/*****************************************************************************
 *  CDMIO.h:                                                                 *
 * ----------                                                                *
 *  This file contains various definitions and function prototypes regarding *
 *  internal utilities for the CDM Input/Output component. The structures    *
 *  defined in this file should be considered CDM internals and should not   *
 *  accessed by non-CDM components, as these structures are subject to       *
 *  change in future versions.                                               *
 *                                                                           *
 *  Copyright 1998-2010 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#include "CDM.h"

#ifndef _CDM_IO_H_

/*
 * The following definition is the header that will always exist at the top of
 * a Collection file. It acts as a "signature", identifying a file as a CDM
 * Collection...
 */
#define CDM_CollectionHeader \
"#Ellogon: Collection File\n" \
"# This is an Auto-generated File. Please DO NOT EDIT.\n" \
"# Ellogon CDM version 1.0 - August 1999 - September 2002\n" \
"#  This file is used by the \"Ellogon\" Text Engineering Platform,\n" \
"#  and contains an \"Ellogon\" CDM Collection specification...\n" \
"#  File Encoding: "CDMIO_DefaultEncoding

/*
 * The following definition is the header that will always exist at the top of
 * a Document file. It acts as a "signature", identifying a file as a CDM
 * Document...
 */
#define CDM_DocumentHeader \
"#Ellogon: Document File\n" \
"# This is an Auto-generated File. Please DO NOT EDIT.\n" \
"# Ellogon CDM version 1.0 - August 1999 - September 2002\n" \
"#  This file is used by the \"Ellogon\" Text Engineering Platform,\n" \
"#  and contains an \"Ellogon\" CDM Document specification...\n" \
"#  File Encoding: "CDMIO_DefaultEncoding

/*
 * GATE I/O Function Prototypes. These functions are implemented in file
 * GateIO.cpp...
 */
extern int CDM_SaveGateCollection(CDM_Collection Collection, Tcl_Channel File);
extern int CDM_LoadGateCollection(CDM_Collection Collection, Tcl_Channel File);
extern int CDM_SaveGateDocument(CDM_Document Document, Tcl_Channel File);
extern int CDM_LoadGateDocument(CDM_Document NewDocument, Tcl_Channel File,
                                char **Id, char **ExternalId);
extern char          *CDM_ReadGateString(Tcl_Channel File);
extern CDM_Attribute  CDM_ReadGateAttribute(Tcl_Channel File);

/*
 * Function Prototypes contained in file CDMFileIO.cpp...
 */
extern Tcl_Channel  CDMIO_OpenCollectionSaveFile(CDM_Collection Collection);
extern Tcl_Channel  CDMIO_OpenCollectionReadFile(CDM_CONST char *path,
                                                 int *format);
extern Tcl_Channel  CDMIO_OpenDocumentSaveFile(CDM_Document Document,
                                              char *Directory);
extern int          CDMIO_WriteCollectionHeader(Tcl_Channel File);
extern int          CDMIO_WriteDocumentHeader(Tcl_Channel File);
extern int          CDMIO_WriteField(Tcl_Channel File, const char *name,
                                     const char *value);
extern int          CDMIO_WriteField(Tcl_Channel File, const char *name,
                                     int   value);
extern int          CDMIO_WriteField(Tcl_Channel File, const char *name,
                                     long  value);
extern int          CDMIO_WriteLengthedObject(Tcl_Channel File, const char *fmt,
                                              Tcl_Obj *value);
extern int          CDMIO_WriteAttributes(Tcl_Channel File,  Tcl_Obj *Atts);
extern int          CDMIO_WriteAnnotations(Tcl_Channel File, Tcl_Obj *Anns);
extern int          CDMIO_WriteRawData(Tcl_Channel File, Tcl_Obj *RawData);
/*
 * GUI Related function prototypes...
 */
extern int          CDMIO_DisplayAppendProgressDialog(Tcl_Interp *interp,
                          const char *key, const char *defMessage);
extern int          CDMIO_UpdateProgressDialog(Tcl_Interp *interp, int progress,
                          Tcl_Obj *win);
extern int          CDMIO_DestroyAppendProgressDialog(Tcl_Interp *interp,
                          Tcl_Obj *win);
#endif /* _CDM_IO_H_ */
