/*****************************************************************************
 *  CDM_CommandSpec.h:                                                       *
 * ------------------------------------                                      *
 *  This file defines a structure that holds the names and function addresses*
 *  of all CDM commands...                                                   *
 *                                                                           *
 *  Copyright 1998-2010 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licensing information, please refer to the license conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifndef _CDM_COMMAND_SPEC
#define _CDM_COMMAND_SPEC
typedef struct CDM_CommandSpec {
  const char *cmd;
  Tcl_ObjCmdProc *proc;
} CDM_CommandSpec;

static const CDM_CommandSpec CommandSpec[] = {
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     C O L L E C T I O N            S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_AnnotateCollection", CDM_AnnotateCollectionObjCmd},
  {"tip_Close", CDM_CloseObjCmd},
  {"tip_CreateCollection", CDM_CreateCollectionObjCmd},
  {"tip_CreateDocument", CDM_CreateDocumentObjCmd},
  {"tip_Destroy", CDM_DestroyObjCmd},
  {"tip_FirstDocument", CDM_FirstDocumentObjCmd},
  {"tip_GetAssociatedInfo", CDM_GetAssociatedInfoObjCmd},
  {"tip_GetAttribute", CDM_GetAttributeObjCmd},
  {"tip_GetAttributes", CDM_GetAttributesObjCmd},
  {"tip_GetByExternalId", CDM_GetByExternalIdObjCmd},
  {"tip_GetComment", CDM_GetCommentObjCmd},
  {"tip_GetDocument", CDM_GetDocumentObjCmd},
  {"tip_GetEncoding", CDM_GetEncodingObjCmd},
  {"tip_GetOwner", CDM_GetOwnerObjCmd},
  {"tip_NextDocument", CDM_NextDocumentObjCmd},
  {"tip_OpenCollection", CDM_OpenCollectionObjCmd},
  {"tip_PutAttribute", CDM_PutAttributeObjCmd},
  {"tip_RemoveAttribute", CDM_RemoveAttributeObjCmd},
  {"tip_RemoveDocument", CDM_RemoveDocumentObjCmd},
  {"tip_SetAssociatedInfo", CDM_SetAssociatedInfoObjCmd},
  {"tip_SetComment", CDM_SetCommentObjCmd},
  {"tip_SetEncoding", CDM_SetEncodingObjCmd},
  {"tip_SetName", CDM_SetNameObjCmd},
  {"tip_SetOwner", CDM_SetOwnerObjCmd},
  {"tip_Sync", CDM_SyncObjCmd},
  {"tip_Status", CDM_StatusObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     D O C U M E N T                S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_AnnotationsAt", CDM_AnnotationsAtObjCmd},
  {"tip_AnnotationsContaining", CDM_AnnotationsContainingObjCmd},
  {"tip_AnnotationsInRange", CDM_AnnotationsInRangeObjCmd},
  {"tip_AnnotationsMatchingRange", CDM_AnnotationsMatchingRangeObjCmd},
  {"tip_ByteSequenceInsertString", CDM_ByteSequenceInsertStringObjCmd},
  {"tip_ByteSequenceReplace", CDM_ByteSequenceReplaceObjCmd},
  {"tip_ByteSequenceReplaceCharacters",CDM_ByteSequenceReplaceCharactersObjCmd},
  {"tip_CreateVectors", CDM_CreateVectorsObjCmd},
  {"tip_DeleteAnnotations", CDM_DeleteAnnotationsObjCmd},
  {"tip_DisplaceAnnotations", CDM_DisplaceAnnotationsObjCmd},
  {"tip_FindMaxUsedAnnotationId", CDM_FindMaxUsedAnnotationIdObjCmd},
  {"tip_FirstAnnotationContaining", CDM_FirstAnnotationContainingObjCmd},
  {"tip_GetAnnotation", CDM_GetAnnotationObjCmd},
  {"tip_GetAnnotations", CDM_GetAnnotationsObjCmd},
  {"tip_GetByteSequence", CDM_GetByteSequenceObjCmd},
  {"tip_GetExternalId", CDM_GetExternalIdObjCmd},
  {"tip_GetFirstAnnotation", CDM_GetFirstAnnotationObjCmd},
  {"tip_GetNextAnnotation", CDM_GetNextAnnotationObjCmd},
  {"tip_GetParent", CDM_GetParentObjCmd},
  {"tip_GetRawData", CDM_GetByteSequenceObjCmd},
  {"tip_NextAnnotations", CDM_NextAnnotationsObjCmd},
  {"tip_RawDataInsertString", CDM_ByteSequenceInsertStringObjCmd},
  {"tip_RawDataReplace", CDM_ByteSequenceReplaceObjCmd},
  {"tip_RawDataReplaceCharacters", CDM_ByteSequenceReplaceCharactersObjCmd},
  {"tip_RemoveAnnotation", CDM_RemoveAnnotationObjCmd},
  {"tip_SelectAnnotations", CDM_SelectAnnotationsObjCmd},
  {"tip_SetByteSequence", CDM_SetByteSequenceObjCmd},
  {"tip_SetExternalId", CDM_SetExternalIdObjCmd},
  {"tip_SetRawData", CDM_SetByteSequenceObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     A T T R I B U T E S            S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_CreateAttribute", CDM_CreateAttributeObjCmd},
  {"tip_CreateAttributeValue", CDM_CreateAttributeValueObjCmd},
  {"tip_CreateAttributeSet", CDM_CreateAttributeSetObjCmd},
  {"tip_GetName", CDM_GetNameObjCmd},
  {"tip_GetValue", CDM_GetValueObjCmd},
  {"tip_GetValueType", CDM_GetValueTypeObjCmd},
  {"tip_GetValueValue", CDM_GetValueValueObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     A N N O T A T I O N S          S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_AnnotationContainsAttributeMatchingValue",
                           CDM_AnnotationContainsAttributeMatchingValueObjCmd},
  {"tip_AnnotationContainsAttributeMatchingValues",
                           CDM_AnnotationContainsAttributeMatchingValuesObjCmd},
  {"tip_AddAnnotation", CDM_AddAnnotationObjCmd},
  {"tip_AnnotationContainsPosition", CDM_AnnotationContainsPositionObjCmd},
  {"tip_AnnotationMatchesRange", CDM_AnnotationMatchesRangeObjCmd},
  {"tip_CompareAnnotations", CDM_CompareAnnotationsObjCmd},
  {"tip_CreateAnnotation", CDM_CreateAnnotationObjCmd},
  {"tip_DisplaceAnnotation", CDM_DisplaceAnnotationObjCmd},
  {"tip_GetFirstSpan", CDM_GetFirstSpanObjCmd},
  {"tip_GetFirstSpanOffsets", CDM_GetFirstSpanOffsetsObjCmd},
  {"tip_GetSpans", CDM_GetSpansObjCmd},
  {"tip_GetType", CDM_GetTypeObjCmd},
  {"tip_GetId", CDM_GetIdObjCmd},
  {"tip_AddSpan", CDM_AddSpanObjCmd},
  {"tip_RemoveSpan", CDM_RemoveSpanObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     A N N O T A T I O N    S E T   S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_Length", CDM_LengthObjCmd},
  {"tip_CreateAnnotationSet", CDM_CreateAnnotationSetObjCmd},
  {"tip_CompareAnnotationSets", CDM_CompareAnnotationSetsObjCmd},
  {"tip_GetAnnotatedTextRanges", CDM_GetAnnotatedTextRangesObjCmd},
  {"tip_GetFirstAnnotatedTextRange", CDM_GetFirstAnnotatedTextRangeObjCmd},
  {"tip_Nth", CDM_NthObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     S P A N                        S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_CreateSpan", CDM_CreateSpanObjCmd},
  {"tip_CreateSpanSet", CDM_CreateSpanSetObjCmd},
  {"tip_GetSpanOffsets", CDM_GetSpanOffsetsObjCmd},
  {"tip_GetStart", CDM_GetStartObjCmd},
  {"tip_GetEnd", CDM_GetEndObjCmd},
  {"tip_MergeAnnotations", CDM_MergeAnnotationsObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     U T I L I T I E S              S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
#ifdef CDM_ENABLE_TIME_ROUTINES
  {"CDM_GetTime", CDM_CdmGetTimeObjCmd},
  {"CDM_Time", CDM_TimeObjCmd},
#endif /* CDM_ENABLE_TIME_ROUTINES */
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |      V A L I D A T I O N           S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"tip_ValidAnnotation", CDM_ValidAnnotationObjCmd},
  {"tip_ValidAttribute", CDM_ValidAttributeObjCmd},
  {"tip_ValidCollection", CDM_ValidCollectionObjCmd},
  {"tip_ValidDocument", CDM_ValidDocumentObjCmd},
  
  {"CDM_ClearCompressedMemoryCache", CDM_ClearCompressedMemoryCacheObjCmd},
  {"CDM_CompressedMemoryCacheInfo", CDM_CompressedMemoryCacheInfoObjCmd},
  {"CDM_CompressedMemoryCacheStatistics", CDM_CompressedMemoryCacheStatisticsObjCmd},
  {"CDM_ClearSavedValues", CDM_ClearSavedValuesObjCmd},
  {"CDM_ClearUnusedSavedValues", CDM_ClearUnusedSavedValuesObjCmd},
  {"CDM_Eval", CDM_EvalObjCmd},
  {"CDM_Exec", CDM_ExecObjCmd},
  {"tip_ExtractTextBasedOnAnnotations", CDM_ExtractTextBasedOnAnnotationsObjCmd},
  {"CDM_GetCollectionFileXIDs", CDM_GetCollectionFileXIDsObjCmd},
  {"CDM_GetCollectionFileIDs", CDM_GetCollectionFileIDsObjCmd},
  {"CDM_GetDocumentAttributes", CDM_GetDocumentAttributesObjCmd},
  {"CDM_GetCurrentThread", CDM_GetCurrentThreadObjCmd},
  {"CDM_LappendUnique", CDM_LappendUniqueObjCmd},
  {"CDM_PresentAnnotationForRawViewer", CDM_PresentAnnotationForRawViewerObjCmd},
  {"CDM_GetRegisteredCollectionTypes", CDM_GetRegisteredCollectionTypesObjCmd},
  {"CDM_DeclareLicensingInformation", CDM_DeclareLicensingInformationObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |      T H R E A D    S A F E T Y    S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"CDM_MutexLock",   CDM_MutexLockObjCmd},
  {"CDM_MutexUnlock", CDM_MutexUnlockObjCmd},
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |  P L A T F O R M   U T I L I T I E S   S E C T I O N  |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
  {"CDM_AddLibraryImage", CDM_AddLibraryImageObjCmd},
  {NULL, NULL}
};
#endif /* _CDM_COMMAND_SPEC */
