/*****************************************************************************
 *  CDM_GlobalObjects.h:                                                     *
 * ----------------------                                                    *
 *  This file defines some utitlity definitions for members stored in thread *
 *  specific data.                                                           *
 *  CDM is the Collection and Document Manager and the core component of the *
 *  Ellogon Text Engineering Platform.                                       *
 *  CDM was designed and implemented by Georgios Petasis,                    *
 *  e-mail: petasis@iit.demokritos.gr                                        *
 *  URL:    http://www.iit.demokritos.gr/~petasis                            *
 *                                                                           *
 *  Copyright 1998-2010 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifndef _CDM_GLOBALOBJECTS_H
#define _CDM_GLOBALOBJECTS_H

#include "CDM_MemoryCache.h"

/*
 * Global Variables (Many of them accesible from Tcl...). Since CDM must be
 * thread safe, each thread has its own version of these variables...
 */
typedef struct ThreadSpecificData {
  CDM_UCHAR CDM_LibraryInitialised;      /* True if CDM has already been
                                            initialised. False otherwise... */
  CDM_UniqIDs ID;
  CDM_ObjectCache ObjectCache;
  Tcl_Interp *CDM_Interp;
  Tcl_Interp *CDMEmbedInterp;
  char *CDM_DefaultEncoding, *CDM_TempDir;
  char  CDM_Temp[MAX_TEMP_STRING+2];     /* A quite large writtable string for
                                            storing strings *temporaly* */
  /*
   * The following pointer will be initialised to hold an integer that will
   * signals that CDM is unlocked, i.e. the caller has a valid license for
   * running Ellogon. Otherwise, CDM will be directed to abort, each time one of
   * its functions is used...
   */
  CDM_UCHAR CDM_UnLocked;
  /*
   * The following flag controls whether CDM will also try to compress 
   * memory by hashing attribute sets from annotations...
   */
  int    CDM_EnableMemoryCompression;
  
  /*
   * The following flag signals that processing will stop. This means that if
   * this flag is true, CDM functions should keep returning errors...
   */
  int    CDM_AbortCurrentProcess;
  
  /*
   * The following flag, if true, causes to output debug information (through
   * CDM_DEBUG) in stdout instead of stderr...
   */
  int    CDM_DebugStdout;
  
  int    CDM_DisplayErrorMessage, CDM_Pid,
         CDM_FullGateCompatibility, CDM_EmulateGateGetValue;
  Tcl_Obj CDM_Obj_GDM_STRING,            /* Object holding the string
                                            "GDM_STRING" */
          CDM_Obj_GDM_STRING_SET,        /* Object holding the string
                                            "GDM_STRING_SET" */
          CDM_Obj_CDM_IMAGE,             /* Object holding the string
                                            "CDM_IMAGE" */
          CDM_Obj_CDM_BASE64_IMAGE,      /* Object holding the string
                                            "CDM_BASE64_IMAGE" */
          CDM_Obj_NULL;                  /* Object holding the empty string */
  int NumberOfDisplacedAnnotations;
  /*
   * Additional Collection Type Plug-ins...
   */
  CDM_RegisteredCollectionTypes
           CDM_AdditionalCollectionTypes[CDM_MAXIMUM_COLLECTION_PLUGINS_NUMBER];
  Tcl_Obj *CDM_MachineIPObject;
  char *CDM_CollectionFilename, *CDM_CollectionTypeFilename;

  /*
   * Structures used by CDM_Utils.cpp
   */
  Tcl_HashTable   AttrSetHash;
  CDM_UCHAR       AttrSetHash_Initialised;
  Tcl_HashKeyType AttrSetHash_TypeStruct;
#ifdef CDM_AYTOCLEAN_COMPRESSEDMEMORYCACHE
  int             AttrSetHash_NumberOfCalls;
  int             AttrSetHash_NumberOfClears;
#endif /* CDM_AYTOCLEAN_COMPRESSEDMEMORYCACHE */
  /*
   * Structures used bu CDM_MemoryCache.cpp
   */
  Tcl_HashTable *LongTablePtr;
} ThreadSpecificData;

/*
 * These are convenience macros used to initialize a thread local storage ptr.
 */
#ifndef TCL_TSD_INIT
#define TCL_TSD_INIT(keyPtr) \
  (ThreadSpecificData *)Tcl_GetThreadData((keyPtr), sizeof(ThreadSpecificData))
#endif /* TCL_TSD_INIT */
#define CDM_tsdPrt(keyPtr) \
   ThreadSpecificData *tsdPtr = (ThreadSpecificData *) \
         Tcl_GetThreadData((keyPtr), sizeof(ThreadSpecificData))

#define CDM_THREAD_SPECIFIC_DATA \
   ThreadSpecificData *tsdPtr = (ThreadSpecificData *) \
         Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));

#define CDM_UPDATE_THREAD_INTERP \
   ThreadSpecificData *tsdPtr = (ThreadSpecificData *) \
         Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));\
   CDM_Interp = interp;

#define CDM_LibraryInitialised        (tsdPtr->CDM_LibraryInitialised)
#define ID                            (tsdPtr->ID)
#define ObjectCache                   (tsdPtr->ObjectCache)
#define CDM_Interp                    (tsdPtr->CDM_Interp)
#define CDMEmbedInterp                (tsdPtr->CDMEmbedInterp)
#define CDM_DefaultEncoding           (tsdPtr->CDM_DefaultEncoding)
#define CDM_TempDir                   (tsdPtr->CDM_TempDir)
#define CDM_Temp                      (tsdPtr->CDM_Temp)
#define CDM_UnLocked                  (tsdPtr->CDM_UnLocked)
#define CDM_EnableMemoryCompression   (tsdPtr->CDM_EnableMemoryCompression)
#define CDM_AbortCurrentProcess       (tsdPtr->CDM_AbortCurrentProcess)
#define CDM_DebugStdout               (tsdPtr->CDM_DebugStdout)
#define CDM_DisplayErrorMessage       (tsdPtr->CDM_DisplayErrorMessage)
#define CDM_Pid                       (tsdPtr->CDM_Pid)
#define CDM_FullGateCompatibility     (tsdPtr->CDM_FullGateCompatibility)
#define CDM_EmulateGateGetValue       (tsdPtr->CDM_EmulateGateGetValue)
#define CDM_Obj_GDM_STRING            (tsdPtr->CDM_Obj_GDM_STRING)
#define CDM_Obj_GDM_STRING_SET        (tsdPtr->CDM_Obj_GDM_STRING_SET)
#define CDM_Obj_CDM_IMAGE             (tsdPtr->CDM_Obj_CDM_IMAGE)
#define CDM_Obj_CDM_BASE64_IMAGE      (tsdPtr->CDM_Obj_CDM_BASE64_IMAGE)
#define CDM_Obj_NULL                  (tsdPtr->CDM_Obj_NULL)
#define NumberOfDisplacedAnnotations  (tsdPtr->NumberOfDisplacedAnnotations)
#define CDM_AdditionalCollectionTypes (tsdPtr->CDM_AdditionalCollectionTypes)
#define CDM_MachineIPObject           (tsdPtr->CDM_MachineIPObject)
#define CDM_CollectionFilename        (tsdPtr->CDM_CollectionFilename)
#define CDM_CollectionTypeFilename    (tsdPtr->CDM_CollectionTypeFilename)
#define AttrSetHash                   (tsdPtr->AttrSetHash)
#define AttrSetHash_Initialised       (tsdPtr->AttrSetHash_Initialised)
#define AttrSetHash_TypeStruct        (tsdPtr->AttrSetHash_TypeStruct)
#define AttrSetHash_NumberOfCalls     (tsdPtr->AttrSetHash_NumberOfCalls)
#define AttrSetHash_NumberOfClears    (tsdPtr->AttrSetHash_NumberOfClears)
#define LongTablePtr                  (tsdPtr->LongTablePtr)

#endif /* _CDM_GLOBALOBJECTS_H */
