/*****************************************************************************
 *  CDM_License.h:                                                           *
 * ----------------------                                                    *
 *  This file implements a class responsible for generating and validating   *
 *  license keys...
 *                                                                           *
 *  Copyright 1998-2004 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifndef _CDM_LICENSE
#define _CDM_LICENSE

#include <tcl.h>

class CDM_LicenseClass {
  private:
    /* charData keeps the "characters" of the key during processing */
    Tcl_HashTable charData;
    /* The current index into charData */
    int charDataIndex;
    /* The checksum is a  hash of the characters */
    int checkSum;
    /* Encode key data in 5-bit words so we can use a 32 char encoding alphabet
       that is single case. 0-9, A-Z, excluding I, L, O and Q */
    char *Map[33], UnMap[129];
    /* These are a function of the size of the word */
    int nbits; int mask;

    /* A temporary dynamic string */
    Tcl_DString Result;

    /* Basic Methods */
    void  Init();
    void  PackInteger(int value, int bits);
    void  PackInteger(int value) {PackInteger(value, 32);};
    char *PackString(char *string);
    char *UnPackString(char *code);
    void  SpreadCheck(void);
    char *KeyToString(void);
    int   UnPack(char *key);
    int   GetInteger(int bits);
    int   HashString(char *string);
    void  PrintCharacterData(void);
    int   ScanDate(Tcl_Obj *date);
    int   GetClockField(int seconds, char *format);

    /* Key Generation methods... */
    char *GenerateKey(int KeyType, char *CustomerID, char *ProductID,
                      char *HostID, char *HostIP, int expire, char *UserKey);
    /* Key Validation methods... */
    int   ValidateKey(char *key, char *CustomerID, char *ProductID,
                      char *HostID, char *HostIP, int expire, char *UserKey,
                      int *expirationDate);
  public:
    /* Key Generation methods... */
    char *GenerateKey(Tcl_Obj *KeyType, Tcl_Obj *CustomerID, Tcl_Obj *ProductID,
                      Tcl_Obj *HostID, Tcl_Obj *HostIP, Tcl_Obj *Expire,
                      Tcl_Obj *UserKey);
    /* Key Validation methods... */
    int   ValidateKey(Tcl_Obj *key, Tcl_Obj *CustomerID, Tcl_Obj *ProductID,
                      Tcl_Obj *HostID, Tcl_Obj *HostIP, Tcl_Obj *Expire,
                      Tcl_Obj *UserKey, int *expirationDate);
    
    CDM_LicenseClass();
    ~CDM_LicenseClass();
};

/*
 * The various License types:
 */
#define LICENSE_TYPE_RESEARCH    1
#define LICENSE_TYPE_COMMERCIAL  2
#define LICENSE_TYPE_DEMO        3

/* 
 * Error Definitions: These errors are returned by ValidateKey to report what
 * went wrong during key validation
 */
#define LICENSE_VALID_KEY        0
#define LICENSE_INV_KEYTYPE      1
#define LICENSE_INV_CUSTOMERID   2
#define LICENSE_INV_PRODUCTID    3
#define LICENSE_INV_HOSTID       4
#define LICENSE_INV_HOSTIP       5
#define LICENSE_INV_USERKEY      6
#define LICENSE_INV_DATETYPE     7
#define LICENSE_INV_YEAR         8
#define LICENSE_INV_MONTH        9
#define LICENSE_INV_DAY         10

#endif
