##############################################################################
##  CDM_LicenseDlg.tcl:                                                     ##
## ---------------------                                                    ##
##  This file implements a dialog, from where the user can create a valid   ##
##  license file from information we has received from e-mail.              ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::CDM {}
proc ::CDM::CDM_LicenseInfoDlg {{win .cmdLicenseInfoDlg}} {
  package require Tk
  package require BWidget
  global CDM_UserLicenseInfo GUI_Text tcl_platform
  catch {source [file join $::HomeDir License License.tcl]}
  
  if {![info exists CDM_UserLicenseInfo(CustomerID)]} {
    if {[info exists ::env(USER)]} {
      set CDM_UserLicenseInfo(CustomerID) $::env(USER)
    }
  }
  if {![info exists CDM_UserLicenseInfo(ProductID)]} {
    set CDM_UserLicenseInfo(ProductID) \
        "$::ApplicationName $::ApplicationVersion"
  }
  if {![info exists CDM_UserLicenseInfo(HostID)]} {
    set CDM_UserLicenseInfo(HostID) "[info hostname] - $tcl_platform(platform)\
        ($tcl_platform(os), $tcl_platform(machine))"
  }
  if {![info exists CDM_UserLicenseInfo(HostIP)]} {
    set CDM_UserLicenseInfo(HostIP) [CDM_GetIP]
  }
  if {![info exists CDM_UserLicenseInfo(Expire)]} {
    set CDM_UserLicenseInfo(Expire) never
  }

  catch {wm withdraw .splashscr}
  catch {destroy $win}
  toplevel $win
  wm withdraw $win
  wm title $win {Please Enter Registration Information:}
  ## A widget with the "keys" image...
  label $win.keys -image [Bitmap::get passwd]
  label $win.customerID  -anchor w -text {Customer ID:}
  entry $win.ecustomerID -textvariable CDM_UserLicenseInfo(CustomerID) -width 55
  label $win.productID   -anchor w -text {Product ID:}
  entry $win.eproductID  -textvariable CDM_UserLicenseInfo(ProductID)
  label $win.hostID      -anchor w -text {Hostname:}
  entry $win.ehostID     -textvariable CDM_UserLicenseInfo(HostID)
  label $win.hostIP      -anchor w -text {Hostname IP:}
  entry $win.ehostIP     -textvariable CDM_UserLicenseInfo(HostIP)
  label $win.expire      -anchor w -text {Expiration Date:}
  entry $win.eexpire     -textvariable CDM_UserLicenseInfo(Expire)
  label $win.userKey     -anchor w -text {User Password:}
  entry $win.euserKey    -textvariable CDM_UserLicenseInfo(UserKey) \
    -show *
  label $win.licenseType -anchor w -text {License Type:}
  ComboBox $win.elicenseType -textvariable CDM_UserLicenseInfo(LicenseType) \
    -values {{Research License} {Demo License} {Commercial License}} \
    -editable 0 -modifycmd "::CDM::CDM_LicenseInfoDlgTypeChanged \
    $win $win.elicenseType $win.eexpire"
  $win.elicenseType setvalue first
  label $win.licenseKey  -anchor w -text {License Key:}
  entry $win.elicenseKey -textvariable CDM_UserLicenseInfo(LicenseKey)

  frame $win.buttons
    if {![info exists GUI_Text(Cancel)]} {
      array set GUI_Text {Update Update Validate Validate
                          Generate Generate Cancel Cancel}
    }
    button $win.buttons.update -textvariable GUI_Text(Update) \
      -command "::CDM::CDM_LicenseInfoDlgUpdate $win"
    button $win.buttons.valid  -textvariable GUI_Text(Validate) \
      -command "::CDM::CDM_LicenseInfoDlgValidate $win"
    button $win.buttons.gene  -textvariable GUI_Text(Generate) \
      -command "::CDM::CDM_LicenseInfoDlgGenerate $win"
    button $win.buttons.cancel -textvariable GUI_Text(Cancel) \
      -command "destroy $win"
    ## Can we generate keys?
    ## We should first check if the command exists. In case CDM has not been
    ## loaded, this command has not been yet registered!
    if {[llength [info commands CDM_GenerateLicenseKey]] == 1} {
      if {[CDM_GenerateLicenseKey GenerateCapable?]} {
        $win.buttons.update configure -command \
          "::CDM::CDM_LicenseInfoDlgUpdate $win 1"
        pack $win.buttons.valid $win.buttons.gene $win.buttons.update \
          $win.buttons.cancel -side left -padx 10 -pady 10
      } else {
        pack $win.buttons.valid $win.buttons.update $win.buttons.cancel \
          -side left -padx 10 -pady 10
      }
    } else {
      pack $win.buttons.valid $win.buttons.update $win.buttons.cancel \
          -side left -padx 10 -pady 10
    }

  grid $win.keys         -padx 9 -pady 2 -sticky snew -rowspan 7
  grid x $win.customerID  $win.ecustomerID  -padx 2 -pady 2 -sticky snew -row 0
  grid x $win.productID   $win.eproductID   -padx 2 -pady 2 -sticky snew -row 1
  grid x $win.hostID      $win.ehostID      -padx 2 -pady 2 -sticky snew -row 2
  grid x $win.hostIP      $win.ehostIP      -padx 2 -pady 2 -sticky snew -row 3
  grid x $win.expire      $win.eexpire      -padx 2 -pady 2 -sticky snew -row 4
  grid x $win.userKey     $win.euserKey     -padx 2 -pady 2 -sticky snew -row 5
  grid x $win.licenseType $win.elicenseType -padx 2 -pady 2 -sticky snew -row 6
  grid x $win.licenseKey  $win.elicenseKey  -padx 2 -pady 2 -sticky snew -row 7
  grid x $win.buttons -columnspan 2
  gut_ModalDlg $win
  wm deiconify $win
  tkwait window $win
};# CDM_LicenseInfoDlg

## ::CDM::CDM_LicenseInfoDlgUpdate
#  This function will create a License file, according to the information
#  obtained from the user...
proc ::CDM::CDM_LicenseInfoDlgUpdate {win {window 0}} {
  global HomeDir CDM_UserLicenseInfo

  ## Are all fields filled?
  foreach name {CustomerID ProductID HostID HostIP Expire UserKey
                LicenseType LicenseKey} {
    if {![string length $CDM_UserLicenseInfo($name)]} {
      gut_warning "Field \"$name\" seems to be empty. Please complete all\
      available fileds."
      return
    }
  }
  
  set valid [::CDM::CDM_LicenseInfoDlgValidate $win 1]
  if {!$valid} {return}

  set LicenseInfo \
"array set CDM_UserLicenseInfo \{
  CustomerID    \{$CDM_UserLicenseInfo(CustomerID)\}
  ProductID     \{$CDM_UserLicenseInfo(ProductID)\}
  HostID        \{$CDM_UserLicenseInfo(HostID)\}
  HostIP        \{$CDM_UserLicenseInfo(HostIP)\}
  Expire        \{$CDM_UserLicenseInfo(Expire)\}
  UserKey       \{$CDM_UserLicenseInfo(UserKey)\}
  LicenseType   \{$CDM_UserLicenseInfo(LicenseType)\}
  LicenseKey    \{$CDM_UserLicenseInfo(LicenseKey)\}
\}
return -code break\;"

  if {$window} {
    ggi_view_text $LicenseInfo
  } else {
    set File [file join $HomeDir License License.tcl]
    file mkdir [file join $HomeDir License]

    if {[file exists $File]} {
      set ans [gut_question \
        {Are you sure you want to update License information?}]
      if {[string equal $ans no]} {return}
    }
    if {[catch {open $File w} file]} {
      error "Cannot open file \"$File\" for writting!\nThe error was:\n$file"
    }
    puts $file $LicenseInfo
    close $file
    destroy $win
    catch {wm deiconify .splashscr}
  }
};# ::CDM::CDM_LicenseInfoDlgUpdate

proc ::CDM::CDM_LicenseInfoDlgValidate {win {dialog_only_on_error 0}} {
  ## If the validation command was not found, all keys are regarded valid :-)
  if {[llength [info commands CDM_ValidateLicenseKey]] == 0} {
    return 1
  }
  
  global CDM_UserLicenseInfo
  set res [CDM_ValidateLicenseKey $CDM_UserLicenseInfo(LicenseKey) \
                                  $CDM_UserLicenseInfo(CustomerID) \
                                  $CDM_UserLicenseInfo(ProductID) \
                                  $CDM_UserLicenseInfo(HostID) \
                                  $CDM_UserLicenseInfo(HostIP) \
                                  $CDM_UserLicenseInfo(Expire) \
                                  $CDM_UserLicenseInfo(UserKey)]
  switch $res {
    LICENSE_VALID_KEY      {set res {The Key is valid!}}
    LICENSE_INV_KEYTYPE    {set res "Invalid Key!\n(Invalid Key Type)"}
    LICENSE_INV_CUSTOMERID {set res "Invalid Key!\n(Invalid Customer ID)"}
    LICENSE_INV_PRODUCTID  {set res "Invalid Key!\n(Invalid Product ID)"}
    LICENSE_INV_HOSTID     {set res "Invalid Key!\n(Invalid Host ID)"}
    LICENSE_INV_HOSTIP     {set res "Invalid Key!\n(Invalid Host IP)"}
    LICENSE_INV_USERKEY    {set res "Invalid Key!\n(Invalid User Password)"}
    LICENSE_INV_DATETYPE   {set res "Invalid Key!\n(Invalid Date Type)"}
    LICENSE_INV_YEAR       {set res "Invalid Key!\n(Invalid Year)"}
    LICENSE_INV_MONTH      {set res "Invalid Key!\n(Invalid Month)"}
    LICENSE_INV_DAY        {set res "Invalid Key!\n(Invalid Day)"}
  }
  
  if {$dialog_only_on_error} {
    if {$res != {The Key is valid!}} {
      gut_info $res
      return 0
    } else {
      return 1
    }
  }
  gut_info $res
};# ::CDM::CDM_LicenseInfoDlgValidate

proc ::CDM::CDM_LicenseInfoDlgGenerate {win} {
  global CDM_UserLicenseInfo
  
  ## Are all fields filled?
  foreach name {CustomerID ProductID HostID HostIP Expire UserKey
                LicenseType} {
    if {![string length $CDM_UserLicenseInfo($name)]} {
      gut_warning "Field \"$name\" seems to be empty. Please complete all\
                   available fileds."
      return
    }
  }

  set res [CDM_GenerateLicenseKey $CDM_UserLicenseInfo(LicenseType) \
                                  $CDM_UserLicenseInfo(CustomerID) \
                                  $CDM_UserLicenseInfo(ProductID) \
                                  $CDM_UserLicenseInfo(HostID) \
                                  $CDM_UserLicenseInfo(HostIP) \
                                  $CDM_UserLicenseInfo(Expire) \
                                  $CDM_UserLicenseInfo(UserKey)]
  set CDM_UserLicenseInfo(LicenseKey) $res
};# ::CDM::CDM_LicenseInfoDlgGenerate

## ::CDM::CDM_LicenseInfoDlgTypeChanged
#  This will be called when the License type changes. Simply we want to
#  change the expiration date...
proc ::CDM::CDM_LicenseInfoDlgTypeChanged {win combo expire} {
  global CDM_UserLicenseInfo
  switch $CDM_UserLicenseInfo(LicenseType) {
    {Demo License} {
      set CDM_UserLicenseInfo(Expire) \
          [clock format [clock scan now+31day] -format "%d/%m/%Y"]
    }
    {Research License} {
      set CDM_UserLicenseInfo(Expire) never
    }
    {Commercial License} {
      set CDM_UserLicenseInfo(Expire) \
          [clock format [clock scan now+1year] -format "%d/%m/%Y"]
    }
    default {set CDM_UserLicenseInfo(Expire) never}
  }
};# ::CDM::CDM_LicenseInfoDlgTypeChanged
