/*****************************************************************************
 *  CDM_ObjectTypes.h:                                                       *
 * --------------------                                                      *
 *  This file implements a set of new Tcl object types. These types are used *
 *  by CDM in order to further reduce the memory required to store them.     *
 *  This memory reduction method works better if the TIPSTER API is used to  *
 *  get the sub-elements. If the various Tcl list handling facilities are    *
 *  convertions among types will be made, which is usually a bad thing...    *
 *                                                                           *
 *  Copyright 1998-2008 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licensing information, please refer to the license conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifdef ENABLE_ANNOTATION_OBJECT

#include "CDM.h"

#define CDM_ANN_OBJ_REP_SHORT 0
#define CDM_ANN_OBJ_REP_LONG  1

typedef struct cdmAnnotationObjInternalRep {
  long id;             /* The Annotation Id. */
  long start;          /* The start of the first span. If -1, the Annotation
                          has more than one span. */
  Tcl_Obj *type;       /* The Annotation object type. */
  Tcl_Obj *attributes; /* The Attribute set object. */
  union {
    long end;          /* The end of the first span. Used if start != -1. */
    Tcl_Obj *spanSet;  /* If start == -1, this holds the span set. */
  } span;
} cdmAnnotationObjInternalRep;

typedef struct cdmCompactAnnotationObjInternalRep {
  unsigned short int id;   /* The Annotation Id. */
  unsigned short int start;/* The start of the first span. If 0, the Annotation
                              has more than one span. */
  Tcl_Obj *type;           /* The Annotation object type. */
  Tcl_Obj *attributes;     /* The Attribute set object. */
  union {
    unsigned short int end;/* The end of the first span. Used if start != 0. */
    Tcl_Obj *spanSet;      /* If start == 0, this holds the span set. */
  } span;
} cdmCompactAnnotationObjInternalRep;

extern Tcl_ObjType  cdmAnnotationType;        /* Annotation Tcl Object */
extern Tcl_ObjType  cdmCompactAnnotationType; /* Annotation Tcl Object */
extern Tcl_Obj     *CDM_NewAnnotationObj(long id, Tcl_Obj *type,
                                         long start, long end,
                                         CDM_AttributeSet attibutes);
extern Tcl_Obj     *CDM_NewAnnotationObj(long id, Tcl_Obj *type,
                                         CDM_SpanSet spans,
                                         CDM_AttributeSet attibutes);

extern void CDMAnnObj_SetId(Tcl_Obj *Ann, long id);
extern void CDMAnnObj_SetType(Tcl_Obj *Ann, Tcl_Obj *type);
extern void CDMAnnObj_SetStart(Tcl_Obj *Ann, long start);
extern void CDMAnnObj_SetEnd(Tcl_Obj *Ann, long end);
extern void CDMAnnObj_SetSpans(Tcl_Obj *Ann, CDM_SpanSet spans);
extern void CDMAnnObj_SetAttributes(Tcl_Obj *Ann, CDM_AttributeSet attibutes);
extern void CDMAnnObj_KeepSpanSet(Tcl_Obj *Ann, CDM_SpanSet spans);
extern void CDMAnnObj_FreeKeptSpanSet(Tcl_Obj *Ann);

extern long     CDMAnnObj_GetId(Tcl_Obj *Ann);
extern Tcl_Obj *CDMAnnObj_GetType(Tcl_Obj *Ann);
extern long     CDMAnnObj_GetStart(Tcl_Obj *Ann);
extern long     CDMAnnObj_GetEnd(Tcl_Obj *Ann);
extern Tcl_Obj *CDMAnnObj_GetSpans(Tcl_Obj *Ann);
extern Tcl_Obj *CDMAnnObj_GetAttributes(Tcl_Obj *Ann);

#define ANNOTATION_OBJ(a) \
  (a->typePtr == &cdmCompactAnnotationType || a->typePtr == &cdmAnnotationType)
#endif /* ENABLE_ANNOTATION_OBJECT */
