/*****************************************************************************
 *  CDM_Utils.h:                                                             *
 * --------------------                                                      *
 *  Header file for CDM_Utils.cpp. CDM_Utils.cpp implements various          *
 *  utilities used internally by the Collection & Document Manager.          *
 *                                                                           *
 *  Copyright 1998-2004 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licensing information, please refer to the license conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifndef _CDM_UTILS
#define _CDM_UTILS

#include <tcl.h>
/*
 * Linked List data structure...
 */

#define CDM_LIST_POINTER_NUMBER 4
#define CDM_DOCUMENT_ID         0
#define CDM_DOCUMENT_XID        1
#define CDM_DOCUMENT_OPENED     2
#define CDM_DOCUMENT_POINTER    3
#define CDM_NewList             CDM_NewListNode

typedef struct CDM_ListNodeStructure {
  struct CDM_ListNodeStructure *prev;
  struct CDM_ListNodeStructure *next;
  char *ptr[CDM_LIST_POINTER_NUMBER];
} CDM_ListNode;
typedef struct CDM_ListNodeStructure CDM_List;

CDM_ListNode  *CDM_NewListNode(void);
CDM_ListNode **CDM_NewListNodeArray(int elements);
CDM_ListNode  *CDM_AppendListNode(CDM_List *list, CDM_ListNode *node);
CDM_List      *CDM_DeleteListNode(CDM_List *list, CDM_ListNode *node);
void           CDM_DeleteList(CDM_List *list);
CDM_ListNode  *CDM_ListFind(CDM_List *list, char *str, int slot);
int            CDM_ListLength(CDM_List *list);
Tcl_Obj       *CDM_ListGetObject(CDM_List *list, int slot);

/*
 * Functions used internally by the CDM in order to further compress memory.
 */
void             CDM_AttributeSetHashInitialise(void);
int              CDM_ClearAttributeSetHash(void);
int              CDM_ClearUnusedAttributeSetHash(void);
CDM_AttributeSet CDM_SearchForAttributeSet(CDM_AttributeSet Set);
#endif /* _CDM_UTILS */
