#!/bin/sh
# Restart using tclsh \
  exec tclsh "$0" "$@"

## CDM Version Info:
set CDM_MAJOR_VERSION          2
set CDM_MINOR_VERSION          0
set CDM_VERSION               \"2.0\"
set CDM_PATCH_LEVEL           \"2.0a1\"
## The following command will produce a date like:
## "04/09/2001"...
set CDM_RELEASE_DATE          \"[clock format [file mtime CDM.cpp] \
                                     -format {%d/%m/%Y}]\"
## The following command will produce a date like:
## "Tue Sep  4 19:08:24 EEST 2001"...
set CDM_RELEASE_DATE_STR      \"[clock format [file mtime CDM.cpp] \
                                     -format {%a %b %d %T %Z %Y}]\"
puts "#####################################################################"
puts "##  Creating CDM Version Headers:                                  ##"
puts "## -------------------------------                                 ##"
puts "##                                                                 ##"

set file [open CDM_Version.h w]
puts $file \
"/****************************************************************************
 **  CDM_Version.h                                                         **
 ** ---------------                                                        **
 **                                                                        **
 ** Various definitions that help in identifying the current CDM version.  **
 ** This file is autogenerated. Do not modify...                           **
 **                                                                        **
 ** Copyright 1998-[clock format [clock seconds] -format {%Y}] by:\
 \                                               **
 ** Georgios Petasis,                                                      **
 ** Software and Knowledge Engineering Laboratory,                         **
 ** Institute of Informatics and Telecommunications,                       **
 ** N.C.S.R. \"Demokritos\",                                                 **
 ** Athens, Greece.                                                        **
 ** E-mail: petasis@iit.demokritos.gr                                      **
 **                                                                        **
 ** For licensing information, please refer to the license conditions of   **
 ** \"Ellogon\" Text Engineering Platform.                                   **
 **                                                                        **
 ****************************************************************************/
 
#ifndef _CDM_VERSION
#define _CDM_VERSION
"
foreach define {CDM_MAJOR_VERSION CDM_MINOR_VERSION CDM_VERSION CDM_PATCH_LEVEL
                CDM_RELEASE_DATE CDM_RELEASE_DATE_STR} {
  puts $file "#define [format %-25.25s $define] [set $define]"
  puts "##    [format %-25.25s $define] = [set $define]"
}
puts $file "\n#endif /* _CDM_VERSION */"
close $file

puts "##                                                                 ##"
puts "#####################################################################"
exit
