/*****************************************************************************
 *  Cdm_Init.h:                                                              *
 * -----------------------------                                             *
 *  This file initialises the Tcl bindings of Collection & Document          *
 *  Manager (CDM).                                                           *
 *                                                                           *
 *  Copyright 1998-2010 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licensing information, please refer to the license conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/
#include "CDM.h"
#include <string.h>

extern Tcl_Interp *CDM_Interp;
extern char *CDM_DefaultEncoding;
extern "C" DLLEXPORT int Cdm_Init(Tcl_Interp *Interp);

/*
 * Tcl Object Function Prototypes 
 */

#ifdef CDM_MEM_DEBUG
CDM_CALLBACK int CDM_IsSharedObjCmd(CDM_TCL_STD_ARGS);
#endif /* CDM_MEM_DEBUG */
CDM_CALLBACK int CDM_StubCommandObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_FreeTclObjCacheObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     C O L L E C T I O N            S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_AnnotateCollectionObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CloseObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateCollectionObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateDocumentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_DestroyObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_FirstDocumentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAssociatedInfoObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAttributeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAttributesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetByExternalIdObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetCommentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetDocumentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetEncodingObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetOwnerObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_NextDocumentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_OpenCollectionObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_PutAttributeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_RemoveAttributeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_RemoveDocumentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetAssociatedInfoObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetCommentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetEncodingObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetNameObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetOwnerObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SyncObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_StatusObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     D O C U M E N T                S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_AnnotationsAtObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationsContainingObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationsInRangeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationsMatchingRangeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ByteSequenceInsertStringObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ByteSequenceReplaceObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ByteSequenceReplaceCharactersObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateVectorsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_DeleteAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_DisplaceAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_FindMaxUsedAnnotationIdObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_FirstAnnotationContainingObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetByteSequenceObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetParentObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetExternalIdObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetFirstAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetNextAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_NextAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_RemoveAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SelectAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetByteSequenceObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_SetExternalIdObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     A T T R I B U T E S            S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_CreateAttributeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateAttributeValueObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateAttributeSetObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetNameObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetValueObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetValueTypeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetValueValueObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     A N N O T A T I O N S          S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_AnnotationContainsAttributeMatchingValueObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationContainsAttributeMatchingValuesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AddAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationContainsPositionObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AnnotationMatchesRangeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CompareAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateAnnotationSetObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CompareAnnotationSetsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateSpanObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CreateSpanSetObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_DisplaceAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetAnnotatedTextRangesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetEndObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetIdObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetFirstAnnotatedTextRangeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetFirstSpanObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetFirstSpanOffsetsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetSpanOffsetsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetSpansObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetStartObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetTypeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_AddSpanObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_RemoveSpanObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_LengthObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_MergeAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_NthObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |      V A L I D A T I O N           S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/

CDM_CALLBACK int CDM_ValidAnnotationObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ValidAttributeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ValidCollectionObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ValidDocumentObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |      U T I L I T I E S           R O U T I N E S      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
#ifdef CDM_ENABLE_TIME_ROUTINES
CDM_CALLBACK void TclpGetTime(Tcl_Time *timePtr);
CDM_CALLBACK int CDM_CdmGetTimeObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_TimeObjCmd(CDM_TCL_STD_ARGS);
#endif /* CDM_ENABLE_TIME_ROUTINES */
CDM_CALLBACK int CDM_ClearCompressedMemoryCacheObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ClearSavedValuesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ClearUnusedSavedValuesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CompressedMemoryCacheInfoObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_CompressedMemoryCacheStatisticsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetIPObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetCurrentThreadObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_EvalObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ExecObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_ExtractTextBasedOnAnnotationsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_LappendUniqueObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     G U I     R E L A T E D      R O U T I N E S      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_GetCollectionFileXIDsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetCollectionFileIDsObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_GetDocumentAttributesObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_PresentAnnotationForRawViewerObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     S I G N A L   H A N D L E R S                     |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
#ifdef CDM_TRAP_SIGNALS
extern int CDM_TrapObjCmd(CDM_TCL_STD_ARGS);
extern void CDM_InitTrap();
#endif /* CDM_TRAP_SIGNALS */

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     C D M    C O L L E C T I O N   P L U G I N S      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_GetRegisteredCollectionTypesObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     C D M     V E R S I O N      R O U T I N E S      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_VersionObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |     C D M     L I C E N S E      R O U T I N E S      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_LicenseKeyObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_DeclareLicensingInformationObjCmd(CDM_TCL_STD_ARGS);

/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |      T H R E A D    S A F E T Y    S E C T I O N      |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_MutexLockObjCmd(CDM_TCL_STD_ARGS);
CDM_CALLBACK int CDM_MutexUnlockObjCmd(CDM_TCL_STD_ARGS);
/******************************************************************************
 ******************************************************************************
 ****    +-------------------------------------------------------+         ****
 ****    |  P L A T F O R M   U T I L I T I E S   S E C T I O N  |         ****
 ****    +-------------------------------------------------------+         ****
 ******************************************************************************
 *****************************************************************************/
CDM_CALLBACK int CDM_AddLibraryImageObjCmd(CDM_TCL_STD_ARGS);
/*
 * Cdm_Init.h - EOF -
 */
