/*****************************************************************************
 *  EllogonConsolePrintStream.java:                                          *
 * ----------------------------------                                        *
 *  This file implements utilities for redirecting Java console output to    *
 *  Ellogon console                                                          *
 *  This class was designed and implemented by Georgios Petasis,             *
 *  e-mail: petasis@iit.demokritos.gr                                        *
 *  URL:    http://www.iit.demokritos.gr/~petasis                            *
 *                                                                           *
 *  Copyright 1998-2004 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

package Ellogon.JCDM;
import java.lang.*;
import java.io.*;
import tcl.lang.*;

public class OCDM_EllogonConsolePrintStream extends PrintStream {
  Interp  interpreter;
  String  channel;
  boolean JavaPrint;
  public OCDM_EllogonConsolePrintStream(PrintStream out, Interp interp,
                                        String tclChannel) {
    super(out);
    this.interpreter = interp;
    this.channel = tclChannel;
    JavaPrint = false;
  }
  public OCDM_EllogonConsolePrintStream(PrintStream out, Interp interp,
                                        String tclChannel, boolean pfJ) {
    super(out);
    this.interpreter = interp;
    this.channel = tclChannel;
    JavaPrint = pfJ;
  }

  public void write(int b) {
    if (JavaPrint) super.write(b);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + String.valueOf(b)
                                                    + "}");
    } catch (Exception e) {}
  }

  public void write(byte buf[], int off, int len) {
    if (JavaPrint) super.write(buf, off, len);
    if (buf == null) {
      throw new NullPointerException();
    } else if ((off < 0) || (off > buf.length) || (len < 0) ||
               ((off + len) > buf.length) || ((off + len) < 0)) {
      throw new IndexOutOfBoundsException();
    } else if (len == 0) {
      return;
    }
    try {
      String tmp = new String();
      for (int i = 0 ; i < len ; i++) {
        tmp += buf[off + i];
      }
      interpreter.eval("puts -nonewline " + channel + "{" + tmp + "}");
    } catch (Exception e) {
    }
  }
  
  public void print(boolean x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(char x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(int x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(long x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(float x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(double x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(char x[]) {
    String tmp = new String(x);
    if (JavaPrint) super.print(tmp);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + tmp + "}");
    } catch (Exception e) {}
  }
  public void print(String x) {
    if (JavaPrint) super.print(x);
    try {
      interpreter.eval("puts -nonewline " + channel + " {" + x + "}");
    } catch (Exception e) {}
  }
  public void print(Object x) {
    if (JavaPrint) super.print(x.toString());
    try {
      interpreter.eval("puts -nonewline " + channel + " {" +x.toString()+ "}");
    } catch (Exception e) {}
  }
  public void println() {
    if (JavaPrint) super.println();
    try {
      interpreter.eval("puts " + channel + " {}");
    } catch (Exception e) {}
  }
  public void println(boolean x) {
    print(x); println();
  }
  public void println(char x) {
    print(x); println();
  }
  public void println(int x) {
    print(x); println();
  }
  public void println(long x) {
    print(x); println();
  }
  public void println(float x) {
    print(x); println();
  }
  public void println(double x) {
    print(x); println();
  }
  public void println(char x[]) {
    print(x); println();
  }
  public void println(String x) {
    print(x); println();
  }
  public void println(Object x) {
    print(x); println();
  }

  public void flush() {
    if (JavaPrint) super.flush();
    try {
      interpreter.eval("flash " + channel);
    } catch (Exception e) {}
  }
}; /* class OCDM_EllogonConsolePrintStream */

/* Typical Usage:

  PrintStream tcl_stdout = new OCDM_EllogonConsolePrintStream(System.out,
                                                              interp, "stdout");
  PrintStream tcl_stderr = new OCDM_EllogonConsolePrintStream(System.err,
                                                              interp, "stderr");
  System.setOut(tcl_stdout);
  System.setErr(tcl_stderr);
 */
