/*****************************************************************************
 *  OCDM.h:                                                                  *
 * ------------------------------------                                      *
 *  This file contains the definition of the object oriented CDM             *
 *  Public API.                                                              *
 *  CDM is the Collection and Document Manager and the core component of the *
 *  Ellogon Text Engineering Platform.                                       *
 *  CDM was designed and implemented by Georgios Petasis,                    *
 *  e-mail: petasis@iit.demokritos.gr                                        *
 *  URL:    http://www.iit.demokritos.gr/~petasis                            *
 *                                                                           *
 *  Copyright 1998-2010 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifdef SWIG
#undef  CDM_EXPORT
#define CDM_EXPORT
%{
#include "OCDM.h"
%}
// %newobject   operator[];
// %newobject   get;
// %newobject   GetValue;
// %newobject   GetAttribute;
// %newobject   GetAttributes;
// %newobject   GetFirstSpan;
// %newobject   GetSpans;
// %newobject   AnnotationsAt;
// %newobject   AnnotationsContaining;
// %newobject   AnnotationsInRange;
// %newobject   AnnotationsMatchingRange;
// %newobject   FirstAnnotationContaining;
// %newobject   GetAnnotation;
// %newobject   SelectAnnotations;
// %newobject   SelectAnnotationsSorted;
// %newobject   NextAnnotations;
// %newobject   GetAnnotation;
// %newobject   GetFirstAnnotatedTextRange;
// %newobject   GetAnnotatedTextRanges;
// %newobject   ByteSequenceInsertString;
// %newobject   ByteSequenceReplace;
// %newobject   ByteSequenceReplaceCharacters;
// %newobject   GetAnnotations;
// %newobject   GetByteSequence(void);
// %newobject   GetFirstAnnotation;
// %newobject   GetNextAnnotation;
// %newobject   GetParent;
// %newobject   GetRawData(void);
// %newobject   Status;
// %newobject   RawDataReplace;
// %newobject   RawDataInsertString;
// %newobject   RawDataReplaceCharacters;
// %newobject   FirstDocument;
// %newobject   GetByExternalId;
// %newobject   GetDocument;
// %newobject   NextDocument;

%rename(get)      operator[];
%rename(set)      operator=;
%rename(base_get) OCDM_Object::operator[];
%rename(base_set) OCDM_Object::operator=;
%rename(get)      OCDM_ByteSequenceSet::operator[];
%rename(get)      OCDM_RawDataSet::operator[];
%rename(get)      OCDM_SpanSet::operator[];
%rename(get)      OCDM_AttributeSet::operator[];
%rename(get)      OCDM_AnnotationSet::operator[];
%rename(set)      OCDM_ByteSequence::operator=;
%rename(_set)     OCDM_ByteSequenceSet::operator=;
%rename(set)      OCDM_RawData::operator=;
%rename(_set)     OCDM_RawDataSet::operator=;
%rename(set)      OCDM_Span::operator=;
%rename(set)      OCDM_SpanSet::operator=;
%rename(set)      OCDM_AttributeValue::operator=;
%rename(set)      OCDM_Attribute::operator=;
%rename(set)      OCDM_AttributeSet::operator=;
%rename(set)      OCDM_Annotation::operator=;
%rename(set)      OCDM_AnnotationSet::operator=;

// Language independent exception handler
%include exception.i
// STL support...
%include stl.i

%exception {
  try {
    $action
  } catch (OCDM_Exception e) {
    SWIG_exception(SWIG_RuntimeError, e.what());
 /* SWIG_exception(SWIG_ValueError, e.what()); */
  }
}

#ifndef SWIGJAVA
%init {
  CDM_GET_LIBRARY_FUNCTION_POINTERS;
  CDM_Initialise("swig");
}
#endif /* SWIGJAVA */

#endif /* SWIG */

#ifdef SWIGJAVA

#ifdef SWIG

%{
/* When running under Java, OCDM can be accessed by various threads. As a
 * result, we need to serialise all access to OCDM. For this purpose we
 * declare a Mutex... */
TCL_DECLARE_MUTEX(OCDM_Mutex);
%}

%exception {
  try {
    Tcl_MutexLock(&OCDM_Mutex);
    $action
    Tcl_MutexUnlock(&OCDM_Mutex);
  } catch (OCDM_Exception e) {
    SWIG_exception(SWIG_RuntimeError, e.what());
 /* SWIG_exception(SWIG_ValueError, e.what()); */
  }
}

/* Define how destructors will be produced for Java shadow classes */
%typemap(javadestruct, methodname="delete", methodmodifiers="public synchronized") SWIGTYPE {
    if(swigCPtr != 0 && swigCMemOwn) {
      $jnicall;
      swigCMemOwn = false;
    }
    swigCPtr = 0;
  }
%typemap(javadestruct_derived, methodname="delete", methodmodifiers="public synchronized") SWIGTYPE {
    if(swigCPtr != 0 && swigCMemOwn) {
      $jnicall;
      swigCMemOwn = false;
    }
    swigCPtr = 0;
    super.delete();
  }

/*
 * Typemaps for correct handling of temporary object references returned by
 * all functions of OCDM that return a new object.
 */
//%typemap(out) const OCDM_AnnotationSet & %{
//    $1_basetype *newObj = new $1_basetype();
//    newObj = $1;
//    printf("obj = %p -> %p\n", _result_ref, $1);
//    *($&1_ltype)&$result = $1;
//%}

/*
 * Macro to help handle references.
 */
// %define reference(type, name)
// %inline %{
// static type &var_##name = initial_value_##name;
// type setref_##name(type &x) {
//     var_##name = x;
//     return var_##name;
// }
// type& createref_##name(type x) {
//     return *new type(x);
// }
// type value_##name(type &x) {
//     return x;
// }
// %}
// %enddef
// reference(const OCDM_AnnotationSet, OCDM_AnnotationSet);

//%typemap(jni)    long "jlong"
//%typemap(jtype)  long "long"
//%typemap(jstype) long "long"

%{
JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void *reserved) {
  /*
   * It is absolutelly safe to always call CDM_Initialise:
   * If this library is loaded under Ellogon, CDM is already initialised, thus
   * this call has no effect. But if we are called from plain Java, we will
   * initialise CDM...
   */
  CDM_GET_LIBRARY_FUNCTION_POINTERS;
  CDM_Initialise("Java");
  return JNI_VERSION_1_4;
} /* JNI_OnLoad */
%}
#endif /* SWIG */
#endif /* SWIGJAVA */

#ifdef SWIGPERL
#ifdef SWIG
%rename(ByteSequence)    OCDM_ByteSequence;
%rename(ByteSequenceSet) OCDM_ByteSequenceSet;
%rename(RawData)         OCDM_RawData;
%rename(RawDataSet)      OCDM_RawDataSet;
%rename(Span)            OCDM_Span;
%rename(SpanSet)         OCDM_SpanSet;
%rename(AttributeValue)  OCDM_AttributeValue;
%rename(Attribute)       OCDM_Attribute;
%rename(AttributeSet)    OCDM_AttributeSet;
%rename(Annotation)      OCDM_Annotation;
%rename(AnnotationSet)   OCDM_AnnotationSet;
%rename(Document)        OCDM_Document;
%rename(Collection)      OCDM_Collection;

%rename(STRING)          OCDM_STRING;
%rename(STRING_SET)      OCDM_STRING_SET;
%rename(IMAGE)           OCDM_IMAGE;
%rename(BASE64_IMAGE)    OCDM_BASE64_IMAGE;
%rename(OK)              OCDM_OK;
%rename(ERROR)           OCDM_ERROR;
%rename(RETURN)          OCDM_RETURN;
%rename(BREAK)           OCDM_BREAK;
%rename(CONTINUE)        OCDM_CONTINUE;

#endif
#endif

#ifdef SWIGPYTHON
#ifdef SWIG
%rename(execute) exec;
%rename(ByteSequence)    OCDM_ByteSequence;
%rename(ByteSequenceSet) OCDM_ByteSequenceSet;
%rename(RawData)         OCDM_RawData;
%rename(RawDataSet)      OCDM_RawDataSet;
%rename(Span)            OCDM_Span;
%rename(SpanSet)         OCDM_SpanSet;
%rename(AttributeValue)  OCDM_AttributeValue;
%rename(Attribute)       OCDM_Attribute;
%rename(AttributeSet)    OCDM_AttributeSet;
%rename(Annotation)      OCDM_Annotation;
%rename(AnnotationSet)   OCDM_AnnotationSet;
%rename(Document)        OCDM_Document;
%rename(Collection)      OCDM_Collection;

%rename(STRING)          OCDM_STRING;
%rename(STRING_SET)      OCDM_STRING_SET;
%rename(IMAGE)           OCDM_IMAGE;
%rename(BASE64_IMAGE)    OCDM_BASE64_IMAGE;
%rename(OK)              OCDM_OK;
%rename(ERROR)           OCDM_ERROR;
%rename(RETURN)          OCDM_RETURN;
%rename(BREAK)           OCDM_BREAK;
%rename(CONTINUE)        OCDM_CONTINUE;

#endif
#endif

#ifndef _OCDM_H_
#define _OCDM_H_
#define OCDM_PRESENT
#define OCDM_BOOL int
#define OCDM_REF(type) const class type
#define OCDM_INDEX_OPERATOR_REF(type) const class type

#ifdef LOG_CONSTRUCTOR_CALLS
#ifdef LOG_THREAD_INFO
#define OCDM_COMMON_CONSTRUCTOR_CODE \
  className = objectType();\
  CDM_Log(" %s(): this: %p, cdmObj: %p, count: %d, " \
          "storedObj: %p\n", className, this, getObject(), \
          (getObject() == NULL)? -100 : getObject()->refCount, \
          getStoredObject()); \
  CDM_Log("CurrentThread Id = %p\n", Tcl_GetCurrentThread());
#else  /* LOG_THREAD_INFO */
  #define OCDM_COMMON_CONSTRUCTOR_CODE \
  className = objectType();\
  className = objectType();\
  CDM_Log(" %s(): this: %p, cdmObj: %p, count: %d, " \
          "storedObj: %p\n", className, this, getObject(), \
          (getObject() == NULL)? -100 : getObject()->refCount, \
          getStoredObject());
#endif /* LOG_THREAD_INFO */
#else  /* LOG_CONSTRUCTOR_CALLS */
#ifdef LOG_DESTRUCTOR_CALLS 
#define OCDM_COMMON_CONSTRUCTOR_CODE className = objectType();
#else  /* LOG_DESTRUCTOR_CALLS */
#define OCDM_COMMON_CONSTRUCTOR_CODE
#endif /* LOG_DESTRUCTOR_CALLS */
#endif /* LOG_CONSTRUCTOR_CALLS */

#if defined(LOG_DESTRUCTOR_CALLS) && defined(LOG_THREAD_INFO)
#define OCDM_COMMON_DESTRUCTOR_CODE \
  CDM_Log("CurrentThread Id = %p\n", Tcl_GetCurrentThread());
#else  /* LOG_DESTRUCTOR_CALLS */
#define OCDM_COMMON_DESTRUCTOR_CODE
#endif /* LOG_DESTRUCTOR_CALLS */

#include <exception>
#include <string>
#include <vector>
#include "CDM.h"

//class CDM_EXPORT OCDM_Object;
//class CDM_EXPORT OCDM_ByteSequence;
//class CDM_EXPORT OCDM_ByteSequenceSet;
//class CDM_EXPORT OCDM_RawData;
//class CDM_EXPORT OCDM_RawDataSet;
//class CDM_EXPORT OCDM_Span;
//class CDM_EXPORT OCDM_SpanSet;
//class CDM_EXPORT OCDM_AttributeValue;
//class CDM_EXPORT OCDM_Attribute;
//class CDM_EXPORT OCDM_AttributeSet;
//class CDM_EXPORT OCDM_Annotation;
//class CDM_EXPORT OCDM_AnnotationSet;
//class CDM_EXPORT OCDM_Document;
//class CDM_EXPORT OCDM_Collection;

/*
 * CDM Defined Constants
 */
#define OCDM_STRING         1
#define OCDM_STRING_SET     2
#define OCDM_IMAGE         10
#define OCDM_BASE64_IMAGE  11
#define OCDM_OK             0
#define OCDM_ERROR          1
#define OCDM_RETURN         2
#define OCDM_BREAK          3
#define OCDM_CONTINUE       4

#ifndef SWIG
/*
 * Class OCDM_Exception:
 * This is the class that will be thrown by OCDM when an error occurs.
 * The caller that catches this exception can call "what" to get a string
 * describing the error.
 */
class CDM_EXPORT OCDM_Exception /* : public std::exception */ {
  private:
    Tcl_Obj *errorObj;
  public:
    OCDM_Exception(const class OCDM_Exception& obj);
    class OCDM_Exception& operator= (const class OCDM_Exception& obj);
    OCDM_Exception() throw();
    OCDM_Exception(const char *msg) throw();
    ~OCDM_Exception() throw();
    const char *what() const throw();
}; /* class OCDM_Exception */
#endif /* SWIG */

/**
 * \brief The base class of all OCDM objects.
 * \par Description:
 * This class is the base class for all OCDM objects, as all OCDM objects
 * inherit this class. It provides all the common functionality offered
 * by all OCDM objects.
 */
class CDM_EXPORT OCDM_Object {
  private:
    mutable CDM_Object cdmObj;
    mutable const class OCDM_Object *storedObj;
  protected:
#if defined (LOG_CONSTRUCTOR_CALLS) || defined (LOG_DESTRUCTOR_CALLS) || \
    defined (LOG_CDM_OBJECT)        || defined (LOG_OCDM_TOSTRING)
    mutable const char *className;
#endif /* LOG_CONSTRUCTOR_CALLS || LOG_DESTRUCTOR_CALLS */
  public:
    void ensureValidity(void) const {
      if (cdmObj == NULL) throw OCDM_Exception("Uninitialised object!");
    }
    /* Constructors */
    OCDM_Object();
#ifndef SWIG
    OCDM_Object(const CDM_Object);
#endif /* SWIG */
    OCDM_Object(const class OCDM_Object&);
    /* Destructor */
    ~OCDM_Object();
    /* Operators */
    virtual class OCDM_Object& operator=  (const class OCDM_Object&);
#ifdef ENABLE_INDEX_OPERATOR_ON_ALL_OBJECTS
    virtual OCDM_INDEX_OPERATOR_REF(OCDM_Object) operator[] (const long index)
                                                                          const;
#endif /* ENABLE_INDEX_OPERATOR_ON_ALL_OBJECTS */
#ifndef SWIG
    /* Private Helper functions */
    void                     initObject(const CDM_Object) const;
    void                     setObject(const CDM_Object) const;
    const CDM_Object         getObject(void) const;
    void                     releaseObject(void) const;
    void                     deleteObject(void) const;
    void                     storeObject(const class OCDM_Object *objPtr) const;
    const class OCDM_Object *getStoredObject(void) const;
    void                     releaseStoredObject(void) const;
#endif /* SWIG */
    /* Public  Helper functions */
    void                     Log(const char *str, ...) const;
    long                     size(void)       const;
    bool                     empty(void)      const {return (size()==0);};
    bool                     isEmpty(void)    const {return (size()==0);};
    OCDM_BOOL                Valid(void)      const;
            const char      *toString(void)   const;
    virtual const char      *objectType(void) const;
    /* Friend class that can access private memebers */
#ifdef    SWIGPYTHON
    /* The following definitions provide a better integration of our objects
     * with Python. */
    long __len__()         {return size();};
    bool __nonzero__()     {return (size()!=0);};
    const char *__str__()  {return toString();};
    const char *__repr__() {return toString();};
#ifdef    SWIG
    /* Support for Python special methods.
     * __iter__ allows code like 'for item in obj:'
     * __getitem__ allows code like 'obj[index]'
     * Code for these methods contributed by
     * Steven Bethard <steven.bethard@colorado.edu>. */
%pythoncode %{
    def __getitem__(self, index):
        if not (0 <= index < len(self)):
            raise IndexError(index)
        return self.get(index)
    def __iter__(self):
        for i in range(len(self)): 
            yield self[i]
    def __unicode__(self):
         return str(self).decode('utf-8')
%}
#endif /* SWIG */
#endif /* SWIGPYTHON */
}; /* class OCDM_Object */

#if 0
/**
 * \brief The base class of all OCDM string/text objects.
 * \par Description:
 * This class is the base class for all OCDM string/text objects,
 * as all OCDM string/text objects inherit this class.
 * It provides all the common functionality offered
 * by all OCDM string/text objects.
 */
class CDM_EXPORT OCDM_StringObject : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_StringObject() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_StringObject(const CDM_Object obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_StringObject(const class OCDM_StringObject& obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
    /* Operators */
    class OCDM_StringObject& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };

    /* Public functions: we try to implement as much as possible form the STL
     * string container... */

    /**
     * Return the number of characters in the string.
     */
    long size(void) const;

    /**
     * Return the number of characters in the string.
     */
    long length(void) const;

    /**
     * Character reference at index i.
     */
    const char *operator[] (const long index) const;
    operator+=
    /**
     * Character reference at index i.
     */
    const char *at(const long index) const;
    const char *c_str() const;
    OCDM_BOOL toUpper(void) const;
    OCDM_BOOL toLower(void) const;
    OCDM_BOOL toTitle(void) const;
    const char *objectType(void) const {return "OCDM_StringObject";}
    
    /* Friend class that can access private members */
}; /* class OCDM_StringObject */
#endif

#if 0
/**
 * \brief The base class of all OCDM set objects.
 * \par Description:
 * This class is the base class for all OCDM set objects, as all OCDM set
 * objects inherit this class. It provides all the common functionality offered
 * by all OCDM set objects.
 */
class CDM_EXPORT OCDM_ObjectSet : public OCDM_Object {
  private:
  protected:
  public:
    /* Constructors */
    OCDM_ObjectSet();
#ifndef SWIG
    OCDM_ObjectSet(const CDM_ObjectSet);
#endif /* SWIG */
    OCDM_ObjectSet(const class OCDM_ObjectSet&);
    /* Destructor */
    ~OCDM_ObjectSet();
    /* Operators */
#ifdef ENABLE_INDEX_OPERATOR_ON_ALL_OBJECTS
    virtual OCDM_INDEX_OPERATOR_REF(OCDM_Object) operator[] (const long index)
                                                                          const;
#endif /* ENABLE_INDEX_OPERATOR_ON_ALL_OBJECTS */

    /* Set manipulation operators */
    void add(const OCDM_Object&);
    void add(const char *);
    void push_back(const OCDM_Object&);
    void push_back(const char *);
    /* Get manipulation operators */
}; /* class OCDM_ObjectSet */
#endif

/**
 * \brief Class for representing textual (UTF-8) information. This is main
 * class for representing text in \b Ellogon.
 * \par Description:
 * Class for representing textual (UTF-8) information. This is main
 * class for representing text in \b Ellogon.
 * \note This class is identical to OCDM_RawData.
 */
class CDM_EXPORT OCDM_ByteSequence : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_ByteSequence() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_ByteSequence(const CDM_Object obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_ByteSequence(const class OCDM_ByteSequence& obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_ByteSequence(const char *str);
    OCDM_ByteSequence(const char *str, const int len);
    /* Destructor */
    //~OCDM_ByteSequence() {deleteObject();}
    /* Operators */
    class OCDM_ByteSequence& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    /* Public  Helper functions */
    void      Set(const char *str);
    void      Set(const char *str, const int len);
    long      length(void) const;
    OCDM_BOOL toUpper(void) const;
    OCDM_BOOL toLower(void) const;
    OCDM_BOOL toTitle(void) const;
    const char *objectType(void) const {return "OCDM_ByteSequence";}
}; /* class OCDM_ByteSequence */

/**
 * \brief Class for representing a set of UTF-8 strings.
 * \note This class is identical to OCDM_RawDataSet.
 */
class CDM_EXPORT OCDM_ByteSequenceSet : public OCDM_ByteSequence {
  public:
    /* Constructors */
    OCDM_ByteSequenceSet() : OCDM_ByteSequence() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_ByteSequenceSet(const CDM_Object obj) : OCDM_ByteSequence(obj)
                                                 {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_ByteSequenceSet(const class OCDM_ByteSequenceSet& obj) :
                          OCDM_ByteSequence(obj) {OCDM_COMMON_CONSTRUCTOR_CODE};
    /* Destructor */
    //~OCDM_ByteSequenceSet() {deleteObject();}
    /* Operators */
    class OCDM_ByteSequenceSet& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    OCDM_INDEX_OPERATOR_REF(OCDM_ByteSequence) operator[] (const long index)
                                                                          const;
    /* Set manipulation operators */
    void add(const OCDM_ByteSequence&);
    void add(const char *);
    void push_back(const OCDM_ByteSequence& obj) {add(obj);};
    void push_back(const char *obj)              {add(obj);};
    /* Get manipulation operators */
    
    /* Public  Helper functions */
    const char *objectType(void) const {return "OCDM_ByteSequenceSet";}
}; /* class OCDM_ByteSequenceSet */

/**
 * \brief Class for representing textual (UTF-8) information. This is main
 * class for representing text in \b Ellogon.
 * \par Description:
 * Class for representing textual (UTF-8) information. This is main
 * class for representing text in \b Ellogon.
 * \note This class is identical to OCDM_ByteSequence.
 */
class CDM_EXPORT OCDM_RawData : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_RawData() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_RawData(const CDM_Object obj) : OCDM_Object(obj)
                                   {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_RawData(const class OCDM_RawData& obj) : OCDM_Object(obj)
                                   {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_RawData(const char *str);
    OCDM_RawData(const char *str, const int len);
    /* Destructor */
    //~OCDM_RawData() {deleteObject();}
    /* Operators */
    class OCDM_RawData& operator= (const class OCDM_Object& obj) {
      return static_cast<OCDM_RawData&>(OCDM_Object::operator= (obj));
    };
    /* Public  Helper functions */
    void      Set(const char *str);
    void      Set(const char *str, const int len);
    long      length(void) const;
    OCDM_BOOL toUpper(void) const;
    OCDM_BOOL toLower(void) const;
    OCDM_BOOL toTitle(void) const;
    const char *objectType(void) const {return "OCDM_RawData";}
}; /* class OCDM_RawData */

/**
 * \brief Class for representing a set of UTF-8 strings.
 * \note This class is identical to OCDM_ByteSequenceSet.
 */
class CDM_EXPORT OCDM_RawDataSet : public OCDM_RawData {
  public:
    /* Constructors */
    OCDM_RawDataSet() : OCDM_RawData() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_RawDataSet(const CDM_Object obj) : OCDM_RawData(obj)
                                       {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_RawDataSet(const class OCDM_RawDataSet& obj) : OCDM_RawData(obj)
                                       {OCDM_COMMON_CONSTRUCTOR_CODE};
    /* Destructor */
    //~OCDM_RawDataSet() {deleteObject();}
    /* Operators */
    class OCDM_RawDataSet& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    OCDM_INDEX_OPERATOR_REF(OCDM_RawData) operator[] (const long index) const;
    /* Set manipulation operators */
    void add(const OCDM_RawData&);
    void add(const char *);
    void push_back(const OCDM_RawData& obj) {add(obj);};
    void push_back(const char *obj)         {add(obj);};
    /* Public  Helper functions */
    const char *objectType(void) const {return "OCDM_RawDataSet";}
}; /* class OCDM_RawDataSet */

/**
*  This class incorporates every function related to the use of Spans 
*
*/
class CDM_EXPORT OCDM_Span : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_Span() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_Span(const CDM_Object obj) : OCDM_Object(obj)
                                {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_Span(const class OCDM_Span& obj) : OCDM_Object(obj)
                                {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_Span(const long start, const long end);
    /* Destructor */
    //~OCDM_Span() {deleteObject();}
    /* Operators */
    class OCDM_Span& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    /* Public  Helper functions */
    long GetStart(void) const;
    long GetEnd(void) const;
#ifndef SWIG
    int GetSpanOffsets(long &start, long &end) const;
#endif /* SWIG */
    const char *objectType(void) const {return "OCDM_Span";}
}; /* class OCDM_Span */

/**
*  \brief This class handles the use of a set of spans.
*  \par Description:
*  This class handles the use of a set of spans.
*  It can be used to create sets of spans,
*  to append a span into a set, etc...
*
*/
class CDM_EXPORT OCDM_SpanSet : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_SpanSet() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_SpanSet(const CDM_Object obj) : OCDM_Object(obj)
                                   {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */ 
    OCDM_SpanSet(const class OCDM_SpanSet& obj) : OCDM_Object(obj)
                                   {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_SpanSet(const long start, const long end);
    /* Destructor */
    //~OCDM_SpanSet() {deleteObject();}
    /* Operators */
    class OCDM_SpanSet& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    OCDM_INDEX_OPERATOR_REF(OCDM_Span) operator[] (const long index) const;
    /* Public  Helper functions */
    void AddSpan(const OCDM_Span& Span);
    const char *objectType(void) const {return "OCDM_SpanSet";}
}; /* class OCDM_SpanSet */

/**
*  \brief This class will be used to set an Attribute value to an Attribute 
*
*/
class CDM_EXPORT OCDM_AttributeValue : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_AttributeValue() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_AttributeValue(const CDM_Object obj) : OCDM_Object(obj)
                                          {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_AttributeValue(const class OCDM_AttributeValue& obj) :
                                OCDM_Object(obj) {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_AttributeValue(const int Type, const char *Value);
    /* Destructor */
    //~OCDM_AttributeValue() {deleteObject();}
    /* Operators */
    class OCDM_AttributeValue& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    /* Public  Helper functions */
    const char *objectType(void) const {return "OCDM_AttributeValue";}
}; /* class OCDM_AttributeValue */

/**
*  This class is related to the use and manipulation of Attributes.
*
*/
class CDM_EXPORT OCDM_Attribute : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_Attribute() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_Attribute(const CDM_Object obj) : OCDM_Object(obj)
                                     {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_Attribute(const class OCDM_Attribute& obj) : OCDM_Object(obj)
                                     {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_Attribute(const char *name, const OCDM_AttributeValue& AttrVal);
    OCDM_Attribute(const char *name, const int Type, const char *Value);
    /* Destructor */
    //~OCDM_Attribute() {deleteObject();}
    /* Operators */
    class OCDM_Attribute& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    /* Public  Helper functions */
    const char                   *GetName(void) const;
    const char                   *GetValueString(void) const;
    OCDM_REF(OCDM_AttributeValue) GetValue(void) const;
    int                           GetValueType(void) const;
    const char                   *GetValueValue(void) const;
    void                          ReconstructAttribute(void) const;
#ifndef SWIG
    void                          ReconstructAttribute(int* type) const;
#endif /* SWIG */
    const char *objectType(void) const {return "OCDM_Attribute";}
}; /* class OCDM_Attribute */

/**
*  \brief OCDM_AttributeSet class incorporates every function related
*  to the use and
*  manipulation of group/sets of Attributes. 
*
*/
class CDM_EXPORT OCDM_AttributeSet : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_AttributeSet() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_AttributeSet(const CDM_Object obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_AttributeSet(const class OCDM_AttributeSet& obj) : OCDM_Object(obj)
                                        {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_AttributeSet(const char *name, const OCDM_AttributeValue& AttrVal);
    /* Destructor */
    //~OCDM_AttributeSet() {deleteObject();}
    /* Operators */
    class OCDM_AttributeSet& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    OCDM_INDEX_OPERATOR_REF(OCDM_Attribute) operator[] (const long index) const;
    /* Public  Helper functions */
    void                        AddAttribute(const OCDM_Attribute& Attr);
    OCDM_BOOL                   AttributeExists(const char *name) const;
    OCDM_REF(OCDM_Attribute)    GetAttribute(const char *name) const;
    int                         RemoveAttribute(const char *name);
    const char *objectType(void) const {return "OCDM_AttributeSet";}
#ifdef    SWIGPYTHON
    /* The following definitions provide a better integration of our objects
     * with Python. */
#ifdef    SWIG
    /* Support for Python special methods.
     * __getitem__ allows code like 'obj[index]' or 'obj[attr_name]'
     * __setitem__ allows code like 'obj[attr_name] = AttributeValue(...)'
     * __contains__ allows code like 'attr_name in obj'
     * Code for these methods contributed by
     * Steven Bethard <steven.bethard@colorado.edu>. */
%pythoncode %{
    def __getitem__(self, item): 
        if isinstance(item, int):
            if not (0 <= item < len(self)):
                raise IndexError(item)
            return self.get(item)
        else: 
            return self.GetAttribute(item)
    def __setitem__(self, attr_name, attr_value): 
        self.AddAttribute(Attribute(attr_name, attr_value))
    def __contains__(self, attr_name):
        return self.AttributeExists(attr_name)
%}
#endif /* SWIG */
#endif /* SWIGPYTHON */
}; /* class OCDM_AttributeSet */

/**
*  \brief OCDM_Annotation class gathers together every function having to do 
*  with the use and manipulation of an Annotation. 
*
*/
class CDM_EXPORT OCDM_Annotation : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_Annotation() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_Annotation(const CDM_Object obj) : OCDM_Object(obj)
                                      {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_Annotation(const class OCDM_Annotation& obj) : OCDM_Object(obj)
                                      {OCDM_COMMON_CONSTRUCTOR_CODE};
    OCDM_Annotation(const char* type, const OCDM_SpanSet& Spans,
                                const OCDM_AttributeSet& Attributes);
    OCDM_Annotation(const char* type, const long start, const long end,
                                const OCDM_AttributeSet& Attributes);
    /* Destructor */
    //~OCDM_Annotation() {deleteObject();}
    /* Operators */
    class OCDM_Annotation& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    /* Public  Helper functions */
    OCDM_BOOL AnnotationContainsPosition(const long Position) const;
    OCDM_BOOL AnnotationMatchesRange(const long Start, const long End) const;
    OCDM_BOOL AttributeExists(const char *name) const;
    int       CompareAnnotations(const OCDM_Annotation &Ann) const;
    void      DisplaceAnnotation(const long offset, const long displacement);
    OCDM_REF(OCDM_Attribute)    GetAttribute(const char *name) const;
    OCDM_REF(OCDM_AttributeSet) GetAttributes(void) const;
    OCDM_REF(OCDM_Span)         GetFirstSpan(void) const;
#ifndef SWIG
    int       GetFirstSpanOffsets(long &start, long &end) const;
#endif /* SWIG */
    OCDM_REF(OCDM_SpanSet)      GetSpans(void) const;
    void                        PutAttribute(const OCDM_Attribute &Attr);
    const char                 *GetType(void) const;
    void                        ReconstructAnnotation(void) const;
    void                        RemoveAttribute(const char *name);
    void                        AddSpan(const OCDM_Span &span);
    void                        RemoveSpan(const long start, const long end);
    long                        GetId(void) const;
    const char *objectType(void) const {return "OCDM_Annotation";}
#ifdef    SWIGPYTHON
    /* The following definitions provide a better integration of our objects
     * with Python. */
#ifdef    SWIG
    /* Support for Python special methods.
     * __getitem__ allows code like 'obj[index]' or 'obj[attr_name]'
     * __setitem__ allows code like 'obj[attr_name] = AttributeValue(...)'
     * __contains__ allows code like 'attr_name in obj'
     * Code for these methods contributed by
     * Steven Bethard <steven.bethard@colorado.edu>. */
%pythoncode %{
    def __getitem__(self, item): 
        if isinstance(item, int):
            if not (0 <= item < len(self)):
                raise IndexError(item)
            return self.get(item) 
        else: 
            return self.GetAttribute(item)
    def __setitem__(self, attr_name, attr_value): 
        self.PutAttribute(Attribute(attr_name, attr_value))
    def __contains__(self, attr_name):
        return attr_name in self.GetAttributes()
%}
#endif /* SWIG */
#endif /* SWIGPYTHON */
}; /* class OCDM_Annotation */

/**
*  \brief As the name implies this class incorporates every function
*  related to  
*  the manipulation of Annotation sets. 
*
*/
class CDM_EXPORT OCDM_AnnotationSet : public OCDM_Object {
  public:
    /* Constructors */
    OCDM_AnnotationSet() : OCDM_Object() {OCDM_COMMON_CONSTRUCTOR_CODE};
#ifndef SWIG
    OCDM_AnnotationSet(const CDM_Object obj) : OCDM_Object(obj)
                                         {OCDM_COMMON_CONSTRUCTOR_CODE};
#endif /* SWIG */
    OCDM_AnnotationSet(const class OCDM_AnnotationSet& obj) :
                                OCDM_Object(obj) {OCDM_COMMON_CONSTRUCTOR_CODE};
    /* Destructor */
    //~OCDM_AnnotationSet() {deleteObject();}
    /* Operators */
    class OCDM_AnnotationSet& operator= (const class OCDM_Object& obj) {
      OCDM_Object::operator= (obj); return *this;
    };
    OCDM_INDEX_OPERATOR_REF(OCDM_Annotation) operator[] (const long index) const;
    /* Public  Helper functions */
    void AddAnnotation(const OCDM_Annotation& Ann);
    int  CompareAnnotations(const OCDM_Annotation &Ann1,
                            const OCDM_Annotation &Ann2) const;
    void                         MergeAnnotations(OCDM_AnnotationSet &AnnSet);
    OCDM_REF(OCDM_AnnotationSet) AnnotationsAt(const long position) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsContaining(const long position)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsContaining(const long position1,
                                              const long position2) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsInRange(const long Start,
                                                    const long End) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsInRange(const OCDM_Annotation& Ann)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsMatchingRange(long Start, long End)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsMatchingRange(const 
                                                    OCDM_Annotation& Ann) const;
    OCDM_REF(OCDM_Annotation)    FirstAnnotationContaining(const long Position)
                                                                          const;
    OCDM_REF(OCDM_Annotation)    FirstAnnotationContaining(const long Position1,
                                                    const long Position2) const;
    OCDM_REF(OCDM_Annotation)    GetAnnotation(const long Id) const;
    void                         RemoveAnnotation(const long Id);
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotations(const char* Type) const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotations(const char* Type,
                                                 const char* Constraints) const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotationsSorted(const char* Type)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotationsSorted(const char* Type,
                                                 const char* Constraints) const;
    void                         SortAnnotationSet(void);
    void                         DisplaceAnnotations(const long offset,
                                                     const long displacement);
    OCDM_REF(OCDM_AnnotationSet) NextAnnotations(const long Position) const;
    const char *objectType(void) const {return "OCDM_AnnotationSet";}
}; /* class OCDM_AnnotationSet */

/** 
*   \brief The class OCDM_Document incorporates every function related to the
*   manipulation of the documents of a collection. Bare in mind that an 
*   Ellogon Document does not consist merely of text. It consists of 
*   textual data as well as linguistic information about the textual data.
*/
class CDM_EXPORT OCDM_Document {
  private:
    CDM_Document Document;
    mutable const class OCDM_Object *storedObj;
    mutable const class OCDM_Collection *storedCollectionObj;

    int          CheckStatus(const int status) const;
    void         ensureValidity(void) const;
    const char  *ReturnString(const char *str) const;
  public:
    /* Constructors */
    OCDM_Document();
    OCDM_Document(const char *TclCmdName);
    OCDM_Document(const class OCDM_Document& obj);
#ifndef SWIG
    OCDM_Document(const Tcl_Obj *TclCmdName);
    OCDM_Document(const CDM_Document);
#endif
    /* Destructor */
    ~OCDM_Document();
    /* Close */
    void Close(void);
    /* Operators */
    class OCDM_Document& operator= (const class OCDM_Document& obj);
    /* Private Helper functions */
    void                     storeObject(const class OCDM_Object *objPtr) const;
    void                     storeObject(const class OCDM_Collection *objPtr)
                                                                          const;
    const class OCDM_Object *getStoredObject(void) const;
    void                     releaseStoredObject(void) const;
    /* Public  Helper functions */
    long                         AddAnnotation(const OCDM_Annotation &Ann);
    OCDM_REF(OCDM_AnnotationSet) AnnotationsAt(const long position) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsContaining(const long position)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsContaining(const long position1,
                                                    const long position2) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsInRange(const long Start,
                                                    const long End) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsInRange(const OCDM_Annotation& Ann)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsMatchingRange(const long Start,
                                                          const long End) const;
    OCDM_REF(OCDM_AnnotationSet) AnnotationsMatchingRange(const 
                                                    OCDM_Annotation& Ann) const;
    OCDM_BOOL                    AttributeExists(const char *name) const;
    int                          DisplaceAnnotations(long offset,
                                                     long displacement);
    OCDM_REF(OCDM_Annotation)    FirstAnnotationContaining(const long Position)
                                                                          const;
    OCDM_REF(OCDM_Annotation)    FirstAnnotationContaining(const long Position1,
                                                    const long Position2) const;
    OCDM_REF(OCDM_RawDataSet)    GetAnnotatedTextRanges(const 
                                                    OCDM_Annotation& Ann) const;
    OCDM_REF(OCDM_Annotation)    GetAnnotation(const long Id) const;
    OCDM_REF(OCDM_Attribute)     GetAttribute(const char *name) const;
    OCDM_REF(OCDM_AttributeSet)  GetAttributes(void) const;
    OCDM_REF(OCDM_ByteSequence)  GetFirstAnnotatedTextRange(
                                              const OCDM_Annotation& Ann) const;
    OCDM_REF(OCDM_AnnotationSet) NextAnnotations(const long Position) const;
    int                          PutAttribute(const OCDM_Attribute &Attr);
    int                          RemoveAnnotation(const long Id);
    const char                  *GetId(void) const;
    int                          RemoveAttribute(const char *name);
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotations(const char* Type) const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotations(const char* Type,
                                                 const char* Constraints) const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotationsSorted(const char* Type)
                                                                          const;
    OCDM_REF(OCDM_AnnotationSet) SelectAnnotationsSorted(const char* Type,
                                                 const char* Constraints) const;
    OCDM_REF(OCDM_ByteSequence)  ByteSequenceInsertString(const long pos,
                                                          const char *string);
    OCDM_REF(OCDM_ByteSequence)  ByteSequenceReplace(const long first,
                                        const long last, const char *newstring);
    OCDM_REF(OCDM_ByteSequence)  ByteSequenceReplaceCharacters(const long first,
                                           const long last, const char *string);
    int                          DeleteAnnotations(const char *Type);
    int                          DeleteAnnotations(const char *Type,
                                                   const char *Constraints);
    long                         FindMaxUsedAnnotationId(void) const;
    OCDM_REF(OCDM_AnnotationSet) GetAnnotations(void) const;
    OCDM_REF(OCDM_ByteSequence)  GetByteSequence(void) const;
    const char                  *GetByteSequence(const char *encoding) const;
    const char                  *GetEncoding(void) const;
    const char                  *GetExternalId(void) const;
    OCDM_REF(OCDM_Annotation)    GetFirstAnnotation(const char *Type) const;
    OCDM_REF(OCDM_Annotation)    GetNextAnnotation(void) const;
    OCDM_REF(OCDM_Collection)    GetParent(void) const;
    OCDM_REF(OCDM_RawData)       GetRawData(void) const;
    const char                  *GetRawData(const char *encoding) const;
    int                          SetByteSequence(const OCDM_ByteSequence &Text);
    void                         SetProcessStatus(const int value) const;
    const char                  *SetEncoding(const char *encoding);
    const char                  *SetExternalId(const char *ExternalId);
    OCDM_REF(OCDM_ByteSequence)  Status(void) const;
    int                          Sync(void) const;
    OCDM_REF(OCDM_RawData)       RawDataReplace(const long first,
                                        const long last, const char *newstring);
    OCDM_REF(OCDM_RawData)       RawDataInsertString(const long pos,
                                                     const char *string);
    OCDM_REF(OCDM_RawData)       RawDataReplaceCharacters(const long first,
                                      const int characters, const char *string);
    void                         Log(const char *str, ...) const;
    OCDM_BOOL                    Valid(void) const;
    const char                  *toString(void) const;
    const char *objectType(void) const {return "OCDM_Document";}
#ifdef    SWIGPYTHON
    /* The following definitions provide a better integration of our objects
     * with Python. */
    const char *__str__()  const {return toString();};
    const char *__repr__() const {return toString();};
#ifdef    SWIG
    /* Support for Python special methods.
     * __getitem__ allows code like 'obj[attr_name]'
     * __setitem__ allows code like 'obj[attr_name] = AttributeValue(...)'
     * __contains__ allows code like 'attr_name in obj'
     * Code for these methods contributed by
     * Steven Bethard <steven.bethard@colorado.edu>. */
%pythoncode %{
    def __getitem__(self, attr_name): 
        return self.GetAttribute(attr_name)
    def __setitem__(self, attr_name, attr_value): 
        self.PutAttribute(Attribute(attr_name, attr_value))
    def __contains__(self, attr_name):
        return attr_name in self.GetAttributes()
%}
#endif /* SWIG */
#endif /* SWIGPYTHON */
    friend class OCDM_Macros;
}; /* class OCDM_Document */

/**
*  \brief The OCDM_Collection class incorporates every function related to the
*  manipulation of the collections. Before start reading the documentation 
*  of the function, you should recollect that the collection is the central
*  element of storing data in Ellogon is the Collection. 
*  A Collection is nothing  more than a finite set of Documents. 
*  In other words, the Collection can be thought of as the corpus we want
*  to process.
*  Then the Documents of that Collection represent the actual
*  documents of the corpus, 
*  bundled together.
* 
*/
class CDM_EXPORT OCDM_Collection {
  private:
    CDM_Collection Collection;
    mutable const class OCDM_Object   *storedObj;
    mutable const class OCDM_Document *storedDocumentObj;

    int          CheckStatus(const int status) const;
    void         ensureValidity(void) const;
    const char  *ReturnString(const char *str) const;
  public:
    /* Constructors */
    OCDM_Collection();
    OCDM_Collection(const char *name);
    OCDM_Collection(const class OCDM_Collection& obj);
    OCDM_Collection(const char* name, const OCDM_AttributeSet &AttrSet,
                    const char* encoding);
#ifndef SWIG
    OCDM_Collection(const CDM_Collection);
#endif
    /* Destructor */
    ~OCDM_Collection();
    /* Operators */
    class OCDM_Collection& operator= (const class OCDM_Collection& obj);
    /* Private Helper functions */
    void                     storeObject(const class OCDM_Object *objPtr) const;
    void                     storeObject(const class OCDM_Document *objPtr)
                                                                          const;
    const class OCDM_Object *getStoredObject(void) const;
    void                     releaseStoredObject(void) const;
    /* Public  Helper functions */
    OCDM_BOOL                   AttributeExists(const char *name) const;
    void CreateDocument(const char *XID, const OCDM_ByteSequence &RawData,
                        const OCDM_AnnotationSet &Annotations,
                        const OCDM_AttributeSet  &Attributes,
                        const char *encoding);
    void CreateDocument(const char *XID, const OCDM_ByteSequence &RawData,
                        const OCDM_AnnotationSet &Annotations,
                        const OCDM_AttributeSet &Attributes);
    int                         Length(void) const;
    OCDM_REF(OCDM_Attribute)    GetAttribute(const char *name) const;
    OCDM_REF(OCDM_AttributeSet) GetAttributes(void) const;
    int                         PutAttribute(const OCDM_Attribute &Attr);
    const char                 *GetName(void) const;
    int                         RemoveAttribute(const char *name);
    const char                 *GetEncoding(void) const;
    const char                 *SetEncoding(const char *encoding);
    OCDM_REF(OCDM_ByteSequence) Status(void) const;
    int                         Sync(void) const;
    int                         AnnotateColection(void);
    int                         Destroy(const char *name);
    OCDM_REF(OCDM_Document)     FirstDocument(void) const;
    OCDM_REF(OCDM_Document)     GetByExternalId(const char *XID) const;
    OCDM_REF(OCDM_Document)     GetDocument(const char *ID) const;
    OCDM_REF(OCDM_Document)     NextDocument(void) const;
    const char                 *GetOwner(void) const;
    int                         RemoveDocument(const char *Id);
    const char                 *SetName(const char *Name);
    const char                 *SetOwner(const char *Owner);
    int                         SetAssociatedInfo(
                                                 const OCDM_ByteSequence& Info);
    void                        Log(const char *str, ...) const;
    long                        size(void) const;
    OCDM_BOOL                   Valid(void) const;
    const char                 *toString(void) const;
    const char *objectType(void) const {return "OCDM_Collection";}
#ifdef    SWIGPYTHON
    /* The following definitions provide a better integration of our objects
     * with Python. */
    long __len__()         const {return Length();};
    bool __nonzero__()     const {return (Length()!=0);};
    const char *__str__()  const {return toString();};
    const char *__repr__() const {return toString();};
#ifdef    SWIG
    /* Support for Python special methods.
     * __getitem__ allows code like 'obj[attr_name]'
     * __setitem__ allows code like 'obj[attr_name] = AttributeValue(...)'
     * __contains__ allows code like 'attr_name in obj'
     * Code for these methods contributed by
     * Steven Bethard <steven.bethard@colorado.edu>. */
%pythoncode %{
    def __getitem__(self, attr_name): 
        return self.GetAttribute(attr_name)
    def __setitem__(self, attr_name, attr_value): 
        self.PutAttribute(Attribute(attr_name, attr_value))
    def __contains__(self, attr_name):
        return attr_name in self.GetAttributes()
%}
#endif /* SWIG */
#endif /* SWIGPYTHON */
    friend class OCDM_Macros;
}; /* class OCDM_Collection */

/**
*  \brief The OCDM_Tcl class provides an easy way to interact with the main
*  scripting language (Tcl). It provides facilities for executing Tcl code,
*  writting to Tcl channels, etc.
* 
*/
class CDM_EXPORT OCDM_Tcl {
  private:
    mutable Tcl_Obj *resultObj;
  public:
    OCDM_Tcl();
    ~OCDM_Tcl();
    int eval(const char *tcl_code) const;
    int exec(const char *command) const;
    const char *GetResult(void) const;
}; /* class OCDM_Tcl */

/**
*  \brief The OCDM_Utilities class provides access to various utility functions
*  of CDM.
* 
*/
/* Under Windows Perl, the symbol DeleteFile is defined and interferes with
 * our class, leading to unresolved symbol... */
#undef DeleteFile
class CDM_EXPORT OCDM_Utilities {
  private:
    mutable Tcl_Obj *resultObj;
  public:
    OCDM_Utilities();
    ~OCDM_Utilities();
    int DeleteFile(const char *path) const;
    int FileExists(const char *path) const;
    int MakeDirectory(const char *directory) const;
    const char *GetIP(void) const;
    const char *GetTempDir(void) const;
    void Panic(const char *error) const;
    void Log(const char *fmt) const;
    const char *GetResult(void) const;
}; /* class OCDM_Utilities */

/**
*  \brief The OCDM_Macros class provides access to a set of macros that allow
*  control of the Graphical Interface.
* 
*/
class CDM_EXPORT OCDM_Macros {
  private:
    mutable CDM_MACRO_SYSTEM             system;
    mutable CDM_MACRO_SYSTEM_COLLECTION  token;
    mutable const class OCDM_Document   *storedDocumentObj;
    mutable const class OCDM_Collection *storedCollectionObj;
  public:
    OCDM_Macros();
    OCDM_Macros(const OCDM_Collection &col);
    ~OCDM_Macros();
    
    /* Private Helper functions */
    void storeObject(const class OCDM_Document *objPtr) const;
    void storeObject(const class OCDM_Collection *objPtr) const;
    void releaseStoredObject(void) const;
    
    /* Public functions */
    void OpenSystem(const char *name, const char *folder = "/") const;
    void EditSystem(const char *name, const char *folder = "/") const;
    void OpenInSystem(const char *collection_name,
                      const char *document_id) const;
    void OpenInSystem(const char *collection_name) const;
    OCDM_REF(OCDM_Collection) GetOpenedCollection(void) const;
    OCDM_REF(OCDM_Document)   GetOpenedDocument(void) const;
    int            RunSystemComponent(const char *Annotator) const;
    int            RunSystemComponent(const char *Annotator,
                      const bool force) const;
    int            RunSystemTool(const char *Annotator) const;
    int            RunSystemTool(const char *Annotator,
                      const bool force) const;
    int            DisplayViewer(const char *viewer_name,
                      const OCDM_Document &doc,
                      const char *parameter1,
                      const char *parameter2) const;
    int            NFoldExecution(OCDM_Collection col,
                      const int folds,
                      OCDM_ByteSequenceSet TrainingCommands,
                      OCDM_ByteSequenceSet EvaluationCommands) const;
    int            NFoldExecution(OCDM_Collection col,
                      const int folds,
                      OCDM_RawDataSet TrainingCommands,
                      OCDM_RawDataSet EvaluationCommands) const;
#ifndef SWIG
    int            NFoldExecution(OCDM_Collection col,
                      const int folds,
                      const std::vector<std::string>& TrainingCommands,
                      const std::vector<std::string>& EvaluationCommands) const;
#endif /* SWIG */
}; /* class OCDM_Macros */

#endif /* _OCDM_H_ */
