##############################################################################
##  Perl.tcl:                                                               ##
## -------------------                                                      ##
##  This file implements various Perl related procedures.                   ##
##                                                                          ##
##  Copyright 2002-2008 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::CDM_Perl {
  variable PerlLoaded 0
  variable PerlInterp
}

## ::CDM_Perl::InitialisePerl
#  Use this function to create a Perl interpreter inside Ellogon.
proc ::CDM_Perl::InitialisePerl {} {
  variable PerlLoaded
  switch $::tcl_platform(platform) {
    windows {
      InitialisePerl_Windows
    }
    unix {
      InitialisePerl_Unix
    }
    default {
    }
  }
  global HomeDir ApplicationInfo GUI_Options ApplicationName SplashPercent
  # Try to load the tclperl package...
  if {![catch {package require tclperl} error]} {
    set PerlLoaded 1
  }
    
  if {!$PerlLoaded} {
    ## The Perl package failed to initialise. We will make another attempt
    ## before we report an error...
    if {$ApplicationInfo(EnableGUI)} {
      set VisScript [bind .splashscr <Visibility>]
      bind .splashscr <Visibility> {}
      lower .splashscr
      tk_messageBox -title Error... -type ok -default ok -icon error -message\
         "Cannot initialise Perl Bindings:\n\
          The error was: $error" -parent .splashscr
      ## Maybe the user wants to disable Perl support?
      set ans [gut_question "Do you want to disable Perl support for the next\
        time $ApplicationName will start?" Question... 1 .splashscr]
      if {[string equal $ans yes]} {
        set GUI_Options(EnablePerl) 0
      }
      unset ans
      bind .splashscr <Visibility> $VisScript
      raise .splashscr
      unset VisScript
    }
  }
  if {$PerlLoaded} {
    ## Initialise the perl interpreter we will be using for executing perl
    ## code...
    variable PerlInterp
    set CWD [pwd]
    cd $HomeDir/CDM/Perl/$ApplicationInfo(LoadBinaryFolderName)
    set PerlInterp [::perl::interp new]
    $PerlInterp eval {print "Perl interpreter succesfully initialised!\n"}
    ## The tcljava was succesfully loaded. Load the CDM Perl bindings (PerlCDM)
    LDM_SetSplashWindowStatus {Loading CDM Perl bindings (PerlCDM)...} \
                                                           $SplashPercent 205
    if {[catch {$PerlInterp eval {use PerlCDM;}} error]} {
      set PerlLoaded 0
      if {$ApplicationInfo(EnableGUI)} {
        set VisScript [bind .splashscr <Visibility>]
        bind .splashscr <Visibility> {}
        lower .splashscr
        tk_messageBox -title Error... -type ok -default ok -icon error -message\
           "Cannot Load PerlCDM:\n\Current Directory: \"[pwd]\"\n\
            The error was: $error" -parent .splashscr
        ## Maybe the user wants to disable Perl support?
        set ans [gut_question "Do you want to disable Perl support for the next\
          time $ApplicationName will start?" Question... 1 .splashscr]
        if {[string equal $ans yes]} {
          set GUI_Options(EnablePerl) 0
        }
        unset ans
        bind .splashscr <Visibility> $VisScript
        raise .splashscr
        unset VisScript
      }
    } else {
      ## We have loaded Perl!
      catch {LDM_SplashWindowNotify language perl}
    }
    cd $CWD
  }
};# ::CDM_Perl::InitialisePerl

## ::CDM_Perl::InitialisePerl_Windows
#  Initialise Perl under Windows
proc ::CDM_Perl::InitialisePerl_Windows {} {
};# ::CDM_Perl::InitialisePerl_Windows

## ::CDM_Perl::InitialisePerl_Unix
#  Initialise Perl under Unix
proc ::CDM_Perl::InitialisePerl_Unix {} {
};# ::CDM_Perl::InitialisePerl_Unix

## ::CDM_Perl::LoadPerlComponent
#  Load a Component defined as Perl functions and make them available as Tcl
#  functions...
proc ::CDM_Perl::LoadPerlComponent {path module} {
  if {!$::CDM_Perl::PerlLoaded} {error "perl bindings disabled"}
  variable PerlInterp
  ## source the perl code stored in $path/$module.pl
  $PerlInterp eval "\$creole_${module}_home = \"[file normalize $path]\";"
  $PerlInterp eval "do '[file normalize $path/$module.pl]';"
  ## Mirror the perl subs into Tcl procs...
  foreach proc [list creole_$module creole_${module}_Initialize \
                     creole_${module}_Finish] {
    set body {$::CDM_Perl::PerlInterp eval }
    append body "\"${proc}(\\\""
    append body {[join $args {", "}]}
    append body {\");"}
    proc ::$proc {args} $body
  }
};# ::CDM_Perl::LoadPerlComponent
