
/* This file is created by the "tcl2c" utility, which is included in
 * most "plus"-patches (e.g. for Tcl7.6 and Tcl8.0). Standalone
 * executables can be made by providing alternative initialization
 * functions which don't read files any more. Sometimes, small
 * adaptations to the original libraries are needed to get the
 * application truly standalone. The "plus"-patches contain these
 * adaptations for Tcl and Tk. If you just create your own
 * Xxx_InitStandAlone() function for your package, you can
 * easyly extend the "tcl2c" utility to your own requirements.
 *
 *	Jan Nijtmans
 *	CMG (Computer Management Group) Arnhem B.V.
 *	email: Jan.Nijtmans@wxs.nl (private)
 *	       Jan.Nijtmans@cmg.nl (work)
 *	url:   http://home.wxs.nl/~nijtmans/
 */
#include "tcl.h"
#ifdef __WIN32__
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#include <malloc.h>
#include <locale.h>

static int setargv _ANSI_ARGS_((char ***argvPtr));
static void TclshPanic _ANSI_ARGS_(TCL_VARARGS(char *,format));
extern void TclWinSetTclInstance(HINSTANCE instance);

#endif

/*
 * Defines to replace the standard Xxx_Init calls to Xxx_InitStandAlone.
 * If you don't have this function, just delete the corresponding
 * define such that the normal initialization function is used.
 * Similar: If SafeInit functions exists, you can use these
 * by commenting out the corresponding lines below.
 */

#if defined(TCL_ACTIVE) && !defined(SHARED)
#define Tcl_Init Tcl_InitStandAlone
#endif

#if TCL_MAJOR_VERSION < 8
#endif

/*
 * Prototypes of all initialization functions and the free() function.
 * So, only "tcl.h" needs to be included now.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef USE_TCLALLOC
#   define USE_TCLALLOC 0
#endif
#if USE_TCLALLOC == 0
extern void free _ANSI_ARGS_((void *));
extern VOID *malloc _ANSI_ARGS_((int));
#endif
extern int  Tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));



#ifdef __cplusplus
}
#endif

/*
 * The array "script" contains the script that is compiled in.
 * It will be executed in tclAppInit() after the other initializations.
 */

static char *lineformat = "%.0s%d";
static int line = (__LINE__ + 1);
static char *script[] = {
"puts \"Execution Server v 1.0 Started...\"",
"flush stdout",
"set command {}",
"while { $command != \"exit\"} {\n\
  set count [gets stdin command]\n\
  if {$count == -1} {\n\
#   puts \"Execution Server Exiting...\"\n\
#   flush stdout\n\
    exit 0\n\
  }\n\
  ## puts $Log \"Executing Command: $command\"\n\
  if {[catch {eval $command} error]} {\n\
    set error \"$error\\n$errorCode\"\n\
    set line_list [split $error \"\\n\"]\n\
    set lines [llength $line_list]\n\
    if {$lines < 1} {set lines 1}\n\
    puts stdout \"Tcl_Error:$lines\\n$error\"\n\
    ## puts $Log \"Tcl_Error:$lines\\n$error\"\n\
  } else {\n\
    set line_list [split $error \"\\n\"]\n\
    set lines [llength $line_list]\n\
    if {$lines < 1} {set lines 1}\n\
    puts stdout \"Tcl_Ok:$lines\\n$error\"\n\
    ## puts $Log \"Tcl_Ok:$lines\\n$error\"\n\
  }\n\
  flush stdout\n\
  ## flush $Log\n\
}",
(char *) NULL
};

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

#if defined(__WIN32__) && defined(HAS_TK)
int APIENTRY
WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
    HINSTANCE hInstance;
    HINSTANCE hPrevInstance;
    LPSTR lpszCmdLine;
    int nCmdShow;
{
    char **argv;
    int argc;
#else
int
#ifdef _USING_PROTOTYPES_
main (int    argc,		/* Number of command-line arguments. */
      char **argv)		/* Values of command-line arguments. */
#else
main(argc, argv)
    int argc;			/* Number of command-line arguments. */
    char **argv;		/* Values of command-line arguments. */
#endif
{
#endif
    Tcl_Interp *interp;
    char **p = script;
    char *q, buffer[16];
    Tcl_DString data;
    Tcl_Channel inChannel, outChannel, errChannel;

#ifdef __WIN32__
    char argv0[128];
#if defined(TCL_ACTIVE) && !defined(SHARED) && defined(HAS_TK)
    TclWinSetTclInstance(hInstance);
    TkWinXInit(hInstance);
    Tcl_CreateExitHandler((Tcl_ExitProc *) TkWinXCleanup, (ClientData) hInstance);
#endif

    Tcl_SetPanicProc(TclshPanic);


    /*
     * Set up the default locale to be standard "C" locale so parsing
     * is performed correctly.
     */

    setlocale(LC_ALL, "C");

    /*
     * Increase the application queue size from default value of 8.
     * At the default value, cross application SendMessage of WM_KILLFOCUS
     * will fail because the handler will not be able to do a PostMessage!
     * This is only needed for Windows 3.x, since NT dynamically expands
     * the queue.
     */
    SetMessageQueue(64);

    argc = setargv(&argv);

    /*
     * Replace argv[0] with full pathname of executable, and forward
     * slashes substituted for backslashes.
     */

    GetModuleFileName(NULL, argv0, sizeof(argv0));
    argv[0] = argv0;
    for (q = argv0; *q != '\0'; q++) {
	if (*q == '\\') {
	    *q = '/';
	}
    }

#endif
    Tcl_FindExecutable(argv[0]);
    interp = Tcl_CreateInterp();

    q = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", q, TCL_GLOBAL_ONLY);
    ckfree(q);
    sprintf(buffer, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buffer, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", argv[0],TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "tcl_interactive","0", TCL_GLOBAL_ONLY);

    if (Tcl_Init(interp) != TCL_OK) {
        goto error;
    }

    /*
     * Execute the script that is compiled in.
     */

    inChannel = Tcl_GetStdChannel(TCL_STDIN);
    outChannel = Tcl_GetStdChannel(TCL_STDOUT);
    Tcl_DStringInit(&data);
    while(*p) {
	Tcl_DStringSetLength(&data,0);
	Tcl_DStringAppend(&data,*p++,-1);
	if (Tcl_Eval(interp,Tcl_DStringValue(&data)) != TCL_OK) {
	    Tcl_DStringFree(&data);
	    while (p-- != script) {
		for (q = *p;*q; q++) {
		    if (*q=='\n') line++;
		}
		line++;
	    }
	    sprintf(buffer,lineformat,"script",line);
	    Tcl_AddErrorInfo(interp,"\n	( Error in file: \"");
	    Tcl_AddErrorInfo(interp,__FILE__);
	    Tcl_AddErrorInfo(interp,"\", line: ");
	    Tcl_AddErrorInfo(interp,buffer);
	    Tcl_AddErrorInfo(interp,")");
	    errChannel = Tcl_GetStdChannel(TCL_STDERR);
	    if (errChannel) {
		Tcl_Write(errChannel,
			Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY), -1);
		Tcl_Write(errChannel, "\n", 1);
	    }
#ifdef __WIN32__
	    TclshPanic(Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY));
#endif
	    sprintf(buffer, "exit %d", 1);
	    Tcl_Eval(interp, buffer);
	}
    }
    Tcl_DStringFree(&data);
    sprintf(buffer, "exit %d", 0);
    Tcl_Eval(interp, buffer);

error:
    errChannel = Tcl_GetStdChannel(TCL_STDERR);
    if (errChannel) {
	Tcl_Write(errChannel,
		"application-specific initialization failed: ", -1);
	Tcl_Write(errChannel, interp->result, -1);
	Tcl_Write(errChannel, "\n", 1);
    }
#ifdef __WIN32__
    TclshPanic(interp->result);
#endif
    sprintf(buffer, "exit %d", 1);
    Tcl_Eval(interp, buffer);
    return 0;
}

#ifdef __WIN32__
/*
 *----------------------------------------------------------------------
 *
 * TclshPanic --
 *
 *	Display a message and exit.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Exits the program.
 *
 *----------------------------------------------------------------------
 */

void
TclshPanic TCL_VARARGS_DEF(char *,arg1)
{
    va_list argList;
    char buf[1024];
    char *format;

    format = TCL_VARARGS_START(char *,arg1,argList);
    vsprintf(buf, format, argList);

    MessageBeep(MB_ICONEXCLAMATION);
    MessageBox(NULL, buf, "Fatal Error in Tclsh",
	    MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
#ifdef _MSC_VER
    _asm {
        int 3
    }
#endif
    ExitProcess(1);
}

/*
 *-------------------------------------------------------------------------
 *
 * setargv --
 *
 *	Parse the Windows command line string into argc/argv.  Done here
 *	because we don't trust the builtin argument parser in crt0.  
 *	Windows applications are responsible for breaking their command
 *	line into arguments.
 *
 *	2N backslashes + quote -> N backslashes + begin quoted string
 *	2N + 1 backslashes + quote -> literal
 *	N backslashes + non-quote -> literal
 *	quote + quote in a quoted string -> single quote
 *	quote + quote not in quoted string -> empty string
 *	quote -> begin quoted string
 *
 * Results:
 *	returns the number of arguments and fill argvPtr with the
 *	array of arguments.
 *
 * Side effects:
 *	Memory allocated.
 *
 *--------------------------------------------------------------------------
 */

static int
setargv(argvPtr)
    char ***argvPtr;		/* Filled with argument strings (malloc'd). */
{
    char *cmdLine, *p, *arg, *argSpace;
    char **argv;
    int argc, size, inquote, copy, slashes;

    cmdLine = GetCommandLine();

    /*
     * Precompute an overly pessimistic guess at the number of arguments
     * in the command line by counting non-space spans.
     */

    size = 2;
    for (p = cmdLine; *p != '\0'; p++) {
	if (isspace(*p)) {
	    size++;
	    while (isspace(*p)) {
		p++;
	    }
	    if (*p == '\0') {
		break;
	    }
	}
    }
    argSpace = (char *) ckalloc((unsigned) (size * sizeof(char *)
	    + strlen(cmdLine) + 1));
    argv = (char **) argSpace;
    argSpace += size * sizeof(char *);
    size--;

    p = cmdLine;
    for (argc = 0; argc < size; argc++) {
	argv[argc] = arg = argSpace;
	while (isspace(*p)) {
	    p++;
	}
	if (*p == '\0') {
	    break;
	}

	inquote = 0;
	slashes = 0;
	while (1) {
	    copy = 1;
	    while (*p == '\\') {
		slashes++;
		p++;
	    }
	    if (*p == '"') {
		if ((slashes & 1) == 0) {
		    copy = 0;
		    if ((inquote) && (p[1] == '"')) {
			p++;
			copy = 1;
		    } else {
			inquote = !inquote;
		    }
                }
                slashes >>= 1;
            }

            while (slashes) {
		*arg = '\\';
		arg++;
		slashes--;
	    }

	    if ((*p == '\0') || (!inquote && isspace(*p))) {
		break;
	    }
	    if (copy != 0) {
		*arg = *p;
		arg++;
	    }
	    p++;
        }
	*arg = '\0';
	argSpace = arg + 1;
    }
    argv[argc] = NULL;

    *argvPtr = argv;
    return argc;
}
#endif /* __WIN32__ */
