##############################################################################
##  CollectionWindow.tcl:                                                   ##
## ---------------------                                                    ##
##  This file contains functions that relate to the dialogs for selecting,  ##
##  opening and managing Collections.                                       ##
##                                                                          ##
##  Copyright 1998-2006 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## Make sure that some needed images are loaded...
foreach image {ITOpenCollection ITCreateCollection ITDeleteCollection
               ITEditCollection ITOpenDocument ITCreateDocument
               ITDeleteDocument ITUpdateDocument ITViewTextLarge
               ITAddCollectionPath ICollection ICollectionGate ITNewMany
               IDocument IDocumentGate IDocumentSmall IAddDocuments
               IRemoveDocuments ITViewAnns ITinyDoc} {
  GUI_LoadImage $image
}
unset image

## GUI_CreateCollectionMenu
#
proc GUI_CreateCollectionMenu {win} {
  global GUI_Text GUI_Underline GUI_Options
  #set menubar [menu ${win}.mb -type menubar -relief $GUI_Options(MenuRelief)
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }

  ## File Menu
  set file $menubar.file; catch {destroy $file}
  $menubar add cascade -label $GUI_Text(File) -menu $file \
     -underline $GUI_Underline(File)
  menu $file
  $file add command -label $GUI_Text(Options)... -underline \
     $GUI_Underline(Options) -command "GUI_EditPreferences $win.edt_Prfs" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $file add separator
  $file add command -label $GUI_Text(Close) -underline $GUI_Underline(Close)\
     -command "destroy $win" -image ITExit \
     -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $file}

  ## View Menu
  set view $menubar.view; catch {destroy $view}
  $menubar add cascade -label $GUI_Text(View) -menu $view \
       -underline $GUI_Underline(View)
  menu $view
  $view add command -label $GUI_Text(Redraw) -underline \
     $GUI_Underline(Redraw) -command {catch {GUI_DrawCollectionDlg}} \
     -image ITRedraw -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(ModifyColSearchPath) -underline \
     $GUI_Underline(ModifyColSearchPath) -command \
     "GUI_EditPref_ModifySearchPath $win CollectionPath directory" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(Options)... -underline \
     $GUI_Underline(Options) -command "GUI_EditPreferences $win.edt_Prfs" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowToolbars) -underline \
     $GUI_Underline(ShowToolbars) -command \
     "Toolbar::deiconify $win.vtf.tb $win.tf.tb1" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $view}

  ## Collection Menu
  set col $menubar.col; catch {destroy $col}
  $menubar add cascade -label $GUI_Text(Collection) -menu $col \
        -underline $GUI_Underline(Collection)
  menu $col
  $col add command -label $GUI_Text(OpenCollectionBT) \
     -command GUI_OpenCollection \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(NewCollectionBT) \
     -command GUI_NewCollection \
     -image ITNewMany -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(DeleteCollectionBT) \
     -command GUI_DeleteCollection \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(EditCollectionBT) \
     -command GUI_EditCollection \
     -image ITEdit -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(NewDocumentBT) \
     -command GUI_AddDocument \
     -image ITNew -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(DeleteDocumentsBT) \
     -command GUI_DeleteDocuments \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(ChangeCollectionFormat) \
     -command GUI_ChangeCollectionFormat \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $col add command -label $GUI_Text(RenameCollectionBT) \
     -command GUI_RenameCollection \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $col add separator
  $col add command -label $GUI_Text(ModifyColSearchPath) -underline \
     $GUI_Underline(ModifyColSearchPath) -command \
     "GUI_EditPref_ModifySearchPath $win CollectionPath directory" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $col}

  ## Document Menu
  set doc $menubar.doc; catch {destroy $doc}
  $menubar add cascade -label $GUI_Text(Document) -menu $doc \
        -underline $GUI_Underline(Document)
  menu $doc
  $doc add command -label $GUI_Text(OpenDocumentBT) \
        -command GUI_OpenDocument \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $doc add command -label $GUI_Text(DeleteDocumentBT) \
        -command GUI_DeleteDocument \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $doc add command -label $GUI_Text(UpdateDocumentBT) \
        -command GUI_UpdateDocument \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $doc add command -label $GUI_Text(ViewDocumentBT) \
        -command GUI_ViewDocument \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $doc}

  ## Tools Menu
  set utils $menubar.utils; catch {destroy $utils}
  $menubar add cascade -label $GUI_Text(Utilities) -underline \
     $GUI_Underline(Utilities) -menu $utils
  menu $utils

    set sgml $utils.sgml
    $utils add cascade -label SGML -underline 0 -menu $sgml
    menu $sgml
    $sgml add command -label $GUI_Text(ConvertFromSGML) -underline \
         $GUI_Underline(ConvertFromSGML) -command \
         GUT_SGML_ConvertCollectionFromSGML
    $sgml add command -label $GUI_Text(ConvertToSGML) -underline \
         $GUI_Underline(ConvertToSGML) -command \
         GUT_SGML_ConvertCollectionToSGML

    set text $utils.text
    $utils add cascade -label $GUI_Text(Text) -underline $GUI_Underline(Text) \
        -menu $text
    menu $text
    $text add command -label {Export Collection Text...} -underline 0 \
        -command "GUT_ExportCollectionTexts {} $win.gut_ExportCollectionTexts"

    set docs $utils.docs
    $utils add cascade -label $GUI_Text(Documents) -menu $docs
    menu $docs
    $docs add command -label $GUI_Text(TransferDocumentsBtwnCols...) -command \
        "CDMTools_TransferCollectionDocuments $win.cdm_toolsTransferColDocs"

    set ling_info $utils.lingistic_information
    $utils add cascade -label $GUI_Text(LinguisticInformation) -menu $ling_info
    menu $ling_info
    $ling_info add command -label $GUI_Text(TransferAnnsBtwnCols...) -command \
        "CDMTools_CopyAnnotationsBetweenCollections \
         $win.cdm_toolsTransferAnnsBtwnCols"
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $utils}
    
  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help $win.infoViewerHTML
  
  return $menubar
}

## GUI_CollectionDlg
#
proc GUI_CollectionDlg {Mode Sys_win} {
  global GUI_Text GUI_Options Locale KeyboardLocale \
         GUI_CollectionDlg GUI_MainWidgets WMIcons

  Agent::Play Idle2_1
  set win .c_dlg
  if {[winfo exists $win]} {return}
  set GUI_CollectionDlg(Window) $win
  set GUI_CollectionDlg(Col2Open) {}
  set GUI_CollectionDlg(Doc2Open) {}
  toplevel $win
  wm title $win "$Mode Collection..."
  wm withdraw $win

  set open [string equal -nocase $Mode open]

  ## Menubar
  set menubar [GUI_CreateCollectionMenu $win]
  $win configure -menu $menubar

  ## Toolbar
  set toolbar_frame ${win}.tf
  set vertical_toolbar ${win}.vtf
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
  Toolbar::ToolbarFrame $vertical_toolbar \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate) -orientation vertical

    ## Load Toolbar
    set toolbar $vertical_toolbar.tb
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)

    if {$open} {
      ## Open Collection
      set w [Toolbar::addbutton $toolbar -image ITOpenCollection -tooltipvar \
         GUI_Text(OpenCollectionBT) -command GUI_OpenCollection \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(CollectionButtons) $w .c_dlg.vtf.tb.fltWin.0
      lappend GUI_CollectionDlg(DocumentButtons) $w .c_dlg.vtf.tb.fltWin.0
    } else {
      ## New Collection
      set w [Toolbar::addbutton $toolbar -image ITCreateCollection -tooltipvar \
         GUI_Text(NewCollectionBT) -command GUI_NewCollection \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      ## Delete Collection
      set w [Toolbar::addbutton $toolbar -image ITDeleteCollection -tooltipvar \
         GUI_Text(DeleteCollectionBT) -command GUI_DeleteCollection \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(CollectionButtons) $w .c_dlg.vtf.tb.fltWin.1
      ## Edit Collection
      set w [Toolbar::addbutton $toolbar -image ITEditCollection -tooltipvar \
         GUI_Text(EditCollectionBT) -command GUI_EditCollection \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(CollectionButtons) $w .c_dlg.vtf.tb.fltWin.2
    }


    if {$open} {
      ## Open Document
      set w [Toolbar::addbutton $toolbar -image ITOpenDocument -tooltipvar \
         GUI_Text(OpenDocumentBT) -command GUI_OpenDocument \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(DocumentButtons) $w .c_dlg.vtf.tb.fltWin.1 \
              .c_dlg.vtf.tb.fltWin.3
    } else {
      ## New Document
      set w [Toolbar::addbutton $toolbar -image ITCreateDocument -tooltipvar \
         GUI_Text(NewDocumentBT) -command GUI_AddDocument \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(CollectionButtons) $w .c_dlg.vtf.tb.fltWin.3
      lappend GUI_CollectionDlg(DocumentButtons)   $w .c_dlg.vtf.tb.fltWin.3
      ## Delete Document
      set w [Toolbar::addbutton $toolbar -image ITDeleteDocument -tooltipvar \
         GUI_Text(DeleteDocumentBT) -command GUI_DeleteDocument \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(DocumentButtons) $w .c_dlg.vtf.tb.fltWin.4
      ## Update Document
      set w [Toolbar::addbutton $toolbar -image ITUpdateDocument -tooltipvar \
         GUI_Text(UpdateDocumentBT) -command GUI_UpdateDocument \
         -statusvar ::GUI_StatusBar::StatusBar($win)]
      lappend GUI_CollectionDlg(DocumentButtons) $w .c_dlg.vtf.tb.fltWin.5 \
              .c_dlg.vtf.tb.fltWin.7
    }

    ## View Document
    Toolbar::addseparator $toolbar
    set w [Toolbar::addbutton $toolbar -image ITViewTextLarge -tooltipvar \
       GUI_Text(ViewDocumentBT) -command GUI_ViewDocument \
       -statusvar ::GUI_StatusBar::StatusBar($win)]
    lappend GUI_CollectionDlg(DocumentButtons) $w
    ## Get all button bindings
    foreach event [bind $w] {
      set GUI_CollectionDlg(ButtonEvent:$event) [bind $w $event]
    }

    pack $toolbar -side top -fill x
    set VToolbar $toolbar

    ## Window Toolbar
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenPropertiesDlg) \
       -image ITProperties -command {GUI_ColDlgProperties %W} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TAddCollectionPath) \
       -image ITAddCollectionPath -command \
       "GUI_EditPref_ModifySearchPath $win CollectionPath directory" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TRedraw) \
       -image ITRedraw -command {catch {GUI_DrawCollectionDlg}} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(DismissButton) \
       -image ITExit -command "GUI_CloseWindow $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Help) -image ITHelp \
       -command "GUI_Help_Contents $win.infoViewerHTML" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar label -relief flat -bd 0 -anchor e \
       -textvariable GUI_CollectionDlg(Col2Open) \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  ## Status Bar
  set ::GUI_StatusBar::StatusBar($win) {}
  set ::GUI_StatusBar::StatusBar($win:Def) {}
  GUI_CreateStatusBar $win.statusbar
  
  ## Button Section
  frame $win.buttons
  button $win.buttons.cancel -textvariable GUI_Text(Dismiss) -command \
    "catch {place info $win.parent.__h1} GUI_CollectionDlg(Pane);destroy $win"
  bind $win.buttons.cancel <Enter> \
    "set ::GUI_StatusBar::StatusBar($win) \"$GUI_Text(DismissButton)\""
  bind $win.buttons.cancel <Leave> \
   "set ::GUI_StatusBar::StatusBar($win) \$::GUI_StatusBar::StatusBar($win:Def)"
  pack $win.buttons.cancel -side bottom -padx 10 -pady 10
  pack $win.buttons -fill x -side bottom
  pack $vertical_toolbar -side right -fill y

  set parent $win.parent
  frame $parent -borderwidth 0 -relief flat -highlightthickness 0
  ## Collection Canvas               ## Document Canvas
  set cfr $parent.cfr;             set dfr $parent.dfr
  set cols $cfr.canvas.col_canvas; set docs $dfr.canvas.doc_canvas
  set GUI_CollectionDlg(Tree) $cols; set GUI_CollectionDlg(TreeView) $docs

  ## Collection Canvas (Tree Widget)
  frame $cfr -relief ridge -bg $GUI_Options(ExplorerBg)
  label $cfr.label -textvariable GUI_Text(AvailableCollections) \
    -relief ridge -justify left -anchor w -padx 15
  ScrolledWidgets::ScrolledCanvas $cfr.canvas -widgetname col_canvas \
    -relief flat -borderwidth $GUI_Options(ExplorerBorderWidth) \
    -bg $GUI_Options(ExplorerBg) -height $GUI_Options(ExplorerTreeHeigth) \
    -width $GUI_Options(ExplorerTreeWidth) -highlightthickness 0 \
    -regionpadx 5 -regionpady 20
  Tree:create $cols
  label $cfr.canvas.label -textvariable GUI_Text(AvailableCollections) \
     -bg $GUI_Options(ExplorerStandoutBg) -fg $GUI_Options(ExplorerStandout) \
     -bd $GUI_Options(BorderWidth) -relief raised -highlightthickness 0 \
     -anchor w
  grid configure $cfr.canvas.label -column 0 -row 0 -sticky new
  pack $cfr.canvas -side left -fill both -expand 1
  pack $cfr -side left -fill y -expand 1

  ## Document Canvas (Tree View Widget)
  frame $dfr -relief ridge -bg $GUI_Options(ExplorerBg)
  label $dfr.label -textvariable GUI_Text(Collection'sDocuments) \
    -relief ridge -justify left -anchor w -padx 15
  ScrolledWidgets::ScrolledCanvas ${dfr}.canvas -widgetname doc_canvas \
    -relief $GUI_Options(ExplorerRelief) \
    -borderwidth $GUI_Options(ExplorerBorderWidth) \
    -bg $GUI_Options(ExplorerBg) -highlightthickness 0 \
    -regionpadx 10 -regionpady 20 \
    -confcommand "TreeView:draw %W no"
  label $dfr.canvas.label -textvariable GUI_Text(Collection'sDocuments) \
     -bg $GUI_Options(ExplorerStandoutBg) -fg $GUI_Options(ExplorerStandout) \
     -bd $GUI_Options(BorderWidth) -relief raised -highlightthickness 0 \
     -anchor w
  TreeView:create $docs
  TreeView:link $docs $cols
  if {$open} {TreeView:Call $docs GUI_OpenDocument}
  grid configure $dfr.canvas.label -column 0 -row 0 -sticky new
  pack $dfr.canvas -side left -fill both -expand 1
  pack $dfr -side left -fill y -expand 1
  pane $cfr $dfr -orient horizontal

  Widget_PaneSet $parent 1 $GUI_Options(CollectionWindowPanePercent)
  pack $parent -fill both -expand 1

  ## Bindings...
  TreeView:UpFolderImage $docs IUp
  TreeView:UpFolderLabel $docs {}
  $cols bind x <<MB_1>> \
  "set lbl \[Tree:labelat %W %x %y\];Tree:setselection \%W \$lbl;\
   TreeView:setselection $docs \$lbl;TreeView:draw $docs;\
   TreeView:drawselection $docs"
  $cols bind x <<MB_2>> \
  "set lbl \[Tree:labelat %W %x %y\];Tree:setselection \%W \$lbl;\
   TreeView:setselection $docs \$lbl;TreeView:draw $docs;\
   TreeView:drawselection $docs"
  ## Left  Menu Handler...
  ## Right Menu Handler...
  bind $docs <<MB_2>> "GUI_CollectionActionPopUp %W $open"

  ## Draw the Dialog...
  catch {GUI_DrawCollectionDlg}

  ## Display Dialog in the screen center and make it application modal...
  if {$open} {set width 500;set height 310} else {set width 510;set height 450}
  wm geometry $win ${width}x$height
  set x [expr {([winfo screenwidth  .] - $width )/2}]
  set y [expr {([winfo screenheight .] - $height)/2}]
  wm geometry $win +${x}+$y
  update
  catch {
    if {$open} {
      winico setwindow $win $WMIcons(Collection)
    } else {
      winico setwindow $win $WMIcons(EditCol)
    }
  }
  wm deiconify $win
  wm geometry $win +${x}+$y
  grab set $win
  trace variable GUI_MainWidgets($win:CurrentSelection) w \
                "GUI_CDlgUpdateStatusBar $win"
  tkwait window $win
  if {![catch {expr {int([lindex $GUI_CollectionDlg(Pane) 3]*100)}} value]} {
    set GUI_Options(CollectionWindowPanePercent) $value
  }
  pane forget $parent
  grab release $win
  catch {unset GUI_CollectionDlg}
  foreach one [array get GUI_MainWidgets $win:*] {
    catch {unset GUI_MainWidgets($one)}
  }
}

## GUI_DrawCollectionDlg
#
proc GUI_DrawCollectionDlg {{selection {}}} {
  global GUI_Options GUI_CollectionDlg GUI_Text

  Agent::Play Search Searching
  GUI_LoadImage ICollectionXML
  GUI_LoadImage IDocumentXML
  GUI_LoadImage ICollectionRegistered
  GUI_LoadImage IDocumentRegistered
  
  ## Here, we have to create the images that will be used as "over" images,
  ## i.e shown when the mouse cursor is over the item...
  catch {image delete IFolderSmallOver IFolderSmallOpenOver IFolderOver\
            ISystemOver ISystemSmallOver ICollectionOver IDocumentOver\
            IDocumentSmallOver \
            ICollectionGateOver ICollectionXMLOver ICollectionRegisteredOver\
            IDocumentGateOver IDocumentXMLOver IDocumentRegisteredOver}
  foreach image {IFolderSmall IFolderSmallOpen IFolder ISystem ISystemSmall\
            ICollection ICollectionGate ICollectionXML ICollectionRegistered\
            IDocument IDocumentGate IDocumentXML IDocumentRegistered\
            IDocumentSmall} {
    image create photo ${image}Over
    ${image}Over copy $image
    ${image}Over configure -gamma $GUI_Options(OverImageGamma)
  }
  
  set Ccan $GUI_CollectionDlg(Tree)
  set Dcan $GUI_CollectionDlg(TreeView)
  catch {Tree:Reset $Ccan}
  Tree:newitem $Ccan /
  array unset GUI_CollectionDlg T2C:*

  # Add Collections
  if {[catch {
    foreach col [GUI_GetCollections] {
      set name [lindex $col 0]
      set format [lindex $col 2]
#     set tail [string map {{ } _} [file tail $name]]
      set tail [file tail $name]
      set root $tail; set i 2
      while {[info exists GUI_CollectionDlg(T2C:$tail)]} {
        set tail "${root} \(${i}\)"
        incr i
      }
      set GUI_CollectionDlg(T2C:$tail) $name
      switch -glob -- $format {
        native {
          Tree:newitem $Ccan /$tail \
            -image IFolderSmall -overimage IFolderSmallOver \
            -openimage IFolderSmallOpen -overopenimage IFolderSmallOpenOver \
            -largeimage ICollection -largeoverimage ICollectionOver \
            -tooltipimg ICollection -tooltip $name
        }
        gate* {
          Tree:newitem $Ccan /$tail \
            -image IFolderSmall -overimage IFolderSmallOver \
            -openimage IFolderSmallOpen -overopenimage IFolderSmallOpenOver \
            -largeimage ICollectionGate -largeoverimage ICollectionGateOver \
            -tooltipimg ICollectionGate -tooltip $name
        }
        XML {
          Tree:newitem $Ccan /$tail \
            -image IFolderSmall -overimage IFolderSmallOver \
            -openimage IFolderSmallOpen -overopenimage IFolderSmallOpenOver \
            -largeimage ICollectionXML -largeoverimage ICollectionXMLOver \
            -tooltipimg ICollectionXML -tooltip $name
        }
        default {
          Tree:newitem $Ccan /$tail \
            -image IFolderSmall -overimage IFolderSmallOver \
            -openimage IFolderSmallOpen -overopenimage IFolderSmallOpenOver \
            -largeimage ICollectionRegistered \
            -largeoverimage ICollectionRegisteredOver \
            -tooltipimg ICollectionRegistered -tooltip $name
        }
      }
      # Add Documents
      set docs [lindex [lindex $col 1] 0]
      if {[llength $docs] > 250} {
        set message " [llength $docs] $GUI_Text(Files...) "
        switch -glob -- $format {
          native {
            Tree:newitem $Ccan /${tail}/$message -image ITNewMany -tooltip \
              $name -largeimage IDocument -tooltipimg IDocument -data $docs
          }
          gate* {
            Tree:newitem $Ccan /${tail}/$message -image ITNewMany -tooltip \
              $name -largeimage IDocumentGate -tooltipimg IDocumentGate \
              -data $docs
          }
          XML {
            Tree:newitem $Ccan /${tail}/$message -image ITNewMany -tooltip \
              $name -largeimage IDocumentXML -tooltipimg IDocumentXML \
              -data $docs
          }
          default {
            Tree:newitem $Ccan /${tail}/$message -image ITNewMany -tooltip \
              $name -largeimage IDocumentRegistered -tooltipimg \
              IDocumentRegistered -data $docs
          }
        }
      } else {
        foreach doc $docs {
          set doc_tail [lindex $doc 0]
          switch -glob -- $format {
            native {
              Tree:newitem $Ccan /${tail}/$doc_tail \
                -image IDocumentSmall -overimage IDocumentSmallOver \
                -largeimage IDocument -largeoverimage IDocumentOver \
                -tooltipimg IDocument -tooltip $name\n$doc_tail
            }
            gate* {
              Tree:newitem $Ccan /${tail}/$doc_tail \
                -image IDocumentSmall -overimage IDocumentSmallOver \
                -largeimage IDocumentGate -largeoverimage IDocumentGateOver \
                -tooltipimg IDocumentGate -tooltip $name\n$doc_tail
            }
            XML {
              Tree:newitem $Ccan /${tail}/$doc_tail \
                -image IDocumentSmall -overimage IDocumentSmallOver \
                -largeimage IDocumentXML -largeoverimage IDocumentXMLOver\
                -tooltipimg IDocumentXML -tooltip $name\n$doc_tail
            }
            default {
              Tree:newitem $Ccan /${tail}/$doc_tail \
                -image IDocumentSmall -overimage IDocumentSmallOver \
                -largeimage IDocumentRegistered \
                -largeoverimage IDocumentRegisteredOver \
                -tooltipimg IDocumentRegistered -tooltip $name\n$doc_tail
            }
          }
        }
      }
    }
  } error]} {
    global errorInfo
    puts stderr "Error Loading Collections:\n   $error\n   $errorInfo"
  }
  if {[string equal $selection {}]} {
    TreeView:setselection $Dcan /
  } else {
    if {[catch {TreeView:setselection $selection}]} {
      TreeView:setselection $Dcan /
    }
  }
  TreeView:draw $Dcan
  global __Tmp__; set __Tmp__ {}
  GUI_CDlgUpdateStatusBar $GUI_CollectionDlg(Window) __Tmp__ {} {}
  unset __Tmp__
  Agent::Play stop Blink
}

## GUI_CDlgUpdateStatusBar
#  Update status bar of Collection dialog according to selection...
proc GUI_CDlgUpdateStatusBar {win name1 name2 op} {
  global GUI_Text GUI_CollectionDlg GUI_MainWidgets $name1
  if {![string equal $name2 {}]} {
    set value [lindex [set "${name1}(${name2})"] 0]
  } else {
    set value [lindex [set $name1] 0]
  }
  set ind [string first / $value 1]
  if {$ind < 0}  {set ind end}

  if {[string length $value] < 2} {
    set Collection "($GUI_Text(none))"
    set Document   "($GUI_Text(none))"
    set GUI_CollectionDlg(Col2Open) {}
    set GUI_CollectionDlg(Doc2Open) {}
    set abutton {}
    set dbutton [concat $GUI_CollectionDlg(CollectionButtons) \
                        $GUI_CollectionDlg(DocumentButtons)]
  } elseif {[string equal $ind end]} {
    set Collection [string range $value 1 end]
    set Document   "($GUI_Text(none))"
    #set tail [string map {{ } _} $Collection]
    set tail $Collection
    catch {set GUI_CollectionDlg(Col2Open) $GUI_CollectionDlg(T2C:$tail)}
    set GUI_CollectionDlg(Doc2Open) {}
    set abutton $GUI_CollectionDlg(CollectionButtons)
    set dbutton $GUI_CollectionDlg(DocumentButtons)
  } else {
    set Collection [string range $value 1 [expr $ind -1]]
    set Document   [string range $value [expr $ind +1] end]
    set pattern {_[0-9][0-9][0-9]*_*..._}
    if {[string match $pattern $Document]} {
      set Document \
         "($GUI_Text(none)) ([string trim $Document])"
      set GUI_CollectionDlg(Doc2Open) {}
      set abutton $GUI_CollectionDlg(CollectionButtons)
      set dbutton $GUI_CollectionDlg(DocumentButtons)
    } else {
      #set GUI_CollectionDlg(Doc2Open) [string map {{ } _} $Document]
      set GUI_CollectionDlg(Doc2Open) $Document
      set abutton $GUI_CollectionDlg(DocumentButtons)
      set dbutton $GUI_CollectionDlg(CollectionButtons)
    }
    #set tail [string map {{ } _} $Collection]
    set tail $Collection
    catch {set GUI_CollectionDlg(Col2Open) $GUI_CollectionDlg(T2C:$tail)}
  }

  ## Activate Appropriate Button
  foreach w $dbutton {catch {$w configure -state disabled}}
  foreach w $abutton {catch {$w configure -state normal}}
  foreach one [array names GUI_CollectionDlg ButtonEvent:*] {
    set event [string range $one 12 end]
    foreach w $dbutton {
      catch {bind $w $event {}}
    }
    foreach w $abutton {
      catch {bind $w $event $GUI_CollectionDlg($one)}
    }
  }

  set value \
   "$GUI_Text(Collection): $Collection,  $GUI_Text(Document): $Document"
  set ::GUI_StatusBar::StatusBar($win) $value
  set ::GUI_StatusBar::StatusBar($win:Def) $value
  catch {GUI_ColDlgPropertiesUpdate}
  return {}
}

## GUI_GetCollections
#
proc GUI_GetCollections {{silent 0}} {
  global CollectionPath CDM_CollectionFilename CDM_CollectionTypeFilename \
         GUI_Options

  if {![info exists CDM_CollectionFilename]} {
    set CDM_CollectionFilename     GuT_mAgIc
    set CDM_CollectionTypeFilename GuT_mAgIc
  }

  foreach dir [lsort -unique $CollectionPath] {
    set colls [glob -nocomplain -type d [file join $dir *]]
    if {![llength $colls]} {continue}
    foreach coll $colls {
      set file  [file join $coll $CDM_CollectionFilename]
      set file2 [file join $coll GuT_mAgIc]
      set file3 [file join $coll $CDM_CollectionTypeFilename]
      if {[file exists $file] || 
         ($GUI_Options(EnableGateCollectionManagement) && [file exists $file2])
         || [file exists $file3]} {
        set CollName [file tail $coll]
        if {[info exists Collections($CollName)]} {
          set Collections($coll) $coll
        } else {
          set Collections($CollName) $coll
        }
      }
    }
  }
  if {![info exists Collections]} {return {}}
  set coll_names [lsort [array names Collections]]
  catch {unset colls}
  foreach coll $coll_names {lappend colls $Collections($coll)}
  catch {unset Collections}
  ## Accumulate the errors in a variable so as to at least show the
  ## Collections that can be handled correctly...
  set ErrorLog {}
  foreach coll $colls {
    set file  [file join $coll $CDM_CollectionFilename]
    set file2 [file join $coll GuT_mAgIc]
    set file3 [file join $coll $CDM_CollectionTypeFilename]
    catch {unset CollectionFiles}
    set CollName $coll
    if {[file exists $file]} {
      if {[catch {CDM_GetCollectionFileIDs $file} DocumentXIDs]} {
        global GUI_Text
        lappend ErrorLog [list \
          "[format $GUI_Text(CannotReadFile) $file]!\n($DocumentXIDs)"]
        continue
      }
      set FileFormat($CollName) native
    } elseif {[file exists $file2]} {
      if {[catch {CDM_GetCollectionFileIDs $file2 gate} DocumentXIDs]} {
        global GUI_Text
        lappend ErrorLog [list \
          "[format $GUI_Text(CannotReadFile) $file2]!\n($DocumentXIDs)"]
        continue
      }
      set FileFormat($CollName) gate
    } elseif {[file exists $file3]} {
      set TypeFile [open $file3]
      set FileFormat($CollName) [string trim [read $TypeFile]]
      close $TypeFile
      ## Is the type a registered one?
      if {[lsearch -exact [CDM_GetRegisteredCollectionTypes] \
            $FileFormat($CollName)] == -1} {
        lappend ErrorLog [list \
          "Unknown Collection type \"$FileFormat($CollName)\" for Collection\
           \"$CollName\"!"]
        continue
      }
      if {[catch {CDM_GetCollectionFileIDs $file3 $FileFormat($CollName)} \
                                                              DocumentXIDs]} {
        global GUI_Text
        lappend ErrorLog [list \
          "[format $GUI_Text(CannotReadFile) $file3]!\n($DocumentXIDs)"]
        continue
      }
    }
    if {[llength $DocumentXIDs] > 0} {
      foreach FileLoc $DocumentXIDs {
        set CollectionFiles($FileLoc) [file join $coll $FileLoc]
      }
    } else {
      set CollectionFiles({}) [file join $coll {}]
    }
    foreach FileName [lsort [array names CollectionFiles]] {
      lappend Files($CollName) [list $FileName $CollectionFiles($FileName)]
    }
  }
  if {!$silent && [llength $ErrorLog]} {
    ## Display the error dialog...
    gut_many_errors {Errors while searching for Collections...} \
       {Errors while searching for Collections} $ErrorLog
  }
  foreach CollName [lsort [array names Files]] {
    lappend Collections [list $CollName [list $Files($CollName)] \
                                         $FileFormat($CollName)]
  }
  if {![info exists Collections]} {return {}}
  return $Collections
}

## GUI_GetCollectionDocuments
#
proc GUI_GetCollectionDocuments {col {silent 0}} {
  global CollectionPath CDM_CollectionFilename CDM_CollectionTypeFilename \
         GUI_Options

  if {![info exists CDM_CollectionFilename]} {
    set CDM_CollectionFilename     GuT_mAgIc
    set CDM_CollectionTypeFilename GuT_mAgIc
  }
  set ErrorLog {}

  set file  [file join $col $CDM_CollectionFilename]
  set file2 [file join $col GuT_mAgIc]
  set file3 [file join $col $CDM_CollectionTypeFilename]
  if {[file exists $file]} {
    if {[catch {CDM_GetCollectionFileIDs $file} DocumentXIDs]} {
      global GUI_Text
      lappend ErrorLog [list \
        "[format $GUI_Text(CannotReadFile) $file]!\n($DocumentXIDs)"]
      continue
    }
    set FileFormat native
  } elseif {[file exists $file2]} {
    if {[catch {CDM_GetCollectionFileIDs $file2 gate} DocumentXIDs]} {
      global GUI_Text
      lappend ErrorLog [list \
        "[format $GUI_Text(CannotReadFile) $file2]!\n($DocumentXIDs)"]
      continue
    }
    set FileFormat gate
  } elseif {[file exists $file3]} {
    set TypeFile [open $file3]
    set FileFormat [string trim [read $TypeFile]]
    close $TypeFile
    ## Is the type a registered one?
    if {[lsearch -exact [CDM_GetRegisteredCollectionTypes] \
          $FileFormat] == -1} {
      lappend ErrorLog [list \
        "Unknown Collection type \"$FileFormat\" for Collection \"$col\"!"]
      continue
    }
    if {[catch {CDM_GetCollectionFileIDs $file3 $FileFormat} DocumentXIDs]} {
      global GUI_Text
      lappend ErrorLog [list \
        "[format $GUI_Text(CannotReadFile) $file3]!\n($DocumentXIDs)"]
      continue
    }
  }
  
  if {!$silent && [llength $ErrorLog]} {
    ## Display the error dialog...
    gut_many_errors {Errors while searching for Collections...} \
       {Errors while searching for Collections} $ErrorLog
  }
  return $DocumentXIDs
};# GUI_GetCollectionDocuments

## GUI_GetCollectionNames
#   Returns the Collection names...
proc GUI_GetCollectionNames {} {
  global CollectionPath CDM_CollectionFilename CDM_CollectionTypeFilename \
         GUI_Options

  if {![info exists CDM_CollectionFilename]} {
    set CDM_CollectionFilename     GuT_mAgIc
    set CDM_CollectionTypeFilename GuT_mAgIc
  }

  foreach dir [lsort -unique $CollectionPath] {
    set colls [glob -nocomplain -type d [file join $dir *]]
    if {[llength $colls] == 0} {continue}
    foreach coll $colls {
      set file  [file join $coll $CDM_CollectionFilename]
      set file2 [file join $coll GuT_mAgIc]
      set file3 [file join $coll $CDM_CollectionTypeFilename]
      if {[file exists $file] || ($GUI_Options(EnableGateCollectionManagement) \
            && [file exists $file2]) || [file exists $file3]} {
         set CollName [file tail $coll]
         if {[info exists Collections($CollName)]} {
           set Collections($coll) $coll
         } else {
           set Collections($CollName) $coll
         }
      }
    }
  }
  if {![info exists Collections]} {return {}}
  foreach col [lsort [array names Collections]] {
    lappend names $Collections($col)
  }
  return $names
};# GUI_GetCollectionNames

## GUI_GetCollectionNameTails
#   Returns all Collection's Name tails...
proc GUI_GetCollectionNameTails {} {
  set names {}
  foreach col [GUI_GetCollectionNames] {
    lappend names [file tail $col]
  }
  return $names
}

## GUI_IsCollectionOpened
#
proc GUI_IsCollectionOpened {Collection} {
  foreach col [info commands {Col[0-9]*}] {
    if {[string equal $Collection [tip_GetName $col]]} {return $col}
  }
  return {}
}

## GUI_IsDocumentOpened
#
proc GUI_IsDocumentOpened {Document Collection} {
  foreach doc [info commands {Doc[0-9]*}] {
    if {[string equal $Document [tip_GetId $doc]]} {lappend Docs $doc}
  }
  if {![info exists Docs]} {return {}}

  foreach doc $Docs {
    if {[string equal $Collection [tip_GetParent $doc]]} {return $doc}
  }
  return {}
}

## GUI_SelectFileEncoding
#
proc GUI_SelectFileEncoding {Name {enc {}} {win .selectFileEncoding}} {
  global GUI_Text GUI_Options GUI_CFileSelectEncoding WMIcons

  if {[string equal $enc {}]} {
    set GUI_CFileSelectEncoding [encoding system]
  } else {
    set GUI_CFileSelectEncoding $enc
  }

  set win [ggi_unused_win $win]
  toplevel $win
  wm withdraw $win
  wm title $win {Select File Encoding...}

  frame $win.buttons
  set menu $win.buttons.enc.mn
  menubutton $win.buttons.enc -relief raised -indicatoron 1 -anchor c \
    -menu $menu -textvariable GUI_CFileSelectEncoding \
    -text $GUI_CFileSelectEncoding \
    -image [::Encoding::GetEncodingImage $enc] -compound left -padx 10
  menu $menu -tearoff 0
  GUI_AppendEncodingsToMenu $menu "set GUI_CFileSelectEncoding \$Encoding
    GUI_SelectFileEncodingCallback $win.text.text \"$Name\"
    $win.buttons.enc configure -image \$EncodingImage -text \$Encoding" 1

  pack $win.buttons.enc -side top -pady 10 -fill x

  ::ScrolledWidgets::ScrolledText $win.text -width 40 -height 10
  GUI_SelectFileEncodingCallback $win.text.text $Name
  button $win.buttons.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
     -width 16
  button $win.buttons.cl -textvariable GUI_Text(Cancel) \
     -command "set GUI_CFileSelectEncoding {}; destroy $win" -width 16
  pack $win.buttons.ok $win.buttons.cl -side top -fill x
  pack $win.buttons -side left -padx 5 -pady 5
  pack $win.text -side left -fill both -pady 5 -padx 5 -expand true
  update
  catch {winico setwindow $win $WMIcons(Collection)}
  wm deiconify $win
  tkwait window $win
  set enc $GUI_CFileSelectEncoding
  unset GUI_CFileSelectEncoding
  return $enc
};# GUI_SelectFileEncoding

proc GUI_SelectFileEncodingCallback {text Name} {
  global GUI_Text GUI_CFileSelectEncoding

  if {[catch {open $Name r} file]} {
    gut_error [format $GUI_Text(CannotReadFile) $Name]
  }
  fconfigure $file -encoding $GUI_CFileSelectEncoding
  set Text [read $file 1024]
  $text configure -state normal
  $text delete 1.0 end
  $text insert end $Text
  close $file
  $text configure -state disabled
}

###########################################################################
## Document Section
###########################################################################

## GUI_OpenDocument
#
proc GUI_OpenDocument {args} {
  global GUI_Current_Opened_Collection GUI_Current_Opened_Document \
         GUI_CollectionDlg
  set GUI_Current_Opened_Collection {}
  set GUI_Current_Opened_Document   {}

  if {[string equal $GUI_CollectionDlg(Doc2Open) {}]} return

  set collection $GUI_CollectionDlg(Col2Open)
  set doc_id     $GUI_CollectionDlg(Doc2Open)
  set col [tip_OpenCollection $collection]
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $collection]} {
    GUI_FixCollectionName $col $ColName $collection
  }
  set doc [tip_GetDocument $col $doc_id]
  set GUI_Current_Opened_Collection $col
  set GUI_Current_Opened_Document $doc
  destroy $GUI_CollectionDlg(Window)
}

## GUI_SW_LoadCallback
#   This function is actually used as a callback in the "Load" button of
#   the "express" system window toolbar.
proc GUI_SW_LoadCallback {collection doc_id} {
  global GUI_Current_Opened_Collection GUI_Current_Opened_Document
  if {[catch {tip_OpenCollection $collection} col]} {
    gut_ErrorDlg $col
    return
  }
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $collection]} {
    GUI_FixCollectionName $col $ColName $collection
  }
  set GUI_Current_Opened_Collection $col
  ## Is the Document Id "All"?
  if {[string equal $doc_id $::GUI_Text(All)]} {
    set GUI_Current_Opened_Document {}
    return
  }
  if {[catch {tip_GetDocument $col $doc_id} doc]} {
    set GUI_Current_Opened_Document {}
    gut_ErrorDlg $doc
    return
  }
  set GUI_Current_Opened_Document $doc
};# GUI_SW_LoadCallback

## GUI_ViewDocument
#
proc GUI_ViewDocument {} {
  global GUI_CollectionDlg

  if {[string equal $GUI_CollectionDlg(Doc2Open) {}]} return

  set collection $GUI_CollectionDlg(Col2Open)
  set doc_id     $GUI_CollectionDlg(Doc2Open)
  ggi_view_document $collection $doc_id $GUI_CollectionDlg(Window)
};# GUI_ViewDocument

## GUI_UpdateDocument
#
proc GUI_UpdateDocument {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Doc2Open) {}]} return

  set col_name   $GUI_CollectionDlg(Col2Open)
  set doc_id     $GUI_CollectionDlg(Doc2Open)
  set win        $GUI_CollectionDlg(Window)

  if {![string equal [GUI_IsCollectionOpened $col_name] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set message [format $GUI_Text(UpdateDocument?) $doc_id]
  if {[string equal [gut_question $message {Update Document?} 1 $win] yes]} {
    set col [tip_OpenCollection $col_name]
    ## Is the Collection's Location in accordance with it's Name?
    set ColName [tip_GetName $col]
    if {![string equal $ColName $col_name]} {
      GUI_FixCollectionName $col $ColName $col_name
    }
    set doc [tip_GetDocument $col $doc_id]
    set source [tip_GetExternalId $doc]
    if {![file exists $source]} {
      gut_error [format $GUI_Text(FileDoesNotExist) $source]
      tip_Close $col
      return
    }
    # Remove All Annotations...
    tip_DeleteAnnotations $doc {}
    # Remove All Attributes...
    foreach attr [tip_GetAttributes $doc] {
      tip_RemoveAttribute $doc [tip_GetName $attr]
    }

    # Get the new text...
    set encoding [GUI_SelectFileEncoding $source [tip_GetEncoding $doc] \
                  $win.selectFileEnc]
    if {[string equal $encoding {}]} {
      gut_warning $GUI_Text(OperationCancel)
      tip_Close $col
      return
    }
    if {[catch {open $source r} file]} {
      gut_error [format $GUI_Text(CannotReadFile) $source]
      tip_Close $col
      return
    }
    fconfigure $file -encoding $encoding
    set Text [read $file]
    tip_SetByteSequence $doc $Text
    tip_SetEncoding $doc $encoding
    tip_Sync $col
    tip_Close $col
  }
};# GUI_UpdateDocument

## GUI_AddDocument
#
proc GUI_AddDocument {} {
  global GUI_CollectionDlg GUI_Text

  set col_name   $GUI_CollectionDlg(Col2Open)
  if {[string equal $col_name {}]} return
  set win        $GUI_CollectionDlg(Window)

  if {![string equal [GUI_IsCollectionOpened $col_name] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set fname [tk_getOpenFile -title {Select Document to Add...} -parent $win]
  if {[string equal $fname {}]} return
  set encoding [GUI_SelectFileEncoding $fname {} $win.selectFileEnc]
  if {[string equal $encoding {}]} {
    gut_warning $GUI_Text(OperationCancel)
    return
  }
  set col [tip_OpenCollection $col_name]
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $col_name]} {
    GUI_FixCollectionName $col $ColName $col_name
  }
  set file [open $fname r]
  fconfigure $file -encoding $encoding
  set RawData [read $file]
  close $file
  set Doc [tip_CreateDocument $col $fname $RawData {} {} $encoding]
  ## Is a Collection Handler available?
  set HandlerAttr [tip_GetAttribute $col {Collection Handler}]
  if {[llength $HandlerAttr]} {
    set Handler [tip_GetValueValue $HandlerAttr]
    set Command [CDM_GetRegisteredCollectionHadlerCommand $Handler]
    if {[catch {eval $Command $col $Doc} error]} {
      ## An error occured inside the handler! Inform the user...
      set ans [MessageDlg $win.errorDlg -parent $win -icon error -default 0 \
         -type user -buttons [list $GUI_Text(Ignore) \
          $GUI_Text(DeleteDocument) $GUI_Text(Abort)] \
         -title {Error during Collection Handler execution...} \
         -message [format $GUI_Text(ErrorExecutingColHandler) \
         [tip_GetExternalId $Doc] $::GUI_CColEncoding($win:Handler) \
           "$error\n($::errorInfo)"]]
      switch $ans {
        0 {## Ignore error!}
        1 {## Delete the Document!
           tip_RemoveDocument $col [tip_GetId $Doc]
           continue
        }
        2 {gut_error <ignore>}
      }
    }
  }
  tip_Sync $col
  tip_Close $col
  #catch {GUI_DrawCollectionDlg \
         [string map {{ } _} [file join [file tail $col_name] $fname]]}
  catch {GUI_DrawCollectionDlg [file join [file tail $col_name] $fname]}
};# GUI_AddDocument

## GUI_DeleteDocument
#
proc GUI_DeleteDocument {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Doc2Open) {}]} return

  set col_name   $GUI_CollectionDlg(Col2Open)
  set doc_id     $GUI_CollectionDlg(Doc2Open)
  set win        $GUI_CollectionDlg(Window)

  if {![string equal [GUI_IsCollectionOpened $col_name] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set message [format $GUI_Text(DeleteDocument?) $doc_id $col_name]
  if {[string equal [gut_question $message {Delete Document?} 1 $win] yes]} {
    set col [tip_OpenCollection $col_name]
    ## Is the Collection's Location in accordance with it's Name?
    set ColName [tip_GetName $col]
    if {![string equal $ColName $col_name]} {
      GUI_FixCollectionName $col $ColName $col_name
    }
    if {[catch {tip_RemoveDocument $col $doc_id} error]} {
      tip_Close $col
      gut_error $error
    }
    tip_Sync $col
    tip_Close $col
    catch {GUI_DrawCollectionDlg}
  }
};# GUI_DeleteDocument

## GUI_DeleteDocuments
#
proc GUI_DeleteDocuments {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return

  set col_name   $GUI_CollectionDlg(Col2Open)
  set win        $GUI_CollectionDlg(Window)

  if {![string equal [GUI_IsCollectionOpened $col_name] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }

  set col [tip_OpenCollection $col_name]
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $col_name]} {
    GUI_FixCollectionName $col $ColName $col_name
  }
  set Docs [$col GetFileIds]
  tip_Close $col
  set docs [gut_MultipleSelectFromList $Docs $GUI_Text(SelectDocuments2Delete) \
            {Delete Documents...} 50 $win.selectDocs]
  if {![llength $docs]} return
  if {[llength $docs] > 35} {
    set dtext \"[join [lrange $docs 0 25] {, }]...\"
  } else {
    set dtext \"[join $docs {, }]\"
  }
  set message [format $GUI_Text(DeleteDocuments?) $dtext $col_name]
  if {[string equal [gut_question $message {Delete Documents?} 1 $win] yes]} {
    set col [tip_OpenCollection $col_name]
    foreach doc_id $docs {
      if {[catch {tip_RemoveDocument $col $doc_id} error]} {
        tip_Close $col
        gut_error $error
      }
    }
    tip_Sync $col
    tip_Close $col
    catch {GUI_DrawCollectionDlg}
  }
};# GUI_DeleteDocuments

########################################################################
## Collection Section
########################################################################

## GUI_OpenCollection
#
proc GUI_OpenCollection {} {
  global GUI_CollectionDlg GUI_Text
  global GUI_Current_Opened_Collection GUI_Current_Opened_Document
  set GUI_Current_Opened_Collection {}
  set GUI_Current_Opened_Document   {}

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return

  set collection $GUI_CollectionDlg(Col2Open)
  set col [tip_OpenCollection $collection]
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $collection]} {
    GUI_FixCollectionName $col $ColName $collection
  }
  set GUI_Current_Opened_Collection $col
  set GUI_Current_Opened_Document {}
  destroy $GUI_CollectionDlg(Window)
};# GUI_OpenCollection

## GUI_EditCollection
#
proc GUI_EditCollection {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return
  set collection $GUI_CollectionDlg(Col2Open)

  set win $GUI_CollectionDlg(Window)
  if {![string equal [GUI_IsCollectionOpened $collection] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set message [format $GUI_Text(UpdateCollection?) $collection]
  if {[string equal [gut_question $message {Update Collection?} 1 $win] no]} \
     {return}
  set col [tip_OpenCollection $collection]
  ## Is the Collection's Location in accordance with it's Name?
  set name [tip_GetName $col]
  if {![string equal $name $collection]} {
    GUI_FixCollectionName $col $name $collection
  }
  set enc    [tip_GetEncoding $col]
  set files  [$col GetFileXIds]
  set format [$col Format]
  ## Is a Collection Handler available?
  set HandlerAttr [tip_GetAttribute $col {Collection Handler}]
  tip_Close  $col
  if {[llength $HandlerAttr]} {
    set Handler [tip_GetValueValue $HandlerAttr]
    GUI_NewCollection -name $name -files $files -encoding $enc \
      -format $format -handler $Handler
  } else {
    GUI_NewCollection -name $name -files $files -encoding $enc -format $format
  }
};# GUI_EditCollection


## GUI_RenameCollection
#   Rename Collection...
proc GUI_RenameCollection {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return
  set collection $GUI_CollectionDlg(Col2Open)

  set win $GUI_CollectionDlg(Window)
  if {![string equal [GUI_IsCollectionOpened $collection] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set col [tip_OpenCollection $collection]
  set name [tip_GetName $col]
  set width [string length $name]
  if {$width > 40} {set width 40}
  set nname [gut_EntryDialog -message $GUI_Text(EnterCollectionName) -title \
             {Rename Collection...} -initial $name -width $width]
  if {[string length $nname] && ![string equal $nname $name]} {
    ## Now rename the directory that holds the Collection
    set dirname [file nativename $nname]
    if {[file exists $dirname]} {
      set res [gut_question $GUI_Text(CollectionExists) \
               {Delete Directory?}]
      if {[string equal $res no]} {
        tip_Close $col
        return
      }
    }
    $col SetName $nname

    catch {file delete -force $dirname}
    file rename -force [file nativename $name] $dirname
    tip_Sync $col
    tip_Close $col
    catch {GUI_DrawCollectionDlg}
  } else {
    tip_Close $col
  }
};# GUI_RenameCollection


## GUI_NewCollection
#   Create a New Collection...
proc GUI_NewCollection {args} {
  global GUI_CollectionDlg GUI_CColEncoding GUI_Text GUI_Options UserHomeDir \
         WMIcons CollectionPath
  if {[info exists GUI_CollectionDlg(Window)]} {
    set win $GUI_CollectionDlg(Window).new_col
  } else {
    set win .new_col
  }
  set files  {}
  if {[info exists CollectionPath]} {
    set Name [file join [lindex $CollectionPath 0] NewCol]
  } else {
    set Name [file join $UserHomeDir Collections NewCol]
  }
  set enc    [encoding system]
  foreach {format sel_filter UserCmd} {native * {}} {}

  foreach {op arg} $args {
    switch -glob -- $op {
      -enc*    {set enc    $arg}
      -fil*    {set files  $arg}
      -for*    {set format $arg}
      -nam*    {set Name   $arg}
      -win*    {set win    $arg}
      -pat*    {set sel_filter $arg}
      -com*    {set UserCmd    $arg}
      -han*    {set Handler    $arg}
      default  {error "Unkown option $op"}
    }
  }

  set window $win
  if {[winfo exists $win]} {return}
  toplevel $win
  wm withdraw $win
  wm title $win "Collection Properties..."
  wm minsize $win 400 470
  set win $win.bgfr
  frame $win -relief flat
  set ColName $win.fr
  frame $ColName -relief groove -borderwidth 2
  label $ColName.path -textvariable GUI_Text(CollectionPath) -anchor w
  label $ColName.label -textvariable GUI_Text(CollectionName) -anchor w
  entry $ColName.location -width 35
  entry $ColName.name     -width 35
  $ColName.name insert 0 [file tail $Name]
  $ColName.name xview end
  $ColName.location insert 0 [file dirname $Name]
  $ColName.location xview end
  GUI_MakeEntryDirectoryDropTarget $ColName.location
  button $ColName.select -text "..." -image ITOpen -compound left -command \
    "GUI_SelectDirectory $ColName.location"
  Tooltip::tooltip $ColName.select -textvariable GUI_Text(SelectDirectory)

  ## Select Encoding Choice
  label $ColName.encLabel -textvariable GUI_Text(Encoding) -anchor w
  set GUI_CColEncoding($win) $enc
  set menu $ColName.enc.mn
  menubutton $ColName.enc -relief raised -indicatoron 1 -anchor c \
    -menu $menu -text $enc -textvariable GUI_CColEncoding($win) -width 150 \
    -image [::Encoding::GetEncodingImage $enc] -compound left -padx 10 \
    -bd $GUI_Options(BorderWidth) -fg #ffff00 -bg #009900 \
    -highlightthickness 0
  menu $menu -tearoff 0
  GUI_AppendEncodingsToMenu $menu "set GUI_CColEncoding($win) \$Encoding
    $ColName.enc configure -image \$EncodingImage -text \$Encoding" 1

  ## Collection's Format
  label $ColName.fmtLabel -textvariable GUI_Text(Format) -anchor w
  ComboBox $ColName.format -entrybg $GUI_Options(InputBackground) \
     -foreground $GUI_Options(InputForeground) \
     -values [CDM_GetRegisteredCollectionTypes] \
     -text $format -editable 0 -width 12 -highlightthickness 0 \
     -textvariable GUI_CColEncoding($win:Format)

  ## Registered Handlers...
  label $ColName.handlerLabel -textvariable GUI_Text(RegColHandlers) -anchor e
  ComboBox $ColName.handler -entrybg $GUI_Options(InputBackground) \
     -foreground $GUI_Options(InputForeground) \
     -values [linsert [CDM_GetRegisteredCollectionHadlerLabels] 0 {}]\
     -text $UserCmd -editable 0 -highlightthickness 0 \
     -textvariable GUI_CColEncoding($win:Handler) -modifycmd \
     "GUI_NewCollectionHandlerChanged $win $ColName.handler_options"
  if {[info exists Handler]} {
    set index [lsearch -exact [$ColName.handler cget -values] $Handler]
    if {$index != -1} {
      $ColName.handler setvalue @$index
    }
  }
  if {[string length $UserCmd]} {
    $ColName.handler configure -state disabled
    set GUI_CColEncoding($win:Handler) \
      [CDM_GetRegisteredCollectionHadlerLabel $UserCmd]
  }
  ## The handlers options button...
  GUI_LoadImage ITExternalEditor
  button $ColName.handler_options -textvariable GUI_Text(Options) -image \
     ITExternalEditor -state disabled
  if {![string equal $::tcl_platform(platform) windows]} {
    $ColName.handler_options configure -compound left
  }
  GUI_NewCollectionHandlerChanged $win $ColName.handler_options

  ## Pack...
  grid $ColName.path     -column 0 -row 0 -sticky ew -ipadx 3
  grid $ColName.location -column 1 -row 0 -sticky ew -ipadx 3 -columnspan 2
  grid $ColName.select   -column 3 -row 0 -sticky  w -ipadx 3 -padx 4
  grid $ColName.label    -column 0 -row 1 -sticky ew -ipadx 3
  grid $ColName.name     -column 1 -row 1 -sticky ew -ipadx 3 -columnspan 2
  grid $ColName.encLabel -column 0 -row 2 -sticky ew -ipadx 3 -pady 3 -padx 4
  grid $ColName.enc      -column 1 -row 2 -sticky  w -ipadx 3 -padx 2
  grid $ColName.fmtLabel -column 0 -row 3 -sticky ew -ipadx 3 -pady 3 -padx 4
  grid $ColName.format   -column 1 -row 3 -sticky  w -ipadx 3 -padx 2
  grid $ColName.handlerLabel -column 2 -row 2 -sticky ew -ipadx 10 -padx 20 \
       -columnspan 2
  grid $ColName.handler      -column 2 -row 3 -sticky ew -ipadx 10 -padx 20 \
       -columnspan 2
  grid $ColName.handler_options -column 2 -row 4 -sticky e -ipadx 10 -padx 20 \
       -columnspan 2
  grid columnconfigure $ColName 2 -weight 1
  pack $ColName -fill x -expand 0 -ipadx 5 -ipady 5 -padx 5 -pady 5 -side top

  set Docs $win.docfr
  frame $Docs -relief ridge -borderwidth 1
  label $Docs.label -textvariable GUI_Text(CurrentFiles) -fg red
  frame $Docs.gr -relief flat -bd 0
  label $Docs.gr.label -textvariable GUI_Text(FileSearchDirectory)
  if {![info exists GUI_Options(NewCollectionFileSearchPath)]} {
    set GUI_Options(NewCollectionFileSearchPath) $UserHomeDir
  }
  entry $Docs.gr.dir -textvariable GUI_Options(NewCollectionFileSearchPath)
  if {[llength $files]} {
    set GUI_Options(NewCollectionFileSearchPath) \
        [file dirname [lindex $files 0]]
  }
  $Docs.gr.dir xview end
  GUI_MakeEntryDirectoryDropTarget $Docs.gr.dir
  button $Docs.gr.select -text "..." -image ITOpen -compound left -command \
    "GUI_SelectDirectory $Docs.gr.dir"
  Tooltip::tooltip $Docs.gr.select -textvariable GUI_Text(SelectDirectory)
  label $Docs.gr.fmes -textvariable GUI_Text(FileFilter)
  entry $Docs.gr.filter
  $Docs.gr.filter insert 0 $sel_filter
  set vertical_toolbar $Docs.vtf
  Toolbar::ToolbarFrame $vertical_toolbar \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate) -orientation vertical
  set toolbar $vertical_toolbar.tb
  Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
  set b+ [Toolbar::addbutton $toolbar -tooltipvar GUI_Text(AddtoCollection) \
     -image IAddDocuments -command "GUI_AddToListBox $Docs.sfr.lb \[glob \
     -nocomplain -types {f l r} \[file join \[$Docs.gr.dir get\]\
     \[$Docs.gr.filter get\]\]\]"]
  Toolbar::addwidget $toolbar label -textvariable GUI_Text(Add) -bd 0
  Toolbar::addseparator $toolbar
  set b- [Toolbar::addbutton $toolbar -image IRemoveDocuments \
     -tooltipvar GUI_Text(RemovefromCollection) -command \
     "GUI_DeleteFromListBox ${Docs}.sfr.lb \
     \[file join \[${Docs}.gr.dir get\] \[${Docs}.gr.filter get\]\]"]
  Toolbar::addwidget $toolbar label -textvariable GUI_Text(Remove) -bd 0
  ::ScrolledWidgets::ScrolledListbox ${Docs}.sfr -widgetname lb \
    -height 1 -selectmode extended -bd $GUI_Options(BorderWidth)
  grid $Docs.gr.label  -column 0 -row 0 -sticky ew -ipadx 3
  grid $Docs.gr.dir    -column 1 -row 0 -sticky ew -ipadx 3 -columnspan 3
  grid $Docs.gr.select -column 4 -row 0 -sticky ew -ipadx 3 -padx 4
  grid $Docs.gr.fmes   -column 0 -row 1 -sticky w  -ipadx 3
  grid $Docs.gr.filter -column 1 -row 1 -sticky ew -ipadx 3
  grid columnconfigure $Docs.gr 1 -weight 1

  ## The Contents info...
  label $Docs.contents -relief flat -bd 0 -fg #006400

  ## If we got files, add them in listbox...
  if {[llength $files]} {GUI_AddToListBox $Docs.sfr.lb $files}

  pack $toolbar
  pack $vertical_toolbar -side right -fill y
  pack $Docs.label -side top -anchor w -padx 5
  pack $Docs.gr -side top -anchor w -padx 5 -pady 2 -fill x -expand 0
  pack $Docs.sfr -side top -fill both -expand 1 -padx 5 -pady 0
  pack $Docs.contents -side bottom -fill x  -padx 7 -pady 0
  pack $Docs -fill both -expand 1 -padx 5 -padx 5

  set Buts $win.buts
  frame $Buts -relief groove -borderwidth 2
  frame $Buts.b -relief flat
  button $Buts.b.ok -textvariable GUI_Text(Ok) -command \
    "GUI_NewCollectionOkCallback $ColName.location $ColName.name \
     $Docs.sfr.lb $win $window"
  button $Buts.b.cancel -textvariable GUI_Text(Cancel) -command \
    "destroy $window;array unset GUI_CColEncoding *$win*"
  pack $Buts.b.ok $Buts.b.cancel -side left -padx 10 -pady 6
  pack $Buts.b
  pack $Buts -fill x -padx 5 -pady 10
  pack $win -fill both -expand 1
  gut_ModalDlg $window 520 410
  catch {winico setwindow $window $WMIcons(NewCollection)}
  ## Make the listbox a Drop target...
  catch {
    dnd bindtarget $Docs.sfr.lb text/uri-list <DragEnter> \
      {%W configure -bg $GUI_Options(DragEnterBackgound);update;return %a}
    dnd bindtarget $Docs.sfr.lb text/uri-list <DragLeave> \
      {%W configure -bg $GUI_Options(InputBackground);update}
    dnd bindtarget $Docs.sfr.lb text/uri-list <Drop> \
      {%W configure -bg $GUI_Options(InputBackground);GUI_AddToListBox %W %D}
  }
  wm deiconify $window
  bind $window <Key-Return> "${b+} invoke"
  bind $window <Key-Delete> "${b-} invoke"
  return $window
};# GUI_NewCollection

proc GUI_NewCollectionHandlerChanged {win button} {
  global GUI_CColEncoding
  
  $button configure -state disabled -command {}
  catch {destroy $win.options}
  if {![info exists GUI_CColEncoding($win:Handler)]} {return}
  set Label $GUI_CColEncoding($win:Handler)
  if {[string length $Label]} {
    set Command [CDM_GetRegisteredCollectionHadlerCommand $Label]
    catch {${Command}_OptionsDialog} error
    if {[llength [info commands ${Command}_OptionsDialog]]} {
      $button configure -state normal -command \
              "${Command}_OptionsDialog $win.options"
    }
  }
};# GUI_NewCollectionHandlerChanged

proc GUI_NewCollectionOkCallback {ColLocEntry ColNameEntry DocListbox
                                                                   win window} {
  global GUI_CColEncoding
  set ::GUI_Options(NewCollectionFileSearchPath)
  
  if {[string length $GUI_CColEncoding($win:Handler)]} {
    set UserCmd [CDM_GetRegisteredCollectionHadlerCommand \
                     $GUI_CColEncoding($win:Handler)]
    GUI_NewCollectionSave $ColLocEntry $ColNameEntry $DocListbox $win $UserCmd \
                          $GUI_CColEncoding($win:Handler)
  } else {
    GUI_NewCollectionSave $ColLocEntry $ColNameEntry $DocListbox $win
  }
  catch GUI_DrawCollectionDlg
  destroy $window
  array unset GUI_CColEncoding *$win*
};# GUI_NewCollectionOkCallback

## GUI_NewCollectionSave
#
proc GUI_NewCollectionSave {loc_obj name_obj file_obj win
                                                    {Command {}} {Handler {}}} {
  global GUI_Text GUI_CColEncoding
  set name [file join [$loc_obj get] [$name_obj get]]
  set files [$file_obj get 0 end]
  ## Check whether the Collection has any documents...
  if {[llength $files] == 0} {
    set ans [gut_question [format "$GUI_Text(CollectionEmpty)\n" $name] \
              Warning... 1 $win]
    if {[string equal $ans no]} {gut_error <ignore>}
  }
  ## Check whether the Collection Directory already exists...
  if {[file exists $name]} {
    set ans [gut_question "$GUI_Text(CollectionExists)\n($name)" \
              Warning... 1 $win]
    if {[string equal $ans no]} {gut_error <ignore>}
    catch {tip_Destroy $name}
  }

  ## Get the Desired Encoding
  if {![info exists GUI_CColEncoding($win)]} {
    set enc iso8859-1
  } else {
    set enc $GUI_CColEncoding($win)
  }
  set col [tip_CreateCollection $name {} $enc]
  $col Format $GUI_CColEncoding($win:Format)
  set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]
  set progress 0.0
  set file_number [llength $files]
  if {!$file_number} {set file_number 1}
  set step [expr {100.0/$file_number}]
  if {[string length $Command]} {
    ## We have to apply a Collection Handler...
    foreach doc $files {
      set progress [expr {$progress+$step}]
      ggi_wait_update {} $progress $wwin
      if {![file isfile $doc]} {
        puts stderr "Skipping not regular file $doc..."
        continue
      }
      set file [open $doc r]
      fconfigure $file -encoding $enc
      set raw_data [read $file]
      close $file
      set Doc [tip_CreateDocument $col $doc $raw_data {} {}]
      if {[catch {eval $Command $col $Doc} error]} {
        ## An error occured inside the handler! Inform the user...
        set ans [MessageDlg $win.errorDlg -parent $win -icon error -default 0 \
           -type user -buttons [list $GUI_Text(Ignore) \
            $GUI_Text(DeleteDocument) $GUI_Text(Abort)] \
           -title {Error during Collection Handler execution...} \
           -message [format $GUI_Text(ErrorExecutingColHandler) \
           [tip_GetExternalId $Doc] $::GUI_CColEncoding($win:Handler) \
             "$error\n($::errorInfo)"]]
        switch $ans {
          0 {## Ignore error!}
          1 {## Delete the Document!
             tip_RemoveDocument $col [tip_GetId $Doc]
             continue
          }
          2 {gut_error <ignore>}
        }
      }
      tip_Sync $Doc
      tip_Close $Doc
    }
    ## We should also add an Attribute, denoting the Handler used...
    if {[string length $Handler]} {
      tip_PutAttribute $col [tip_CreateAttribute {Collection Handler} \
          [tip_CreateAttributeValue GDM_STRING $Handler]]
    }
  } else {
    foreach doc $files {
      set progress [expr {$progress+$step}]
      ggi_wait_update {} $progress $wwin
      if {![file isfile $doc]} {
        puts stderr "Skipping not regular file $doc..."
        continue
      }
      set file [open $doc r]
      fconfigure $file -encoding $enc
      set raw_data [read $file]
      close $file
      set Doc [tip_CreateDocument $col $doc $raw_data {} {}]
      tip_Sync $Doc
      tip_Close $Doc
    }
  }
  tip_Sync $col
  tip_Close $col
  ggi_destroy_wait $wwin
};# GUI_NewCollectionSave

## GUI_ChangeCollectionFormat
#
proc GUI_ChangeCollectionFormat {} {
  global GUI_CollectionDlg GUI_CColEncoding GUI_Text GUI_Options WMIcons

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return
  set collection $GUI_CollectionDlg(Col2Open)

  set win $GUI_CollectionDlg(Window)
  if {[winfo exists $win.change_format]} return
  if {![string equal [GUI_IsCollectionOpened $collection] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set win $win.change_format
  toplevel $win
  wm withdraw $win
  wm protocol $win WM_DELETE_WINDOW \
     "set GUI_CColEncoding($win:Format2Convert) {};destroy $win"
  wm title $win {Change Collection Format...}

  ## Collection's Name
  set ColName $win.fr
  frame $ColName -relief groove -borderwidth 2
  label $ColName.label -textvariable GUI_Text(CollectionName)
  label $ColName.name -width [string length $collection] \
    -background $GUI_Options(InputBackground) \
    -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
    -relief sunken -text $collection -anchor w

  set col [tip_OpenCollection $collection]
  ## Is the Collection's Location in accordance with it's Name?
  set CollectionName [tip_GetName $col]
  if {![string equal $CollectionName $collection]} {
    GUI_FixCollectionName $col $CollectionName $collection
  }
  set Format [$col Format]
  set GUI_CColEncoding($win:Format2Convert) $Format

  ## Collection's Format
  label $ColName.fmtLabel -textvariable GUI_Text(Format) -anchor w
  ComboBox $ColName.format -entrybg $GUI_Options(InputBackground) \
     -foreground $GUI_Options(InputForeground) \
     -values [CDM_GetRegisteredCollectionTypes] \
     -highlightthickness 0 -text $Format -editable 0 -width 12 \
     -textvariable GUI_CColEncoding($win:Format2Convert)

  ## Pack...
  grid $ColName.label    -column 0 -row 0 -sticky ew -ipadx 3
  grid $ColName.name     -column 1 -row 0 -sticky ew -ipadx 3
  grid $ColName.fmtLabel -column 0 -row 2 -sticky ew -ipadx 3 -pady 3 -padx 4
  grid $ColName.format   -column 1 -row 2 -sticky  w -ipadx 3 -padx 2
  grid columnconfigure $ColName 1 -weight 1
  pack $ColName -fill x -expand 1 -ipadx 5 -ipady 5 -padx 5 -pady 5

  set Buts $win.buts
  frame $Buts -relief groove -borderwidth 2
  frame $Buts.b -relief flat
  button $Buts.b.ok -textvariable GUI_Text(Ok) -command "destroy $win"
  button $Buts.b.cancel -textvariable GUI_Text(Cancel) -command \
     "set GUI_CColEncoding($win:Format2Convert) {};destroy $win"
  pack $Buts.b.ok $Buts.b.cancel -side left -padx 10 -pady 6
  pack $Buts.b
  pack $Buts -fill x -padx 5 -pady 10
  update
  gut_ModalDlg $win
  catch {winico setwindow $win $WMIcons(Collection)}
  wm deiconify $win
  tkwait window $win
  if {[string equal $GUI_CColEncoding($win:Format2Convert) {}] ||
      [string equal $GUI_CColEncoding($win:Format2Convert) $Format]} {
    unset GUI_CColEncoding($win:Format2Convert)
    tip_Close $col
    return
  }
  set Format $GUI_CColEncoding($win:Format2Convert)
  unset GUI_CColEncoding($win:Format2Convert)
  set file_nu [llength [$col GetFileIds]]
  if {$file_nu < 1} {set file_nu 1}
  set progress 0.0
  set step [expr {100.0/$file_nu}]
  set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]
  set doc [tip_FirstDocument $col]
  while {[string length $doc]} {
    $doc Modified 1
    $doc Format $Format
    $doc Sync
    tip_Close $doc
    set progress [expr {$progress+$step}]
    ggi_wait_update {} $progress $wwin
    set doc [tip_NextDocument $col]
  }
  $col Format $Format
  $col Modified 1
  tip_Sync $col
  set Name [tip_GetName $col]
  if {[string equal $Format native]} {
    global CDM_CollectionTypeFilename
    catch {file delete -force [file join $Name GuT_mAgIc]}
    catch {file delete -force [file join $Name $CDM_CollectionTypeFilename]}
  } elseif {[string match gate* $Format]} {
    global CDM_CollectionFilename CDM_CollectionTypeFilename
    catch {file delete -force [file join $Name $CDM_CollectionFilename]}
    catch {file delete -force [file join $Name $CDM_CollectionTypeFilename]}
  } else {
    global CDM_CollectionFilename
    catch {file delete -force [file join $Name GuT_mAgIc]}
    catch {file delete -force [file join $Name $CDM_CollectionFilename]}
  }
  tip_Close $col
  ggi_destroy_wait $wwin
  catch {GUI_DrawCollectionDlg}
};# GUI_ChangeCollectionFormat

## GUI_AddToListBox
#
proc GUI_AddToListBox {lb what} {
  global GUI_Text
  foreach one [lsort $what] {
    ## Files dropped here can also have as source dnd, so may be formed as
    ## uri-lists...
    $lb insert end [GUI_NormaliseURI $one]
    #-image IDocument -text $one -indent 10
  }
  catch {
    [winfo parent [winfo parent $lb]].contents conf -text \
      "[$lb size] $GUI_Text(Files...)"
  }
}

## GUI_DeleteFromListBox
#
proc GUI_DeleteFromListBox {lb what} {
  global GUI_Text
  set delete [$lb curselection]
  if {![string equal $delete {}]} {
    set delete [lsort -integer -decreasing $delete]
    foreach index $delete {
      $lb delete $index
    }
    [winfo parent [winfo parent $lb]].contents conf -text \
      "[$lb size] $GUI_Text(Files...)"
    return
  }
  set contents [$lb get 0 end]
  set i 0
  foreach line $contents {
    if {[regexp $what $line]} {
      $lb delete $i
    } else {
      incr i
    }
  }
  [winfo parent [winfo parent $lb]].contents conf -text \
      "[$lb size] $GUI_Text(Files...)"
}

## GUI_DeleteCollection
#
proc GUI_DeleteCollection {} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return

  set collection $GUI_CollectionDlg(Col2Open)
  set win $GUI_CollectionDlg(Window)
  if {![string equal [GUI_IsCollectionOpened $collection] {}]} {
    gut_warning $GUI_Text(CollectionIsOpen)
  }
  set ans [gut_question "$GUI_Text(RemoveCollection?) ${collection}$GUI_Text(?)"\
            Warning... 1 $win]
  if {[string equal $ans no]} return
  tip_Destroy $collection
  GUI_DrawCollectionDlg
}

##############################################################################
## PopUp Menu Handlers...
##############################################################################
proc GUI_CollectionActionPopUp {widget open} {
  global GUI_CollectionDlg GUI_Text

  if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return
  set menu $widget.actions_popup
  set index 0
  catch {menu $menu -tearoff false}
  catch {$menu delete 0 end}

  ## Collection Section
  if {$open} {
    $menu add command -label $GUI_Text(OpenCollectionBT) \
          -command GUI_OpenCollection
  } else {
    $menu add command -label $GUI_Text(NewCollectionBT) \
          -command GUI_NewCollection
    $menu add command -label $GUI_Text(DeleteCollectionBT) \
          -command GUI_DeleteCollection
    $menu add command -label $GUI_Text(EditCollectionBT) \
          -command GUI_EditCollection
    $menu add command -label $GUI_Text(NewDocumentBT) \
          -command GUI_AddDocument
    $menu add command -label $GUI_Text(DeleteDocumentsBT) \
          -command GUI_DeleteDocuments
  }
  $menu add separator
  $menu add command -label $GUI_Text(ChangeCollectionFormat) \
        -command "update; GUI_ChangeCollectionFormat"
  $menu add command -label $GUI_Text(RenameCollectionBT) \
        -command GUI_RenameCollection

  ## Document Section
  if {![string equal $GUI_CollectionDlg(Doc2Open) {}]} {
    $menu add separator
    if {$open} {
      $menu add command -label $GUI_Text(OpenDocumentBT) \
            -command GUI_OpenDocument
    } else {
      $menu add command -label $GUI_Text(DeleteDocumentBT) \
            -command GUI_DeleteDocument
      $menu add command -label $GUI_Text(UpdateDocumentBT) \
            -command GUI_UpdateDocument
      $menu add command -label $GUI_Text(ViewDocumentBT) \
            -command GUI_ViewDocument
    }
  }

  ## Post Menu...
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
}

##############################################################################
## Properties Dialog
##############################################################################

## GUI_ColDlgProperties
#   Create Properties Dialog
proc GUI_ColDlgProperties {widget} {
  global GUI_CollectionDlg HomeDir GUI_Options WMIcons

  set win $GUI_CollectionDlg(Window).propertiesDlg

  if {[winfo exists $win]} {
    foreach one $GUI_CollectionDlg(PropertiesOpenedCollections) {
      catch {tip_Close $one}
    }
    destroy $win
    return
  }
  set GUI_CollectionDlg(PropertiesWindow) $win
  set GUI_CollectionDlg(PropertiesOpenedCollections) {}

  toplevel $win
  bind $win <Destroy> \
    {foreach one $GUI_CollectionDlg(PropertiesOpenedCollections) \
       {catch {$one Close}}
     catch {unset one}}
  wm withdraw $win
  wm title $win {Properties...}

  foreach image {IProperties ICore} {
    GUI_LoadImage $image
  }

  set bg white
  set Header $win.header; frame $Header -relief flat -bd 0 -bg $bg
    label $Header.title -font {Helvetica 18 bold italic} \
       -anchor e -bg $bg -fg red -width 30
    label $Header.image -image IProperties -bg $bg
    pack $Header.image  -side left -padx 5 -pady 5 -anchor w
    pack $Header.title  -side left -padx 5 -pady 5 -anchor w -fill x -expand 1
  pack $Header -side top -fill x -expand 0

  frame $win.buttons
    button $win.buttons.ok -textvariable GUI_Text(Dismiss) \
             -command "destroy $win"
    pack $win.buttons.ok -padx 2 -pady 2
  pack ${win}.buttons -side bottom

  set Body $win.body
  ::ScrolledWidgets::ScrolledText $Body
  pack $Body -side top -fill both -expand 1

  catch {GUI_ColDlgPropertiesUpdate}
  update
  catch {winico setwindow $win $WMIcons(Info)}
  wm deiconify $win
}

## GUI_ColDlgPropertiesUpdate
#
proc GUI_ColDlgPropertiesUpdate {} {
  global GUI_CollectionDlg

  set win    $GUI_CollectionDlg(PropertiesWindow)
  set Title  $win.header.title
  set Image  $win.header.image
  set Text   $win.body.text

  catch {$Image configure -image IProperties}
  if {![string equal $GUI_CollectionDlg(Doc2Open) {}]} {
    $Title configure -text $GUI_CollectionDlg(Doc2Open)
    set doc [GUI_IsDocumentOpened $GUI_CollectionDlg(Doc2Open) \
                                  $GUI_CollectionDlg(Col2Open)]
    if {![string length $doc]} {
      set col [GUI_IsCollectionOpened $GUI_CollectionDlg(Col2Open)]
      if {![string length $col]} {
        if {[catch {tip_OpenCollection $GUI_CollectionDlg(Col2Open)} col]} {
          GUI_ColDlgPropertiesUpdateError $col
          return
        }
        set ColName [tip_GetName $col]
        if {![string equal $ColName $GUI_CollectionDlg(Col2Open)]} {
          GUI_FixCollectionName $col $ColName $GUI_CollectionDlg(Col2Open)
        }
        lappend GUI_CollectionDlg(PropertiesOpenedCollections) $col
      }
      if {[catch {tip_GetDocument $col $GUI_CollectionDlg(Doc2Open)} doc]} {
        if {![string equal $GUI_CollectionDlg(Doc2Open) \{\}]} {
          GUI_ColDlgPropertiesUpdateError $doc
        } else {
          $Title configure -text {No Available Documents!}
          $Text configure -state normal
          $Text delete 1.0 end
          $Text configure -state disabled
        }
        return
      }
    }
    $Text configure -state normal
    $Text delete 1.0 end
    #$Text insert end [$doc Status]
    $Text insert end "Document Id: [tip_GetId $doc]\n"
    $Text insert end "Document External Id: [tip_GetExternalId $doc]\n"
    $Text insert end "Document Encoding: [tip_GetEncoding $doc]\n"
    $Text insert end "Document Format: [$doc Format]\n"
    $Text insert end "Document Tcl Command: $doc\n"
    if {[$doc Modified]} {
       $Text insert end "Document is Modified\n"
    } else {
       $Text insert end "Document is not Modified\n"
    }
    set Anns [tip_GetAnnotations $doc]
    $Text insert end "Document has [llength $Anns] Annotations ("
    set start [$Text index end-1c]
    $Text image create end -image ITViewAnns
    $Text insert end " View Annotations"
    $Text tag add SeeAnns $start end-1c
    $Text tag configure SeeAnns -underline 1 -foreground blue
    set Title "Annotations of Document [tip_GetId $doc]..."
    $Text tag bind SeeAnns <<MB_1>> \
         "ggi_view_raw $doc {} {} \"$Title\" 0 $Text.anns"
    $Text tag bind SeeAnns <Enter> "$Text configure -cursor hand2"
    $Text tag bind SeeAnns <Leave> "$Text configure -cursor {}"
    $Text insert end ")\n"
    set Attrs [tip_GetAttributes $doc]
    $Text insert end "Document has [llength $Attrs] Attributes:\n"
    foreach attr $Attrs {
      $Text insert end [format "         %s : (%s)\n" [lindex $attr 0] \
                  [lindex [lindex $attr 1] end]]
    }

    $Text insert end "Document Text:\n[tip_GetRawData $doc]"
    $Text configure -state disabled
  } elseif {![string equal $GUI_CollectionDlg(Col2Open) {}]} {
    $Title configure -text $GUI_CollectionDlg(Col2Open)
    set col [GUI_IsCollectionOpened $GUI_CollectionDlg(Col2Open)]
    if {[string equal $col {}]} {
      if {[catch {tip_OpenCollection $GUI_CollectionDlg(Col2Open)} col]} {
        GUI_ColDlgPropertiesUpdateError $col
        return
      }
      set ColName [tip_GetName $col]
      if {![string equal $ColName $GUI_CollectionDlg(Col2Open)]} {
        GUI_FixCollectionName $col $ColName $GUI_CollectionDlg(Col2Open)
      }
      lappend GUI_CollectionDlg(PropertiesOpenedCollections) $col
    }
    $Text configure -state normal
    $Text delete 1.0 end
    #$Text insert end [$col Status]
    $Text insert end "Collection Name: [tip_GetName $col]\n"
    $Text insert end "Collection Encoding: [tip_GetEncoding $col]\n"
    $Text insert end "Collection Format: [$col Format]\n"
    $Text insert end "Collection Tcl Command: $col\n"
    if {[$col Modified]} {
       $Text insert end "Collection is Modified\n"
    } else {
       $Text insert end "Collection is not Modified\n"
    }
    $Text insert end "Collection Owner: [tip_GetOwner $col]\n"
    set Attrs [tip_GetAttributes $col]
    $Text insert end "Document has [llength $Attrs] Attributes:\n"
    foreach attr $Attrs {
      $Text insert end [format "         %s : (%s)\n" [lindex $attr 0] \
                  [lindex [lindex $attr 1] end]]
    }

    $Text insert end "Collection Documents:\n"
    foreach id [$col GetFileIds] xid [$col GetFileXIds] {
      $Text insert end {  }
      $Text image create end -image IDocumentSmall
      $Text insert end " $id ($xid)\n"
    }

    $Text insert end "Collection Opened Documents:\n"
    foreach id [$col GetOpenedIds] cmd [$col GetOpenedDocs] {
      $Text insert end {  }
      $Text image create end -image ITinyDoc
      $Text insert end " $id ($cmd)\n"
    }

    $Text configure -state disabled
  } else {
    $Title configure -text {}
    $Text configure -state normal
    $Text delete 1.0 end
    $Text configure -state disabled
  }
};# GUI_ColDlgPropertiesUpdate

proc GUI_ColDlgPropertiesUpdateError error {
  global GUI_CollectionDlg
  set win    $GUI_CollectionDlg(PropertiesWindow)
  set Title  $win.header.title
  set Image  $win.header.image
  set Text   $win.body.text

  catch {$Image configure -image ICore}
  $Title configure -text Error!
  $Text configure -state normal
  $Text delete 1.0 end
  $Text insert end $error
  $Text configure -state disabled
}

###############################################################################
## GUI_FixCollectionName
###############################################################################
proc GUI_FixCollectionName {col name location {silent 0}} {
  global GUI_Options GUI_Text

  ## There is the possibility that the two provided paths are the same, but
  ## have different encoding. Do not bother the user for such a change...
  set normalised_1 [file normalize $name]
  set normalised_2 [file normalize $location]
  if {[string equal $normalised_1 $normalised_2]} {
    ## Is the stored path the correct one?
    if {![string equal [tip_GetName $col] $normalised_2]} {
      $col SetName $normalised_2
      tip_Sync $col
    }
    return
  }

  if {!$silent} {
    set msg "$GUI_Text(FixColLocation)\n\n($name) -> ($location)"
    if {[catch {gut_question $msg Warning... 0} ans]} {
      gut_error <ignore>
    }
    if {[string equal $ans no]} {
      error $GUI_Text(CannotManageCollection)
    }
  }
  $col SetName $location
  tip_Sync $col
}; # GUI_FixCollectionName

###############################################################################
## Register Some Useful Collection Handlers...
###############################################################################
if {[catch {package require ELEP::ImportHandlers}]} {
  catch {::CollectionHandlers::Initialise}
  catch {
    CDM_RegisterCollectionHandler {Import HTML Metadata} \
                ::CollectionHandlers::ImportHTMLDocumentMetadataDirectory
    CDM_RegisterCollectionHandler {Import HTML Images...} \
                ::CollectionHandlers::ImportHTML&Images
    CDM_RegisterCollectionHandler {SGML tags to Annotations} \
                ::CollectionHandlers::ConvertSGMLTags2Anns
  }
}
