##############################################################################
##  ContextSensitivePopUp.tcl:                                              ## 
## ----------------------------                                             ##
##                                                                          ##
##  This file contains some standart pop-up windows that we have associated ##
##  with some common widgets...                                             ##
##                                                                          ##
##  Author:                                                                 ##
##    Georgios Petasis, petasis@iit.demokritos.gr                           ##
##                                                                          ##
##  Copyright 1998-2005 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  National Center for Scientific Research "Demokritos",                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  the "Ellogon" Text Engineering Platform.                                ##
##############################################################################

GUI_LoadImage ITEditCopy
GUI_LoadImage ITEditCut
GUI_LoadImage ITEditPaste
GUI_LoadImage ITVirtualKeyboard

proc GUI_TextSensitivePopup {text} {
  global GUI_Text GUI_Options TextPopup

  set menu $text.actions_popup
  catch {menu $menu -tearoff false} 
  catch {$menu delete 0 end}

  ## Commands
  $menu add command -label $GUI_Text(Copy)  -command \
         "event generate $text <<Copy>>" \
         -image ITEditCopy -compound $GUI_Options(MenuImageLocation)
  if {[string equal [$text cget -state] normal]} {
    $menu add command -label $GUI_Text(Cut)   -command \
         "event generate $text <<Cut>>" \
         -image ITEditCut -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Paste) -command \
         "event generate $text <<Paste>>" \
         -image ITEditPaste -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(VirtualKeyboard) -command \
         "::GUI::VirtualKeyboard $text" \
         -image ITVirtualKeyboard -compound $GUI_Options(MenuImageLocation)
  } else {
    $menu add command -label $GUI_Text(Cut)   -command \
         "event generate $text <<Cut>>"   -state disabled \
         -image ITEditCut -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Paste) -command \
         "event generate $text <<Paste>>" -state disabled \
         -image ITEditPaste -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(VirtualKeyboard) -command \
         "::GUI::VirtualKeyboard $text" -state disabled \
         -image ITVirtualKeyboard -compound $GUI_Options(MenuImageLocation)
  }
  $menu add separator
  $menu add command -label $GUI_Text(Find...) -command \
        "::GUI_TextSearchFacilities::TextFindDialog $text" \
         -image ITZoom -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(Save) -command \
        "ggi_SaveTextWidgetContents $text" \
         -image ITSave -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  set TextPopup(Undo:$text) [$text cget -undo]
  switch -exact [$text cget -wrap] {
    word {set TextPopup(Wrap:$text) 1}
    none {set TextPopup(Wrap:$text) 0}
    char {set TextPopup(Wrap:$text) 0}
  }
  switch -glob [$text cget -state] {
    n*   {set TextPopup(Edit:$text) 1}
    d*   {set TextPopup(Edit:$text) 0}
  }
  $menu add checkbutton -label $GUI_Text(EnableUndo) \
    -variable TextPopup(Undo:$text) \
    -command "if \"!$TextPopup(Undo:$text)\" \"$text configure -undo 1\" \
              else \"$text configure -undo 0\"" \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $menu add checkbutton -label $GUI_Text(WordWrap) \
    -variable TextPopup(Wrap:$text) \
    -command "if \"!$TextPopup(Wrap:$text)\" \"$text configure -wrap word\" \
              else \"$text configure -wrap none\"" \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $menu add checkbutton -label $GUI_Text(Edit) \
    -variable TextPopup(Edit:$text) \
    -command "if \"!$TextPopup(Edit:$text)\" \"$text configure -state normal\" \
              else \"$text configure -state disabled\"" \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  ## Is the text widget using a fixed or a variable length font...
  switch [$text cget -font] {
    FixedTextFont {
      $menu add command -label $GUI_Text(FontVariable) \
        -command "$text conf -font TextFont" \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
    }
    TextFont {
      $menu add command -label $GUI_Text(FontFixed) \
        -command "$text conf -font FixedTextFont" \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
    }
  }
  $menu add command -label $GUI_Text(ChangeFont) -command \
        "::FontDialog::FontSelect $text $text.__fontdlg" \
         -image ITSelectFont -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(Colours...) -command \
        "ggi_SetWidgetColours $text" \
         -image ITSelectColour -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(Size...) -command \
        "ggi_SetWidgetSize $text" \
         -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}

  tk_popup $menu [winfo pointerx .] [winfo pointery .]
  bind $menu <Destroy> \
    "unset -nocomplain TextPopup(Wrap:$text) TextPopup(Edit:$text)"
};# GUI_TextSensitivePopup

proc GUI_ListBoxSensitivePopup {widget} {
  global GUI_Text GUI_Options

  set menu $widget.actions_popup
  catch {menu $menu -tearoff false} 
  catch {$menu delete 0 end}

  ## Commands
  $menu add command -label $GUI_Text(ChangeFont) -command \
     "::FontDialog::FontSelect $widget $widget.__fontdlg" \
     -image ITSelectFont -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(Find...) -command "FindDialog $widget" \
     -image ITZoom -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# GUI_ListBoxSensitivePopup

proc GUI_EntrySensitivePopup {entry} {
  global GUI_Text GUI_Options

  set menu $entry.actions_popup
  catch {menu $menu -tearoff false} 
  catch {$menu delete 0 end}
  GUI_LoadImage ITEntryClear

  ## Commands
  $menu add command -label $GUI_Text(Copy)  -command \
         "event generate $entry <<Copy>>" \
         -image ITEditCopy -compound $GUI_Options(MenuImageLocation)
  if {[string equal [$entry cget -state] normal]} {
    $menu add command -label $GUI_Text(Cut)   -command \
         "event generate $entry <<Cut>>" \
         -image ITEditCut -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Paste) -command \
         "event generate $entry <<Paste>>" \
         -image ITEditPaste -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Clear) -command \
         "if {\[$entry selection present]} {$entry delete sel.first sel.last} \
          else {$entry delete 0 end}" \
         -image ITEntryClear -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(VirtualKeyboard) -command \
         "::GUI::VirtualKeyboard $entry" \
         -image ITVirtualKeyboard -compound $GUI_Options(MenuImageLocation)
  } else {
    $menu add command -label $GUI_Text(Cut)   -command \
         "event generate $entry <<Cut>>"   -state disabled \
         -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Paste) -command \
         "event generate $entry <<Paste>>" -state disabled \
         -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(Clear) -command \
         "if {\[$entry selection present]} {$entry delete sel.first sel.last} \
          else {$entry delete 0 end}" -state disabled \
         -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $menu add command -label $GUI_Text(VirtualKeyboard) -command \
         "::GUI::VirtualKeyboard $entry" \
         -image ITVirtualKeyboard -compound $GUI_Options(MenuImageLocation)
  }
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# GUI_EntrySensitivePopup

proc GUI_NoteBookSensitivePopup {widget} {
  global GUI_Text

  set menu $widget.actions_popup
  catch {destroy $menu}
  menu $menu -tearoff false
  $menu delete 0 end

  ## Commands
  if {[$widget cget -homogeneous]} {
    $menu add command -label $GUI_Text(UnHomogeneousLabels) -command \
        "$widget configure -homogeneous 0"
  } else {
    $menu add command -label $GUI_Text(HomogeneousLabels) -command \
        "$widget configure -homogeneous 1"
  }
  $menu add separator
  if {[string equal [$widget cget -side] top]} {
    $menu add command -label $GUI_Text(LabelSideBottom) -command \
      "$widget configure -side bottom"
  } else {
    $menu add command -label $GUI_Text(LabelSideTop) -command \
      "$widget configure -side top"
  }
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# GUI_NoteBookSensitivePopup

proc GUI_CanvasSensitivePopup {widget} {
  global GUI_Text GUI_Options

  set menu $widget.actions_popup
  catch {destroy $menu}
  menu $menu -tearoff false
  $menu delete 0 end

  ## Commands
  $menu add separator
  $menu add command -label $GUI_Text(Save) -command \
        "GUI_Canvas2SVG $widget" \
         -image ITSave -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(Colours...) -command \
        "ggi_SetWidgetColours $widget" \
         -image ITSelectColour -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}

  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# GUI_CanvasSensitivePopup
