##############################################################################
##  CreateModifySystem.tcl:                                                 ##
## -------------------------                                                ##
##  This file implements a GUI for creating a new System or modifying an    ##
##  existing one...                                                         ##
##                                                                          ##
##  Copyright 1998-2005 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
namespace eval ::CreateModifySystem {
  variable Dummy
}
GUI_LoadImage IDynamicComponentSmall

## ::CreateModifySystem::CreateWindowMenu
#  Create the menubar or our window...
proc ::CreateModifySystem::CreateWindowMenu {win} {
  global GUI_Text GUI_Underline GUI_Options
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }
  
  ## File Menu
  set system $menubar._system_ ;# Avoid the System Entry under Windows...
  catch {destroy $system}
  $menubar add cascade -label $GUI_Text(System) -menu $system \
     -underline $GUI_Underline(System)
  menu $system
  $system add command -label $GUI_Text(Save) -underline \
     $GUI_Underline(Save) -command "" \
     -image ITSave -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(Close) -underline \
     $GUI_Underline(Close) -command "GUI_CloseWindow $win $win" \
     -image ITCancel -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $system}

  ## view Menu
  set view $menubar.view; catch {destroy $view}
  $menubar add cascade -label $GUI_Text(View) -menu $view \
     -underline $GUI_Underline(View)
  menu $view
  $view add command -label $GUI_Text(ShowToolbars) -underline \
     $GUI_Underline(ShowToolbars) -command \
     "Toolbar::deiconify $win.tf.toolbar1" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $view}

  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $help}

  return $menubar
};# ::CreateModifySystem::CreateWindowMenu

## ::CreateModifySystem::CreateWindow
#  Create a window able to create/modify a System...
proc ::CreateModifySystem::CreateWindow {{system {}}} {
  global GUI_Options GUI_Text
  ## If system is empty, we are called to create a new system...
  if {[string length $system]} {
    set graph ::SystemManagement::Graph::$system
    set title "Modifying System: \"$system\"..."
    set destroyGraph 0
  } else {
    ## Create an empty graph...
    set id 1; set graph ::SystemManagement::Graph::new_system_1_system
    while {[llength [info commands $graph]]} {
      set graph ::SystemManagement::Graph::new_system_${id}_system
      $graph set -key Geometry 550x400+200+150
      $graph set -key TimeLog    0
      $graph set -key Grid     1x1
    }
    ::struct::graph $graph
    $graph set -key Name new_system_${id}_system
    set title "Creating new System..."
    set destroyGraph 1
  }
  set win [ggi_unused_win .createModifySystem]
  toplevel $win
  wm withdraw $win
  wm title $win $title
  set menubar [CreateWindowMenu $win]
  $win configure -menu $menubar
  ## Status Bar
  set ::GUI_StatusBar::StatusBar($win) {}
  set ::GUI_StatusBar::StatusBar($win:Def) {}
  GUI_CreateStatusBar $win.statusbar
  ## Toolbar...
  set tf $win.tf
  Toolbar::ToolbarFrame $tf \
    -relief $GUI_Options(ToolbarFrameRelief) \
    -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
    -decorate $GUI_Options(ToolbarDecorate)

    set toolbar $tf.toolbar1 
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) -image ITSave \
       -command "" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command "destroy $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left 
  pack $tf -side top -fill x
  ## Main window area
  panedwindow $win.marea -orient vertical 
    ## The System canvas...
    ::ScrolledWidgets::ScrolledCanvas $win.marea.systemCanvas -width 600 \
      -height 450 -highlightthickness 0
    $graph set -key _CanvasWindow $win.marea.systemCanvas.canvas
    ## The Component notebook...
    set NoteBookBg [$win cget -background]
    NoteBook $win.marea.components -background $NoteBookBg -width 600 
    set ModuleSpec [GetModuleInfoAccordingLanguage]
    if {[string equal $ModuleSpec {}]} {
      gut_warning $GUI_Text(NoModulesLoaded)
      destroy $win
      return {}
    }
    foreach {lang moduleList} $ModuleSpec {
      set path [$win.marea.components insert end $lang -text $lang \
                  -background $NoteBookBg]
      set img [::Encoding::GetCountryImage $lang]
      $win.marea.components itemconfigure $lang -image $img -createcmd \
         "::CreateModifySystem::CreateNotebookLanguagePage $graph \
	  $win.marea.systemCanvas.canvas $path $moduleList"
      $win.marea.components raise $lang
    }
  #pane $win.marea.systemCanvas $win.marea.components -orient vertical
  update
  $win.marea add $win.marea.systemCanvas $win.marea.components -sticky snew
  pack $win.marea -fill both -expand 1 -ipadx 10 -ipady 10
  $win.marea.components raise English
  $win.marea.components compute_size
  $win.marea sash place 0 10 250
  ## Show our window
  catch {winico setwindow $win $WMIcons(NewSystem)}
  wm deiconify $win
  ## Arrange the graph to be destroyed on window destruction...
  if {$destroyGraph} {
    bind $win.marea.systemCanvas.canvas <Destroy> "$graph destroy"
  }
  return $win
};# ::CreateModifySystem::CreateWindow

## ::CreateModifySystem::GetModuleInfoAccordingLanguage
#  Returns a list containing all the modules, sorted accoring to their
#  langauge...
proc ::CreateModifySystem::GetModuleInfoAccordingLanguage {} {
  global creole_config

  set modules [array names creole_config]
  if {[string equal $modules {}]} {return {}}

  foreach mod $modules {
    set title {  }
    append title [lindex $creole_config($mod) \
        [expr [lsearch $creole_config($mod) title] +1]]
    append title "  ($mod)"
    lappend Titles $title
    set Module($title) $mod
    set language English
    foreach el \
       [PUL_plugin_module_config $mod pre_conditions document_attributes] {
      if {[string match language_* $el]} {
        set lang [string range $el 9 end]
        set language [string toupper [string range $lang 0 0]]
        append language [string range $lang 1 end] 
      }
    }
    set Language($mod) $language
  }
  set Titles [lsort -ascii $Titles]

  ## Language English must exist always!
  set Lang(English) {}
  foreach title $Titles {
    set m $Module($title)
    set lang $Language($m)
    lappend Lang($lang) [list $title $m]
  }
  foreach lang [lsort [array names Lang]] {
    lappend Result $lang [list $Lang($lang)]
  }
  return $Result
};# ::CreateModifySystem::GetModuleInfoAccordingLanguage

## ::CreateModifySystem::CreateNotebookLanguagePage
#  Create a page in our module notebook...
proc ::CreateModifySystem::CreateNotebookLanguagePage {graph canvas win mods} {
  set text [::ScrolledWidgets::ScrolledText $win.text -width 70 -height 8 \
              -highlightthickness 0]
  set fg [$text cget -foreground]
  set bg [$text cget -background]
  checkbutton $text.temp
  set abg [$text.temp cget -activebackground]
  destroy $text.temp
  set i 1
  foreach item $mods {
    foreach {label component} $item {break}
    regexp {(.+)\s+(\([^\)]+\))} $label _ title _
    checkbutton $text.c$i -text $title -image IDynamicComponentSmall \
      -compound left -padx 0 -anchor w -bg $bg -fg $fg -relief flat \
      -highlightthickness 0 -variable ::CreateModifySystem::Dummy($text.c$i) \
      -command [list ::CreateModifySystem::ToggleComponentExistenceInGraph \
       $text.c$i $graph $component $title $fg $bg $abg]
    bind $text.c$i <Destroy> "unset ::CreateModifySystem::Dummy($text.c$i)"
    bind $text.c$i <MouseWheel> \
      "$text yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $text.c$i <4> "$text yview scroll -5 units"
    bind $text.c$i <5> "$text yview scroll  5 units"
    bind $text.c$i <Enter> \
      "::CreateModifySystem::Page_SelectComponentsCheckEnter %W\
       \"[string trim $title]\" \"$component\""
    bind $text.c$i <Leave> {destroy %W.popup}
    $text window create $i.0 -window $text.c$i
    $text insert end "\n"
    incr i
  }
  $text delete $i.1 end
  $text configure -state disabled
  pack $win.text -fill both -expand 1
};# ::CreateModifySystem::CreateNotebookLanguagePage

proc ::CreateModifySystem::Page_SelectComponentsCheckEnter {w t c} {
  global GUI_Options
  toplevel $w.popup -bg $GUI_Options(TooltipBackground) -highlightthickness 1\
    -highlightbackground $GUI_Options(TooltipForeground) \
    -highlightcolor $GUI_Options(TooltipForeground)
  wm overrideredirect $w.popup 1
  label $w.popup.l -text "Component Name: \"$c\"\nComponent Title: \"$t\"" \
    -relief flat -wraplength 350 \
    -bg $GUI_Options(TooltipBackground) -padx 2 -pady 0 \
    -fg $GUI_Options(TooltipForeground) -anchor w
    pack $w.popup.l -side left -padx 0 -pady 0 -fill both -expand 1
  wm geometry $w.popup \
    +[expr {[winfo rootx $w] + [winfo width $w] + 10}]+[winfo rooty $w]
};# ::CreateModifySystem::Page_SelectComponentsCheckEnter

## ::CreateModifySystem::ToggleComponentExistenceInGraph
#  This function will insert/remove a component into a system graph...
proc ::CreateModifySystem::ToggleComponentExistenceInGraph {cb graph component
                                                            label fg bg abg} {
  ## Is the node already in the graph?
  if {[$graph node exists $component]} {
    $graph node delete $component
    $cb configure -fg $fg -background $bg -activebackground $abg
  } else {
    $graph node insert $component
    $graph node set $component -key Label $label
    ::SystemManagement::SetDefaultNodeKeys $graph $component
    $cb configure -fg black -background #8fbc8f -activebackground #66cdaa
  }
  ## Re-calculate the system graph arcs...
  CreateGraphArcs $graph
  ::SystemManagement::LayoutGraph $graph
  set canvas [$graph get -key _CanvasWindow]
  ::SystemManagement::DrawSystemGraph $graph $canvas
  ::ScrolledWidgets::CanvasSetScrollRegion $canvas
};# ::CreateModifySystem::ToggleComponentExistenceInGraph

## ::CreateModifySystem::CreateGraphArcs
#  This function will remove all arcs that exist in the graph and place new
#  ones, according to the pre/post-conditions of the system components...
proc ::CreateModifySystem::CreateGraphArcs {graph} {
  ## Remove all existing arcs...
  set arcs [$graph arcs]
  if {[llength $arcs]} {
    eval $graph arc delete $arcs
  }
  ## Phase 1: Selecting Initial Modules
  foreach {InitModules NonInitModules InitPostConditions PostConditionsList} \
          {{} {} {} {}} {}
  set Modules [$graph nodes]
  foreach M $Modules {
    set PreConditions($M) {}
    set PostConditions($M) {}
    foreach one [PUL_plugin_module_config_list $M pre_conditions annotations] {
      lappend PreConditions($M)  $one}
    foreach one [PUL_plugin_module_config_list $M post_conditions annotations] {
      lappend PostConditions($M) $one}
    if {![llength $PreConditions($M)]} {
      lappend InitModules $M
      foreach one $PostConditions($M) {lappend InitPostConditions $one}
    } else {
      lappend NonInitModules $M
    }
    foreach one $PostConditions($M) {lappend PostConditionsList $one}
    foreach condition $PostConditions($M) {
      lappend PostConditionsModules($condition) $M
    }
    foreach condition $PreConditions($M) {
      lappend PreConditionsModules($condition) $M
    }
  }
  # puts "Initial Modules:   $InitModules"
  # puts "Dependend Modules: $NonInitModules"
  ## Phase 2: Order components according to their pre-conditions...
  foreach module $Modules {
    foreach one $Modules {
      if {[string equal $module $one]} {continue}
      set Common($module,$one) [ListIntersection \
             $PostConditions($module) $PreConditions($one)]
      if {[llength $Common($module,$one)]} {
        set Lower($module<$one) 1
      } else {
        set Lower($module<$one) 0
      }
    }
  }
  ## Now, we can use Lower(a<b) in order to see if a module "b" should be placed
  ## after "a"...
  ## Phase 3: Starting from the initial modules, we find the modules that
  ## should be placed just after them. This is repeated for all modules...
  set LayerModules $InitModules
  while {[llength $LayerModules]} {
    set newLayerModules {}
    foreach module $LayerModules {
      set newNonInitModules {}
      foreach mod $NonInitModules {
        if {$Lower($module<$mod)} {
          ## Add an arc from the module to mod...
          set arc [$graph arc insert $module $mod]
          ::SystemManagement::SetDefaultArcKeys $graph $arc
	  lappend newLayerModules $mod
        } else {
          lappend newNonInitModules $mod
        }
      }
      set NonInitModules $newNonInitModules
    }
    set LayerModules $newLayerModules
  }
};# ::CreateModifySystem::CreateGraphArcs

## ::CreateModifySystem::ListIntersection
#  This function accepts two lists and returns the list of their common
#  elements...
proc ::CreateModifySystem::ListIntersection {list1 list2} {
  set Common {}
  foreach element $list1 {
    if {[lsearch -exact $list2 $element] != -1} {
      lappend Common $element
    }
  }
  return $Common
};# ::CreateModifySystem::ListIntersection

##############################################
## Test section...
::SystemManagement::ReadConfigurationFile ~/.gate.props
::CreateModifySystem::CreateWindow;# ayto_system
