##############################################################################
##  BgError.tcl:                                                            ##
## ---------------------                                                    ##
##  This file implements a replacement dialog for the standard "bgerror"    ##
##  dialog.                                                                 ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

proc bgerror {err} {
  if {[string equal $err <ignore>]} {
    bgerror_clear
    return
  }
  global errorInfo tcl_platform GUI_Text

  set info $errorInfo
  if {[string length $info]} {
    set buttons [list $GUI_Text(Ok) {Stack Trace}]
  } else {
    set buttons $GUI_Text(Ok)
  }

  set button [MessageDlg .bgerrorDialog -title Error... -justify left -message \
      "Error: $err" -icon error -type user -buttons $buttons -width 250 \
      -default 0]
  if {$button != 1} {bgerror_clear;return}

    set w .bgerrorTrace
    catch {destroy $w}
    toplevel $w -class ErrorTrace
    wm minsize $w 1 1
    wm title $w "Stack Trace for Error"
    wm iconname $w "Stack Trace"
    button $w.ok -text $GUI_Text(Ok) -command "destroy $w;bgerror_clear" \
      -default active
    if {![string compare $tcl_platform(platform) "macintosh"]} {
      text $w.text -relief flat -bd 2 -highlightthickness 0 -setgrid true \
	    -yscrollcommand "$w.scroll set" -width 60 -height 20
    } else {
      text $w.text -relief sunken -bd 2 -yscrollcommand "$w.scroll set" \
	    -setgrid true -width 60 -height 20
    }
    scrollbar $w.scroll -relief sunken -command "$w.text yview"
    pack $w.ok -side bottom -padx 3m -pady 2m
    pack $w.scroll -side right -fill y
    pack $w.text -side left -expand yes -fill both
    $w.text insert 0.0 $info
    $w.text mark set insert 0.0

    bind $w <Return> "destroy $w"
    bind $w.text <Return> "destroy $w; break"

    # Center the window on the screen.

    wm withdraw $w
    update
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
	    - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	    - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w

    # Be sure to release any grabs that might be present on the
    # screen, since they could make it impossible for the user
    # to interact with the stack trace.

    if {[string compare [grab current .] {}]} {
	grab release [grab current .]
    }
}

## bgerror_clear --
## This function will recursively check the toplevel hierarchy and will:
##  1) Destroy any Wait Message Dialogs (thus releasing their grab...)
##  2) Set the default cursor for all other toplevels...
proc bgerror_clear {{parent .}} {
  foreach child [winfo children $parent] {
    if {[string equal [winfo toplevel $child] $child]} {
      ## A toplevel...
      if {[string match *.ggi_wait_message* $child]} {
        catch {destroy $child}
	continue
      } else {
        catch {$child configure -cursor {}}
      }
      bgerror_clear $child
    }
  }
  catch {
    foreach win [grab current] {
      grab release $win
    }
  }
}
