##############################################################################
##  Encoding.tcl:                                                           ##
## ---------------                                                          ##
##  This file implements some dialogs that relate to encoding handling...   ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::Encoding {
  variable GUI_SelectEncoding
}

proc ::Encoding::GUI_SelectEncoding {args} {
  global GUI_Text
  set win .__encoding__GUI_SelectEncoding
  set msg $GUI_Text(PleaseSelectEncoding...)
  foreach {arg val} $args {
    switch -glob -- $arg {
      default {}
    }
  }
  set win [ggi_unused_win $win]
  set menu $win.button.menu
  toplevel $win
  wm withdraw $win
  wm title $win {Please Select Encoding...}
  label $win.message -text $msg
  set ::Encoding::GUI_SelectEncoding($win) [encoding system]
  menubutton $win.button -relief raised -indicatoron 1 -anchor c \
    -menu $menu -textvariable ::Encoding::GUI_SelectEncoding($win) \
    -text [encoding system] -compound left -padx 10 \
    -width 15 -bd $::GUI_Options(BorderWidth) -fg #ffff00 -bg #009900 \
    -image [::Encoding::GetEncodingImage [encoding system]]
  menu $menu -type normal -tearoff 0
  GUI_AppendEncodingsToMenu $menu \
    "set ::Encoding::GUI_SelectEncoding($win) \$Encoding
     $win.button configure -image \$EncodingImage -text \$Encoding"
  button $win.ok     -textvariable GUI_Text(Ok) -command "destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -command \
    "set ::Encoding::GUI_SelectEncoding($win) {}; destroy $win"
  grid $win.message -columnspan 2 -sticky snew
  grid $win.button  -columnspan 2 -sticky snew -padx 20 -pady 5
  grid $win.ok $win.cancel -padx 10 -pady 10 -sticky snew
  gut_ModalDlg $win
  wm deiconify $win
  tkwait window $win
  set val $::Encoding::GUI_SelectEncoding($win)
  unset ::Encoding::GUI_SelectEncoding($win)
  return $val
};# ::Encoding::GUI_SelectEncoding

## ::Encoding::GetEncodingImage
#  This function will try to load an image that corresponds to the provided
#  encoding. If this fails, a blank image will be created and returned. Note
#  that for efficiency reasons, images are loaded only if they do not exist...
namespace eval ::Encoding {}
proc ::Encoding::GetEncodingImage {encoding} {
  if {[llength [info command ::Encoding::IT_$encoding]]} {
    return ::Encoding::IT_$encoding
  }
  if {[catch {image create photo ::Encoding::IT_$encoding \
                -file $::HomeDir/share/Locale/EncodingFlags/$encoding.gif}]} {
    image create photo ::Encoding::IT_$encoding -width 21 -height 14
  }
  return ::Encoding::IT_$encoding
};# ::Encoding::GetEncodingImage

## ::Encoding::GetCountryImage
#  This function will try to load an image that corresponds to the provided
#  country. If this fails, a blank image will be created and returned. Note
#  that for efficiency reasons, images are loaded only if they do not exist...
proc ::Encoding::GetCountryImage {country} {
  if {[llength [info command ::Encoding::ITC_$country]]} {
    return ::Encoding::ITC_$country
  }
  if {[catch {image create photo ::Encoding::ITC_$country \
                -file $::HomeDir/share/Locale/CountryFlags/$country.gif}]} {
    image create photo ::Encoding::ITC_$country -width 21 -height 14
  }
  return ::Encoding::ITC_$country
};# ::Encoding::GetEncodingImage
