##############################################################################
##  HTTPD.tcl:                                                              ##
## ---------------------                                                    ##
##  This file loads TclHTTPD (an HTTP deamon) inside Ellogon.               ##
##                                                                          ##
##  Copyright 1998-2008 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

################################################################################
################################################################################
##  HTTPD Deamon for Ellogon...
##  Author: George Petasis, petasis@iit.demokritos.gr
##         13/04/2000
################################################################################
################################################################################

## The following function overides the package "srvui" that comes with the
## "TclHTTPD" package. It provides an alternative (nicer!) status log window,
## which integrates better with the application...
package provide httpd::srvui 2.0
proc SrvUI_Init {title} {
  global ApplicationName GUI_Options GUI_Text Httpd Doc 
   
  set win .httpdStatus
  toplevel $win
  wm withdraw $win
  wm title $win "$ApplicationName - HTTP Deamon: $Httpd(name):$Httpd(port)"
  wm protocol $win WM_DELETE_WINDOW {GUI_StopHttpd}
  wm iconname $win $Httpd(name):$Httpd(port)
  TitleFrame $win.fr -text "$ApplicationName - HTTP Deamon Status" \
    -fg green -font {helvetica 14 bold} -relief groove -bd 2 -bg [.mw cget -bg]
  set frame [$win.fr getframe]
  foreach {w message val} [list deamon HTTPDeamonInUse $title \
                             host   RunningHost        $Httpd(name) \
                             port   PortInUse          $Httpd(port) \
                             doc    DocumentFolders    $Doc(root) ] {
    label $frame.$w -textvariable GUI_Text($message) -anchor w -bg #eee8aa
    label $frame.${w}_val -text " $val" -bd $GUI_Options(BorderWidth) \
        -bg $GUI_Options(InputBackground) -fg $GUI_Options(InputForeground) \
        -relief sunken -anchor w
    grid $frame.$w $frame.${w}_val -sticky snew -padx 2 -pady 2
  }

  foreach {counter label bg fg} {urlhits  "URL Requests"       #f5f5dc #8b4513
                                 urlreply "URL Replies"        #8fbc8f #2f4f4f
                                 cgihits  "CGI Hits"           #d2b48c #8b0000
                                 tclhits  "Tcl Safe-CGIO Hits" #d2b48c #8b0000
                                 maphits  "Image Map Hits"     #b0c4de #0000cd
                                 errors   "Errors"             #ffb6c1 #FF0000
  } {
    label $frame.l$counter -text $label -anchor w -bg $bg -fg $fg
    label $frame.n$counter -textvariable [CountVarName $counter] \
          -bd $GUI_Options(BorderWidth) -bg $GUI_Options(InputBackground) \
          -fg $fg -relief sunken -anchor e
    grid $frame.l$counter $frame.n$counter -sticky snew -padx 2 -pady 2
  }
  counter::init maphits
  counter::init errors
  counter::init tclhits
  grid columnconfigure $frame 1 -weight 1
  $frame.nurlreply config -fg #006400 
  $frame.lerrors config -bg #ffb6c1 -fg red
  $frame.nerrors config -fg red
  pack $win.fr -fill x -expand 1 -padx 2 -pady 2 -ipadx 2 -ipadx 2
  GUI_LoadImage ITError
  button $win.quit -compound left -textvariable GUI_Text(StopHttpd) \
               -image ITError -command {GUI_StopHttpd}
  pack $win.quit -pady 8
  wm deiconify $win
};# SrvUI_Init

proc HTTPD_StartHTTPd {{port 8015}} {
  global HomeDir GUI_HTTPD
  if {[info exists GUI_HTTPD(Port)]} {
    if {$GUI_HTTPD(Port) == $port} {return}
    error "HTTPD_StartHTTPd: Server already running in port $GUI_HTTPD(Port)!"
  }
  set GUI_HTTPD(Port) $port
  ##
  ## Ensure GUI_HTTPD(ConfirmStop) is defined!
  if {![info exists GUI_HTTPD(ConfirmStop)]} {
    set GUI_HTTPD(ConfirmStop) 0
  }
  ##
  ## Prepare the list of arguments to pass to the server...
  ##
  set args [list -port $port -webmaster {} -gui 1  -threads 0]
  set ::Config(AuthDefaultFile) $::CDM_TempDir/tclhttpd.default
  ##
  ## Check whether threads are supported. If they are supported, start the
  ## deamon into a new thread...
  ##
  if {$::tcl_platform(threaded) && ![catch {package require Thread}]} {
    # console show
    # puts "Initial threads: [thread::names]"
    ## Treads are available!
    set GUI_HTTPD(Threads) 1
    set GUI_HTTPD(Interp) [thread::create]
    thread::preserve $GUI_HTTPD(Interp)
    # puts "HTTPD Thread: $GUI_HTTPD(Interp)"
    ##
    ## Register some functions for evaluation code in the HTTPD server
    ## context...
    ##
    proc HTTPD_eval {args} {
      if {[thread::send $::GUI_HTTPD(Interp) $args result]} {
        error $result
      }
      return $result
    };# HTTPD_eval
    proc HTTPD_eval_async {args} {
      thread::send -async $::GUI_HTTPD(Interp) $args
    };# HTTPD_eval_async
    ##
    ## Register some functions which can be used by the HTTPD server context
    ## to evaluate code in the main application context...
    ##
    HTTPD_eval set ::ELEP(ThreadId) [thread::id]
    HTTPD_eval package require Thread
    HTTPD_eval proc ELEP_eval {args}  {
      if {[thread::send $::ELEP(ThreadId) $args result]} {
        error $result
      }
      return $result
    };# ELEP_eval
    HTTPD_eval proc ELEP_eval_async {args} {
      thread::send -async $::ELEP(ThreadId) $args
    };# ELEP_eval_async
    HTTPD_eval proc ELEP_eval_async_result_variable {var args} {
      thread::send -async $::ELEP(ThreadId) $args $var
    };# ELEP_eval_async_result_variable
  } else {
    set GUI_HTTPD(Threads) 0
    set GUI_HTTPD(Interp) [interp create]
    proc HTTPD_eval {args} {
      return [eval interp eval $::GUI_HTTPD(Interp) $args]
    };# HTTPD_eval
    proc HTTPD_eval_async {args} {
      return [eval interp eval $::GUI_HTTPD(Interp) $args]
    };# HTTPD_eval_async
    HTTPD_eval proc ELEP_eval_async_result_variable {var args} {
      return [eval interp eval $::GUI_HTTPD(Interp) $args]
    };# ELEP_eval_async_result_variable
  }
  HTTPD_eval set argv $args
  HTTPD_eval set argc [llength $args]
  HTTPD_eval set ::Config(AuthDefaultFile) $::CDM_TempDir/tclhttpd.default
  HTTPD_eval_async source $HomeDir/Httpd/bin/httpd.tcl
  # puts "Current Threads: [thread::names]"
  catch {raise .httpdStatus}
};# HTTPD_StartHTTPd

proc HTTPD_StopHTTPd {} {
  global GUI_HTTPD
  if {![info exists GUI_HTTPD(Port)]} {
    unset -nocomplain GUI_HTTPD
    return
  }
  # puts "Current Threads: [thread::names]"
  # puts "HTTPD Thread: $GUI_HTTPD(Interp)"
  catch {HTTPD_eval Httpd_Shutdown}
  thread::release -wait $GUI_HTTPD(Interp)
  # puts "Released HTTPD Thread: $GUI_HTTPD(Interp)"
  # puts "Current Threads: [thread::names]"
  unset -nocomplain GUI_HTTPD
  catch {destroy .httpdStatus}
};# HTTPD_StopHTTPd

proc HTTPD_HTTPdRunning {} {
  return [info exists ::GUI_HTTPD(Port)]
};# HTTPD_HTTPdRunning

## GUI_StartHttpd
#   This procedure starts the HTTP Deamon...
proc GUI_StartHttpd {menu} {
  global HomeDir GUI_Text GUI_Options GUI_HTTPD

  if {[info exists GUI_HTTPD(Menu)]} {
    ## The server is already running! return...
    return
  }
  set GUI_HTTPD(Menu) $menu

  if {![info exists GUI_HTTPD(ConfirmStop)]} {
    set GUI_HTTPD(ConfirmStop) 1
  }

  if {![info exists GUI_HTTPD(Port)]} {
    set port [gut_EntryDialog -message $GUI_Text(EnterPort) -initial 8015 \
                              -title {Enter Port...}]
    if {![string length $port]} return
    ## Display the wait message. There is really no need to display it, but that
    ## ensures that the BWidget package gets used, so we won't have any problems
    ## later...
    set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]

    HTTPD_StartHTTPd $port

    # ggi_wait_update {} 50 $wwin
    # HTTPD_eval Doc_AddRoot /ellogon $HomeDir/doc
    # if {[catch {Direct_Url /demo HTTP_Demo} error]} {
    #   puts stderr "Could not add virtual URL /demo"
    #   puts stderr "The error was: $error"
    # }
    # if {[catch {Doc_AddRoot /ellogon [file join $HomeDir doc]} error]} {
    #   puts stderr "Could not add Document Root at \"$HomeDir/doc\"..."
    #   puts stderr "The error was: $error"
    # }
    ggi_wait_update {} 100 $wwin
    ggi_destroy_wait $wwin
    Agent::Play stop RestPose DoMagic1 DoMagic2 RestPose
  }
  $menu entryconfigure $GUI_Text(StartHttpd) -state disabled
  $menu entryconfigure $GUI_Text(StopHttpd) -state normal
  update
};# GUI_StartHttpd

## GUI_StopHttpd
#   This procedure stops the HTTP Deamon. Note that a few more requests will
#   still be handled by our server, untill the OS updates the status of the
#   socket (on some OSes there is a delay up to 5 minutes!)
proc GUI_StopHttpd {} {
  global GUI_Text GUI_HTTPD
  if {[HTTPD_HTTPdRunning] && $GUI_HTTPD(ConfirmStop)} {
    set result [gut_question $GUI_Text(StopHTTPD?) {Stop HTTP Deamon?}]
    if {[string equal $result no]} {return 0}
  }
  if {[info exists GUI_HTTPD(Menu)]} {
    set menu $GUI_HTTPD(Menu)
    $menu entryconfigure $GUI_Text(StopHttpd) -state disabled
    $menu entryconfigure $GUI_Text(StartHttpd) -state normal
    GUI_SOAPStopServer
  }
  HTTPD_StopHTTPd
  
  update
  return 1
};# GUI_StopHttpd
  
##########################################################################
## What follows is a simple demo of the server capabilities...
##########################################################################

if {0} {
## HTTP_Demo
global HTTP_Demo HTTP_Demo
set HTTP_Demo "text/html;charset=iso-8859-7"
proc HTTP_Demo {args} {
  global GUI_Text GUI_Progress
  ## Write standart header...
  set html "<html>
  <meta http-equiv=\"Content-Type\" content=\"text/html;charset=iso-8859-7\">
  <Head><Title>Welcome to the Ellogon Demo Page...</Title></Head>
  <Body background=\"/ellogon/Images/BackGround.jpg\">
  <img src=\"/ellogon/Images/Ellogon-Logo.gif\">
  <h1><center>The Ellogon HTTP Demo Page!</center></h1><hr>"

  ## Handle the various cases and generate the pages accordingly...
  switch [lindex $args 0] {
    Collection {
      set val [lindex $args 1]
      ## This is the second demo page...
      set i [lsearch -exact [GUI_GetCollectionNameTails] $val]
      if {$i == -1} {
        append html "<font size=+5 color=\"red\">Error: Cannot Locate
        the specified Collection: \"$val\"</font><br><br><ul>
        Please try again (<a href=\"/ellogon\">Back</a>)</ul></html>"
        return $html
      }
      set col [lindex [GUI_GetCollectionNames] $i]
      append html "<p><h2>2. Select Document:</h2>
      <form action=\"/demo\" method=\"POST\"><ul><li>Document:
      <input type=\"hidden\" name=\"selcol\" value=\"$col\">
      <select name=\"Document\">[HTTP_GD $col]</select>
      </ul><p><h2>3. Select Modules to Run:</h2>
      You can specify up to five modules to run. The modules will be run
      in the order you will specify, i.e. first the module in the
      slot \"Module 1:\" will be run, then the following, etc. When
      the first empty slot is found (having \"None\" as value),
      the execution will stop and the results will be returned...<ul>"
      global creole_config
      foreach mod [array names creole_config] {
        set title [lindex $creole_config($mod) \
            [expr {[lsearch $creole_config($mod) title]+1}]]
        set Modules($title) $mod
      }
      set html_combo "<option value=\"none\">None</option>\n"
      foreach mod [lsort [array names Modules]] {
        append html_combo "<option value=\"$Modules($mod)\">$mod</option>\n"
      }
      for {set i 1} {$i < 6} {incr i} {
        append html "<li>Module $i: <select name=\"$i\">
                     $html_combo</select></li>"
      }
      append html "</ul><br><center>
      <input type=\"submit\" name=\"next\" value=\"Next >>\">
      </center></form></html>"
    }
    selcol {
      ## This is the last demo page (the one with the results...)
      set Col [lindex $args 1]
      set Doc [lindex $args 3]
      set col [tip_OpenCollection   $Col]
      set doc [tip_GetDocument $col $Doc]
      
      tip_DeleteAnnotations $doc {}
      append html "<p><h2>Selected Annotators:</h2>"
      set annotators {}
      foreach {id mod} [lrange $args 4 end] {
        switch -exact $mod {
          none      {break}
          "Next >>" {break}
          default {
            ## Run module...
            if {[catch {tip_RunAnnotator $doc $mod} result]} {
              append html "Error running \"$mod\": $result<br>"
            } else {
              if {[string equal $result {Annotate Ok}]} {
                append html "Annotator \"<font color=\"#00AA00\">$mod</font>\"\
                             completed succesfully...<br>"
                append annotators \
                  "<input type=\"hidden\" name=\"$mod\" value=\"$mod\">\n"
              } else {
                append html "Error running \"<font color=\"#0000FF\">$mod</font\
                            >\": <font color=red>[lindex $result 1]</font><br>"
                append html "Stack Trace: <pre>[lindex $result 2]</pre><br>"
              }
            }
          }
        }
      }
      append html "<hr><p><h2>Single Span Viewer:</h2>"
      append html "<form action=\"/demo\" method=\"POST\">
      <input type=\"hidden\" name=\"seldoc\" value=\"$Doc\">
      <input type=\"hidden\" name=\"selcol\" value=\"$Col\">
      Select what to view: <select name=\"type\">
      <option value=\"token type\">Token Types</option>
      <option value=\"token pos\">Token POS Tags</option>
      <option value=\"lookup type\">Lookup Types</option>
      </select>
      <input type=\"submit\" name=\"view\" value=\"View\">
      $annotators
      </form>"
      append html "<hr><p><h2>Document Text...</h2><table width=100%><tr><td>"
      append html [encoding convertto iso8859-7 [tip_GetRawData $doc]]
      append html "</td></tr></table><hr><br>"
      append html "<p><h2>Annotations...</h2>
                   <table width=\"100%\" BORDER=\"1\">"
      foreach ann [$doc GetAnnotations] {
        foreach {id type spans attrs} $ann {}
        foreach {start end} $spans {break}
        set Attr {}
        foreach attr $attrs {
          foreach {name value} $attr {break}
          append Attr "$name=[lrange $value 1 end] "
        }
        append html \
          "<tr><td> $id </td><td> $type </td><td> $start $end </td>\
           <td> [encoding convertto iso8859-7 $Attr] </td></tr>\n"
      }
      append html </table>
      $doc Close
      $col Close
    }
    seldoc {
      ## This is a request for a viewer...
      foreach {seldoc Doc selcol Col type Type} $args {break}
      set col [tip_OpenCollection   $Col]
      set doc [tip_GetDocument $col $Doc]
      tip_DeleteAnnotations $doc {}
      foreach {mod Mod} [lrange $args 8 end] {
        catch {tip_RunAnnotator $doc $mod}
      }
      foreach {Ann Attr} $Type {break}
      set Anns [tip_SelectAnnotations $doc $Ann]
      set Text [tip_GetRawData $doc]
      catch {destroy .http_text}
      text .http_text -setgrid 0
      .http_text insert end $Text
      
      global GUI_Options HTTP_GetHTMLFromText
      foreach ann $Anns {
        set attr [tip_GetAttribute $ann $Attr]
        if {[llength $attr]} {
          set value [lindex [lindex $attr 1] 1]
          set HTTP_GetHTMLFromText($value) {}
          foreach {start end} [lindex [lindex $ann 2] 0] break
          .http_text tag add $value 1.0+${start}c 1.0+${end}c
        }
      }
      foreach val [lsort [array names HTTP_GetHTMLFromText]] \
              col $GUI_Options(Colors) {
        if {![string length $val]} break
        set HTTP_GetHTMLFromText($val) $col
      }
      set HTTP_GetHTMLFromText(:Text:) $html
      .http_text dump -all -command HTTP_GetHTMLFromText 1.0 end
      set html $HTTP_GetHTMLFromText(:Text:)
      unset HTTP_GetHTMLFromText(:Text:)
      append html "<hr><table><tr>\n"
      ## Create the key...
      set i 0
      foreach key [lsort [array names HTTP_GetHTMLFromText]] {
        incr i
        if {$i == 11} {
          append html "</tr>\n<tr>"
          set i 0
        }
        foreach {bg fg} $HTTP_GetHTMLFromText($key) {}
        append html \
        "<td align=center bgcolor=\"$bg\"><font color=\"$fg\"><b>\
        &nbsp;&nbsp;$key&nbsp;</b></font></td>"
      }
      append html "</tr></table><hr>\n" 
      array unset HTTP_GetHTMLFromText
    }
    default {append html $args}
  }
  append html </html>
  return $html
}

proc HTTP_GetHTMLFromText {type tag index} {
  global HTTP_GetHTMLFromText
  switch -exact -- $type {
    text   {append HTTP_GetHTMLFromText(:Text:) \
            [encoding convertto iso8859-7 $tag]}
    tagon  {
      foreach {bg fg} $HTTP_GetHTMLFromText($tag) {}
      append HTTP_GetHTMLFromText(:Text:) \
        "<span style=\"background-color: $bg\"><font color=\"$fg\" bgcolor=$bg><b>"
    }
    tagoff {
      append HTTP_GetHTMLFromText(:Text:) "</b></font></span>"
    }
  }
}

proc HTTP_GC {} {
  set data {}
  foreach col [GUI_GetCollectionNameTails] {
    append data "<option value=\"$col\">$col</option>\n"
  }
  return $data
}
proc HTTP_GD {col} {
  set data {}
  foreach doc [gut_GetCollectionDocuments $col] {
    append data "<option value=\"$doc\">$doc</option>\n"
  }
  return $data
}
};# if {0}
