##############################################################################
##  InfoViewer.tcl:                                                         ##
## ---------------------                                                    ##
##  This file implements the Documentation Viewer system of the "Ellogon"   ##
##  NLP platform.                                                           ##
##  It utilises a feature-rich HTML viewer component (TkHtml) from          ##
##  Dr. Richard Hipp, drh@acm.org, http://www.hwaci.com/drh/                ##
##  Initial version date: 20/01/2000                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::HtmlInfoViewer {
  variable InfoViewer
  catch {
    set InfoViewer(showTableStruct) 0
    set ::HtmlTraceMask 0
  }
}
image create photo ::HtmlInfoViewer::HTML_biggray -data {
  R0lGODdhJAAkAPEAAACQkADQ0PgAAAAAACwAAAAAJAAkAAACmISPqcsQD6OcdJqKM71PeK15
  AsSJH0iZY1CqqKSurfsGsex08XuTuU7L9HywHWZILAaVJssvgoREk5PolFo1XrHZ29IZ8oo0
  HKEYVDYbyc/jFhz2otvdcyZdF68qeKh2DZd3AtS0QWcDSDgWKJXY+MXS9qY4+JA2+Vho+YPp
  FzSjiTIEWslDQ1rDhPOY2sXVOgeb2kBbu1AAADv/
}
image create photo ::HtmlInfoViewer::nogifsm -data {
    R0lGODdhEAAQAPEAAACQkADQ0PgAAAAAACwAAAAAEAAQAAACNISPacHtD4IQz80QJ60as25d
    3idKZdR0IIOm2ta0Lhw/Lz2S1JqvK8ozbTKlEIVYceWSjwIAO///
}

## Make sure that needed Images are loaded...
foreach image {IWWW IBook IBookOpen IHelpDoc BackNice NextNice} {
  GUI_LoadImage $image
}
unset image

## GUI_CreateCollectionMenu
#
proc GUI_CreateInfoViewerMenu {win} {
  global GUI_Text GUI_Underline GUI_Options
  #set menubar [menu ${win}.mb -type menubar -relief $GUI_Options(MenuRelief) 
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }
  
  ## File Menu
  set file $menubar.file; catch {destroy $file}
  $menubar add cascade -label $GUI_Text(File) -menu $file \
     -underline $GUI_Underline(File)
  menu $file
  $file add command -label $GUI_Text(Close) -underline $GUI_Underline(Close)\
     -command "destroy $win" -image ITCancel \
     -compound $GUI_Options(MenuImageLocation)

  ## Options Menu
  set options $menubar.options; catch {destroy $options}
  $menubar add cascade -label $GUI_Text(Options) -menu $options \
     -underline $GUI_Underline(Options)
  menu $options
  set html $win.main.html.html.html
  $options add checkbutton -label $GUI_Text(ShowTableStructure) \
        -variable ::HtmlInfoViewer::InfoViewer(showTableStruct) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
  trace variable ::HtmlInfoViewer::InfoViewer(showTableStruct) \
        w "::HtmlInfoViewer::ShowTableStruct"
  $options add checkbutton -label $GUI_Text(ShowImages) \
        -variable ::HtmlInfoViewer::InfoViewer($html:showImages) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
  trace variable ::HtmlInfoViewer::InfoViewer($html:showImages) \
        w ::HtmlInfoViewer::RefreshTraceCallback
  $options add checkbutton -label $GUI_Text(ShowBgImages) \
        -variable ::HtmlInfoViewer::InfoViewer($html:showBgImages) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
  trace variable ::HtmlInfoViewer::InfoViewer($html:showBgImages) \
        w ::HtmlInfoViewer::RefreshTraceCallback
  $options add separator
  $options add command -label $GUI_Text(ViewSource...) \
        -underline 0 -command "::HtmlInfoViewer::ShowSource $html" \
        -image ITViewText -compound $GUI_Options(MenuImageLocation)

  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help
  
  return $menubar
};# GUI_CreateInfoViewerMenu

proc ::HtmlInfoViewer::ShowTableStruct {n1 n2 op} {
  global $n1 HtmlTraceMask
  variable InfoViewer
  ## regexp {([^:]+):.*} $n2 dummy html
  if {[set ${n1}($n2)]} {
    set HtmlTraceMask [expr {$HtmlTraceMask|0x8}]
    foreach entry [array names InfoViewer *:showImages] {
      catch {
        set html [string range $entry 0 end-11]
        $html configure -tablerelief flat
        ::HtmlInfoViewer::Refresh $html
      }
    }
  } else {
    set HtmlTraceMask [expr {$HtmlTraceMask&~0x8}]
    foreach entry [array names InfoViewer *:showImages] {
      catch {
        set html [string range $entry 0 end-11]
        $html configure -tablerelief raised
        ::HtmlInfoViewer::Refresh $html
      }
    }
  }
};# ::HtmlInfoViewer::ShowTableStruct

proc ::HtmlInfoViewer::RefreshTraceCallback {n1 n2 op} {
  regexp {([^:]+):.*} $n2 dummy html
  ::HtmlInfoViewer::Refresh $html 1
};# ::HtmlInfoViewer::RefreshTraceCallback

## ::HtmlInfoViewer::ShowSource
#   Show the source of the current page...
proc ::HtmlInfoViewer::ShowSource {widget} {
  variable InfoViewer
  if {[catch {set InfoViewer($widget:CurrentHTML)} HTML]} {
    set HTML [$widget source]
  }
  if {[info exists InfoViewer($widget:CurrentURL)]} {
    set URL $InfoViewer($widget:CurrentURL)
  } else {
    set URL [$widget cget -base]
  }
  ggi_view_text_file $HTML {} {} "HTML Source: $URL"
};# ::HtmlInfoViewer::ShowSource

## HtmlInfoViewer
#   Create and display the help system...
proc ::HtmlInfoViewer::InfoViewer {{URL {}} {win .infoViewerHTML}} {
  if {[winfo exists $win]} {
    raise $win;
    if {[string length $URL]} {GUT_WWWDownloadURL $win.main.html.html.html $URL}
    return
  }
  variable InfoViewer
  global GUI_Text GUI_Options HomeDir PANE WMIcons \
         Locale UserAppDir TkHtmlWWWCache ApplicationName

  Agent::Play Read ReadContinued ReadReturn

  toplevel $win
  wm withdraw $win
  wm title $win "$ApplicationName Info Viewer..."
  
  set html $win.main.html.html.html
  if {![info exists InfoViewer($html:showImages)]} {
    set InfoViewer($html:showImages)   1  
    set InfoViewer($html:showBgImages) 1
  }
  set InfoViewer($html:CurrentURL) {}
  ## Menubar
  set menubar [GUI_CreateInfoViewerMenu $win]
  $win configure -menu $menubar
  
  ## Define Widget Names...
  frame $win.main -width 830 -height 450 -bd 0 -relief flat
  frame $win.main.explorer -bd 0 -relief flat
  set explorer $win.main.explorer.explorer.canvas
  frame $win.main.html -bd 0 -relief flat
  set DocDir [file join $HomeDir doc]
  
  ## Create the window Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    set types {{{{HTML Files} {.html .htm}}}}
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Open) \
        -image ITOpen -command "GUT_WWWOpenURL $html" \
        -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar ::HtmlInfoViewer::ToolButton $html Back \
        -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar ::HtmlInfoViewer::ToolButton $html Next \
        -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadPage) \
        -image ITRedraw -command "::HtmlInfoViewer::Refresh $html" \
        -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
        -image ITExit -command "destroy $win" \
        -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  set ::GUI_StatusBar::StatusBar($win) {} 
  set ::GUI_StatusBar::StatusBar($win:Def) {}
  GUI_CreateStatusBar $win.statusbar
  ProgressBar $win.statusbar.prg -relief sunken -borderwidth \
     $GUI_Options(BorderWidth) -width 1 -background $GUI_Options(Background) \
     -foreground blue -variable \
     ::HtmlInfoViewer::InfoViewer($html:ProgressStatus)
  pack $win.statusbar.prg -before $win.statusbar.message \
     -padx 0 -pady 1 -side right -fill y
 
  ## Create the Tree Widget...
  ScrolledWidgets::ScrolledCanvas $win.main.explorer.explorer \
    -relief flat -borderwidth $GUI_Options(ExplorerBorderWidth) \
    -bg $GUI_Options(ExplorerBg) -height $GUI_Options(ExplorerTreeHeigth) \
    -width $GUI_Options(ExplorerTreeWidth) -highlightthickness 0 \
    -regionpadx 5 -regionpady 20
  label $win.main.explorer.explorer.label -anchor w \
     -textvariable GUI_Text(AvailableManuals) \
     -bg $GUI_Options(ExplorerStandoutBg) -fg $GUI_Options(ExplorerStandout) \
     -bd $GUI_Options(BorderWidth) -relief raised -highlightthickness 0
  Tree:create $explorer
  $explorer bind x <<MB_1>> \
    "::HtmlInfoViewer::ExplorerCallback single $html $explorer %x %y"
  $explorer bind x <<DMB_1>> \
    "::HtmlInfoViewer::ExplorerCallback double $html $explorer %x %y"
  grid configure $win.main.explorer.explorer.label -column 0 -row 0 -sticky new
  pack $win.main.explorer.explorer -side left -fill both -expand 1

  ## Here, we have to create the images that will be used as "over" images,
  ## i.e shown when the mouse cursor is over the item...
  catch {image delete IBookOver IHelpDocOver IBookOpenOver IWWWOver}
  foreach image {IBook IHelpDoc IBookOpen IWWW} {
    image create photo ${image}Over
    ${image}Over copy $image
    ${image}Over configure -gamma $GUI_Options(OverImageGamma)
  }
  ## Load the Tree Widget...
  source [file join $DocDir Index]
  catch {Tree:Reset $explorer};Tree:newitem $explorer /
  
  foreach {type entry loc} $EllogonDocumentSpec {
    switch $type {
      f {Tree:newitem $explorer $entry -tags [list $loc] -image IHelpDoc \
         -overimage IHelpDocOver}
      d {Tree:newitem $explorer $entry -tags [list $loc] \
         -image IBook -overimage IBookOver -openimage IBookOpen \
         -overopenimage IBookOpenOver}
      w {Tree:newitem $explorer $entry -tags [list $loc] -image IWWW \
         -overimage IWWWOver}
      default {Tree:newitem $explorer $entry -tags [list $loc] -image IHelpDoc \
         -overimage IHelpDocOver}
    }
  }
  Tree:open $explorer {/Tcl-Tk Reference Manuals}

  ## Create the HTML Widget...
  
  ::ScrolledWidgets::ScrolledHtml $win.main.html.html -bg $GUI_Options(TextBg) \
    -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
    -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
    -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
    -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
    -imagecommand     "::HtmlInfoViewer::ImageCmd $html" \
    -bgimagecommand   "::HtmlInfoViewer::BgImageCmd $html" \
    -framecommand     "::HtmlInfoViewer::FrameCmd $html" \
    -fontcommand      "::HtmlInfoViewer::PickFont $html" \
    -formcommand      "::HtmlInfoViewer::FormCmd $html"
#   -scriptcommand    "::HtmlInfoViewer::ScriptCmd $html" \
#   -appletcommand    "::HtmlInfoViewer::AppletCmd $html"\
#   -hyperlinkcommand "::HtmlInfoViewer::Load $html" \
#   -isvisitedcommand "::HtmlInfoViewer::IsVisitedCmd $html"
  pack $win.main.html.html -fill both -expand 1

  pane $win.main.explorer $win.main.html -orient horizontal
  pack $win.main -fill both -expand 1
  Widget_PaneSet $win.main 1 0.265
  ArrowButton $win.main.__h1.b1 -type button -width 6 -height 60 \
    -dir left -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.main 1 0"
  ::$win.main.__h1.b1:cmd configure -cursor left_side
  ArrowButton $win.main.__h1.b2  -type button -width 6 -height 60 \
    -dir right -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.main 1 0.265"
  ::$win.main.__h1.b2:cmd configure -cursor right_side
  pack $win.main.__h1.b1 $win.main.__h1.b2 -side top

  wm deiconify $win
  set InfoViewer($html:ProgressBar) $win.statusbar.prg
  ## Create HTML Cache Directory...
  file mkdir [file join $UserAppDir WWWCache]
  catch {source [file join $UserAppDir WWWCache Index]}
  update
  if {[string length $URL]} {GUT_WWWDownloadURL $html $URL}
  update
  catch {winico setwindow $win $WMIcons(Help)}

  ## Register Drag'n'Drop of HTML pages...
  if {![catch {package require tkdnd}]} {
    dnd bindtarget $html.x text/uri-list <DragEnter> \
      "$html configure -background \$GUI_Options(DragEnterBackgound)
       update
       return default"
    dnd bindtarget $html.x text/uri-list <DragLeave> \
      "$html configure -background \$GUI_Options(InputBackground)
       update"
    dnd bindtarget $html.x text/uri-list <Drop> \
      "$html configure -background \$GUI_Options(InputBackground)
       GUT_WWWDownloadURL $html %D"
  }
  return $win
};# ::InfoViewer::InfoViewer

## This function handles mouse clicks on the tree widget...
proc ::HtmlInfoViewer::ExplorerCallback {mode html widget x y} {
  global Tree
  ## Get the label under the mouse position...
  set label [Tree:labelat $widget $x $y]
  ## Set the selection to the retrieved label...
  if {![string length $label]} {set label /}
  Tree:setselection $widget $label
  ## Two modes: one for single click and one for double-click...
  switch $mode {
    single {
      ## Is there a URL associated with this item?
      if {![info exists Tree($widget:$label:tags)]} {return}
      set URL [lindex $Tree($widget:$label:tags) 0]
      if {[string length $URL]} {
        catch {GUT_WWWDownloadURL $html $URL}
      } else {
        ## The URL is empty. Try to construct and display an html page that
        ## contains the children nodes...
        set Topic [string trim $label /]
        regsub -all -- {/} $Topic { -> } Topic
        set HTML "<html><head><title>$Topic</title></head>\
                  <body bgcolor=#FFFFFF><h1>$Topic</h1><hr>\n<ul>"
        foreach child [Tree:NodeChildren $widget $label] {
          if {![info exists Tree($widget:$label/$child:tags)]} {continue}
          set URL [lindex $Tree($widget:$label/$child:tags) 0]
          append HTML "\n<li><a href=\"$URL\">$child</a></li>"
        }
        append HTML "</ul></body></html>"
        $html clear
        $html parse $HTML
      }
    }
    double {
      ## We got a double click. The user wants to open/close the sub-tree...
      if {![info exists Tree($widget:$label:open)]} {return}
      if {$Tree($widget:$label:open)} {
        Tree:close $widget $label
      } else {
        Tree:open $widget $label
      }
    }
  }
};# ::HtmlInfoViewer::ExplorerCallback

## This function creates the "Combo Button" widget on the toolbar...
proc ::HtmlInfoViewer::ToolButton {widget html dir args} {
  global GUI_Options
  eval frame $widget $args -bg $GUI_Options(ToolbarBackground)
  switch $dir {
    Back {
      Tooltip::tooltip $widget -textvar GUI_Text(LastPage)
      button $widget.image -image BackNice -command \
        "::HtmlInfoViewer::Back $html" \
        -relief flat -bd 0 -bg $GUI_Options(ToolbarBackground) \
        -highlightbackground $GUI_Options(ToolbarBackground) \
        -activebackground $GUI_Options(ToolbarBackground)
      ArrowButton $widget.arrow -dir bottom -relief flat -bd 0 -ipadx 0 -width \
        9 -height 12 -command \
        "::HtmlInfoViewer::GoTo $html $widget.arrow back" \
        -bg $GUI_Options(ToolbarBackground) -highlightbackground \
        $GUI_Options(ToolbarBackground) -highlightthickness 0
      pack $widget.image $widget.arrow -side left -padx 0 -pady 0 -fill both \
        -expand 1
    }
    Next {
      Tooltip::tooltip $widget -textvar GUI_Text(NextPage)
      button $widget.image -image NextNice -command \
        "::HtmlInfoViewer::Next $html" \
        -relief flat -bd 0 -bg $GUI_Options(ToolbarBackground) \
        -highlightbackground $GUI_Options(ToolbarBackground) \
        -activebackground $GUI_Options(ToolbarBackground)
      ArrowButton $widget.arrow -dir bottom -relief flat -bd 0 -ipadx 0 \
        -width 9 -height 12 -bg $GUI_Options(ToolbarBackground) \
        -command "::HtmlInfoViewer::GoTo $html $widget.arrow next" \
        -highlightbackground $GUI_Options(ToolbarBackground) \
        -highlightthickness 0
      pack $widget.image $widget.arrow -side left -padx 0 -pady 0 -fill both \
        -expand 1
    }
  }
};# ::HtmlInfoViewer::ToolButton

## ::HtmlInfoViewer::HRef
#    Handle the <href> HTML tag...
proc ::HtmlInfoViewer::HRef {widget x y {loc {}}} {
  if {[string length $widget]} {
    focus $widget
    set loc [lindex [$widget href $x $y] 0]
  }
  if {[string length $loc]} {
    if {[string match {tcl://*} $loc]} {
      set script [string map {## { }} [string range $loc 6 end]]
      set script [string map {{source } {} {load } {} {file } {} {encoding } {}\
                 {exec } {} {cd } {}} $script]
      catch "destroy $widget.warningW"
      set ::HtmlInfoViewer::InfoViewer(This) $widget
      eval $script
    } else {
      catch {GUT_WWWDownloadURL $widget $loc}
    }
  } elseif {[string length $widget]} {
    set ::HtmlInfoViewer::InfoViewer(${widget}::SelectionStart) \
                                     [$widget index @$x,$y]
    $widget selection clear
  }
};# ::HtmlInfoViewer::HRef

## ::HtmlInfoViewer::Load
#    Given a URL, load the target file in the widget...
proc ::HtmlInfoViewer::Load {widget Loc {AppendHistory 1}} {
  set Loc [html reformat url plain $Loc]
  switch -glob -- $Loc {
    file:*  {set Loc [string range $Loc 5 end]}
    default   {}
  }
  set tail [file tail $Loc]
  set index [string last \# $tail]
  if {$index > -1} {
    set base [file dirname $Loc]
    set ending [string range $tail 0 [expr {$index-1}]]
    set loc [file join $base $ending]
    set NAME [string range $tail [expr {$index+1}] end]
  } else {
    set loc $Loc
    set NAME {}
  }
  if {![file isfile $loc]} return
  variable InfoViewer
  global WMIcons
  if {![info exists InfoViewer($widget:History)]} {
    set InfoViewer($widget:CurrentURL) {}
    set InfoViewer($widget:CurrentHistoryIndex) -1
    set InfoViewer($widget:History) {}
  }
  #if {![string equal $InfoViewer($widget:CurrentURL) $Loc]} {}
    ## Read URL...
    if {[catch {open $loc r} fp]} {
      tk_messageBox -icon error -message $fp -type ok
      return {}
    } else {
      fconfigure $fp -translation binary
      set Data [string trim [read $fp [file size $loc]]]
      close $fp
      ## We have just red the file with binary translation. Is there a charset
      ## information?
      if {[regexp -nocase -- {<meta.*charset=([^\ \"\;]*)} $Data dummy enc]} {
        ## If changed, modify also Viewers/Viewers.tcl
        switch -glob -- [string tolower $enc] {
          windows-* {set encoding cp[string range $enc 8 end]}
          iso-*     {set encoding iso[string range $enc 4 end]}
          iso*      {set encoding iso[string range $enc 3 end]}
          utf-8     {set encoding utf-8}
          shift-jis {set encoding shiftjis}
          default   {set encoding {}}
        }
        if {[string length $encoding]} {
          set Data [encoding convertfrom $encoding $Data]
        }
      }
    }

    ::HtmlInfoViewer::Clear $widget

    ## Remember the displayed file...
    set ::HtmlInfoViewer::InfoViewer($widget:CurrentURL) $Loc
    if {$AppendHistory} {
      set id [lsearch -exact $::HtmlInfoViewer::InfoViewer($widget:History) $Loc]
      if {$id < 0} {
        incr InfoViewer($widget:CurrentHistoryIndex)
        set InfoViewer($widget:History) [linsert \
           $InfoViewer($widget:History) \
           $InfoViewer($widget:CurrentHistoryIndex) $Loc]
      } else {
        set InfoViewer($widget:CurrentHistoryIndex) $id
      }
    }

    $widget configure -base $loc
    $widget configure -cursor watch -bg white; #grab set $widget
    ## Before we parse the data, we have to check whether it is an html file
    ## or not. If it is not html, enclose the data in <pre>...</pre>
    if {![regexp -nocase {.*<html>.*} $Data]} {
      set Data "<HTML><PRE>$Data</PRE></HTML>"
    }
    $widget parse $Data
    set InfoViewer($widget:CurrentHTML) $Data
    ::HtmlInfoViewer::LoadImagesCmd $widget
    $widget configure -cursor {}; #grab release $widget
    
    if {[regexp -nocase -- {<title>([^<]*)</title>} $Data dummy Title]} {
      wm title [winfo toplevel $widget] [string map {&quot\; \"} $Title]
    }
#  {}

  #::HtmlInfoViewer::DefaultBingings $widget
  ## Jump to the correct tag, if any...
  if {[string length $NAME]} {
    after idle "$widget yview $NAME"
  }
  set toplevel [winfo toplevel $widget]
  set ::GUI_StatusBar::StatusBar($toplevel:Def) "file:$loc"
  set ::GUI_StatusBar::StatusBar($toplevel) "file:$loc"
  update
  catch {winico setwindow $toplevel $WMIcons(Help)}
  focus $widget
};# ::HtmlInfoViewer::Load

## ::HtmlInfoViewer::Refresh
#    Reload displayed file...
proc ::HtmlInfoViewer::Refresh {widget {force 0}} {
  variable InfoViewer

  if {$force} {
    if {[info exists InfoViewer($widget:CurrentURL)]} {
      GUT_WWWDownloadURL $widget $InfoViewer($widget:CurrentURL) 0
    } else {
      set HTML [$widget source]
      $widget clear
      $widget parse $HTML
    }
  }
  $widget refresh
};# ::HtmlInfoViewer::Refresh

## ::HtmlInfoViewer::Back
#    Load the previous page (from history)
proc ::HtmlInfoViewer::Back {widget} {
  variable InfoViewer

  if {![info exists InfoViewer($widget:CurrentHistoryIndex)]} return
  if {$InfoViewer($widget:CurrentHistoryIndex)} {
    incr InfoViewer($widget:CurrentHistoryIndex) -1
    set Loc [lindex $InfoViewer($widget:History) \
                    $InfoViewer($widget:CurrentHistoryIndex)]
    GUT_WWWDownloadURL $widget $Loc 0
  }
};# ::HtmlInfoViewer::Back

## ::HtmlInfoViewer::Next
#    Load the previous page (from history)
proc ::HtmlInfoViewer::Next {widget} {
  variable InfoViewer

  if {![info exists InfoViewer($widget:CurrentHistoryIndex)]} return
  set index [expr {$InfoViewer($widget:CurrentHistoryIndex)+1}]
  if {$index == [llength $InfoViewer($widget:History)]} return
  set Loc [lindex $InfoViewer($widget:History) $index]
  set InfoViewer($widget:CurrentHistoryIndex) $index
  GUT_WWWDownloadURL $widget $Loc 0
};# ::HtmlInfoViewer::Next

## ::HtmlInfoViewer::GoTo
#    Display the history list, so as the user to select the destination...
proc ::HtmlInfoViewer::GoTo {widget path direction} {
  if {[winfo exists $path.shell]} return
  variable InfoViewer
  if {![info exists InfoViewer($widget:CurrentHistoryIndex)]} return
  set shell [menu $path.shell -tearoff 0 -relief flat -bd 0]
  wm overrideredirect $shell 1
  wm withdraw $shell
  wm transient $shell [winfo toplevel $path]
  wm group $shell [winfo toplevel $path]

  if {[string equal $direction next]} {
    set lval [lrange $InfoViewer($widget:History) \
       [expr {$InfoViewer($widget:CurrentHistoryIndex)+1}] end]
  } else {
    set lval [lrange $InfoViewer($widget:History) \
       0 [expr {$InfoViewer($widget:CurrentHistoryIndex)-1}]]
  }
  set h 0
  set sb 0
  if {$h <= 0} {
    set len [llength $lval]
    if { $len < 3 } {
      set h 3
    } elseif { $len > 10 } {
      set h  10
      set sb 1
    }
  }
  set frame [frame $shell.frame -relief sunken -bd 2]
  set listb [listbox $shell.listb -relief flat -bd 0 -highlightthickness 0 \
      -exportselection false -height $h]
  $listb delete 0 end
  foreach val $lval {
    $listb insert end $val
  }

  if {$sb} {
    set scroll [scrollbar $shell.scroll -orient vertical -takefocus 0 \
        -command "$shell.listb yview" -highlightthickness 0 -width 9]
    $listb configure -yscrollcommand "$scroll set"
    pack $scroll -in $frame -side right -fill y
  }
  pack $listb  -in $frame -side left  -fill both -expand yes
  pack $frame  -fill both -expand yes -padx 1 -padx 1
  bind $path  <ButtonPress-1>   "::HtmlInfoViewer::unmapliste $path"
  bind $listb <ButtonRelease-1> "::HtmlInfoViewer::select $widget $path @%x,%y"
  bind $listb <Return>          "::HtmlInfoViewer::select $widget $path active"
  bind $listb <Escape>          "::HtmlInfoViewer::unmapliste $path"
  ArrowButton::configure $path -dir top
  BWidget::place $path.shell 450 0 below $path
  wm deiconify $path.shell
  raise $path.shell
  BWidget::grab global $path
  focus $listb
};# ::HtmlInfoViewer::GoTo
proc ::HtmlInfoViewer::unmapliste {path} {
  BWidget::grab release $path
  ArrowButton::configure $path -dir bottom
  after idle "destroy $path.shell"
}
proc ::HtmlInfoViewer::select {widget path index} {
  set index [$path.shell.listb index $index]
  if {$index != -1} {
    variable InfoViewer
    set Loc [$path.shell.listb get $index]
    set InfoViewer($widget:CurrentHistoryIndex) $index
    GUT_WWWDownloadURL $widget $Loc 0
  }
  ::HtmlInfoViewer::unmapliste $path
  return -code break
}

## ::HtmlInfoViewer::IsVisitedCmd
#  Checks whether an HREF is visited or not...
proc ::HtmlInfoViewer::IsVisitedCmd {widget href args} {
# variable InfoViewer
# if {[info exists InfoViewer(${widget}::ISV:$href)]} {
#   set visited 1
# } else {
#   set visited 0
#   set InfoViewer(${widget}::ISV:$href) 1
# }
# return $visited
  return 0
};# ::HtmlInfoViewer::IsVisitedCmd

## ::HtmlInfoViewer::Clear
#    Clear an HTML widget...
proc ::HtmlInfoViewer::Clear {widget} {
  variable Images
  catch {after cancel [TkGUI_AccessPriv HTMLafterId]}
  $widget clear
  foreach img [array names Images $widget:*] {
    if {[catch {image delete $Images($img)} error]} {puts $error}
  }
  catch {array unset Images $widget:*}
};# ::HtmlInfoViewer::Clear

## ::HtmlInfoViewer::ParseBodyTag
#    Process the <body> tag to get background/foreground colour...
proc ::HtmlInfoViewer::ParseBodyTag {widget tag args} {
  foreach {arg val} [lindex $args 1] {
    switch -glob -- [string tolower $arg] {
      backgr* {
        ## Actually, this cores :-)
        catch {$widget bgimage \
               [::HtmlInfoViewer::BgImageCmd $widget $val]} error
      }
      bgcolo*      {catch {$widget configure -background     $val}}
      text         {catch {$widget configure -foreground     $val}}
      link         {catch {$widget configure -unvisitedcolor $val}}
      vlink        {catch {$widget configure -visitedcolor   $val}}
      marginwidth  {catch {$widget configure -marginwidth    $val}}
      marginheight {catch {$widget configure -marginheight   $val}}
      leftmargin   {catch {$widget configure -leftmargin     $val}}
      topmargin    {catch {$widget configure -topmargin      $val}}
    }
  }
};# ::HtmlInfoViewer::ParseBodyTag

## ::HtmlInfoViewer::ParseLinkTag
#  A callback to parse the <link> tag, which usually holds a stylesheet...
proc ::HtmlInfoViewer::ParseLinkTag {widget id tag args} {
  foreach {href rel type} {{} stylesheet {}} {}
  foreach {option value} [lindex $args 0] {
    switch -exact -- $option {
      href {set href $value}
      rel  {set rel  $value}
      type {set type $value}
      default {}
    }
  }
  if {[string equal stylesheet $rel] && [string length $href]} {
    ## We have a found a style sheet. Is it something we can find locally?
    set base [$widget cget -base]
    switch -glob -- $base {
      http://* {set base [string range $base 7 end]}
      http:/*  {set base [string range $base 6 end]}
      file:*   {set base [string range $base 5 end]}
      default  {}
    }
    set urls [list $href]
    lappend urls [file dirname $base]/$href $base/$href
    ## Maybe a DOS Disk name in front of the url?
    if {[regexp {[A-Z]\:.*} $href]} {
      set href [string range $href 2 end]
      lappend urls [file dirname $base]/$href $base/$href
    }
    foreach file $urls {
      if {[file exists $file]} {
        puts "Found Stylesheet: $file"
        catch {
          set file [open $file]
          set data [string trim [read $file]]
          close $file
          ParseStyleSheet $widget $data
        }
        break
      }
    }
  }
};# ::HtmlInfoViewer::ParseLinkTag

## ::HtmlInfoViewer::ParseStyleSheet
#  This routine will parse a style sheet and prepare an html widget to process
#  the instructions...
proc ::HtmlInfoViewer::ParseStyleSheet {widget data} {
  catch {
    foreach {name spec} $data {
      foreach {tag class} [split $name .] {break}
      CDM_LappendUnique Tags($tag) $class
      puts $spec
      CDM_LappendUnique TagSpec($tag:$class) "style=\"$spec\"" \
    }
    foreach tag [array names Tags] {
      foreach class $Tags($tag) {
        if {[info exists TagSpec($tag:$class)]} {
          append Options($tag) " $class $TagSpec($tag:$class)"
        }
      }
    }
    parray Options
  } error
  puts $error
};# ::HtmlInfoViewer::ParseStyleSheet

## ::HtmlInfoViewer::ParseTitleTag
#    Process the <title> tag to get window title...
proc ::HtmlInfoViewer::ParseTitleTag {widget tag args} {
  foreach {arg val} [lindex $args 0] {
    switch -glob -- [string tolower $arg] {
    }
  }
};# ::HtmlInfoViewer::ParseTitleTag

## ::HtmlInfoViewer::PickFont
#    Handle the <font> HTML tag...
proc ::HtmlInfoViewer::PickFont {widget size attrs} {
# puts "FontCmd: $size $attrs"
  global GUI_Options
  if {$GUI_Options(HTMLUseDefaultFonts)} {return {}}
  foreach {bold italic} {normal roman} {break}
  set family $GUI_Options(HTMLVariableFont)
  foreach attr $attrs {
    switch $attr {
      bold    {set bold bold}
      italic  {set italic italic}
      fixed   {set family $GUI_Options(HTMLFixedFont)}
      default {}
    }
  }
  set size [set GUI_Options(HTMLFontSize$size)]
  set FontName HtmlFont${family}${size}${bold}${italic}
  if {[lsearch -exact [font names] $FontName] != -1} {
    ## Have we recreated the font?
    font configure $FontName -family $family -size $size \
      -weight $bold -slant $italic
  } else {
    ## Its the first time we create this font...
    font create $FontName -family $family -size $size \
      -weight $bold -slant $italic
  }
  return $FontName
# set a [expr {-1<[lsearch $attrs fixed]?{courier}:{charter}}]
# set b [expr {-1<[lsearch $attrs italic]?{italic}:{roman}}]
# set c [expr {-1<[lsearch $attrs bold]?{bold}:{normal}}]
# set d [expr {int(12*pow(1.2,$size-4))}]
# list $a $d $b $c
};# ::HtmlInfoViewer::PickFont


proc ::HtmlInfoViewer::BgImageCmd {widget args} {
  variable InfoViewer
  if {!$InfoViewer($widget:showBgImages)} {return {}}
  return [eval ::HtmlInfoViewer::ImageCmd $widget $args]
};# ::HtmlInfoViewer::BgImageCmd

## ::HtmlInfoViewer::ImageCmd
#    Handle the <image> HTML tag...
proc ::HtmlInfoViewer::ImageCmd {widget args} {
  variable InfoViewer
  if {!$InfoViewer($widget:showImages)} {return {}}
  ## The arguments are: src=, width=, height=, list of all other values...
  foreach {w h} {1 1} {}
  foreach {fn w h spec} $args {break}
  if {!([string match *.gif $fn]|[string match *.ppm $fn])} {
    ## Check whether the Img package is loaded
    if {[catch {package require Img}]} {
      return {}
    }
  }

  ## Is the location a local file we can open?
  if {[file exists $fn]} {
    if {[string length $w] && [string length $h]} {
      return [image create photo -file $fn -width $w -height $h]
    } else {
      return [image create photo -file $fn]
    }
  }

  ## If it is a local file, then maybe it has an incomplete uri. Try to
  ## reconstruct the original uri...
  set base [$widget cget -base]
  switch -glob -- $base {
    http://* {set base [string range $base 7 end]
              $widget configure -base http://$base}
    http:/*  {set base [string range $base 6 end]
              $widget configure -base http://$base}
    file:*   {set base [string range $base 5 end]
              $widget configure -base $base}
    default  {}
  }

  ## A local file?
  foreach file [list [file dirname $base]/$fn $base/$fn] {
    if {[file exists $file]} {
      if {[string length $w] && [string length $h]} {
        return [image create photo -file $file -width $w -height $h]
      } else {
        return [image create photo -file $file]
      }
    }
  }
  
  ## We failed to find the image locally. We have to download the image...
  variable Images2Load
  variable TkHtmlWWWCache
  variable Images

  ## Does our uri start from http or www.?
  if {[string match http* $fn] || [string match www.* $fn]} {
    set urls $fn
  } elseif {[string length [file extension $base]]} {
    set urls [list http://[file dirname $base]/$fn http://$base/$fn]
  } else {
    set urls [list http://$base/$fn http://[file dirname $base]/$fn]
  }
  foreach file $urls {
    if {[info exists TkHtmlWWWCache($file)]} {
      if {[string length $w] && [string length $h]} {
        if {![catch {image create photo -file $TkHtmlWWWCache($file) \
                      -width $w -height $h} img]} {
          return $img
        }
      } else {
        if {![catch {image create photo -file $TkHtmlWWWCache($file)} img]} {
          return $img
        }
      }
    }
  }
  ## We failed to load the image from disk. Then it might be located over
  ## Internet. DO NOT DOWNLOAD it now. Just create a blank image, and we
  ## will download the actual data later...
  if {[string length $w] && [string length $h]} {
    set img [image create photo -width $w -height $h]
  } else {
    set img [image create photo]
  }
  set Images2Load($img) [list $urls $w $h]
  set Images($widget:$img) $img;# This will return "$img" as well...
};# ::HtmlInfoViewer::ImageCmd

## ::HtmlInfoViewer::LoadImagesCmd
#    This procedure tries to download the postponed so far images...
proc ::HtmlInfoViewer::LoadImagesCmd {widget} {
  variable Images2Load
  variable Images
  variable InfoViewer
  variable TkHtmlWWWCache
  if {![array exists Images2Load]} return
  array set Images [array get Images2Load]
  array unset Images2Load

  foreach img [lsort [array names Images]] {
    foreach {urls w h} $Images($img) {break}
    foreach url $urls {
      set InfoViewer($widget:ProgressStatus) 0
      set ::GUI_StatusBar::StatusBar([winfo toplevel $widget]) \
           "Downloading Image: [file tail $url]"
       ## Save the data in the cache...
       global UserAppDir
       set file [file join $UserAppDir WWWCache \
                             "[pid][clock seconds]$img[file tail $url]"]
       switch -glob -- [string tolower [file tail $url]] {
         *.gif   {set mime_type image/gif}
         *.png   {set mime_type image/png}
         *.jpeg  {set mime_type image/jpeg}
         default {set mime_type image/gif}
       }
       set f [open $file w]
       fconfigure $f  -encoding binary -translation binary
       if {![catch {::http::geturl $url -blocksize 1024 -type $mime_type \
               -progress "GUT_WWWDownloadURLProgress $widget" \
               -channel $f -type $} token]} {
         close $f
         #puts "Success on \"$url\" (w=$w) (h=$h) <$mime_type>"
         set InfoViewer($widget:ProgressStatus) 100
         set TkHtmlWWWCache($url) $file

         ## If we fail to create the image, the Data we got are wrong...
         #puts "file=[file size $file] $file"
         if {[string length $w] && [string length $h]} {
           if {[catch {image create photo $img -file $file \
                                    -width $w -height $h} error]} {
             $img copy ::HtmlInfoViewer::HTML_biggray
             puts "      Error: $error"
           }
         } else {
           if {[catch {image create photo $img -file $file} error]} {
             $img copy ::HtmlInfoViewer::HTML_biggray
             puts "      Error: $error"
           }
         }
         ::http::cleanup $token
         break
         ## Redefine variable, as the user may have changed image, and the
         ## array may have been unsetted (so we may leak ... images!)
         set Images($widget:$img) $img
      } else {
        close $f
        catch {file delete -force $file}
      }
      set InfoViewer($widget:ProgressStatus) 0
      $img copy ::HtmlInfoViewer::HTML_biggray
    }
  }
  catch {$InfoViewer($widget:ProgressBar) configure -width 1}
};# ::HtmlInfoViewer::LoadImagesCmd

## ::HtmlInfoViewer::FormCmd
#    Handle the <form> HTML tag...
proc ::HtmlInfoViewer::FormCmd {widget n cmd args} {
  #puts "FormCmd: $n $cmd $args"
catch {
  foreach {wspec name fspec} $args {break}

  ## Handle some of the widget options...
  foreach {fg bg} {{} {}} {}
  foreach {arg val} $wspec {
    switch -glob -- [string tolower $arg] {
      color   {set fg $val}
      bgcolor {set bg $val}
    }
  }
  switch $cmd {
    form {# We ignore this info. Our forms are not usuable anyway...}
    flash {# We should free any resources used by this form, except widgets...}
    input {
      ## We should create an input widget...
      foreach {size value} {-1 {}} {}
      foreach {arg val} $fspec {
        switch -glob -- [string tolower $arg] {
          type {
            switch -glob -- [string tolower $val] {
              text     {entry       $name -foreground $fg -background $bg}
              button   -
              submit   {button      $name}
              reset    {button      $name}
              radio    {radiobutton $name -foreground $fg -background $bg}
              checkbox {checkbutton $name -foreground $fg -background $bg}
              default  {entry       $name -foreground $fg -background $bg}
            }
          }
          name  {# Ignore it...}
          size  {set size $val}
          value {set value $val}
        }
      }
      ## If the widget has not yet been created, then "type" was missing.
      ## Create an entry widget.
      if {![winfo exists $name]} {
        entry $name -foreground $fg -background $bg
      }
      switch [winfo class $name] {
        Button {$name configure -text $value}
        Entry  {$name insert end $value}
      }
      if {$size != -1} {
        switch [winfo class $name] {
          Button {$name configure -width $size}
          Entry  {$name configure -width $size}
        }
      }
      update
    }
    textarea {
      ScrolledWidgets::ScrolledText $name -foreground $fg -background $bg
      foreach {arg val} $fspec {
        switch -glob -- [string tolower $arg] {
          rows  {$name.text configure -height $val}
          cols  {$name.text configure -width  $val}
        }
      }
      set text [lindex $args 3]
      if {[string length $text]} {$name.text insert 1.0 $text}
      update
    }
    select {
      ## ComboBox...
      set ValueSpec [lindex $args 3]
      set values {}
      set i 0
      foreach menu_entry [lindex $args 3] {
        foreach {index value label} $menu_entry {
          lappend values $label
          if {$index} {set see $i}
        }
        incr i
      }
      ComboBox $name -foreground $fg -background $bg -values $values
      catch {$name setvalue @$see}
      update
    }
  }
}
};# ::HtmlInfoViewer::FormCmd

## ::HtmlInfoViewer::FrameCmd
proc ::HtmlInfoViewer::FrameCmd {widget args} {
  ## Just inform the user that the widget does not support forms...
  return {}
};# ::HtmlInfoViewer::FrameCmd

## ::HtmlInfoViewer::ScriptCmd
#    Handle the <script> HTML tag...
proc ::HtmlInfoViewer::ScriptCmd {widget args} {
};# ::HtmlInfoViewer::ScriptCmd

## ::HtmlInfoViewer::AppletCmd
#    Handle the <applet> HTML tags...
proc ::HtmlInfoViewer::AppletCmd {widget args} {
# puts "AppletCmd: w=$w arglist=$arglist"
  label $w -text "The Applet $w" -bd 2 -relief raised
};# ::HtmlInfoViewer::AppletCmd

## ::HtmlInfoViewer::MotionOver
#  This is intented to run as a <Motion> binding over the HTML widget. It
#  changes the cursor to a hand if over a HREF, and displays the target in the
#  status bar, if any...
proc ::HtmlInfoViewer::MotionOver {widget x y} {
  set html [winfo parent $widget]
  set href [lindex [$html href $x $y] 0]
  set toplevel [winfo toplevel $widget]
  global GUI_Text
  if {[string length $href] > 0} {
    if {[string match {tcl://*} $href]} {
      $html configure -cursor hand2
      set ::GUI_StatusBar::StatusBar($toplevel) $GUI_Text(ExecuteTcl)
      if {![winfo exists $html.warningW]} {
        toplevel $html.warningW -bg white
        wm withdraw $html.warningW
        wm transient $html.warningW $toplevel
        wm overrideredirect $html.warningW 1
        wm geometry $html.warningW +[expr {[winfo pointerx \
                    $html]+20}]+[expr {[winfo pointery $html]+20}]
        if {![llength [info commands IWarning]]} {
          GUI_LoadImage IWarning
        }
        label $html.warningW.image -image IWarning -bg white
        pack $html.warningW.image
        wm deiconify $html.warningW
        after 5000 "destroy $html.warningW"
      } else {
        wm geometry $html.warningW +[expr {[winfo pointerx \
                    $html]+20}]+[expr {[winfo pointery $html]+20}]
      }
    } else {
      $html configure -cursor hand2
      set ::GUI_StatusBar::StatusBar($toplevel) $href
      catch {destroy $html.warningW}
    }
  } else {
    $html configure -cursor {}
    catch {set ::GUI_StatusBar::StatusBar($toplevel) \
              $::GUI_StatusBar::StatusBar($toplevel:Def)}
    catch {destroy $html.warningW}
  }
};# ::HtmlInfoViewer::MotionOver

## ::HtmlInfoViewer::DefaultBingings
#  Sets the default bindings (sort of text widget emulation) for an HTML
#  widget...
proc ::HtmlInfoViewer::DefaultBingings {widget} {
  variable InfoViewer
  if {![info exists InfoViewer($widget:showImages)]} {
    set InfoViewer($widget:showImages)   1  
  }
  if {![info exists InfoViewer($widget:showBgImages)]} {
    set InfoViewer($widget:showBgImages) 1
  }
  bind $widget   <Destroy> \
                 {catch {array unset ::HtmlInfoViewer::InfoViewer(%W:*)}}
  bind $widget.x <<MB_1>>          "::HtmlInfoViewer::HRef $widget %x %y"
  ## Selection Capabilities....
  bind $widget.x <B1-Motion>       "TkGUI_SetPriv x %x;TkGUI_SetPriv y %y;\
                                    TkGUI_SetPriv selectMode char;\
                                    ::HtmlInfoViewer::SelectTo $widget %x %y"
  bind $widget.x <B1-Leave>        "TkGUI_SetPriv x %x;TkGUI_SetPriv y %y;\
                                    TkGUI_SetPriv selectMode char;\
                                    ::HtmlInfoViewer::AutoScan $widget"
  bind $widget.x <B1-ButtonRelease> \
                          {catch {after cancel [TkGUI_AccessPriv HTMLafterId]}}
  bind $widget.x <Motion>           {::HtmlInfoViewer::MotionOver %W %x %y}

  bind $widget.x <Double-1>        "TkGUI_SetPriv selectMode word;\
                                    ::HtmlInfoViewer::SelectTo $widget %x %y"
  bind $widget.x <Triple-1>        "TkGUI_SetPriv selectMode line;\
                                    ::HtmlInfoViewer::SelectTo $widget %x %y"
  bind $widget <Shift-Left> \
    "::HtmlInfoViewer::ExpandSelection $widget left char"
  bind $widget <Shift-Right> \
    "::HtmlInfoViewer::ExpandSelection $widget right char"
  
  $widget token handler body        "::HtmlInfoViewer::ParseBodyTag $widget"
  #$widget token handler link        "::HtmlInfoViewer::ParseLinkTag $widget"
# $widget token handler title      "::HtmlInfoViewer::ParseTitleTag $widget"

  ## Contextual Menu...
  bind $widget.x  <<MB_2>>          "::HtmlInfoViewer::ContextualMenuHandler $widget"
};# ::HtmlInfoViewer::DefaultBingings

## ::HtmlInfoViewer::AutoScan
# This procedure is invoked when the mouse leaves a HTML widget
# with button 1 down.  It scrolls the window up, down, left, or right,
# depending on where the mouse is (this information was saved in
# tkPriv(x) and tkPriv(y)), and reschedules itself as an "after"
# command so that the window continues to scroll until the mouse
# moves back into the window or the mouse button is released.
proc ::HtmlInfoViewer::AutoScan {widget} {
  if {![winfo exists $widget]} {return}
  if {[TkGUI_AccessPriv y] >= [winfo height $widget]-20} {
      $widget yview scroll 2 units
  } elseif {[TkGUI_AccessPriv y] < 0} {
      $widget yview scroll -2 units
  } elseif {[TkGUI_AccessPriv x] >= [winfo width $widget]-20} {
      $widget xview scroll 2 units
  } elseif {[TkGUI_AccessPriv x] < 0} {
      $widget xview scroll -2 units
  } else {
      return
  }
  ::HtmlInfoViewer::SelectTo $widget [TkGUI_AccessPriv x] [TkGUI_AccessPriv y]
  TkGUI_SetPriv HTMLafterId   [after 100 ::HtmlInfoViewer::AutoScan $widget]
};# ::HtmlInfoViewer::AutoScan

## ::HtmlInfoViewer::SelectTo
#  Aranges the selection in a HTML widget...
proc ::HtmlInfoViewer::SelectTo {widget x y} {
  variable InfoViewer
  switch [TkGUI_AccessPriv selectMode] {
    char {
      catch {
        $widget selection set $InfoViewer(${widget}::SelectionStart) @$x,$y
        ## Trim the selection from space & markup elements...
        set start  [$widget index $InfoViewer(${widget}::SelectionStart)]
        set end    [$widget index @$x,$y]
        set tokens [$widget token list $start $end]
        ## Trim selection start...
        set i 0
        foreach token $tokens {
          foreach {id type} $token {break}
          switch $type {
            Text {
              if {$i} {set start $id.0}
              break
            }
            Space -
            Markup {}
          }
          incr i
        }
        ## Trim selection end...
        set len [llength $tokens]
        set i [expr {$len-1}]
        while {$i >= 0} {
          set token [lindex $tokens $i]
          foreach {id type} $token {break}
          switch $type {
            Text {
              if {$i != $len-1} {
                set end $id.[string bytelength [lindex $token 2]]
              }
              break
            }
            Space -
            Markup {}
          }
          incr i -1
        }
        $widget selection set $start $end
      }
    }
    word {
      catch {
        #set index [$widget index @$x,$y]
        ### This will be an index of the form token.char
        #if {[regexp {([0-9]+)\.([0-9]+)} $index tmp token char]} {
        #  set tokenText [$widget text ascii $token.0 $token.last]
        #  set start [tcl_wordBreakBefore $tokenText $char]
        #  set end   [tcl_wordBreakAfter  $tokenText $char]
        #  if {$start == -1} {set start 0}
        #  if {$end   == -1} {set end [string bytelength $tokenText]}
        #  $widget selection set $token.$start $token.$end
        #} else {
        #  $widget selection clear
        #}
        set offset [$widget indexToCharacterOffset @$x,$y]
        set text   [$widget source]
        set start  [tcl_wordBreakBefore $text $offset]
        set end    [tcl_wordBreakAfter  $text $offset]
        #puts "text: [string range $text $start $end]"
        $widget selection set [$widget characterOffsetToIndex $start] \
                              [$widget characterOffsetToIndex $end]
      }
    }
    line {
      catch {
        set index [$widget index @$x,$y]
        ## This will be an index of the form token.char
        if {[regexp {([0-9]+)\.([0-9]+)} $index tmp token char]} {
          ## We have the token the user has clicked. Start adding tokens
          ## before and after this token, until we change line...
          foreach {x top y bottom} [$widget coords $token] {}
          set i [expr {$token-1}]
          while {$i} {
            foreach {i_x i_top i_y i_bottom} [$widget coords $i.0] {}
            if {abs($i_bottom - $bottom) > 5} {incr i; break}
            incr i -1
          }
          set start $i.0
          set i [expr {$token+1}]
          set end [expr {$token+250}]
          while {$i < $end} {
            foreach {i_x i_top i_y i_bottom} [$widget coords $i.0] {}
            if {abs($i_bottom - $bottom) > 5} {incr i -1; break}
            incr i
          }
          set end $i.end
          $widget selection set $start $end
        } else {
          $widget selection clear
        }
      }
    }
  }
  clipboard clear
  catch {
    clipboard append -- [$widget text ascii sel.first sel.last]
  }
};# ::HtmlInfoViewer::SelectTo

## ::HtmlInfoViewer::ExpandSelection
#  This function tries to expand the selection in the specified direction by
#  1 units...
proc ::HtmlInfoViewer::ExpandSelection {widget direction units} {
  ## Is there any selected text?
  set sel_first [$widget index sel.first]
  if {[string length $sel_first]} {
    set sel_last [$widget index sel.last]
    ## A selection exists. Set the index according to the direction...
    switch $direction {
      left  - up   {set index $sel_last}
      right - down {set index $sel_last}
      default {set index $sel_first}
    }
  } else {
    ## No text is selected. Use the last clicked index, if available...
    variable InfoViewer
    if {[info exists InfoViewer(${widget}::SelectionStart)]} {
      set index $InfoViewer(${widget}::SelectionStart)
      set sel_first $index
      set sel_last  $index
    }
  }
  if {![info exists index]} {return}
  ## Ensure that the index is within text...
  set token [lindex [$widget token list $index] 0]
  foreach {tok_id tok_type text} $token {break}
  foreach {id char} [split $index .] {break}
  switch $tok_type {
    Text  {
      ## Convert the byte-index to character index...
      set len [string length $text]
      for {set i 0} {$i < $len} {incr i} {
        if {[string bytelength [string range $text 0 $i]] > $char} {
          if {$i == 0} {
            set char 0
          } else {
            set char [string length [string range $text 0 $i]]
          }
          break
        }
      }
      set lcontext [expr {$char - 1}]
      set rcontext [expr {[string length $text] - $char}]
    }
    Space {
      set lcontext 0
      set rcontext 0
      set text { }
      set char 0
    }
    default {return}
  }
  puts "text='$text', id=$id, char=$char, \
        lcontext=$lcontext, rcontext=$rcontext"
  switch $units {
    char {
      ## We must move the selection 1 chars in the specified direction...
      switch $direction {
        left  {
          if {$lcontext >= 0} {
            if {$char} {
              set sel_first \
                $id.[string bytelength [string range $text 0 [incr char -1]]]
            } else {
              set sel_first \
                $id.[string bytelength [string index $text 0]]
            }
          }
        }
        right {
          if {$rcontext >= 0} {
            if {$char} {
              set sel_last \
                $id.[string bytelength [string range $text 0 [incr char]]]
            } else {
              set sel_last \
                $id.[string bytelength [string index $text 0]]
            }
          }
        }
      }
    }
    default {return}
  }
  $widget selection set $sel_first $sel_last
  puts "[$widget index sel.first] [$widget index sel.last]"
};# ::HtmlInfoViewer::ExpandSelection

###############################################################################
###############################################################################
##
##  The following routines add Internet capabilities to the HTML display
##  component.
##
##  Author: George Petasis, 22/01/2000
##
###############################################################################
###############################################################################

## GUT_WWWOpenURL
#  Displays a dialog that the user can enter a URL to be displayed in an HTML
#  widget...
proc GUT_WWWOpenURL {widget {win .gut_www_open_url}} {
  if {[winfo exists $win]} {raise $win;return}
  
  toplevel $win
  wm withdraw $win
  global ApplicationName GUI_Text
  wm title $win "$ApplicationName: Open URL..."

  label $win.label -textvariable GUI_Text(EnterURL) -justify left -anchor w
  entry $win.entry -width 55
  ## Can we get a valid url from HTML widget?
  set base [$widget cget -base]
  switch $base {
    file://*   -
    http://*   {$win.entry insert end $base}
    default    {
      if {[file exists $base]} {
        $win.entry insert end $base
      } else {
        $win.entry insert end $base
      }
    }
  }
  ImageButton::Button $win.choose -image ITOpen \
     -textvariable GUI_Text(SelectFile) \
     -command "GUI_SelectFile $win.entry -defaultextension *.html \
     -filetypes [list [list [list {HTML Files} [list .html .htm]] \
                            [list $GUI_Text(All) [list * *.*]]]]"
  GUI_MakeEntryFileDropTarget $win.entry
  frame $win.separator -height 2 -bd 1 -relief sunken
  frame $win.buttons -relief flat -bd 0
  button $win.buttons.ok -textvariable GUI_Text(Ok) -default active -command \
   "wm withdraw $win;GUT_WWWDownloadURL $widget \[$win.entry get\];destroy $win"
  button $win.buttons.cancel -textvariable GUI_Text(Cancel) \
    -command "destroy $win"
  pack $win.buttons.ok $win.buttons.cancel -side left -padx 10 -pady 10
  bind $win.buttons.ok <Return> {%W invoke}
  bind $win.entry      <Return> "catch \"$win.buttons.ok invoke\""
  bind $win            <Return> "catch \"$win.buttons.ok invoke\""

  grid $win.label -columnspan 2 -sticky snew
  grid $win.entry -sticky snew
  grid $win.choose -row 1 -column 1 -sticky sn
  grid $win.separator -columnspan 2 -sticky ew -pady 5 -padx 4
  grid $win.buttons -columnspan 2
  grid columnconfigure $win 0 -weight 1
  gut_ModalDlg $win
  wm deiconify $win
  grab set $win
};# GUT_WWWOpenURL


## GUT_WWWDownloadURL
#  This procedure starts a connection with the server specified by the given 
#  URL and displays the HTML into the given widget. The routine periodicaly
#  checks for the existence of GUT_WWWDownload($html_widget:Abort). If this
#  variable exists (no matter the value) then the current transaction is
#  aborted...
proc GUT_WWWDownloadURL {widget URL {AppendHistory 1} {block_size 512}} {
  package require http

  set URL [string trim [string trim $URL \{\}]]
  if {![string match *.htm* $URL]} {
    if {![string equal [string index $URL end] /]} {
      append URL /
    }
  }

  
  ## puts "\nPage: $URL"

  ## Does the URL has a NAME reference?
  set tail [file tail $URL]
  set index [string last \# $tail]
  if {$index > -1} {
    switch -glob -- $URL {
      http://* {set base http://[file dirname [string range $URL 7 end]]}
      http:/*  {set base http://[file dirname [string range $URL 6 end]]}
      default  {set base [file dirname $URL]}
    }
    set ending [string range $tail 0 [expr {$index-1}]]
    set url $base/$ending
    set NAME [string range $tail [expr {$index+1}] end]
  } else {
    set url $URL
    set NAME {}
  } 
  ## Is it an Internet URL (starts with http:// ?) or a simple file?
  if {[string match file:* $url] || [file isfile $url]} {
    return [::HtmlInfoViewer::Load $widget $URL]
  }
  
  global GUI_Text UserAppDir TkHtmlWWWCache WMIcons
  if {![info exists ::HtmlInfoViewer::InfoViewer($widget:History)]} {
    set ::HtmlInfoViewer::InfoViewer($widget:CurrentURL) {}
    set ::HtmlInfoViewer::InfoViewer($widget:CurrentHistoryIndex) -1
    set ::HtmlInfoViewer::InfoViewer($widget:History) {}
  }
  
  if {![string equal $::HtmlInfoViewer::InfoViewer($widget:CurrentURL) $URL]} {
    ## Download URL...
    catch {$::HtmlInfoViewer::InfoViewer($widget:ProgressBar) \
                                                          configure -width 100}
    if {[catch {::http::geturl $url -blocksize $block_size \
        -progress "GUT_WWWDownloadURLProgress $widget"} token]} {
      catch {$::HtmlInfoViewer::InfoViewer($widget:ProgressBar) \
                                                         configure -width 1}
      set ::HtmlInfoViewer::InfoViewer($widget:ProgressStatus) -1
      gut_ErrorDlg "[format $GUI_Text(CannotShowURL) $url]\n($token)"
      return
    }

    ## Do we got an error?
    upvar #0 $token state
    foreach {arg val} $state(meta) {
      set Meta($arg) $val
    }
    foreach {protocol code status} [::http::code $token] break
    if {$code != 200} {
      if {$code >= 300 && $code < 400} {
        ## This is a redirection error...
        set url $Meta(Location)
      } else {
        ## This is a "not found" error
        if {[string match *.htm $url] || [string match *.html $url]} {
          set url {}
        } else {
          set url [string range $url 0 [expr {[string length $url]-2}]]
        }
      }
      if {[string length $url]} {
        if {[catch {::http::geturl $url -blocksize $block_size \
            -progress "GUT_WWWDownloadURLProgress $widget"} token]} {
          catch {$::HtmlInfoViewer::InfoViewer($widget:ProgressBar) \
                                                       configure -width 1}
          set ::HtmlInfoViewer::InfoViewer($widget:ProgressStatus) -1
          gut_ErrorDlg "[format $GUI_Text(CannotShowURL) $url]\n($token)"
          return
        }
        upvar #0 $token state
        foreach {arg val} $state(meta) {
          set Meta($arg) $val
        }
      }
    }
    set Data [::http::data $token]
    ::http::cleanup $token

    ## If it is a text file, span an external viewer...
    if {[info exists Meta(Content-Type)] && \
        [string match -nocase *text/plain* $Meta(Content-Type)]} {
      ggi_view_text_file $Data {} {} $url
    } else {
      ::HtmlInfoViewer::Clear $widget
      $widget configure -base $url
      $widget configure -cursor watch -bg white; #grab set $widget 
      ## Before we parse the data, we have to check whether it is an html file
      ## or not. If it is not html, enclose the data in <pre>...</pre>
      if {![regexp -nocase {.*<html>.*} $Data]} {
        set Data "<HTML><PRE>$Data</PRE></HTML>"
      }
      $widget parse $Data
      set ::HtmlInfoViewer::InfoViewer($widget:CurrentHTML) $Data
      ::HtmlInfoViewer::LoadImagesCmd $widget
      $widget configure -cursor {}; #grab release $widget
      if {[regexp -nocase -- {<title>([^<]*)</title>} $Data dummy Title]} {
        wm title [winfo toplevel $widget] [string map {&quot\; \"} $Title]
      }
    }
  }

  ## Remember the displayed file...
  set ::HtmlInfoViewer::InfoViewer($widget:CurrentURL) $URL
  if {$AppendHistory} {
    set id [lsearch -exact $::HtmlInfoViewer::InfoViewer($widget:History) $URL]
    if {$id < 0} {
      incr ::HtmlInfoViewer::InfoViewer($widget:CurrentHistoryIndex)
      set ::HtmlInfoViewer::InfoViewer($widget:History) [linsert \
         $::HtmlInfoViewer::InfoViewer($widget:History) \
         $::HtmlInfoViewer::InfoViewer($widget:CurrentHistoryIndex) $URL]
    } else {
      set ::HtmlInfoViewer::InfoViewer($widget:CurrentHistoryIndex) $id
    }
  }

  #::HtmlInfoViewer::DefaultBingings $widget
  ## Jump to the correct tag, if any...
  if {[string length $NAME]} {
    after idle "$widget yview $NAME"
  }
  set ::HtmlInfoViewer::InfoViewer($widget:ProgressStatus) -1
  catch {$::HtmlInfoViewer::InfoViewer($widget:ProgressBar) configure -width 1}
  set toplevel [winfo toplevel $widget]
  catch {set ::GUI_StatusBar::StatusBar($toplevel) \
    "$url, Context Type:$Meta(Content-Type)"}
  set ::GUI_StatusBar::StatusBar($toplevel) \
     $::GUI_StatusBar::StatusBar($toplevel:Def)

  set f [open [file join $UserAppDir WWWCache Index] w]
  puts $f "array set TkHtmlWWWCache \{[array get TkHtmlWWWCache]\}"
  close $f
  update
  catch {winico setwindow $toplevel $WMIcons(Html)}
  focus $widget
};# GUT_WWWDownloadURL

## GUT_WWWDownloadURLProgress
#  Update the progress dialog...
proc GUT_WWWDownloadURLProgress {widget token total current} {
  global GUT_WWWDownload
  
  if {[info exists GUT_WWWDownload($widget:Abort)]} {
    # Abort, but how??
    ::http::reset $token
    set ::HtmlInfoViewer::InfoViewer($widget:ProgressStatus) 100
    return
  }
  catch {set ::HtmlInfoViewer::InfoViewer($widget:ProgressStatus) \
             [expr {$current*100/$total}]}
};# GUT_WWWDownloadURLProgress

proc HTMLDocument {name data} {
  set widget $::HtmlInfoViewer::InfoViewer(This)
  ::HtmlInfoViewer::Clear $widget
  set data [string map [list "<A NAME=\"$name\">" "<A NAME=\"$name\">$data"] \
                       $::HtmlInfoViewer::InfoViewer($widget:CurrentHTML)]
  $widget configure -cursor watch -bg white; #grab set $widget 
  ## Before we parse the data, we have to check whether it is an html file
  ## or not. If it is not html, enclose the data in <pre>...</pre>
  if {![regexp -nocase {.*<html>.*} $data]} {
    set Data "<HTML><PRE>$data</PRE></HTML>"
  }
  $widget parse $data
  set ::HtmlInfoViewer::InfoViewer($widget:CurrentHTML) $data
  ::HtmlInfoViewer::LoadImagesCmd $widget
  $widget configure -cursor {}; #grab release $widget
  if {[regexp -nocase -- {<title>([^<]*)</title>} $data dummy Title]} {
    wm title [winfo toplevel $widget] [string map {&quot\; \"} $Title]
  }
}

#############################################################################
##
#############################################################################
## ::HtmlInfoViewer::ShowSource
#   Show the token tree of the current page...
proc ::HtmlInfoViewer::ShowTokenTree {widget} {
  if {[catch {set ::HtmlInfoViewer::InfoViewer($widget:CurrentHTML)} HTML]} {
    set HTML [$widget source]
  }
  package require BWidget
  set win [ggi_unused_win $widget.tokenTreeViewer]
  toplevel $win
  wm withdraw $win
  wm title $win {HTML Tree Viewer...}
  global GUI_Options GUI_Text

  ## Create the window
  ## Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
        -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  ## The main widget is a BWidget Tree...
  set sw   [ScrolledWindow $win.main -background [$win cget -bg]]
  set tree [Tree $sw.tree -padx 28 -deltay 22 -width 65 -height 18]
  $sw setwidget $tree
  pack $sw -fill both -expand 1

  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  pack $win.buttons.dismiss -padx 2 -pady 10
  focus $tree

  ## Load some needed images...
  foreach image {HTMLTagAnchor HTMLTagCubes HTMLTagH3 HTMLTagI  HTMLTagP
                 HTMLTagTr     HTMLTagBold  HTMLTagH1 HTMLTagH4 HTMLTagLi
                 HTMLTagTable  HTMLTagU     HTMLTagBr HTMLTagH2 HTMLTagH5
                 HTMLTagMisc   HTMLTagTd    HTMLTagHr IHelpDoc  ITImage
                 ITStar} {
    GUI_LoadImage $image
  }
  ## Fill the tree with the HTML tokens...
  set parent root
  set stack {root root}
  foreach token [$widget token list begin end] {
    foreach {id type} $token {break}
    switch -glob -- $type {
      Space {
        foreach {count new_line} [lrange $token 2 end] {break}
        if {$new_line} {
          $tree insert end $parent $id \
            -text "$count newline character(s) (id=$id)" -image IHelpDoc
        } else {
          $tree insert end $parent $id \
            -text "$count space character(s) (id=$id)" -image IHelpDoc
        }
      }
      Markup {
        set tag [string tolower [lindex $token 2]]
        set attrs {}
        foreach {attr val} [lrange $token 3 end] {
          lappend attrs "$attr=\"$val\""
        }
        set attrs [string map {\n \\n} [join $attrs ,\ ]]
        switch -exact -- $tag {
          a       {set image HTMLTagAnchor}
          b       {set image HTMLTagBold}
          br      {set image HTMLTagBr}
          font    {set image ITSelectFont}
          h1      {set image HTMLTagH1}
          h2      {set image HTMLTagH2}
          h3      {set image HTMLTagH3}
          h4      {set image HTMLTagH4}
          h5      {set image HTMLTagH5}
          hr      {set image HTMLTagHr}
          i       {set image HTMLTagI}
          img     {set image ITImage}
          li      {set image HTMLTagLi}
          p       {set image HTMLTagP}
          table   {set image HTMLTagTable}
          td      {set image HTMLTagTd}
          tr      {set image HTMLTagTr}
          u       {set image HTMLTagU}
          default {set image HTMLTagMisc}
        }
        switch -glob -- $tag {
          meta      -
          br        -
          hr        -
          link      -
          img       -
          area      -
          base      -
          bgsound   -
          iframe    -
          input     -
          isindex   -
          nextid    -
          plaintext -
          */      {
            $tree insert end $parent $id -text "<$tag $attrs> (id=$id)" \
                                         -image $image
          }
          /* {
            #$tree insert end $parent $id -text "<$tag $attrs> (id=$id)" \
            #                             -image $image
            ## If our id is in the stack, we must advance the level...
            set index [lsearch -exact $stack $id]
            if {$index > 0} {
              set stack  [lrange $stack 0 [incr index -2]]
              set parent [lindex $stack end-1]
              #if {![string length $parent]} {set parent root}
            }
          }
          li {
            set index [lsearch -exact $stack $id]
            if {$index > 0} {
              set stack  [lrange $stack 0 [incr index -2]]
              set parent [lindex $stack end-1]
            }
            $tree insert end $parent $id -text "<$tag $attrs> (id=$id)" \
                                         -image $image
            set parent $id
            set end [$widget token getend $id.0]
            #if {[string length $end]} {incr end -1}
            if {[string length $end]} {lappend stack $id $end}
          }
          * {
            $tree insert end $parent $id -text "<$tag $attrs> (id=$id)" \
                                         -image $image
            set parent $id
            set end [$widget token getend $id.0]
            #if {[string length $end]} {incr end -1}
            if {[string length $end]} {lappend stack $id $end}
          }
        }
      }
      Text {
        set text [lindex $token 2]
        switch -glob -- [string tolower $text] {
          *<script*\</script\>* {
            $tree insert end $parent $id -text [string map {\n \\n} \
              "Script: \"[string range $text 0 80]...\""] \
              -image ITStar
          }
          default {
            $tree insert end $parent $id -text "\"$text\" (id=$id)" \
                                         -image IHelpDoc
          }
        };# switch -glob -- [string tolower $text]
      }
    }
  }

  update

  ## Try to transfer the selection to the tree...
  set first [$widget index sel.first]
  if {[string length $first]} {
    set last [$widget index sel.last]
    set selected {}
    foreach token [$widget token list sel.first sel.last] {
      foreach {id type tag} $token {break}
      switch -glob -- $type {
        Markup {
          if {![string match /* $tag]} {lappend selected $id}
        }
        default {
          lappend selected $id
        }
      }
    }
    eval $tree selection set $selected
    ## Open all the nodes that lead to the selection...
    set node [$tree parent [lindex $selected 0]]
    while {$node != "root"} {
      $tree opentree $node 0
      set node [$tree parent $node]
    }
    update
    $tree see [lindex $selected 0]
  }
  wm deiconify $win
  update
  catch {$tree see [lindex $selected 0]}
};# ::HtmlInfoViewer::ShowTokenTree

#############################################################################
## Contextual Menu Handler...
#############################################################################
proc ::HtmlInfoViewer::ContextualMenuHandler {widget {doc {}}} {
  global GUI_Text GUI_Options
  set menu $widget.contextualMenu
  catch {destroy $menu}
  menu $menu -tearoff 0

  $menu add checkbutton -label $GUI_Text(ShowTableStructure) \
        -variable ::HtmlInfoViewer::InfoViewer(showTableStruct) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
# if {![llength [trace list variable ::HtmlInfoViewer::InfoViewer(showTableStruct)]]} {
#   trace variable ::HtmlInfoViewer::InfoViewer(showTableStruct) \
#         w "::HtmlInfoViewer::ShowTableStruct $widget"
# }
  $menu add checkbutton -label $GUI_Text(ShowImages) \
        -variable ::HtmlInfoViewer::InfoViewer($widget:showImages) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {![llength [trace info variable \
                       ::HtmlInfoViewer::InfoViewer($widget:showImages)]]} {
    if {[string length $doc]} {
      trace variable ::HtmlInfoViewer::InfoViewer($widget:showImages) \
            w "ggi_view_documentHTMLRetrieveImagesClean $doc $widget;\
               ::HtmlInfoViewer::RefreshTraceCallback"
    } else {
      trace variable ::HtmlInfoViewer::InfoViewer($widget:showImages) \
            w ::HtmlInfoViewer::RefreshTraceCallback
    }
  }
  $menu add checkbutton -label $GUI_Text(ShowBgImages) \
        -variable ::HtmlInfoViewer::InfoViewer($widget:showBgImages) \
        -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {![llength [trace info variable \
                       ::HtmlInfoViewer::InfoViewer($widget:showBgImages)]]} {
    if {[string length $doc]} {
      trace variable ::HtmlInfoViewer::InfoViewer($widget:showBgImages) \
            w "ggi_view_documentHTMLRetrieveImagesClean $doc $widget;\
               ::HtmlInfoViewer::RefreshTraceCallback"
    } else {
      trace variable ::HtmlInfoViewer::InfoViewer($widget:showBgImages) \
            w ::HtmlInfoViewer::RefreshTraceCallback
    }
  }
  $menu add separator
  $menu add command -label $GUI_Text(Find...) -command \
        "::GUI_TextSearchFacilities::HtmlFindDialog $widget" \
         -image ITZoom -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(ChangeFont) -underline 0 -command \
    "GUI_EditPreferences .edt_prfrncs HTMLFonts" \
        -image ITSelectFont -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(Colours...) -command \
        "ggi_SetWidgetColours $widget" \
         -image ITSelectColour -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(ReloadPage) -underline 0 -command \
    "::HtmlInfoViewer::Refresh $widget" \
        -image ITRedraw -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(ViewHTMLTree...) \
        -underline 0 -command "::HtmlInfoViewer::ShowTokenTree $widget" \
        -image ITViewText -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(ViewSource...) \
        -underline 0 -command "::HtmlInfoViewer::ShowSource $widget" \
        -image ITViewText -compound $GUI_Options(MenuImageLocation)
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# ::HtmlInfoViewer::ContextualMenuHandler
