##############################################################################
##  MSAgent.tcl:                                                            ##
## ---------------------                                                    ##
##  This file contains functions for handling a Microsoft Agent.            ##
##  Initial version date: 01/10/2000                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval Agent {
global HomeDir
#catch {load [file join $HomeDir lib TclAgent TclAgent.dll]}

## LoadAgent
#  Initialise agent
proc LoadAgent {} {
  global GUI_Options tcl_platform

  ## Is there another agent instance? Destroy it...
  Destroy

  if {$GUI_Options(AgentUse) && \
      [string equal $tcl_platform(platform) windows]} {
    catch {package require TclAgent}
    if {[catch {agent create $GUI_Options(AgentFile)} GUI_Options(AgentID)]} {
      unset -nocomplain GUI_Options(AgentID)
      return
    }
    $GUI_Options(AgentID) bind <Click>        ::Agent::Click
    $GUI_Options(AgentID) bind <DblClick>     ::Agent::DblClick
    $GUI_Options(AgentID) bind <DragStart>    ::Agent::DragStart
    $GUI_Options(AgentID) bind <DragComplete> ::Agent::DragComplete
  }
};# LoadAgent

## Click
#  Click Callback
proc Click {} {
  global GUI_Options 
  if {![info exists GUI_Options(AgentID)]} {return}
  $GUI_Options(AgentID) stop
};# Click

## Click
#  Double Click Callback
proc DblClick {} {
  global GUI_Options
  if {![info exists GUI_Options(AgentID)]} {return}
  $GUI_Options(AgentID) stop
};# DblClick

## DragStart
#  Drag Start Callback
proc DragStart {} {
  global GUI_Options
  if {![info exists GUI_Options(AgentID)]} {return}
  $GUI_Options(AgentID) stop
};# DragStart

## DragComplete
#  Drag Complete Callback
proc DragComplete {} {
  global GUI_Options
  if {![info exists GUI_Options(AgentID)]} {return}
  $GUI_Options(AgentID) stop
  foreach {GUI_Options(Agent_X) GUI_Options(Agent_Y)} \
      [$GUI_Options(AgentID) position] {}
  $GUI_Options(AgentID) play RestPose
};# DragComplete

## Destroy
#  Destroy current agent...
proc Destroy {} {
  global GUI_Options

  ## Is there another agent instance? Destroy it...
  if {![info exists GUI_Options(AgentID)]} {return}
  if {[string length $GUI_Options(AgentID)]} {
    catch {$GUI_Options(AgentID) destroy}
  }
  set GUI_Options(AgentID)      {}
  set GUI_Options(AgentVisible) 0
};# Destroy

## Agent
#  Give access to the agent commands...
proc Agent {command args} {
  global GUI_Options
  
  if {![info exists GUI_Options(AgentID)]} {return}
  if {[string length $GUI_Options(AgentID)]} {
    if {!$GUI_Options(AgentUse)} {
      Destroy
    } else {
      eval $GUI_Options(AgentID) $command $args
    }
  }
};# Agent

## Move
#  Move Agent to a specific position
proc Move {{x {}} {y {}}} {
  global GUI_Options

  if {![string length $x]} {
    set x 0
    catch {set x $GUI_Options(Agent_X)}
  }
  if {![string length $y]} {
    set y 0
    catch {set y $GUI_Options(Agent_Y)}
  }
  
  if {![info exists GUI_Options(AgentID)]} {return}
  if {[string length $GUI_Options(AgentID)]} {
    if {!$GUI_Options(AgentUse)} {
      Destroy
    } else {
      $GUI_Options(AgentID) move $x $y
      set GUI_Options(Agent_X) $x
      set GUI_Options(Agent_Y) $y
    }
  }
};# Move

## Hide
#  Hide Agent
proc Hide {} {
  global GUI_Options

  if {![info exists GUI_Options(AgentID)]} {return}
  if {[string length $GUI_Options(AgentID)]} {
    catch {$GUI_Options(AgentID) hide}
    set GUI_Options(AgentVisible) 0
  }
};# Hide

## Show
#  Show Agent
proc Show {} {
  global GUI_Options
  set speak 0

  if {![info exists GUI_Options(AgentID)]} {return}
  if {$GUI_Options(AgentUse) && ![string length $GUI_Options(AgentID)]} {
    LoadAgent; set speak 1
  } elseif {!$GUI_Options(AgentUse)} {
    Destroy
  }
  if {[string length $GUI_Options(AgentID)]} {
    Move
    $GUI_Options(AgentID) show
    set GUI_Options(AgentVisible) 1
  }
  if {$speak} {Agent::Agent speak "[Agent::Agent description]"}
};# Show

## Play
#  Play an Agent animation...
proc Play {args} {
  global GUI_Options

  if {![info exists GUI_Options(AgentID)]} {return}
  if {[string length $GUI_Options(AgentID)]} {
    if {!$GUI_Options(AgentUse)} {
      Destroy
    } else {
      foreach animation $args {
        if {[string equal $animation stop]} {
          $GUI_Options(AgentID) stop
        } else {
          $GUI_Options(AgentID) play $animation
        }
      }
    }
  }
};# Play
};# namespace Agent
