##############################################################################
##  SelectFileSystem.tcl:                                                   ##
## ---------------------                                                    ##
##  This file contains utilities related to the selection of files and      ##
##  directories.                                                            ##
##                                                                          ##
##  Copyright 1998-2006 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

############################################################################
## Selection Dialogs...
##

## GUI_SelectFile
#  This function will display the file selection dialog and place the result
#  in the provided entry widget.
proc GUI_SelectFile {entry args} {
  if {[string length $entry]} {
    set InitialDir [$entry get]
    set parent [winfo toplevel $entry]
  } else {
    set InitialDir {}
    set parent .mw
  }
  set defaultextension {}
  set filetypes [list [list $::GUI_Text(All) *]]
  set initialfile {}
  set multiple 0
  set message {}
  set relativedir {}
  set title $::GUI_Text(Open)

  foreach {opt val} $args {
    switch -- $opt {
      -defaultextension {set defaultextension $val}
      -filetypes        {set filetypes $val}
      -initialdir       {set InitialDir $val}
      -initialfile      {set initialfile $val}
      -multiple         {set multiple $val}
      -message          {set message $val}
      -parent           {set parent $val}
      -relativedir      {set relativedir $val}
      -title            {set title $val}
      default {error "unkown option $opt"}
    }
  }

  if {[file exists $InitialDir]} {
    if {![file isdirectory $InitialDir]} {
      set InitialDir [file dirname $InitialDir]
    }
    set File [tk_getOpenFile -parent $parent -initialdir $InitialDir \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -multiple $multiple -title $title]
  } else {
    set File [tk_getOpenFile -parent $parent \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -multiple $multiple -title $title]
  }

  ## Remove the relative dir, if any...
  if {[string length $relativedir]} {
    if {$multiple} {
      set nFiles {}
      foreach file $File {
        if {[regexp ${relativedir}(.*) $file tmp nFile]} {
          lappend nFiles [string trimleft $nFile /\\]
        }
      }
      set File $nFiles
    } else {
      if {[regexp ${relativedir}(.*) $File tmp nFile]} {
        set File [string trimleft $nFile /\\]
      }
    }
  }
  if {[string length $entry]} {
    if {[string length $File]} {
      $entry delete 0 end
      $entry insert 0 $File
      $entry xview end
    }
  }
  return $File
};# GUI_SelectFile

## GUI_SelectFiles
#  This function will display the file selection dialog in order to select
#  a set of files. The resulting files will be inserted in the provided
#  listbox widget.
proc GUI_SelectFiles {listbox args} {
  if {[string length $listbox]} {
    set InitialDir [$listbox get 0]
    set parent [winfo toplevel $listbox]
  } else {
    set InitialDir {}
    set parent .mw
  }
  set defaultextension {}
  set filetypes [list [list $::GUI_Text(All) *]]
  set initialfile {}
  set multiple 1
  set message {}
  set relativedir {}
  set title $::GUI_Text(Open)

  foreach {opt val} $args {
    switch -- $opt {
      -defaultextension {set defaultextension $val}
      -filetypes        {set filetypes $val}
      -initialdir       {set InitialDir $val}
      -initialfile      {set initialfile $val}
      -multiple         {set multiple $val}
      -message          {set message $val}
      -parent           {set parent $val}
      -title            {set title $val}
      -relativedir      {set relativedir $val}
      default {error "unkown option $opt"}
    }
  }

  if {[file exists $InitialDir]} {
    if {![file isdirectory $InitialDir]} {
      set InitialDir [file dirname $InitialDir]
    }
    set File [tk_getOpenFile -parent $parent -initialdir $InitialDir \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -multiple $multiple -title $title]
  } else {
    set File [tk_getOpenFile -parent $parent \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -multiple $multiple -title $title]
  }
  if {[string length $listbox]} {
    if {[string length $relativedir]} {
      foreach file $File {
        if {[regexp ${relativedir}(.*) $file tmp nFile]} {
          lappend nFiles [string trimleft $nFile /\\]
        }
      }
      set File $nFiles
    }
    if {[string length $File]} {
      $listbox delete 0 end
      eval $listbox insert 0 $File
      $listbox xview moveto 1.0
    }
  }
  return $File
};# GUI_SelectFiles

## GUI_SelectSaveFile
#  This function will display the file selection dialog (in save mode)
#  and place the result in the provided entry widget.
proc GUI_SelectSaveFile {entry args} {
  if {[string length $entry]} {
    set InitialDir [file dirname [$entry get]]
    set parent [winfo toplevel $entry]
  } else {
    set InitialDir {}
    set parent .mw
  }
  set defaultextension {}
  set filetypes [list [list $::GUI_Text(All) *]]
  set initialfile {}
  set message {}
  set title $::GUI_Text(Save)

  foreach {opt val} $args {
    switch -- $opt {
      -defaultextension {set defaultextension $val}
      -filetypes        {set filetypes $val}
      -initialdir       {set InitialDir $val}
      -initialfile      {set initialfile $val}
      -message          {set message $val}
      -parent           {set parent $val}
      -title            {set title $val}
      default {error "unkown option $opt"}
    }
  }

  if {[file exists $InitialDir]} {
    if {![file isdirectory $InitialDir]} {
      set InitialDir [file dirname $InitialDir]
    }
    set File [tk_getSaveFile -parent $parent -initialdir $InitialDir \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -title $title]
  } else {
    set File [tk_getSaveFile -parent $parent \
       -filetypes $filetypes -defaultextension $defaultextension \
       -initialfile $initialfile -title $title]
  }
  if {[string length $entry]} {
    if {[string length $File]} {
      $entry delete 0 end
      $entry insert 0 $File
      $entry xview end
    }
  }
  return $File
};# GUI_SelectSaveFile

############################################################################
## Entry Widget Utilities...
##

## GUI_MakeEntryFileDropTarget
#  This function will register an entry widget as a file drop target.
proc GUI_MakeEntryFileDropTarget {entry} {
  if {!$::ApplicationInfo(DNDEnabled)} {return}

  ## According to tkdnd docs, we have to issue dnd commands only on mapped
  ## windows. So, check if our window is mapped and if it is not, postpone
  ## this call for some time (~ 2 sec), untill the window gets mapped...
  if {![winfo ismapped $entry]} {
    after 2000 "catch \"GUI_MakeEntryFileDropTarget $entry\""
    return
  }

  ## Register a dnd drop target...
  dnd bindtarget $entry text/uri-list <DragEnter> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <DragEnter>}
  ## We don't actually need a <Drag> event...
  ## dnd bindtarget $entry Files <Drag>
  dnd bindtarget $entry text/uri-list <DragLeave> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <DragLeave>}
  dnd bindtarget $entry text/uri-list <Drop> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <Drop> %D}
};# GUI_MakeEntryFileDropTarget

## GUI_MakeEntrySaveFileDropTarget
#  This function will register an entry widget as a file drop target.
proc GUI_MakeEntrySaveFileDropTarget {entry} {
  if {!$::ApplicationInfo(DNDEnabled)} {return}

  ## According to tkdnd docs, we have to issue dnd commands only on mapped
  ## windows. So, check if our window is mapped and if it is not, postpone
  ## this call for some time (~ 2 sec), untill the window gets mapped...
  if {![winfo ismapped $entry]} {
    after 2000 "catch \"GUI_MakeEntrySaveFileDropTarget $entry\""
    return
  }

  ## Register a dnd drop target...
  dnd bindtarget $entry text/uri-list <DragEnter> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <DragEnter>}
  ## We don't actually need a <Drag> event...
  ## dnd bindtarget $entry Files <Drag>
  dnd bindtarget $entry text/uri-list <DragLeave> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <DragLeave>}
  dnd bindtarget $entry text/uri-list <Drop> \
      {GUI_MakeEntryFileDropTargetHandleEvent %W <SaveDrop> %D}
};# GUI_MakeEntrySaveFileDropTarget

## GUI_MakeEntryFileDropTargetHandleEvent
#  This function will handle all registered dnd events.
proc GUI_MakeEntryFileDropTargetHandleEvent {win event {data {}}} {
  global GUI_Options GUI_Text
  switch -exact $event {
    <DragEnter> {
      $win configure -background $GUI_Options(DragEnterBackgound)
      update
    }
    <DragLeave> {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
    <Drop> {
      $win configure -background $GUI_Options(InputBackground)
      set data [lindex $data 0]
      ## Files dropped here can also have as source dnd, so may be formed as
      ## uri-lists...
      set data [GUI_NormaliseURI $data]
      if {[file isdirectory $data]} {
        gut_info [format $GUI_Text(NotFile-Directory) $data]
        return
      }
      $win delete 0 end
      $win insert end $data
      update
    }
    <SaveDrop> {
      $win configure -background $GUI_Options(InputBackground)
      set data [lindex $data 0]
      ## Files dropped here can also have as source KDE, so may be formed as
      ## uri-lists...
      set data [GUI_NormaliseURI $data]
      if {[file isdirectory $data]} {
        gut_info [format $GUI_Text(NotFile-Directory) $data]
        return
      }
      if {[file exists $data]} {
        set reply [gut_question [format $GUI_Text(OverwriteFile?) $data]] 
        if {[string equal $reply "no"]} {return}
      }
      $win delete 0 end
      $win insert end $data
      update
    }
    default {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
  }
  return default
};# GUI_MakeEntryFileDropTargetHandleEvent

## GUI_SelectDirectory
#  This function will display the directory selection dialog
#  and place the result in the provided entry widget.
proc GUI_SelectDirectory {entry args} {
  global GUI_Text
  if {[string length $entry]} {
    set InitialDir [$entry get]
    set parent [winfo toplevel $entry]
  } else {
    set InitialDir {}
    set parent .mw
  }
  set Dir [eval {tk_getDirectory -initialdir $InitialDir -ok $GUI_Text(Ok) \
      -cancel $GUI_Text(Cancel) -open $GUI_Text(Open) -msg1 \
      $GUI_Text(Directory:) -msg2 $GUI_Text(DirectoryName:)} $args]
  if {![string equal $Dir {}] && [string length $entry]} {
    $entry delete 0 end
    $entry insert 0 $Dir
    catch {$entry xview end}
  }
  return $Dir
};# GUI_SelectDirectory

## GUI_MakeEntryDirectoryDropTarget
#  This function will register an entry widget as a file drop target.
proc GUI_MakeEntryDirectoryDropTarget {entry {cmd {}}} {
  if {!$::ApplicationInfo(DNDEnabled)} {return}

  ## According to tkdnd docs, we have to issue dnd commands only on mapped
  ## windows. So, check if our window is mapped and if it is not, postpone
  ## this call for some time (~ 2 sec), untill the window gets mapped...
  if {![winfo ismapped $entry]} {
    after 2000 "catch \"GUI_MakeEntryDirectoryDropTarget $entry [list $cmd]\""
    return
  }

  ## Register a dnd drop target...
  dnd bindtarget $entry text/uri-list <DragEnter> \
      {GUI_MakeEntryDirectoryDropTargetHandleEvent %W <DragEnter>}
  ## We don't actually need a <Drag> event...
  ## dnd bindtarget $entry Files <Drag>
  dnd bindtarget $entry text/uri-list <DragLeave> \
      {GUI_MakeEntryDirectoryDropTargetHandleEvent %W <DragLeave>}
  if {![string length $cmd]} {
    dnd bindtarget $entry text/uri-list <Drop> \
        {GUI_MakeEntryDirectoryDropTargetHandleEvent %W <Drop> %D}
  } else {
    dnd bindtarget $entry text/uri-list <Drop> \
        "GUI_MakeEntryDirectoryDropTargetHandleEvent %W <Drop> %D
         $cmd"
  }
};# GUI_MakeEntryDirectoryDropTarget

## GUI_MakeEntryDirectoryDropTargetHandleEvent
#  This function will handle all registered dnd events.
proc GUI_MakeEntryDirectoryDropTargetHandleEvent {win event {dirs {}}} {
  global GUI_Options GUI_Text
  switch -exact $event {
    <DragEnter> {
      $win configure -background $GUI_Options(DragEnterBackgound)
      update
    }
    <DragLeave> {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
    <Drop> {
      $win configure -background $GUI_Options(InputBackground)
      ## Directories dropped here can also have as source KDE,
      ## so may be formed as uri-lists...
      set data [GUI_NormaliseURI [lindex $dirs 0]]
      if {![file isdirectory $data]} {
        gut_info [format $GUI_Text(NotDirectory-File) $data]
        return
      }
      $win delete 0 end
      $win insert end $data
      update
    }
    default {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
  }
  return default
};# GUI_MakeEntryDirectoryDropTargetHandleEvent

############################################################################
## Listbox Widget Utilities...
##

## GUI_MakeListboxDirectoryDropTarget
#  This function will register an entry widget as a file drop target.
proc GUI_MakeListboxDirectoryDropTarget {listbox} {
  if {!$::ApplicationInfo(DNDEnabled)} {return}

  ## According to tkdnd docs, we have to issue dnd commands only on mapped
  ## windows. So, check if our window is mapped and if it is not, postpone
  ## this call for some time (~ 2 sec), untill the window gets mapped...
  if {![winfo ismapped $listbox]} {
    after 2000 "catch \"GUI_MakeListboxDirectoryDropTarget $listbox\""
    return
  }

  ## Register a dnd drop target...
  dnd bindtarget $listbox text/uri-list <DragEnter> \
      {GUI_MakeListboxDirectoryDropTargetHandleEvent %W <DragEnter>}
  ## We don't actually need a <Drag> event...
  ## dnd bindtarget $listbox Files <Drag>
  dnd bindtarget $listbox text/uri-list <DragLeave> \
      {GUI_MakeListboxDirectoryDropTargetHandleEvent %W <DragLeave>}
  dnd bindtarget $listbox text/uri-list <Drop> \
      {GUI_MakeListboxDirectoryDropTargetHandleEvent %W <Drop> %D}
};# GUI_MakeListboxDirectoryDropTarget

## GUI_MakeListboxDirectoryDropTargetHandleEvent
#  This function will handle all registered dnd events.
proc GUI_MakeListboxDirectoryDropTargetHandleEvent {win event {dirs {}}} {
  global GUI_Options GUI_Text
  switch -exact $event {
    <DragEnter> {
      $win configure -background $GUI_Options(DragEnterBackgound)
      update
    }
    <DragLeave> {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
    <Drop> {
      $win configure -background $GUI_Options(InputBackground)
      foreach data $dirs {
        ## Directories dropped here can also have as source KDE,
        ## so may be formed as uri-lists...
        set data [GUI_NormaliseURI $data]
        if {![file isdirectory $data]} {
          gut_info [format $GUI_Text(NotDirectory-File) $data]
          continue
        }
        if {[lsearch -exact [$win get 0 end] $data] == -1} {
          $win insert end $data
        }
      }
      update
    }
    default {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
  }
  return default
};# GUI_MakeListboxDirectoryDropTargetHandleEvent

## GUI_MakeListboxFileDropTarget
#  This function will register an entry widget as a file drop target.
proc GUI_MakeListboxFileDropTarget {listbox} {
  if {!$::ApplicationInfo(DNDEnabled)} {return}

  ## According to tkdnd docs, we have to issue dnd commands only on mapped
  ## windows. So, check if our window is mapped and if it is not, postpone
  ## this call for some time (~ 2 sec), untill the window gets mapped...
  if {![winfo ismapped $listbox]} {
    after 2000 "catch \"GUI_MakeListboxFileDropTarget $listbox\""
    return
  }

  ## Register a dnd drop target...
  dnd bindtarget $listbox text/uri-list <DragEnter> \
      {GUI_MakeListboxFileDropTargetHandleEvent %W <DragEnter>}
  ## We don't actually need a <Drag> event...
  ## dnd bindtarget $listbox Files <Drag>
  dnd bindtarget $listbox text/uri-list <DragLeave> \
      {GUI_MakeListboxFileDropTargetHandleEvent %W <DragLeave>}
  dnd bindtarget $listbox text/uri-list <Drop> \
      {GUI_MakeListboxFileDropTargetHandleEvent %W <Drop> %D}
};# GUI_MakeListboxFileDropTarget

## GUI_MakeListboxFileDropTargetHandleEvent
#  This function will handle all registered dnd events.
proc GUI_MakeListboxFileDropTargetHandleEvent {win event {files {}}} {
  global GUI_Options GUI_Text
  switch -exact $event {
    <DragEnter> {
      $win configure -background $GUI_Options(DragEnterBackgound)
      update
    }
    <DragLeave> {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
    <Drop> {
      $win configure -background $GUI_Options(InputBackground)
      foreach data $files {
        ## Files dropped here can also have as source KDE,
        ## so may be formed as uri-lists...
        set data [GUI_NormaliseURI $data]
        if {![file isfile $data]} {
          gut_info [format $GUI_Text(NotFile-Directory) $data]
          continue
        }
        if {[lsearch -exact [$win get 0 end] $data] == -1} {
          $win insert end $data
        }
      }
      update
    }
    default {
      $win configure -background $GUI_Options(InputBackground)
      update
    }
  }
  return default
};# GUI_MakeListboxFileDropTargetHandleEvent

###############################################################################
## URI Utilities...
###############################################################################
proc GUI_NormaliseURI {uri} {
  switch -glob -- $uri {
    file:///*  {set uri [string range $uri 7 end]}
    file:/*    {set uri [string range $uri 5 end]}
    default    {}
  }
  ## Has the uri and %?? escapes in it?
  set index [string first % $uri 0]
  if {$index != -1} {
    foreach HexCode [regexp -inline -all -- {%[A-Za-z0-9][A-Za-z0-9]} $uri] {
      set Mapping($HexCode) [format %c [scan [string range $HexCode 1 end] %x]]
    }
    set uri [string map [array get Mapping] $uri]
  }
  return $uri
};# GUI_NormaliseURI
