##############################################################################
##  SplashScreen.tcl:                                                       ##
## ---------------------                                                    ##
##  This file implements the Ellogon splash-screen.                         ##
##  Initial version date: 20/12/1998                                        ##
##                                                                          ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## GUT_ShowSplashWindow
#   Shows a splash screen!
proc LDM_ShowSplashWindow {{imageData {}}} {
  global ApplicationInfo
  if {!$ApplicationInfo(EnableGUI) ||
      !$ApplicationInfo(DisplaySplashScreen)} {return}
  global HomeDir SplashStatus GUI_Options
  set win .splashscr
  toplevel $win -bd 1 -relief raised
  wm withdraw $win
  wm overrideredirect $win 1
  ## If application is embedded, don't insist on visibility...
  if {!$ApplicationInfo(CDM_Embeded)} {
    bind $win <Visibility> [list raise $win]
  }

  if {![string length $imageData]} {
    ## Create Image and Center splash-window on screen...
    if {[catch {image create photo Splash -file [file join $HomeDir share \
                          Themes $GUI_Options(GUI_Theme) EllogonSplash.gif]}]} {
      if {[catch {
        image create photo Splash -file \
              [file join $HomeDir share Themes Default EllogonSplash.gif]
        if {$GUI_Options(FancyStartup)} {
          image create photo SplashActiveBar -file \
              [file join $HomeDir share Themes Default EllogonSplashAcBar.ppm]
          image create photo SplashInactiveBar -file \
              [file join $HomeDir share Themes Default EllogonSplashInBar.ppm]
        }
      }]} {
        image create photo Splash
        image create photo SplashActiveBar
        image create photo SplashInactiveBar
      }
    } else {
      if {$GUI_Options(FancyStartup)} {
        if {[catch {
          image create photo SplashActiveBar -file [file join $HomeDir share \
              Themes $GUI_Options(GUI_Theme) EllogonSplashAcBar.ppm]
          image create photo SplashInactiveBar -file [file join $HomeDir share \
              Themes $GUI_Options(GUI_Theme) EllogonSplashInBar.ppm]
        }]} {
          catch {image create photo SplashActiveBar}
          catch {image create photo SplashInactiveBar}
        }
      }
    }
  } else {
    ## The caller specified image data...
    image create photo Splash -data $imageData
    image create photo SplashActiveBar
    image create photo SplashInactiveBar
  }
  set sw [winfo screenwidth  .]
  set sh [winfo screenheight .]
  set iw [image width  Splash]
  set ih [image height Splash]
  set x [expr {($sw-$iw)/2}]
  set y [expr {($sh-$ih)/2}]
  wm geometry $win +$x+$y
  set w $win.fr
  frame $w -relief ridge -bd 1
  canvas $w.image -width $iw -height $ih -bd 0 -highlightthickness 0 \
     -relief flat
  $w.image create image 0 0 -anchor nw -image Splash
  if {$GUI_Options(FancyStartup)} {
    $w.image create image -1 170 -anchor nw -image SplashInactiveBar
  }
  Progress $w.progress
  set SplashStatus { }
  label $w.label -textvariable SplashStatus -background white -foreground \
    navy
  ## Languages Bar...
  frame $w.f -bg #7e7e7c
    frame $w.f.languages -bg #7e7e7c
    label $w.f.languages.tcl -text {[Tcl]} -bg #7e7e7c -fg white
    label $w.f.languages.cpp -text {[C++]} -bg #7e7e7c -fg gray
    label $w.f.languages.java -text {[Java]} -bg #7e7e7c -fg gray
    label $w.f.languages.perl -text {[Perl]} -bg #7e7e7c -fg gray
    label $w.f.languages.python -text {[Python]} -bg #7e7e7c -fg gray
    pack $w.f.languages.tcl $w.f.languages.cpp $w.f.languages.java \
         $w.f.languages.perl $w.f.languages.python -pady 0 -padx 6 -side left
    pack $w.f.languages
  pack $w.image -side top -padx 0 -pady 0 -ipadx 0 -ipady 0
  pack $w.f -side top -fill x -expand 1
  pack $w.progress -side top -fill x -expand 1
  pack $w.label -side left -fill x -expand 1
  pack $w -padx 5 -pady 5
  wm deiconify $win
  update
  wm geometry $win [wm geometry $win]
  return $win
};# LDM_ShowSplashWindow

proc LDM_DeleteSplashWindow {{win .splashscr}} {
  if {!$::ApplicationInfo(EnableGUI) ||
      !$::ApplicationInfo(DisplaySplashScreen)} {return}
  global SplashStatus
  catch {unset SplashStatus}
  catch {image delete Splash} 
  catch {image delete SplashInactiveBar}
  catch {image delete SplashActiveBar}
  catch {destroy $win}
};# LDM_DeleteSplashWindow

## Set the status bar text in the splash window...
proc LDM_SetSplashWindowStatus { message {percent {}} {ImgWidth 0}} {
  if {!$::ApplicationInfo(EnableGUI) ||
      !$::ApplicationInfo(DisplaySplashScreen)} {return}
  if {[winfo exists .splashscr]} {
    if {[string length $percent]} {SetProgress .splashscr.fr.progress $percent}
    set ::SplashStatus $message
    if {$percent > 90} {
      catch {SplashInactiveBar copy SplashActiveBar}
    } else {
      catch {SplashInactiveBar copy SplashActiveBar -from 0 0 $ImgWidth 60}
    }
    update
  }
};# LDM_SetSplashWindowStatus

## Notify the user about various things visually...
proc LDM_SplashWindowNotify {mode value} {
  switch -exact $mode {
    language {
      .splashscr.fr.f.languages.$value configure -fg white
    }
    default {}
  }
};# LDM_SplashWindowNotify
catch {source $HomeDir/share/Themes/$GUI_Options(GUI_Theme)/SplashScreen.tcl}
