##############################################################################
##  TipsDialog.tcl:                                                         ##
## ---------------------                                                    ##
##  This file implements the tips dialog and database.                      ##
##  Initial version date: 23/9/1998                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

proc LDM_ShowTip {{parent .}} {
  set win .gut_ShowTipWindow
  if {[winfo exists $win] && [winfo exists ${win}.tipArea.handle.tip]} {
    raise $win
    catch {LDM_NextTip ${win}.tipArea.handle}
    return
  }
  if {[catch {toplevel $win}]} return
  wm title $win {Tip of the Day}
  wm transient $win $parent
  image create photo TipWindowImageIdea -data \
{R0lGODlhMgAyAMIAAAAAAAD//4SEhMbGxv//AP///////////yH+Dk1hZGUgd2l0aCBHSU1QACH5
BAEKAAUALAAAAAAyADIAAAP+WLrc/jDKSau9OGtFBtkg1H1hyQxmWhACqUqe2wiSbHboAnAAuTee
16J1YQkZhJ8CwGw2escHk0CtMqOSXnVLVWKfXK43ei1orefucvxKd93J6IdqDk/Fq/kqNBgIrnB3
WztMAn05IHMfgYF5ezYljHZfZW6Cb2ZsLy1heF9DdWiDNJ8FpGmXO6RYq6FJZ0qtmwoBBbWBtQG1
sia8S4MPvhs7urbGAQACyn/HuWZtEACHA5qaIT/OyMvMubdHymxMfWU6ypvL2MZN6WbbKtvc6k7q
8OfoE8nuQvDEx/T637ZVq/eFn798y0qZElgOYCmDCM0pHCKQ4ESK8CReZJAUUdhFixsdJAwZwSPJ
kyhTqlyZIQEAOw==}
  
  wm withdraw $win
  global GUI_Options GUI_Text tcl_platform

  # Get Tip...
  
  # Tip Section
  set top ${win}.tipArea
  frame $top -bg #c8cde0 -relief ridge -bd 2 
  frame $top.handle -bg #c8cde0 -relief flat -bd 0 

  label $top.handle.image -bg $GUI_Options(TextBg) -relief groove \
        -bd 2 -image TipWindowImageIdea
  label $top.handle.title -bg black -fg white -relief flat \
        -textvariable GUI_Text(Didyouknow?) \
        -font {helvetica 14 bold}
  label $top.handle.tipTitle -bg #c8cde0 -fg black -relief flat -bd 0 \
        -height 1 -font {helvetica 16 bold italic underline} \
	-fg #009000 -height 2
  if {[string equal $tcl_platform(platform) windows]} {
    $top.handle.tipTitle configure -font {helvetica 14 bold italic underline}
  }
  ::ScrolledWidgets::ScrolledText $top.handle.tip -bg $GUI_Options(TextBg) \
        -fg $GUI_Options(TextFg) -relief groove -bd 2 -height 8 -state disabled\
	-highlightthickness 0 -wrap word
  
  grid $top.handle.image $top.handle.title -sticky snew -padx 5 -pady 5
  grid $top.handle.tipTitle -sticky snew -padx 5 -pady 5 -columnspan 2
  grid $top.handle.tip -sticky snew -padx 5 -pady 5 -columnspan 2
  grid columnconfigure $top.handle 1 -weight 1
  grid rowconfigure $top.handle 1 -weight 1
  
  pack $top.handle -fill both -expand 1 -padx 3 -pady 3
  pack $top -fill both -expand 1 -padx 8 -pady 8

  # Button Section
  frame $win.sel -relief flat -bd 0
    checkbutton $win.sel.check -relief flat \
       -variable GUI_Options(ExplorerShowTipsDlg) \
       -textvariable GUI_Text(showTipofthedayonnextstart)    
    #$win.sel.check select
    pack $win.sel.check -side left -padx 15
  pack $win.sel -fill x -expand 1 -side top
  frame $win.buttons -relief flat -bd 0
    frame $win.buttons.c -relief flat -bd 0
      button $win.buttons.c.next  -relief raised \
         -textvariable GUI_Text(NextTip) -width 16 -command \
	 "LDM_NextTip $win.tipArea.handle"
      button $win.buttons.c.close -relief raised \
         -textvariable GUI_Text(Close) -width 16 \
         -command "destroy $win" 
      bind $win.buttons.c.close <Return> {%W invoke}
      bind $win.buttons.c.next  <Return> {%W invoke}
      pack $win.buttons.c.next $win.buttons.c.close -padx 20 -pady 5 -side left
    pack $win.buttons.c 
  pack $win.buttons -fill x -expand 1 
  LDM_NextTip $win.tipArea.handle
  update 
  gut_ModalDlg $win
  bind $win <Visibility> {}
  focus $win.buttons.c.close
  wm deiconify $win
  update
  tkwait window $win
  catch {image delete TipWindowImageIdea}
}

proc LDM_NextTip parent {
  global HomeDir TipDatabase Locale LDM_TipIndex
  if {![info exists TipDatabase($Locale)]} {
    set Path [file join $HomeDir share TipDatabase]
    if {[catch {open [file join $Path ${Locale}.tcl]} file]} {
      catch {source [file join $Path English.tcl] }
      catch {set TipDatabase($Locale) $TipDatabase(English)}
    } else {
      gets $file
      set enc [lindex [gets $file] end]
      fconfigure $file -encoding $enc
      catch {eval [read $file]}
      close $file
    }
  }
  if {![info exists TipDatabase($Locale)]} {
    return { - Cannot Load Tips Database! - }
  }

  set tips [llength $TipDatabase($Locale)]
  incr tips -1
  if {![info exists LDM_TipIndex]} {
    set LDM_TipIndex [expr $tips + 10]
    while {$LDM_TipIndex > $tips} {
      set LDM_TipIndex [expr int(rand()*($tips + 2))]
    }
  }
  set tip [lindex $TipDatabase($Locale) $LDM_TipIndex] 
  incr LDM_TipIndex
  if {$LDM_TipIndex > $tips} {set LDM_TipIndex 0}
  $parent.tipTitle configure -text [lindex $tip 0]
  $parent.tip.text configure -state normal
  $parent.tip.text delete 1.0 end
  $parent.tip.text insert end [lindex $tip 1]
  $parent.tip.text configure -state disabled
  return $tip
}
