#########################################################
# Directory Selector TCL version 1.0
#
# Daniel Roche, <dan@lectra.com>
#########################################################


#########################################################
# 
# tk_getDirectory [option value ...]
#
#  options are :
#   [-initialdir dir]     display in dir
#   [-title string]       make string title of dialog window
#   [-ok string]          make string the label of OK button
#   [-open string]        make string the label of OPEN button
#   [-cancel string]      make string the label of CANCEL button
#   [-msg1 string]        make string the label of the first directory message
#   [-msg2 string]        make string the label of the second directory message
#
#########################################################

proc tk_getDirectory {args} {
 variable fini
 global tcl_platform drives GUI_Options

#
# arguments
#
 set _titre {Select Directory...}
 set _ldir Directory:
 set _ldnam {Directory Name:}
 set _open Ok
 set _expand Open
 set _cancel Cancel
 set initialdir [pwd]

 foreach {opt val} $args {
   switch -exact -- $opt {
     -initialdir {set initialdir $val}
     -title      {set _titre $val}
     -ok         {set _open $val}
     -open       {set _expand $val}
     -cancel     {set _cancel $val}
     -msg1       {set _ldir $val}
     -msg2       {set _ldnam $val}
     default     {error "unknown option $opt"}
   }
 }

 if {![string length $initialdir]} {
   global UserHomeDir
   if {[catch {set initialdir $UserHomeDir}]} {
     set initialdir [pwd]
   }
 }

 ## Check for existance of tk_chooseDirectory
 if {[string equal $tcl_platform(platform) windows]} {
  return [tk_chooseDirectory -initialdir $initialdir -title $_titre]
 }

#
# variables et data
#
 set fini 0

 switch -exact $tcl_platform(platform) {
  unix {
   catch {font create tkGetDirFont -family courier -size 13 -weight bold}
  }
  windows {
   catch {font create tkGetDirFont -family courier -size 12}
  }
 }

#
# widgets
#
 toplevel .dirsel
 wm withdraw .dirsel
 grab set .dirsel
 wm geometry .dirsel 360x250
 wm title .dirsel $_titre

 frame .dirsel.f1 -relief flat -borderwidth 0
 #frame .dirsel.f2 -relief sunken -borderwidth 1 
 ScrolledWidgets::ScrolledCanvas .dirsel.f2 -widgetname cv -borderwidth 1 \
   -bg $GUI_Options(ExplorerBg) -relief sunken -bd $GUI_Options(BorderWidth) \
   -highlightthickness 0  -scrollwidth 12 -height 1
 frame .dirsel.f3 -relief flat -borderwidth 0
 frame .dirsel.f4 -relief flat -borderwidth 0

 label .dirsel.f1.lab -text $_ldir
 #menubutton .dirsel.f1.dir -relief raised -indicatoron 1 -anchor w \
 #    -menu .dirsel.f1.dir.m
 #menu .dirsel.f1.dir.m -tearoff 0
 ComboBox .dirsel.f1.dir -editable 0 -entrybg $GUI_Options(InputBackground) \
   -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)
 #button .dirsel.f1.up -image b_up -command UpDir
 button .dirsel.f1.up -image [TkGUI_AccessPriv updirImage] -command UpDir \
   -bd $GUI_Options(BorderWidth) -relief flat

 pack .dirsel.f1.up -side right -padx 4 -pady 4
 pack .dirsel.f1.lab -side left -padx 4 -pady 4
 pack .dirsel.f1.dir -side right -padx 4 -pady 4 -fill x -expand 1

 
 # canvas .dirsel.f2.cv -borderwidth 0 -yscrollcommand ".dirsel.f2.sb set" \
 #   -background $GUI_Options(ExplorerBg)
 ##if ![string compare $tcl_platform(platform) windows] {
  ##.dirsel.f2.cv configure -background white
 ##}
 #scrollbar .dirsel.f2.sb -command ".dirsel.f2.cv yview"
 #set scw 16
 #place .dirsel.f2.cv -x 0 -relwidth 1.0 -width [expr -$scw ] -y 0 \
         -relheight 1.0
 #place .dirsel.f2.sb -relx 1.0 -x [expr -$scw ] -width $scw -y 0 \
         -relheight 1.0
 #unset scw

 .dirsel.f2.cv bind TXT <Any-Enter> EnterItem
 .dirsel.f2.cv bind TXT <Any-Leave> LeaveItem
 .dirsel.f2.cv bind TXT <Any-Button-1> ClickItem
 .dirsel.f2.cv bind TXT <Double-Button-1> DoubleClickItem
 .dirsel.f2.cv bind IMG <Any-Enter> EnterItem
 .dirsel.f2.cv bind IMG <Any-Leave> LeaveItem
 .dirsel.f2.cv bind IMG <Any-Button-1> ClickItem
 .dirsel.f2.cv bind IMG <Double-Button-1> DoubleClickItem

 label .dirsel.f3.lnam -text $_ldnam
 entry .dirsel.f3.chosen -takefocus 0 -bd $GUI_Options(BorderWidth)
 pack .dirsel.f3.lnam -side left -padx 4 -pady 4
 pack .dirsel.f3.chosen -side right -fill x -expand 1 -padx 4 -pady 4

 button .dirsel.f4.open -text $_open -command { 
  set tmp [.dirsel.f3.chosen get]
  if {[string length $tmp]} {
   set fini 1 
  } else {
  }
 }
 button .dirsel.f4.expand -text $_expand -command DownDir
 button .dirsel.f4.cancel -text $_cancel -command { 
  set fini -1 
 }
 
 pack .dirsel.f4.open .dirsel.f4.expand -side left -padx 10 -pady 4
 pack .dirsel.f4.cancel -side right -padx 10 -pady 4

 pack .dirsel.f1 -fill x
 pack .dirsel.f2 -fill both -expand 1 -padx 6 -pady 6 
 pack .dirsel.f3 -fill x
 pack .dirsel.f4 -fill x
 #pack .dirsel.f2 -fill both -expand 1 -padx 6 -pady 6
#
# realwork
#
 ShowDir $initialdir
#update
 gut_ModalDlg .dirsel
 wm deiconify .dirsel

#
# wait user
#
 tkwait variable fini

 if { $fini == 1 } {
  set curdir [.dirsel.f1.dir cget -text]
  set nnam [.dirsel.f3.chosen get]

  set retval [file join [file dirname $curdir] $nnam]
  if {![file isdirectory $retval]} {
    set retval [file join $curdir $nnam]
  }
 } else {
  set retval {}
 }

 catch {font delete tkGetDirFont}
 destroy .dirsel
 unset drives fini
 return $retval
}

proc ShowDir {curdir} {
 global tcl_platform
 variable drives 

 #cd $curdir
 .dirsel.f1.dir configure -text $curdir
 .dirsel configure -cursor watch
 update

 set hi1 [font metrics tkGetDirFont -linespace]
 #set hi2 [image height b_dir]
 set hi2 [image height [TkGUI_AccessPriv folderImage]]
 if [expr $hi1 > $hi2] {
  set hi $hi1
 } else {
  set hi $hi2
 }
 #set wi1 [image width b_dir]
 set wi1 [image width [TkGUI_AccessPriv folderImage]]
 ## The following value affects icon+text spacing...
 incr wi1 6
 set wi2 [winfo width .dirsel.f2.cv]

 set lidir [list]; set lifile [list]
 foreach file [glob -nocomplain [file join $curdir *]] {
  if [file isdirectory [string trim $file "~"] ] { 
   lappend lidir $file
  } else {
   lappend lifile $file
  }
 }
 set sldir  [lsort $lidir]
 set slfile [lsort $lifile]

 .dirsel.f2.cv delete all
 set ind 0
 .dirsel.f2.cv create rectangle 2 2 2 3 -fill {} -outline {}
 foreach file $sldir {
  if [file isdirectory $file] { 
   #.dirsel.f2.cv create image 2 [expr $ind * $hi] \
   #           -anchor nw -image b_dir -tags IMG
   .dirsel.f2.cv create image 2 $ind \
           -anchor nw -image [TkGUI_AccessPriv folderImage] -tags IMG
   .dirsel.f2.cv create text $wi1 $ind \
           -anchor nw -text [file tail $file] -font tkGetDirFont -tags TXT
   incr ind $hi
  }
 }

 set hi2 [image height [TkGUI_AccessPriv fileImage]]
 incr hi2 6
 if {[expr $hi1 > $hi2]} { set hi $hi1 } else { set hi $hi2 }
 set wi1 [image width [TkGUI_AccessPriv fileImage]]
 incr wi1 34
 incr ind 10
 foreach file $slfile {
   .dirsel.f2.cv create image 30 $ind \
     -anchor nw -image [TkGUI_AccessPriv fileImage]
   .dirsel.f2.cv create text $wi1 $ind \
     -anchor nw -text [file tail $file] -font tkGetDirFont -fill \#555555
   incr ind $hi
 }

 set ha [expr $ind * $hi]
 #.dirsel.f2.cv configure -scrollregion [list 0 0 $wi2 $ha]
 .dirsel.f2.cv configure -scrollregion [.dirsel.f2.cv bbox all]
 .dirsel.f2.cv xview moveto 0.0
 .dirsel.f2.cv yview moveto 0.0

 set curlst [file split $curdir]
 set nbr [llength $curlst]

 #.dirsel.f1.dir.m delete 0 last
 incr nbr -2
 set VALUES {}
 for {set ind $nbr} {$ind >= 0} {incr ind -1} {
  set tmplst [ lrange $curlst 0 $ind] 
  set tmpdir [ eval file join $tmplst] 
     #.dirsel.f1.dir.m add command -label $tmpdir -command "ShowDir {$tmpdir}"
     lappend VALUES $tmpdir
 }
 .dirsel.f1.dir configure -values $VALUES -modifycmd \
   "ShowDir \[lindex \{$VALUES\} \[.dirsel.f1.dir getvalue\]\]"
 if {[info exist drives] == 0} {
   set drives [file volume]
 }
 if ![string compare $tcl_platform(platform) windows] {
  foreach drive $drives {
    .dirsel.f1.dir.m add command -label $drive -command "ShowDir {$drive}"
  }
 }
 .dirsel configure -cursor {}
 .dirsel.f3.chosen delete 0 end
 .dirsel.f3.chosen insert end [file tail $curdir]
}

proc UpDir {} {
 set curdir [.dirsel.f1.dir cget -text]
 set curlst [file split $curdir]

 set nbr [llength $curlst]
 if [ expr $nbr < 2 ] {
  return
 }
 set tmp [expr $nbr - 2]
 
 set newlst [ lrange $curlst 0 $tmp ]
 set newdir [ eval file join $newlst ]

 .dirsel.f3.chosen delete 0 end
 ShowDir $newdir
}

proc DownDir {} {
 set curdir [.dirsel.f1.dir cget -text]
 set nnam [.dirsel.f3.chosen get]

 set newdir [file join $curdir $nnam]
 if {![file isdirectory $newdir]} {
   set newdir [file join [file dirname $curdir] $nnam]
   if {![file isdirectory $newdir]} {
     global GUI_Text
     return [gut_ErrorDlg [format $GUI_Text(DirectoryDoesNotExists) $nnam]]
   }
 }

 .dirsel.f3.chosen delete 0 end
 ShowDir $newdir
}

proc EnterItem {} {
 global tcl_platform

 set id [.dirsel.f2.cv find withtag current]
 set wt [.dirsel.f2.cv itemcget $id -tags]
 if {[lsearch -exact $wt IMG] >= 0} {
  set id [.dirsel.f2.cv find above $id]
 }
 if [string compare $tcl_platform(platform) windows] {
   set cocol #00FF00
 } else {
   set cocol #0000FF
 }
 .dirsel.f2.cv itemconfigure $id -fill $cocol
}

proc LeaveItem {} {
 set id [.dirsel.f2.cv find withtag current]
 set wt [.dirsel.f2.cv itemcget $id -tags]
 if {[lsearch -exact $wt IMG] >= 0} {
  set id [.dirsel.f2.cv find above $id]
 }
 .dirsel.f2.cv itemconfigure $id -fill black
}

proc ClickItem {} {
 .dirsel.f2.cv delete BOX
 set id [.dirsel.f2.cv find withtag current]
 set wt [.dirsel.f2.cv itemcget $id -tags]
 if {[lsearch -exact $wt IMG] >= 0} {
  set id [.dirsel.f2.cv find above $id]
 }
 set bxr [.dirsel.f2.cv bbox $id]
 eval .dirsel.f2.cv create rectangle $bxr -fill #a2a2ff -outline #a2a2ff -tags BOX
 .dirsel.f2.cv lower BOX
 set nam [.dirsel.f2.cv itemcget $id -text]
 .dirsel.f3.chosen delete 0 end
 .dirsel.f3.chosen insert 0 $nam
}

proc DoubleClickItem {} {
 set id [.dirsel.f2.cv find withtag current]
 DownDir
}

