##############################################################################
##  DragNDrop.tcl:                                                          ##
## ---------------------                                                    ##
##  This file contains functions related to drag'n'drop.                    ##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## GUI_DragInitCmd
proc GUI_DragInitCmd {pathname x y win} {
  global GUI_MainWidgets
  catch {$GUI_MainWidgets(TreeView) delete -?Special?-SelectionRectangular-}
  set toplevel [winfo toplevel $pathname]
  if {[info exists GUI_MainWidgets($toplevel:CurrentSelection)]} {
    set sel [string map {_ { }} [file tail [string trim \
                        $GUI_MainWidgets($toplevel:CurrentSelection) \{\}]]]
    # There was no selection...
    if {![string length $sel] || [string equal $sel .]} {return {}}
    global GUI_Systems
    if {[info exists GUI_Systems($sel)] && \
       [string match *_task [lindex $GUI_Systems($sel) 2]] } {
      # This is a Folder...
      return {}
    } else {
      label $win.l -image ISystemSmall -relief flat -bd 0
      pack $win.l
      return [list SYSTEM move $sel]
    }
  }
  return {}
};# GUI_DragInitCmd

## GUI_DragEndCmd
proc GUI_DragEndCmd {source target operation type data drop_result} {
  if {![string length $target] || ![string equal $operation copy]} {return}
  global GUI_Text GUI_Options
  #puts "Drag Event:"
  #puts "  source:      $source"
  #puts "  target:      $target"
  #puts "  operation:   $operation"
  #puts "  type:        $type"
  #puts "  data:        $data"
  #puts "  drop_result: $drop_result"
  if {[string equal [winfo class $target] Label]} {
    catch {$target configure -background $GUI_Options(ExplorerStandoutBg)}
  }
  set Folder [string map {_ { }} [file tail $drop_result]]
  if {![string length $Folder]} {set Folder {(Root Folder)}}
  if {![string equal yes [gut_question [format \
        $GUI_Text(MoveSystemInFolder?) [string map {_ { }} $data] \
        $Folder]]]} return
  GUI_MW_MoveSystem $data $drop_result
};# GUI_DragEndCmd

## GUI_DropOverRootCmd
proc GUI_DropOverRootCmd {target source event x y operation type data} {
  switch -exact $event {
    enter {$target configure -background #32cd32}
    leave {
      global GUI_Options
      $target configure -background $GUI_Options(ExplorerStandoutBg)
    }
  }
  DropSite::setcursor based_arrow_down
  global GUI_MainWidgets
  set GUI_MainWidgets($target:DropTarget) /
  return 3
};# GUI_DropOverRootCmd

## GUI_DropOverCmd
proc GUI_DropOverCmd {target source event x y operation type data} {
  catch {$target delete DropTarget}
  if {![string equal $event motion]} {return 2}
  global GUI_MainWidgets
  set GUI_MainWidgets($target:DropTarget) {}
  set cx   [$target canvasx [expr {$x-[winfo rootx $target]}]]
  set cy   [$target canvasy [expr {$y-[winfo rooty $target]}]]
  set cx_1 [expr {$cx+1}]
  set cy_1 [expr {$cy+1}]
  ## Find all tags that are under the mouse...
  set tags [$target find overlapping $cx $cy $cx_1 $cy_1]
  ## ... and select the topmost...
  set length [llength $tags]
  if {!$length} {
    DropSite::setcursor X_cursor
    return 2
  } elseif {$length == 1} {
    set Item $tags
  } else {
    set Item [$target find closest $cx $cy]
  }
  
  set tags [$target gettags $Item]
  set id [lsearch -glob $tags \/*]
  if {$id != -1} {
    set path [lindex $tags $id]
    set type [lindex $tags [expr {$id - 1}]]
    set items [$target find withtag $path]
    foreach {x1 y1 x2 y2} [eval $target bbox $items] {
      set bbox [list [incr x1 -6] $y1 [incr x2 6] $y2]; break
    }
    set rec [eval $target create rectangle $bbox -tags DropTarget]
    set img [$target create image [incr x2 12] [expr {$y1+($y2-$y1)/2}] \
               -anchor c -tags DropTarget]
    $target lower DropTarget
    if {[string equal $type folder]} {
      set GUI_MainWidgets($target:DropTarget) $path
      DropSite::setcursor based_arrow_down
      $target itemconfigure $rec -fill #32cd32 -outline {}
      #$target itemconfigure $img -image ITik
      $target itemconfigure $img -image IAddArrowGreen
      return 3
    }
    DropSite::setcursor X_cursor
    $target itemconfigure $rec -fill #ff3e00 -outline {}
    $target itemconfigure $img -image ITCancel
    return 2
  }
  DropSite::setcursor X_cursor
  return 2
};# GUI_DropOverCmd

## GUI_DropCmd
proc GUI_DropCmd {source target x y mod type data} {
  global GUI_MainWidgets
  catch {$source delete DropTarget}
  return $GUI_MainWidgets($source:DropTarget)
}

## GUI_MW_MoveSystem
proc GUI_MW_MoveSystem {System Folder} {
  global GUI_Systems GUI_Graphs GUI_MainWidgets

  set folder [string map {_ { }} [file tail $Folder]]
  set system $System

  set Names  [join $GUI_Graphs($system) |]
  if {[string length $folder]} {
    foreach {id Name name cons} $GUI_Systems($folder) {break}
    if {[string first $Names $cons] != -1} {
      global GUI_Text
      gut_ErrorDlg [format $GUI_Text(SystemExistsInFolder) \
                           $System $folder]
      return
    }
    if {[string length $cons]} {
      append cons =$Names
    } else {
      set cons $Names
    }
  } else {
    ## Place System on "Root" folder...
    set ID 0
    foreach one [array names GUI_Systems] {
      set id [lindex $GUI_Systems($one) 0]
      if {$id > $ID} {set ID $id}
    }
    incr ID
  }
  unset GUI_Graphs($system)
  #lappend GUI_Systems($folder) $Names
  #catch {unset GUI_Systems($system)}
  if {[catch {GUI_MW_DeleteSystem $System 1} error]} {
    #puts $error
  }
  if {[string length $folder]} {
    set GUI_Systems($folder) [list $id $Name $name $cons]
  } else {
    set sys_name [string tolower [string map {{ } {}} $system]_system]
    set GUI_Systems($system) [list $ID $system $sys_name {}]
    set GUI_Graphs($system)  [list $sys_name $system]
  }
  opt_LoadSystemTree $GUI_MainWidgets(Tree)
  catch {Tree:open $GUI_MainWidgets(Tree) [string map {{ } _} $Folder]}
  if {[catch {TreeView:setselection $GUI_MainWidgets(TreeView) \
              [string map {{ } _} $Folder]}]} {
    TreeView:setselection $GUI_MainWidgets(TreeView) /
  }
  TreeView:draw $GUI_MainWidgets(TreeView)
}

##############################################################################
## Our tkdnd package!
##  Petasis, 17/01/2001
##############################################################################

## GUI_TkDND_Drag
#   This function initiates a drag action...
proc GUI_TkDND_DragInit {canvas x y} {
  global GUI_MainWidgets
  catch {$GUI_MainWidgets(TreeView) delete -?Special?-SelectionRectangular-}
  set toplevel [winfo toplevel $canvas]
  if {[info exists GUI_MainWidgets($toplevel:CurrentSelection)]} {
    set sel [string map {_ { }} \
                 [file tail $GUI_MainWidgets($toplevel:CurrentSelection)]]
    # There was no selection...
    if {![string length $sel] || [string equal $sel .]} {return}
    global GUI_Systems tcl_platform
    if {[info exists GUI_Systems($sel)] && \
       [string match *_task [lindex $GUI_Systems($sel) 2]] } {
      # This is a Folder...
      return
    } else {
      ## This is a system. We can create a Cursor window under unix...
#     if {[string equal $tcl_platform(platform) unix]} {
#       set win .tkdnd_cursor_window
#       catch {destroy $win}
#       toplevel $win
#       wm withdraw $win
#       wm overrideredirect $win 1
#       label $win.image -image ISystemSmall -bd 0
#       pack $win.image
#       wm deiconify $win
#       update
#       dnd drag $canvas -cursor $win \
#                        -callback "GUI_TkDND_CursorCallback $win %%A"
#       destroy $win
#     } else {
        dnd drag $canvas 
#     }
    }
  }
};# GUI_TkDND_DragInit

## GUI_TkDND_CursorCallback
#   This function updates the image in our cursor window, according to the
#   current drag state...
proc GUI_TkDND_CursorCallback {win action} {
  if {[string length $action]} {
    $win.image conf -image ISystemSmall
  } else {
    $win.image conf -image {}
  }
};# GUI_TkDND_CursorCallback

## Various tkdnd events...
proc GUI_TkDND_DragEnter {win x y} {
  $win conf -bg #8fbc8f
  return default
};# GUI_TkDND_DragEnter
proc GUI_TkDND_Drag {win x y} {
  return default
};# GUI_TkDND_Drag
proc GUI_TkDND_DragLeave {win x y} {
  global GUI_Options
  $win conf -bg $GUI_Options(ExplorerBg)
  return copy
};# GUI_TkDND_DragLeave
proc GUI_TkDND_Drop {win x y data} {
  global GUI_Options
  $win conf -bg $GUI_Options(ExplorerBg)
};# GUI_TkDND_Drop
proc GUI_TkDND_GetData {win x y} {
};# GUI_TkDND_GetData
