##############################################################################
##  EditPreferences.tcl:                                                    ##
## ---------------------                                                    ##
##  This file contains functions related to managing platform's preferences.##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
##############################################################################
##  George Petasis, 29/9/1999                                               ##
##############################################################################

## GUI_EditPreferences
#
proc GUI_EditPreferences {{win .edt_prfrncs} {showPage {}}} {
  global GUI_Text GUI_Options Edit_GUI_Options Edit_GUI_OptionsCmds \
         tcl_platform
  array set Edit_GUI_Options [array get GUI_Options]
  if {[winfo exists $win]} {
    raise $win
    return
  }
  set Edit_GUI_OptionsCmds {}
  if {![string equal $win .edt_prfrncs]} {catch {destroy .edt_prfrncs}}
  ## Load Images that may not have been already loaded...
  GUI_LoadImage ICollectionXML
  GUI_LoadImage IDocumentXML
  GUI_LoadImage ICollectionRegistered
  GUI_LoadImage IDocumentRegistered
  ## Create the window...
  toplevel $win
  wm withdraw $win
  wm title $win {Edit Preferences...}
  update

  ## Create the Tabbed Notebook Widget...
  set NoteBook $win.nt_book
  set NoteBookBg [$win cget -bg]
  NoteBook $NoteBook -bg $NoteBookBg
  set Pages [list Collections Documents Components Explorer System Fonts \
                  HTMLFonts GATE Appearence Themes Startup/Close OpSystem \
                  Java Perl Python] 
  if {[string equal $tcl_platform(platform) windows]} {
    lappend Pages Agents
  }
  foreach Page $Pages {
    set Parent [$NoteBook insert end $Page -text $GUI_Text($Page) \
                                           -background $NoteBookBg]
    $NoteBook itemconfigure $Page -createcmd \
         "GUI_EditPref_CreateModulePage $NoteBook $Parent $Page"
  }
  pack $NoteBook -fill both -expand 1

  ## Button Section...
  set buttons $win.but
  frame $buttons
  button $buttons.ok -textvariable GUI_Text(Ok) -command \
    "GUI_EditPref_ApplySettings; destroy $win"
  button $buttons.cancel -textvariable GUI_Text(Cancel) -command \
    "destroy $win" -default active
  bind $win <Return> "$buttons.cancel invoke"
  button $buttons.apply -textvariable GUI_Text(Apply) -command \
    GUI_EditPref_ApplySettings
  pack $buttons.ok $buttons.apply $buttons.cancel \
       -padx 15 -pady 2 -side left
  pack $buttons -padx 2 -pady 2
  
  foreach Page $Pages {
    $NoteBook raise $Page
    update
  }
  if {![string length $showPage]} {set showPage Collections}
  $NoteBook raise $showPage
  $NoteBook see   $showPage
  update
  wm deiconify $win
  tkwait window $win
  array unset Edit_GUI_Options
}

## GUI_EditPref_ApplySettings
#   Apply the User Specified Settings, wherever possible of course...
proc GUI_EditPref_ApplySettings {} {
  global GUI_Options Edit_GUI_Options Edit_GUI_OptionsCmds HomeDir
  ## Themes are a special case, so we have to handle it here seperately...
  if {![string equal $GUI_Options(GUI_Theme) $Edit_GUI_Options(GUI_Theme)]} {
    ## Well, we cannot live without the following entries...
    foreach entry {Background SelectBackground SelectForeground MenuSelectBg\
                   MenuSelectFg BorderWidth InputForeground InputBackground} {
      set local($entry) $GUI_Options($entry)
    }
    ## Remove completely the options array...
    array unset GUI_Options *
    array set GUI_Options [array get local]
    ## Reset Theme Settings to default values...
    puts " ** Reseting Theme options to default values..."
    source [file join $HomeDir share Themes ResetTheme.tcl]
    ## Try to load the user-specified theme...
    puts " ** Loading Theme \"$Edit_GUI_Options(GUI_Theme)\""
    catch {source [file join $HomeDir share Themes \
               $Edit_GUI_Options(GUI_Theme) Theme.tcl]}
    ## Unset all image locations...
    foreach image [image names] {
      if {[info exists GUI_Options(ImgLocation:$image)]} {
        unset -nocomplain GUI_Options(ImgLocation:$image)
      }
      if {[info exists Edit_GUI_Options(ImgLocation:$image)]} {
        unset -nocomplain Edit_GUI_Options(ImgLocation:$image)
      }
    }
    ## Reset all theme-related options...
    array set Edit_GUI_Options [array get GUI_Options]
  }
  array set GUI_Options [array get Edit_GUI_Options]
  foreach cmd $Edit_GUI_OptionsCmds {
    eval $cmd
  }
  GUI_CreateMouseOverImages
  catch {GUI_DrawCollectionDlg}
  catch {Agent::Agent stop
         Agent::Show}
}

## GUI_EditPref_CreateModulePage
#   Create each one of the pages displayed in the NoteBook Widget...
proc GUI_EditPref_CreateModulePage {NoteBook Path Page} {
  global GUI_Text GUI_Options Edit_GUI_Options Edit_GUI_OptionsCmds HomeDir
  set win [winfo toplevel $NoteBook]
  set Parent $Path.frame
  frame $Parent -bd 0 -relief flat
  switch -exact $Page {
    Collections {
      set Images $Parent.images
      frame $Images -relief groove -bd 2
        label $Images.col -textvariable GUI_Text(CollectionImage) -anchor e
        entry $Images.col_e; GUI_MakeEntryFileDropTarget $Images.col_e
        $Images.col_e insert end [ICollection cget -file]
        button $Images.colimg -image ICollection -command \
           "GUI_SelectImage -button $Images.colimg -entry $Images.col_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ICollection blank} \
            \"set GUI_Options(ImgLocation:ICollection) \[$Images.col_e get\]\"\
            \"ICollection configure -file \[$Images.col_e get\]\""
        label $Images.gcol -textvariable GUI_Text(CollectionImageGate) -anchor e
        entry $Images.gcol_e; GUI_MakeEntryFileDropTarget $Images.gcol_e
        $Images.gcol_e insert end [ICollectionGate cget -file]
        button $Images.gcolimg -image ICollectionGate -command \
           "GUI_SelectImage -button $Images.gcolimg -entry $Images.gcol_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ICollectionGate blank} \
            \"set GUI_Options(ImgLocation:ICollectionGate) \
               \[$Images.gcol_e get\]\" \
            \"ICollectionGate configure -file \[$Images.gcol_e get\]\""
        ## XML Collection Images...
        label $Images.xcol -textvariable GUI_Text(CollectionImageXML) -anchor e
        entry $Images.xcol_e; GUI_MakeEntryFileDropTarget $Images.xcol_e
        $Images.xcol_e insert end [ICollectionXML cget -file]
        button $Images.xcolimg -image ICollectionXML -command \
           "GUI_SelectImage -button $Images.xcolimg -entry $Images.xcol_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ICollectionXML blank} \
            \"set GUI_Options(ImgLocation:ICollectionXML) \
               \[$Images.xcol_e get\]\" \
            \"ICollectionXML configure -file \[$Images.xcol_e get\]\""
        ## Registered Collection Images...
        label $Images.rcol -textvariable GUI_Text(CollectionImageRegistered) \
           -anchor e
        entry $Images.rcol_e; GUI_MakeEntryFileDropTarget $Images.rcol_e
        $Images.rcol_e insert end [ICollectionRegistered cget -file]
        button $Images.rcolimg -image ICollectionRegistered -command \
           "GUI_SelectImage -button $Images.rcolimg -entry $Images.rcol_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ICollectionRegistered blank} \
            \"set GUI_Options(ImgLocation:ICollectionRegistered) \
               \[$Images.rcol_e get\]\" \
            \"ICollectionRegistered configure -file \[$Images.rcol_e get\]\""
        ## Modify Collection Search Path...
        label $Images.cpath -textvariable GUI_Text(ModifyColSearchPath) \
           -anchor e
        global CollectionPath
        entry $Images.cpath_e -textvariable CollectionPath
        $Images.cpath_e xview end
        button $Images.cpath_b -text ... -image ITAddCollectionPath -command \
           "GUI_EditPref_ModifySearchPath $Images CollectionPath dir"
        grid $Images.col $Images.col_e $Images.colimg -sticky ew -padx 2
        grid $Images.gcol $Images.gcol_e $Images.gcolimg -sticky ew -padx 2
        grid $Images.xcol $Images.xcol_e $Images.xcolimg -sticky ew -padx 2
        grid $Images.rcol $Images.rcol_e $Images.rcolimg -sticky ew -padx 2
        grid $Images.cpath -columnspan 3 -sticky w -padx 2
        grid $Images.cpath_e -columnspan 2 -sticky ew -padx 2
        grid $Images.cpath_b -row 4 -column 2 -rowspan 2 -sticky snew -padx 2
            
        grid columnconfigure $Images 1 -weight 1
      pack $Images -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill x -expand 1
      checkbutton $Parent.gate -textvariable GUI_Text(ManageGateCollections) \
      -variable Edit_GUI_Options(EnableGateCollectionManagement) -anchor w
      checkbutton $Parent.xml -textvariable GUI_Text(ManageXMLCollections) \
      -variable Edit_GUI_Options(EnableXMLCollectionManagement) -anchor w
      
      pack $Parent.gate -fill x -expand 1 -padx 10 -pady 3
      pack $Parent.xml  -fill x -expand 1 -padx 10 -pady 3
    }
    Documents {
      set Images $Parent.images
      frame $Images -relief groove -bd 2
        label $Images.doc -textvariable GUI_Text(DocumentImage) -anchor e
        entry $Images.doc_e; GUI_MakeEntryFileDropTarget $Images.doc_e
        $Images.doc_e insert end [IDocument cget -file]
        button $Images.docimg -image IDocument -command \
           "GUI_SelectImage -button $Images.docimg -entry $Images.doc_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IDocument blank} \
            \"set GUI_Options(ImgLocation:IDocument) \[$Images.doc_e get\]\" \
            \"IDocument configure -file \[$Images.doc_e get\]\""
        label $Images.gdoc -textvariable GUI_Text(DocumentImageGate) -anchor e
        entry $Images.gdoc_e; GUI_MakeEntryFileDropTarget $Images.gdoc_e
        $Images.gdoc_e insert end [IDocumentGate cget -file]
        button $Images.gdocimg -image IDocumentGate -command \
           "GUI_SelectImage -button $Images.gdocimg -entry $Images.gdoc_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IDocumentGate blank} \
            \"set GUI_Options(ImgLocation:IDocumentGate) \
               \[$Images.gdoc_e get\]\" \
            \"IDocumentGate configure -file \[$Images.gdoc_e get\]\""
        ## XML Document Images...
        label $Images.xdoc -textvariable GUI_Text(DocumentImageXML) -anchor e
        entry $Images.xdoc_e; GUI_MakeEntryFileDropTarget $Images.xdoc_e
        $Images.xdoc_e insert end [IDocumentXML cget -file]
        button $Images.xdocimg -image IDocumentXML -command \
           "GUI_SelectImage -button $Images.xdocimg -entry $Images.xdoc_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IDocumentXML blank} \
            \"set GUI_Options(ImgLocation:IDocumentXML) \
               \[$Images.xdoc_e get\]\" \
            \"IDocumentXML configure -file \[$Images.xdoc_e get\]\""
        ## Registered Collection Images...
        label $Images.rdoc -textvariable GUI_Text(DocumentImageRegistered) \
           -anchor e
        entry $Images.rdoc_e; GUI_MakeEntryFileDropTarget $Images.rdoc_e
        $Images.rdoc_e insert end [IDocumentRegistered cget -file]
        button $Images.rdocimg -image IDocumentRegistered -command \
           "GUI_SelectImage -button $Images.rdocimg -entry $Images.rdoc_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IDocumentRegistered blank} \
            \"set GUI_Options(ImgLocation:IDocumentRegistered) \
               \[$Images.rdoc_e get\]\" \
            \"IDocumentRegistered configure -file \[$Images.rdoc_e get\]\""
        grid $Images.doc $Images.doc_e $Images.docimg -sticky ew -padx 2
        grid $Images.gdoc $Images.gdoc_e $Images.gdocimg -sticky ew -padx 2
        grid $Images.xdoc $Images.xdoc_e $Images.xdocimg -sticky ew -padx 2
        grid $Images.rdoc $Images.rdoc_e $Images.rdocimg -sticky ew -padx 2
            
        grid columnconfigure $Images 1 -weight 1
        pack $Images -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill both -expand 1
    }
    Components {
      global PluginModulePath
      set Comp $Parent.components
      frame $Comp -relief flat -bd 0
        label $Comp.cpath -textvariable GUI_Text(ModifyComponentSearchPath) \
           -anchor e
        entry $Comp.cpath_e -textvariable PluginModulePath
        $Comp.cpath_e xview end
        button $Comp.cpath_b -image ITAddCollectionPath -width 50 -command \
           "GUI_EditPref_ModifySearchPath $Comp PluginModulePath dir"
        grid $Comp.cpath -columnspan 3 -sticky w -padx 2
        grid $Comp.cpath_e -columnspan 2 -sticky ew -padx 2
        grid $Comp.cpath_b -row 0 -column 2 -rowspan 2 -sticky snew -padx 2
            
        grid columnconfigure $Comp 1 -weight 1
      pack $Comp -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill both -expand 1
    }
    Explorer {
      set Images $Parent.images
      frame $Images -relief groove -bd 2
        label $Images.fol -textvariable GUI_Text(FolderImage) -anchor e
        entry $Images.fol_e; GUI_MakeEntryFileDropTarget $Images.fol_e
        $Images.fol_e insert end [IFolder cget -file]
        button $Images.folimg -image IFolder -command \
           "GUI_SelectImage -button $Images.folimg -entry $Images.fol_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IFolder blank} \
            \"set GUI_Options(ImgLocation:IFolder) \[$Images.fol_e get\]\"\
            \"IFolder configure -file \[$Images.fol_e get\]\""
        label $Images.sys -textvariable GUI_Text(SystemImage) -anchor e
        entry $Images.sys_e; GUI_MakeEntryFileDropTarget $Images.sys_e
        $Images.sys_e insert end [ISystem cget -file]
        button $Images.sysimg -image ISystem -command \
           "GUI_SelectImage -button $Images.sysimg -entry $Images.sys_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ISystem blank} \
            \"set GUI_Options(ImgLocation:ISystem) \[$Images.sys_e get\]\"\
            \"ISystem configure -file \[$Images.sys_e get\]\""
        label $Images.ssys -textvariable GUI_Text(SmallSystemImage) -anchor e
        entry $Images.ssys_e; GUI_MakeEntryFileDropTarget $Images.ssys_e
        $Images.ssys_e insert end [ISystemSmall cget -file]
        button $Images.ssysimg -image ISystemSmall -height 42 -command \
           "GUI_SelectImage -button $Images.ssysimg -entry $Images.ssys_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {ISystemSmall blank} \
           \"set GUI_Options(ImgLocation:ISystemSmall) \[$Images.ssys_e get\]\"\
            \"ISystemSmall configure -file \[$Images.ssys_e get\]\""
        label $Images.sfol -textvariable GUI_Text(SmallFolderImage) -anchor e
        entry $Images.sfol_e; GUI_MakeEntryFileDropTarget $Images.sfol_e
        $Images.sfol_e insert end [IFolderSmall cget -file]
        button $Images.sfolimg -image IFolderSmall -height 42 -command \
           "GUI_SelectImage -button $Images.sfolimg -entry $Images.sfol_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IFolderSmall blank} \
           \"set GUI_Options(ImgLocation:IFolderSmall) \[$Images.sfol_e get\]\"\
            \"IFolderSmall configure -file \[$Images.sfol_e get\]\""
        label $Images.sfolo -textvariable GUI_Text(SmallOpenFolderImage) \
           -anchor e
        entry $Images.sfolo_e; GUI_MakeEntryFileDropTarget $Images.sfolo_e
        $Images.sfolo_e insert end [IFolderSmallOpen cget -file]
        button $Images.sfoloimg -image IFolderSmallOpen -height 42 -command \
           "GUI_SelectImage -button $Images.sfoloimg -entry $Images.sfolo_e \
             -win $win.imgSelectDlg
           lappend Edit_GUI_OptionsCmds {IFolderSmallOpen blank} \
           \"set GUI_Options(ImgLocation:IFolderSmallOpen) \
             \[$Images.sfolo_e get\]\"\
            \"IFolderSmallOpen configure -file \[$Images.sfolo_e get\]\""
        label $Images.sdoc -textvariable GUI_Text(SmallDocumentImage) -anchor e
        entry $Images.sdoc_e; GUI_MakeEntryFileDropTarget $Images.sdoc_e
        $Images.sdoc_e insert end [IDocumentSmall cget -file]
        button $Images.sdocimg -image IDocumentSmall -height 42 -command \
         "GUI_SelectImage -button $Images.sdocimg -entry $Images.sdoc_e \
             -win $win.imgSelectDlg
         lappend Edit_GUI_OptionsCmds {IDocumentSmall blank} \
         \"set GUI_Options(ImgLocation:IDocumentSmall) \[$Images.sdoc_e get\]\"\
         \"IDocumentSmall configure -file \[$Images.sdoc_e get\]\""
        grid $Images.fol $Images.fol_e $Images.folimg -sticky ew -padx 2
        grid $Images.sys $Images.sys_e $Images.sysimg -sticky ew -padx 2
        grid $Images.ssys $Images.ssys_e $Images.ssysimg -sticky ew -padx 2
        grid $Images.sfol $Images.sfol_e $Images.sfolimg -sticky ew -padx 2
        grid $Images.sfolo $Images.sfolo_e $Images.sfoloimg -sticky ew -padx 2
        grid $Images.sdoc $Images.sdoc_e $Images.sdocimg -sticky ew -padx 2
        grid columnconfigure $Images 1 -weight 1
      pack $Images -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill x -expand 1
      button $Parent.colours -text $GUI_Text(Colours...) -command \
        {ggi_SetWidgetColours $GUI_MainWidgets(TreeView)}
      pack $Parent.colours  -fill x -expand 1 -padx 10 -pady 3
    }
    System {
      set frame $Parent.system
      frame $frame
      ## Toolbar dimensions...
      label $frame.et_colwidth -textvariable GUI_Text(ExprToolbarColWidth) \
         -anchor w
      spinbox $frame.et_cw -from 5 -to 100 -state readonly -justify c \
         -textvariable Edit_GUI_Options(SystemExprToolbarCollectionWidth)
      $frame.et_cw set $GUI_Options(SystemExprToolbarCollectionWidth)
      label $frame.et_docwidth -textvariable GUI_Text(ExprToolbarDocWidth) \
         -anchor w
      spinbox $frame.et_dw -from 5 -to 100 -state readonly -justify c \
         -textvariable Edit_GUI_Options(SystemExprToolbarDocumentWidth)
      $frame.et_dw set $GUI_Options(SystemExprToolbarDocumentWidth)
      grid $frame.et_colwidth $frame.et_cw -sticky snew -padx 2 -pady 2
      grid $frame.et_docwidth $frame.et_dw -sticky snew -padx 2 -pady 2
      ## Module dimensions...
      label $frame.mdl_ModuleWidth_l -textvariable GUI_Text(ModuleWidth:) \
         -anchor w
      spinbox $frame.mdl_ModuleWidth -from 5 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleWidth)
      $frame.mdl_ModuleWidth set $GUI_Options(ModuleWidth)
      grid $frame.mdl_ModuleWidth_l $frame.mdl_ModuleWidth \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleHeight_l -textvariable GUI_Text(ModuleHeight:) \
         -anchor w
      spinbox $frame.mdl_ModuleHeight -from 5 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleHeight)
      $frame.mdl_ModuleHeight set $GUI_Options(ModuleHeight)
      grid $frame.mdl_ModuleHeight_l $frame.mdl_ModuleHeight \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleBorderWidth_l -textvariable \
         GUI_Text(ModuleBorderWidth:) -anchor w
      spinbox $frame.mdl_ModuleBorderWidth -from 0 -to 100 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleBorderWidth)
      $frame.mdl_ModuleBorderWidth set $GUI_Options(ModuleBorderWidth)
      grid $frame.mdl_ModuleBorderWidth_l $frame.mdl_ModuleBorderWidth \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleTextWidth_l -textvariable \
         GUI_Text(ModuleTextWidth:) -anchor w
      spinbox $frame.mdl_ModuleTextWidth -from 5 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleTextWidth)
      $frame.mdl_ModuleTextWidth set $GUI_Options(ModuleTextWidth)
      grid $frame.mdl_ModuleTextWidth_l $frame.mdl_ModuleTextWidth \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleTextJustify_l -textvariable \
         GUI_Text(ModuleTextJustify:) -anchor w
      ComboBox $frame.mdl_ModuleTextJustify -values {center left right} \
         -editable 0 -textvariable Edit_GUI_Options(ModuleTextJustify) \
         -justify center 
      grid $frame.mdl_ModuleTextJustify_l $frame.mdl_ModuleTextJustify \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleTextAnchor_l -textvariable \
         GUI_Text(ModuleTextAnchor:) -anchor w
      ComboBox $frame.mdl_ModuleTextAnchor -values {center n s e w ne nw se sw}\
         -editable 0 -textvariable Edit_GUI_Options(ModuleTextAnchor) \
         -justify center
      grid $frame.mdl_ModuleTextAnchor_l $frame.mdl_ModuleTextAnchor \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleTextPadX_l -textvariable \
         GUI_Text(ModuleTextPadX:) -anchor w
      spinbox $frame.mdl_ModuleTextPadX -from -1000 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleTextPadX)
      $frame.mdl_ModuleTextPadX set $GUI_Options(ModuleTextPadX)
      grid $frame.mdl_ModuleTextPadX_l $frame.mdl_ModuleTextPadX \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleTextPadY_l -textvariable \
         GUI_Text(ModuleTextPadY:) -anchor w
      spinbox $frame.mdl_ModuleTextPadY -from -1000 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleTextPadY)
      $frame.mdl_ModuleTextPadY set $GUI_Options(ModuleTextPadY)
      grid $frame.mdl_ModuleTextPadY_l $frame.mdl_ModuleTextPadY \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleImageAnchor_l -textvariable \
         GUI_Text(ModuleImageAnchor:) -anchor w
      ComboBox $frame.mdl_ModuleImageAnchor \
         -editable 0 -textvariable Edit_GUI_Options(ModuleImageAnchor) \
         -justify center -values {center n s e w ne nw se sw}
      grid $frame.mdl_ModuleImageAnchor_l $frame.mdl_ModuleImageAnchor \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleImagePadX_l -textvariable \
         GUI_Text(ModuleImagePadX:) -anchor w
      spinbox $frame.mdl_ModuleImagePadX -from -1000 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleImagePadX)
      $frame.mdl_ModuleImagePadX set $GUI_Options(ModuleImagePadX)
      grid $frame.mdl_ModuleImagePadX_l $frame.mdl_ModuleImagePadX \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleImagePadY_l -textvariable \
         GUI_Text(ModuleImagePadY:) -anchor w
      spinbox $frame.mdl_ModuleImagePadY -from -1000 -to 1000 -state readonly \
         -justify c -textvariable Edit_GUI_Options(ModuleImagePadY)
      $frame.mdl_ModuleImagePadY set $GUI_Options(ModuleImagePadY)
      grid $frame.mdl_ModuleImagePadY_l $frame.mdl_ModuleImagePadY \
         -sticky snew -padx 2 -pady 2
      label $frame.mdl_ModuleShape_l -textvariable \
         GUI_Text(ModuleShape:) -anchor w
      set Edit_GUI_Options(ModuleShape) $GUI_Options(ModuleShape)
      ComboBox $frame.mdl_ModuleShape -values {rectangle oval} -justify center \
         -editable 0 -textvariable Edit_GUI_Options(ModuleShape)
      grid $frame.mdl_ModuleShape_l $frame.mdl_ModuleShape \
         -sticky snew -padx 2 -pady 2
      set Edit_GUI_Options(ModuleTextJustify) $GUI_Options(ModuleTextJustify)
      grid columnconfigure $frame 1 -weight 1
      pack $frame -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill x -expand 1
      lappend Edit_GUI_OptionsCmds {catch {GUI_RedrawAllOpenedSystemWindows}}
    }
    Fonts {
      set frame $Parent.fonts
      frame $frame
      foreach font [list TextFont FixedTextFont ModuleTextFont ExplorerFont \
                         ExplorerFontItalic ExplorerLargeFont ListBoxFont] {
        set lfont [string tolower $font]
        catch {font delete TempFont_$font}
        font create TempFont_$font -family [font configure $font -family] \
          -size       [font configure $font -size] \
          -weight     [font configure $font -weight] \
          -slant      [font configure $font -slant] \
          -underline  [font configure $font -underline] \
          -overstrike [font configure $font -overstrike]
        label $frame.$lfont -text $font     -anchor w -padx 6
        entry $frame.val$lfont -width 20 -font TempFont_$font
        button $frame.select$lfont -textvar GUI_Text(Select...) -padx 6 \
          -command "::FontDialog::FontDialog TempFont_$font
                    GUI_EditPref_SelectFont TempFont_$font $frame.val$lfont"
        GUI_EditPref_SelectFont TempFont_$font $frame.val$lfont
        grid $frame.$lfont $frame.val$lfont $frame.select$lfont \
          -sticky ew -padx 2 -pady 2
        grid columnconfigure $frame 1 -weight 1
      }
      lappend Edit_GUI_OptionsCmds GUI_EditPref_UpdateFonts
      pack $frame -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill x -expand 1
    }
    HTMLFonts {
      set frame $Parent.fonts
      frame $frame
      foreach font [list HTMLVariableFont HTMLFixedFont] {
        set lfont [string tolower $font]
        catch {font delete TempFont_$font}
        font create TempFont_$font -family $Edit_GUI_Options($font) -size 10
        label $frame.$lfont -text $font     -anchor w -padx 6
        entry $frame.val$lfont -width 20 -font TempFont_$font -textvariable \
          Edit_GUI_Options($font)
        button $frame.select$lfont -textvar GUI_Text(Select...) -padx 6 \
          -command "::FontDialog::FontDialog TempFont_$font
                    GUI_EditPref_SelectFont TempFont_$font $frame.val$lfont 1"
        GUI_EditPref_SelectFont TempFont_$font $frame.val$lfont 1
        grid $frame.$lfont $frame.val$lfont $frame.select$lfont \
          -sticky ew -padx 2 -pady 2
        grid columnconfigure $frame 1 -weight 1
      }
      ## The Sizes...
      for {set i 1} {$i<8} {incr i} {
        label $frame.size$i -text "$GUI_Text(Size:) $i" -anchor w -padx 6
        spinbox $frame.sizeval$i -from 1 -to 64 -textvariable \
          Edit_GUI_Options(HTMLFontSize$i)
        $frame.sizeval$i set $GUI_Options(HTMLFontSize$i)
        grid $frame.size$i $frame.sizeval$i -sticky ew -padx 2 -pady 2
      }
      checkbutton $frame.default -textvariable GUI_Text(UseDefaultFonts) \
      -variable Edit_GUI_Options(HTMLUseDefaultFonts) -anchor w
      grid $frame.default -columnspan 2 -sticky snew -padx 2 -pady 2
      pack $frame -padx 2 -pady 2 -ipadx 2 -ipady 2 -fill x -expand 1
    }
    OpSystem {
      set op $Parent.operating_system
      frame $op
      label $op.editLabel -textvariable GUI_Text(EditCmd)
      entry $op.editLabelEntry -textvariable Edit_GUI_Options(Editor)
      GUI_MakeEntryFileDropTarget $op.editLabelEntry
      button $op.buttton -image ITOpen -text ... -compound left -command \
          "GUI_SelectFile $op.editLabelEntry"
      Tooltip::tooltip $op.buttton -textvariable GUI_Text(SelectFile)
      checkbutton $op.executionServer -textvariable \
        GUI_Text(EnableExecutionServer) -variable \
        Edit_GUI_Options(EnableExecutionServer) -anchor w
      grid $op.editLabel $op.editLabelEntry $op.buttton -sticky snew -padx 2
      grid $op.executionServer -sticky snew -padx 2 -columnspan 3
      grid columnconfigure $op 1 -weight 1
      pack $op -fill both -expand 1
    }
    GATE {
      frame $Parent.fr
      checkbutton $Parent.fr.compatibility -textvariable \
        GUI_Text(FullGateCompatibility) -variable \
        Edit_GUI_Options(FullGateCompatibility) -anchor w
      GUI_LoadImage ITItemHelp
      button $Parent.fr.help -image ITItemHelp -command \
        [list ::HtmlInfoViewer::InfoViewer \
              $HomeDir/doc/GATE/GATECompatibility.html]
      Tooltip::tooltip $Parent.fr.help -textvariable GUI_Text(GetItemHelp)
      pack $Parent.fr.compatibility -side left -fill x -expand 1
      pack $Parent.fr.help -side left -padx 3
      frame $Parent.fr2
      checkbutton $Parent.fr2.getValue -textvariable \
        GUI_Text(EmulateGateGetValue) -variable \
        Edit_GUI_Options(EmulateGateGetValue) -anchor w
      button $Parent.fr2.help -image ITItemHelp -command \
        [list ::HtmlInfoViewer::InfoViewer \
              $HomeDir/doc/GATE/GATECompatibility.html#tip_GetValue]
      Tooltip::tooltip $Parent.fr2.help -textvariable GUI_Text(GetItemHelp)
      pack $Parent.fr2.getValue -side left -fill x -expand 1
      pack $Parent.fr2.help -side left -padx 3
      checkbutton $Parent.gate -textvariable GUI_Text(ManageGateCollections) \
        -variable Edit_GUI_Options(EnableGateCollectionManagement) -anchor w
      pack $Parent.fr   -fill x -expand 1 -padx 10 -pady 3
      pack $Parent.fr2  -fill x -expand 1 -padx 10 -pady 3
      pack $Parent.gate -fill x -expand 1 -padx 10 -pady 3
      lappend Edit_GUI_OptionsCmds {global CDM_FullGateCompatibility} \
       {set CDM_FullGateCompatibility $Edit_GUI_Options(FullGateCompatibility)}\
       {global CDM_EmulateGateGetValue} \
       {set CDM_EmulateGateGetValue $Edit_GUI_Options(EmulateGateGetValue)}
    }
    Appearence {
      checkbutton $Parent.menuTearOff -textvariab GUI_Text(EnableTearOffMenus) \
        -variable Edit_GUI_Options(MenuTearOff) -anchor w
      pack $Parent.menuTearOff -fill x -expand 1 -padx 10 -pady 3
      checkbutton $Parent.menuImages -textvariable GUI_Text(EnableMenuImages) \
        -variable Edit_GUI_Options(MenuImageLocation) -anchor w \
        -onvalue left -offvalue none
      pack $Parent.menuImages -fill x -expand 1 -padx 10 -pady 3
      lappend Edit_GUI_OptionsCmds \
        {option add *Menu.TearOff $Edit_GUI_Options(MenuTearOff)}
    }
    Themes {
      set ThemeDir [file join $HomeDir share Themes]
      catch {source [file join $ImageDir Theme.tcl]}
      set Themes {}
      foreach theme [lsort [glob -types d [file join $ThemeDir *]]] {
        lappend Themes [file tail $theme]
      }
      image create photo EditPref_PreviewTheme
      label $Parent.current -textvariable GUI_Text(CurrentTheme)
      ScrolledWidgets::ScrolledCanvas $Parent.image \
        -bg white -width 400 -height 280
      $Parent.image.canvas create image 0 0 -image EditPref_PreviewTheme
      ComboBox $Parent.theme \
        -foreground $GUI_Options(InputForeground) -values $Themes \
        -text $GUI_Options(GUI_Theme) -editable 0 -width 12 \
        -textvariable Edit_GUI_Options(GUI_Theme) -modifycmd \
        "GUI_EditPref_PreviewTheme $Parent.image.canvas"
      grid $Parent.current $Parent.theme -padx 4 -sticky snew
      grid $Parent.image -columnspan 2 -pady 2 -sticky snew
      grid columnconfigure $Parent 1 -weight 1
      grid rowconfigure    $Parent 1 -weight 1
      GUI_EditPref_PreviewTheme $Parent.image.canvas
    }
    Startup/Close {
      checkbutton $Parent.fancy_startup -textvariable GUI_Text(FancyStartup) \
        -variable Edit_GUI_Options(FancyStartup) -anchor w
      checkbutton $Parent.exitDefaultNo -textvariable GUI_Text(ExitDefNo) \
        -variable Edit_GUI_Options(ExitDefaultNo) -anchor w
      checkbutton $Parent.executionServer -textvariable \
        GUI_Text(EnableExecutionServer) -variable \
        Edit_GUI_Options(EnableExecutionServer) -anchor w
      checkbutton $Parent.console -textvariable GUI_Text(OpenStartupConsole) \
        -variable Edit_GUI_Options(OpenStartupConsole) -anchor w
      grid $Parent.fancy_startup -sticky snew
      grid $Parent.exitDefaultNo -sticky snew
      grid $Parent.executionServer -sticky snew
      grid $Parent.console -sticky snew
    }
    Java {
      checkbutton $Parent.enableJava -textvariable GUI_Text(EnableJava) \
        -variable Edit_GUI_Options(EnableJava) -anchor w
      checkbutton $Parent.linkStreams -textvariable \
        GUI_Text(JavaLinkStreamsToTclChannels) \
        -variable Edit_GUI_Options(JavaLinkStreams) -anchor w
      grid $Parent.enableJava -sticky snew
      grid $Parent.linkStreams -sticky snew
    }
    Perl {
      checkbutton $Parent.enablePerl -textvariable GUI_Text(EnablePerl) \
        -variable Edit_GUI_Options(EnablePerl) -anchor w
      grid $Parent.enablePerl -sticky snew
    }
    Python {
      checkbutton $Parent.enablePython -textvariable GUI_Text(EnablePython) \
        -variable Edit_GUI_Options(EnablePython) -anchor w
      grid $Parent.enablePython -sticky snew
    }
    Agents {
      GUI_LoadImage IAgentPreview
      unset -nocomplain Edit_GUI_Options(AgentID)
      label $Parent.image -image IAgentPreview
      grid $Parent.image -columnspan 3 -sticky snew -padx 2 -pady 8
      checkbutton $Parent.useAgent -textvariable GUI_Text(UseAgent) \
        -variable Edit_GUI_Options(AgentUse) -anchor w
      grid $Parent.useAgent -columnspan 3 -sticky snew -padx 2 -pady 4
      label $Parent.editLabel -textvariable GUI_Text(AgentFile)
      entry $Parent.editLabelEntry -textvariable Edit_GUI_Options(AgentFile)
      GUI_MakeEntryFileDropTarget $Parent.editLabelEntry
      $Parent.editLabelEntry xview end
      button $Parent.buttton -image ITOpen -text ... -compound left -command \
          "GUI_SelectFile $Parent.editLabelEntry -defaultextension *.* \
          -filetypes [list [list [list Agents [list *.acs]]]]"
      GUI_MakeEntryFileDropTarget $Parent.editLabelEntry
      Tooltip::tooltip $Parent.buttton -textvariable GUI_Text(SelectFile)
      grid $Parent.editLabel $Parent.editLabelEntry $Parent.buttton \
           -sticky snew -padx 2 -pady 4
      grid rowconfigure    $Parent 0 -weight 1
      grid columnconfigure $Parent 1 -weight 1
    }
    default {
      label $Parent.message -text \
      {Sorry, but this Section is not implemented yet...}
      pack $Parent.message
    }
  }
  pack $Parent -padx 3 -pady 3 -ipadx 2 -ipady 2 -fill both -expand 1
  $NoteBook compute_size
}

proc GUI_EditPref_PreviewTheme {widget} {
  global HomeDir Edit_GUI_Options

  if {[catch {image create photo EditPref_PreviewTheme -file \
     [file join $HomeDir share Themes $Edit_GUI_Options(GUI_Theme) Preview.gif]\
     }]} {
     image create photo EditPref_PreviewTheme
     EditPref_PreviewTheme blank
  } 
#  $widget configure -image EditPref_PreviewTheme
  update
  Widget_SetScrollRegion $widget
  $widget xview moveto 0
  $widget yview moveto 0
}

## GUI_SelectImage
#    Displays an Image Explorer, where the user is able to select an Image...
proc GUI_SelectImage {args} {
  global HomeDir GUI_Text GUI_Options GUI_SelectImage
  foreach {File button entry win} {{} {} {} .imgExplorer} {}
  ## Search our Themes directory to find out all directories possibly
  ## containing images...
  set values [glob -directory [file join $HomeDir share Themes] -types {d} *]
  lappend values [file join $HomeDir share Images]
  set Path [file join $HomeDir share Images]
  
  foreach {opt val} $args {
    switch -glob -- $opt {
      -bu*  {set button $val}
      -en*  {
        set entry  $val
        ## Can we get a directory from the entry widget?
        set dir [file dirname [$entry get]]
        if {[file isdir $dir]} {set Path $dir}
      }
      -pa*  {set Path   $val}
      -se*  {set File   $val}
      -wi*  {set win    $val}
      -va*  {set values $val}
      default {}
    }
  }

  if {[winfo exists $win]} {raise $win; return -code return}
  toplevel $win
  wm withdraw $win
  wm title $win {Select Image...}

  ## Path Entry...
  frame $win.path
    label  $win.path.label -textvariable GUI_Text(Path)
    ComboBox $win.path.entry -editable 1 -values $values \
      -modifycmd "GUI_SelectImageDraw $win.explorer.canvas $win.path.entry.e"
    #entry  $win.path.entry
    $win.path.entry.e insert end $Path; $win.path.entry.e xview end
    GUI_MakeEntryDirectoryDropTarget $win.path.entry.e \
       "GUI_SelectImageDraw $win.explorer.canvas $win.path.entry.e"
    button $win.path.button -text "..." -image ITOpen -compound left -command \
      "GUI_SelectDirectory $win.path.entry.e
       GUI_SelectImageDraw $win.explorer.canvas $win.path.entry.e"
    Tooltip::tooltip $win.path.button -textvariable GUI_Text(SelectDirectory)
    grid $win.path.label $win.path.entry $win.path.button -sticky snew \
      -padx 1 -pady 2
    grid columnconfigure $win.path 1 -weight 1
  pack $win.path -padx 2 -pady 6 -fill x 

  ## Canvas Widget
  ScrolledWidgets::ScrolledCanvas $win.explorer \
    -relief flat -borderwidth $GUI_Options(ExplorerBorderWidth) \
    -bg $GUI_Options(ExplorerBg) -height $GUI_Options(ExplorerTreeHeigth) \
    -width $GUI_Options(ExplorerTreeWidth) -highlightthickness 0 \
    -regionpadx 6 -regionpady 6 -width 505 -height 190
  pack $win.explorer -padx 2 -pady 0 -fill both -expand 1

  ## Image Location label...
  label $win.imageLabel -relief groove -anchor c -bd 2 -text { }
  pack $win.imageLabel -padx 50 -pady 4 -fill x

  ## Close Button
  button $win.close -textvariable GUI_Text(Close) -command "destroy $win"
  pack $win.close -side bottom -pady 4

  ## Draw Widget
  set GUI_SelectImage($win:Images) {}
  set GUI_SelectImage($win:Windows) {}
  GUI_SelectImageDraw $win.explorer.canvas $win.path.entry.e
  wm deiconify $win
  tkwait window $win
  foreach one $GUI_SelectImage($win:Images) {
    catch {image delete $one}
  }
  set Image $GUI_SelectImage($win:Selection)
  catch {unset GUI_SelectImage($win:Selection)}
  if {![string length $Image]} {return -code return}
  if {[string length $entry]} {
    catch {$entry delete 0 end;$entry insert end $Image;$entry xview end}
  }
  if {[string length $button]} {
    catch {image delete $button.Image}
    image create photo $button.Image -file $Image
    if {[catch {$button configure -image $button.Image}]} {
      image delete $button.Image
    } else {
      bind $button <Destroy> {catch {image delete %W.Image}}
    }
  }
  return Image
}

## GUI_SelectImageDraw
proc GUI_SelectImageDraw {canvas entry} {
  global HomeDir GUI_Options GUI_SelectImage
  set win [winfo toplevel $canvas]
  foreach one $GUI_SelectImage($win:Images) {
    catch {image delete $one}
  }
  catch {$canvas delete all}
  label $canvas.hide -bg [$canvas cget -bg]
  place $canvas.hide -relwidth 1 -relheight 1
  Progress $win.imageLabel.progress
  place $win.imageLabel.progress -relwidth 1 -relheight 1
  update
  set GUI_SelectImage($win:Selection) {}
  set Path [$entry get]
  if {![file isdirectory $Path]} {set Path [file join $HomeDir share Images]}
  foreach {Files x y count max_height max_width} {{} 50 50 0 50 50} {}
  foreach format {.gif .png .jpg .jpeg .bmp .xpm .ppm} {
    append Files "[glob -nocomplain [file join $Path *$format]] "
  }
  set canvas_width [winfo width $canvas]
  if {$canvas_width < 10} {set canvas_width [$canvas cget -width]}

  ## Progress...
  set Progress 0.0
  set LastProgress 0.0
  set Step [expr {100.0/([llength $Files]+1)}]
  foreach img [lsort -dictionary $Files] {
    set Progress [expr {$Progress+$Step}]
    if {$Progress - $LastProgress > 1.0} {
      SetProgress $win.imageLabel.progress $Progress
      set LastProgress $Progress
      update
    }
    if {[catch {image create photo $canvas.Large$count -file $img}]} {
      continue
    }
    #frame  $canvas.$count -bd 0 -relief flat -bg $GUI_Options(ExplorerBg)
    if {[image width  $canvas.Large$count] > $max_width || 
        [image height $canvas.Large$count] > $max_height} { 
      set Image [GUI_ShrinkImage $canvas.Large$count $max_width $max_height]
      image delete $canvas.Large$count
    } else {
      set Image $canvas.Large$count
    }
    set rec [$canvas create rectangle $x $y [expr {$x+$max_width}] \
      [expr {$y+$max_height}] -fill $GUI_Options(ExplorerBg) -outline \
      $GUI_Options(Background) -width 1]
    
    set img_id [$canvas create image [expr {$x+($max_width/2)}] \
      [expr {$y+($max_height/2)}] -image $Image -anchor c]
    foreach Id $img_id {
      $canvas bind $Id <Enter> \
        "$canvas itemconfigure $rec -fill $GUI_Options(Background)
         $win.imageLabel configure -text \"[file tail $img]\"
         $Image configure -gamma 2.0"
      $canvas bind $Id <Leave> \
        "$canvas itemconfigure $rec -fill $GUI_Options(ExplorerBg)
         $win.imageLabel configure -text { }
         $Image configure -gamma 1.0"
      $canvas bind $Id <<MB_1>> \
      "set GUI_SelectImage($win:Selection) \"$img\"; destroy $win"
    }
    lappend GUI_SelectImage($win:Images) $Image
    #lappend GUI_SelectImage($win:Windows) $canvas.$count
    #pack $canvas.$count.im 
    #$canvas create window $x $y -window $canvas.$count
    incr x 60
    if {$x > $canvas_width} {set x 50; incr y 60}
    incr count
  }
  destroy $win.imageLabel.progress $canvas.hide
  update
  ::ScrolledWidgets::CanvasUpdateScrollRegion $canvas
  $canvas yview moveto 0
  focus $canvas
}

## GUI_ShrinkImage
#   Shrinks image for display in a small button, maintaining aspect ratio.
proc GUI_ShrinkImage {im max_width max_height} {
  # Determine proper aspect ratio to get size.
  set w [image width $im]
  set h [image height $im]
  if {![string length $max_width]}  {set max_width $w}
  if {![string length $max_height]} {set max_height $h}
  if {[string match *% $max_width]} {
    set max_width [expr {int([string range $max_width 0 end-1]*$w/100.0)}]
  }
  if {[string match *% $max_height]} {
    set max_height [expr {int([string range $max_height 0 end-1]*$h/100.0)}]
  }
  # Determine size of small proportional image.
  set iw [expr {($w * $max_height)/$h}]
  # If iw is too big, use ih.
  if {$iw >= $max_width} {
    set iw $max_width
    set ih [expr {($h * $max_width)/$w}]
  } else {
    set ih $max_height
  }
  if {!$ih} {incr ih}
  # Create new, blank, small image.
  #image create photo $im.small -format gif -width $iw -height $ih
  image create photo $im.small -format gif 
  if {$w > $h} {
    set samp_x [expr {$w/$iw}]
    set samp_y $samp_x
  } else {
    set samp_y [expr {$h/$ih}]
    set samp_x $samp_y
  }
  # We always shrink.
  if {$samp_x == 1} {
    foreach {samp_x samp_y} {2 2} {break}
  }
  $im.small copy $im -subsample $samp_x $samp_y -shrink -to 0 0;# $iw $ih
  if {[image width  $im.small] > $max_width} {
    $im.small configure -width $max_width
  }
  if {[image height $im.small] > $max_height} {
    $im.small configure -height $max_height
  }
  return $im.small
};# GUI_ShrinkImage

##############################################################################
## Modify Collection Search Path Dialog
##############################################################################

## GUI_EditPref_ModifySearchPath
#   This function will create a dialog (child of the specified widget) that
#   will enable the user to change the value of $variable which holds a list
#   of files or directories (according to type)
proc GUI_EditPref_ModifySearchPath {widget variable type} {
  global GUI_CollectionDlg HomeDir $variable GUI_Options WMIcons

  if {[info exists GUI_CollectionDlg(Window)]} {
    set win $GUI_CollectionDlg(Window).modifySearchPathDlg
  } else {
    set win $widget.modifySearchPathDlg$variable
  }

  if {[winfo exists $win]} {
    raise $win
    focus $win
    return
  }

  toplevel $win
  wm withdraw $win
  catch {wm transient $win $GUI_CollectionDlg(Window)}
  wm title $win "Modify variable \"$variable\"..."

  ::ScrolledWidgets::ScrolledListbox $win.listbox -height 5 -width 40 \
    -listvar $variable
  GUI_MakeListboxDirectoryDropTarget $win.listbox.listbox
  pack $win.listbox -side left -fill both -expand 1
  frame $win.buttons -relief flat
    button $win.buttons.plus -text + -font {arial 16 bold} -foreground green \
    -command "GUI_EditPref_ModifySearchPathCallback add $variable $type $win"
    button $win.buttons.minus -text - -font {arial 16 bold} -foreground red \
    -command "GUI_EditPref_ModifySearchPathCallback remove $variable $type $win"
    button $win.buttons.ok -textvariable GUI_Text(Dismiss) \
       -command "destroy $win"
    pack $win.buttons.plus $win.buttons.minus $win.buttons.ok -side top \
       -fill x -expand 1 -pady 2 -padx 2 
  pack $win.buttons -side right

  ## Bind <Delete> to delete selected entries...
  bind $win <Delete> "$win.buttons.minus invoke"

  gut_ModalDlg $win 450 200
  catch {winico setwindow $win $WMIcons(Collection)}
  wm deiconify $win
};# GUI_EditPref_ModifySearchPath

## GUI_ColDlgModifyColSearchPathCallback
#   Callback for add/remove buttons...
proc GUI_EditPref_ModifySearchPathCallback {mode variable type win} {
  global UserHomeDir GUI_Text $variable

  switch -exact $mode {
    add     {
      entry $win.tmp_entry
      switch -glob $type {
        d* {
          GUI_SelectDirectory $win.tmp_entry -initialdir $UserHomeDir \
             -ok $GUI_Text(Ok) -cancel $GUI_Text(Cancel) -open \
             $GUI_Text(Open) -msg1 $GUI_Text(Directory:) -msg2 \
             $GUI_Text(DirectoryName:)
        }
        f* {
          GUI_SelectFile $win.tmp_entry
        }
        default {}
      }
      set Path [$win.tmp_entry get]
      destroy $win.tmp_entry
      if {![string length $Path]} {return}
      if {[lsearch -exact [set $variable] $Path] < 0} {
        lappend $variable $Path
      }
    }
    remove  {
      set index [$win.listbox.listbox curselection]
      if {![string length $index]} {return}
      set $variable [lreplace [set $variable] $index $index]
    }
    default {
    }
  }
  catch {GUI_DrawCollectionDlg}
};# GUI_ColDlgModifyColSearchPathCallback

##############################################################################
## Font Selection Utilities...
##############################################################################
proc GUI_EditPref_SelectFont {font widget {only_family 0}} {
  set string {}
  set family {}
  foreach {opt val} [font actual $font] {
    switch -exact -- $opt {
     -family     {append string $val; set family $val}
     -size       {append string ", $val"}
     -weight     {append string ", $val"}
     -slant      {append string ", $val"}
     -underline  {append string ", underline"}
     -overstrike {append string ", overstrike"}
    }
  }
  $widget delete 0 end
  if {$only_family} {
    $widget insert end $family
  } else {
    $widget insert end $string
  }
};# GUI_EditPref_SelectFont

proc GUI_EditPref_UpdateFonts {} {
  global GUI_Options
  foreach font [list TextFont FixedTextFont ModuleTextFont ExplorerFont \
                     ExplorerFontItalic ExplorerLargeFont ListBoxFont] {
    font configure $font \
      -family     [font configure TempFont_$font -family] \
      -size       [font configure TempFont_$font -size] \
      -weight     [font configure TempFont_$font -weight] \
      -slant      [font configure TempFont_$font -slant] \
      -underline  [font configure TempFont_$font -underline] \
      -overstrike [font configure TempFont_$font -overstrike]
    set GUI_Options(${font}Styles) {}
    foreach {opt val} [font actual $font] {
      switch -exact -- $opt {
       -family     {set GUI_Options($font) $val}
       -size       {set GUI_Options(${font}Size) $val}
       -weight     {lappend GUI_Options(${font}Styles) -weight $val}
       -slant      {lappend GUI_Options(${font}Styles) -slant  $val}
       -underline  {if {$val} {lappend GUI_Options(${font}Styles) -underline 1}
       }
       -overstrike {if {$val} {lappend GUI_Options(${font}Styles) -overstrike 1}
       }
      }
    }
    puts "Configured Fond \"$font\"..."
  }
};# GUI_EditPref_UpdateFonts
