##############################################################################
##  GUI.tcl:                                                                ##
## ---------------------                                                    ##
##  This file contains functions related to the GUI.                        ##
##                                                                          ##
##  Copyright 1998-2006 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
##############################################################################
# This file defines some new tcl widgets.
# George Petasis, 15/9/1998
##############################################################################
if {$::ApplicationInfo(EnableGUI)} {
  catch {package require BWidget}
  if {$::tcl_platform(os) != "Darwin"} {
    catch {package require Img}
  }
}

## Rename the NoteBook command from BWidget, so as to add a context sensitive
## pop-up...
catch {::NoteBook}
if {![catch {rename ::NoteBook ::GUI_BW_NoteBook} error]} {
  proc ::NoteBook {pathname args} {
    set widget [eval ::GUI_BW_NoteBook $pathname $args]
    bind $widget.c <<MB_2>> "GUI_NoteBookSensitivePopup $pathname"
    ## BWidget bug (corrected by Andreas Kupries)...
    ## Do not allow wheel mouse events to destroy our widget!
    # bind $widget.c <MouseWheel> {break}
    # bind $widget.c <4> {break}
    # bind $widget.c <5> {break}
  }
}

## Create the namespace where we keep information about status bars...
namespace eval ::GUI_StatusBar {
  variable StatusBar
}

## GUI_CreateImages
#
proc GUI_CreateImages {} {
  global HomeDir GUI_Options WMIcons tcl_platform
  set ImageDir    $HomeDir/share/Themes/$GUI_Options(GUI_Theme)
  set DefImageDir $HomeDir/share/Themes/Default

  ## The blank image...
  image create photo Blank
  if {![string equal $GUI_Options(GUI_Theme) Default]} {
    set DoNotLoadImages 1
    catch {source $ImageDir/Theme.tcl}
  }

  ## Explorer Icons
  foreach image {IUp IFolder IFolderSmall IFolderSmallOpen
      ISystem ISystemSmall ICollection IDocument IDocumentSmall
      IAddDocuments IRemoveDocuments ITextDocument ITinyDoc
      ICollectionGate IDocumentGate IAddArrowGreen
      IUpDown IRightLeft ICommentSmall} {
    if {[info exists  GUI_Options(ImgLocation:$image)] &&
        [file exists $GUI_Options(ImgLocation:$image)]} {
      ## Use an image other than the one provided by the theme...
      if {[catch {image create photo $image \
                               -file $GUI_Options(ImgLocation:$image)}]} {
        GUI_LoadImage $image
      }
    } else {
      GUI_LoadImage $image
    }
  }
  if {[llength [info globals tkPriv]]} {
    global tkPriv
    set tkPriv(updirImage)  IUp
    set tkPriv(folderImage) IFolderSmall
    set tkPriv(fileImage)   IDocumentSmall
    namespace eval ::tk {}
  } else {
    set ::tk::Priv(updirImage)  IUp
    set ::tk::Priv(folderImage) IFolderSmall
    set ::tk::Priv(fileImage)   IDocumentSmall
  }

  ## Module Icons...
  foreach image {IModule IModulePreCons IModulePostCons IModuleViewers
      IModuleProps} {
    if {[catch {image create photo $image -file $ImageDir/$image.gif}]} {
      if {[catch {image create photo $image -file $DefImageDir/$image.gif}]} {
        image create photo $image
        puts stderr \
           "Warning: Cannot find file: [file join $ImageDir $image.gif]"
      }
    }
  }

  ## Toolbar Icons...
  foreach image {ITBlank ITNew ITOpen ITSave ITEdit ITHelp ITExit ITNewMany
    ITViewText ITExplore ITViewAnns ITViewDocAtt ITViewColAtt ITViewTimeLg
    ITSaveLayout ITSetGrid ITAddCurveP ITCancel ITAddDepend ITRemDepend
    ITEditSystem ITRedraw ITConsole ITCreateSystem ITLogo IBookOpen
    ITCreateModule ITReloadModule ITEditModule ITUpdate ITOpenSystem
    ITReLoadModules ITReloadLastLM ITProperties ITLoad ITViewTextLarge
    ITOpenDocument ITOpenCollection ITSelectFont ITZoom ITCreateCollection
    ITDeleteCollection ITCreateDocument ITEditCollection ITUpdateDocument
    ITDeleteDocument ITSelectColour ITAddCollectionPath ICpu ITik ITTool
    ITAddModuleDir ITSearch ITEllogonMode ITWarning ITRenameSystem ITNewFolder
    ITDeleteSystem ITNewSystem ITShowTags ITCompile ITStar ITMemory
    ITMemoryClear ITTools ITImportSystem ITExportSystem ITRenameModule
    ITAdd ITRemove} {
    set found 0
    foreach FileName [list $ImageDir/$image.png $ImageDir/$image.gif \
                           $DefImageDir/$image.png $DefImageDir/$image.gif] {
      if {![catch {image create photo $image -file $FileName}]} {
        set found 1
        break
      }
    }
    if {!$found} {
      image create photo $image
      $image copy ITBlank
      puts stderr \
           "Warning: Cannot find file: [file join $ImageDir $image.gif]"
    }
  }

  ## Module (optional) Images
  if {[info exists GUI_Options(ModuleImageInactive)] &&
         [string length $GUI_Options(ModuleImageInactive)]} {
    if {[catch {
      image create photo IModuleEnabled -file $GUI_Options(ModuleImageEnabled)
      image create photo IModuleActive -file $GUI_Options(ModuleImageActive)
      image create photo IModuleRun -file $GUI_Options(ModuleImageRun)
      image create photo IModuleInactive -file $GUI_Options(ModuleImageInactive)
    } error]} {
      array set GUI_Options [list \
        ModuleImageInactive $DefImageDir/IModuleInactive.gif \
        ModuleImageEnabled  $DefImageDir/IModuleEnabled.gif  \
        ModuleImageActive   $DefImageDir/IModuleActive.gif   \
        ModuleImageRun      $DefImageDir/IModuleRun.gif]
      #gut_warning \
      #     "The Following Error Occured while Loading Module Images:\n$error\n"
      GUI_LoadImage IModuleEnabled
      GUI_LoadImage IModuleActive
      GUI_LoadImage IModuleRun
      GUI_LoadImage IModuleInactive
    }
  } else {
    image create photo IModuleEnabled
    image create photo IModuleActive
    image create photo IModuleRun
    image create photo IModuleInactive
  }

  ## Try to load the Icons from libCDM.dll. This operation is valid only under
  ## windows...
  if {[string equal $tcl_platform(platform) windows]} {
    catch {
      set dll $HomeDir/CDM/$::ApplicationInfo(LoadBinaryFolderName)/libCDM.dll
      foreach icon {AnnView Collection Console EditCol Help Html Info Main \
                    NewCollection NewModule NewSystem SelectDir System} {
        set WMIcons($icon) [winico load ${icon}WindowIcon $dll]
      }
    }
  }

  ## Finally, create the mouse "over" images...
  GUI_CreateMouseOverImages
};# GUI_CreateImages

## GUI_LoadImage
#   Makes sure that an image is loaded...
proc GUI_LoadImage {image {force 0}} {
  global HomeDir GUI_Options
  set ImageDir $HomeDir/share/Themes/$GUI_Options(GUI_Theme)

  if {!$force} {
    if {![catch {image inuse $image}]} {return $image}
  }
  if {[info exists GUI_Options(ImgLocation:$image)]} {
    ## Use an image other than the one provided by the theme...
    if {![catch {image create photo $image \
                             -file $GUI_Options(ImgLocation:$image)}]} {
      return $image
    }
  }
  foreach FileName [list $ImageDir/$image.png $ImageDir/$image.gif \
                         $HomeDir/share/Themes/Default/$image.png \
                         $HomeDir/share/Themes/Default/$image.gif] {
    if {![catch {image create photo $image -file $FileName}]} {return $image}
  }
  image create photo $image
  puts stderr "Warning: Cannot find file: [file join $ImageDir $image.gif]"
  return $image
};# GUI_LoadImage

## GUI_SaveOptions
#
proc GUI_SaveOptions {} {
  global UserAppDir Locale KeyboardLocale ApplicationName \
         CDM_EnableMemoryCompression CollectionPath PluginModulePath
  set FileName [file join $UserAppDir UserPreferences.tcl]

  ## Remove some options that have to be guessed next time Ellogon starts...
  array set GUI_Options [array get ::GUI_Options]
  unset -nocomplain GUI_Options(Background) GUI_Options(Colors) \
    GUI_Options(Encodings) \
    GUI_Options(SelectBackground) GUI_Options(SelectForeground) \
    GUI_Options(InputForeground) GUI_Options(InputBackground) \
    GUI_Options(CannotViewCollectionReported)

  if {![catch {place info .mw.fr.__h1} pane]} {
    ## Locate the "-relx" entry...
    set relx_index [lsearch -exact $pane -relx]
    set GUI_Options(MainWindowPanePercent) \
        [expr {int([lindex $pane [incr relx_index]]*100)}]
  }

  foreach font {TextFont FixedTextFont ExplorerFont ExplorerLargeFont ListBoxFont} {
    break
    set GUI_Options(${font}Styles) {}
    foreach {opt val} [font actual $font] {
      switch -exact -- $opt {
       -family     {set GUI_Options($font) $val}
       -size       {set GUI_Options(${font}Size) $val}
       -weight     {lappend GUI_Options(${font}Styles) -weight $val}
       -slant      {lappend GUI_Options(${font}Styles) -slant  $val}
       -underline  {if {$val} {lappend GUI_Options(${font}Styles) -underline 1}
       }
       -overstrike {if {$val} {lappend GUI_Options(${font}Styles) -overstrike 1}
       }
      }
    }
  }

  if {[catch {open $FileName w} File]} {
    puts "Cannot save options: $File"
    return
  }
  fconfigure $File -encoding utf-8
  puts $File {## Autogenerated - DO NOT EDIT!}
  puts $File {global Locale KeyboardLocale GUI_Options\
              CDM_EnableMemoryCompression}
  puts $File "set Locale $Locale"
  puts $File "set KeyboardLocale $KeyboardLocale"
  puts $File "array set GUI_Options \{[array get GUI_Options]\}"
  puts $File "set GUI_Options(EnableMemoryCompression)\
              $CDM_EnableMemoryCompression"
  puts $File "set CollectionPath \{$CollectionPath\}"
  puts $File "set PluginModulePath \{$PluginModulePath\}"
  close $File
};# GUI_SaveOptions

## GUI_CloseWindow
#
proc GUI_CloseWindow {widgetname {col {}}} {
  global GUI_Text GUI_GraphInfo GUI_Options GUI_HTTPD
  # If main window, prompt.
  if {[string equal $widgetname .mw]} {
    Agent::Play stop Wave
    ## Is the HTTP server running?
    if {[llength [array names GUI_HTTPD]]} {
      if {![GUI_StopHttpd]} {return}
    }
    ## If the user has left Collections opened, display a confirmation...
    if {[llength [info commands Col*]]} {
      set result [gut_question $GUI_Text(Doyoureallywanttoquit?) {Exit ?} \
                               $GUI_Options(ExitDefaultNo)]
    } else {
      set result yes
    }
    if {[string equal $result yes]} {
      global GUI_Options Babel_ExecServer GUI_SystemsModified
      ## Are Systems modified?
      if {$GUI_SystemsModified} {
        set result [gut_question $GUI_Text(SaveModifiedSystems?) {Exit ?} 0]
        if {[string equal $result yes]} {opt_SaveConfig}
      }
      Agent::Hide
      catch {GUI_SaveOptions}
      Agent::Destroy
      catch {close $Babel_ExecServer}
      exit 0
    }
  } else {
    # Not the main window, destroy it.
    update
    if {[string equal $col $widgetname]} {
      set canvas $widgetname.frame.canvas
      if {[info exists GUI_GraphInfo(CCollection_${canvas})]} {
        set col $GUI_GraphInfo(CCollection_${canvas})
      } else {
        set col {}
      }
    }
    ## System Window...
    if {![string equal $col {}]} {
      if {[info exists \
          GUI_GraphInfo(CollectionChanged_$widgetname.frame.canvas)] && \
          [$col Modified]} { 
        set result [gut_question $GUI_Text(DoyouwanttosavecurrentCollection?) \
                       {Save Collection ?} 1 $widgetname {yes no cancel}]
        switch $result {
          yes {tip_Sync $col}
          cancel {return}
          default {}
        }
      }
      tip_Close $col
      CDM_ClearUnusedSavedValues
    }
    ## Clean Up global arrays
    set unset [array names GUI_GraphInfo "*${widgetname}*"]
    foreach name $unset {unset GUI_GraphInfo($name)}
    destroy $widgetname
    unset -nocomplain ::GUI_StatusBar::StatusBar($widgetname) \
                      ::GUI_StatusBar::StatusBar($widgetname:Def)
  }
};# GUI_CloseWindow

## GUI_SetLocale
#
proc GUI_SetLocale {window x y} {
  global Locale
  
  if {[string length $window]} {GUI_SelectLocale $window Locale;return {}}
  GUI_ReadLocale
  GUI_CreateMainMenu
  if {[info exists ::GUI_StatusBar::StatusBar]} {
    foreach win [array names ::GUI_StatusBar::StatusBar] {
      if {[winfo exists $win]} {
        ## Change the Locale Image
        if {[catch {$win.statusbar.locale configure -image LocaleImg-$Locale \
                 -width 21}]} {
          catch {$win.statusbar.locale configure -image {} -width 10}
        }
        if {[string equal $win .mw]} {continue}
        if {[string equal $win .c_dlg]} {        ;# Collection Dialog
          GUI_CreateCollectionMenu $win
        } elseif {[string match \.handle_system_* $win]} { ;# Edit System Dialog
          ::EditSystem::GUI_CreateHandleSystemsMenu $win
        } elseif {[string match \.system_board_*  $win]} { ;# System Board
          ::SystemBoard::CreateMainMenu $win
        } else {
          GUI_CreateSystemMenu $win
        }
      } else {
        ## Clear the Status Bar array...
        #unset ::GUI_StatusBar::StatusBar($win)
      }
    }
  }
};# GUI_SetLocale

## GUI_SetKeyboardLocale
#
proc GUI_SetKeyboardLocale {window x y} {
  global KeyboardLocale HomeDir GUI_Options

  ## Select Encoding Choice
  set menu $window.klocale_menu
  if {[string equal $window .]} {set menu .klocale_menu}
  catch {destroy $menu}
  menu $menu -type normal -tearoff 0
  set Cmd {set KeyboardLocale $Encoding;puts "Loading Keyboard Locale: $EncodingLabel ($KeyboardLocale)"}
  GUI_AppendEncodingsToMenu $menu $Cmd
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  tk_popup $menu [winfo pointerx .mw] [winfo pointery .mw]
}

## GUI_SelectLocale
#
proc GUI_SelectLocale {window Variable} {
  global HomeDir

  set Locales [lsort [glob -nocomplain [file join $HomeDir share Locale *.msg]]]
  foreach Locale $Locales {
    if {[file isfile $Locale]} {
      lappend locales [file rootname [file tail $Locale]]
    }
  }
  set menu $window.locale_menu
  if {[string equal $window .]} {set menu .locale_menu}
  catch {destroy $menu}
  menu $menu -tearoff 0
  foreach locale $locales {
    set imgFile [file join $HomeDir share Locale $locale.gif]
    set img ITBlank
    if {[file exists $imgFile]} {
      if {![catch [list image create photo LocaleImg-$locale -file $imgFile]]} {
        set img LocaleImg-$locale
      }
    }
    $menu add command -label $locale -command "set $Variable $locale" \
      -image $img -compound left
  }
  if {$::GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  tk_popup $menu [winfo pointerx .mw] [winfo pointery .mw]
}

proc GUI_SetSystemLocale {args} {
  global KeyboardLocale CDM_DefaultEncoding
  encoding system $KeyboardLocale
  fconfigure stderr -encoding $KeyboardLocale
  fconfigure stdout -encoding $KeyboardLocale
  set CDM_DefaultEncoding $KeyboardLocale
}

proc GUI_SetInterfaceLocale {args} {
  GUI_SetLocale {} 0 0
}

proc GUI_AppendEncodingsToMenu {menu_parent {Cmd {}} {showBinary 0}} {
  global GUI_Options GUI_Text
  set new_col 0
  set menu $menu_parent
  foreach encoding $GUI_Options(Encodings) {
    foreach {enc label} $encoding {break}
    switch -- $enc {
      - { 
        if {[string equal $label -]} { ;# Menu Separator
          set new_col 1
        } else { ;# Cascade Menu
          set menu $menu_parent.[string tolower [string map {{ } _} $label]]
          menu $menu -tearoff 0
          set image [::Encoding::GetEncodingImage [string map {{ } _} $label]]
          if {[info exists GUI_Text($label)]} {set label $GUI_Text($label)}
          $menu_parent add cascade -menu $menu -label $label \
            -image $image -compound $GUI_Options(MenuImageLocation) \
        }
      }
      binary {
        if {$showBinary} {
          if {![string length $label]} {
            set label [string toupper $enc]
          } else {
            if {[info exists GUI_Text($label)]} {set label $GUI_Text($label)}
          }
          set ECmd $Cmd
          set image [::Encoding::GetEncodingImage $enc]
          regsub -all -- {\$EncodingLabel} $Cmd  $label ECmd
          regsub -all -- {\$EncodingImage} $ECmd $image ECmd
          regsub -all -- {\$Encoding}      $ECmd $enc   ECmd
          $menu add command -label $label -accelerator "($enc)" \
            -image $image -compound $GUI_Options(MenuImageLocation) \
            -columnbreak $new_col -command $ECmd
          set new_col 0
        }
      }
      default {
        if {![string length $label]} {
          set label [string toupper $enc]
        } else {
          if {[info exists GUI_Text($label)]} {set label $GUI_Text($label)}
        }
        set ECmd $Cmd
        set image [::Encoding::GetEncodingImage $enc]
        regsub -all -- {\$EncodingLabel} $Cmd  $label ECmd
        regsub -all -- {\$EncodingImage} $ECmd $image ECmd
        regsub -all -- {\$Encoding}      $ECmd $enc   ECmd
        $menu add command -label $label -accelerator "($enc)" \
          -image $image -compound $GUI_Options(MenuImageLocation) \
          -columnbreak $new_col -command $ECmd
        set new_col 0
      }
    };# switch 
  }
  if {$::GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
};# GUI_AppendEncodingsToMenu

## GUI_ReadLocale
#
proc GUI_ReadLocale {} {
  global HomeDir Locale LocaleEncoding
  global GUI_Text GUI_Underline
  if {[catch {open [file join $HomeDir share Locale $Locale.msg] r} File]} { 
    gut_error \
    "Cannot Open File [file join $HomeDir share Locale $Locale.msg] for Reading"
    return {}
  } else {
    fconfigure $File -encoding utf-8
    set encoding [gets $File]
    set LocaleEncoding [lindex [split [gets $File]] end]
    puts "Loading Locale $Locale ($LocaleEncoding)"
    if {![string equal $LocaleEncoding utf-8]} {
      fconfigure $File -encoding $LocaleEncoding
    }
    set FileData [read $File]
    close $File
    eval $FileData
    unset FileData

    ## Load Image, if available...
    catch {image create photo LocaleImg-$Locale -file \
           $HomeDir/share/Locale/$Locale.gif}
    ## Read Resources, if available...
    set SysEnc [encoding system]
    encoding system $LocaleEncoding
    catch {option readfile $HomeDir/share/Locale/$Locale.rc}
    encoding system $SysEnc
  }
};# GUI_ReadLocale

## GUI_RegisterTools
proc GUI_RegisterTools {} {
  global creole_config GUI_ToolConfig
  if {[array exists GUI_ToolConfig]} {
    foreach module [array names GUI_ToolConfig] {
      set index [string first :: $module]
      set Class [string range $module 0 [incr index -1]]
      set Tool  [string range $module [incr index 3] end]
      # puts "Class: \"$Class\", Tool: \"$Tool\""
      switch -exact -- [PUL_ToolConfig $Tool type $Class] {
        button {}
        default {unset -nocomplain creole_config($Tool)}
      }
    }
  }
};# GUI_RegisterTools

## GUI_GetScrollRegion
proc GUI_GetScrollRegion {canvas} {
  return [$canvas bbox all]
}

## GUI_CreateMouseOverImages
proc GUI_CreateMouseOverImages {} {
  global GUI_Options
  ## Here, we have to create the images that will be used as "over" images,
  ## i.e shown when the mouse cursor is over the item...
  foreach image {IFolderSmall IFolderSmallOpen IFolder ISystem ISystemSmall} {
    catch {image delete ${image}Over}
    image create photo ${image}Over
    ${image}Over copy $image
    ${image}Over configure -gamma $GUI_Options(OverImageGamma)
  }
};# GUI_CreateMouseOverImages

if {$ApplicationInfo(EnableGUI)} {
  GUI_CreateImages;# Load them always at startup...
}
