##############################################################################
##  GUI_Utilities.tcl:                                                      ##
## --------------------                                                     ##
##  This file contains some commonly needed GUI utility functions...        ##
##                                                                          ##
##  Copyright 1998-2005 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::GUI {}

proc ::GUI::FillAttributeComboFromAnnotationCombo {ann_combo attr_combo} {
  set Annotation [lindex [$ann_combo cget -values] [$ann_combo getvalue]]
  $attr_combo configure -values [::Annotation::AnnotationAttributes $Annotation]
  $attr_combo setvalue first
};# ::GUI::FillAttributeComboFromAnnotationCombo

proc ::GUI::VirtualKeyboard {widget} {
  set win $widget.virtualKeyboardToplevel
  if {[winfo exists $win]} {raise $win; return}
  toplevel $win
  wm withdraw $win
  wm title $win "Virtual Keyboard"
  pack [virtualKeyboard::virtualKeyboard $win.vkf -receiver [list $widget]] \
    -fill both -expand 1
  wm deiconify $win
};# ::GUI::VirtualKeyboard

proc ::GUI::RemoveAllImagesFromMenu {menu} {
  if {![string length $menu]} {return}
  set last [$menu index last]
  if {[string equal $last none]} {return}
  for {set index 0} {$index <= $last} {incr index} {
    switch [$menu type $index] {
      separator - tear-off {}
      cascade {
        ## Cascade entries are different. We have to also follow its menu...
        $menu entryconfigure $index -image {}
        RemoveAllImagesFromMenu [$menu entrycget $index -menu]
      }
      default {
        $menu entryconfigure $index -image {}
      }
    }
  }
};# ::GUI::RemoveAllMenuImages

proc ::GUI::TraceCommandExecution {} {
  foreach dir {GUI Utils} {
    foreach file [glob -directory $dir *.tcl] {
      catch {uplevel #0 source $file}
    }
  }
  namespace eval ::GUI::TraceCommandExecution {
    catch {file delete -force $::CDM_TempDir/command_log.txt}
    proc __Log {msg} {
      set file [open $::CDM_TempDir/command_log.txt a]
      puts $file $msg
      close $file
    }
  }
  set images [image names]
  foreach command [lsort [info commands ::*]] {
    set command [string range $command 2 end]
    if {[lsearch $images $command] != -1} {continue}
    if {[winfo exists $command]} {continue}
    switch -glob  $command {
      after - error - interp - pkg_mkIndex - tcl_startOfPreviousWord -
      append - eval - join - proc - tcl_wordBreakAfter - array -
      exec - lappend - puts - tcl_wordBreakBefore - auto_execok -
      exit - lassign - pwd - tcltest - auto_import - lindex -
      auto_load - fblocked - linsert - read - tell - auto_mkindex -
      fconfigure - list - regexp - time - auto_mkindex_old - fcopy -
      llength - registry - auto_qualify - file - regsub - trace - 
      auto_reset - fileevent - lrange - rename - unknown - bgerror -
      lrepeat - return - unload - binary - flush - lreplace - scan - unset - 
      break - for - lsearch - seek - update - catch - foreach - lset - set -
      uplevel - cd - format - lsort - socket - upvar - clock - gets -
      memory - source - variable - close - glob - msgcat - split -
      vwait - concat - global - namespace - string - while - continue -
      history - open - subst - dde - http - package - switch - dict -
      parray - tcl_endOfWord - info - incr - if - expr {}
      winfo - tk {}
      auto_* - *:cmd {}
      default {
        puts $command
        rename ::$command ::${command}_original_cmd
        proc ::$command {args} "\
          ::GUI::TraceCommandExecution::__Log \"Enter: $command \$args\"
          set result \[eval ${command}_original_cmd \$args\]
          ::GUI::TraceCommandExecution::__Log \"Leave: $command \$args\"
          return \$result"
      }
    }
  }
};# ::GUI::TraceCommandExecution
