##############################################################################
##  HandleSystems.tcl:                                                      ##
## ---------------------                                                    ##
##  This file contains functions related to System editing.                 ##
##                                                                          ##
##  Copyright 1998-2005 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
############################################################################
## Petasis George, 3/11/1998
############################################################################
namespace eval ::EditSystem {

## GUI_CreateHandleSystemMenu
#
proc GUI_CreateHandleSystemsMenu {win} {
  global GUI_Text GUI_Underline GUI_Options
  variable Win2System
  set System $Win2System($win)
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }
  
  ## File Menu
  set system $menubar._system_ ;# Avoid the System Entry under Windows...
  catch {destroy $system}
  $menubar add cascade -label $GUI_Text(System) -menu $system \
     -underline $GUI_Underline(System)
  menu $system
  $system add command -label $GUI_Text(Save) -underline \
     $GUI_Underline(Save) -command [list EditSystem::SaveSystem $System] \
     -image ITSave -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(Close) -underline \
     $GUI_Underline(Close) -command "GUI_CloseWindow $win $win" \
     -image ITCancel -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $file}

  ## View Menu
  set view $menubar.view; catch {destroy $view}
  $menubar add cascade -label $GUI_Text(View) -menu $view \
     -underline $GUI_Underline(View)
  menu $view
  $view add command -label $GUI_Text(ShowToolbars) -underline \
     $GUI_Underline(ShowToolbars) -command \
     "Toolbar::deiconify $win.tf.toolbar1" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $view}
  
  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help

  return $menubar
}

variable Win2System
## GUI_CreateHandleSystemsWindow
#
proc GUI_CreateHandleSystemsWindow {{system New} {title {}}} {
  global GUI_Text GUI_Underline Locale KeyboardLocale \
         GUI_Options WMIcons
  variable Win2System

  set win .handle_system_[string map {{ } _} $system]
  if {[winfo exists $win]} {raise $win; focus $win; return}
  set title [join [split $title _]]
  toplevel $win
  wm withdraw $win
  wm title $win "$title"
  set Win2System($win) $system
  set menubar [GUI_CreateHandleSystemsMenu $win]
  bind $win <Destroy> "catch \"unset ::EditSystem::Win2System($win)\""
  $win configure -menu $menubar
  
  ## Status Bar
  set ::GUI_StatusBar::StatusBar($win) {}
  set ::GUI_StatusBar::StatusBar($win:Def) {}
  GUI_CreateStatusBar $win.statusbar

  ## Toolbar...
  set tf $win.tf
  Toolbar::ToolbarFrame $tf \
    -relief $GUI_Options(ToolbarFrameRelief) \
    -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
    -decorate $GUI_Options(ToolbarDecorate)

    set toolbar $tf.toolbar1 
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) -image ITSave \
       -command [list ::EditSystem::SaveSystem $system] \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command "destroy $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left 
  pack $tf -side top -fill x

  ## Main Window Area...
  set frame $win.frame; panedwindow $frame -relief flat -orient vertical

    ## New System Section
    set section $frame.sys; set sys_canvas $section.canvas
    global GUI_Graphs
    if {[string equal $system {Custom_New}]} {
      set GUI_Graphs($system) [list $system 1x1+0+0 0 1x1 {} {}]
    }
    GUI_CreateSystemWindow $system {} $section
    variable SystemModules
    set SystemModules($system) {}
    foreach m [lindex $GUI_Graphs($system) 4] {
      lappend SystemModules($system) [lindex $m 0]
    }
  
    ## Module Section
    set msection $frame.mods
    set NoteBookBg [$win cget -bg]
    NoteBook $msection -bg $NoteBookBg
    set ModuleSpec [GUI_HS_GetModules]
    if {![llength $ModuleSpec]} {
      gut_warning $GUI_Text(NoModulesLoaded)
      destroy $win
      return {}
    }
    foreach {lang list} $ModuleSpec {
      set path [$msection insert end $lang -text $lang -background $NoteBookBg]
      $msection itemconfigure $lang -createcmd \
         "::EditSystem::GUI_HS_CreatePage \"$system\" $section $path $list
          $msection compute_size"
    }
  #pane $section $msection -orient vertical
  $frame add $section $msection -sticky snew -padx 0 -pady 0
  $frame sash place 0 0 150
  pack $frame -fill both -expand 1 -ipadx 10 -ipady 10
  $msection raise English
  update
  catch {winico setwindow $win $WMIcons(NewSystem)}
  wm deiconify $win
  return $win
}

proc CreateSystem {} \
   {GUI_CreateHandleSystemsWindow Custom_New {Creating New System...}}

proc EditSystem {system win} {
  global GUI_Graphs GUI_GraphInfo
  set Title $system
  catch {set Title [lindex $GUI_Graphs($system) 0]}
  #set w [gut_MakeWindowBusy $win]
  wm withdraw $win
  set canvas .sys_[string map {{ } _} $system].frame.canvas
  set window \
      [GUI_CreateHandleSystemsWindow $system "Editing System \"$Title\"..."]
  if {[string length $window]} {
    set Graph $GUI_Graphs($system)
    set Modules [lindex $Graph 4]
    tkwait window $window
    foreach Module [lindex $GUI_Graphs($system) 4] {
      if {[lsearch -exact $Modules $Module] == -1} {
        foreach module [lindex $GUI_Graphs($system) 4] {
          set m [lindex $module 0]
          if {[info exists GUI_GraphInfo(${canvas}${m})]} {continue}
          set GUI_GraphInfo(${canvas}${m}) {}
          lappend GUI_GraphInfo(${canvas}${m}) 0
          set result [GUI_GetModuleOptions $m silent]
          if {[string equal [lindex $result 0] Error]} {
            lappend LoadingModulesErrors [lindex $result 1]
            set result {{} {}}
          }
          set GUI_GraphInfo(MOptions_${canvas}${m}) $result
        }
        catch {unset GUI_GraphInfo(RunOrder_${canvas})}
        break
      }
    }
    wm deiconify $win
  } else {
    wm deiconify $win
  }
  GUI_DrawSystem $canvas $system
}


## GUI_HS_GetModules
#   Return a list containing all modules, devided according to language
proc GUI_HS_GetModules {} {
  global creole_config

  set modules [array names creole_config]
  if {[string equal $modules {}]} {return {}}

  foreach mod $modules {
    set title {  }
    append title [lindex $creole_config($mod) \
        [expr {[lsearch $creole_config($mod) title] +1}]]
    append title "  ($mod)"
    lappend Titles $title
    set Module($title) $mod
    set language English
    foreach el \
       [PUL_plugin_module_config $mod pre_conditions document_attributes] {
      if {[string match language_* $el]} {
        set lang [string range $el 9 end]
        set language [string toupper [string range $lang 0 0]]
        append language [string range $lang 1 end] 
      }
    }
    set Language($mod) $language
  }
  set Titles [lsort -ascii $Titles]

  ## Language English must exist always!
  set Lang(English) {}
  foreach title $Titles {
    set m $Module($title)
    set lang $Language($m)
    lappend Lang($lang) [list $title $m]
  }
  foreach lang [lsort [array names Lang]] {
    lappend Result $lang [list $Lang($lang)]
  }
  return $Result
}

## GUI_HS_CreatePage
#   
proc GUI_HS_CreatePageListBox {system canvas Path List} {
  set list ${Path}.list
  set deltax [image width  ISystemSmall]
  set deltay [image height ISystemSmall]
  incr deltax 15; incr deltay 15
  set w [::ScrolledWidgets::ScrolledListBox $list \
           -multicolumn true -deltax $deltax -deltay $deltay \
           -padx [expr $deltax + 10]]
  foreach one $List {
    set title [lindex $one 0]
    set m     [lindex $one 1]
    $w insert end $m -text $title -image ISystemSmall -fill navy 
  }
  $w bindImage <<MB_1>> \
    [list ::EditSystem::GUI_HS_AddInNewSystem $w $canvas $system]
  $w bindText  <<MB_1>> \
    [list ::EditSystem::GUI_HS_AddInNewSystem $w $canvas $system]
  pack $list -fill both -expand 1
  $list configure -relief raised -bd 3
}
proc GUI_HS_CreatePage {system canvas Path List} {
  global GUI_Options
  variable SystemModules
  set Modules $SystemModules($system)
  set text $Path.text
  set text [::ScrolledWidgets::ScrolledText $Path.text -wrap none -width 70 \
    -font [list $GUI_Options(TextFont) $GUI_Options(TextFontSize) italic] \
    -highlightthickness 0 -height 8]
  ## Get some needed colours...
  set fg [$text cget -foreground]
  set bg [$text cget -background]
  checkbutton $text.temp
  set abg [$text.temp cget -activebackground]
  destroy $text.temp
  
  set i 1
  foreach item $List {
    foreach {label m} $item {break}
    regexp {(.+)\s+(\([^\)]+\))} $label _ title _
    checkbutton $text.c$i -text $title -image ISystemSmall \
      -compound left -padx 0 -anchor w -bg $bg -fg $fg -relief flat \
      -highlightthickness 0 -variable ::EditSystem::Dummy($text.c$i) \
      -command [list ::EditSystem::GUI_HS_AddInNewSystem \
                     $text.c$i $canvas $system $m]
    bind $text.c$i <Destroy> {unset -nocomplain ::EditSystem::Dummy(%W)}
    bind $text.c$i <MouseWheel> \
      "$text yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $text.c$i <4> "$text yview scroll -5 units"
    bind $text.c$i <5> "$text yview scroll  5 units"
    bind $text.c$i <Enter> "::EditSystem::Page_SelectComponentsCheckEnter %W\
       \"[string trim $title]\" \"$m\""
    bind $text.c$i <Leave> {destroy %W.popup}
    if {[lsearch $Modules $m] != -1} {
      $text.c$i configure -background red
    }
    $text window create $i.0 -window $text.c$i
    $text insert end "\n"
    incr i
  }
  focus $text
  $text configure -state disabled
  pack $Path.text -fill both -expand 1
}

proc ::EditSystem::Page_SelectComponentsCheckEnter {w t c} {
  global GUI_Options
  toplevel $w.popup -bg $GUI_Options(TooltipBackground) -highlightthickness 1\
    -highlightbackground $GUI_Options(TooltipForeground) \
    -highlightcolor $GUI_Options(TooltipForeground)
  wm overrideredirect $w.popup 1
  label $w.popup.l -text "Component Name: \"$c\"\nComponent Title: \"$t\"" \
    -relief flat -wraplength 350 \
    -bg $GUI_Options(TooltipBackground) -padx 2 -pady 0 \
    -fg $GUI_Options(TooltipForeground) -anchor w
    pack $w.popup.l -side left -padx 0 -pady 0 -fill both -expand 1
  wm geometry $w.popup \
    +[expr {[winfo rootx $w] + [winfo width $w] + 10}]+[winfo rooty $w]
};# ::EditSystem::Page_SelectComponentsCheckEnter

variable SystemModules
## GUI_HS_AddInNewSystem
#   Adds a module in the system section's canvas
proc GUI_HS_AddInNewSystem {w canvas system obj} {
  variable SystemModules
  if {![info exists SystemModules($system)]} {set SystemModules($system) {}}
  if {[lsearch $SystemModules($system) $obj] > -1} {
    #$w tag configure $obj -background bisque
    $w configure -background bisque
    DeleteFromSystem $canvas $system $obj
  } else {
    #$w tag configure $obj -background red
    $w configure -background red
    AddInSystem $canvas $system $obj
  }
  focus $w
}

proc AddInSystem {canvas system module} {
  variable SystemModules

  if {![info exists SystemModules($system)]} {
    lappend SystemModules($system) $module
  } elseif {[lsearch -exact $SystemModules($system) $module] == -1} {
    lappend SystemModules($system) $module
  }
  RedrawSystem $canvas $system
}

proc DeleteFromSystem {canvas system module} {
  variable SystemModules
  set i [lsearch -exact $SystemModules($system) $module]
  if {$i != -1} {
    set SystemModules($system) [lreplace $SystemModules($system) $i $i]
    RedrawSystem $canvas $system
  }
}

proc RedrawSystem {canvas system} {
  global GUI_Graphs
  variable SystemModules
  set spec [CreateGraphSpecification $canvas $system $SystemModules($system)]

  set GUI_Graphs($system) $spec
  GUI_CreateSystemWindow $system {} $canvas
}

proc SatisfyConditions {Conditions Available} {
  set NList {}
  foreach el $Available {lappend NList $el}
  foreach cond $Conditions {
    if {[lsearch -exact $NList $cond] == -1} {
      #puts -nonewline "Nismatch: ($cond) List: "
      #foreach el $NList {puts -nonewline "($el) "}
      #puts {}
      return 0
    }
  }
  return 1
}

## CreateGraphSpecification
#   Creates a proper GUI_Graphs($system) element
proc CreateGraphSpecification {canvas system Modules} {
  ## Initialisation
#  catch {unset PreConditions PostConditions Level}
  foreach {InitModules NonInitModules InitPostConditions PostConditionsList} \
          {{} {} {} {}} {}
  set InitialModuleList $Modules
  ## Phase 1: Selecting Initial Modules
  foreach M $Modules {
    foreach [list PreConditions($M) PostConditions($M)] {{} {}} {}
    foreach one [PUL_plugin_module_config_list $M pre_conditions annotations] {
      lappend PreConditions($M)  $one }
    foreach one [PUL_plugin_module_config_list $M post_conditions annotations] {
      lappend PostConditions($M) $one }
    if {![llength $PreConditions($M)]} {
      lappend InitModules $M
      foreach one $PostConditions($M) {lappend InitPostConditions $one}
    } else {
      lappend NonInitModules $M
    }
    foreach one $PostConditions($M) {lappend PostConditionsList $one}
    foreach condition $PostConditions($M) {
      lappend PostConditionsModules($condition) $M
    }
    #foreach condition $PreConditions($M) {
    #  lappend PreConditionsModules($condition) $M
    #}
  }
  set Modules {}; set UnSatisfiedInitModules {}
  foreach M $NonInitModules {
    if {[SatisfyConditions $PreConditions($M) $PostConditionsList]} {
      lappend Modules $M
    } else {
      lappend UnSatisfiedInitModules $M; lappend InitModules $M
      eval lappend InitPostConditions $PostConditions($M)
    }
  }
  #puts "Init Modules: $InitModules\nUnsatisfied: $UnSatisfiedInitModules"
  #puts "Modules: $Modules"
  ## Phase 2: Preconstruction of Graph: Splitting into layers
  foreach M $InitModules {
    lappend Level(0) $M
  }
  set level 1; set SatisfiedConditions $InitPostConditions
  while {1} {
    set TempConditions {}; set UnSatisfiedModules {};
    foreach M $Modules {
      if {[SatisfyConditions $PreConditions($M) $SatisfiedConditions]} {
        lappend Level($level) $M
        eval lappend TempConditions $PostConditions($M)
      } else {
        lappend UnSatisfiedModules $M
      }
    }
    if {[string equal $UnSatisfiedModules {}]} break
    set Modules $UnSatisfiedModules
    if {$level > 100} {puts "To Many Iterations: Giving up..."; break}
    incr level
    foreach one $TempConditions {
      if {[lsearch -exact $SatisfiedConditions $one] == -1} {
        lappend SatisfiedConditions $one
      }
    }
  }
  #puts "\n"; catch {parray Level}
  ## Phase 3: Finding Longest Chain of Connected Modules
  foreach M $InitModules {
    set TempConditions $PostConditions($M); set Chain $M; set AddedModules {}
    foreach level [lrange [lsort -integer [array names Level]] 1 end] {
      foreach m $Level($level) {
        if {[SatisfyConditions $PreConditions($M) $TempConditions]} {
          lappend AddedModules $m
        }
      }
      if {![llength $AddedModules]} break
      foreach m $AddedModules {
        eval lappend TempConditions $PostConditions($m); lappend Chain $m
      }
    }
    set BackBone($M) $Chain; set BackBoneLength($M) [llength $Chain]
  }
  # catch {parray BackBone}
  ## Phase 4: Create Vertical Layers
  ## catch {unset Level};
  #set PlacedModules {}
  #while 1 {
  #  if {![array exists BackBoneLength]} break
  #  if {[array get BackBoneLength] == {}} break
  #  set MaxLength 0
  #  foreach InitModule [array names BackBoneLength] {
  #    if {$BackBoneLength($InitModule) >= $MaxLength} {
  #      set Module $InitModule; set MaxLength $BackBoneLength($InitModule)
  #    }
  #  }
  #  for {set i 0} {$i < [llength $BackBone($Module)]} {incr i} {
  #    set M [lindex $BackBone($Module) $i]
  #    if {[lsearch $PlacedModules $M] == -1} {
  #      lappend Level($i) $M; lappend PlacedModules $M
  #    }
  #  }
  #  unset BackBoneLength($Module)
  #}
  #catch {parray Level}
  set PlacedModules {}
  foreach i [array names Level] {
    foreach M $Level($i) {lappend PlacedModules $M}
  }
  ## Have we lost any modules? Append them to the first column...
  if {[llength $PlacedModules] != [llength $InitialModuleList]} {
    if {[llength $PlacedModules] < [llength $InitialModuleList]} {
      set From $InitialModuleList; set To $PlacedModules
    } else {
      set From $PlacedModules;     set To$InitialModuleList 
    }
    foreach M $From {
      if {[lsearch $To $M] == -1} {lappend Level(0) $M; puts "Added Missed $M"}
    }
  }
  ## Phase 5: Placing Modules
  global GUI_Options
  set ModuleSpecification {}; set ArcSpecification {}; set Modules {}
  set padx [expr {$GUI_Options(ModuleWidth)  + $GUI_Options(ModuleWidth)/2}]
  set pady [expr {$GUI_Options(ModuleHeight) + $GUI_Options(ModuleHeight)/2}]
  set x [expr {$padx/2}]; set y [expr {$pady/2}]
  foreach level [lsort -integer [array names Level]] {
    foreach M $Level($level) {
      lappend ModuleSpecification \
              [list $M $x $y [PUL_plugin_module_config $M title]]
      lappend Modules $M
      incr y $pady
    }
    incr x $padx
    set y [expr {$pady/2}]
  }
  ## Phase 6: Placing Arcs...
  foreach M $Modules {
    ## Placing Direct Arcs
    set Connected {}
    foreach m $Modules {
      if {[string equal $M $m]} continue
      if {![llength $PreConditions($M)]} continue
      if {[SatisfyConditions $PreConditions($M) $PostConditions($m)]} {
        lappend Connected $m
        lappend ArcSpecification [list $m $M {}]
      }
    }
    ## Placing Compound Arcs
    set ToConnect {}
    foreach PreCondition $PreConditions($M) {
      ## lappend_uniq
      if {![info exists PostConditionsModules($PreCondition)]} {
        set PostConditionsModules($PreCondition) {}
      }
      foreach el $PostConditionsModules($PreCondition) {
        if {[lsearch $ToConnect $el] == -1} {lappend ToConnect $el}
      }
      ## lappend_uniq
    }
    set ToConnectWith($M) $ToConnect
  }

  foreach M $Modules {
    ## Remove Excess (Unwanted) links...
#   puts -nonewline "$M ->"
    foreach {Connected ToConnect} {{} {}} {}
    foreach m $ToConnectWith($M) {
      foreach subm $ToConnectWith($m) {
        if {[lsearch -exact $ToConnectWith($M) $subm] == -1} {
          lappend ToConnect $m
#   puts -nonewline "$m "
        }
      }
    }
#   puts {}
    ## Finally, create the arcs...
    foreach m $ToConnect {
      if {[string equal $M $m]} continue
      if {[lsearch $Connected $m] == -1} {
        lappend ArcSpecification [list $m $M {}]
        lappend Connected $m
#       puts "Connecting $m -> $M"
      }
    }
  }
# puts "\n"
  return [list $system 600x350+200+200 0 1x1 \
             $ModuleSpecification $ArcSpecification]
}

## Save System
proc SaveSystem {System} {
  global GUI_Graphs GUI_Systems GUI_Text
  if {[string equal $System Custom_New]} {
    set res [gut_EntryDialog -message $GUI_Text(EnterSystemName) \
      -title {Saving New System...} -regexp {[_=\;|\"\#\$\(\)\[\]\{\}]} \
      -width 20 -message2 $GUI_Text(EnterFolderName) -type2 ComboBox \
      -default2 $GUI_Text(none) -values2 [GUI_GetSystemNames]]
    foreach {name Folder} $res {break}
    if {![string length $name]} {return}
    set nname [string tolower $name]
    set nname [string map {{ } {}} $nname]_system
    if {[info exists GUI_Graphs($nname)]} {
      gut_ErrorDlg [format $GUI_Text(SystemExists!) $name]
      return
    }
    if {[info exists GUI_Systems($name)]} {
      if {[string length [lindex $GUI_Systems($name) end]]} {
        gut_ErrorDlg [format $GUI_Text(CannotCreateSystem:FolderExists) $name]
        return
      }
    }
    
    ## Find a new ID
    set ID 1
    foreach system [array names GUI_Systems] {
      set id [lindex $GUI_Systems($system) 0]
      if {$id > $ID} {set ID $id}
    }
    incr ID

    ## Create the System Specification...
    ## GUI_Graphs is already in place!
    set GUI_Systems($name) [list $ID $name $nname {}]

    if {[string equal $Folder $GUI_Text(none)]} {set Folder {}}
    set folder [string map {_ { }} [file tail $Folder]]
    if {[string length $folder]} {
      if {![info exists GUI_Systems($folder)]} {
        set res [gut_question [format $GUI_Text(CreateFolder?) \
                    [file tail $Folder]]]
        if {[string equal $res no]} {
          foreach {Folder folder} {{} {}} {break}
        } else {
          set GUI_Graphs($name)  [list $nname $name]
          set Spec $GUI_Graphs($System)
          if {[string equal [lindex $Spec 1] 1x1+0+0]} {
            set Spec [lreplace $Spec 1 1 600x300+200+200]
          }
          set GUI_Graphs($nname) [lreplace $Spec 0 0 $name]
          GUI_MW_CreateFolder $Folder
        }
      }
    }
    set GUI_Graphs($name)  [list $nname $name]
    set Spec $GUI_Graphs($System)
    if {[string equal [lindex $Spec 1] 1x1+0+0]} {
      set Spec [lreplace $Spec 1 1 600x300+200+200]
    }
    set GUI_Graphs($nname) [lreplace $Spec 0 0 $name]

    ## Now, the new system is created in the Root directory.
    ## Move it to the desired one...
    if {[string length $folder]} {
      GUI_MW_MoveSystem $name $Folder
    } 
    ## Update the main window view...
    global GUI_MainWidgets
    opt_LoadSystemTree $GUI_MainWidgets(Tree)
    TreeView:setselection $GUI_MainWidgets(TreeView) /
    TreeView:draw $GUI_MainWidgets(TreeView)
  }
}

## AddModulesInSystem
#  Append a list of modules in a system, without altering its layout
proc AddModulesInSystem {canvas system {modules {}}} {
  global GUI_Graphs GUI_GraphInfo GUI_Text
  if {![llength $modules]} {
    ## Get the Components to run...
    set modules [GUI_MW_GetModule2Reload \
      $GUI_Text(PleaseSelectDesiredComponents) {Select Component...}]
  }
  set spec [lindex $GUI_Graphs($system) 4]
  set x 0;
  foreach module $modules {
    set module [file tail $module]
    if {[catch {PUL_plugin_module_config $module title} title]} {
      set title $module
    }
    lappend spec [list $module [incr x 20] 0 $title]
    set GUI_GraphInfo(${canvas}${module}) 0
    set result [GUI_GetModuleOptions $module silent]
    if {[string equal [lindex $result 0] Error]} {
      lappend LoadingModulesErrors [lindex $result 1]
      set result [list {} {}]
    }
    set GUI_GraphInfo(MOptions_${canvas}${module}) $result
  }
  lset GUI_Graphs($system) 4 $spec
  GUI_DrawSystem $canvas $system
  if {[info exists LoadingModulesErrors]} {
    gut_many_errors {Opening System Error...} \
       $GUI_Text(LoadingModulesErrors) [list $LoadingModulesErrors]
  }
};# AddModulesInSystem

## RemoveModuleFromSystem
#  Remove a module from a system, without altering its layout
proc RemoveModuleFromSystem {canvas system {modules {}}} {
  global GUI_Graphs GUI_GraphInfo GUI_Text
  if {![llength $modules]} {return}
  set moduleSpec [lindex $GUI_Graphs($system) 4]
  set arcSpec    [lindex $GUI_Graphs($system) 5]
  foreach module $modules {
    set ans [gut_question [format $GUI_Text(RemoveModule%s?) $module]]
    if {$ans == "no"} {continue}
    set index [lsearch -glob $moduleSpec ${module}*]
    if {$index != -1} {
      ## The module is in the system...
      set moduleSpec [lreplace $moduleSpec $index $index]
    }
    foreach index [lsearch -glob -all $arcSpec *${module}*] {
      lset arcSpec $index {}
    }
  }
  lset GUI_Graphs($system) 4 $moduleSpec
  set newArcSpec {}
  foreach arc $arcSpec {
    if {[llength $arc]} {lappend newArcSpec $arc}
  }
  lset GUI_Graphs($system) 5 $newArcSpec
  GUI_DrawSystem $canvas $system
};# RemoveModuleFromSystem

}; #namespace

proc GUI_GetSystemNames {} {
  global GUI_Systems
  set Names {}
  foreach system [array names GUI_Systems] {
    if {[string match *_task [lindex $GUI_Systems($system) 2]]} {
      lappend Names [lindex $GUI_Systems($system) 1]
    }
  }
  return $Names
};# GUI_GetSystems
