##############################################################################
##  MainWindow.tcl:                                                         ##
## ---------------------                                                    ##
##  This file implements the main window of the platform. Through this      ##
##  window the user can manage Components, Systems and Collections.         ##
##                                                                          ##
##  Copyright 1998-2006 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## GUI_CreateHelpMenuEntries
#  This function creates the standart Help menu entries...
proc GUI_CreateHelpMenuEntries {help {win {}}} {
  global GUI_Text GUI_Options GUI_Underline
  $help add command -label $GUI_Text(HelpContents) -underline \
  $GUI_Underline(HelpContents) -command "GUI_Help_Contents $win" \
  -image IBookOpen -compound $GUI_Options(MenuImageLocation)
  $help add command -label $GUI_Text(ReleaseNotes) -underline \
  $GUI_Underline(ReleaseNotes) -command "GUI_Help_Release_Notes $win" \
  -image ITLogo -compound $GUI_Options(MenuImageLocation)
  $help add command -label $GUI_Text(ProductInfo&Support) -underline \
  $GUI_Underline(ProductInfo&Support) -command \
  "GUI_Help_Product_Info_Support $win" \
  -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $help add separator
  $help add command -label $GUI_Text(SoftwareUpdate) -underline \
  $GUI_Underline(SoftwareUpdate) -command "GUI_Help_Software_Update $win" \
  -image ITUpdate -compound $GUI_Options(MenuImageLocation)
  $help add separator
  $help add command -label $GUI_Text(RegistrationInfo) -underline \
  $GUI_Underline(RegistrationInfo) -command "::CDM::CDM_LicenseInfoDlg" \
  -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $help add separator
  $help add command -label $GUI_Text(Copyright) -underline \
  $GUI_Underline(Copyright) -command "GUI_Help_Copyright $win" \
  -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $help add command -label $GUI_Text(About) -underline \
  $GUI_Underline(About) -command "GUI_Help_About $win" \
  -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $help}
};# GUI_CreateHelpMenuEntries

## GUI_CreateToolMenuEntries
#  This function will add the proper entries in the specified menu that
#  execute tools...
proc GUI_CreateToolMenuEntries {utilities {canvas {}}} {
  global GUI_ToolConfig GUI_Options GUI_Text GUI_Underline
  foreach tool [lsort -dictionary [array names GUI_ToolConfig]] {
    set toolIcon ITTool
    if {[regexp -- {([^:]+)::([^:]+)} $tool match Class Tool]} {
      ## Maybe the tool has a special icon?
      if {[info exists ::creole_${Tool}_home]} {
        foreach file [glob -nocomplain -directory [set ::creole_${Tool}_home] \
                           ${Tool}ToolbarIcon.*] {
          if {![catch {image create photo IT_${Tool} -file $file}]} {
            set toolIcon IT_${Tool}
            break
          }
        }
      }
      if {[string length $Class]} {
        catch {
          set class [string map {{ } _ . _} $Class]
          menu $utilities._class$class
          $utilities add cascade -label $Class -menu $utilities._class$class \
          -image IFolderSmallOpen -compound $GUI_Options(MenuImageLocation)
        }
        $utilities._class$class add command \
          -command [list GUI_RunTool $canvas $Tool] \
          -label [PUL_ToolConfig $Tool title $Class] \
          -image $toolIcon -compound $GUI_Options(MenuImageLocation)
      } else {
        $utilities add command -label [PUL_ToolConfig $tool title] \
          -command [list GUI_RunTool $canvas $tool] \
          -image $toolIcon -compound $GUI_Options(MenuImageLocation)
      }
    } else {
      if {[regexp -- {::([^:]+)} $tool match Tool]} {
        ## Maybe the tool has a special icon?
        if {[info exists ::creole_${Tool}_home]} {
          foreach file [glob -nocomplain -directory [set ::creole_${Tool}_home]\
                             ${Tool}ToolbarIcon.*] {
            if {![catch {image create photo IT_${Tool} -file $file}]} {
              set toolIcon IT_${Tool}
              break
            }
          }
        }
        $utilities add command -label [PUL_ToolConfig $Tool title] \
          -command [list GUI_RunTool $canvas $Tool] \
          -image $toolIcon -compound $GUI_Options(MenuImageLocation)
      }
    }
  }
  ## Finally add an entry to start the Toolbox:-)
  $utilities add separator
  $utilities add command -label $GUI_Text(ExploreToolbox) \
    -underline $GUI_Underline(ExploreToolbox) \
    -command [list ::GUI_Toolbox::Explorer $canvas] \
    -image ITTools -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $utilities}
};# GUI_CreateToolMenuEntries

## GUI_CreateMainMenu
#
proc GUI_CreateMainMenu {} {
  global GUI_Text GUI_Underline GUI_Options GUI_ToolConfig CDM_Embeded
  set win .mw

  #if catch menu .mw.mb -type menubar -relief $GUI_Options(MenuRelief) 
  if {[catch {menu .mw.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar .mw.mb
    $menubar delete 0 end
  }

  ## File Menu
  set file $menubar.top_file; catch {destroy $file}
  $menubar add cascade -label $GUI_Text(File) -menu $file \
              -underline $GUI_Underline(File)
  menu $file
#              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
#              -activebackground  $GUI_Options(MenuSelectBg) \
#              -activeforeground  $GUI_Options(MenuSelectFg) file
  $file add command -label $GUI_Text(OpenConsole) -image ITConsole \
              -underline $GUI_Underline(OpenConsole) -command {gut_Console} \
              -compound $GUI_Options(MenuImageLocation)
  if {$::tcl_platform(platform) == "windows"} {
    $file add command -label \
      "$GUI_Text(OpenConsole) ([file tail [info nameofexecutable]])" \
      -image ITConsole -command {console show} \
      -compound $GUI_Options(MenuImageLocation)
  }
  if {!$CDM_Embeded} {
    $file add separator
    $file add command -label $GUI_Text(StartHttpd) -underline \
              $GUI_Underline(StartHttpd) -command "GUI_StartHttpd $file" \
              -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $file add command -label $GUI_Text(StopHttpd) -state disabled -underline \
              $GUI_Underline(StopHttpd) -command {GUI_StopHttpd} \
              -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $file add command -label $GUI_Text(StartSOAP) -underline \
              $GUI_Underline(StartHttpd) -command "GUI_SOAPStartServer $file" \
              -image ITBlank -compound $GUI_Options(MenuImageLocation)
    $file add command -label $GUI_Text(StopSOAP) -state disabled -underline \
              $GUI_Underline(StopHttpd) -command {GUI_SOAPStopServer} \
              -image ITBlank -compound $GUI_Options(MenuImageLocation)
  }
  $file add separator
  $file add command -label $GUI_Text(Options)... \
              -underline $GUI_Underline(Options) -command GUI_EditPreferences\
              -image ITBlank -compound $GUI_Options(MenuImageLocation)
  
  if {!$CDM_Embeded} {
    $file add separator
    $file add command -label $GUI_Text(Exit) -underline $GUI_Underline(Exit) \
              -command {GUI_CloseWindow .mw} -image ITExit \
              -compound $GUI_Options(MenuImageLocation)
  }
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $file}

  ## View Menu
  set view $menubar.view; catch {destroy $view}
  $menubar add cascade -label $GUI_Text(View) -menu $view \
       -underline $GUI_Underline(View)
  menu $view
  $view add command -label $GUI_Text(SaveLayout) -image ITSaveLayout -underline\
     $GUI_Underline(SaveLayout) -command {opt_SaveConfig} \
     -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowTips) -underline \
     $GUI_Underline(ShowTips) -command {LDM_ShowTip} \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowCachedMemory) -underline \
     $GUI_Underline(ShowCachedMemory) -command \
     "ggi_view_CDM_CachedMemoryObjects" \
     -image ITMemory -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(ClearCachedMemory) -underline \
     $GUI_Underline(ClearCachedMemory) -command \
     "ggi_view_CDM_CachedMemoryObjectsClearCache" \
     -image ITMemoryClear -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowToolbars) -underline \
     $GUI_Underline(ShowToolbars) -command \
     "Toolbar::deiconify $win.tf.tb $win.tf.tb2 $win.tf.tb3 $win.tf.tb4" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $view}
  
  ## System Menu
  set system $menubar.top_system; catch {destroy $system}
  $menubar add cascade -label $GUI_Text(System) -menu $system \
     -underline $GUI_Underline(System)
  menu $system
  $system add command -label $GUI_Text(OpenSelectedSystem) -underline \
    $GUI_Underline(OpenSelectedSystem) -command \
      {GUI_MW_Open/EditSelectedSystem Open} \
    -image ITOpenSystem -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(OpenSelectedSystemRibbon) -underline \
    $GUI_Underline(OpenSelectedSystem) -command \
      {GUI_MW_Open/EditSelectedSystem OpenRibbon} \
    -image ITOpenSystem -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(CreateNewSystem) -underline \
    $GUI_Underline(CreateNewSystem) -image ITNewSystem -command \
      {::EditSystem::CreateSystem} -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(RenameSelectedSystem) -underline \
    $GUI_Underline(RenameSelectedSystem) -command \
    {if {[info exists GUI_MainWidgets([winfo toplevel \
         $GUI_MainWidgets(Tree)]:CurrentSelection)]} {
       GUI_MW_RenameSystem [file tail [lindex $GUI_MainWidgets([winfo toplevel \
         $GUI_MainWidgets(Tree)]:CurrentSelection) 0]]
     }} \
     -image ITRenameSystem -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(EditSelectedSystem) -underline \
    $GUI_Underline(EditSelectedSystem) -image ITCreateSystem -command \
      {GUI_MW_Open/EditSelectedSystem Edit} \
      -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(OpenSystemBoard) -underline \
    $GUI_Underline(OpenSystemBoard) -command \
      {::SystemBoard::Open} \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $system add separator
  $system add command -label $GUI_Text(NewFolder) \
    -command {GUI_MW_CreateFolder} \
    -image ITNewFolder -compound $GUI_Options(MenuImageLocation)
  $system add separator
  $system add command -label $GUI_Text(DeleteSystemFolder) -underline \
    $GUI_Underline(DeleteSystemFolder) -command \
    {if {[info exists GUI_MainWidgets([winfo toplevel \
         $GUI_MainWidgets(Tree)]:CurrentSelection)]} {
       GUI_MW_DeleteSystem [file tail [lindex $GUI_MainWidgets([winfo toplevel \
         $GUI_MainWidgets(Tree)]:CurrentSelection) 0]]
     }} \
    -image ITDeleteSystem -compound $GUI_Options(MenuImageLocation)
  $system add separator
  $system add command -label $GUI_Text(ImportSystem) -command \
      {GUI_MW_ImportExportSelectedSystem Import} -image ITImportSystem \
      -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(ExportSelectedSystem) -command \
        {GUI_MW_ImportExportSelectedSystem Export} -image ITExportSystem \
        -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(ExportSelectedSystemAsComponent) \
        -command \
        {GUI_MW_ImportExportSelectedSystem Component} -image ITExportSystem \
        -compound $GUI_Options(MenuImageLocation)
  $system add command -label $GUI_Text(SaveSystems) -underline \
    $GUI_Underline(SaveSystems) -image ITSaveLayout -command {opt_SaveConfig} \
    -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $system}
  
  ## Collection Menu
  set collection $menubar.collection; catch {destroy $collection}
  $menubar add cascade -label $GUI_Text(Collection) -menu $collection \
     -underline $GUI_Underline(Collection)
  menu $collection
  $collection add command -label $GUI_Text(CreateCollection) -underline \
     $GUI_Underline(CreateCollection) -image ITNewMany \
     -command "GUI_NewCollection -win $win.ccw" \
     -compound $GUI_Options(MenuImageLocation)
  $collection add command -label $GUI_Text(EditCollections) -underline \
     $GUI_Underline(EditCollections) -command "GUI_CollectionDlg Edit $win" \
     -image ITEdit -compound $GUI_Options(MenuImageLocation)
  $collection add separator
  $collection add command -label $GUI_Text(ModifyColSearchPath) -underline \
     $GUI_Underline(ModifyColSearchPath) -command \
     "GUI_EditPref_ModifySearchPath $win CollectionPath directory" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $collection}
    
  ## Modules Menu
  set modules $menubar.modules; catch {destroy $modules}
  $menubar add cascade -label $GUI_Text(Modules) -menu $modules \
     -underline $GUI_Underline(Modules)
  menu $modules
  $modules add command -label $GUI_Text(AddModulesDir) -underline \
    $GUI_Underline(AddModulesDir) -command \
    {PUL_LoadPluginsInDir [gut_SelectDirectory];GUI_RegisterTools} \
    -image ITAddModuleDir -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(CreateNewModule) -underline \
    $GUI_Underline(CreateNewModule) -command {::Component::Create} \
    -image ITCreateModule -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(RenameModule) -underline \
    $GUI_Underline(RenameModule) -command {::Component::RenameComponentWizard} \
    -image ITRenameModule -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add command -label $GUI_Text(ReloadModule) -underline \
    $GUI_Underline(ReloadModule) -command \
    {PUL_LoadPlugin [GUI_MW_GetModule2Reload];GUI_RegisterTools} \
    -image ITReloadModule -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(ReloadLastLoaded) -underline \
    $GUI_Underline(ReloadLastLoaded) -command \
    {PUL_LoadPlugin {};GUI_RegisterTools} \
    -image ITReloadLastLM -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(ReloadAllModules) -underline \
    $GUI_Underline(ReloadAllModules) -command \
    {PUL_LoadPlugins;GUI_RegisterTools} \
    -image ITReLoadModules -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add command -label $GUI_Text(CompileModule) -underline \
    $GUI_Underline(CompileModule) -command "::CompileComponentWizard::Init"\
    -image ITCompile -compound $GUI_Options(MenuImageLocation)
  $modules add cascade -label $GUI_Text(Debug) -underline \
    $GUI_Underline(Debug) -menu $modules.debug \
    -image ITStar -compound $GUI_Options(MenuImageLocation)
    menu $modules.debug
      $modules.debug add command -label $GUI_Text(TclProfiler) -underline \
      $GUI_Underline(TclProfiler) -command ::CDM_Debug::TclProfiler::Init \
      -image ITViewTimeLg -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add cascade -label $GUI_Text(ComponentLanguages) -underline \
    $GUI_Underline(ComponentLanguages) -menu $modules.languages \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
    menu $modules.languages
    foreach language {C C++ Tcl} {
      $modules.languages add command -label $language \
      -image ITStar -compound $GUI_Options(MenuImageLocation)
    }
    foreach language {Java Perl Python} {
      if {[info exists CDM_${language}::${language}Loaded] &&
          [set CDM_${language}::${language}Loaded]} {
        $modules.languages add command -label $language \
          -image ITStar -compound $GUI_Options(MenuImageLocation)
      } else {
        $modules.languages add command -label $language \
          -image ITBlank -compound $GUI_Options(MenuImageLocation)
      }
    }
  $modules add command -label $GUI_Text(ModifyComponentSearchPath) -underline \
    $GUI_Underline(ModifyComponentSearchPath) -command \
    "GUI_EditPref_ModifySearchPath $win PluginModulePath directory" \
    -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $modules}

  ## Utilities Menu
  if {[array exists GUI_ToolConfig]} {
    set utilities $menubar.utilities; catch {destroy $utilities}
    $menubar add cascade -label $GUI_Text(Utilities) -menu $utilities \
       -underline $GUI_Underline(Utilities)
    menu $utilities
    GUI_CreateToolMenuEntries $utilities
  }
  
  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help

  return $menubar
}

## GUI_CreateStatusBar
#  This function creates a default status bar, containing a label and
#  facilities for changing UI/Keyboard language...
proc GUI_CreateStatusBar {statusbar} {
  global GUI_Options Locale
  frame $statusbar -borderwidth 0
  set win [winfo toplevel $statusbar]
  label $statusbar.locale  -relief sunken -anchor w -borderwidth \
     $GUI_Options(BorderWidth) -textvariable Locale -width 12
  catch {$statusbar.locale configure -image LocaleImg-$Locale \
         -width [image width LocaleImg-$Locale]}
  label $statusbar.keybd   -relief sunken -anchor w -borderwidth \
     $GUI_Options(BorderWidth) -textvariable KeyboardLocale -width 12
  label $statusbar.message -relief sunken -anchor w -borderwidth \
     $GUI_Options(BorderWidth) -textvariable ::GUI_StatusBar::StatusBar($win)
  pack $statusbar.keybd   -padx 2 -pady 1 -side right
  pack $statusbar.locale  -padx 2 -pady 1 -side right -fill y
  pack $statusbar.message -padx 2 -pady 1 -fill x -expand 1
  pack $statusbar -fill x -side bottom
  bind $statusbar.locale <Enter> "set ::GUI_StatusBar::StatusBar($win) \
       \$GUI_Text(ChangeGUILocale)"
  bind $statusbar.locale <Leave> "set ::GUI_StatusBar::StatusBar($win) \
       \$::GUI_StatusBar::StatusBar($win:Def)"
  bind $statusbar.keybd  <Enter> "set ::GUI_StatusBar::StatusBar($win) \
       \$GUI_Text(ChangeKBDLocale)"
  bind $statusbar.keybd  <Leave> "set ::GUI_StatusBar::StatusBar($win) \
       \$::GUI_StatusBar::StatusBar($win:Def)"
  bind $statusbar.locale <ButtonRelease-1> "GUI_SetLocale $win %x %y"
  bind $statusbar.keybd  <ButtonRelease-1> "GUI_SetKeyboardLocale $win %x %y"
  bind $statusbar.message <Destroy> "unset -nocomplain \
    ::GUI_StatusBar::StatusBar($win) ::GUI_StatusBar::StatusBar($win:Def)"
  if {![info exists ::GUI_StatusBar::StatusBar($win:Def)]} {
    if {[info exists ::GUI_StatusBar::StatusBar($win)]} {
      set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
    } else {
      set ::GUI_StatusBar::StatusBar($win:Def) {}
      set ::GUI_StatusBar::StatusBar($win) {}
    }
  }
};# GUI_CreateStatusBar

## GUI_CreateMainWindow
#
proc GUI_CreateMainWindow {} {
  global UserHomeDir HomeDir IUp IFolder IFolderSmall ISystem ISystemSmall \
         GUI_Text GUI_Underline Locale KeyboardLocale \
         GUI_Options PANE WMIcons tcl_platform \
         ApplicationName ApplicationVersion argv0 argv GUI_MainWidgets \
         GUI_Systems GUI_SystemsModified CDM_Embeded
  set win .mw
  catch {toplevel $win -class ${::ApplicationName}MainWindow}
  wm withdraw $win
  if {$CDM_Embeded} {
    wm protocol $win WM_DELETE_WINDOW "GUI_DisableGUI"
  } else {
    . configure -background {}
    wm protocol $win WM_DELETE_WINDOW "GUI_CloseWindow $win"
    wm protocol $win WM_SAVE_YOURSELF {GUI_SaveOptions; exit 0}
    catch {CDM_Trap "GUI_CloseWindow $win" SIGINT}
    if {[info exists argv0]} {
      wm command .mw   "[info nameofexecutable] $argv0 $argv"
      wm client  .mw   [info hostname]
    }
  }
  wm title $win "$ApplicationName v $ApplicationVersion!"
  set PANE_Percent $GUI_Options(MainWindowPanePercent)
  set lines [opt_GetLines [file join $UserHomeDir .gate.props] \
                    __START_TASKS __start_graphs]
  wm sizefrom $win user
  foreach line $lines {
    if {[string match "__top_geometry\=\*" $line]} {
      scan $line "__top_geometry=%s" geometry
    }
    if {[string match "__top_pane_percent\=\*" $line]} {
      scan $line "__top_pane_percent=%s" PANE_Percent
    }
  }
  if {![info exists geometry]} {
    if {$::tcl_platform(os) == "Darwin"} {
      set geometry "600x400+100+100"
    } else {
      set geometry "600x400+0+0"
    }
  }
  wm geometry $win $geometry
  
  ## Create the menubar...
  $win configure -menu [GUI_CreateMainMenu]
  
  ## Create the status bar...
  set ::GUI_StatusBar::StatusBar($win) \
    [format $GUI_Text(Welcome) $ApplicationName $ApplicationVersion]
  set ::GUI_StatusBar::StatusBar($win:Def) \
    "$GUI_Text(System): ( $GUI_Text(none) )"
  GUI_CreateStatusBar $win.statusbar

  ## Toolbar...
  set toolbar_frame $win.tf
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
    
    ## File Toolbar
    set toolbar $toolbar_frame.tb
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TCreateSystem) \
       -image ITCreateSystem -command {::EditSystem::CreateSystem} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TConsole) \
       -image ITConsole -command {gut_Console} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TSaveLayout) \
       -image ITSaveLayout -command {opt_SaveConfig} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left -fill y

    ## Module Toolbar
    set toolbar $toolbar_frame.tb2
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(AddModulesDir) -image \
       ITAddModuleDir -statusvar ::GUI_StatusBar::StatusBar($win) -command \
       {PUL_LoadPluginsInDir [gut_SelectDirectory];GUI_RegisterTools}
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(CreateNewModule) \
       -image ITCreateModule -command {::Component::Create} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadModule) \
       -image ITReloadModule -statusvar ::GUI_StatusBar::StatusBar($win) \
       -command \
       {PUL_LoadPlugin [GUI_MW_GetModule2Reload];GUI_RegisterTools}
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadLastLoaded) \
       -image ITReloadLastLM -command {PUL_LoadPlugin {};GUI_RegisterTools} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadAllModules) \
       -image ITReLoadModules -command {PUL_LoadPlugins;GUI_RegisterTools} \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left -fill y
    
    ## Window Toolbar
    set toolbar $toolbar_frame.tb4
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                    -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
    if {!$CDM_Embeded} {
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "GUI_CloseWindow $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
    }
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Help) -image ITHelp \
       -command {GUI_Help_Contents} -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x
  
  ## Put everything in a frame...
  set Width 650; set Height 350
  frame $win.fr -width $Width -height $Height
  ## Create a tree widget
  set tr_fr $win.fr.tr_fr; set tree $tr_fr.tree.canvas
  frame $tr_fr -bg $GUI_Options(ExplorerBg) -relief \
    $GUI_Options(ExplorerRelief) -borderwidth $GUI_Options(ExplorerBorderWidth)
  ScrolledWidgets::ScrolledCanvas $tr_fr.tree -relief flat \
   -borderwidth $GUI_Options(ExplorerBorderWidth) \
   -bg $GUI_Options(ExplorerBg) -height $GUI_Options(ExplorerTreeHeigth) \
   -width $GUI_Options(ExplorerTreeWidth) -highlightthickness 0 \
   -regionpadx 5 -regionpady 20
  Tree:create $tree
  label $tr_fr.tree.label -textvariable GUI_Text(AvailableSystems) \
   -bg $GUI_Options(ExplorerStandoutBg) -fg $GUI_Options(ExplorerStandout) \
   -bd $GUI_Options(BorderWidth) -relief raised -highlightthickness 0 -anchor w
  #pack ${tr_fr}.label -side top -fill none -anchor w
  grid configure $tr_fr.tree.label -column 0 -row 0 -sticky new
  pack $tr_fr.tree  -side left -fill both -expand 1 
  pack $tr_fr -side left -fill y -expand 1

  ## Create a tree view widget
  set tv_fr $win.fr.tv_fr; set tr_v $tv_fr.tree_view.canvas
  frame $tv_fr -bg $GUI_Options(ExplorerBg) -bd 0 -relief flat
  ScrolledWidgets::ScrolledCanvas $tv_fr.tree_view -relief \
   $GUI_Options(ExplorerRelief) -borderwidth $GUI_Options(ExplorerBorderWidth)\
   -bg $GUI_Options(ExplorerBg) -highlightthickness 0 -regionpadx 10 \
   -confcommand "TreeView:draw %W no"
  TreeView:create $tr_v
  TreeView:link $tr_v $tree
  TreeView:Call $tr_v GUI_MW_CreateSystemWindow
  pack $tv_fr.tree_view -fill both -expand 1 
  pack $tv_fr -side left -fill both -expand 1
  pane $tr_fr $tv_fr -orient horizontal 

  ## The pane buttons...
  ArrowButton $win.fr.__h1.b1 -type button -width 6 -height 25 \
    -dir left -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.fr 1 0"
  ::$win.fr.__h1.b1:cmd configure -cursor left_side
  button $win.fr.__h1.b2 -image IRightLeft -padx 0 -pady 0 -width 4 -height 25 \
    -bd 0 -cursor dot -relief flat -highlightthickness 0 \
    -activebackground [$win.fr.__h1.b1 cget -activebackground] -anchor w \
    -command "Widget_PaneSet $win.fr 1 $PANE_Percent" \
    -bg $PANE(Color)
  ArrowButton $win.fr.__h1.b3 -type button -width 6 -height 25 \
    -dir right -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.fr 1 100"
  ::$win.fr.__h1.b3:cmd configure -cursor right_side
  place $win.fr.__h1.b1 -rely 0.0
  place $win.fr.__h1.b2 -rely 0.45
  place $win.fr.__h1.b3 -rely 1.0 -anchor sw
  
  pack $win.fr -fill both -expand 1

  $tree bind x <<MB_1>> \
  "set lbl \[Tree:labelat %W %x %y\];Tree:setselection \%W \$lbl;\
   TreeView:setselection $tr_v \$lbl;TreeView:draw $tr_v;\
   TreeView:drawselection $tr_v"
  $tree bind x <<MB_2>> \
  "set lbl \[Tree:labelat %W %x %y\];Tree:setselection \%W \$lbl;\
   TreeView:setselection $tr_v \$lbl;TreeView:draw $tr_v;\
   TreeView:drawselection $tr_v"
  ## Left Menu Handler...
  bind $tr_v <<Motion_1>> {catch {GUI_MW_AppendSelection %W %x %y}}
  bind $tr_v <<Release_1>> {catch {%W delete -?Special?-SelectionRectangular-}}
  bind $tree <<MB_1>> {GUI_MW_Deselect %W %x %y}
  bind $tr_v <<MB_1>> {GUI_MW_Deselect %W %x %y}
  ## Keyboard Shortcuts
  bind $tr_v <Return> {GUI_MW_Open/EditSelectedSystem Open}
  bind $tr_v <Alt-o>  {GUI_MW_Open/EditSelectedSystem Open}
  bind $tr_v <Alt-n>  {::EditSystem::CreateSystem}
  bind $tr_v <Alt-e>  {GUI_MW_Open/EditSelectedSystem Edit}
  bind $tr_v <Alt-c>  {catch {GUI_MW_CreateFolder}}
  bind $tr_v <Delete> {if {[info exists GUI_MainWidgets([winfo toplevel \
       $GUI_MainWidgets(Tree)]:CurrentSelection)]} {
       GUI_MW_DeleteSystem [file tail [lindex $GUI_MainWidgets([winfo toplevel \
       $GUI_MainWidgets(Tree)]:CurrentSelection) 0]]
     }}
  ## Right Menu Handler...
  bind $tree <<MB_2>> {GUI_MW_Deselect %W %x %y;GUI_MW_ActionPopUp %W}
  bind $tr_v <<MB_2>> {GUI_MW_Deselect %W %x %y;GUI_MW_ActionPopUp %W}

  ## Drag-n-Drop /* Petasis, 6/12/1999 */
  if {[catch {package present tkdnda}]} {
    ## Use BWidget drag & drop protocol
    DragSite::register $tr_v -draginitcmd GUI_DragInitCmd \
         -dragendcmd GUI_DragEndCmd -dragevent 1
    DropSite::register $tr_v -dropcmd    GUI_DropCmd \
         -droptypes {SYSTEM {move {}}} -dropovercmd GUI_DropOverCmd
    DropSite::register $tree -dropcmd    GUI_DropCmd \
         -droptypes {SYSTEM {move {}}} -dropovercmd GUI_DropOverCmd
    DropSite::register $tr_fr.tree.label -dropcmd    GUI_DropCmd \
         -droptypes {SYSTEM {move {}}} -dropovercmd GUI_DropOverRootCmd
    ## We may get an error, due to BWidgets bug. Just define the variable:
    set DragSite::_state {}; ## Petasis, 11/02/2000
  } else {
    ## Use tkdnd extension...
    dnd bindsource $tr_v TK_MW_SYSTEM {GUI_TkDND_GetData %W %x %y}
    bind $tr_v <1> {GUI_TkDND_DragInit %W %x %y}
    dnd bindtarget $tr_v TK_MW_SYSTEM <DragEnter> \
        {GUI_TkDND_DragEnter %W %x %y}
    dnd bindtarget $tr_v TK_MW_SYSTEM <Drag> \
        {GUI_TkDND_Drag %W %x %y}
    dnd bindtarget $tr_v TK_MW_SYSTEM <DragLeave> \
        {GUI_TkDND_DragLeave %W %x %y}
    dnd bindtarget $tr_v TK_MW_SYSTEM <Drop> \
        {GUI_TkDND_Drop %W %x %y %D}
  }
  opt_ReadConfig [file join $UserHomeDir .gate.props]
  opt_LoadSystems [file join $UserHomeDir .gate.props]
  set GUI_MainWidgets(Tree) $tree
  set GUI_MainWidgets(TreeView) $tr_v
  opt_LoadSystemTree $tree
  TreeView:UpFolderImage $tr_v IUp
  TreeView:UpFolderLabel $tr_v {Up Folder}
  TreeView:setselection $tr_v /
  TreeView:draw $tr_v 
  Widget_PaneSet $win.fr 1 $PANE_Percent
  trace add variable GUI_MainWidgets($win:CurrentSelection) write \
    "GUI_UpdateStatusBar $win"
  trace add variable KeyboardLocale write GUI_SetSystemLocale
  trace add variable Locale         write GUI_SetInterfaceLocale
  update
  wm deiconify $win
  wm geometry $win $geometry
  focus $tr_v
  update
  ## Try to load the Merlin Agent if under windows...
  if {[string equal $tcl_platform(platform) windows]} {
    catch {winico setwindow $win $WMIcons(Main)}
    foreach {w h x y} [split $geometry x+] {}
    Agent::LoadAgent
    if {[info exists GUI_Options(Agent_X)]} {
      Agent::Move $GUI_Options(Agent_X) $GUI_Options(Agent_Y)
    } else {
      Agent::Move [expr {$x+$w+10}] [expr {$y+20}]
    }
    Agent::Show
  }

  ## Set a trace on GUI_Systems array...
  trace variable GUI_Systems w GUI_MW_GUI_SystemsChanged
  set GUI_SystemsModified 0
  catch {LDM_DeleteSplashWindow .splashscr}
  if {$GUI_Options(ExplorerShowTipsDlg)} {catch {LDM_ShowTip $win}}
};# GUI_CreateMainWindow

proc GUI_MW_GUI_SystemsChanged {name1 name2 op} {
  global GUI_SystemsModified 
  set GUI_SystemsModified 1
};# GUI_MW_GUI_SystemsChanged
      
## GUI_MW_CreateSystemWindow
#   Opens Selected System
proc GUI_MW_CreateSystemWindow {system} {
  global GUI_Graphs
  set name $system
  if {[info exists GUI_Graphs($name)]} {
    set name [lindex $GUI_Graphs($name) 0]
    GUI_CreateSystemWindow $name $system
    #if {[catch {GUI_CreateSystemWindow $name $system} error]} {
    #  gut_error $error
    #}
  }
};# GUI_MW_CreateSystemWindow

## GUI_UpdateStatusBar
#   Update Status Bar of win according to given variable.
proc GUI_UpdateStatusBar {win name1 name2 op} {
  global GUI_Text GUI_Systems GUI_MainWidgets $name1
  if {[string length $name2]} {
    set value [lindex [set "${name1}(${name2})"] 0]
  } else {
    set value [lindex [set $name1] 0]
  }
  set selection [file tail $value]
  if {  [info exists GUI_Systems($selection)] && \
        [string match *_task [lindex $GUI_Systems($selection) 2]] } {
    set Directory [file tail $value]
    set System "($GUI_Text(none))"
  } else {
    set System [file tail $value]
    set Directory [file tail [file dirname $value]]
  }
  if {![string length $Directory]} {set Directory "($GUI_Text(none))"}
  set value "$GUI_Text(Folder): $Directory,  $GUI_Text(System): $System"
  set ::GUI_StatusBar::StatusBar($win) $value
  set ::GUI_StatusBar::StatusBar($win:Def) $value
  return {}
}

##############################################################################
## Mouse Button Handlers...                                                 ##
##############################################################################
## GUI_MW_Deselect
proc GUI_MW_Deselect {canvas x y} {
  catch {$canvas delete -?Special?-SelectionRectangular-}
  set current [$canvas find withtag current]
  if {[string equal $current {}]} {
    global GUI_MainWidgets Tree
    set tr_v $GUI_MainWidgets(TreeView)
    set tree $GUI_MainWidgets(Tree)
    set twin [winfo toplevel $tree]
    set selection [lindex $GUI_MainWidgets($twin:CurrentSelection) 0]
    if {[string compare $canvas $tr_v] == 0} {
      if {[info exists Tree($tree:$selection:children)] &&\
          [string compare $Tree($tree:$selection:children) {}] != 0} {
          if {$Tree($tree:$selection:open) == 0} {
            set selection [file dirname $selection]
          }
      } else {
        set selection [file dirname $selection]
      }
    } else {
      set selection [file dirname $selection]
    }
    TreeView:setselection $tr_v $selection
    TreeView:draw $tr_v 
    set GUI_MainWidgets(TreeView:x) $x
    set GUI_MainWidgets(TreeView:y) $y
    ##focus $tr_v
  }
};# GUI_MW_Deselect

## GUI_MW_GetSelection
#   Returns a list containing: the selection, its type, and its path
proc GUI_MW_GetSelection {} {
  global GUI_MainWidgets GUI_Systems
  set twin [winfo toplevel $GUI_MainWidgets(Tree)]
  set selection [file tail [lindex $GUI_MainWidgets($twin:CurrentSelection) 0]]
  set Type Directory
  if {[string length $selection]} {
    if {[info exists GUI_Systems($selection)] && \
        [string match *_task [lindex $GUI_Systems($selection) 2]] } {
      set Type Directory
    } else {
      set Type System
    }
  }
  return [list $selection $Type $GUI_MainWidgets($twin:CurrentSelection)]
}

## GUI_MW_Open/EditSelectedSystem
proc GUI_MW_Open/EditSelectedSystem {Mode {selection {}}} {
  global GUI_Graphs
  if {![llength $selection]} {
    set selection [GUI_MW_GetSelection]
  }
  foreach {system type path} $selection {break}
  ## If "system" is an empty string, then the user may have selected many
  ## items. If that is the case, arrange an iteration over these items...
  if {![string length $system] || [string equal $type Directory]} {
    global GUI_MainWidgets GUI_Systems
    if {![info exists GUI_MainWidgets(TreeView:Selected)]} {return}
    set selection $GUI_MainWidgets(TreeView:Selected)
    if {![llength $selection]} {return}
    foreach one $selection {
      set tail [file tail $one]
      if {[info exists GUI_Systems($tail)] && \
          [string match *_task [lindex $GUI_Systems($tail) 2]] } {
      } else {
        lappend Selection $tail System $one
      }
    }
    if {![info exists Selection]} return
    set selection $Selection
  }

  puts $selection

  ## Iterate over the items..
  foreach {system type path} $selection {
    if {[string equal $type Directory]} return
    if {[catch {set sys [lindex $GUI_Graphs($system) 0]}]} return
    switch $Mode {
      OpenRibbon {
        if {[catch {GUI_CreateSystemWindow_Ribbon $sys $system} system_window]} {
          gut_ErrorDlg $system_window
        }}
      Open {
        if {[catch {GUI_CreateSystemWindow $sys $system} system_window]} {
          gut_ErrorDlg $system_window
        }
      }
    }
    if {[string equal [string tolower $Mode] edit]} {
      set win .sys_$sys
      catch {::EditSystem::EditSystem $sys $win}
    }
  }
  return $system_window
};# GUI_MW_Open/EditSelectedSystem

## GUI_MW_ActionPopUp
proc GUI_MW_ActionPopUp {win args} {
  global GUI_MainWidgets GUI_Systems GUI_Text GUI_Options
  set twin [winfo toplevel $GUI_MainWidgets(Tree)]
  set selection [file tail [lindex $GUI_MainWidgets($twin:CurrentSelection) 0]]
  set Type Directory
  if {[string length $selection]} {
    if {  [info exists GUI_Systems($selection)] && \
          [string match *_task [lindex $GUI_Systems($selection) 2]] } {
      set Type Directory
    } else {
      set Type System
    }
  }
  set menu ${win}.actions_popup
  set index 0
  catch {menu $menu -tearoff false} 
  catch {$menu delete 0 end}
  switch -exact $Type {
    System {
      $menu add command -label $GUI_Text(RenameSelectedSystem) -command \
        "GUI_MW_RenameSystem \"$selection\"" -image ITRenameSystem \
        -compound $GUI_Options(MenuImageLocation)
      $menu add command -label $GUI_Text(OpenSelectedSystem) -command \
        {GUI_MW_Open/EditSelectedSystem Open} -image ITOpenSystem \
        -compound $GUI_Options(MenuImageLocation)
      $menu add command -label $GUI_Text(OpenSelectedSystemRibbon) -command \
        {GUI_MW_Open/EditSelectedSystem OpenRibbon} -image ITOpenSystem \
        -compound $GUI_Options(MenuImageLocation)
      $menu add command -label $GUI_Text(EditSelectedSystem) -command \
        {GUI_MW_Open/EditSelectedSystem Edit} -image ITEditModule \
        -compound $GUI_Options(MenuImageLocation)
      $menu add command -label $GUI_Text(ExportSelectedSystem) -command \
        {GUI_MW_ImportExportSelectedSystem Export} -image ITExportSystem \
        -compound $GUI_Options(MenuImageLocation)
      $menu add command -label $GUI_Text(ExportSelectedSystemAsComponent) \
        -command \
        {GUI_MW_ImportExportSelectedSystem Component} -image ITExportSystem \
        -compound $GUI_Options(MenuImageLocation)
      incr index 5
    }
    Directory {
    }
  }
  if {![string length $selection]} {
    GUI_LoadImage ITNewFolder
    $menu add command -label $GUI_Text(NewFolder) -image ITNewFolder \
      -command {GUI_MW_CreateFolder} -compound $GUI_Options(MenuImageLocation)
    incr index
  }
  $menu add command -label $GUI_Text(NewSystem) -image ITNewSystem -command \
    {::EditSystem::CreateSystem} -compound $GUI_Options(MenuImageLocation)
  incr index
  if {[string length $selection]} {
    $menu add separator
    incr index
    if {[string equal $Type Directory]} {
      GUI_LoadImage ITDeleteFolder
      $menu add command -label $GUI_Text(DeleteFolder) -command \
         "GUI_MW_DeleteSystem \{$selection\}" -image ITDeleteFolder \
         -compound $GUI_Options(MenuImageLocation)
      incr index
    } else {
      $menu add command -label $GUI_Text(DeleteSystem) -command \
         "GUI_MW_DeleteSystem \{$selection\}" -image ITDeleteSystem \
         -compound $GUI_Options(MenuImageLocation)
      if {[string compare $selection "{}"] == 0} {
        $menu entryconfigure $index -state disabled
      }  
      incr index
    }
  } else {
    $menu add command -label $GUI_Text(ImportSystem) -command \
      {GUI_MW_ImportExportSelectedSystem Import} -image ITImportSystem \
      -compound $GUI_Options(MenuImageLocation)
    incr index
  }
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
}

## GUI_MW_CreateFolder
proc GUI_MW_CreateFolder {{name {}}} {
  global GUI_Text GUI_Systems
  if {![string length $name]} {
    set name [gut_EntryDialog -message $GUI_Text(EnterFolderName) \
      -title {Creating New Folder...} -regexp {[_=\;|\"\#]} -width 20]
    if {![string length $name]} {return}
  }

  ## Does the Name exists?
  if {[info exists GUI_Systems($name)]} {
    if {[string length [lindex $GUI_Systems($name) end]]} {
      gut_ErrorDlg [format $GUI_Text(CannotCreate:FolderExists) $name]
    } else {
      gut_ErrorDlg [format $GUI_Text(CannotCreate:SystemExists) $name]
    }
    return
  }
  set ID 1
  foreach system [array names GUI_Systems] {
    set id [lindex $GUI_Systems($system) 0]
    if {$id > $ID} {set ID $id}
  }
  incr ID
  lappend NewEl $ID $name
  set nname [string tolower $name]
  set nname [join [split $nname " "] {}]
  append nname _task
  lappend NewEl $nname {}
  set GUI_Systems($name) $NewEl
  ## Reload Explorer
  global GUI_MainWidgets
  #set Folder [join [split $name { }] _]
  opt_LoadSystemTree $GUI_MainWidgets(Tree)
  TreeView:setselection $GUI_MainWidgets(TreeView) /
  TreeView:draw $GUI_MainWidgets(TreeView)
}

## GUI_MW_DeleteSystem
#   Delete a system or a *folder*
proc GUI_MW_DeleteSystem {system {move 0}} {
  global GUI_Systems GUI_Graphs GUI_Text

  ## If "system" is an empty string, then the user may have selected many
  ## items. If that is the case, arrange an iteration over these items...
  if {![string length $system]} {
    global GUI_MainWidgets
    if {![info exists GUI_MainWidgets(TreeView:Selected)]} {return}
    set selection $GUI_MainWidgets(TreeView:Selected)
    if {![llength $selection]} {return}
    foreach one $selection {
      lappend Selection [file tail $one]
    }
    set system $Selection
  } else {
    ## The user has selected only a single item. Place that into a list, so as
    ## not to iterate over its sub-elements (if its name contains spaces...)
    set system [list $system]
  }
  set removed 0

  ## Iterate over the items..
  foreach System $system {
    set Folder /
    if {[info exists GUI_Systems($System)] && \
         [string match *_task [lindex $GUI_Systems($System) 2]] } {
      set Type Directory
      if {!$move} {
        if {[string equal [gut_question \
              [format $GUI_Text(DeleteFolder?) $System]] no]} \
            {continue}
      }
      set systems [split [lindex $GUI_Systems($System) end] =]
      foreach system $systems {
        unset GUI_Graphs([lindex [split $system |] 0])
      }
      unset GUI_Systems($System)
      incr removed
    } else {
      set Type System
      if {!$move} {
        if {[string equal [gut_question \
              [format $GUI_Text(DeleteSystem?) $System]] no]} \
            {continue}
      }
      catch {lindex $GUI_Systems($System) 1} GraphName
      if {[catch {unset GUI_Systems($System)}]} {
        foreach folder [array names GUI_Systems] {
          set contents [lindex $GUI_Systems($folder) end]
          if {[llength $contents] == 0} {continue}
          set contents [split $contents "="]
          set index [lsearch -glob $contents "*|$System"]
          if {$index < 0} {continue}
          set GraphName [lindex [split [lindex $contents $index] |] 0]
          if {!$move} {unset GUI_Graphs($GraphName)}
          set contents [join [lreplace $contents $index $index] "="]
          set GUI_Systems($folder) \
             [lreplace $GUI_Systems($folder) end end $contents]
          append Folder $folder
          regsub -all -- { } $Folder {_} Folder 
          break
        }
      } else {
        set nname "[string map {{ } {}} [string tolower $GraphName]]_system"
        unset -nocomplain GUI_Graphs($GraphName)
        if {!$move} {unset -nocomplain GUI_Graphs($nname)}
      }
      incr removed
    }
  }
  if {$removed} {
    global GUI_MainWidgets GUI_SystemsModified
    set GUI_SystemsModified 1
    opt_LoadSystemTree $GUI_MainWidgets(Tree)
    catch {Tree:open $GUI_MainWidgets(Tree) $Folder}
    catch {TreeView:setselection $GUI_MainWidgets(TreeView) $Folder}
    TreeView:draw $GUI_MainWidgets(TreeView)
  }
  if {$removed > 1} {
    gut_info [format $GUI_Text(Removed%dItems) $removed] {Removing Items...}
  }
}

## GUI_MW_RenameSystem
#  Rename a system...
proc GUI_MW_RenameSystem {system} {
  global GUI_Systems GUI_Graphs GUI_Text GUI_MainWidgets

  if {![string length $system]} {return}
  if {![info exists GUI_Graphs($system)]} {
    ## This is a folder!
    return
  }
  ## Get the new system name...
  set name [gut_EntryDialog -message $GUI_Text(EnterSystemName) \
    -title "Renaming System \"$system\"..." \
    -regexp {[_=\;|\"\#\$\(\)\[\]\{\}]} \
    -width 30 -initial $system]
  if {![string length $name]} {return}
  if {[string equal $name $system]} {return}
  set nname [string map {{ } {}} [string tolower $name]]_system
  if {[info exists GUI_Graphs($nname)]} {
    gut_ErrorDlg [format $GUI_Text(SystemExists!) $name]
    return
  }
  if {[info exists GUI_Systems($name)]} {
    if {[string length [lindex $GUI_Systems($name) end]]} {
      gut_ErrorDlg [format $GUI_Text(CannotCreateSystem:FolderExists) $name]
      return
    }
  }

  ## Get Current info...
  set Graphs1 $GUI_Graphs($system)
  set Graphs2 $GUI_Graphs([lindex $Graphs1 0])
  set GUI_Graphs($name)  [list $nname $name]
  set GUI_Graphs($nname) [lreplace $Graphs2 0 0 $name]
  ## Is our system in a folder?
  if {[info exists GUI_Systems($system)]} {
    set Systems $GUI_Systems($system)
    set GUI_Systems($name) [list [lindex $Systems 0] $name $nname {}] 
    if {[catch {GUI_MW_DeleteSystem $system 1} error]} {
      puts $error
    }
  } else {
    ## Our system is in a folder...
    ## Try to locate the folder...
    foreach one [array names GUI_Systems] {
      if {[string match *_system [lindex $GUI_Systems($one) 2]]} {continue}
      set system_list [split [lindex $GUI_Systems($one) end] =|]
      if {[lsearch -exact $system_list $system]} {
        set GUI_Systems($name) [list [lindex $GUI_Systems($one) 0] \
                                     $name $nname {}]
        catch {GUI_MW_MoveSystem $name $one}
        catch {GUI_MW_DeleteSystem $system 1}
        break
      }
    }
  }
  unset -nocomplain GUI_Graphs($system) \
    GUI_Graphs([string map {{ } {}} [string tolower $system]]_system)
  opt_LoadSystemTree $GUI_MainWidgets(Tree)
  catch {Tree:open $GUI_MainWidgets(Tree) $Folder}
  if {[catch {TreeView:setselection $GUI_MainWidgets(TreeView) $Folder}]} {
    TreeView:setselection $GUI_MainWidgets(TreeView) /
  }
  TreeView:draw $GUI_MainWidgets(TreeView)
};# GUI_MW_RenameSystem

## GUI_MW_GetModule2Reload
#   Prompts the user to select a module for reloading it...
proc GUI_MW_GetModule2Reload \
  {{prompt {Select Module to Reload...}} {title {Reload Module...}}} {
  global creole_config 
  set result {}
  foreach module [gut_MultipleSelectFromList \
                     [lsort [array names creole_config]] $prompt $title] {
    lappend result [file normalize [set ::creole_${module}_home]]
  }
  return $result
}

## GUI_MW_AppendSelection
proc GUI_MW_AppendSelection {w x y} {
  global GUI_MainWidgets GUI_Options Tree

  catch {$w delete -?Special?-SelectionRectangular-}
  set rootx [winfo x $w]
  set rooty [winfo y $w]
  if {$x < $rootx} {set x [incr rootx]}
  if {$y < $rooty} {set y [incr rooty]}
  set x1 [$w canvasx $GUI_MainWidgets(TreeView:x)]
  set y1 [$w canvasy $GUI_MainWidgets(TreeView:y)]
  set x2 [$w canvasx $x] 
  set y2 [$w canvasy $y]
  foreach {sx1 sy1 sx2 sy2} [$w cget -scrollregion] {
#   puts "$sx1 $sy1 $sx2 $sy2 -> $x1 $y1 $x2 $y2"
#   if {int($x1) < $rootx} {set x1 [incr rootx]}
#   if {int($y1) < $rooty} {set y1 [incr rooty]}
    if {int($x2) > $sx2} {set x2 [incr sx2 -2]}
    if {int($y2) > $sy2} {set y2 [incr sy2 -2]}
#   puts "----$sx1 $sy1 $sx2 $sy2 -> $x1 $y1 $x2 $y2"
  }
  set x $GUI_MainWidgets(TreeView:x)
  set y $GUI_MainWidgets(TreeView:y)
  set Selected {}
  foreach one [$w find overlapping $x1 $y1 $x2 $y2] {
    lappend Selected [lindex [$w gettags $one] end]
  }
  $w create rectangle $x1 $y1 $x2 $y2 -tags -?Special?-SelectionRectangular- \
     -outline $GUI_Options(ExplorerStandout) -dash {6 4}
  set Selected [lsort -unique $Selected]
  if {![string length $Selected]} {
    set Selected /
  } elseif {[string equal [lindex $Selected 0] {}]} {
    set Selected [lrange $Selected 1 end]
  }
  if {[info exists GUI_MainWidgets(TreeView:Selected)] && \
      [string equal $GUI_MainWidgets(TreeView:Selected) $Selected]} {
    return
  }
  set GUI_MainWidgets(TreeView:Selected) $Selected
  set T $Tree($w:Tree)
  set Tree($T:selection) $GUI_MainWidgets(TreeView:Selected)
  TreeView:drawselection $w
  set $GUI_MainWidgets(TreeView:x) $x
  set $GUI_MainWidgets(TreeView:y) $y
  set GUI_MainWidgets([winfo toplevel $GUI_MainWidgets(Tree)]:CurrentSelection)\
    {}
}

proc GUI_MW_ImportExportSelectedSystem {mode} {
  global GUI_MainWidgets
  switch $mode {
    Import {
      set file [GUI_SelectFile {} -title \
         {Select File containing System to Import...} \
         -parent $GUI_MainWidgets(TreeView)]
      if {![string length $file]} {return}
      opt_ReadConfig  $file
      opt_LoadSystems $file
      opt_LoadSystemTree $GUI_MainWidgets(Tree)
      TreeView:draw $GUI_MainWidgets(TreeView)
    }
    Component {
      # Export system as a Component.
      global GUI_Graphs
      set selection [GUI_MW_GetSelection]
      if {![llength $selection]} {return}
      set System [lindex $selection 0]
      if {![string length $System]} {return}
      set info $GUI_Graphs($System)
      if {[llength $info] != 2} {return}
      set system [lindex $info 0]
      
      set directory [GUI_SelectDirectory {} -title \
         {Select Folder to Export System as Component...}]
      if {![string length $directory]} {return}
      set System [string map {{ } _} $System]
      set directory $directory/System_$System
      file mkdir $directory
      set info $GUI_Graphs($system)
      set modules {}
      foreach module [lindex $info 4] {
        lappend modules [lindex $module 0]
      }
      # Create the creole_config.tcl file.
      set file [open $directory/creole_config.tcl w]
      puts $file "set creole_config(System_$System) \{  
    title \{$System\}
    pre_conditions \{
        collection_attributes      {}
        document_attributes        {}
        annotations                {}
    \}
    post_conditions \{
        collection_attributes      {}
        document_attributes        \{$System\}
        annotations                {} 
    \}
    viewers {
      {} AnnotationExplorer {Explore Annotations...}
    }
    parameters \{
      Modules \{$modules\} \{$modules\}
    \}
    coupling loose
\};# $System"
      close $file
      # Create the component main file.
      set file [open $directory/System_$System.tcl w]
      puts $file "proc creole_System_$System {doc modules args} {
  foreach module \$modules {
    tip_RunAnnotator \$doc \$module
  }
};# creole_System_$System

proc creole_System_${System}_Initialize {col doc modules args} {
  foreach module \$modules {
    tip_InitialiseAnnotator \$doc \$module \$col
  }
};# creole_System_${System}_Initialize

proc creole_System_${System}_Finish {col doc modules args} {
  foreach module \$modules {
    tip_FinishAnnotator \$doc \$module \$col
  }
};# creole_System_${System}_Finish
"
      close $file
    }
    Export {
      global GUI_Graphs
      set selection [GUI_MW_GetSelection]
      if {![llength $selection]} {return}
      set System [lindex $selection 0]
      if {![string length $System]} {return}
      set info $GUI_Graphs($System)
      if {[llength $info] != 2} {return}
      set system [lindex $info 0]
      
      set file [GUI_SelectSaveFile {} -title \
         {Select File to Export System...} -parent $GUI_MainWidgets(TreeView)]
      if {![string length $file]} {return}

      set info $GUI_Graphs($system)
      set geometry [lindex $info 1]
      set arci 1
      set file [open $file w]
      fconfigure $file -encoding utf-8
      puts $file "__START_CONFIG\n__START_TASKS\n__top_geometry=500x350+0+0\n__top_pane_percent=35\n__start_graphs"
      puts $file "1\;$System\;$system\;\n__end_graphs"
      puts $file \
         "__START_GRAPH=$system\n__geometry=$geometry\n__timeLog=0"
      puts $file "__grid=[lindex $info 3]\n__start_nodes"
      foreach module [lindex $info 4] {
        puts -nonewline $file "$system\,[join [lrange $module 0 2] \;]\;0\;"
        set Module_Label [lindex $module 3]
        regsub -all "\n" $Module_Label {\\n} Module_Label
        puts $file "$Module_Label\;\;\;unrunnable\;"
      }
      puts $file "__end_nodes\n__start_arcs"
      foreach arc [lindex $info 5] {
        puts -nonewline $file \
          "$system\,$arci\;$system\,[lindex $arc 0]\;$system\,"
        puts $file "[lindex $arc 1]\;[lindex $arc 2]\;\;unrunnable\;"
        incr arci
      }
      puts $file "__end_arcs\n__END_GRAPH=$system\n__END_CONFIG"
      close $file
    }
  }
};# GUI_MW_ImportExportSelectedSystem

##############################################################################
##############################################################################
## Procedures for Enabling/Disabling our graphical Interface...
##############################################################################
##############################################################################

proc GUI_EnableGUI {} {
  if {![winfo exists .mw]} {
    GUI_CreateMainWindow
  }
  foreach toplevel [winfo children .] {
    if {[string match -nocase {.#*} $toplevel] ||
        [string match -nocase {.tkcon*} $toplevel]} {continue}
    catch {wm deiconify $toplevel}
  }
}

proc GUI_DisableGUI {} {
  foreach toplevel [winfo children .] {
    if {[string match {.#*} $toplevel]} {continue}
    catch {wm withdraw $toplevel}
  }
}
