##############################################################################
##  SystemBoard.tcl:                                                        ##
## ---------------------                                                    ##
##  This file implements the "System Board" Environment of the "Ellogon"    ##
##  Text Engineering Platform.                                              ##
##                                                                          ##
##  Copyright 1998-2005 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
###############################################################################
## Petasis George, 20/08/2000
##   petasis@iit.demokritos.gr
###############################################################################
namespace eval ::SystemBoard {

## CreateMainMenu
#   Creates the menu bar displayed on top of the window
proc CreateMainMenu {win} {
  global GUI_Text GUI_Underline GUI_Options
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }

  ## File Menu
  set file $menubar.file; catch {destroy $file}
  $menubar add cascade -label $GUI_Text(File) -menu $file \
     -underline $GUI_Underline(File)
  menu $file
  $file add command -label $GUI_Text(Close) -underline \
     $GUI_Underline(Close) -command "GUI_CloseWindow $win $win" \
     -image ITCancel -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $file}

  ## Help Menu
  set help $menubar.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help

  return $menubar
};# CreateMainMenu

## Open
#  This function creates the System Board and places an existing system in it,
#  if any selected in the main Explorer window...
proc Open {{system {}}} {
  global GUI_Text GUI_Underline Locale KeyboardLocale GUI_Options PANE
  set win .system_board_$system
  if {[winfo exists $win]} {raise $win; focus $win; return}

  toplevel $win
  wm withdraw $win
  wm geometry $win 650x450
  
  if {[string length $system]} {
    wm title $win "System Board: System [string map {_ { }} $system]"
  } else {
    wm title $win {System Board: Creating a new system...}
  }
  set menubar [CreateMainMenu $win]
  $win configure -menu $menubar

  ## Status Bar
  set ::GUI_StatusBar::StatusBar($win) {}
  set ::GUI_StatusBar::StatusBar($win:Def) {}
  GUI_CreateStatusBar $win.statusbar

  ## Toolbar...

  ## Main window area
  frame $win.main -bd 0 -relief flat
    ## We will have a pane on the left side with the modules vertical toolbar
    set ModuleToolbar [::TabbedToolbar::TabbedToolbar \
      $win.main.modules_toolbar -bg #A0A0A0]
    ## And a frame on the right for the rest of the window
    frame $win.main.frame -bd 0 -relief flat
    ## Add the pane...
    pane $win.main.modules_toolbar $win.main.frame -orient horizontal
  pack $win.main -fill both -expand 1
  Widget_PaneSet $win.main 1 0.270
  ArrowButton $win.main.__h1.b1 -type button -width 6 -height 60 \
    -dir left -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.main 1 0"
  ::$win.main.__h1.b1:cmd configure -cursor left_side
  ArrowButton $win.main.__h1.b2  -type button -width 6 -height 60 \
    -dir right -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.main 1 0.270"
  ::$win.main.__h1.b2:cmd configure -cursor right_side
  pack $win.main.__h1.b1 $win.main.__h1.b2 -side top
  set canvases [AddModulesInToolbar $ModuleToolbar]
  gut_ModalDlg $win 650 450
  wm deiconify $win
  update
  wm geometry $win 650x450
  foreach canvas $canvases {
    $canvas yview moveto 0
  }
  return $win
};# Open

## AddModulesInToolbar
#   Adds the modules in the Module Toolbar according to their Language.
proc AddModulesInToolbar {toolbar} {
  set ModuleSpec [::EditSystem::GUI_HS_GetModules]
  set canvases {}
  foreach {lang list} $ModuleSpec {
    foreach {x y} {0 0} {break}
    if {[catch {GUI_LoadImage $lang}]} {
      set canvas [$toolbar addpage $lang -label $lang -image ITHelp]
    } else {
      set canvas [$toolbar addpage $lang -label $lang -image $lang]
    }
    foreach module_spec [lindex $list 0] {
      incr y [AddModuleInCanvas $canvas.canvas $module_spec $x $y]
    }
    lappend canvases $canvas.canvas
  }
  return $canvases
};# AddModulesInToolbar

## AddModuleInCanvas
#   Add a single module in a canvas widget
GUI_LoadImage TRayTracedSystem
proc AddModuleInCanvas {canvas module_spec x y} {
  global GUI_Options Tree
  foreach {title module} $module_spec {break}
  set title [string map {{  } \n} [string trim $title]]

  $canvas create image $x $y -image TRayTracedSystem \
    -tags [list $module img_$module]
  incr y 40
  $canvas create text  $x $y -justify center -text $title \
    -font $Tree(font) -fill black -tags [list $module text_$module]
  foreach {x1 y1 x2 y2} [$canvas bbox img_$module] {break}
  incr x1 -5; incr y1 -5
  incr x2  5; incr y2  5
  $canvas lower [$canvas create rectangle $x1 $y1 $x2 $y2 \
    -fill #A0A0A0 -outline #A0A0A0 -tags [list $module rect_$module]] all
  foreach {x1 y1 x2 y2} [$canvas bbox text_$module] {break}
  $canvas lower [$canvas create rectangle $x1 $y1 $x2 $y2 \
    -fill #A0A0A0 -outline #A0A0A0 -tags [list $module rect_$module]] all
  $canvas bind $module <Enter> \
    "$canvas itemconfigure text_$module -fill #a52a2a
     $canvas itemconfigure rect_$module -fill #ffe4b5 -outline #ffe4b5"
  $canvas bind $module <Leave> \
    "$canvas itemconfigure text_$module -fill black
     $canvas itemconfigure rect_$module -fill #A0A0A0 -outline #A0A0A0"
  return 100
};# AddModuleInCanvas
}
