##############################################################################
##  SystemWindow.tcl:                                                       ##
## ---------------------                                                    ##
##  This file contains the implementation code of System windows. System    ##
##  windows are the main graphical component of the platform, as they are   ##
##  the windows in which language processing occurs. The file was initially ##
##  created on 21/9/1998, by Georgios Petasis and has been modified numerous##
##  times since then :-) to fix bugs or add new features/functionality.     ##
##                                                                          ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## GUI_CreateSystemMenu
#
proc GUI_CreateSystemMenu {win {system {}}} {
  global GUI_Text GUI_Underline GUI_Options GUI_ToolConfig \
         CDM_EnableMemoryCompression
  #set menubar [menu ${win}.mb -type menubar -relief $GUI_Options(MenuRelief)
  if {[catch {menu $win.mb -relief $GUI_Options(MenuRelief) \
              -activeborderwidth $GUI_Options(MenuReliefBorderWidth) \
              -borderwidth $GUI_Options(MenuReliefBorderWidth)} menubar]} {
    set menubar $win.mb
    $menubar delete 0 end
  }

  set canvas $win.frame.canvas
  if {![string length $system]} {
    set system $::GUI_GraphInfo(SystemName_$canvas)
  }
  
  ## File Menu
  set file $menubar.file; catch {destroy $file}
  $menubar add cascade -label $GUI_Text(File) -menu $file \
     -underline $GUI_Underline(File)
  menu $file
  $file add command -label $GUI_Text(OpenCollection) -underline \
     $GUI_Underline(OpenCollection) -image ITOpen\
     -command "GUI_CollectionDlg Open $win; GUI_System_Get_Opened $win" \
     -compound $GUI_Options(MenuImageLocation)
  $file add command -label $GUI_Text(CreateCollection) -underline \
     $GUI_Underline(CreateCollection) -image ITNewMany \
     -command "GUI_NewCollection -win $win.ccw" \
     -compound $GUI_Options(MenuImageLocation)
  $file add command -label $GUI_Text(EditCollections) -underline \
     $GUI_Underline(EditCollections) -command "GUI_CollectionDlg Edit $win" \
     -image ITEdit -compound $GUI_Options(MenuImageLocation)
  $file add command -label $GUI_Text(SaveCollection) -underline \
     $GUI_Underline(SaveCollection) -command "GUI_CollectionSave $win" \
     -image ITSave -compound $GUI_Options(MenuImageLocation)
  $file add separator
  $file add command -label $GUI_Text(InsEditCollectionComment) -underline \
     $GUI_Underline(InsEditCollectionComment) -command \
     "GUI_ColView EditCom $win" -image ICommentSmall \
     -compound $GUI_Options(MenuImageLocation)
  $file add command -label $GUI_Text(CollectionProperties) -underline \
     $GUI_Underline(CollectionProperties) -command "GUI_ColView Status $win" \
     -image ITProperties -compound $GUI_Options(MenuImageLocation)
  $file add separator
  $file add command -label $GUI_Text(Options)... -image ITBlank \
     -underline $GUI_Underline(Options) -command GUI_EditPreferences \
     -compound $GUI_Options(MenuImageLocation)
  $file add separator
  $file add command -label $GUI_Text(Close) -underline $GUI_Underline(Close)\
     -command "GUI_CloseWindow $win $win" -image ITExit \
     -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $file}

  ## View Menu
  set view $menubar.view; catch {destroy $view}
  $menubar add cascade -label $GUI_Text(View) -menu $view \
     -underline $GUI_Underline(View)
  menu $view
  $view add command -label $GUI_Text(Text) -underline $GUI_Underline(Text) \
     -command "GUI_DocView Text $win" \
     -image ITViewText -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(ExploreAnnotations) -underline \
  $GUI_Underline(ExploreAnnotations) -command "GUI_DocView ExAnnotations $win" \
     -image ITExplore -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(Annotations) -underline \
  $GUI_Underline(Annotations) -command "GUI_DocView Annotations $win" \
     -image ITViewAnns -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(DocumentAttributes) -underline \
  $GUI_Underline(DocumentAttributes) -command "GUI_DocView Attributes $win" \
     -image ITViewDocAtt -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(CollectionAttributes) -underline \
  $GUI_Underline(CollectionAttributes) -command "GUI_ColView Attributes $win" \
     -image ITViewColAtt -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(TimeLog) -underline \
  $GUI_Underline(TimeLog) -command "GUI_ColView TimeLog $win" \
     -image ITViewTimeLg -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(ErrorLog) -underline \
  $GUI_Underline(TimeLog) -command "GUI_ColView ErrorLog $win" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowCachedMemory) -underline \
     $GUI_Underline(ShowCachedMemory) -command \
     "ggi_view_CDM_CachedMemoryObjects" \
     -image ITMemory -compound $GUI_Options(MenuImageLocation)
  $view add command -label $GUI_Text(ClearCachedMemory) -underline \
     $GUI_Underline(ClearCachedMemory) -command \
     "ggi_view_CDM_CachedMemoryObjectsClearCache" \
     -image ITMemoryClear -compound $GUI_Options(MenuImageLocation)
  $view add separator
  $view add command -label $GUI_Text(ShowToolbars) -underline \
     $GUI_Underline(ShowToolbars) -command \
     "Toolbar::deiconify $win.tf.express $win.tf.module $win.tf.tb $win.tf.tb2 \
                         $win.tf.tb3 $win.tf.tb4" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $view}

  ## Layout Menu
  set layout $menubar.layout; catch {destroy $layout}
  $menubar add cascade -label $GUI_Text(Layout) -menu $layout \
     -underline $GUI_Underline(Layout)
  menu $layout
  $layout add command -label $GUI_Text(Redraw) -underline \
     $GUI_Underline(Redraw) -command [list GUI_DrawSystem $canvas $system] \
     -image ITRedraw -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(AddCurvePoint) -underline \
     $GUI_Underline(AddCurvePoint) -command \
     [list GUI_SystemAddCurvePoint $system $canvas] \
     -image ITAddCurveP -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(ClearCurvePoints) -underline \
     $GUI_Underline(ClearCurvePoints) -command \
     [list GUI_SystemClearCurvePoints $system $canvas] \
     -image ITCancel -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(AddDependency) -underline \
     $GUI_Underline(AddDependency) -command \
     [list GUI_SystemAddDepend $system $canvas] \
     -image ITAddDepend -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(RemoveDependency) -underline \
     $GUI_Underline(RemoveDependency) -command \
     [list GUI_SystemRemDepend $system $canvas] \
     -image ITRemDepend -compound $GUI_Options(MenuImageLocation)
  $layout add separator
  $layout add command -label $GUI_Text(SetGrid) -underline \
     $GUI_Underline(SetGrid) -command [list GUI_SystemSetGrid $canvas $system] \
     -image ITSetGrid -compound $GUI_Options(MenuImageLocation)
  $layout add separator
  $layout add command -label $GUI_Text(AddModulesInSystem) -underline \
     $GUI_Underline(AddModulesInSystem) \
     -command [list ::EditSystem::AddModulesInSystem $canvas $system] \
     -image ITAdd -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(RemoveSystemModule) -underline \
     $GUI_Underline(RemoveSystemModule) \
     -command "::EditSystem::RemoveModuleFromSystem $canvas \"$system\" \
     \[GUI_SystemGraph_GetSelectedModule $win\]" \
     -image ITRemove -compound $GUI_Options(MenuImageLocation)
  $layout add command -label $GUI_Text(EditSystem) -underline \
     $GUI_Underline(EditSystem) \
     -command [list ::EditSystem::EditSystem $system $win] \
     -image ITEditSystem -compound $GUI_Options(MenuImageLocation)
  $layout add separator
  $layout add command -label $GUI_Text(SaveLayout) -underline \
     $GUI_Underline(SaveLayout) -command {opt_SaveConfig} \
     -image ITSaveLayout -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $layout}

  ## Options Menu
  set options $menubar.options; catch {destroy $options}
  $menubar add cascade -label $GUI_Text(Options) -menu $options \
     -underline $GUI_Underline(Options)
  menu $options
  $options add command -label $GUI_Text(LogExecutionTime) -underline \
  $GUI_Underline(LogExecutionTime) -command \
  "if \{\[catch \{frame $canvas.timeDisplay -highlightbackground black \
       -highlightthickness 1 -highlightcolor black -bg #ffffaa
       label $canvas.timeDisplay.img -bg #ffffaa -image ICpu
       label $canvas.timeDisplay.l -relief flat -bg #ffffaa -fg black \
           -padx 2 -pady 0 -anchor w -text \[clock format \
      \$GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1\]
       pack $canvas.timeDisplay.img $canvas.timeDisplay.l -side left -fill y
       place $canvas.timeDisplay -in $canvas -x 2 -rely 1.0 -y \
              -[expr {[image height ICpu] + 6}]\}\]\} \{
       destroy $canvas.timeDisplay\}" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $options add checkbutton -label $GUI_Text(IgnoreChanges) \
     -underline $GUI_Underline(IgnoreChanges) -indicatoron 1 \
     -offvalue 0 -onvalue 1 -variable GUI_GraphInfo(IgnoreChanges_$canvas) \
     -command "if \$GUI_GraphInfo(IgnoreChanges_$canvas) \"set GUI_GraphInfo(AutoSave_$canvas) 0\"" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $options add checkbutton -label $GUI_Text(Auto-SaveCollections) \
     -underline $GUI_Underline(Auto-SaveCollections) -indicatoron 1 \
     -offvalue 0 -onvalue 1 -variable GUI_GraphInfo(AutoSave_$canvas) \
     -command "if \$GUI_GraphInfo(AutoSave_$canvas) \"set GUI_GraphInfo(IgnoreChanges_$canvas) 0\"" \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $options add checkbutton -label $GUI_Text(OnErrorsContinue) \
     -underline $GUI_Underline(OnErrorsContinue) -indicatoron 1 \
     -offvalue 0 -onvalue 1 -variable GUI_GraphInfo(IgnoreErrors_$canvas) \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $options add checkbutton -label $GUI_Text(EnableMemoryCompression) \
     -underline $GUI_Underline(EnableMemoryCompression) -indicatoron 1 \
     -offvalue 0 -onvalue 1 -variable CDM_EnableMemoryCompression \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  $options add separator
  $options add command -label $GUI_Text(Options)... \
     -underline $GUI_Underline(Options) -command GUI_EditPreferences \
     -image ITBlank -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $options}

  ## Modules Menu
  set modules $menubar.modules; catch {destroy $modules}
  $menubar add cascade -label $GUI_Text(Modules) -menu $modules \
     -underline $GUI_Underline(Modules)
  menu $modules
  $modules add command -label $GUI_Text(EditModule) -underline \
     $GUI_Underline(EditModule) -command \
      "catch \{GUI_EditModuleSrc \[GUI_SystemGraph_GetSelectedModule $win\]\}" \
     -image ITEditModule -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(ModuleParameters) -underline \
  $GUI_Underline(ModuleParameters) -command "GUI_EditModuleParameters $win" \
     -image ITProperties -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(ViewModuleDirectory) -underline \
  $GUI_Underline(ViewModuleDirectory) -command \
      "GUI_ViewModuleDir \[GUI_SystemGraph_GetSelectedModule $win\]" \
     -image IFolderSmall -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add command -label $GUI_Text(CompileModule) -underline \
      $GUI_Underline(CompileModule) -command "::CompileComponentWizard::Init\
      \[GUI_SystemGraph_GetSelectedModule $win\]" \
     -image ITCompile -compound $GUI_Options(MenuImageLocation)
  $modules add cascade -label $GUI_Text(Debug) -underline \
    $GUI_Underline(Debug) -menu $modules.debug \
    -image ITStar -compound $GUI_Options(MenuImageLocation)
    menu $modules.debug
      $modules.debug add command -label $GUI_Text(TclProfiler) -underline \
      $GUI_Underline(TclProfiler) -command ::CDM_Debug::TclProfiler::Init \
      -image ITViewTimeLg -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add command -label $GUI_Text(CreateNewModule) -underline \
    $GUI_Underline(CreateNewModule) -command {::Component::Create} \
     -image ITCreateModule -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(RenameModule) -underline \
    $GUI_Underline(RenameModule) -command {::Component::RenameComponentWizard} \
    -image ITRenameModule -compound $GUI_Options(MenuImageLocation)
  $modules add separator
  $modules add command -label $GUI_Text(ReloadModule) -underline \
    $GUI_Underline(ReloadModule) -command "PUL_LoadPlugin \
         \[list \[set creole_\[GUI_SystemGraph_GetSelectedModule $win\]_home\]\]
         GUI_RegisterTools" \
    -image ITReloadModule -compound $GUI_Options(MenuImageLocation)
  $modules add command -label $GUI_Text(ReloadAllModules) -underline \
    $GUI_Underline(ReloadAllModules) -command \
    {PUL_LoadPlugins;GUI_RegisterTools} \
     -image ITReLoadModules -compound $GUI_Options(MenuImageLocation)
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $modules}

  ## Utilities Menu
  if {[array exists GUI_ToolConfig]} {
    set utilities ${menubar}.utilities; catch {destroy $utilities}
    $menubar add cascade -label $GUI_Text(Utilities) -menu $utilities \
       -underline $GUI_Underline(Utilities)
    menu $utilities
    GUI_CreateToolMenuEntries $utilities $canvas
  }

  ## Help Menu
  set help ${menubar}.help; catch {destroy $help}
  $menubar add cascade -label $GUI_Text(Help) -menu $help \
     -underline $GUI_Underline(Help)
  menu $help
  GUI_CreateHelpMenuEntries $help

  return $menubar
};# GUI_CreateSystemMenu

proc GUI_CreateSystemWindow_UpdateDocExpressToolbar {c1 c2} {
  set docs [gut_GetCollectionDocuments [lindex [GUI_GetCollectionNames]\
                                              [$c2 getvalue]]]
  if {[llength $docs]} {
    global GUI_Text
    $c1 configure -values [concat $GUI_Text(All) $docs]
  } else {
    $c1 configure -values {}
  }
};# GUI_CreateSystemWindow_UpdateDocExpressToolbar
  
## GUI_CreateSystemWindow_Ribbon
#
proc GUI_CreateSystemWindow_Ribbon {system {title {System Window}} {toplevel {}} } {
  global GUI_Graphs GUI_Text GUI_Underline Locale KeyboardLocale WMIcons \
      GUI_GraphInfo GUI_Options GUI_SystemWindow GUI_SystemErrorLog

  package require ELEP::System::System
  set win .system_[string map {{ } _} $system]
  # if {[winfo exists $win]} {raise $win; return $win}
  if {[catch {
  set systemObj [ELEP::System::System create $win]
  $systemObj initFromEllogonMemoryLegacy $system
  $systemObj show
  # return $systemObj
  } error]} {
    puts stderr $error
    puts $::errorInfo
  }
};# GUI_CreateSystemWindow_Ribbon

## GUI_CreateSystemWindow
#
proc GUI_CreateSystemWindow {system {title {System Window}} {toplevel {}} } {
  global GUI_Graphs GUI_Text GUI_Underline Locale KeyboardLocale WMIcons \
      GUI_GraphInfo GUI_Options GUI_SystemWindow GUI_SystemErrorLog

  package require ELEP::System::System
  set win .system_[string map {{ } _} $system]
  if {[winfo exists $win]} {raise $win; return $win}

  if {![string length $toplevel]} {
    set win .sys_[string map {{ } _} $system]
    if {[winfo exists $win]} {raise $win; return $win}
    set title [string map {_ { }} $title]
    toplevel $win -class ${::ApplicationName}SystemWindow
    wm withdraw $win
    bind $win <Destroy> {array unset GUI_SystemWindow %W:*}
    wm title $win "System: $title"
    wm sizefrom $win user
    catch {wm geometry $win [lindex $GUI_Graphs($system) 1]}
    set menubar [GUI_CreateSystemMenu $win $system]
    $win configure -menu $menubar
  
    ## Status Bar
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):( $GUI_Text(none) )    \
       $GUI_Text(Document):( $GUI_Text(none) )"
    set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
    GUI_CreateStatusBar $win.statusbar
    label $win.statusbar.comment -relief sunken -anchor w -image Blank \
       -borderwidth 0 -highlightthickness 0 -width 0
    checkbutton $win.statusbar.compat -relief sunken -anchor w -borderwidth \
       $GUI_Options(BorderWidth) -highlightthickness 0 -indicatoron 1 \
       -variable CDM_FullGateCompatibility \
       -image ITEllogonMode -selectimage ITWarning
    pack $win.statusbar.compat  -after $win.statusbar.locale \
       -padx 2 -pady 1 -side right -fill y
    pack $win.statusbar.comment -after $win.statusbar.compat \
       -padx 0 -pady 1 -side right -fill y

    set frame $win.frame; set canvas $frame.canvas
    ## Toolbar...
    set toolbar_frame $win.tf; 
    Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)
      
      ## Open Document/Collection Express Toolbar
      set toolbar $toolbar_frame.express
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -float 0 
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Collection) -tooltipvar GUI_Text(TSelectCollection) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -name $toolbar.widgets.collection -entercmd \
         {%W configure -values [GUI_GetCollectionNameTails]} -modifycmd \
         "$toolbar.widgets.document configure -text \$GUI_Text(All)" \
         -tooltipvar GUI_Text(TSelectCollection) -width \
         $GUI_Options(SystemExprToolbarCollectionWidth) -takefocus 0 -pady 3 \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Document) -tooltipvar GUI_Text(TSelectDocument) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -name $toolbar.widgets.document -entercmd \
         "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
          $toolbar.widgets.document $toolbar.widgets.collection" \
         -tooltipvar GUI_Text(TSelectDocument) -width \
         $GUI_Options(SystemExprToolbarDocumentWidth) -takefocus 0 -pady 3 \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
         -textvariable GUI_Text(Load) -image ITLoad -command \
         "GUI_SW_LoadCallback \[lindex \[GUI_GetCollectionNames\] \
           \[$toolbar.widgets.collection getvalue\]\] \
           \[$toolbar.widgets.document.e get\]; GUI_System_Get_Opened $win" \
         -tooltipvar GUI_Text(TSelectedLoad) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      pack $toolbar -side top -anchor w

      ## Module Toolbar...
      set toolbar $toolbar_frame.module
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(EditModule) \
         -image ITEditModule -command \
         "GUI_EditModuleSrc \[GUI_SystemGraph_GetSelectedModule $win\]" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadModule) \
         -image ITReloadModule -command "PUL_LoadPlugin \
         \[list \[set creole_\[GUI_SystemGraph_GetSelectedModule $win\]_home\]\]
         GUI_RegisterTools" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ModuleParameters) \
         -image ITProperties -command "GUI_EditModuleParameters $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ViewModuleDirectory) \
         -image IFolderSmall -command \
         "GUI_ViewModuleDir \[GUI_SystemGraph_GetSelectedModule $win\]" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      pack $toolbar -side right -anchor ne -in $toolbar_frame.express -fill y
      Toolbar::deactivate $toolbar 

      ## Add a horizontal separator
      Toolbar::addseparator $toolbar -name $toolbar_frame.horiz1 \
           -orient horizontal

      ## Collection Toolbar
      set toolbar $toolbar_frame.tb
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(NewCollectionBT) \
         -image ITNewMany -command "GUI_NewCollection -win $win.ccw" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
         -image ITOpen -command \
         "GUI_CollectionDlg Open $win; GUI_System_Get_Opened $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TSaveCol) \
         -image ITSave -command "GUI_CollectionSave $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TEditCol) \
         -image ITEdit -command "GUI_CollectionDlg Edit $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      pack $toolbar -side left -fill y
    
      ## Viewers Toolbar
      set toolbar $toolbar_frame.tb2
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewDocText) \
         -image ITViewText -command "GUI_DocView Text $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ExploreAnnotations) \
         -image ITExplore -command "GUI_DocView ExAnnotations $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewDocAnns) \
        -image ITViewAnns -command "GUI_DocView Annotations $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewDocAttrs) \
         -image ITViewDocAtt -command "GUI_DocView Attributes $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewColAttrs) \
         -image ITViewColAtt -command "GUI_ColView Attributes $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TimeLog) \
         -image ITViewTimeLg -command "GUI_ColView TimeLog $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ExploreToolbox) \
         -image ITTools -command "::GUI_Toolbox::Explorer $canvas" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      set widget [Toolbar::addwidget $toolbar ArrowButton -dir bottom \
         -relief flat -bd 0 -ipadx 0 -width 6 -height 10 \
         -bg $GUI_Options(ToolbarBackground) \
         -highlightbackground $GUI_Options(ToolbarBackground) \
         -highlightthickness 0 -statusvar ::GUI_StatusBar::StatusBar($win)]
      $widget configure \
        -command "$widget configure -dir \[GUI_DocView SelectViewer $win\]"
      pack $toolbar -side left -fill y
      Toolbar::deactivate $toolbar
      set GUI_SystemWindow($win:ViewersToolbar) $toolbar

      ## Layout Toolbar
      set toolbar $toolbar_frame.tb3
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                      -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TRedraw) \
         -image ITRedraw -command [list GUI_DrawSystem $canvas $system] \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TAddCurvePoint) \
      #   -image ITAddCurveP -command "GUI_SystemAddCurvePoint $system $canvas"\
      #   -statusvar ::GUI_StatusBar::StatusBar($win)
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TClrCurvePoint) \
      #   -image ITCancel -statusvar ::GUI_StatusBar::StatusBar($win) \
      #   -command "GUI_SystemClearCurvePoints $system $canvas"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TAddDepend) \
         -image ITAddDepend -command [list GUI_SystemAddDepend $system $canvas]\
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TRemDepend) \
         -image ITRemDepend -command [list GUI_SystemRemDepend $system $canvas]\
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addseparator $toolbar
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TSetGrid) \
      #   -image ITSetGrid -command "GUI_SystemSetGrid $canvas $system" \
      #   -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ITAddModulesInSystem) \
         -command [list ::EditSystem::AddModulesInSystem $canvas $system] \
         -image ITAdd -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ITRemoveSystemModule) \
         -command "::EditSystem::RemoveModuleFromSystem $canvas \"$system\" \
         \[GUI_SystemGraph_GetSelectedModule $win\]" \
         -image ITRemove -statusvar ::GUI_StatusBar::StatusBar($win)
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TEditSystem) \
      #   -image ITEditSystem -command "::EditSystem::EditSystem $system $win" \
      #   -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TSaveLayout) \
         -image ITSaveLayout -command {opt_SaveConfig} \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      pack $toolbar -side left -fill y

      ## Window Toolbar
      set toolbar $toolbar_frame.tb4
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
                      -borderwidth $GUI_Options(ToolbarReliefBorderWidth)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
         -image ITExit -command "GUI_CloseWindow $win $win" \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Help) -image ITHelp \
        -command {GUI_Help_Contents} -statusvar ::GUI_StatusBar::StatusBar($win)
      pack $toolbar -side left -fill y
    pack $toolbar_frame -side top -fill x
  } else {
    set win $toplevel
    set frame $win; set canvas $frame.canvas
  }
  
  ## Main Canvas...
  if {![winfo exists $frame]} {
    ScrolledWidgets::ScrolledCanvas $frame -highlightthickness 0 \
      -borderwidth $GUI_Options(SystemBorderWidth) \
      -relief $GUI_Options(SystemRelief) \
      -bg $GUI_Options(SystemBg) \
      -scrollbd $GUI_Options(SystemScrollbarBorderWidth) \
      -regionpadx $GUI_Options(SystemPadX) \
      -regionpady $GUI_Options(SystemPadY) \
      -confcomm [list GUI_ChangeSystemGeometry $win $system]
    bind $canvas <<MB_1>>  \
      {TkGUI_SetPriv PrevX [%W canvasx %x]; TkGUI_SetPriv PrevY [%W canvasy %y]
       %W delete _my_area_tag_
       TkGUI_SetPriv x %x; TkGUI_SetPriv y %y}
    bind $canvas <<MBR_1>>  "TkGUI_SetPriv x %x; TkGUI_SetPriv y %y;\
       GUI_SystemDeselect $canvas \"$system\""
    bind $canvas <<CMBR_1>> "TkGUI_SetPriv x %x; TkGUI_SetPriv y %y;\
       GUI_SystemDeselect $canvas \"$system\""
    bind $canvas <<Motion_1>> \
      "GUI_SystemSelectArea $canvas \"$system\" %x %y"
    bind $canvas <Destroy> {catch {array unset GUI_SystemErrorLog *%W*}}
    set GUI_SystemErrorLog($canvas) {}
    pack $frame -fill both -expand 1
  }

  ## Grid Settings
  set grid [lindex $GUI_Graphs($system) 3]
  scan $grid "%dx%d" GUI_GraphInfo(GridX_,$canvas) GUI_GraphInfo(GridY_,$canvas)

  ## Initialise each module status (Deactivate all...)
  set info $GUI_Graphs($system)
  set modules [lindex $info 4]
  foreach module $modules {
    set m [lindex $module 0]
    set GUI_GraphInfo(${canvas}${m}) 0
    set result [GUI_GetModuleOptions $m silent $canvas]
    if {[string equal [lindex $result 0] Error]} {
      lappend LoadingModulesErrors [lindex $result 1]
      set result [list {} {}]
    }
    set GUI_GraphInfo(MOptions_${canvas}${m}) $result
  }
  catch {unset GUI_GraphInfo(RunOrder_${canvas})}
  set GUI_GraphInfo(AutoSave_$canvas) 0
  set GUI_GraphInfo(TimeLog_$canvas) {}
  set GUI_GraphInfo(SystemName_$canvas) $system
  GUI_DrawSystem $canvas $system
  if {[info exists LoadingModulesErrors]} {
    global GUI_Text
    gut_many_errors {Opening System Error...} \
       $GUI_Text(LoadingModulesErrors) [list $LoadingModulesErrors]
  }
  set GUI_SystemWindow($win:TimeElapsed) 0
  
  update
  catch {
    set icon [winico load SystemWindowIcon [file join $HomeDir CDM libCDM.dll]]
    winico setwindow $win $icon
    bind $win <Destroy> "+winico delete $icon"
  }
  if {![string length $toplevel]} {
    catch {winico setwindow $win $WMIcons(System)}
    wm deiconify $win
    catch {wm geometry $win [lindex $GUI_Graphs($system) 1]}
  }
  ::ScrolledWidgets::CanvasUpdateScrollRegion $canvas
  return $win
};# GUI_CreateSystemWindow

## GUI_SystemWindowGetInfo
#  Retrieve system information like AutoSave, Error Ignore, etc...
proc GUI_SystemWindowGetInfo {col what args} {
  switch -glob -- $col {
    Col* {
      ## A Collection specifier...
      set info [tip_GetAssociatedInfo $col]
      if {![llength $info]} {
        switch -glob -- $what {
          CollectionOpenInSystem {return 0}
          AutoSave               {return 0}
          IgnoreErrors           {return 0}
          IgnoreChanges          {return 0}
          CollectionChanged      {return 0}
          TimeLog                {return {}}
          GridX                  {return 0}
          GridY                  {return 0}
          Document               {return {}}
          Collection             {return {}}
          Modules                {return {}}
          ModuleState            {return 0}
          ModuleOptionValues     {return {}}
          Canvas                 {return {}}
          SystemName             {return {}}
          SystemWindow           {return {}}
        }
      }
      set canvas [lindex $info 1]
    }
    .*.frame.canvas {
      ## A canvas specifier...
      set canvas $col
    }
    default {error "Unknown Collection/Canvas \"$col\"!"}
  }
  switch -glob -- $what {
    CollectionOpenInSystem {
      ## If this is a collection, ruturn a boolean value denoting whether the
      ## Collection "lives" inside a system...
      switch -glob -- $col {
        Col* {
          ## A Collection specifier...
          set info [tip_GetAssociatedInfo $col]
          if {![llength $info]} {return 0}
          set canvas [lindex $info 1]
          ## Is it really a canvas?
          return [string match .*.frame.canvas $canvas]
        }
        .*.frame.canvas {
          ## A canvas specifier...
          if {[string length $::GUI_GraphInfo(CCollection_$canvas)]} {
            return 1
          } else {
            return 0
          }
        }
      }
    }
    AutoSave          {
      if {![info exists ::GUI_GraphInfo(AutoSave_$canvas)]} {return 0}
      return $::GUI_GraphInfo(AutoSave_$canvas)}
    IgnoreErrors      {
      if {![info exists ::GUI_GraphInfo(IgnoreErrors_$canvas)]} {return 0}
      return $::GUI_GraphInfo(IgnoreErrors_$canvas)}
    IgnoreChanges     {
      if {![info exists ::GUI_GraphInfo(IgnoreChanges_$canvas)]} {return 0}
      return $::GUI_GraphInfo(IgnoreChanges_$canvas)}
    CollectionChanged {
      if {![info exists ::GUI_GraphInfo(CollectionChanged_$canvas)]} {return 0}
      return $::GUI_GraphInfo(CollectionChanged_$canvas)}
    TimeLog           {
      if {![info exists ::GUI_GraphInfo(TimeLog_$canvas)]} {return {}}
      return $::GUI_GraphInfo(TimeLog_$canvas)}
    GridX             {
      if {![info exists ::GUI_GraphInfo(GridX_,$canvas)]} {return 0}
      return $::GUI_GraphInfo(GridX_,$canvas)}
    GridY             {
      if {![info exists ::GUI_GraphInfo(GridY_,$canvas)]} {return 0}
      return $::GUI_GraphInfo(GridY_,$canvas)}
    Document          {
      if {![info exists ::GUI_GraphInfo(CDocument_$canvas)]} {return {}}
      return $::GUI_GraphInfo(CDocument_$canvas)}
    Collection        {
      if {![info exists ::GUI_GraphInfo(CCollection_$canvas)]} {return {}}
      return $::GUI_GraphInfo(CCollection_$canvas)}
    Modules           {
      set modules {}
      set len [string length MOptions_${canvas}]
      foreach entry [lsort [array names ::GUI_GraphInfo MOptions_${canvas}*]] {
        lappend modules [string range $entry $len end]
      }
      return $modules
    }
    ModuleState       {
      set module [lindex $args 0]
                       return [lindex $::GUI_GraphInfo(${canvas}${module}) 0]}
    ModuleOptionValues {
      set module [lindex $args 0];
      if {![info exists ::GUI_GraphInfo(MOptions_${canvas}${module})]} {
        return {}
      }
      return [lindex $::GUI_GraphInfo(MOptions_${canvas}${module}) 0]}
    Canvas            {return $canvas}
    SystemName        {
      if {![info exists ::GUI_GraphInfo(SystemName_$canvas)]} {return {}}
      return $::GUI_GraphInfo(SystemName_$canvas)
    }
    SystemWindow {return [winfo toplevel $canvas]}
  }
};# GUI_SystemWindowGetInfo

## GUI_SystemWindowSetInfo
#  Set system information like AutoSave, Error Ignore, etc...
proc GUI_SystemWindowSetInfo {col what value args} {
  switch -glob -- $col {
    Col* {
      ## A Collection specifier...
      set info [tip_GetAssociatedInfo $col]
      if {![llength $info]} {return {}}
      set canvas [lindex $info 1]
    }
    .*.frame.canvas {
      ## A canvas specifier...
      set canvas $col
    }
    default {error "Unknown Collection/Canvas \"$col\"!"}
  }
  switch -glob -- $what {
    AutoSave          {set ::GUI_GraphInfo(AutoSave_$canvas) $value}
    IgnoreErrors      {set ::GUI_GraphInfo(IgnoreErrors_$canvas) $value}
    IgnoreChanges     {set ::GUI_GraphInfo(IgnoreChanges_$canvas) $value}
    CollectionChanged {set ::GUI_GraphInfo(CollectionChanged_$canvas) $value}
    TimeLog           {set ::GUI_GraphInfo(TimeLog_$canvas) $value}
    GridX             {set ::GUI_GraphInfo(GridX_,$canvas) $value}
    GridY             {set ::GUI_GraphInfo(GridY_,$canvas) $value}
    Document          {set ::GUI_GraphInfo(CDocument_$canvas) $value}
    Collection        {set ::GUI_GraphInfo(CCollection_$canvas) $value}
    ModuleState       {set module $value; set value [lindex $args 0]
                       set ::GUI_GraphInfo(${canvas}${module}) [lreplace \
                         $::GUI_GraphInfo(${canvas}${module}) 0 0 $value]
    }
    ModuleOptionValues {set module $value;  set value [lindex $args 0]
      set ::GUI_GraphInfo(MOptions_${canvas}${module}) [lreplace \
        $::GUI_GraphInfo(MOptions_${canvas}${module}) 0 0 $value]
      set ::GUI_GraphInfo(MOptionsCV_${canvas}${module}) $value
    }
  }
};# GUI_SystemWindowSetInfo

## GUI_ChangeSystemGeometry
#    Changes the geometry specification of a system
proc GUI_ChangeSystemGeometry {win system} {
  global GUI_Graphs
  catch {
    set GUI_Graphs($system) [lreplace $GUI_Graphs($system) 1 1 \
                             [wm geometry $win]]
  }
  return {}
};# GUI_ChangeSystemGeometry

## GUI_ResetSystemGraph
#   Reset System Graph (Initialise each module status (Deactivate all...))
proc GUI_ResetSystemGraph {canvas system} {
  global GUI_Graphs GUI_GraphInfo
  set info $GUI_Graphs($system)
  set modules [lindex $info 4]
  foreach module $modules {
    set m [lindex $module 0]
    set GUI_GraphInfo(${canvas}${m}) \
      [lreplace $GUI_GraphInfo(${canvas}${m}) 0 0 0]
  }
  catch {unset GUI_GraphInfo(RunOrder_$canvas)} 
  set GUI_GraphInfo(CollectionChanged_$canvas) 1
};# GUI_ResetSystemGraph

## GUI_DrawSystem
#  Draws and adds bindings for the canvas containing the system
proc GUI_DrawSystem {canvas system {ArcFrom {}} {ArcTo {}}} {
  global GUI_Graphs GUI_GraphInfo GUI_Options GUI_Text
  set info $GUI_Graphs($system)
  set win [winfo toplevel $canvas]
  set modules [lindex $info 4]
  $canvas delete all
  set STATE(FILL_0) $GUI_Options(ModuleInactive)
  set STATE(FILL_1) $GUI_Options(ModuleEnabled)
  set STATE(FILL_2) $GUI_Options(ModuleActive)
  set STATE(FILL_3) $GUI_Options(ModuleRun)
  array set STATE [list Text_0 $GUI_Options(ModuleTextInactive) \
                        Text_1 $GUI_Options(ModuleTextEnabled) \
                        Text_2 $GUI_Options(ModuleTextActive) \
                        Text_3 $GUI_Options(ModuleTextRun) \
                        Im_0 IModuleInactive Im_1 IModuleEnabled \
                        Im_2 IModuleActive   Im_3 IModuleRun]
  
  set index 0
  ## Add the modules
  foreach module $modules {
    foreach {m x y label} $module {break}
    #set l [join [split $label " "] "\n"]
    #set l [join [split $l "_"] "\ "]
    set l [string map {\\n \n} $label]
    set tags [list $m ${m}_all index_[incr index]_$system]
    if {[catch {PUL_plugin_module_config $m type} type]} {
      set type module
    }
    if {![string length $type]} {set type module}
    set state [lindex $GUI_GraphInfo(${canvas}$m) 0]
    
    GUI_DrawModule $canvas $m $GUI_Options(ModuleShape) $type $state $x $y \
      $GUI_Options(ModuleWidth) $GUI_Options(ModuleHeight) \
      $GUI_Options(ModuleBorderWidth) $STATE(FILL_$state) \
      $GUI_Options(ModuleBorderColor) $tags \
      $l $GUI_Options(ModuleTextWidth) $GUI_Options(ModuleTextPadX) \
      $GUI_Options(ModuleTextPadY) $STATE(Text_$state) \
      ModuleTextFont $GUI_Options(ModuleTextJustify) \
      $GUI_Options(ModuleTextAnchor) $GUI_Options(ModuleImageAnchor) \
      [list ${m}_text ${m}_all] $STATE(Im_$state) \
      $GUI_Options(ModuleImagePadX) $GUI_Options(ModuleImagePadY) \
      [list ${m}_img ${m}_all]
    $canvas bind ${m}_all <<MB_M>> [list GUI_SystemModulePopUp $canvas $m]

    ## Check for a module Description...
    if {[catch {PUL_plugin_module_config $m description} ToolTip]} {
      set ToolTip {}
    }

    ## Append the Location of the module to the Tooltip...
    if {[info exists ::creole_${m}_home]} {
      set ToolTip "$GUI_Text(ModuleName): $m\n\
                 \n$GUI_Text(ModuleLocation): [set ::creole_${m}_home]\n\
                 \n$ToolTip"
    }
    
    ## Bindings...
    switch -exact -- $type {
      button {
        $canvas._button$m configure -command \
          [list GUI_HandleModule_Button $canvas $m $label]
        bind $canvas._button$m <<Motion_1>> \
          [list GUI_DragModule $canvas $system $m $type $index %X %Y]
        if {[string length $ToolTip]} {
          Tooltip::tooltip $canvas._button$m -text $ToolTip
        }
      }
      combobox {
        $canvas._combo$m.combobox configure -modifycmd \
          [list GUI_HandleModule_ComboBox $canvas $m $label]
        bind $canvas._combo$m.label <<Motion_1>> \
          [list GUI_DragModule $canvas $system $m $type $index %X %Y]
        if {[string length $ToolTip]} {
          Tooltip::tooltip $canvas._combo$m.label -text $ToolTip
        }
      }
      imageset {
        $canvas bind index_${index}_$system <<MBR_1>> \
          [list GUI_HandleModule $canvas $m $label]
        $canvas bind index_${index}_$system <<MB_2>> \
          [list GUI_HandleModule_ImageSet $canvas $m $label $tags]
        $canvas bind index_${index}_$system <<Motion_1>> \
          [list GUI_DragModule $canvas $system $m $type $index %X %Y \
                               index_${index}_$system]
      }
      module -
      default {
        $canvas bind ${m}_all <<MBR_1>> \
          [list GUI_HandleModule $canvas $m $label]
        $canvas bind ${m}_all <<MB_2>> \
          [list GUI_ResetModule $canvas $m $label]
        $canvas bind ${m}_all <<Motion_1>> \
          [list GUI_DragModule $canvas $system $m $type $index %X %Y]
        if {[string length $ToolTip]} {
          Tooltip::canvas_tooltip $canvas ${m}_all $ToolTip
          ## Display the Component location when entering the module...
          $canvas bind ${m}_all <Enter> \
            "+catch \{\
              set ::GUI_StatusBar::StatusBar($win) \$::creole_${m}_home\}"
          $canvas bind ${m}_all <Leave> \
            "+catch \{\
                set ::GUI_StatusBar::StatusBar($win)\
                \$::GUI_StatusBar::StatusBar($win:Def)\}"
        }
      }
    }
  }
  ## Add the arcs
  set CallDeselect 0
  set arcs [lindex $info 5]
  foreach arc $arcs {
    foreach {source dest c_points} $arc {break}
    lappend GUI_GraphInfo(${canvas}${source}) $dest
    set sc  [$canvas coords $source]
    if {[llength $sc] == 2} {
      foreach {x y} $sc {}
      set sx1 [expr {int($x-$GUI_Options(ModuleWidth)/2)}]
      set sy1 [expr {int($y-$GUI_Options(ModuleHeight)/2)}]
      set sx2 [expr {int($x+$GUI_Options(ModuleWidth)/2)}]
      set sy2 [expr {int($y+$GUI_Options(ModuleHeight)/2)}]
    } else {
      foreach {sx1 sy1 sx2 sy2} $sc {break}
    }
    set dc  [$canvas coords $dest]
    if {[llength $dc] == 2} {
      foreach {x y} $dc {break}
      set dx1 [expr {int($x-$GUI_Options(ModuleWidth)/2)}]
      set dy1 [expr {int($y-$GUI_Options(ModuleHeight)/2)}]
      set dx2 [expr {int($x+$GUI_Options(ModuleWidth)/2)}]
      set dy2 [expr {int($y+$GUI_Options(ModuleHeight)/2)}]
    } else {
      foreach {dx1 dy1 dx2 dy2} $dc {break}
    }
    ## puts "$sx1 $sy1 $sx2 $sy2 $dx1 $dy1 $dx2 $dy2"
    ## Find modules centers...
    set scx [expr {($sx2 - $sx1)/2 + $sx1}]
    set scy [expr {($sy2 - $sy1)/2 + $sy1}]
    set dcx [expr {($dx2 - $dx1)/2 + $dx1}]
    set dcy [expr {($dy2 - $dy1)/2 + $dy1}]
    set points [GUI_SpecifyLine [list $scx $scy $c_points $dcx $dcy] \
                $sx1 $sy1 $sx2 $sy2 $dx1 $dy1 $dx2 $dy2]
    
#    set points [join [concat $lx1 $ly1 [join $c_points] $lx2 $ly2]]
#    puts $points
    set tags [list From_${source} To_${dest} Line_$source Line_$dest]
    if {[string equal $ArcFrom $source] && [string equal $ArcTo $dest]} {
      lappend tags current _current_arc_
      set CallDeselect 1
    }
    $canvas lower [GUI_DrawArc $canvas $points \
       $GUI_Options(ArcColor)  $GUI_Options(ArcActiveColor) \
       $GUI_Options(ArcWidth)  $GUI_Options(ArcActiveWidth) \
       $GUI_Options(ArcSmooth) $tags]
  }
  if {$CallDeselect} {GUI_SystemDeselect $canvas $system}
  ## Enlarge a bit...
  #Widget_SetScrollRegion $canvas $GUI_Options(SystemPadX) \
  #                               $GUI_Options(SystemPadY)
  update
  ::ScrolledWidgets::CanvasUpdateScrollRegion $canvas
};# GUI_DrawSystem

## GUI_DragModule
proc GUI_DragModule {canvas system m type index x y {moveTag {}}} {
  global GUI_GraphInfo GUI_Options
  ## Fix mouse coordinates to be relative to the canvas widget...
  set x [expr {$x-[winfo rootx $canvas]}]
  set y [expr {$y-[winfo rooty $canvas]}]

  ## If the ::tk::Priv(Prev?) variables do not exist, define them and return...
  if {![TkGUI_ExistsPriv PrevX]} {
    TkGUI_SetPriv PrevX [$canvas canvasx $x $GUI_GraphInfo(GridX_,$canvas)]
    TkGUI_SetPriv PrevY [$canvas canvasy $y $GUI_GraphInfo(GridY_,$canvas)]
  }
  if {![TkGUI_ExistsPriv x]} {
    TkGUI_SetPriv x [$canvas canvasx $x $GUI_GraphInfo(GridX_,$canvas)]
    TkGUI_SetPriv y [$canvas canvasy $y $GUI_GraphInfo(GridY_,$canvas)]
    return
  }

  ## Has the module been moved since the last click on the canvas?
  if {abs($x-[TkGUI_AccessPriv x]) < 20 && abs($y-[TkGUI_AccessPriv y]) < 20} {
    return
  }
  
  set Xx [$canvas canvasx $x $GUI_GraphInfo(GridX_,$canvas)]
  set Yy [$canvas canvasy $y $GUI_GraphInfo(GridY_,$canvas)]
  
  ## Is a grid drawn? If not, draw one...
  set grid_tags [$canvas gettags _my_grid_tags_]
  if {[llength $grid_tags] == 0} {
    GUI_DrawCanvasGrid $canvas $Xx $Yy \
      [expr {$GUI_Options(ModuleWidth)+2*$GUI_Options(ModuleBorderWidth)}] \
      [expr {$GUI_Options(ModuleHeight)+2*$GUI_Options(ModuleBorderWidth)}] \
      _my_grid_tags_
  }
  
  ## Create the small window displaying the current module coordinates, if it
  ## does not already exist...
  if {![winfo exists $canvas.geometry_display]} {
    frame $canvas.geometry_display -highlightbackground navy \
       -highlightthickness 1 -highlightcolor black -bg #ffffaa
    label $canvas.geometry_display.text -relief flat -bg #ffffaa -fg #666699\
       -padx 2 -pady 0 -anchor w -font {helvetica 12 bold}
    pack  $canvas.geometry_display.text
    place $canvas.geometry_display -relx 0.5 -rely 0.5 -anchor c
  }

  ## Raise the module that is moved so as to be on top of everything else...
  $canvas raise ${m}_all

  catch  {$canvas.geometry_display.text configure -text "($Xx,$Yy)"}
  set Dx [expr {$Xx - [TkGUI_AccessPriv PrevX]}]
  set Dy [expr {$Yy - [TkGUI_AccessPriv PrevY]}]
  TkGUI_SetPriv PrevX $Xx;        TkGUI_SetPriv PrevY $Yy
  TkGUI_SetPriv x [expr {$x-25}]; TkGUI_SetPriv y [expr {$y+25}]
  switch -exact -- $type {
    button {
      $canvas move ${m}_all $Dx $Dy
      foreach {X Y} [$canvas coords $m] break
      bind $canvas._button$m <<Release_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
      bind $canvas._button$m <<CRelease_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
    }
    combobox {
      $canvas move ${m}_all $Dx $Dy
      foreach {X Y} [$canvas coords $m] break
      bind $canvas._combo$m.label <<Release_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
      bind $canvas._combo$m.label <<CRelease_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
    } 
    imageset {
      $canvas move $moveTag $Dx $Dy
      foreach {X Y} [$canvas coords $moveTag] {break}
      $canvas bind $moveTag <<Release_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y $moveTag;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
      $canvas bind $moveTag <<CRelease_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y $moveTag;\
         TkGUI_UnsetPriv PrevX; TkGUI_UnsetPriv x; break"
    }
    module -
    default {
      $canvas move ${m}_all $Dx $Dy
      foreach {x1 y1 x2 y2} [$canvas coords $m] break
      set X [expr {$x1+int($x2-$x1)/2}]
      set Y [expr {$y1+int($y2-$y1)/2}]
      $canvas bind ${m}_all <<Release_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; break"
      $canvas bind ${m}_all <<CRelease_1>> \
        "GUI_MoveModule \"$system\" $canvas $m $index $X $Y;\
         TkGUI_UnsetPriv PrevX; break"
    }
  };# switch -exact -- $type
};# GUI_DragModule

## GUI_DrawCanvasGrid
#   Draws a canvas grid. Useful when dragging a Component, in order to place
#   it correctly... Petasis, 21/07/2001
proc GUI_DrawCanvasGrid {canvas x y dx dy tag} {
  ## Get the bounding box of all items. We don't want our grid to be larger
  ## than what needed...
  foreach {bx1 by1 bx2 by2} [$canvas bbox all] {}
  set bx1 [$canvas canvasx [expr {$bx1-4*$dx}] $dx]
  set by1 [$canvas canvasy [expr {$by1-4*$dy}] $dy]
  set bx2 [$canvas canvasx [expr {$bx2+10*$dx}] $dx]
  set by2 [$canvas canvasy [expr {$by2+15*$dy}] $dy]

  set l1_tags [list $tag l1]
  set l2_tags [list $tag l2]
  set l3_tags [list $tag l3]

  for {set x $bx1} {$x < $bx2} {set x [expr {$x+$dx}]} {
    set x1 [expr {$x+  $dx/4.0}]
    set x2 [expr {$x+  $dx/2.0}]
    set x3 [expr {$x+3*$dx/4.0}]
    set x4 [expr {$x+  $dx/8.0}]
    set x5 [expr {$x+3*$dx/8.0}]
    set x6 [expr {$x+5*$dx/8.0}]
    set x7 [expr {$x+7*$dx/8.0}]
    $canvas create line $x  $by1 $x  $by2 -fill black   -tags $l1_tags -width 2
    $canvas create line $x1 $by1 $x1 $by2 -fill #4682b4 -tags $l2_tags -width 1
    $canvas create line $x2 $by1 $x2 $by2 -fill black   -tags $l1_tags -width 1
    $canvas create line $x3 $by1 $x3 $by2 -fill #4682b4 -tags $l2_tags -width 1
    ## Very fine devisions...
    $canvas create line $x4 $by1 $x4 $by2 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $x5 $by1 $x5 $by2 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $x6 $by1 $x6 $by2 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $x7 $by1 $x7 $by2 -fill #228b22 -tags $l3_tags -width 1
  }
  $canvas create line $bx2 $by1 $bx2 $by2 -fill black   -tags $l1_tags -width 2
  for {set y $by1} {$y < $by2} {set y [expr {$y+$dy}]} {
    set y1 [expr {$y+  $dy/4.0}]
    set y2 [expr {$y+  $dy/2.0}]
    set y3 [expr {$y+3*$dy/4.0}]
    set y4 [expr {$y+  $dy/8.0}]
    set y5 [expr {$y+3*$dy/8.0}]
    set y6 [expr {$y+5*$dy/8.0}]
    set y7 [expr {$y+7*$dy/8.0}]
    $canvas create line $bx1 $y  $bx2 $y  -fill black   -tags $l1_tags -width 2
    $canvas create line $bx1 $y1 $bx2 $y1 -fill #4682b4 -tags $l2_tags -width 1
    $canvas create line $bx1 $y2 $bx2 $y2 -fill black   -tags $l1_tags -width 1
    $canvas create line $bx1 $y3 $bx2 $y3 -fill #4682b4 -tags $l2_tags -width 1
    ## Very fine devisions...
    $canvas create line $bx1 $y4 $bx2 $y4 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $bx1 $y5 $bx2 $y5 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $bx1 $y6 $bx2 $y6 -fill #228b22 -tags $l3_tags -width 1
    $canvas create line $bx1 $y7 $bx2 $y7 -fill #228b22 -tags $l3_tags -width 1
  }
  $canvas create line $bx1 $by2 $bx2 $by2 -fill black   -tags $l1_tags -width 2
  
  $canvas lower $tag
  $canvas raise l2 l3
  $canvas raise l1 l2
};# GUI_DrawCanvasGrid

## GUI_MoveModule
#   Moves a module in a system graph.
#   Arguments: the system name, the canvas object, the module name or tag and 
#   the new x and y coordinates
proc GUI_MoveModule {system canvas module index x y {moveTag {}}} {
  global GUI_Graphs GUI_Options GUI_SystemsModified
  set GUI_SystemsModified 1
  set info $GUI_Graphs($system)
  set modules [lindex $info 4]
  
  ## Change the Module position in System Graph
  set counter 0
  foreach mod $modules {
    set m [lindex $mod 0]
    if {[incr counter] == $index} {
      set label [lindex $mod 3]
      lappend NewModuleSpec [list $module [expr {int($x)}] [expr {int($y)}] \
                                  $label]
    } else {
      lappend NewModuleSpec $mod
    }
  }
  set info [lreplace $info 1 1 [winfo geometry [winfo toplevel $canvas]]]
  set GUI_Graphs($system) [lreplace $info 4 4 $NewModuleSpec]
  
  ## Redraw the System Graph
  ## GUI_DrawSystem $canvas $system
  if {[catch {PUL_plugin_module_config $module type} type]} {
    set type module
  }
  if {![string length $type]} {set type module}
  switch -exact -- $type {
    button {
      bind $canvas._button$module <<Release_1>>  {}
      bind $canvas._button$module <<CRelease_1>> {}
    }
    combobox {
      bind $canvas._combo${module}.label <<Release_1>>  {}
      bind $canvas._combo${module}.label <<CRelease_1>> {}
    }
    imageset {
      $canvas bind $moveTag <<Release_1>>  {}
      $canvas bind $moveTag <<CRelease_1>> {}
    }
    default {
      $canvas bind ${module}_all <<Release_1>>  {}
      $canvas bind ${module}_all <<CRelease_1>> {}
    }
  }
  catch {destroy $canvas.geometry_display}
  catch {$canvas delete _my_grid_tags_}

  ## Get all lines that start or end in the moved module
  set Lines [$canvas find withtag Line_$module]
  foreach line $Lines {
    set tags [$canvas gettags $line]
    foreach tag $tags {
      if {[string match From_* $tag]} {set source [string range $tag 5 end]}
      if {[string match To_*   $tag]} {set dest   [string range $tag 3 end]}
    }
    set sc  [$canvas coords $source]
    if {[llength $sc] == 2} {
      foreach {x y} $sc {break}
      set sx1 [expr {int($x-$GUI_Options(ModuleWidth)/2)}]
      set sy1 [expr {int($y-$GUI_Options(ModuleHeight)/2)}]
      set sx2 [expr {int($x+$GUI_Options(ModuleWidth)/2)}]
      set sy2 [expr {int($y+$GUI_Options(ModuleHeight)/2)}]
    } else {
      foreach {sx1 sy1 sx2 sy2} $sc {break}
    }
    set dc  [$canvas coords $dest]
    if {[llength $dc] == 2} {
      foreach {x y} $dc {break}
      set dx1 [expr {int($x-$GUI_Options(ModuleWidth)/2)}]
      set dy1 [expr {int($y-$GUI_Options(ModuleHeight)/2)}]
      set dx2 [expr {int($x+$GUI_Options(ModuleWidth)/2)}]
      set dy2 [expr {int($y+$GUI_Options(ModuleHeight)/2)}]
    } else {
      foreach {dx1 dy1 dx2 dy2} $dc {break}
    }
    ## Find modules centers...
    set scx [expr {($sx2 - $sx1)/2 + $sx1}]
    set scy [expr {($sy2 - $sy1)/2 + $sy1}]
    set dcx [expr {($dx2 - $dx1)/2 + $dx1}]
    set dcy [expr {($dy2 - $dy1)/2 + $dy1}]
    set line_coords [$canvas coords $line]
    set llength [llength $line_coords]
    if {$llength > 4} {
      set end_3 [expr {$llength -3}]
      set c_points [list [lrange $line_coords 2 $end_3]]
    } else {
      set c_points {}
    }
    catch {
    set points [GUI_SpecifyLine [list $scx $scy $c_points $dcx $dcy] \
                $sx1 $sy1 $sx2 $sy2 $dx1 $dy1 $dx2 $dy2]
    
    $canvas lower [eval $canvas coords $line $points]
    }
  }
  ## Enlarge a bit...
  #Widget_SetScrollRegion $canvas $GUI_Options(SystemPadX) \
  #                               $GUI_Options(SystemPadY)
  update
  ::ScrolledWidgets::CanvasUpdateScrollRegion $canvas
};# GUI_MoveModule

## GUI_SpecifyLine 
#
proc GUI_SpecifyLine {points sx1 sy1 sx2 sy2 dx1 dy1 dx2 dy2} {
  foreach {x1 y1 point_list x2 y2} $points {break}
  set last [expr {[llength $point_list] -1}]
  if {$last >= 0} {
    set p  [lindex $point_list 0]
    foreach {x2 y2} $p {break}
  }

  ## If I have a vertical line, its easy...
  if {$x1 == $x2} {
    set lx1 $x1
    set lx2 $x1
    if {$y2 > $sy2} {set ly1 $sy2} else {set ly1 $sy1}
    if {$y1 > $dy2} {set ly2 $dy2} else {set ly2 $dy1}
    return [join [concat $lx1 $ly1 [join $point_list] $lx2 $ly2]]
  }
  
  ## Find start point
  catch {set a [expr {($x1*$y2 - $x2*$y1) / double($x1-$x2)}]}
  catch {set b [expr {($y1-$y2) / double($x1-$x2)}]}
  if {$x2 > $sx2} {set exit_x $sx2} else {set exit_x $sx1}
  if {$y2 > $sy2} {set exit_y $sy2} else {set exit_y $sy1}
  set lx1 $exit_x
  set ly1 [expr {int($a+$b*$lx1)}]
  if {$ly1 < $sy1 || $ly1 > $sy2} {
    set ly1 $exit_y
    set lx1 [expr {int(($ly1-$a)/$b)}]
  }

  if {$last >= 0} {
    set x2 [lindex $points 3]
    set y2 [lindex $points 4]
    set p  [lindex $point_list $last]
    foreach {x1 y1} $p {break}
  }
  ## Find end point
  catch {set a [expr {($x1*$y2 - $x2*$y1) / double($x1-$x2)}]}
  catch {set b [expr {($y1-$y2) / double($x1-$x2)}]}
  if {$x1 > $dx2} {set enter_x $dx2} else {set enter_x $dx1}
  if {$y1 > $dy2} {set enter_y $dy2} else {set enter_y $dy1}
  set lx2 $enter_x
  set ly2 [expr {int($a+$b*$lx2)}]
  if {$ly2 < $dy1 || $ly2 > $dy2} {
    set ly2 $enter_y
    set lx2 [expr {int(($ly2-$a)/$b)}]
  }
  return [join [concat $lx1 $ly1 [join $point_list] $lx2 $ly2]]
};# GUI_SpecifyLine

## GUI_ResetModule
#
proc GUI_ResetModule {canvas module label {silent {}}} {
  global GUI_GraphInfo GUI_Text creole_config
  
  set state [lindex $GUI_GraphInfo(${canvas}${module}) 0]
  if {$state < 4} {
     switch -exact $state {
       0 {return {}}
       1 {if {[string equal $silent _FollowModule_]} {
            set state 0
          }
          set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
          set GUI_GraphInfo(${canvas}${module}) [concat $state $next_modules]
          GUI_Draw_Module_Status $canvas $module {}
          foreach mod $next_modules {
            GUI_ResetModule $canvas $mod $label _FollowModule_
          }
          catch {
          set modules $GUI_GraphInfo(RunOrder_${canvas})
          set i [lsearch -exact $modules $module]
          if {$i != -1} {
            set GUI_GraphInfo(RunOrder_${canvas}) [lreplace $modules $i $i]
          }
          }
          return {}
         }
       2 {if {[string equal $silent _FollowModule_]} {
            set state 0
          } else {
            set state 1
          }
          set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
          set GUI_GraphInfo(${canvas}${module}) [concat $state $next_modules]
          GUI_Draw_Module_Status $canvas $module {}
          foreach mod $next_modules {
            GUI_ResetModule $canvas $mod {} _FollowModule_
          }
          catch {
          set modules $GUI_GraphInfo(RunOrder_${canvas})
          set i [lsearch -exact $modules $module]
          if {$i != -1} {
            set GUI_GraphInfo(RunOrder_${canvas}) [lreplace $modules $i $i]
          }
          }
          return {}
         }
       3 {if {[string equal $silent _FollowModule_]} {
            GUI_ResetModule $canvas $module $label {}
            return {}
          }
         }
       default {return {}}
     }
  }

  ## If $silent == {}, display a confirmation dialog...
  if {[string equal $silent {}]} {
    set label [join [split $label _] { }]
    if {[string equal $label {}]} {set label $module}
    set message "$GUI_Text(ResetModule) ${label}$GUI_Text(?)"
    set ans [gut_question $message {Reset Module ?} 1 [winfo toplevel $canvas]]
    set GUI_GraphInfo(CollectionChanged_$canvas) 1
  } else {
    set ans yes
  }
  
  if {[string equal $ans yes]} {
    ## gut_Debug "Resetting module $module..."
    if {[string equal $silent {}]} {
      set state 1
    } else {
      set state [expr {$silent - 1}]
      if {$state < 0} {set state 0}
    }
    set col $GUI_GraphInfo(CCollection_${canvas})
    set dc  $GUI_GraphInfo(CDocument_${canvas})
    set autosave $GUI_GraphInfo(AutoSave_${canvas})
    set enc [tip_GetEncoding $col]
    if {[string equal $dc {}]} {
      set doc [tip_FirstDocument $col]
      set wwin [ggi_wait_message $GUI_Text(PleaseWait)]
      set progress 0.0
      set step [expr {100.0/([tip_Length $col]+1)}]
      if {!$step} {incr step}
      ggi_wait_update {} 0 $wwin
    } else {
      set doc $dc
    } 

    set info $creole_config($module)
    set post [lindex $info [expr {[lsearch $info post_conditions] +1}]]
    set post_anns [lindex $post [expr {[lsearch $post annotations] +1}]]
    set post_anns [lrange $post_anns 0 end]
    while {[string length $doc]} {
      ## Remove the annotations from CDM...
      foreach ann $post_anns {
        tip_DeleteAnnotations $doc $ann {}
      }
      catch {tip_RemoveAttribute $doc $module}
      if {![string length $dc]} {
        if {$autosave} {
          tip_Sync $doc
          tip_Close $doc
          CDM_ClearUnusedSavedValues
        }
        set progress [expr {$progress+$step}]
        ggi_wait_update {} $progress $wwin
        set doc [tip_NextDocument $col]
      } else {
        set doc {}
      }
    }
    if {![string length $dc]} {ggi_destroy_wait $wwin}
    catch {tip_RemoveAttribute $col $module}

    set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
    set GUI_GraphInfo(${canvas}${module}) [concat $state $next_modules]
    GUI_Draw_Module_Status $canvas $module {}
    foreach mod $next_modules {
      GUI_ResetModule $canvas $mod {} $state
    }
    catch {
      set modules $GUI_GraphInfo(RunOrder_${canvas})
      set i [lsearch -exact $modules $module]
      if {$i != -1} {
        set GUI_GraphInfo(RunOrder_${canvas}) [lreplace $modules $i $i]
      }
    }
  }
};# GUI_ResetModule

## GUI_HandleModule_Button
#  This function handles user requests (mouse clicks) on system buttons
proc GUI_HandleModule_Button {canvas module {title {}}} {
  global GUI_GraphInfo GUI_Text GUI_Options GUI_SystemAddDepend GUI_Progress \
         CDM_CurrentStatus GUI_SystemWindow GUI_SystemErrorLog

  if {[info exists CDM_CurrentStatus(NowAnnotating)]} {
    if {$CDM_CurrentStatus(NowAnnotating) == 1} {return {}}
  }
  set col {}; set doc {}
  if {[info exists GUI_GraphInfo(CCollection_${canvas})]} {
    set col $GUI_GraphInfo(CCollection_${canvas})
  }
  if {[info exists GUI_GraphInfo(CDocument_${canvas})]} {
    set doc $GUI_GraphInfo(CDocument_${canvas})
  }

  creole_${module}_Initialize $col $doc \
    [lindex $GUI_GraphInfo(MOptions_${canvas}$module) 0]
};# GUI_HandleModule_Button

## GUI_HandleModule_ComboBox
#  This function handles user requests (mouse clicks) on system combo boxes
proc GUI_HandleModule_ComboBox {canvas module {title {}}} {
  global GUI_GraphInfo GUI_Text GUI_Options GUI_SystemAddDepend GUI_Progress \
         CDM_CurrentStatus GUI_SystemWindow GUI_SystemErrorLog

  if {[info exists CDM_CurrentStatus(NowAnnotating)]} {
    if {$CDM_CurrentStatus(NowAnnotating) == 1} {return {}}
  }
  set col {}; set doc {}
  if {[info exists GUI_GraphInfo(CCollection_${canvas})]} {
    set col $GUI_GraphInfo(CCollection_${canvas})
  }
  if {[info exists GUI_GraphInfo(CDocument_${canvas})]} {
    set doc $GUI_GraphInfo(CDocument_${canvas})
  }

  creole_${module} $doc $col \
    $canvas._combo$module.combobox $canvas._combo$module.label $canvas \
    [lindex $GUI_GraphInfo(MOptions_${canvas}$module) 0]
};# GUI_HandleModule_ComboBox

## GUI_HandleModule_ImageSet
#  This function handles user requests (mouse clicks) on system imageset
#  modules...
proc GUI_HandleModule_ImageSet {canvas module {title {}} {tags {}}} {
  global GUI_GraphInfo GUI_Text GUI_Options GUI_SystemAddDepend GUI_Progress \
         CDM_CurrentStatus GUI_SystemWindow GUI_SystemErrorLog

  if {[info exists CDM_CurrentStatus(NowAnnotating)]} {
    if {$CDM_CurrentStatus(NowAnnotating) == 1} {return {}}
  }
  set col {}; set doc {}
  if {[info exists GUI_GraphInfo(CCollection_${canvas})]} {
    set col $GUI_GraphInfo(CCollection_${canvas})
  }
  if {[info exists GUI_GraphInfo(CDocument_${canvas})]} {
    set doc $GUI_GraphInfo(CDocument_${canvas})
  }

  ## Retrieve the list of available images...
  set images [creole_${module}_Finish $doc $col $canvas \
                    [lindex $GUI_GraphInfo(MOptions_${canvas}$module) 0]]
  if {[llength $images]} {
    set menu $canvas.viewers_menu
    catch {destroy $menu}
    menu $menu -tearoff false
    $menu delete 0 end
    foreach image $images {
      ## The module must find a way to keep its state among platform
      ## invocations...
      $menu add command -label $image -command \
        [list creole_${module} $doc $col $canvas $tags $image]
    }
    tk_popup $menu [winfo pointerx .] [winfo pointery .]
    catch {GUI_Draw_Module_Status $canvas $module $title}
  }
};# GUI_HandleModule_ImageSet

## GUI_HandleModule
#  This function handles user requests (mouse clicks) on system components
proc GUI_HandleModule {canvas module {title {}}} {
  global GUI_GraphInfo GUI_Text GUI_Options GUI_SystemAddDepend GUI_Progress \
         CDM_CurrentStatus GUI_SystemWindow GUI_SystemErrorLog

  if {[info exists CDM_CurrentStatus(NowAnnotating)]} {
    if {$CDM_CurrentStatus(NowAnnotating) == 1} {return {}}
  }

  if {[catch {PUL_plugin_module_config $module type} type]} {
    set type module
  }
  if {![string length $type]} {set type module}

  set state [lindex $GUI_GraphInfo(${canvas}${module}) 0]
  set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
  set selected [$canvas find withtag _current_]
  catch {$canvas dtag _current_ _current_}
  foreach mod $selected {
    ## Try to recover the _module name from the canvas tag...
    set _module [lindex [$canvas gettags $mod] 0]
    if {[catch {PUL_plugin_module_config $_module type} _type]} {
      set _type module
    }
    if {![string length $_type]} {set _type module}
    switch -exact -- $_type {
      button   {}
      combobox {}
      imageset {}
      module   {
        $canvas itemconfigure $mod -width $GUI_Options(ModuleBorderWidth) \
                -outline $GUI_Options(ModuleBorderColor)
      }
      default  {}
    }
  }
  $canvas addtag _current_ withtag $module
  switch -exact -- $type {
    button   {}
    combobox {}
    imageset {}
    module   {
      $canvas itemconfigure $module \
        -width $GUI_Options(ModuleBorderWidthSelected) \
        -outline $GUI_Options(ModuleBorderColorSelected)
    }
    default  {}
  }
  if [info exists GUI_SystemAddDepend(Bind,$canvas)] {return {}}
 
  ## Enable the modules toolbar...
  set parent_win [winfo toplevel $canvas]
  catch {Toolbar::activate $parent_win.tf.module}

  ## If the Module is selected
  switch -exact -- $type {
    button   {}
    combobox {}
    imageset {}
    module   {
      switch -exact -- $state {
        1 {
          incr state
          set GUI_GraphInfo(${canvas}${module}) [concat $state $next_modules]
          foreach mod $next_modules {
            set mst [lindex $GUI_GraphInfo(${canvas}${mod}) 0]
            if {$mst == 0} {
              set n_modules [lrange $GUI_GraphInfo(${canvas}${mod}) 1 end]
              set GUI_GraphInfo(${canvas}${mod}) [concat 1 $n_modules]
            }
          }
          lappend GUI_GraphInfo(RunOrder_${canvas}) $module
        }
        2 {
          ## The module is ready to run...
          set CDM_CurrentStatus(TimeLog) {}
          set GUI_SystemErrorLog($canvas) {}
          set run_modules $GUI_GraphInfo(RunOrder_${canvas})
          set col $GUI_GraphInfo(CCollection_${canvas})
          set dc  $GUI_GraphInfo(CDocument_${canvas})
          set autosave $GUI_GraphInfo(AutoSave_${canvas})
          set enc [tip_GetEncoding $col]
          set win [winfo toplevel $canvas]
          set wm_win $win.ggi_wait_message_win
          set TimeDisplay $canvas.timeDisplay
          catch {after cancel $GUI_SystemWindow($win:TimeElapsed_ID)}
          catch {frame $TimeDisplay -bg #ffffaa -highlightbackground black \
             -highlightthickness 1 -highlightcolor black
                 label $TimeDisplay.img -relief flat -bg #ffffaa -image ICpu
                 label $TimeDisplay.l -relief flat -bg #ffffaa -fg black \
                    -padx 2 -pady 0 -anchor w -text 00:00:00
                 pack $TimeDisplay.img $TimeDisplay.l -side left -fill y
                 place $TimeDisplay -in $canvas -x 2 -rely 1.0 -y \
                    -[expr {[image height ICpu] + 6}]}
          set StartTime [clock seconds]
          catch {ggi_wait_message \
              "$GUI_Text(ProcessingDocument):" 0 1 $wm_win}
          if {[string equal $dc {}]} {
            set doc [tip_FirstDocument $col]
          } else {
            set doc $dc
          }
          ##
          ## Process options, to remove empty elements...
          ##
          foreach mod $run_modules {
            set options($mod) {}
            foreach one [lindex $GUI_GraphInfo(MOptions_${canvas}$mod) 0] {
              if {$one ne ""} {lappend options($mod) $one}
            }
            puts "Module: $mod, options: \"[join $options($mod) \",\ \"]\""
          }

          foreach mod $run_modules {
            incr GUI_SystemWindow($win:TimeElapsed) \
               [expr {[clock seconds] - $StartTime}]
            catch {$TimeDisplay.l configure -text [clock format \
               $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
            set StartTime [clock seconds]
            if {[llength [info commands creole_${mod}_Initialize]]} {
              if {[catch {creole_${mod}_Initialize $col $doc \
                               {*}$options($mod)} InitError]} {
               bgerror [format $GUI_Text(ModuleInitialize) $mod $mod $InitError]
                return
              }
            }
          }
          incr GUI_SystemWindow($win:TimeElapsed) \
             [expr {[clock seconds] - $StartTime}]
          catch {$TimeDisplay.l configure -text [clock format \
             $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
          set StartTime [clock seconds]
          
          set CDM_CurrentStatus(CurrentCollectionEncoding) $enc
    
          set progress 0.0
          set doc_number [tip_Length $col]
          if {!$doc_number} {set doc_number 1}
          set step [expr {100.0/$doc_number}]
    
          ## Display a status dialog showing the overal Collection status...
          while {[string length $doc]} {
            catch {ggi_wait_message \
              "$GUI_Text(ProcessingDocument): [tip_GetId $doc]" \
              $progress 1 $wm_win}
            set progress [expr {$progress+$step}]
            ## The following was a bug: We reseted the following value for
            ## each new module instead of Document...
            set allow_save 1;# Bug reported by Vangelis, 25/01/2001
            ## Reset the modules states to the initial one for each new
            ## document...
            foreach mod $run_modules {
              ## Check if the annotator has already run for this module:
              ## Reset only these modules not already run (petasis, 26/06/2000)
              if {![llength [tip_GetAttribute $doc $mod]]} {
                set nmods [lrange $GUI_GraphInfo(${canvas}${mod}) 1 end]
                set GUI_GraphInfo(${canvas}${mod}) [concat 2 $nmods]
                GUI_Draw_Module_Status $canvas $mod $title
              }
            }
            
            ## Run the Selected Annotators
            foreach mod $run_modules {
              ## Check if the annotator has already run for this module...
              ## Was BUG: Petasis 03/02/2000...
              if {[llength [tip_GetAttribute $doc $mod]]} {
                set ann_result {Annotate Ok}
                continue
              }
              ## gut_Debug "Running Module $mod..."
              catch {ggi_wait_append "$GUI_Text(PleaseWait)\n\
                        $GUI_Text(RunningModule) $mod $GUI_Text(OnDocument)\n\
                        [tip_GetId $doc]\n" -1.0}
              set GUI_Progress(WindowID:$doc) $wm_win
    
              incr GUI_SystemWindow($win:TimeElapsed) \
                 [expr {[clock seconds] - $StartTime}]
              catch {$TimeDisplay.l configure -text [clock format \
                 $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
              set StartTime [clock seconds]
              ## Run the annotator
              set ann_result [tip_Annotate $doc creole_$mod {*}$options($mod)]
              catch {ggi_wait_update {} 100 $wm_win}
              catch {ggi_destroy_append $wm_win}
    
              ## Handle Errors
              if {![string equal $ann_result {Annotate Ok}]} {
                ## Is Error Logging/Ignoring enabled? If so, keep the error and
                ## continue...
                if {$GUI_GraphInfo(IgnoreErrors_$canvas)} {
                  append GUI_SystemErrorLog($canvas) \
                   "$GUI_Text(ErrorWhileExecutingModule:) \"$module\"\
                    \n$GUI_Text(OnDocument:) \"[$doc GetId]\":\
                    [lindex $ann_result 1]\
                    \nError Log:\n[lindex $ann_result 2]\n\n"
                  ## Was the error due to an abort request?
                  if {[string match "CDM_*: Aborting..." $ann_result]} {
                    break
                  }
                  set ann_result {Annotate Ok}
                  set allow_save 0;# Do not save this Document...
                  if {![winfo exists $canvas.errorLog]} {
                    GUI_LoadImage IWarning
                    label $canvas.errorLog -image IWarning
                    place $canvas.errorLog -in $canvas -relx 1.0 -rely 1.0 \
                      -x -[expr {[image width  IWarning] + 6}] \
                      -y -[expr {[image height IWarning] + 6}]
                    bind $canvas.errorLog <<MB_1>> \
                      {GUI_ColView ErrorLog [winfo toplevel %W]}
                  }
                } else {
                  break
                }
              }
              set nmods [lrange $GUI_GraphInfo(${canvas}${mod}) 1 end]
              set GUI_GraphInfo(${canvas}${mod}) [concat 3 $nmods]
              GUI_Draw_Module_Status $canvas $mod $title
              set GUI_GraphInfo(CollectionChanged_${canvas}) 1
              incr GUI_SystemWindow($win:TimeElapsed) \
                 [expr {[clock seconds] - $StartTime}]
              catch {$TimeDisplay.l configure -text [clock format \
                 $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
              set StartTime [clock seconds]
              if {[string length $dc]} {
                if {$autosave && $allow_save} {
                  tip_Sync $doc
                  update
                  incr GUI_SystemWindow($win:TimeElapsed) \
                     [expr {[clock seconds] - $StartTime}]
                  catch {$TimeDisplay.l configure -text [clock format \
                     $GUI_SystemWindow($win:TimeElapsed) \
                     -format %H:%M:%S -gmt 1]}
                  set StartTime [clock seconds]
                }
              }
            }
            if {![string equal $ann_result {Annotate Ok}]} {break}
            if {![string length $dc]} {
              
              ## If Autosave Collection is enabled, save the document and free
              ## occupied memory
              if {$autosave} {
                if {$allow_save} {tip_Sync $doc}
                tip_Close $doc
                CDM_ClearUnusedSavedValues
              }
              ## Petasis, 05/02/2001: If "Ignore Changes" is activated,
              ## reject all changes and close Document.
              ## We save memory if changes are not important...
              if {$GUI_GraphInfo(IgnoreChanges_$canvas)} {
                catch {tip_Close $doc}
                CDM_ClearUnusedSavedValues
              }
              update
              set doc [tip_NextDocument $col]
            } else {
              set doc {}
            }
            incr GUI_SystemWindow($win:TimeElapsed) \
               [expr {[clock seconds] - $StartTime}]
            catch {$TimeDisplay.l configure -text [clock format \
               $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
            set StartTime [clock seconds]
          }
          #puts "Removed [CDM_ClearCompressedMemoryCache] objects..."
          CDM_ClearCompressedMemoryCache
          if {[string equal $ann_result {Annotate Ok}]} {
            foreach mod $run_modules {
              if {[string length [info commands creole_${mod}_Finish]]} {
                if {[catch {creole_${mod}_Finish $col $doc \
                               {*}$options($mod)} FinError]} {
                  error [format $GUI_Text(ModuleFinalize) $mod $mod $FinError]
                }
              }
              incr GUI_SystemWindow($win:TimeElapsed) \
                 [expr {[clock seconds] - $StartTime}]
              catch {$TimeDisplay.l configure -text [clock format \
                 $GUI_SystemWindow($win:TimeElapsed) -format %H:%M:%S -gmt 1]}
              set StartTime [clock seconds]
            }
          }
          catch {ggi_destroy_wait $wm_win}; # Destroy Append
          catch {ggi_destroy_wait $wm_win}; # Destroy the Actual dialog
          set GUI_SystemWindow($win:TimeElapsed_ID) \
              [after 10000 "catch \"destroy $canvas.timeDisplay\""]
    
          catch {unset GUI_GraphInfo(RunOrder_${canvas})}
          append GUI_GraphInfo(TimeLog_${canvas}) $CDM_CurrentStatus(TimeLog)
    
          ## Handle Errors!
          if {![string equal $ann_result {Annotate Ok}]} {
            Agent::Agent stop
            set GUI_GraphInfo(${canvas}${module}) [concat 1 $next_modules]
            GUI_ResetModule $canvas $module {}
            GUI_Draw_Module_Status $canvas $module $title
            error "\n$GUI_Text(ErrorWhileExecutingModule:) $module\
                   \n$GUI_Text(OnDocument:) [tip_GetId $doc] \
                   \n\n[lindex $ann_result 1]" "[lindex $ann_result 2]"
          }
          
          set GUI_GraphInfo(${canvas}${module}) [concat 3 $next_modules]
        }
        3 {
          ## The module has run. Show viewers window
          set col $GUI_GraphInfo(CCollection_${canvas})
          set doc $GUI_GraphInfo(CDocument_${canvas})
          set viewers [PUL_plugin_module_config $module viewers]
          set lviewers [llength $viewers]
          if {$lviewers > 0} {
            ## Check whether user has opened a whole collection...
            if {![string length $doc]} {
              set DocIds [$col GetFileIds]
              ## If collection has only one document, no need to prompt the user
              ## to select it :-)
              if {[llength $DocIds] > 1} {
                if {![info exists GUI_Options(CannotViewCollectionReported)]} {
                  gut_ErrorDlg $GUI_Text(CannotViewCollection) \
                               [winfo toplevel $canvas]
                  set GUI_Options(CannotViewCollectionReported) 1
                }
                set document [gut_MultipleSelectFromList \
                                  $DocIds \
                                  $GUI_Text(TSelectDocument) \
                                  {Select Document to View...}]
                if {![llength $document]} {return}
                set doc [$col GetDocument [lindex $document 0]]
              } else {
                set doc [tip_FirstDocument $col]
              }
            }
            set enc [tip_GetEncoding $col]
            set CDM_CurrentStatus(CurrentCollectionEncoding) $enc
            set menu $canvas.viewers_menu
            catch {destroy $menu}
            menu $menu -tearoff false
            $menu delete 0 end
            ## Do not forget to register as global the location of the
            ## component...
            global creole_${module}_home
            for {set i 0} {$i < $lviewers} {incr i 3} {
              set what [lindex $viewers $i]
              regsub -all {%D%} $what [tip_GetId $doc] what
              ## As the entry may contain variable references, ensure that the
              ## proper substitutions occur...
              catch {set what [subst $what]}
              set viewer   [lindex $viewers [expr {$i+1}]]
              switch -- $viewer {
                single_span            {set image ITShowTags}
                raw                    {set image ITViewAnns}
                text_file              -
                document               {set image ITViewText}
                AnnotationExplorer     {set image ITExplore}
                HTMLAnnotationExplorer {set image ITExplore}
                Attributes             {set image ITViewDocAtt}
                parse                  {set image ITViewText}
                morphology             {set image ITShowTags}
                VideoFile              {set image ITViewText}
                default                {
                  set image [CDM_GetRegisteredViewerImage \
                                [CDM_GetRegisteredViewerLabel ggi_view_$viewer]]
                }
              }
              set text     [lindex $viewers [expr {$i+2}]]
              set title "\"Viewer: $viewer - Document: [tip_GetId $doc], From\
                  Collection: [file tail [tip_GetName $col]]\""
              if {[llength $what] == 2} {
                $menu add command -label $text -command \
                      [list ggi_view_$viewer $doc\
                         [lindex $what 0] [lindex $what 1] $title] \
                         -image $image -compound $GUI_Options(MenuImageLocation)
              } else {
                $menu add command -label $text -command \
                      [list ggi_view_$viewer $doc\
                         [lindex $what 0] {} $title] \
                         -image $image -compound $GUI_Options(MenuImageLocation)
              }
            }
            if {[string equal [$menu index end] none]} {
              destroy $menu
            } else {
              if {$GUI_Options(MenuImageLocation) == "none"} {
                ::GUI::RemoveAllImagesFromMenu $menu}
              tk_popup $menu [winfo pointerx .] [winfo pointery .]
            }
          };# if {$lviewers > 0}
        }
      };# switch -exact -- $state
    }
    default  {}
  };# switch -exact -- $type
  catch {GUI_Draw_Module_Status $canvas $module $title}
};# GUI_HandleModule

## GUI_Draw_Module_Status
#
proc GUI_Draw_Module_Status {canvas module title} {
  global GUI_Draw_Module_Status_Drawn
  catch {unset GUI_Draw_Module_Status_Drawn}
  set res [GUI_Draw_Module_Status_Recursive $canvas $module $title]
  catch {unset GUI_Draw_Module_Status_Drawn}
  return $res
};# GUI_Draw_Module_Status

## GUI_Draw_Module_Status_Recursive
#   GUI_Draw_Module_Status was splitted, in order to avoid loops in graph
#   cycles... (Petasis, 22/1/1999)
proc GUI_Draw_Module_Status_Recursive {canvas module title} {
  global GUI_GraphInfo GUI_Options GUI_Draw_Module_Status_Drawn

  set state [lindex $GUI_GraphInfo(${canvas}${module}) 0]

  if {[catch {PUL_plugin_module_config $module type} type]} {
    set type module
  }
  if {![string length $type]} {set type module}

  switch -exact -- $type {
    button {
      # switch $state {
      #   1 {$canvas._button$module configure -bg $GUI_Options(ModuleEnabled) \
      #       -activebackground $GUI_Options(ModuleEnabled) \
      #       -fg $GUI_Options(ModuleTextEnabled) \
      #       -activeforeground $GUI_Options(ModuleTextEnabled)}
      #   2 {$canvas._button$module configure -bg $GUI_Options(ModuleActive) \
      #       -activebackground $GUI_Options(ModuleActive) \
      #       -fg $GUI_Options(ModuleTextActive) \
      #       -activeforeground $GUI_Options(ModuleTextActive)}
      #   3 {$canvas._button$module configure -bg $GUI_Options(ModuleRun) \
      #       -activebackground $GUI_Options(ModuleRun) \
      #       -fg $GUI_Options(ModuleTextRun) \
      #       -activeforeground $GUI_Options(ModuleTextRun)}
      # }
    }
    combobox {}
    imageset {}
    module -
    default {
      switch $state {
        1       {set MState ModuleEnabled;  set textFill ModuleTextEnabled}
        2       {set MState ModuleActive;   set textFill ModuleTextActive}
        3       {set MState ModuleRun;      set textFill ModuleTextRun}
        default {set MState ModuleInactive; set textFill ModuleTextInactive}
      }
      $canvas itemconfigure $module        -fill $GUI_Options($MState)
      $canvas itemconfigure ${module}_text -fill $GUI_Options($textFill)
      ## Do we have two images?
      set tags [$canvas find withtag ${module}_img]
      if {[llength $tags] == 1} {
        catch {$canvas itemconfigure ${module}_img -image I$MState}
        catch {$canvas itemconfigure ${module}_img -image ModImg_$module}
      } else {
        foreach {lower upper} $tags {break}
        catch {$canvas itemconfigure $lower -image I$MState}
        catch {$canvas itemconfigure $upper -image ModImg_$module}
      }
    }
  };# switch -exact -- $type

  lappend GUI_Draw_Module_Status_Drawn $module

  ## Avoid Cycles...
  set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
  foreach mod $next_modules {
    if {[lsearch -exact $GUI_Draw_Module_Status_Drawn $mod] == -1} {
      GUI_Draw_Module_Status_Recursive $canvas $mod $title
    }
  }
};# GUI_Draw_Module_Status_Recursive


## GUI_GetModuleOptions module
#   Get the module parameters from the creole_config file...
#   Returns a list of the following form:
#   { { {Val1} {Val2} ... {ValN} }
#     { {Def1} {Def2} ... {DefN} } }
#      where Def? is of form:
#        Def? = { {Parameter Text} {Patern} {Default Value} }
proc GUI_GetModuleOptions {mod {silent no} {Canvas {}}} {
  global creole_${mod}_home HomeDir UserHomeDir HomeDir gate_home
  set gate_home $HomeDir
  catch {winfo toplevel $Canvas} System

  ## Get Definitions from creole_config.tcl file
  if {[catch {PUL_plugin_module_config $mod parameters} arg_def]} {
    if {[string equal $silent no] || $silent == 0} {
      gut_ErrorDlg $arg_def
    } else {
      return [list Error $arg_def]
    }
    return {{} {}}
  }
  set args {}

  ## Set Initial Values
  foreach {label pattern default} $arg_def {
    if {![catch {uplevel #0 subst [list $pattern]} new]} {
      set pattern $new
    } else {set warning "$new\n$::errorInfo"}
    if {![catch {uplevel #0 subst [list $default]} new]} {
      set default $new
    } else {set warning "$new\n$::errorInfo"}

    set value $default
    switch -glob -- $pattern {
      {*%FILE%*}       {set p %FILE%      }
      {*%DIRECTORY%*}  {set p %DIRECTORY% }
      {*%NUMBER%*}     {set p %NUMBER%    }
      {*%COMBO%*}      {set p %COMBO%;    set value [lindex $default 0]}
      {*%ENCODING%*}   {set p %ENCODING%  }
      {*%ENTRY%*}      {set p %ENTRY%     }
      {*%COMMAND%*}    {set p %COMMAND%;  set value {} }
      {*%BOOLEAN%*}    {set p %BOOLEAN%   }
      default          {set p {}          }
    }

    if {$p ne "" && $value ne ""} {
      regsub -all -- $p $pattern $value value
    }

    if {[info exists warning]} {
      gut_warning "Cannot handle options for module $mod...\n$warning"
      set value {}
    }
    lappend args $value
  }
  return [list $args $arg_def]
};# GUI_GetModuleOptions

## GUI_RunTool
#   Runs a registered tool.
proc GUI_RunTool {canvas tool} {
  global GUI_GraphInfo
  foreach {col doc} {{} {}} {}
  catch {set col $GUI_GraphInfo(CCollection_${canvas})}
  catch {set doc $GUI_GraphInfo(CDocument_${canvas})}
  #if {[string equal $col {}]} return
  tip_RunTool $col $doc $tool
};# GUI_RunTool

## GUI_System_Get_Opened system_window
#   It is called after the user has chosen a collection/document, in order
#   to proper initialize the system window
proc GUI_System_Get_Opened {system_win} {
  global GUI_Current_Opened_Collection GUI_Current_Opened_Document \
   GUI_GraphInfo GUI_SystemWindow
  if {![info exists GUI_Current_Opened_Collection] || 
       [string equal $GUI_Current_Opened_Collection {}]} {return}

  set canvas $system_win.frame.canvas
  ## Close the previous opened collection or document, if any...
  set result [GUI_CloseCollection $system_win]
  if {[string equal $result cancel]} {return}
  set GUI_SystemWindow($system_win:TimeElapsed) 0
  catch {destroy $canvas.timeDisplay}

  set col $GUI_Current_Opened_Collection
  set doc $GUI_Current_Opened_Document
  
  set GUI_Current_Opened_Collection {}
  set GUI_Current_Opened_Document {}
  set system [GUI_SystemWindowGetInfo $canvas SystemName]
  GUI_ResetSystemGraph $canvas $system
  set GUI_GraphInfo(CollectionChanged_$canvas) 0

  ## Get Modules contained in the current system
  set modules [array names GUI_GraphInfo "$canvas*"]
  set module_names {}
  foreach module $modules {
    regsub $canvas $module {} name
    lappend module_names $name
  }

  ## Find the "Initial" modules...
  set init_modules {}
  foreach module $modules {
    eval lappend names [lrange $GUI_GraphInfo($module) 1 end]
  }
  foreach module $module_names {
    if {[lsearch -exact $names $module] == -1} {lappend init_modules $module}
  }
  if {![llength $init_modules] && [llength $module_names]} {
    global GUI_Text
    set init_modules [gut_MultipleSelectFromList $module_names \
      $GUI_Text(CannotResolveInitModules) {Select Initial Modules...}]
  }

  ## Enable the Initial Modules (Make them Selectable...)
  foreach init_m $init_modules {
    if {![info exists GUI_GraphInfo(${canvas}${init_m})]} {
      catch {puts stderr "Module missing? $init_m (init_m)"}
      continue
    }
    set info $GUI_GraphInfo(${canvas}${init_m})
    set info [lreplace $info 0 0 1]
    set GUI_GraphInfo(${canvas}${init_m}) $info
  }

  ## Petasis, 30 Nov 2009: Ask the user to collect attributes if documents are
  ## more than 200...
  set collect_attributes 1
  set length [$col Length]
  if {![string length $doc] && $length > 200} {
    set answer [gut_question "A Collection with $length documents detected. Do\
    you want to skip search for modules that have run on all documents?" \
    "Time-consuming task..." 1 $system_win]
    if {$answer eq "yes"} {set collect_attributes 0}
  }
  
  set AlreadyRunModules {}
  if {![string length $doc]} { ;# The user has opened a whole collection
    global GUI_Text
    set GUI_GraphInfo(CCollection_${canvas}) $col
    set GUI_GraphInfo(CDocument_${canvas}) {}
    set ColName [file tail [tip_GetName $col]]
    set ::GUI_StatusBar::StatusBar($system_win:Def) \
      "$GUI_Text(Collection):([tip_GetName $col]), \
       $GUI_Text(Document):($GUI_Text(All))"
    catch {$system_win.tf.express.widgets.collection configure -text $ColName}
    catch {$system_win.tf.express.widgets.document configure \
           -text $GUI_Text(All)}
    set ::GUI_StatusBar::StatusBar($system_win) \
       $::GUI_StatusBar::StatusBar($system_win:Def)
    if {$collect_attributes} {
    set wwin [ggi_wait_message $GUI_Text(CollectingDocumentAttributes) 0]
    catch {grab release [grab current]}
    update
    set FileIDs [$col GetFileIds]
    set FileIDsNu [llength $FileIDs]
    if {!$FileIDsNu} {incr FileIDsNu}
      set step [expr {100.0/$FileIDsNu}]
      set Progress 0.0
      ggi_wait_update {} $Progress $wwin

      ## Find All module that have already been done for the whole collection
      set Format  [$col Format]
      set Name    [tip_GetName $col]
      set DocumentNumber $FileIDsNu
      unset -nocomplain ModuleCoundHash
      ## Here, we try to locate whether some components have been ran on all
      ## Documents. The algorithm is simple:
      ## We set the entries in the ModuleCoundHash array we are interested,
      ## and then we try to increment their frequency. If the frequency is
      ## equal to the number of Documents, the component has been ran on the
      ## whole Collection...
      foreach attr_name $module_names {
        set ModuleCoundHash($attr_name) 0
      }
      if {[catch {
        foreach ID $FileIDs {
          foreach attr [CDM_GetDocumentAttributes $Name/$ID $Format] {
            catch {incr ModuleCoundHash([tip_GetName $attr])}
          }
          set Progress [expr {$Progress + $step}]
          ggi_wait_update {} $Progress $wwin
        }
      } msg]} {
        if {[string equal $msg "CDM_GetDocumentAttributes: Aborting..."]} {
          gut_info $GUI_Text(CollectingDocumentAttributesAborted) \
                   {Process Aborted...} $canvas
        } else {
          set GUI_GraphInfo(CCollection_${canvas}) {}
          set GUI_GraphInfo(CDocument_${canvas})   {}
          tip_Close $col
          error $msg $::errorInfo
        }
      }
      foreach module [array names ModuleCoundHash] {
        if {[lsearch -exact $module_names $module] == -1} {continue}
        if {$ModuleCoundHash($module) == $DocumentNumber} {
          lappend AlreadyRunModules $module
        }
      }
      ggi_destroy_wait $wwin
    }
  } else { ;# a single document
    global GUI_Text
    set GUI_GraphInfo(CCollection_${canvas}) $col
    set GUI_GraphInfo(CDocument_${canvas}) $doc
    set ColName [file tail [tip_GetName $col]]
    set DocName [tip_GetId $doc]
    set ::GUI_StatusBar::StatusBar($system_win:Def) \
        "$GUI_Text(Collection):($ColName), $GUI_Text(Document):($DocName)"
    catch {$system_win.tf.express.widgets.collection configure -text $ColName}
    catch {$system_win.tf.express.widgets.document   configure -text $DocName}
    set ::GUI_StatusBar::StatusBar($system_win) \
       $::GUI_StatusBar::StatusBar($system_win:Def)

    ## Get Document Attributes and set the proper state to the modules that
    ## have already run...
    foreach attr [tip_GetAttributes $doc] {
      set attr_name [tip_GetName $attr]
      if {[lsearch -exact $module_names $attr_name] != -1} {
        lappend AlreadyRunModules $attr_name
      }
    }
  }

  ## Set the proper initial state in all modules in the system
  set GUI_Draw_Module_Status_Drawn $AlreadyRunModules
  foreach module $AlreadyRunModules {
    if {![info exists GUI_GraphInfo(${canvas}${module})]} {
      catch {puts stderr "Module missing? $module (module)"}
      continue
    }
    set info $GUI_GraphInfo(${canvas}${module})
    set info [lreplace $info 0 0 3]
    set GUI_GraphInfo(${canvas}${module}) $info
    set next_modules [lrange $GUI_GraphInfo(${canvas}${module}) 1 end]
    foreach nm $next_modules {
      if {![info exists GUI_GraphInfo(${canvas}${nm})]} {
        catch {puts stderr "Module missing? $nm (nm)"}
        continue
      }
      set info $GUI_GraphInfo(${canvas}${nm})
      if {[lindex $info 0] != 3} {
        set info [lreplace $info 0 0 1]
        set GUI_GraphInfo(${canvas}${nm}) $info
      }
    }
    GUI_Draw_Module_Status $canvas $module {}
  }
  
  set GUI_GraphInfo(InitialModuleList_${canvas}) $init_modules
  foreach init_m $init_modules {
    GUI_Draw_Module_Status $canvas $init_m {}
  }
  wm protocol $system_win WM_DELETE_WINDOW \
     "GUI_CloseWindow \"$system_win\" $col"
  catch {Toolbar::activate $GUI_SystemWindow($system_win:ViewersToolbar)}
  set index [lsearch -exact [GUI_GetCollectionNames] $ColName]
  catch {$system_win.tf.express.widgets.collection setvalue $index}
  set GUI_GraphInfo(CollectionChanged_$canvas) 0
  tip_SetAssociatedInfo $col [list $system_win $canvas]
  ## Has Collection any comments?
  if {[string length [tip_GetComment $col]]} {
    global GUI_Options
    $system_win.statusbar.comment configure -image ICommentSmall -borderwidth \
       $GUI_Options(BorderWidth)
    bind $system_win.statusbar.comment <<MB_1>> \
      "ggi_view_text \[tip_GetComment $col\] \"Comments of Collection\
      \[tip_GetName $col\]\""
  } else {
    $system_win.statusbar.comment configure -image Blank -width 0 -borderwidth 0
    bind $system_win.statusbar.comment <<MB_1>> {}
  }
  ## Close the selectViewer window, if displayed...
  if {[winfo exists $system_win.selectViewer]} {
    destroy $system_win.selectViewer
    GUI_DocView SelectViewer $system_win
  }
};# GUI_System_Get_Opened

## GUI_CollectionSave
#   Save a system's current collection
proc GUI_CollectionSave {win} {
  global GUI_GraphInfo GUI_Text
  set canvas $win.frame.canvas
  set result yes
  if [info exists GUI_GraphInfo(CCollection_$canvas)] {
    set col $GUI_GraphInfo(CCollection_$canvas)
    if {![string length [info commands $col]]} {
      unset -nocomplain GUI_GraphInfo(CCollection_$canvas)
      return
    }
    if {![$col Modified]} {return}
    set result [gut_question [format "%s\n(%s)" \
           $GUI_Text(DoyouwanttosavecurrentCollection?) \
           [tip_GetName $col]] {Save Collection ?} \
           0 $win {yes no cancel}]
    if {[string equal $result yes]} {tip_Sync $col}
  }
  return $result
};# GUI_CollectionSave

## GUI_CloseCollection
#   Asks the user whether to save the current collection, closes collection
#   and does some cleaning up...
proc GUI_CloseCollection {win} {
  global GUI_GraphInfo GUI_Text GUI_SystemWindow
  set canvas ${win}.frame.canvas
  if [info exists GUI_GraphInfo(CCollection_${canvas})] {
    set col $GUI_GraphInfo(CCollection_${canvas})
    set result [GUI_CollectionSave $win]
    if {[string equal $result cancel]} {
      ## The user has canceled closing...
      return cancel
    }
    tip_Close $col
    CDM_ClearUnusedSavedValues
    catch {unset GUI_GraphInfo(CCollection_${canvas})}
    catch {unset GUI_GraphInfo(CDocument_${canvas})}
    catch {Toolbar::deactivate $GUI_SystemWindow($win:ViewersToolbar)}
  }
  return yes
};# GUI_CloseCollection

## GUI_DocView {Text, Annotations, Attributes} win
proc GUI_DocView {What win} {
  global GUI_GraphInfo GUI_Text GUI_Options
  set canvas $win.frame.canvas
  if [info exists GUI_GraphInfo(CDocument_${canvas})] {
    set doc $GUI_GraphInfo(CDocument_${canvas})
  } else { return {}}
  if {[catch {set GUI_GraphInfo(CCollection_${canvas})} col]} return
  
  ## Has the user Opened a whole Collection?
  if {![string length $doc]} {
    set DocIds [$col GetFileIds]
    if {[llength $DocIds] > 1} {
      if {![info exists GUI_Options(CannotViewCollectionReported)]} {
        gut_ErrorDlg $GUI_Text(CannotViewCollection) [winfo toplevel $canvas]
        set GUI_Options(CannotViewCollectionReported) 1
      }
      set document [gut_MultipleSelectFromList \
                       $DocIds \
                       $GUI_Text(TSelectDocument) \
                       {Select Document to View...}]
    
      if {[string equal $document {}]} return
      set doc [$col GetDocument [lindex $document 0]]
    } else {
      set doc [tip_FirstDocument $col]
    }
  }

  set ColMsg "From Collection [file tail [tip_GetName $col]]"
  switch -exact $What {
    Text  {
      set title "Text of Document [$doc GetId], $ColMsg"
      ggi_view_text_file $doc {} {} $title
    }
    Annotations  {
      set title "Annotations of Document [$doc GetId], $ColMsg"
      ggi_view_raw $doc {} {} $title
    }
    ExAnnotations {
      set title "Explore Annotations of Document [$doc GetId], $ColMsg"
      ggi_view_AnnotationExplorer $doc {} {} $title
    }
    Attributes   {
     set title "Attributes of Document [$doc GetId], $ColMsg"
     ggi_view_Attributes $doc {} 1 $title
    }
    SelectViewer {
      ## This option is not a real viewer. Instead, we display a window, from
      ## which the user can select which viewer and with what parameters to
      ## run...
      set Win $win.selectViewer
      ## Does the window exists? Destroy it...
      if {[winfo exists $Win]} {destroy $Win; return bottom}
      toplevel $Win
      wm withdraw $Win
      wm title $Win "Select A Viewer..."

      ## Create a Tabbed Notebook widget...
      set NoteBook $Win.nt_book
      set NoteBookBg [$win cget -bg]
      NoteBook $NoteBook -bg $NoteBookBg

      ## Get Annotation Types...
      set AnnTypes [lsort -dictionary [$doc GetAnnotationTypes]]

      ########################################################################
      ## Create a page for the Single Span viewer...
      set Frame [$NoteBook insert end SingleSpan -text {Single Span} \
                                           -background $NoteBookBg]
      label $Frame.lanntype -textvariable GUI_Text(AnnotationType) -anchor w
      ComboBox $Frame.anntype -postcommand "$Frame.anntype configure -values \
        \[lsort -dictionary \[$doc GetAnnotationTypes]]" -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -modifycmd "$Frame.annattr configure -values \
        \[lsort -dictionary \[$doc GetAnnotationAttributeNames \
        \[lindex \[lsort -dictionary \[$doc GetAnnotationTypes]] \
        \[$Frame.anntype getvalue]]]];$Frame.annattr setvalue first"
      label $Frame.lannattr -textvariable GUI_Text(Attribute) -anchor w
      ComboBox $Frame.annattr -highlightthickness 0 -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)
      
      grid $Frame.lanntype $Frame.anntype -sticky snew -pady 1 -padx 2
      grid $Frame.lannattr $Frame.annattr -sticky snew -pady 1 -padx 2
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_single_span $doc \
           \[lindex \[$Frame.anntype cget -values] \[$Frame.anntype getvalue]] \
           \[lindex \[$Frame.annattr cget -values] \[$Frame.annattr getvalue]] \
           {}"] -columnspan 2 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1

      ########################################################################
      ## Create a page for the Raw viewer...
      set Frame [$NoteBook insert end Raw -text {Raw} -background $NoteBookBg]
      label $Frame.lanntype -textvariable GUI_Text(AnnotationType) -anchor w
      ComboBox $Frame.anntype -postcommand "$Frame.anntype configure -values \
        \[concat {{}} \[lsort -dictionary \[$doc GetAnnotationTypes]]]" \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -editable 0
      label $Frame.lannattr -textvariable GUI_Text(Attribute) -anchor w
      ComboBox $Frame.annattr -highlightthickness 0 -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)
      
      grid $Frame.lanntype $Frame.anntype -sticky snew -pady 1 -padx 2
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_raw $doc \
           \[lindex \[$Frame.anntype cget -values] \[$Frame.anntype getvalue]] \
           {} \
           {}"] -columnspan 2 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1

      ########################################################################
      ## Create a page for the Annotation Explorer viewer...
      set Frame [$NoteBook insert end Explorer -text {Annotation Explorer} \
                                           -background $NoteBookBg]
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_AnnotationExplorer $doc {} {} {}"] -pady 10 -padx 10

      ########################################################################
      ## Create a page for the HTML Annotation Explorer viewer...
      set Frame [$NoteBook insert end HTMLExplorer \
         -text {HTML Annotation Explorer} -background $NoteBookBg]
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_HTMLAnnotationExplorer $doc {} {} {}"] -pady 10 -padx 10

      ########################################################################
      ## Create a page for the Attribute viewer...
      set Frame [$NoteBook insert end Attributes -text {Attributes} \
                                           -background $NoteBookBg]
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_Attributes $doc"] -pady 10 -padx 10
   
      ########################################################################
      ## Create a page for the Parse viewer...
      set Frame [$NoteBook insert end Parse -text {Parse} \
                                           -background $NoteBookBg]
      label $Frame.lanntype -textvariable GUI_Text(AnnotationType) -anchor w
      ComboBox $Frame.anntype -postcommand "$Frame.anntype configure -values \
        \[lsort -dictionary \[$doc GetAnnotationTypes]]" -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -modifycmd "$Frame.annattr configure -values \
        \[lsort -dictionary \[$doc GetAnnotationAttributeNames \
        \[lindex \[lsort -dictionary \[$doc GetAnnotationTypes]] \
        \[$Frame.anntype getvalue]]]];$Frame.annattr setvalue first"
      label $Frame.lannattr -textvariable GUI_Text(Attribute) -anchor w
      ComboBox $Frame.annattr -highlightthickness 0 -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)
      
      grid $Frame.lanntype $Frame.anntype -sticky snew -pady 1 -padx 2
      grid $Frame.lannattr $Frame.annattr -sticky snew -pady 1 -padx 2
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_parse $doc \
           \[lindex \[$Frame.anntype cget -values] \[$Frame.anntype getvalue]] \
           \[lindex \[$Frame.annattr cget -values] \[$Frame.annattr getvalue]] \
           {}"] -columnspan 2 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1
      
      ########################################################################
      ## Create a page for the Morphology viewer...
      set Frame [$NoteBook insert end Morphology -text {Morphology} \
                                           -background $NoteBookBg]
      label $Frame.lanntype -textvariable GUI_Text(AnnotationType) -anchor w
      ComboBox $Frame.anntype -postcommand "$Frame.anntype configure -values \
        \[lsort -dictionary \[$doc GetAnnotationTypes]]" -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -modifycmd "$Frame.annattr configure -values \
        \[lsort -dictionary \[$doc GetAnnotationAttributeNames \
        \[lindex \[lsort -dictionary \[$doc GetAnnotationTypes]] \
        \[$Frame.anntype getvalue]]]];$Frame.annattr setvalue first"
      label $Frame.lannattr -textvariable GUI_Text(Attribute) -anchor w
      ComboBox $Frame.annattr -highlightthickness 0 -editable 0 \
        -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)
      
      grid $Frame.lanntype $Frame.anntype -sticky snew -pady 1 -padx 2
      grid $Frame.lannattr $Frame.annattr -sticky snew -pady 1 -padx 2
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_morphology $doc \
           \[lindex \[$Frame.anntype cget -values] \[$Frame.anntype getvalue]] \
           \[lindex \[$Frame.annattr cget -values] \[$Frame.annattr getvalue]] \
           {}"] -columnspan 2 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1
      
      ########################################################################
      ## Create a page for the Document viewer...
      set Frame [$NoteBook insert end Document -text {Document} \
                                           -background $NoteBookBg]
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_document $col [tip_GetId $doc] $win"] -pady 10 -padx 10

      ########################################################################
      ## Create a page for the Text File viewer...
      set Frame [$NoteBook insert end TextFile -text {Text File} \
                                           -background $NoteBookBg]
      global GUI_CColEncoding
      label $Frame.editLabel -textvariable GUI_Text(File) -anchor w
      entry $Frame.editLabelEntry 
      GUI_MakeEntryFileDropTarget $Frame.editLabelEntry
      button $Frame.buttton -image ITOpen -text ... -compound left -command \
          "GUI_SelectFile $Frame.editLabelEntry"
      Tooltip::tooltip $Frame.buttton -textvariable GUI_Text(SelectFile)
      label $Frame.encLabel -textvariable GUI_Text(Encoding) -anchor w
      set menu $Frame.enc.mn
      set GUI_CColEncoding($win) [encoding system]
      menubutton $Frame.enc -relief raised -indicatoron 1 -anchor c \
        -menu $menu -textvariable GUI_CColEncoding($win) -width 200 \
        -bd $GUI_Options(BorderWidth) -fg #ffff00 -bg #009900 \
        -image [::Encoding::GetEncodingImage $GUI_CColEncoding($win)] \
        -compound left -padx 10 -text $GUI_CColEncoding($win)
      bind $Win <Destroy> "+unset -nocomplain GUI_CColEncoding($win)"
      menu $menu -tearoff 0
      GUI_AppendEncodingsToMenu $menu "set GUI_CColEncoding($win) \$Encoding
        $Frame.enc configure -image \$EncodingImage -text \$Encoding" 1
      grid $Frame.editLabel $Frame.editLabelEntry $Frame.buttton \
        -sticky snew -padx 2 -pady 1
      grid $Frame.encLabel -sticky snew -padx 2 -pady 1
      grid $Frame.enc -row 1 -column 1 -columnspan 2 -sticky snew \
        -padx 2 -pady 1
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
      "set CDM_CurrentStatus(CurrentCollectionEncoding) \$GUI_CColEncoding($win)
       ggi_view_text_file $doc \[$Frame.editLabelEntry get] \
       \$GUI_CColEncoding($win) {}"] -columnspan 3 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1

      ########################################################################
      ## Create a page for the Document XML DOM Tree viewer...
      set Frame [$NoteBook insert end XMLDOM -text {Document XML DOM Tree} \
                                           -background $NoteBookBg]
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_XMLDomTree $doc {} {} {} $win.xmlDomTree_viewer"] \
        -pady 10 -padx 10
      
      ########################################################################
      ## Create a page for the Document XML File DOM Tree viewer...
      set Frame [$NoteBook insert end FileXMLDOM -text {File XML DOM Tree} \
                                           -background $NoteBookBg]
      global GUI_CColEncoding
      label $Frame.editLabel -textvariable GUI_Text(File) -anchor w
      entry $Frame.editLabelEntry 
      GUI_MakeEntryFileDropTarget $Frame.editLabelEntry
      button $Frame.buttton -image ITOpen -text ... -compound left -command \
          "GUI_SelectFile $Frame.editLabelEntry"
      Tooltip::tooltip $Frame.buttton -textvariable GUI_Text(SelectFile)
      label $Frame.encLabel -textvariable GUI_Text(Encoding) -anchor w
      set menu $Frame.enc.mn
      set GUI_CColEncoding($win) utf-8
      menubutton $Frame.enc -relief raised -indicatoron 1 -anchor c \
        -menu $menu -textvariable GUI_CColEncoding($win) -width 10 \
        -bd $GUI_Options(BorderWidth) -fg #ffff00 -bg #009900 \
        -text $GUI_CColEncoding($win) -compound left -image \
        [::Encoding::GetEncodingImage $GUI_CColEncoding($win)] -padx 10
      bind $Win <Destroy> "+unset -nocomplain GUI_CColEncoding($win)"
      menu $menu -tearoff 0
      GUI_AppendEncodingsToMenu $menu "set GUI_CColEncoding($win) \$Encoding
        $Frame.enc configure -image \$EncodingImage -text \$Encoding"
      grid $Frame.editLabel $Frame.editLabelEntry $Frame.buttton \
        -sticky snew -padx 2 -pady 1
      grid $Frame.encLabel -sticky snew -padx 2 -pady 1
      grid $Frame.enc -row 1 -column 1 -columnspan 2 -sticky snew \
        -padx 2 -pady 1
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
        "ggi_view_XML_file $doc \[$Frame.editLabelEntry get]\
        \$GUI_CColEncoding($win) {XML FileViewer...} $win.xmlDomTree_viewer"] \
        -columnspan 3 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1

      ########################################################################
      ## Create a page for the Video File viewer...
      set Frame [$NoteBook insert end VideoFile -text {Video File} \
                                           -background $NoteBookBg]
      global GUI_CColEncoding
      label $Frame.editLabel -textvariable GUI_Text(File) -anchor w
      entry $Frame.editLabelEntry 
      GUI_MakeEntryFileDropTarget $Frame.editLabelEntry
      button $Frame.buttton -image ITOpen -text ... -compound left -command \
          "GUI_SelectFile $Frame.editLabelEntry"
      Tooltip::tooltip $Frame.buttton -textvariable GUI_Text(SelectFile)
      grid $Frame.editLabel $Frame.editLabelEntry $Frame.buttton \
        -sticky snew -padx 2 -pady 1
      grid [button $Frame.view -textvariable GUI_Text(View) -command \
      "ggi_view_VideoFile $doc \[$Frame.editLabelEntry get] \
       {} {}"] -columnspan 3 -pady 10 -padx 10
      grid columnconfigure $Frame 1 -weight 1

      pack $NoteBook -fill both -expand 1
      $NoteBook compute_size
      $NoteBook raise SingleSpan
      $NoteBook see   SingleSpan
      update
      wm deiconify $Win
      return top
    }
   default {return {}}
  }
};# GUI_DocView

## GUI_ColView { Attributes, TimeLog } win
proc GUI_ColView {What Win} {
  global GUI_GraphInfo

  set canvas $Win.frame.canvas
  if [info exists GUI_GraphInfo(CCollection_$canvas)] {
    set col $GUI_GraphInfo(CCollection_$canvas)
  } else {return}

  switch -exact $What {
   Attributes  {
     set title "Attributes of Collection [file tail [$col GetName]]"
     ggi_view_Attributes $col {} 1 $title
   }
   TimeLog {
     set Log $GUI_GraphInfo(TimeLog_$canvas)
     ggi_view_text_file $Log {} default {Time Log}
   }
   ErrorLog {
     global GUI_SystemErrorLog GUI_Options
     set text [ggi_view_text_file $GUI_SystemErrorLog($canvas) {} default \
                                  {Error Log}]
     catch {destroy $canvas.errorLog}
     ## Highlight in the text widget the first line of each error...
     set index 1.0
     while {1} {
       set index [$text search -count chars -regexp \
         {An error occured while executing module: \"[^\"]*\"} $index end]
       if {![string length $index]} {break}
       regexp {(\d+).(\d+)} $index match line char
       $text tag add hi $index [expr {$line+2}].0
       append index +${chars}chars
     }
     $text tag configure hi -foreground red -background yellow\
       -font "$GUI_Options(TextFont) $GUI_Options(TextFontSize) bold"
   }
   EditComSave {
     global GUI_Options
     set Text $Win.col_edtCommentDlg.text.text
     tip_SetComment $col [string trim [$Text get 1.0 end]]
     if {[string length [tip_GetComment $col]]} {
       $Win.statusbar.comment configure -image ICommentSmall -borderwidth \
          $GUI_Options(BorderWidth)
       bind $Win.statusbar.comment <<MB_1>> \
         "ggi_view_text \[tip_GetComment $col\] \"Comments of Collection\
          \[tip_GetName $col\]\""
     } else {
       $Win.statusbar.comment configure -image Blank -width 0 -borderwidth 0
       bind $Win.statusbar.comment <<MB_1>> {}
     } 
     return
   }
   EditCom {
     global GUI_Options GUI_Text
     set win $Win.col_edtCommentDlg
     if {[winfo exists $win]} {
       raise $win
       return
     }
     toplevel $win
     wm title $win "Editing Collection Comment: \"[tip_GetName $col]\""
     wm withdraw $win

     ## Toolbar
     set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
     set Text $win.text.text
     Toolbar::ToolbarFrame $toolbar_frame \
             -relief $GUI_Options(ToolbarFrameRelief) \
             -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
             -decorate $GUI_Options(ToolbarDecorate)

       set tw $win.frame.text
       set toolbar $toolbar_frame.tb1
       Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
           -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
           -image ITSave -command "GUI_ColView EditComSave $Win"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
           -image ITSelectFont -command "::FontDialog::FontSelect $tw \
           $tw._fontdlg"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
           -image ITSelectColour -command "ggi_SetWidgetColours $tw"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
           -image ITZoom -command \
           "::GUI_TextSearchFacilities::TextFindDialog $tw"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
            -image ITExit -command "destroy $win"
       pack $toolbar -side left -fill y
     pack $toolbar_frame -side top -fill x

     frame $win.buttons
       button $win.buttons.ok -textvariable GUI_Text(Dismiss) \
             -command "destroy $win"
        pack $win.buttons.ok -padx 2 -pady 2
     pack $win.buttons -side bottom
     
     ::ScrolledWidgets::ScrolledText $win.text
     pack $win.text -side top -fill both -expand 1
     
     $Text configure -state normal
     $Text delete 1.0 end
     $Text insert end [tip_GetComment $col]
     wm deiconify $win
     return   
   }
   Status {
     global GUI_Options GUI_Text
     set win $Win.col_statusDlg
     if {[winfo exists $win]} {
       raise $win
       return
     }
     toplevel $win
     wm title $win "Collection Status: \"[tip_GetName $col]\""
     wm withdraw $win

     ## Toolbar
     set toolbar_frame ${win}.tf; set toolbar $toolbar_frame.tb
     Toolbar::ToolbarFrame $toolbar_frame \
             -relief $GUI_Options(ToolbarFrameRelief) \
             -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
             -decorate $GUI_Options(ToolbarDecorate)

       set tw $win.frame.text
       set toolbar $toolbar_frame.tb1
       Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
           -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
           -image ITSave -command "ggi_SaveTextWidgetContents $tw"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
           -image ITSelectFont -command "::FontDialog::FontSelect $tw \
           $tw._fontdlg"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
           -image ITSelectColour -command "ggi_SetWidgetColours $tw"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
           -image ITZoom -command \
           "::GUI_TextSearchFacilities::TextFindDialog $tw"
       Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
            -image ITExit -command "destroy $win"
       pack $toolbar -side left -fill y
     pack $toolbar_frame -side top -fill x

     frame $win.buttons
       button $win.buttons.ok -textvariable GUI_Text(Dismiss) \
             -command "destroy $win"
        pack $win.buttons.ok -padx 2 -pady 2
     pack ${win}.buttons -side bottom
     
     set Text $win.text.text
     ::ScrolledWidgets::ScrolledText $win.text
     pack $win.text -side top -fill both -expand 1
     
     $Text configure -state normal
     $Text delete 1.0 end
     $Text insert end "Collection Name: [tip_GetName $col]\n"
     $Text insert end "Collection Document Number: [tip_Length $col]\n"
     $Text insert end "Collection Encoding: [tip_GetEncoding $col]\n"
     $Text insert end "Collection Format: [$col Format]\n"
     $Text insert end "Collection Comment:\n[$col GetComment]\n"
     $Text insert end "Collection Tcl Command: $col\n"
     if {[$col Modified]} {
       $Text insert end "Collection is Modified\n"
     } else {
       $Text insert end "Collection is not Modified\n"
     }
     $Text insert end "Collection Owner: [tip_GetOwner $col]\n"
     set Attrs [tip_GetAttributes $col]
     $Text insert end "Document has [llength $Attrs] Attributes:\n"
     foreach attr $Attrs {
       $Text insert end [format "         %s : (%s)\n" [lindex $attr 0] \
                   [lindex [lindex $attr 1] end]]
     }

     $Text insert end "Collection Documents:\n"
     foreach id [$col GetFileIds] xid [$col GetFileXIds] {
       $Text insert end {  }
       $Text image create end -image IDocumentSmall
       $Text insert end " $id ($xid)\n"
     }

     $Text insert end "Collection Opened Documents:\n"
     foreach id [$col GetOpenedIds] cmd [$col GetOpenedDocs] {
       $Text insert end {  }
       $Text image create end -image ITinyDoc
       $Text insert end " $id ($cmd)\n"
     }
 
     $Text configure -state disabled
     wm deiconify $win
     return   
   }
   default return
  }
}

## GUI_SystemDeselect
#   "Raw" canvas callback. If mouse clicks:
#   1) a module, just returns
#   2) an arc, selects that arc and displays its curve points for editing
#   3) empty space, deselects any selected module and arc
proc GUI_SystemDeselect {canvas system} {
  global GUI_Options
  
  set current [$canvas find withtag current]
  if {![string length $current]} {
    ## Disable the modules toolbar...
    set parent_win [winfo toplevel $canvas]
    catch {Toolbar::deactivate $parent_win.tf.module}
    set selected [$canvas find withtag _current_]
    catch {$canvas dtag _current_ _current_}
    foreach mod $selected {
      ## Try to recover the module name from the canvas tag...
      set module [lindex [$canvas gettags $mod] 0]
      if {[catch {PUL_plugin_module_config $module type} type]} {
        set type module
      }
      if {![string length $type]} {set type module}
      switch -exact -- $type {
        button   {}
        combobox {}
        imageset {}
        module   {
          $canvas itemconfigure $mod -width $GUI_Options(ModuleBorderWidth) \
                  -outline $GUI_Options(ModuleBorderColor)
        }
        default  {}
      }
    }
    set selected [$canvas find withtag _current_arc_]
    catch {$canvas dtag _current_arc_ _current_arc_}
    foreach mod $selected {
      $canvas itemconfigure $mod -width $GUI_Options(ArcWidth)
    }
    ## Remove Arc Points...
    set selected [$canvas find withtag _current_arc_points_]
    catch {$canvas dtag _current_arc_points_ _current_arc_points_}
    foreach mod $selected {
      $canvas delete $mod
    }
    return {}
  }

  ## Arc Stuff
  if {[string equal [$canvas type $current] line]} {
    ## Deselect Arcs...
    set selected [$canvas find withtag _current_arc_]
    catch {$canvas dtag _current_arc_ _current_arc_}
    foreach mod $selected {
      $canvas itemconfigure $mod -width $GUI_Options(ArcWidth)
    }
    ## Remove Arc Points...
    set selected [$canvas find withtag _current_arc_points_]
    catch {$canvas dtag _current_arc_points_ _current_arc_points_}
    foreach mod $selected {
      $canvas delete $mod
    }
    
    $canvas itemconfigure $current -width [expr {$GUI_Options(ArcWidth) +2}]
    $canvas addtag _current_arc_ withtag current

    foreach tag [$canvas itemcget _current_arc_ -tags] {
      if {[string match From_* $tag]} {
        set From [string range $tag 5 end]
      }
      if {[string match To_* $tag]} {
        set To   [string range $tag 3 end]
      }
    }
    global GUI_Graphs
    foreach arc [lindex $GUI_Graphs($system) end] {
      if {[string equal [lindex $arc 0] $From] && \
          [string equal [lindex $arc 1] $To]} {
        set points {}
        foreach pointset [lindex $arc end] {
          eval lappend points $pointset
        }
        if {[string equal $points {}]} {return {}}
        break
      }
    }
    for {set i 0} {$i < [llength $points]} {incr i 2} {
      set x [lindex $points $i]
      set y [lindex $points [expr {$i +1}]]
      set id [$canvas create rectangle $x $y [expr {$x +5}] [expr {$y +5}] \
          -fill $GUI_Options(ArcColor) -tags _current_arc_points_]
      $canvas bind $id <<Motion_1>> \
        "GUI_SystemMoveArcPoint $canvas $id \[$canvas canvasx %x\] \
         \[$canvas canvasy %y\] [expr {$i +2}]"
      $canvas bind $id <<Release_1>> \
         "eval GUI_DrawSystem $canvas \\\"$system\\\"\
          \[GUI_SystemChangeArcPoints \
          $canvas \"$system\"]; $canvas dtag all current"
      set last_id $id; set last_i $i
    }
    ## Remove "current" tag...
    
    return [list $last_id $last_i]
  }
  return {}
};# GUI_SystemDeselect

##############################################################################
## Arc Curve Point Stuff...                                                 ##
##############################################################################

## GUI_SystemMoveArcPoint
proc GUI_SystemMoveArcPoint {canvas id x y i} {
  $canvas coords $id $x $y [expr {$x +5}] [expr {$y +5}]
  set points [lreplace [$canvas coords _current_arc_] $i [expr {$i+1}] $x $y]
  eval $canvas coords _current_arc_ $points
};# GUI_SystemMoveArcPoint

## GUI_SystemChangeArcPoints
proc GUI_SystemChangeArcPoints {canvas system {newx {}} {newy {}}} {
  global GUI_Graphs
  foreach tag [$canvas itemcget _current_arc_ -tags] {
    if {[string match From_* $tag]} {
      set From [string range $tag 5 end]
    }
    if {[string match To_* $tag]} {
      set To   [string range $tag 3 end]
    }
  }
  set Arcs {}
  foreach arc [lindex $GUI_Graphs($system) end] {
    if {[string equal [lindex $arc 0] $From] && \
        [string equal [lindex $arc 1] $To]} {
      set points [$canvas coords _current_arc_] 
      set points [lrange $points 2 [expr {[llength $points] -3}]]
      for {set i 0} {$i < [llength $points]} {incr i 2} {
        lappend Points [lrange $points $i [expr {$i +1}]]
      }
      if {[string match d* [string tolower $newx]]} {
          set Points {}
      } else {
        if {[string length $newx] && [string length $newy]} {
          lappend Points [list $newx $newy]
        }
      }
      set Arc [list $From $To $Points]
      lappend Arcs $Arc
    } else {
      lappend Arcs $arc
    }
  }
  set GUI_Graphs($system) [lreplace $GUI_Graphs($system) end end $Arcs]
  return [list $From $To]
};# GUI_SystemChangeArcPoints

## GUI_SystemClearCurvePoints
proc GUI_SystemClearCurvePoints {system canvas} {
  if {[string equal [$canvas find withtag _current_arc_] {}]} {return {}}
  set result [GUI_SystemChangeArcPoints $canvas $system Delete]
  eval GUI_DrawSystem $canvas $system $result
  GUI_SystemDeselect $canvas $system
  #$canvas dtag _current_arc_ current
  $canvas dtag all current
};# GUI_SystemClearCurvePoints

## GUI_SystemAddCurvePointEvent
proc GUI_SystemAddCurvePointEvent {system canvas x y} {
  set win $canvas
  set result [GUI_SystemChangeArcPoints $canvas $system \
               [$canvas canvasx $x] [$canvas canvasy $y]]
  eval GUI_DrawSystem $canvas [list $system] $result
  GUI_SystemDeselect $canvas $system
  #$canvas dtag _current_arc_ current
  $canvas dtag all current
  bind $canvas <Enter> {}
};# GUI_SystemAddCurvePointEvent

## GUI_SystemAddCurvePoint
proc GUI_SystemAddCurvePoint {system canvas} {
  if {[$canvas find withtag _current_arc_] != {}} {
    bind $canvas <Enter> \
      [list GUI_SystemAddCurvePointEvent $system $canvas %x %y]
  }
};# GUI_SystemAddCurvePoint

############################################################################
## Module Parameters...                                                   ##
############################################################################
proc GUI_SystemGraph_GetSelectedModule {win} {
  set id [$win.frame.canvas find withtag _current_]
  switch -exact -- [llength $id] {
    0       {return {}}
    1       {return [lindex [$win.frame.canvas gettags $id] 0]}
    default {return [lindex [$win.frame.canvas gettags [lindex $id 0]] 0]}
  }
}; #GUI_SystemGraph_GetSelectedModule

## GUI_EditModuleParameters
#   Shows a window, by which the user can see and change a module's
#   parameters...
proc GUI_EditModuleParameters {Win} {
  global GUI_Text GUI_GraphInfo HomeDir UserHomeDir GUI_Options

  set canvas $Win.frame.canvas
  set Canvas $canvas
  set System $Win
  set module_id [$Win.frame.canvas find withtag _current_]
  if {![string length $module_id]} {return {}}
  set module [lindex [$canvas gettags $module_id] 0]
  global creole_${module}_home
  set pwin $Win.mp_$module
  if {[winfo exists $pwin]} {
    raise $pwin
    focus $pwin
    return {}
  }

  ## Get the parameters from creole_config array, and their current values...
  set arguments $GUI_GraphInfo(MOptions_${canvas}$module)

  ## Create the main window
  toplevel $pwin
  wm title $pwin "Module Parameters: $module"
  frame $pwin.lf
  GUI_LoadImage IModuleProperties
  label $pwin.lf.image -image IModuleProperties -bg #708090
  label $pwin.lf.label -textvariable GUI_Text(ModuleParameters) \
        -font {arial 14 bold italic} -anchor e -fg #008000 -bg #8fbc8f
  label $pwin.lf.module -text $module -font {arial 14 bold italic} \
        -anchor w -bg #8fbc8f -fg #6a5acd
  grid $pwin.lf.image  -row 0 -column 0 -rowspan 2 -sticky snew
  grid $pwin.lf.label  -row 0 -column 1 -sticky snew
  grid $pwin.lf.module -row 0 -column 2 -sticky snew
  set Description {}
  catch {PUL_plugin_module_config $module description} Description
  if {[string length $Description]} {
    label $pwin.lf.des_module -text $Description -bg #ffffaa -fg navy
    grid $pwin.lf.des_module -row 1 -column 1 -columnspan 2 -sticky snew
  }
  grid columnconfigure $pwin.lf 2 -weight 1
  pack $pwin.lf -side top -fill x -expand 1 -padx 4

  ## For each option, do:
  set values [lindex $arguments 0]
  set arg_def [lindex $arguments 1]
  set index 0
  set current_value_index 0
  set win $pwin.frame
  set CheckButtonWidgets {}
  frame $win -relief groove -borderwidth 2
  foreach {label pattern default} $arg_def c_value $values {
    incr index
    if {[catch {uplevel #0 subst [list $default]} default]} {
      gut_warning $default
    }
    set tmp_value $default
    ## Check whether we need an entry widget or not...
    switch -glob -- $pattern {
      {*%FILE%*}       {set entry file}
      {*%DIRECTORY%*}  {set entry directory}
      {*%NUMBER%*}     {set entry number}
      {*%COMBO%*}      {set entry combo; set tmp_value [lindex $default 0]}
      {*%ENCODING%*}   {set entry encoding}
      {*%ENTRY%*}      {set entry entry}
      {*%COMMAND%*}    {set entry command}
      {*%BOOLEAN%*}    {set entry boolean}
      default          {set entry none}
    }

    if {[info exists GUI_GraphInfo(MOptionsCV_${canvas}$module)]} {
      set tmp_value [lindex $GUI_GraphInfo(MOptionsCV_${canvas}$module) \
                            [expr {$index-1}]]
    }

    switch -exact $entry {
      file    {
        label $win.label_$index -text $label -anchor e
        set width [string length $c_value]
        if {$width < 20} {set width 20}
        if {$width > 50} {set width 50}
        entry $win.entry_$index -width $width
        $win.entry_$index insert 0 $tmp_value
        $win.entry_$index xview end
        button $win.buttton_$index -compound left -image ITOpen \
          -textvariable GUI_Text(SelectFile) \
          -command "GUI_SelectFile $win.entry_$index"
        GUI_MakeEntryFileDropTarget $win.entry_$index
        grid $win.label_$index $win.entry_$index $win.buttton_$index \
          -sticky snew -padx 3 -pady 2
      }
      directory {
        label $win.label_$index -text $label -anchor e
        set width [string length $c_value]
        if {$width < 20} {set width 20}
        if {$width > 50} {set width 50}
        entry $win.entry_$index -width $width
        $win.entry_$index insert 0 $tmp_value
        $win.entry_$index xview end
        GUI_MakeEntryDirectoryDropTarget $win.entry_$index
        button $win.buttton_$index -compound left -image ITOpen \
          -textvariable GUI_Text(SelectFile) \
          -command "GUI_SelectDirectory $win.entry_$index"
        grid $win.label_$index $win.entry_$index $win.buttton_$index \
          -sticky snew -padx 3 -pady 2
      }
      number  {
        label $win.label_$index -text $label -anchor e
        entry $win.entry_$index 
        $win.entry_$index insert 0 $tmp_value
        grid $win.label_$index $win.entry_$index -sticky snew -padx 3 -pady 2
      }
      entry   {
        label $win.label_$index -text $label -anchor e
        entry $win.entry_$index 
        $win.entry_$index insert 0 $tmp_value
        grid $win.label_$index $win.entry_$index -sticky snew -padx 3 -pady 2
      }
      combo   {
        label $win.label_$index -text $label -anchor e
        ttk::combobox $win.entry_$index -values $default -state readonly
        $win.entry_$index set $tmp_value
        grid $win.label_$index $win.entry_$index -sticky snew -padx 3 -pady 2
      }
      encoding {
        label $win.label_$index -text $label -anchor e
        set ::GUI_CColEncoding($win:$index) $tmp_value
        menubutton $win.entry_$index -relief raised -indicatoron 1 -anchor c \
          -menu $win.entry_$index.menu \
          -textvariable ::GUI_CColEncoding($win:$index) \
          -text $::GUI_CColEncoding($win:$index) -compound left -padx 10 \
          -width 10 -bd $GUI_Options(BorderWidth) -fg #ffff00 -bg #009900 \
          -image [::Encoding::GetEncodingImage $::GUI_CColEncoding($win:$index)]
        bind $win.entry_$index <Destroy> \
          "+unset -nocomplain GUI_CColEncoding($win:$index)"
        menu $win.entry_$index.menu -tearoff 0
        GUI_AppendEncodingsToMenu $win.entry_$index.menu \
          "set GUI_CColEncoding($win:$index) \$Encoding
           $win.entry_$index configure -image \$EncodingImage -text \$Encoding"
        grid $win.label_$index $win.entry_$index -sticky snew -padx 3 -pady 2
      }
      command {
        ttk::button $win.entry_$index -text $label {*}$default
        grid x $win.entry_$index -sticky snew -padx 3 -pady 2
      }
      boolean {
        set GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index) \
          $tmp_value
        checkbutton $win.check_$index -text $label -variable \
          GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index)
        lappend CheckButtonWidgets \
          "GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index)"
        grid $win.check_$index -columnspan 3 -sticky snew -padx 3 -pady 2
      }
      none    {
        checkbutton $win.check_$index -text $label -variable \
          GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index) \
          -onvalue $pattern -offvalue $default
        lappend CheckButtonWidgets \
          "GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index)"
        if {$tmp_value eq $default} {
          $win.check_$index deselect
        } else {
          $win.check_$index select
        }
        grid $win.check_$index -columnspan 3 \
          -sticky snew -padx 3 -pady 2
      }
    }
  
  }
  grid columnconfigure $win 0 -weight 0 
  grid columnconfigure $win 1 -weight 1 
  grid columnconfigure $win 2 -weight 0
  pack $win -fill both -expand 1 -ipadx 5 -ipady 5 -padx 3 -pady 3
  frame $pwin.bf
  button $pwin.bf.apply -textvariable GUI_Text(Apply) -command \
    "GUI_ApplyModuleParameters $Win $module" -default active
  bind $pwin <Return> "$pwin.bf.apply invoke;update;$pwin.bf.close invoke"
  button $pwin.bf.close -textvariable GUI_Text(Close) -command \
    "destroy $pwin; catch \"unset $CheckButtonWidgets\""
  button $pwin.bf.def   -textvariable GUI_Text(Default) -command \
    "GUI_SetDefaultModuleParameters $Win $module"
  ## Check whether this module has parameters or not...
  if {$index == 0} {
    $pwin.bf.def   configure -state disabled
    $pwin.bf.apply configure -state disabled
    label $win.none -textvariable GUI_Text(NoModuleParameters)
    grid x $win.none
  }
  pack $pwin.bf.close $pwin.bf.apply $pwin.bf.def -padx 3 -pady 5 -side left
  pack $pwin.bf
};# GUI_EditModuleParameters

## GUI_SetDefaultModuleParameters
proc GUI_SetDefaultModuleParameters {win module} {
  global GUI_GraphInfo creole_${module}_home HomeDir UserHomeDir

  set pwin $win.mp_$module
  set canvas $win.frame.canvas
  set System $win
  set Canvas $canvas

  ## Get the parameters from creole_config array, and their current values...
  set arguments [GUI_GetModuleOptions $module $canvas]

  ## For each option, do:
  set arg_vals [lindex $arguments 0]
  set arg_def  [lindex $arguments 1]
  set index 0
  set win $pwin.frame
  foreach {label pattern default} $arg_def c_value $arg_vals {
    incr index
    set default [uplevel #0 subst [list $default]]
    ## Check whether we need an entry widget or not...
    switch -glob -- $pattern {
      {*%FILE%*}       {set entry file}
      {*%DIRECTORY%*}  {set entry directory}
      {*%NUMBER%*}     {set entry number}
      {*%COMBO%*}      {set entry combo}
      {*%ENCODING%*}   {set entry encoding}
      {*%ENTRY%*}      {set entry entry}
      {*%COMMAND%*}    {set entry command}
      {*%BOOLEAN%*}    {set entry boolean}
      default          {set entry none}
    }

    switch -exact $entry {
      file    {
        $win.entry_$index delete 0 end
        $win.entry_$index insert 0 $default
        $win.entry_$index xview end
      }
      directory {
        $win.entry_$index delete 0 end
        $win.entry_$index insert 0 $default
        $win.entry_$index xview end
      }
      entry   -
      number  {
        $win.entry_$index delete 0 end
        $win.entry_$index insert 0 $default
        $win.entry_$index xview end
      }
      combo   {
        $win.entry_$index current 0
      }
      encoding {
        set ::GUI_CColEncoding($win:$index) $default
      }
      command {}
      boolean {
        set GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index) \
            $default
      }
      none    {
        $win.check_$index deselect
      }
    }
  
  }
  update
};# GUI_SetDefaultModuleParameters

## GUI_ApplyModuleParameters Win module
proc GUI_ApplyModuleParameters {win module} {
  global GUI_GraphInfo

  set pwin $win.mp_$module
  set canvas $win.frame.canvas
  set System $win
  set Canvas $canvas
  ## Get the parameters from creole_config array, and their current values...
  set arguments $GUI_GraphInfo(MOptions_${canvas}$module)

  ## For each option, do:
  set arg_def [lindex $arguments 1]
  set index 0
  set win $pwin.frame
  set c_value {}
  set CurrentValues {}
  foreach {label pattern default} $arg_def {
    incr index
    ## Check whether we need an entry widget or not...
    switch -glob -- $pattern {
      {*%FILE%*}       {set entry file}
      {*%DIRECTORY%*}  {set entry directory}
      {*%NUMBER%*}     {set entry number}
      {*%COMBO%*}      {set entry combo}
      {*%ENCODING%*}   {set entry encoding}
      {*%ENTRY%*}      {set entry entry}
      {*%COMMAND%*}    {set entry command}
      {*%BOOLEAN%*}    {set entry boolean}
      default          {set entry none}
    }

    switch -exact $entry {
      file    {
        set value [$win.entry_$index get]
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%FILE%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      directory {
        set value [$win.entry_$index get]
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%DIRECTORY%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      number  {
        set value [$win.entry_$index get]
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%NUMBER%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      entry  {
        set value [$win.entry_$index get]
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%ENTRY%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      combo   {
        set value [$win.entry_$index get]
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%COMBO%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      encoding {
        set value $::GUI_CColEncoding($win:$index)
        lappend CurrentValues $value
        set result $value
        if {[string length $value]} {
          regsub -all -- {%ENCODING%} $pattern $value result
          lappend c_value $result
        } else {
          lappend c_value {}
        }
      }
      command {
        lappend CurrentValues {}
        lappend c_value {}
      }
      boolean {
        set value \
          $GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index)
        regsub -all -- {%BOOLEAN%} $pattern $value result
        lappend CurrentValues $value
        lappend c_value $result
      }
      none    {
        set value \
          $GUI_GraphInfo(ModParamsCheck_${canvas}_${module}_check_$index)
        lappend CurrentValues $value
        lappend c_value $value
      }
    }
  }
  set GUI_GraphInfo(MOptions_${canvas}$module) [list $c_value $arg_def]
  set GUI_GraphInfo(MOptionsCV_${canvas}$module) $CurrentValues
};# GUI_ApplyModuleParameters

proc GUI_EditModuleSrc {module} {
  global GUI_Options creole_config creole_${module}_home

  set dir [set creole_${module}_home]

  ## Get coupling type
  set i [lsearch -exact $creole_config($module) coupling]
  set type [lindex $creole_config($module) [incr i]]
  if {[string match loo* $type]} {
    set file [file join $dir $module.tcl]
  } elseif {[string match jav* $type]} {
    set file [file join $dir $module.java]
  } elseif {[string match per* $type]} {
    set file [file join $dir $module.pl]
  } elseif {[string match pyt* $type]} {
    set file [file join $dir $module.py]
  } else {
    foreach ext {.cpp .lex .c .c++} {
      set file [file join $dir $module$ext]
      if {[file exists $file]} {
        break
      } else {
        set file {}
      }
    }
    ## set file [lindex [glob -type f $dir/*$module*] 0]
  }

  if {[string length $file]} {GUI_OpenExternalEditor $file}
};# GUI_EditModuleSrc

proc GUI_ViewModuleDir {module} {
  if {[string length $module]} {
    global creole_${module}_home tcl_platform
    set dir [set creole_${module}_home]
    switch -glob [string tolower $tcl_platform(os)] {
      linux {
        foreach prgm [list konqueror gmc] {
          if {![catch {exec $prgm $dir &} error]} {return} else {
            puts "Error while \"$prgm $dir\":\n$error"
          }
        }
      }
      win* {
        exec explorer [file native $dir] &
      }
      sunos {
        foreach prgm [list konqueror gmc] {
          if {![catch {exec $prgm $dir &} error]} {return} else {
            puts "Error while \"$prgm $dir\":\n$error"
          }
        }
        if {![catch {exec dtfile -standalone -folder $dir &}]} {return}
      }
    }
  }
};# GUI_ViewModuleDir

## GUI_OpenExternalEditor
proc GUI_OpenExternalEditor {file {text {}} {enc {}}} {
  global GUI_Options

  if {[string length $file]} {
   if {[catch {gut_exec_original $GUI_Options(Editor) [file native $file] &}]} {
    catch {exec $GUI_Options(Editor) [file native $file] &}
   }
  } elseif {[string length $text]} {
    global CDM_TempDir
    ## We were given only text. Create a temporary file...
    set tmp_name [file join $CDM_TempDir TempFile[pid].txt]
    set tmp [open $tmp_name w]
    if {[string length $enc]} {fconfigure $tmp -encoding $enc}
    puts $tmp $text
    close $tmp
    if {[catch {gut_exec_original $GUI_Options(Editor) $tmp_name &}]} {
      catch {exec $GUI_Options(Editor) $tmp_name &}
    }
    after 1000 "file delete -force $tmp_name"
  }
};# GUI_OpenExternalEditor

#############################################################################
## Grid Settings...                                                        ##
#############################################################################
## GUI_SystemSetGrid
proc GUI_SystemSetGrid {canvas system} {
  global GUI_Graphs GUI_GraphInfo GUI_Text
  set win $canvas.__set_grid
  if {[winfo exists $win]} {
    raise $win
    return
  }
  toplevel $win
  wm withdraw $win
  wm title $win {Grid Settings...}
  label $win.width  -textvariable GUI_Text(EnterXGrid:) -anchor w
  spinbox $win.bwidth -from 1 -to 700 -justify c
  $win.bwidth set $GUI_GraphInfo(GridX_,$canvas)
  label $win.height -textvariable GUI_Text(EnterYGrid:) -anchor w
  spinbox $win.bheight -from 1 -to 700 -justify c
  $win.bheight set $GUI_GraphInfo(GridY_,$canvas)

  grid $win.width  $win.bwidth  -sticky snew -padx 4 -pady 4
  grid $win.height $win.bheight -sticky snew -padx 4 -pady 4
  button $win.ok -textvariable GUI_Text(Ok) -width 9 -command \
    "set GUI_GraphInfo(GridX_,$canvas) \[$win.bwidth get\]
     set GUI_GraphInfo(GridY_,$canvas) \[$win.bheight get\]
     destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -width 9 -command \
    "destroy $win"
  grid $win.ok $win.cancel -padx 10 -pady 10

  gut_ModalDlg $win
  wm deiconify $win
  tkwait window $win
  set grid $GUI_GraphInfo(GridX_,$canvas)x$GUI_GraphInfo(GridY_,$canvas)
  set GUI_Graphs($system) [lreplace $GUI_Graphs($system) 3 3 $grid]
};# GUI_SystemSetGrid

##############################################################################
## Dependency (Arc) Add/Remove Functions...                                 ##
##############################################################################

## GUI_SystemAddDepend
proc GUI_SystemAddDepend {system canvas {status {}}} {
  global GUI_Graphs GUI_Text GUI_SystemAddDepend

  ## If this is the first Call...
  if {![info exists GUI_SystemAddDepend(CallNu,$canvas)]} {
    ## Show the First Message
    gut_ShowTooltip $canvas $GUI_Text(SelectDependencySource) \
      .toolmes [expr {[winfo rootx $canvas]+4}] [expr {[winfo rooty $canvas]+4}]
    set GUI_SystemAddDepend(CallNu,$canvas) 1
    set GUI_SystemAddDepend(Bind,$canvas) [bind $canvas <<MBR_1>>]
    bind $canvas <<MBR_1>> "+GUI_SystemAddDepend \"$system\" $canvas"
    return {}
  }
  ## If the user hasn't clicked a module, cancel operation...
  set selected [$canvas find withtag _current_]
  ## Cancel Operation...
  if {[string equal $selected {}] && [string equal $status {}]} {
    gut_ShowTooltip $canvas $GUI_Text(OperationCancel) \
     .toolmes [expr {[winfo rootx $canvas]+4}] [expr {[winfo rooty $canvas]+4}]
    set GUI_SystemAddDepend(CallNu,$canvas) -1
    after 1000 "GUI_SystemAddDepend \"$system\" $canvas Cancel"
    return 0
  }
  
  if {$GUI_SystemAddDepend(CallNu,$canvas) == 1} {
    ## Show the Second Message
    gut_ShowTooltip $canvas $GUI_Text(SelectDependencyDestination) \
      .toolmes [expr {[winfo rootx $canvas]+4}] [expr {[winfo rooty $canvas]+4}]
    set GUI_SystemAddDepend(CallNu,$canvas) 2
    foreach tag [$canvas itemcget _current_ -tags] {
      if {[string match "*_all" $tag]} {
        set From [string range $tag 0 [expr {[string length $tag] -5}]]
        break
      }
    }
    set GUI_SystemAddDepend(Source,$canvas) $From
    return {}
  }
  if {$GUI_SystemAddDepend(CallNu,$canvas) == 2} {
    foreach tag [$canvas itemcget _current_ -tags] {
      if {[string match "*_all" $tag]} {
        set  To [string range $tag 0 [expr {[string length $tag] -5}]]
        break
      }
    }
    set GUI_SystemAddDepend(Destination,$canvas) $To
    set From $GUI_SystemAddDepend(Source,$canvas)
    ## Check if Source and Destination Module is the same
    if {[string equal $To $From]} {
      gut_ShowTooltip $canvas $GUI_Text(OperationCancel) \
           .toolmes [expr {[winfo rootx $canvas] +4}] \
                    [expr {[winfo rooty $canvas] +4}]
      set GUI_SystemAddDepend(CallNu,$canvas) -1
      after 1000 "GUI_SystemAddDepend \"$system\" $canvas Cancel"
      return 0
    }
    ## Check if the New Arc Exists...
    set Arcs [lindex $GUI_Graphs($system) end]
    foreach arc $Arcs {
      if {[string equal [lindex $arc 0] $From] && \
          [string equal [lindex $arc 1] $To]} {
        gut_ShowTooltip $canvas $GUI_Text(DependencyExists) .toolmes \
          [expr {[winfo rootx $canvas] +4}] [expr {[winfo rooty $canvas] +4}]
        set GUI_SystemAddDepend(CallNu,$canvas) -1
        after 1000 "GUI_SystemAddDepend \"$system\" $canvas Cancel"
        return 0
      }
      ## Check for Cycles...
      if {[string equal [lindex $arc 1] $From] && \
          [string equal [lindex $arc 0] $To]} {
        if {[string equal [gut_question [format $GUI_Text(DependencyCycle?) \
          $From $To] {Confirm Dependency...} 1 [winfo toplevel $canvas]] no]} {
          gut_ShowTooltip $canvas $GUI_Text(OperationCancel) \
           .toolmes [expr {[winfo rootx $canvas] +4}] \
                    [expr {[winfo rooty $canvas] +4}]
          set GUI_SystemAddDepend(CallNu,$canvas) -1
          after 1000 "GUI_SystemAddDepend \"$system\" $canvas Cancel"
          return 0
        } else {
          break
        }
      }
    }
    lappend Arcs [list $From $To {}]
    set GUI_Graphs($system) [lreplace $GUI_Graphs($system) end end $Arcs]
    GUI_DrawSystem $canvas $system $From $To
    after 2000 "GUI_DrawSystem $canvas \"$system\""
    set GUI_SystemAddDepend(CallNu,$canvas) -1
    GUI_SystemAddDepend $system $canvas Done
    return 1
  }
  if {$GUI_SystemAddDepend(CallNu,$canvas) == -1} {
    ## Finish Up...
    destroy .toolmes
    bind $canvas <<MBR_1>> $GUI_SystemAddDepend(Bind,$canvas)
    foreach var [array names GUI_SystemAddDepend "*,$canvas"] {
      unset GUI_SystemAddDepend($var)
    }
    return 0
  }
};# GUI_SystemAddDepend

## GUI_SystemRemDepend
proc GUI_SystemRemDepend {system canvas} {
  global GUI_Graphs GUI_Text
  set selected [$canvas find withtag _current_arc_]
  if {![string length $selected]} {return}
  foreach tag [$canvas itemcget _current_arc_ -tags] {
    if {[string match From_* $tag]} {
      set From [string range $tag 5 end]
    }
    if {[string match To_* $tag]} {
      set To   [string range $tag 3 end]
    }
  }
  set Arcs {}
  foreach arc [lindex $GUI_Graphs($system) end] {
    if {!([string equal [lindex $arc 0] $From] && \
          [string equal [lindex $arc 1] $To])} {
      lappend Arcs $arc
    }
  }
  if {[string equal [gut_question [format $GUI_Text(RemoveDependency?) \
         $From $To] {Remove Dependency...} 1 [winfo toplevel $canvas]] yes]} {
    set GUI_Graphs($system) [lreplace $GUI_Graphs($system) end end $Arcs]
    GUI_DrawSystem $canvas $system
  }
  return [list $From $To]
};# GUI_SystemRemDepend

###############################################################################
## Module Pop-Up window...                                                   ##
###############################################################################

proc GUI_SystemModulePopUp {canvas module} {
  global GUI_Text GUI_Options GUI_Underline CDM_CurrentStatus
  ## Module Menu
  set win  [winfo toplevel $canvas]
  set system [string range $win 5 end]
  set menu $canvas.module$module; catch {destroy $menu}
  $canvas dtag current
  catch {GUI_SystemDeselect $canvas $system} error
  $canvas dtag _current_
  $canvas addtag _current_ withtag $module
  menu $menu -tearoff 0
  if {[catch {GUI_SystemGraph_GetSelectedModule $win} Selected]} {
    ## We have been called from system creation window. No popup can be
    ## defined. Simply return...
    return
  }
  GUI_LoadImage ITRun
  $menu add command -label $GUI_Text(EditModule) -underline \
    $GUI_Underline(EditModule) -command "catch \{GUI_EditModuleSrc $Selected\}"\
    -image ITEditModule -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(ModuleParameters) -underline \
    $GUI_Underline(ModuleParameters) -command \
    "$canvas addtag _current_ withtag $module;GUI_EditModuleParameters $win" \
    -image ITProperties -compound $GUI_Options(MenuImageLocation)
  $menu add command -label $GUI_Text(ViewModuleDirectory) -underline \
     $GUI_Underline(ViewModuleDirectory) -command "GUI_ViewModuleDir $Selected"\
     -image IFolderSmall -compound $GUI_Options(MenuImageLocation)
  $menu add command -label  $GUI_Text(ReloadModule) -command \
    "PUL_LoadPlugin \[list \[set creole_${Selected}_home\]\]
     GUI_RegisterTools" \
    -image ITReloadModule -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  set col [GUI_SystemWindowGetInfo $canvas Collection]
  set doc [GUI_SystemWindowGetInfo $canvas Document]
  $menu add command -label  $GUI_Text(RunSelectedComponent) -underline \
     $GUI_Underline(RunSelectedComponent) \
     -command [list creole_Ext_RunComponentTool $doc $col $Selected] \
     -image ITRun -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label  $GUI_Text(RemoveSystemModule) -underline \
     $GUI_Underline(RemoveSystemModule) \
     -command [list ::EditSystem::RemoveModuleFromSystem \
                    $canvas $system $Selected] \
     -image ITRemove -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(CompileModule) -underline \
    $GUI_Underline(CompileModule) -command \
    "::CompileComponentWizard::Init \[GUI_SystemGraph_GetSelectedModule $win\]"\
    -image ITCompile -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(CreateNewModule) -underline \
    $GUI_Underline(CreateNewModule) -command {::Component::Create} \
    -image ITCreateModule -compound $GUI_Options(MenuImageLocation)
  $menu add separator
  $menu add command -label $GUI_Text(ReloadAllModules) -underline \
    $GUI_Underline(ReloadAllModules) -command \
    {PUL_LoadPlugins;GUI_RegisterTools} \
    -image ITReLoadModules -compound $GUI_Options(MenuImageLocation)
  if {[info exists CDM_CurrentStatus(NowAnnotating)]} {
    set status $CDM_CurrentStatus(NowAnnotating) 
  }
  ## We define the following variable so as to prevent the menu from
  ## GUI_HandleMenu to be posted...
  set CDM_CurrentStatus(NowAnnotating) 1
  if {$GUI_Options(MenuImageLocation) == "none"} {
    ::GUI::RemoveAllImagesFromMenu $menu}
  tk_popup $menu [winfo pointerx $canvas] [winfo pointery $canvas]
  if {[info exists status]} {
    set CDM_CurrentStatus(NowAnnotating) $status
  } else {
    unset CDM_CurrentStatus(NowAnnotating)
  }
  return -code break
};# GUI_SystemModulePopUp

## GUI_SystemSelectArea
proc GUI_SystemSelectArea {canvas module x y} {
  if {![TkGUI_ExistsPriv PrevX]} {
    TkGUI_SetPriv PrevX $x
    TkGUI_SetPriv PrevY $y
    return
  }
  set selected [$canvas find withtag current]
  if {[llength $selected] != 0} {return}
  set x [$canvas canvasx $x]
  set y [$canvas canvasy $y]
  if {([TkGUI_AccessPriv PrevX] != $x) && ([TkGUI_AccessPriv PrevY] != $y)} {
    $canvas delete _my_area_tag_
    $canvas addtag _my_area_tag_ withtag [$canvas create rect \
            [TkGUI_AccessPriv PrevX] [TkGUI_AccessPriv PrevY] \
            $x $y -outline black -dash {6 4}]
    #::ScrolledWidgets::CanvasUpdateScrollRegion $canvas
  }
};# GUI_SystemSelectArea

## GUI_RedrawAllOpenedSystemWindows
proc GUI_RedrawAllOpenedSystemWindows {} {
  foreach win [winfo children .] {
    if {[string match *_system $win]} {
      set canvas $win.frame.canvas
      catch {set ::GUI_GraphInfo(SystemName_$canvas)} system
      catch {GUI_DrawSystem $canvas $system}
    }
  }
};# GUI_RedrawAllOpenedSystemWindows
