##############################################################################
##  ToolboxExplorer.tcl:                                                    ##
## ----------------------                                                   ##
##  This file implements an explorer for the loaded tools. The user can see ##
##  and execute all the available tools...                                  ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
 
namespace eval ::GUI_Toolbox {
  GUI_LoadImage ITIconBoxLarge
  GUI_LoadImage ITIconBoxList
  GUI_LoadImage ITIconBoxSmall
  GUI_LoadImage ToolBox
  variable ToolCategory
}

proc ::GUI_Toolbox::Explorer {canvas} {
  package require mkWidgets
  global GUI_Text GUI_Options GUI_ToolConfig
  variable ToolCategory

  set win $canvas.guiToolboxExplorerWindow
  catch {destroy $win}
  unset -nocomplain ToolCategory

  toplevel $win
  wm withdraw $win
  wm title $win {Toolbox Explorer...}

  ## Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  set TabIconBox $win.mainarea.iconbox.tabIconBox
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITIconBoxLarge -command "$TabIconBox config -view large"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITIconBoxSmall -command "$TabIconBox config -view small"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITIconBoxList  -command "$TabIconBox config -view list"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  ## Header Section...
  set TitleFrame $win.title
  frame $TitleFrame
    label $TitleFrame.image 
    label $TitleFrame.text -height 2 -anchor w -justify left
    grid $TitleFrame.image $TitleFrame.text \
      -sticky snew -padx 3 -pady 6
    grid columnconfigure $TitleFrame 1 -weight 1
  pack $TitleFrame -fill x -expand 1
  
  ## Our main area:
  ## Create a button area, and
  frame $win.cbuttons
  pack $win.cbuttons -side left -fill y -expand 1

  frame $win.mainarea
    ## Create an IconBox...
    frame $win.mainarea.iconbox
    set TIB $TabIconBox
    set ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($TIB.scrh) 0
    set ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($TIB.scrh_args)\
        [eval list -sticky snew -column 0 -row 2]
    set ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($TIB.scrv)   0
    set ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($TIB.scrv_args) \
        [eval list -sticky snew -column 1 -row 0 -rowspan 2]
      iconbox $TabIconBox -columns {{Category l 250} {Description l 500}} \
        -onselect ::GUI_Toolbox::SelectionChanged -cursor left_ptr
      $TIB configure -xscrollcommand \
          [list ::ScrolledWidgets::ScrollbarSet $TIB.scrh set] \
        -yscrollcommand \
          [list ::ScrolledWidgets::ScrollbarSet $TIB.scrv set]
      grid forget $TIB.scrh $TIB.scrv
      grid rowconfigure    $win.mainarea.iconbox 0 -weight 1
      grid columnconfigure $win.mainarea.iconbox 0 -weight 1
      grid $TabIconBox -row 0 -column 0 -sticky snew
    pack $win.mainarea.iconbox -side left -fill both -expand 1
  pack $win.mainarea -fill both -expand 1
  
  ## Button Section...
  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  button $win.buttons.exec -textvariable GUI_Text(Run)
  pack $win.buttons.exec $win.buttons.dismiss -side left -padx 10 -pady 10
  
  ## Load the Tools...
  foreach tool [array names GUI_ToolConfig] {
    set category Generic
    if {[regexp -- {([^:]+)::([^:]+)} $tool match Class Tool]} {
      set tool $Tool
      if {[string length $Class]} {set category $Class}
    } else {
      set tool [string trim $tool :]
    }
    lappend ToolCategory($category) $tool
  }

  ## Create the categories buttons...
  set index 0
  foreach category [lsort [array names ToolCategory]] {
    radiobutton $win.cbuttons.cat_$index -text $category -compound top \
      -indicatoron 0 -variable ::GUI_Toolbox::selectedButton($win) \
      -image IModule -relief flat -overrelief raised -bd 1 \
      -value $category -command \
      [list ::GUI_Toolbox::SelectionChanged $canvas]
    pack $win.cbuttons.cat_$index -side top -fill x -pady 0 -padx 2
    incr index
  }
  set ::GUI_Toolbox::selectedButton($win) Generic
  ::GUI_Toolbox::SelectionChanged $canvas
  update
  wm deiconify $win
};# ::GUI_Toolbox::Explorer

proc ::GUI_Toolbox::SelectionChanged {canvas args} {
  variable ToolCategory
  global GUI_Text
  set win $canvas.guiToolboxExplorerWindow
  set TabIconBox $win.mainarea.iconbox.tabIconBox
  
  ## Update the Header...
  foreach {col doc} {{} {}} {}
  catch {set col $::GUI_GraphInfo(CCollection_$canvas)}
  catch {set doc $::GUI_GraphInfo(CDocument_$canvas)}
  if {[string length $doc]} {
    $win.title.image conf -image IDocument
    $win.title.text conf -text "$GUI_Text(EditingDocument): [tip_GetId $doc]\
               \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
  } elseif {[string length $col]} {
    $win.title.image conf -image ICollection
    $win.title.text conf -text \
               "$GUI_Text(EditingCollection): [file tail [tip_GetName $col]]\
               \n([tip_GetName $col])"
  } else {
    $win.title.image conf -image ToolBox
    $win.title.text conf -text { }
  }
  set category $::GUI_Toolbox::selectedButton($win)

  $TabIconBox delete
  grid forget $TabIconBox.scrh $TabIconBox.scrv
  set index 0
  foreach tool [lsort $ToolCategory($category)] {
    if {[string equal $category Generic]} {
      set text [PUL_ToolConfig $tool title]
    } else {
      set text [PUL_ToolConfig $tool title $category]
    }
    $TabIconBox insert end obj$index -text $text \
      -image IModule -smallimage ITTool \
      -values [list $text]
    incr index
  }
  catch {$TabIconBox selection set obj0}
};# ::GUI_Toolbox::SelectionChanged
