##############################################################################
#
# balloon.tcl - procedures used by balloon help
#
# Copyright (C) 1998-2004 George Petasis, e-mail petasis@iit.demokritos.gr
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

##############################################################################
#
#global GUI_Options
#set GUI_Options(TooltipForeground) black
#set GUI_Options(TooltipBackground) #ffffaa

# Tooltips help.

namespace eval Tooltip {

variable Tooltip
 # 
 # Register a widget with a tooltip
 #
proc tooltip {widget args} {
  variable Tooltip
  set image {}
  foreach {opt val} $args {
    switch -glob -- $opt {
    -text   {set message $val}
    -textv* {
       if {![info exists ::$val]} {set ::$val {}}
       set message [set ::$val]
       trace variable ::$val w "Tooltip::ChangeMessage $widget"
    }
    -im*    {set image $val}
    default {return -code error "unknown option $opt"}
    }
  }
  if {[string equal $message {}]} return
  set Tooltip($widget) $message
  if {![string equal $image {}]} {set Tooltip(${widget}:Image) $image}
  bind $widget <Enter> {+
    set Tooltip::Tooltip(set) 0; set Tooltip::Tooltip(first) 1
    set Tooltip::Tooltip(id) [after 500 \
      {Tooltip::tooltip_show %W $Tooltip::Tooltip(%W)}]
  }
  bind $widget <Button> {+
    set Tooltip::Tooltip(first) 0
    Tooltip::kill_balloon
  }
  bind $widget <Leave> {+
    set Tooltip::Tooltip(first) 0
    Tooltip::kill_balloon
  }
  bind $widget <Motion> {+
    if {$Tooltip::Tooltip(set) == 0} {
      after cancel $Tooltip::Tooltip(id)
      set Tooltip::Tooltip(id) [after 500 \
          {Tooltip::tooltip_show %W $Tooltip::Tooltip(%W)}]
    }
  }
}
# Private function needeb by -textvariable option 
proc ChangeMessage {widget name1 name2 op} {
  variable Tooltip
  global $name1
  if {[string length $name2]} {
    set Tooltip($widget) [set "${name1}(${name2})"]
  } else {
    set Tooltip($widget) [set $name1]
  }
  return {}
}

 # 
 # Register a canvas item (!) with a tooltip
 #
proc canvas_tooltip {widget tag_or_id message {image {}}} {
  variable Tooltip
  if {![string length $message]} {return}
  set Tooltip($widget,:,$tag_or_id) [list $message]
  $widget bind $tag_or_id <Enter> "+
    set Tooltip::Tooltip(set) 0; set Tooltip::Tooltip(first) 1
    set Tooltip::Tooltip(id) \[after 2500 \
      \"Tooltip::tooltip_show %W \$Tooltip::Tooltip(%W,:,$tag_or_id) $image\"\]"
  $widget bind $tag_or_id <Button> \
    {+set Tooltip::Tooltip(first) 0;Tooltip::kill_balloon}
  $widget bind $tag_or_id <Leave> \
    {+set Tooltip::Tooltip(first) 0;Tooltip::kill_balloon}
  $widget bind $tag_or_id <Motion> "+
    if \{\$Tooltip::Tooltip(set) == 0\} \{
      after cancel \$Tooltip::Tooltip(id)
      set Tooltip::Tooltip(id) \[after 2500 \
       \"Tooltip::tooltip_show %W \$Tooltip::Tooltip(%W,:,$tag_or_id) $image\"\]
    \}"
}

# Private procedure to show tooltip.
proc tooltip_show {widget {text {}} {image {}} {win {.__tooltipWindow__hopeItsUnique}} args} {
  variable Tooltip
  ## Has this widget already a tooltip?
  if {![winfo exists $widget]} {return}

  set focus [focus]
  ## Has application lost the focus?
  if {![string length $focus]} {return}
  ## Is the focus still in the same toplevel?
  if {![string equal [winfo toplevel $widget] [winfo toplevel $focus]]} {
    ## If the class of the widget is a canvas, do not display tooltips if has
    ## lost the focus...
    if {[string equal Canvas [winfo class $widget]]} {return}
  }
  foreach {mx my} [winfo pointerxy $widget] {break}
  ## Is the mouse still in the same screen as the widget?
  if {$mx < 0 || $my < 0} {return}
  ## Is the mouse still in the widget?
  set rootx [winfo rootx $widget]; set rooty [winfo rooty $widget]
  if {$mx < $rootx || $my < $rooty} {return}
  if {$mx > $rootx+[winfo width  $widget] ||
      $my > $rooty+[winfo height $widget]} {
    return
  }

  if {$Tooltip(first) == 1} {
    global GUI_Options
    set Tooltip(first) 2
    set x [expr {[winfo rootx $widget] + ([winfo width $widget]/2)}]
    set y [expr {[winfo rooty $widget] + [winfo height $widget] + 4}]
    catch {destroy $win}
    toplevel $win -bg $GUI_Options(TooltipForeground) -highlightthickness 1 \
      -highlightbackground $GUI_Options(TooltipForeground) \
      -highlightcolor $GUI_Options(TooltipForeground)
    bind $win <Leave> "destroy $win"
    bind $win <Destroy> [list catch [list after cancel [after 8000 "destroy \
                                                              $win"]]]
    wm overrideredirect $win 1
    if {![string length $text]} {set text $Tooltip($widget)}

    label ${win}.l -text $text -relief flat -wraplength 350 -padx 2 -pady 0 \
      -bg $GUI_Options(TooltipBackground) \
      -fg $GUI_Options(TooltipForeground) -anchor w
    if {[string length $image]} {
      set image $image
    } elseif {[info exists Tooltip(${widget}:Image)]} {
      set image $Tooltip(${widget}:Image)
    } 
    if {[string length $image]} {
      label $win.img -bg $GUI_Options(TooltipBackground) -image $image -bd 0
      pack $win.img -side left -padx 0 -pady 0 -fill both -expand 1
    }
    pack $win.l -side left -padx 0 -pady 0 -fill both -expand 1
    wm geometry $win +$x+$y
    set Tooltip(set) 1
  }
  return $win
}

proc kill_balloon {} {
  variable Tooltip
  catch {after cancel $Tooltip(id)}
  if {[winfo exists .__tooltipWindow__hopeItsUnique]} {
    destroy .__tooltipWindow__hopeItsUnique
  }
  set Tooltip(set) 0
}

};# Namespace Tooltip
