#
# Creates a font dialog to select a font.
#
# Comment out test code at end of file for real usage.
#

#
# Creates a font dialog like that in Windows 95.
# returns 1 if user chose a font, 0 otherwise.
#
namespace eval ::FontDialog {
  variable fd_family 
  variable fd_style 
  variable fd_size 
  variable fd_close 
  variable fd_strikeout 
  variable fd_underline
  variable fd_show_widget
}

proc ::FontDialog::FontDialog {font_name {dlg .fontdialog_dialogs_instance}} {
    global GUI_Text GUI_Options
    variable fd_family 
    variable fd_style 
    variable fd_size 
    variable fd_close 
    variable fd_strikeout 
    variable fd_underline
    variable fd_show_widget

    set fd_family {}
    set fd_style  {}
    set fd_size   {}
    set fd_strikeout 0
    set fd_underline 0
    set fd_close  -1
    if {[winfo exists $dlg]} {return}

    set unsorted_fam [font families]

    set families [lsort $unsorted_fam]

    #
    # Get current font's family and so on.
    #
        # Calculate font style.
    set slant     [font actual $font_name -slant]
    set weight    [font actual $font_name -weight]

        # Default style.
    set fd_style "Regular"

    if { $slant == "italic" } {
        if { $weight == "bold" } {
            set fd_style "Bold Italic"
        } else {
            set fd_style "Italic"
        }
    } else {
        if { $weight == "bold" } {
            set fd_style "Bold"
        } 
    }

    set family [font actual $font_name -family]
    set fd_family $family
    set size   [font actual $font_name -size]
    set fd_size   $size
    set strikeout [font actual $font_name -underline]
    set fd_strikeout $strikeout
    set underline [font actual $font_name -overstrike]
    set fd_underline $underline

    #
    # Create font dialog.
    #
    set LocalFont [list $GUI_Options(ListBoxFont) $GUI_Options(ListBoxFontSize)]
    toplevel $dlg
    wm withdraw $dlg
    wm protocol $dlg WM_DELETE_WINDOW {set ::FontDialog::fd_close 0}

    wm title $dlg {Select Font...}
    wm transient $dlg 

    label $dlg.family_lbl -textvariable GUI_Text(Font:) -anchor w
    entry $dlg.family_ent -textvariable ::FontDialog::fd_family
    bind  $dlg.family_ent <Key-Return> \
         "::FontDialog::FontDialogRegen $font_name"
    grid config $dlg.family_lbl -column 0 -row 0 \
         -sticky w 
    grid config $dlg.family_ent -column 0 -row 1 \
         -sticky snew 


    label $dlg.style_lbl  -textvariable GUI_Text(FontStyle:) -anchor w
    entry $dlg.style_ent  -textvariable ::FontDialog::fd_style \
         -width 11
    bind  $dlg.style_ent  <Key-Return> \
         "::FontDialog::FontDialogRegen $font_name"
    grid config $dlg.style_lbl  -column 1 -row 0 \
         -sticky w 
    grid config $dlg.style_ent  -column 1 -row 1 \
         -sticky snew 

    label $dlg.size_lbl   -textvariable GUI_Text(Size:) -anchor w
    entry $dlg.size_ent   -textvariable ::FontDialog::fd_size -width 6
    bind  $dlg.size_ent   <Key-Return> \
         "::FontDialog::FontDialogRegen $font_name"
    grid config $dlg.size_lbl   -column 2 -row 0 \
         -sticky w
    grid config $dlg.size_ent   -column 2 -row 1 \
         -sticky snew 
 
    # Font family listbox.
    set fr $dlg.family_list
    ::ScrolledWidgets::ScrolledListbox $fr -widgetname list \
         -borderwidth $GUI_Options(BorderWidth) -scrollwidth 12 -width 40 \
         -height 6 -exportselection 0 -font $LocalFont
    #frame $fr -bd 0
    #listbox $fr.list -height 6 -selectmode single \
    #    -yscrollcommand "$fr.scroll set" \
    #     -width 40

    #scrollbar $fr.scroll -command "$fr.list yview" -borderwidth 1

    foreach f $families {
        $fr.list insert end $f
    }

    set Cmd \
     "::FontDialog::FontDialogFamily $fr.list $font_name $dlg.family_ent"
    foreach event {<Double-Button-1> <<MB_1>> <Key>} {
      bind $fr.list $event "+after idle $Cmd"
    }

    #pack $fr.scroll -side right -fill y
    #pack $fr.list -side left -fill both -expand 1
    grid config $fr -column 0 -row 2 -sticky snew

    # Font style listbox.
    set fr $dlg.style_list
    ::ScrolledWidgets::ScrolledListbox $fr -widgetname list \
         -borderwidth 1 -scrollwidth 12 -width 8 -height 6 -exportselection 0 \
	  -font $LocalFont
    #frame $fr -bd 0
    #listbox $fr.list -height 6 -selectmode single \
    #    -yscrollcommand "$fr.scroll set" \
    #    -width 11 

    #scrollbar $fr.scroll -command "$fr.list yview" -borderwidth 1

    foreach Style {Regular Bold Italic {Bold Italic}} {
      $fr.list insert end $Style
    }

    set Cmd \
     "::FontDialog::FontDialogStyle $fr.list $font_name $dlg.style_ent"
    foreach event {<Double-Button-1> <<MB_1>> <Key>} {
      bind $fr.list $event "+after idle $Cmd"
    }

    #pack $fr.scroll -side right -fill y
    #pack $fr.list -side left
    grid config $fr -column 1 -row 2 -sticky snew


    # Font size listbox.
    set fr $dlg.size_list
    ::ScrolledWidgets::ScrolledListbox $fr -widgetname list \
         -borderwidth 1 -scrollwidth 12 -width 4 -height 6 -exportselection 0 \
	  -font $LocalFont
    #frame $fr -bd 0
    #listbox $fr.list -height 6 -selectmode single \
    #    -yscrollcommand "$fr.scroll set" \
    #    -width 4

    #scrollbar $fr.scroll -command "$fr.list yview" -borderwidth 1

    set i 6
    while { $i <= 72 } {
        $fr.list insert end $i
        incr i 2
    }

    set Cmd \
     "::FontDialog::FontDialogSize $fr.list $font_name $dlg.size_ent"
    foreach event {<Double-Button-1> <<MB_1>> <Key>} {
      bind $fr.list $event "+after idle $Cmd"
    }

    #pack $fr.scroll -side right -fill y
    #pack $fr.list -side left
    grid config $fr -column 2 -row 2 -sticky snew
    grid columnconfigure $dlg 0 -weight 1
    grid rowconfigure $dlg 2 -weight 1

    # OK/Cancel
    set fr $dlg.ok_cancel
    frame $fr -bd 0

    button $fr.ok -textvariable GUI_Text(Ok) \
           -command "set ::FontDialog::fd_close 1"
    button $fr.cancel -textvariable GUI_Text(Cancel) \
         -command "set ::FontDialog::fd_close 0"
    pack $fr.ok -side top -fill x 
    pack $fr.cancel -side top -fill x -pady 2
    grid config $fr -column 4 -row 1 -rowspan 2 \
         -sticky snew -padx 12

    # Effects.
    set fr $dlg.effects
    frame $fr -relief groove -borderwidth 2

    label $fr.effects -textvariable GUI_Text(Effects) -anchor w
    checkbutton $fr.strikeout -variable ::FontDialog::fd_strikeout \
        -textvariable GUI_Text(Strikeout) -anchor w \
        -command "::FontDialog::FontDialogRegen $font_name"
    checkbutton $fr.underline -variable ::FontDialog::fd_underline \
        -textvariable GUI_Text(Underline) -anchor w \
        -command "::FontDialog::FontDialogRegen $font_name"
    #label $fr.tmp1 -text "  " -anchor w
    #label $fr.tmp2 -text "  " -anchor w
    #label $fr.tmp3 -text "  " -anchor w

    pack $fr.effects $fr.strikeout $fr.underline \
         -side top -fill x
    #pack $fr.tmp1 $fr.tmp2 $fr.tmp3

    grid config $fr -column 1 -columnspan 2 -row 3 -sticky snew -pady 10 

    # Sample text
    set fr $dlg.sample
    frame $fr -relief groove -borderwidth 2 -height 100
    label $fr.l_sample -textvariable GUI_Text(Sample) -anchor w

    entry $fr.sample -width 20 -justify center \
        -font $font_name -bd 0 -relief flat -bg [$fr.l_sample cget -bg]
    $fr.sample insert end $GUI_Text(TestString...)
    focus $fr.sample
    set fd_show_widget $fr.sample

    pack  $fr.l_sample -side top -fill x -pady 4
    pack  $fr.sample -side top -pady 4 -padx 15 -ipady 20 -fill x -expand 1
    pack propagate $fr 0

    grid config $fr -column 0 -row 3 -sticky snew -pady 10 -padx 2
    #pack propagate $fr 0

    update
    gut_ModalDlg $dlg
    wm deiconify $dlg
    # Make this a modal dialog.
    tkwait variable ::FontDialog::fd_close

    # Get rid of dialog and return value.
    destroy $dlg

    #
    # Restore old font characteristics on a cancel.
    #
    if { $fd_close == 0 } {
            font configure $font_name -family $family \
                    -size $size -underline $underline \
                    -overstrike $strikeout -slant $slant \
                    -weight $weight
    }

    return $fd_close
}


proc ::FontDialog::FontDialogFamily { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        ::FontDialog::FontDialogRegen $font_name
    }
}


proc ::FontDialog::FontDialogStyle { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        ::FontDialog::FontDialogRegen $font_name
    }
}



proc ::FontDialog::FontDialogSize { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        ::FontDialog::FontDialogRegen $font_name
    }
}




    # Regenerates font from attributes.
proc ::FontDialog::FontDialogRegen { font_name {widget {}} } {
    variable fd_family 
    variable fd_style 
    variable fd_size 
    variable fd_close 
    variable fd_strikeout 
    variable fd_underline

    set weight "normal"
    if { $fd_style == "Bold Italic" || $fd_style == "Bold" } {
        set weight "bold"
    }

    set slant "roman"
    if { $fd_style == "Bold Italic" || $fd_style == "Italic" } {
        set slant "italic"
    }

    #
    # Change font to have new characteristics.
    #
    font configure $font_name -family $fd_family \
        -size $fd_size -underline $fd_underline \
        -overstrike $fd_strikeout -slant $slant \
        -weight $weight
    variable fd_show_widget
    lappend nfontlist $fd_family $fd_size $slant $weight
    if {$fd_underline == 1} {lappend nfontlist underline}
    if {$fd_strikeout == 1} {lappend nfontlist overstrike}
    if {![string length $widget]} {
      $fd_show_widget configure -font $nfontlist
    } else {
      catch {$widget configure -font $nfontlist}
    }
}



# Test code. Comment out for real usage.
proc ::FontDialog::FontSelect { widget {win {}}} {
  variable fd_family 
  variable fd_style 
  variable fd_size 
  variable fd_close 
  variable fd_strikeout 
  variable fd_underline
  variable fd_show_widget
  set test_font __test__font; set delete_font 1

  set cfont [$widget cget -font]
  if {[llength $cfont] == 1} {
    if {[catch {font actual $cfont -display $widget -family} family]} {
      lappend cfontlist -family $cfont 
    } else {
      set test_font $cfont; set delete_font 0
      lappend cfontlist -family [font actual $cfont -display $widget -family]
      lappend cfontlist -size   [font actual $cfont -display $widget -size]
      lappend cfontlist -weight [font actual $cfont -display $widget -weight]
      lappend cfontlist -slant  [font actual $cfont -display $widget -slant]
      lappend cfontlist -underline \
              [font actual $cfont -display $widget -underline]
      lappend cfontlist -overstrike \
              [font actual $cfont -display $widget -overstrike]
    }
  } else {
    lappend cfontlist -family [lindex $cfont 0] -size [lindex $cfont 1]
    foreach attr [lrange $cfont 2 end] {
      switch -exact -- attr {
        normal      {lappend cfontlist -weight normal}
        roman       {lappend cfontlist -slant roman} 
        bold        {lappend cfontlist -weight bold}
        italic      {lappend cfontlist -slant italic}
        underline   {lappend cfontlist -underline 1}
        overstrike  {lappend cfontlist -overstrike 1}
      }
    }
  }
    # Create a dummy font for use.
    if {$delete_font == 1} {
      eval font create $test_font $cfontlist
    } else {
      eval font configure $test_font $cfontlist
    }

    # Ask user to select font.
    set status [::FontDialog::FontDialog $test_font $win]

    if { $status == 1} {
      ::FontDialog::FontDialogRegen $test_font $widget
      if {[string equal [winfo class $widget] Text]} {
        global GUI_Options
        set GUI_Options(TextFontStyles) {}
        foreach {opt val} [font actual $test_font] {
          switch -exact -- $opt {
            -family     {set GUI_Options(TextFont) $val}
            -size       {set GUI_Options(TextFontSize) $val}
            -weight     {lappend GUI_Options(TextFontStyles) -weight $val}
            -slant      {lappend GUI_Options(TextFontStyles) -slant  $val}
            -underline  {if {$val} {
                           lappend GUI_Options(TextFontStyles) -underline 1
                         }
            }
            -overstrike {if {$val} {
                           lappend GUI_Options(TextFontStyles) -overstrike 1
                        }
            }
          }
        }
      }
    }

    if {$delete_font == 1} {font delete $test_font}
}

# fontdlg.tcl

