#!/usr/bin/wish
#########################################################################
##  
##  This file implements the ImageButton widget. That means a button that
##  can display an image and text at the same time...
##
#########################################################################

#
# Copyright (C) 2000 George P. Petasis
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   petasis@iit.demokritos.gr
#   http://www.iit.demokritos.gr/~petasis
#
# $Revision: 1.0 $
#

namespace eval ImageButton {
  variable ImageButton
  ## Initialise Widget...
  button .__imageButton;# only for getting defaults
  foreach one {-background -foreground \
               -activebackground -activeforeground -font} {
    set ImageButton($one) [.__imageButton cget $one]
  }
  destroy .__imageButton

  proc Button {w args} {
    variable ImageButton
    global GUI_Options

    foreach {image bitmap side command expand ipadx ipady tooltip tooltipvar} \
            {{} {} left {} 1 2 2 {} {}} {}
    foreach {bg fg relief abg afg} \
      [list $ImageButton(-background) $ImageButton(-foreground) raised \
      $ImageButton(-activebackground) $ImageButton(-activeforeground)] break
    set button_args [list -bg $bg -font $ImageButton(-font)]
    set bd $GUI_Options(BorderWidth)
    set frame_args  {}
    foreach {arg val} $args {
      switch -glob -- $arg {
        -bitmap      {set bitmap  $val}
	-image       {set image   $val}
	-side        {set side    $val}
	-command     {set command $val}
	-expand      {set expand  $val}
	-activebackground {set abg $val}
	-activeforeground {set afg $val}
	-relief      {set relief $val}
	-borderwidth -
	-bd          {set bd $val}
	-borderwidth {lappend frame_args  $arg $val}
	-bg          -
	-background  {lappend frame_args  $arg $val
	              lappend button_args $arg $val
		      set bg $val}
        -foreground  {lappend button_args $arg $val
		      set fg $val}
	-ipadx       {set ipadx $val}
	-ipady       {set ipady $val}
	-tooltip     {set tooltip $val}
	-tooltipvar* {set tooltipvar $val}
	default {lappend button_args $arg $val}
      }
    }
	
    eval frame $w $frame_args -relief $relief -bg $bg -bd $bd
    if {[string length $image]} {
        label $w.b -image $image -bg $bg
    } elseif {[string length $bitmap]} {
        label $w.b -bitmap $bitmap -bg $bg
    }
    eval label $w.t $button_args -bg $bg
    pack $w.b $w.t -side $side -fill both -expand $expand \
                   -ipadx $ipadx -ipady $ipady
    Bind $w <Enter> \
      "ImageButton::Configure %W -bg $abg -fg $afg;$w configure -relief raised;update"
    Bind $w <Leave> \
      "ImageButton::Configure %W -bg $bg -fg $fg;$w configure -relief $relief;update"
    Bind $w <ButtonPress-1> \
      "$w configure -relief sunken;update"
    Bind $w <ButtonRelease-1> \
      "if \{\[string equal sunken \[$w cget -relief\]\]\} \{
         eval [list $command];catch \{$w configure -relief $relief\}\}"
    if {[string length $tooltipvar]} {
      Tooltip::tooltip $w -textvariable $tooltipvar
    } elseif {[string length $tooltip]} {
      Tooltip::tooltip $w -text $tooltip
    }
  }
  
  proc Bind {w event body} {
    if ![llength [winfo children $w]] {set w [winfo parent $w]}
    foreach i [concat $w [winfo children $w]] {
        bind $i $event $body
    }
  } ;# binds to children and parent
  
  proc Configure {w args} {
    if ![llength [winfo children $w]] {set w [winfo parent $w]}
    foreach i [concat $w [winfo children $w]] {
        catch {eval $i configure $args}
    }
  }
} 
