##############################################################################
#
# ImageProgressBar.tcl - procedures used for creating a progressbar
#
# Copyright (C) 1998-2004 George Petasis, e-mail petasis@iit.demokritos.gr
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###############################################################################
###### ImageProgress Widget
###############################################################################
option add *ImageProgress.leftEmptyCornerImage \
                     ImageProgress::LeftEmptyCorner  widgetDefault
option add *ImageProgress.rightEmptyCornerImage \
                     ImageProgress::RightEmptyCorner widgetDefault
option add *ImageProgress.emptyTileImage \
                     ImageProgress::EmptyTile        widgetDefault
option add *ImageProgress.filledTileImage \
                     ImageProgress::FilledTile       widgetDefault
option add *ImageProgress.egdePixelOffset       4    widgetDefault
option add *ImageProgress.tilePixelSpacing      2    widgetDefault
option add *ImageProgress.tileHeight           23    widgetDefault
option add *ImageProgress.tileWidth             6    widgetDefault
option add *ImageProgress.tileNumber           25    widgetDefault

namespace eval ImageProgress {
  namespace export ImageProgress SetImageProgress

  proc ImageProgress {w args} {
    uplevel 1 [list frame $w -class ImageProgress] $args
    ## Retrieve defaults from the option database...
    foreach {val} {leftEmptyCornerImage rightEmptyCornerImage
                   emptyTileImage       filledTileImage
                   egdePixelOffset      tilePixelSpacing
                   tileHeight           tileWidth   tileNumber} {
      set class [string toupper $val][string range $val 0 end]
      set $val [option get $w $val $class]
    }
    set width [expr {$tileNumber*$tileWidth-1+
                     ($tileNumber-1)*$tilePixelSpacing+2*$egdePixelOffset}]
    canvas $w.c -height $tileHeight -width $width -bg white \
      -bd 0 -relief flat -highlightthickness 0
    PlaceTiles $w $width

    SetImageProgress $w 0
    pack $w.c -padx 0 -pady 0 -fill x
    bind $w.c <Configure> [namespace code [list PlaceTiles $w "%w"]]
    return $w
  }

  proc PlaceTiles {w width} {
    ## Retrieve defaults from the option database...
    foreach {val} {leftEmptyCornerImage rightEmptyCornerImage
                   emptyTileImage       filledTileImage
                   egdePixelOffset      tilePixelSpacing
                   tileHeight           tileWidth   tileNumber} {
      set class [string toupper $val][string range $val 0 end]
      set $val [option get $w $val $class]
    }
    ## Place items in canvas...
    $w.c delete all
    ## Fill background:
    set max_x [expr {$width-$egdePixelOffset-$tileWidth}]
    for {set x $egdePixelOffset} {$x < $max_x} {incr x $tileWidth} {
      $w.c create image $x 0 -anchor nw -image $emptyTileImage
    }
    $w.c create image [expr {$width-$egdePixelOffset}] 0 \
      -anchor ne -image $emptyTileImage
    ## Place endings...
    set end_x [expr {$width-[image width $rightEmptyCornerImage]}]
    $w.c create image 0 0 -anchor nw -image $leftEmptyCornerImage
    $w.c create image $end_x 0 -anchor nw -image $rightEmptyCornerImage
    ## Finally, create the filled tiles...
    set step [expr {$tileWidth+$tilePixelSpacing}]
    incr max_x
    for {set x $egdePixelOffset;set i 0} {$x <= $max_x} {incr x $step;incr i} {
      $w.c create image $x 0 -anchor nw -tags "tile_$i tile"
    }
    variable ImageProgressInfo
    if {[info exists ImageProgressInfo($w)]} {
      SetImageProgress $w $ImageProgressInfo($w)
    }
  }

  proc SetImageProgress {win value {range 100}} {
    ## How many tiles are in the progressbar?
    if {[catch {llength [$win.c find withtag tile]]} tag_nu]} {return}
    variable ImageProgressInfo
    set ImageProgressInfo($win) $value
    set filledTileImage [option get $win filledTileImage {}]
    set emptyTileImage  [option get $win emptyTileImage  {}]
    set relwidth [expr {double($value)/double($range)}]
    if {$relwidth > 1.0} {set relwidth 1.0}
    set up_to_tile [expr {int($relwidth*$tag_nu)}]
    for {set tile 0} {$tile < $up_to_tile} {incr tile} {
      $win.c itemconfigure tile_$tile -image $filledTileImage
    }
    for {set tile $up_to_tile} {$tile < $tag_nu} {incr tile} {
      $win.c itemconfigure tile_$tile -image $emptyTileImage
    }
  }
}

image create photo ImageProgress::LeftEmptyCorner -data \
{R0lGODlhBQAXAOMAAKyrp39+fWhoaKyrpnd3d76+vu/v7/////v7+/j4+PX19fPz8/Dw8PLy8vr6
+v///yH5BAEKAA8ALAAAAAAFABcAAAQt8IEgxCAlh2yMKN3xGQcylmdyKudyMqcHHs2ans5pgqTu
z4lNCEDoGB4DiiACADs=}
image create photo ImageProgress::RightEmptyCorner -data \
{R0lGODlhBAAXAIQAAGhoaL6+vrOzs+/v74GBgf////v7+/j4+PX19fPz8/Dw8PLy8vr6+n9+fXd3
d6yrpqyrp////////////////////////////////////////////////////////////yH5BAEK
AB8ALAAAAAAEABcAAAUtICB+QQAIwxAQRQoY7RsfMRInsRIP8VLPMUYMNgC0ikRZkVY8NlKOB6AB
+YQAADs=}
image create photo ImageProgress::EmptyTile -data \
{R0lGODlhBgAXAMIAAGhoaL6+vu/v7////////////////////yH5BAEKAAQALAAAAAAGABcAAAMT
CLrRLjCOSau9OOvNu/9UFC1LAgA7}
image create photo ImageProgress::FilledTile -data \
{R0lGODlhBgAXAIQAAGhoaL6+vu/v76vtrJXplpTplXzkfnvjfWbfaE3aUE7aUDXVODXVNynSLCjS
Ky7TMS/TMkHXRELXRVvdXVvdXnbid3bieHXid4/okI7nkP///////////////////////yH5BAEK
AB8ALAAAAAAGABcAAAVEICCOQWkKaDqsLEEULmEYx2wgeJ4ovJIsQCCj4SgSiUYHUslMLo9F5mM6
hUSukohkQpl4KRXLZVzBYDLojGbNTqVGoxAAOw==}

##namespace import ImageProgress::ImageProgress ImageProgress::SetImageProgress
#proc ImageProgress {win args} {
#  eval ImageProgress::ImageProgress $win $args
#}
#
#proc SetImageProgress {win value {range 100}} {
#  ImageProgress::SetImageProgress $win $value $range
#}
#
### Test code...
#pack [ImageProgress .p ] -fill both -expand 1 -padx 10 -pady 10
#pack [ImageProgress .p2] -fill y    -expand 1 -padx 10 -pady 10
#pack [ImageProgress .p3] -fill both -expand 1 -padx 10 -pady 10
#set ImageProgress 0
#proc Show {} {
#  global ImageProgress; incr ImageProgress
#  if {$ImageProgress > 100} {set ImageProgress 0}
#  SetImageProgress .p $ImageProgress
#  SetImageProgress .p2 $ImageProgress
#  SetImageProgress .p3 $ImageProgress
#  after 100 Show
#}
#Show
