##
## Keyboard.tcl --
##
##   A virtual keyboard widget implementation.
##
## Copyright (c) 1999-2006 by:
##   George Petasis,
##   Institute of Informatics and Telecommunications,
##   National Centre for Scientific Research "Demokritos",
##   Aghia Paraskevi, Athens, Greece.
##
## Author contact information:
##   e-mail: petasis@iit.demokritos.gr
##   URL:    http://www.iit.demokritos.gr/~petasis
##
##
## The following terms apply to all files associated
## with the software unless explicitly disclaimed in individual files.
##
## The authors hereby grant permission to use, copy, modify, distribute,
## and license this software and its documentation for any purpose, provided
## that existing copyright notices are retained in all copies and that this
## notice is included verbatim in any distributions. No written agreement,
## license, or royalty fee is required for any of the authorized uses.
## Modifications to this software may be copyrighted by their authors
## and need not follow the licensing terms described here, provided that
## the new terms are clearly indicated on the first page of each file where
## they apply.
## 
## IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
## FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
## ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
## DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
## 
## THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
## INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
## IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
## NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
## MODIFICATIONS.
##
package provide virtualKeyboard 1.0

namespace eval virtualKeyboard {
  ## The format for providing an additional keyboard is simple:
  ## 1) Create an array entry named after the language you are adding.
  ## 2) For each character you want to add, add an entry similar to the
  ##    following:
  ##  english_key_on_keyboard {label_to_be_shown character_to_be_inserted}
  ## 3) In case a character is generated by a sequence of characters,
  ##    this sequence must be specified in english_key_on_keyboard AND
  ##    all characters except the last one must be also placed in an entry named
  ##    specifierKeys. Also note that keys declared as specifier keys can
  ##    additionaly operate as normal keys.
  ## 4) The keys Control, Alt and AltGr can be used in character definitions by
  ##    using the following codes:
  ##           Control    Alt   AltGr
  ##    Left      \x01   \x04    \x07   (Not Implemented)
  ##    Right     \x02   \x05    \x08   (Not Implemented)
  ##    Any       \x03   \x06    \x0E
  ##    Please note that after the "\x??" code, you have to escape numeric keys
  ##    and the hex numbers a, b, c, d, e, f, as they will be simply appended to
  ##    the end of the "\x??" code. Instead, you can use the following maps:
  ##    \x??0 -> \x??\u0030 \x??1 -> \x??\u0031 \x??2 -> \x??\u0032
  ##    \x??3 -> \x??\u0033 \x??4 -> \x??\u0034 \x??5 -> \x??\u0035
  ##    \x??6 -> \x??\u0036 \x??7 -> \x??\u0037 \x??8 -> \x??\u0038
  ##    \x??9 -> \x??\u0039
  ##    \x??a -> \x??\u0061 \x??b -> \x??\u0062 \x??a -> \x??\c
  ##    \x??d -> \x??\d     \x??e -> \x??\e     \x??f -> \x??\u0066
  ##
  ## A good example for these features is the Greek entry for the Hellenic
  ## language.
  
  set KeyboardConfig(English) {
    a {a a} b {b b} c {c c} d {d d} e {e e} f {f f} g {g g} h {h h} i {i i}
    j {j j} k {k k} l {l l} m {m m} n {n n} o {o o} p {p p} q {q q} r {r r}
    s {s s} t {t t} u {u u} v {v v} w {w w} x {x x} y {y y} z {z z}
    A {A A} B {B B} C {C C} D {D D} E {E E} F {F F} G {G G} H {H H} I {I I}
    J {J J} K {K K} L {L L} M {M M} N {N N} O {O O} P {P P} Q {Q Q} R {R R}
    S {S S} T {T T} U {U U} V {V V} W {W W} X {X X} Y {Y Y} Z {Z Z}
    \x03\` {^C \u0003}     \x03\u0032 {^A \u0001} \x03\u0033 {Esc \u001b}
    \x03\u0034 {^\\\\ \u001c}                     \x03\u0035 {^] \u001d}
    \x03\u0036 {^^ \u001e} \x03\u0037 {^_ \u001f} \x03\u0038 {^H \u0008}
    \x03\u0030 {^0 \u00ff} \x03\u0061 {^A \u0001} \x03\u0062 {^B \u0002}
    \x03\[ {^\[ \u001b}    \x03\] {^] \u001d}     \x03\c {^C \u0003}
    \x03\d {^D \u0004}     \x03\e {^E \u0005}     \x03\u0066 {^F \u0006}
    \x03\g {^G \u0007}     \x03\h {^H \u0008}     \x03\i {^I \u0009}
    \x03\j {^@ \u0000}     \x03\k {^K \u000b}     \x03\l {^L \u000c}
    \x03\m {^M \u000d}     \x03n  {^N \u000e}     \x03\o {^O \u000f}
    \x03\p {^P \u0010}     \x03\q {^Q \u0011}     \x03r  {^R \u0012}
    \x03\s {^S \u0013}     \x03t  {^T \u0014}     \x03\u {^U \u0015}
    \x03v  {^V \u0016}     \x03\w {^W \u0017}     \x03\x {^X \u0018}
    \x03\y {^Y \u0019}     \x03\z {^Z \u001A}
    \x0E\e {\u20ac \u20ac}
    specifierKeys {\x01 \x02 \x03 \x04 \x05 \x06 \x07 \x08 \x0E}
  }

  set KeyboardConfig(Greek) {
    a {\u03b1 \u03b1}   b {\u03b2 \u03b2}  c {\u03c8 \u03c8}  d {\u03b4 \u03b4}
    e {\u03b5 \u03b5}   f {\u03c6 \u03c6}  g {\u03b3 \u03b3}  h {\u03b7 \u03b7}
    i {\u03b9 \u03b9}   j {\u03be \u03be}  k {\u03ba \u03ba}  l {\u03bb \u03bb}
    m {\u03bc \u03bc}   n {\u03bd \u03bd}  o {\u03bf \u03bf}  p {\u03c0 \u03c0}
    q {\;     \;    }   r {\u03c1 \u03c1}  s {\u03c3 \u03c3}  t {\u03c4 \u03c4}
    u {\u03b8 \u03b8}   v {\u03c9 \u03c9}  w {\u03c2 \u03c2}  x {\u03c7 \u03c7}
    y {\u03c5 \u03c5}   z {\u03b6 \u03b6}
   ;a {\u03ac \u03ac}  ;e {\u03ad \u03ad} ;h {\u03ae \u03ae} ;i {\u03af \u03af}
   ;o {\u03cc \u03cc}  ;y {\u03cd \u03cd} ;v {\u03ce \u03ce}
   :i {\u03ca \u03ca} ;:i {\u0390 \u0390}
   :y {\u03cb \u03cb} ;:y {\u03b0 \u03b0}
    
    A {\u0391 \u0391}   B {\u0392 \u0392}  C {\u03a8 \u03a8}  D {\u0394 \u0394}
    E {\u0395 \u0395}   F {\u03a6 \u03a6}  G {\u0393 \u0393}  H {\u0397 \u0397}
    I {\u0399 \u0399}   J {\u039e \u039e}  K {\u039a \u039a}  L {\u039b \u039b}
    M {\u039c \u039c}   N {\u039d \u039d}  O {\u039f \u039f}  P {\u03a0 \u03a0}
    Q {:      :     }   R {\u03a1 \u03a1}  S {\u03a3 \u03a3}  T {\u03a4 \u03a4}
    U {\u0398 \u0398}   V {\u03a9 \u03a9}  W {\u03a3 \u03a3}  X {\u03a7 \u03a7}
    Y {\u03a5 \u03a5}   Z {\u0396 \u0396}
   ;A {\u0386 \u0386}  ;E {\u0388 \u0388} ;H {\u0389 \u0389} ;I {\u038a \u038a}
   ;O {\u038c \u038c}  ;Y {\u038e \u038e} ;V {\u038f \u038f}
   :I {\u03aa \u03aa} ;:I {\u03aa \u03aa}
   :Y {\u03ab \u03ab} ;:Y {\u03ab \u03ab}
    specifierKeys {\x01 \x02 \x03 \x04 \x05 \x06 \x07 \x08 \x0E ; :}
  }

  set KeyboardConfig(GermanTest) {
    \x0E\u0032 {\u00b2 \u00b2} \x0E\u0033 {\u00b3 \u00b3}
    \x0E\u0037 {\\\{ \\\{}     \x0E\u0038 {\[ \[}
    \x0E\u0039 {\] \]}         \x0E\u0030 {\\\} \\\}}
    \x0E-      {\\\\ \\\\}     \x0E\q     {@ @}
    \x0E\e     {\u20ac \u20ac} \x0E+      {~ ~}
    \x0E,      {| |}           \x0EM      {\u0385 \u0385}
    specifierKeys {\x01 \x02 \x03 \x04 \x05 \x06 \x07 \x08 \x0E}
  }

  set KeyboardConfig(SwedishTest) {
    \x0E\e {\u20ac \u20ac}
    \x0E\u0037 {\\\{ \\\{}
    \x0E\u0030 {\\\} \\\}}
    \x0E\] {\] \]}
    \x0E\]o {\u00f5 \u00f5}
    \x0E\- {\\\\ \\\\}
    specifierKeys {\x0E \x0E\]}
  }

  set KeyboardConfig(Igbo) {
    a {a a} b {b b} c {c c} d {d d} e {e e} f {f f} g {g g} h {h h} i {i i}
    j {j j} k {k k} l {l l} m {m m} n {n n} o {o o} p {p p} q {q q} r {r r}
    s {s s} t {t t} u {u u} v {v v} w {w w} x {x x} y {y y} z {z z}
    A {A A} B {B B} C {C C} D {D D} E {E E} F {F F} G {G G} H {H H} I {I I}
    J {J J} K {K K} L {L L} M {M M} N {N N} O {O O} P {P P} Q {Q Q} R {R R}
    S {S S} T {T T} U {U U} V {V V} W {W W} X {X X} Y {Y Y} Z {Z Z}
    \- {\u1E45 \u1E45} \= {\u0301 \u0301} \[ {\u1EE5 \u1EE5} \] {\u0304 \u0304}
    \; {\u1ECD \u1ECD} \' {\u1ECB \u1ECB} \\\\ {\' \'}
    \_ {\u1E44 \u1E44} \+ {\u0300 \u0300} \\\{ {\u1EE4 \u1EE4} \\\} {\~ \~}
    \: {\u1ECC \u1ECC} \\\" {\u1ECA \u1ECA} \\\| {\\\" \\\"}
    \< {\; \;} \> {\: \:}
    \x03\` {^C \u0003}     \x03\u0032 {^A \u0001} \x03\u0033 {Esc \u001b}
    \x03\u0034 {^\\\\ \u001c}                     \x03\u0035 {^] \u001d}
    \x03\u0036 {^^ \u001e} \x03\u0037 {^_ \u001f} \x03\u0038 {^H \u0008}
    \x03\u0030 {^0 \u00ff} \x03\u0061 {^A \u0001} \x03\u0062 {^B \u0002}
    \x03\[ {^\[ \u001b}    \x03\] {^] \u001d}     \x03\c {^C \u0003}
    \x03\d {^D \u0004}     \x03\e {^E \u0005}     \x03\u0066 {^F \u0006}
    \x03\g {^G \u0007}     \x03\h {^H \u0008}     \x03\i {^I \u0009}
    \x03\j {^@ \u0000}     \x03\k {^K \u000b}     \x03\l {^L \u000c}
    \x03\m {^M \u000d}     \x03n  {^N \u000e}     \x03\o {^O \u000f}
    \x03\p {^P \u0010}     \x03\q {^Q \u0011}     \x03r  {^R \u0012}
    \x03\s {^S \u0013}     \x03t  {^T \u0014}     \x03\u {^U \u0015}
    \x03v  {^V \u0016}     \x03\w {^W \u0017}     \x03\x {^X \u0018}
    \x03\y {^Y \u0019}     \x03\z {^Z \u001A}
    \x0E\e {\u20ac \u20ac}
    specifierKeys {\x01 \x02 \x03 \x04 \x05 \x06 \x07 \x08 \x0E}
  }
  
  ## virtualKeyboard:
  #  Creates a virtual keyboard.
  proc virtualKeyboard {win args} {
    variable KeyboardConfig
    variable $win
    set font {arial 12 bold}
    set w_1 3
    set w_2 5
    set w_3 7
    set w_4 9
    set w_5 30
    set language Greek
    set receiver {}
    ## Process options...
    foreach {opt val} $args {
      switch -glob -- $opt {
        -receiver {set receiver $val}
        default {error "unknown option \"$opt\""}
      }
    }
    ## Save settings...
    set ${win}(Language) $language
    set ${win}(Shift)    1
    set ${win}(Control)  0
    set ${win}(Alt)      0
    set ${win}(Altgr)    0
    set ${win}(Capitals) 0
    set ${win}(Receiver) $receiver
    bind <Destroy> $win "namespace code {unset $win}"
    frame $win -class VirtualKeyboard
    bind $win <KeyPress>   "virtualKeyboard::OnKeyPress   $win %A %K %s"
    bind $win <KeyRelease> "virtualKeyboard::OnKeyRelease $win %A %K %s"
    bind $win <Expose>     "focus $win"
    ## First row of buttons:
    frame $win.row1
    foreach c {` 1 2 3 4 5 6 7 8 9 0 - =} {
      pack [button $win.row1.[CharacterToName $c] -takefocus 0 \
                   -text $c -width $w_1 -font $font] \
                   -side left -padx 1 -pady 1
    }
    pack [button $win.row1.[CharacterToName \b] -takefocus 0 \
                   -text Backspace -font $font \
                   -command "[namespace current]::Backspace $win"] \
                   -side left -padx 1 -pady 1 -fill x -expand 1
    pack $win.row1 -fill x -expand 1
    ## Second row of buttons:
    frame $win.row2
    pack [button $win.row2.[CharacterToName \t] -takefocus 0 \
                   -text Tab -width $w_2 -font $font \
                   -command [list [namespace current]::Insert $win \t \t]] \
                   -side left -padx 1 -pady 1
    foreach c {q w e r t y u i o p [ ]} {
      pack [button $win.row2.[CharacterToName $c] -takefocus 0 \
                   -text $c -width $w_1 -font $font] \
                   -side left -padx 1 -pady 1
    }
    pack [button $win.row2.[CharacterToName delete] -takefocus 0 \
                   -text Del -width $w_3 -font $font \
                   -command "[namespace current]::Delete $win"] \
                   -side left -padx 1 -pady 1 -fill x -expand 1
    pack $win.row2 -fill x -expand 1
    ## Third row of buttons:
    frame $win.row3
    pack [button $win.row3.[CharacterToName caps_lock] -takefocus 0 \
                   -text Caps -width $w_3 -font $font \
                   -command "[namespace current]::ToggleCapitals $win"] \
                   -side left -padx 1 -pady 1
    foreach c {a s d f g h j k l ; '} {
      pack [button $win.row3.[CharacterToName $c] -takefocus 0 \
                   -text $c -width $w_1 -font $font] \
                   -side left -padx 1 -pady 1
    }
    pack [button $win.row3.[CharacterToName \n] -takefocus 0 \
                   -text Return -width $w_4 -font $font \
                   -command [list [namespace current]::Insert $win \n \n]] \
                   -side left -padx 1 -pady 1 -fill x -expand 1
    pack $win.row3 -fill x -expand 1
    ## Fourth row of buttons:
    frame $win.row4
    pack [button $win.row4.[CharacterToName shift_l] -takefocus 0 \
                   -text Shift -width $w_4 -font $font \
                   -command "[namespace current]::ToggleShift $win"] \
                   -side left -padx 1 -pady 1
    foreach c {z x c v b n m , . /} {
      pack [button $win.row4.[CharacterToName $c] -takefocus 0 \
                   -text $c -width $w_1 -font $font] \
                   -side left -padx 1 -pady 1
    }
    pack [button $win.row4.[CharacterToName shift_r] -takefocus 0 \
                   -text Shift -width $w_3 -font $font \
                   -command "[namespace current]::ToggleShift $win"] \
                   -side left -padx 1 -pady 1 -fill x -expand 1
    pack [button $win.row4.[CharacterToName \\] -takefocus 0 \
                   -text \\ -width $w_1 -font $font] \
                   -side left -padx 1 -pady 1
    pack $win.row4 -fill x -expand 1
    ## Fifth row of buttons:
    frame $win.row5
    foreach {c a ga} "Control_l \x01 \x03 Alt_l \x04 \x06 Altgr_l \x07 \x0E" {
      pack [button $win.row5.[CharacterToName $c] -takefocus 0 \
                   -text [string range $c 0 end-2] -width $w_2 -font $font \
                   -command [list [namespace current]::Insert $win $ga $ga] \
                   -command "[namespace current]::Toggle[string range $c \
                                                                0 end-2] $win"]\
                   -side left -padx 1 -pady 1
    }
    pack [button $win.row5.[CharacterToName space] -takefocus 0 \
                   -text Space -width $w_5 -font $font \
                   -command [list [namespace current]::Insert $win { } { }]] \
                   -side left -padx 1 -pady 1 -fill x -expand 1
    foreach {c a ga} "Alt_r \x05 \x06 Altgr_r \x08 \x0E Control_r \x05 \x03" {
      pack [button $win.row5.[CharacterToName $c] -takefocus 0 \
                   -text [string range $c 0 end-2] -width $w_2 -font $font \
                   -command [list [namespace current]::Insert $win $ga $ga] \
                   -command "[namespace current]::Toggle[string range $c \
                                                                0 end-2] $win"]\
                   -side left -padx 1 -pady 1
    }
    pack $win.row5 -fill x -expand 1
    ## Sixth row of buttons:
    frame $win.row6
    pack $win.row6 -fill x -expand 1

    ## Menubutton for changing the language...
    pack [menubutton $win.row6.language -menu $win.row6.language.menu \
           -textvariable "[namespace current]::${win}(Language)" -takefocus 0] \
      -side left -padx 10 -pady 1
    menu $win.row6.language.menu
    foreach lang [lsort [array names KeyboardConfig]] {
      $win.row6.language.menu add command -label $lang -command \
        "set [namespace current]::${win}(Language) $lang
         set [namespace current]::${win}(Shift) 1
         set [namespace current]::${win}(Control) 1
         set [namespace current]::${win}(Alt) 1
         set [namespace current]::${win}(Altgr) 1
         set [namespace current]::${win}(AccentHistory) {}
         [namespace current]::ToggleControl $win
         [namespace current]::ToggleAlt $win
         [namespace current]::ToggleAltgr $win
         [namespace current]::ToggleShift $win"
    }

    ## Label for showing accent history...
    set ${win}(AccentHistory) { }
    pack [label $win.row6.accent_history -width 20 -anchor e -textvariable \
           "[namespace current]::${win}(AccentHistory)" -takefocus 0 \
           -background white -foreground navy -relief raised -bd 1] \
      -side left -padx 10 -pady 1

    ## Checkbox for emmiting Unicode codes...
    pack [checkbutton $win.row6.unicode_codes -text "Unicode Codes" \
           -variable "[namespace current]::${win}(UnicodeCodes)" -takefocus 0] \
      -side right -padx 10 -pady 1
    set ${win}(AccentHistory) {}
    Shift $win 0
    focus $win
    return $win
  };# virtualKeyboard

  proc Insert {win c k} {
    variable $win
    ## Is it a accent key?
    # puts "c=\"$c\", k=\"$k\""
    append ${win}(AccentHistory) $k
    set history [set ${win}(AccentHistory)]
    set acckeys [set ${win}(AccentKeys)]
    set keys    [set ${win}(Keys)]
    set index   [lsearch $acckeys $k]
    if {$index != -1} {
      ## An accented key!
      return
    } else {
      ## A normal character. Is there a match in our key specification?
      set index [lsearch $keys $history]
      set ${win}(AccentHistory) {}
      if {$index == -1} {
        if {[string length $history] > 1} {
          ## If all keys in the history are valid keys also, insert them...
          set c {}
          foreach one [split $history {}] {
            set index [lsearch $keys $one]
            if {$index == -1} {bell; return}
            append c [lindex [lindex $keys [incr index]] 1]
          }
        }
      } else {
        set c [lindex [lindex $keys [incr index]] 1]
      }
    }
    if {[set ${win}(UnicodeCodes)]} {
      ## Convert the character to a \u???? escape code.
      set _c {}
      foreach one [split $c {}] {
        scan $one %c value
        if {$value >= 129} {
          append _c "\\u[format %04x $value]"
        } else {
          append _c $one
        }
      }
      set c $_c
    }
    foreach w [set ${win}(Receiver)] {
      if {![winfo exists $w]} {continue}
      switch [winfo class $w] {
        Text   {catch {tk::TextInsert $w $c}}
        TEntry -
        Entry  {catch {tk::CancelRepeat}
                catch {tk::EntryInsert $w $c}}
        default {}
      }
    }
  };# Insert

  proc Delete {win} {
    variable $win
    foreach w [set ${win}(Receiver)] {
      if {![winfo exists $w]} {continue}
      switch [winfo class $w] {
        Text  {
          if {[$w tag nextrange sel 1.0 end] ne ""} {
            $w delete sel.first sel.last
          } else {
            $w delete insert
            $w see insert
          }
        }
        Entry {
          if {[$w selection present]} {
            $w delete sel.first sel.last
          } else {
            $w delete insert
          }
        }
        default {}
      }
    }
  };# Delete

  proc Backspace {win} {
    variable $win
    foreach w [set ${win}(Receiver)] {
      if {![winfo exists $w]} {continue}
      switch [winfo class $w] {
        Text  {
          if {[$w tag nextrange sel 1.0 end] ne ""} {
            $w delete sel.first sel.last
          } elseif {[$w compare insert != 1.0]} {
            $w delete insert-1c
            $w see insert
          }
        }
        Entry {catch {tk::EntryBackspace $w}}
        default {}
      }
    }
  };# Backspace

  proc ExpandState {state} {
    foreach {shift caps control alt altgr} {0 0 0 0 0} {break}
    if {$state & 1<<0} {
      set shift 1
    } elseif {$state & 1<<1} {
      set caps 1
    } elseif {$state & 1<<2} {
      set control 1
    } elseif {$state & 1<<3} {
      set alt 1
    } elseif {$state & 1<<4} {
      set altgr 1
    } else {
      #puts "State: $state"
    }
    return [list $shift $caps $control $alt $altgr]
  };# ExpandState

  proc OnKeyPress {win button detail state args} {
    set name [CharacterToName $detail]
    foreach {shift caps control alt altgr} [ExpandState $state] {break}
    foreach row {1 2 3 4 5} {
      if {[winfo exists $win.row$row.$name]} {
        switch -glob $name {
          caps_lock - 
          default {
            $win.row$row.$name configure -relief sunken
            update idletasks
          }
        }
        $win.row$row.$name invoke
        break
      }
    }
    switch -glob $name {
      caps_lock {
        if {$caps} {
          $win.row3.[CharacterToName caps_lock] configure -relief raised
          Capitals $win 0
        } else {
          $win.row3.[CharacterToName caps_lock] configure -relief sunken
          Capitals $win 1
        }
      }
      shift_* {
        if {$shift} {
          $win.row4.[CharacterToName shift_r] configure -relief raised
          $win.row4.[CharacterToName shift_l] configure -relief raised
          Shift $win 0
        } else {
          $win.row4.[CharacterToName shift_r] configure -relief sunken
          $win.row4.[CharacterToName shift_l] configure -relief sunken
          Shift $win 1
        }
      }
      control_* {
        if {$control} {
          $win.row5.[CharacterToName control_r] configure -relief raised
          $win.row5.[CharacterToName control_l] configure -relief raised
          Control $win 0
        } else {
          $win.row5.[CharacterToName control_r] configure -relief sunken
          $win.row5.[CharacterToName control_l] configure -relief sunken
          Control $win 1
        }
      }
      alt_* {
        if {$alt} {
          $win.row5.[CharacterToName alt_r] configure -relief raised
          $win.row5.[CharacterToName alt_l] configure -relief raised
          Alt $win 0
        } else {
          $win.row5.[CharacterToName alt_r] configure -relief sunken
          $win.row5.[CharacterToName alt_l] configure -relief sunken
          Alt $win 1
        }
      }
    }
    # puts "OnKeyPress $detail -> $button \[$name\] $state ($args)"
    # puts "Shift: $shift"
    # puts "Caps:  $caps"
  };# OnKeyPress

  proc OnKeyRelease {win button detail state args} {
    set name [CharacterToName $detail]
    switch -glob $name {
      caps_lock {return}
      iso_prev_group -
      shift_l   {$win.row4.shift_l configure -relief raised
                 #$win.row4.shift_r configure -relief raised
                 Shift $win 0
                 return}
      iso_next_group -
      shift_r   {$win.row4.shift_r configure -relief raised
                 #$win.row4.shift_l configure -relief raised
                 Shift $win 0
                 return}
      control_l -
      iso_levelthree_shift -
      control_r {$win.row5.control_r configure -relief raised
                 $win.row5.control_l configure -relief raised
                 Control $win 0
                 return}
      alt_l     {$win.row5.alt_l configure -relief raised}
      menu      -
      alt_r     {$win.row5.alt_r configure -relief raised}
      default   {#puts $name}
    }
    foreach row {1 2 3 4 5} {
      if {[winfo exists $win.row$row.$name]} {
        $win.row$row.$name configure -relief raised
        break
      }
    }
    
    foreach {shift caps control alt altgr} [ExpandState $state] {break}
    ## Fix the shift & caps lock buttons...
    if {$caps} {
      #$win.row3.[CharacterToName caps_lock] configure -relief sunken
    } else {
      $win.row3.[CharacterToName caps_lock] configure -relief raised
      Capitals $win 0
    }
    if {$shift} {
      #$win.row4.[CharacterToName shift_l] configure -relief sunken
      #$win.row4.[CharacterToName shift_r] configure -relief sunken
    } else {
      $win.row4.[CharacterToName shift_l] configure -relief raised
      $win.row4.[CharacterToName shift_r] configure -relief raised
    }
    foreach k {control alt altgr} {
      if {![set $k]} {
        $win.row5.[CharacterToName ${k}_l] configure -relief raised
        $win.row5.[CharacterToName ${k}_r] configure -relief raised
        #[string totitle $k] $win 0
      }
    }
    # puts "OnKeyRelease: $detail -> $button \[$name\] $state ($args)"
    # puts "Shift: $shift"
    # puts "Caps:  $caps"
  };# OnKeyRelease

  proc ToggleCapitals {win} {
    variable $win
    if {[set ${win}(Capitals)]} {
      Capitals $win 0
      $win.row3.[CharacterToName caps_lock] configure -relief raised
    } else {
      Capitals $win 1
      $win.row3.[CharacterToName caps_lock] configure -relief sunken
    }
  };# ToggleCapitals

  proc ToggleShift {win} {
    variable $win
    if {[set ${win}(Shift)]} {
      Shift $win 0
      $win.row4.[CharacterToName shift_l] configure -relief raised
      $win.row4.[CharacterToName shift_r] configure -relief raised
    } else {
      Shift $win 1
      $win.row4.[CharacterToName shift_l] configure -relief sunken
      $win.row4.[CharacterToName shift_r] configure -relief sunken
    }
  };# ToggleShift

  proc ToggleControl {win} {
    variable $win
    if {[set ${win}(Control)]} {
      Control $win 0
      $win.row5.[CharacterToName control_l] configure -relief raised
      $win.row5.[CharacterToName control_r] configure -relief raised
    } else {
      Control $win 1
      $win.row5.[CharacterToName control_l] configure -relief sunken
      $win.row5.[CharacterToName control_r] configure -relief sunken
    }
  };# ToggleControl

  proc ToggleAlt {win} {
    variable $win
    if {[set ${win}(Alt)]} {
      Alt $win 0
      $win.row5.[CharacterToName alt_l] configure -relief raised
      $win.row5.[CharacterToName alt_r] configure -relief raised
    } else {
      Alt $win 1
      $win.row5.[CharacterToName alt_l] configure -relief sunken
      $win.row5.[CharacterToName alt_r] configure -relief sunken
    }
  };# ToggleAlt

  proc ToggleAltgr {win} {
    variable $win
    if {[set ${win}(Altgr)]} {
      Altgr $win 0
      $win.row5.[CharacterToName altgr_l] configure -relief raised
      $win.row5.[CharacterToName altgr_r] configure -relief raised
    } else {
      Altgr $win 1
      $win.row5.[CharacterToName altgr_l] configure -relief sunken
      $win.row5.[CharacterToName altgr_r] configure -relief sunken
    }
  };# ToggleAltgr
  
  proc Shift {win on} {
    variable $win
    if {[set ${win}(Shift)] == $on} {return}
    variable KeyboardConfig
    array set key [subst $KeyboardConfig(English)]
    array set key [subst $KeyboardConfig([set ${win}(Language)])]
    catch {unset ${win}(Keys)}
    set ${win}(Keys) [array get key]
    ## Set the accented keys...
    set ${win}(AccentKeys) {}
    if {[info exists key(specifierKeys)]} {
      set ${win}(AccentKeys) [split [subst $key(specifierKeys)]]
    }
    set ${win}(Shift) $on
    if {[set ${win}(Capitals)]} {
      set on [expr {!$on}]
    }
    if {$on} {
      ## Set the keyboard to display the characters that should be input when
      ## shift is pressed.
      foreach c {~ ! @ # $ % ^ & * ( ) _ +} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row1.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $c $c]
      }
      foreach c {Q W E R T Y U I O P \{ \}} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {A S D F G H J K L : \"} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {Z X C V B N M < > ? |} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
    } else {
      ## Set the keyboard to display the characters that should be input when
      ## shift is not pressed.
      foreach c {` 1 2 3 4 5 6 7 8 9 0 - =} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row1.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $c $c]
      }
      foreach c {q w e r t y u i o p [ ]} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {a s d f g h j k l ; '} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {z x c v b n m , . / \\} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
    }
  };# Shift

  proc Capitals {win on} {
    variable $win
    if {[set ${win}(Capitals)] == $on} {return}
    variable KeyboardConfig
    array set key $KeyboardConfig(English)
    array set key $KeyboardConfig([set ${win}(Language)])
    set ${win}(Capitals) $on
    if {$on} {
      ## Set the keyboard to display the characters that should be input when
      ## shift is pressed.
      foreach c {Q W E R T Y U I O P} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {A S D F G H J K L} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {Z X C V B N M} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
    } else {
      ## Set the keyboard to display the characters that should be input when
      ## shift is not pressed.
      foreach c {q w e r t y u i o p} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {a s d f g h j k l} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {z x c v b n m} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
    }
  };# Capitals

  proc Control {win on} {
    return [ControlAltAltGr Control \x03 $win $on]
  };# Control
  
  proc Alt {win on} {
    return [ControlAltAltGr Alt \x06 $win $on]
  };# Alt

  proc Altgr {win on} {
    return [ControlAltAltGr Altgr \x0E $win $on]
  };# Altgr

  proc ControlAltAltGr {var CC win on} {
    variable $win
    if {[set ${win}($var)] == $on} {return}
    variable KeyboardConfig
    array set key [subst $KeyboardConfig(English)]
    array set key [subst $KeyboardConfig([set ${win}(Language)])]
    catch {unset ${win}(Keys)}
    set ${win}(Keys) [array get key]
    ## Set the accented keys...
    set ${win}(AccentKeys) {}
    if {[info exists key(specifierKeys)]} {
      set ${win}(AccentKeys) [split [subst $key(specifierKeys)]]
    }
    set ${win}($var) $on
    if {$on} {
      ## Set the keyboard to display the characters that should be input when
      ## Control/Alt/AltGr is pressed.
      foreach c {` 1 2 3 4 5 6 7 8 9 0 - =} {
        set C $c
        if {[info exists key($CC$c)]} {
          foreach {l v} $key($CC$c) break; set C $CC$c} \
        elseif {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row1.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $C]
      }
      foreach c {q w e r t y u i o p [ ]} {
        set C $c
        if {[info exists key($CC$c)]} {
          foreach {l v} $key($CC$c) break; set C $CC$c} \
        elseif {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $C]
      }
      foreach c {a s d f g h j k l ; '} {
        set C $c
        if {[info exists key($CC$c)]} {
          foreach {l v} $key($CC$c) break; set C $CC$c} \
        elseif {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $C]
      }
      foreach c {z x c v b n m , . / \\} {
        set C $c
        if {[info exists key($CC$c)]} {
          foreach {l v} $key($CC$c) break; set C $CC$c} \
        elseif {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $C]
      }
    } else {
      ## Set the keyboard to display the characters that should be input when
      ## Control/Alt/AltGr is not pressed.
      foreach c {` 1 2 3 4 5 6 7 8 9 0 - =} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row1.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {q w e r t y u i o p [ ]} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row2.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {a s d f g h j k l ; '} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row3.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      foreach c {z x c v b n m , . / \\} {
        if {[info exists key($c)]} {foreach {l v} $key($c) break} \
        else {set l $c; set v $c}
        $win.row4.[CharacterToName $c] configure -text $l \
          -command [list [namespace current]::Insert $win $v $c]
      }
      if {[set ${win}(Capitals)]} {
        set ${win}(Capitals) 0
        return [Capitals $win 1]
      }
    }
  };# ControlAltAltGr

  proc CharacterToName {c} {
    if {[string equal $c _]} {return minus}
    return [string map {
       `  grave         ~  grave         asciitilde   grave
       1  one           !  one           exclam       one
       2  two           @  two           at           two
       3  three         #  three         numbersign   three
       4  four          $  four          dollar       four
       5  five          %  five          percent      five
       6  six           ^  six           asciicircum  six
       7  seven         &  seven         ampersand    seven
       8  eight         *  eight         asterisk     eight
       9  nine          (  nine          parenleft    nine
       0  zero          )  zero          parenright   zero
       -  minus                          underscore   minus
       =  equal         +  equal         plus         equal
       [  bracketleft  \{  bracketleft   braceleft    bracketleft
       ]  bracketright \}  bracketright  braceright   bracketright
      \t  tab          \n  return      
       ,  comma         <  comma         less         comma
       .  period        >  period        greater      period
       /  slash         ?  slash         question     slash
      \;  semicoln      :  semicoln      colon        semicoln
                                         semicolon    semicoln
       '  apostrophe   \"  apostrophe    quotedbl     apostrophe
      \\  backslash     |  backslash     bar          backslash
      \   space        \b  backspace} [string tolower $c]]
  };# CharacterToName

};# namespace eval virtualKeyboard

######################################
## Test Code
######################################

if {![info exists ::HomeDir]} {
  catch {console show}
  pack [entry .entry] -fill x -expand 1
  pack [text .text -height 8]   -fill both -expand 1
  set win .virtualKeyboard
  toplevel $win
  wm withdraw $win
  wm title $win "Virtual Keyboard"
  pack [virtualKeyboard::virtualKeyboard $win.vkf -receiver {.text .entry}] \
    -fill both -expand 1
  wm deiconify $win
  bind $win <Destroy> exit
  raise $win
}
