#!/bin/sh
# Restart, using wish \
 exec wish "$0" "$@"
#
# Copyright (C) 1998-2010 George P. Petasis
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   petasis@iit.demokritos.gr
#   http://www.edu.physics.uch.gr/~petasis
#
# $Revision: 1.0 $
#

package require Tk 8.0

namespace eval ::ScrolledWidgets {
  variable ScrolledWidgets_ScrollbarStatus
  namespace export ScrollbarSet ScrolledCanvas ScrolledText ScrolledListbox
}

## These virtual events control the scrolling of widgets...
global $tcl_platform(platform)
switch $tcl_platform(platform) {
  unix    {
    event add <<SW:Focus>>        <ButtonRelease-1>
    event add <<SW:ScrollPress>>  <ButtonPress-2>
    event add <<SW:ScrollMotion>> <B2-Motion>
  }
  windows {
    event add <<SW:Focus>>        <ButtonRelease-2>
    event add <<SW:ScrollPress>>  <ButtonPress-2>
    event add <<SW:ScrollMotion>> <B2-Motion>
  }
  default {
    event add <<SW:Focus>>        <Enter>
    event add <<SW:ScrollPress>>  <ButtonPress-2>
    event add <<SW:ScrollMotion>> <B2-Motion>
  }
}
#event add <<SW:Expose>>       <Expose> <Configure>
event add <<SW:Expose>>       <Configure>
event add <<SW:Prior>>        <Prior> 
event add <<SW:Next>>         <Next>

###############################################################################
###                                                                         ###
###     Generic Functions (Internal Functions)                              ###
###                                                                         ###
###############################################################################

#
# This function, automatically hides/shows the scrollbar
#
proc ::ScrolledWidgets::ScrollbarSet {widget args} {
  variable ScrolledWidgets_ScrollbarStatus
  set parent [winfo parent $widget]
  if {[winfo width $parent] == 1} {
    if {[winfo height $parent] == 1} {
      ## Parent has geometry 1x1 (is unmapped). Don't pack scrollbars
      return {}
    }
  }
  if {[string equal [lindex $args 0] set] && \
      [lindex $args end] == 1 && [lindex $args 1] == 0} {
    ## Hide scrollbar...
    if {$ScrolledWidgets_ScrollbarStatus($widget) == 1} {
      grid forget $widget
      set ScrolledWidgets_ScrollbarStatus($widget) 0
    }
  } else {
    if {$ScrolledWidgets_ScrollbarStatus($widget) == 0} {
      set grid_args $ScrolledWidgets_ScrollbarStatus(${widget}_args)
      set parent [winfo toplevel $widget]
      #grid propagate $widget 0
      eval grid $widget $grid_args
      catch {wm geometry $parent [wm geometry $parent]}
      set ScrolledWidgets_ScrollbarStatus($widget) 1
    }
    eval $widget $args
    ## The following prevents TkHtml from freezing wish under Windows...
    #if {[lindex $args end] >= 1.0-[$widget delta 1 1]} {
    #  eval [$widget cget -command] scroll +1 pages
    #  $widget set [lindex $args 1] 1.0
    #}
    ## But prevents Annotation Viewers with many Annotations to scroll...
  }
};# ::ScrolledWidgets::ScrollbarSet

###############################################################################
###                                                                         ###
###     Scrolled Canvas Widget                                              ###
###                                                                         ###
###############################################################################

#
# Scrolled Canvas
#  Arguments: all these of the canvas widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#   -regionpadx: adds extra region when calculating canvas scrollregion (pixels)
#                (x-direction)
#   -regionpady: adds extra region when calculating canvas scrollregion (pixels)
#                (y-direction)
#   -confcommand: a command that is executed before the standard configure
#                 bindings for the canvas object
#   -mapvertical: map vertical scrollbar at creation
#   -maphorizontal: map horizontal scrollbar at creation
#   -widgetwidth:   total width (in pixels) of widget + scrollbars
#   -widgetheight:  total heigth (in pixels) of widget + scrollbars
#
proc ::ScrolledWidgets::ScrolledCanvas {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  foreach {canvas_args general_args grid_args scroll_args frame_args \
           widgetname conf_command map_vertical map_horizontal} \
          {{} {} {} {} {} canvas {} 0 0} {break}
  ## Enlarge canvas scroll region by the following values (pixels)...
  foreach {canvas_scroll_padx canvas_scroll_pady} {0 0} {break}
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -regionpadx {set canvas_scroll_padx  $arg}
      -regionpady {set canvas_scroll_pady  $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args    $op $arg}
      -widgetname {set     widgetname       $arg}
      -confcom*   {set     conf_command     $arg}
      -mapver*    {set     map_vertical     [string tolower $arg]}
      -maphor*    {set     map_horizontal   [string tolower $arg]}
      -widgetw*   {lappend frame_args -width  $arg}
      -widgeth*   {lappend frame_args -height $arg}
      default     {lappend canvas_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval canvas $w.$widgetname $general_args $canvas_args -takefocus 1 \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet $w.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet $w.vertical set]]
    eval scrollbar $w.horizontal $scroll_args \
      -command [list [list $w.$widgetname xview]] -orient horizontal 
    eval scrollbar $w.vertical   $scroll_args \
      -command [list [list $w.$widgetname yview]] -orient vertical 

    ## Map widgets...
    if {[string is true $map_vertical]} {
      eval grid $w.$widgetname $w.vertical -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus($w.vertical)   1
    } else {
      eval grid $w.$widgetname -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus($w.vertical)   0
    }
    if {[string is true $map_horizontal]} {
      eval grid $w.horizontal -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus($w.horizontal) 1
    } else {
      set ScrolledWidgets_ScrollbarStatus($w.horizontal) 0
    }
    set ScrolledWidgets_ScrollbarStatus($w.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus($w.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]

    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W::padx)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W::pady)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}

    ## Pad info
    set ScrolledWidgets_ScrollbarStatus($w.${widgetname}::padx) \
        $canvas_scroll_padx
    set ScrolledWidgets_ScrollbarStatus($w.${widgetname}::pady) \
        $canvas_scroll_pady

    append conf_command "\;::ScrolledWidgets::CanvasSetScrollRegion %W \
                         $canvas_scroll_padx $canvas_scroll_pady"
    bind ${w}.$widgetname <<SW:Expose>> $conf_command
    bind ${w}.$widgetname <<SW:ScrollPress>>  {%W scan mark %x %y}
    bind ${w}.$widgetname <<SW:ScrollMotion>> {%W scan dragto %x %y}
    bind ${w}.$widgetname <<SW:Prior>> {%W yview scroll -1 pages}
    bind ${w}.$widgetname <<SW:Next>>  {%W yview scroll  1 pages}
    bind ${w}.$widgetname <<SW:Focus>> {focus %W}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return $w.$widgetname
};#::ScrolledWidgets::ScrolledCanvas

#
#  This calculates a canvas widget scroll region
#
proc ::ScrolledWidgets::CanvasGetScrollRegion {canvas {padx 0} {pady 0}} {
  set width  [winfo width  $canvas]
  set height [winfo height $canvas]
  if {$width == 1 && $height == 1} {
    update
    set width  [winfo width  $canvas]
    set height [winfo height $canvas]
  }
  if {$width == 1 && $height == 1} {
    set width  [winfo reqwidth  $canvas]
    set height [winfo reqheight $canvas]
  }
  set pwidth  [winfo width  [winfo parent $canvas]]
  set pheight [winfo height [winfo parent $canvas]]
  set bd [expr {2*[$canvas cget -borderwidth]}]
  set hw [expr {2*[$canvas cget -highlightthickness]}]
  set bbox   [$canvas bbox all]
  foreach {x1 y1 x2 y2} $bbox {break}
  incr x1 -$padx
  incr y1 -$pady
  incr x2  $padx
  incr y2  $pady
  set scroll_width  [expr {$x2-$x1}]
  set scroll_height [expr {$y2-$y1}]
  if {$scroll_width < $pwidth && $scroll_height < $pheight} {
    set x2 [expr {$x1+$width-$bd-$hw}]
    set y2 [expr {$y1+$height-$bd-$hw}]
    return [list $x1 $y1 $x2 $y2]
  }
  if {$scroll_width < $width} {
    set x2 [expr {$x1+$width-$bd-$hw}]
  }
  if {$scroll_height < $height} {
    set y2 [expr {$y1+$height-$bd-$hw}]
  }
  return [list $x1 $y1 $x2 $y2]
}

#
# Set a canvas widget scroll region
#
proc ::ScrolledWidgets::CanvasSetScrollRegion {canvas {padx 0} {pady 0}} {
  catch {$canvas configure -scrollregion \
    [::ScrolledWidgets::CanvasGetScrollRegion $canvas $padx $pady]}
}

#
# Update Scroll Region
#
proc ::ScrolledWidgets::CanvasUpdateScrollRegion {canvas} {
  variable ScrolledWidgets_ScrollbarStatus
  catch {
  set padx $ScrolledWidgets_ScrollbarStatus(${canvas}::padx)
  set pady $ScrolledWidgets_ScrollbarStatus(${canvas}::pady)
  $canvas configure -scrollregion \
    [::ScrolledWidgets::CanvasGetScrollRegion $canvas $padx $pady]}
}

###############################################################################
###                                                                         ###
###     Scrolled Text Widget                                                ###
###                                                                         ###
###############################################################################

#
# Scrolled Text
#  Arguments: all these of the text widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#   -mapvertical:   map vertical scrollbar at creation
#   -maphorizontal: map horizontal scrollbar at creation
#
proc ::ScrolledWidgets::ScrolledText {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  foreach {text_args general_args grid_args scroll_args frame_args widgetname \
           map_vertical map_horizontal} \
          {{} {} {} {} {} text 0 0} {break}
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args  $op $arg}
      -widgetname {set     widgetname     $arg}
      -mapver*    {set     map_vertical     [string tolower $arg]}
      -maphor*    {set     map_horizontal   [string tolower $arg]}
      default     {lappend text_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval text ${w}.$widgetname $general_args $text_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.vertical set]]
    eval scrollbar ${w}.horizontal $scroll_args \
      -command [list [list ${w}.$widgetname xview]] -orient horizontal  
    eval scrollbar ${w}.vertical   $scroll_args \
      -command [list [list ${w}.$widgetname yview]] -orient vertical  
    ## Map widgets...
    if {[string is true $map_vertical]} {
      eval grid ${w}.$widgetname ${w}.vertical -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   1
    } else {
      eval grid ${w}.$widgetname -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    }
    if {[string is true $map_horizontal]} {
      eval grid ${w}.horizontal -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 1
    } else {
      set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 0
    }
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return ${w}.$widgetname
};#::ScrolledWidgets::ScrolledText


###############################################################################
###                                                                         ###
###     Scrolled Listbox Widget                                             ###
###                                                                         ###
###############################################################################

#
# Scrolled Listbox
#  Arguments: all these of the listbox widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#
proc ::ScrolledWidgets::ScrolledListbox {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  set listbox_args {}; set general_args {}; set grid_args {}; set scroll_args {}
  set frame_args {}; set widgetname listbox
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args     $op $arg}
      -widgetname {set     widgetname        $arg}
      default     {lappend listbox_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval listbox ${w}.$widgetname $general_args $listbox_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.vertical set]]
    eval scrollbar ${w}.horizontal $scroll_args \
      -command [list [list ${w}.$widgetname xview]] -orient horizontal  
#     -highlightthickness 0
    eval scrollbar ${w}.vertical   $scroll_args \
      -command [list [list ${w}.$widgetname yview]] -orient vertical  
#     -highlightthickness 0
    eval grid ${w}.$widgetname -sticky snew $grid_args
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 0
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  bind $w.$widgetname <ButtonPress> {+focus %W}
  return ${w}.$widgetname
};#::ScrolledWidgets::ScrolledListbox

###############################################################################
###                                                                         ###
###     Scrolled ListBox Widget (BWidget ListBox...)                        ###
###                                                                         ###
###############################################################################

#
# Scrolled ListBox
#  Arguments: all these of the listbox widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#
proc ::ScrolledWidgets::ScrolledListBox {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  set listbox_args {}; set general_args {}; set grid_args {}; set scroll_args {}
  set frame_args {}; set widgetname listbox
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args     $op $arg}
      -widgetname {set     widgetname        $arg}
      default     {lappend listbox_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval ListBox ${w}.$widgetname $general_args $listbox_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.vertical set]]
    eval scrollbar ${w}.horizontal $scroll_args \
      -command [list [list ${w}.$widgetname xview]] -orient horizontal  
#     -highlightthickness 0
    eval scrollbar ${w}.vertical   $scroll_args \
      -command [list [list ${w}.$widgetname yview]] -orient vertical  
#     -highlightthickness 0
    eval grid ${w}.$widgetname -sticky snew $grid_args
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 0
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return ${w}.$widgetname
};#::ScrolledWidgets::ScrolledListBox

###############################################################################
###                                                                         ###
###     Scrolled Multi Column Listbox Widget                                ###
###                                                                         ###
###############################################################################

#
# Scrolled Mclistbox
#  Arguments: all these of the mclistbox widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#   -mapvertical: map vertical scrollbar at creation
#   -maphorizontal: map horizontal scrollbar at creation
#
proc ::ScrolledWidgets::ScrolledMclistbox {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  set listbox_args {}; set general_args {}; set grid_args {}; set scroll_args {}
  set frame_args {}; set widgetname mclistbox
  foreach {map_vertical map_horizontal} {0 0} {}
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args     $op $arg}
      -widgetname {set     widgetname        $arg}
      -mapver*    {set     map_vertical     [string tolower $arg]}
      -maphor*    {set     map_horizontal   [string tolower $arg]}
      default     {lappend listbox_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval ::mclistbox::mclistbox ${w}.$widgetname $general_args $listbox_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.vertical set]]
    eval scrollbar ${w}.horizontal $scroll_args \
      -command [list [list ${w}.$widgetname xview]] -orient horizontal 
#      -highlightthickness 0
    eval scrollbar ${w}.vertical   $scroll_args \
      -command [list [list ${w}.$widgetname yview]] -orient vertical 
#      -highlightthickness 0
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    ## Map widgets...
    if {[string is true $map_vertical]} {
      eval grid ${w}.$widgetname ${w}.vertical -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   1
    } else {
      eval grid ${w}.$widgetname -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    }
    if {[string is true $map_horizontal]} {
      eval grid ${w}.horizontal -sticky snew $grid_args
      set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 1
    } else {
      set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 0
    }
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return ${w}.$widgetname
};#::ScrolledWidgets::ScrolledMclistbox

###############################################################################
###                                                                         ###
###     Scrolled Html Widget (TkHtml...)                                    ###
###                                                                         ###
###############################################################################

#
# Scrolled Html
#  Arguments: all these of the html widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#
proc ::ScrolledWidgets::ScrolledHtml {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  set html_args {}; set general_args {}; set grid_args {}; set scroll_args {}
  set frame_args {}; set widgetname html
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args  $op $arg}
      -widgetname {set     widgetname     $arg}
      default     {lappend html_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval html $w.$widgetname $general_args $html_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet $w.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet $w.vertical set]]
    ## Bindings...
    catch {::HtmlInfoViewer::DefaultBingings $w.$widgetname}
    
    eval scrollbar $w.horizontal $scroll_args \
      -command [list [list $w.$widgetname xview]] -orient horizontal  
#     -highlightthickness 0
    eval scrollbar $w.vertical   $scroll_args \
      -command [list [list $w.$widgetname yview]] -orient vertical  
#     -highlightthickness 0
    eval grid $w.$widgetname -sticky snew $grid_args
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    set ScrolledWidgets_ScrollbarStatus($w.horizontal) 0
    set ScrolledWidgets_ScrollbarStatus($w.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return $w.$widgetname
};#::ScrolledWidgets::ScrolledHtml

###############################################################################
###                                                                         ###
###     Scrolled Table Widget (TkTable...)                                    ###
###                                                                         ###
###############################################################################

#
# Scrolled Table
#  Arguments: all these of the html widget, plus:
#   -widgetname : name of the internal widget
#   -scrollborderwidth (or -scrollbd) : the borderwidth of scrollbars
#   -scrollforeground  (or -scrollfg) : the foreground color of scrollbars
#   -scrollbackground  (or -scrollbg) : the background color of scrollbars
#
proc ::ScrolledWidgets::ScrolledTable {w args} {
  variable ScrolledWidgets_ScrollbarStatus
  ## Process Arguments
  set table_args {}; set general_args {}; set grid_args {}; set scroll_args {}
  set frame_args {}; set widgetname table
  foreach {op arg} $args {
    switch -glob -- $op {
      -bd         -
      -bord*      {lappend general_args $op $arg}
      -scroll*    {lappend scroll_args "-[string range $op 7 end]" $arg}
      -xsc*       -
      -ysc*       {}
      -padx       -
      -pady       -
      -ipadx      -
      -ipady      {lappend grid_args  $op $arg}
      -widgetname {set     widgetname     $arg}
      default     {lappend table_args  $op $arg}
    }
  }
  ## Create the Widget
  if {[catch {
    eval frame  $w -borderwidth 0 -relief flat -highlightthickness 0 $frame_args
    eval table ${w}.$widgetname $general_args $table_args \
      -xscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.horizontal set]] \
      -yscrollcommand \
        [list [list ::ScrolledWidgets::ScrollbarSet ${w}.vertical set]]
    ## Bindings...
    #catch {InfoViewer_DefaultBingings ${w}.$widgetname}
    
    eval scrollbar ${w}.horizontal $scroll_args \
      -command [list [list ${w}.$widgetname xview]] -orient horizontal  
#     -highlightthickness 0
    eval scrollbar ${w}.vertical   $scroll_args \
      -command [list [list ${w}.$widgetname yview]] -orient vertical  
#     -highlightthickness 0
    eval grid ${w}.$widgetname -sticky snew $grid_args
    grid rowconfigure    $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    #grid propagate       $w 0
    bind $w <Destroy> {catch {\
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.horizontal_args)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical)
    unset ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus(%W.vertical_args)}}
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal) 0
    set ScrolledWidgets_ScrollbarStatus(${w}.horizontal_args) \
      [eval list -sticky snew -column 0 -row 1 $grid_args]
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical)   0
    set ScrolledWidgets_ScrollbarStatus(${w}.vertical_args) \
      [eval list -sticky snew -column 1 -row 0 $grid_args]
    catch {focus ${w}.$widgetname}
  } error]} {
    catch {destroy $w}
    error $error
  }
  return $w.$widgetname
};#::ScrolledWidgets::ScrolledTable
#
# EOF - End of File
#
