#!/bin/sh
# Restart, using wish \
 exec wish "$0" "$@"
#
# Copyright (C) 2000-2004 George P. Petasis
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   petasis@iit.demokritos.gr
#   http://www.iit.demokritos.gr/~petasis
#
# $Revision: 1.0 $
#

## 
## This library provides a widget that is a toolbar that can contain a single
## line of items composed out of an image and a label (both optionally)...
##
package require Tk 8.0

namespace eval ::TabbedToolbar {
  variable Pages
  namespace export {}
}

#
# This function creates a TabbedToolbar widget
#  Arguments:
#     All arguments of the frame widget
#
proc ::TabbedToolbar::TabbedToolbar {widget args} {
  eval frame $widget $args -class TabbedToolbar -takefocus 1
  bind $widget <Destroy> \
    {array unset ::TabbedToolbar::Pages *%W; rename ::%W {}}
  set bg [$widget cget -background]
  frame $widget.top    -relief flat -bd 0 -bg $bg
  frame $widget.middle -relief flat -bd 0 -bg $bg
  frame $widget.bottom -relief flat -bd 0 -bg $bg
  grid $widget.top    -sticky snew
  grid $widget.middle -sticky snew
  grid $widget.bottom -sticky snew
  grid columnconfigure $widget 0 -weight 1
  grid rowconfigure    $widget 0 -weight 0
  grid rowconfigure    $widget 1 -weight 1
  grid rowconfigure    $widget 2 -weight 0
  rename ::$widget ::TabbedToolbar::widget_$widget
  proc   ::$widget {command args} \
     "eval ::TabbedToolbar::WidgetCommand $widget \$command \$args"
  grid columnconfigure $widget.top    0 -weight 1
  grid columnconfigure $widget.middle 0 -weight 2
  grid columnconfigure $widget.bottom 0 -weight 1
  return $widget
};# TabbedToolbar

#
# Handle widget commands...
#
proc ::TabbedToolbar::WidgetCommand {widget command args} {
  switch -glob -- $command {
    addp* {::TabbedToolbar::AddPage $widget $args}
    addi* {::TabbedToolbar::AddItem $widget $args}
    default {error "invalid option $command"}
  }
};# ::TabbedToolbar::WidgetCommand

#
# Create a new page...
#
proc ::TabbedToolbar::AddPage {widget args} {
  variable Pages 
  set args [lindex $args 0]
  
  foreach {name label image compound  fg   padx pady} \
          { {}    {}    {}    left   black  10    5 } {break}
  set bg [::TabbedToolbar::widget_$widget cget -bg]
  set name [lindex $args 0]
  foreach {opt val} [lrange $args 1 end] {
    switch -glob -- $opt {
      -bg   -
      -ba*  {set bg       $val}
      -co*  {set compound $val}
      -fg   -
      -fo*  {set fg       $val}
      -im*  {set image    $val}
      -la*  {set label    $val}
      -padx {set padx     $val}
      -pady {set pady     $val}
      default {error "invalid option $opt"}
    }
  }
  if {[info exists Pages($widget)] && 
      [lsearch -exact $Pages($widget) $name] != -1} {
    error "page $name already exists!"
  }
  lappend Pages($widget) $name
  button $widget.top.page_$name    -compound $compound -image $image \
    -text $label -anchor w -padx $padx -pady $pady -highlightthickness 0 \
    -command "::TabbedToolbar::ShowPage $widget $name top"
  button $widget.bottom.page_$name -compound $compound -image $image \
    -text $label -anchor w -padx $padx -pady $pady -highlightthickness 0 \
    -command "::TabbedToolbar::ShowPage $widget $name bottom"
  set pos [lsearch -exact $Pages($widget) $name]
  ## The returned value is a window, where the user can embed other windows...
  ::ScrolledWidgets::ScrolledCanvas $widget.middle.area_$name -relief flat \
    -bd 0 -bg $bg
  if {$pos == 0} {
    grid $widget.top.page_$name    -row $pos -sticky snew
    grid $widget.middle.area_$name -row $pos -sticky snew
    grid rowconfigure $widget.middle $pos -weight 1
  } else {
    grid $widget.bottom.page_$name -row $pos -sticky snew
  }
  return $widget.middle.area_$name
};# ::TabbedToolbar::AddPage

#
# Show a specific page... 
#
proc ::TabbedToolbar::ShowPage {widget name direction} {
  variable Pages
  set pos  [lsearch -exact $Pages($widget) $name]
  set last [llength $Pages($widget)]
  if {$pos == -1} {error "page \"$name\" does not exists!"}
  if {$pos == $last-1} {$widget.bottom configure -height 1}
  for {set i 0} {$i < $last} {incr i} {
    set page [lindex $Pages($widget) $i]
    if {$i <= $pos} {
      grid $widget.top.page_$page -row $i -sticky snew
      grid forget $widget.bottom.page_$page
    } else {
      grid forget $widget.top.page_$page
      grid $widget.bottom.page_$page -row $i -sticky snew
    }
    grid forget $widget.middle.area_$page
    grid rowconfigure $widget.middle $i -weight 0
  }
  set page [lindex $Pages($widget) $pos]
  grid $widget.middle.area_$page -row $pos -sticky snew
  grid rowconfigure $widget.middle $pos -weight 1
  update
  $widget.middle.area_$page.canvas yview moveto 0
};# ::TabbedToolbar::ShowPage

#
# EOF - End of File
#
