##############################################################################
##  TkInternals.tcl:                                                        ##
## ------------------                                                       ##
##  This file contains some useful procedures for accessing some tk         ##
##  internal variables, that are subject to change in future releases...    ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## Use this function to get values stored in ::tk::Priv or tkPriv arrays...
proc TkGUI_AccessPriv {key} {
  if {[llength [info globals tkPriv]]} {
    ## This will work up to 8.4a2...
    return [set ::tkPriv($key)]
  } else {
    ## This is needed for tk8.4a3 or newer...
    return [set ::tk::Priv($key)]
  }
};# TkGUI_AccessPriv

## Use this function to store new values in ::tk::Priv or tkPriv arrays...
proc TkGUI_SetPriv {key val} {
  if {[llength [info globals tkPriv]]} {
    ## This will work up to 8.4a2...
    return [set ::tkPriv($key) $val]
  } else {
    ## This is needed for tk8.4a3 or newer...
    return [set ::tk::Priv($key) $val]
  }
};# TkGUI_SetPriv

## Use this function to remove values in ::tk::Priv or tkPriv arrays...
proc TkGUI_UnsetPriv {key} {
  if {[llength [info globals tkPriv]]} {
    ## This will work up to 8.4a2...
    catch {unset ::tkPriv($key)}
  } else {
    ## This is needed for tk8.4a3 or newer...
    unset -nocomplain ::tk::Priv($key)
  }
};# TkGUI_UnsetPriv

## Use this function to check if a specific key is defined in
## ::tk::Priv or tkPriv arrays...
proc TkGUI_ExistsPriv {key} {
  if {[info exists ::tk::Priv($key)]} {
    return 1
  }
  if {[info exists ::tkPriv($key)]} {
    return 1
  }
  return 0
};# TkGUI_ExistsPriv
