#!/usr/bin/wish
#
# Copyright (C) 1997,1998 D. Richard Hipp
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   drh@acm.org
#   http://www.hwaci.com/drh/
#
# $Revision: 1.6 $
#

#switch $tcl_platform(platform) {
#  unix {
#    set Tree(font) \
#      -adobe-helvetica-medium-r-normal-*-11-80-100-100-p-56-iso8859-1
#  }
#  windows {
#    set Tree(font) \
#      -adobe-helvetica-medium-r-normal-*-14-100-100-100-p-76-iso8859-1
#  }
#}
set Tree(font) ExplorerFont
set Tree(font_underline) ExplorerFontUnderline
set Tree(font_italic) ExplorerFontItalic

#
# Create a new tree widget.  $args become the configuration arguments to
# the canvas widget from which the tree is constructed.
#
proc Tree:create {w args} {
  global Tree
  if {[winfo exists $w] && [string equal [winfo class $w] Canvas]} {
    catch {eval $w configure $args}
  } else {
    eval canvas $w $args
    bind $w <Expose> {catch {Widget_SetScrollRegion %W 10}}
  }
  bind $w <Destroy> \
    "Tree:delitem $w /; array unset Tree $w:*;unset Tree(SortMode:$w)"
  Tree:dfltconfig $w /
  Tree:buildwhenidle $w
  set Tree($w:selection) {}
  set Tree($w:selidx) {}
  set Tree(SortMode:$w) 1
}

#
# Reset Tree State
#
proc Tree:Reset w {
  global Tree
  Tree:delitem $w /
  Tree:dfltconfig $w /
  set Tree($w:selection) {}
  set Tree($w:selidx) {}
}


# Initialise a element of the tree.
# Internal use only
#
proc Tree:dfltconfig {w v} {
  global Tree
  array set Tree [list\
    $w:$v:children {} $w:$v:open    0 $w:$v:tags  {} \
    $w:$v:icon     {} $w:$v:Oicon  {} $w:$v:Licon {} $w:$v:LOicon {} \
    $w:$v:icono    {} $w:$v:Oicono {}]
  #set Tree($w:$v:Data)      {}
}

#
# Pass configuration options to the tree widget
#
proc Tree:config {w args} {
  eval $w config $args
}

proc Tree:sort {w args} {
  global Tree
  if {[llength $args]} {
    set Tree(SortMode:$w) [lindex $args 0]
  } else {
    return $Tree(SortMode:$w)
  }
}

#
# Insert a new element $v into the tree $w.
#
proc Tree:newitem {w v args} {
  global Tree
  set dir [file dirname $v]
  set n [file tail $v]
  if {![info exists Tree($w:$dir:open)]} {
    error "parent item \"$dir\" is missing"
  }
  set i [lsearch -exact $Tree($w:$dir:children) $n]
  if {$i>=0} {
    error "item \"$v\" already exists"
  }
  lappend Tree($w:$dir:children) $n
  if {$Tree(SortMode:$w)} {
    set Tree($w:$dir:children) [lsort $Tree($w:$dir:children)]
  }
  Tree:dfltconfig $w $v
  foreach {op arg} $args {
    switch -exact -- $op {
      -image          {set Tree($w:$v:icon)          $arg}
      -overimage      {set Tree($w:$v:Oicon)         $arg}
      -largeimage     {set Tree($w:$v:Licon)         $arg}
      -largeoverimage {set Tree($w:$v:LOicon)        $arg}
      -openimage      {set Tree($w:$v:icono)         $arg}
      -overopenimage  {set Tree($w:$v:Oicono)        $arg}
      -tags           {set Tree($w:$v:tags)          $arg}
      -data           {set Tree($w:$v:Data)          $arg}
      -tooltip        {set Tree($w:$v:Tooltip)       $arg}
      -tooltipimg     {set Tree($w:$v:TooltipImg)    $arg}
    }
  }
  Tree:buildwhenidle $w
}

#
# Delete element $v from the tree $w.  If $v is /, then the widget is
# deleted.
#
proc Tree:delitem {w v} {
  global Tree
  if {![info exists Tree($w:$v:open)]} {return}
  if {[string equal $v /]} {
    # delete the whole widget
    # catch {destroy $w} Petasis, 11/8/1999 - don't delete widget
    array unset Tree $w:*
    ## Petasis, 10/2/1999: We have removed all excess entries from array,
    ## so, we can return...
    return
  }
  foreach c $Tree($w:$v:children) {
    catch {Tree:delitem $w $v/$c}
  }
  array unset Tree $w:$v:*
  set dir [file dirname $v]
  set n [file tail $v]
  set i [lsearch -exact $Tree($w:$dir:children) $n]
  if {$i>=0} {
    set Tree($w:$dir:children) [lreplace $Tree($w:$dir:children) $i $i]
  }
  Tree:buildwhenidle $w
}

proc Tree:NodeChildren {w node} {
  global Tree
  if {[info exists Tree($w:$node:children)]} {
    return $Tree($w:$node:children)
  } else {
    if {![info exists Tree($w:$node:open)]} {
      error "node \"$node\" does not exists in Tree $w!"
    }
    return {}
  }
}

#
# Change the selection to the indicated item
#
proc Tree:setselection {w args} {
  global Tree
  foreach v $args {
    set Tree($w:selection) [list $v]
  }
  Tree:drawselection $w
}

#
# Retrieve the current selection
#
proc Tree:getselection w {
  global Tree
  return $Tree($w:selection)
}

#
# Bitmaps used to show which parts of the tree can be opened.
#
set maskdata "#define solid_width 9\n#define solid_height 9"
append maskdata {
  static unsigned char solid_bits[] = {
   0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01,
   0xff, 0x01, 0xff, 0x01, 0xff, 0x01
  };
}
set data "#define open_width 9\n#define open_height 9"
append data {
  static unsigned char open_bits[] = {
   0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x7d, 0x01, 0x01, 0x01,
   0x01, 0x01, 0x01, 0x01, 0xff, 0x01
  };
}
image create bitmap Tree:openbm -data $data -maskdata $maskdata \
  -foreground $GUI_Options(ExplorerLineFg) \
  -background $GUI_Options(ExplorerLineBg)
set data "#define closed_width 9\n#define closed_height 9"
append data {
  static unsigned char closed_bits[] = {
   0xff, 0x01, 0x01, 0x01, 0x11, 0x01, 0x11, 0x01, 0x7d, 0x01, 0x11, 0x01,
   0x11, 0x01, 0x01, 0x01, 0xff, 0x01
  };
}
image create bitmap Tree:closedbm -data $data -maskdata $maskdata \
  -foreground $GUI_Options(ExplorerLineFg) \
  -background $GUI_Options(ExplorerLineBg)

# Internal use only.
# Draw the tree on the canvas
proc Tree:build w {
  global Tree
  $w delete all
  catch {unset Tree($w:buildpending)}
  set Tree($w:y) 30
  Tree:buildlayer $w / 10
  ## Widget_SetScrollRegion $w
  Tree:drawselection $w
  event generate $w <Configure>
  event generate $w <Expose>
}

# Internal use only.
# Build a single layer of the tree on the canvas.  Indent by $in pixels
proc Tree:buildlayer {w v in} {
  global Tree GUI_Options
  set TreeXincr $GUI_Options(ExplorerPadX)
  set TreeYincr $GUI_Options(ExplorerPadY)
  if {[string equal $v /]} {
    set vx {}
    set start [expr {$Tree($w:y) - (($TreeYincr + 25) / 2) + 2}];# -10
  } else {
    set vx $v
    set root_icon $Tree($w:$v:icon)
    set root_ih 25
    catch {set root_ih [image height $root_icon]}
    set start [expr {$Tree($w:y) - (($TreeYincr + $root_ih) / 2) + 2}];# -10
  }

  foreach c $Tree($w:$v:children) {
    set iw 0; set ih 0
    set y $Tree($w:y)
    set icon $Tree($w:$vx/$c:icon)
    set icono $Tree($w:$vx/$c:icono)
    catch {set iw [image width $icon]; set ih [image height $icon]}
    incr Tree($w:y) [expr {$TreeYincr + $ih}]
    $w create line $in $y [expr {$in+10}] $y -fill $GUI_Options(ExplorerLineFg)
    set taglist [concat x $Tree($w:$vx/$c:tags)]
    set x [expr {$in+12}]
    if {[string length $icon]>0} {
      set k [$w create image $x $y -image $icon -anchor w -tags $taglist]
      ## Change to over icon if mouse is over...
      $w bind $k <Enter> "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:Oicon)\""
      $w bind $k <Leave> "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:icon)\""
      incr x [expr {$TreeXincr + $iw}]
      set Tree($w:tag:$k) $vx/$c
      if {[info exists Tree($w:$vx/$c:Tooltip)]} {
        if {[info exists Tree($w:$vx/$c:TooltipImg)]} {
          Tooltip::canvas_tooltip $w $k $Tree($w:$vx/$c:Tooltip) \
            $Tree($w:$vx/$c:TooltipImg)
        } else {
          Tooltip::canvas_tooltip $w $k $Tree($w:$vx/$c:Tooltip)
        }
      }
    }
    set text_label $c
    if {[string equal $text_label {{}}]} {
      set text_label ${::GUI_Text(<Empty>)}
      set text [$w create text $x $y -text $text_label -font $Tree(font_italic)\
                -fill $GUI_Options(ExplorerFg) \
                -activefill $GUI_Options(ExplorerActiveFg) \
                -anchor w -tags $taglist]
    } elseif {[string match { [0123456789][0123456789][0123456789]* *... } $c]} {
      set text [$w create text $x $y -text $text_label -font $Tree(font_italic)\
                   -fill $GUI_Options(ExplorerFg) \
                   -activefill $GUI_Options(ExplorerActiveFg) \
                   -anchor w -tags $taglist]
    } else {
      ## Petasis, 13/12/2001: The following command is needed in order to be
      ## able to use special characters as "/" (\u002f) as unicode chars, in
      ## the form \u????...
      set text_label [subst -nocommands -novariables $text_label]
      set text [$w create text $x $y -text $text_label -font $Tree(font) \
                   -fill $GUI_Options(ExplorerFg) \
                   -activefill $GUI_Options(ExplorerActiveFg) \
                   -anchor w -tags $taglist]
      $w bind $text <Enter> "$w itemconf $text -font $Tree(font_underline);\
         catch \"$w itemconf $k -imag $Tree($w:$vx/$c:Oicon)\""
      $w bind $text <Leave> "$w itemconf $text -font $Tree(font);\
         catch \"$w itemconf $k -imag $Tree($w:$vx/$c:icon)\""
    }
    if {[info exists Tree($w:$vx/$c:Tooltip)]} {
      if {[info exists Tree($w:$vx/$c:TooltipImg)]} {
        Tooltip::canvas_tooltip $w $text $Tree($w:$vx/$c:Tooltip) \
            $Tree($w:$vx/$c:TooltipImg)
      } else {
        Tooltip::canvas_tooltip $w $text $Tree($w:$vx/$c:Tooltip)
      }
    }
    set Tree($w:tag:$text) $vx/$c
    set Tree($w:$vx/$c:tag) $text
    if {$ih == 0} {
      set bbox [$w bbox $text]
      set height [expr {[lindex $bbox 3] - [lindex $bbox 1]}]
      incr Tree($w:y) [expr {$TreeYincr + 2 * $height}]
    }
    if {[string length $Tree($w:$vx/$c:children)]} {
      if {$Tree($w:$vx/$c:open)} {
         set j [$w create image $in $y -image Tree:openbm]
         if {$ih == 0} {
           incr Tree($w:y) [expr int($TreeYincr + [image height Tree:openbm])]
         }
         $w bind $j <<MB_1>> [list Tree:close $w $vx/$c]
         $w bind $j <<MB_2>> [list Tree:close $w $vx/$c]
         #$w bind $j <<MB_2>> "set \"Tree($w:$vx/$c:open)\" 0; Tree:build $w"
         Tree:buildlayer $w $vx/$c [expr {$in+18}]
         if {[string length $icono]>0} {
           $w itemconfigure $k -image $icono
           $w bind $k <Enter> \
             "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:Oicono)\""
           $w bind $k <Leave> \
             "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:icono)\""
           $w bind $text <Enter> \
             "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:Oicono)\""
           $w bind $text <Leave> \
             "catch \"$w itemconf $k -imag $Tree($w:$vx/$c:icono)\""
         }
      } else {
         set j [$w create image $in $y -image Tree:closedbm]
         if {$ih == 0} {
           incr Tree($w:y) [image height Tree:openbm]
         }
         $w bind $j <<MB_1>> [list Tree:open $w $vx/$c]
         $w bind $j <<MB_2>> [list Tree:open $w $vx/$c]
         #$w bind $j <<MB_2>> "set \"Tree($w:$vx/$c:open)\" 1; Tree:build $w"
      }
    }
  }
  catch {
    set j [$w create line $in $start $in [expr {$y+1}] \
                     -fill $GUI_Options(ExplorerLineFg)]
    $w lower $j
  }
}

# Open a branch of a tree
#
proc Tree:open {w v} {
  global Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==0
      && [info exists Tree($w:$v:children)]
      && [string length $Tree($w:$v:children)]>0} {
    set Tree($w:$v:open) 1
    Tree:build $w
  }
}

proc Tree:close {w v} {
  global Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==1} {
    set Tree($w:$v:open) 0
    Tree:build $w
  }
}

# Internal use only.
# Draw the selection highlight
proc Tree:drawselection w {
  global Tree GUI_Options
  if {[string length $Tree($w:selidx)]} {
    foreach selidx $Tree($w:selidx) tag $Tree($w:selidx:tag) {
      $w delete $selidx
      $w itemconfigure $tag -fill $GUI_Options(ExplorerFg)
    }
  }
# if {[string length $Tree($w:selidx)]} {
#   $w delete $Tree($w:selidx)
#   $w itemconfigure $Tree($w:selidx:tag) -fill $GUI_Options(ExplorerFg)
# }
# set v $Tree($w:selection)
  set Tree($w:selidx) {}
  foreach v $Tree($w:selection) {
    if {![string length $v]} {continue}
    if {![info exists Tree($w:$v:tag)]} {continue}
    set bbox [$w bbox $Tree($w:$v:tag)]
    if {[llength $bbox]==4} {
      set i [eval $w create rectangle $bbox -fill \
      $GUI_Options(ExplorerHilightBg) -outline {{}}]
      $w itemconfigure $Tree($w:$v:tag) -fill $GUI_Options(ExplorerHilightFg)
      lappend Tree($w:selidx) $i
      lappend Tree($w:selidx:tag) $Tree($w:$v:tag)
      $w lower $i
    }
  }
}

# Internal use only
# Call Tree:build then next time we're idle
proc Tree:buildwhenidle w {
  global Tree
  if {![info exists Tree($w:buildpending)]} {
    set Tree($w:buildpending) 1
    after idle "Tree:build $w"
  }
}

#
# Return the full pathname of the label for widget $w that is located
# at real coordinates $x, $y
#
proc Tree:labelat {w x y} {
  set x [$w canvasx $x]
  set y [$w canvasy $y]
  global Tree
  foreach m [$w find overlapping $x $y $x $y] {
    if {[info exists Tree($w:tag:$m)]} {
      return $Tree($w:tag:$m)
    }
  }
  return {}
}
