#!/usr/bin/wish
#
# Copyright (C) 1998-2004 George P. Petasis
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   petasis@iit.demokritos.gr
#   http://www.edu.physics.uch.gr/~petasis
#
# $Revision: 1.0 $
#


#
# Create a new TreeView widget.  $args become the configuration arguments to
# the canvas widget from which the tree view is constructed.
#
proc TreeView:create {w args} {
  global Tree
  ## Create the default image
  set Tree(TreeView:IUp:${w}:Image) \
      [image create photo "TreeView:IUp:Default:$w"]
  set Tree(TreeView:IUp:${w}:Label) ".."
  set Tree(TreeView:$w:Cmd) {}
  set Tree(TreeView:$w:RedrawRequest) 0
  if {[winfo exists $w] && [string equal [winfo class $w] Canvas]} {
    catch {eval $w configure $args}
  } else {
    eval canvas $w $args
    bind $w <Configure> "+TreeView:draw $w"
  }
  bind $w <Destroy> "TreeView:UpFolderImage $w {}; \
  unset -nocomplain Tree(TreeView:IUp:${w}:Image) Tree(TreeView:IUp:${w}:Label)"
  return $w
}

## Link a TreeView Widget with a Tree Widget
proc TreeView:link {w TreeWidget} {
  global Tree
  set Tree($w:Tree) $TreeWidget
  set Tree($w:selidx) {}
}

## Define what function to call when a "system" gets double-clicked
proc TreeView:Call {w Cmd} {
  global Tree
  set Tree(TreeView:$w:Cmd) $Cmd
}

## Define Up Folder Image
proc TreeView:UpFolderImage {w image} {
  global Tree
  if {[string equal $Tree(TreeView:IUp:$w:Image) TreeView:IUp:Default:$w]} {
    catch {image delete "TreeView:IUp:Default:$w"}
  }
  set Tree(TreeView:IUp:$w:Image) $image
  catch {image delete $Tree(TreeView:IUp:$w:LightImage)}
  if {[string length $image]} {
    set Tree(TreeView:IUp:$w:LightImage) \
        [image create photo TreeView:IUp:ImageOver:$w]
    $Tree(TreeView:IUp:$w:LightImage) copy $image
    $Tree(TreeView:IUp:$w:LightImage) configure \
       -gamma $::GUI_Options(OverImageGamma)
  }
  catch {TreeView:draw $w}
  return $image
}
## Define Up Folder Label
proc TreeView:UpFolderLabel {w label} {
  global Tree
  set Tree(TreeView:IUp:${w}:Label) $label
  catch {TreeView:draw $w}
  return $label
}

#
# Pass configuration options to the tree view widget
#
proc TreeView:config {w args} {
  eval $w config $args
}

#
# Change the selection to the indicated item
#
proc TreeView:setselection {w args} {
  global Tree GUI_MainWidgets
  set T $Tree($w:Tree)
  set Tree($T:selection) $args
  ##set Tree($w:selidx) {}
  Tree:drawselection $T
  TreeView:drawselection $w
  set GUI_MainWidgets([winfo toplevel $w]:CurrentSelection) $args
  if {![string equal $args /]} {
    catch {unset GUI_MainWidgets(TreeView:x) GUI_MainWidgets(TreeView:y)}
  }
}

#
# Retrieve the current selection
#
proc TreeView:getselection w {
  global Tree
  set T $Tree($w:Tree)
  return $Tree($T:selection)
}

# Internal use only.
# Draw the tree on the canvas
#proc TreeView:draw {w {update {1}} } {
#  global Tree
#  incr Tree(TreeView:$w:RedrawRequest)
#  puts "Arranging TreeView Redraw... ($Tree(TreeView:$w:RedrawRequest))"
#  after idle [list catch [list TreeView:drawOnlyOnce $w $update]]
#}
#proc TreeView:drawOnlyOnce {w update} {
proc TreeView:draw {w {update {1}} } {
  global Tree GUI_Graphs GUI_Options
#  if {$Tree(TreeView:$w:RedrawRequest) > 1} {
#    incr Tree(TreeView:$w:RedrawRequest) -1
#    puts "Redraw postponed... ($Tree(TreeView:$w:RedrawRequest))"
#    #after idle [list TreeView:drawOnlyOnce $w $update]
#    return
#  }
#  set Tree(TreeView:$w:RedrawRequest) 0
  #puts "Drawing TreeView..."
  set T $Tree($w:Tree)
  set v [lindex [TreeView:getselection $w] 0]
  if {![string length $v] || ![info exists Tree($T:$v:children)]} {return}

  $w delete all
  set x_visible [$w xview]
  set y_visible [$w yview]
  $w configure -scrollregion {0 0 0 0}
  set canvas_width [winfo width $w]

  ## find the largest's image geometry
  set vx $v
  if {[llength $Tree($T:$v:children)] == 0} {set v [file dir $v]}
  set children $Tree($T:$v:children)
  if {[string equal $v /]} {
    set v {}
  }
  set image_width  [image width  $Tree(TreeView:IUp:$w:Image)]
  set image_height [image height $Tree(TreeView:IUp:$w:Image)]
  ## Petasis, 28/11/1999: If the item has the Data field filled, then this
  ## node should be expanded...
  if {![info exists Tree($T:$v/[lindex $children 0]:Data)]} {
    set Tree($T:$v/[lindex $children 0]:Data) {}
  }
  if {[llength $children] == 1 && \
      [llength $Tree($T:$v/[lindex $children 0]:Data)] > 1} {
    set add_files 1
    if {[winfo exists ${w}_listbox.listbox]} {
      if {![info exists Tree($w:ListBox:selection)] || \
          ![string equal $Tree($w:ListBox:selection) $v]} {
        ${w}_listbox.listbox delete 0 end
      } else {
        set add_files 0
      }
    } else {
      ::ScrolledWidgets::ScrolledListbox ${w}_listbox \
         -relief $GUI_Options(ExplorerRelief) \
         -borderwidth $GUI_Options(ExplorerBorderWidth) \
         -bg $GUI_Options(ExplorerBg) -highlightthickness 0 \
         -scrollwidth 12 -fg $GUI_Options(ExplorerFg) \
         -font ExplorerFont

      set label [winfo parent $w].label
      grid forget $w
      grid ${w}_listbox -row 0 -column 0 -sticky snew
      #raise $label ${w}_listbox
      #raise ${w}_listbox.vertical $label
    }
    ## Listbox bindings...
    if {![string length $v]} {set vx /}; set vx [file dir $vx]
    bind ${w}_listbox.listbox <<MB_1>> [list +after idle [list \
         TreeView:HandleListbox %W $w single_mouse $vx $v]]
    bind ${w}_listbox.listbox <Double-Button-1> [list +after idle [list \
         TreeView:HandleListbox %W $w double_mouse $vx $v]]
    bind ${w}_listbox.listbox <KeyRelease> [list +after idle [list \
         TreeView:HandleListbox %W $w key $vx $v]]

    set Data $Tree($T:$v/[lindex $children 0]:Data)
    ${w}_listbox.listbox selection clear 0 end
    if {$add_files} {
      set Tree($w:ListBox:selection) $v
      ${w}_listbox.listbox insert end ..
      foreach doc $Data {
        ${w}_listbox.listbox insert end [lindex $doc 0]
      }
    }
  } else {
    if {[winfo exists ${w}_listbox.listbox]} {
      catch {unset Tree($w:ListBox:selection)}
      destroy ${w}_listbox
      grid $w -row 0 -column 0 -sticky snew
    }
    foreach c $children {
      set image $Tree($T:$v/$c:Licon)
      if {![string length $image]} {set image $Tree($T:$v/$c:icon)}
      catch {
        if {[image width $image] > $image_width} {
          set image_width [image width $image]
        }
        if {[image height $image] > $image_height} {
          set image_height [image height $image]
        }
      }
    }

    set padx $GUI_Options(ExplorerVPadX)
    set pady $GUI_Options(ExplorerVPadY)
    set x [expr {$padx + ($image_width  / 2.0)}]
    set y [expr {$pady + ($image_height / 2.0)}]

    if {[string length $v]} {
    ## The Up folder...
    set k [$w create image $x $y -image $Tree(TreeView:IUp:$w:Image) \
              -activeimage $Tree(TreeView:IUp:$w:LightImage) -anchor c]
    set j [$w create text $x [expr {$y+($image_height/2.0)}] -text \
        $Tree(TreeView:IUp:${w}:Label) -width [expr {$image_width + $padx -1}] \
        -font $Tree(font_italic) -fill $GUI_Options(ExplorerFg) \
        -activefill $GUI_Options(ExplorerActiveFg) -anchor n -justify c]
    foreach widg [list $k $j] {
      if {![string length $v]} {set vx /}
      set vx [file dir $vx]
      $w bind $widg <<MB_1>> [list TreeView:setselection $w $vx]
      $w bind $widg <Double-Button-1> "TreeView:setselection $w /;\
        TreeView:draw $w;[list Tree:open $T $vx]"
      }
      $w bind $widg <Enter> \
        "catch \"$w itemconf $k -image $Tree(TreeView:IUp:$w:LightImage)\""
      $w bind $widg <Leave> \
        "catch \"$w itemconf $k -image $Tree(TreeView:IUp:$w:Image)\""
      set x [expr {$x + $padx + $image_width}]
    }
    set NULL_Index [lsearch -exact $children {{}}]
    if {$NULL_Index != -1} {
      set children [lreplace $children $NULL_Index $NULL_Index]
    }
    foreach c $children {
      #if {[string equal $c {{}}]} continue
      if {$x > $canvas_width} {
        set x [expr {$padx + ($image_width  / 2.0)}]
        set y [expr {$y + $image_height + $pady}]
      }
      ## Get the Proper Image...
      set image $Tree($T:$v/$c:Licon)
      set omage $Tree($T:$v/$c:LOicon)
      if {![string length $image]} {
        set image $Tree($T:$v/$c:icon)
        set omage $Tree($T:$v/$c:Oicon)
      }

      if {![llength $Tree($T:$v/$c:children)]} {
        set type system
        set tags [list system $v/$c]
        if {[string length $image]} {
          set k [$w create image $x $y -image $image -activeimage $omage \
                    -anchor c -tags $tags]
        }
      } else {
        set type folder
        set tags [list folder $v/$c]
        if {[string length $image]} {
          set k [$w create image $x $y -image $image -activeimage $omage \
                    -anchor c -tags $tags]
        }
      }

      set j [$w create text $x [expr {$y+($image_height/2.0)}] -text \
        $c -font $Tree(font) -fill $GUI_Options(ExplorerFg) -anchor n\
        -activefill $GUI_Options(ExplorerActiveFg) -justify c \
        -width [expr {$image_width + $padx -1}]  -tags $tags]
      $w bind $j <Enter> "$w itemconf $j -font $Tree(font_underline);\
                          $w itemconf $k -image $omage"
      $w bind $j <Leave> "$w itemconf $j -font $Tree(font);\
                          $w itemconf $k -image $image"
      $w bind $k <Enter> "$w itemconf $j -font $Tree(font_underline)\
                                         -fill $GUI_Options(ExplorerActiveFg)"
      $w bind $k <Leave> "$w itemconf $j -font $Tree(font)\
                                         -fill $GUI_Options(ExplorerFg)"

      ## Is a Tooltip registered?
      if {[info exists Tree($T:$v/$c:Tooltip)]} {
        if {[info exists Tree($T:$v/$c:TooltipImg)]} {
          Tooltip::canvas_tooltip $w $k $Tree($T:$v/$c:Tooltip) \
            $Tree($T:$v/$c:TooltipImg)
          Tooltip::canvas_tooltip $w $j $Tree($T:$v/$c:Tooltip) \
            $Tree($T:$v/$c:TooltipImg)
        } else {
          Tooltip::canvas_tooltip $w $k $Tree($T:$v/$c:Tooltip)
          Tooltip::canvas_tooltip $w $j $Tree($T:$v/$c:Tooltip)
        }
      }
      set Tree($w:$v/$c:tag) [list $j $k]
      set Tree($w:tag:$k) $v/$c
      if {[string equal $type system]} {
        if {[string length $Tree(TreeView:$w:Cmd)]} {
          $w bind $k <Double-Button-1> "[list $Tree(TreeView:$w:Cmd) $c]"
          $w bind $j <Double-Button-1> "[list $Tree(TreeView:$w:Cmd) $c]"
        }
        if {[info exists GUI_Graphs($c)]} {
          set system [lindex $GUI_Graphs($c) 0]
          set command "[list TreeView:setselection $w $v/$c];\
             [list Tree:open $T $v/$c];after idle \"TreeView:draw $w\""
        } else {
          set command "[list TreeView:setselection $w $v/$c];TreeView:draw $w"
        }
      } else {
        set command "[list TreeView:setselection $w $v/$c];TreeView:draw $w"
        $w bind $k <Double-Button-1> "$command;[list Tree:open $T $v/$c]"
        $w bind $j <Double-Button-1> "$command;[list Tree:open $T $v/$c]"
        set command "[list TreeView:setselection $w $v/$c];\
                     TreeView:drawselection $w;update"
      }
      $w bind $k <<MB_1>> $command
      $w bind $j <<MB_1>> $command
      $w bind $k <<MB_2>> $command
      $w bind $j <<MB_2>> $command
      set x [expr {$x + $padx + $image_width}]
    }
  }
  set update [string tolower $update]
  #if {$update == "1" || $update == "yes"} {Widget_SetScrollRegion $w 10}
  if {$update == "1" || [string equal $update yes]} {
    event generate $w <Configure>
    event generate $w <Expose>
  }
  TreeView:drawselection $w
  set Tree($w:TreeView:RedrawDone) 1
  foreach {x1 x2} $x_visible {y1 y2} $y_visible {
    $w xview moveto [expr {$x1/$x2}]
    $w yview moveto [expr {$y1/$y2}]
    break
  }
}

proc TreeView:HandleListbox {listbox w event vx v} {
  if {![winfo exists $listbox]} return
  global Tree
  set T $Tree($w:Tree)
  set csel [$listbox curselection]
  if {[llength $csel] == 0} {return}
  set Tree($w:ListBox:selection) $v
  set sel [$listbox get [$listbox curselection]]
  if {![info exists Tree($T:$v/$sel:children)]} {
    set Tree($T:$v/$sel:children) {}
  }
  if {[string equal $Tree($T:$v/$sel:children) {}]} {
    set type system
  } else {
    set type folder
  }
  switch $event {
    double_mouse {
      if {[string equal $sel ..]} {
        TreeView:setselection $w $vx;TreeView:draw $w;Tree:open $T $vx
      } else {
        set Tree($w:$v/$sel:tag) {}
        if {[string equal $type system]} {
          if {![string length $Tree(TreeView:$w:Cmd)]} {
            $Tree(TreeView:$w:Cmd) $sel
          }
        } else {
          TreeView:setselection $w $v/$sel;TreeView:draw $w;Tree:open $T $v/$sel
        }
      }
    }
    key -
    single_mouse {
      if {[string equal $sel ..]} {return}
      if {[string equal $type system]} {
        if {[info exists GUI_Graphs($sel)]} {
          TreeView:setselection $w $v/$sel
          #Tree:open $T $v/$sel;
          #after idle {TreeView:draw $w}
        } else {
          TreeView:setselection $w $v/$sel
          #TreeView:draw $w
        }
      } else {
        TreeView:setselection $w $v/$sel
        TreeView:drawselection $w
        update
      }
    }
  }
  catch {focus $listbox}
}

# Open an item of a tree view
#
proc TreeView:open {w v} {
  global Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==0
      && [info exists Tree($w:$v:children)]
      && [string length $Tree($w:$v:children)]>0} {
    set Tree($w:$v:open) 1
    Tree:build $w
  }
}

# Internal use only.
# Draw the selection highlight
proc TreeView:drawselection {w} {
  global Tree GUI_Options
  if {[string length $Tree($w:selidx)]} {
    foreach selEntry $Tree($w:selidx) itemEntry $Tree($w:selidx:tag) {
      foreach {selidx selidximg} $selEntry {break}
      foreach {text img} $itemEntry {break}
      catch {$w delete $selidx $selidximg}
      catch {$w itemconfigure $text -fill $GUI_Options(ExplorerFg)}
      catch {$w bind $img <Leave> "$w itemconf $text -font $Tree(font)\
                                         -fill $GUI_Options(ExplorerFg)"}
    }
  }
  set Tree($w:selidx) {}; set Tree($w:selidx:tag) {}
  set T $Tree($w:Tree)
  foreach v $Tree($T:selection) {
    #if {[llength $Tree($T:$v:children)] == 0} {set v [file dir $v]}
    #if {$v == "/"} {set v {}}
    if {![string length $v]} {return}
    if {![info exists Tree($w:$v:tag)]} {return}
    foreach {text_tag_id img_tag_id} $Tree($w:$v:tag) {break}
    set bbox [$w bbox [lindex $Tree($w:$v:tag) 1]]
    if {[llength $bbox]==4} {
      foreach {x y W H} $bbox {break}
      incr x  2; incr y  2
      incr W -2
      set img [$w create rectangle $x $y $W $H -stipple gray50 \
        -fill $GUI_Options(ExplorerHilightBg) -outline {}]
      $w lower $img [lindex $Tree($w:$v:tag) 1]
    }
    set bbox [$w bbox $text_tag_id];# [lindex $Tree($w:$v:tag) 1]]
    if {[llength $bbox]==4} {
      foreach {x y W H} $bbox {break}
      incr x  1; incr W -1
      set i [$w create rectangle $x $y $W $H \
               -fill $GUI_Options(ExplorerHilightBg) -outline {}]
      $w itemconfigure $text_tag_id -fill $GUI_Options(ExplorerHilightFg)
      $w bind $img_tag_id <Leave> "$w itemconf $text_tag_id -font $Tree(font)\
                                         -fill $GUI_Options(ExplorerHilightFg)"
      lappend Tree($w:selidx) [list $i $img]
      lappend Tree($w:selidx:tag) [list $text_tag_id $img_tag_id]
      $w lower $i
      focus $w
    }
  }
}
