##############################################################################
##  Utilities.tcl:                                                          ##
## ---------------------                                                    ##
##  This file implements some tk widget facilities.                         ##
##  Initial version date: 15/9/1998                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## Widget_CanvasSeeItem
#  Scroll a canvas viewable area so as item to be centered on the canvas
#  viewable area...
proc Widget_CanvasSeeItem {canvas id} {
  foreach {x1 y1 x2 y2} [$canvas bbox $id] {}
  ## Get the (x,y) of the item...
  set x [expr {($x2-$x1)/2.0+$x1}]; set y [expr {($y2-$y1)/2.0+$y1}]
  ## Measure distances from left & top corners...
  foreach {x1 y1 x2 y2} [$canvas cget -scrollregion] {}
  set dx [expr {$x-$x1}]; set dy [expr {$y-$y1}]
  ## Calculate the franctions...
  set fx [expr {$dx/($x2-$x1)}]; set fy [expr {$dy/($y2-$y1)}]
  ## Center the item within the widget...
  set w [winfo width  $canvas]
  set h [winfo height $canvas]
  set dfx [expr {$w/2.0/($x2-$x1)}]; set dfy [expr {$h/2.0/($y2-$y1)}]
  $canvas xview moveto [expr {$fx-$dfx}]
  $canvas yview moveto [expr {$fy-$dfy}]
};# Widget_CanvasSeeItem

## Widget_GetScrollRegion
proc Widget_GetScrollRegion { canvas {padx 0} {pady 0} } {
  set bbox   [$canvas bbox all]
  set width  [winfo width $canvas]
  set height [winfo height $canvas]
  set bd [expr {2*[$canvas cget -borderwidth]}]
  set hw [expr {2*[$canvas cget -highlightthickness]}]
  set x1 [expr {[lindex $bbox 0] - $padx}]
  set y1 [expr {[lindex $bbox 1] - $pady}]
  set x2 [expr {[lindex $bbox 2] + $padx}]
  set y2 [expr {[lindex $bbox 3] + $pady}]
  set scroll_width  [expr {$x2-$x1}]
  set scroll_height [expr {$y2-$y1}]
  if {$scroll_width < $width} {
    set x2 [expr {$x1+$width-$bd-$hw}]
  }
  if {$scroll_height < $height} {
    set y2 [expr {$y1+$height-$bd-$hw}]
  }
  return [list $x1 $y1 $x2 $y2]
}

proc Widget_SetScrollRegion { canvas {padx 0} {pady 0} } {
  $canvas configure -scrollregion [Widget_GetScrollRegion $canvas $padx $pady]
  #$canvas xview moveto 0
  #$canvas yview moveto 0
}

## PANE Widget Utilities...
proc Widget_PaneSet {master handle percent} {
  if {$percent < 0.0 || $percent > 100.0} {return 0}
  set handle ${master}.__h$handle
  if {![winfo exists $handle]} {return 0}
  global PANE 
  set PANE(OffSet) 0.0
  if {$percent <= 1.0} {set percent [expr {$percent*100.0}]}
  eval [bind $handle <Button-1>]
  set orientation $PANE(${master},o)

  if {[string match hor* $orientation]} {
    set area [expr {[winfo width $PANE(W0)] + [winfo width $PANE(W1)]}]
    set start [winfo rootx $PANE(W0)]
  } else {
    set area [expr {[winfo height $PANE(W0)] + [winfo height $PANE(W1)]}]
    set start [winfo rooty $PANE(W0)]
  }
  if {$area == 2} {
    set PANE(C0) [expr {$percent/100.0}]
    set PANE(C1) [expr {$percent/100.0}]
  }
  
  set percent [expr {int($start + ($area *($percent/100.0)))}]
  set cmd [bind $handle <ButtonRelease-1>]
  eval [lreplace $cmd 1 1 $percent]
  unset PANE(OffSet)
  return 1
}
