#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  BuildNullSoftInstaller.tcl:                                             ##
## ---------------------                                                    ##
##  A script for writting a configureation file for building a windows      ##
##  with NullSoft's installer builder.                                      ##
##                                                                          ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

catch {console show;update}
set ScriptDir [file dirname [file normalize [info script]]]
cd $ScriptDir

set binaries 1
set sources  1

## Retrieve Ellogon version...
set EllogonVersion [lindex [exec [info nameofexe] ../ellogon \
                                                  -printVersion] end]
puts ">>> Building tar.gz distribution for Ellogon version $EllogonVersion..."
set EllogonVersionNoDots [string map {. {}} $EllogonVersion]
if {[string length $EllogonVersionNoDots] == 2} {
  set EllogonVersionNoDots ${EllogonVersionNoDots}0
}

if {$binaries} {
## Gather the list of all files to be installed...
  set EllogonInstallationImage ../cmake/runtime
  
  puts "  Copying Ellogon files..."; update
  catch {file delete -force ./Ellogon-$EllogonVersionNoDots}
  file copy -force $EllogonInstallationImage Ellogon-$EllogonVersionNoDots
  puts "  Executing tar..."; update
  exec tar -cf Ellogon-$EllogonVersionNoDots.tar Ellogon-$EllogonVersionNoDots
  puts "  Executing bzip2..."; update
  exec bzip2 --compress --keep --best --force Ellogon-$EllogonVersionNoDots.tar
  puts "  Executing gzip..."; update
  exec gzip                    --best --force Ellogon-$EllogonVersionNoDots.tar
  catch {file delete -force ./Ellogon-$EllogonVersionNoDots.tar}
  catch {file delete -force ./Ellogon-$EllogonVersionNoDots}
  puts "  Creating OS X bundle..."; update
  exec [info nameofexe] Darwin/OSXBundle.tcl
  file rename -force Darwin/Ellogon-$EllogonVersionNoDots.tar.gz \
                   ./Ellogon-$EllogonVersionNoDots-OSX-Bundle.tar.gz
}

proc DeleteBinaries {dir} {
  ## Delete folder that can contain binaries/unwanted files...
  foreach one {Darwin Linux SunOS Windows .svn .cvs} {
    catch {file delete -force $dir/$one}
  }
  foreach one [glob -nocomplain -type d $dir/*] {
    DeleteBinaries $one
  }
}

if {$sources} {
  ## Generate a sources distribution...
  set EllogonSources [file normalize ../../Ellogon]
  set EllogonSrcTarget [file normalize ./Ellogon-$EllogonVersionNoDots-src]
  catch {file delete -force $EllogonSrcTarget}
  ## Execute svn export...
  exec svn export [file native $EllogonSources] [file native $EllogonSrcTarget]

  foreach tool   { ARFFVectorCreationTool
                   ExportSGML
                   MAT } {
    puts "  Deleting Tool: $tool"
    file delete -force $EllogonSrcTarget/share/Tools/$tool
  }

  foreach module { HBrillCases
                   HBrillParole
                   HCFGParser
                   HLemmatiser
                   HMorph
                   HPcPatr
                   HStemmer
                   PatternLookup
                   TNTStatisticalTagger } {
    puts "  Deleting module: $module"
    file delete -force $EllogonSrcTarget/share/modules/$module
  }

  DeleteBinaries $EllogonSrcTarget

  puts "  Executing tar..."; update
  exec tar -cf Ellogon-$EllogonVersionNoDots-src.tar \
               Ellogon-$EllogonVersionNoDots-src
  puts "  Executing bzip2..."; update
  exec bzip2 --compress --keep --best --force \
               Ellogon-$EllogonVersionNoDots-src.tar
  puts "  Executing gzip..."; update
  exec gzip                    --best --force \
               Ellogon-$EllogonVersionNoDots-src.tar
  catch {file delete -force ./Ellogon-$EllogonVersionNoDots-src.tar}
  catch {file delete -force ./Ellogon-$EllogonVersionNoDots-src}
}
exit 0
