##############################################################################
##  AnnotationConfiguartion.tcl:                                            ##
## ------------------------------                                           ##
##  An interface for loading Annotation/Attribute associations, stored in a ##
##  format compatible with GATE 1.x...                                      ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

namespace eval ::Annotation {}

## InitialiseMatConfiguration
#  This is the only function users should call. It initialises the 
proc ::Annotation::Init {args} {
  foreach filename [list \
                        $::HomeDir/share/Config/AnnotationTypes.cfg \
                        $::UserAppDir/AnnotationTypes.cfg \
                        $::UserHomeDir/.AnnotationTypes.cfg \
                        $::UserHomeDir/AnnotationTypes.cfg \
                        $::UserHomeDir/.gate.types] {
    if {![file exists $filename]} {continue}
    if {[catch {loadMatConfig $filename} error]} {
      set msg "The following error occured while reading Annotation\
               configurations from file \"$filename\":\n$error"
      catch {bgerror $msg}
    }
  }
};# ::Annotation::Init

## ::Annotation::AnnotationTypes
#  Get a list of all Annotations stored in configuration files...
proc ::Annotation::AnnotationTypes {} {
  lsort [array names ::matTypeIsVisible]
};# ::Annotation::AnnotationTypes

## ::Annotation::AnnotationAttributes
#  Return all registered Attributes for a specific Annotation
proc ::Annotation::AnnotationAttributes {AnnotationType} {
  foreach attribute [array names ::matAttType] {
    foreach {a t} [split $attribute ,] {break}
    if {[string equal $AnnotationType $a]} {
      set Values($t) 0
    }
  }
  if {[info exists Values]} {
    return [lsort [array names Values]]
  }
  return {}
};# ::Annotation::AnnotationAttributes

proc loadMatConfig {fileName} {
  global matTypeIsVisible matAttType matAttOptional matAttValues
  if {![file exists $fileName]} {
    return
  }
  set file [open $fileName r]
  fconfigure $file -encoding utf-8
  set getAtts 0
  while {[gets $file line] >= 0} {
    set line [string trim $line]
    if {!$getAtts && [string match "annotation type *" $line]} {
      set splitStr [split $line " "]
      set type [lindex $splitStr 2]
      if {[info exists matTypeIsVisible($type)]} {
        unset -nocomplain matTypeIsVisible($type)
        foreach i [array names matAttType] {
          if {[string equal [lindex [split $i ,] 0] $type]} {
            unset -nocomplain matAttType($i) matAttOptional($i) matAttValues($i)
          }
        }
      }
      set matTypeIsVisible($type) 0
      set getAtts 1
    } elseif {$getAtts && [string match "\}\;" $line]} {
      set getAtts 0
    } elseif {$getAtts} {
      if {[string match "*:" [lindex $line 0]]} {
        set line [string trim $line ,]
        set att [string trim [lindex $line 0] :]
        set attList [lrange $line 1 end]
        if {[string equal [lindex $attList 0] optional]} {
          set matAttOptional($type,$att) 1
          set attList [lrange $attList 1 end]
        } else {
          set matAttOptional($type,$att) 0
        }
        if {[string equal [lrange $attList 0 1] {sequence of}]} {
          set matAttType($type,$att) [lrange $attList 0 2]
          set matAttValues($type,$att) [lrange $attList 3 end]
        } elseif {[string match "\{*\}" $attList]} {
          set matAttType($type,$att) enumerated
          regsub -all ", " [lindex $attList 0] { } enumVals
          set matAttValues($type,$att) $enumVals
        } else {
          set matAttType($type,$att) [lindex $attList 0]
          set matAttValues($type,$att) [lrange $attList 1 end]
        }
      }
    }
  }
  close $file
};# loadMatConfig
