##############################################################################
##  CompileComponentWizard.tcl:                                             ##
## -----------------------------                                            ##
##  This file implements a wizard for exporting a system or a collection of ##
##  out of Ellogon, in a stand-alone application. This application can have ##
##  a completelly custom interface or run in a batch mode, without requiring##
##  a graphical interface at all...                                         ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

##############################################################################
##############################################################################
## Actually this tool is implemented as a wizard: The user has to be asked a
## series of questions. At the end, the stand-alone application is created,
## according to the user gathered information...
## In order to use this wizard, the caller has to only execute 
## "::CompileComponentWizard::Init"...
##############################################################################
##############################################################################

package ifneeded WizardDialog 1.0 "source $HomeDir/GUI/Dialogs/Wizard.tcl"

namespace eval ::CompileComponentWizard {
  variable Page; # Inernal information about the current wizard page
  ## The following variable holds the sequence of the wizard pages. The pages
  ## will be navigated according to this order. For each such page, a
  ## procedure named ::CompileComponentWizard::Page_<page> should be
  ## available...
  variable Pages {Welcome SelectComponent CheckComponent ProcessConfiguration
                  SelectFiles PreActions PostActions EnableDebug
                  CompilerSpecs Compile}
  variable Info
  set Info(WizardId) 0
}

## ::CompileComponentWizard::Init
#  Initialise internal data structures and start the wizard...
proc ::CompileComponentWizard::Init {{component {}}} {
  package require WizardDialog
  package require tBuild
  variable Page
  variable Info
  variable Components
  set id CCW_$Info(WizardId)
  incr Info(WizardId)
  set ::WizardDialog::Wizard($id|CancelCommand) \
    "array unset ::WizardDialog::Wizard $id|*"
  set ::WizardDialog::Wizard($id|BackCommand) \
    "::CompileComponentWizard::DisplayPage $id prev"
  set ::WizardDialog::Wizard($id|NextCommand) \
    "::CompileComponentWizard::DisplayPage $id next"
  set ::WizardDialog::Wizard($id|FinishCommand) \
    "::CompileComponentWizard::DisplayPage $id last"
  GUI_LoadImage Compile
  ## Set default values...
  set Info($id|Component) $component
  set Info($id|PreCompileScript)  {}
  set Info($id|PostCompileScript) {}
  set Info($id|Library)           {}
  set Info($id|Defines)           {}
  set Info($id|EnableDebug)        0
  ## Display the first page...
  if {[string length $component]} {
    ## Skip "Welcome" and "SelectComponent" pages...
    set Page($id|Current) 1
    DisplayPage $id next
  } else {
    set Page($id|Current) 0
    DisplayPage $id start
  }
};# ::CompileComponentWizard::Init

## ::CompileComponentWizard::DisplayPage
#  This function will display a page according to the current displayed page.
#  It accepts an argumend which can have one of the following values:
#  "start", "next", "prev", "last"
proc ::CompileComponentWizard::DisplayPage {id page} {
  variable Page
  variable Pages

  #if {![string equal $page last] && [string equal $Page(Current) end]} {return}
  switch $page {
    start {set  Page($id|Current)   0}
    next  {incr Page($id|Current)    }
    prev  {incr Page($id|Current)  -1}
    last  {set  Page($id|Current) [expr {[llength $Pages]-1}]}
    default {
      ## This is one of the pages...
      set Page($id|Current) [lsearch $Pages $page]
    }
  }
  ## Make some routine checks...
  if {![string equal $Page($id|Current) end]} {
    if {$Page($id|Current) < 0} {set Page($id|Current) 0}
    if {$Page($id|Current) >= [llength $Pages]} {return}
  }
  ## Run the function for the specific page...
  ::CompileComponentWizard::Page_[lindex $Pages $Page($id|Current)] $id $page
};# ::CompileComponentWizard::DisplayPage

##############################################################################
#### Page: Welcome
proc ::CompileComponentWizard::Page_Welcome {id page args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 0 1 0} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Welcome to the "Compile Dynamic Component" Wizard!} \
    -message "This wizard will help you to compile and link a component,\
    provided that a suitable compiler/linker is installed."]
};# ::CompileComponentWizard::Page_Welcome

##############################################################################
#### Page: SelectComponent
proc ::CompileComponentWizard::Page_SelectComponent {id page args} {
  GUI_LoadImage IDynamicComponentSmall
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 0} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Select Dynamic Component...} -show 0 \
    -message "Please specify the Component that will be compiled/linked:"]
  ## Create a scrolled text and place some radiobuttons inside...
  ::ScrolledWidgets::ScrolledText $area.text -width 70 -height 12 \
    -highlightthickness 0
  set text $area.text.text
  set bg [$text cget -bg]
  ## Get all components...
  set i 1; set index 0
  global PUL_LoadPlugin
  foreach entry [lsort [array names PUL_LoadPlugin m::*]] {
    set component [string range $entry 3 end]
    switch $PUL_LoadPlugin(c::$component) {
      loose   {continue}
      dynamic {}
      tcl     {continue}
      static  {continue}
      java    {continue}
      tight   {continue}
      c++     {}
      default {continue}
    }
    ## Add the radio button...
    radiobutton $text.c$i -text $component -image IDynamicComponentSmall \
      -variable ::CompileComponentWizard::Info($id|Component) -value $component\
      -bg $bg -compound left -padx 4 -anchor w -relief flat \
      -highlightthickness 0
    bind $text.c$i <MouseWheel> \
      "$text yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $text.c$i <4> "$text yview scroll -5 units"
    bind $text.c$i <5> "$text yview scroll  5 units"
    bind $text.c$i <Enter> \
      "::CompileComponentWizard::Page_SelectComponentCheckEnter %W\
       \"$component\""
    bind $text.c$i <Leave> {destroy %W.popup}
    $text window create $i.0 -window $text.c$i
    $text insert end "\n"
    incr i
  }
  $text delete $i.1 end
  $text configure -state disabled
  pack $area.text -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CompileComponentWizard::Page_SelectComponent

proc ::CompileComponentWizard::Page_SelectComponentCheckEnter {w component} {
  global GUI_Options PUL_LoadPlugin
  set l $PUL_LoadPlugin(m::$component)
  set c $PUL_LoadPlugin(c::$component)
  toplevel $w.popup -bg $GUI_Options(TooltipBackground) -highlightthickness 1\
    -highlightbackground $GUI_Options(TooltipForeground) \
    -highlightcolor $GUI_Options(TooltipForeground)
  wm overrideredirect $w.popup 1
  label $w.popup.l -text "Component Name: \"$c\"\nLocation: \"$l\"\nComponent Coupling Type: \"$c\"" \
    -relief flat -wraplength 550 \
    -bg $GUI_Options(TooltipBackground) -padx 2 -pady 0 \
    -fg $GUI_Options(TooltipForeground) -anchor w
    pack $w.popup.l -side left -padx 0 -pady 0 -fill both -expand 1
  wm geometry $w.popup \
    +[expr {[winfo rootx $w] + [winfo width $w] + 10}]+[winfo rooty $w]
};# ::CompileComponentWizard::Page_SelectComponentCheckEnter

##############################################################################
#### Page: CheckComponent
proc ::CompileComponentWizard::Page_CheckComponent {id dir args} {
  variable Components
  variable Info
  ## Has the user selected any component?
  if {[string length $Info($id|Component)]} {
    ## Everything went fine, advance to the next/previous page...
    DisplayPage $id $dir
    return
  }
  ## The user didnot select any component. What is he up to?
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 0 0} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Error: No Component selected!} \
    -message "You haven't selected any Linguistic Processing Component!\n\
      \nPlease use the \"$::GUI_Text(Back)\" button to return to the previous\
      dialog in order to select a component."]
};# ::CompileComponentWizard::Page_CheckComponent

##############################################################################
#### Page: ProcessConfiguration
proc ::CompileComponentWizard::Page_ProcessConfiguration {id dir args} {
  ## Actually, this page does not display a dialog. Instead it creates the
  ## default configuration file if it does not exist or parses an existing
  ## one...

  ## Does a configuration file exists?
  variable Info
  set component $Info($id|Component)
  set CompileConfFilename \
    [file join $::PUL_LoadPlugin(m::$component) CompileComponent.cfg]
  if {![file exists $CompileConfFilename]} {
    GenerateDefaultCompilationConfiguration $component
  }
  if {[catch {source $CompileConfFilename} error]} {
    global GUI_Text
    gut_error \
     "[format $GUI_Text(ErrorCompilationConfig) $CompileConfFilename]\n$error"
  }
  foreach var {PreCompileScript CppFiles Library PostCompileScript} {
    if {![info exists $var]} {
      gut_error "[format $GUI_Text(ErrorCompilationConfig) \
         $CompileConfFilename]\nVariable \"$var\" not defined!"
    }
    set Info($id|$var) [string trim [set $var]]
  }
  DisplayPage $id $dir
};# ::CompileComponentWizard::Page_ProcessConfiguration

##############################################################################
#### Page: SelectFiles
proc ::CompileComponentWizard::Page_SelectFiles {id page args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 1} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Select Component C/C++ Files...} \
    -message "Please specify the C/C++ files containing the source code of the Component:"]
  variable Info

  ## A message and a button to select the files...
  set component $Info($id|Component)
  set dir $::PUL_LoadPlugin(m::$component)
  frame $area.select
    label $area.select.dnd -text \
      "Use drag & drop or the button on the right\
       \nto select the files to be processed:"
    button $area.select.sel -image ITOpen -text ... -compound left \
      -command [list GUI_SelectFiles $area.list.listbox -initialdir $dir \
      -filetypes [list [list C/C++            [list *.c *.cpp]] \
                       [list $::GUI_Text(All) *]] \
      -initialfile $component.cpp -relativedir $dir]
    grid $area.select.dnd $area.select.sel -sticky snew -padx 3 -pady 2
    grid columnconfigure $area.select 0 -weight 1
  pack $area.select -fill x -expand 1
  ## A listbox to drop file names...
  ::ScrolledWidgets::ScrolledListbox $area.list -height 8 -width 90 \
    -listvariable ::CompileComponentWizard::Info($id|CppFiles)
  pack $area.list -fill both -expand 1
  GUI_MakeListboxFileDropTarget $area.list.listbox
};# ::CompileComponentWizard::Page_SelectFiles

##############################################################################
#### Page: PreActions
proc ::CompileComponentWizard::Page_PreActions {id args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 1} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Specify Pre-Compilation Script...} -show 0 \
    -message "Please specify the actions (as a Tcl script) that should be\
    performed before the compilation of C/C++ files:"]
  variable Info
  ::ScrolledWidgets::ScrolledText $area.text -width 70 -height 12 \
    -highlightthickness 0
  bind $area.text.text <Key> "+set ::CompileComponentWizard::Info($id|PreCompileScript) \[string trim \[%W get 1.0 end\]\]"
  $area.text.text insert 1.0 $Info($id|PreCompileScript)
  pack $area.text -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CompileComponentWizard::Page_PreActions

##############################################################################
#### Page: PostActions
proc ::CompileComponentWizard::Page_PostActions {id args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 1} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Specify Post-Compilation Script...} -show 0 \
    -message "Please specify the actions (as a Tcl script) that should be\
    performed after the compilation and linking of C/C++ files:"]
  variable Info
  ::ScrolledWidgets::ScrolledText $area.text -width 70 -height 12 \
    -highlightthickness 0
  bind $area.text.text <Key> "+set ::CompileComponentWizard::Info($id|PostCompileScript) \[string trim \[%W get 1.0 end\]\]"
  $area.text.text insert 1.0 $Info($id|PostCompileScript)
  pack $area.text -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CompileComponentWizard::Page_PostActions

##############################################################################
#### Page: EnableDebug
proc ::CompileComponentWizard::Page_EnableDebug {id args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 1} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Specify Compilation Options...} -show 0 \
    -message "Please specify options related to the compilation/linking process:"]
  variable Info
  GUI_LoadImage ITSearchLarge
  pack [checkbutton $area.debug -text \
    "Enable Debugging Information" -image ITSearchLarge \
    -variable ::CompileComponentWizard::Info($id|EnableDebug) \
    -compound left -padx 5 -anchor w] -fill x -expand 1
  pack [label $area.lbl -text "Please specify the name of the shared library\nobject file, without the \".so\" or \".dll\" suffix:"] -fill x -expand 1
  pack [entry $area.lib -textvariable \
    ::CompileComponentWizard::Info($id|Library)] -fill x -expand 1
  ::WizardDialog::DialogShow $area
};# ::CompileComponentWizard::Page_EnableDebug

##############################################################################
#### Page: CompilerSpecs
proc ::CompileComponentWizard::Page_CompilerSpecs {id args} {
  set area [::WizardDialog::Dialog -image Compile -buttonvisible {1 1 1 1} \
    -id $id -title {Compile Dynamic Component Wizard!} \
    -header {Compiler/Linker Specifications...} -show 0 \
    -message "The following Compiler/Linker Information has been identified:"]
  set i 0
  labelframe $area.f -text $::GUI_Text(CompilationParameters):
  foreach key {OS OSVersion Compiler CompileFlag CompileSharedFlag
    TclIncludeDirs ObjectSuffix OptimiseFlags
    WarningFlags LinkSharedLib LinkLibPaths LinkLibs} {
    label  $area.f.l$i -text $key: -anchor w
    entry  $area.f.e$i -width 40 -highlightthickness 0
    $area.f.e$i insert end $::tBuild::tBuild($key)
    $area.f.e$i configure -state readonly
    grid $area.f.l$i $area.f.e$i -sticky snew -padx 1 -pady 1
    incr i
  }
  grid columnconfigure $area.f 1 -weight 1
  pack $area.f -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CompileComponentWizard::Page_CompilerSpecs

##############################################################################
#### Page: Compile
proc ::CompileComponentWizard::Page_Compile {id args} {
  variable Info
  global GUI_Text GUI_Options
  GUI_LoadImage ISourceCpp
  GUI_LoadImage IBinaryFile
  GUI_LoadImage IDynamicComponent
  GUI_LoadImage IAddArrowGreen

  set component $Info($id|Component)
  set InitDir $::PUL_LoadPlugin(m::$component)
  ## We are ready to create the Compilation window...
  ## 1) Create the window
  set win [ggi_unused_win .compileComponent$id]
  toplevel $win
  wm withdraw $win
  wm title $win "Compiling Component: \"$component\"..."

  ## 2) Create the window toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Back) \
        -image ITPrev -command "::CompileComponentWizard::DisplayPage $id prev
	destroy $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Close) \
        -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  ## 3) Our main area frame
  set MainArea $win.main_area
  frame $MainArea -relief flat
    ## Create a Header...
    frame $MainArea.header -relief flat -bg #8fbc8f
      label $MainArea.header.info -image ISourceCpp -text "$GUI_Text(Compile):"\
        -compound left -fg #008000 -bg #8fbc8f \
        -font {arial 14 bold italic} -anchor e
      label $MainArea.header.comp -font {arial 14 bold italic} \
        -anchor w -bg #8fbc8f -fg #6a5acd -text $component...
      grid $MainArea.header.info -sticky snew -padx 0 -pady 0
      grid $MainArea.header.comp -row 0 -column 1 -sticky snew -padx 2 -pady 0
      grid columnconfigure $MainArea.header 1 -weight 1
    pack $MainArea.header -fill x

    ## The parameters section...
    labelframe $MainArea.parameters -text $GUI_Text(Parameters):
      ## Pre-Compile script:
      label  $MainArea.parameters.pre -text {Pre-Compile Script:} -anchor w
      ::ScrolledWidgets::ScrolledText $MainArea.parameters.pree \
        -height 2 -width 40 -wrap none
      $MainArea.parameters.pree.text insert end \
        [string trim $Info($id|PreCompileScript)]
      ## Defines:
      label  $MainArea.parameters.def -text {Defines:} -anchor w
      entry  $MainArea.parameters.defe -width 40 \
             -textvariable ::CompileComponentWizard::Info($id|Defines)
      ## C++ Files:
      label  $MainArea.parameters.cpp -text {C++ Files:} -anchor w
      entry  $MainArea.parameters.cppe -width 40 \
             -textvariable ::CompileComponentWizard::Info($id|CppFiles)
      button $MainArea.parameters.cppb -compound left -image ITOpen \
        -textvariable GUI_Text(SelectFile) \
        -command [list GUI_SelectFile $MainArea.parameters.cppe \
                  -initialdir $InitDir -relativedir $InitDir -multiple 1 \
                  -filetypes [list [list C/C++ [list *.c *.cpp]] \
                       [list $::GUI_Text(All) *]]]
      GUI_MakeEntryFileDropTarget $MainArea.parameters.cppe
      ## Library name
      label  $MainArea.parameters.lib -text {Output Library Name:} -anchor w
      entry  $MainArea.parameters.libe -width 40 \
             -textvariable ::CompileComponentWizard::Info($id|Library)
      ## Post-Compile script:
      label  $MainArea.parameters.post -text {Post-Compile Script:} -anchor w
      ::ScrolledWidgets::ScrolledText $MainArea.parameters.poste \
        -height 2 -width 40 -wrap none -maphorizontal 1 -mapvertical 1
      $MainArea.parameters.poste.text insert end \
        [string trim $Info($id|PostCompileScript)]
      ## Debug:
      checkbutton $MainArea.parameters.dbg -text {Enable Debug} \
        -variable ::CompileComponentWizard::Info($id|EnableDebug)

      grid $MainArea.parameters.pre -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.pree -row 0 -column 1 -columnspan 2 \
           -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.def $MainArea.parameters.defe \
           -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.cpp $MainArea.parameters.cppe \
           $MainArea.parameters.cppb -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.lib $MainArea.parameters.libe \
           -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.post -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.poste -row 4 -column 1 -columnspan 2 \
           -sticky snew -padx 1 -pady 1
      grid $MainArea.parameters.dbg -sticky snew -padx 1 -pady 1 -columnspan 3
      grid columnconfigure $MainArea.parameters 1 -weight 1
      grid rowconfigure $MainArea.parameters 0 -weight 1
      grid rowconfigure $MainArea.parameters 4 -weight 1
      
    pack $MainArea.parameters -fill both -expand 1

    ## Compilation Options...
    labelframe $MainArea.comppars -text $GUI_Text(CompilationParameters):
    set i 0
    foreach key {OS OSVersion Compiler CompileFlag CompileSharedFlag
      TclIncludeDirs ObjectSuffix OptimiseFlags
      WarningFlags LinkSharedLib LinkLibPaths LinkLibs} {
      label  $MainArea.comppars.l$i -text $key: -anchor w
      entry  $MainArea.comppars.e$i -width 40 -highlightthickness 0
      $MainArea.comppars.e$i insert end $::tBuild::tBuild($key)
      $MainArea.comppars.e$i configure -state readonly
      grid $MainArea.comppars.l$i $MainArea.comppars.e$i \
           -sticky snew -padx 1 -pady 1
      incr i
    }
    grid columnconfigure $MainArea.comppars 1 -weight 1
    pack $MainArea.comppars -fill x
    
  pack $MainArea -fill both -expand 1 -padx 2 -pady 2
  
  ## 4) The button section...
  set ButtonArea $win.buttons
  frame $ButtonArea -relief flat
    ## Add a horizontal separator:-)
    frame $ButtonArea.sep -relief sunken -bd 1 -height 2
    button $ButtonArea.compile -textvariable GUI_Text(Compile) \
      -image ITCompile -compound left -command \
      "::CompileComponentWizard::ExecuteCompile $id $component $win"
    button $ButtonArea.save -textvariable GUI_Text(Save) \
      -image ITSave -compound left -command \
      "::CompileComponentWizard::SaveConfiguration $id $component"
    button $ButtonArea.cancel -textvariable GUI_Text(Close) \
      -image ITCancel -compound left -command "destroy $win"
    bind $ButtonArea.cancel <Destroy> "unset -nocomplain $win"
    grid $ButtonArea.sep -sticky ew -padx 5 -columnspan 3
    grid $ButtonArea.compile -row 1 -column 0 -sticky e -padx 10 -pady 10
    grid $ButtonArea.save    -row 1 -column 1 -sticky w -padx 10 -pady 10
    grid $ButtonArea.cancel  -row 1 -column 2 -sticky w -padx 10 -pady 10
    grid columnconfigure $ButtonArea 0 -weight 1
    grid columnconfigure $ButtonArea 2 -weight 1
  pack $ButtonArea -fill x
  wm deiconify $win
};# ::CompileComponentWizard::Page_Compile

## GenerateDefaultCompilationConfiguration
## This function will generate the default Compilation configuration file for
## a component, if it does not already exist.
proc ::CompileComponentWizard::GenerateDefaultCompilationConfiguration {
  component {force 0} {PreCompileScript {}} {CppFiles {}} {Library {}}
  {PostCompileScript {}} {Debug 0} {Defines {}}} {
  global PUL_LoadPlugin
  set CompileConfFilename \
    [file join $PUL_LoadPlugin(m::$component) CompileComponent.cfg]
  if {![string length $Library]} {
    set Library lib${component}
  }
  set Library [file rootname $Library]
  if {![llength $CppFiles]} {
    set CppFiles {}
    foreach file [glob -nocomplain \
         -directory $PUL_LoadPlugin(m::$component) *.cpp] {
      regexp $PUL_LoadPlugin(m::$component)(.*) $file tmp nFile
      lappend CppFiles [string trimleft $nFile /\\]
    }
  }
  if {!$force} {
    if {[file exists $CompileConfFilename]} {return}
  }
  set file [open $CompileConfFilename w]
  puts $file "##########################################################"
  puts $file "## Component Compilation Information."
  puts $file "## Auto-generated by the $::ApplicationName platform."
  puts $file "##########################################################"
  puts $file ""
  puts $file "set PreCompileScript  \{\n$PreCompileScript\n\}"
  puts $file "set Defines           \"$Defines\""
  puts $file "set CppFiles          \"$CppFiles\""
  puts $file "set Library           \"$Library\""
  puts $file "set PostCompileScript \{\n$PostCompileScript\n\}"
  puts $file "set Debug             \"$Debug\""
  puts $file ""
  puts $file "##"
  puts $file "## End of file"
  puts $file "##"
  close $file
};# ::CompileComponentWizard::GenerateDefaultCompilationConfiguration

## ::CompileComponentWizard::SaveConfiguration
## The a customised configuration for a component...
proc ::CompileComponentWizard::SaveConfiguration {id component} {
  variable Info
  GenerateDefaultCompilationConfiguration $component 1 \
    [string trim $Info($id|PreCompileScript)] \
    [string trim $Info($id|CppFiles)] \
    [string trim $Info($id|Library)] \
    [string trim $Info($id|PostCompileScript)] \
    [string trim $Info($id|EnableDebug)] \
    [string trim $Info($id|Defines)]
};# ::CompileComponentWizard::SaveConfiguration

## ::CompileComponentWizard::ExecuteCompile
## This function does the actual compilation...
proc ::CompileComponentWizard::ExecuteCompile {id component parent} {
  global GUI_Text GUI_Options
  variable Info
  set MainArea $parent.main_area
  
  ## Create a feedback dialog...
  set win $parent.feedback
  catch {destroy $win}
  toplevel $win
  wm withdraw $win
  wm group $win $parent
  wm transient $win $parent
  wm title $win "Compiling Component \"$component\"..."

  ## Create a vertical feedback with images...
  frame $win.vert -bg #8fbc8f -width 50
    label $win.vert.i1 -image ISourceCpp     -bg #8fbc8f \
      -width 48 -height 48 -bd 2 -relief flat
    label $win.vert.i2 -image IAddArrowGreen -bg #8fbc8f \
      -width 48 -height 48 -bd 2 -relief flat
    label $win.vert.i3 -image IBinaryFile    -bg #8fbc8f \
      -width 48 -height 48 -bd 2 -relief flat
    pack $win.vert.i3 $win.vert.i2 $win.vert.i1 -padx 10 -pady 15 -side bottom
  pack $win.vert -fill y -side left

  ## Our main area...
  set MainArea $win.main_area
  frame $MainArea
    label $MainArea.title -text $component... -font {arial 14 bold italic} \
      -anchor w -bg #8fbc8f -fg #6a5acd
    pack $MainArea.title -fill x
    ::ScrolledWidgets::ScrolledText $MainArea.feedback -height 12 -width 80
    pack $MainArea.feedback -fill both -expand 1
    Progress $MainArea.progress
    pack $MainArea.progress -fill x -padx 45 -pady 15

    ## Finally a stop/close button
    button $MainArea.stop -textvariable GUI_Text(Close) -command "destroy $win"
    pack $MainArea.stop -pady 10
    
  pack $MainArea -fill both -expand 1

  gut_ModalDlg $win
  wm deiconify $win
  grab $win

  set Text $MainArea.feedback.text
  $Text tag configure exec -foreground #8fbc8f
  $Text tag configure msg  -foreground #6a5acd -font {arial 14 bold italic}
  $Text tag configure error -foreground red
  ## Register a callback in the tBuild package...
  tBuild::Reset
  ::tBuild::Configure -execallback \
    "::CompileComponentWizard::ExecCallback $win $Text"
  set CWD [pwd]
  cd $::PUL_LoadPlugin(m::$component)
  set CDM [file join $::HomeDir CDM]
  ## 1) Execute any possible pre-script...
  $Text insert end "Executing Pre-Compilation Script:\n" msg
  $Text insert end "[subst $Info($id|PreCompileScript)]\n" exec
  catch {eval $Info($id|PreCompileScript)} result
  $Text insert end "$result\n\n" exec

  ## 2) Compile our files...
  set Defines $Info($id|Defines)
  lappend Defines CDM_USE_STUBS
  set defines [join $Defines " -define "]
  foreach file $Info($id|CppFiles) {
   lappend Objs [eval tBuild::CompileSharedObject -debug $Info($id|EnableDebug)\
      -include [list $CDM] -define $defines [list $file]]
  }
  SetProgress $MainArea.progress 50
  ## 3) Link our shared object...
  $win.vert.i1 configure -image IBinaryFile
  $win.vert.i3 configure -image IDynamicComponent
  set osDir $::ApplicationInfo(LoadBinaryFolderName)
  catch {file mkdir $osDir}
  eval tBuild::LinkSharedLibrary -debug $Info($id|EnableDebug) \
     -libpath [list [file join $CDM $osDir]] \
     -libpath [list $CDM] \
     -lib libCDM.lib \
     -output [file join $osDir new_$Info($id|Library)[info sharedlibextension]]\
     $Objs
  
  ## 4) Execute any possible pre-script...
  $Text configure -state normal
  $Text insert end "Executing Post-Compilation Script:\n" msg
  $Text insert end "[subst $Info($id|PostCompileScript)]\n" exec
  catch {eval $Info($id|PostCompileScript)} result
  $Text insert end "$result\n\n" exec

  SetProgress $MainArea.progress 100
  $Text insert end \
    "Please rename file \"[file join \
    $osDir new_$Info($id|Library)[info sharedlibextension]]\" to \"[file join \
    $osDir $Info($id|Library)[info sharedlibextension]]\"" msg
  $Text configure -state disabled
  $Text see end
  cd $CWD
  grab release $win
  #$win.vert.i1 configure -image IDynamicComponent
  #$win.vert.i2 configure -image ITBlank
  #$win.vert.i3 configure -image ITBlank
  AnimateFeedback $win 11
};# ::CompileComponentWizard::ExecuteCompile

proc ::CompileComponentWizard::ExecCallback {win text args} {
  ## Insert the command to be executed in the text widget...
  set cmd [join $args { }]
  $text configure -state normal
  $text insert end "Executing Command:\n" msg
  $text insert end "$cmd\n" exec
  $text configure -state disabled
  update

  ## Execute our command...
  if {[catch {eval exec $args} error]} {
    $text configure -state normal
    $text insert end "$error\n\n" error
  } else {
    $text configure -state normal
    $text insert end "==> Completed without errors <==\n\n" msg
  }
  $text configure -state disabled
  $text see end
  update
};# ::CompileComponentWizard::ExecCallback

## ::CompileComponentWizard::AnimateFeedback
## Simply for visual purposes. It animates the feedback window...
proc ::CompileComponentWizard::AnimateFeedback {win {state 0} {T 400}} {
  if {![winfo exists $win]} {return}
  set l1 $win.vert.i1
  set l2 $win.vert.i2
  set l3 $win.vert.i3
  set i1 ISourceCpp
  set i2 IAddArrowGreen
  set i3 IBinaryFile
  set i4 IDynamicComponent
  set i5 ITBlank
  switch $state {
    0  {$l1 conf -image $i5;$l2 conf -image $i5;$l3 conf -image $i5}
    1  {$l1 conf -image $i1;$l2 conf -image $i5;$l3 conf -image $i5}
    2  {$l1 conf -image $i1;$l2 conf -image $i2;$l3 conf -image $i5}
    3  {$l1 conf -image $i1;$l2 conf -image $i2;$l3 conf -image $i3}
    4  {$l1 conf -image $i5;$l2 conf -image $i2;$l3 conf -image $i3}
    5  {$l1 conf -image $i5;$l2 conf -image $i5;$l3 conf -image $i3;set T 100}
    6  {$l1 conf -image $i5;$l2 conf -image $i3;$l3 conf -image $i5}
    7  {$l1 conf -image $i3;$l2 conf -image $i5;$l3 conf -image $i5;set T 400}
    8  {$l1 conf -image $i3;$l2 conf -image $i2;$l3 conf -image $i5}
    9  {$l1 conf -image $i3;$l2 conf -image $i2;$l3 conf -image $i4}
    10 {$l1 conf -image $i5;$l2 conf -image $i2;$l3 conf -image $i4}
    11 {$l1 conf -image $i5;$l2 conf -image $i5;$l3 conf -image $i4}
    12 {$l3 conf -relief groove ;#-bg #6a5acd}
    13 {$l3 conf -relief flat  ;#-bg #8fbc8f}
    14 {$l3 conf -relief groove ;#-bg #008000}
    15 {$l3 conf -relief flat  ;#-bg #8fbc8f}
  }
  incr state
  if {$state == 16} {set state 0}
  after $T "::CompileComponentWizard::AnimateFeedback $win $state $T"
};# ::CompileComponentWizard::AnimateFeedback
