##############################################################################
##  Configure.tcl:                                                          ##
## ---------------------                                                    ##
##  This file implements facilities for serialising Systems into files.     ##
##  Initial version date: 15/09/1998                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

## opt_ReadConfig
#
proc opt_ReadConfig {filename} {
  global GUI_Systems GUI_Graphs

  set Systems [opt_GetLines $filename __start_graphs __end_graphs]
  foreach line $Systems {
    set info [split $line \;]
    foreach {info0 info1 info2 info3} $info {break}
    set GUI_Systems($info1) $info
    if {![string match *_task $info2]} {
      set GUI_Graphs($info1) [list $info2 $info1]
    } else {
      set subsystems [split $info3 =]
      if {[llength $subsystems]} {
        foreach sub $subsystems {
          foreach {sub_mod sub_nam} [split $sub |] {break}
          set GUI_Graphs($sub_nam) [list $sub_mod $sub_nam]
        }
      }
    }
  }
};# opt_ReadConfig

## opt_SaveConfig
#   Save Configure File (default=~/.gate_props)...
proc opt_SaveConfig {{filename {}}} {
  global GUI_Systems GUI_Graphs GUI_SystemsModified

  if {![string length $filename]} {
    global UserHomeDir
    set filename [file join $UserHomeDir .gate.props]
  }

  append Text "__START_CONFIG\n__START_TASKS\n__top_geometry="
  append Text "[wm geometry .mw]\n__top_pane_percent="
  append Text "35\n";#CHANGE
  append Text "__start_graphs\n"
  foreach {i arci} {1 1} {}
  ## Save Systems...
  foreach system [lsort [array names GUI_Systems]] {
    append Text "$i\;[join [lrange $GUI_Systems($system) 1 end] "\;"]\n"
    incr i
  }
  append Text "__end_graphs\n__END_TASKS\n"

  ## Save Graphs...
  foreach system [lsort [array names GUI_Graphs]] {
    set info $GUI_Graphs($system)
    if {[llength $info] == 2} {continue}
    if {[catch {wm geometry .sys_$system} geometry]} {
      set geometry [lindex $info 1]
    }
    append Text \
    "__START_GRAPH=$system\n__geometry=$geometry\n__timeLog=0\n__grid="
    append Text "[lindex $info 3]\n__start_nodes\n"
    ## Displace modules so as coordinates allways start at 0.0
    set min_x {}; set min_y {}
    foreach module [lindex $info 4] {
      set x [lindex $module 1]
      set y [lindex $module 2]
      if {[string length $min_x]} {
        if {$x < $min_x} {set min_x $x}
        if {$y < $min_y} {set min_y $y}
      } else {
        set min_x $x; set min_y $y
      }
    }
    foreach module [lindex $info 4] {
      set x [expr {[lindex $module 1]-$min_x}]
      set y [expr {[lindex $module 2]-$min_y}]
      append Text "$system\,[lindex $module 0]\;$x;$y\;0\;"
      set Module_Label [lindex $module 3]
      regsub -all "\n" $Module_Label {\\n} Module_Label
      append Text "$Module_Label\;\;\;unrunnable\;\n"
    }
    append Text "__end_nodes\n__start_arcs\n"
    foreach arc [lindex $info 5] {
      append Text "$system\,$arci\;$system\,[lindex $arc 0]\;$system\,"
      append Text "[lindex $arc 1]\;[lindex $arc 2]\;\;unrunnable\;\n"
      incr arci
    }
    append Text "__end_arcs\n__END_GRAPH=$system\n"
  }
  append Text "__END_CONFIG"
  if [catch {open $filename w} File] {
    gut_error "Cannot Open File $filename for Writting.\n($File)"
    return {}
  }
  fconfigure $File -encoding utf-8
  puts $File $Text
  close $File
  set GUI_SystemsModified 0
};# opt_SaveConfig

## opt_GetLines
#
proc opt_GetLines {filename start end} {
  if {[catch {open $filename r} File]} {
    if {![file exists $filename]} {
      global HomeDir
      file copy -force [file join $HomeDir share Templates GateProps] \
      [file join [file dirname $filename] .gate.props]
    }
    if {[catch {open $filename r} File]} {
      gut_error "Cannot Open File $filename for Reading"
    }
  }
  fconfigure $File -encoding utf-8
  foreach {st text} {0 {}} {}
  while {[gets $File line] != -1} {
    switch -glob -- $line [list \
      $start {incr st} \
      $end   {break} \
      default {
        if {$st} {lappend text $line}
      }\
    ]
  }
  close $File
  return $text
};# opt_GetLines


## opt_LoadSystemTree
#
proc opt_LoadSystemTree {tree} {
  global GUI_Options GUI_Systems GUI_Graphs GUI_Text Tree

  set systems [lsort [array names GUI_Systems]]
  Tree:newitem $tree /
  foreach system $systems {
    foreach {info0 info1 info2 info3} $GUI_Systems($system) {break}
    set name /$info1
    if {[string match *_task $info2]} {
      Tree:newitem $tree $name -image IFolderSmall -overimage IFolderSmallOver\
          -largeimage IFolder -largeoverimage IFolderOver \
          -openimage IFolderSmallOpen -overopenimage IFolderSmallOpenOver \
          -tags [list folder $name]
      set subsystems [split $info3 =]
      if {[llength $subsystems]} {
        set ContainedSystems {}
        foreach sub $subsystems {
          foreach {sub_mod sub_nam} [split $sub |] {break}
          set subname $name/$sub_nam
          set GUI_Graphs([file tail $subname]) [split $sub |]
          set Modules {}
          foreach spec [lindex $GUI_Graphs($sub_mod) 4] {
            lappend Modules [lindex $spec 3]
          }
          Tree:newitem $tree $subname -image ISystemSmall -overimage \
              ISystemSmallOver -largeimage ISystem -largeoverimage ISystemOver\
              -tags [list system $subname] -tooltip [join $Modules {, }] \
              -tooltipimg IModule
          lappend ContainedSystems [file tail $subname]
        }
        set Tree($tree:$name:Tooltip) [join $ContainedSystems {, }]
        set Tree($tree:$name:TooltipImg) ISystem
      } else {
        Tree:newitem $tree "$name/{}" -tags {empty {}} -tooltip $GUI_Text(none)\
          -tooltipimg ISystem
      }
    } else {
      set Modules {}
      foreach spec [lindex $GUI_Graphs($info2) 4] {
        lappend Modules [lindex $spec 3]
      }
      Tree:newitem $tree $name -image ISystemSmall -overimage ISystemSmallOver\
             -largeimage ISystem -largeoverimage ISystemOver\
             -tags [list system $name] -tooltip [join $Modules {, }] \
             -tooltipimg IModule
    }
  }
};# opt_LoadSystemTree


## opt_LoadSystems
#
proc opt_LoadSystems {filename} {
  global GUI_Graphs
  set system_arcs {}
  foreach system [array names GUI_Graphs] {
    set name [lindex $GUI_Graphs($system) 0]
    set lines [opt_GetLines $filename "__START_GRAPH=$name" "__END_GRAPH=$name"]
    if {![llength $lines]} {continue}
    foreach {geometry time_log grid} $lines {break}
    set geometry [string range $geometry 11 end]
    set time_log [string range $time_log 10 end]
    set grid     [string range $grid 7 end]
    set lines    [lrange $lines 4 end]
    foreach {read_nodes system_modules system_arcs} {1 {} {}} {break}
    foreach line $lines {
      if {[string equal $line __end_nodes]}  {continue}
      if {[string equal $line __end_arcs]}   {break}
      if {[string equal $line __start_arcs]} {
        set read_nodes 0
        continue
      }
      if {$read_nodes} {         ;# Reading Nodes...
        foreach {mname X Y _ Label} [split $line \;] {break}
        set mname [lindex [split $mname ,] 1]
        if {[catch {PUL_plugin_module_config $mname title} label]} {
           set label $Label
        }
        lappend system_modules [list $mname $X $Y $label]
      } else {                         ;# Reading Arcs
        foreach {_ id _ source _ target cpoints} [split $line \;,] {break}
        lappend system_arcs [list $source $target $cpoints]
      }
    }
    set GUI_Graphs($name) [list $system $geometry $time_log $grid \
                                $system_modules $system_arcs]
  }
};# opt_LoadSystems

## opt_ImportSystemsFromFile
#  This function imports the systems contained in a file...
proc opt_ImportSystemsFromFile {file} {
};# opt_ImportSystemsFromFile
