##############################################################################
##  CopyCollectionAnnotations.tcl:                                          ##
## --------------------------------                                         ##
##  This file implements a utility for copying Annotations between          ##
##  Collections.                                                            ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

#############################################################################
## A Small Utility for copying Annotations between Collections...          ##

proc CDMTools_CopyAnnotationsBetweenCollections {{win .cdm_toolsCopyColsAnns}} {
  global GUI_Text GUI_Options
  set win [ggi_unused_win $win]
  toplevel $win
  wm withdraw $win
  wm title $win {Copying Annotations between Collections...}

  ## Comboboxes of Collection Names...
  set CFrame $win.comboframe
  set LFrame $win.doclistboxes
  GUI_LoadImage Step1
  GUI_LoadImage Step2
  GUI_LoadImage Step3
  GUI_LoadImage Step4
  GUI_LoadImage Step5
  GUI_LoadImage Step6
  GUI_LoadImage NextNice
  frame $CFrame
    set ColNames [GUI_GetCollectionNames]
    label $CFrame.keylab -image Step1 -compound left -justify l -textvar \
      GUI_Text(SelectSourceCollectionContainingAnnsToCopy) -anchor w \
      -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.keylab \
      $CFrame.keycol.e
    ComboBox $CFrame.keycol -editable 0 -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -values $ColNames \
      -modifycmd "CDMTools_TransferCollectionUpdateListboxes $CFrame.keycol \
      $CFrame.keydocs.listbox"
    label $CFrame.reslab -image Step2 -compound left -justify l -textvar \
      GUI_Text(SelectTargerCollectionToCopyAnns) -anchor w \
      -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.reslab \
      $CFrame.rescol.e
    ComboBox $CFrame.rescol -editable 0 -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -values $ColNames \
      -modifycmd "CDMTools_TransferCollectionUpdateListboxes $CFrame.rescol \
      $CFrame.resdocs.listbox"
    label $CFrame.dkeylab -image Step3 -compound left -justify l -textvar \
      GUI_Text(SelectSourceDocumentsContainingAnnsToCopy) -anchor w \
      -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.dkeylab \
      $CFrame.keydocs.listbox
    ::ScrolledWidgets::ScrolledListbox $CFrame.keydocs -height 8 -width 45 \
      -selectmode extended -bd $GUI_Options(BorderWidth) -exportsel 0
    bind $CFrame.keydocs.listbox <Destroy> "catch {
      array unset CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes \
      %W:*}"
    bind $CFrame.keydocs.listbox <<ListboxSelect>> \
    "catch {after cancel \
       \$CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes(%W:Id)}
     set CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes(%W:Id) \
     \[after 2000 \"CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes \
     $CFrame.keycol %W $CFrame.anns.listbox $CFrame.dobutton\"\]"
    label $CFrame.dreslab -image Step4 -compound left -justify l -textvar \
      GUI_Text(SelectTargerDocumentsToCopyAnns) -anchor w \
      -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.dreslab \
      $CFrame.resdocs.listbox
    ::ScrolledWidgets::ScrolledListbox $CFrame.resdocs -height 8 -width 45 \
      -selectmode extended -bd $GUI_Options(BorderWidth) -exportsel 0
    label $CFrame.dannlab -image Step5 -compound left -justify l -textvar \
      GUI_Text(SelectAnnsToCopy) -anchor w -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.dannlab \
      $CFrame.anns.listbox
    ::ScrolledWidgets::ScrolledListbox $CFrame.anns -height 5 -width 45 \
      -selectmode extended -bd $GUI_Options(BorderWidth) -exportsel 0
    label $CFrame.dolab -image Step6 -compound left -justify l -textvar \
      GUI_Text(PressButtonToCopy) -anchor w -padx 10 -pady 5
    CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel $CFrame.dolab \
      $CFrame.dobutton orange
    button $CFrame.dobutton -image NextNice -textvariable GUI_Text(Copy) \
      -compound r -bg orange -fg red -pady 5 -padx 5 -state disabled -command \
      [list CDMTools_CopyCollectionAnnotations $CFrame.keycol \
      $CFrame.keydocs.listbox \
      $CFrame.rescol $CFrame.resdocs.listbox $CFrame.anns.listbox]
    grid $CFrame.keylab  $CFrame.reslab  -sticky snew -padx 2 -pady 2
    grid $CFrame.keycol  $CFrame.rescol  -sticky snew -padx 2 -pady 2
    grid $CFrame.keydocs $CFrame.resdocs -sticky snew -padx 2 -pady 2
    grid $CFrame.dkeylab $CFrame.dreslab -sticky snew -padx 2 -pady 2
    grid $CFrame.anns                    -sticky snew -padx 2 -pady 2
    grid $CFrame.dannlab                 -sticky snew -padx 2 -pady 2
    grid $CFrame.dobutton -column 1 -row 4 -sticky ew -padx 60 -pady 10
    grid $CFrame.dolab    -column 1 -row 5 -sticky snew -padx 2 -pady 2
    grid columnconfigure $CFrame 0 -weight 1
    grid columnconfigure $CFrame 1 -weight 1
  pack $CFrame -fill x -expand 1 -padx 4

  set BFrame $win.button
  frame $BFrame
    button $BFrame.close -textvar GUI_Text(Close) -command "destroy $win"
    pack $BFrame.close -side top -pady 10
  pack $BFrame -fill x -expand 1 -padx 4
  update
  wm deiconify $win
};# CDMTools_TransferAnnotationsBetweenCollections

## CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel
#  Make a label to point to a widget, by changing its appearence...
proc CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel {label widget
              {colour {$GUI_Options(InputBackground)}}} {
  bind $label <Enter> \
    "$widget configure -background \$GUI_Options(DragEnterBackgound)"
  bind $label <Leave> \
    "$widget configure -background $colour"
};# CDMTools_CopyAnnotationsBetweenCollections_MakeHintLabel

## CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes
#  Fill the Annotation type listbox from Annotations found in the selected
#  Documents...
proc CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes {combo DocList
                                                               AnnList button} {
  $AnnList delete 0 end
  $button configure -state disabled
  ## Get the name of the selected Collection...
  set index [$combo getvalue]
  if {$index < 0} {return}
  ## Get Selected Documents...
  set selected [$DocList curselection]
  if {![llength $selected]} {return}
  
  set cols    [$combo cget -values]
  set ColName [lindex $cols $index]
  set Docs {}
  foreach index $selected {
    lappend Docs [$DocList get $index]
  }
  ## We never have to look more than 100 Documents...
  set Docs [lrange $Docs 0 100]

  ## Open the Collection, and iterate over the Documents to collect the
  ## Annotation types...
  set Types {}
  set Col [tip_OpenCollection $ColName]
  set keycolname [tip_GetName $Col]
  if {![string equal $ColName $keycolname]} {
    GUI_FixCollectionName $Col $keycolname $ColName
  }
  set wwin [ggi_wait_message {} 0.0]
  set Progress 0.0
  set Step [expr {100.0/[llength $selected]}]
  foreach Doc $Docs {
    set doc [tip_GetDocument $Col $Doc]
    foreach type [$doc GetAnnotationTypes] {
      CDM_LappendUnique Types $type
    }
    tip_Close $doc
    set Progress [expr {$Progress+$Step}]
    ggi_wait_update {} $Progress $wwin
  }
  tip_Close $Col
  ggi_destroy_wait $wwin
  eval $AnnList insert end [lsort -dictionary $Types]
  $button configure -state normal
};# CDMTools_CopyAnnotationsBetweenCollections_CollectAnnTypes

## CDMTools_CopyCollectionAnnotations
#  This function does the actual transfer of linguistic information...
proc CDMTools_CopyCollectionAnnotations {keycom keylist rescom reslist
                                                annlist} {
  global GUI_Text
  ## Get the name of the key Collection...
  set index [$keycom getvalue]
  if {$index < 0} {return}
  set keycolname [lindex [$keycom cget -values] $index]
  ## Get the name of the target Collection...
  set index  [$rescom getvalue]
  if {$index < 0} {return}
  set rescolname [lindex [$rescom cget -values] $index]
  
  ## Get the selected documents from key collection...
  set keysel [$keylist curselection]
  if {![llength $keysel]} {return}
  set ressel [$reslist curselection]
  if {![llength $ressel]} {return}
  set annsel [$annlist curselection]
  if {![llength $annsel]} {return}
  
  set keycol  [tip_OpenCollection $keycolname]
  set ColName [tip_GetName $keycol]
  if {![string equal $ColName $keycolname]} {
    GUI_FixCollectionName $keycol $ColName $keycolname
  }
  set rescol  [tip_OpenCollection $rescolname]
  set ColName [tip_GetName $rescol]
  if {![string equal $ColName $rescolname]} {
    GUI_FixCollectionName $rescol $ColName $rescolname
  }

  ## Get the selected Annotation types...
  set Types {}
  foreach index $annsel {
    lappend Types [$annlist get $index]
  }

  set keydocs {}
  set resdocs {}
  foreach key_index $keysel res_index $ressel {
    if {![string length $key_index] || ![string length $res_index]} {break}
    lappend keydocs [$keylist get $key_index]
    lappend resdocs [$reslist get $res_index]
  }

  ## Itelate over selected documents...
  set Step [expr {100.0/[llength $keydocs]}]
  set Progress 0.0
  set wwin [ggi_wait_message {} 0.0 1]
  foreach keydoc $keydocs resdoc $resdocs {
    ggi_wait_message \
      [format $GUI_Text(CopyingAnnsFromDoc2Doc) $keydoc $resdoc] $Progress 1
    set kd [tip_GetDocument $keycol $keydoc]
    set rd [tip_GetDocument $rescol $resdoc]
    ## Copy the Annotations of the selected types...
    foreach type $Types {
      foreach ann [tip_SelectAnnotations $kd $type] {
        set ann [lreplace $ann 0 0 {}]
	tip_AddAnnotation $rd $ann
      }
    }
    ## Save the target Document...
    tip_Sync $rd
    tip_Close $kd
    tip_Close $rd
    set Progress [expr {$Progress+$Step}]
    CDM_ClearCompressedMemoryCache
  }
  tip_Close $keycol
  tip_Sync  $rescol
  tip_Close $rescol
  ggi_wait_message $GUI_Text(UpdateCollectionBT) 100.0 1
  catch {GUI_DrawCollectionDlg}
  ggi_destroy_wait $wwin
}
