##############################################################################
##  CreateStandAloneApplication.tcl:                                        ##
## ----------------------------------                                       ##
##  This file implements a wizard for exporting a system or a collection of ##
##  out of Ellogon, in a stand-alone application. This application can have ##
##  a completelly custom interface or run in a batch mode, without requiring##
##  a graphical interface at all...                                         ##
##                                                                          ##
##  Copyright 1998-2008 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

##############################################################################
##############################################################################
## Actually this tool is implemented as a wizard: The user has to be asked a
## series of questions. At the end, the stand-alone application is created,
## according to the user gathered information...
## In order to use this wizard, the caller has to only execute 
## "::CreateStandAloneApplication::Init"...
##############################################################################
##############################################################################

package ifneeded WizardDialog 1.0 "source $HomeDir/GUI/Dialogs/Wizard.tcl"

namespace eval ::CreateStandAloneApplication {
  variable Page; # Inernal information about the current wizard page
  ## The following variable holds the sequence of the wizard pages. The pages
  ## will be navigated according to this order. For each such page, a
  ## procedure named ::CreateStandAloneApplication::Page_<page> should be
  ## available...
  variable Pages {Welcome ApplicationType ApplicationCodeType AppParts 
                  CollectionInfo FileInfo GUI_Options
                  SelectComponents CheckComponents ComponentOrder
                  ComponentParameters OutputDir Review Finish}
  variable Info
  variable Components
  variable AppParts
  variable ComponentArgs
}

## ::CreateStandAloneApplication::Init
#  Initialise internal data structures and start the wizard...
proc ::CreateStandAloneApplication::Init {} {
  package require WizardDialog
  variable Page
  variable Info
  variable Components
  variable AppParts
  variable ComponentArgs
  if {[info exists Info(WizardActive)]} {return}
  set Info(WizardActive) 1
  unset -nocomplain Components AppParts ComponentArgs
  set ::WizardDialog::Wizard(CSAA|CancelCommand) \
    {unset -nocomplain ::CreateStandAloneApplication::Info(WizardActive)
     array unset ::WizardDialog::Wizard CSAA|*}
  set ::WizardDialog::Wizard(CSAA|BackCommand) \
    {::CreateStandAloneApplication::DisplayPage prev}
  set ::WizardDialog::Wizard(CSAA|NextCommand) \
    {::CreateStandAloneApplication::DisplayPage next}
  set ::WizardDialog::Wizard(CSAA|FinishCommand) \
    {::CreateStandAloneApplication::DisplayPage last}
  GUI_LoadImage Install
  ## Set default values...
  set Info(GUI)                     1
  set Info(ScriptApp)               0
  set Info(ScriptFileName)          "$::UserHomeDir/EllogonApp/AppScript.tcl"
  set Info(ApplicationName)         "${::ApplicationName}_based_Application" 
  set Info(AppDirName)              "$::UserHomeDir/EllogonApp"
  set Info(SelectedComponents)      {}
  set Info(FilesToProcess)          {$argv}
  set Info(FileEncoding)            {[encoding system]}
  set Info(FileEncodingSelectable)  1
  set Info(CollectionName)          {$::CDM_TempDir/TempCollection}
  set Info(SyncCollection)          1
  set Info(DeleteCollection)        1
  set Info(FileSelectionMode)       directory
  set Info(FileExtensions)          {*.txt *.*htm*}
  set Info(OutputSelectionMode)     directory
  set Info(GUI_Title)               {Application Window Title...}
  set Info(GUI_TitleText)           {Application version 1.0}
  set Info(GUI_TitleTextBackground) white
  set Info(GUI_TitleTextForeground) navy
  set Info(GUI_TitleTextFont)       {arial 14 bold}
  set Info(GUI_InputMessage)        Input:
  set Info(GUI_OutputMessage)       Output:
  set Info(GUI_EncodingMessage)     Encoding:
  set Info(GUI_ExecMessage)         Execute
  set Info(GUI_ExitMessage)         Exit
  set Info(OS_Linux)                1
  set Info(OS_Windows)              1
  set Info(OS_SunOS)                1
  set Info(LoadedProjectSettings)   0
  ## The following images will be defined by the following function, defined
  ## in a different file...
  #set Info(GUI_SplashImageData)
  #set Info(GUI_TitleImageData)
  #set Info(GUI_WindowTitleImageData)
  InitialiseImages
  ## The following function will initialise the list for available sub-parts
  ## that can be included in an application...
  InitialiseParts
  ## Display the first page...
  set  Page(CSAA|Current)   0
  DisplayPage start
};# ::CreateStandAloneApplication::Init

## ::CreateStandAloneApplication::DisplayPage
#  This function will display a page according to the current displayed page.
#  It accepts an argumend which can have one of the following values:
#  "start", "next", "prev", "last"
proc ::CreateStandAloneApplication::DisplayPage {page} {
  variable Page
  variable Pages

  #if {![string equal $page last] && [string equal $Page(Current) end]} {return}
  switch $page {
    start {set  Page(CSAA|Current)   0}
    next  {incr Page(CSAA|Current)    }
    prev  {incr Page(CSAA|Current)  -1}
    last  {set  Page(CSAA|Current) end}
    default {
      ## This is one of the pages...
      set Page(CSAA|Current) [lsearch $Pages $page]
    }
  }
  ## Make some routine checks...
  if {![string equal $Page(CSAA|Current) end]} {
    if {$Page(CSAA|Current) < 0} {set Page(CSAA|Current) 0}
    if {$Page(CSAA|Current) >= [llength $Pages]} {return}
  }
  ## Run the function for the specific page...
  ::CreateStandAloneApplication::Page_[lindex $Pages $Page(CSAA|Current)] $page
};# ::CreateStandAloneApplication::DisplayPage

##############################################################################
#### Page: Welcome
proc ::CreateStandAloneApplication::Page_Welcome {args} {
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 0 1 0} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Welcome to the "Create Stand-Alone Application" Wizard!} \
    -message "This wizard will help you to create a new\
    application, based on the \"$::ApplicationName\" technology.\n\
    \n\nThe resulting application will be able to apply specific linguistic\
    processing components on files without the need to use $::ApplicationName!"]

  ## Add a button for loading an already saved Script/Application...
  pack [button $area.load -image ITOpen -compound left -text ... -command \
               "::CreateStandAloneApplication::LoadSettings $area"] -side right
  pack [label  $area.loadLabel -text {Load Settings:}] -side right -padx 5
};# ::CreateStandAloneApplication::Page_Welcome

##############################################################################
#### Page: ApplicationType
proc ::CreateStandAloneApplication::Page_ApplicationType {args} {
  GUI_LoadImage IConsole
  GUI_LoadImage IGUI
  variable Info
  if {$Info(LoadedProjectSettings)} {
    set visible {1 1 1 1}
  } else {
    set visible {1 1 1 0}
  }
  set area [::WizardDialog::Dialog -image Install -buttonvisible $visible \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Specifying Application Type...} \
    -message "Please specify the desired application type:"]
  pack [radiobutton $area.gui -text {Graphical Application} \
    -image IGUI -variable ::CreateStandAloneApplication::Info(GUI) \
    -value 1 -compound left -padx 5 -anchor w] -fill x -expand 1
  pack [radiobutton $area.console -text {Console Application} \
    -image IConsole -variable ::CreateStandAloneApplication::Info(GUI) \
    -value 0 -compound left -padx 5 -anchor w] -fill x -expand 1
};# ::CreateStandAloneApplication::Page_ApplicationType

##############################################################################
#### Page: ApplicationCodeType
proc ::CreateStandAloneApplication::Page_ApplicationCodeType {args} {
  GUI_LoadImage IScript
  GUI_LoadImage IApplication
  variable Info
  if {$Info(LoadedProjectSettings)} {
    set visible {1 1 1 1}
  } else {
    set visible {1 1 1 0}
  }
  set area [::WizardDialog::Dialog -image Install -buttonvisible $visible \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Specifying Application Type...} \
    -message "Please specify the desired application type:"]
  pack [radiobutton $area.script -text \
    "Script Application: The script will be\
     given as\nargument to Ellogon for batch processing..." \
    -image IScript -variable ::CreateStandAloneApplication::Info(ScriptApp) \
    -value 1 -compound left -padx 5 -anchor w] -fill x -expand 1
  pack [radiobutton $area.app -text \
    "Complete Application: The generated\
     Application\nwill run without requiring Ellogon..." -image IApplication \
    -variable ::CreateStandAloneApplication::Info(ScriptApp) \
    -value 0 -compound left -padx 5 -anchor w] -fill x -expand 1
};# ::CreateStandAloneApplication::Page_ApplicationCodeType

##############################################################################
#### Page: CollectionInfo
proc ::CreateStandAloneApplication::Page_CollectionInfo {args} {
  GUI_LoadImage ITrashCanFull
  GUI_LoadImage IFolderLocked
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Information regarding Collection created by Application...} \
    -message "Please select the name of the Collection that will be created\
              by the Application during processing:"]
  frame $area.frame
  entry  $area.frame.entry
  button $area.frame.button -image ITOpen -text ... -compound left
  $area.frame.entry configure -textvar \
    ::CreateStandAloneApplication::Info(CollectionName)
  GUI_MakeEntryDirectoryDropTarget $area.frame.entry
  $area.frame.button configure -command "GUI_SelectDirectory $area.frame.entry"
  grid $area.frame.entry $area.frame.button -sticky snew -padx 3 -pady 2
  grid columnconfigure $area.frame 0 -weight 1
  pack $area.frame -fill x -expand 1

  pack [checkbutton $area.sync -text \
    "Sync Collection: The Collection will be saved after\neach Linguistic\
     Processing Component\nhas been executed..." -image IFolderLocked \
    -variable ::CreateStandAloneApplication::Info(SyncCollection) \
    -compound left -padx 5 -anchor w -justify left] -fill x -expand 1
  pack [checkbutton $area.del -text \
    "Delete Collection: The Collection will be deleted after\nprocessing\
     has been finished..." -image ITrashCanFull \
    -variable ::CreateStandAloneApplication::Info(DeleteCollection) \
    -compound left -padx 5 -anchor w -justify left] -fill x -expand 1
};# ::CreateStandAloneApplication::Page_CollectionInfo

##############################################################################
#### Page: FileInfo
proc ::CreateStandAloneApplication::Page_FileInfo {args} {
  GUI_LoadImage IEarth
  GUI_LoadImage IConsoleSmall
  GUI_LoadImage IMultipleFiles
  GUI_LoadImage ISearchFolder
  GUI_LoadImage IFileBroken
  variable Info
  if {$Info(GUI)} {
    package require BWidget
    set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
      -id CSAA -title {Create Stand-Alone Application Wizard!} \
      -header {Information regarding Files to be processed by Application...} \
      -message "Please specify whether the Application will accept a set of\
                files or will search for files in a directory:" \
      -fill x -expand 0]
      NoteBook $area.book -bg [$area cget -background] \
        -borderwidth $::GUI_Options(BorderWidth)
      set input  [$area.book insert end input  -text {Input Section} \
                    -background [$area cget -background]]
      set output [$area.book insert end output -text {Output Section} \
                    -background [$area cget -background]]
      pack [radiobutton $input.files -text \
        "Files Mode: The user must select the files\nto be processed." \
        -image IMultipleFiles -compound left -padx 5 -anchor w -justify left \
        -variable ::CreateStandAloneApplication::Info(FileSelectionMode) \
        -value files] -fill x
      pack [radiobutton $input.directory -text \
        "Directory Mode: The user must select a\ndirectory. All files\
         having the specified\nextensions will be processed." \
        -image ISearchFolder -compound left -padx 5 -anchor w -justify left \
        -variable ::CreateStandAloneApplication::Info(FileSelectionMode) \
        -value directory] -fill x
      frame $input.ext
        label $input.ext.label -text "File extensions:"
        entry $input.ext.exts -textvariable \
          ::CreateStandAloneApplication::Info(FileExtensions)
        grid $input.ext.label $input.ext.exts -sticky snew -padx 2
        grid columnconfigure $input.ext 1 -weight 1
      pack $input.ext -fill x -expand 1 -expand 1 -padx 30 -pady 4
      ## Allow users to select the encoding of the files?
      checkbutton $input.selectEncoding -image IEarth -compound left \
        -justify left -anchor w -padx 5 -variable \
        ::CreateStandAloneApplication::Info(FileEncodingSelectable) -text \
        "Allow users to select the Encoding\nof the files..."
      pack $input.selectEncoding -fill x
      pack [radiobutton $output.file -text \
        "File Mode: The user must select a file\nfor saving the\
         processing results." \
        -image IDocument -compound left -padx 5 -anchor w -justify left \
        -variable ::CreateStandAloneApplication::Info(OutputSelectionMode) \
        -value file] -fill x -expand 1
      pack [radiobutton $output.directory -text \
        "Directory Mode: The user must select a directory\nfor saving the\
         processing results." \
        -image IFolder -compound left -padx 5 -anchor w -justify left \
        -variable ::CreateStandAloneApplication::Info(OutputSelectionMode) \
        -value directory] -fill x -expand 1
      pack [radiobutton $output.none -text \
        "None: The user will not be prompt to select\nan output destination\
         for saving\nthe processing results." \
        -image IFileBroken -compound left -padx 5 -anchor w -justify left \
        -variable ::CreateStandAloneApplication::Info(OutputSelectionMode) \
        -value none] -fill x -expand 1
      $area.book raise input
    pack $area.book -fill both -expand 1
  } else {
    set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
      -id CSAA -title {Create Stand-Alone Application Wizard!} \
      -header {Information regarding Files to be processed by Application...} \
      -message "Please select the encoding of the files that will be processed\
                by the Application:" \
      -fill x -expand 0]
    ## File Encoding...
    frame $area.frame -padx 20
      label $area.frame.label -image IEarth
      menubutton $area.frame.enc -relief raised -indicatoron 1 -anchor c \
        -menu $area.frame.enc.menu \
        -textvariable ::CreateStandAloneApplication::Info(FileEncoding) \
        -text $::CreateStandAloneApplication::Info(FileEncoding) \
        -compound left -padx 10 -image [::Encoding::GetEncodingImage \
          $::CreateStandAloneApplication::Info(FileEncoding)] \
        -width 200 -bd $::GUI_Options(BorderWidth) -fg #ffff00 -bg #009900
      menu $area.frame.enc.menu -tearoff 0
      GUI_AppendEncodingsToMenu $area.frame.enc.menu \
        "set ::CreateStandAloneApplication::Info(FileEncoding) \$Encoding
         $area.frame.enc configure -image \$EncodingImage -text \$Encoding"
      pack $area.frame.label $area.frame.enc -side left -padx 3 -pady 2
    pack $area.frame -side top -fill x -expand 0
    ## Use files from arguments...
    pack [checkbutton $area.args -text \
      "Use Application arguments as file names..." \
      -image IConsoleSmall -onvalue "\$argv" -offvalue {} \
      -variable ::CreateStandAloneApplication::Info(FilesToProcess) \
      -compound left -padx 5 -anchor w] -fill x -expand 0
    frame $area.select
      label $area.select.dnd -text \
        "Or use drag & drop to select the files to be processed:"
      button $area.select.sel -image ITOpen -text ... -compound left \
        -command "GUI_SelectFiles $area.list.listbox"
      grid $area.select.dnd $area.select.sel -sticky snew -padx 3 -pady 2
      grid columnconfigure $area.select 0 -weight 1
    pack $area.select -fill x -expand 1
    ## A listbox to drop file names...
    ::ScrolledWidgets::ScrolledListbox $area.list -height 8 -width 90 \
      -listvariable ::CreateStandAloneApplication::Info(FilesToProcess)
    pack $area.list -fill both -expand 1
    GUI_MakeListboxFileDropTarget $area.list.listbox
  };# if $Info(GUI)
};# ::CreateStandAloneApplication::Page_FileInfo

##############################################################################
#### Page: GUI_Options
proc ::CreateStandAloneApplication::Page_GUI_Options {args} {
  variable Info
  if {!$Info(GUI)} {
    ## No GUI is specified, advance to the next/previous page...
    DisplayPage [lindex $args 0]
    return
  }
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Information regarding the Application GUI...} \
    -message "Please click on GUI elements to modify them:" -show 0 \
      -fill x -expand 0]
  ## Set the needed GUI_Elements...
  namespace eval ::GUI_MainWindow {}
  set ::GUI_MainWindow::Title                    $Info(GUI_Title)
  set ::GUI_MainWindow::TitleImage               $Info(GUI_TitleImageData)
  set ::GUI_MainWindow::TitleText                $Info(GUI_TitleText)
  set ::GUI_MainWindow::TitleTextBackground      $Info(GUI_TitleTextBackground)
  set ::GUI_MainWindow::TitleTextForeground      $Info(GUI_TitleTextForeground)
  set ::GUI_MainWindow::TitleTextFont            $Info(GUI_TitleTextFont)
  set ::GUI_MainWindow::InputMessage             $Info(GUI_InputMessage)
  set ::GUI_MainWindow::OutputMessage            $Info(GUI_OutputMessage)
  set ::GUI_MainWindow::EncodingMessage          $Info(GUI_EncodingMessage)
  set ::GUI_MainWindow::ExecMessage              $Info(GUI_ExecMessage)
  set ::GUI_MainWindow::ExitMessage              $Info(GUI_ExitMessage)
  set ::GUI_MainWindow::FileSelectionMode        $Info(FileSelectionMode)
  set ::GUI_MainWindow::OutputSelectionMode      $Info(OutputSelectionMode)
  set ::GUI_MainWindow::DirectorySelectionExtensions $Info(FileExtensions)
  set ::GUI_MainWindow::EncodingSelectableByUser $Info(FileEncodingSelectable)
  switch $Info(FileSelectionMode) {
    files     {set ::UserInputPaths { --- Select or Drop one or more Files ---}}
    directory {set ::UserInputPaths { --- Select or Drop a Directory ---}}
    default   {set ::UserInputPaths {}}
  }
  switch $Info(OutputSelectionMode) {
    file      {set ::UserOutputPaths { --- Select or Drop one File ---}}
    directory {set ::UserOutputPaths { --- Select or Drop a Directory ---}}
    default   {set ::UserOutputPaths {}}
  }
  set ::Encoding [encoding system]
  ## We will create a notebook, from where the user can specify the splash
  ## screen and the dialog options...
  NoteBook $area.book -bg [$area cget -background] \
        -borderwidth $::GUI_Options(BorderWidth)
  if {!$Info(ScriptApp)} {
    ## The splash screen section...
    set splash [$area.book insert end splash -text {Splash Screen} \
                    -background [$area cget -background]]
    image create photo ::GUI_MainWindow::SplashImage \
          -data $Info(GUI_SplashImageData)
    ScrolledWidgets::ScrolledCanvas $splash.image -bg white
    label $splash.image.canvas.image -image ::GUI_MainWindow::SplashImage
    $splash.image.canvas create window 0 0 -window $splash.image.canvas.image
    bind $splash.image.canvas.image <<MB_1>> \
     "::CreateStandAloneApplication::Edit image \
      ::CreateStandAloneApplication::Info(GUI_SplashImageData) %W"
    
    pack $splash.image -fill both -expand 1
  }
  
  ## The Window section
  set dialog [$area.book insert end dialog -text {Application Window} \
                    -background [$area cget -background]]
  image create photo ::GUI_MainWindow::WindowDecorationImage \
        -data $Info(GUI_WindowTitleImageData)
  canvas $dialog.title -background [$dialog cget -background] \
        -borderwidth 0 -relief flat -highlightthickness 0\
        -height [image height ::GUI_MainWindow::WindowDecorationImage] \
        -width  [image width  ::GUI_MainWindow::WindowDecorationImage]
  $dialog.title create image 1 1 -anchor nw \
     -image ::GUI_MainWindow::WindowDecorationImage
  $dialog.title create text 60 14 -anchor w -fill white -activefill yellow \
     -font {Arial 12 bold} -text $Info(GUI_Title) -tags text
  $dialog.title bind text <Enter> \
    "$dialog.title itemconfigure text -fill yellow"
  $dialog.title bind text <Leave> \
    "$dialog.title itemconfigure text -fill white"
  $dialog.title bind text <<MB_1>> \
    "::CreateStandAloneApplication::Edit canvastext \
    ::CreateStandAloneApplication::Info(GUI_Title) $dialog.title text"
  pack $dialog.title -fill x
  grid $dialog.title -sticky snew
  frame $dialog.frame -borderwidth 1 -relief raised
  CreateGUIAppWindow $dialog.frame.
  grid $dialog.frame -sticky snew -padx 1

  ## Configure the editing bindings...
  bind $dialog.frame.title.image <<MB_1>> \
   "::CreateStandAloneApplication::Edit image \
    ::CreateStandAloneApplication::Info(GUI_TitleImageData) %W"
  bind $dialog.frame.title.text <<MB_1>> \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_TitleText) %W"
  bind $dialog.frame.inout_select.ilabel <<MB_1>> \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_InputMessage) %W"
  bind $dialog.frame.inout_select.olabel <<MB_1>> \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_OutputMessage) %W"
  bind $dialog.frame.inout_select.iencLabel <<MB_1>> \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_EncodingMessage) %W"
  foreach widget [list $dialog.frame.title.image $dialog.frame.title.text \
          $dialog.frame.inout_select.ilabel $dialog.frame.inout_select.olabel \
          $dialog.frame.inout_select.iencLabel] {
    bind $widget <Enter> {%W configure -relief raised -borderwidth 1}
    bind $widget <Leave> {%W configure -relief flat   -borderwidth 1}
  }
  $dialog.frame.buttons.exec configure -command \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_ExecMessage) \
    $dialog.frame.buttons.exec"
  $dialog.frame.buttons.exit configure -command \
   "::CreateStandAloneApplication::Edit text \
    ::CreateStandAloneApplication::Info(GUI_ExitMessage) \
    $dialog.frame.buttons.exec"
  foreach widget [list $dialog.frame.buttons.exec $dialog.frame.buttons.exit] {
    $widget configure -activebackground white
  }
  pack $area.book -fill both -expand 1
  $area.book raise dialog
  ## Finally, replace some commands so as not to get errors...
  switch $Info(FileSelectionMode) {
    files     {$dialog.frame.inout_select.ibrowse configure -command \
               {gut_info "Dialog for selecting one or more Files..."}}
    directory {$dialog.frame.inout_select.ibrowse configure -command \
               {gut_info "Dialog for selecting a Directory..."}}
    default   {$dialog.frame.inout_select.ibrowse configure -command {}}
  }
  switch $Info(OutputSelectionMode) {
    file      {$dialog.frame.inout_select.obrowse configure -command \
               {gut_info "Dialog for selecting a single File..."}}
    directory {$dialog.frame.inout_select.obrowse configure -command \
               {gut_info "Dialog for selecting a Directory..."}}
    default   {$dialog.frame.inout_select.obrowse configure -command {}}
  }
  ::WizardDialog::DialogShow $area 810 430
};# ::CreateStandAloneApplication::Page_GUI_Options

##############################################################################
#### Page: AppParts
proc ::CreateStandAloneApplication::Page_AppParts {args} {
  variable Info
  if {$Info(ScriptApp)} {
    ## This is a script Application, no need to select the exported parts...
    DisplayPage [lindex $args 0]
    return
  }
  GUI_LoadImage IPartAlwaysDownload
  GUI_LoadImage IPartDownload
  GUI_LoadImage IPartNoDownload
  ## The user didnot select any components. What is he up to?
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header "Selecting $::ApplicationName Parts to be included in the\
             Application..." \
    -message "Please select the desired $::ApplicationName sub-systems:" \
      -fill x -expand 0 -show 0]
  ## Create a scrolled text and place some checkbuttons inside...
  ::ScrolledWidgets::ScrolledText $area.text -width 50 -height 12 \
    -highlightthickness 0
  set text $area.text.text
  set bg [$text cget -bg]
  ## Get all components...
  set i 1; set index 0
  set cols {#ffebcd #ffd700 #00bfff #b0c4de #32cd32 #f5deb3 #fff5ee #b0e0e6}
  foreach {col} [lrange $::GUI_Options(Colors) 0 20] {
    lappend cols [lindex $col 0]
  }
  InitialiseParts
  foreach {section entries} $Info(AppParts) {
    set col [lindex $cols $index]; incr index
    ## Add the language Indentification...
    set section_tag [string map {{ } _} $section]
    $text tag configure $section_tag -background black -foreground white \
      -font {helvetica 14 bold}
    $text tag configure i$section -background $col  -foreground $col
    $text insert end "$section\n" $section_tag
    incr i; set start $i.0
    foreach {label mode paths} $entries {
      switch $mode {
        opt_on {
          ## If the global variable does not exist, make the part active...
          if {![info exists ::CreateStandAloneApplication::AppParts($label)]} {
            set select 1
          } else {
            set select 0
          }
          checkbutton $text.c$i -text $label -image IPartNoDownload \
            -selectimage IPartDownload -indicatoron 0 \
            -offrelief flat -selectcolor {} -bd 0 -padx 10 \
            -variable ::CreateStandAloneApplication::AppParts($label) \
            -compound left -anchor w -bg $col -fg black -relief flat \
            -highlightthickness 0 -onvalue $paths -offvalue {}
          if {$select} {$text.c$i select}
        }
        optional {
          checkbutton $text.c$i -text $label -image IPartNoDownload \
            -selectimage IPartDownload -indicatoron 0 \
            -offrelief flat -selectcolor {} -bd 0 -padx 10 \
            -variable ::CreateStandAloneApplication::AppParts($label) \
            -compound left -anchor w -bg $col -fg black -relief flat \
            -highlightthickness 0 -onvalue $paths -offvalue {}
        }
        obligatory -
        default {
          label $text.c$i -text $label -image IPartAlwaysDownload \
            -compound left -anchor w -bg $col -fg black -relief flat \
            -highlightthickness 0 -bd 0 -padx 10
          set ::CreateStandAloneApplication::AppParts($label) $paths
        }
      }
      bind $text.c$i <MouseWheel> \
        "$text yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $text.c$i <4> "$text yview scroll -5 units"
      bind $text.c$i <5> "$text yview scroll  5 units"
      $text window create $i.0 -window $text.c$i
      $text insert end "\n"
      incr i
    }
    $text insert end "\n"
    incr i
    $text tag add i$section $start end
  }
  $text configure -bg $col
  $text delete $i.1 end
  $text configure -state disabled
  pack $area.text -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CreateStandAloneApplication::Page_AppParts

##############################################################################
#### Page: SelectComponents
proc ::CreateStandAloneApplication::Page_SelectComponents {args} {
  GUI_LoadImage IDynamicComponentSmall
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Selecting Processing Components...} -show 0 \
    -message "Please select the desired Linguistic Processing Components:" \
      -fill x -expand 0]
  ## Create a scrolled text and place some checkbuttons inside...
  ::ScrolledWidgets::ScrolledText $area.text -width 70 -height 12 \
    -highlightthickness 0
  set text $area.text.text
  set bg [$text cget -bg]
  ## Get all components...
  set i 1; set index 0
  set cols {#ffebcd #ffd700 #00bfff #b0c4de #32cd32 #f5deb3 #fff5ee #b0e0e6}
  foreach {col fg} [lrange $::GUI_Options(Colors) 0 20] {lappend cols $col}
  foreach {lang list} [::EditSystem::GUI_HS_GetModules] {
    set col [lindex $cols $index]; incr index
    ## Add the language Indentification...
    $text tag configure $lang  -background black -foreground white \
      -font {helvetica 14 bold}
    $text tag configure i$lang -background $col  -foreground $col
    $text insert end "Language: $lang\n" $lang
    incr i; set start $i.0
    foreach item [lindex $list 0] {
      foreach {mess component} $item {break}
      regexp {(.+)\s+(\([^\)]+\))} $mess _ title _
      checkbutton $text.c$i -text $title -image IDynamicComponentSmall \
        -variable ::CreateStandAloneApplication::Components($component) \
        -compound left -padx 0 -anchor w -bg $col -fg black -relief flat \
        -highlightthickness 0
      bind $text.c$i <MouseWheel> \
        "$text yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $text.c$i <4> "$text yview scroll -5 units"
      bind $text.c$i <5> "$text yview scroll  5 units"
      bind $text.c$i <Enter> \
        "::CreateStandAloneApplication::Page_SelectComponentsCheckEnter %W\
         \"[string trim $title]\" \"$component\""
      bind $text.c$i <Leave> {destroy %W.popup}
      $text window create $i.0 -window $text.c$i
      $text insert end "\n"
      incr i
    }
    $text tag add i$lang $start end
  }
  $text configure -bg $col
  $text delete $i.1 end
  $text configure -state disabled
  pack $area.text -fill both -expand 1
  ::WizardDialog::DialogShow $area
};# ::CreateStandAloneApplication::Page_SelectComponents

proc ::CreateStandAloneApplication::Page_SelectComponentsCheckEnter {w t c} {
  global GUI_Options
  toplevel $w.popup -bg $GUI_Options(TooltipBackground) -highlightthickness 1\
    -highlightbackground $GUI_Options(TooltipForeground) \
    -highlightcolor $GUI_Options(TooltipForeground)
  wm overrideredirect $w.popup 1
  label $w.popup.l -text "Component Name: \"$c\"\nComponent Title: \"$t\"" \
    -relief flat -wraplength 350 \
    -bg $GUI_Options(TooltipBackground) -padx 2 -pady 0 \
    -fg $GUI_Options(TooltipForeground) -anchor w
    pack $w.popup.l -side left -padx 0 -pady 0 -fill both -expand 1
  wm geometry $w.popup \
    +[expr {[winfo rootx $w] + [winfo width $w] + 10}]+[winfo rooty $w]
};# ::CreateStandAloneApplication::Page_SelectComponentsCheckEnter

##############################################################################
#### Page: CheckComponents
proc ::CreateStandAloneApplication::Page_CheckComponents {args} {
  variable Components
  variable Info
  ## Has the user selected any components?
  set SelectedComponents {}
  foreach component [array names Components] {
    if {$Components($component)} {lappend SelectedComponents $component}
  }
  ## We update the Component list only if the lists are not equal...
  if {![string equal [lsort $Info(SelectedComponents)] \
                     [lsort $SelectedComponents]]} {
    set Info(SelectedComponents) [lsort $SelectedComponents]
  }
  if {[llength $SelectedComponents]} {
    ## Everything went fine, advance to the next/previous page...
    DisplayPage [lindex $args 0]
    return
  }
  ## The user did not select any components. What is he up to?
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Error: No Components selected!} \
    -message "You haven't selected any Linguistic Processing Components!\n\
      \nPlease use the \"$::GUI_Text(Back)\" button to return to the previous\
      dialog in order to select some components.\n\
      \nHowever, you can proceed with this wizard: In this case an empty\
      Application framework will be created. You will have to manually edit\
      the generated code in order to use the Application."]
};# ::CreateStandAloneApplication::Page_CheckComponents

##############################################################################
#### Page: ComponentOrder
proc ::CreateStandAloneApplication::Page_ComponentOrder {args} {
  variable Info
  ## If no component is selected, advance to the next page...
  if {![llength $Info(SelectedComponents)] ||
      ([llength $Info(SelectedComponents)] == 1)} {
    DisplayPage [lindex $args 0]
    return
  }
  
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Selecting Processing Components Order...} \
    -message "Please select the order in which Linguistic Processing\
              Components will be executed:\n\
            \nSimply drag an item of the list in order to change its order..." \
      -fill x -expand 0]
  ::ScrolledWidgets::ScrolledListbox $area.list -listvariable \
   ::CreateStandAloneApplication::Info(SelectedComponents)
  pack $area.list -fill both -expand 1
  bind $area.list.listbox <<ListboxSelect>> \
    {::CreateStandAloneApplication::Page_ComponentOrderListboxSelect %W}
  bind $area.list.listbox <<MB_1>> \
    {set ::CreateStandAloneApplication::Page(PrevListboxSelection) {}}
};# ::CreateStandAloneApplication::Page_ComponentOrder

proc ::CreateStandAloneApplication::Page_ComponentOrderListboxSelect {l} {
  variable Page
  variable Info
  set currentIndex [$l curselection]
  if {[string length $Page(PrevListboxSelection)]} {
    ## We have to swap item positions...
    # puts "Swapping $Page(PrevListboxSelection) <-> $currentIndex"
    set prev [lindex $Info(SelectedComponents) $Page(PrevListboxSelection)]
    set curr [lindex $Info(SelectedComponents) $currentIndex]
    set list [lreplace $Info(SelectedComponents) $currentIndex $currentIndex \
                       $prev]
    set Info(SelectedComponents) [lreplace $list $Page(PrevListboxSelection) \
                       $Page(PrevListboxSelection) $curr]
  } else {
    # puts "Setting Page(PrevListboxSelection) = $currentIndex"
  }
  set Page(PrevListboxSelection) $currentIndex
};# ::CreateStandAloneApplication::Page_ComponentOrderListboxSelect

##############################################################################
#### Page: ComponentParameters
proc ::CreateStandAloneApplication::Page_ComponentParameters {args} {
  variable Info
  variable ComponentArgs
  ## If no component is selected, advance to the next page...
  if {![llength $Info(SelectedComponents)]} {
    DisplayPage [lindex $args 0]
    return
  }
  GUI_LoadImage IFolderSmall
  GUI_LoadImage IDocumentSmall
  GUI_LoadImage ITError
  GUI_LoadImage ITEarth
  GUI_LoadImage ITBoolean
  GUI_LoadImage ITComboBox
  GUI_LoadImage ITIntegers
  GUI_LoadImage ITEditText
  GUI_LoadImage IModuleProperties
  
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Selecting Processing Components Parameters...} \
    -message "You can use the facilities below to specify the Parameters that\
              will be provided to each Component during execution" \
      -fill x -expand 0]
  NoteBook $area.book -bg [$area cget -background] \
        -borderwidth $::GUI_Options(BorderWidth)
  foreach component $Info(SelectedComponents) {
    set frame [$area.book insert end $component -text $component \
                    -background [$area cget -background]]
    set index 0
    foreach {label spec defval} [lindex \
                                   [GUI_GetModuleOptions $component silent] 1] {
      if {![info exists ComponentArgs(Val:$component:$index)]} {
        set ComponentArgs(Val:$component:$index) $defval
        set ComponentArgs(Sel:$component:$index) Default
      }
      label $frame.image$index
      label $frame.label$index -text $label -anchor w
      entry $frame.value$index -width 16 -textvariable \
        ::CreateStandAloneApplication::ComponentArgs(Val:$component:$index)
      ComboBox $frame.combo$index -editable 0 -width 20 -textvariable \
        ::CreateStandAloneApplication::ComponentArgs(Sel:$component:$index)
      switch -glob -- $spec {
        {*%FILE%*}      {
          $frame.combo$index configure -values [list \
          Default {Select File...} \$::UserInputPaths \$::UserOutputPaths \
          {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback\
          $frame.combo$index $frame.value$index $spec $component $index $defval]
          $frame.image$index configure -image IDocumentSmall
        }
        {*%DIRECTORY%*} {
          $frame.combo$index configure -values [list \
          Default {Select Directory...} \
          \$::UserInputPaths \$::UserOutputPaths {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index $spec \
          $component $index $defval]
          $frame.image$index configure -image IFolderSmall
        }
        {*%NUMBER%*}    {
          $frame.combo$index configure -values [list \
          Default {Enter Number...} {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index $spec \
          $component $index $defval]
          $frame.image$index configure -image ITIntegers
        }
        {*%COMBO%*}     {
          if {[string equal $ComponentArgs(Sel:$component:$index) Default]} {
            set ComponentArgs(Val:$component:$index) [lindex $defval 0]
          }
          $frame.combo$index configure -values [concat Default \
                                                {{Parameter Handler}} $defval] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index $spec $component $index $defval]
          $frame.image$index configure -image ITComboBox
        }
        {*%ENCODING%*}  {
          $frame.combo$index configure -values [list \
          Default {Select Encoding...} {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index $spec \
          $component $index $defval]
          $frame.image$index configure -image ITEarth
        }
        {*%ENTRY%*}     {
          $frame.combo$index configure -values [list \
          Default {Enter Text...} {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index $spec \
          $component $index $defval]
          $frame.image$index configure -image ITEditText
        }
        {*%COMMAND%*}   {
          $frame.combo$index configure -state disabled
          $frame.image$index configure -image ITError
        }
        default         {
          ## Boolean value...
          if {[string equal $ComponentArgs(Sel:$component:$index) Default]} {
            switch -exact -- $defval {
              {} {set ComponentArgs(Val:$component:$index) false
                  set defval false}
              1  {set ComponentArgs(Val:$component:$index) true
                  set defval true}
            }
          }
          $frame.combo$index configure -values [list \
          Default TRUE FALSE {Parameter Handler}] \
          -modifycmd [list \
          ::CreateStandAloneApplication::Page_ComponentParametersComboCallback \
          $frame.combo$index $frame.value$index %BOOLEAN% \
          $component $index $defval]
          $frame.image$index configure -image ITBoolean
        }
      }
      $frame.value$index configure -state readonly
      grid $frame.image$index $frame.label$index \
           $frame.combo$index $frame.value$index\
           -row $index -sticky snew
      if {![string equal $ComponentArgs(Sel:$component:$index) Default]} {
        $frame.value$index configure -readonlybackground yellow
      }
      incr index
    }
    ## If index == 0, the Components has no parameters...
    if {$index == 0} {
      GUI_LoadImage ICancel48x48
      label $frame.cancel -image ICancel48x48 -anchor center
      label $frame.image -image IModuleProperties -bg #708090
      label $frame.label -textvariable GUI_Text(ModuleParameters) \
            -font {arial 14 bold italic} -anchor e -fg #008000 -bg #8fbc8f
      label $frame.module -text $component \
            -font {arial 14 bold italic} -anchor w -bg #8fbc8f -fg #6a5acd
      label $frame.des_module -textvariable GUI_Text(NoModuleParameters) \
            -bg #ffffaa -fg navy
      grid $frame.image  -row 0 -column 0 -rowspan 2 -sticky snew
      grid $frame.label  -row 0 -column 1 -sticky snew
      grid $frame.module -row 0 -column 2 -sticky snew
      grid $frame.des_module -row 1 -column 1 -columnspan 2 -sticky snew
      grid $frame.cancel -sticky snew -pady 30 -columnspan 3
      grid columnconfigure $frame 2 -weight 1
    } else {
      #grid columnconfigure $frame 2 -weight 2
      grid columnconfigure $frame 3 -weight 1
    }
  }
  entry $area.temp -state readonly
  set Info(ReadonlyEntryBackground) [$area.temp cget -readonlybackground]
  destroy $area.temp
  update
  $area.book raise [lindex $Info(SelectedComponents) 0]
  pack $area.book -fill both -expand 1
};# ::CreateStandAloneApplication::Page_ComponentParameters

proc ::CreateStandAloneApplication::Page_ComponentParametersComboCallback {
                                    combo entry spec component index def} {
  variable ComponentArgs
  variable Info
  set comboVal $ComponentArgs(Sel:$component:$index)
  if {![string equal $comboVal Default]} {
    $entry configure -readonlybackground yellow
  } else {
    $entry configure -readonlybackground $Info(ReadonlyEntryBackground)
  }
  switch -glob -- $spec {
    {*%FILE%*} {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) $def}
        {Select File...} {
          set file [GUI_SelectFile $entry]
          if {[string length $file]} {
            set ComponentArgs(Val:$component:$index) $file
          } else {
            $combo setvalue first
            Page_ComponentParametersComboCallback $combo $entry $spec \
                                                  $component $index $def
          }
        }
        \$::UserInputPaths {
          if {[string equal $Info(FileSelectionMode) files]} {
            set pos [gut_EntryDialog -message \
              "Please specify the position of the file in the list value\nof\
               the variable \"::UserInputPaths\":" -allow {\d} -initial 0]
            if {[string length $pos]} {
              set ComponentArgs(Val:$component:$index) \
                "\[lindex \$::UserInputPaths $pos\]"
            } else {
              $combo setvalue first
              Page_ComponentParametersComboCallback $combo $entry $spec \
                                                    $component $index $def
            }
          } else {
            set ComponentArgs(Val:$component:$index) {$::UserInputPaths}
          }
        }
        \$::UserOutputPaths {
          set ComponentArgs(Val:$component:$index) {$::UserOutputPaths}
        }
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
           SpecifyComponentParameters $component $index $spec $def]\]
        }
      }
    }
    {*%DIRECTORY%*} {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) $def}
        {Select Directory...} {
          if {![file exists $ComponentArgs(Val:$component:$index)]} {
            set ComponentArgs(Val:$component:$index) {}
          }
          set dir [GUI_SelectDirectory $entry]
          if {[string length $dir]} {
            set ComponentArgs(Val:$component:$index) $dir
          } else {
            $combo setvalue first
            Page_ComponentParametersComboCallback $combo $entry $spec \
                                                  $component $index $def
          }
        }
        \$::UserInputPaths {
          if {[string equal $Info(FileSelectionMode) files]} {
            set pos [gut_EntryDialog -message \
              "Please specify the position of the file in the list value\nof\
               the variable \"::UserInputPaths\":" -allow {\d} -initial 0]
            if {[string length $pos]} {
              set ComponentArgs(Val:$component:$index) \
                "\[lindex \$::UserInputPaths $pos\]"
            } else {
              $combo setvalue first
              Page_ComponentParametersComboCallback $combo $entry $spec \
                                                    $component $index $def
            }
          } else {
            set ComponentArgs(Val:$component:$index) {$::UserInputPaths}
          }
        }
        \$::UserOutputPaths {
          set ComponentArgs(Val:$component:$index) {$::UserOutputPaths}
        }
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
            SpecifyComponentParameters $component $index $spec $def]\]
        }
      }
    }
    {*%COMBO%*} {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) [lindex $def 0]}
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
            SpecifyComponentParameters $component $index $spec $def]\]
        }
        default {set ComponentArgs(Val:$component:$index) $comboVal}
      }
    }
    {*%NUMBER%*} {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) $def}
        {Enter Number...} {
          set pos [gut_EntryDialog -message \
            "Please specify the position of the directory in the list value\nof\
             the variable \"::UserInputPaths\":" -allow {\d} -initial 0]
          if {[string length $pos]} {
            set ComponentArgs(Val:$component:$index) $pos
          } else {
            $combo setvalue first
            Page_ComponentParametersComboCallback $combo $entry $spec \
                                                  $component $index $def
          }
        }
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
            SpecifyComponentParameters $component $index $spec $def]\]
        }
      }
    }
    {*%ENCODING%*} {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) $def}
        {Select Encoding...} {
          set encoding [::Encoding::GUI_SelectEncoding]
          if {[string length $encoding]} {
            set ComponentArgs(Val:$component:$index) $encoding
          } else {
            $combo setvalue first
            Page_ComponentParametersComboCallback $combo $entry $spec \
                                                  $component $index $def
          }
        }
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
            SpecifyComponentParameters $component $index $spec $def]\]
        }
      }
    }
    %BOOLEAN% {
      switch -exact -- $comboVal {
        Default {set ComponentArgs(Val:$component:$index) $def}
        TRUE    {set ComponentArgs(Val:$component:$index) true}
        FALSE   {set ComponentArgs(Val:$component:$index) false}
        {Parameter Handler} {
          set ComponentArgs(Val:$component:$index) \[[list \
            SpecifyComponentParameters $component $index $spec $def]\]
        }
      }
    }
    default {
    }
  }
  $entry xview 0
};# ::CreateStandAloneApplication::Page_ComponentParametersComboCallback

##############################################################################
#### Page: OutputDir
proc ::CreateStandAloneApplication::Page_OutputDir {args} {
  variable Info
  if {$Info(ScriptApp)} {
    set message "Please specify the file name into which the Application\
       script will be stored.\n\nIf a file with the same file name already\
       exists, it will be overwritten.\nIf a file with the same file name does\
       not exist, it will be created."
  } else {
    set message "Please specify the directory into which the Application will\
       be stored.\n\
       \nIf the directory already exists, all its contents will be deleted.\
       \nIf the directory does not exist, it will be created."
  }
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Specifying where to store Application...} \
    -message $message \
      -fill x -expand 0]
  frame $area.frame
    entry  $area.frame.entry
    button $area.frame.button -image ITOpen -text ... -compound left
  grid $area.frame.entry $area.frame.button -sticky snew -padx 3 -pady 2
  grid columnconfigure $area.frame 0 -weight 1
  pack $area.frame -fill x -expand 1
  if {$Info(ScriptApp)} {
    $area.frame.entry configure -textvar \
      ::CreateStandAloneApplication::Info(ScriptFileName)
    GUI_MakeEntryFileDropTarget $area.frame.entry
    $area.frame.button configure -command "GUI_SelectSaveFile $area.frame.entry"
  } else {
    $area.frame.entry configure -textvar \
      ::CreateStandAloneApplication::Info(AppDirName)
    GUI_MakeEntryDirectoryDropTarget $area.frame.entry
    $area.frame.button configure -command \
    "GUI_SelectDirectory $area.frame.entry"

    ## This is a complete Application. Prompt the user to also specify an
    ## Application name...
    frame $area.name_frame
      label $area.name_frame.label -text {Please specify an Application Name:}
      entry $area.name_frame.name -textvariable \
            ::CreateStandAloneApplication::Info(ApplicationName)
      grid $area.name_frame.label $area.name_frame.name \
           -sticky snew -padx 3 -pady 2
    grid columnconfigure $area.name_frame 1 -weight 1
    pack $area.name_frame -fill x -expand 1

    ## Finally, ask the user what OS to support...
    GUI_LoadImage IOS_Linux
    GUI_LoadImage IOS_Windows
    GUI_LoadImage IOS_Solaris
    frame $area.os
      label $area.os.msg -text "Please select the Operating Systems that the\
                                Application will run on:"
      grid $area.os.msg -columnspan 3 -sticky snw
      label $area.os.llinux   -text Linux
      label $area.os.lwindows -text Windows
      label $area.os.lsolaris -text SunOS
      checkbutton $area.os.linux   -image IOS_Linux \
        -variable ::CreateStandAloneApplication::Info(OS_Linux)
      checkbutton $area.os.windows -image IOS_Windows \
        -variable ::CreateStandAloneApplication::Info(OS_Windows)
      checkbutton $area.os.solaris -image IOS_Solaris \
        -variable ::CreateStandAloneApplication::Info(OS_SunOS)
      grid $area.os.linux $area.os.windows $area.os.solaris -sticky snew \
           -padx 10
      grid $area.os.llinux $area.os.lwindows $area.os.lsolaris -sticky snew
      grid columnconfigure $area.os 0 -weight 1
      grid columnconfigure $area.os 1 -weight 1
      grid columnconfigure $area.os 2 -weight 1
    pack $area.os -fill x -expand 1
  }
};# ::CreateStandAloneApplication::Page_OutputDir

##############################################################################
#### Page: Review
proc ::CreateStandAloneApplication::Page_Review {args} {
  variable Info
  ## Generate HTML that reviews the user preferences...
  set HTML "<HTML><BODY><center><H1>Application\
            &quot;$Info(ApplicationName)&quot; Information</H1></center>\n"
  ## Application type:
  append HTML "\n<H2>Application Type:</H2>"
  if {$Info(GUI)} {
    append HTML "<p><img src=\"IGUI\" align=middle> Application <b>will</b>\
                 have a Graphical User Interface.</p>"
  } else {
    append HTML "<p><img src=\"IConsole\" align=middle> Application\
                 <b>will not</b> have a Graphical User Interface.</p>"
  }
  append HTML "\n<H2>Application Type:</H2>"
  if {$Info(ScriptApp)} {
    append HTML "<p><img src=\"IScript\" align=middle> Application will\
                 be a <b>script</b> Application. Ellogon will be required in\
                 order to run the script.</p>
                 <H2>Application File:</H2>\
                 <p><img src=\"IDocument\" align=middle>\
                 $Info(ScriptFileName)</p>"
  } else {
    append HTML "\n<p><img src=\"IApplication\" align=middle> Application\
                 will be a <b>complete Application</b>. Ellogon will not be\
                 required in order to run the script.</p>\
                 <H2>Application Directory:</H2>\
                 <p><img src=\"ICollection\" align=middle>\
                 $Info(AppDirName)</p>"
    ## Application Parts:
    append HTML "\n<H2>Selected $::ApplicationName Parts:</H2>"
    variable AppParts
    foreach element [lsort [array names AppParts]] {
      if {![llength $AppParts($element)]} {continue}
      append HTML "\n<p><img src=\"IPartDownload\" align=middle> $element</p>"
    }
  }
  ## Selected Components:
  append HTML "\n<H2>Selected Components Sequence:</H2>"
  set index 1
  foreach component $Info(SelectedComponents) {
    append HTML "\n<p>$index. <img src=\"IDynamicComponentSmall\" align=middle>\
                 $component</p>"
    incr index
  }
  ## Collection Info:
  append HTML "\n<H2>Temporary Collection Name:</H2>"
  append HTML "\n<p><img src=\"ICollection\" align=middle>\
               $Info(CollectionName)</p>"
  append HTML "<p><img src=\"IFolderLocked\" align=middle>\
               Sync Collection: $Info(SyncCollection)</p>"
  append HTML "<p><img src=\"ITrashCanFull\" align=middle>\
               Delete Collection: $Info(DeleteCollection)</p>\n"
  ## Files to be processed:
  append HTML "\n<H2>Files to be processed:</H2>"
  append HTML "<p><img src=\"IEarth\" align=middle>\
               Encoding: &quot;$Info(FileEncoding)&quot;</p><ul>\n"
  if {$Info(FileEncodingSelectable)} {
    append HTML "\n<p>Encoding selectable by users.</p>"
  } else {
    append HTML "\n<p>Encoding not selectable by users.</p>"
  }
  foreach file $Info(FilesToProcess) {
    append HTML "<li>$file</li>\n"
  }
  append HTML "</ul>"
  append HTML "\n</BODY></HTML>"
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 0 1} \
    -id CSAA -title {Create Stand-Alone Application Wizard!} \
    -header {Reviewing Application Information...} \
    -html { }]
  $area.html.html clear
  $area.html.html configure -base . -imagecommand \
    ::CreateStandAloneApplication::Page_ReviewHtmlImage
  $area.html.html parse $HTML
};# ::CreateStandAloneApplication::Page_Review

proc ::CreateStandAloneApplication::Page_ReviewHtmlImage {image args} {
  return $image
}

##############################################################################
#### Page: Finish
proc ::CreateStandAloneApplication::Page_Finish {args} {
  global GUI_Text
  variable Info
  variable Components
  variable AppParts
  variable ComponentArgs
  ## Is it a script or an application?
  if {$Info(ScriptApp)} {
    ## Does the file exist?
    if {[file exists $Info(ScriptFileName)]} {
      set a [gut_question [format $GUI_Text(OverwriteFile?) \
                 $Info(ScriptFileName)]]
      if {[string equal $a no]} {DisplayPage OutputDir;return}
    } else {
      set a [gut_question [format $GUI_Text(CreateFile?) $Info(ScriptFileName)]]
      if {[string equal $a no]} {DisplayPage OutputDir;return}
    }
    SaveSettings [file dirname $Info(ScriptFileName)]/WizardProjectSettings.txt
    if {$Info(GUI)} {GenerateScriptGUI} else {GenerateScriptNoGUI}
  } else {
    if {[file isdirectory $Info(AppDirName)]} {
      set a [gut_question [format $GUI_Text(OverwriteFolder?) \
        $Info(AppDirName)]]
      if {[string equal $a no]} {DisplayPage OutputDir;return}
      ## Take special care so as not to overwrite a the user handlers file...
      if {[file exists $Info(AppDirName)/UserHandlers.tcl]} {
        set file [open $Info(AppDirName)/UserHandlers.tcl]
          set Handlers [string trim [read $file]]
        close $file
      }
      catch {file delete -force -- $Info(AppDirName)}
      file mkdir $Info(AppDirName)
      if {[info exists Handlers]} {
        set file [open $Info(AppDirName)/UserHandlers.tcl w]
          puts $file $Handlers
        close $file
      }
    } else {
      set a [gut_question [format $GUI_Text(CreateFolder?) $Info(AppDirName)]]
      if {[string equal $a no]} {DisplayPage OutputDir;return}
      file mkdir $Info(AppDirName)
    }
    SaveSettings $Info(AppDirName)/WizardProjectSettings.txt
    if {$Info(GUI)} {GenerateAppGUI}    else {GenerateAppNoGUI}
  }
  unset -nocomplain Info Components AppParts ComponentArgs
  array unset ::WizardDialog::Wizard CSAA|*
};# ::CreateStandAloneApplication::Page_Finish

proc ::CreateStandAloneApplication::LoadSettings {parent} {
  set filename [tk_getOpenFile -parent [winfo toplevel $parent]]
  if {![string length $filename]} {return}
  variable Info
  variable Components
  variable AppParts
  variable ComponentArgs
  source $filename
  set Info(LoadedProjectSettings) 1
};# ::CreateStandAloneApplication::LoadSettings

proc ::CreateStandAloneApplication::SaveSettings {filename} {
  variable Info
  variable Components
  variable AppParts
  variable ComponentArgs
  ## Remove some unwanted entries...
  unset -nocomplain Info(AppParts)
  ## Warn the user if the file exists...
  if {[file exists $filename]} {
    set reply [gut_question [format $::GUI_Text(OverwriteFile?) $filename]] 
    if {[string equal $reply "no"]} {
      set filename [GUI_SelectSaveFile -initialfile $filename]
      if {![string length $filename]} {return}
    }
  }
  set file [open $filename w]
  puts $file "array set Info \{[array get Info]\}"
  puts $file "array set Components \{[array get Components]\}"
  puts $file "array set AppParts \{[array get AppParts]\}"
  puts $file "array set ComponentArgs \{[array get ComponentArgs]\}"
  close $file
};# ::CreateStandAloneApplication::SaveSettings

##############################################################################
## Application Generation Procedures...
##############################################################################
proc ::CreateStandAloneApplication::GenerateScriptGUI {} {
  variable Info
  
  ## Read the template
  set file [open $::HomeDir/share/Templates/GenerateScriptGUI.tcl]
  set text [read $file]
  close $file

  ## Perform standart substitutions...
  set text [PerformSubstitutions $text]
  set file [open $Info(ScriptFileName) w]
  puts $file $text
  close $file

  ## Save the User Handlers Template
  GenerateUserHandlersTemplate \
          [file dirname $Info(ScriptFileName)]/UserHandlers.tcl

  ## Display the generate file...
  #ggi_view_text_file {} $Info(ScriptFileName) {} \
  #  "Generated Script Application - GUI"
};# ::CreateStandAloneApplication::GenerateScriptGUI

proc ::CreateStandAloneApplication::GenerateScriptNoGUI {} {
  variable Info

  ## Read the template
  set file [open $::HomeDir/share/Templates/GenerateScriptNoGUI.tcl]
  set text [read $file]
  close $file

  ## Perform standart substitutions...
  set text [PerformSubstitutions $text]
  set file [open $Info(ScriptFileName) w]
  puts $file $text
  close $file
  
  ## Save the User Handlers Template
  GenerateUserHandlersTemplate \
          [file dirname $Info(ScriptFileName)]/UserHandlers.tcl
  
  ## Display the generate file...
  #ggi_view_text_file {} $Info(ScriptFileName) {} \
  #  "Generated Script Application - No GUI"
};# ::CreateStandAloneApplication::GenerateScriptNoGUI

proc ::CreateStandAloneApplication::GenerateAppGUI {} {
  variable Info
  
  ## Read the template
  set file [open $::HomeDir/share/Templates/GenerateAppGUI.tcl]
  set text [read $file]
  close $file

  ## Perform standart substitutions...
  set text [PerformSubstitutions $text]
  set file [open $Info(AppDirName)/Init.tcl w]
  puts $file $text
  close $file

  ## Save the User Handlers Template
  GenerateUserHandlersTemplate $Info(AppDirName)/UserHandlers.tcl
  
  ## Copy the requested parts...
  CopyApplicationParts

  ## Display the generate file...
  #ggi_view_text_file {} $Info(AppDirName)/Init.tcl {} \
  #  "Generated Complete Application - GUI"
};# ::CreateStandAloneApplication::GenerateScriptGUI

proc ::CreateStandAloneApplication::GenerateAppNoGUI {} {
  variable Info
  
  ## Read the template
  set file [open $::HomeDir/share/Templates/GenerateAppGUI.tcl]
  set text [read $file]
  close $file

  ## Perform standart substitutions...
  set text [PerformSubstitutions $text]
  set file [open $Info(AppDirName)/Init.tcl w]
  puts $file $text
  close $file

  ## Save the User Handlers Template
  GenerateUserHandlersTemplate $Info(AppDirName)/UserHandlers.tcl
  
  ## Copy the requested parts...
  CopyApplicationParts

  ## Display the generate file...
  #ggi_view_text_file {} $Info(AppDirName)/Init.tcl {} \
  #  "Generated Complete Application - No GUI"
};# ::CreateStandAloneApplication::GenerateScriptNoGUI

proc ::CreateStandAloneApplication::GenerateUserHandlersTemplate {filename} {
  if {[file exists $filename]} {
    set reply [gut_question [format $::GUI_Text(OverwriteFile?) $filename]] 
    if {[string equal $reply "no"]} {return}
  }
  ## Write the Handlers file...
  set file [open $filename w]
    set code [string trim [info body \
                ::CreateStandAloneApplication::UserProvidedFumctionsTemplate]]
    while {[regsub -all -line {^(\ +)(\S+(\ +\S+)*)(\ \ \ ?)(\S+)} $code \
                                  "\\1\\2 \\\n\\1  \\5" code]} {}
    puts $file $code
  close $file
};# ::CreateStandAloneApplication::GenerateUserHandlersTemplate

## ::CreateStandAloneApplication::PerformSubstitutions
#  Perform known substitutions in provided data...
proc ::CreateStandAloneApplication::PerformSubstitutions {text} {
  variable Info
  variable ComponentArgs

  ## Substitute string values...
  foreach {pattern variable} {
           @AppName@                               ::ApplicationName
           @ApplicationName@                       Info(ApplicationName)
           @Components@                            Info(SelectedComponents)
           @Files@                                 Info(FilesToProcess)
           @Encoding@                              Info(FileEncoding)
           @EncodingSelectableByUser@              Info(FileEncodingSelectable)
           @CollectionName@                        Info(CollectionName)
           @FileSelectionMode@                     Info(FileSelectionMode)
           @DirectorySelectionExtension@           Info(FileExtensions)
           @OutputSelectionMode@                   Info(OutputSelectionMode)
           @GUI_MainWindow::Title@                 Info(GUI_Title)
           @GUI_MainWindow::TitleText@             Info(GUI_TitleText)
           @GUI_MainWindow::TitleImage@            Info(GUI_TitleImageData)
           @GUI_MainWindow::TitleTextBackground@   Info(GUI_TitleTextBackground)
           @GUI_MainWindow::TitleTextForeground@   Info(GUI_TitleTextForeground)
           @GUI_MainWindow::TitleTextFont@         Info(GUI_TitleTextFont)
           @GUI_MainWindow::InputMessage@          Info(GUI_InputMessage)
           @GUI_MainWindow::EncodingMessage@       Info(GUI_EncodingMessage)
           @GUI_MainWindow::OutputMessage@         Info(GUI_OutputMessage)
           @GUI_MainWindow::ExecMessage@           Info(GUI_ExecMessage)
           @GUI_MainWindow::ExitMessage@           Info(GUI_ExitMessage)
           @GUI_MainWindow::SplashImage@           Info(GUI_SplashImageData)
                                                                        } {
    set text [string map [list $pattern \"[set $variable]\"] $text]
  }
  ## Substitute flag values...
  foreach {pattern variable} {@SyncCollection@   Info(SyncCollection)
                              @DeleteCollection@ Info(DeleteCollection)
                                                                        } {
    set text [string map [list $pattern [set $variable]] $text]
  }
  LoadProcsToBeExported
  ## Substitute various "peculiar" values...
  foreach {pattern proc} {@ApplicationExecuteProcBody@ AppExecutionProc
                          @DisplayMainWindowProcBody@  CreateGUIAppWindow
                          @Initialise@                 AppInitialisationProc} {
    set code [string trim [info body ::CreateStandAloneApplication::$proc]]
    while {[regsub -all -line {^(\ +)(\S+(\ +\S+)*)(\ \ \ ?)(\S+)} $code \
                                  "\\1\\2 \\\n\\1  \\5" code]} {}
    set text [string map [list $pattern $code] $text]
  }
  ## Generate the default Annotator parameter list...
  set Parameters {}
  foreach Annotator $Info(SelectedComponents) {
    ## Search the Parameters array and get all non-default values...
    set argList {}
    foreach key [lsort [array names ComponentArgs Sel:$Annotator:*]] {
      if {![string equal $ComponentArgs($key) Default]} {
        set index [lindex [split $key :] end]
        lappend argList $index $ComponentArgs(Val:$Annotator:$index)
      }
    }
    if {[llength $argList]} {
      append Parameters "  set AnnotatorParameters($Annotator) \[list \\\n"
      foreach {index value} $argList {
        append Parameters "         $index \"$value\" \\\n"
      }
      append Parameters "  ]\n"
    } else {
      append Parameters "  set AnnotatorParameters($Annotator) {}\n"
    }
  }
  set text [string map \
      [list @AnnotatorDefaultParameterList@ $Parameters] $text]
  return $text
};# ::CreateStandAloneApplication::PerformSubstitutions

## ::CreateStandAloneApplication::CopyApplicationParts
#  This function will copy the specified by user Application parts into
#  the specified directory, thus creating a stand-alone Application...
proc ::CreateStandAloneApplication::CopyApplicationParts {} {
  variable AppParts
  variable Info
  global HomeDir GUI_Progress

  ## Iterate over all defined AppParts, and for those that their value is not
  ## empty, performed the associated files copy...
  set elements [array names AppParts]
  set Step [expr \
    {100.0/([llength $elements]+[llength $Info(SelectedComponents)]+1)}]
  set Progress 0.0
  set wwin [ggi_wait_message {} $Progress]
  ## Copy Components...
  file mkdir $Info(AppDirName)/share/modules
  foreach Annotator $Info(SelectedComponents) {
    ## Skip non-existent Annotators...
    if {![info exists ::creole_${Annotator}_home]} {
      file mkdir $Info(AppDirName)/share/modules/$Annotator
      continue
    }
    set AnnotatorDir [set ::creole_${Annotator}_home]
    set GUI_Progress(Message:$wwin) \
        "Copying \"$AnnotatorDir\" ->\
                 \"$Info(AppDirName)/share/modules\""
    set Progress [expr {$Progress+$Step}]
    ggi_wait_update {} $Progress $wwin
    update
    catch {file copy -force -- $AnnotatorDir $Info(AppDirName)/share/modules}
    ## Remove any OS that must not exist in the produced Application...
    foreach os {Linux Windows SunOS} {
      if {!$Info(OS_$os)} {
        if {[file exists $Info(AppDirName)/share/modules/$AnnotatorDir/$os]} {
          catch {file delete -force -- \
                             $Info(AppDirName)/share/modules/$AnnotatorDir/$os}
        }
      };# if {!$Info(OS_$os)}
    };# foreach os {Linux Windows SunOS}
  };# foreach Annotator 
  foreach element $elements {
    foreach path $AppParts($element) {
      ## If the path is an absolute path, then this is a tool...
      if {[file pathtype $path] == "absolute"} {
        ## Perform a direct copy to the modules directory...
        set GUI_Progress(Message:$wwin) \
        "Copying \"$path\" ->\
                 \"[file dirname $Info(AppDirName)/share/modules]\""
        catch {file copy -force -- $path $Info(AppDirName)/share/modules}
        continue
      }
      ## Filter paths according to the supported platforms...
      switch -glob -- $path {
        *Linux*   {if {!$Info(OS_Linux)}   continue}
        *Windows* {if {!$Info(OS_Windows)} continue}
        *SunOS*   {if {!$Info(OS_SunOS)}   continue}
        default   {}
      }
      ## Ensure that the target directory exists...
      set target $Info(AppDirName)/$path
      set targetDir [file dirname $target]
      if {![file exists $targetDir]} {
        file mkdir $targetDir
      }
      ## If target is a directory and it exists, we have to delete it...
      if {[file exists $target] && [file isdir $target]} {
        catch {file delete -force -- $target}
      }
      set GUI_Progress(Message:$wwin) \
        "Copying \"$HomeDir/$path\" ->\
                 \"[file dirname $Info(AppDirName)/$path]\""
      #puts "Copying \"$HomeDir/$path\" ->\
      #           \"[file dirname $Info(AppDirName)/$path]\""
      update
      catch {
        file copy -force $HomeDir/$path \
                              [file dirname $Info(AppDirName)/$path]
      }
    }
    set Progress [expr {$Progress+$Step}]
    ggi_wait_update {} $Progress $wwin
  }
  ggi_destroy_wait $wwin
};# ::CreateStandAloneApplication::CopyApplicationParts

proc ::CreateStandAloneApplication::Edit {what var widget {tag {}}} {
  switch $what {
    image {
      set imageFile [tk_getOpenFile -parent $widget \
       -filetypes {{GIF *.gif} {PNG *.png} {BMP *.bmp} {{All Files} *.*}} \
       -title {Please select an Image file...}]
      if {![string length $imageFile]} {return}
      set file [open $imageFile]
      fconfigure $file -encoding binary -translation binary
      set data [read $file]
      close $file
      package require base64
      set data [base64::encode $data]
      set image [$widget cget -image]
      $image blank
      $image configure -data $data
      set $var $data
    }
    text {
      set text [gut_EntryDialog -message {Please enter the desired text:} \
                   -initial [set $var] -title {Change Widget Text Label...}]
      if {![string length $text]} {return}
      $widget configure -text $text
      set $var $text
    }
    canvastext {
      set text [gut_EntryDialog -message {Please enter the desired text:} \
                   -initial [set $var] -title {Change Widget Text Label...}]
      if {![string length $text]} {return}
      $widget itemconfigure $tag -text $text -fill white
      set $var $text
    }
    default {}
  }
};# ::CreateStandAloneApplication::Edit
