##############################################################################
##  CreateStandAloneApplicationParts.tcl:                                   ##
## ---------------------------------------                                  ##
##  This file implements a wizard for exporting a system or a collection of ##
##  out of Ellogon, in a stand-alone application. This application can have ##
##  a completelly custom interface or run in a batch mode, without requiring##
##  a graphical interface at all...                                         ##
##                                                                          ##
##  Copyright 1998-2008 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

proc ::CreateStandAloneApplication::InitialiseParts {} {
  set ::CreateStandAloneApplication::Info(AppParts) {
    {Component Languages Support} {
       {Include Java Support} optional {
          {CDM/Java/JCDM.jar}
          {CDM/Java/Windows/libJCDM.dll}
          {CDM/Java/Windows/x86_32/libJCDM.dll}
          {CDM/Java/Windows/x86_64/libJCDM.dll}
          {CDM/Java/Linux/libJCDM.so}
          {CDM/Java/Linux/x86_32/libJCDM.so}
          {CDM/Java/Linux/x86_64/libJCDM.so}
          {CDM/Java/SunOS/libJCDM.so}
          {CDM/Java/SunOS/x86_32/libJCDM.so}
          {CDM/Java/SunOS/x86_64/libJCDM.so}
          {CDM/Java/Darwin/libJCDM.so}
          {CDM/Java/Darwin/x86_32/libJCDM.so}
          {CDM/Java/Darwin/x86_64/libJCDM.so}
          {CDM/Java.tcl}
          {lib/tcljava1.4.1}
          {lib/xputils}
       }
       {Include Perl Support} optional {
          {CDM/Perl/Windows}
          {CDM/Perl/Linux}
          {CDM/Perl/SunOS}
          {CDM/Perl/Darwin}
          {CDM/Perl.tcl}
          {lib/tclperl-3.2/Windows}
          {lib/tclperl-3.2/Linux}
          {lib/tclperl-3.2/SunOS}
          {lib/tclperl-3.2/Darwin}
          {lib/tclperl-3.2/pkgIndex.tcl}
       }
       {Include Python Support} optional {
          {CDM/Python/Windows}
          {CDM/Python/Linux}
          {CDM/Python/SunOS}
          {CDM/Python/Darwin}
          {CDM/Python.tcl}
          {lib/tclpython-4.1/Windows}
          {lib/tclpython-4.1/Linux}
          {lib/tclpython-4.1/SunOS}
          {lib/tclpython-4.1/Darwin}
          {lib/tclpython-4.1/pkgIndex.tcl}
       }
    }
    {Graphical User Interface Elements} {
       {Include BWidgets} optional {{GUI/Widgets/BWidget}}
       {Include Drag and Drop Support (tkDND 1.0)} opt_on {{lib/tkdnd/lib}}
       {Include HTML Display Support (TkHtml)} optional {
         {CDM/Windows/x86_32/libTkHtml.dll}
         {CDM/Windows/x86_64/libTkHtml.dll}
         {CDM/Linux/x86_32/libTkHtml.so}
         {CDM/Linux/x86_64/libTkHtml.so}
         {CDM/SunOS/x86_32/libTkHtml.so}
         {CDM/SunOS/x86_64/libTkHtml.so}
         {CDM/Darwin/x86_32/libTkHtml.so}
         {CDM/Darwin/x86_64/libTkHtml.so}
       }
       {Include Encoding Images} opt_on {share/Locale/EncodingFlags}
    }
    {Tcl Language Extensions} {
       {Include ODBC Support (tclodbc 2.2)}        optional {{lib/tclodbc2.2}}
    }
    {SGML Support} {
      {Importing/Exporting SGML} optional {
        {Utils/ImportExportSGML.tcl} {Utils/tclIndex}
      }
    }
    {XML Support} {
       {Include XML (tclXML 2.0 & tDOM 0.7)} optional {
       }
    }
    {Graphical Viewers} {
      {Annotation Explorer} optional {Viewers/AnnotationExplorer.tcl}
      {Single Span Viewer} optional {Viewers/SingleSpanViewer.tcl}
      {Annotation Raw Viewer} optional {Viewers/AnnotationRawViewer.tcl}
      {XML DOM Tree Viewer (Requires also XML Support)} optional {
        Viewers/XMLDomTreeViewer.tcl
      }
      {Annotation Viewer} optional {Viewers/AnnotationViewer.tcl}
      {Attribute Viewer} optional {Viewers/AttributeViewer.tcl}
      {HTML Viewers} optional {Viewers/HTMLViewers.tcl}
      {Morphological Viewer} optional {Viewers/MorphologyViewer.tcl}
      {Parse Tree Viewer} optional {Viewers/ParseViewer.tcl}
      {Stem Viewer} optional {Viewers/StemViewer.tcl}
    }
    {Tools} {##TOOLS##}
    {Essential Elements} {
       {CDM Core Libraries} obligatory {
         {CDM/Windows/x86_32/libCDM.dll}    {CDM/Windows/x86_64/libCDM.dll}
         {CDM/Windows/x86_32/libWinIco.dll} {CDM/Windows/x86_64/libWinIco.dll}
         {CDM/Linux/x86_32/libCDM.so}       {CDM/Linux/x86_64/libCDM.so}
         {CDM/SunOS/x86_32/libCDM.so}       {CDM/SunOS/x86_64/libCDM.so}
         {CDM/Darwin/x86_32/libCDM.so}      {CDM/Darwin/x86_64/libCDM.so}
         {CDM/CDM.tcl} {CDM/tip_Annotate.tcl}  {CDM/CDM_LicenseDlg.tcl}
         {CDM/tip_SelectAnnotationsSorted.tcl} {CDM/tclIndex}
       }
       {Default Configuration Options} obligatory {
         {share/Config/OptionDatabase.tcl}
         {share/Config/Options.tcl}
       }
       {Loading Components Support} obligatory {
         {Utils/Configure.tcl} {Utils/LoadPlugins.tcl} {Utils/tclIndex}
         {Utils/GUT.tcl} {Utils/AnnotationConfiguration.tcl}
       }
       {Default Messages (English)} obligatory {
         {share/Locale/English.msg} {share/Locale/English.rc}
       }
       {Splash Screen Support} obligatory {{GUI/Dialogs/SplashScreen.tcl}}
       {File/Directory Drag & Drop Handlers} obligatory 
                                          {{GUI/Dialogs/SelectFileSystem.tcl}}
       {Tcl Graphical Error Handler} obligatory {{GUI/Dialogs/BgError.tcl}}
       {Progress Bar Widget} obligatory {
         {GUI/Widgets/ProgressBar.tcl} {GUI/Widgets/BWidget/progressbar.tcl}
         {GUI/Widgets/BWidget/progressdlg.tcl} {GUI/Widgets/BWidget/widget.tcl}
       }
       {Scrolled Widget Support} obligatory {{GUI/Widgets/ScrolledWidgets.tcl}}
       {Graphical Console} obligatory {{GUI/Widgets/TkCon.tcl}}
       {Various Graphical Utilities} obligatory {
          {GUI/Widgets/Utilities.tcl} {GUI/GUI.tcl} {GUI/Dialogs/Encoding.tcl}
          {GUI/Widgets/BWidget/images} {GUI/Widgets/BWidget/lang/en.rc}
          {GUI/Widgets/BWidget/messagedlg.tcl} {GUI/Widgets/BWidget/dialog.tcl}
          {GUI/Widgets/BWidget/pkgIndex.tcl} {GUI/Widgets/BWidget/buttonbox.tcl}
          {GUI/Widgets/BWidget/button.tcl} {GUI/Widgets/BWidget/dynhelp.tcl}
          {GUI/Widgets/BWidget/bitmap.tcl} {GUI/Widgets/BWidget/utils.tcl}
          {GUI/Widgets/BWidget/init.tcl} {GUI/Dialogs/MSAgent.tcl}
          {Viewers/Viewers.tcl} {Viewers/Colors.tcl} {Viewers/tclIndex}
          {GUI/SystemWindow.tcl} {GUI/GUI_Utilities.tcl}
       }
       {Widgets/Dialogs Index Files} obligatory {
         {GUI/Widgets/tclIndex} {GUI/Dialogs/tclIndex} {GUI/tclIndex}
       }
       {Abort Dialog} obligatory {{GUI/Abort.tcl}}
    }
  }
  
  ##
  ## Collect all the registered tools and add them to the list...
  ##
  set toolsEntry {}
  foreach toolSpec [lsort -dictionary [array names ::GUI_ToolConfig]] {
    regexp -- {([^:]*)::([^:]+)} $toolSpec match Class Tool
    if {![info exists ::creole_${Tool}_home]} {continue}
    if {[string length $Class]} {
      set title [PUL_ToolConfig $Tool title $Class]
    } else {
      set title [PUL_ToolConfig $Tool title]
    }
    lappend toolsEntry "$Class -> $title ($Tool)" \
                        optional [set ::creole_${Tool}_home]
  }
  set index [lsearch $::CreateStandAloneApplication::Info(AppParts) \
                     {##TOOLS##}]
  if {$index != -1} {
    set ::CreateStandAloneApplication::Info(AppParts) [lreplace \
        $::CreateStandAloneApplication::Info(AppParts) \
        $index $index $toolsEntry]
  }
};# ::CreateStandAloneApplication::InitialiseParts
