##############################################################################
##  ExportText.tcl:                                                         ##
## ---------------------                                                    ##
##  This file implements a utility for exporting the textual data of a      ##
##  Collection. Exports the text of All Documents is a specified Collection ##
##  as plain text files in a user-defined directory. A new directory, named ##
##  after the Collection to be exported will be created, and the texts will ##
##  be saved in there, under the Document Id of each Document.              ##
##  Initial version date: 21/01/2000                                        ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

proc GUT_ExportCollectionTexts {{col {}} {win .gut_ExportCollectionTexts}} {
  if {[winfo exists $win]} {raise $win; return}
  global GUI_Text UserHomeDir

  if {![string length $col]} {
    global GUI_CollectionDlg
    if {[string equal $GUI_CollectionDlg(Col2Open) {}]} return
    set col [tip_OpenCollection $GUI_CollectionDlg(Col2Open)]
  }
  
  toplevel $win
  wm withdraw $win
  wm title $win {Export Collection Documents As Text...}

  set main $win.main
  frame $main
  ## Place an Image at the left side (like wizards:-))
  label $main.image -text image
  pack $main.image -fill y -expand 1

  ## The actual window widgets
  frame $main.data
    label $main.data.colnamel -textvariable GUI_Text(CollectionName)
    entry $main.data.colname -state disabled
          $main.data.colname insert end [tip_GetName $col]
    grid $main.data.colnamel $main.data.colname -sticky snew
    
  set DestDir [file join $UserHomeDir [file tail [tip_GetName $col]]]
  puts "Destination Dir: $DestDir"
  file mkdir $DestDir
  ## Itarate over Collection Documents...
  set files [$col GetFileIds]; set file_nu [llength $files]
  if {!$file_nu} {set file_nu 1}
  set progress 0.0
  set step [expr {100.0/$file_nu}]
  set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]
  foreach file $files {
    set doc [tip_GetDocument $col $file]
    set Text [tip_GetRawData $doc]
    set filename [file join $DestDir [tip_GetId $doc]]
    set file [open $filename w]
    fconfigure $file -encoding [tip_GetEncoding $doc]
    puts -nonewline $file $Text
    close $file
    #puts "  +++ Written file: $filename"
    tip_Close $doc
    set progress [expr {$progress+$step}]
    catch {ggi_wait_update {} $progress $wwin}
  }
  catch {ggi_destroy_wait $wwin}
};# GUT_ExportCollectionTexts
