##############################################################################
##  GUT.tcl:                                                                ##
## --------------------------------                                         ##
##  This file implements various utilities.                                 ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

#############################################################################
## Do not remove the folowing command. It is needed in case where an
## error occurs while loading a module...
if {$::ApplicationInfo(EnableGUI)} {
  catch {package require BWidget};# Do not remove this. It is needed for dialogs
}

## gut_not_done_yet
#   Displays the "Undone" Message
proc gut_not_done_yet {} {
  gut_Debug "Not Done yet!"
}

## gut_error
proc gut_error {message {info 0}} {
  error $message
}

## gut_ErrorDlg
#    Displays an Error Dialog
proc gut_ErrorDlg {message {win {}}} {
  global ApplicationInfo tcl_platform GUI_Options
  if {!$ApplicationInfo(EnableGUI)} {
    catch {puts stderr $message}
    return
  }

  ## Try to find a window to act as a parent, as "." is "hidden" to
  ## the top-left corner of the screen...
  if {![string length $win]} {catch {set win [focus]}}
  if {[string length $win] && [winfo exists $win]} {
    set parent [winfo toplevel $win]
  } else {
    if {[winfo exists .mw]} {set parent .mw} else {set parent .}
  }

  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }
  
  switch -exact $tcl_platform(platform) {
    unix {
      bell
      if {[catch {.mw cget -bg} bg]} {
        set bg $GUI_Options(Background)
      }
      MessageDlg .mw_mesgDlg -title Error... -message $message -parent $parent \
             -type ok -default 0 -icon error -justify left -bg $bg
    }
    default {
      catch {$GUI_Options(AgentID) play Announce
             $GUI_Options(AgentID) play Confused}
      tk_messageBox -title Error... -message $message -parent $parent \
             -type ok -default ok -icon error
    }
  }
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
}

## gut_warning
#    Displays a Warning Dialog box...
proc gut_warning {message {win {}}} {
  global ApplicationInfo tcl_platform GUI_Options
  if {!$ApplicationInfo(EnableGUI)} {
    catch {puts stderr $message}
    return
  }
  LDM_DeleteSplashWindow
  ## Try to find a window to act as a parent, as "." is "hidden" to
  ## the top-left corner of the screen...
  if {![string length $win]} {catch {set win [focus]}}
  if {[string length $win] && [winfo exists $win]} {
    set parent [winfo toplevel $win]
  } else {
    if {[winfo exists .mw]} {set parent .mw} else {set parent .}
  }

  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }

  switch -exact $tcl_platform(platform) {
    unix {
      bell
      if {[catch {.mw cget -bg} bg]} {
        set bg $GUI_Options(Background)
      }
      if {[catch {
        MessageDlg .mw_mesgDlg -title Warning... -message $message -parent \
             $parent -type ok -default 0 -icon warning -justify left -bg $bg
                 }]} {
        tk_messageBox -title Warning... -message $message -parent $parent \
             -type ok -default ok -icon warning
      }
    }
    default {
      catch {$GUI_Options(AgentID) play GetAttention
             $GUI_Options(AgentID) play GetAttentionContinued
             $GUI_Options(AgentID) play GetAttentionReturn}
      tk_messageBox -title Warning... -message $message -parent $parent \
             -type ok -default ok -icon warning
    }
  }
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
}

## gut_errexit
#     Displays an Error Dialog and exits application with an error...
proc gut_errexit {message} {
  gut_ErrorDlg $message
  puts $message
  puts stderr $message
  exit -1
}

## gut_tk_dialog
#     Display a Dialog...
proc gut_tk_dialog {Title Message Type {Icon warning} {win {}}} {
  global ApplicationInfo tcl_platform GUI_Options
  if {!$ApplicationInfo(EnableGUI)} {
    catch {puts stderr $Message}
    return
  }
  set type [string tolower $Type]
  ## Try to find a window to act as a parent, as "." is "hidden" to
  ## the top-left corner of the screen...
  if {![string length $win]} {catch {set win [focus]}}
  if {[catch {winfo viewable $win} viewable]} {set viewable 0}
  if {[string length $win] && $viewable} {
    set parent [winfo toplevel $win]
  } else {
    if {[winfo exists .mw]} {set parent .mw} else {set parent .}
  }

  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }

  switch -exact $tcl_platform(platform) {
    unix {
      if {[catch {.mw cget -bg} bg]} {
        set bg $GUI_Options(Background)
      }
      set res [MessageDlg .mw_mesgDlg -title $Title -message $Message \
                   -parent $parent -type $type -icon $Icon -justify left \
                   -bg $bg]
      switch -exact $type {
        abortretryignore {set vals {abort retry ignore}; set def abort}
        ok               {return ok}
        okcancel         {set vals {ok cancel};          set def cancel}
        retrycancel      {set vals {retry cancel};       set def cancel}
        yesno            {set vals {yes no};             set def no}
        yesnocancel      {set vals {yes no cancel};      set def cancel}
      }
      if {$res < 0} {return $def}
      return [lindex $vals $res]
    }
    default {
      catch {$GUI_Options(AgentID) play Surprised
             $GUI_Options(AgentID) play Confused}
      return [tk_messageBox -title $Title -message $Message -parent $parent \
             -type $type -icon $Icon]
    }
  }
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
}

## gut_question
proc gut_question {message {title Question...} {def 1} {win {}}
                   {type {yes no}}}  {
  global ApplicationInfo tcl_platform GUI_Options
  ## Try to find a window to act as a parent, as "." is "hidden" to
  ## the top-left corner of the screen...
  if {![string length $win]} {catch {set win [focus]}}
  if {[string length $win] && [winfo exists $win]} {
    set parent [winfo toplevel $win]
  } else {
    if {[winfo exists .mw]} {set parent .mw} else {set parent .}
  }

  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }

  switch -exact $tcl_platform(platform) {
    unix {
      if {[catch {.mw cget -bg} bg]} {
        set bg $GUI_Options(Background)
      }
      set res [MessageDlg .mw_mesgDlg -title $title -message $message \
          -parent $parent -type [join $type {}] -default $def -icon question \
          -justify left -bg $bg]
      if {$res < 0} {return no}
      return [lindex $type $res]
    }
    default {
      catch {$GUI_Options(AgentID) play Confused
             $GUI_Options(AgentID) play LookDownBlink
             $GUI_Options(AgentID) play LookDownReturn}
      return [tk_messageBox -title $title -message $message -parent $parent \
             -type [join $type {}] -default no -icon question]
    }
  }
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
};# gut_question

## gut_info
proc gut_info {message {title Info...} {win {}}} {
  global ApplicationInfo tcl_platform GUI_Options
  if {!$ApplicationInfo(EnableGUI)} {
    catch {puts stderr $Message}
    return
  }
  ## Try to find a window to act as a parent, as "." is "hidden" to
  ## the top-left corner of the screen...
  if {![string length $win]} {catch {set win [focus]}}
  if {[string length $win] && [winfo exists $win]} {
    set parent [winfo toplevel $win]
  } else {
    if {[winfo exists .mw]} {set parent .mw} else {set parent .}
  }

  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }
  
  switch -exact $tcl_platform(platform) {
    unix {
      if {[catch {.mw cget -bg} bg]} {
        set bg $GUI_Options(Background)
      }
      set res [MessageDlg .mw_mesgDlg -title $title -message $message \
          -parent $parent -type ok -icon info -justify left -bg $bg]
      return 
    }
    default {
      catch {$GUI_Options(AgentID) play Explain}
      return [tk_messageBox -title $title -message $message -parent $parent \
             -type ok -icon info]
    }
  }
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
};# gut_info

## gut_SelectDirectory: Select a directory...
proc gut_SelectDirectory {{dir {}}} {
  global GUI_Text
  if {![string length $dir]} {set dir [pwd]}
  tk_getDirectory -initialdir $dir -ok $GUI_Text(Ok) \
      -cancel $GUI_Text(Cancel) -open $GUI_Text(Open) -msg1 \
      $GUI_Text(Directory:) -msg2 $GUI_Text(DirectoryName:)
};# gut_SelectDirectory

## gut_many_errors
#  Display a dialog that contains accumulated many errors. This dialog is
#  modal, i.e. the user has to close it in order to continue.
proc gut_many_errors {Title Message Text} {
  global ApplicationInfo GUI_Options GUI_Text
  if {!$ApplicationInfo(EnableGUI)} {
    catch {puts stderr $Text}
    return
  }
  catch {$GUI_Options(AgentID) play Announce
         $GUI_Options(AgentID) play Confused}
  set win [ggi_unused_win .many_errors_viewer] 
  LDM_DeleteSplashWindow
  ## Perhaps a grab has been set that will stop interaction with this dialog?
  set grabbedWindows [grab current]
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab release $window}
  }
  toplevel $win
  wm withdraw $win
  wm title $win $Title
  #bind $win <Visibility> "raise $win"

  ## Display the Message:
  ttk::label $win.l -text $Message -foreground red \
    -font {helvetica 14 bold}
  pack $win.l -side top -pady 5
  ## Create a Text Widget
  ::ScrolledWidgets::ScrolledText $win.f -width 40 -height 10
  set Text [join [join $Text "\n"] "\n\n"]
  $win.f.text insert 1.0 $Text
  $win.f.text configure -state disabled
  pack $win.f -fill both -padx 2 -pady 2 -expand 1
  ttk::button $win.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
    -default active
  pack $win.ok -pady 20
  bind $win.ok <Return> {%W invoke}
  bind $win    <Return> "$win.ok invoke"
  gut_ModalDlg $win
  wm deiconify $win
  focus $win.ok
  tkwait window $win
  if {[llength $grabbedWindows]} {
    foreach window $grabbedWindows {grab set $window}
  }
};# gut_many_errors

## gut_Debug
#   Report debug info, only if debugging is enabled...
proc gut_Debug {info} {
  puts "Debug: $info"
}

## gut_MakeWindowBusy
proc gut_MakeWindowBusy {win {window {}}} {
  if {![winfo exists $win]} {return}
  if {[string length $window] == 0} {
    if {![string equal $win .]} {
      set window [ggi_unused_win $win._block_window_events]
    } else {
      set window [ggi_unused_win ._block_window_events]
    }
  } else {
    catch {destroy $window}
  }
  set width [winfo width $win]; set height [winfo height $win]
  frame $window -bg {} -cursor watch
  if {[string compare $win [winfo toplevel $win]] == 0} {
    ## A toplevel window
    set x 0; set y 0
  } else {
    set x [winfo x $win]; set y [winfo y $win]
  }
  update
  place $window -x $x -y $y -relwidth 1.0 -relheight 1.0
  update
  bind $window <Expose> "gut_MakeWindowBusy $win $window"
  return $window
}

## gut_Console
#   Open the a TkCon Console
proc gut_Console {} {
  global tcl_platform GUI_Options
  catch {$GUI_Options(AgentID) play Greet}
  ## Initialise only if we haven't yet
  ##
  if {![info exists ::tkcon::PRIV(root)]} {
    global HomeDir ApplicationName argv argc
    ## Set some tkcon's default values to match our look'n'feel...
    namespace eval ::tkcon {}
    #set ::tkcon::COLOR(bg) $GUI_Options(TextBg)
    #set ::tkcon::OPT(cols) $GUI_Options(TextWidth)
    #set ::tkcon::OPT(rows) $GUI_Options(TextHeight)
    #set ::tkcon::OPT(font) TextFont
    #set ::tkcon::OPT(exec) {}
    #set ::tkcon::OPT(showstatusbar) 1
    set ::tkcon::OPT(usehistory)    0
    #uplevel \#0 source [list [file join $HomeDir GUI Widgets TkCon.tcl]]
    ::tkcon::Init -color-bg $GUI_Options(TextBg) -font TextFont \
                  -exec {} -root .tkcon
    
    gut_ConsoleConfigure {} title "$ApplicationName Main Console..."
    gut_ConsoleConfigure {} attach Main
    wm protocol .tkcon WM_DELETE_WINDOW {gut_ConsoleConfigure {} hide}
    catch {
      load [file join $HomeDir CDM libWinIco.dll]
      set icon [winico load ConsoleWindowIcon \
               [file join $HomeDir CDM libCDM.dll]]
      after idle "winico setwindow .tkcon $icon"
      bind .tkcon <Destroy> "+winico delete $icon"
    }
    ## In the main console, we bind our own text popup...
    bind .tkcon <<TkCon_Popup>> {}
    return {}
  } else {
    if {[winfo exists .tkcon] && [string equal [wm state .tkcon] withdrawn]} {
      wm deiconify .tkcon
      #tkcon show
      return {}
    } else {
      set interpreter [uplevel \#0 tkcon new]
      gut_ConsoleConfigure $interpreter eval wm protocol . WM_DELETE_WINDOW \
                              [list tkcon destroy]
      gut_ConsoleConfigure $interpreter attach Main
      set font [$interpreter eval font create \
                 [font actual TextFont]]
      gut_ConsoleConfigure $interpreter console configure \
        -bg $GUI_Options(TextBg) -font $font \
        -width $GUI_Options(TextWidth) -height $GUI_Options(TextHeight)
      return $interpreter
    }
  }
}

## gut_ConsoleConfigure
proc gut_ConsoleConfigure {interpeter args} {
  if {![string length $interpeter]} {
    return [eval tkcon $args]
  } else {
    return [$interpeter eval tkcon $args]
  }
}

## gut_SplitLine
#
proc gut_SplitLine {list {char { }} } {
  set Data [split $list $char]
  set Clear {}
  foreach element $Data {
    if {[string length $element] && ![string equal $element { }]} {
     lappend Clear $element
    }
  }
  return $Clear
}

## gut_SetKeyboardLocale
#   Set Application's Keyboard Locale
proc gut_SetKeyboardLocale {locale} {
  set ::KeyboardLocale $locale
  catch {encoding system $locale}
}

## gut_FindCharacterOffset
#   This function returns the character offset given a text widget and a
#   valid text index in that widget...
proc gut_FindCharacterOffset {text_obj index} {
  ## Well, the answer is obvious. No need for time consuming calculations...
  return [string length [$text_obj get 1.0 $index]]
  #set start_ind [$text_obj search -exact \n 1.0]
  #if {[lindex [split $start_ind .] 0] == [lindex [split $index .] 0]} {
  #    return [lindex [split $index .] 1]
  #}

  #if {[string length $start_ind]} {
  #   set count [lindex [split $start_ind .] 1]
  #   incr count
  #   set ind [$text_obj search -exact \n "$start_ind + 1 char"]
  #   while {($ind != $start_ind) && [lindex [split $ind .] 0] != \
  #          [lindex [split $index .] 0]} {
  #      incr count [lindex [split $ind .] 1]
  #      incr count
  #      set ind [$text_obj search -exact \n "$ind + 1 char"]
  #   }
  #   incr count [lindex [split $index .] 1]
  #} else {
  #   return 0
  #}
  #return $count
}; # gut_FindByteOffset

## gut_RegisterTool
#   Registers a Plugin Module as a Tool...
proc gut_RegisterTool {module {class {}}} {
  global creole_config GUI_ToolConfig
  set GUI_ToolConfig(${class}::$module) $creole_config($module)
  puts -nonewline { [Tool]}
}; # gut_RegisterTool

## gut_GetCollectionDocuments
#    Returns all document names of the given collection name
proc gut_GetCollectionDocuments {col} {
  if {![string length $col]} {return {}}
  global CDM_CollectionFilename
  set file  [file join $col $CDM_CollectionFilename]
  set file2 [file join $col CDM_CollectionType.txt]
  set file3 [file join $col GuT_mAgIc]
  if {[file readable $file]} {
    return [CDM_GetCollectionFileIDs $file]
  }
  if {[file readable $file2]} {
    set TypeFile [open $file2]
    gets $TypeFile type
    close $TypeFile
    return [CDM_GetCollectionFileIDs $file2 [string trim $type]]
  }
  if {[file readable $file3]} {
    return [CDM_GetCollectionFileIDs $file3 gate]
  }
  foreach c [GUI_GetCollections] {
    if {[lindex $c 0] == $col} {
      foreach d [lindex [lindex $c 1] 0] {
        lappend names [lindex $d 0]
      }
      return $names
    }
  }
  return {}
}

## gut_lappend_uniq
#    Appends item to list, only if it doesn't exist...
proc gut_lappend_uniq {list args} {
  upvar $list List
  eval CDM_LappendUnique List $args
}

########################################################################
####### Validating Input Widgets                              ##########
########################################################################
## gut_Ventry
proc gut_Ventry {widget args} {
  set command ".\n"
  set entry_args {}; set mode default; set Keysyms {}
  foreach {op arg} $args {
    switch -glob -- $op {
      -allow  -
      -permit {set command $arg; set mode permit}
      -reject -
      -regexp {set command $arg; set mode default}
      -keys*  {set Keysyms $arg}
      default {lappend entry_args $op $arg}
    }
  }
  ttk::entry $widget {*}$entry_args
  bind $widget <Key> \
    "gut_ValidateVentry %W \{$command\} \{$Keysyms\} %A %K $mode"
  return $widget
};# gut_Ventry
proc gut_ValidateVentry {widget List Keysyms Char KeySym Mode} {
  if {[string compare $Mode default] == 0} {
    if {[regexp -- "$List" $Char]} {
      bell
      return -code break
    }
    foreach keysym $Keysyms {
      if {[string match $keysym $KeySym]} {
        bell
        return -code break
      }
    }
  } else {
    switch -glob $KeySym {
      BackSpace - Delete - Left - Right - Home - End - 
      Alt* - Shift* - Control* - Caps_Lock - Insert -
      Return {return {}}
      default {
        if {![regexp -- "$List" $Char]} {
          foreach keysym $Keysyms {
            if {[string match $keysym $KeySym]} {return {}}
          }
          bell
          ## puts $KeySym
          return -code break
        }
      }
    }
  }
  return {}
}


########################################################################
####### Execution Server                                      ##########
#######   Added by G. Petasis, 30/10/1998                     ##########
########################################################################

proc gut_EXEC {args} {
  global Babel_ExecServer Babel_ExecServerDataAvailable TIPSTER_Time

  set Babel_ExecServerDataAvailable 0
  set CurrentTime [clock seconds]
  fileevent $Babel_ExecServer readable \
  {if {![eof $Babel_ExecServer]} {
    set Babel_ExecServerDataAvailable 1
  }}
  ## Set the same environments between the two processes...
  ## Note that we have to pass all the commands in a single puts statement...
  puts $Babel_ExecServer \
    "array set ::env \[list [array get ::env]\];cd [list [pwd]]; exec $args"
  flush $Babel_ExecServer
  while {$Babel_ExecServerDataAvailable == 0} {
    vwait Babel_ExecServerDataAvailable
  }
  fileevent $Babel_ExecServer readable {}
  gets $Babel_ExecServer data
  set data [split $data ":"]
  set status [lindex $data 0]
  set lines [lindex $data 1]
  set cmd_out {}
  for {set i 0} {$i < $lines} {incr i} {
      lappend cmd_out [gets $Babel_ExecServer]
  }
  if {[llength $cmd_out]} {
    set output [join $cmd_out "\n"]
  } else {
    set output {}
  }
  set Now [clock seconds]
  append TIPSTER_Time [format "\t %d seconds executing external command" \
     [expr $Now - $CurrentTime]]
  append TIPSTER_Time " ($args).\n"
  if {[string equal $status Tcl_Ok]} {
    return $output
  } else {
    set end [expr {[llength $cmd_out] -2}]
    set output [lrange $cmd_out 0 $end]
    set output [join $output "\n"]
    set Code [split [lindex $cmd_out end] { }]
    error $output {} $Code
  }
}
  
## gut_InitExecServer
#   This procedure initializes the execution server: another process that
#   would fork and execute requests with exec. It redifines the "cd" and
#   "exec" commands...
proc gut_InitExecServer {} {
  global HomeDir Babel_ExecServer SplashStatus
  
  ## Start Execution Server
  set server [string map {{ } {\ }} "|[file join $HomeDir ExecServer.exe]"]
  if {[catch {open $server w+} Babel_ExecServer]} {
    ## We couldn't execute ExecServer as a single executable. Can we execute
    ## it at least as a script?
    set tclsh [string map {{ } {\ } wish tclsh WISH TCLSH} \
              [info nameofexecutable]]
    set server "|$tclsh \"[file join $HomeDir ExecServer.tcl]\""
    if {[catch {open $server w+} Babel_ExecServer]} {
      set message "Failed in Starting Execution Server..."
      set SplashStatus $message
      puts "$message\nError was: $Babel_ExecServer"
      return
    }
  }

  ## Print "Hello" from Execution Server
  set message [gets $Babel_ExecServer]
  set SplashStatus $message
  puts $message
  ## fconfigure $Babel_ExecServer -blocking 0
  ## fileevent $Babel_ExecServer readable \
  ## "if {![eof $Babel_ExecServer]} {
  ##   set Babel_ExecServerDataAvailable 1
  ## }"
  
  ## Make required procedure renames...
  rename exec gut_exec_original
  rename gut_EXEC exec
}
  
##############################################################################
### Usefull Dialogs                                                        ###
### G. Petasis...                                                          ###
##############################################################################

## gut_ModalDlg
proc gut_ModalDlg {win {req_x 0} {req_y 0}} {
  update
  set sw [winfo screenwidth  .]
  set sh [winfo screenheight .]
  if {$req_x && $req_y} {
    set x [expr {($sw - $req_x)/2}]
    set y [expr {($sh - $req_y)/2}]
    wm geometry $win ${req_x}x$req_y+${x}+$y
  } else {
    set x [expr {($sw - [winfo reqwidth $win])/2}]
    set y [expr {($sh - [winfo reqheight $win])/2}]
    if {$x < 10 && $y < 10} {
      set x [expr {($sw - [winfo width $win])/2}]
      set y [expr {($sh - [winfo height $win])/2}]
    }
    if {$x < 10 && $y < 10} {
      set x [expr {$sw / 2}]
      set y [expr {$sh / 2}]
    }
    wm geometry $win +${x}+$y
  }
  update
  #bind $win <Visibility> "raise $win"
}

## gut_EntryDialog
proc gut_EntryDialog {args} {
  global GUI_Options
  catch {$GUI_Options(AgentID) play Write
         $GUI_Options(AgentID) play WriteContinued
         $GUI_Options(AgentID) play WriteReturn}
  foreach {Message Message2 DefaultValue DefaultValue2 win command Title} \
          {{} {} {} {} {} {} {Entry Dialog}} {break}
  foreach {Type1 Type2 Vals1 Vals2} {gut_Ventry gut_Ventry {} {}} {break}
  foreach {op arg} $args {
    switch -glob -- $op {
      -message  {set Message       $arg}
      -message2 {set Message2      $arg}
      -type     {set Type1         $arg}
      -type2    {set Type2         $arg}
      -win      {set win           $arg}
      -tit*     {set Title         $arg}
      -ini*     {set DefaultValue  $arg}
      -default2 {set DefaultValue2 $arg}
      -values   {set Vals1         $arg}
      -values2  {set Vals2         $arg}
      default   {lappend command $op $arg}
    }
  }
    
  if {[string length $win]} {
    if {[winfo exists $win]} {raise $win; return {}}
  } else {
    set win [ggi_unused_win .gut_entry_dlg]
  }
  global gut_EntryDialog; set gut_EntryDialog(1:$win) {}
  toplevel $win
  wm title $win $Title
  wm withdraw $win
  frame $win.f -relief groove -borderwidth 2
  frame $win.bf -relief flat -borderwidth 0
  label $win.f.label -text $Message -anchor w
  if {[string equal $Type1 gut_Ventry]} {
    eval gut_Ventry $win.f.name $command
    $win.f.name insert end $DefaultValue
    set EntryWidget1 $win.f.name
  } else {
    $Type1 $win.f.name -values [concat $DefaultValue $Vals1] -editable 1
    $win.f.name setvalue first
    set EntryWidget1 $win.f.name.e
  }
  grid $win.f.label $win.f.name -sticky ew -padx 2
  set EntryWidget2 $win.f.name2
  if {[string length $Message2]} {
    set gut_EntryDialog(2:$win) {}
    label $win.f.label2 -text $Message2 -anchor w
    if {[string equal $Type2 gut_Ventry]} {
      eval gut_Ventry $win.f.name2 $command
      $win.f.name2 insert end $DefaultValue2
    } else {
      $Type2 $win.f.name2 -values [concat $DefaultValue2 $Vals2] -editable 1
      $win.f.name2 setvalue first
      set EntryWidget2 $win.f.name2.e
    }
    grid $win.f.label2 $win.f.name2 -sticky ew -padx 2
  }
  grid columnconfigure $win.f 1 -weight 1
  pack $win.f -ipadx 10 -ipady 10 -padx 2 -pady 2 -fill both -expand 1
  button $win.bf.ok -textvariable GUI_Text(Ok) -default active -command \
    "set gut_EntryDialog(1:$win) \[$EntryWidget1 get\]
     catch {set gut_EntryDialog(2:$win) \[$EntryWidget2 get\]};destroy $win"
  button $win.bf.cancel -textvariable GUI_Text(Cancel) -command \
    "set gut_EntryDialog(1:$win) {};destroy $win"
  pack $win.bf.ok $win.bf.cancel -side left -padx 5 -pady 6
  pack $win.bf 
  bind $win <Return> "$win.bf.ok invoke"
  update
  gut_ModalDlg $win
  focus $win.f.name
  wm deiconify $win
  grab set $win
  tkwait window $win
  grab release $win
  if {[string length $Message2]} {
    global gut_EntryDlg2$win
    set result [list [set gut_EntryDialog(1:$win)] \
                     [set gut_EntryDialog(2:$win)]]
    array unset gut_EntryDialog 2:$win
  } else {
    set result [set gut_EntryDialog(1:$win)]
  }
  array unset gut_EntryDialog 1:$win
  return $result
};# gut_EntryDialog

## gut_MultipleSelectFromList
#   Displays a ListBox, where the user can select many entries...
proc gut_MultipleSelectFromList {List {Text {}} {Title {Select Items...}} \
                                  {width 30} {win {}} {embed_win {}} } {
  global GUI_Options
  catch {$GUI_Options(AgentID) play Uncertain}
  if {[string length $win]} {
    if {[winfo exists $win]} {raise $win; return {}}
  } else {
    set win [ggi_unused_win .gut_multilistselect_dlg]
  }
  toplevel $win
  wm withdraw $win
  wm title $win $Title
  frame $win.f -relief flat
  label $win.f.label -text $Text
  ::ScrolledWidgets::ScrolledListbox $win.f.name -width $width  \
     -foreground navy -selectmode extended -setgrid 1 -bd 1
  eval $win.f.name.listbox insert end $List
  bind $win.f.name.listbox <Double-1> "$win.buttons.ok invoke"
  pack $win.f.label -fill x -padx 4 -pady 4
  pack $win.f.name -fill both -expand 1
  pack $win.f -ipadx 10 -ipady 10 -padx 5 -pady 5 -fill both -expand 1
  if {[string length $embed_win] && [winfo exists $embed_win]} {
    pack $embed_win -fill both -expand 1
  }
  frame $win.buttons -bd 0
    button $win.buttons.ok -textvariable GUI_Text(Ok) -default active -command \
       "set gut_MultipleSelectFromList$win $win"
    button $win.buttons.cancel -textvariable GUI_Text(Cancel) -command \
       "set gut_MultipleSelectFromList$win -1Cancel0"
    pack $win.buttons.ok $win.buttons.cancel -padx 10 -pady 10 -side left
  pack $win.buttons
  bind $win <Return> "$win.buttons.ok invoke"
  wm protocol $win WM_DELETE_WINDOW \
     "set gut_MultipleSelectFromList$win -1Cancel0"
  global gut_MultipleSelectFromList$win
  gut_ModalDlg $win
  wm deiconify $win
  update
  #raise $win;# Sometimes, for no reason this dialog "hides" beneath others...
  tkwait variable gut_MultipleSelectFromList$win
  if {[string equal [set gut_MultipleSelectFromList$win] -1Cancel0]} {
    destroy $win
    gut_error <ignore> 
  }
  set result {}
  for {set i 0} {$i < [llength $List]} {incr i} {
    if {[${win}.f.name.listbox selection includes $i]} {
      lappend result [lindex $List $i]
    }
  }
  destroy $win
  unset gut_MultipleSelectFromList$win
  return $result
};# gut_MultipleSelectFromList

## gut_ShowTooltip
proc gut_ShowTooltip {Widget Text {Win {.gut_showtooltip}} {x {}} {y {}}} {
  if {![string length $Win]} {set Win .gut_showtooltip}
  if {[winfo exists $Win]} {destroy $Win}
  if {![string length $x]} {
    set x [expr {[winfo rootx $Widget] + ([winfo width $Widget]/2)}]
  }
  if {![string length $y]} {
    set y [expr {[winfo rooty $Widget] + [winfo height $Widget] + 4}]
  }
  toplevel $Win -bg black
  wm overrideredirect $Win 1
  label ${Win}.l -text $Text -relief flat -bg #ffffaa -fg black \
     -padx 2 -pady 0 -anchor w
  pack ${Win}.l -side left -padx 1 -pady 1
  wm geometry $Win +${x}+${y}
  return $Win
};# gut_ShowTooltip

## gut_UTF8ToHex
#  This function will convert a UTF-8 string to an equivalent string that will
#  contain only ASCII characters. All Unicode characters will be encoded as a
#  hexadecimal code...
proc gut_UTF8ToHex {string} {
  set result {}
  foreach ch [split $string {}] {
    scan $ch %c value
    if {$value < 129} {
      append result $ch
    } else {
      append result "\\u[format %04x $value]"
    }
  }
  return $result
};# gut_UTF8ToHex

## gut_UTF8ToOctal
#  This function will convert a UTF-8 string to an equivalent string that will
#  contain only ASCII characters. All Unicode characters will be encoded as an
#  octal code...
proc gut_UTF8ToOctal {string} {
  set result {}
  foreach ch [split $string {}] {
    scan $ch %c value
    if {$value < 129} {
      append result $ch
    } else {
      append result "\\[format %o $value]"
    }
  }
  return $result
};# gut_UTF8ToOctal

## gut_HexToUTF8
#  This function will convert a string encoded by gut_UTF8ToHex back to the
#  original UTF-8 string.
proc gut_HexToUTF8 {string} {
  subst -novariables -nocommands $string
};# gut_HexToUTF8

proc gut_CharacterStringClass {string} {
  set result {}
  foreach ch [split $string {}] {
    set classes {}
    foreach class {alnum alpha ascii boolean control digit double false graph
                   integer lower print punct space true upper wordchar xdigit} {
      if {[string is $class $ch]} {
        lappend classes $class
      }
    }
    lappend result [list $ch $classes]
  }
  return $result
};# gut_CharacterStringClass
