#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  CheckDependencies.tcl:                                                  ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
#if {$argc == 0} {
#  puts stderr "$argv0: This script checks the dependencies of shared libraries"
#  puts stderr "  Usage: $argv0 file-or-dir1 \[file-or-dir2 ...\]"
#  exit -1
#}

set HomeDir [exec ellogon -printHomeDir]
set OS [exec uname]
set CDM $HomeDir/CDM/$OS
append env(LD_LIBRARY_PATH) ":$CDM"
append env(LD_PRELOAD) $CDM/libCDM.so

proc CheckFileLinux {file} {
  ## Skip some files with known problems...
  switch -glob [file tail $file] {
    setuid.so - utime.so - crypt.so - limit.so -
    CORE.so - Redland.so -
    libTclHtml.so {return}
  }
  catch {exec chmod 755 $file}
  puts "-----------------------------------------------------"
  puts "  Library: [file tail $file]"
  puts "-----------------------------------------------------"
  ## Execute ldd and check the returned dependencies...
  if {[catch {exec ldd -d -r $file} result]} {
    puts "ldd Output:\n$result"
  }
  #puts "  ($file)"
  ## Perform simple checks on the output...
  if {[string match "undefined symbol: *OCDM_*" $result]} {
    ## Undefined OCDM objects:
    puts "OCDM References: FAIL"
  } else {
    puts "OCDM References: Ok"
  }
  if {[string match "undefined symbol: *CDM_*" $result]} {
    ## Undefined OCDM objects:
    puts "CDM  References: FAIL"
  } else {
    puts "CDM  References: Ok"
  }
  if {[string match "*undefined symbol: *Tcl_*" $result]} {
    ## Undefined OCDM objects:
    puts "Tcl  References: FAIL"
  } else {
    puts "Tcl  References: Ok"
  }
  puts -nonewline "Tcl  Libraries:  "
  set found 0
  foreach Major {8 9} {
    foreach Minor {0 1 2 3 4  5 6 7 8 9} {
      if {[string match "*libtcl$Major.$Minor.so =>*" $result]} {
        ## Undefined OCDM objects:
        puts "libtcl$Major.$Minor.so "
        set found 1
      }
    }
  }
  if {!$found} {puts "Ok"}
  puts -nonewline "Tk   Libraries:  "
  set found 0
  foreach Major {8 9} {
    foreach Minor {0 1 2 3 4  5 6 7 8 9} {
      if {[string match "*libtk$Major.$Minor.so =>*" $result]} {
        ## Undefined OCDM objects:
        puts "libtk$Major.$Minor.so "
        set found 1
      }
    }
  }
  if {!$found} {puts "Ok"}
}

proc CheckDir {dir} {
  exec chmod 755 $dir
  if {[catch {glob [file join $dir *] [file join $dir .*]} files_dirs]} return
  foreach file_dir $files_dirs {
    if {[string equal [file tail $file_dir] .] ||
        [string equal [file tail $file_dir] ..] ||
        [string equal [file tail $file_dir] .svn]} {
      continue
    }
    if {[file isfile $file_dir]} {
      if {[string match *Linux/GCC2x/*.so $file_dir]} {
      } elseif {[string match *Linux/*.so $file_dir]} {
        CheckFileLinux $file_dir
      }
    } elseif {[file isdirectory $file_dir]} {
      CheckDir $file_dir
    }
  }
};# CheckDir

if {$argc == 0} {
  set argv [list $HomeDir]
}

foreach file_dir $argv {
  if {[file isfile $file_dir]} {
    CheckFile $file_dir
  } elseif {[file isdirectory $file_dir]} {
    CheckDir $file_dir
  }
}
