#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  FixPermitions.tcl:                                                      ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
if {$argc == 0} {
  puts stderr "$argv0: This script resets the permitions of a file"
  puts stderr "                    or directory to more sane values..."
  puts stderr "  Usage: $argv0 file-or-dir1 \[file-or-dir2 ...\]"
  exit -1
}

proc FixFile {file} {
  ## We have to decide on the type of the file...

  ## First look extension.
  switch -glob -- [string tolower [file extension $file]] {
    .rpm         -
    .rtf         -
    .doc         -
    .c           -
    .cc          -
    .cpp         -
    .c++         -
    .lex         -
    .class       -
    .java        -
    .rc          -
    .h           -
    .tek         -
    .txt         -
    .doc         -
    .pdf         -
    .ps          -
    .test        -
    .log         -
    .frx         -
    .trm         -
    .m4          -
    .macros      -
    .in          -
    .n           -
    .db          -
    .xbm         -
    .xpm         -
    .ppm         -
    .gif         -
    .jpg         -
    .jpeg        -
    .png         -
    .bmp         -
    .ico         -
    .cur         -
    .cdr         -
    .fig         -
    .ncb         -
    .opt         -
    .tdb         -
    .idx         -
    .cpt         -
    .tar         -
    .zip         -
    .gz          -
    .tgz         -
    .xml         -
    .sgml        -
    .mdb         -
    .dtd         -
    .grm         -
    .acs         -
    .css         -
    .wc          -
    .mcp         -
    .py          -
    .pyc         -
    .pl          -
    .*htm        -
    .html {exec chmod 644 $file; return}

    .so   -
    .dll  -
    .a    -
    .o    -
    .aps  -
    .tlb  -
    .rgs  -
    .jar  -
    .def  -
    .lib  {exec chmod 644 $file; return}

    .exe  {exec chmod 755 $file; return}
    .swp  {}
    default {
      if {[string match *.so.* $file]} {
        exec chmod 644 $file; return
      }
    }
  }

  ## Is it a known file format?
  switch -glob [string tolower [file tail $file]] {
    makefile        -
    makefile.vc     -
    *.mak           -
    makefile*       -
    entries         -
    repository      -
    root            -
    license.terms   -
    license*        -
    changelog       -
    changes*        -
    copying*        -
    copyright*      -
    tclindex        -
    readme.cygwin   -
    index           -
    readme*          {exec chmod 644 $file; return}

    ellogon         -
    configure       -
    install-sh      -
    mkinstalldirs   {exec chmod 755 $file; return}
    default {}
  }

  ## Check the possibility to be a script...
  set f [open $file]
  set line [gets $f]
  close $f
  if {[string match {#!*} $line]} {
    ## It is a script!
    exec chmod 755 $file
    return
  }

  ## Does it have a known script ending?
  switch -exact [string tolower [file extension $file]] {
    .tcl   -
    .ksh   -
    .sh    -
    .bat   -
    .prl   -
    .perl  {exec chmod 644 $file; return}
    default {}
  }

  ## Try to exec file...
  if {![catch {exec file $file} response]} {
    if {[string match "* image*" $response]} {exec chmod 644 $file;return}
    if {[string match "* text*" $response]}  {exec chmod 644 $file;return}
    if {[string match "* executable*" $response]} {exec chmod 755 $file;return}
  }

  puts stderr "Unknown file type: $file"
}

proc FixDir {dir} {
  exec chmod 755 $dir
  if {[catch {glob [file join $dir *] [file join $dir .*]} files_dirs]} return
  foreach file_dir $files_dirs {
    if {[string equal [file tail $file_dir] .]  ||
        [string equal [file tail $file_dir] ..] ||
        [string equal [file tail $file_dir] .svn]} {
      continue
    }
    if {[file isfile $file_dir]} {
      FixFile $file_dir
    } elseif {[file isdirectory $file_dir]} {
      FixDir $file_dir
    }
  }
};# FixDir

foreach file_dir $argv {
  if {[file isfile $file_dir]} {
    FixFile $file_dir
  } elseif {[file isdirectory $file_dir]} {
    FixDir $file_dir
  }
}
