#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  FixWindowsFiles.tcl:                                                    ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
if {$argc == 0} {
  puts stderr "$argv0: This script converts the files in a"
  puts stderr "                    directory to unix format..."
  puts stderr "  Usage: $argv0 file-or-dir1"
  exit -1
}

proc RewriteFile {file} {
  set f [open $file]
  set data [read $f]
  close $f
  set f [open $file w]
  puts -nonewline $f $data
  close $f
}

proc FixFile {file} {
  ## We have to decide on the type of the file...

  ## First look extension.
  switch -exact [string tolower [file extension $file]] {
    .c    -
    .cc   -
    .cpp  -
    .c++  -
    .tcl   -
    .ksh   -
    .sh    -
    .bat   -
    .prl   -
    .perl  -
    .lex  -
    .java  -
    .h    -
    .decls -
    .txt  -
    .test -
    .log  -
    .msg  -
    .m4   -
    .macros -
    .in   -
    .n    -
    .rc   -
    .xbm  -
    .xpm  -
    .ps   -
    .eps  -
    .bmp  -
    .manifest -
    .pch  -
    .xml  -
    .sgml -
    .spec -
    .file1 -
    .file2 -
    .dtd  -
    .grm  -
    .vc   -
    .1    -
    .3    -
    .r    -
    .htm  -
    .html {RewriteFile $file; return}
    .dll      -
    .exe      -
    .lib      -
    .obj      -
    .cur      -
    .ppm      -
    .gif      -
    .png      -
    .jpg      -
    .mpg      -
    .mpeg     -
    .avi      -
    .mp3      -
    .ico      -
    .res      -
    .pdb      -
    .ilk      -
    .doc      -
    .hqx      -
    .pbxuser  -
    .pbxproj  -
    .icns     -
    .bat      {return}
    default {}
  }

  ## Is it a known file format?
  switch -glob [string tolower [file tail $file]] {
    makefile        -
    makefile.vc     -
    *.mak           -
    makefile*       -
    mklinks         -
    entries         -
    repository      -
    root            -
    license.terms   -
    license*        -
    changelog       -
    changes*        -
    copying*        -
    copyright*      -
    browse          -
    hello           -
    ixset           -
    rmt             -
    rolodex         -
    square          -
    tcolor          -
    timer           -
    widget          -
    mkLinks         -
    tclindex        -
    readme.cygwin   -
    index           -
    readme*         -
    ellogon         -
    configure       -
    install-sh      -
    mkinstalldirs   {RewriteFile $file; return}
    default {}
  }

  puts stderr "Unknown file type: $file"
}

proc FixDir {dir} {
  exec chmod 755 $dir
  if {[catch {glob [file join $dir *] [file join $dir .*]} files_dirs]} return
  foreach file_dir $files_dirs {
    if {[string equal [file tail $file_dir] .]  ||
        [string equal [file tail $file_dir] ..] ||
        [string equal [file tail $file_dir] .svn]} {
      continue
    }
    if {[file isfile $file_dir]} {
      FixFile $file_dir
    } elseif {[file isdirectory $file_dir]} {
      FixDir $file_dir
    }
  }
};# FixDir

foreach file_dir $argv {
  if {[file isfile $file_dir]} {
    FixFile $file_dir
  } elseif {[file isdirectory $file_dir]} {
    FixDir $file_dir
  }
}
