#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  GenerateCollectionIndex.tcl:                                            ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
if {$argc == 0} {
  puts stderr "$argv0: This script will convert a directory containing"
  puts stderr "\t\t\t     some Ellogon Documents into a Collection..."
  puts stderr "  Usage: $argv0 directory"
  exit -1
}

## This function returns true if the file is an Ellogon Document...
proc EllogonDocument {filename} {
  set file [open $filename]
  gets $file line
  close $file
  string equal [string trim $line] \
               "#Ellogon: Document File"
};# EllogonDocument

proc GetDocumentEntry {entry filename} {
  set file [open $filename]
  fconfigure $file -encoding utf-8 -translation lf
  while {[gets $file line] != -1} {
    if {[string match ${entry}::* $line]} {
      close $file
      return [string range [string trim $line] \
                     [expr {[string length $entry]+2}] end]
    }
  }
  close $file
  return {}
};# GetDocumentEntry

## The following function is responsible for generating the Collection index
## file.
proc GenerateIndex {dir} {
  puts " -- Processing directory \"$dir\"..."
  ## Collect all the files in the directory...
  set files [glob -directory $dir -nocomplain -type f *]
  set docs   {}
  set docids {}
  foreach file $files {
    ## Is it an Ellogon Document?
    if {![EllogonDocument $file]} {
      puts "   ** Skipping file: \"$file\"..."
      continue
    }
    puts "   ++ Adding file: \"$file\"..."
    set nameTail   [file tail $file]
    lappend docs   [GetDocumentEntry ExternalID $file]
    set DocumentId [GetDocumentEntry ID         $file]
    if {![string equal $nameTail $DocumentId]} {
      set DocumentId $nameTail
      puts "   ++ Changing the Id of the Document to \"$DocumentId\"..."
    }
    set index 0; set DocumentIdRoot $DocumentId
    while {[info exists DocumentIds($DocumentId)]} {
      set DocumentId $DocumentIdRoot_[incr index]
    }
    set DocumentIds($DocumentId) 1
    lappend docids $DocumentId
  }
  if {![llength $docs]} {
    puts "   << No Ellogon Documents found!"
    return
  }
  
  ## Generate the index file...
  set file [open $dir/.CDM_CollectionIndex.txt w]
  fconfigure $file -encoding utf-8 -translation lf
  puts $file \
"#Ellogon: Collection File
# This is an Auto-generated File. Please DO NOT EDIT.
# Ellogon CDM version 1.0 - August 1999 - September 2002
#  This file is used by the \"Ellogon\" Text Engineering Platform,
#  and contains an \"Ellogon\" CDM Collection specification...
#  File Encoding: utf-8
Encoding::[encoding system]
FileFormat::5
Translation::lf
Name::$dir
Owner::CDM
DocumentIDs::[string length $docids]
$docids
DocumentXIDs::[string length $docs]
$docs
Attributes::0

## - End of File -"
close $file
};# GenerateIndex

foreach dir $argv {
  if {[file isdirectory $dir]} {
    GenerateIndex [file normalize $dir]
  }
}
