#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  GenerateMissingDocuments.tcl:                                           ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
if {$argc == 0} {
  puts stderr "$argv0: This script will create missing Documents"
  puts stderr "\t\t\t     into an Ellogon Collection..."
  puts stderr "  Usage: $argv0 directory"
  exit -1
}

proc ProcessIndex {dir} {
  set filename $dir/.CDM_CollectionIndex.txt
  set file [open $filename]
  fconfigure $file -encoding utf-8 -translation lf
  while {[gets $file line] != -1} {
    switch -glob -- $line {
      DocumentIDs::* {
        gets $file DocIDs
      }
      DocumentXIDs::* {
        gets $file DocXIDs
      }
      default {}
    }
  }
  close $file
  ## Ensure all Documents are in place...
  if {[llength $DocIDs] != [llength $DocXIDs]} {
    puts stderr "Corrupted Index File: IDs != XIDs!"
    exit -1
  }
  set missing 0
  foreach ID $DocIDs XID $DocXIDs {
    set file [file join $dir $ID]
    if {![file exists $file]} {
      incr missing
      puts "  Restoring missing Document ($missing): \"$ID\"..."
      RestoreDocument $file $ID $XID
    }
  }
};# ProcessIndex

proc RestoreDocument {filename ID XID} {
  set msg "Restored Document by the \"$::argv0\" Tool..."
  set file [open $filename w]
  puts $file \
"#Ellogon: Document File
# This is an Auto-generated File. Please DO NOT EDIT.
# Ellogon CDM version 1.0 - Summer 1999
#  This file is used by the \"Ellogon\" Text Engineering Platform,
#  and contains a \"Ellogon\" CDM Document specification...
#  File Encoding: utf-8
Encoding::utf-8
FileFormat::5
ExternalID::$XID
ID::$ID
AnnotationMaxID::0
Attributes::1
Restored_by {GDM_STRING {$::argv0}}
Annotations::0

RawData::[string length $msg]
$msg

## - End of File -
"
close $file
};# RestoreDocument

foreach dir $argv {
  if {[file isdirectory $dir]} {
    ProcessIndex [file normalize $dir]
  }
}
