#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  MergeDirectories.tcl:                                                   ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
if {$argc != 2 && $argc != 3} {
  puts stderr "$argv0: This script merges two directories by copying only"
  puts stderr "  newer files from the source-dir to target-dir..."
  puts stderr "  Usage: $argv0 source-dir target-dir2 ?check?"
  exit -1
}

if {$argc == 3} {
  set PerformCopy [lindex $argv 2]
} else {
  set PerformCopy 0
}

if {!$PerformCopy} {
  puts "## Only Checking: No modifications! ##"
}

proc CopyFile {source dest} {
  if {!$::PerformCopy} {return}
  if {[catch {file copy -force -- $source $dest} error]} {
    puts "Error: $error"
  }
}

proc UpdateFile {source dest} {
  ## Is it a "probelmatic" file?
  if {[string match *~* $source]} {
    puts "Attention: $source"
    return
  }
  ## Does the target file exists?
  if {![file exists $dest]} {
    puts "Creating: $dest"
    CopyFile $source $dest
    return
  }
  ## Is the target file older than the source one?
  set smtime [file mtime $source]
  set dmtime [file mtime $dest]
  if {$dmtime < $smtime} {
    puts "Updating: $dest"
    CopyFile $source $dest
  }
};# UpdateFile

proc UpdateDir {source dest} {
  ## Get the files of the source directory...
  if {[catch {glob -directory $source *} files_dirs]} {
    return
  }
  foreach file_dir $files_dirs {
    if {[string equal [file tail $file_dir] .] ||
        [string equal [file tail $file_dir] ..]} {
      continue
    }
    if {[file isfile $file_dir]} {
      UpdateFile $file_dir [file join $dest [file tail $file_dir]]
    } elseif {[file isdirectory $file_dir]} {
      file mkdir [file join $dest [file tail $file_dir]]
      exec chmod 755 [file join $dest [file tail $file_dir]]
      UpdateDir  $file_dir [file join $dest [file tail $file_dir]]
    }
  }
};# FixDir

foreach {source dest} $argv {break}
UpdateDir $source $dest
