#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  Statistics.tcl:                                                         ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

##########################################################################
## Statistics.tcl:
## A simple tcl script for calculating a few statistics on the Ellogon
## source code...
set HomeDir [file dirname [file normalize [info script]]]
cd $HomeDir
cd ../..

## Collect all Ellogon Tcl code...
set DirList [list \
  . \
  CDM/* \
  ellogon2.0/* \
  GUI \
  GUI/Dialogs \
  GUI/Widgets \
  Setup \
  Utils/* \
  Viewers/* \
  doc/* \
  share/* \
  lib/TclAgent/* \
  lib/tBuild/* \
  lib/tkdnd/* \
]

## Construct a list of all directories that will be searched for Tcl and C/C++
## files...

proc AppendDir {var dir} {
  global $var
  ## Has the dir a /* at the end? If yes, we have to also append its
  ## subdirs...
  if {[string equal "/*" [string range $dir end-1 end]]} {
    set dir [string range $dir 0 end-2]
    lappend $var $dir
    ## Get all subdirectories...
    if {[catch {glob -nocomplain -type d $dir/*} subdirs]} {return}
    foreach subdir $subdirs {
      AppendDir $var $subdir/*
    }
  } else {
    lappend $var $dir
  }
};# AppendDir

set Directories {}
foreach dir $DirList {
  ## If the directory ends in /*, we have to add all its subdirectories...
  AppendDir Directories $dir
}

proc GetDirStats {dir exts} {
  set file_count 0
  set line_count 0
  foreach ext $exts {
    ## Get all files in directory matching the extension...
    if {[catch {glob $dir/$ext} files]} {continue}
    foreach file $files {
      set count [exec wc $file]
      incr file_count
      incr line_count [lindex $count 0]
    }
  }
  return [list $file_count $line_count]
}

## Statistics about Tcl files...
set TclFiles 0
set TclLines 0
foreach dir $Directories {
  foreach {files lines} [GetDirStats $dir *.tcl] {}
  incr TclFiles $files
  incr TclLines $lines
}

puts "*** Tcl Statistics:"
puts "    Number of Files: $TclFiles"
puts "    Number of Lines: $TclLines"

## Statistics about C/C++ files...
set CppFiles 0
set CppLines 0
foreach dir $Directories {
  foreach {files lines} [GetDirStats $dir {*.c *.cpp *.lex}] {}
  incr CppFiles $files
  incr CppLines $lines
}

puts "*** C/C++ Statistics:"
puts "    Number of Files: $CppFiles"
puts "    Number of Lines: $CppLines"
