##############################################################################
##  TransferCollectionFiles.tcl:                                            ##
## --------------------------------                                         ##
##  This file implements a utility for transfering Documents between        ##
##  Collections.                                                            ##
##                                                                          ##
##  Copyright 1998-2007 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

#############################################################################
## A Small Utility for transfering Documents between Collections...        ##

proc CDMTools_TransferCollectionDocuments {{win .cdm_toolsTransferColDocs}} {
  global GUI_Text GUI_Options
  set win [ggi_unused_win $win]
  toplevel $win
  wm withdraw $win
  wm title $win {Transfering Documents between Collections...}

  ## Comboboxes of Collection Names...
  set CFrame $win.comboframe
  set LFrame $win.doclistboxes
  frame $CFrame
    set ColNames [GUI_GetCollectionNames]
    #label $CFrame.keylab -textvar GUI_Text(SourceCollection) -anchor w
    ComboBox $CFrame.keycol -editable 0 -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -values $ColNames \
      -modifycmd "CDMTools_TransferCollectionUpdateListboxes $CFrame.keycol \
      $LFrame.keydocs.listbox"
    #label $CFrame.reslab -textvar GUI_Text(TargetCollection) -anchor e
    ComboBox $CFrame.rescol -editable 0 -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -values $ColNames \
      -modifycmd "CDMTools_TransferCollectionUpdateListboxes $CFrame.rescol \
      $LFrame.resdocs.listbox"
    #grid $CFrame.keylab $CFrame.reslab -sticky snew -padx 2 -pady 2
    grid $CFrame.keycol $CFrame.rescol -sticky snew -padx 2 -pady 2
    grid columnconfigure $CFrame 0 -weight 1
    grid columnconfigure $CFrame 1 -weight 1
  pack $CFrame -fill x -expand 1 -padx 4

  ## Document Listboxes...
  frame $LFrame
    ::ScrolledWidgets::ScrolledListbox $LFrame.keydocs -height 20 -width 45 \
      -selectmode extended -bd $GUI_Options(BorderWidth)
    label $LFrame.keydocslen -text { }
    ::ScrolledWidgets::ScrolledListbox $LFrame.resdocs -height 20 -width 45 \
      -selectmode extended -bd $GUI_Options(BorderWidth)
    label $LFrame.resdocslen -text { }
    GUI_LoadImage NextNice
    GUI_LoadImage BackNice
    button $LFrame.lmbutton -image BackNice -textvariable GUI_Text(Move) \
      -compound l -bg red -fg white -command \
      "CDMTools_TransferCollection $CFrame.rescol $LFrame.resdocs.listbox \
                                   $CFrame.keycol $LFrame.keydocs.listbox move"
    button $LFrame.lcbutton -image BackNice -textvariable GUI_Text(Copy) \
      -compound l -bg orange -fg red -command \
      "CDMTools_TransferCollection $CFrame.rescol $LFrame.resdocs.listbox \
                                   $CFrame.keycol $LFrame.keydocs.listbox copy"
    button $LFrame.rmbutton -image NextNice -textvariable GUI_Text(Move) \
      -compound r -bg red -fg white -command \
      "CDMTools_TransferCollection $CFrame.keycol $LFrame.keydocs.listbox \
                                   $CFrame.rescol $LFrame.resdocs.listbox move"
    button $LFrame.rcbutton -image NextNice -textvariable GUI_Text(Copy) \
      -compound r -bg orange -fg red -command \
      "CDMTools_TransferCollection $CFrame.keycol $LFrame.keydocs.listbox \
                                   $CFrame.rescol $LFrame.resdocs.listbox copy"
    grid $LFrame.keydocs -row 0 -column 0 -rowspan 6 -sticky snew \
      -padx 2 -pady 2
    grid $LFrame.resdocs -row 0 -column 2 -rowspan 6 -sticky snew \
      -padx 2 -pady 2
    grid $LFrame.lmbutton -row 1 -column 1 -sticky ew -padx 4 -pady 10
    grid $LFrame.lcbutton -row 2 -column 1 -sticky ew -padx 4 -pady 10
    grid $LFrame.rmbutton -row 3 -column 1 -sticky ew -padx 4 -pady 10
    grid $LFrame.rcbutton -row 4 -column 1 -sticky ew -padx 4 -pady 10
    grid $LFrame.keydocslen x $LFrame.resdocslen -sticky snew \
      -padx 2 -pady 2
    grid columnconfigure $LFrame 0 -weight 1
    grid columnconfigure $LFrame 2 -weight 1
    grid rowconfigure $LFrame 0 -weight 1
    grid rowconfigure $LFrame 5 -weight 1
  pack $LFrame -fill both -expand 1 -padx 4

  set BFrame $win.button
  frame $BFrame
    button $BFrame.close -textvar GUI_Text(Close) -command "destroy $win"
    pack $BFrame.close -side top -pady 10
  pack $BFrame -fill x -expand 1 -padx 4
  update
  wm deiconify $win
};# CDMTools_TransferCollectionDocuments

## CDMTools_TransferCollectionDocumentsUpdateDocumentCount
#  Update the Document count below each listbox...
proc CDMTools_TransferCollectionDocumentsUpdateDocumentCount {listbox} {
  global GUI_Text
  set frame [winfo parent [winfo parent $listbox]]
  if {[winfo exists $frame.keydocslen]} {
    $frame.keydocslen configure -text \
      "[$frame.keydocs.listbox index end] $GUI_Text(Documents)"
  }
  if {[winfo exists $frame.resdocslen]} {
    $frame.resdocslen configure -text \
      "[$frame.resdocs.listbox index end] $GUI_Text(Documents)"
  }
};# CDMTools_TransferCollectionDocumentsUpdateDocumentCount

## CDMTools_TransferCollectionUpdateListboxes
#  Update the Document listboxes, according to the selected values...
proc CDMTools_TransferCollectionUpdateListboxes {combo listbox} {
  global CDM_CollectionFilename
  set index [$combo getvalue]
  if {$index < 0} {return}
  set cols [$combo cget -values]
  set col [lindex $cols $index]

  set file  [file join $col $CDM_CollectionFilename]
  set file2 [file join $col CDM_CollectionType.txt]
  set file3 [file join $col GuT_mAgIc]
  if {[file exists $file]} {
    ## This is a native format Collection...
    if {[catch {CDM_GetCollectionFileIDs $file} DocumentXIDs]} {
      global GUI_Text
      gut_error "[format $GUI_Text(CannotReadFile) $file]!\n($DocumentXIDs)"
    }
  } elseif {[file exists $file2]} {
    ## This is a registered format Collection...
    set TypeFile [open $file2]
    gets $TypeFile type
    close $TypeFile
    if {[catch {CDM_GetCollectionFileIDs $file2 [string trim $type]} \
                                                               DocumentXIDs]} {
      global GUI_Text
      gut_error "[format $GUI_Text(CannotReadFile) $file2]!\n($DocumentXIDs)"
    }
  } elseif {[file exists $file3]} {
    ## This is a GATE Collection...
    if {[catch {CDM_GetCollectionFileIDs $file3 gate} DocumentXIDs]} {
      global GUI_Text
      gut_error "[format $GUI_Text(CannotReadFile) $file3]!\n($DocumentXIDs)"
    }
  } else {
    ## We failed to understant the Collection type...
    $listbox delete 0 end
    return
  }

  $listbox delete 0 end 
  eval $listbox insert end $DocumentXIDs
  CDMTools_TransferCollectionDocumentsUpdateDocumentCount $listbox 
};# CDMTools_TransferCollectionUpdateListboxes

## CDMTools_TransferCollectionApply
#  This function does the actual transfer of documents...
proc CDMTools_TransferCollection {keycom keylist rescom reslist mode} {
  global GUI_Text
  set cols   [$keycom cget -values]
  set index  [$keycom getvalue]
  if {$index < 0} {return}
  set keycolname [lindex $cols $index]

  set cols   [$rescom cget -values]
  set index  [$rescom getvalue]
  if {$index < 0} {return}
  set rescolname [lindex $cols $index]
  if {[string equal $keycolname $rescolname] && [string equal $mode move]} {
    gut_error $GUI_Text(SelectDifferentCols)
  }
  unset cols
  
  ## Get the selected documents from key collection...
  set sel [$keylist curselection]
  if {![llength $sel]} {return}
  set keycol  [tip_OpenCollection $keycolname]
  set ColName [tip_GetName $keycol]
  if {![string equal $ColName $keycolname]} {
    GUI_FixCollectionName $keycol $ColName $keycolname
  }
  set rescol  [tip_OpenCollection $rescolname]
  set ColName [tip_GetName $rescol]
  if {![string equal $ColName $rescolname]} {
    GUI_FixCollectionName $rescol $ColName $rescolname
  }
  set keydocs [$keycol GetFileIds]

  ## Itelate over selected documents...
  set Step [expr {100.0/([llength $sel]+1)}]
  set Progress 0.0
  set wwin [ggi_wait_message {} 0.0 1]
  foreach index $sel {
    set dname [lindex $keydocs $index]
    ggi_wait_message "$GUI_Text(TransferingDoc): $dname..." $Progress 1
    set doc [tip_GetDocument $keycol $dname]
    set id  [file tail [tip_GetExternalId $doc]]
    ## Ensure that we have not seen again this file, taking also into account
    ## filesystems that ignore case!
    set ID $id
    if {[file exists $ColName/$ID]} {
      set ID ${id}-1; set i 1
      while {[file exists $ColName/$ID]} {
        set ID ${id}-[incr i]
      }
      set ID ${id}-$i
    }
    ## Create a new document in the target collection...
    set rdoc [tip_CreateDocument $rescol \
      $ColName/$ID [tip_GetRawData $doc] [tip_GetAnnotations $doc] \
      [tip_GetAttributes $doc] [tip_GetEncoding $doc]]
    tip_SetExternalId $doc [tip_GetExternalId $doc]
    if {[string equal $mode move]} {
      tip_RemoveDocument $keycol $dname
    } else {
      tip_Close $doc
    }
    $reslist insert end [tip_GetId $rdoc]
    $reslist see end
    tip_Sync  $rdoc
    tip_Close $rdoc
    set Progress [expr {$Progress+$Step}]
    CDM_ClearCompressedMemoryCache
  }
  unset keydocs

  if {[string equal $mode move]} {
    tip_Sync $keycol
    $keylist delete 0 end
    eval $keylist insert end [$keycol GetFileIds]
  }
  tip_Close $keycol
  tip_Sync  $rescol
  tip_Close $rescol
  ggi_wait_message $GUI_Text(UpdateCollectionBT) 100.0 1
  catch {GUI_DrawCollectionDlg}
  ggi_destroy_wait $wwin
  CDMTools_TransferCollectionDocumentsUpdateDocumentCount $keylist
}
