##############################################################################
##  AnnotationRawViewer.tcl:                                                ##
## --------------------------                                               ##
##  This file implements the "raw" Annotation viewer...                     ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

CDM_RegisterViewer {Annotation Raw Viewer} ggi_view_raw ITViewAnns

## ggi_view_raw
#  Show annotations...
proc ggi_view_raw {doc annot attr title {HilightAll 0} {win .ggi_view_raw} \
                   {start -1} {end -1}} {
  set doc_name [tip_GetExternalId $doc]
  global GUI_Text GUI_Options PANE WMIcons
  set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
  set win [ggi_unused_win $win]
  if {![string length $title]} {
    set title "Viewer: Raw, Document: [tip_GetId $doc],\
               Annotation: $annot, Attribute: $attr"
  }
  toplevel $win
  wm title $win $title
  wm withdraw $win

  ## Define Text widget...
  set text_frame $win.f.fs; set text $win.f.fs.text

  ## Toolbar...
  set toolbar_frame $win.tf 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.f.fs.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(SelectAnnsInRange...) \
        -image ITSearch -command [list ggi_view_raw_range $doc $annot $attr \
         $title $HilightAll $win $text]
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(AnnotationProperties) \
        -image ITProperties -command \
        [list ggi_view_Annotation $doc {} {} {} $tw.annotation_properties]
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    set ActLbl [Toolbar::addwidget $toolbar label -relief flat -bd 0 -anchor e]
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x
  
  frame $win.f -relief flat -width 650 -height 450
  set listbox_frame $win.f.fr; set listbox $win.f.fr.mclistbox
  ::ScrolledWidgets::ScrolledMclistbox $listbox_frame -font FixedTextFont \
     -selectmode browse -width $GUI_Options(TextWidth) -height \
     $GUI_Options(TextHeight) -highlightthickness 0 -selectbackground red \
     -selectforeground white -selectborderwidth 1 -columnrelief flat \
     -labelanchor w -columnborderwidth 0 -labelborderwidth 1 \
     -labelrelief raised -labelforeground navy -labelbackground bisque \
     -fillcolumn type -labelfont TextFont -bg $GUI_Options(InputBackground) \
     -selectcommand "ggi_view_raw_hilight $doc $win.f.fs.text $listbox"

  $listbox column add id    -label ID         -width 7
  $listbox column add type  -label Type       -width 15
  $listbox column add start -label Start      -width 7
  $listbox column add end   -label End        -width 7
  $listbox column add Attr  -label Attributes 

  ## Sort Bindings...
  $listbox label bind id    <ButtonPress-1> {ggi_view_raw_sort %W id}
  $listbox label bind type  <ButtonPress-1> {ggi_view_raw_sort %W type}
  $listbox label bind start <ButtonPress-1> {ggi_view_raw_sort %W start}
  $listbox label bind end   <ButtonPress-1> {ggi_view_raw_sort %W end}
  bind $listbox <<MB_1>> {+focus %W}
  bind $listbox <<MB_2>> {eval ggi_view_rawContextMenu \
       [::mclistbox::convert %W -W -x %x -y %y] %X %Y; break}

  ## Sample Text Widget
  set Text [tip_GetByteSequence $doc]
  ::ScrolledWidgets::ScrolledText $text_frame -height 11 
  bind $text <<MB_1>> {+focus %W}
  $text insert end $Text
  unset Text
  
  ## Load the Annotations Text Object...
  if {[string length $annot]} {
    set anns [tip_SelectAnnotations $doc $annot $attr]
  } else {
    set anns [tip_GetAnnotations $doc]
  }

  ## Do we need to select Annotations according to range?
  if {$start != -1 && $end != -1} {
    set anns [tip_AnnotationsInRange $anns $start $end]
  }
  set AnnNu [llength $anns]
  if {$AnnNu < 1} {set AnnNu 1}
  set ProgressStep [expr {60.0/$AnnNu}]; set Progress 0.0
  set max_attr_width 0
  set Items {}
  if {$HilightAll == 0} {
    foreach ann $anns {
      set Progress [expr {$Progress + $ProgressStep}]
      ggi_wait_update {} $Progress $wwin
      foreach {width display_ann} [CDM_PresentAnnotationForRawViewer $ann] \
                                  {break}
      ##$listbox insert end $display_ann
      lappend Items $display_ann
      if {$width > $max_attr_width} {set max_attr_width $width}
    }
  } else {
    foreach {i HilightNu Colors} {0 150 {}} {}
    foreach color $GUI_Options(Colors) {
      if {[string compare [winfo rgb $win [lindex $color 1]] {0 0 0}] != 0} {
        lappend Colors $color
      }
    }
    set color_nu [llength $Colors]
    if {$HilightNu > $color_nu} {set HilightNu $color_nu}
    foreach ann $anns {
      set Progress [expr {$Progress + $ProgressStep}]
      ggi_wait_update {} $Progress $wwin
      foreach {width display_ann} [CDM_PresentAnnotationForRawViewer $ann] \
                                  {break}
      #$listbox insert end $display_ann
      lappend Items $display_ann
      if {$width > $max_attr_width} {set max_attr_width $width}
      # tip_GetSpanOffsets [tip_GetFirstSpan $ann] start end
      foreach span [tip_GetSpans $ann] {
        tip_GetSpanOffsets $span start end
        $text tag add $i 1.0+${start}c 1.0+${end}c
      }
      incr i;  if {$i == $HilightNu} {set i 0}
    }
    for {set i 0} {$i < $HilightNu} {incr i} {
      set color [lindex $Colors $i]
      set bg [lindex $color 1]
      $text tag configure $i -foreground [lindex $color 0] 
    }
  }
  set AnnNu [llength $Items]
  if {$AnnNu} {
    set Process 61
    ggi_wait_update {} $Process $wwin
    set step [expr {$AnnNu/4}]
    for {set start 0; set end $step; set i 1} {$i < 5} \
        {incr i; incr end; set start $end; set end [expr {$i*$step}]} {
      if {$i == 4} {set end $AnnNu}
      eval $listbox insert end [lrange $Items $start $end]
      incr Process 10
      ggi_wait_update {} $Process $wwin
    }
    ggi_wait_update {} 100 $wwin
  }
  $listbox column configure Attr -width $max_attr_width
  ggi_destroy_wait $wwin

  ## Packing...
  pane $listbox_frame $text_frame -orient vertical 
  ## Add Pane Arrows...
  ArrowButton $win.f.__h1.b1 -type button -width 45 -height 7 \
    -dir top -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.f 1 0"
  ::$win.f.__h1.b1:cmd configure -cursor based_arrow_up
  button $win.f.__h1.b2 -image IUpDown -padx 0 -pady 0 -width 45 -height 5 \
    -bd 0 -cursor dot -relief flat -highlightthickness 0 \
    -activebackground [$win.f.__h1.b1 cget -activebackground] -anchor n \
    -command "Widget_PaneSet $win.f 1 0.5" -bg $PANE(Color)
    
#  ::$win.f.__h1.b2:cmd configure -cursor dot
  ArrowButton $win.f.__h1.b3 -type button -width 45 -height 7 \
    -dir bottom -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.f 1 100"
  ::$win.f.__h1.b3:cmd configure -cursor based_arrow_down
  place $win.f.__h1.b1 -relx 0.0
  place $win.f.__h1.b2 -relx 0.47
  place $win.f.__h1.b3 -relx 1.0 -anchor ne
  $win.f.__h1 configure -height [expr {[$win.f.__h1 cget -height]+2}]
  
  pack $win.f -fill both -expand 1
  $text configure -state disabled -exportselection 1
  button $win.dismiss -textvariable GUI_Text(Dismiss) -command \
    "destroy $win; pane forget $win.f"
  pack $win.dismiss -side bottom -pady 3
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  focus $text

  ## Arrange the character index to be shown each time the user clicks on the
  ## text widget...
  bind $text <<MB_1>> "+ggi_DisplayTextIndexInLabel $text $ActLbl %x %y"
  return $win
}

proc ggi_view_raw_sort {w id} {
  set data [$w get 0 end]
  set index [lsearch -exact [$w column names] $id]
  switch -exact $id {
    id      -
    start   -
    end     {set result [lsort -integer -index $index $data]}
    default {set result [lsort -index $index $data]}
  }
  $w delete 0 end
  # ... and add our sorted data in
  eval $w insert end $result
}

proc ggi_view_raw_range {doc annot attr title HilightAll win text} {
  ## Get the range from text widget...
  if {[catch {$text get sel.first sel.last} sel]} {
    global GUI_Text
    set sel [$text get 1.0 end-1c]
    set start 0
    set end [expr {$start+[string length $sel]}]
    set res [gut_EntryDialog -message $GUI_Text(Start) -message2 $GUI_Text(End)\
                -title {Select Span Range...} -initial $start -default2 $end]
    foreach {start end} $res {break}
  } else {
    set start [gut_FindCharacterOffset $text [$text index sel.first]]
    set end [expr {$start+[string length $sel]}]
  }
  if {[string length $start]} {
    append title " - \[$start $end\]"
    ggi_view_raw $doc $annot $attr $title $HilightAll $win $start $end
  }
}

proc ggi_view_raw_hilight {doc text listbox {index {}} args} {
  catch {$text tag delete show}
  if {[string length $index]} {
    foreach {ID Type x y} [$listbox get $index] {break}
    ## Update our Annotation Properties Dialog, if opened...
    if {[winfo exists $text.annotation_properties]} {
      ggi_view_Annotation $doc $ID {} {} $text.annotation_properties
      bind $text.annotation_properties \
        <<ggi_view_Annotation_AnnotationChanged>> \
        [list ggi_view_raw_AnnotationChangedCallback ggi_view_raw_hilight \
                                                 $doc $text $listbox $index $ID]
    }
    if {![info exists x] || ![info exists y] || \
        ![string length $x] || ![string length $y]} {return} 
    $text configure -state normal
    #$text tag add show  1.0+${x}chars 1.0+${y}chars
    foreach span [tip_GetSpans [tip_GetAnnotation $doc $ID]] {
      tip_GetSpanOffsets $span start end
      $text tag add show 1.0+${start}c 1.0+${end}c
    }
    $text tag configure show -background blue -foreground white
    $text see 1.0+${x}chars
    $text configure -state disabled
  }
  update
};# ggi_view_raw_hilight

## ggi_view_raw_AnnotationChangedCallback
#  This function gets called when the highligted Annotation is changed from
#  inside the Annotation properties window...
proc ggi_view_raw_AnnotationChangedCallback {proc doc text listbox index id} {
  set ann [tip_GetAnnotation $doc $id]
  set ann [lindex [CDM_PresentAnnotationForRawViewer $ann] 1]
  ## Delete the previous entry...
  $listbox delete $index
  $listbox insert $index $ann
  ## Simulate selection...
  $proc $doc $text $listbox $index
  #puts "ggi_view_raw_AnnotationChangedCallback [winfo toplevel $listbox]"
};# ggi_view_raw_AnnotationChangedCallback

proc ggi_view_rawContextMenu {w x y rootx rooty} {
  global GUI_Text
  set win .ggi_view_raw_contextMenu
  catch {destroy $win}
  menu $win -tearoff false
  # ask the widget for what column this is
  set column [$w column nearest $x]
  set columnLabel [$w column cget $column -label]
  $win configure -title "$columnLabel"
  $win add command -label "$GUI_Text(Sortby) $columnLabel" \
          -command [list ggi_view_raw_sort $w $column]
  $win add command -label "$GUI_Text(HideColumn) $columnLabel" \
          -command [list $w column configure $column -visible false]
  $win add separator
  $win add command -label $GUI_Text(ShowAllHiddenColumns) \
          -command "ggi_view_rawShowAllColumns $w"
  $win add command -label $GUI_Text(ChangeFont) -command \
   "::FontDialog::FontSelect $w $w.__fontdlg"
  tk_popup $win $rootx $rooty
}

proc ggi_view_rawShowAllColumns {w} {
  foreach column [$w column names] {
    $w column configure $column -visible true
  }
}
