##############################################################################
## AnnotationViewer.tcl:                                                    ##
## ---------------------                                                    ##
##  This file implements a viewer for viewing Annotations stored in         ##
##  Documents.                                                              ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

CDM_RegisterViewer {Single Annotation Viewer} ggi_view_Annotation

proc ggi_view_Annotation {doc {Id {}} {unused {}} {title {}} {win {}}} {
  global GUI_Text GUI_Options ggi_view_Annotation

  if {![string length $win]} {
    set win [ggi_unused_win .annotation_viewer]
  }
  if {![string length $title]} {
    set title "Annotation Viewer: Annotation from Document\
               \"[tip_GetId $doc]\"..."
  }

  set Info $win.annotation_info
  set listbox $Info.e_Spans.mclistbox
  set listbox2 $Info.e_Attrs.mclistbox
  set start_spinbox $Info.span_edit.span_start
  set end_spinbox   $Info.span_edit.span_end
  if {![winfo exists $win]} {
    toplevel $win
    wm withdraw $win
    wm title $win $title

    ## Create the window Toolbar...
    set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
    Toolbar::ToolbarFrame $toolbar_frame \
            -relief $GUI_Options(ToolbarFrameRelief) \
            -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
            -decorate $GUI_Options(ToolbarDecorate)

      set tw $win.frame.text
      set toolbar $toolbar_frame.tb1
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
          -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
           -image ITExit -command "destroy $win"
      pack $toolbar -side left -fill y

    pack $toolbar_frame -side top -fill x

    frame $win.buttons
    pack  $win.buttons -side bottom
    button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
      -command [list destroy $win]
    pack $win.buttons.dismiss -padx 2 -pady 2

    ## Annotation Information...
    frame $Info
      label $Info.l_ID -text ID: -anchor nw -bg #8fbc8f -fg #2f4f4f
      entry $Info.e_ID -fg #2f4f4f -highlightbackground #8fbc8f
      grid $Info.l_ID   $Info.e_ID   -sticky snew -padx 0 -pady 0

      label $Info.l_Type -text Type: -anchor nw -bg #ffb6c1 -fg #FF0000
      entry $Info.e_Type -fg #FF0000 -highlightbackground #ffb6c1
      grid $Info.l_Type $Info.e_Type -sticky snew -padx 0 -pady 0

      label $Info.l_Spans -text {Span Set:} -anchor nw -bg #b0c4de -fg #0000cd
      ::ScrolledWidgets::ScrolledMclistbox $Info.e_Spans -font TextFont \
         -selectmode browse -width 62 -height 2 -highlightthickness 0 \
         -selectbackground red -selectforeground white -selectborderwidth 1 \
         -columnrelief flat -labelanchor w -columnborderwidth 0 \
         -labelborderwidth 1 -labelrelief raised \
         -labelforeground #0000cd -labelbackground #b0c4de \
         -fillcolumn end -labelfont TextFont -bg $GUI_Options(InputBackground) \
         -fg #0000cd
      $listbox column add start -label Start -width 31 \
        -background $GUI_Options(InputBackground)
      $listbox column add end   -label End   -width 31 \
        -background $GUI_Options(InputBackground)
      ## Sort Bindings...
      $listbox label bind start <ButtonPress-1> {ggi_view_raw_sort %W start}
      $listbox label bind end   <ButtonPress-1> {ggi_view_raw_sort %W end}
      bind $listbox <<MB_1>> {+focus %W}
      bind $listbox <<MB_2>> {eval ggi_view_rawContextMenu \
       [::mclistbox::convert %W -W -x %x -y %y] %X %Y; break}
      grid $Info.l_Spans $Info.e_Spans -sticky snew -padx 0 -pady 0
      ## Facilities for modifying the Spans...
      frame $Info.span_edit -bg #b0c4de
        spinbox $Info.span_edit.span_start -width 8 -state disabled -from 0 \
          -increment 1
        spinbox $Info.span_edit.span_end   -width 8 -state disabled -from 0 \
          -increment 1
        grid $Info.span_edit.span_start -sticky snw -padx 80
        grid $Info.span_edit.span_end   -sticky sne -padx 80 -row 0 -column 1
        grid columnconfigure $Info.span_edit 0 -weight 1
        grid columnconfigure $Info.span_edit 1 -weight 1
      grid [frame $Info.dummy -bg #b0c4de] $Info.span_edit \
          -sticky snew -padx 0 -pady 0

      label $Info.l_Attrs -text {Attribute Set:} -anchor nw \
            -bg #d2b48c -fg #8b0000
      frame $Info.e_Attrs
      set listbox2 [ggi_view_Attributes {} {} 0 {} $Info.e_Attrs]
      $listbox2 configure -height 8 -width 50
      $listbox2 column configure Name  -background #d2b48c -foreground #8b0000\
           -borderwidth 1
      $listbox2 column configure Value -background #d2b48c -foreground #8b0000\
           -borderwidth 1
      $listbox2 column configure Type  -background #d2b48c -foreground #8b0000\
           -borderwidth 1
      grid $Info.l_Attrs $Info.e_Attrs -sticky snew -padx 0 -pady 0

      grid columnconfigure $Info 1 -weight 1
      grid rowconfigure    $Info 3 -weight 1
    pack $Info -fill both -expand 1 -padx 2 -pady 2
    update
    wm deiconify $win
  };# if {![info exists $win]}

  ## Retrieve the Annotation, if possible...
  if {[llength $doc] == 4} {
    set Ann $doc
  } elseif {[llength $doc] == 1 && [string length $Id]} {
    set Ann [tip_GetAnnotation $doc $Id]
  } else {
    set Ann {{} {} {} {}}
  }
  foreach {Id Type Spans Attributes} $Ann {break}
  ## Set Annotation Id & Type...
  $Info.e_ID delete 0 end
  $Info.e_ID insert end $Id
  $Info.e_Type delete 0 end
  $Info.e_Type insert end $Type
  ## Fill the Span Set listbox...
  $listbox delete 0 end
  foreach span $Spans {
    $listbox insert end $span
  }
  if {[llength $Spans]} {
    $Info.l_Spans configure -text "Span Set:\n[llength $Spans]"
  }
  if {[llength $Attributes]} {
    $Info.l_Attrs configure -text "Attribute Set:\n[llength $Attributes]"
  }
  ## Fill the Attribute Set listbox...
  ggi_view_Attributes {} $Attributes 0 {} $Info.e_Attrs
  $listbox configure -selectcommand \
    [list ggi_view_Annotation_SelectSpanCallback $doc $Id $listbox \
          $start_spinbox $end_spinbox]
  $start_spinbox delete 0 end
  $end_spinbox   delete 0 end
  update
  $start_spinbox configure -command {} -state disabled
  $end_spinbox   configure -command {} -state disabled
  return $win
};# ggi_view_Annotation

## ggi_view_Annotation_SelectSpanCallback
#  This callback gets called when the user selects a span in the main viewer
#  window...
proc ggi_view_Annotation_SelectSpanCallback {doc id listbox
                                         start_spinbox end_spinbox args} {
  if {![string match Doc* $doc]} {return}
  set index [lindex $args 0]
  if {[string length $index]} {
    $start_spinbox configure -state normal
    $end_spinbox configure   -state normal
    $start_spinbox delete 0 end
    $end_spinbox   delete 0 end
    ## Get the selected span...
    set ann   [tip_GetAnnotation $doc $id]
    set spans [tip_GetSpans $ann]
    set span  [lindex $spans $index]
    tip_GetSpanOffsets $span start end
    $start_spinbox insert 0 $start
    $end_spinbox   insert 0 $end
    $start_spinbox configure -to [expr {$start + 1000}] -command [list \
      ggi_view_Annotation_SpanSpinboxCallback $start_spinbox $end_spinbox \
                                              $doc $id $listbox $index]
    $end_spinbox   configure -to [expr {$end + 1000}]   -command [list \
      ggi_view_Annotation_SpanSpinboxCallback $start_spinbox $end_spinbox \
                                              $doc $id $listbox $index]
  } else {
    $start_spinbox delete 0 end
    $end_spinbox   delete 0 end
    $start_spinbox configure -state disabled -command {}
    $end_spinbox   configure -state disabled -command {}
  }
};# ggi_view_Annotation_SelectSpanCallback

proc ggi_view_Annotation_SpanSpinboxCallback {start_spin end_spin doc id
                                              listbox index} {
  set ann   [tip_GetAnnotation $doc $id]
  set spans [tip_GetSpans $ann]
  set span  [tip_CreateSpan [$start_spin get] [$end_spin get]]
  set spans [lset spans $index $span]
  ## Re-create the Annotation...
  set ann   [lset ann 2 $spans]
  tip_AddAnnotation $doc $ann
  ## Replace the listox view...
  $listbox delete $index
  $listbox insert $index $span
  update
  ggi_view_Annotation_SelectSpanCallback $doc $id $listbox \
                                         $start_spin $end_spin $index
  ## Generate an event on the window that the Annotaton was changed...
  event generate [winfo toplevel $listbox] \
    <<ggi_view_Annotation_AnnotationChanged>>
  ## Make sure that the listbox entry is again selected...
  $listbox select $index
  ggi_view_Annotation_SelectSpanCallback $doc $id $listbox \
                                         $start_spin $end_spin $index
};# ggi_view_Annotation_SpanSpinboxCallback
