##############################################################################
## CachedMemoryViewer.tcl:                                                  ##
## -----------------------                                                  ##
##  This file implements a viewer for viewing the objects cached internally ##
##  by the CDM Object Cache.                                                ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

proc ggi_view_CDM_CachedMemoryObjects {args} {
  global GUI_Text GUI_Options

  set win [ggi_unused_win ._cached_memory_objects_viewer_]
  toplevel $win
  wm withdraw $win
  wm title $win {Cached Memory Object Viewer...}

  ## Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addwidget $toolbar checkbutton -textvariable \
        GUI_Text(EnableMemoryCompression) -indicatoron 1 -offvalue 0 \
        -onvalue 1 -variable ::CDM_EnableMemoryCompression \
        -activebackground $GUI_Options(ToolbarBackground) \
        -tooltipvar GUI_Text(EnableMemoryCompression)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ReloadPage) \
         -image ITRedraw -command \
         "ggi_view_CDM_CachedMemoryObjectsRefresh $win.mainArea.noteBook"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ClearCachedMemory) \
         -image ITMemoryClear -command \
         "ggi_view_CDM_CachedMemoryObjectsClearCache $win.mainArea.noteBook"
    Toolbar::addseparator $toolbar 
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  ## Button Section...
  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  pack $win.buttons.dismiss -padx 2 -pady 10

  ## In the main window area create a NoteBook widget...
  set NoteBook $win.mainArea.noteBook
  set NoteBookBg [$win cget -bg]
  frame $win.mainArea
  NoteBook $NoteBook -bg $NoteBookBg -side top
  ## Insert the NoteBook pages...
  foreach {page label array} {string {Cached String Objects} StringTable
                 long    {Cached Long Integer Objects}    LongTable
                 attr    {Cached Attribute Objects}       AttributeTable
                 attrval {Cached Attribute Value Objects} AttributeValueTable
                 attrset {Cached Attribute Set Objects}   AttributeSetTable} {
    set Area [$NoteBook insert end $page -background $NoteBookBg]
    set array ::CDM::CDM_Cache::$array
    ## Create a multi-column listbox...
    ::ScrolledWidgets::ScrolledMclistbox $Area.listbox -font TextFont \
         -selectmode browse -highlightthickness 0 -width 90 -height 16\
         -selectbackground red -selectforeground white -selectborderwidth 1 \
         -columnrelief flat -labelanchor w -columnborderwidth 0 \
         -labelborderwidth 1 -labelrelief raised \
         -labelforeground navy -labelbackground bisque \
         -fillcolumn 0 -labelfont TextFont -bg $GUI_Options(InputBackground) \
         -fg #0000cd -selectcommand "" -resizablecolumns 1
    set listbox $Area.listbox.mclistbox
    $listbox column add object -label {Object Value} -width 80 \
        -background $GUI_Options(InputBackground) -resizable 1
    $listbox column add count  -label {Count} -width 10 \
        -background $GUI_Options(InputBackground)
    ## Fill the listbox with the desired values...
    set Items {}
    foreach key [lsort -dictionary [array names $array]] {
      catch {lappend Items [list $key [set ${array}($key)]]}
    }
    eval $listbox insert end $Items
    $NoteBook itemconfigure $page -text "$label ([llength $Items])"
    #$listbox column configure object -width 0
    pack $Area.listbox -fill both -expand 1
  }
  $NoteBook raise string
  $NoteBook see string
  pack $NoteBook -fill both -expand 1
  pack $win.mainArea -fill both -expand 1

  update
  wm deiconify $win
}; # ggi_view_CDM_CachedMemoryObjects

## ggi_view_CDM_CachedMemoryObjectsRefresh
#  Refresh the values displayed in the listboxes...
proc ggi_view_CDM_CachedMemoryObjectsRefresh {NoteBook} {

  foreach page [$NoteBook pages] array {StringTable LongTable AttributeTable
                                        AttributeValueTable AttributeSetTable} {
    set Area [$NoteBook getframe $page]
    set array ::CDM::CDM_Cache::$array
    set listbox $Area.listbox.mclistbox
    ## Fill the listbox with the desired values...
    set Items {}
    $listbox delete 0 end
    foreach key [lsort -dictionary [array names $array]] {
      catch {lappend Items [list $key [set ${array}($key)]]}
    }
    eval $listbox insert end $Items
    $NoteBook itemconfigure $page -text \
      "[string trim [$NoteBook itemcget $page -text] (0123456789)\ ] ([llength $Items])"
  }
  #CDM_CompressedMemoryCacheInfo
};# ggi_view_CDM_CachedMemoryObjectsRefresh

## ggi_view_CDM_CachedMemoryObjectsClearCache
#  Remove all objects from the memory cache...
proc ggi_view_CDM_CachedMemoryObjectsClearCache {{NoteBook {}} {ask 1}} {
  # Its better to ask the user before removing anything from the cache...
  if {$ask} {
    set ans [gut_question $::GUI_Text(ClearCachedObjects?)]
    if {[string equal $ans no]} {return}
  }
  # CDM_FreeTclObjCache;# Intented to force Tcl free objects. Not implemented
  
  # Clear unused AttributeSets. Four of them will do a full clear...
  #puts "Clearing unused Attribute sets..."
  CDM_ClearCompressedMemoryCache
  CDM_ClearCompressedMemoryCache
  CDM_ClearCompressedMemoryCache
  CDM_ClearCompressedMemoryCache

  # Crear unused objects from the CDM_ObjectCache (strings, Attributes &
  # Attribute Values)...
  #puts "Clearing unused strings, Attributes & Attribute Values..."
  CDM_ClearUnusedSavedValues

  # Clear completely the objects from the CDM_ObjectCache (strings, Attributes &
  # Attribute Values)...
  #puts "Clearing cached strings, Attributes & Attribute Values..."
  CDM_ClearSavedValues

  # Refresh our view...
  if {[string length $NoteBook] && [winfo exists $NoteBook]} {
    #puts "Refresing view..."
    ggi_view_CDM_CachedMemoryObjectsRefresh $NoteBook
  }
  #puts "Done!"
};# ggi_view_CDM_CachedMemoryObjectsClearCache
