##############################################################################
## HTMLAnnotationViewer.tcl:                                                ##
## -------------------------                                                ##
##  This file implements the "HTML Annotation Explorer", a viewer for       ##
##  viewing Annotations stored in a Document as a tree.                     ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
package require BWidget
CDM_RegisterViewer {HTML Annotation Explorer} ggi_view_HTMLAnnotationExplorer ITExplore

proc ggi_view_HTMLAnnotationExplorer {doc type attr title
                                      {win .ggi_view_explannhtml}} {
  global GUI_Text GUI_Options GUI_ParseViewer WMIcons PANE \
         ggi_view_HTMLAnnotationExplorer_Priv
  set Colors $GUI_Options(Colors)
  set win [ggi_unused_win $win]
  toplevel $win
  wm title $win $title
  wm withdraw $win

  ## Toolbar...
  set Text     $win.sc.top_section.text
  set Tree     $win.sc.top_section.tree
  set AnnTable $win.sc.bot_section.table.mclistbox
  set toolbar_frame $win.tf
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)
    set tw $Text.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "tip_Sync [tip_GetParent $doc]"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(SelectAnnsInRange...) \
        -image ITSearch -command [list ggi_view_raw_range $doc {} {} $title \
        0 $win $tw]
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(AnnotationProperties) \
        -image ITProperties -command \
        [list ggi_view_Annotation $doc {} {} {} $tw.annotation_properties]
    set HideMenuButton [Toolbar::addwidget $toolbar menubutton \
        -tooltipvar GUI_Text(HideTag) -image ITHideTags -activebackground \
        $GUI_Options(ToolbarBackground) -state disabled]
    set ShowMenuButton [Toolbar::addwidget $toolbar menubutton \
        -tooltipvar GUI_Text(ShowTag) -image ITShowTags -activebackground \
        $GUI_Options(ToolbarBackground) -pady 0 -state disabled]
    menu $HideMenuButton.menu -tearoff 0 -postcommand \
      "$HideMenuButton configure -relief flat
       $ShowMenuButton configure -relief flat"
    $HideMenuButton configure -menu $HideMenuButton.menu
    menu $ShowMenuButton.menu -tearoff 0 -postcommand \
      "$HideMenuButton configure -relief flat
       $ShowMenuButton configure -relief flat"
    $ShowMenuButton configure -menu $ShowMenuButton.menu
    Toolbar::addseparator $toolbar
    set mode_button [Toolbar::addbutton $toolbar \
         -tooltipvar GUI_Text(EnableAnnotationDeletion) -image ITWarning]
    Toolbar::configure $toolbar $mode_button -command \
         "ggi_view_HTMLAnnotationExplorer_EnableAnnotationDeletionMode \
                   $win $mode_button $toolbar $AnnTable $doc"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    set ActiveLabel [Toolbar::addwidget $toolbar label -relief flat -bd 0 \
      -anchor e -textvariable \
       ggi_view_HTMLAnnotationExplorer_Priv($win>CurrentTag)]
    bind $ActiveLabel <Destroy> \
      "unset -nocomplain ggi_view_HTMLAnnotationExplorer_Priv($win>CurrentTag)"
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  frame $win.sc             -relief flat -bd 0 -width 750 -height 650
  frame $win.sc.top_section -relief flat -bd 0
  frame $win.sc.bot_section -relief flat -bd 0
  
  ## Text Widget
  ::ScrolledWidgets::ScrolledHtml $Text -widgetname text \
    -bg $GUI_Options(TextBg) \
    -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
    -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
    -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
    -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
    -fontcommand      "::HtmlInfoViewer::PickFont $Text.text" \
    -imagecommand     "ggi_view_documentHTMLRetrieveImages $doc $Text.text" \
    -formcommand      "::HtmlInfoViewer::FormCmd $Text.text" \
    -bgimagecommand   "ggi_view_documentHTMLRetrieveBgImages $doc $Text.text"
  bind $Text.text.x <<MB_1>> \
      "set ::HtmlInfoViewer::InfoViewer($Text.text::SelectionStart) \
                                        \[$Text.text index @%x,%y\]
       $Text.text selection clear"
  bind $Text.text.x <<MB_2>> \
      [list ::HtmlInfoViewer::ContextualMenuHandler $Text.text $doc]
  bind $Text.text <Destroy> \
      [list +ggi_view_documentHTMLRetrieveImagesClean {} $Text.text]
  catch {$Text.text configure -base [tip_GetExternalId $doc]}
  ## We extend the scrolled widget to add search facilities...
  set cmdList \
    $::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($Text.horizontal_args)
  set \
    ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($Text.horizontal_args)\
    [lreplace $cmdList 5 5 2]
  set grid_args [lrange $cmdList 6 end]
  eval grid $Text.text -sticky snew -row 0 -column 0 -rowspan 2 $grid_args
  frame $Text.frame -bd 0 -relief flat
    button $Text.frame.up      -image DoubleArrowUp   -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -command "ggi_view_single_spanFind $Text up"
    menubutton $Text.frame.dot -image GreyDot         -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -direction left -menu $Text.frame.dot.menu \
      -relief raised -text {{} 1.0}
    menu $Text.frame.dot.menu -tearoff 0
    button $Text.frame.bottom  -image DoubleArrowDown -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -command "ggi_view_single_spanFind $Text down"
    pack $Text.frame.up $Text.frame.dot $Text.frame.bottom \
      -padx 0 -pady 0 -side top -expand 1 -fill x
  grid $Text.frame -sticky snew -row 1 -column 1
  ## Load the text widget...
  $Text.text parse [tip_GetByteSequence $doc]
  bind $Text.text <<MB_1>> "focus $Text.text"

  ## Tree Widget
  set tree $Tree.canvas
  ::ScrolledWidgets::ScrolledCanvas $Tree -regionpadx 10 -regionpady 10
  set xscrollcommand [$tree cget -xscrollcommand]
  set yscrollcommand [$tree cget -yscrollcommand]
  destroy $Tree.canvas
  Tree $tree -relief flat -borderwidth 0 -background $GUI_Options(ExplorerBg) \
       -linesfill $GUI_Options(ExplorerLineFg) -deltay 35 -deltax 20 \
       -xscrollcommand $xscrollcommand -yscrollcommand $yscrollcommand
  ## A temporary fix to remove the vertical bars until BWidgets gets fixed...
  $tree:cmd configure -padx 0 -pady 0
  grid $tree -row 0 -column 0 -sticky snew
  bind $tree <<MB_1>> "focus $tree"
    ## Make the widget a BWidget tree widget and load it with the annotation
    ## types and their attributes...
    set index 0; set cindex 0
    foreach type [$doc GetAnnotationTypes] {
      foreach {bg fg} [lindex $Colors $cindex] {}; incr cindex
      ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow root \
          $win $tree.i$index$cindex $type $fg $bg {arial 14 bold}
      $tree.i$index$cindex.ck conf -command \
          "ggi_view_HTMLAnnotationExplorer_TagAnnotation $win $Text.text $doc \
           [list $type] $tree.i$index$cindex.ack $tree.i$index$cindex.ck \
           $HideMenuButton.menu $ShowMenuButton.menu"
      bind $tree.i$index$cindex.lb <<MB_1>> \
          "ggi_view_HTMLAnnotationExplorer_AnnotationCallback $win $doc \
           [list $type] $tree $fg $bg $tree.i$index$cindex.lb \
           $tree.i$index$cindex.ck $AnnTable"
      bind $tree.i$index$cindex.lb <<MB_2>> \
        "ggi_view_HTMLAnnotationExplorer_ExplorerContextualMenu $tree $type \
           $tree.i$index$cindex.ck $tree.i$index$cindex.ack %W"
      checkbutton $tree.i$index$cindex.ack -bg $bg -fg $fg \
      -overrelief raised -relief flat -highlightthickness 0 -bd 0 \
      -variable ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type) \
      -textvariable ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type) \
      -font {arial 14 bold} -indicatoron 0 -offvalue { + } \
      -onvalue { - } -selectcolor {} -command \
        "ggi_view_HTMLAnnotationExplorer_AnnotationToggle $win $doc \
         [list $type] $tree $fg $bg $tree.i$index$cindex.lb \
         $tree.i$index$cindex.ack $AnnTable"
      bind $tree.i$index$cindex.ack <MouseWheel> \
        "$tree yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $tree.i$index$cindex.ack <4> "$tree yview scroll -5 units"
      bind $tree.i$index$cindex.ack <5> "$tree yview scroll  5 units"
      pack $tree.i$index$cindex.ack -side left -fill y
      $tree insert $index root [string map \
            {: _:_ & <amp> | <or> ^ <cir> ! <ex>} $type] -drawcross allways\
            -open 0 -window $tree.i$index$cindex
      set position 0
      foreach attribute [lsort  [$doc GetAnnotationAttributeNames $type]] {
        foreach {bg fg} [lindex $Colors $cindex] {break}; incr cindex
        ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow $type $win \
          $tree.i$index$position$cindex $attribute $fg $bg {arial 12 bold} \
          "ggi_view_HTMLAnnotationExplorer_AttributeCallback $win $doc \
           [list $type] [list $attribute] $tree $type$attribute $Text.text \
           $HideMenuButton.menu $ShowMenuButton.menu $AnnTable"
        $tree.i$index$position$cindex.ck conf -command \
          "ggi_view_HTMLAnnotationExplorer_TagAnnotation $win $Text.text $doc \
           [list $type] $tree.i$index$position$cindex.ack \
           $tree.i$index$position$cindex.ck \
           $HideMenuButton.menu $ShowMenuButton.menu \
           \"\[llength ann::$attribute \]\" [list $type.$attribute]"
        bind $tree.i$index$position$cindex.lb <<MB_2>> \
          "ggi_view_HTMLAnnotationExplorer_ExplorerContextualMenu $tree \
           $type.$attribute $tree.i$index$position$cindex.ck \
           $tree.i$index$position$cindex.ack %W"
        checkbutton $tree.i$index$position$cindex.ack -bg $bg -fg $fg \
          -overrelief raised -relief flat -highlightthickness 0 -bd 0 \
          -variable \
           ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type.$attribute) \
          -textvariable \
           ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type.$attribute) \
          -font {arial 12 bold} -indicatoron 0 -offvalue { + } \
          -onvalue { - } -selectcolor {} -command \
          "ggi_view_HTMLAnnotationExplorer_AnnotationToggle $win $doc \
           [list $type] $tree $fg $bg $tree.i$index$position$cindex.lb \
           $tree.i$index$position$cindex.ack $AnnTable \
           \{\[string length ann::[list $attribute] \]\}"
        bind $tree.i$index$position$cindex.ack <MouseWheel> \
          "$tree yview scroll \[expr {- (%D / 120) * 4}\] units"
        bind $tree.i$index$position$cindex.ack <4> "$tree yview scroll -5 units"
        bind $tree.i$index$position$cindex.ack <5> "$tree yview scroll  5 units"
        pack $tree.i$index$position$cindex.ack -side left -padx 0 -fill y
        $tree insert $position [string map \
            {: _:_ & <amp> | <or> ^ <cir> ! <ex>} $type] \
            [string map {: _:_ & <amp> | <or> ^ <cir> ! <ex>} \
            $type$attribute] -drawcross allways -open 0 \
            -window $tree.i$index$position$cindex
        incr position
      }
      incr index
    }
  set ggi_view_HTMLAnnotationExplorer_Priv($win>MaxColor) $cindex
  set ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress) 0

  ## Pack the two main views...
  pane $Text $Tree -orient horizontal
  ArrowButton $win.sc.top_section.__h1.b1 -type button -width 6 -height 60 \
    -dir left -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.sc.top_section 1 0.7"
  ::$win.sc.top_section.__h1.b1:cmd configure -cursor left_side
  ArrowButton $win.sc.top_section.__h1.b2  -type button -width 6 -height 60 \
    -dir right -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.sc.top_section 1 1"
  ::$win.sc.top_section.__h1.b2:cmd configure -cursor right_side
  Widget_PaneSet $win.sc.top_section 1 0.7
  pack $win.sc.top_section.__h1.b1 $win.sc.top_section.__h1.b2 -side top

  ## Table widget...
  ::ScrolledWidgets::ScrolledMclistbox $win.sc.bot_section.table \
     -font FixedTextFont \
     -selectmode browse -highlightthickness 0 -selectbackground red \
     -selectforeground white -selectborderwidth 1 -columnrelief flat \
     -labelanchor w -columnborderwidth 0 -labelborderwidth 1 \
     -labelrelief raised -labelforeground navy -labelbackground bisque \
     -fillcolumn id -labelfont TextFont -bg $GUI_Options(InputBackground) \
     -selectcommand "ggi_view_HTMLAnnotationExplorer_HighlightCallback \
                                                     $doc $Text.text $AnnTable"
  $AnnTable column add id    -label ID         -width 7
  $AnnTable column add type  -label Type       -width 15
  $AnnTable column add start -label Start      -width 7
  $AnnTable column add end   -label End        -width 7
  $AnnTable column add Attr  -label Attributes 

  ## Sort Bindings...
  $AnnTable label bind id    <ButtonPress-1> {ggi_view_raw_sort %W id}
  $AnnTable label bind type  <ButtonPress-1> {ggi_view_raw_sort %W type}
  $AnnTable label bind start <ButtonPress-1> {ggi_view_raw_sort %W start}
  $AnnTable label bind end   <ButtonPress-1> {ggi_view_raw_sort %W end}
  bind $AnnTable <<MB_1>> {+focus %W}
  bind $AnnTable <<MB_2>> "eval ggi_view_rawContextMenu\
             \[::mclistbox::convert %W -W -x %x -y %y\] %X %Y;break"
  pack $win.sc.bot_section.table -side top -fill both -expand 1

  ## Put the two main widget frames in a pane...
  pane $win.sc.top_section $win.sc.bot_section -orient vertical
  ## Add Pane Arrows...
  ArrowButton $win.sc.__h1.b1 -type button -width 45 -height 7 \
    -dir top -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.sc 1 0"
  ::$win.sc.__h1.b1:cmd configure -cursor based_arrow_up
  button $win.sc.__h1.b2 -image IUpDown -padx 0 -pady 0 -width 45 -height 5 \
    -bd 0 -cursor dot -relief flat -highlightthickness 0 \
    -activebackground [$win.sc.__h1.b1 cget -activebackground] -anchor n \
    -command "Widget_PaneSet $win.sc 1 0.5" -bg $PANE(Color)
  ArrowButton $win.sc.__h1.b3 -type button -width 45 -height 7 \
    -dir bottom -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.sc 1 100"
  ::$win.sc.__h1.b3:cmd configure -cursor based_arrow_down
  place $win.sc.__h1.b1 -relx 0.0
  place $win.sc.__h1.b2 -relx 0.47
  place $win.sc.__h1.b3 -relx 1.0 -anchor ne
  $win.sc.__h1 configure -height [expr {[$win.sc.__h1 cget -height]+2}]
  
  pack $win.sc -side top -fill both -expand 1

  ## Button frame
  frame $win.buttons -relief flat -bd 0
    button $win.buttons.ok -textvar GUI_Text(Ok) -command "destroy $win"
    bind $win.buttons.ok <Destroy> "pane forget $win.top_section;\
    array unset ggi_view_HTMLAnnotationExplorer_Checked $win>*
    array unset ggi_view_HTMLAnnotationExplorer_Priv $win>*
    array unset ggi_view_HTMLAnnotationExplorer_Priv $win:*
    if \[info exists ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu\] {
      foreach one \[array names ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu\] {
        image delete \[set ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu(\$one)\]
      }
      unset -nocomplain one ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu
    }"
    pack $win.buttons.ok -side top -pady 2
  pack $win.buttons -side bottom -fill x -expand 0
  ## Arrange the character index to be shown each time the user clicks on the
  ## text widget...
  bind $Text.text <<MB_1>> \
    "+ggi_DisplayTextIndexInLabel $Text.text $ActiveLabel %x %y"
  update
  wm deiconify $win
  return $win
};# ggi_view_HTMLAnnotationExplorer

proc ggi_view_HTMLAnnotationExplorer_AttributeCallback {win doc type name \
         tree parent text HM SM AnnTable widget1 widget2} {
  set process_anns 0
  set AnnSet [tip_SelectAnnotations $doc $type {}]
  foreach ann $AnnSet {
    set attr [tip_GetAttribute $ann $name]
    if {[llength $attr]} {
      if {![string equal [tip_GetValueType $attr] GDM_STRING_SET]} {
        set process_anns 1
      }
      break
    }
  }
  if {$process_anns} {
    foreach ann $AnnSet {
      set attr [tip_GetAttribute $ann $name]
      if {[llength $attr]} {set Values([tip_GetValueValue $attr]) 0}
    }
    set Vals [lsort [array names Values]]
    if {[llength $Vals] < 200} {
      global GUI_Options ggi_view_HTMLAnnotationExplorer_Priv
      if {[llength $Vals] > 1} {
        ## Create a label to hilight all attribute values...
        ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow $parent $win \
         $tree.v$type${name}_all All \
         $GUI_Options(ExplorerFg) $GUI_Options(ExplorerBg) {arial 12 bold}
        bind $tree.v$type${name}_all.lb <<MB_1>> \
           "$tree.v$type${name}_all.ck invoke"
        $tree insert 0 [string map {: _:_ & <amp> | <or> ^ <cir> ! <ex>} \
          $parent] [string map {: _:_ & <amp> | <or> ^ <cir> ! <ex>} \
          $type$name>All] -drawcross never -open 0\
          -window $tree.v$type${name}_all
      }
      set index $ggi_view_HTMLAnnotationExplorer_Priv($win>MaxColor)
      foreach value $Vals {
        foreach {bg fg} [lindex $GUI_Options(Colors) $index] {break}
        ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow $parent $win \
          $tree.v$type$name$index $value $fg $bg {arial 10 bold}
        bind $tree.v$type$name$index.lb <<MB_1>> \
            "$tree.v$type$name$index.ck invoke"
        bind $tree.v$type$name$index.lb <<MB_2>> \
          "ggi_view_HTMLAnnotationExplorer_ExplorerContextualMenu $tree \
             $type.$name.$value $tree.v$type$name$index.ck \
             $tree.v$type$name$index.ack %W"
        lappend CKWidgets [list $tree.v$type$name$index.ck invoke]
        $tree.v$type$name$index.ck conf -command \
          "ggi_view_HTMLAnnotationExplorer_TagAnnotation $win $text $doc \
           [list $type] $tree.v$type$name$index.ack \
           $tree.v$type$name$index.ck $HM $SM \
           \{\[string equal ann::$name [list $value] \]\} \
           [list $type.$name.$value]"
        $tree insert $index [string map {: _:_ & <amp> | <or> ^ <cir> ! <ex>} \
          $parent] [string map {: _:_ & <amp> | <or> ^ <cir> ! <ex>} \
          $type$name>$value] -drawcross never -open 0 \
          -window $tree.v$type$name$index

        ## Create the "+" checkbutton...
        checkbutton $tree.v$type$name$index.ack -bg $bg -fg $fg \
          -overrelief raised -relief flat -highlightthickness 0 -bd 0 \
          -variable \
           ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type.$name.$value) \
          -textvariable \
           ggi_view_HTMLAnnotationExplorer_Checked($win>SAroot>$type.$name.$value) \
          -font {arial 10 bold} -indicatoron 0 -offvalue { + } \
          -onvalue { - } -selectcolor {} -command \
          "ggi_view_HTMLAnnotationExplorer_AnnotationToggle $win $doc $type $tree \
           $fg $bg $tree.v$type$name$index.lb $tree.v$type$name$index.ack \
           $AnnTable \{\[string equal [list ann::$name] [list $value]\]\}"
        bind $tree.v$type$name$index.ack <MouseWheel> \
          "$tree yview scroll \[expr {- (%D / 120) * 4}\] units"
        bind $tree.v$type$name$index.ack <4> "$tree yview scroll -5 units"
        bind $tree.v$type$name$index.ack <5> "$tree yview scroll  5 units"
        pack $tree.v$type$name$index.ack -side left -padx 0 -fill y
        incr index
      }
      set ggi_view_HTMLAnnotationExplorer_Priv($win>MaxColor) $index
      $tree opentree $parent
      $tree itemconfigure $parent -drawcross allways
      if {[llength $Vals] > 1} {
        set CKWidgets [linsert $CKWidgets 0 \
            "set ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress) 1"]
        set CKWidgets [linsert $CKWidgets end \
            "set ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress) 0" \
            "destroy $win.ggi_wait_message"]
        $tree.v$type${name}_all.ck configure -command [join $CKWidgets \;]
      }
    } else {
      $widget2 invoke
    }
  } else {
    $widget2 invoke
  }
  bind $widget1 <<MB_1>> "$widget2 invoke"
  #$widget2 configure -command {}
};# ggi_view_HTMLAnnotationExplorer_AttributeCallback

proc ggi_view_HTMLAnnotationExplorer_AnnotationCallback {win doc ann_type tree\
     fg bg w_label w_check w_list} {
  $tree itemconfigure $ann_type -drawcross allways
  $tree opentree $ann_type
};# ggi_view_HTMLAnnotationExplorer_AnnotationCallback

proc ggi_view_HTMLAnnotationExplorer_AnnotationToggle {win doc ann_type tree\
     fg bg w_label w_check w_list {cons {}}} {
  set var [$w_check cget -variable]
  if {[string match *-* [set ::$var]]} {
    ## We should add the annotations in the listbox...
    set max_attr_width 0
    set AnnSet [tip_SelectAnnotations $doc $ann_type $cons]
    set AnnNu [llength $AnnSet]
    if {$AnnNu == 0} {return}
    set wwin [ggi_wait_message $::GUI_Text(LoadingAnnotations) 0.0]
    set ProgressStep [expr {60.0/$AnnNu}]; set Progress 0.0
    set Items {}
    foreach ann $AnnSet {
      set Progress [expr {$Progress + $ProgressStep}]
      ggi_wait_update {} $Progress $wwin
      foreach {width display_ann} [CDM_PresentAnnotationForRawViewer $ann] \
                                  {break}
      #$w_list insert end $display_ann
      ## Since, we need colour...
      #$w_list.frametype.listbox itemconf end -foreground $fg -background $bg
      if {$width > $max_attr_width} {set max_attr_width $width}
      lappend Items $display_ann
    }
    if {$max_attr_width > [$w_list column cget Attr -width]} {
      $w_list column configure Attr -width $max_attr_width
    }
    set AnnNu [llength $Items]
    if {$AnnNu} {
      set Process 61
      ggi_wait_update {} $Process $wwin
      set step [expr {$AnnNu/4}]
      for {set start 0; set end $step; set i 1} {$i < 5} \
          {incr i; incr end; set start $end; set end [expr {$i*$step}]} {
        if {$i == 4} {set end $AnnNu}
        eval $w_list insert end [lrange $Items $start $end]
        incr Process 10
        ggi_wait_update {} $Process $wwin
      }
      ggi_wait_update {} 100 $wwin
    }
    ggi_destroy_wait $wwin
  } else {
    ## We should remove the annotations in the listbox...
    set col $w_list.text.frametype.listbox
    set id  $w_list.text.frameid.listbox
    set end [$col index end]
    set wwin [ggi_wait_message $::GUI_Text(LoadingAnnotations) 0.0]
    set AnnNu $end
    if {$AnnNu < 1} {set AnnNu 1}
    set ProgressStep [expr {100.0/$AnnNu}]; set Progress 0.0
    set same_end 0
    if {[string length $cons]} {
      for {set i 0} {$i <= $end} {incr i; set same_end $i} {
        set Progress [expr {$Progress + $ProgressStep}]
        ggi_wait_update {} $Progress $wwin
        if {[string equal [$col get $i] $ann_type]} {
          ## An Annotation that matches our type...
          set AnnSet [list \
            [tip_GetAnnotation $doc [$id get $i]]]
          if {[llength [tip_SelectAnnotations $AnnSet $ann_type $cons]] == 0} {
            continue
          }
          for {incr same_end} {$same_end <= $end} {incr same_end} {
            if {![string equal [$col get $same_end] $ann_type]} {
              incr same_end -1
              break
            }
            ## An Annotation that matches our type...
            set AnnSet [list [tip_GetAnnotation $doc \
                                [$id get $same_end]]]
            if {[llength [tip_SelectAnnotations $AnnSet $ann_type $cons]] \
                                                                        == 0} {
              incr same_end -1
              break
            }
            set Progress [expr {$Progress + $ProgressStep}]
            ggi_wait_update {} $Progress $wwin
          }
          $w_list delete $i $same_end
          incr i -1
          set end [$col index end]
        }
      }
    } else {
      for {set i 0} {$i <= $end} {incr i; set same_end $i} {
        set Progress [expr {$Progress + $ProgressStep}]
        ggi_wait_update {} $Progress $wwin
        if {[string equal [$col get $i] $ann_type]} {
          for {incr same_end} {$same_end <= $end} {incr same_end} {
            if {![string equal [$col get $same_end] $ann_type]} {
              incr same_end -1
              break
            }
            set Progress [expr {$Progress + $ProgressStep}]
            ggi_wait_update {} $Progress $wwin
          }
          $w_list delete $i $same_end
          incr i -1
          set end [$col index end]
        }
      }
    };# if {[string length $cons]}...
    ::mclistbox::InvalidateScrollbars $w_list
    if {[$col index end] == 0} {
      $w_list column configure Attr -width 20
    }
    ggi_destroy_wait $wwin
  }
};# ggi_view_HTMLAnnotationExplorer_AnnotationToggle

proc ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow {parent toplevel \
     win label fg bg font {cmd {}}} {
  global GUI_Options ggi_view_HTMLAnnotationExplorer_Checked
  frame $win -bd 0 -relief flat -bg $GUI_Options(ExplorerBg)
    checkbutton $win.ck -bg $GUI_Options(ExplorerBg) -overrelief raised \
      -relief flat -highlightthickness 0 \
      -variable ggi_view_HTMLAnnotationExplorer_Checked($toplevel>$parent>$label)
    set canvas [winfo parent $win]
    bind $win.ck <MouseWheel> \
        "$canvas yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $win.ck <4> "$canvas yview scroll -5 units"
    bind $win.ck <5> "$canvas yview scroll  5 units"
    label $win.lb -bg $bg -fg $fg -text $label -font $font
    bind $win.lb <MouseWheel> \
        "$canvas yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $win.lb <4> "$canvas yview scroll -5 units"
    bind $win.lb <5> "$canvas yview scroll  5 units"
  pack $win.ck $win.lb -side left -fill y
  if {[string length $cmd]} {
    bind $win.lb <<MB_1>> "$cmd $win.lb $win.ck"
    #$win.ck configure -command "$cmd $win.lb $win.ck; $win.ck conf -command {}"
  }
};# ggi_view_HTMLAnnotationExplorer_CreateTreeItemWindow

proc ggi_view_HTMLAnnotationExplorer_TagAnnotation {win text doc type colour_ckbox \
                            ckbox HideMenu ShowMenu {cons {}} {label {}}} {
  set fg [$colour_ckbox cget -foreground]
  set bg [$colour_ckbox cget -background]
  set value [set ::[$ckbox cget -variable]]
  if {$value} {
    ## Create a special progress window that is embeded inside our main
    ## window!
    set wwin $win.ggi_wait_message
    if {!$::ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress)} {
      catch {destroy $wwin}
    }
    set ::GUI_Progress(Percent:$wwin) 0
    set ::GUI_Progress(Message:$wwin) $::GUI_Text(LoadingAnnotations)
    if {![winfo exists $wwin]} {
      frame $wwin -relief raised -bd 1
      label $wwin.label -textvariable ::GUI_Progress(Message:$wwin) \
        -width 45 -height 4 -justify left -anchor w
      ProgressBar $wwin.progress -variable ::GUI_Progress(Percent:$wwin) \
        -borderwidth 1
      pack $wwin.label -fill x -padx 20 -pady 20
      pack $wwin.progress -fill x -padx 30 -pady 10
      update
      place $wwin -relx 0.5 -rely 0.5 -anchor c
    }
    
    #set wwin [ggi_wait_message $::GUI_Text(LoadingAnnotations) 0.0 1 $dialog]
    set Values {}
    set Anns [tip_SelectAnnotations $doc $type $cons]
    if {[string length $label]} {
      set type $label
    }
    set AnnNu [llength $Anns]
    if {$AnnNu < 1} {
      #ggi_destroy_wait $wwin
      if {!$::ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress)} {
        destroy $wwin
      }
      return
    }
    set ProgressStep [expr {100.0/ $AnnNu}]; set Progress 0.0
    foreach {prev prev_index} {0 1.0} {}
    foreach ann $Anns {
      set Progress [expr {$Progress + $ProgressStep}]
      foreach span [tip_GetSpans $ann] {
        tip_GetSpanOffsets $span start end
        lappend Values [$text characterOffsetToIndex $start] \
                       [$text characterOffsetToIndex $end]
      }
      ggi_wait_update {} $Progress $wwin
    }
    if {[llength $Values]} {eval $text tag add [list $type] $Values}
    ggi_wait_update {} 100 $wwin
    if {[llength $Values]} {
      $text tag configure $type -background $bg -foreground $fg
      ggi_view_HTMLAnnotationExplorer_AddMenuEntry $text $HideMenu \
         $ShowMenu $type $fg $bg
      $text tag bind $type <Enter> \
        "set ::ggi_view_HTMLAnnotationExplorer_Priv($win>CurrentTag) [list $type]"
      $text tag bind $type <Leave> \
        "set ::ggi_view_HTMLAnnotationExplorer_Priv($win>CurrentTag) {}"
    }
    #ggi_destroy_wait $wwin
    if {!$::ggi_view_HTMLAnnotationExplorer_Priv($win:EmbbedProgress)} {
      destroy $wwin
    }
  } else {
    if {[string length $label]} {
      set type $label
    }
    catch {$text tag delete $type}
    ggi_view_HTMLAnnotationExplorer_RemoveMenuEntry $text $HideMenu \
      $ShowMenu $type $fg $bg
  }
};# ggi_view_HTMLAnnotationExplorer_TagAnnotation

proc ggi_view_HTMLAnnotationExplorer_AddMenuEntry {text HideMenu ShowMenu
                                                   label fg bg} {
  set cur_index [$HideMenu index end]
  set col_break 0
  if {[string equal $cur_index none]} {
    set cur_index 0
  } else {
    incr cur_index
  }
  if {[expr {$cur_index%15}] == 0} {set col_break 1} 
  set HideMenuButton [winfo parent $HideMenu]
  set ShowMenuButton [winfo parent $ShowMenu]
  $HideMenuButton configure -state normal
  $ShowMenuButton configure -state normal
  $HideMenu add command -label $label -background $bg -foreground $fg -command \
    "$text tag configure [list $label] -elide 1
     $HideMenu entryconfigure $cur_index -state disabled \
               -background {} -foreground {}
     $ShowMenu entryconfigure $cur_index -state normal \
               -background $bg -foreground $fg
     $HideMenuButton configure -relief flat
     $ShowMenuButton configure -relief flat" -columnbreak $col_break
  $ShowMenu add command -label $label -state disabled \
    -background {} -foreground {} -command \
    "$text tag configure [list $label] -elide 0
     $HideMenu entryconfigure $cur_index -state normal \
              -background $bg -foreground $fg
     $ShowMenu entryconfigure $cur_index -state disabled \
              -background {} -foreground {}
     $HideMenuButton configure -relief flat
     $ShowMenuButton configure -relief flat" -columnbreak $col_break
  set Viewer [winfo parent $text]
  $Viewer.frame.dot.menu add command -label $label -background $bg\
     -foreground $fg -columnbreak $col_break -command \
        [list $Viewer.frame.dot configure -bg $bg -text [list $label 1.0]]
};# ggi_view_HTMLAnnotationExplorer_AddMenuEntry

proc ggi_view_HTMLAnnotationExplorer_RemoveMenuEntry {text HideMenu ShowMenu \
                                                  label fg bg} {
  set cur_index [$HideMenu index end]
  if {[string equal $cur_index none]} {
    set cur_index 0
  }
  set Viewer [winfo parent $text]

  ## Iterate over all menu entries and remove all entries matching our
  ## label...
  for {set i $cur_index} {$i >= 0} {incr i -1} {
    if {[string equal [$HideMenu entrycget $i -label] $label]} {
      if {[string equal [$HideMenu entrycget $i -background] $bg] && \
          [string equal [$HideMenu entrycget $i -foreground] $fg]} {
        $HideMenu delete $i
        $ShowMenu delete $i
        $Viewer.frame.dot.menu delete $i
      } elseif {[string equal [$ShowMenu entrycget $i -background] $bg] && \
                [string equal [$ShowMenu entrycget $i -foreground] $fg]} {
        $HideMenu delete $i
        $ShowMenu delete $i
        $Viewer.frame.dot.menu delete $i
      }
    }
  }

  ## Should we deactivate the buttons?
  set cur_index [$HideMenu index end]
  if {[string equal $cur_index none]} {
    set HideMenuButton [winfo parent $HideMenu]
    set ShowMenuButton [winfo parent $ShowMenu]
    $HideMenuButton configure -state disabled
    $ShowMenuButton configure -state disabled
  }
};# ggi_view_HTMLAnnotationExplorer_RemoveMenuEntry

##############################################################################
## Explorer Annotation Deletion Mode...
##############################################################################

proc ggi_view_HTMLAnnotationExplorer_EnableAnnotationDeletionMode {win toolbutton
                 toolbar annTable doc} {
  Toolbar::configure $toolbar $toolbutton -image ITCancel \
    -tooltipvar GUI_Text(DeleteSelectedAnnotations) -command \
    "ggi_view_HTMLAnnotationExplorer_DeleteSelectedAnnotations $annTable $doc"
  $annTable configure -selectmode extended
};# ggi_view_HTMLAnnotationExplorer_EnableAnnotationDeletionMode

proc ggi_view_HTMLAnnotationExplorer_DeleteSelectedAnnotations {annTable doc} {
  set selection [$annTable curselection]
  if {![llength $selection]} {return}
  if {[llength $selection] == 1} {
    set msg $::GUI_Text(RemoveSelectedAnnotation?)
  } else {
    set msg [format $::GUI_Text(RemoveSelected%dAnnotations?) \
                        [llength $selection]]
  }
  if {[string equal [gut_question $msg] no]} {return}
  foreach index [lsort -decreasing -integer $selection] {
    set id [lindex [$annTable get $index] 0]
    ## Remove Annotation from Document...
    tip_RemoveAnnotation $doc $id
    $annTable delete $index
  }
};# ggi_view_HTMLAnnotationExplorer_DeleteSelectedAnnotations

##############################################################################
## Explorer Contextual Menu
##############################################################################

proc ggi_view_HTMLAnnotationExplorer_ExplorerContextualMenu {canvas path
     highlight_checkbutton annotation_checkbutton label_widget} {
  global GUI_Text ggi_view_HTMLAnnotationExplorer_Priv tcl_platform
  set win [winfo toplevel $canvas]
  set menu $canvas.contextualMenu
  catch {destroy $menu}
  set fg   [$annotation_checkbutton cget -foreground]
  set bg   [$annotation_checkbutton cget -background]
  set font [$annotation_checkbutton cget -font]
  menu $menu -tearoff 0
    $menu add command -label $path -background $bg -foreground $fg \
      -font $font -activebackground $bg -activeforeground $fg \
      -hidemargin 1
    set ggi_view_HTMLAnnotationExplorer_Priv($win:$menu:CM:TempVar1) \
      [set ::[$highlight_checkbutton cget -variable]]
    $menu add checkbutton -label "$GUI_Text(Highlight): $path" \
      -variable ggi_view_HTMLAnnotationExplorer_Priv($win:$menu:CM:TempVar1) \
      -command "$highlight_checkbutton invoke"
    set ggi_view_HTMLAnnotationExplorer_Priv($win:$menu:CM:TempVar2) \
      [set ::[$annotation_checkbutton cget -variable]]
    $menu add checkbutton -label "$GUI_Text(ShowAnnotations): $path" \
      -variable ggi_view_HTMLAnnotationExplorer_Priv($win:$menu:CM:TempVar2) \
      -onvalue { - } -offvalue { + } \
      -command "$annotation_checkbutton invoke"
    $menu add separator
    menu $menu.foreground -tearoff 0 -activeborderwidth 0 -activebackground \
      white
    ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu $menu.foreground \
       $annotation_checkbutton -foreground $label_widget
    $menu add cascade -label $GUI_Text(ChangeForegroundColour) \
      -background $fg -foreground $bg -menu $menu.foreground
    menu $menu.background -tearoff 0 -activeborderwidth 0 -activebackground \
      white
    ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu $menu.background \
       $annotation_checkbutton -background $label_widget
    $menu add cascade -label $GUI_Text(ChangeBackgroundColour) \
      -background $bg -foreground $fg -menu $menu.background
    if {[string equal $tcl_platform(platform) windows]} {
      $menu.foreground configure -background SystemMenu
      $menu.background configure -background SystemMenu
    }
    
  ## The colour menus...
  tk_popup $menu [winfo pointerx .] [winfo pointery .]
};# ggi_view_HTMLAnnotationExplorer_ExplorerContextualMenu

## ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu
#  This procedure will add the proper entries in a menu, that will enable
#  the user to select a colour from a set of predefined colours, or through a
#  selection dialog. The selected value will be set to the specified widget,
#  through the "colour_option" option (i.e. -background, -foreground etc.)
proc ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu {menu widget 
                                    colour_option args} {
  global GUI_Options ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu
  set initial_colour [$widget cget $colour_option]
  set Colors $GUI_Options(Colors)
  set index 0
  #foreach colour $Colors {}
  #  foreach {bg fg} $colour {}
  foreach r {0000 3300 6600 9900 CC00 FF00} {
    foreach g {0000 3300 6600 9900 CC00 FF00} {
      foreach b {0000 3300 6600 9900 CC00 FF00} {
        set bg "#$b$r$g" ;#"#$r$g$b"
        if {[info exists \
          ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu($bg)]} {
          set image $ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu($bg)
        } else {
          set image [image create photo ggi_view_HTMLAnnotationExplorer_Image$bg \
                           -height 16 -width 16]
          $image put $bg -to 0 0 16 16
          set ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu($bg) $image
        }
        set Cmd "$widget configure $colour_option $bg"
        foreach w $args {
          append Cmd "\;$w configure $colour_option $bg"
        }
        if {$index == 6} {
          $menu add command -image $image -columnbreak 1 \
            -command $Cmd -hidemargin 1
          set index 0
        } else {
          $menu add command -image $image \
            -command $Cmd -hidemargin 1
        }
        incr index
      }
    }
  }
  $menu add command -image [Bitmap::get palette] -hidemargin 1 -command \
    "ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenuSelectColour \
    $menu $widget $colour_option $args"
};# ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenu
proc ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenuSelectColour {
                                              menu widget colour_option args} {
  set initial [$widget cget $colour_option]
  ## Under Unix, show the BWidget dialog...
  if {[string equal $::tcl_platform(platform) unix]} {
    set colour [SelectColor::dialog $widget.selectBWColour \
                -parent [winfo toplevel $widget] -color $initial]
  } else {
    set colour [tk_chooseColor -parent [winfo toplevel $widget] \
                  -initialcolor $initial]
  }
  if {[string length $colour]} {
    $widget configure $colour_option $colour
    foreach w $args {
      $w configure $colour_option $colour
    }
  }
};# ggi_view_HTMLAnnotationExplorer_CreateColourSelectionMenuSelectColour

proc ggi_view_HTMLAnnotationExplorer_HighlightCallback {
                                             doc text listbox {index {}} args} {
  catch {$text tag delete show}
  if {[string length $index]} {
    foreach {ID Type x y} [$listbox get $index] {break}
    ## Update our Annotation Properties Dialog, if opened...
    if {[winfo exists $text.annotation_properties]} {
      ggi_view_Annotation $doc $ID {} {} $text.annotation_properties
      bind $text.annotation_properties \
        <<ggi_view_Annotation_AnnotationChanged>> \
        [list ggi_view_raw_AnnotationChangedCallback \
                  ggi_view_HTMLAnnotationExplorer_HighlightCallback \
                      $doc $text $listbox $index $ID]
    }
    if {![info exists x] || ![info exists y] || \
        ![string length $x] || ![string length $y]} {return} 
    set startIndex [$text characterOffsetToIndex $x]
    $text tag add show  $startIndex \
                        [$text characterOffsetToIndex $y]
    $text tag configure show -background blue -foreground white
    catch {
      ::GUI_TextSearchFacilities::HtmlSee $text $startIndex
    }
  }
  update
};# ggi_view_HTMLAnnotationExplorer_HighlightCallback
