##############################################################################
##  HTMLSingleSpanViewer.tcl:                                               ##
## ---------------------------                                              ##
##  This file implements the "single span" Annotation viewer for HTML       ##
##  Documents...                                                            ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

CDM_RegisterViewer {HTML Annotation Single Span Viewer} ggi_view_HTML_single_span ITShowTags
## ggi_view_HTML_single_span
#
proc ggi_view_HTML_single_span {doc annot attribute title} {
  if {![string length $attribute]} {
    return [ggi_view_raw $doc $annot {} $title 1]
  }
  global GUI_Text GUI_Options WMIcons ggi_view_single_span

  set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
  ## Get Document Annotations and text
  if [catch {set text_str [tip_GetByteSequence $doc]}] {
    ggi_destroy_wait $wwin
    error "ggi_view_HTML_single_span: Not Valid Document! ($doc)" {}
  }

  if {![string length $title]} {
    set title "Viewer: HTML Single Span, Document: [tip_GetId $doc],\
               Annotation: $annot, Attribute: $attribute"
  }

  ## Create the window
  set win [ggi_unused_win .ggi_view_single_span_html]
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Toolbar...
  set toolbar_frame $win.tf 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.viewer.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command \
        "GUI_EditPreferences .edt_prfrncs HTMLFonts"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::HtmlFindDialog $tw"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(SelectAnnsInRange...) \
        -image ITSearch -command [list ggi_view_raw_range $doc {} {} $title \
        0 $win $win.viewer.text]
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltip HTML... -image IWWW -command \
        [list ggi_view_single_span $doc $annot $attribute $title]
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    set ActiveLabel [Toolbar::addwidget $toolbar label -relief flat -bd 0 \
       -anchor e -textvariable ggi_view_single_span($win:CurrentTag)]
    bind $ActiveLabel <Destroy> \
      "unset -nocomplain ggi_view_single_span($win:CurrentTag)"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x
  
  set BFrame $win.but_frame
  set Viewer $win.viewer
  frame $BFrame -relief flat

  ## HTML Widget
  ::ScrolledWidgets::ScrolledHtml $Viewer -widgetname text \
    -bg $GUI_Options(HtmlBg) \
    -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
    -fg $GUI_Options(HtmlFg) -relief $GUI_Options(TextRelief) \
    -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
    -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
    -fontcommand      "::HtmlInfoViewer::PickFont $Viewer.text" \
    -imagecommand     "ggi_view_documentHTMLRetrieveImages $doc $Viewer.text" \
    -formcommand      "::HtmlInfoViewer::FormCmd $Viewer.text" \
    -bgimagecommand   "ggi_view_documentHTMLRetrieveBgImages $doc $Viewer.text"
  bind $Viewer.text.x <<MB_1>> \
      "set ::HtmlInfoViewer::InfoViewer($Viewer.text::SelectionStart) \
                                        \[$Viewer.text index @%x,%y\]
       $Viewer.text selection clear"
  bind $Viewer.text.x <<MB_2>> \
      [list ::HtmlInfoViewer::ContextualMenuHandler $Viewer.text $doc]
  bind $Viewer.text <Destroy> \
      [list +ggi_view_documentHTMLRetrieveImagesClean {} $Viewer.text]
  catch {$Viewer.text configure -base [tip_GetExternalId $doc]}
  ## We extend the scrolled widget to add search facilities...
  set cmdList \
    $::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($Viewer.horizontal_args)
  set \
    ::ScrolledWidgets::ScrolledWidgets_ScrollbarStatus($Viewer.horizontal_args)\
    [lreplace $cmdList 5 5 2]
  set grid_args [lrange $cmdList 6 end]
  eval grid $Viewer.text -sticky snew -row 0 -column 0 -rowspan 2 $grid_args
  frame $Viewer.frame -bd 0 -relief flat
    button $Viewer.frame.up      -image DoubleArrowUp   -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -command "ggi_view_single_spanFind $Viewer up"
    menubutton $Viewer.frame.dot -image GreyDot         -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -direction left -menu $Viewer.frame.dot.menu \
      -relief raised -text {{} 1.0}
    menu $Viewer.frame.dot.menu -tearoff 0
    button $Viewer.frame.bottom  -image DoubleArrowDown -highlightthickness 0 \
      -anchor c -bg #d4d0c8 -command "ggi_view_single_spanFind $Viewer down"
    pack $Viewer.frame.up $Viewer.frame.dot $Viewer.frame.bottom \
      -padx 0 -pady 0 -side top -expand 1 -fill x
  grid $Viewer.frame -sticky snew -row 1 -column 1
  #$Viewer.text insert end $text_str
  ## Add the text into the HTML widget...
  $Viewer.text parse $text_str

  ## Add Tags to Text Viewer...
  set Annotations [tip_SelectAnnotations $doc $annot]
  foreach {Values Data max_length Progress prev} {{} {} 3 0.0 0} {}
  set AnnNu [llength $Annotations]
  if {$AnnNu < 1} {set AnnNu 1}
  set ProgressStep [expr {100.0/ $AnnNu}]
#puts [time {
  foreach ann $Annotations {
    set Progress [expr {$Progress + $ProgressStep}]
    set attr [tip_GetAttribute $ann $attribute]
    if {[llength $attr]} {
      set value [tip_GetValueValue $attr]
      tip_GetSpanOffsets [tip_GetFirstSpan $ann] start end
      $Viewer.text tag add $value \
           [$Viewer.text characterOffsetToIndex $start] \
           [$Viewer.text characterOffsetToIndex $end]
      set ViewerTags($value) 0
      ggi_wait_update {} $Progress $wwin
    }
  }
  foreach value [lsort -ascii [array names ViewerTags]] {
    lappend Values $value
    set val [string length $value]
    if {$val > $max_length} {set max_length $val}
  }
  incr max_length 2
  set MaxButtonNu [expr {18- $max_length}]
  if {$MaxButtonNu < 6} {
    set MaxButtonNu 5
  } else {
    incr max_length -1
  }

  ## Hilight All Taggs and Add Buttons
  set Colors $GUI_Options(Colors)
  foreach {index bindex ival} {0 1 0} {}
  set color_nu [llength $Colors]
  set frame $BFrame.b
  frame $frame
  pack $frame -side top 
  button $frame.__all -textvariable GUI_Text(All) -borderwidth 1 -command \
   "ggi_view_HTML_single_span_hilight $Viewer.text {} [list $Values]" \
   -width $max_length -padx 1 -pady 1 -highlightthickness 0
  pack $frame.__all -side left -padx 1 -pady 0
  foreach value $Values {
    set val [list $value]
    if {$index == $color_nu} {set index 0}
    foreach {bg fg} [lindex $Colors $index] {}
    $Viewer.text tag configure $value -background $bg -foreground $fg
    $Viewer.text tag bind $value <Enter> \
      "set ggi_view_single_span($win:CurrentTag) [list \ $value\ ]"
    $Viewer.text tag bind $value <Leave> \
      "set ggi_view_single_span($win:CurrentTag) {}"
    if {$bindex == $MaxButtonNu} {
      set frame [ggi_unused_win ${BFrame}.b]
      frame $frame -relief flat
      pack $frame -side top -padx 0 -pady 0
      set bindex 0
    }
    set button [ggi_unused_win $frame.b${index}_]
    button $button -text "$value" -background $bg \
     -foreground $fg -borderwidth 1 -padx 1 -pady 1 -command \
     "ggi_view_HTML_single_span_hilight $Viewer.text $val \
       [list $Values] $fg $bg" -width $max_length -highlightthickness 0
    bind $button <<MB_2>> \
     "ggi_view_HTML_single_span_hilight $Viewer.text $val {} $fg $bg"
    pack $button -side left -padx 1 -pady 0
    incr index
    incr bindex
  }
#}]
  
  frame $BFrame.ok
  button $BFrame.ok.ok -textvariable GUI_Text(Dismiss) -command "destroy $win"
  ggi_destroy_wait $wwin
  pack $BFrame.ok.ok -pady 2
  pack $BFrame.ok -side bottom -fill x
  pack $BFrame -side bottom -fill x -expand 0
  pack $Viewer -side top -fill both -expand 1
  $Viewer.text configure -exportselection 1
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  ## Arrange the character index to be shown each time the user clicks on the
  ## text widget...
  bind $Viewer.text <<MB_1>> \
    "+ggi_DisplayTextIndexInLabel $Viewer.text $ActiveLabel %x %y"
  wm deiconify $win
  focus $Viewer.text
  return [list $win $Viewer.text]
};# ggi_view_single_span

## ggi_view_HTML_single_span_hilight
#   Hilight the appropriate attribute in single span viewer, or all if 
#   $tag == ""
proc ggi_view_HTML_single_span_hilight {text tag Tags {fg {}} {bg {}}} {
  global GUI_Options
  if {[string equal $tag {}]} {
    set index 0; set end [llength $GUI_Options(Colors)]
    foreach value $Tags {
      if {$index == $end} {set index 0}
      set color [lindex $GUI_Options(Colors) $index]
      $text tag configure $value -background \
       [lindex $color 0] -foreground [lindex $color 1]
      incr index
      $text tag lower $value
    }
  } else {
    if {![llength $Tags]} {
      ## See if tag is already activated, and if so, deactivate it.
      ## New feature, added 16/02/1999
      if {[$text tag cget $tag -background] == $bg} {
        if {[$text tag cget $tag -foreground] == $fg} {
           $text tag configure $tag -background $GUI_Options(HtmlBg) \
             -foreground $GUI_Options(HtmlFg)
           $text tag lower $tag
           return {}
        }
      }
    } else {
      foreach value $Tags {
        $text tag configure $value -background $GUI_Options(HtmlBg) \
         -foreground $GUI_Options(HtmlFg)
        $text tag lower $tag
      }
    }
    $text tag configure $tag -background $bg -foreground $fg
    $text tag raise $tag
  }
};# ggi_view_HTML_single_span_hilight
