##############################################################################
##  MorphologyViewer.tcl:                                                   ##
## -----------------------                                                  ##
##  This file implements a morphological information viewer...              ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

##############################################################################
## Morphological Viewer
##############################################################################
CDM_RegisterViewer {Morphology Viewer} ggi_view_morphology ITShowTags
## ggi_view_morphology
#
proc ggi_view_morphology {doc annot attribute title} {
  package require TclLexicon
  if {![string length $attribute]} {
    return [ggi_view_raw $doc $annot {} $title 1]
  }
  global GUI_Text GUI_Options WMIcons creole_HMorph_home

  set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
  ## Get Document Annotations and text
  if [catch {set text_str [tip_GetByteSequence $doc]}] {
    ggi_destroy_wait $wwin
    error "ggi_view_single_span: Not Valid Document! ($doc)" {}
  }

  ## Create the window
  set win [ggi_unused_win .ggi_view_morphology]
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.contents.left_frame.viewer.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x
  
  
  frame $win.contents -bd 0 -height 510 -width 850
  frame $win.contents.left_frame -bd 0
  set BFrame $win.contents.left_frame.but_frame
  set Viewer $win.contents.left_frame.viewer
  set MFrame $win.contents.morph_frame
  
  ## Create our Lexicon instance and morphology entries...
  frame $MFrame -relief flat -bd 0
  ## Initialise our Lexicon...
  ## Has already HMorph created a Lexicon?
  if {[llength [info commands creole_HMorph_LexiconObject]]} {
    set Lex creole_HMorph_LexiconObject
    set destroy_lexicon 0
  } else {
    set Lex [Lexicon Create \
                     [file join $creole_HMorph_home HellinicMorphLexicon]]
    set destroy_lexicon 1
  }
  ggi_view_morphologyCreateValuesViewer $MFrame $Lex
  
  frame $BFrame -relief flat

  ## Text Widget
  ::ScrolledWidgets::ScrolledText $Viewer
  $Viewer.text insert end $text_str

  ## Add Tags to Text Viewer...
  set Annotations [tip_SelectAnnotations $doc $annot {}]
  set Values {}; set max_length 3
  set AnnNu [llength $Annotations]
  if {$AnnNu < 1} {set AnnNu 1}
  set ProgressStep [expr {20.0/ $AnnNu}]; set Progress 0.0
# puts [time {
  foreach ann $Annotations {
    set Progress [expr {$Progress + $ProgressStep}]
    set attr [tip_GetAttribute $ann $attribute]
    if {[llength $attr]} {
      set value [tip_GetValueValue $attr]
      foreach {start end} [tip_GetFirstSpan $ann] {break}
      set Val [list $value]
      if {![info exists ViewerTags($Val)]} {
        CDM_LappendUnique Values $value
        set val [string length $value]
        if {$val > $max_length} {set max_length $val}
        ggi_wait_update {} $Progress $wwin
      }
      lappend ViewerTags($Val) 1.0+${start}c 1.0+${end}c
    }
  }
  set Values [lsort -ascii $Values]
  incr max_length 2
  set MaxButtonNu [expr {12- $max_length}]
  if {$MaxButtonNu < 3} {set MaxButtonNu 3}
  incr max_length -1

  ## Hilight All Taggs and Add Buttons
  set Colors $GUI_Options(Colors)
  set index 0; set bindex 1; set color_nu [llength $Colors]
  set frame ${BFrame}.b
  frame $frame
  pack $frame -side top 
  button ${frame}.__all -textvariable GUI_Text(All) -borderwidth 1 -command \
   "ggi_view_single_span_hilight $Viewer.text {} [list $Values]" \
   -width $max_length -padx 1 -pady 1 -highlightthickness 0
  pack ${frame}.__all -side left -padx 1 -pady 0
  set Progress 20.0
  set ProgressStep [expr {(100.0-$Progress)/($AnnNu*2.0)}]
  foreach value $Values {
    set val [list $value]
    set Progress [expr {$Progress+$ProgressStep*[llength $ViewerTags($val)]}]
    eval $Viewer.text tag add $val $ViewerTags($val)
    if {$index == $color_nu} {set index 0}
    foreach {bg fg} [lindex $Colors $index] {break}
    $Viewer.text tag configure $value -background $bg -foreground $fg
    $Viewer.text tag bind $value <<MB_1>> "ggi_view_morphologyHandleClick \
       $Lex $MFrame $doc $annot $attribute %W %x %y"
    if {$bindex == $MaxButtonNu} {
      set frame [ggi_unused_win $BFrame.b]
      frame $frame -relief flat -bd 0
      pack $frame -side top -padx 0 -pady 0
      set bindex 0
    }
    set button [ggi_unused_win ${frame}.b${index}_]
    button $button -text "$value" -background $bg \
     -foreground $fg -borderwidth 1 -padx 1 -pady 1 -command \
     "ggi_view_single_span_hilight ${Viewer}.text $val \
       [list $Values] $fg $bg" -width $max_length -highlightthickness 0
    bind $button <<MB_2>> \
     "ggi_view_single_span_hilight ${Viewer}.text $val {} $fg $bg"
    pack $button -side left -padx 1 -pady 0
    incr index
    incr bindex
    ggi_wait_update {} $Progress $wwin
  }
# }]
  $Viewer.text configure -state disabled -exportselection 1
  
  pack $BFrame -side bottom -fill x -expand 1
  pack $Viewer -side top -fill both -expand 1
  pane $win.contents.left_frame $win.contents.morph_frame
  pack $win.contents -side top -fill both -expand 1

  ## Button Frame
  frame $win.buttons
  button $win.buttons.ok -textvariable GUI_Text(Dismiss) -command "destroy $win"
  if {$destroy_lexicon} {
    bind $win.buttons.ok <Destroy> "rename $Lex {}"
  }
  pack $win.buttons.ok
  pack $win.buttons -side top -fill x -pady 5
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  gut_ModalDlg $win
  ggi_destroy_wait $wwin
  wm deiconify $win
  return [list $win $Viewer.text]
};# ggi_view_morphology

## This function is our callback when the user clicks any visible item in our
## text widget. Form the index, we find one annotation to get the text. Then
## we lookup this form in the Lexicon and we display the results...
proc ggi_view_morphologyHandleClick {Lex win doc annot attribute text x y} {
  if {[catch {$text index "@$x,$y"} index]} {return}
  set anns [tip_AnnotationsContaining $doc \
               [gut_FindCharacterOffset $text $index]]
  foreach ann $anns {
    if {[string equal $annot [tip_GetType $ann]]} {break}
  }
  foreach {start end} [lindex [tip_GetSpans $ann] 0] {break}
  incr end -1
  set Text [tip_GetByteSequence $doc]
  set word [string tolower [string range $Text $start $end]]
  ggi_view_morphologyUpdateDlg $Lex $win $word
};# ggi_view_morphologyHandleClick

## Create Morphological Values Viewer...
proc ggi_view_morphologyCreateValuesViewer {win Lex} {
  global GUI_Text
  set frame $win.word_properties
  frame $frame -bg #8fbc8f
    ## Word Form:
    label $frame.l_Form -textvar GUI_Text(Form:) -anchor w \
          -bg #8fbc8f -fg #2f4f4f
    entry $frame.e_Form -fg #2f4f4f -justify center \
          -highlightbackground #8fbc8f
    grid  $frame.l_Form -sticky snew -padx 2 -pady 1
    grid  $frame.e_Form -row 0 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
   
    ## Word Lemma:
    label $frame.l_Lemma -textvar GUI_Text(Lemma:) -anchor w \
          -bg #8fbc8f -fg #2f4f4f
    entry $frame.e_Lemma -fg #2f4f4f -justify center \
          -highlightbackground #8fbc8f
    grid  $frame.l_Lemma -sticky snew -padx 2 -pady 1
    grid  $frame.e_Lemma -row 1 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1

    ## Word Stem & Suffix:
    label $frame.l_Stem -textvar GUI_Text(Stem:) -anchor w \
          -bg #8fbc8f -fg #2f4f4f
    entry $frame.e_Stem -fg #2f4f4f -justify center \
          -highlightbackground #8fbc8f
    grid  $frame.l_Stem -row 2 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Stem -row 2 -column 1 -sticky snew -padx 2 -pady 1
    label $frame.l_Suffix -textvar GUI_Text(Suffix:) -anchor w \
          -bg #8fbc8f -fg #2f4f4f
    entry $frame.e_Suffix -fg #2f4f4f -justify center \
          -highlightbackground #8fbc8f
    grid  $frame.l_Suffix -row 2 -column 2 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Suffix -row 2 -column 3 -sticky snew -padx 2 -pady 1

    ## Syllables:
    label $frame.l_Syllables -textvar GUI_Text(Syllables:) -anchor w \
          -bg #8fbc8f -fg #2f4f4f
    entry $frame.e_Syllables -fg #2f4f4f -justify center \
          -highlightbackground #8fbc8f
    grid  $frame.l_Syllables -sticky snew -padx 2 -pady 1
    grid  $frame.e_Syllables -row 3 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1

    grid columnconfigure $frame 1 -weight 1
    grid columnconfigure $frame 3 -weight 1
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2
    
  set frame $win.addword_properties
  frame $frame -bg #ffb6c1
    ## Pos & Number:
    label $frame.l_Pos -textvar GUI_Text(Pos:) -anchor w -bg #ffb6c1 -fg #FF0000
    entry $frame.e_Pos -fg #FF0000 -justify center \
          -highlightbackground #ffb6c1
    grid  $frame.l_Pos -row 0 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Pos -row 0 -column 1 -sticky snew -padx 2 -pady 1
    label $frame.l_Number -textvariable GUI_Text(Number:) -anchor w \
          -bg #ffb6c1 -fg #FF0000
    entry $frame.e_Number -fg #FF0000 -justify center \
          -highlightbackground #ffb6c1
    grid  $frame.l_Number -row 0 -column 2 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Number -row 0 -column 3 -sticky snew -padx 2 -pady 1
    label $frame.l_PosDetail -textvar GUI_Text(PosDetail:) -anchor w \
          -bg #ffb6c1 -fg #FF0000
    entry $frame.e_PosDetail -fg #FF0000 -justify center \
          -highlightbackground #ffb6c1
    grid  $frame.l_PosDetail -row 1 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_PosDetail -row 1 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
    
    ## AccentType & AccentedSyllable:
    label $frame.l_AccentType -textvar GUI_Text(AccentType:) -anchor w \
          -bg #ffb6c1 -fg #FF0000 -height 2 -justify left
    entry $frame.e_AccentType -fg #FF0000 -justify center \
          -highlightbackground #ffb6c1
    grid  $frame.l_AccentType -row 2 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_AccentType -row 2 -column 1 -sticky   ew -padx 2 -pady 1
    label $frame.l_AccentedSyllable -textvar GUI_Text(AccentedSyllable:) \
          -anchor w -bg #ffb6c1 -fg #FF0000 -height 2 -justify left
    entry $frame.e_AccentedSyllable -fg #FF0000 -justify center \
          -highlightbackground #ffb6c1
    grid  $frame.l_AccentedSyllable -row 2 -column 2 -sticky snew \
          -padx 2 -pady 1
    grid  $frame.e_AccentedSyllable -row 2 -column 3 -sticky   ew \
          -padx 2 -pady 1
    
    grid columnconfigure $frame 1 -weight 1
    grid columnconfigure $frame 3 -weight 1
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2

  set frame $win.noun_properties
  frame $frame -bg #b0c4de
    ## Gender & Case:
    label $frame.l_Gender -textvar GUI_Text(Gender:) -anchor w \
          -bg #b0c4de -fg #0000cd
    entry $frame.e_Gender -fg #0000cd -justify center \
          -highlightbackground #b0c4de
    grid  $frame.l_Gender -row 0 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Gender -row 0 -column 1 -sticky snew -padx 2 -pady 1
    label $frame.l_Case -textvar GUI_Text(Case:) -anchor w \
          -bg #b0c4de -fg #0000cd
    entry $frame.e_Case -fg #0000cd -justify center \
          -highlightbackground #b0c4de
    grid  $frame.l_Case -row 0 -column 2 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Case -row 0 -column 3 -sticky snew -padx 2 -pady 1

    ## Inflexion:
    label $frame.l_Inflexion -textvar GUI_Text(Inflexion:) -anchor w \
          -bg #b0c4de -fg #0000cd
    entry $frame.e_Inflexion -fg #0000cd -justify center \
          -highlightbackground #b0c4de
    grid  $frame.l_Inflexion -row 1 -column 0 -columnspan 1 -sticky snew \
          -padx 2 -pady 1
    grid  $frame.e_Inflexion -row 1 -column 1 -columnspan 3 -sticky snew \
          -padx 2 -pady 1

    grid columnconfigure $frame 1 -weight 1
    grid columnconfigure $frame 3 -weight 1
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2

  set frame $win.verb_properties
  frame $frame -bg #d2b48c
    ## Tense:
    label $frame.l_Tense -textvar GUI_Text(Tense:) -anchor w \
          -bg #d2b48c -fg #8b0000
    entry $frame.e_Tense -fg #8b0000 -justify center \
          -highlightbackground #d2b48c
    grid  $frame.l_Tense -sticky snew -padx 2 -pady 1
    grid  $frame.e_Tense -row 0 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
   
    ## Person & Voice:
    label $frame.l_Person -textvar GUI_Text(Person:) -anchor w \
          -bg #d2b48c -fg #8b0000
    entry $frame.e_Person -fg #8b0000 -justify center \
          -highlightbackground #d2b48c
    grid  $frame.l_Person -row 1 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Person -row 1 -column 1 -sticky snew -padx 2 -pady 1
    label $frame.l_Voice -textvar GUI_Text(Voice:) -anchor w \
          -bg #d2b48c -fg #8b0000
    entry $frame.e_Voice -fg #8b0000 -justify center \
          -highlightbackground #d2b48c
    grid  $frame.l_Voice -row 1 -column 2 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Voice -row 1 -column 3 -sticky snew -padx 2 -pady 1
  
    ## Mood & Egklish:
    label $frame.l_Mood -textvar GUI_Text(Mood:) -anchor w \
          -bg #d2b48c -fg #8b0000
    entry $frame.e_Mood -fg #8b0000 -justify center \
          -highlightbackground #d2b48c
    grid  $frame.l_Mood -row 2 -column 0 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Mood -row 2 -column 1 -sticky snew -padx 2 -pady 1
    label $frame.l_Egklish -textvar GUI_Text(Egklish:) -anchor w \
          -bg #d2b48c -fg #8b0000
    entry $frame.e_Egklish -fg #8b0000 -justify center \
          -highlightbackground #d2b48c
    grid  $frame.l_Egklish -row 2 -column 2 -sticky snew -padx 2 -pady 1
    grid  $frame.e_Egklish -row 2 -column 3 -sticky snew -padx 2 -pady 1
    
    grid columnconfigure $frame 1 -weight 1
    grid columnconfigure $frame 3 -weight 1
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2

  set frame $win.various_properties
  frame $frame -bg #f5f5dc
    ## Info:
    label $frame.l_Info -textvar GUI_Text(Info:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Info -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Info -sticky snew -padx 2 -pady 1
    grid  $frame.e_Info -row 0 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
    ## Explanation:
    label $frame.l_Explanation -textvar GUI_Text(Explanation:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Explanation -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Explanation -sticky snew -padx 2 -pady 1
    grid  $frame.e_Explanation -row 1 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
    ## Examples:
    label $frame.l_Examples -textvar GUI_Text(Examples:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Examples -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Examples -sticky snew -padx 2 -pady 1
    grid  $frame.e_Examples -row 2 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
    ## Synonyms:
    label $frame.l_Synonyms -textvar GUI_Text(Synonyms:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Synonyms -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Synonyms -sticky snew -padx 2 -pady 1
    grid  $frame.e_Synonyms -row 3 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1
    ## Translation:
    label $frame.l_Translation -textvar GUI_Text(Translation:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Translation -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Translation -sticky snew -padx 2 -pady 1
    grid  $frame.e_Translation -row 4 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1

    ## Category:
    label $frame.l_Category -textvar GUI_Text(Category:) -anchor w \
          -bg #f5f5dc -fg #8b4513
    entry $frame.e_Category -fg #8b4513 -justify center \
          -highlightbackground #f5f5dc
    grid  $frame.l_Category -sticky snew -padx 2 -pady 1
    grid  $frame.e_Category -row 5 -column 1 -columnspan 3 \
          -sticky snew -padx 2 -pady 1

    grid columnconfigure $frame 1 -weight 1
    grid columnconfigure $frame 3 -weight 1
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2
  set frame $win.spinbox
  frame $frame -bg #ffdab9
    label $frame.info -textvar GUI_Text(DisplayEntry:) -bg #ffdab9 -fg #cd853f
    spinbox $frame.spin -from 1 -to 1 -state readonly -exportselection 0 \
       -highlightbackground #ffdab9 \
       -command "ggi_view_morphologyUpdateDlg $Lex $win"
    label $frame.info2 -bg #ffdab9 -fg #cd853f
    grid $frame.info $frame.spin $frame.info2 -sticky snew -padx 2 -pady 2
  grid $frame -columnspan 3 -sticky snew -padx 2 -ipadx 2 -ipady 2
  
  grid columnconfigure $win 1 -weight 1
  grid rowconfigure    $win 6 -weight 1
};# ggi_view_morphologyCreateValuesViewer

## ggi_view_morphologyUpdateDlg
#  This function updates the displayed into the dialog values, according to
#  the word in the specified entry widget
proc ggi_view_morphologyUpdateDlg {Lex win {word {}}} {
  if {[string length $word]} {
    set entries [$Lex FindWord $word]
    if {$entries} {
      $win.spinbox.spin configure -to $entries
      $win.spinbox.spin set 1
      $win.spinbox.info2 configure -text "of $entries"
    }
    set entry 1
  } else {
    set entries [$Lex GetEntryCount]
    set entry   [$win.spinbox.spin get]
  }

  if {$entries} {
    incr entry -1
    set obj [$Lex GetEntry $entry]
    if {[string length $word]} {
      $win.word_properties.e_Form delete 0 end
      $win.word_properties.e_Form insert end $word
    }
    foreach {cat} {Lemma Stem Suffix Syllables} {
      $win.word_properties.e_$cat delete 0 end
      $win.word_properties.e_$cat insert end [eval $obj Get$cat]
    }
    foreach {cat} {Pos PosDetail Number AccentType AccentedSyllable} {
      $win.addword_properties.e_$cat delete 0 end
      $win.addword_properties.e_$cat insert end \
        [ggi_view_morphologyUpdateDlgTranslate [eval $obj Get$cat]]
    }
    foreach {cat} {Gender Case Inflexion} {
      $win.noun_properties.e_$cat delete 0 end
      $win.noun_properties.e_$cat insert end \
        [ggi_view_morphologyUpdateDlgTranslate [eval $obj Get$cat]]
    }
    foreach {cat} {Tense Person Voice Mood Egklish} {
      $win.verb_properties.e_$cat delete 0 end
      $win.verb_properties.e_$cat insert end \
        [ggi_view_morphologyUpdateDlgTranslate [eval $obj Get$cat]]
    }
    foreach {cat} {Info Explanation Examples Synonyms Translation Category} {
      $win.various_properties.e_$cat delete 0 end
      $win.various_properties.e_$cat insert end [eval $obj Get$cat]
    }
    rename $obj {}
  } else {
    foreach {cat} {Form Lemma Stem Suffix Syllables} {
      $win.word_properties.e_$cat delete 0 end
    }
    foreach {cat} {Pos PosDetail Number AccentType AccentedSyllable} {
      $win.addword_properties.e_$cat delete 0 end
    }
    foreach {cat} {Gender Case Inflexion} {
      $win.noun_properties.e_$cat delete 0 end
    }
    foreach {cat} {Tense Person Voice Mood Egklish} {
      $win.verb_properties.e_$cat delete 0 end
    }
    foreach {cat} {Info Explanation Examples Synonyms Translation Category} {
      $win.various_properties.e_$cat delete 0 end
    }
    $win.spinbox.spin configure -to 1
    $win.spinbox.spin set 1
    $win.spinbox.info2 configure -text {}
  }
};# ggi_view_morphologyUpdateDlg
proc ggi_view_morphologyUpdateDlgTranslate {text} {
  if {[info exists ::GUI_Text($text)]} {return $::GUI_Text($text)}
  return $text
}
