##############################################################################
##############################################################################
##
## This file implements the Tree parse viewer.
## Author: Georgios Petasis, petasis@iit.demokritos.gr
## This file is part of the "Ellogon" text engineering platform.
##
#############################################################################
#############################################################################
package require Tree
CDM_RegisterViewer {Parse Tree Viewer} ggi_view_parse

proc ggi_view_parse {doc type attr title {win .ggi_view_parse}} {
  global GUI_Text GUI_Options GUI_ParseViewer WMIcons PANE
  set win [ggi_unused_win $win]
  toplevel $win
  wm title $win $title
  wm withdraw $win

  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)
    set tw $win.frame.viewer.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  set Viewer   $win.frame.viewer
  set area     $win.frame.area
  set Tree     $area.tree
  set TreeText $area.text
  frame $win.frame -relief flat -bd 0 -width 750 -height 450
  ## Text Widget
  ::ScrolledWidgets::ScrolledText $Viewer

    ## Load the text widget...
    $Viewer.text insert end [tip_GetByteSequence $doc]
    foreach sentence [tip_SelectAnnotations $doc sentence] {
      foreach {id ann_type spans attrs} $sentence {}
      foreach {start end} [lindex $spans 0] {}
      $Viewer.text tag add s_$id 1.0+${start}c 1.0+${end}c
      $Viewer.text tag bind s_$id <Enter> "ggi_view_parseTextCallback \
        $doc $type $attr $Viewer.text $area s_$id <Enter>"
      $Viewer.text tag bind s_$id <Leave> "ggi_view_parseTextCallback \
        $doc $type $attr $Viewer.text $area s_$id <Leave>"
      $Viewer.text tag bind s_$id <1>     "ggi_view_parseTextCallback \
        $doc $type $attr $Viewer.text $area s_$id <1>"
    }

  frame $win.frame.area -relief flat -bd 0
  ## Tree Widget
  ::ScrolledWidgets::ScrolledCanvas $Tree -regionpadx 10 -regionpady 10
  tree $Tree.canvas.tree -layout vertical
  ::ScrolledWidgets::ScrolledText $TreeText -height 4 -state disabled
  pack $Tree     -side top -pady 0 -padx 0 -fill both -expand 1
  pack $TreeText -side top -pady 0 -padx 0 -fill x
  pane $Viewer $win.frame.area -orient horizontal
  ArrowButton $win.frame.__h1.b1 -type button -width 6 -height 60 \
    -dir left -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.frame 1 0"
  ::$win.frame.__h1.b1:cmd configure -cursor left_side
  ArrowButton $win.frame.__h1.b2  -type button -width 6 -height 60 \
    -dir right -bd 0 -highlightthickness 0 -bg $PANE(Color) -command \
    "Widget_PaneSet $win.frame 1 0.3"
  ::$win.frame.__h1.b2:cmd configure -cursor right_side
  Widget_PaneSet $win.frame 1 0.3
  pack $win.frame.__h1.b1 $win.frame.__h1.b2 -side top
  pack $win.frame -fill both -expand 1

  ## Button frame
  frame $win.buttons -relief flat -bd 0
    button $win.buttons.ok -textvar GUI_Text(Ok) -command "destroy $win"
    bind $win.buttons.ok <Destroy> "pane forget $win.frame
       array unset ggi_view_parseTextCallback_FillTree $Tree.canvas*"
    pack $win.buttons.ok -side top -pady 2
  pack $win.buttons -side bottom -fill x -expand 0
  update
  wm deiconify $win
};# ggi_view_parse

proc ggi_view_parseTextCallback {doc type attr text area tag event} {
  $text configure -state normal
  switch -exact $event {
    <Enter> {
      $text tag configure $tag -background blue -foreground white
    }
    <Leave> {
      $text tag configure $tag -background [$text cget -background] \
                               -foreground [$text cget -foreground]
    }
    <1> {
      set canvas $area.tree.canvas
      set c_text $area.text.text
      set id [string range $tag 2 end]
      set sentence [tip_GetAnnotation $doc $id]

      ## Place the sentence text in the text widget under the tree widget...
      set Text [tip_GetFirstAnnotatedTextRange [tip_GetRawData $doc] $sentence]
      $c_text configure -state normal
      $c_text delete 1.0 end
      $c_text insert end $Text
      $c_text configure -state disabled

      ## Handle the sentence parse...
      foreach {start end} [tip_GetFirstSpan $sentence] {}
      set Anns [tip_AnnotationsInRange [tip_SelectAnnotations $doc $type] \
               $start $end]
      ## Reset the tree widget...
      foreach child [$canvas.tree subnodes {}] {
        $canvas.tree rmlink $child
      }

      global ggi_view_parseTextCallback_FillTree GUI_Options
      set ggi_view_parseTextCallback_FillTree($canvas:ID) 0
      set ggi_view_parseTextCallback_FillTree($canvas:Categories) {}
      set ggi_view_parseTextCallback_FillTree($canvas:Words) {}
      set ggi_view_parseTextCallback_FillTree($canvas:TextStart) \
        [lindex [tip_GetFirstSpan $sentence] 0]

      switch -exact -- $type {
        parse {
          foreach ann $Anns {
            set parse [tip_GetAttribute $ann $attr]
            if {![llength $parse]} {continue}
            ggi_view_parseTextCallback_FillTree $canvas {} \
                                            [tip_GetValueValue $parse]
          }
        }
        edge -
        default {
          ## 1. Find the annotations covering the whole sentence...
          set Heads [tip_AnnotationsMatchingRange $Anns $start $end]
          foreach head $Heads {
           ggi_view_parseTextCallback_TreeFromAnns $doc $head $canvas $c_text {}
          }
        }
      }
      ## Set some nice colors...
      set Colors $GUI_Options(Colors)
      set index 0; set color_nu [llength $Colors]
      lappend ggi_view_parseTextCallback_FillTree($canvas:Categories) _Word
      foreach cat $ggi_view_parseTextCallback_FillTree($canvas:Categories) {
        foreach {bg fg} [lindex $Colors $index] {break}
        $canvas itemconfigure r_$cat -fill $bg
        $canvas itemconfigure t_$cat -fill $fg
        if {$index >= $color_nu} {set index 0}
        incr index
      }
    }
  }
  $text configure -state disabled
};# ggi_view_parseTextCallback

## This function creates the parse tree, by simply checking the "constituents"
## attribute of the parent annotation...
proc ggi_view_parseTextCallback_TreeFromAnns {doc parent canvas text tp} {
  global ggi_view_parseTextCallback_FillTree
  set cons [tip_GetAttribute $parent constituents]
  if {![llength $cons]} {return}
  set cons [tip_GetValueValue $cons]

  set head [tip_GetAttribute $parent type]
  if {![llength $head]} {return}
  set head [tip_GetValueValue $head]
  set id   [tip_GetId $parent]

  ## Is it a word?
  if {[llength $cons]} {
    ## A category...
    CDM_LappendUnique ggi_view_parseTextCallback_FillTree($canvas:Categories) \
        $head
    set bgid [$canvas create rectangle 0 0 64 25 -tags [list r_$head n$id]]
    set txid [$canvas create text 32 12 -text $head -tags [list t_$head n$id] \
      -font ExplorerFont]
  } else {
    ## A word...
    ## Get Document text...
    set head [tip_GetFirstAnnotatedTextRange [tip_GetRawData $doc] $parent]
    set bgid [$canvas create oval 0 0 64 25 -tags [list r__Word n$id]]
    set txid [$canvas create text 32 12 -text $head -font ExplorerFont \
      -tags [list t__Word n$id]]
  }
  foreach {x y w h} [$canvas bbox $txid] {break}
  if {$w > 64} {$canvas coords $bgid $x 0 $w 25}
  set line [$canvas create line 0 0 0 0 -arrow last -fill blue -smooth 1 \
                                        -width 1]
  $canvas.tree addlink $tp n$id $line -border 3
  $canvas bind n$id <<MB_1>> \
    "ggi_view_parseTextCallback_Hilight $canvas $text $doc $id"

  ## Add children...
  foreach Id $cons {
    ggi_view_parseTextCallback_TreeFromAnns $doc [tip_GetAnnotation $doc $Id] \
                                                      $canvas $text n$id
  }
};# ggi_view_parseTextCallback_TreeFromAnns

## This function creates the parse tree...
proc ggi_view_parseTextCallback_FillTree {canvas parent parse} {
  global ggi_view_parseTextCallback_FillTree
  set item $ggi_view_parseTextCallback_FillTree($canvas:ID)
  incr ggi_view_parseTextCallback_FillTree($canvas:ID)
  if {[catch {lindex $parse 0} head]} {
    if {[string equal $parse \"]} {
      set parse {\"}
      set head  \"
    } else {
      puts "Error: $parse"
      return
    }
  }
  ## Is a syntactic category or a word?
  if {[llength $parse] == 1} {
    lappend ggi_view_parseTextCallback_FillTree($canvas:Words) $head
    set word_id [llength $ggi_view_parseTextCallback_FillTree($canvas:Words)]
    set bgid [$canvas create oval 0 0 64 25 -tags \
                      [list r__Word ${item}_$head wi:$word_id]]
    set txid [$canvas create text 32 12 -text $head -font ExplorerFont -tags \
                      [list t__Word ${item}_$head wi:$word_id]]
    #$canvas bind ${item}_$head <<MB_1>> "puts [ $word_id]"
  } else {
    CDM_LappendUnique ggi_view_parseTextCallback_FillTree($canvas:Categories) \
        $head
    set bgid [$canvas create rectangle 0 0 64 25    -tags \
                      [list r_$head ${item}_$head]]
    set txid [$canvas create text 32 12 -text $head -tags \
                      [list t_$head ${item}_$head] -font ExplorerFont]
  }
  foreach {x y w h} [$canvas bbox $txid] {break}
  if {$w > 64} {$canvas coords $bgid $x 0 $w 25}
  set line [$canvas create line 0 0 0 0 -arrow last -fill blue -smooth 1 \
                                        -width 1]
  $canvas.tree addlink $parent ${item}_$head $line -border 3
  foreach child [lrange $parse 1 end] {
     ggi_view_parseTextCallback_FillTree $canvas ${item}_$head $child
  }
}

proc ggi_view_parseTextCallback_Hilight {canvas text doc id} {
  global ggi_view_parseTextCallback_FillTree
  set ann [tip_GetAnnotation $doc $id]
  foreach {start end} [tip_GetFirstSpan $ann] {}
  incr start -$ggi_view_parseTextCallback_FillTree($canvas:TextStart)
  incr end   -$ggi_view_parseTextCallback_FillTree($canvas:TextStart)
  $text configure -state normal
  $text tag delete sel_span
  $text tag add sel_span 1.0+${start}c 1.0+${end}c
  $text tag configure sel_span -background blue -foreground white
  $text see 1.0+${start}c
  $text configure -state disable
};# ggi_view_parseTextCallback_Hilight
