##############################################################################
##  ParseViewer2.tcl:                                                       ##
## -------------------                                                      ##
##  This file implements a parse tree viewer...                             ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
CDM_RegisterViewer {Parse Tree Viewer (2)} ggi_view_parse2

## ggi_view_parse2
#   Tree Hierarchy Viewer
proc ggi_view_parse2 {doc type dummy title {win .ggi_view_parse}} {
  global GUI_Text GUI_Options GUI_ParseViewer WMIcons
  set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
  set win [ggi_unused_win $win]
  toplevel $win
  wm title $win $title
  wm withdraw $win

  set Text      [tip_GetRawData $doc]
  set Sentences [tip_SelectAnnotations $doc sentence]
  set Progress  0.0; set nu [llength $Sentences]; incr nu
  set step      [expr {90.0/$nu}]

  ## Toolbar...
  set toolbar_frame ${win}.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    set ComboBox $toolbar.widgets.sentence; set NoteBook $win.tree.nb
    Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) \
         -foreground $GUI_Options(InputForeground) \
         -name $ComboBox -text {Select Sentence...} \
         -modifycmd "ggi_view_parseAddPage $NoteBook $ComboBox" \
         -tooltipvar GUI_Text(TSelectCollection) -width 75
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y
  pack $toolbar_frame -side top -fill x

  set ComboList {}
  foreach sentence $Sentences {
    set Progress [expr {$Progress + $step}]
    foreach {start end} [tip_GetFirstSpan $sentence] {break}
    lappend ComboList [string range $Text $start $end]
    ggi_wait_update {} $Progress $wwin
  }
  $ComboBox configure -values $ComboList

  ## NoteBook Widget...
  frame $win.tree
  NoteBook $NoteBook -side top -width 450 -height 350
  pack $NoteBook -fill both -expand 1
  pack $win.tree -fill both -expand 1

  ## Buttons...
  frame $win.buttons
  set progress1 $win.buttons.prg1; set progress2 $win.buttons.prg2
  ProgressBar $progress1 -variable GUI_ParseViewer($NoteBook:Prg1) -bd 1
  ProgressBar $progress2 -type infinite -variable \
    GUI_ParseViewer($NoteBook:Prg2) -bd 1
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command "destroy $win"
  grid $progress1 $win.buttons.dismiss $progress2 -sticky ew -padx 4
  grid columnconfigure $win.buttons 0 -weight 1
  grid columnconfigure $win.buttons 1 -weight 0
  grid columnconfigure $win.buttons 2 -weight 1
  pack $win.buttons -side bottom -pady 3 -expand 1 -fill x

  ## Global Variables...
  set GUI_ParseViewer($NoteBook:Prg1) -1
  set GUI_ParseViewer($NoteBook:Prg2) -1
  set GUI_ParseViewer($NoteBook:Doc)  $doc
  set GUI_ParseViewer($NoteBook:Type) $type
  set GUI_ParseViewer($NoteBook:Anns) [tip_SelectAnnotationsSorted $doc $type]
  set GUI_ParseViewer($NoteBook:Sentences) $Sentences
  ggi_wait_update {} 100.0 $wwin
  bind $win <Destroy> "array unset GUI_ParseViewer($NoteBook:*)"

  ggi_destroy_wait $wwin
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
}

## ggi_view_parseAddPage
#   Add a new page for the selected sentence...
proc ggi_view_parseAddPage {NoteBook ComboBox} {
  set index     [$ComboBox getvalue]
  set pages     [$NoteBook pages]
  set id        [lsearch -exact $pages page_$index]
  if {$id != -1} {
    set page [lindex $pages $id]
    $NoteBook raise $page
    $NoteBook see $page
    return
  }
  set id   [llength $pages]
  set page page_$index
  $NoteBook insert end $page -text $id -createcmd \
     "ggi_view_parseCreatePage $NoteBook $ComboBox $page $index"
  $NoteBook raise $page
  $NoteBook see $page
}

## ggi_view_parseCreatePage
#   Create the widgets for each page in the notebook widget of the "parse"
#   viewer...
proc ggi_view_parseCreatePage {NoteBook ComboBox page index} {
  global GUI_Options
  set Sentences [$ComboBox cget -values]
  set text      [lindex $Sentences $index]
  set page_frame [$NoteBook getframe $page]
  ::ScrolledWidgets::ScrolledText $page_frame.label -height 4 
  $page_frame.label.text insert end $text
  $page_frame.label.text configure -state disabled
  ::ScrolledWidgets::ScrolledCanvas $page_frame.canvas
  pack $page_frame.label -fill x -expand 1 -side top -padx 5 -pady 2
  pack $page_frame.canvas -fill both
  ggi_view_parseCreateTree $page_frame.canvas.canvas $NoteBook $ComboBox $index
};# ggi_view_parseCreatePage

## ggi_view_parseCreateTree
#    Given a canvas object, draw the tree that annotations define...
proc ggi_view_parseCreateTree {Canvas NoteBook ComboBox index} {
  global GUI_ParseViewer GUI_Text

  set sentence [lindex $GUI_ParseViewer($NoteBook:Sentences) $index]
  set Anns $GUI_ParseViewer($NoteBook:Anns)
  set doc $GUI_ParseViewer($NoteBook:Doc)
  set text [tip_GetRawData $doc]
  set step [expr {100.0/([llength $Anns]+1)}]
  set progress 0.0

  ## Select the Annotations that are within the span range of the selected
  ## sentence..
  set sentence_span [tip_GetFirstSpan $sentence]
  foreach {start end} $sentence_span {break}
  set AnnsInRange [lindex [tip_AnnotationsInRange $Anns $start $end] 0]
  set GUI_ParseViewer($NoteBook:Prg1) 1
  
  ## Make the canvas a tree widget...
  set parse [tip_GetValueValue [tip_GetAttribute \
                 $AnnsInRange $GUI_ParseViewer($NoteBook:Type)]]
  set tree $Canvas
  Tree:create  $tree
  Tree:sort    $tree 0
  Tree:newitem $tree /
  image create photo null
  ggi_view_parseCreateTree_FillTree $tree / $parse
  set GUI_ParseViewer($NoteBook:Prg1) 100
  after 1000 "set GUI_ParseViewer($NoteBook:Prg1) -1"
# ggi_destroy_wait $wwin
};# ggi_view_parseCreateTree

## This function creates the parse tree...
proc ggi_view_parseCreateTree_FillTree {tree parent parse} {
  foreach child $parse {
    set head     [lindex $child 0]
    set children [lrange $child 1 end]
    set i 1
    while {[catch {Tree:newitem $tree ${parent}$head -image ISystemSmall}]} {
      set head "[lindex $child 0]_$i"
      incr i
    }
    foreach one $children {
      ggi_view_parseCreateTree_FillTree $tree ${parent}$head/ $one
    }
    Tree:open $tree ${parent}$head
  }
}
