##############################################################################
##  StemViewer.tcl:                                                         ##
## -------------------                                                      ##
##  This file implements a simple stem viewer...                            ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
CDM_RegisterViewer {Stem Viewer} ggi_view_stem

## ggi_view_stem
#
proc ggi_view_stem {doc annot attribute title} {
  global GUI_Text GUI_Options WMIcons

  set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
  ## Get Document Annotations and text
  if [catch {set text_str [tip_GetByteSequence $doc]}] {
    ggi_destroy_wait $wwin
    error "ggi_view_stem: Not Valid Document! ($doc)" {}
  }

  ## Create the window
  set win [ggi_unused_win .ggi_view_stem]
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Toolbar...
  set toolbar_frame ${win}.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.viewer.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x
  
  set BFrame ${win}.but_frame
  set Viewer ${win}.viewer
  frame $BFrame -relief flat

  ## Text Widget
  ::ScrolledWidgets::ScrolledText $Viewer
  $Viewer.text insert end $text_str

  ## Add Tags to Text Viewer...
  set Annotations [tip_SelectAnnotations $doc $annot {}]
  set Values {}; set AnnNu [llength $Annotations]
  if {$AnnNu < 1} {set AnnNu 1}
  set ProgressStep [expr {20.0/ $AnnNu}]; set Progress 0.0
# puts [time {
  foreach ann $Annotations {
    set Progress [expr {$Progress + $ProgressStep}]
    set attr [tip_GetAttribute $ann $attribute]
    if {[llength $attr]} {
      set value [lindex [lindex $attr 1] 1]
      set val [string length $value]
      if {!$val} {continue}
      foreach {start end} [tip_GetFirstSpan $ann] {break}
      set text [string range $text_str $start $end]
      ggi_wait_update {} $Progress $wwin
      set pos [tip_GetAttribute $ann pos]
      if {[string length $pos]} {
        set pos_val [tip_GetValueValue $pos]
        lappend ViewerTags($pos_val) 1.0+${start}c 1.0+${end}c
      } else {
        lappend ViewerTags(Default) 1.0+${start}c 1.0+${end}c
      }
      set index [string last $value $text]
      if {$index != -1} {
        set start [expr {$start+$index}]
      } else {
        if {![string equal $attribute stem]} {
          set start [expr {$start+[string length $text]-$val-1}]
        }
      }
      set end   [expr {$start+$val}]
      lappend ViewerTags(\ $attribute) 1.0+${start}c 1.0+${end}c
    }
  }
  catch {set Values [lsort [array names ViewerTags]]}
  set MaxButtonNu 10

  ## Hilight All Taggs and Add Buttons
  set Colors $GUI_Options(Colors)
  set index 0; set bindex 1; set color_nu [llength $Colors]
  set frame ${BFrame}.b
  frame $frame
  pack $frame -side top 
  button ${frame}.__all -textvariable GUI_Text(All) -borderwidth 1 -command \
   "ggi_view_single_span_hilight $Viewer.text {} [list $Values]" \
   -padx 6 -pady 2 -highlightthickness 0
  pack ${frame}.__all -side left -padx 1 -pady 0
  set Progress 20.0
  set ProgressStep [expr {(100.0-$Progress)/($AnnNu*2.0)}]
  foreach value $Values {
    set val [list $value]
    catch \
    {set Progress [expr {$Progress+$ProgressStep*[llength $ViewerTags($val)]}]}
    if {[catch {eval $Viewer.text tag add $val $ViewerTags($val)}]} {
      catch {eval $Viewer.text tag add $val $ViewerTags($value)}
    }
    if {$index == $color_nu} {set index 0}
    foreach {bg fg} [lindex $Colors $index] {break}
    ${Viewer}.text tag configure $value -background $bg -foreground $fg
    if {$bindex == $MaxButtonNu} {
      set frame [ggi_unused_win ${BFrame}.b]
      frame $frame -relief flat
      pack $frame -side top -padx 0 -pady 0
      set bindex 0
    }
    set button [ggi_unused_win ${frame}.b${index}_]
    button $button -text "$value" -background $bg \
     -foreground $fg -borderwidth 1 -padx 7 -pady 2 -command \
     "ggi_view_single_span_hilight ${Viewer}.text $val \
       [list $Values] $fg $bg" -highlightthickness 0
    bind $button <<MB_2>> \
     "ggi_view_single_span_hilight ${Viewer}.text $val {} $fg $bg"
    pack $button -side left -padx 1 -pady 0
    incr index
    incr bindex
    ggi_wait_update {} $Progress $wwin
  }
# }]
  
  frame ${BFrame}.ok
  button ${BFrame}.ok.ok -textvariable GUI_Text(Dismiss) -command "destroy $win"
  ggi_destroy_wait $wwin
  pack ${BFrame}.ok.ok -pady 2
  pack ${BFrame}.ok -side bottom -fill x
  pack $BFrame -side bottom -fill x -expand 1
  pack $Viewer -side top -fill both -expand 1
  ${Viewer}.text configure -state disabled -exportselection 1
  focus ${Viewer}.text
  foreach {bg fg} [lindex $Colors 0] break
  catch {${BFrame}.b.b0__0 invoke;${BFrame}.b.__all invoke}
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  return [list $win ${Viewer}.text]
};# ggi_view_stem
