##############################################################################
##############################################################################
##
## This file implements same simple Video viewers.
## Currently, these viewers work only under the Windows family of operating
## systems...
## Author: Georgios Petasis, petasis@iit.demokritos.gr
## This file is part of the "Ellogon" text engineering platform.
##
#############################################################################
#############################################################################

GUI_LoadImage TMCILogo
switch $tcl_platform(platform) {
  windows {
    catch {package require tmci}
  }
  unix {
    catch {package require xanim}
  }
  default {}
}

## A simple Video viewer. It is a simple Tk window, having a label widget
## where video is drawn by MCI Windows services. Thus, the actual work on
## displaying video is done by MCI...
proc ggi_view_VideoFile {doc annot attribute title} {
  global GUI_Options GUI_Text WMIcons ggi_view_VideoFile
  ## Create the Video window...
  set win [ggi_unused_win .ggi_view_VideoFile]
  toplevel $win
  wm withdraw $win
  wm title $win $title
  set VideoFile $annot

  ## 2) Create the window toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.viewer.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltip {} \
         -image ITOpen -command "ggi_view_VideoFileOpenFile $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  ## 3) The widget where the Video is displayed...
  frame $win.video_frame -bg black
    pack [label $win.video_frame.video -bg black -relief flat -bd 0 \
                -image TMCILogo] -fill both -expand 1 -padx 4 -pady 4
  pack $win.video_frame -side top -fill both -expand 1
  set VideoWidget $win.video_frame.video
  bind $VideoWidget <Destroy> "array unset ggi_view_VideoFile $win:*"
  set ggi_view_VideoFile($win:Status) Stop
  set ggi_view_VideoFile($win:VideoWidget) $VideoWidget
  after 6000 "catch \{$VideoWidget configure -image {}\}"
  
  ## 4) The area where additional Video controls are placed...
  frame $win.video_control
    set Control $win.video_control
    ## a) A play button...
    if {![info exists GUI_Text(Play)]} {set GUI_Text(Play) Play}
    button $Control.play -textvariable GUI_Text(Play) -command \
           "mci \{play $win from 0:0:0\}
	    set ggi_view_VideoFile($win:Status) Play"
    if {![info exists GUI_Text(Stop)]} {set GUI_Text(Stop) Stop}
    button $Control.stop -textvariable GUI_Text(Stop) -command \
           "mci \{stop $win\}
	    set ggi_view_VideoFile($win:Status) Stop"
    if {![info exists GUI_Text(Configure)]} {set GUI_Text(Configure) Configure}
    button $Control.configure -textvariable GUI_Text(Configure) -command \
           "mci \{configure $win\}"
    if {![info exists GUI_Text(Freeze)]} {set GUI_Text(Freeze) Freeze}
    if {![info exists GUI_Text(UnFreeze)]} {set GUI_Text(UnFreeze) UnFreeze}
    button $Control.freeze -textvariable GUI_Text(Freeze) -command \
           "mci \{pause $win\}
	    set ggi_view_VideoFile($win:Status) Pause"
    button $Control.unfreeze -textvariable GUI_Text(UnFreeze) -command \
           "mci \{resume $win\}
	    set ggi_view_VideoFile($win:Status) Play"
    pack $Control.play $Control.stop $Control.configure \
         $Control.freeze $Control.unfreeze \
         -side left -padx 2 -pady 2
    scale $Control.colour -from 0 -to 100 -orient horizontal -command \
      "ggi_view_VideoFile_Scale $win seek to"
    scale $Control.contrast   -orient horizontal
    scale $Control.brightness -orient horizontal
    pack $Control.colour -fill x
  pack $win.video_control -side top -fill x

  ## 5) Button section...
  frame $win.buttons
    pack [button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
                 -command "destroy $win"] -padx 2
  pack $win.buttons -side top -fill x -pady 4
  
  ## 6) Display the window
  wm geometry $win 450x450
  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  focus $VideoWidget
  
  ## We have to make sure all windows are visible, before we can display video
  ## inside them...
  update

  if {[file exists $VideoFile]} {
    mci "open $VideoFile alias $win"
    scan [winfo id $VideoWidget] 0x%x id  
    mci "window $win handle $id"
    bind $VideoWidget <Destroy> "mci \"close $win\""
  }
};# ggi_view_VideoFile

proc ggi_view_VideoFile_Scale {win command option value} {
  global ggi_view_VideoFile
  set val [expr {$value*100}]
  if {[info exists ggi_view_VideoFile($win:AfterEvent)]} {
    after cancel $ggi_view_VideoFile($win:AfterEvent)
  }
  catch {mci "$command $win $option $val"}
  switch $::ggi_view_VideoFile($win:Status) {
    Play {
      set ggi_view_VideoFile($win:AfterEvent) [after idle \
          "catch \{mci \"pause $win\"; mci \"resume $win\"\}"]
    }
    default {
      set ggi_view_VideoFile($win:AfterEvent) [after idle \
          "catch \{mci \"pause $win\"\}"]
    }
  }
};# ggi_view_VideoFile_Scale

proc ggi_view_VideoFileOpenFile {win} {
  global ggi_view_VideoFile
  set VideoWidget $ggi_view_VideoFile($win:VideoWidget)

  ## Display a Dialog for selecting the Video file...
  set VideoFile [tk_getOpenFile -initialdir {C:/My Documents} -parent $win \
      -title {Select a Video File...}]
  if {[file exists $VideoFile]} {
    ## Close Previous video file, if any opened...
    catch {mci "close $win"}
    puts "open $VideoFile alias $win"
    mci "open \"$VideoFile\" alias $win"
    scan [winfo id $VideoWidget] 0x%x id  
    mci "window $win handle $id"
    bind $VideoWidget <Destroy> "mci \"close $win\""
  }
};# ggi_view_VideoFileOpenFile
