##############################################################################
## Viewers.tcl:                                                             ##
## ---------------------                                                    ##
##  This file implements a series of text viewers.                          ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################
##############################################################################
## Petasis, 08/10/1998
## Viewers Section...
##############################################################################

## Load some more images used only by procedures in this file...
if {$::ApplicationInfo(EnableGUI)} {
  GUI_LoadImage ITExternalEditor
  GUI_LoadImage ITHideTags
  GUI_LoadImage IWWW
  GUI_LoadImage DoubleArrowUp
  GUI_LoadImage DoubleArrowDown
  GUI_LoadImage GreyDot
}
catch {package require base64}
namespace eval ::HtmlInfoViewer {}

CDM_RegisterViewer {Document Viewer} ggi_view_document

## Simple Document Viewer
## ggi_view_document Collection_name Doc_ID [parent_window]
#
proc ggi_view_document {col doc {parentWin {}}} {
  global GUI_Text GUI_Options WMIcons
  eval set win $parentWin.text_viewer
  set win [ggi_unused_win $win]
  toplevel $win
  wm withdraw $win
  wm title $win "Collection $col: [file tail $doc]"

  set doc_id [GUI_IsDocumentOpened $doc $col]
  if {![string length $doc_id]} {
    ## open a new tmp collection (producing a duplicate if it's already open)
    set col_id [tip_OpenCollection $col]
    ## Is the Collection's Location in accordance with it's Name?
    if {![string equal [tip_GetName $col_id] $col]} {
      GUI_FixCollectionName $col [tip_GetName $col_id] $col
    }
    set doc_id [tip_GetDocument $col_id $doc]
    set text_str [tip_GetByteSequence $doc_id]
    tip_Close $col_id
  } else {
    set text_str [tip_GetByteSequence $doc_id]
  }

  ## Create the window Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set hw $win.frame.html
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Edit) \
        -image ITExternalEditor -command \
        [list GUI_OpenExternalEditor {} $text_str]
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
        -image ITExit -command "destroy $win"
    set ActLbl [Toolbar::addwidget $toolbar label -relief flat -bd 0 -anchor e]
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x
  
  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  pack $win.buttons.dismiss -padx 2 -pady 2

  set f $win.frame; set text $f.text
  ::ScrolledWidgets::ScrolledText $f 
  $text insert end $text_str
  pack $f -fill both -expand true

  $text configure -state disabled -exportselection 1

  ## Is it an html document?
  if {[ggi_view_IsHTMLText $text_str]} {
    catch {
      html $hw -bg $GUI_Options(TextBg) \
        -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
        -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
        -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
        -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
        -fontcommand  "::HtmlInfoViewer::PickFont $hw" \
        -framecommand "::HtmlInfoViewer::FrameCmd $hw" \
        -formcommand  "::HtmlInfoViewer::FormCmd $hw"
      set ::HtmlInfoViewer::InfoViewer($hw:showImages)   1
      set ::HtmlInfoViewer::InfoViewer($hw:showBgImages) 1
      if {[string match Doc* $doc]} {
        $hw configure \
          -imagecommand     "ggi_view_documentHTMLRetrieveImages $doc $hw" \
          -bgimagecommand   "ggi_view_documentHTMLRetrieveBgImages $doc $hw"
        bind $win.frame <Destroy> \
          "+ggi_view_documentHTMLRetrieveImagesClean $doc $hw
           array unset ::HtmlInfoViewer::InfoViewer $hw:*"
        catch {$hw configure -base [tip_GetExternalId $doc]}
      }
      catch {::HtmlInfoViewer::DefaultBingings $hw}
      bind $hw.x <<MB_2>> \
           [list ::HtmlInfoViewer::ContextualMenuHandler $hw $doc]
      after 1000 [list $hw parse $text_str]
      Toolbar::addseparator $toolbar
      Toolbar::addbutton $toolbar -tooltip HTML... \
        -image IWWW -command "ggi_SwapGriddedScrolledWidgets $tw $hw"
    }
  }

  ## Arrange the character index to be shown each time the user clicks on the
  ## text widget...
  bind $text <<MB_1>> "+ggi_DisplayTextIndexInLabel $text $ActLbl %x %y"

  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  focus $text
};# ggi_view_document
proc ggi_SwapGriddedScrolledWidgets {w1 w2} {
  set parent [winfo parent $w1]
  if {[llength [grid info $w1]]} {
    grid forget $w1 $w2
    grid $w2 -column 0 -row 0 -sticky snew
    $w2 conf -xscrollcommand [$w1 cget -xscrollcommand] \
             -yscrollcommand [$w1 cget -yscrollcommand]
    $w1 conf -xscrollcommand {} -yscrollcommand {}
    $parent.horizontal conf -command [list $w2 xview]
    $parent.vertical   conf -command [list $w2 yview]
    focus $w2
  } else {
    grid forget $w1 $w2
    grid $w1 -column 0 -row 0 -sticky snew
    $w1 conf -xscrollcommand [$w2 cget -xscrollcommand] \
             -yscrollcommand [$w2 cget -yscrollcommand]
    $w2 conf -xscrollcommand {} -yscrollcommand {}
    $parent.horizontal conf -command [list $w1 xview]
    $parent.vertical   conf -command [list $w1 yview]
    focus $w1
  }
}

proc ggi_view_documentHTMLRetrieveBgImages {doc widget args} {
  if {[info exists ::HTML_InfoViewer($widget:showBgImages)]} {
    if {!$::HTML_InfoViewer($widget:showBgImages)} {return {}}
  }
  return [eval ggi_view_documentHTMLRetrieveImages $doc $widget $args]
};# ggi_view_documentHTMLRetrieveBgImages

## ggi_view_documentHTMLRetrieveImages
#  See if requested images exist as Attributes in Document...
proc ggi_view_documentHTMLRetrieveImages {doc widget args} {
  if {[info exists ::HtmlInfoViewer::InfoViewer($widget:showImages)]} {
    if {!$::HtmlInfoViewer::InfoViewer($widget:showImages)} {return {}}
  }
  foreach {fn w h spec} $args {break}
  if {[catch {tip_GetAttribute $doc $fn} attr]} {return Blank}
  if {[llength $attr]} {
    set value [tip_GetValueValue $attr]
    set type  [tip_GetValueType  $attr]
    switch -exact $type {
      CDM_IMAGE {
        ## Is the data in binary format?
        set image_data $value
        binary scan $image_data {}
        if {[catch {image create photo -data $image_data} image]} {
          ## We failed to create the image directly from the data. Perhaps
          ## Base64 encoded?
          set image_data [::base64::decode $value]
          binary scan $image_data {}
          if {[catch {image create photo -data $image_data} image]} {
            ## We failed again. No image can be retrieved...
            set image {}
          }
        }
      }
      CDM_BASE64_IMAGE {
        set image_data [::base64::decode $value]
        binary scan $image_data {}
        if {[catch {image create photo -data $image_data} image]} {
          ## We failed again. No image can be retrieved...
          set image {}
        }
      }
      default {
        set image {};# The attribute does not contain an image...
      }
    }
    if {[string length $image]} {
      lappend ::ggi_view_documentHTMLRetrieveImages($widget:$doc) $image
      ## Do we have to shrink the image?
      if {[string length $w] && [string length $h]} {
        ## Image dimensions are present. Is our image larger?
        if {[image width $image] > $w || [image height $image] > $h} {
          set _image [GUI_ShrinkImage $image $w $h]
          image delete $image
          set image $_image
        }
      }
      return $image
    }
  }
  return Blank
};# ggi_view_documentHTMLRetrieveImages
proc ggi_view_documentHTMLRetrieveImagesClean {doc win} {
  catch {
    if {[info exists ::ggi_view_documentHTMLRetrieveImages($win:$doc)]} {
      foreach image $::ggi_view_documentHTMLRetrieveImages($win:$doc) {
        catch {image delete $image}
      }
      unset ::ggi_view_documentHTMLRetrieveImages($win:$doc)
    } else {
      if {![string length $doc]} {
        foreach key [array names ::ggi_view_documentHTMLRetrieveImages $win:*] {
          foreach image $::ggi_view_documentHTMLRetrieveImages($key) {
            catch {image delete $image}
          }
        }
        array unset ::ggi_view_documentHTMLRetrieveImages $win:*
      }
    }
  }
};# ggi_view_documentHTMLRetrieveImagesClean

CDM_RegisterViewer {Text File Viewer} ggi_view_text_file
## Simple File Viewer
#
proc ggi_view_text_file {doc text_file enc title {win {}}} {
  global GDM_Collections GUI_Text GUI_Underline GUI_Options WMIcons
  if {[string length $enc]} {
    set GDM_Collections(CurrentAnnotatedCollectionEncoding) $enc
  } else {
    set enc [encoding system]
    set GDM_Collections(CurrentAnnotatedCollectionEncoding) $enc
  }
  if {![info exists ::CDM_CurrentStatus(CurrentCollectionEncoding)]} {
    set ::CDM_CurrentStatus(CurrentCollectionEncoding) $enc
  }

  set HtmlFile 0
  if {[string length $text_file]} {
    ## Open File and read text...
    # if {[catch {CDM_OpenWithCorrectEncoding $text_file r} File]} {
    #   gut_error [format "$GUI_Text(CannotReadFile)\n(%s)" \
    #   $text_file $File]
    # }
    if {[catch {open $text_file r} File]} {
      gut_error [format "$GUI_Text(CannotReadFile)\n(%s)" \
      $text_file $File]
    }
    fconfigure $File -encoding $enc
    set ExternalEditorCommand [list GUI_OpenExternalEditor $text_file]
    set text [read $File [file size $text_file]]
    close $File
    set extension [file extension $text_file]
    if {[string equal $extension .html] || [string equal $extension .htm]} {
      set HtmlFile 1
      if {[regexp -nocase -- {<meta.*charset=([^\ \"\;]*)} $text dummy enc]} {
        ## If changed, modify also GUI/Dialogs/InfoViewer.tcl
        switch -glob -- [string tolower $enc] {
          windows-* {set encoding cp[string range $enc 8 end]}
          iso-*     {set encoding iso[string range $enc 4 end]}
          iso*      {set encoding iso[string range $enc 3 end]}
          utf-8     {set encoding utf-8}
          shift-jis {set encoding shiftjis}
          default   {set encoding {}}
        }
        set File [open $text_file]
        fconfigure $File -encoding $encoding
        set text [read $File [file size $text_file]]
        close $File
      }
    }
  } else {
    set text_file $title
    set text $doc
    catch {set text [tip_GetByteSequence $doc]}
    if {[string equal $text_file {}]} {
      catch {set text_file [tip_GetId $doc]}
    }
    set ExternalEditorCommand [list GUI_OpenExternalEditor {} $text]
  }

  ## Display it...
  if {![string length $win]} {
    set win [ggi_unused_win .text_viewer]
  }
  toplevel $win
  wm withdraw $win
  wm title $win "Text Viewer: $text_file"

  ## Create the window
  ## Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    if {!$HtmlFile} {
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Edit) \
        -image ITExternalEditor -command $ExternalEditorCommand
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    } else {
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Edit) \
        -image ITExternalEditor -command $ExternalEditorCommand
    }
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    set ActLbl [Toolbar::addwidget $toolbar label -relief flat -bd 0 -anchor e]
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  pack $win.buttons.dismiss -padx 2 -pady 2

  set f $win.frame
  if {$HtmlFile} {
    ScrolledWidgets::ScrolledHtml $f -widgetname text -bg $GUI_Options(TextBg) \
    -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
    -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
    -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
    -tablerelief raised -fontcommand "::HtmlInfoViewer::PickFont $f.text"
    $f.text configure -base $text_file
    $f.text parse $text
  } else {
    ::ScrolledWidgets::ScrolledText $f
    $f.text insert end $text
    $f.text configure -state disabled -exportselection 1
    bind $f.text <<MB_1>> "+ggi_DisplayTextIndexInLabel $f.text $ActLbl %x %y"
    ## Is it an html document?
    if {[ggi_view_IsHTMLText $text]} {
      set hw $f.html
      catch {
        html $hw -bg $GUI_Options(TextBg) \
          -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
          -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
          -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
          -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
          -fontcommand  "::HtmlInfoViewer::PickFont $hw.html" \
          -framecommand "::HtmlInfoViewer::FrameCmd $hw" \
          -formcommand  "::HtmlInfoViewer::FormCmd $hw"
        set ::HtmlInfoViewer::InfoViewer($hw:showImages)   1
        set ::HtmlInfoViewer::InfoViewer($hw:showBgImages) 1
        if {[string match Doc* $doc]} {
          $hw configure \
            -imagecommand     "ggi_view_documentHTMLRetrieveImages $doc $hw" \
            -bgimagecommand   "ggi_view_documentHTMLRetrieveBgImages $doc $hw"
          bind $f <Destroy> \
                  "+ggi_view_documentHTMLRetrieveImagesClean $doc $hw
                    array unset ::HtmlInfoViewer::InfoViewer $hw:*"
          catch {$hw configure -base [tip_GetExternalId $doc]}
        }
        catch {::HtmlInfoViewer::DefaultBingings $hw}
        bind $hw.x <<MB_2>> \
             [list ::HtmlInfoViewer::ContextualMenuHandler $hw $doc]
        after 1000 [list $hw parse $text]
        Toolbar::addseparator $toolbar
        Toolbar::addbutton $toolbar -tooltip HTML... \
          -image IWWW -command "ggi_SwapGriddedScrolledWidgets $f.text $hw"
      }
    }
  }
  pack $f -fill both -expand true

  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  focus $f.text
  return $f.text
}; # ggi_view_text_file

CDM_RegisterViewer {Text Viewer} ggi_view_text
## Simple Text Viewer
#
proc ggi_view_text {text {title {Text Viewer...}} {win {}}} {
  global GDM_Collections GUI_Text GUI_Underline GUI_Options WMIcons

  set HtmlFile [ggi_view_IsHTMLText $text]

  ## Display it...
  if {![string length $win]} {
    set win [ggi_unused_win .text_viewer]
  }

  #set ExternalEditorCommand [list GUI_OpenExternalEditor {} $text]
  
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Create the window
  ## Toolbar...
  set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
  Toolbar::ToolbarFrame $toolbar_frame \
          -relief $GUI_Options(ToolbarFrameRelief) \
          -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
          -decorate $GUI_Options(ToolbarDecorate)

    set tw $win.frame.text
    set toolbar $toolbar_frame.tb1
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
        -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
    if {!$HtmlFile} {
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Save) \
        -image ITSave -command "ggi_SaveTextWidgetContents $tw"
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Edit) \
      #  -image ITExternalEditor -command $ExternalEditorCommand
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeFont) \
        -image ITSelectFont -command "::FontDialog::FontSelect $tw $tw._fontdlg"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(ChangeColour...) \
        -image ITSelectColour -command "ggi_SetWidgetColours $tw"
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Find...) \
        -image ITZoom -command "::GUI_TextSearchFacilities::TextFindDialog $tw"
    } else {
      #Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Edit) \
      #  -image ITExternalEditor -command $ExternalEditorCommand
    }
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
         -image ITExit -command "destroy $win"
    set ActLbl [Toolbar::addwidget $toolbar label -relief flat -bd 0 -anchor e]
    pack $toolbar -side left -fill y

  pack $toolbar_frame -side top -fill x

  frame $win.buttons
  pack  $win.buttons -side bottom
  button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
    -command [list destroy $win]
  pack $win.buttons.dismiss -padx 2 -pady 2

  set f $win.frame
  if {$HtmlFile} {
    ScrolledWidgets::ScrolledHtml $f -widgetname text -bg $GUI_Options(TextBg) \
    -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
    -fg $GUI_Options(TextFg) -relief $GUI_Options(TextRelief) \
    -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
    -tablerelief raised -width 350 -height 150 \
    -fontcommand "::HtmlInfoViewer::PickFont $f.html"
    $f.text parse $text
  } else {
    ::ScrolledWidgets::ScrolledText $f
    $f.text insert end $text
    $f.text configure -state disabled -exportselection 1
    ## Arrange the character index to be shown each time the user clicks on the
    ## text widget...
    bind $f.text <<MB_1>> "+ggi_DisplayTextIndexInLabel $f.text $ActLbl %x %y"
  }
  pack $f -fill both -expand true

  update
  catch {winico setwindow $win $WMIcons(AnnView)}
  wm deiconify $win
  focus $f.text
  return $f.text
}; # ggi_view_text

CDM_RegisterViewer {Annotation Multiple Span Viewer} ggi_view_multiple_span
## ggi_view_multiple_span
#
proc ggi_view_multiple_span {doc type dummy title} {
}

## ggi_unused_win
#   Get an unused window name based on string given as name
proc ggi_unused_win {name} {
  if {![winfo exists $name]} {return $name}
  set win_nu 0
  while {[incr win_nu]} {
    if {![winfo exists ${name}_$win_nu]} {return ${name}_$win_nu}
  }
};# ggi_unused_win

## ggi_wait_message
#  Create a small window showing the message
proc ggi_wait_message {{message {}} {Percent -1.0} {reuse_win 0} \
                        {win .ggi_wait_message}} {
  global GUI_Progress tcl_platform GUI_Options GUI_Text
  if {![string length $message]} {
    set message $GUI_Text(PleaseWait)
  }

  if {!$::ApplicationInfo(EnableGUI)} {
    catch {puts $message}
    return $win
  }
  
  Agent::Play Processing
  switch $tcl_platform(platform) {
    unix    -
    windows {set parent {}}
    default {set parent .mw}
  }
  foreach toplevel [winfo children .] {
    catch {$toplevel configure -cursor watch}
  }
  if {!$reuse_win} {set win [ggi_unused_win $win]}
  set GUI_Progress(Message:$win) $message
  set GUI_Progress(Percent:$win) $Percent
  set GUI_Progress(Resized:$win) 0
  if {![winfo exists $win]} {
    if {[catch {.mw cget -bg} bg]} {set bg $GUI_Options(Background)}
    if {[catch \
      {ProgressDlg $win -textvariable GUI_Progress(Message:$win) \
        -variable GUI_Progress(Percent:$win) -title "Message..." \
        -foreground blue -height 5 -width 45 -parent $parent \
        -bg $bg -bd 1 -relief sunken}]} {
      toplevel $win
      wm title $win "Message..."
      set x_y [winfo pointerxy $win]
      wm geometry $win "+[lindex $x_y 0]+[lindex $x_y 1]"
      frame $win.fr -relief ridge -borderwidth 2
      label $win.fr.message -textvariable GUI_Progress(Message:$win)
      pack $win.fr -padx 5 -pady 5
      pack $win.fr.message -padx 10 -pady 10
      update
      catch {grab set $win}
    }
  }
  update
  catch "wm protocol $win WM_DELETE_WINDOW {GUI_AbortCurrentProcess}"
  return $win
};# ggi_wait_message

## ggi_wait_update_message
#  Update the message displayed...
proc ggi_wait_update_message {doc message {win {}}} {
  if {!$::ApplicationInfo(EnableGUI)} {
    puts $message
    return
  }
  global GUI_Options GUI_Progress
  if {![string length $win]} {
    if {![info exists GUI_Progress(WindowID:$doc)]} {
      update
      return
    }
    set win $GUI_Progress(WindowID:$doc)
  }
  set GUI_Progress(Message:$win) $message
  update
};# ggi_wait_update_message

## ggi_wait_update
#  Redraws windows while waiting (and possibly doing cpu-intensive tasks...)
proc ggi_wait_update {doc percent {win {}}} {
  if {!$::ApplicationInfo(EnableGUI)} {return}
  global GUI_Options GUI_Progress
  if {![string length $win]} {
    if {![info exists GUI_Progress(WindowID:$doc)]} {
      update
      return
    }
    set win $GUI_Progress(WindowID:$doc)
  }
 
  if {[info exists GUI_Progress(Message2:$win)]} {
    if {abs(int($percent-$GUI_Progress(Percent2:$win)))
                                               >= $GUI_Options(ProgressStep)
        || $percent == 100} {
      set GUI_Progress(Percent2:$win) $percent
    }
  } else {
    if {abs(int($percent-$GUI_Progress(Percent:$win)))
                                               >= $GUI_Options(ProgressStep)
        || $percent == 100} {
      set GUI_Progress(Percent:$win) $percent
    }
  }
  update
};# ggi_wait_update

## ggi_wait_append
proc ggi_wait_append {message {percent -1} {type normal}} {
  if {!$::ApplicationInfo(EnableGUI)} {
    catch {puts $message}
    return
  }

  set path [grab current]
  if {![string match *ggi_wait_message* $path]} {
    return [ggi_wait_message $message $percent]
  }
  global GUI_Progress
  set GUI_Progress(Message2:$path) $message
  set GUI_Progress(Percent2:$path) $percent
  
  set frame $path
  catch {frame $frame.f2 -relief flat -bd 0}
  label $frame.f2.msg2 -height 4 \
    -relief flat -borderwidth 0 -highlightthickness 0 -anchor w -justify left \
    -textvariable GUI_Progress(Message2:$path)
  pack $frame.f2.msg2 -side top -pady 3m -anchor nw -fill x -expand 1 -padx 6
  eval ProgressBar::create $frame.f2.pb2 \
    [Widget::subcget "$path#ProgressDlg" .frame.pb] 
  $frame.f2.pb2 configure -variable GUI_Progress(Percent2:$path) -type $type \
    -bd 1 -relief sunken
  pack $frame.f2.pb2 -side bottom -anchor w -fill x -expand 1 -padx 3 -pady 3
  pack $frame.f2 -side bottom -fill x -expand 1
  if {!$GUI_Progress(Resized:$path)} {
    wm geometry $path {}
    update
    set GUI_Progress(Resized:$path) 1
    wm geometry $path [wm geometry $path]
    $frame.f2 configure -height [winfo height $frame.f2]
  }
  return $path
};# ggi_wait_append

## ggi_destroy_append
#
proc ggi_destroy_append {path} {
  if {!$::ApplicationInfo(EnableGUI)} {return}
  global GUI_Progress
  if {[info exists GUI_Progress(Message2:$path)]} {
    destroy $path.f2.msg2
    destroy $path.f2.pb2
    unset -nocomplain GUI_Progress(Message2:$path) GUI_Progress(Percent2:$path)
  } else {
    ggi_destroy_wait $path
  }
};# ggi_destroy_append

## ggi_destroy_wait
#  Destroy a message window
proc ggi_destroy_wait {win} {
  if {!$::ApplicationInfo(EnableGUI)} {return}
  global GUI_Progress GUI_Options
  if {[info exists GUI_Progress(Message2:$win)]} {
    return [ggi_destroy_append $win]
  }
  if {$GUI_Progress(Percent:$win) >= 0.0} {set GUI_Progress(Percent:$win) 100}
  catch {grab release $win}
  destroy $win
  foreach toplevel [winfo children .] {
    catch {$toplevel configure -cursor {}}
  }
  unset -nocomplain GUI_Progress(Message:$win)  GUI_Progress(Percent:$win) \
                    GUI_Progress(Message2:$win) GUI_Progress(Percent2:$win) \
                    GUI_Progress(Resized:$win)
  Agent::Play stop Pleased
};# ggi_destroy_wait

## ggi_TextTagToggle
#  This function will toggle (blink) a specific tag in a text widget.
proc ggi_TextTagToggle {text tag fg1 bg1 {fg2 {}} {bg2 {}}} {
  catch {
    $text tag configure $tag -background $bg1 -foreground $fg1
    if {[string length $bg1] && ![llength [$text tag ranges $tag]]} {return}
    after 800 [list ggi_TextTagToggle $text $tag $fg2 $bg2 $fg1 $bg1]
  }
};# ggi_TextTagToggle

## ggi_SaveTextWidgetContents
#  Save text contents in a file...
proc ggi_SaveTextWidgetContents {win} {
  global GUI_Text
  if {![string equal [winfo class $win] Text]} {return}
  set filename [tk_getSaveFile -parent $win]
  if {[string equal $filename {}]} return
  if {[catch {open $filename w} file]} {
    gut_error "[format $GUI_Text(CannotWriteFile) filename]\n  ($file)"
  }
  puts -nonewline $file [$win get 1.0 end-1c]
  close $file
};# ggi_SaveTextWidgetContents

## ggi_SetWidgetColours
#  Change Widgets foreground/background etc...
proc ggi_SetWidgetColours {widget} {
  global GUI_Text GUI_Options ggi_SetWidgetColours
  set class [winfo class $widget]
  switch -exact $class {
    Text -
    Html {}
    default {return}
  }
  set win $widget.widgetColours
  if {[winfo exists $win]} {return}
  set ggi_SetWidgetColours(Win) $win
  toplevel $win
  wm withdraw $win
  wm title $win {Change Colours...}
  label $win.foreground     -textvariable GUI_Text(ForegroundColour) -anchor w
  label $win.background     -textvariable GUI_Text(BackgroundColour) -anchor w
  if {[string equal $::tcl_platform(platform) windows]} {
    button $win.bforeground -text ... -bg $GUI_Options(${class}Fg) \
    -fg $GUI_Options(${class}Bg) -command \
      "set ggi_SetWidgetColours(FG) \[tk_chooseColor -parent $win \
         -initialcolor \$ggi_SetWidgetColours(FG) \
         -title {Choose Foreground Color...}\]
       if {\[string length \$ggi_SetWidgetColours(FG)\]} {
          \$ggi_SetWidgetColours(Win).bforeground configure -background \
          \$ggi_SetWidgetColours(FG)
        } else {
          set ggi_SetWidgetColours(FG) \
            \[\$ggi_SetWidgetColours(Win).bforeground cget -background\]
        }"
    button $win.bbackground -text ... -bg $GUI_Options(${class}Bg) \
    -fg $GUI_Options(${class}Fg) -command \
      "set ggi_SetWidgetColours(BG) \[tk_chooseColor -parent $win \
         -initialcolor \$ggi_SetWidgetColours(BG) \
         -title {Choose Background Color...}\]
       if {\[string length \$ggi_SetWidgetColours(BG)\]} {
          \$ggi_SetWidgetColours(Win).bbackground configure -background \
          \$ggi_SetWidgetColours(BG)
        } else {
          set ggi_SetWidgetColours(BG) \
            \[\$ggi_SetWidgetColours(Win).bbackground cget -background\]
        }"
  } else {
    if {[catch {SelectColor $win.bforeground -type menubutton \
     -color $GUI_Options(${class}Fg) -variable ggi_SetWidgetColours(FG) \
     -title {Choose Foreground Color...}}]} {
     button $win.bforeground -background $GUI_Options(${class}Fg) \
       -foreground $GUI_Options(${class}Bg) -text ... -command \
       "set ggi_SetWidgetColours(FG) \[SelectColor::dialog $win.bforeground.sc \
         -color \$ggi_SetWidgetColours(FG) -title {Choose Foreground Color...}\]
        if {!\[string equal \$ggi_SetWidgetColours(FG) {}\]} {
          \$ggi_SetWidgetColours(Win).bforeground configure -background \
          \$ggi_SetWidgetColours(FG)
        } else {
          set ggi_SetWidgetColours(FG) \
            \[\$ggi_SetWidgetColours(Win).bforeground cget -background\]
        }"
    }
    
    if {[catch {SelectColor $win.bbackground -type menubutton \
      -color $GUI_Options(${class}Fg) -variable ggi_SetWidgetColours(BG) \
      -title {Choose Background Color...}}]} {
      button $win.bbackground   -background $GUI_Options(${class}Bg) \
       -foreground $GUI_Options(${class}Fg) -text ... -command \
       "set ggi_SetWidgetColours(BG) \[SelectColor::dialog $win.bbackground.sc \
         -color \$ggi_SetWidgetColours(BG) -title {Choose Background Color...}\]
        if {!\[string equal \$ggi_SetWidgetColours(BG) {}\]} {
          \$ggi_SetWidgetColours(Win).bbackground configure -background \
          \$ggi_SetWidgetColours(BG)
        } else {
         set ggi_SetWidgetColours(BG) \
           \[\$ggi_SetWidgetColours(Win).bbackground cget -background\]
       }"
    }
  }

  grid $win.foreground $win.bforeground -sticky snew -padx 2 -pady 2
  grid $win.background $win.bbackground -sticky snew -padx 2 -pady 2
  button $win.ok -textvariable GUI_Text(Ok) -width 9 -command \
    "set ggi_SetWidgetColours(Button) ok;destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -width 9 -command \
    "set ggi_SetWidgetColours(Button) cancel;destroy $win"
  grid $win.ok $win.cancel -padx 10 -pady 10

  gut_ModalDlg $win
  update
  wm deiconify $win
  set ggi_SetWidgetColours(FG) $GUI_Options(${class}Fg)
  set ggi_SetWidgetColours(BG) $GUI_Options(${class}Bg)
  tkwait window $win
  if {[string equal $ggi_SetWidgetColours(Button) ok]} {
    if {![string equal $ggi_SetWidgetColours(FG) {}]} {
      set GUI_Options(${class}Fg) $ggi_SetWidgetColours(FG)
    }
    if {![string equal $ggi_SetWidgetColours(BG) {}]} {
      set GUI_Options(${class}Bg) $ggi_SetWidgetColours(BG)
    }
    $widget configure -foreground $GUI_Options(${class}Fg) \
                      -background $GUI_Options(${class}Bg)
    option add *${class}.Foreground   $GUI_Options(${class}Fg)
    option add *${class}.Background   $GUI_Options(${class}Bg)
  }
  unset ggi_SetWidgetColours
};# ggi_SetWidgetColours

## ggi_SetWidgetSize
proc ggi_SetWidgetSize {widget} {
  global GUI_Text GUI_Options
  if {![string equal [winfo class $widget] Text]} {return}
  set win $widget.widgetSize
  if {[winfo exists $win]} return
  toplevel $win
  wm withdraw $win
  wm title $win {Change Size...}
  label $win.width  -textvariable GUI_Text(TextWidth) -anchor w
  SpinBox $win.bwidth -editable 0 -exportselection 0 \
    -text $GUI_Options(TextWidth) -range [list 20 180 5]
  label $win.height -textvariable GUI_Text(TextHeight) -anchor w
  SpinBox $win.bheight -editable 0 -exportselection 0 \
    -text $GUI_Options(TextHeight) -range [list 6 60 1]

  grid $win.width  $win.bwidth -sticky snew
  grid $win.height $win.bheight -sticky snew
  button $win.ok -textvariable GUI_Text(Ok) -width 9 -command \
    "set GUI_Options(TextWidth) \[expr \[$win.bwidth getvalue\]*5+20\]
     set GUI_Options(TextHeight) \[expr \[$win.bheight getvalue\]+6\]
     catch {$widget configure -width \$GUI_Options(TextWidth) \
                              -height \$GUI_Options(TextHeight)}
     destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -width 9 -command \
    "destroy $win"
  grid $win.ok $win.cancel 

  gut_ModalDlg $win
  wm deiconify $win
};# ggi_SetWidgetSize

## ggi_message
proc ggi_message {message} {
  gut_warning $message
};# ggi_message

## ggi_DisplayTextIndexInLabel
#  This function will display the index of a text widget (as pointed by x & y
#  coordinates) in the specified label widget...
proc ggi_DisplayTextIndexInLabel {text label x y} {
  global GUI_Text
  set index [$text index @$x,$y]
  set charOffset [string length [$text get 1.0 $index]]
  foreach {line char} [split $index .] {break}
  set msg "$GUI_Text(CharacterOffset): $charOffset ($GUI_Text(Line) $line,\
           $GUI_Text(Character) $char)"
  ## Has the label a text variable?
  set tvar [$label cget -textvariable]
  if {[string length $tvar]} {
    set ::$tvar $msg
  } else {
    $label configure -text $msg
  }
};# ggi_DisplayTextIndexInLabel

## ggi_view_IsHTMLText
#  Returns true if the document is an HTML document.
proc ggi_view_IsHTMLText {text} {
  set text [string trim $text]
  set first_char [string index $text 0]
  ## There is the possibility the file to start from a special signature,
  ## denoting a UTF-8 file...
  if {[string equal $first_char \uFEFF]} {
    set text [string range $text 1 end]
    set first_char [string index $text 0]
  }
  if {![string equal < $first_char]} {return 0}
  if {![string equal -nocase -length 6   <html> $text]}          {return 1}
  if {![string equal -nocase -length 14 "<!DOCTYPE HTML" $text]} {return 1}
  ## Search for specific html tags...
  set text [string tolower $text]
  if {[string first  <html>   $text] != -1} {return 1}
  if {[string first "<a href" $text] != -1} {return 1}
  if {[string first "<table"  $text] != -1} {return 1}
  if {[string first  <body>   $text] != -1} {return 1}
  return 0
};# ggi_view_IsHTMLText
