##############################################################################
## XMLDomTreeViewer.tcl:                                                    ##
## ---------------------                                                    ##
##  This file implements a viewer for viewing the DOM tree of an XML        ##
##  Document.                                                               ##
##                                                                          ##
##  Copyright 1998-2004 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

CDM_RegisterViewer {XML DOM Tree Viewer} ggi_view_XMLDomTree

proc ggi_view_XMLDomTree {doc {xml {}} {unused {}} {title {}}
                          {win .xmlDomTree_viewer}} {
  package require tdom
  global GUI_Text GUI_Options ggi_view_Annotation

  set win [ggi_unused_win $win]
  if {![string length $title]} {
    set title "XML Dom Tree Viewer..."
    if {[string length $doc]} {
      append title ": XML Document contained in Document\
               \"[tip_GetId $doc]\""
    }
    append title ...
  }

  ## Make sure we have the XML Data...
  if {![string length $xml] && [string length $doc]} {
    set xml [tip_GetRawData $doc]
  }

  ## Is it a true XML document?
  if {![string match {<?xml*} [string trim $xml]]} {
    ## This is not XML...
    return [ggi_view_text $xml]
  }

  set Dom $win.dom
  if {![winfo exists $win]} {
    toplevel $win
    wm withdraw $win
    wm title $win $title

    ## Create the window
    ## Toolbar...
    set toolbar_frame $win.tf; set toolbar $toolbar_frame.tb
    Toolbar::ToolbarFrame $toolbar_frame \
            -relief $GUI_Options(ToolbarFrameRelief) \
            -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
            -decorate $GUI_Options(ToolbarDecorate)

      set tw $win.frame.text
      set toolbar $toolbar_frame.tb1
      Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
          -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -hide 0
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewDocText) \
          -image ITViewText -command \
           [list ggi_view_text $xml {Text Viewer...} $win.xmlv]
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(Exit) \
          -image ITExit -command "destroy $win"
      pack $toolbar -side left -fill y

    pack $toolbar_frame -side top -fill x

    ## We will display the DOM Tree in a text widget!
    ::ScrolledWidgets::ScrolledText $Dom -wrap none -cursor left_ptr
    switch $::tcl_platform(platform) {
      unix {
          set Cour12    8x13
          set CourB12   8x13b
          set Helv10    "-Adobe-helvetica-medium-r-normal--*-100-*"
          set Helv12    "-Adobe-helvetica-medium-r-normal--*-120-*"
          set HelvB10   "-Adobe-helvetica-bold-r-normal--*-100-*"
          set HelvB12   "-Adobe-helvetica-bold-r-normal--*-120-*"
      }
      windows {
          set Cour12    "{Courier New} 10"
          set CourB12   "{Courier New} 10 bold"
          set Helv10    "Arial 9"
          set Helv12    "Arial 10"
          set HelvB10   "Arial 9  bold"
          set HelvB12   "Arial 10 bold"
      }
    }
    set tagFont   $HelvB12
    set attrFont  $Cour12
    set opnclFont $Cour12
    $Dom.text tag configure tag  -font $tagFont \
         -foreground #40004000D000
    $Dom.text tag configure comment -font $attrFont \
         -background #d000e800d000 -foreground black
    $Dom.text tag configure textValue -font $attrFont \
         -background #d200d200f000 -foreground black
    $Dom.text tag configure attrName -font $attrFont \
         -foreground black
    $Dom.text tag configure attrValue -font $attrFont \
         -background #f000d000d000 -foreground black         
    $Dom.text tag configure header -background gray90 \
         -foreground red2   
    $Dom.text tag configure query -background gray95 \
         -foreground red2   
    $Dom.text tag configure hot   -background #a666a666ffff
    $Dom.text tag configure open  -font $opnclFont 
    $Dom.text tag configure close -font $opnclFont 
    $Dom.text tag configure leave -font $opnclFont
    $Dom.text tag bind open  <Enter> \
      "ggi_view_XMLDomTree::xmlHighlight $Dom.text @%x,%y open hot"
    $Dom.text tag bind open  <Motion> \
      "ggi_view_XMLDomTree::xmlHighlightMotion $Dom.text @%x,%y open hot"
    $Dom.text tag bind open  <Leave>  "$Dom.text tag remove hot 1.0 end"
    $Dom.text tag bind open  <1> \
      "ggi_view_XMLDomTree::xmlOpen $Dom.text @%x,%y"

    $Dom.text tag bind close <Enter> \
      "ggi_view_XMLDomTree::xmlHighlight $Dom.text @%x,%y open hot"
    $Dom.text tag bind close <Motion> \
      "ggi_view_XMLDomTree::xmlHighlightMotion $Dom.text @%x,%y close hot"
    $Dom.text tag bind close <Leave>  "$Dom.text tag remove hot 1.0 end"
    $Dom.text tag bind close <1> \
      "ggi_view_XMLDomTree::xmlClose $Dom.text @%x,%y"
    pack $Dom -fill both -expand true

    frame $win.buttons
    pack  $win.buttons -side bottom
    button $win.buttons.dismiss -textvariable GUI_Text(Dismiss) \
      -command [list destroy $win]
    pack $win.buttons.dismiss -padx 2 -pady 10

    update
    wm deiconify $win
  };# if {![info exists $win]}

  ## We have to execute any <Destroy> scripts, if any...
  set cmd [bind $Dom.text <Destroy>]
  if {[string length $cmd]} {eval uplevel #0 $cmd}
  $Dom.text delete 1.0 end
  set dom [ggi_view_XMLDomTree::xmlTextWidgetLoad $Dom.text $xml]
  bind $Dom.text <Destroy> \
    "$dom delete;unset -nocomplain ggi_view_XMLDomTree::$Dom.text"
};# ggi_view_XMLDomTree

proc ggi_view_XML_file {doc xml_file {enc utf-8} {title {XML FileViewer...}}
                          {win .xmlDomTree_viewer}} {
  if {[catch {open $xml_file} xml]} {
    gut_error [format "$::GUI_Text(CannotReadFile)\n(%s)" \
      $xml_file $xml]
  }
  fconfigure $xml -encoding $enc
  set XML [read $xml [file size $xml_file]]
  close $xml
  return [ggi_view_XMLDomTree {} $XML {} {} $win]
};# ggi_view_XML_file

## Convert a DOM tree into an XML string...
proc ggi_view_XMLDomTree_DOMTreeToXML {tree} {
  set XML "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
  for {set root [$tree documentElement]} {[string length $root]} \
      {set root [$root nextSibling]} {
    append XML [$root asXML]
  }
  return [gut_HexToUTF8 $XML]
};# ggi_view_XMLDomTree_DOMTreeToXML

################Lexicon$lang##############################################################
## The following functions are modified versions found in the "xe" application
## from the tDOM package...
##############################################################################
namespace eval ggi_view_XMLDomTree {
#----------------------------------------------------------------------------
#   xmlTextWidgetLoad_Recurs
#
#----------------------------------------------------------------------------
proc xmlTextWidgetLoad_Recurs {path doSiblings level node maxlevel} {
  upvar #0 ggi_view_XMLDomTree::$path levels

  incr maxlevel -1
  if {$maxlevel < 0} { return }

  while {[string length $node]} {
    set levels($node) $level

    $path insert insert $level

    set type [$node nodeType]
    if {[string equal $type "ELEMENT_NODE"]} {
      set firstChild [$node firstChild]
      if {![string length $firstChild]} {
        $path insert insert " = " leave
      } else {
        if {$maxlevel > 0} {
          $path insert insert " - " [list close $node]
        } else {
          $path insert insert " + " [list open $node]
        }
      }
      $path insert insert "[subst -novariables -nocommands \
                                  [$node nodeName]] " [list tag $node]
      foreach {attr_line_width attr_name_width} {0 0} {break}
      set attr_value_width [string length $level]
      foreach attr [$node attributes] {
        ## Petasis: Does the attribute exists?
        if {[catch {$node getAttribute $attr}]} {continue}
        set l [string length $attr]
        if {$l > $attr_name_width} { 
          set attr_name_width $l
        }
        incr attr_line_width $l
        set l [string length [$node getAttribute $attr]]
        if {$l > $attr_value_width} {
          set attr_value_width $l
        }
        incr attr_line_width $l
      }

      set recurseToChilds 1
      set attrDisplayMode [expr $attr_line_width > 80]
   
      if {$attrDisplayMode} {
        foreach attr [$node attributes] {
          $path insert insert "\n"
          $path insert insert "$level    " 
          #$path insert insert [format " %-${attr_name_width}s = %-${attr_value_width}s  " \
          #                            $attr [$node getAttribute $attr] \
          #                    ] attr
          $path insert insert [format " %-${attr_name_width}s= " \
                              [subst -novariables -nocommands $attr]] attrName
          $path insert insert [subst -novariables -nocommands \
                                     [$node getAttribute $attr]] attrValue
          #$path insert insert [format "%-${attr_value_width}s  "    \
          #                            '[$node getAttribute $attr]'  \
          #                    ] attrValue
        }
        $path insert insert "\n"
      } else {
        if {![llength [$node attributes]]} {
          set childs [$node childNodes] 
          if {[llength $childs] == 1} {
            if {[$childs nodeType] == "TEXT_NODE"} {
              set value [$childs nodeValue]
              if {([string length $value] < 60) && 
                ([string first \n $value] == -1)} {
                $path insert insert [subst -novariables -nocommands \
                                           $value] textValue
                set recurseToChilds 0
              }
            }
          }
        } else {
          foreach attr [$node attributes] {
            ## Petasis: Does the attribute exists?
            if {[catch {$node getAttribute $attr}]} {continue}
            $path insert insert " [subst -novariables -nocommand \
                                         $attr]=" attrName
            $path insert insert [subst -novariables -nocommands [$node \
                                       getAttribute $attr]] attrValue
            #$path insert insert ' attrName
          }
        }
        $path insert insert "\n"
      }
      set recurseToChilds 1
      if {$recurseToChilds} {
        foreach child [$node childNodes] {
          xmlTextWidgetLoad_Recurs $path 1 "$level   " $child $maxlevel
        }
      }
    } else {
      switch $type {
        COMMENT_NODE {
          $path insert insert " C "
          $path insert insert [subst -novariables -nocommands \
                                     [$node nodeValue]] comment
          $path insert insert "\n"
        }
        CDATA_SECTION_NODE -
        TEXT_NODE {
          set lines 0
          foreach line [split [$node nodeValue] \n] {
            if {$lines == 0} {         
              $path insert insert " T "
            } else {
              $path insert insert "$level   "
            }
            if {![string length $line]} {
              $path insert insert " " textValue
            } else {
              $path insert insert [subst -novariables -nocommands \
                                         $line] textValue
            }
            $path insert insert "\n"
            incr lines
          }
        }
        PROCESSING_INSTRUCTION_NODE {
          $path insert insert " P "
          $path insert insert [subst -novariables -nocommands \
                                     [$node target]] tag
          set lines 0
          foreach line [split [$node data] \n] {
            if {$lines == 0} {
              $path insert insert " "
            } else {
              $path insert insert "$level    "
            }
            $path insert insert [subst -novariables -nocommands $line] attrValue
            $path insert insert "\n"
            incr lines
          }
        }
        default {
          $path insert insert " ? "
          $path insert insert [subst -novariables -nocommands \
                                     [$node nodeValue]] attrValue
          $path insert insert "\n"
        }
      }
    }

    if {!$doSiblings} {
      return
    }
    break
    #set node [$node nextSibling]
  }
};# xmlTextWidgetLoad_Recurs

#----------------------------------------------------------------------------
#   xmlTextWidgetLoad
#
#----------------------------------------------------------------------------
proc xmlTextWidgetLoad {path xml {query {}}} {
  #variable doc root keepEmpties useSimple
  set keepEmpties 0
  set useSimple 0
  set xml [gut_UTF8ToHex $xml]

  if {$useSimple} {
    if {$keepEmpties} { 
      set doc [dom parse -keepEmpties -simple $xml]
    } else {
      set doc [dom parse -simple $xml]
    }
  } else {
    if {$keepEmpties} { 
      set doc [dom parse -keepEmpties $xml]
    } else {
      set doc [dom parse $xml]
    }
  }
  set root [$doc documentElement]

  set query [string trim $query]
  if {![string length $query]} {
    set query /
  }

  set nodes 0
  set rows  0

  set results [$root selectNodes $query type]

  switch $type { 
    nodes { 
      foreach node $results {
        $path mark set insert end
        xmlTextWidgetLoad_Recurs $path 1 "" $node 2
        $path insert end \n
        incr nodes
      }
    }
    attrnodes {
      foreach {attrName attrValue} $results {
        $path insert end [subst -novariables -nocommands $attrName] attrName 
        $path insert end " "
        $path insert end [subst -novariables -nocommands $attrValue] attrValue
        $path insert end \n
        incr rows
      }
    }
    attrvalues {
      foreach result $results {
        $path insert end "[subst -novariables -nocommands $result]\n"
        incr rows
      }
    }
    default {
      $path insert end "[subst -novariables -nocommands $results]\n"
    }
  }
  #if {$rows  != 0} { $path insert end "---$rows result(s)---\n" }
  #if {$nodes != 0} { $path insert end "---$nodes node(s)---\n"  }
  $path yview -pickplace 1.0
  return $doc
}

#----------------------------------------------------------------------------
#   xmlHighlight
#
#----------------------------------------------------------------------------
proc xmlHighlight { path pos tag highlight_tag} {
  set range [$path tag nextrange $tag $pos [$path index "$pos lineend"] ]
  if {$range == ""} {
    set range [$path tag prevrange $tag $pos [$path index "$pos linestart"] ]
  }
  if {$range != ""} {
    eval $path tag add $highlight_tag [lrange $range 0 1]
  }
}


#----------------------------------------------------------------------------
#   xmlHighlightMotion
#
#----------------------------------------------------------------------------
proc xmlHighlightMotion { path pos tag highlight_tag} {
  set tags [$path tag names $pos]
  if {[lsearch -exact $tags $highlight_tag] < 0} {
    $path tag remove $highlight_tag 1.0 end
  } 
  xmlHighlight $path $pos $tag $highlight_tag
}

#----------------------------------------------------------------------------
#   xmlOpen
#
#----------------------------------------------------------------------------
proc xmlOpen {path pos} {
  #global levels
  upvar #0 ggi_view_XMLDomTree::$path levels
  foreach tag [$path tag names $pos] {
    if {($tag != "open") && ($tag != "hot") && ($tag != "sel")} {
      $path configure -state normal
      set start [$path index "$pos linestart"]
      set end   [$path index "$start + 1 lines"]
      $path delete $start $end
      while 1 {
        set end   [$path index "$start + 1 lines"]
        set nextLine [$path get $start $end]
        if {[string match "$levels($tag)    *" $nextLine]} {
          $path delete $start $end
        } else {
          break
        }
      }
      $path mark set insert $start
      xmlTextWidgetLoad_Recurs $path 0 $levels($tag) $tag 2
      $path see $start
      $path configure -state disabled
    }
  }

  # that's a hack to remove selections, which occur sometimes
  after 50 "$path tag remove sel 1.0 end"
}

#----------------------------------------------------------------------------
#   xmlClose
#
#----------------------------------------------------------------------------
proc xmlClose {path pos} {
  #global levels
  upvar #0 ggi_view_XMLDomTree::$path levels
  foreach tag [$path tag names $pos] {
    if {($tag != "close") && ($tag != "hot") && ($tag != "sel")} {
      $path configure -state normal
      set start [$path index "$pos linestart"]
      set end   [$path index "$start + 1 lines"]
      $path delete $start $end

      while 1 {
        set end   [$path index "$start + 1 lines"]
        set nextLine [$path get $start $end]
        if {[string match "$levels($tag)    *" $nextLine]} {
          $path delete $start $end
        } else {
          break
        }
      }
      $path mark set insert $start
      xmlTextWidgetLoad_Recurs $path 0 $levels($tag) $tag 1
      $path see $start
      $path configure -state disabled
    }
  }

  # that's a hack to remove selections, which occur sometimes
  after 50 "$path tag remove sel 1.0 end"
}

};# namespace ggi_view_XMLDomTree
