#!/bin/sh
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}
##############################################################################
##  PerformSubstitutions.tcl:                                               ##
## --------------------------------                                         ##
##                                                                          ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Software and Knowledge Engineering Laboratory,                          ##
##  Institute of Informatics and Telecommunications,                        ##
##  N.C.S.R. "Demokritos"                                                   ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Text Engineering Platform.                                    ##
##############################################################################

set ScriptDir [file dirname [file normalize [info script]]]
set HomeDir [file dirname [file dirname $ScriptDir]]
cd $HomeDir

set Modify 1

set DirList [list \
  ellogon2.0/lib_elep \
  ellogon2.0/Tools \
]

proc processDirectory {dir} {
  foreach d [lsort -dictionary [glob -nocomplain -directory $dir -types d *]] {
    processDirectory $d
  }
  puts "Directory: $dir"
  foreach f [lsort -dictionary [glob -nocomplain -directory $dir -types f *.tcl]] {
    processFile $f
  }
};# processDirectory

proc processFile {filename} {
  global Filename Edit 
  set Filename $filename
  set Edit     1
  set fd [open $filename]
  fconfigure $fd -encoding utf-8
  set data [string trim [read $fd]]
  close $fd
  set d [processData $data]
  if {![string equal $d $data]} {
    puts "  + Updating: $filename"
    if {$::Modify} {
      set fd [open $filename w]
      fconfigure $fd -encoding utf-8
      puts $fd $d
      close $fd
    }
  }
  if {$Edit} {
    exec gvim [file nativename [file normalize $filename]]
  }
};# processFile

proc processData {data} {
  # set data [fixCopyrightDates $data]
  # set data [fixItcl $data]
  set data [fixTile $data]
  # set data [fixUnderline $data]
  return [string trim $data]
};# processData

proc fixUnderline {data} {
  regsub -all {##\s+\-+\s+##} $data {## --------------------------------------------------                       ##} data
  return $data
};# fixUnderline

proc fixCopyrightDates {data} {
  set year [clock format [clock seconds] -format %Y]
  regsub -all {Copyright \d+-\d+ by:} $data "Copyright 1998-$year by:" data
  return $data
};# fixCopyrightDates

proc fixItcl {data} {
  regsub -all {nnItcl}                            $data {TclOO}             data
  regsub -all {ITcl}                            $data {TclOO}             data
  regsub -all {itcl::class}                     $data {oo::class}         data
  regsub -all {\$this }                         $data {[self] }           data
  regsub -all { \$this\}}                       $data " \[self\]\}"       data
  regsub -all { \$this\"}                       $data " \[self\]\""       data
  regsub -all -line { \$this\s*$}               $data { [self]}           data
  regsub -all -line {^(\s*)chain\s*$}           $data {\1next}            data
  regsub -all {itcl::scope}                     $data {my varname}        data
  regsub -all {itcl::local}                     $data {ELEP::Base::local} data
  regsub -all {itcl::delete object \[self\]}    $data {[self] destroy}    data
  regsub -all {\[info class\]}                  $data {[self class]}      data
  foreach {pattern subs} {inherit superclass} {
    if {[regexp $pattern $data]} {
      puts "    => $pattern match!"
      set ::Edit 1
      regsub -all $pattern $data $subs data
    }
  }
  return $data
};# fixItcl

proc fixTile {data} {
  regsub -all {package require tile} $data {# package require tile} data
  regsub -all {package require Tablelist_tile} $data \
      {package require tablelist_tile} data
  return $data
};# fixTile

if {$argc} {
  set DirList $argv
}

foreach dir $DirList {
  processDirectory $dir
}
exit 0
