##########################################################
## This file automatically generates an index for the TclLib package.
##
set tclLibPath /disk2b/local/sources/tcl/tcllib
set CWD [pwd]
catch {
  foreach file [glob *.html] {
    puts "Removing $file..."
    file delete -force -- $file
  }
}

## Generate the list of packages...
cd $tclLibPath/modules
set Modules [lsort [glob -type d *]]
cd $CWD

## Open the two files we write our output...
set index [open index.html w]
set Index {d {/Tcl Standart Library (tcllib)}  [file join $DD TclLib index.html]\\
}

## Write an HTML Header in the index.html file...
puts $index "<html><head><title>Tcl Standart Library Modules</title></head>"
puts $index "<body><h1>Tcl Standart Library Modules</h1>"

proc Module {module} {
  if {[string equal CVS $module]} {return}
  global tclLibPath CWD index Index 
  
  puts "Processing Module: \"$module\"..."
  ## See how many submodules are defined by this module...
  cd $tclLibPath/modules/$module
  set subModules {}
  set Files {}
  if {![catch {glob *.man} man]} {set Files [concat $Files $man]}
  if {![catch {glob *.n}   n]}   {set Files [concat $Files $n]}
  foreach file $Files {
    ## Is it a dev page?
    if {[string match *.dev.man $file]} {continue}
    lappend subModules [file rootname $file]
  }
  set subModules [lsort -unique $subModules]
  if {[llength $subModules] == 0} {
    puts "    >>> No documentation found. Skipping Module..."
    append Index "f {/Tcl Standart Library (tcllib)/$module} {}\\\n"
    puts $index "<li><font color=\"#ffd700\"><b>$module</b></font>: <font color=\"#c0c0c0\">&lt;No documentation available&gt;</font></li>"
    cd $CWD
    return
  }
  puts "    + Sub-Modules: ([llength $subModules]) $subModules"

  ## Examine each sub-module...
  foreach submod $subModules {
    ## Is there an html file?
    if {[file exists $tclLibPath/doc/html/$submod.html]} {
      puts "      Located HTML page for \"$submod\""
      file copy -force $tclLibPath/doc/html/$submod.html $CWD
    } else {
      ## We didn't find an html page. Maybe we can generate one?
      if {[file exists $submod.n]} {
        cd $tclLibPath/doc
        exec man2html $tclLibPath/modules/$module/$submod.n > $CWD/$submod.html
        #exec $tclLibPath/modules/doctools/mpexpand -module $module html \
             $tclLibPath/modules/$module/$submod.n $CWD/$submod.html
        puts "      ! Generating HTML page for \"$submod\""
	cd $tclLibPath/modules/$module
      } else {
        puts "    >>> No man pages found. Skipping Module..."
        append Index "f {/Tcl Standart Library (tcllib)/$module} {}\\\n"
        puts $index "<li><font color=\"#ffd700\"><b>$module</b></font>: <font color=\"#c0c0c0\">&lt;No documentation available&gt;</font></li>"
        cd $CWD
        return
      }
    }
  }

  ## We have copied/generated all html files. Now, update the indexes..
  if {[llength $subModules] == 1} {
    append Index "f {/Tcl Standart Library (tcllib)/$subModules} \[file join \$DD TclLib $subModules.html\]\\\n"
    puts $index "<li><font color=\"#6b8e23\"><b>$module:</b></font> <a href=\"$subModules.html\">$subModules</a></li>"
  } else {
    if {[file exists $CWD/$module.html]} {
      append Index "d {/Tcl Standart Library (tcllib)/$module} \[file join \$DD TclLib $module.html\]\\\n"
    } else {
      append Index "d {/Tcl Standart Library (tcllib)/$module} \[file join \$DD TclLib index.html\]\\\n"
    }
    puts $index "<li><font color=\"#6b8e23\"><b>$module:</b></font> "
    foreach submod $subModules {
      append Index "f {/Tcl Standart Library (tcllib)/$module/$submod} \[file join \$DD TclLib $submod.html\]\\\n"
      puts $index "<a href=\"$submod.html\">$submod</a> "
    }
    puts $index "</li>"
  }
  cd $CWD
}

foreach module $Modules {
  Module $module
}

## Generate the root Index...
cd $CWD
set IndexFile [open ../Index.in]
set Data [read $IndexFile]
close $IndexFile
regsub -all -- {###TCL_LIB_INDEX###} $Data [string trim $Index] Data
set IndexFile [open ../Index w]
puts -nonewline $IndexFile $Data
close $IndexFile
