package provide tcanvas 1.0

##  tcanvas.tcl
##
##	Tiled-canvas : an extension to the canvas widget.
##
##  Copyright (c) 2003 <Irrational Numbers> : <aburatti@libero.it> 
##
##  NOTE: package "snit" (v0.91) is required.
##   It is freely downloadable from the Web:
##       http://www.wjduquette.com/snit
##
##  This program is free software; you can redistibute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  See the file "license" for information on usage and
##  redistribution of this program, and for a disclaimer of all warranties.
##

package require snit 

#
# How to use tcanvas:
#   Read "tcanvas.txt" for detailed info.
#   Sample code in provided in "demo*.tcl".
#


::snit::widgetadaptor tcanvas {
   variable bg

   option -bgimage
   option -bgzoom 1
   option -bgimagemargin 0
    # this is not a new option; it is for trapping changes to -scrollregion
   option -scrollregion


   delegate method * to hull except lower 
   delegate option * to hull except -scrollregion

   constructor {args} {
      installhull using canvas

      set bg [image create photo ${win}_bg]
      $hull create image 0 0 -image $bg -anchor nw -tags bgimage

      $self configurelist $args

       # WARNING: next bindings for <Configure> event should be appended !
      bind $win <Configure> [mymethod RedrawBg %w %h]
   }

   destructor {
      image delete $bg
   }

   onconfigure -bgimage {value} {
      set options(-bgimage) $value
      $win RedrawBg [winfo width $win] [winfo height $win]
   }

   onconfigure -bgzoom {value} {
      set options(-bgzoom) $value
      $win RedrawBg [winfo width $win] [winfo height $win]
   }
   
   onconfigure -bgimagemargin {value} {
      set options(-bgimagemargin) $value
      $win RedrawBg [winfo width $win] [winfo height $win]
   }

    # warning: this is critical for intercepting calls to $win
    #  that would be directly delegated to the hull
    #    (delegate method * to hull)
    # Instead, some special processing is done, and then the
    #  method is passed to the hull.
    # (Note: to be correct, first the method is passed to the hull,
    #        then some special processing (RedrawBg) is done. )
    # 
   onconfigure -scrollregion {value} {
      set options(-scrollregion) $value
       # note that we should call the $hull object;
       #  if we call the $win object, it causes a never ending recursion!
      $hull configure -scrollregion $value
      $win RedrawBg [winfo width $win] [winfo height $win]
   }

   method lower {args} {
       # note that we should call the $hull object;
       #  if we call the $win object, it causes a never ending recursion!
      eval $hull lower $args
       # be sure bgimage is always the lowest 
      $hull lower bgimage
   }

    # delete all items on canvas (leaving the background-image intact)
   method clean {} {
	$win delete !bgimage
   }

   proc max { a b } { expr $a > $b ? $a : $b }

    # Private
   method RedrawBg { w h } {
	 $bg blank
	 if { $options(-bgimage) != {} } {
            set scrollReg [ $win cget -scrollregion ]
	    if { $scrollReg == {} } {
	        set x0 0
		set y0 0
		set rW 0
		set rH 0
	    }  else {
                 # be careful to convert them in pixels !
	        set x0 [winfo pixels . [lindex $scrollReg 0]]
	        set y0 [winfo pixels . [lindex $scrollReg 1]]
	        set x1 [winfo pixels . [lindex $scrollReg 2]]
	        set y1 [winfo pixels . [lindex $scrollReg 3]]
		set rW [expr $x1-$x0]
		set rH [expr $y1-$y0]
	    }
	    set W [max $w $rW]
	    set H [max $h $rH]

	    set D $options(-bgimagemargin)
	    set W [expr $W -$D]
	    set H [expr $H -$D]
	    if { $W > $D  && $H > $D } { 
	       set zoom $options(-bgzoom)
	       $bg copy $options(-bgimage) -zoom $zoom $zoom -to $D $D $W $H
	       $win coords bgimage $x0 $y0
            }
         }
   }

}
