#
# Sample XML editing application using package xmledit
#
# (c) Solagem Oy 2002
#

lappend auto_path /disk2a/users/petasis/Ellogon/ellogon2.0/lib
package require xmledit

#
# GUI
#
menu .menubar
. conf -menu .menubar

.menubar add cascade -label File -menu .menubar.file -underline 0
menu .menubar.file -tearoff 0

.menubar.file add command -label "New..." -underline 0 -command {file_new}
.menubar.file add command -label "Open..." -underline 0 -command {file_open}
.menubar.file add command -label "Save" -underline 0 -command {file_save $filename}
.menubar.file add command -label "Save As..." -underline 5 -command {file_save {}}
.menubar.file add separator
.menubar.file add command -label "Exit" -underline 1 -command {destroy .}

# set default size for the window
wm geometry . =600x400

#
# Procedures
#
proc file_open {{f {}}} {
  global filename document

  if {[string length $f]} {
    set filename $f
  } else {
    set filename [tk_getOpenFile -filetypes {{{XML files} {.xml}}} -defaultextension .xml]
  }
  
  if {[string length $filename]} {
    # read file
    set document [xmledit::doc_from_file $filename]
    
    # try to find out schema. Here we assume that the schema must be a local file.
    set root [dom::document cget $document -documentElement]
    set schemafile [dom::element getAttribute $root noNamespaceSchemaLocation]
    
    switch -glob $schemafile {
      http* {
        set schema [xmledit::doc_from_url $schemafile]
      }
      default {
        set schemafile [file join [file dirname $filename] $schemafile]
        if {![file isfile $schemafile]} {
          error "Could not locate schema file \"$schemafile\" for document \"$filename\""
        }
        set schema [xmledit::doc_from_file $schemafile]
      }
    }    
    
    # now start xmledit
    xmledit::configure .xmledit -schema $schema
    xmledit::edit .xmledit $document
    
    # set filename to title
    wm title . $filename
  }
}

proc file_save {new_filename} {
  global filename document

  # if this is called with empty filename (Save As... or New...), ask name from user
  if {![string length $new_filename]} {
    set new_filename [tk_getSaveFile -filetypes {{{XML files} {.xml}}} -defaultextension .xml -initialfile $filename]
  }

  if {[string length $new_filename]} {
    # save file
    set data [xmledit::serialize $document 1]
    set f [open $new_filename w]
    fconfigure $f -translation crlf -encoding binary
    puts -nonewline $f $data
    close $f
    
    set filename $new_filename

    # set filename to title
    wm title . $filename
  }
}

proc file_new {{f {}}} {
  global filename document

  if {[string length $f]} {
    set schemafile $f
  } else {
    set schemafile [tk_getOpenFile -title "Select XML schema for new document" -filetypes {{{XML schemas} {.xsd}}} -defaultextension .xsd]
  }
  
  if {[string length $schemafile]} {
    # read schema
    set schema [xmledit::doc_from_file $schemafile]
    
	# init .xmledit for the given schema
    xmledit::configure .xmledit -schema $schema
    
    # now start xmledit, create new document
    set document [xmledit::new .xmledit -noNamespaceSchemaLocation $schemafile]
    
    # set filename to title
    set filename "" ;# this is indicator in file_save that user must be prompted from filename
    wm title . "New Document"
  }
}


# initialize
xmledit::init .xmledit -menubar .menubar
pack .xmledit -fill both -expand 1

# open file given in command line
file_open [lindex $argv 0]
