<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:xs="http://www.w3.org/2001/XMLSchema">

<!-- 
  Stylesheet for preprocessing schema file for xmledit 
  
  (c) Solagem Oy 2002
-->

<!-- This template generates XPath expression for creating node label at runtime -->
<xsl:template name="nodelabel">
  <xsl:param name="node"/>
  <xsl:choose>
    <xsl:when test="$node/xs:complexType/xs:attribute">
      <xsl:value-of select="'@'"/>
      <xsl:value-of select="$node/xs:complexType/xs:attribute/@name"/>
    </xsl:when>
    <xsl:when test="$node/xs:complexType/xs:simpleContent/xs:extension/xs:attribute">
      <xsl:value-of select="'@'"/>
      <xsl:value-of select="$node/xs:complexType/xs:simpleContent/xs:extension/xs:attribute/@name"/>
    </xsl:when>
    <xsl:when test="$node/xs:complexType/xs:sequence/xs:element">
      <xsl:value-of select="$node/xs:complexType/xs:sequence/xs:element/@name"/>
    </xsl:when>
    <xsl:when test="$node/xs:complexType/xs:all/xs:element">
      <xsl:value-of select="$node/xs:complexType/xs:all/xs:element/@name"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<!-- this template preprocesses all element nodes for xmledit -->
<xsl:template match="xs:element">
  <xsl:copy>
    <!-- 
      Non-simple elements are added id attribute that points to element implementation.
      Nodes tagged with id have a GUI of their own in editing frame. 
      Non-tagged nodes are considered one-to-one childs to parent node, having a field
      in the parent's GUI.
    -->
    <xsl:choose>
     <xsl:when test="@ref">
      <xsl:variable name='ref'><xsl:value-of select='@ref'/></xsl:variable>
      <xsl:attribute name="id"><xsl:value-of select="generate-id(/xs:schema/xs:element[@name=$ref])"/></xsl:attribute>
     </xsl:when>    
     <xsl:when test=".//@maxOccurs='unbounded' or .//@ref or local-name(..) = 'schema'" >
      <xsl:attribute name="id"><xsl:value-of select="generate-id(.)"/></xsl:attribute>
     </xsl:when>    
    </xsl:choose>

    <!--
      nodelabel attribute generated to non-unique nodes. The attribute value
      is used as xpath expression fo generating node identifier in the tree widget.
    -->
    <xsl:if test="@maxOccurs='unbounded'">
      <xsl:attribute name="nodelabel">
      <xsl:choose>
        <xsl:when test="@ref">
          <xsl:variable name='ref'><xsl:value-of select='@ref'/></xsl:variable>
          <xsl:call-template name="nodelabel">
            <xsl:with-param name="node" select="/xs:schema/xs:element[@name=$ref]"/>
          </xsl:call-template>
        </xsl:when>    
        <xsl:otherwise>
          <xsl:call-template name="nodelabel">
            <xsl:with-param name="node" select="."/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
      </xsl:attribute>
    </xsl:if>

    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

<!-- copy everything else as it is -->
<xsl:template match="@*|node()">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

</xsl:stylesheet>

