<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/TR/xhtml1/strict" xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xsl:param name="proc" select="'GUI'"/>
  <xsl:output method="text" encoding="ISO-8859-1"/>
<!--
  Stylesheet for generating Tcl GUI code for xmledit extension.
  
  (c) Solagem Oy 2002
-->

<!--
  Root Template
  This generates the main GUI procedure, and switch to draw
  specified element frame.
  
  GUI procedure parameters:
    w  : host widget path
    id : id attiribute for the schema element
-->
  <xsl:template match="/xs:schema">
proc <xsl:value-of select="$proc"/> {id {w {}}} {
 # Establish framedata array. This array is used for field value storage.
 # Field values are referenced using array items framedata([xpath]) where
 #
 # xpath is the relative xpath to the element or attribute.
 # For implementation reasons, the xpath value should always
 # start with ".", so that "." refers to the element itself, ./xx to element xx
 # and ./@yy to attribute yy.

 set xmldata data$w 
 upvar #0 data$w framedata
 if {[info exist framedata]} {unset framedata}
 set framedata(FRAME) $w
 set xpath .

 # create notebook widget to hold fields
 set notebook [NoteBook $w.notebook]
 
 switch $id {
 <!-- process only elements having id value set -->
    <xsl:for-each select="//xs:element[@id and not(@ref)]">
      <xsl:value-of select="@id"/> {
  <xsl:apply-templates select="."/>
  }
  </xsl:for-each>
  default {
   error "Invalid element identifier: $id"
  }
 }

 $notebook raise [$notebook pages 0]
 pack $notebook -fill both -expand 1 -side top -padx 5 -pady 5 -anchor w

 # standard buttons, OK and Cancel
 frame $w.fAccept
 button $w.fAccept.bOk -text OK -width 10 -command "xmledit::frame_save $w"
 helptext $w.fAccept.bOk {Save values}
 button $w.fAccept.bCancel -text Cancel -width 10 -command "xmledit::frame_display $w"
 helptext $w.fAccept.bCancel {Return original values}
 pack $w.fAccept.bOk $w.fAccept.bCancel -side left -padx 5 -pady 10

 # apply button, if proc apply defined
 if {[llength [info procs ::config::apply]]} {
   button $w.fAccept.bApply -text Apply -width 10 -command "
     xmledit::frame_save $w
     ::config::apply
   "
   helptext $w.fAccept.bApply {Save and apply}
   pack $w.fAccept.bApply -side left -padx 5 -pady 10
 }
 
 pack $w.fAccept -side bottom -anchor e

}
</xsl:template>

<!-- 
  Simple element templates
-->

  <!-- Generic simple field type, this matches anything -->
  <xsl:template match="xs:element">
    <xsl:if test="@id">
  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame]
  pack $f -side top -anchor nw

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
      <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
      <xsl:apply-templates select=".//xs:attribute"/>
    </xsl:if>

  # "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  entry $f.e<xsl:value-of select="@name"/> -textvariable ${xmldata}(${xpath})
  helptext $f.e<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.e<xsl:value-of select="@name"/> -sticky nwe
</xsl:template>

  <!-- Enumerated element, use radio set -->
  <xsl:template match="xs:element[xs:simpleType/xs:restriction/xs:enumeration]">
  # "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
    <xsl:if test="@id">
    # new page to notebook
    set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
    # create subframe for page for easier packing, f is referenced in all subtemplates
    set f [frame $page.frame]
    pack $f -side top -anchor nw

    <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
    <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
    <xsl:apply-templates select=".//xs:attribute"/>
    </xsl:if>
    <xsl:variable name="name" select="@name"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:  
  frame $f.f<xsl:value-of select="@name"/> -relief groove -borderwidth 2
  helptext $f.f<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
    <xsl:if test="@use != 'required' and not(@default)">
  radiobutton $f.f<xsl:value-of select="$name"/>.r0 -text {Not Selected} -value {} -variable ${xmldata}(${xpath})
  set framedata(${xpath}) {}
  pack $f.f<xsl:value-of select="$name"/>.r0 -side left -anchor nw
</xsl:if>
    <xsl:for-each select="xs:simpleType/xs:restriction/xs:enumeration">
  radiobutton $f.f<xsl:value-of select="$name"/>.r<xsl:value-of select="position()"/> -text <xsl:value-of select="@value"/> -value <xsl:value-of select="@value"/> -variable ${xmldata}(${xpath})
  pack $f.f<xsl:value-of select="$name"/>.r<xsl:value-of select="position()"/> -side left -anchor nw
  </xsl:for-each>
    <xsl:if test="@default">
set framedata(${xpath}) {<xsl:value-of select="@default"/>}
</xsl:if>
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.f<xsl:value-of select="@name"/> -sticky nw
</xsl:template>
  <!-- Enumerated element, use combobox -->
  <xsl:template match="xs:element[count(xs:simpleType/xs:restriction/xs:enumeration) > 4]">
  # enumerated "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
    <xsl:if test="@id">
  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame]
  pack $f -side top -anchor nw

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
      <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
      <xsl:apply-templates select=".//xs:attribute"/>
    </xsl:if>

  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  set values {<xsl:for-each select="xs:simpleType/xs:restriction/xs:enumeration">{<xsl:value-of select="@value"/>} </xsl:for-each>}
<xsl:if test="@use != 'required' and not(@default)">
set values [concat {{}} $values]
</xsl:if>
  ComboBox $f.c<xsl:value-of select="@name"/> -values $values -textvariable ${xmldata}(${xpath})
  <xsl:if test="@default">
set framedata(${xpath}) {<xsl:value-of select="@default"/>}
</xsl:if>
  helptext $f.c<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> $f.c<xsl:value-of select="@name"/> -sticky w
</xsl:template>

  <!-- Boolean element, generate checkbox -->
  <xsl:template match="xs:element[@type='xs:boolean']">
  # boolean "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
    <xsl:if test="@id">
  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame]
  pack $f -side top -anchor nw

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
      <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
      <xsl:apply-templates select=".//xs:attribute"/>
    </xsl:if>

  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  checkbutton $f.c<xsl:value-of select="@name"/> -onvalue true -offvalue false -variable ${xmldata}(${xpath})
<xsl:if test="@default">
set framedata(${xpath}) {<xsl:value-of select="@default"/>}
</xsl:if>
  helptext $f.c<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> $f.c<xsl:value-of select="@name"/> -sticky w
</xsl:template>

  <!-- Custom type longstring displayed in text widget -->
  <xsl:template match="xs:element[@type='longstring']">
  # longstring "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
    <xsl:if test="@id">
  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame]
  pack $f -side top -anchor nw

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
      <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
      <xsl:apply-templates select=".//xs:attribute"/>
    </xsl:if>

  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>
  frame $f.f<xsl:value-of select="@name"/>
  text $f.f<xsl:value-of select="@name"/>.text -height 8 -width 40 -yscrollcommand "$f.f<xsl:value-of select="@name"/>.vScroll set"
  # special hook to associate a variable with a text widget using trace
  textvariable $f.f<xsl:value-of select="@name"/>.text ${xmldata}(${xpath}) 
  helptext $f.f<xsl:value-of select="@name"/>.text {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  scrollbar $f.f<xsl:value-of select="@name"/>.vScroll -command "$f.f<xsl:value-of select="@name"/>.text yview"
  pack $f.f<xsl:value-of select="@name"/>.text -side left -fill both -expand 1
  pack $f.f<xsl:value-of select="@name"/>.vScroll -side left -fill y -anchor e
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.f<xsl:value-of select="@name"/> -sticky nwe
</xsl:template>
  <!-- Generic simple attribute -->
  <xsl:template match="xs:attribute">
  # "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  entry $f.e<xsl:value-of select="@name"/> -textvariable ${xmldata}(${xpath}/@<xsl:value-of select="@name"/>)
  helptext $f.e<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.e<xsl:value-of select="@name"/> -sticky nwe
</xsl:template>

  <!-- Enumerated attribute, use radio set -->
  <xsl:template match="xs:attribute[xs:simpleType/xs:restriction/xs:enumeration]">
  # "@<xsl:value-of select="@name"/>"
  <xsl:variable name="name" select="@name"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:  
  frame $f.f<xsl:value-of select="@name"/> -relief groove -borderwidth 2
  helptext $f.f<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  <xsl:if test="@use != 'required' and not(@default)">
 radiobutton $f.f<xsl:value-of select="$name"/>.r0 -text {Not Selected} -value {} -variable ${xmldata}(${xpath}/@<xsl:value-of select="$name"/>)
set framedata(${xpath}/@<xsl:value-of select="$name"/>) {}
  pack $f.f<xsl:value-of select="$name"/>.r0 -side left -anchor nw
</xsl:if>
    <xsl:for-each select="xs:simpleType/xs:restriction/xs:enumeration">
  radiobutton $f.f<xsl:value-of select="$name"/>.r<xsl:value-of select="position()"/> -text <xsl:value-of select="@value"/> -value <xsl:value-of select="@value"/> -variable ${xmldata}(${xpath}/@<xsl:value-of select="$name"/>)
  pack $f.f<xsl:value-of select="$name"/>.r<xsl:value-of select="position()"/> -side left -anchor nw
  </xsl:for-each>
    <xsl:if test="@default">
set framedata(${xpath}/@<xsl:value-of select="$name"/>) {<xsl:value-of select="@default"/>}
</xsl:if>
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.f<xsl:value-of select="@name"/> -sticky nw
</xsl:template>

  <!-- Enumerated attribute, use combobox -->
  <xsl:template match="xs:attribute[count(xs:simpleType/xs:restriction/xs:enumeration) > 4]">
  # "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
    set values {<xsl:for-each select="xs:simpleType/xs:restriction/xs:enumeration">{<xsl:value-of select="@value"/>} </xsl:for-each>}
<xsl:if test="@use != 'required' and not(@default)">
set values [concat {{}} $values]
</xsl:if>
  ComboBox $f.c<xsl:value-of select="@name"/> -values $values -textvariable ${xmldata}(${xpath}/@<xsl:value-of select="@name"/>)
<xsl:if test="@default">
set framedata(${xpath}/@<xsl:value-of select="@name"/>) {<xsl:value-of select="@default"/>}
</xsl:if>
  helptext $f.c<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> $f.c<xsl:value-of select="@name"/> -sticky w
</xsl:template>
  <!-- Boolean attribute, generate checkbox -->
  <xsl:template match="xs:attribute[@type='xs:boolean']">
  # boolean "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  checkbutton $f.c<xsl:value-of select="@name"/> -onvalue true -offvalue false -variable ${xmldata}(${xpath}/@<xsl:value-of select="@name"/>)
<xsl:if test="@default">
set framedata(${xpath}/@<xsl:value-of select="@name"/>) {<xsl:value-of select="@default"/>}
</xsl:if>
  helptext $f.c<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> $f.c<xsl:value-of select="@name"/> -sticky w
</xsl:template>

  <!-- Custom type longstring displayed in text widget -->
  <xsl:template match="xs:attribute[@type='longstring']">
  # longstring "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  frame $f.f<xsl:value-of select="@name"/>
  text $f.f<xsl:value-of select="@name"/>.text -height 8 -width 40 -yscrollcommand "$f.f<xsl:value-of select="@name"/>.vScroll set"
  # special hook to associate a variable with a text widget using trace
  textvariable $f.f<xsl:value-of select="@name"/>.text ${xmldata}(${xpath}/@<xsl:value-of select="@name"/>) 
  helptext $f.f<xsl:value-of select="@name"/>.text {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  scrollbar $f.f<xsl:value-of select="@name"/>.vScroll -command "$f.f<xsl:value-of select="@name"/>.text yview"
  pack $f.f<xsl:value-of select="@name"/>.text -side left -fill both -expand 1
  pack $f.f<xsl:value-of select="@name"/>.vScroll -side left -fill y -anchor e
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.f<xsl:value-of select="@name"/> -sticky nwe
  </xsl:template>

  <!-- Visible documentation -->
  <xsl:template match="xs:documentation">
  # element documentation shown in a label widget
  label $f.lannotation -justify left -wraplength 300 -text {<xsl:value-of select="."/>} 
  grid $f.lannotation -columnspan 2 -pady 5 -sticky nw
  </xsl:template>

  <!-- 
  Generic complex element template.

  This should match most calls from main template, but this can also
  be called recursively.

  Complex elements define their own page in notebook.
  -->

  <xsl:template match="xs:element[xs:complexType]">
  # element "<xsl:value-of select="@name"/>"
  
  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame] 
  pack $f -side top -anchor nw

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
    <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
    <xsl:apply-templates select="xs:complexType/xs:attribute"/>
    <xsl:for-each select="xs:complexType/xs:sequence/xs:element[not(@id) and not(xs:complexType)]|xs:complexType/xs:all/xs:element[not(@id) and not(xs:complexType)]|xs:complexType/xs:sequence/xs:choice/xs:element[not(@id) and not(xs:complexType)]">
    push xpath <xsl:value-of select="@name"/>
      <xsl:apply-templates select="."/>
    pop xpath
  </xsl:for-each>
  
  # set minumum size for the second column
  grid columnconfigure $f 1 -minsize 200

  <!-- complexType elements are displayed in their own pages -->
    <xsl:for-each select="xs:complexType/xs:sequence/xs:element[not(@id) and xs:complexType]|xs:complexType/xs:all/xs:element[not(@id) and xs:complexType]">
  push xpath <xsl:value-of select="@name"/>
      <xsl:apply-templates select="."/>
  pop xpath

  </xsl:for-each>
  </xsl:template>

  <!-- Complex type, simple content -->
  <xsl:template match="xs:element[xs:complexType/xs:simpleContent]">
  # "<xsl:value-of select="@name"/>" <xsl:value-of select="local-name(.)"/>

  # new page to notebook
  set page [$notebook insert end <xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>]
  
  # create subframe for page for easier packing, f is referenced in all subtemplates
  set f [frame $page.frame]
  pack $f -side top -anchor nw 

  <!-- Attributes and simple elements with type attribute set, are displayed on the first page -->
    <xsl:apply-templates select="xs:annotation/xs:documentation[1]"/>
    <xsl:apply-templates select=".//xs:attribute"/>
    <!-- datatype support for complexType/simpleContent elements is here -->
    <xsl:choose>
      <xsl:when test="xs:complexType/xs:simpleContent/xs:extension/@base='longstring'">
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  frame $f.f<xsl:value-of select="@name"/>
  text $f.f<xsl:value-of select="@name"/>.text -height 8 -width 40 -yscrollcommand "$f.f<xsl:value-of select="@name"/>.vScroll set"
  # special hook to associate a variable with a text widget using trace
  textvariable $f.f<xsl:value-of select="@name"/>.text ${xmldata}(${xpath}) 
  helptext $f.f<xsl:value-of select="@name"/>.text {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  scrollbar $f.f<xsl:value-of select="@name"/>.vScroll -command "$f.f<xsl:value-of select="@name"/>.text yview"
  pack $f.f<xsl:value-of select="@name"/>.text -side left -fill both -expand 1
  pack $f.f<xsl:value-of select="@name"/>.vScroll -side left -fill y -anchor e
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.f<xsl:value-of select="@name"/> -sticky nwe
  </xsl:when>
      <xsl:when test="xs:complexType/xs:simpleContent/xs:extension/@base='xs:boolean'">
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  checkbutton $f.c<xsl:value-of select="@name"/> -onvalue true -offvalue false -variable ${xmldata}(${xpath})
<xsl:if test="@default">
set framedata(${xpath}) {<xsl:value-of select="@default"/>}
</xsl:if>
  helptext $f.c<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> $f.c<xsl:value-of select="@name"/> -sticky w
  </xsl:when>
      <xsl:otherwise>
  label $f.l<xsl:value-of select="@name"/> -text <xsl:value-of select="@name"/>:
  entry $f.e<xsl:value-of select="@name"/> -textvariable ${xmldata}(${xpath})
  helptext $f.e<xsl:value-of select="@name"/> {<xsl:value-of select="xs:annotation/xs:documentation"/>}
  grid $f.l<xsl:value-of select="@name"/> -sticky nw
  grid ^ $f.e<xsl:value-of select="@name"/> -sticky nwe
  </xsl:otherwise>
    </xsl:choose>

  # set minumum size for the second column
  grid columnconfigure $f 1 -minsize 200
</xsl:template>

</xsl:stylesheet>
