#! /bin/sh
# \
exec tclsh8.3 "$0" ${1+"$@"}

lappend auto_path [file join [file dirname [info script]] ..]
package require ylex
package require yeti

if {$argv == "-v"} {
    set verbosity 10
} else {
    set verbosity 0
}

#
# This tests handling of the dangling else
#
# It also demonstrates a shift/reduce conflict upon reading ELSE. In the
# state `expr --> IF expr . ELSE expr', yeti sees that it is legitimate
# to shift ELSE. Hoever, yeti also notices that ELSE is a valid lookahead
# for reducing expr (as in an inner if statement). yeti always resolves
# such a conflict by preferring the shift.
#

set scannergenerator [yeti::ylex #auto -name dscanner -verbose $verbosity]

$scannergenerator add {
    if				{ return IF }
    else			{ return ELSE }
    ;				{ return SEMICOLON }
    {[a-zA-Z_][a-zA-Z0-9_]*}	{ return [list IDENTIFIER $yytext] }
}

set parsergenerator [yeti::yeti #auto -name dangling -verbose $verbosity]

$parsergenerator add {
    start		{expr SEMICOLON} -
    start		{start expr SEMICOLON} -
    expr		{IF expr else} {
	puts "read: if $2 else $3"
	return "(inner if with else)"
    }
    |			{IF expr} {
	puts "read: if $2"
	return "(inner if w/o else)"
    }
    |			{IDENTIFIER} -
    else		{ELSE expr} {
	return $2
    }
}

eval [$scannergenerator dump]
eval [$parsergenerator dump]

if {$verbosity} {
    set f [open "foo.tcl" w]
    puts $f [$scannergenerator dump]
    puts $f [$parsergenerator dump]
    close $f
}

set scanner [dscanner #auto -verbose $verbosity]
set parser  [dangling #auto -scanner $scanner -verbose $verbosity]

$scanner start {
    if foo;
    if if bar else foobar;
    if foo;
}

$parser parse
