#! /bin/sh
# \
exec tclsh8.3 "$0" ${1+"$@"}

lappend auto_path [file join [file dirname [info script]] ..]
package require ylex
package require yeti

if {$argv == "-v"} {
    set verbosity 10
} else {
    set verbosity 0
}

#
# This tests arbitration of the reduction to use by lookahead.
# Up until PAREN_CLOSE, both fwd_header and func_header are
# appropriate, but the lookahead (BRACE_OPEN or SEMICOLON) can
# be used to determine which reduction to use.
#

set scannergenerator [yeti::ylex #auto -name iscanner -verbose $verbosity]

$scannergenerator add {
    void			{ return VOID }
    {\(}			{ return PAREN_OPEN }
    {\)}			{ return PAREN_CLOSE }
    ;				{ return SEMICOLON }
    {[a-zA-Z_][a-zA-Z0-9_]*}	{ return [list IDENTIFIER $yytext] }
    {\{}			{ return BRACE_OPEN }
    {\}}			{ return BRACE_CLOSE }
}

set parsergenerator [yeti::yeti #auto -name forward -verbose $verbosity]

$parsergenerator add {
    start		{decl SEMICOLON} -
    start		{start decl SEMICOLON} -
    decl		{forward_dcl} {
	puts "forward declaration of $1"
    }
    |			{func_dcl} {
	puts "function definition of $1"
    }
    forward_dcl		{fwd_header} -
    func_dcl		{func_header BRACE_OPEN BRACE_CLOSE} -
    fwd_header		{VOID IDENTIFIER PAREN_OPEN PAREN_CLOSE} {
	return $2
    }
    func_header		{VOID IDENTIFIER PAREN_OPEN PAREN_CLOSE} {
	return $2
    }
}

eval [$scannergenerator dump]
eval [$parsergenerator dump]

if {$verbosity} {
    set f [open "foo.tcl" w]
    puts $f [$scannergenerator dump]
    puts $f [$parsergenerator dump]
    close $f
}

set scanner [iscanner #auto -verbose $verbosity]
set parser  [forward #auto -scanner $scanner -verbose $verbosity]

$scanner start {
    void this_is_a_forward_dcl ();
    void not_a_forward_dcl () {};
}

$parser parse
