##############################################################################
##  CollectionCreator.tcl:                                                  ##
## --------------------------------------------------                       ##
##  This class implements a widget for creating a new Collection.           ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase
package require ELEP::WindowUtilities::ProgressWidget
package require ELEP::ImportHandlers
package require tablelist_tile

GUI_LoadImage ITOpen
GUI_LoadImage ITAdd
GUI_LoadImage ITRemove

oo::class create ELEP::CollectionUtilities::CollectionCreator {
  superclass ELEP::WindowUtilities::WidgetBase

  variable {*}[info class variables ELEP::WindowUtilities::WidgetBase] \
           collection_path \
           collection_name \
           collection_format \
           collection_encoding \
           collection_handler \
           collection_file_names \
           directory_pattern \
           collection_files_widget \
           collection_file_names_number_label

  constructor {parent_win args} {
    next ttk::frame [::ELEP::WindowUtilities::UniqueWindowName \
                             $parent_win.collectionCreator]
    my variable update_collection
    set collection_name       SampleCollection_1
    set collection_format     native
    set collection_handler    {}
    set collection_file_names {}
    set directory_pattern     *.htm*
    set collection_file_names_number_label {}
    set update_collection     0

    set collection_path     [lindex $::CollectionPath 0]
    set collection_encoding [encoding system]
    my configure {*}$args
    my createWidgets
  };# constructor

  ##########################################################################
  ## Method: updateCollection
  ##########################################################################
  method updateCollection {boolean} {
    my variable update_collection
    set update_collection $boolean
  };# updateCollection

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    grid [my createCollectionPropertiesWidget] -sticky snew -padx 2 -pady 2
    grid [my createCollectionFilesWidget] -sticky snew -padx 2 -pady 2
    my updateFileNameNumberLabel
    grid columnconfigure $widget 0 -weight 1
    grid rowconfigure $widget 1 -weight 1
  };# createWidgets

  ##########################################################################
  ## Method: createCollectionPropertiesWidget
  ##########################################################################
  method createCollectionPropertiesWidget {} {
    ttk::labelframe $widget.cp -text { Collection Properties }
      ## Collection location on disk:
      ttk::label $widget.cp.pathl    -anchor w -text {Disk Location:}
      ttk::combobox $widget.cp.path -textvariable \
        [my varname collection_path] -state readonly -width 60
      grid $widget.cp.pathl $widget.cp.path - -sticky snew -padx 2 -pady 2
      bind $widget.cp.path <Enter> "%W configure -values \[set CollectionPath\]"
      ## Collection name:
      ttk::label $widget.cp.namel    -anchor w -text {Name:}
      ttk::entry $widget.cp.name -textvariable [my varname collection_name]
      grid $widget.cp.namel $widget.cp.name - -sticky snew -padx 2 -pady 2
      ## Collection default encoding:
      ttk::label $widget.cp.dencl    -anchor w -text {Default Encoding:}
      ttk::menubutton $widget.cp.denc -textvariable \
        [my varname collection_encoding] -menu $widget.cp.denc.menu \
        -image [::Encoding::GetEncodingImage $collection_encoding] \
        -compound left
      grid $widget.cp.dencl $widget.cp.denc - -sticky snew -padx 2 -pady 2
      menu $widget.cp.denc.menu -tearoff 0
      GUI_AppendEncodingsToMenu $widget.cp.denc.menu \
        "set \{[my varname collection_encoding]\} \$Encoding;\
         $widget.cp.denc configure -image \$EncodingImage -text \$Encoding" 1
      ## Collection format:
      ttk::label $widget.cp.formatl  -anchor w -text {Format:}
      ttk::combobox $widget.cp.format -textvariable \
        [my varname collection_format] -state readonly \
        -values [CDM_GetRegisteredCollectionTypes]
      grid $widget.cp.formatl $widget.cp.format - -sticky snew -padx 2 -pady 2
      ## Collection default handler:
      ttk::label $widget.cp.handlerl  -anchor w -text {Default Handler:}
      ttk::combobox $widget.cp.handler -textvariable \
        [my varname collection_handler] -state readonly
      bind $widget.cp.handler <<ComboboxSelected>> \
        [list [self] onCollectionHandlerChanged]
      ttk::button $widget.cp.handlero -text {Handler Options} -state disabled
      grid $widget.cp.handlerl $widget.cp.handler $widget.cp.handlero \
        -sticky snew -padx 2 -pady 2
      bind $widget.cp.handler <Enter> "%W configure -values \[linsert\
        \[CDM_GetRegisteredCollectionHadlerLabels\] 0 {}\]"
      my onCollectionHandlerChanged
      grid columnconfigure $widget.cp 1 -weight 1
    return $widget.cp
  };# createCollectionPropertiesWidget

  ##########################################################################
  ## Method: createCollectionFilesWidget
  ##########################################################################
  method createCollectionFilesWidget {} {
    ttk::labelframe $widget.cf -text { Collection Files }
      ## Add a section for adding files in the Collection:
      ttk::labelframe $widget.cf.add -text { Add Files from Directory }
        ttk::label $widget.cf.add.dirl -anchor w \
          -textvariable GUI_Text(FileSearchDirectory)
        ttk::entry $widget.cf.add.dir \
          -textvariable GUI_Options(NewCollectionFileSearchPath)
        catch {$widget.cf.add.dir xview end}
        GUI_MakeEntryDirectoryDropTarget $widget.cf.add.dir
        ttk::button $widget.cf.add.add -text {...} -image ITOpen \
          -compound left -command [list GUI_SelectDirectory $widget.cf.add.dir]
        grid $widget.cf.add.dirl $widget.cf.add.dir $widget.cf.add.add \
          -sticky snew -padx 2 -pady 2
        ttk::label $widget.cf.add.patternl -anchor w \
          -textvariable GUI_Text(FileFilter)
        ttk::combobox $widget.cf.add.pattern -textvariable \
          [my varname directory_pattern] -values {*.txt *.htm* *.* *}
        ttk::button $widget.cf.add.apply -image ITAdd -command \
          [list [self] onAddFiles]
        grid $widget.cf.add.patternl $widget.cf.add.pattern \
          $widget.cf.add.apply -sticky snew -padx 2 -pady 2
        ttk::checkbutton $widget.cf.add.recsrch \
          -text { Recursivelly check all sub-directories} \
          -variable GUI_Options(NewCollectionFileSearchPathRecursively)
        grid $widget.cf.add.recsrch -columnspan 3 -sticky snew -padx 2 -pady 2
        ttk::checkbutton $widget.cf.add.autodetect_enc \
          -text { Auto-detect file encoding} \
          -variable GUI_Options(NewCollectionAutodetectEncoding)
        grid $widget.cf.add.autodetect_enc -columnspan 3 \
          -sticky snew -padx 2 -pady 2
        grid columnconfigure $widget.cf.add 1 -weight 1
      ## Add a section for showing the files and a button to remove them:
      ttk::labelframe $widget.cf.files -text \
        { Files to be added to the Collection }
        ScrolledWindow $widget.cf.files.sw
        set collection_files_widget [$widget.cf.files.sw getframe].l
        tablelist::tablelist $collection_files_widget -height 10 \
          -columns {0 {File Name} 12 Encoding center 0 Handler} \
          -listvariable [my varname collection_file_names] \
          -selectmode extended -stretch {0 2} \
          -editstartcommand [list [self] onFileNameRowEdit]
        $collection_files_widget columnconfigure 1 -editable yes \
          -editwindow ttk::menubutton -sortmode dictionary
        $collection_files_widget columnconfigure 2 -editable yes \
          -editwindow ttk::combobox -sortmode dictionary
        $widget.cf.files.sw setwidget $collection_files_widget
        ttk::button $widget.cf.files.del -image ITRemove -command \
          [list [self] onDeleteFiles]
        grid $widget.cf.files.sw $widget.cf.files.del \
          -sticky snew -padx 2 -pady 2
        ## Finally, add a label for showing the number of files:
        ttk::label $widget.cf.files.number -textvariable \
          [my varname collection_file_names_number_label]
        grid $widget.cf.files.number - -sticky snew -padx 2 -pady 2
        grid columnconfigure $widget.cf.files 0 -weight 1
        grid rowconfigure $widget.cf.files 0 -weight 1
      grid $widget.cf.add -sticky snew -padx 2 -pady 2
      grid $widget.cf.files -sticky snew -padx 2 -pady 2
      grid columnconfigure $widget.cf 0 -weight 1
      grid rowconfigure $widget.cf 1 -weight 1
    return $widget.cf
  };# createCollectionFilesWidget

  ##########################################################################
  ## Event: onCollectionHandlerChanged
  ##########################################################################
  method onCollectionHandlerChanged {} {
    catch {destroy $widget.cp.handlero.options}
    $widget.cp.handlero configure -state disabled -command {}
    if {[string length $collection_handler]} {
      set cmd [CDM_GetRegisteredCollectionHadlerCommand $collection_handler]
      catch {${cmd}_OptionsDialog};# Autoload command!
      if {[llength [info commands ${cmd}_OptionsDialog]]} {
        $widget.cp.handlero configure -state normal -command \
          [list ${cmd}_OptionsDialog $widget.cp.handlero.options]
      }
    }
  };# onCollectionHandlerChanged

  ##########################################################################
  ## Event: onCreateNewCollection
  ##########################################################################
  method onCreateNewCollection {} {
    my notifyListeners
  };# onCreateNewCollection

  ##########################################################################
  ## Event: onReCreateNewCollection
  ##########################################################################
  method onReCreateNewCollection {} {
  };# onReCreateNewCollection

  ##########################################################################
  ## Event: onFileNameRowEdit
  ##########################################################################
  method onFileNameRowEdit {tbl row col text} {
    set w [$tbl editwinpath]
    switch -exact $col {
      1 {
        menu $w.menu -tearoff 0
        GUI_AppendEncodingsToMenu $w.menu \
          "$w configure -text \$Encoding"
        $w configure -menu $w.menu
      }
      2 {
        $w configure -values [concat {{}} \
                [CDM_GetRegisteredCollectionHadlerLabels]]
      }
    }
    return $text
  };# onFileNameRowEdit

  ##########################################################################
  ## Event: onDeleteFiles
  ##########################################################################
  method onDeleteFiles {} {
    ## Are any file names selected?
    set sel [$collection_files_widget curselection]
    if {[llength $sel]} {
      $collection_files_widget delete $sel
    } else {
      ## No selection. Apply pattern...
      set pattern [file join \
            $::GUI_Options(NewCollectionFileSearchPath) $directory_pattern]
      set new_collection_file_names {}
      foreach item $collection_file_names {
        set file_name [lindex $item 0]
        if {![string match $pattern $file_name]} {
          lappend new_collection_file_names $item
        }
      }
      set collection_file_names $new_collection_file_names
    }
    my updateFileNameNumberLabel
  };# onDeleteFiles

  ##########################################################################
  ## Event: onAddFiles
  ##########################################################################
  method onAddFiles {} {
    if {$::GUI_Options(NewCollectionFileSearchPathRecursively)} {
      package require fileutil
      set files [::fileutil::findByPattern \
        $::GUI_Options(NewCollectionFileSearchPath) -glob -- \
        [list $directory_pattern]]
    } else {
      set files [glob -nocomplain -types {f l r} [file join \
              $::GUI_Options(NewCollectionFileSearchPath) $directory_pattern]]
    }
    if {$::GUI_Options(NewCollectionAutodetectEncoding)} {
      package require ELEP::DocumentUtilities::EncodingDetection
      set detector [ELEP::DocumentUtilities::EncodingDetection \
          create [ELEP::Base::auto ELEP::CollectionUtilities::] \
          $collection_encoding]
      foreach file $files {
        lappend collection_file_names [list \
          $file [$detector encoding $file] $collection_handler]
      }
      $detector destroy
    } else {
      foreach file $files {
        lappend collection_file_names [list \
          $file $collection_encoding $collection_handler]
      }
    }
    my updateFileNameNumberLabel
  };# onAddFiles

  ##########################################################################
  ## Method: addFileName
  ##########################################################################
  method addFileName {file_name {encoding -none-} {handler -none-}} {
    if {[string equal $encoding -none-]} {
      set encoding $collection_encoding
    }
    if {[string equal $handler -none-]} {
      set handler $collection_handler
    }
    lappend collection_file_names [list $file_name $encoding $handler]
    my updateFileNameNumberLabel
  };# addFileName

  ##########################################################################
  ## Method: addFileNames
  ##########################################################################
  method addFileNames {file_name_list {encoding -none-} {handler -none-}} {
    if {[string equal $encoding -none-]} {
      set encoding $collection_encoding
    }
    if {[string equal $handler -none-]} {
      set handler $collection_handler
    }
    foreach file_name $file_name_list {
      lappend collection_file_names [list $file_name $encoding $handler]
    }
    my updateFileNameNumberLabel
  };# addFileName

  ##########################################################################
  ## Method: updateFileNameNumberLabel
  ##########################################################################
  method updateFileNameNumberLabel {} {
    set collection_file_names_number_label \
      "[llength $collection_file_names] $::GUI_Text(Files...)"
  };# updateFileNameNumberLabel

  ##########################################################################
  ## Method: saveCollection
  ##########################################################################
  method saveCollection {} {
    my variable update_collection
    set collection_xid [file join $collection_path $collection_name]
    ## Step 1: Inform the user if Collection is going to be empty!
    if {![llength $collection_file_names]} {
      set ans [gut_question [format "$::GUI_Text(CollectionEmpty)\n" \
                   $collection_xid] Warning... 1 $widget]
      if {[string equal $ans no]} {return 0}
    }

    ## Step 2: Check whether the Collection Directory already exists:
    if {[file exists $collection_xid] && !$update_collection} {
      set ans [gut_question "$::GUI_Text(CollectionExists)\n($collection_xid)" \
                             Warning... 1 $widget]
      if {[string equal $ans no]} {return 0}
      catch {tip_Destroy $collection_xid}
    }
    
    ## Step 3: Create the Collection.
    if {$update_collection} {
      set col [tip_OpenCollection $collection_xid]
      ## Is the Collection's Location in accordance with it's Name?
      set name [tip_GetName $col]
      if {![string equal [file native $name] [file native $collection_xid]]} {
        GUI_FixCollectionName $col $name $collection_xid
      }
    } else {
      set col [tip_CreateCollection $collection_xid {} $collection_encoding]
    }

    ## Step 4: Set its format.
    if {!$update_collection} {
      $col Format $collection_format
    }

    ## Step 5: Create a progress widget, to show progress.
    set progress [ELEP::WindowUtilities::ProgressWidget \
      create [ELEP::Base::auto ELEP::CollectionUtilities::] \
      $widget [llength $collection_file_names]]
    $progress placeInToplevelCenter

    ## Step 6: Iterate over all file names, and add them to the Collection.
    foreach item $collection_file_names {
      foreach {file_name encoding handler} $item {}
      if {![info exists HandlerAttribute($handler)]} {
        set handler_attribute [tip_CreateAttribute {Document Handler} \
                               [tip_CreateAttributeValue GDM_STRING $handler]]
        set HandlerAttribute($handler) $handler_attribute
      } else {
        set handler_attribute $HandlerAttribute($handler)
      }
      set file_dirname         [file dirname $file_name]
      set file_name_tail       [file tail $file_name]
      set file_name_tail_lower [string tolower $file_name_tail]
      set XID $file_name
      set ID  $file_name_tail
      $progress setMessage&Advance "Processing File: $file_name_tail..."
      ## Examine file existence...
      if {![file isfile $file_name]} {
        puts stderr "Skipping not regular file $file_name..."; continue
      }
      ## Read file content...
      set file [open $file_name r]
      fconfigure $file -encoding $encoding
      set raw_data [read $file]
      close $file
      ## Ensure that we have not seen again this file, taking also into account
      ## filesystems that ignore case!
      if {[info exists seen($file_name_tail_lower)]} {
        set i 1; set ID ${file_name_tail_lower}-1
        while {[info exists seen($ID)]} {
          set ID ${file_name_tail_lower}-[incr i]
        }
        set ID ${file_name_tail}-$i
      }
      set seen([string tolower $ID]) 0
      ## Create a Document in the Collection, from the context we have just red.
      set doc [tip_CreateDocument $col $file_dirname/$ID $raw_data \
                                    {} {} $encoding]
      tip_SetExternalId $doc $XID
      ## Should we apply a Collection handler?
      if {[string length $handler]} {
        set cmd [CDM_GetRegisteredCollectionHadlerCommand $handler]
        if {[catch {eval $cmd $col $doc} error]} {
          ## An error occured inside the handler! Inform the user and request an
          ## action...
          set ans [MessageDlg $widget.errorDlg -parent $widget -icon error \
             -default 0 -type user -buttons [list $::GUI_Text(Ignore) \
              $::GUI_Text(DeleteDocument) $::GUI_Text(Abort)] \
             -title {Error during Collection Handler execution...} \
             -message [format $::GUI_Text(ErrorExecutingColHandler) \
             [tip_GetExternalId $doc] $handler "$error\n($::errorInfo)"]]
          switch $ans {
            0 {## Ignore error!}
            1 {## Delete the Document!
               tip_RemoveDocument $col [tip_GetId $doc]; continue
            }
            2 {
              ## We must abort. Destroy the Collection built so far, and
              ## return.
              catch {tip_Destroy $collection_xid}
              catch {tip_Close $col}
              $progress destroyWindow
              return 0
            }
          }
        } else {
          ## The handler application was succesfull.
          ## Check whether an action has been requested by the handler...
          switch -glob $error {
            delete {
              tip_RemoveDocument $col [tip_GetId $doc]
            }
            default {
              ## Set a proper Attribute to denote success.
              tip_PutAttribute $doc $handler_attribute
            }
          }
        };# if {[catch {eval $cmd $col $doc} error]}
      };# if {[string length $handler]}
      ## Save Document on disk, and close it to save memory
      tip_Sync $doc
      tip_Close $doc
    }

    ## Step 7: Save the default Collection handler, if one exists.
    if {[string length $collection_handler]} {
      tip_PutAttribute $col [tip_CreateAttribute {Collection Handler} \
              [tip_CreateAttributeValue GDM_STRING $handler]]
    }

    ## Step 8: Save and close the Collection.
    tip_Sync $col
    tip_Close $col
    $progress destroyWindow
    return 1
  };# saveCollection

  ##########################################################################
  ## Method: initFromCollection
  ##########################################################################
  method initFromCollection {col} {
    my variable update_collection
    set name [tip_GetName $col]
    ## Is there a default handler?
    set handler [tip_GetAttribute $col {Collection Handler}]
    if {[llength $handler]} {set handler [tip_GetValueValue $handler]}
    ## Construct our file names list:
    set encoding [tip_GetEncoding $col]
    set collection_file_names {}
    if {!$update_collection} {
      set doc [tip_FirstDocument $col]
      while {[string length $doc]} {
        set dh [tip_GetAttribute $doc {Document Handler}]
        if {[llength $dh]} {set dh [tip_GetValueValue $dh]}
        lappend collection_file_names [list \
          [tip_GetExternalId $doc] [tip_GetEncoding $doc] $dh]
        tip_Close $doc
        set doc [tip_NextDocument $col]
      }
    }

    set collection_path       [file dirname $name]
    set collection_name       [file tail $name]
    set collection_format     [$col Format]
    set collection_encoding   $encoding
    set collection_handler    $handler
    my updateFileNameNumberLabel
  };# initFromCollection

  ##########################################################################
  ## Method: initFromCollectionPath
  ##########################################################################
  method initFromCollectionPath {col_name} {
    if {![string length $col_name]} {return}
    if {![file exists $col_name]} {return}
    if {[string length [GUI_IsCollectionOpened $col_name]]} {
      gut_warning $::GUI_Text(CollectionIsOpen)
    }
    my variable update_collection
    if {!$update_collection} {
      set message [format $::GUI_Text(UpdateCollection?) $col_name]
      if {[string equal [gut_question $message {Update Collection?} 1 $widget] \
                        no]} {return 0}
    }
    set col [tip_OpenCollection $col_name]
    ## Is the Collection's Location in accordance with it's Name?
    set name [tip_GetName $col]
    if {![string equal [file native $name] [file native $col_name]]} {
      GUI_FixCollectionName $col $name $col_name
    }
    my initFromCollection $col
    tip_Close $col
    return 1
  };# initFromCollectionPath

};# class ELEP::CollectionUtilities::CollectionCreator

## Register ttk::menubutton for tablelist interactive cell editing
proc tablelist::addTileMenubutton {{name ttk::menubutton}} {
     checkEditWinName $name

     array set ::tablelist::editWin [list \
         $name-creationCmd       "createTileMenubutton %W" \
         $name-putValueCmd       "%W configure -text %T" \
         $name-getValueCmd       "%W cget -text" \
         $name-putTextCmd        "%W configure -text %T" \
         $name-getTextCmd        "%W cget -text" \
         $name-putListCmd        "" \
         $name-getListCmd        "" \
         $name-selectCmd         "" \
         $name-invokeCmd         "event generate %W <<Invoke>>" \
         $name-fontOpt           "" \
         $name-useFormat         1 \
         $name-useReqWidth       0 \
         $name-usePadX           1 \
         $name-isEntryLike       0 \
         $name-focusWin          %W \
         $name-reservedKeys      {} \
     ]

     return $name
};# tablelist::addTileMenubutton
#------------------------------------------------------------------------------
# tablelist::createTileCombobox
#
# Creates a tile combobox widget with the given path name for interactive cell
# editing in a tablelist widget.
#------------------------------------------------------------------------------
proc tablelist::createTileMenubutton {w args} {
     ttk::menubutton $w

     foreach {opt val} $args {
         switch -- $opt {
             -font  {}
             -state { $w configure $opt $val }
         }
     }
};# tablelist::createTileMenubutton
tablelist::addTileMenubutton

package provide ELEP::CollectionUtilities::CollectionCreator 1.0
