##############################################################################
##  CollectionCreatorDialog.tcl:                                            ##
## --------------------------------------------------                       ##
##  This class implements a dialog for creating a new Collection.           ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::DialogWindow
package require ELEP::CollectionUtilities::CollectionCreator

oo::class create ELEP::CollectionUtilities::CollectionCreatorDialog {
  superclass ELEP::WindowUtilities::DialogWindow

  constructor {} {
    my variable collection_creator_object
    next
    set collection_creator_object {}
    my createWidgets
  };# constructor

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    my variable collection_creator_object
    ## Create a Collection creator object.
    my createCollectionCreatorObject
    pack [$collection_creator_object getframe] -fill both -expand 1
    ## Add a frame with the ok & cancel buttons.
    my addStdButton ok     [list [self] onOk]
    my addStdButton cancel [list [self] onDestroyDialog]
  };# createWidgets

  ##########################################################################
  ## Method: createCollectionCreatorObject
  ##########################################################################
  method createCollectionCreatorObject {} {
    my variable client_area collection_creator_object
    set collection_creator_object [ELEP::CollectionUtilities::CollectionCreator\
      create [ELEP::Base::auto ELEP::CollectionUtilities::cc] $client_area]
  };# createCollectionCreatorObject

  ##########################################################################
  ## Event: onOk
  ##########################################################################
  method onOk {} {
    my variable collection_creator_object
    if {[$collection_creator_object saveCollection]} {
      my onDestroyDialog
    }
  };# onOk

  ##########################################################################
  ## Method: updateCollection
  ##########################################################################
  method updateCollection {boolean} {
    my variable collection_creator_object
    return [$collection_creator_object updateCollection $boolean]
  };# updateCollection

  ##########################################################################
  ## Method: initFromCollection
  ##########################################################################
  method initFromCollection {col} {
    my variable collection_creator_object
    return [$collection_creator_object initFromCollection $col]
  };# initFromCollection

  ##########################################################################
  ## Method: initFromCollectionPath
  ##########################################################################
  method initFromCollectionPath {col_name} {
    my variable collection_creator_object
    return [$collection_creator_object initFromCollectionPath $col_name]
  };# initFromCollectionPath

};# class ELEP::CollectionUtilities::CollectionCreatorDialog

package provide ELEP::CollectionUtilities::CollectionCreatorDialog 1.0
