##############################################################################
##  CollectionSimpleManagerToplevel.tcl:                                    ##
## --------------------------------------------------                       ##
##  This class implements a toplevel window that presents a simple          ##
##  interface: The user can see/create/modify/rename/delete Collections     ##
##  and run some tools on them.                                             ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ToplevelWindow
package require ELEP::CollectionUtilities::CollectionsViewer
package require ELEP::CollectionUtilities::CollectionSimpleOperations

GUI_LoadImage IApplication

oo::class create ELEP::CollectionUtilities::CollectionSimpleManagerToplevel {
  superclass ELEP::WindowUtilities::ToplevelWindow

  constructor {args} {
    my variable annotator
    set annotator ELEP::VisualAnnotators::HtmlTemplateFiller
    next true
    my createWidgets
    my configure {*}$args
  };# constructor

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    my variable client_area collections_viewer_object \
                            collections_operations_object
    ttk::panedwindow $client_area.paned -orient horizontal
    set collections_viewer_object [ELEP::CollectionUtilities::CollectionsViewer\
      create [ELEP::Base::auto ELEP::CollectionUtilities::] \
      $client_area.paned vertical]
    set collections_operations_object \
      [ELEP::CollectionUtilities::CollectionSimpleOperations create \
      [ELEP::Base::auto ELEP::CollectionUtilities::] $client_area.paned]
    $collections_viewer_object registerListener $collections_operations_object
    $collections_operations_object registeredAsListener [self]
    $client_area.paned add [$collections_viewer_object getframe] -weight 1
    $client_area.paned add [$collections_operations_object getframe] -weight 1
    pack $client_area.paned -fill both -expand 1 -padx 2 -pady 2
    my registerAdditionalButtons
  };# createWidgets

  ##########################################################################
  ## Method: registerAdditionalButtons
  ##########################################################################
  method registerAdditionalButtons {} {
    my variable collections_operations_object
    set w [$collections_operations_object addWidget]
    $w configure -image IApplication -text {Annotate Selected Collection} \
      -compound left -command [list [self] onAnnotateSelectedCollection $w]
  };# registerAdditionalButtons

  ##########################################################################
  ## Event: onAnnotateSelectedCollection
  ##########################################################################
  method onAnnotateSelectedCollection {w} {
    my variable collections_viewer_object
    set col [$collections_viewer_object openSelectedCollection]
    if {![string length $col]} {return}
    set doc_name [$collections_viewer_object getSelectedDocumentName]

    $w configure -state disabled
    set doc {}
    set obj [my createAnnotatorObject $col $doc]
    $obj show
    if {[string length $doc_name]} {
      $obj loadDocument $doc_name
    }
    $obj wait
    tip_Close $col
    $w configure -state normal
    $collections_viewer_object update
  };# onAnnotateSelectedCollection

  ##########################################################################
  ## Event: onUpdateRequestFromRegisteredListener
  ##########################################################################
  method onUpdateRequestFromRegisteredListener {obj} {
    my variable collections_viewer_object
    $collections_viewer_object update
  };# onUpdateRequestFromRegisteredListener

  ##########################################################################
  ## Method: createAnnotatorObject
  ##########################################################################
  method createAnnotatorObject {col doc} {
    my variable annotator
    package require $annotator
    return [$annotator create [ELEP::Base::auto ::.creole_Annotator_] $col $doc]
  };# createAnnotatorObject

};# class ELEP::CollectionUtilities::CollectionSimpleManagerToplevel

package provide ELEP::CollectionUtilities::CollectionSimpleManagerToplevel 1.0
