##############################################################################
##  CollectionSimpleOperations.tcl:                                         ##
## --------------------------------------------------                       ##
##  This class implements a tool for editing Annotations on a text widget.  ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase
package require ELEP::WindowUtilities::ObjectListenerClient
package require ELEP::CollectionUtilities::CollectionCreatorDialog

GUI_LoadImage ITCreateCollection
GUI_LoadImage ITDeleteCollection
GUI_LoadImage ITEditCollection
GUI_LoadImage IAddDocuments
GUI_LoadImage IRemoveDocuments

oo::class create ELEP::CollectionUtilities::CollectionSimpleOperations {
  superclass ELEP::WindowUtilities::WidgetBase \
             ELEP::WindowUtilities::ObjectListenerClient
  variable padx pady col_name doc_name registed_button_index

  constructor {parent_win {orientation horizontal}} {
    set padx 4
    set pady 8

    set col_name {}
    set doc_name {}

    set registed_button_index 0
    next ttk::frame [::ELEP::WindowUtilities::UniqueWindowName \
                           $parent_win.collectionSimpleOperations]
    my initListenerObject
    my createWidgets
  };# constructor

  method createWidgets {} {
    my variable widget
    grid [ttk::button $widget.create  -text [my msg {Create New Collection}] \
      -image ITCreateCollection -compound left \
      -command [list [self] onCreateNewCollection] \
      ] -sticky snew -padx $padx -pady $pady
    grid [ttk::button $widget.rebuild -text \
      [my msg {Re-create Selected Collection}] \
      -image ITEditCollection -compound left \
      -command [list [self] onReCreateSelectedCollection] \
      ] -sticky snew -padx $padx -pady $pady
    grid [ttk::button $widget.delete  -text \
      [my msg {Delete Selected Collection}] \
      -image ITDeleteCollection -compound left \
      -command [list [self] onDeleteSelectedCollection] \
      ] -sticky snew -padx $padx -pady $pady
    grid [ttk::separator $widget.separator1 -orient horizontal] \
      -sticky snew -padx $padx -pady $pady
    grid [ttk::button $widget.adddocs -text \
      [my msg {Add Documents to Selected Collection}] \
      -image IAddDocuments -compound left \
      -command [list [self] onAddDocumentsToSelectedCollection] \
      ] -sticky snew -padx $padx -pady $pady
    grid [ttk::button $widget.deldocs -text \
      [my msg {Delete Documents from Selected Collection}] \
      -image IRemoveDocuments -compound left \
      -command [list [self] onDeleteDocumentsFromSelectedCollection] \
      ] -sticky snew -padx $padx -pady $pady
    grid [ttk::separator $widget.separator2 -orient horizontal] \
      -sticky snew -padx $padx -pady $pady
    # grid [ttk::button $widget.rename  -text {Rename Selected Collection} \
    # -image ITCreateCollection -compound left \
    #   -command [list [self] onRenameSelectedCollection] \
    #   ] -sticky snew -padx $padx -pady $pady
    grid columnconfigure $widget 0  -weight 1
    grid rowconfigure    $widget 0 -uniform buttons
    grid rowconfigure    $widget 1 -uniform buttons
    grid rowconfigure    $widget 2 -uniform buttons
    grid rowconfigure    $widget 4 -uniform buttons
    grid rowconfigure    $widget 5 -uniform buttons
    grid rowconfigure    $widget 7 -uniform buttons
    grid rowconfigure    $widget 20 -weight 1
  };# createWidgets

  ##########################################################################
  ## Method: addWidget
  ##########################################################################
  method addWidget {{wt ttk::button}} {
    my variable widget registed_button_index
    set w [$wt $widget.b$registed_button_index]
    grid $w -sticky snew -padx $padx -pady $pady
    incr registed_button_index
    return $w
  };# addWidget

  ##########################################################################
  ## Event: onCreateNewCollection
  ##########################################################################
  method onCreateNewCollection {} {
    my variable widget
    set dlg_obj [ELEP::CollectionUtilities::CollectionCreatorDialog \
      create [ELEP::Base::auto ::$widget.createNewCollection_]]
    $dlg_obj title {Create New Collection...}
    $dlg_obj showWait
    my notifyListeners
  };# onCreateNewCollection

  ##########################################################################
  ## Event: onReCreateSelectedCollection
  ##########################################################################
  method onReCreateSelectedCollection {} {
    my variable widget
    if {![string length $col_name]} {return}
    if {![file exists $col_name]} {return}
    set dlg_obj [ELEP::CollectionUtilities::CollectionCreatorDialog \
      create [ELEP::Base::auto ::$widget.createNewCollection_]]
    $dlg_obj title {Re-Create Existing Collection...}
    if {[$dlg_obj initFromCollectionPath $col_name]} {
      $dlg_obj showWait
    } else {
      update
      $dlg_obj destroyWindow
    }
    my notifyListeners
  };# onReCreateSelectedCollection

  ##########################################################################
  ## Event: onDeleteSelectedCollection
  ##########################################################################
  method onDeleteSelectedCollection {} {
    my variable widget
    if {![string length $col_name]} {return}
    if {![file exists $col_name]} {return}

    if {[string length [GUI_IsCollectionOpened $col_name]]} {
      gut_warning [my msg CollectionIsOpen]
    }

    set ans [gut_question "[my msg RemoveCollection?]\
        ${col_name}[my msg ?]" Warning... 1 $widget]
    if {[string equal $ans no]} {return}
    tip_Destroy $col_name
    my notifyListeners
  };# onDeleteSelectedCollection

  ##########################################################################
  ## Event: onAddDocumentsToSelectedCollection
  ##########################################################################
  method onAddDocumentsToSelectedCollection {} {
    my variable widget
    if {![string length $col_name]} {return}
    if {![file exists $col_name]} {return}
    set dlg_obj [ELEP::CollectionUtilities::CollectionCreatorDialog \
      create [ELEP::Base::auto ::$widget.createNewCollection_]]
    $dlg_obj updateCollection true 
    $dlg_obj title {Add Documents to Existing Collection...}
    if {[$dlg_obj initFromCollectionPath $col_name]} {
      $dlg_obj showWait
    } else {
      update
      $dlg_obj destroyWindow
    }
    my notifyListeners
  };# onAddDocumentsToSelectedCollection

  ##########################################################################
  ## Event: onDeleteDocumentsFromSelectedCollection
  ##########################################################################
  method onDeleteDocumentsFromSelectedCollection {} {
    my notifyListeners
  };# onDeleteDocumentsFromSelectedCollection

  ##########################################################################
  ## Event: onRenameSelectedCollection
  ##########################################################################
  method onRenameSelectedCollection {} {
    my notifyListeners
  };# onRenameSelectedCollection

  ##########################################################################
  ## Event: onCollectionDocumentSelectionChange
  ##########################################################################
  method onCollectionDocumentSelectionChange {caller_obj} {
    set col_name [$caller_obj getSelectedCollectionName]
    set doc_name [$caller_obj getSelectedDocumentName]
  };# onCollectionDocumentSelectionChange

  method getSelectedCollectionName {} {return $col_name}
  method getSelectedDocumentName   {} {return $doc_name}

  ##########################################################################
  ## Method: openSelectedCollection
  ##########################################################################
  method openSelectedCollection {} {
    if {![string length $col_name]} {return {}}
    set col [tip_OpenCollection $col_name]
    ## Is the Collection's location in accordance with it's name?
    set _saved_col_name [tip_GetName $col]
    if {![string equal $_saved_col_name $col_name]} {
      GUI_FixCollectionName $col $_saved_col_name $col_name
    }
    return $col
  };# openSelectedCollection

};# class ELEP::CollectionUtilities::CollectionSimpleOperations

package provide ELEP::CollectionUtilities::CollectionSimpleOperations 1.0
