##############################################################################
##  ProcessingComponent.tcl:                                                ##
## --------------------------------------------------                       ##
##  This file implements a Processing Component. Processing Components are  ##
##  the basic Component type.                                               ## 
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
package require TclOO    ;# TclOO adds object-oriented facilities to Tcl.

oo::class create ELEP::ComponentBase::ProcessingComponent {

  public variable component_home_directory
  public variable option         ;# An array holding Component's options.
  public variable option_default ;# An array holding Component's default
                                  # option values.
  public variable option_valid   ;# An array holding Component's valid
                                  # option values.
  public variable option_type    ;# An array holding Component's type for
                                  # each option.
  variable option_names {}       ;# A list holding all option names, in their
                                  # definition order.

  constructor {} {
    onLoad
  };# constructor

  destructor {
    onUnload
  };# destructor

  ##########################################################################
  ## Method: initialiseProcessingSession
  ##########################################################################
  method initialiseProcessingSession {col_name_list doc_ids_per_col_list} {
  };# initialiseProcessingSession

  ##########################################################################
  ## Method: initialiseCollectionProcessingSession
  ##########################################################################
  method initialiseCollectionProcessingSession {col doc_ids_list} {
  };# initialiseCollectionProcessingSession

  ##########################################################################
  ## Method: processDocument
  ##########################################################################
  method processDocument {col doc} {
  };# processDocument

  ##########################################################################
  ## Method: unprocessDocument
  ##########################################################################
  method unprocessDocument {col doc} {
  };# unprocessDocument

  ##########################################################################
  ## Method: finaliseCollectionProcessingSession
  ##########################################################################
  method finaliseCollectionProcessingSession {col doc_ids_list} {
  };# finaliseCollectionProcessingSession

  ##########################################################################
  ## Method: finaliseProcessingSession
  ##########################################################################
  method finaliseProcessingSession {col_name_list doc_ids_per_col_list} {
  };# finaliseProcessingSession

  ##########################################################################
  ## Event: onLoad
  ##########################################################################
  ## This method is invoked when the Component is loaded (created). Usually,
  ## this is done through the Component constructor.
  method onLoad {} {
  };# onLoad

  ##########################################################################
  ## Event: onReload
  ##########################################################################
  method onReload {} {
  };# onReload

  ##########################################################################
  ## Event: onUnload
  ##########################################################################
  ## This method is called when the object is unloaded (destroyed), usually
  ## from the destructor.
  method onUnload {} {
  };# onUnload

  ##########################################################################
  ## Method: whatProvides
  ##########################################################################
  method whatProvides {} {
  };# whatProvides

  ##########################################################################
  ## Method: whatRequires
  ##########################################################################
  method whatRequires {} {
  };# whatRequires

  ##########################################################################
  ## Method: supportedLanguages
  ##########################################################################
  method supportedLanguages {} {
  };# supportedLanguages

  ##########################################################################
  ## Method: processingCategory
  ##########################################################################
  method processingCategory {} {
  };# processingCategory

  ##########################################################################
  ## Method: componentType
  ##########################################################################
  method componentType {} {
  };# componentType

  ##########################################################################
  ## Method: declareOption
  ##########################################################################
  method declareOption {op_name op_type op_def_val op_valid_vals op_cur_val} {
    set option($op_name)         $op_cur_val
    set option_type($op_name)    $op_type
    set option_default($op_name) $op_def_val
    set option_valid($op_name)   $op_valid_vals
    lappend option_names $op_name
  };# declareOption

  ##########################################################################
  ## Method: deleteOption
  ##########################################################################
  method deleteOption {op_name} {
    unset -nocomplain option($op_name) option_type($op_name) \
                      option_default($op_name) option_valid($op_name)
    set index [lsearch $option_names $op_name]
    if {$index != -1} {set option_names [lreplace $option_names $index $index]}
  };# deleteOption

  ##########################################################################
  ## Method: getOptionNames
  ##########################################################################
  method getOptionNames {} {
    return $option_names
  };# getOptionNames

  ##########################################################################
  ## Method: serialiseCurrentOptionValues
  ##########################################################################
  method serialiseCurrentOptionValues {} {
    return [array get option]
  };# serialiseCurrentOptionValues

  ##########################################################################
  ## Method: restoreOptionValues
  ##########################################################################
  method restoreOptionValues {data} {
    array set $data
  };# restoreOptionValues

};# class ELEP::ComponentBase::ProcessingComponent

package provide ELEP::ComponentBase::ProcessingComponent 1.0
