##############################################################################
##  NaiveTokenizer.tcl:                                                     ##
## --------------------------------------------------                       ##
##  This class implements a simple tokenizer, that splits words on          ##
##  whitespace...                                                           ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
package require TclOO    ;# TclOO adds object-oriented facilities to Tcl.

oo::class create ELEP::DocumentUtilities::NaiveTokenizer {

  constructor {} {
  };# constructor

  classmethod tokenize {text} {
    set tcl_symbols_re_pattern  {[;,.'\"?]}
    set tokens {}
    set length [string length $text]
    
    ## We have to find the first word...
    set start 0
    set end [tcl_endOfWord $text $start]
    if {[tcl_startOfNextWord $text $start] < $end} {
      set start [tcl_startOfNextWord $text $start]
    }
    set textStart $start
    set token [string trim [string range $text $start $end]]
    regsub -all -- $tcl_symbols_re_pattern $token {} token
    if {[string length $token]} {lappend tokens $token}
  
    ## And now iterate over all other words...
    while {$start >= 0} {
      set start [tcl_startOfNextWord $text $start]
      set end   [tcl_endOfWord $text $start]
      if {$start != -1 && $end != -1} {
        set token [string trim [string range $text $start $end]]
        regsub -all -- $tcl_symbols_re_pattern $token {} token
        if {[string length $token]} {lappend tokens $token}
      }
    }
    return $tokens
  };# tokenize

};# class ELEP::DocumentUtilities::NaiveTokenizer

package provide ELEP::DocumentUtilities::NaiveTokenizer 1.0
