##############################################################################
##  HTMLPageLinkCrawler.tcl:                                                ##
## --------------------------------------------------                       ##
##  This class implements methods for examining and downloading the links   ##
##  of an HTML page.                                                        ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::FileUtilities::HTMLPageDownloader

oo::class create ELEP::FileUtilities::HTMLPageLinkCrawler {
  public variable storageDirectory $::CDM_TempDir/HTMLPageLinkCrawler
  public variable resourceIsText              0
  public variable resourceIsHTML              0
  public variable resourceBaseURI            {}
  public variable resourceTextFullPathName   {}
  public variable resourceBinaryFullPathName {}
  public variable resourceURI                {}

  variable html_downloader   {}

  constructor {args} {
    eval configure $args
    init
  };# constructor

  destructor {
    foreach object {html_downloader} {
      if {[string length [set $object]]} {
        catch {itcl::delete object [set $object]}
      }
    }
  };# destructor

  method init {} {
    set html_downloader [ELEP::FileUtilities::HTMLPageDownloader \
        ELEP::FileUtilities::html_downloader_#auto \
          -storageDirectory $storageDirectory]
  };# init

  ##########################################################################
  ## Method: downloadURI
  ##########################################################################
  method downloadURI {uri} {
    $html_downloader downloadURI $uri
    foreach var {resourceIsText resourceIsHTML resourceBaseURI
                 resourceTextFullPathName resourceBinaryFullPathName
                 resourceURI} {
      set $var [$html_downloader cget -$var]
    }
  };# downloadURI

  ##########################################################################
  ## Method: getHTML
  ##########################################################################
  method getHTML {} {
    return [$html_downloader getHTML]
  };# getHTML

  ##########################################################################
  ## Method: getHTMLLinks
  ##########################################################################
  method getHTMLLinks {} {
    return [$html_downloader getHTMLLinks]
  };# getHTMLLinks

  method printLinks {} {
    foreach one [getHTMLLinks] {
      puts $one; update
    }
  };# printLinks

  method test {} {
    foreach url {http://www.ellogon.org} {
      puts ===============================================================
      puts " $url"
      puts ===============================================================
      downloadURI $url
      printLinks
    }
  };# test

};# class ELEP::FileUtilities::HTMLPageLinkCrawler

package provide ELEP::FileUtilities::HTMLPageLinkCrawler 1.0

if {1} {
  package require ELEP::FileUtilities::HTMLPageLinkCrawler
  set c [ELEP::FileUtilities::HTMLPageLinkCrawler #auto]
  $c test
  itcl::delete object $c
}
