##############################################################################
##  GradientItem.tcl:                                                       ##
## --------------------------------------------------                       ##
##  This class implements an item that can be filled with a gradient.       ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::FlowControl::Item

oo::class create ELEP::FlowControl::GradientItem {
  superclass ELEP::FlowControl::Item

  constructor {c args} {
    my variable bgGradient
    set bgGradient {}
    next $c {*}$args
    my createGradients
  };# constructor

  ##########################################################################
  ## Method: updateStyle
  ##########################################################################
  method updateStyle {} {
    next
    [ELEP::Style::getStyle] initFlowControlGradientItem
  };# updateStyle

  ##########################################################################
  ## Method: createGradients
  ##########################################################################
  method createGradients {} {
    my variable bgGradient gradientOrientation gradientType \
                gradientColourA gradientColourB canvas
    if {!$::ELEP::FlowControl::FlowControl(tkpath)} {return}
    switch $gradientOrientation {
      diagonal-down {set lineartransition {0 0 1 1}}
      default       {set lineartransition {0 0 1 1}}
    }
    if {[string length $bgGradient]} {$canvas gradient delete $bgGradient}
    set bgGradient [$canvas gradient create $gradientType \
        -stops [list [list 0 $gradientColourA] [list 1 $gradientColourB]] \
        -lineartransition $lineartransition]
  };# createGradients

  ##########################################################################
  ## Method: onBackgroundColourChange
  ##########################################################################
  method onBackgroundColourChange {} {
    if {$::ELEP::FlowControl::FlowControl(tkpath)} {
      my variable canvas bgColour bgShapeId bgGradient outlineColour \
                  gradientColourA
      set gradientColourA $bgColour
      $canvas itemconfigure $bgShapeId -fill {}
      my createGradients
      $canvas itemconfigure $bgShapeId -fill $bgGradient -stroke $outlineColour
    } else {
      next
    }
  };# onBackgroundColourChange

  ##########################################################################
  ## Method: drawBackground
  ##########################################################################
  method drawBackground {} {
    next
    if {$::ELEP::FlowControl::FlowControl(tkpath)} {
      my variable canvas bgShapeId bgGradient
      $canvas itemconfigure $bgShapeId -fill $bgGradient
    }
  };# drawBackground

  ##########################################################################
  ## Method: drawSelected
  ##########################################################################
  method drawSelected {} {
    my variable selected
    set selected 1
    if {$::ELEP::FlowControl::FlowControl(tkpath)} {
      my variable canvas bgShapeId gradientColourA lockIndicator labelTextId \
                  labelActiveColour
      $canvas itemconfigure $bgShapeId -strokewidth 3 -fill $gradientColourA
      $canvas itemconfigure $lockIndicator -state normal
      $canvas itemconfigure $labelTextId -fill $labelActiveColour
    } else {
      next
    }
  };# drawSelected

  ##########################################################################
  ## Method: drawSelected
  ##########################################################################
  method drawDeselected {} {
    my variable selected
    set selected 0
    if {$::ELEP::FlowControl::FlowControl(tkpath)} {
      my variable canvas bgShapeId bgGradient lockIndicator \
                  labelTextId labelColour
      $canvas itemconfigure $bgShapeId -strokewidth 1 -fill $bgGradient
      $canvas itemconfigure $lockIndicator -state hidden
      $canvas itemconfigure $labelTextId -fill $labelColour
    } else {
      next
    }
  };# drawDeselected

  destructor {
    my variable canvas bgGradient
    catch {$canvas gradient delete $bgGradient}
    next
  };# destructor

};# class ELEP::FlowControl::GradientItem

package provide ELEP::FlowControl::GradientItem 1.0
