##############################################################################
##  GraphEdge.tcl:                                                          ##
## --------------------------------------------------                       ##
##  This class implements an item that can appear in a flow control object. ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::FlowControl::ConnectionItem

oo::class create ELEP::FlowControl::GraphEdge {
  superclass ELEP::FlowControl::ConnectionItem

  protected variable arrow_shape polygon

  constructor {c _source _sourceIndex _target _targetIndex args} {
    chain $c $_source $_sourceIndex $_target $_targetIndex
  } {
    set colour #808080
    set activeColour #606060
    set activeWidth $width
    if {$::ELEP::FlowControl::FlowControl(tkpath)} {set arrow_shape ppolygon}
    eval configure $args
  };# constructor
  
  ##########################################################################
  ## Method: arrowCoords
  ##########################################################################
  method arrowCoords {} {
    foreach {x1 y1} [$source getXY] {break}
    foreach {x2 y2} [$target getXY] {break}
    set coordsList [list [expr {$x1-2}] [expr {$y1+2}] $x2 $y2 \
                         [expr {$x1+2}] [expr {$y1-2}]]
  };# arrowCoords

  ##########################################################################
  ## Method: DrawArrow
  ##########################################################################
  method DrawArrow {} {
    arrowCoords
    switch $arrow_shape {
      ppolygon {
        set arrowId [$canvas create $arrow_shape $coordsList \
          -fill $colour -stroke $colour]
      }
      default {
        set arrowId [$canvas create $arrow_shape $coordsList -arrow $arrowMode \
          -smooth $smooth -width $width -activewidth $activeWidth \
          -fill $colour -activefill $activeColour -tags [list [self] graphedge]]
      }
    }
    $canvas lower $arrowId
  };# DrawArrow

  ##########################################################################
  ## Method: UpdateCoordinates
  ##########################################################################
  method UpdateCoordinates {} {
    if {[catch {arrowCoords}]} {
      ## There was an error in retrieving at least one node coordinates.
      ## Perhaps the node no longer exists. We should delete ourself...
      after idle "catch {::[self] destroy\}"
      return
    }
    $canvas coords $arrowId $coordsList
    if {[string length $labelTextId]} {
      set x1 [lindex $coordsList 0]
      set y1 [lindex $coordsList 1]
      set x2 [lindex $coordsList end-1]
      set y2 [lindex $coordsList end]
      set x [expr {$x1+($x2-$x1)/2.0}]
      set y [expr {$y1+($y2-$y1)/2.0}]
      catch {
        $canvas coords $labelTextId $x $y
        $canvas coords $labelBackgroundId [$canvas bbox $labelTextId]
      }
    }
  };# UpdateCoordinates

};# class ELEP::FlowControl::GraphEdge

package provide ELEP::FlowControl::GraphEdge 1.0
