##############################################################################
##  GraphLayoutDot.tcl:                                                     ##
## --------------------------------------------------                       ##
##  This class implements a graph layout algorithm based on the Tcldot      ##
##  package (from graphviz).                                                ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
package require TclOO    ;# TclOO adds object-oriented facilities to Tcl.

oo::class create ELEP::FlowControl::GraphLayoutDot {
  protected {
    variable m_graphManager {}            ;# The the graph manager, holding the                                            # graph being manipulated by
                                           # this strategy.
    variable m_dot_graph_object {}
  }

  constructor {GraphManager} {
    package require Tcldot
    set m_graphManager $GraphManager
    graphContentsChanged
  };# constructor

  ##########################################################################
  ## Method: getGraph
  ##########################################################################
  method getGraphManager {} {
    return $m_graphManager
  };# getGraphManager

  ##########################################################################
  ## Method: graphContentsChanged
  ##########################################################################
  ## Called when the graph has been completely changed.
  method graphContentsChanged {} {
    set inch2pixels [winfo fpixels . 1.0i]
    catch {$m_dot_graph_object delete}
    set m_dot_graph_object [dotnew digraph rankdir TB margin 1 center 1]
    foreach node [$m_graphManager getNodes] {
      set w [expr {[$node getW]/$inch2pixels}]
      set h [expr {[$node getH]/$inch2pixels}]
      set N($node) [$m_dot_graph_object addnode label [$node cget -label] \
          obj $node width $w height $h fixedsize true pos 100,100] \
    }
    foreach edge [$m_graphManager getEdges] {
      set from [$edge getFrom]
      set to   [$edge getTo]
      set E($edge) [$m_dot_graph_object addedge $N($to) $N($from) obj $edge]
    }
  };# graphContentsChanged

  ##########################################################################
  ## Method: notifyGraphLayoutUpdated
  ##########################################################################
  ## Notifies the strategy that the graph has changed due to an event that
  ## is not caused by the strategy itself.
  method notifyGraphLayoutUpdated {} {
  };# notifyGraphLayoutUpdated

  ##########################################################################
  ## Method: moveNodes
  ##########################################################################
  ## Moves nodes for the computed delta.
  method moveNodes {} {
    foreach node_handle [$m_dot_graph_object listnodes] {
      array set attrs [eval $node_handle queryattributevalues \
                           pos obj]
      foreach {x y} [split $attrs(pos) ,] {break}
      $attrs(obj) Place $x $y 1
    }
  };# moveNodes

  ##########################################################################
  ## Method: onItemMove
  ##########################################################################
  method onItemMove {} {
  };# onItemMove

  ##########################################################################
  ## Method: runLayout
  ##########################################################################
  method runLayout {} {
    $m_dot_graph_object layout
    set file [open $::CDM_TempDir/dot.txt w]
    catch {$m_dot_graph_object write $file}
    close $file
    moveNodes
  };# runLayout

  destructor {
    catch {$m_dot_graph_object delete}
  };# destructor

};# class ELEP::FlowControl::GraphLayoutDot

package provide ELEP::FlowControl::GraphLayoutDot 1.0
