##############################################################################
##  GraphManager.tcl:                                                       ##
## --------------------------------------------------                       ##
##  This class implements an object that manages a graph, by keeping its    ##
##  nodes and edges.                                                        ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::FlowControl::GraphManager {
  protected {
    variable l_nodes;# The list of nodes in the graph.
    variable l_edges;# The list of edges in the graph.

    variable m_dx;
    variable m_dy
    variable m_nodeMovement

    variable m_selectedNodes {}
  }

  constructor {{nodes {}} {edges {}}} {
    set l_nodes $nodes
    set l_edges $edges
  };# constructor

  ##########################################################################
  ## Method: addNode
  ##########################################################################
  method addNode {node} {
    lappend l_nodes [$node name]
    $node locked 1
  };# addNode
  
  ##########################################################################
  ## Method: removeNode
  ##########################################################################
  method removeNode {node} {
    set index [lsearch -exact $l_nodes [$node name]]
    if {$index != -1} {
      set l_nodes [lreplace $l_nodes $index $index]
    }
  };# removeNode

  ##########################################################################
  ## Method: addEdge
  ##########################################################################
  method addEdge {edge} {
    lappend l_edges [$edge name]
  };# addEdge
  
  ##########################################################################
  ## Method: removeEdge
  ##########################################################################
  method removeEdge {edge} {
    set index [lsearch -exact $l_edges [$edge name]]
    if {$index != -1} {
      set l_edges [lreplace $l_edges $index $index]
    }
  };# removeEdge

  ##########################################################################
  ## Method: clear
  ##########################################################################
  method clear {} {
    set l_nodes {}
  };# clear

  ##########################################################################
  ## Method: mergeWith
  ##########################################################################
  method mergeWith {GraphManager} {
    set l_nodes [concat $l_nodes [$GraphManager getNodes]]
    set l_edges [concat $l_edges [$GraphManager getEdges]]
  };# mergeWith

  ##########################################################################
  ## Method: getNodes
  ##########################################################################
  method getNodes {} {
    return $l_nodes
  };# getNodes

  ##########################################################################
  ## Method: getVisibleNodes
  ##########################################################################
  method getVisibleNodes {} {
    return $l_nodes
  };# getVisibleNodes
  
  ##########################################################################
  ## Method: getEdges
  ##########################################################################
  method getEdges {} {
    return $l_edges
  };# getEdges

  ##########################################################################
  ## Method: getVisibleEdges
  ##########################################################################
  method getVisibleEdges {} {
    return $l_edges
  };# getVisibleEdges

  ##########################################################################
  ## Method: getSubGraph
  ##########################################################################
  method getSubGraph {node} {
    set p_edges [$node getDirectParentConnectors]
    set c_edges [$node getDirectChildrenConnectors]
    set nodes {}
    foreach edge $p_edges {
      lappend nodes [$edge Source]
    }
    foreach edge $c_edges {
      lappend nodes [$edge Target]
    }
    return [ELEP::FlowControl::GraphManager ELEP::FlowControl::sg#auto \
                  $nodes [concat $p_edges $c_edges]]
  };# getSubGraph


  ##########################################################################
  ## Method: Draw
  ##########################################################################
  method Draw {} {
    foreach obj $l_nodes {$obj Draw}
    foreach obj $l_edges {$obj Draw}
    foreach obj $m_selectedNodes {$obj DrawSelected}
  };# Draw
  
  ##########################################################################
  ## Method: select
  ##########################################################################
  method select {nodes} {
    foreach obj $m_selectedNodes {$obj DrawDeselected}
    set m_selectedNodes $nodes
    foreach obj $m_selectedNodes {$obj DrawSelected}
  };# select

  if {0} {
  ##########################################################################
  ## Method: 
  ##########################################################################
  method  {} {
  };#

  ##########################################################################
  ## Method: 
  ##########################################################################
  method  {} {
  };#
  }

};# class ELEP::FlowControl::GraphManager

package provide ELEP::FlowControl::GraphManager 1.0
