##############################################################################
##  Alignment_TradosWinAlign.tcl:                                           ##
## --------------------------------------------------                       ##
##  This class provides facilities for importing alignment data from the    ##
##  Trados WinAlignment application.                                        ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
package require TclOO    ;# TclOO adds object-oriented facilities to Tcl.


oo::class create ELEP::ImportHandlers::Alignment_TradosWinAlign {

  constructor {} {
  };# constructor

  ##########################################################################
  ## Procedure: run
  ##########################################################################
  proc run {col doc args} {
    set Text [string trim [tip_GetRawData $doc]]
    foreach line [split $Text \n] {
      set line [string trim $line]
      switch -glob -nocase -- $line {
        <TrU>* {
          # Initialise our structures...
          array set attr {Quality 0 CrU {} CrD {}}
          set segments  {}
          set languages {}
        }
        <Quality>* {set attr(Quality) [string trim [string range $line 9 end]]}
        <CrU>*     {set attr(CrU)     [string trim [string range $line 5 end]]}
        <CrD>*     {set attr(CrD)     [string trim [string range $line 5 end]]}
        <Seg*      {
          regexp -nocase -- {L=([^>]+)} $line _ lang
          set index [string first > $line]
          if {$index != -1} {
            lappend segments [string trim [string range $line [incr index] end]]
            lappend languages [string trim $lang]
          }
        }
        </TrU>* {
          set spans {}
          set attrs {}
          # Find the maximum segment length...
          set len 0
          foreach s $segments {
            set l [string length $s]
            if {$l > $len} {set len $l}
          }
          foreach s $segments l $languages {
            if {![info exists text($l)]} {set text($l) {}}
            set     ln        [string length $s]
            set     start     [string length $text($l)]
            append  text($l) "[format %-${len}s $s]\n"
            lappend spans     [list $start [expr {$start+$ln}]]
          }
          # Create the annotation...
          foreach key [lsort [array names attr]] {
            lappend attrs [tip_CreateAttribute $key \
              [tip_CreateAttributeValue CDM_STRING $attr($key)]]
          }
          lappend attrs [tip_CreateAttribute languages \
              [tip_CreateAttributeValue CDM_STRING_SET $languages]]
          tip_AddAnnotation $doc [tip_CreateAnnotation alignment $spans $attrs]
        }
      };# switch -nocase $line
    };# foreach line
    set Text {}
    foreach l [lsort [array names text]] {
      set start [string length $Text]
      set len   [string length $text($l)]
      set span  [list $start [expr {$start+$len}]]
      append Text "$text($l)\n"
      tip_AddAnnotation $doc [tip_CreateAnnotation alignment-info \
       [list $span] \
       [tip_CreateAttributeSet language \
              [tip_CreateAttributeValue CDM_STRING $l]]]
    }
    tip_SetRawData $doc [string trimright $Text]
  };# run

  destructor {
  };# destructor

};# class ELEP::ImportHandlers::Alignment_TradosWinAlign

package provide ELEP::ImportHandlers::Alignment_TradosWinAlign 1.0
CDM_RegisterCollectionHandler {Alignment: Trados WinAlign} \
    ELEP::ImportHandlers::Alignment_TradosWinAlign::run
