##############################################################################
##  CASAM.tcl:                                                              ##
## --------------------------------------------------                       ##
##  This class provides facilities for importing data for the CASAM demo    ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
package require TclOO    ;# TclOO adds object-oriented facilities to Tcl.


oo::class create ELEP::ImportHandlers::CASAM {

  constructor {} {
  };# constructor

  ##########################################################################
  ## Procedure: run
  ##########################################################################
  proc run {col doc args} {
    ## Get the original file name...
    set filename [file normalize [tip_GetExternalId $doc]]
    set filename_root [file rootname $filename]
    set annotation_filename $filename_root.csv
    ## Open the file...
    set fd [open $annotation_filename]
    fconfigure $fd -encoding utf-8
    ## Skip the first line
    gets $fd line
    while {[gets $fd line]} {
      foreach {id start end keyword confidence operator synonym} \
        [split [string trim $line] \t] {break}
      set attrs {}
      foreach one {keyword confidence operator synonym} {
        lappend attrs [tip_CreateAttribute $one \
          [tip_CreateAttributeValue CDM_STRING [set $one]]]
      }
      tip_AddAnnotation $doc [tip_CreateAnnotation term \
        [tip_CreateSpanSet $start $end] $attrs]
    }
    close $fd
  };# run

  destructor {
  };# destructor

};# class ELEP::ImportHandlers::CASAM

package provide ELEP::ImportHandlers::CASAM 1.0
CDM_RegisterCollectionHandler {CASAM: Demo} \
    ELEP::ImportHandlers::CASAM::run
