##############################################################################
##  SYNC3.tcl:                                                              ##
## --------------------------------------------------                       ##
##  This class provides facilities for importing data for the SYNC3 project ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

##
## Load some needed packages.
##
catch {package require tdom}

namespace eval ELEP::ImportHandlers::SYNC3 {

  ##########################################################################
  ## Procedure: run
  ##########################################################################
  proc run {col doc args} {
    set xml [tip_GetRawData $doc]
    set text {}

    set document [dom parse -simple $xml]
    set root     [$document documentElement]

    ## 14 July 2010: Add the publishing date if found:
    catch {
      foreach node [$root selectNodes published] {
        set milliseconds [$node text]
        set seconds      [expr {$milliseconds / 1000}]
        append text "Publishing Date: [clock format $seconds -gmt 1]\n\n"
      }
    }

    set cleanTextNodes [$root selectNodes cleanText]
    foreach node $cleanTextNodes {
      append text [string trim [$node text]]
    }

    $document delete
    tip_SetRawData $doc [string trim $text]
  };# run

};# namespace ELEP::ImportHandlers::SYNC3

package provide ELEP::ImportHandlers::SYNC3 1.0
CDM_RegisterCollectionHandler {SYNC3: Import Clean Text} \
    ELEP::ImportHandlers::SYNC3::run
