##############################################################################
##  ABoxInstancesViewer.tcl:                                                ##
## --------------------------------------------------                       ##
##  This class implements a viewer for displaying instances of an ABox.     ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase

oo::class create ELEP::Ontology::ABoxInstancesViewer {
  superclass ELEP::WindowUtilities::WidgetBase

  public {
    variable instances {}
    variable instances_simple_name {}
    variable instances_description {}
    variable selected_instances {}
    variable selected_instances_simple_name {}
    variable selected_instances_description {}
  }

  protected {
    variable window_obj     {};# The object that holds our toplevel window.
    variable tree_widget    {};# The tree widget.
    variable listbox_widget {};# The listbox widget.
  }

  constructor {win parent_win} {
    chain ttk::notebook [::ELEP::WindowUtilities::UniqueWindowName \
                           $parent_win.aboxInstanceViewer]
  } {
    set window_obj $win
    createListboxWidget
    #createTreeWidget
  };# constructor

  ##########################################################################
  ## Method: createListboxWidget
  ##########################################################################
  method createListboxWidget {} {
    ScrolledWindow $widget.listboxsw
    set listbox_widget $widget.listboxsw.listbox
    listbox $listbox_widget -listvariable [my varname instances_simple_name]
    $widget.listboxsw setwidget $listbox_widget
    $widget add $widget.listboxsw -text {Instances}
    bind $listbox_widget <<ListboxSelect>> [list [self] onListboxSelectionChange]
  };# createListboxWidget

  ##########################################################################
  ## Method: createTreeWidget
  ##########################################################################
  method createTreeWidget {} {
    ScrolledWindow $widget.treesw
    set tree_widget $widget.treesw.tree
    Tree $tree_widget
    $widget.treesw setwidget $tree_widget
    $widget add $widget.treesw -text {Concepts}
  };# createTreeWidget

  ##########################################################################
  ## Method: updateView
  ##########################################################################
  method updateView {reasoner} {
    #$tree_widget delete [$tree_widget nodes root]
    set instances {}
    set instances_simple_name {}
    set instances_description {}
    foreach instance [lsort -dictionary [$reasoner all_individuals]] {
      lappend instances $instance
      lappend instances_description [$reasoner instance_as_table $instance]
      lappend instances_simple_name \
        [join [lrange [split [string range $instance 1 end-1] \#] 1 end] \#]
    }
  };# updateView

  ##########################################################################
  ## Method: onListboxSelectionChange
  ##########################################################################
  method onListboxSelectionChange {} {
    set selected_instances {}
    set selected_instances_simple_name {}
    set selected_instances_description {}
    foreach index [$listbox_widget curselection] {
      lappend selected_instances [lindex $instances $index]
      lappend selected_instances_simple_name \
                                 [lindex $instances_simple_name $index]
      lappend selected_instances_description \
                                 [lindex $instances_description $index]
    }
    event generate $widget <<SelectionChanged>>
  };# onListboxSelectionChange

  destructor {
  };# destructor

};# class ELEP::Ontology::ABoxInstancesViewer

package provide ELEP::Ontology::ABoxInstancesViewer 1.0
