##############################################################################
##  OWLHierarchyViewer.tcl:                                                 ##
## --------------------------------------------------                       ##
##  This class implements a viewer for an OWL class hierarchy.              ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase
package require ELEP::Ontology::OWLParser

oo::class create ELEP::Ontology::OWLHierarchyViewer {
  superclass ELEP::WindowUtilities::WidgetBase

  variable window_obj          {};# The object that holds our toplevel window.
  variable tree_widget         {};# The tree widget.

  variable owl_parser_object   {}
  variable OWL_hierarchy       {};# A tree structure that holds the class
                                  # hierarchy of the ontology.

  constructor {win parent_win} {
    chain ScrolledWindow [::ELEP::WindowUtilities::UniqueWindowName \
                           $parent_win.owlHierarchyViewer]
  } {
    set window_obj $win
    createTreeWidget
  };# constructor

  ##########################################################################
  ## Method: createTreeWidget
  ##########################################################################
  method createTreeWidget {} {
    set tree_widget $widget.tree
    Tree $tree_widget
    $widget setwidget $tree_widget
  };# createTreeWidget

  ##########################################################################
  ## Method: initHierarchy
  ##########################################################################
  method initHierarchy {hierarchy} {
    $tree_widget delete [$tree_widget nodes root]
    set OWL_hierarchy $hierarchy
    $OWL_hierarchy walkproc root [list [self] initHierarchy_NodeCallback]
  };# initHierarchy

  ##########################################################################
  ## Method: initHierarchy_NodeCallback
  ##########################################################################
  method initHierarchy_NodeCallback {tree node action} {
    if {[string equal $node root]} {return}
    if {![string length $node]} {return}
    if {[catch {
    set parent [$tree parent $node]
    set children [lsort [concat [$tree_widget nodes $parent] $node]]
    set pos [lsearch -exact $children $node]
    $tree_widget insert $pos $parent $node -text $node
    } error]} {
      puts "node=$node"
      puts "$error\n$::errorInfo"
    }
  };# initHierarchy_NodeCallback

  destructor {
  };# destructor

};# class ELEP::Ontology::OWLHierarchyViewer

package provide ELEP::Ontology::OWLHierarchyViewer 1.0
