##############################################################################
##  OWLViewer.tcl:                                                          ##
## --------------------------------------------------                       ##
##  This class implements a tool for vieweing an OWL Ontology.              ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ToplevelWindow
package require ELEP::Ontology::OWLHierarchyViewer

oo::class create ELEP::Ontology::OWLViewer {
  superclass ELEP::WindowUtilities::ToplevelWindow

  variable owl_parser_object {}
  variable owl_class_hierarchy_viewer_object {}

  constructor {} {
    createWidgets
    createOWLParserObject
  };# constructor

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    ## Add a vertical pane.
    ttk::panedwindow $client_area.pane -orient horizontal
    ## On left, add a class hierarchy viewer (OWLHierarchyViewer object).
    $client_area.pane add [createClassHierarchyDisplayWidget $client_area.pane]
    ## On right, add a ?? object.
    #$client_area.pane add [ttk::frame $client_area.pane.x]
    ## Pack the pane widget.
    pack $client_area.pane -fill both -expand 1
    title {OWL Ontology Viewer...}
  };# createWidgets

  ##########################################################################
  ## Method: createClassHierarchyDisplayWidget
  ##########################################################################
  method createClassHierarchyDisplayWidget {parent} {
    set owl_class_hierarchy_viewer_object [ELEP::Ontology::OWLHierarchyViewer \
        ELEP::Ontology::#auto [self] $parent]
    return [$owl_class_hierarchy_viewer_object getframe]
  };# createClassHierarchyDisplayWidget

  ##########################################################################
  ## Method: createOWLParserObject
  ##########################################################################
  method createOWLParserObject {} {
    set owl_parser_object [ELEP::Ontology::OWLParser ELEP::Ontology::#auto]
  };# createOWLParserObject

  ##########################################################################
  ## Method: onLoadFile
  ##########################################################################
  method onLoadFile {filename} {
    $owl_parser_object parseLocalFile $filename
    $owl_class_hierarchy_viewer_object initHierarchy \
      [$owl_parser_object getHierarchyObject]
  };# onLoadFile

  destructor {
    if {[string length $owl_parser_object]} {
      itcl::delete object $owl_parser_object
    }
    next
  };# destructor

};# class ELEP::Ontology::OWLViewer

package provide ELEP::Ontology::OWLViewer 1.0
