##############################################################################
##  JAPEBNF.tcl:                                                            ##
## --------------------------------------------------                       ##
##  This class uses YETI to create a JAPE parser class from BNF.            ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ylex
package require yeti

oo::class create ELEP::PatternEngine::JAPEBNF {
  variable sg {}; # The lexical scanner generator object.
  variable pg {}; # The parser generator object.

  constructor {} {
    constractScannerGenerator
    constractParserGenerator
  };# constructor

  ##########################################################################
  ## Method: constractScannerGenerator
  ##########################################################################
  method constractScannerGenerator {} {
    set sg [yeti::ylex ::ELEP::PatternEngine::#auto \
                             -name ELEP::PatternEngine::JAPEScanner]
    $sg add {
      {\s+}            {return SPACE}
      Rule:            {return RULE}
      {\(}             {return OPEN}
      {\)}             {return CLOSE}
      {-->}            {return RULESEP}
      {\|}             {return BAR}
      {.+}             {return CODE}
    }
  };# constractScannerGenerator

  ##########################################################################
  ## Method: constractParserGenerator
  ##########################################################################
  method constractParserGenerator {} {
    set pg [yeti::yeti ::ELEP::PatternEngine::#auto \
                             -name ELEP::PatternEngine::JAPEParser]

    $pg add {
      start                    {SinglePhaseTransducer}              {hello}
      SinglePhaseTransducer    {Rule}                               -
      |                        {Macro}                              -
      Rule                     {LeftHandSide RULESEP RightHandSide} -
      Macro                    {}                                   -
      LeftHandSide             {ConstraintGroup}                    -
      RightHandSide            {CODE}                               -
      ConstraintGroup          {PatternElement}                     -
      |                        {PatternElement BAR PatternElement}  -
      |                        {PatternElement PatternElement}      -
      PatternElement           {BasicPatternElement}                -
      |                        {ComplexPatternElement}              -
    }
  };# constractParserGenerator

  ##########################################################################
  ## Method: generateCode
  ##########################################################################
  method generateCode {} {
    return "[generateScannerCode]\n[generateParserCode]"
  };# generateCode

  ##########################################################################
  ## Method: generateScannerCode
  ##########################################################################
  method generateScannerCode {} {
    return [$sg dump]
  };# generateScannerCode

  ##########################################################################
  ## Method: generateParserCode
  ##########################################################################
  method generateParserCode {} {
    return [$pg dump]
  };# generateParserCode

};# class ELEP::PatternEngine::JAPEBNF

package provide ELEP::PatternEngine::JAPEBNF 1.0

set bnf [ELEP::PatternEngine::JAPEBNF #auto]
set dir [file dirname [info script]]
set file [open JAPELexerParser.tcl w]
puts $file [$bnf generateCode]
close $file
exit
