oo::class create ELEP::PatternEngine::JAPEScanner {
    public variable case 1
    public variable verbose 0
    public variable verbout stderr
    private variable yyistcl83 -1

    public variable yydata ""
    public variable yyindex 0
    private variable yymatches

    public variable yystate "INITIAL"
    public variable yytext
    public variable yystart
    public variable yyend

    private common yyregexs
    private common yyigncase
    private common yystates

    array set yyregexs {
        1 {\s+}
        2 {Rule:}
        3 {\(}
        4 {\)}
        5 {-->}
        6 {\|}
        7 {.+}
    }

    array set yyigncase {
        1 0
        2 0
        3 0
        4 0
        5 0
        6 0
        7 0
    }

    array set yystates {
        __all__ {
            1
            2
            3
            4
            5
            6
            7
        }
    }

    constructor {args} {
        eval [self] configure $args
    }
    
    public method reset {} {
        if {$verbose} {
            puts $verbout "ELEP::PatternEngine::JAPEScanner: reset, entering state INITIAL"
        }
        
        set yyindex 0
        set yystate "INITIAL"
        catch {unset yymatches}
    }
    
    public method start {yynewstr} {
        reset
        set yydata $yynewstr
    }
    
    protected method yyerror {yyerrmsg} {
        puts $verbout "ELEP::PatternEngine::JAPEScanner: $yyerrmsg"
    }
    
    private method yyupdate {} {
        if {$yyistcl83 == -1} {
            if {[package vcompare [info tclversion] 8.3] >= 0} {
                set yyistcl83 1
            } else {
                set yyistcl83 0
            }
        }
        
        set yybestmatch -1
        set yybestbegin [string length $yydata]
        set yybestend   $yybestbegin
        
        if {$verbose >= 2} {
            set yycurdata [string range $yydata $yyindex [expr {$yyindex + 16}]]
            set yycurout [string map {\r \\r \n \\n} $yycurdata]
            if {$yyindex + 16 < [string length $yydata]} {
                append yycurout "..."
            }
            puts $verbout "ELEP::PatternEngine::JAPEScanner: looking for match at position $yyindex: `$yycurout'"
        }
        
        if {[info exists yystates($yystate)]} {
            set yyruleset [concat $yystates($yystate) $yystates(__all__)]
        } else {
            set yyruleset $yystates(__all__)
        }
        
        if {[llength $yyruleset] == 0} {
            if {$verbose} {
                puts $verbout "ELEP::PatternEngine::JAPEScanner: no active rules in state $yystate"
            }
        }


        foreach yyruleno $yyruleset {
            #
            # if the last match is in the past, rerun regexp
            #

            if {![info exists yymatches($yyruleno)] ||  [lindex [lindex $yymatches($yyruleno) 0] 0] < $yyindex} {
                if {$yyistcl83} {
                    if {!$case || $yyigncase($yyruleno)} {
                        set yyfound [regexp -nocase -start $yyindex  -inline -indices --  $yyregexs($yyruleno) $yydata]
                    } else {
                        set yyfound [regexp -start $yyindex  -inline -indices --  $yyregexs($yyruleno) $yydata]
                    }
                } else {
                    if {!$case || $yyigncase($yyruleno)} {
                        set yyres [regexp -nocase -start $yyindex  -indices -- $yyregexs($yyruleno)  $yydata yyfound]
                    } else {
                        set yyres [regexp -start $yyindex  -indices -- $yyregexs($yyruleno)  $yydata yyfound]
                    }
                    
                    if {!$yyres} {
                        set yyfound [list]
                    } else {
                        set yyfound [list $yyfound]
                    }
                }

                if {[llength $yyfound] > 0} {
                    set yymatches($yyruleno) $yyfound
                } else {
                    set yymatches($yyruleno) [list  [string length $yydata]  [string length $yydata]]
                }

                if {$verbose >= 3 && [llength $yymatches($yyruleno)] > 0} {
                    set yymatchbegin [lindex [lindex $yymatches($yyruleno) 0] 0]
                    set yymatchend [lindex [lindex $yymatches($yyruleno) 0] 1]
                    if {$yymatchbegin != [string length $yydata]} {
                        set yymatchdata [string range $yydata $yymatchbegin $yymatchend]
			set yymatchout [string map {\r "\\r" \n "\\n"} $yymatchdata]
			if {[string length $yymatchout] > 20} {
			    set yymatchout [string range $yymatchout 0 16]
			    append yymatchout "..."
			}
                        puts $verbout "ELEP::PatternEngine::JAPEScanner: next match for rule $yyruleno is $yymatchbegin-$yymatchend: `$yymatchout'"
                    } else {
                        puts $verbout "ELEP::PatternEngine::JAPEScanner: rule $yyruleno does not match anwhere in remaining text"
		    }
                }
            }

            #
            # see if the match is better than the one we already have
            #

            set yymatchbegin [lindex [lindex $yymatches($yyruleno) 0] 0]
            set yymatchend [lindex [lindex $yymatches($yyruleno) 0] 1]

            if {($yymatchbegin < $yybestbegin) ||  ($yymatchbegin == $yybestbegin && $yymatchend > $yybestend)} {
                set yybestmatch $yyruleno
                set yybestbegin $yymatchbegin
                set yybestend   $yymatchend

                if {$verbose >= 2} {
                    set yybestdata [string range $yydata $yybestbegin $yybestend]
		    set yymatchout [string map {\r "\\r" \n "\\n"} $yybestdata]
		    if {[string length $yymatchout] > 20} {
			set yymatchout [string range $yymatchout 0 16]
			append yymatchout "..."
		    }
                    puts $verbout "ELEP::PatternEngine::JAPEScanner: new best match rule $yyruleno, $yybestbegin-$yybestend: `$yymatchout'"
                }
            }
        }
                    
        return $yybestmatch
    }
    
    public method step {} {
        if {$yyindex >= [string length $yydata]} {
            if {$verbose} {
                puts $verbout "ELEP::PatternEngine::JAPEScanner: scanner at EOF"
            }
            return [list -1 1 ""]
        }
        
        set yyruleno [yyupdate]
        
        if {$yyruleno == -1} {
            if {$verbose} {
                puts $verbout "ELEP::PatternEngine::JAPEScanner: no further match until EOF"
            }
            set yyindex [string length $yydata]
            return [list -1 1 ""]
        }
        
        set yymatch $yymatches($yyruleno)
        set yyindices [lindex $yymatch 0]
        set yystart [lindex $yyindices 0]
        set yyend [lindex $yyindices 1]
        set yytext [string range $yydata $yystart $yyend]
        
        if {$verbose} {
            set yymatchout [string map {\r \\r \n \\n} $yytext]
            if {[string length $yymatchout] > 20} {
                set yymatchout [string range $yymatchout 0 16]
                append yymatchout "..."
            }
            puts $verbout "ELEP::PatternEngine::JAPEScanner: best match at $yyindex using rule $yyruleno, $yystart-$yyend: `$yymatchout'"
        }
        
        set yyindex [expr {$yyend + 1}]
        
        for {set yyi 0} {$yyi < [llength $yymatch]} {incr yyi} {
            set yysubidxs  [lindex $yymatch $yyi]
            set yysubbegin [lindex $yysubidxs 0]
            set yysubend   [lindex $yysubidxs 1]
            set $yyi [string range $yydata $yysubbegin $yysubend]
        }
        
        set yyoldstate $yystate
        set yyretcode [catch {
            switch -- $yyruleno {
                1 {
return SPACE
                }
                2 {
return RULE
                }
                3 {
return OPEN
                }
                4 {
return CLOSE
                }
                5 {
return RULESEP
                }
                6 {
return BAR
                }
                7 {
return CODE
                }
            }
        } yyretdata]
        
        if {$verbose} {
            if {![string match $yyoldstate $yystate]} {
                puts $verbout "ELEP::PatternEngine::JAPEScanner: leaving state $yyoldstate, entering $yystate"
            }
        }
        
        if {$yyretcode == 1} {
            global errorInfo
            set yyerrmsg "script for rule # $yyruleno failed: $yyretdata"
            yyerror $yyerrmsg
            append errorInfo "\n    while executing script for rule # $yyruleno"
            error $yyerrmsg $errorInfo
        }
        
        if {$yyretcode == 2} {
            return [list $yyruleno 1 $yyretdata]
        }
        
        return [list $yyruleno 0 ""]
    }
    
    public method next {} {
        while {42} {
            set yysd [step]
            if {[lindex $yysd 1] == 1} {
                return [lindex $yysd 2]
            }
        }
    }
    
    public method run {} {
        set yyresult [list]
        set yysd [next]
        while {$yysd != ""} {
            lappend yyresult $yysd
            set yysd [next]
        }
        return $yyresult
    }
}


oo::class create ELEP::PatternEngine::JAPEParser {
    public variable scanner ""
    public variable verbose 0
    public variable verbout stderr
    private variable yystate 1
    private variable yysstack {1}
    private variable yydstack {}
    private variable yyreadnext 1
    private variable yylookahead
    private variable yyfinished 0
    private common yytrans
    private common yyredux
    private common yyrules

    array set yytrans {
        1,ComplexPatternElement 2
        1,PatternElement 3
        1,Rule 8
        1,BasicPatternElement 7
        1,ConstraintGroup 9
        1,Macro 10
        1,SinglePhaseTransducer 11
        1,start 12
        1,LeftHandSide 13
        3,ComplexPatternElement 2
        3,PatternElement 4
        3,BAR 5
        3,BasicPatternElement 7
        5,ComplexPatternElement 2
        5,PatternElement 6
        5,BasicPatternElement 7
        13,RULESEP 14
        14,RightHandSide 15
        14,CODE 16
    }

    array set yyredux {
        1 6
        2 13
        3 9
        4 11
        6 10
        7 12
        8 3
        9 7
        10 4
        11 2
        12 0
        15 5
        16 8
    }

    array set yyrules {
        0 {__init__ {start}}
        2 {start {SinglePhaseTransducer}}
        3 {SinglePhaseTransducer {Rule}}
        4 {SinglePhaseTransducer {Macro}}
        5 {Rule {LeftHandSide RULESEP RightHandSide}}
        6 {Macro {}}
        7 {LeftHandSide {ConstraintGroup}}
        8 {RightHandSide {CODE}}
        9 {ConstraintGroup {PatternElement}}
        10 {ConstraintGroup {PatternElement BAR PatternElement}}
        11 {ConstraintGroup {PatternElement PatternElement}}
        12 {PatternElement {BasicPatternElement}}
        13 {PatternElement {ComplexPatternElement}}
    }

    constructor {args} {
        eval [self] configure $args
    }
    
    public method reset {} {
        if {$verbose} {
            puts $verbout "ELEP::PatternEngine::JAPEParser: reset, entering state 1"
        }
        set yystate 1
        set yysstack [list $yystate]
        set yydstack [list]
        set yyreadnext 1
        set yyfinished 0
    }
    
    protected method yyerror {yyerrmsg} {
        puts $verbout "ELEP::PatternEngine::JAPEParser: $yyerrmsg"
    }
    
    private method yyshift {yytoken yydata} {
        set yynewstate $yytrans($yystate,$yytoken)
        lappend yysstack $yynewstate
        lappend yydstack $yydata
        
        if {$verbose} {
            puts $verbout "ELEP::PatternEngine::JAPEParser: shifting token $yytoken, entering state $yynewstate"
        }
        
        set yystate $yynewstate
    }

    private method yyreduce {yyruleno} {
        set yylhs [lindex $yyrules($yyruleno) 0]
        set yyrhs [lindex $yyrules($yyruleno) 1]
        set yycount [llength $yyrhs]
        set yyssdepth [llength $yysstack]
        set yydsdepth [llength $yydstack]
        
        if {$verbose} {
            puts -nonewline $verbout "ELEP::PatternEngine::JAPEParser: reducing rule # $yyruleno: $yylhs -->"
            foreach yytoken $yyrhs {
                puts -nonewline $verbout " "
                puts -nonewline $verbout $yytoken
            }
            puts $verbout ""
        }

        for {set yyi 0} {$yyi < $yycount} {incr yyi} {
            set yyvarname [expr {$yyi + 1}]
            set yyvarval  [lindex $yydstack [expr {$yydsdepth-$yycount+$yyi}]]
            set $yyvarname $yyvarval
        }
        
        set yyretcode [catch {
            switch -- $yyruleno {
                2 {
hello
                }
            }
        } yyretdata]
        
        if {$yyretcode == 0} {
            if {$yycount} {
                set yyretdata $1
            } else {
                set yyretdata ""
            }
        } elseif {$yyretcode == 1} {
            global errorInfo
            set yyerrmsg "script for rule # $yyruleno ($yylhs --> $yyrhs) failed: $yyretdata"
            yyerror $yyerrmsg
            append errorInfo "\n    while executing script for rule # $yyruleno ($yylhs --> $yyrhs)"
            error $yyerrmsg $errorInfo
        }
        
        set yysstack [lrange $yysstack 0 [expr {$yyssdepth-$yycount-1}]]
        set yydstack [lrange $yydstack 0 [expr {$yydsdepth-$yycount-1}]]
        set yystate [lindex $yysstack end]
        
        if {$verbose} {
            puts $verbout "ELEP::PatternEngine::JAPEParser: entering state $yystate"
        }
        
        return $yyretdata
    }
    
    public method step {} {
        if {$yyfinished} {
           error "step beyond end of parse"
        }
        
        if {$yyreadnext} {
            set yylookahead [$scanner next]
            set yyreadnext 0
        }
        
        set yyterm [lindex $yylookahead 0]
        set yydata [lindex $yylookahead 1]
        
        if {[info exists yytrans($yystate,$yyterm)]} {
            yyshift $yyterm $yydata
            set yyreadnext 1
            return [list $yyterm $yydata]
        } elseif {[info exists yyredux($yystate)] || \
                [info exists yyredux($yystate,$yyterm)]} {
            if {[info exists yyredux($yystate)]} {
                set yyruleno $yyredux($yystate)
            } else {
                set yyruleno $yyredux($yystate,$yyterm)
            }
            
            set yyreduxlhs [lindex $yyrules($yyruleno) 0]
            set yyreduxdata [yyreduce $yyruleno]
            
            if {$yyreduxlhs == "__init__"} {
                set yyfinished 1
            } else {
                yyshift $yyreduxlhs $yyreduxdata
            }
            return [list $yyreduxlhs $yyreduxdata]
        } else {
            global errorInfo
            set yyerrmsg "parse error reading \"$yyterm\" in state $yystate"
            set errorInfo $yyerrmsg
            append errorInfo "\n    expecting one of:"
            foreach yylaname [array names yytrans $yystate,*] {
                append errorInfo "\n        " [lindex [split $yylaname ,] 1]
            }
            yyerror $yyerrmsg
            error $yyerrmsg $errorInfo
        }
        
        error "oops: unreachable code"
    }
    
    public method parse {} {
        if {$scanner == ""} {
            global errorInfo
            set yyerrmsg "cannot start parsing without a scanner"
            set errorInfo $yyerrmsg
            yyerror $yyerrmsg
            error $yyerrmsg $errorInfo
        }
        
        while {!$yyfinished} {
            set yydata [step]
        }
        
        if {[lindex $yylookahead 0] != ""} {
            global errorInfo
            set yyerrmsg "parser finished, but not at EOF (lookahead: [lindex $yylookahead 0])"
            set errorInfo $yyerrmsg
            yyerror $yyerrmsg
            error $yyerrmsg
        }
        
        return [lindex $yydata 1]
    }
}
