##############################################################################
##  Memory.tcl:                                                             ##
## --------------------------------------------------                       ##
##  This class implements a class for getting memory information of a       ##
##  process.                                                                ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::Process::Memory {
 
  constructor {args} {
  };# constructor

  ##########################################################################
  ## Proc: _get_os
  ##########################################################################
  proc _get_os {} {
    return $::tcl_platform(platform)
  };# _get_os

  ##########################################################################
  ## Proc: memory_bytes
  ##########################################################################
  proc memory_bytes {pid} {
    return [memory_bytes_[_get_os] $pid]
  };# memory_bytes

  ##########################################################################
  ## Proc: memory_bytes_windows
  ##########################################################################
  proc memory_bytes_windows {pid} {
    package require twapi
    set bytes [lindex [twapi::get_process_info $pid -workingset] 1]
    return $bytes
  };# memory_bytes_windows

  ##########################################################################
  ## Proc: memory_KB
  ##########################################################################
  proc memory_KB {pid} {
    return [expr {[memory_bytes $pid]/1024.0}]
  };# memory_KB

  ##########################################################################
  ## Proc: memory_MB
  ##########################################################################
  proc memory_MB {pid} {
    return [expr {[memory_KB $pid]/1024.0}]
  };# memory_MB

  ##########################################################################
  ## Proc: memory
  ##########################################################################
  proc memory {{pid {}} {fmt {%.2f MB}}} {
    if {![string length $pid]} {set pid [pid]}
    if {[catch {memory_MB $pid} mem]} {
      return "Unsupported operating system by ELEP::Process::Memory"
    } else {
      return [format $fmt $mem]
    }
  };# memory

  destructor {
  };# destructor

};# class ELEP::Process::Memory

package provide ELEP::Process::Memory 1.0
