##############################################################################
##  Style.tcl:                                                              ##
## --------------------------------------------------                       ##
##  This class implements an Ellogon Style.                                 ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::Style::Style {
  
  constructor {args} {
    my init
    my configure {*}$args
  };# constructor

  ##########################################################################
  ## Method: init
  ##########################################################################
  method init {} {
    my variable NamedGradients NamedTransitions
    my checkTkPath
    set NamedGradients(blue)     {{0 #bfd4ea} {1 #5a8fc7}}
    set NamedGradients(green)    {{0 #bbef89} {1 #7bdb1e}}
    set NamedGradients(green2)   {{0 #7ECD00} {1 #46A300}}
    set NamedGradients(orange)   {{0 #fee440} {1 #fda221}}
    set NamedGradients(red)      {{0 #f9a2a2} {1 #ef2929}}
    set NamedGradients(gray)     {{0 white}   {1 black}}
    set NamedGradients(default)  $NamedGradients(blue)

    set NamedTransitions(default) {0 0 1 1}
  };# init

  ##########################################################################
  ## Method: checkTkPath
  ##########################################################################
  method checkTkPath {} {
    my variable tkpath_available canvas_type
    if {[catch {package require tkpath 0.3.0}]} {
      set canvas_type canvas
      set tkpath_available 0
    } else {
      set canvas_type tkp::canvas
      set tkpath_available 1
    }
  };# checkTkPath

  ##########################################################################
  ## Method: gradient
  ##########################################################################
  method gradient {{name default} {stops {}}} {
    my variable NamedGradients
    if {[llength $stops]} {
      set NamedGradients($name) $stops
    }
    return $NamedGradients($name)
  };# gradient

  ##########################################################################
  ## Method: transition
  ##########################################################################
  method transition {{name default} {transition {}}} {
    my variable NamedTransitions
    if {[llength $transition]} {
      set NamedTransitions($name) $transition
    }
    return $NamedTransitions($name)
  };# transition

  ##########################################################################
  ## Method: initFlowControlComponentItem
  ##########################################################################
  method initFlowControlComponentItem {} {
    set my [uplevel namespace which my]
    $my variable backgroundColour foregroundColour activeForegroundColour
    array set backgroundColour {
      inactive  #ff6060
      enabled   orange
      active    #6dc8fe
      running   green
      completed #008000
      error     red
    }

    array set foregroundColour {
      inactive  #0F36AD
      enabled   #0F36AD
      active    black
      running   orange
      completed orange
      error     white
    }

    array set activeForegroundColour {
      inactive  #0F36AD
      enabled   #0F36AD
      active    white
      running   yellow
      completed yellow
      error     white
    }
  };# initFlowControlComponentItem

  ##########################################################################
  ## Method: initFlowControlGradientItem
  ##########################################################################
  method initFlowControlGradientItem {} {
    set my [uplevel namespace which my]
    $my variable gradientColourA gradientColourB gradientType \
                 gradientOrientation bgColour
    set gradientColourA     $bgColour
    set gradientColourB     white
    set gradientType        linear; # linear or radial
    set gradientOrientation diagonal-down
  };# initFlowControlGradientItem

  ##########################################################################
  ## Method: initFlowControlItem
  ##########################################################################
  method initFlowControlItem {} {
    set my [uplevel namespace which my]
    $my variable bgType bgRadius bgColour bgStipple outlineColour \
                 disabledColour outlineDisabledColour

    set bgType rounded-rectangle ;# Item's shape
    set bgRadius 6               ;# The radius pixels, in case of rounded
    set bgColour red             ;# Item's background colour
    set bgStipple {}             ;# Item's background stipple bitmap
    set outlineColour  black     ;# Item's outline colour
    set disabledColour #c0c0c0   ;# Item's colour when disabled
    set outlineDisabledColour \
                       black     ;# Item's outline colour when disabled

    $my variable inNodeRadius inNodeBgColour inNodeActiveBgColour \
                inNodeDisabledBgColour inNodeFillColour \
                inNodeActiveFillColour inNodeDisabledFillColour
    ## Input node characteristics
    set inNodeRadius              4    ;# Item's input  node's radius in pixels
    set inNodeBgColour            yellow
    set inNodeActiveBgColour      green
    set inNodeDisabledBgColour    #c0c0c0
    set inNodeFillColour          blue
    set inNodeActiveFillColour    blue
    set inNodeDisabledFillColour  blue
  
    $my variable outNodeRadius outNodeBgColour outNodeActiveBgColour \
                 outNodeDisabledBgColour outNodeFillColour \
                 outNodeActiveFillColour outNodeDisabledFillColour
    ## Output node characteristics
    set outNodeRadius             4    ;# Item's output node's radius in pixels
    set outNodeBgColour           yellow
    set outNodeActiveBgColour     green
    set outNodeDisabledBgColour   #c0c0c0
    set outNodeFillColour         blue
    set outNodeActiveFillColour   blue
    set outNodeDisabledFillColour blue

    $my variable label labelColour labelBackgroundColour labelActiveColour \
                 labelDisabledColour labelFont inputNodeNumber outputNodeNumber\
                 repulsion inputNodesSide outputNodesSide
    set label               {<no label>}
    set labelColour         black
    set labelBackgroundColour {}
    set labelActiveColour   #0F36AD
    set labelDisabledColour #b0c4de ;# lightsteelblue
    set labelFont           ModuleTextFont
    set inputNodeNumber     1 ;# Number of input nodes  (GUI Connectors)
    set outputNodeNumber    1 ;# Number of output nodes (GUI Connectors)
    set inputNodesSide   left ;# The side that input nodes are drawn
    set outputNodesSide  right;# The side that input nodes are drawn
    set repulsion 1.0         ;# The repulsion factor (specifies how much does
                               # this node repulses other nodes). (used by
                               # graph layout algorithms)

    $my variable image  activeimage  disabledimage  image_coords  image_anchor \
                 image2 activeimage2 disabledimage2 image2_coords image2_anchor
    ## Images: up to 2 images are supported...
    set image {}                ;# Image #1.
    set activeimage {}
    set disabledimage {}
    set image_coords {0.5 0.5}  ;# (x, y) relative to item (width, height)
    set image_anchor c          ;# Anchor of image #1
    set image2 {}               ;# Image #2
    set activeimage2 {}
    set disabledimage2 {}
    set image2_coords {0.5 0.5} ;# (x, y), relative to item width, height
    set image2_anchor c         ;# Anchor of image #1

    $my variable x y width height
    set x 80             ;# Item's horizontal coordinate on a canvas
    set y 50             ;# Item's vertical   coordinate on a canvas
    set width 120        ;# Item's width  in pixels
    set height 80        ;# Item's height in pixels

    $my variable x1 x2 y1 y2
    ## Drawing cache (automatically updated)
    set x1  20
    set x2 140
    set y1  10
    set y2  90
  };# initFlowControlItem

  destructor {
  };# destructor

};# class ELEP::Style::Style

proc ELEP::Style::getStyle {} {
  variable Style
  if {![info exists Style]} {
    set Style [ELEP::Style::Style create style]
  }
  return $Style
};# ELEP::Style::getStyle

proc ELEP::Style::setStyle {obj} {
  variable Style
  catch {$Style delete}
  set Style $obj
};# ELEP::Style::setStyle

##
## Create a global style object, kept in the variable:
## ELEP::Style::Style
##
ELEP::Style::getStyle

package provide ELEP::Style::Style 1.0
