##############################################################################
##  CollectionManager.tcl:                                                  ##
## --------------------------------------------------                       ##
##  This class implements a class for querying Collection information.      ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::System::CollectionManager {

  constructor {args} {
    my variable collectionPaths allowCollectionOpen \
                selectedCollection selectedDocument
    set selectedCollection  -1
    set selectedDocument     0
    set allowCollectionOpen  1
    set collectionPaths     $::CollectionPath
    my configure {*}$args
  };# constructor

  ##########################################################################
  ## Method: getCollectionNames
  ##########################################################################
  method getCollectionNames {} {
    my variable collections
    set collections [GUI_GetCollectionNames]
    return $collections
  };# getCollectionNames

  ##########################################################################
  ## Method: getCollectionNameTails
  ##########################################################################
  method getCollectionNameTails {} {
    my getCollectionNames
    my variable collections collectionTails
    set collectionTails {}
    foreach col $collections {
      lappend collectionTails [file tail $col]
    }
    return $collectionTails
  };# getCollectionNameTails

  ##########################################################################
  ## Method: getDocumentNames
  ##########################################################################
  method getDocumentNames {col} {
    my variable documents allowCollectionOpen
    if {$allowCollectionOpen} {
      set documents [list [::ELEP::WindowUtilities::msg All]]
    } else {
      set documents {}
    }
    set documents [concat $documents [gut_GetCollectionDocuments $col]]
    return $documents
  };# getDocumentNames

  ##########################################################################
  ## Method: collectionAtIndex
  ##########################################################################
  method collectionAtIndex {index} {
    my variable collections
    return [lindex $collections $index]
  };# collectionAtIndex

  ##########################################################################
  ## Method: documentAtIndex
  ##########################################################################
  method documentAtIndex {index} {
    my variable documents
    return [lindex $documents $index]
  };# documentAtIndex

  ##########################################################################
  ## Method: selectCollection
  ##########################################################################
  method selectCollection {index} {
    my variable selectedCollection documents
    set documents {}
    set selectedCollection $index
    my selectDocument 0
  };# selectCollection

  ##########################################################################
  ## Method: getSelectedCollectionIndex
  ##########################################################################
  method getSelectedCollectionIndex {} {
    my variable selectedCollection
    return $selectedCollection
  };# getSelectedCollection

  ##########################################################################
  ## Method: getSelectedCollection
  ##########################################################################
  method getSelectedCollection {} {
    my variable selectedCollection collections
    if {$selectedCollection < 0} {return {}}
    return [lindex $collections $selectedCollection]
  };# getSelectedCollection

  ##########################################################################
  ## Method: getDocumentNamesOfSelectedCollection
  ##########################################################################
  method getDocumentNamesOfSelectedCollection {} {
    set col [my getSelectedCollection]
    if {$col eq ""} {return {}}
    return [my getDocumentNames $col]
  };# getDocumentNamesOfSelectedCollection

  ##########################################################################
  ## Method: selectDocument
  ##########################################################################
  method selectDocument {index} {
    my variable selectedDocument
    set selectedDocument $index
  };# selectDocument

  ##########################################################################
  ## Method: getSelectedDocumentIndex
  ##########################################################################
  method getSelectedDocumentIndex {} {
    my variable selectedDocument
    return $selectedDocument
  };# getSelectedDocument

  ##########################################################################
  ## Method: getSelectedDocument
  ##########################################################################
  method getSelectedDocument {} {
    my variable selectedDocument documents
    if {$selectedDocument < 1} {return {}}
    return [lindex $documents $selectedDocument]
  };# getSelectedDocument

};# class ELEP::System::CollectionManager

package provide ELEP::System::CollectionManager 1.0
