##############################################################################
##  Component.tcl:                                                          ##
## --------------------------------------------------                       ##
##  This file implements a class that represents a component.               ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::System::Component {

  constructor {{comp {}} args} {
    my setComponent $comp {*}$args
  };# constructor

  ##########################################################################
  ## Method: setComponent
  ##########################################################################
  method setComponent {comp args} {
    my variable component options cmd_init cmd cmd_finish exists \
                gui_object
    set gui_object {}
    set component  $comp
    set options    $args

    set cmd        creole_$component
    set cmd_init   ${cmd}_Initialize
    set cmd_finish ${cmd}_Finish

    ## Ensure existence...
    foreach one {cmd_init cmd cmd_finish} {
      set exists($one) [llength [info commands [set $one]]]
    }
  };# setComponent

  ##########################################################################
  ## Method: setOptions
  ##########################################################################
  method setOptions {opts} {
    my variable options
    set options    $opts
  };# setOptions

  ##########################################################################
  ## Method: setGraphicalObject
  ##########################################################################
  method setGraphicalObject {obj} {
    my variable gui_object
    set gui_object $obj
  };# setGraphicalObject

  ##########################################################################
  ## Method: init
  ##########################################################################
  method init {col doc} {
    my variable exists cmd_init options
    if {!$exists(cmd_init)} {return m}
    if {[catch {$cmd_init $col $doc {*}$options} error err_options]} {
      return [list e $error $err_options]
    }
    return [list o $error]
  };# init

  ##########################################################################
  ## Method: run
  ##########################################################################
  method run {col doc {force 0}} {
    my variable component exists cmd options gui_object
    if {!$exists(cmd)} {return m}
    ## Ensure that the component has not run on the component...
    if {[my hasRun $doc]} {
      catch {$gui_object setState completed}
      if {!$force} {return r}
    }
    catch {$gui_object drawWithState running}
    if {[catch {$cmd $doc {*}$options} error err_options]} {
      catch {$gui_object setState error}
      return [list e $error $err_options]
    }
    catch {$gui_object setState completed}
    return [list o $error]
  };# run

  ##########################################################################
  ## Method: finish
  ##########################################################################
  method finish {col doc} {
    my variable exists cmd_finish options
    if {!$exists(cmd_finish)} {return m}
    if {[catch {$cmd_finish $col $doc {*}$options} error err_options]} {
      return [list e $error $err_options]
    }
    return [list o $error]
  };# finish

  ##########################################################################
  ## Method: reset
  ##########################################################################
  method reset {col doc} {
    my variable post_annotations post_attributes
    foreach one $post_annotations {
      tip_DeleteAnnotations $doc $one
    }
    foreach one $post_attributes {
      catch {tip_RemoveAttribute $doc $one}
    }
    return {o {}}
  };# reset

  ##########################################################################
  ## Method: call
  ##########################################################################
  method call {col doc} {
    set result [my init $col $doc]
    if {[lindex $result 0] eq "e"} {return $result}
    set result [my run $col $doc]
    ## Ensure finish is always called...
    catch {my finish $col $doc}
    return $result
  };# call

  ##########################################################################
  ## Method: hasRun
  ##########################################################################
  method hasRun {doc} {
    my variable component
    return [llength [tip_GetAttribute $doc $component]]
  };# hasRun

  ##########################################################################
  ## Method: component
  ##########################################################################
  method component {} {
    my variable component
    return $component
  };# component

};# class ELEP::System::Component

package provide ELEP::System::Component 1.0
