##############################################################################
##  Progress.tcl:                                                           ##
## --------------------------------------------------                       ##
##  This class implements a widget that displays progress during running a  ##
##  a system.                                                               ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ProgressWidget

oo::class create ELEP::System::Progress {
  superclass ELEP::WindowUtilities::ProgressWidget

  constructor {parent_win args} {
    my variable progress_count document_name component_name widget time_variable
    set progress_count 2
    set time_variable  time
    set document_name  [my msg Document]
    set component_name [my msg ModuleName]
    my configure {*}$args
    next $parent_win
    for {set i 1} {$i < $progress_count} {incr i} {
      my addGroupWidgets $i
    }
    $widget.client_area.pgr_1 configure -variable GUI_Progress(Percent:[self])
    my timerStart
  };# constructor

  ##########################################################################
  ## Method: addCustomGroupWidgets
  ##########################################################################
  method addCustomGroupWidgets {group} {
    my variable widget message_wrap_length
    switch $group {
      0 {set txt "[my msg ProcessingDocument]:"; set var document_name}
      1 {set txt "[my msg RunningModule]:";      set var component_name}
      default {set txt [my msg PleaseWait]}
    }
    $widget.client_area.lbl_$group configure -text $txt \
      -width [expr {[string length $txt]+1}]
    ttk::label $widget.client_area.lbld_$group -textvar [my varname $var] \
      -wraplength $message_wrap_length -justify left -anchor nw
  };# addCustomGroupWidgets

  ##########################################################################
  ## Method: packGroupWidgets
  ##########################################################################
  method packGroupWidgets {group} {
    my variable widget
    grid $widget.client_area.lbl_$group $widget.client_area.lbld_$group \
                                          -sticky snew -padx 2 -pady 2
    grid $widget.client_area.pgr_$group - -sticky snew -padx 2 -pady 2
    grid columnconfigure $widget.client_area 0 -weight 0
    grid columnconfigure $widget.client_area 1 -weight 1
  };# packGroupWidgets

  ##########################################################################
  ## Method: numberOfDocumentsToProcess
  ##########################################################################
  method numberOfDocumentsToProcess {count} {
    my variable widget current_value
    set current_value(0) 0
    $widget.client_area.pgr_0 configure -maximum $count
    update
  };# numberOfDocumentsToProcess

  ##########################################################################
  ## Method: processingDocument
  ##########################################################################
  method processingDocument {doc} {
    my variable document_name
    catch {set document_name [tip_GetId $doc]}
    set ::GUI_Progress(WindowID:$doc) [self]
    update
  };# processingDocument

  ##########################################################################
  ## Method: processingDocumentFinished
  ##########################################################################
  method processingDocumentFinished {doc} {
    my advance 0
    unset ::GUI_Progress(WindowID:$doc)
  };# processingDocumentFinished

  ##########################################################################
  ## Method: processingComponent
  ##########################################################################
  method processingComponent {name} {
    my variable component_name current_value
    set component_name   $name
    # set current_value(1) 0
    set ::GUI_Progress(Percent:[self]) 0
    update
  };# processingComponent

  ##########################################################################
  ## Method: processingComponentFinished
  ##########################################################################
  method processingComponentFinished {} {
    my variable current_value
    # set current_value(1) 100
    set ::GUI_Progress(Percent:[self]) 100
    my timerPoll
    update
  };# processingComponentFinished

  ##########################################################################
  ## Method: nextDocument
  ##########################################################################
  method nextDocument {} {
    my advance 0
  };# nextDocument

  ##########################################################################
  ## Method: timerStart
  ##########################################################################
  method timerStart {} {
    my variable time_variable start_time
    set $time_variable 00:00:00
    set start_time [clock seconds]
  };# timerStart

  ##########################################################################
  ## Method: timerPoll
  ##########################################################################
  method timerPoll {} {
    my variable time_variable start_time ellapsed_time
    set ellapsed_time [expr {[clock seconds] - $start_time}]
    set $time_variable [clock format $ellapsed_time -format %H:%M:%S -gmt 1]
  };# timerPoll

};# class ELEP::System::Progress

package provide ELEP::System::Progress 1.0
