##############################################################################
##  System.tcl:                                                             ##
## --------------------------------------------------                       ##
##  This class implements an Ellogon System.                                ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::RibbonToplevelWindow
package require ELEP::System::CollectionManager
package require ELEP::System::SystemDesign
package require ELEP::System::SystemDeploy
package require ELEP::System::SystemExpose
package require ELEP::System::SystemManager
package require ELEP::Style::Style

oo::class create ELEP::System::System {
  superclass ELEP::WindowUtilities::RibbonToplevelWindow

  constructor {args} {
    my variable win title RibbonLoaded AutoSave IgnoreChanges RaisedPage \
                LogErrors LogTimings RaisedPage SelectedComponent \
                collection document system_object system_manager style 
    set AutoSave           0
    set IgnoreChanges      0
    set LogTimings         0
    set LogErrors          0
    set RaisedPage         H
    set SelectedComponent {}
    set collection        {}
    set document          {}
    set style             [ELEP::Style::getStyle]

    set title [my msg System:]
    next $::HomeDir/CDM/Windows/x86_32/libCDMRibbons.dll SYSTEM_RIBBON
    my createObjects
    my restoreState
    my createWidgets
    set title "[my msg System:] [$system_manager cget -name]"
    my configure {*}$args
    my title $title
    if {$RibbonLoaded} {
      wm geometry $win 800x500
    }
  };# constructor

  ##########################################################################
  ## ==================================================================== ##
  ## Ribbon Control                                                       ##
  ## ==================================================================== ##
  ##########################################################################

  ##########################################################################
  ## Event: onRibbonUpdateProperty
  ##########################################################################
  method onRibbonUpdateProperty {cmd id property args} {
    catch {my checkRaisedPage $cmd}
    switch -glob $property {
      UI_PKEY_Enabled {
        switch -glob $cmd {
          *DataDocument {
            my variable collection_manager
            if {[$collection_manager getSelectedCollectionIndex] == -1} {
              return 0
            }
            return 1
          }
          *ComponentOptions         -
          *ComponentOptionsMenu     -
          *ComponentFolder          -
          *ComponentReset           -
          *ComponentReload          {
            my variable SelectedComponent
            if {$SelectedComponent eq ""} {return 0}
            return 1
          }
          *CollectionNotes          -
          *CollectionInfo           -
          *CloseCollection          -
          *DataSave                 -
          *DataSaveAs               -
          *ViewDocument             -
          *ViewDocumentText         -
          *ViewDocumentHTML         -
          *ViewAnnotations          -
          *ViewAnnotationsExplorer  -
          *ViewAnnotationsRaw       -
          *ViewAttributes           -
          *ViewAttributesCollection -
          *ViewAttributesDocument   -
          *ViewMoreViewers          {
            my variable collection
            if {$collection eq ""} {return 0}
            return 1
          }
        }
        return -code break
      }
      UI_PKEY_RepresentativeString {
        ## This property relates to combo boxes...
        switch -glob $cmd {
          *DataCollection -
          *DataDocument   {
            set width [expr {([my opt SystemExprToolbarDocumentWidth] +
                              [my opt SystemExprToolbarCollectionWidth] ) / 2}]
          }
          default      {set width 20}
        }
        return [string repeat X $width]
      }
      UI_PKEY_ItemsSource {
        ## This property relates to combo boxes...
        switch -glob $cmd {
          *DataCollection {
            my variable collection_manager
            return [list [$collection_manager getCollectionNameTails] \
                         [my ribbonId CHDataCollection_SmallImages_96__RESID] \
                         {}]
          }
          *DataDocument   {
            my variable collection_manager toolbar
            set docs [$collection_manager getDocumentNamesOfSelectedCollection]
            if {[llength $docs] < 50} {
              return [list $docs \
                           [my ribbonId CHDataDocument_SmallImages_96__RESID] \
                           {}]
            }
            return [list $docs {} {}]
          }
          *Tools* {
            set names {}
            set categories {}
            set category 0
            foreach cat [my getToolCategories $cmd] {
              foreach name [my getToolNamesForCategory $cat] {
                lappend names      $name
                lappend categories $category
              }
              incr category
            }
            return [list \
                    $names \
                    [my ribbonId GroupHomeComponent_SmallImages_144__RESID] \
                    $categories]
          }
        }
      }
      UI_PKEY_Categories {
        switch -glob $cmd {
          *Tools* {
            return [list [my getToolCategories $cmd] {}]
          }
          default {
            puts "$cmd (id: $id), property: $property, args: $args"
          }
        }
      }
      UI_PKEY_SelectedItem {
        ## This property relates to combo boxes...
        switch -glob $cmd {
          *DataCollection {
            my variable collection_manager
            return [$collection_manager getSelectedCollectionIndex]
          }
          *DataDocument   {
            my variable collection_manager
            return [$collection_manager getSelectedDocumentIndex]
          }
        }
      }
      UI_PKEY_BooleanValue {
        ## This property relates to checkboxes...
        set variable [my ribbonCommand2VariableBoolean $cmd]
        if {[string length $variable]} {
          catch {my variable $variable}; return [set $variable]
        }
      }
      *Label              {return "Label"}
      *LabelDescription   {return "LabelDescription"}
      *TooltipTitle       {return "TOOLTIP"}
      *TooltipDescription {return "This is a tooltip!"}
      default {
        puts "$cmd (id: $id), property: $property, args: $args"
      }
    }
    return -code break
  };# onRibbonUpdateProperty

  ##########################################################################
  ## Event: onRibbonExecute
  ##########################################################################
  method onRibbonExecute {cmd id args} {
    my variable toolbar collection_manager
    switch -glob $cmd {
      *AutoSave - *IgnoreChanges - *LogTimings - *ErrorLogging -
      *CompressMemory {
        set variable [my ribbonCommand2VariableBoolean $cmd]
        catch {my variable $variable}
        set $variable [$toolbar get_property UI_PKEY_BooleanValue $id]
      }
      *DataCollection {
        set index [$toolbar get_property UI_PKEY_SelectedItem $id]
        $collection_manager selectCollection $index
        set doc_id [my ribbonId CHDataDocument]
        if {$index < 0} {
          $toolbar set_property UI_PKEY_Enabled $doc_id 0
        } else {
          $toolbar set_property UI_PKEY_Enabled $doc_id 1
          $toolbar invalidate_property UI_PKEY_ItemsSource  $doc_id
          $toolbar invalidate_property UI_PKEY_SelectedItem $doc_id
        }
      }
      *DataDocument   {
        set index [$toolbar get_property UI_PKEY_SelectedItem $id]
        $collection_manager selectDocument $index
        puts "Selected Document: [$collection_manager getSelectedDocument]"
      }
      *DataOpenCollection {
        my open [$collection_manager getSelectedCollection]
        $collection_manager selectDocument 0
        set doc_id [my ribbonId CHDataDocument]
        $toolbar invalidate_property UI_PKEY_SelectedItem $doc_id
      }
      *DataOpenDocument - *DataOpen {
        my open [$collection_manager getSelectedCollection] \
                [$collection_manager getSelectedDocument]
      }
      *CloseCollection          {my close}
      *CloseSystem              {my variable win; destroy $win}
      *DataSave                 {my save}
      *DataSaveAs               {my saveAs}
      *DataNew                  {my new  Collection}
      *ViewDocument             {my view Document}
      *ViewDocumentText         {my view DocumentText}
      *ViewDocumentHTML         {my view DocumentHTML}
      *ViewAnnotations          {my view Annotations}
      *ViewAnnotationsExplorer  {my view AnnotationsExplorer}
      *ViewAnnotationsRaw       {my view AnnotationsRaw}
      *ViewAttributes           {my view Attributes}
      *ViewAttributesCollection {my view AttributesCollection}
      *ViewAttributesDocument   {my view AttributesDocument}
      *ViewMoreViewers          {my view MoreViewers}
      *ViewConsole              {my view Console}

      *ComponentOptions         -
      *ComponentOptionsMenu     {my component Options}
      *ComponentFolder          {my component Folder}
      *ComponentReset           {my component Reset}
      *ComponentReload          {my component Reload}

      *ExecutionOptions         {}
      *ExecutionLog             {my execution Logs}
      *ExecutionLogExecution    {my execution LogExecution}
      *ExecutionLogTime         {my execution LogTime}
      *ExecutionLogErrors       {my execution LogErrors}

      default {
        puts "Command: $cmd (id: $id) $args"
      }
    }
  };# onRibbonExecute

  ##########################################################################
  ## Method: checkRaisedPage
  ##########################################################################
  method checkRaisedPage {cmd} {
    my variable RaisedPage
    set page [string index $cmd 1]
    if {$page ne $RaisedPage} {
      my raisePage $page
    }
  };# checkRaisedPage

  ##########################################################################
  ## Method: raisePage
  ##########################################################################
  method raisePage {page} {
    switch $page {
      H - T - Deploy {set p Deploy}
      E -     Expose {set p Expose}
      D -     Design {set p Design}
      default {error "unkown page $page"}
    }
    my variable client_area system_widget RaisedPage
    foreach slave [grid slaves $client_area] {
      grid remove $slave
    }
    grid $system_widget($p) -row 0 -column 0 -sticky snew
    set RaisedPage $page
  };# raisePage

  ##########################################################################
  ## Method: ribbonCommand2VariableBoolean
  ##########################################################################
  method ribbonCommand2VariableBoolean {cmd} {
    switch -glob $cmd {
      *AutoSave       {return AutoSave}
      *IgnoreChanges  {return IgnoreChanges}
      *LogTimings     {return LogTimings}
      *ErrorLogging   {return LogErrors}
      *CompressMemory {return [my optVar EnableMemoryCompression]}
      default         {return {}}
    }
  };# ribbonCommand2VariableBoolean

  ##########################################################################
  ## Method: ribbonUpdateCommandState
  ##########################################################################
  method ribbonUpdateCommandState {category} {
    my variable toolbar collection SelectedComponent
    set state 1
    switch $category {
      document  {
        set commands {
          CHDataSave CHDataSaveAs
          CHViewDocument CHViewDocumentText CHViewDocumentHTML
          CHViewAttributes CHViewAttributesCollection CHViewAttributesDocument
          CHViewAnnotations CHViewAnnotationsExplorer CHViewAnnotationsRaw
          CHViewMoreViewers CFileCollectionNotes CFileCollectionInfo
          CFileCloseCollection
        }
        if {$collection eq ""} {set state 0}
      }
      component {
        set commands {
          CHComponentOptions CHComponentOptionsMenu
          CHComponentFolder  CHComponentReset CHComponentReload
        }
        if {$SelectedComponent eq ""} {set state 0}
      }
    }
    update idletasks
    foreach command $commands {
      catch {
        $toolbar set_property UI_PKEY_Enabled [my ribbonId $command] $state
      }
    }
    my ribbonModes modes_add 0
  };# ribbonUpdateCommandState

  ##########################################################################
  ## ==================================================================== ##
  ## Ribbon Control                                                       ##
  ## ==================================================================== ##
  ##########################################################################

  ##########################################################################
  ## Method: getToolCategories
  ##########################################################################
  method getToolCategories {button} {
    switch -glob $button {
      *Annotators        {set cats { {Ellogon 2.0 Annotation Tools...}
                                     {Annotation Tools...} } }
      *Editors           {set cats { {Attribute Tools...} 
                                     {Ellogon 2.0 Viewers...} } }
      *Collections       {set cats { {Ellogon 2.0 Collection Tools...}
                                     {Collection Tools...} } }
      *Transfer          {set cats { {Collection Tools...} } }
      *MachineLearning   {set cats { {Machine Learning...} } }
      *Import            {set cats {Import...} }
      *Export            {set cats {Export...} }
      *Other             {set cats { {} Debug Comparisons... Statistics...} }
    }
    return $cats
  };# getToolCategories

  ##########################################################################
  ## Method: getToolNamesForCategory
  ##########################################################################
  method getToolNamesForCategory {cat} {
    global GUI_ToolConfig
    set names {}
    foreach one [lsort -dictionary [array names GUI_ToolConfig ${cat}::*]] {
      set index [string first :: $one]
      set class [string range $one 0        $index-1]
      set tool  [string range $one $index+2 end]
      set label [PUL_ToolConfig $tool title $class]
      lappend names $label
    }
    return $names
  };# getToolNamesForCategory

  ##########################################################################
  ## Method: createObjects
  ##########################################################################
  method createObjects {} {
    my variable collection_manager system_manager
    set collection_manager [ELEP::System::CollectionManager new]
    set system_manager     [ELEP::System::SystemManager new]
  };# createObjects

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    my variable client_area
    ## One frame is created, for each page...
    my addPages
    my raisePage Deploy
    grid columnconfigure $client_area 0 -weight 1
    grid rowconfigure    $client_area 0 -weight 1
  };# createWidgets

  ##########################################################################
  ## Method: addPages
  ##########################################################################
  method addPages {} {
    my variable pages
    set pages {Deploy Expose Design}
    foreach page $pages {
      my addPage $page
    }
  };# addPages

  ##########################################################################
  ## Method: addPage
  ##########################################################################
  method addPage {page} {
    my variable client_area system_object system_widget system_manager style
    set system_object($page) \
       [ELEP::System::System$page new $client_area.p$page [self] \
         -manager $system_manager -style $style]
    set system_widget($page) [$system_object($page) getframe]
  };# addPage

  ##########################################################################
  ## Method: open
  ##########################################################################
  method open {col {doc {}}} {
    my close
    my variable collection document
    set collection [tip_OpenCollection $col]
    ## Is the Collection's Location in accordance with it's Name?
    if {![string equal [tip_GetName $collection] $col]} {
      GUI_FixCollectionName $collection [tip_GetName $collection] $col
    }
    if {$doc ne ""} {
      set document [tip_GetDocument $collection $doc]
    }

    my updateStatusFromDocument $collection $document
    my ribbonUpdateCommandState document
    my notifyObjects onDocumentChange $collection $document
    my saveState
  };# open

  ##########################################################################
  ## Method: close
  ##########################################################################
  method close {} {
    my save
    my variable collection document
    catch {tip_Close $document}
    catch {tip_Close $collection}
    set collection {}
    set document   {}
    my updateStatusFromDocument $collection $document
    my ribbonUpdateCommandState document
    my notifyObjects onDocumentChange $collection $document
    CDM_ClearUnusedSavedValues
  };# close

  ##########################################################################
  ## Method: save
  ##########################################################################
  method save {} {
    my variable collection win
    if {$collection eq ""}       {return}
    if {![$collection Modified]} {return}
    set result [gut_question [format "%s\n(%s)" \
           [my msg DoyouwanttosavecurrentCollection?] \
           [tip_GetName $collection]] [my msg {Save Collection ?}] \
           0 $win {yes no cancel}]
    if {[string equal $result yes]} {tip_Sync $collection}
  };# save

  ##########################################################################
  ## Method: saveAs
  ##########################################################################
  method saveAs {} {
    my save
    error todo
  };# saveAs

  ##########################################################################
  ## Method: new
  ##########################################################################
  method new {what} {
    switch $what {
      default {error "unsupported option: \"$what\""}
    }
  };# new

  ##########################################################################
  ## Method: view
  ##########################################################################
  method view {what} {
    switch -glob $what {
      *Collection*  {return [my viewCollection $what]}
      *Annotations* -
      *Attributes*  -
      *Document*    {return [my viewDocument   $what]}
      *Console*     {tkcon show}
      default {error "unsupported option: \"$what\""}
    }
  };# view

  ##########################################################################
  ## Method: viewCollection
  ##########################################################################
  method viewCollection {what} {
    my variable collection
    if {$collection eq ""} {return}
    set window [my childWindow viewCollection]
    set colMsg "of Collection [file tail [tip_GetName $collection]]"
    switch $what {
      AttributesCollection {
        ggi_view_Attributes $collection {} 1 "Attributes $colMsg" $window
        
      }
      default {error "unsupported option: \"$what\""}
    }
  };# viewCollection

  ##########################################################################
  ## Method: viewDocument
  ##########################################################################
  method viewDocument {what} {
    set doc [my getCurrentDocument]
    if {$doc eq ""} {return}

    my variable collection document win
    set window [my childWindow viewDocument]
    set colMsg "from Collection [file tail [tip_GetName $collection]]"
    set docMsg "of Document [$doc GetId], $colMsg"
    switch $what {
      Document -
      DocumentHTML -
      DocumentText {
        ggi_view_text_file $doc {} {} "Text $docMsg" $window
        # ggi_view_document $collection $doc $win
      }
      Annotations -
      AnnotationsExplorer {
        ggi_view_AnnotationExplorer $doc {} {} \
                                   "Explore Annotations $docMsg" $window
      }
      AnnotationsRaw {
        ggi_view_raw $doc {} {} "Annotations $docMsg" 0 $window
      }
      Attributes -
      AttributesDocument {
        ggi_view_Attributes $doc {} 1 "Attributes $docMsg" $window
      }
      MoreViewers {
       todo
      }
      default {error "unsupported option: \"$what\""}
    }
  };# viewDocument

  ##########################################################################
  ## Method: getCurrentDocument
  ##########################################################################
  method getCurrentDocument {} {
    my variable collection document win cannotViewWholeCollectionReported
    if {$collection eq ""} {return {}}
    ## If a document has been opened, return it...
    if {$document ne ""}   {return $document}
    ## Get the documents from the collection...
    set docIds [$collection GetFileIds]
    switch [llength $docIds] {
      0 {return {}}
      1 {return [tip_FirstDocument $collection]}
    }
    ## More than one document...
    if {![info exists cannotViewWholeCollectionReported]} {
      set cannotViewWholeCollectionReported 1
      gut_ErrorDlg [my msg CannotViewCollection] $win
    }
    set doc [gut_MultipleSelectFromList $docIds [my msg TSelectDocument] \
                       [my msg {Select Document to View...}]]
    if {$doc eq ""} {return {}}
    return [$collection GetDocument [lindex $doc 0]]
  };# getCurrentDocument

  ##########################################################################
  ## Method: component
  ##########################################################################
  method component {what} {
    my variable SelectedComponent
    if {$SelectedComponent eq ""} {return}
    switch $what {
      Options -
      Folder  -
      Reset   -
      Reload  {
        my notifyObject Deploy component $what $SelectedComponent
      }
      default {error "unsupported option: \"$what\""}
    }
  };# component

  ##########################################################################
  ## Method: execution
  ##########################################################################
  method execution {what} {
    switch $what {
      Logs         {my notifyObject Deploy widget logs      toggle}
      LogExecution {my notifyObject Deploy widget execution raise}
      LogTime      {my notifyObject Deploy widget time      raise}
      LogErrors    {my notifyObject Deploy widget error     raise}
      default      {error "unsupported option: \"$what\""}
    }
  };# execution

  ##########################################################################
  ## Method: initFromEllogonMemoryLegacy
  ##########################################################################
  method initFromEllogonMemoryLegacy {system} {
    my variable system_manager
    $system_manager initFromEllogonMemoryLegacy $system
    my notifyObjects reset
    my title "[my msg System:] [$system_manager cget -name]"
  };# initFromEllogonMemoryLegacy

  ##########################################################################
  ## Method: notifyObjects
  ##########################################################################
  method notifyObjects {args} {
    my variable system_object
    foreach page [array names system_object] {
      catch {$system_object($page) {*}$args}
    }
  };# notifyObjects

  ##########################################################################
  ## Method: notifyObject
  ##########################################################################
  method notifyObject {page args} {
    my variable system_object
    $system_object($page) {*}$args
  };# notifyObject

  ##########################################################################
  ## Method: childWindow
  ##########################################################################
  method childWindow {{prefix child}} {
    my variable win
    return [ELEP::WindowUtilities::UniqueWindowName $win.$prefix]
  };# childWindow

  ##########################################################################
  ## Event: onTabChange
  ##########################################################################
  method onTabChange {} {
    my variable notebook pages system_object exposed_system_object
    set page [lindex $pages [$notebook index current]]
    catch {$exposed_system_object onHide}
    set exposed_system_object $system_object($page)
    catch {$exposed_system_object onExpose}
  };# onTabChange

  ##########################################################################
  ## Event: onComponentSelect
  ##########################################################################
  method onComponentSelect {component} {
    my variable SelectedComponent
    set SelectedComponent $component
    my ribbonUpdateCommandState component
  };# onComponentSelect

  ##########################################################################
  ## Method: saveState
  ##########################################################################
  method saveState {} {
    my variable collection document
    if {$collection eq ""} {return}
    set col [tip_GetName $collection]
    set doc {}
    if {$document ne ""} {
      set doc [tip_GetId $document]
    }
    set ::GUI_Options([self class]) [list $col $doc]
  };# saveState

  ##########################################################################
  ## Method: restoreState
  ##########################################################################
  method restoreState {} {
    if {![info exists ::GUI_Options([self class])]} {
      return
    }
    foreach {col doc} $::GUI_Options([self class]) { break }
    my variable collection_manager
    set index [lsearch -exact [$collection_manager getCollectionNames] $col]
    if {$index == -1} {return}
    $collection_manager selectCollection $index
    set index [lsearch -exact [$collection_manager \
                                 getDocumentNamesOfSelectedCollection] $doc]
    if {$index == -1} {set index 0}
    $collection_manager selectDocument $index
  };# restoreState

  destructor {
    my variable collection_manager
    catch {my close}
    catch {$collection_manager destroy}
  };# destructor

};# class ELEP::System::System

package provide ELEP::System::System 1.0
