##############################################################################
##  ToolToplevel.tcl:                                                       ##
## --------------------------------------------------                       ##
##  This class implements an automatic encoding detection facility.         ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ToplevelWindow
package require ELEP::ViewerBase::DocumentSelector

oo::class create ELEP::ToolBase::ToolToplevel {
  superclass ELEP::WindowUtilities::ToplevelWindow

  variable {*}[info class variables ELEP::WindowUtilities::ToplevelWindow] \
           collection \
           document \
           selector_object \
           allow_collection_open

  constructor {{col {}} {doc {}} {allow_col_open 0}} {
    set collection            $col
    set document              $doc
    set allow_collection_open $allow_col_open
    next
  };# constructor

  ##########################################################################
  ## Method: fillToolBar
  ##########################################################################
  method fillToolBar {} {
    set selector_object [ELEP::ViewerBase::DocumentSelector \
        create [ELEP::Base::auto ELEP::ToolBase::ds] \
               [self] {} [my getCollection] [my getDocument] \
               $allow_collection_open]
    $selector_object registerListener [self]
  };# fillToolBar

  ##########################################################################
  ## Method: show
  ##########################################################################
  method show {} {
    update
    next
    $selector_object show
  };# show

  ##########################################################################
  ## Method: loadDocument
  ##########################################################################
  method loadDocument {doc_id} {
    $selector_object loadDocument $doc_id
  };# loadDocument

  ##########################################################################
  ## Method: onDocumentChange
  ##########################################################################
  method onDocumentChange {col doc} {
    set collection $col
    set document   $doc
    event generate $win <<onDocumentChange>>
  };# onDocumentChange

  method getCollection {} {return $collection}
  method getDocument   {} {return $document}

};# class ELEP::ToolBase::ToolToplevel

package provide ELEP::ToolBase::ToolToplevel 1.0
