##############################################################################
##  AnnotationAndTemplateSpecificationSelector.tcl:                         ##
## --------------------------------------------------                       ##
##  This class superclass an AnnotationSpecificationSelector and a          ##
##  TemplateSpecificationSelector.                                          ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::ViewerBase::AnnotationSpecificationSelector
package require ELEP::ViewerBase::TemplateSpecificationSelector

oo::class create ELEP::ViewerBase::AnnotationAndTemplateSpecificationSelector {
  # superclass ELEP::ViewerBase::AnnotationSpecificationSelector \
  #            ELEP::ViewerBase::TemplateSpecificationSelector

  variable dialog_window ann_selector templ_selector \
           language \
           annotation \
           attribute \
           alternative \
           groups \
           values \
           template \
           tables

  constructor {{init_dialog 1}} {
    # set language      {}
    # set annotation    {}
    # set attribute     {}
    # set alternative   {}
    # set groups        {}
    # set values        {}
    # set template      {}
    # set tables        {}
    if {$init_dialog} {my init}
  };# constructor

  method init {} {
    set ann_selector   [ELEP::ViewerBase::AnnotationSpecificationSelector \
                          create [::ELEP::Base::auto ::ELEP::ViewerBase::] 0]
    set templ_selector [ELEP::ViewerBase::TemplateSpecificationSelector \
                          create [::ELEP::Base::auto ::ELEP::ViewerBase::] 0]
    oo::objdefine $ann_selector export varname
    foreach var {language annotation attribute alternative groups values} {
      upvar [$ann_selector varname $var] [my varname $var]
    }
    oo::objdefine $templ_selector export varname
    foreach var {template tables} {
      upvar [$templ_selector varname $var] [my varname $var]
    }

    my createReader
    my createDialog
    my populateDialogFrame [$dialog_window getframe]
    my restoreState        [$dialog_window getframe]
  };# init

  method createReader {} {
    $ann_selector   createReader
    $templ_selector createReader
  };# createReader

  method createDialog {} {
    set dialog_window [Dialog [::ELEP::WindowUtilities::UniqueWindowName \
     .annotatorSpecificationSelector] -modal local -separator 1 \
     -title "Select Annotation & Template Characteristics..." \
     -default 0 -cancel 1]
    $dialog_window add -textvariable GUI_Text(Ok)
    $dialog_window add -textvariable GUI_Text(Cancel)
  };# createDialog

  method populateDialogFrame {frame} {
    ttk::labelframe $frame.annotation -text {Annotation Details:}
    $ann_selector   populateDialogFrame $frame.annotation
    # trace add variable [$ann_selector varname language] write \
    #    [list [self] onLanguageChange $templ_selector $frame.template \
    #          [$ann_selector varname language]]

    ttk::labelframe $frame.template -text {Template Details:}
    $templ_selector populateDialogFrame $frame.template
    # trace add variable [$templ_selector varname language] write \
    #    [list [self] onLanguageChange $ann_selector $frame.annotation \
    #          [$templ_selector varname language]]

    grid $frame.annotation $frame.template -sticky snew -padx 2
    grid rowconfigure    $frame 0 -weight 1
    grid columnconfigure $frame 0 -weight 1
    grid columnconfigure $frame 1 -weight 1
    # Arrange value listbox widths...
    $frame.annotation.lf.sc.values configure -width 12
    $frame.template.lf.sc.values   configure -width 15
  };# populateDialogFrame

  # method onLanguageChange {obj frame name name1 name2 op} {
  #   catch {console show}
  #   puts "name1: $name $name1"
  #   set language [set $name]
  #   puts "onLanguageChange: language: $language"
  #   $obj configure -language $language
  #   $obj onLanguageChange $frame
  # };# onLanguageChange

  method show {} {
    if {[$dialog_window draw] == 0} {
      my saveState
      return ok
    } else {
      return cancel
    }
  };# show

  destructor {
    catch {$ann_selector   destroy}
    catch {$templ_selector destroy}
  };# destructor

  method saveStateVariableName {} {return ::GUI_Options([self class])}

  method saveState {} {
    $ann_selector   saveState
    $templ_selector saveState
  };# saveState

  method restoreState {frame} {
    $ann_selector   restoreState $frame.annotation
    $templ_selector restoreState $frame.template
  };# restoreState

  method unknown {args} {
    ## Try to call the aggragated objects...
    if {![catch {$ann_selector {*}$args} result]} {
      return $result
    }
    if {![catch {$templ_selector {*}$args} result]} {
      return $result
    }
    # catch {console show}
    # puts "unknown $args"
    next unknown {*}$args
  };# unknown

};# class ELEP::ViewerBase::AnnotationAndTemplateSpecificationSelector

package provide ELEP::ViewerBase::AnnotationAndTemplateSpecificationSelector 1.0
