##############################################################################
##  AttributesWidget.tcl:                                                   ##
## --------------------------------------------------                       ##
##  This class implements a widget for displaying Attributes.               ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase
package require widget::scrolledwindow
package require tablelist_tile

GUI_LoadImage IFileBroken

oo::class create ELEP::ViewerBase::AttributesWidget {
  superclass ELEP::WindowUtilities::WidgetBase

  constructor {parent_win} {
    next ScrolledWindow [::ELEP::WindowUtilities::UniqueWindowName \
                             $parent_win.viewer]
    my variable images max_width max_height
    set images       {}
    set max_width   120
    set max_height  120

    my createWidgets
  };# constructor

  ##########################################################################
  ## Method: createWidgets
  ##########################################################################
  method createWidgets {} {
    my variable widget tablelist
    set tablelist $widget.tbl
      tablelist::tablelist $tablelist -height 15 -width 100 \
        -columns {20 Name 30 Value 15 Type} -stretch {1}
      $tablelist columnconfigure 0 -sortmode dictionary
      $tablelist columnconfigure 1 -sortmode dictionary
      $tablelist columnconfigure 2 -sortmode dictionary
    $widget setwidget $tablelist
  };# createWidgets

  ##########################################################################
  ## Method: clear
  ##########################################################################
  method clear {} {
    my variable tablelist images
    $tablelist delete 0 end
    foreach image $images {
      catch {image delete $image}
    }
    set images {}
  };# clear

  ##########################################################################
  ## Method: getImage
  ##########################################################################
  method getImage {data} {
    my variable images max_width max_height
    ## Create the image...
    binary scan $data {}
    if {[catch {image create photo -data $data} image]} {
      set image IFileBroken
    } else {
      ## Maybe we need to resize the image?
      if {[image width  $image] > $max_width || 
          [image height $image] > $max_height} { 
        set shrinked [GUI_ShrinkImage $image $max_width $max_height]
        image delete $image
        set image $shrinked
      }
      lappend images $image
    }
    return $image
  };# getImage

  ##########################################################################
  ## Event: onDocumentChange
  ##########################################################################
  method onDocumentChange {col doc} {
    if {[string length $doc]} {
      set attributes [tip_GetAttributes $doc]
    } elseif {[string length $col]} {
      set attributes [tip_GetAttributes $col]
    } else {
      set attributes {}
    }
    my onUpdateAttributes $attributes
  };# onDocumentChange

  #############################################################################
  ## Event: onUpdateAttributes
  #############################################################################
  method onUpdateAttributes {attributes} {
    my variable tablelist
    my clear
    set row 0
    foreach attr $attributes {
      set name  [tip_GetName       $attr]
      set type  [tip_GetValueType  $attr]
      set value [tip_GetValueValue $attr]
      switch -exact $type {
        GDM_STRING - CDM_STRING {
          $tablelist insert end [list $name $value $type]
        }
        GDM_STRING_SET - CDM_STRING_SET {
          $tablelist insert end [list $name [join $value ,] $type]
        }
        CDM_IMAGE {
          $tablelist insert end [list $name {} $type]
          $tablelist cellconfigure $row,1 -image [my getImage $value]
        }
        CDM_BASE64_IMAGE {
          $tablelist insert end [list $name {} $type]
          set value [::base64::decode $value]
          $tablelist cellconfigure $row,1 -image [my getImage $value]
        }
        default {
          $tablelist insert end [list $name $value $type]
        }
      }
      incr row
    }
  };# onUpdateAttributes

  destructor {
    my variable images
    foreach image $images {
      catch {image delete $image}
    }
    next
  };# destructor

};# class ELEP::ViewerBase::AttributesWidget

package provide ELEP::ViewerBase::AttributesWidget 1.0
