##############################################################################
##  ColourManager.tcl:                                                      ##
## --------------------------------------------------                       ##
##  This class manages a set of colour values (foreground/background).      ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require TclOO
package require ELEP::ViewerBase::ColourPairs

oo::class create ELEP::ViewerBase::ColourManager {
  # This class assumes that the variable ::ELEP::ViewerBase::ColourPairs exists,
  # and is a list of background/foreground colour pairs.

  variable current_colour_index;# The current colour from the list of colours.
  
  constructor {} {
    set current_colour_index 0
  }

  method next {} {
    incr current_colour_index
    if {$current_colour_index >= [llength $::ELEP::ViewerBase::ColourPairs]} {
      set current_colour_index 0
    }
  };# next

  # Returns a list with 2 elements (background foreground).
  method nextColourPair {} {
    my next
    return [lindex $::ELEP::ViewerBase::ColourPairs $current_colour_index]
  };# nextColourPair

  # Returns a list with 2 elements (background foreground).
  method ColourPairNext {} {
    set pair [lindex $::ELEP::ViewerBase::ColourPairs $current_colour_index]
    my next
    return $pair
  };# ColourPairNext

  # Returns a list with 2 elements (background foreground).
  method colourPair {} {
    return [lindex $::ELEP::ViewerBase::ColourPairs $current_colour_index]
  };# colourPair

  method colourPairBackground {} {
    return [lindex [lindex $::ELEP::ViewerBase::ColourPairs \
                           $current_colour_index] 0]
  };# colourPairBackground

  method colourPairForeground {} {
    return [lindex [lindex $::ELEP::ViewerBase::ColourPairs \
                           $current_colour_index] 1]
  };# colourPairForeground

  export ColourPairNext

};# class ELEP::ViewerBase::ColourManager

package provide ELEP::ViewerBase::ColourManager 1.0
