##############################################################################
##  CoreferenceAnnotator.tcl:                                               ##
## --------------------------------------------------                       ##
##  This class implements a widget that allows co-reference relations,      ##
##  added as Annotations to a Document.                                     ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase
package require ELEP::ViewerBase::ColourManager
package require ELEP::ViewerBase::MultiAttributedAnnotationSpecificationSelector
package require ELEP::ViewerBase::AnnotationHighlighter
package require ELEP::ViewerBase::1ClickSelector

ELEP::Base::loadImage IBack
ELEP::Base::loadImage Remove

ELEP::Base::loadImage ITPrev
ELEP::Base::loadImage ITRemove

oo::class create ELEP::ViewerBase::CoreferenceAnnotator {
  superclass ELEP::WindowUtilities::WidgetBase

  variable {*}[info class variables ELEP::WindowUtilities::WidgetBase] \
           window_obj \
           display_object \
           annotation \
           attributes \
           attributeValues \
           attributeValueDescriptions \
           attributeType \
           selector \
           colour_manager \
           annotation_highlighter_object \
           1_click_selector_object \
           value2coreference \
           accelerator_key \
           TEXT \
           START \
           END \
           current_id \
           ignore_unselect_events

  constructor {win parent_win {disp_obj {}}} {
    set window_obj     {};# The object that holds our toplevel window.
    set display_object {};# The object that shows annotated information.
    set annotation     {}
    set attributes     {}
    # set attributeValues
    # set attributeValueDescriptions
    # set attributeType
    set selector       {}
    set colour_manager {}
    set annotation_highlighter_object {}
    set 1_click_selector_object {};# An object that extends the way selection
                                   # is performed
    # set value2coreference
    set accelerator_key {}
    # set TEXT
    # set START
    # set END
    set current_id {}
    set ignore_unselect_events 0
    
    next ttk::notebook [::ELEP::WindowUtilities::UniqueWindowName \
                           $parent_win.buttonAnnotator]
    
    set window_obj $win
    set display_object $disp_obj
    if {[string length $display_object]} {
      $display_object call registerAnnotatorObject [self]
    }
    ## Toolbar.
    my createToolbar

    ## Annotator Coreference area.
    ScrolledWindow $widget.buttonAnnotatorSW
    $widget add $widget.buttonAnnotatorSW -text {Coreference Annotator}
    ScrollableFrame $widget.buttonAnnotatorSW.client_area -constrainedwidth 1
    $widget.buttonAnnotatorSW setwidget $widget.buttonAnnotatorSW.client_area

    ## Annotation Highlighter area.
    my createAnnotationHighlighterObject
    $widget add [$annotation_highlighter_object getframe] \
      -text {Annotation Highlighter}

    ## 1-Click Selector area.
    my create1ClickSelectorObject
    $widget add [$1_click_selector_object getframe] -text {1-Click Selector}
      
    set client_area [$widget.buttonAnnotatorSW.client_area getframe]
    ELEP::WindowUtilities::AutoFocusOnMouseEnter $client_area
    my addScrollBindings $widget.buttonAnnotatorSW.client_area $client_area
  };# constructor

  method createToolbar {} {
  };# createToolbar

  method createAutomaticAnnotatorObject {} {
    # set automatic_annotator_object [ELEP::ViewerBase::AutomaticTagger \
    #                   ::ELEP::ViewerBase::#auto $widget $display_object]
    # $automatic_annotator_object configure -attribute_values_object [self]
  };# createAutomaticAnnotatorObject

  method createAnnotationHighlighterObject {} {
    set annotation_highlighter_object [ELEP::ViewerBase::AnnotationHighlighter \
      create [::ELEP::Base::auto ::ELEP::ViewerBase::] $widget $display_object]
  };# createAnnotationHighlighterObject

  method create1ClickSelectorObject {} {
    set 1_click_selector_object [ELEP::ViewerBase::1ClickSelector \
      create [::ELEP::Base::auto ::ELEP::ViewerBase::] $widget $display_object]
  };# create1ClickSelectorObject

  method createSpecificationSelectorObject {} {
    set selector \
      [ELEP::ViewerBase::MultiAttributedAnnotationSpecificationSelector \
             create [::ELEP::Base::auto ::ELEP::ViewerBase::]]
  };# createSpecificationSelectorObject

  method createColourManagerObject {} {
    set colour_manager [ELEP::ViewerBase::ColourManager \
               create [::ELEP::Base::auto ::ELEP::ViewerBase::]]
  };# createColourManagerObject

  method show {} {
    if {[string length $colour_manager]} {
      error "init can be called only once!"
    }
    my createSpecificationSelectorObject
    my createColourManagerObject
    set answer [$selector show]
    if {[string equal $answer ok]} {my init}
    $selector destroy
    return $answer
  };# show

  method init {} {
    set annotation            [$selector cget -annotation]
    set attributes            [$selector cget -attributes]
    set attributeType         [$selector cget -attributeType]
    array set attributeValues [$selector getArray attributeValues]
    array set attributeValueDescriptions \
                              [$selector getArray attributeValueDescriptions]
    my createAnnotatorWidgets
    focus $client_area
  };# init

  method createAnnotatorWidgets {} {
    my variable TEXT START END
    grid [ttk::label $client_area.lat -textvariable [my msgVar Attribute]] \
         [ttk::label $client_area.lte -textvariable [my msgVar Text]] \
         [ttk::label $client_area.lst -textvariable [my msgVar Start]] \
         [ttk::label $client_area.len -textvariable [my msgVar End]] \
         [ttk::label $client_area.lid -textvariable [my varname current_id]] \
         - -sticky snew -padx 1 -pady 1
    grid columnconfigure $client_area 1 -weight 1
    set i 0
    if {[llength $attributes] > 10} {
      set image(add) ITPrev
      set image(del) ITRemove
    } else {
      set image(add) IBack
      set image(del) IRemove
    }
    foreach attribute $attributes type $attributeType {
      # puts "attribute: \"$attribute\", type: \"$type\""
      if {![string length $attribute]} {continue}
      set TEXT($attribute)  {}
      set START($attribute) {}
      set END($attribute)   {}
      set description [subst -nocommands \
          [$selector getAttributeDescription $attribute]]
      switch -glob $type {
        check* {
          ## Add a checkbutton...
          set TEXT($attribute) 0
          grid [ttk::label    $client_area.l$i -text $description \
                    -anchor e -justify right] \
               [ttk::checkbutton $client_area.c$i \
                            -variable [my varname TEXT]($attribute) \
                            -text "($attribute)"] \
               - - - - -sticky ew -padx 1 -pady 1
        }
        combo* {
          ## Add a combobox...
          grid [ttk::label    $client_area.l$i -text $description \
                    -anchor e -justify right] \
               [ttk::combobox $client_area.c$i -width 16 \
                            -textvariable [my varname TEXT]($attribute) \
                            -values $attributeValueDescriptions($attribute)] \
               - - - - -sticky ew -padx 1 -pady 1
        }
        entry {
          ## Add a simple entry...
          grid [ttk::label  $client_area.l$i -text $description \
                    -anchor e -justify right] \
               [ttk::entry  $client_area.e$i -width 16 \
                            -textvariable [my varname TEXT]($attribute)] \
               - - - - -sticky ew -padx 1 -pady 1
        }
        title {
          grid [label $client_area.l$i -text "  \u25ba  $description" \
                  -font {arial 10 bold} -anchor w \
                  -foreground white -background #78B1F3] \
               - - - - - -sticky ew -padx 1 -pady 1
        }
        span -
        default {
          ## The default type allows the user to put document text...
          grid [ttk::label  $client_area.l$i -text $description \
                    -anchor e -justify right] \
               [ttk::entry  $client_area.t$i -width 16 \
                            -textvariable [my varname TEXT]($attribute)] \
               [ttk::entry  $client_area.s$i -width 7 \
                            -textvariable [my varname START]($attribute)] \
               [ttk::entry  $client_area.e$i -width 7 \
                            -textvariable [my varname END]($attribute)] \
               [ttk::button $client_area.b$i -image $image(add) \
                 -command [list [self] onButtonPress $attribute $i]] \
               [ttk::button $client_area.bc$i -image $image(del) \
                 -command [list [self] onFieldClear $attribute $i]] \
               -sticky ew -padx 1 -pady 1
          $client_area.t$i state readonly
          $client_area.s$i state readonly
          $client_area.e$i state readonly
          bind $client_area.b$i <<MB_2>> \
               [list [self] onButtonAcceleratorChange $client_area.b$i]
          bind $client_area.bc$i <<MB_2>> \
               [list [self] onButtonAcceleratorChange $client_area.bc$i]
        }
      };# switch attribute type
      incr i
    }
    grid [ttk::button $client_area.annotate -textvariable [my msgVar Annotate] \
               -command [list [self] onAnnotate]] -columnspan 6 -sticky snew
    bind $client_area.annotate <<MB_2>> \
           [list [self] onButtonAcceleratorChange $client_area.annotate]
  };# createAnnotatorWidgets

  method resetAnnotatorWidgets {} {
    my variable TEXT START END
    foreach attribute $attributes {
      set TEXT($attribute)  {}
      set START($attribute) {}
      set END($attribute)   {}
    }
    set current_id {}
    $client_area.annotate configure -textvariable [my msgVar Annotate]
  };# resetAnnotatorWidgets

  method editAnnotation {id} {
    my variable TEXT START END
    set current_id $id
    set doc [$display_object call getDocument]
    set ann [tip_GetAnnotation $doc $id]
    set text [tip_GetRawData $doc]
    foreach attribute $attributes type $attributeType {
      set attr [tip_GetAttribute $ann $attribute]
      puts "$type $attribute $attr"
      if {[llength $attr]} {
        switch -glob $type {
          title {
            set TEXT($attribute) {}
          }
          combo* {
            set TEXT($attribute) [my getDescriptionFromValue \
               [tip_GetValueValue $attr] $attribute $type]
          }
          check* - entry {
            set TEXT($attribute) [tip_GetValueValue $attr]
          }
          default {
            set span [tip_GetValueValue $attr]
            if {![catch {tip_GetSpanOffsets $span start end}]} {
              set START($attribute) $start
              set END($attribute)   $end
              set TEXT($attribute)  [string range $text $start [incr end -1]]
            }
          }
        }
      } else {
        set TEXT($attribute)  {}
        set START($attribute) {}
        set END($attribute)   {}
      }
    }
    $client_area.annotate configure -textvariable [my msgVar Update]
  };# editAnnotation

  method getValueFromDescription {attribute {type {}}} {
    my variable TEXT START END
    switch -glob $type {
      combo* {
        set description $TEXT($attribute)
        set index [lsearch -exact $attributeValueDescriptions($attribute) \
                                  $description]
        if {$index == -1} {return $description}
        return [lindex $attributeValues($attribute) $index]
      }
    }
    return $TEXT($attribute)
  };# getValueFromDescription

  method getDescriptionFromValue {value attribute {type {}}} {
    switch -glob $type {
      combo* {
        set index [lsearch -exact $attributeValues($attribute) \
                                  $value]
        if {$index == -1} {return $value}
        return [lindex $attributeValueDescriptions($attribute) $index]
      }
    }
    return $value
  };# getDescriptionFromValue


  ##########################################################################
  ## Event: onFieldClear
  ##########################################################################
  method onFieldClear {attribute i} {
    my variable TEXT START END
    set TEXT($attribute)  {}
    set START($attribute) {}
    set END($attribute)   {}
  };# onFieldClear

  ##########################################################################
  ## Event: onButtonPress
  ##########################################################################
  method onButtonPress {attribute i} {
    if {![string length $display_object]} {return}
    if {![$display_object call selectionExists]} {return}
    my variable TEXT START END
    ## Get the selection indexes...
    set doc               [$display_object call getDocument]
    set offsetWRTDocument [$display_object call offsetWRTDocument]
    set start             [$display_object call index2CharacterOffset \
                             [$display_object call getSelectionStartIndex]]
    set end               [$display_object call index2CharacterOffset \
                             [$display_object call getSelectionEndIndex]]
    if {$start < 0 || $end < 0} {return}
    if {$start > $end} {
      set s $start; set start $end; set end $s
    }
    incr start $offsetWRTDocument; incr end $offsetWRTDocument

    set START($attribute) $start
    set END($attribute)   $end
    incr end -1
    set TEXT($attribute) [string range [tip_GetRawData $doc] $start $end]
    $display_object call selectionClear
  };# onButtonPress

  ##########################################################################
  ## Event: onAnnotate
  ##########################################################################
  method onAnnotate {} {
    if {![string length $display_object]} {return}
    my variable TEXT START END
    $display_object call selectionClear
    ## Create the annotation...
    set spans {}
    set attrs {}
    foreach attribute $attributes type $attributeType {
      switch -glob $type {
        title {}
        combo* {
          set v [my getValueFromDescription $attribute $type]
          lappend attrs [tip_CreateAttribute $attribute \
               [tip_CreateAttributeValue GDM_STRING $v]]
        }
        check* - entry {
          lappend attrs [tip_CreateAttribute $attribute \
               [tip_CreateAttributeValue GDM_STRING $TEXT($attribute)]]
        }
        default {
          set start $START($attribute)
          set end   $END($attribute)
          if {![string length $start] || $start < 0}    {continue}
          if {![string length $end]   || $start > $end} {continue}
          set span [tip_CreateSpan $start $end]
          lappend spans $span
          lappend attrs [tip_CreateAttribute $attribute \
                     [tip_CreateAttributeValue GDM_STRING_SET $span]]
        }
      }
    }
    if {![llength $spans]} {return}
    set ann [tip_CreateAnnotation $annotation $spans $attrs]
    if {[string length $current_id]} {
      set ann [lreplace $ann 0 0 $current_id]
      $display_object callAll deleteSelection
    }
    set doc [$display_object call getDocument]
    set id  [tip_AddAnnotation $doc $ann] 
    set ann [tip_GetAnnotation $doc $id]
    $display_object callAll annotateAnnotationSet_NoAttribute [list $ann] 0 0
    $display_object callAll selectionClear
    my resetAnnotatorWidgets
  };# onAnnotate

  ##########################################################################
  ## Event: onButtonAcceleratorChange
  ##########################################################################
  method onButtonAcceleratorChange {win} {
    if {![string length $display_object]} {return}
    set dialog_window [Dialog [::ELEP::WindowUtilities::UniqueWindowName \
      [$window_obj getframe].editAcceleratorDlg] -modal local -separator 1 \
      -title "Setting Coreference Accelerator..." -default 0 -cancel 1]
    $dialog_window add -textvariable GUI_Text(Ok)
    $dialog_window add -textvariable GUI_Text(Cancel)
    ## Try to retrieve any previously defined accelarator key.
    set accelerator_key {}
    foreach event [bind $win] {
        switch -glob -- $event {
          ?       {set accelerator_key $event}
          <Key-*> {set accelerator_key [string range $event 5 end-1]}
          default {}
        }
      }
    set frame [$dialog_window getframe]
      message $frame.label -width 250 -text \
       "Please press the keyboard keys combination you want to assign as an\
       accelerator to the button with value \"[$win cget -text]\""
      ttk::label $frame.accl -text "Key:" -anchor e
      ttk::label $frame.acc -textvariable [my varname accelerator_key] \
           -anchor w
      bind [winfo toplevel $frame] <Key> \
           [list set [my varname accelerator_key] %K]
      grid $frame.label -sticky snew -columnspan 2
      grid $frame.accl $frame.acc -sticky snew
    if {[$dialog_window draw] == 0} {
      set toplevel [winfo toplevel $win]
      ## Remove any previously set accelerators...
      foreach event [bind $win] {
        switch -glob -- $event {
          ? - <Key-*> { bind $win $event {}; bind $toplevel $event {} }
          default {}
        }
      }
      ## Add the binding...
      if {[string length $accelerator_key]} {
        bind $win      <Key-$accelerator_key> "$win invoke"
        bind $toplevel <Key-$accelerator_key> "$win invoke"
      }
    }
  };# onButtonAcceleratorChange

  ##########################################################################
  ## Event: invokeAutomaticAnnotator
  ##########################################################################
  method invokeAutomaticAnnotator {} {
    if {![string length $automatic_annotator_object]} {return}
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {![string length $doc]} {return}
    set col [$display_object call getCollection]
    foreach id [$automatic_annotator_object applyAutomaticTagger $col $doc] {
      lappend anns [tip_GetAnnotation $doc $id]
    }
    if {[info exists anns]} {
      $display_object call annotateAnnotationSet $anns $annotation $attribute
    }
  };# invokeAutomaticAnnotator

  ##########################################################################
  ## Event: select
  ##########################################################################
  method select {value {id {}} {tag {}}} {
    # puts "[self] select $value"
    set ignore_unselect_events 1
    catch {$display_object callAll selectTag $tag 0}
    if {[catch {my editAnnotation $id} error]} {
      catch {puts stderr "CoreferenceAnnotator::select: $error\n$::errorInfo"}
    }
    set ignore_unselect_events 0
  };# select

  ##########################################################################
  ## Event: unselect
  ##########################################################################
  method unselect {value {id {}} {tag {}}} {
    # puts "[self] unselect $value"
    if {!$ignore_unselect_events} {
      catch {$display_object callAll unselectSelectedTags 0}
      my resetAnnotatorWidgets
    }
  };# unselect

  ##########################################################################
  ## Event: getValueColourPair
  ##########################################################################
  method getValueColourPair {value} {
    if {![info exists value2coreference($value)]} {
      my addValueCoreference $value
    }
    return $value2coreference($value)
  };# getValueColourPair

  method addValueCoreference {value} {
    foreach {bg fg} [$colour_manager ColourPairNext] {break}
    set value2coreference($value) [list $bg $fg]
  };# addValueCoreference

  ##########################################################################
  ## Event: getValues
  ##########################################################################
  method getValues {} {
    return [lsort [array names value2coreference]]
  };# getValues

  ##########################################################################
  ## Event: onDocumentChange
  ##########################################################################
  method onDocumentChange {col doc} {
    # if {![string length $automatic_annotator_object]} {return}
    # $automatic_annotator_object configure -annotation $annotation \
    #   -attribute $attribute
    # $automatic_annotator_object    onDocumentChange $col $doc
    $annotation_highlighter_object onDocumentChange $col $doc
  };# onDocumentChange

  ##########################################################################
  ## Event: annotationWillBeDeleted
  ##########################################################################
  method annotationWillBeDeleted {id} {
  };# annotationWillBeDeleted

  ##########################################################################
  ## Event: annotationAttributeWillBeDeleted
  ##########################################################################
  method annotationAttributeWillBeDeleted {id attr} {
    my annotationWillBeDeleted $id
  };# annotationAttributeWillBeDeleted

  ##########################################################################
  ## Event: deletionHandled
  ##########################################################################
  method deletionHandled {ann_document ann_id ann_attribute ann_tag} {
    set ann [tip_GetAnnotation $ann_document $ann_id]
    puts "Removing Annotation: $ann"
    tip_RemoveAnnotation $ann_document $ann_id
    $display_object callAll deleteTag $ann_tag
    return 1
  };# deletionHandled

  method addScrollBindings {sw win} {
    bind $win <MouseWheel> \
       "$sw yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $win <4> "$sw yview scroll -5 units"
    bind $win <5> "$sw yview scroll  5 units"
  };# addScrollBindings

  method addScrollBindingsToWidgetList {sw args} {
    foreach win $args {addScrollBindings $sw $win}
  };# addScrollBindingsToWidgetList

  method getAnnotation {} {return $annotation}
  method getAttribute  {} {return {}}

  destructor {
    catch {$colour_manager destroy}
    catch {$1_click_selector_object destroy}
    catch {$automatic_annotator_object destroy}
  };# destructor

};# class ELEP::ViewerBase::CoreferenceAnnotator

package provide ELEP::ViewerBase::CoreferenceAnnotator 1.0
