##############################################################################
##  TemplateFiller.tcl:                                                     ##
## --------------------------------------------------                       ##
##  This class implements a widget that presents a set of tables, that the  ##
##  user can fill with either annotations, or text portions.                ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

GUI_LoadImage ITNew
GUI_LoadImage ITShowTags
GUI_LoadImage ITHideTags
GUI_LoadImage ITAdd
GUI_LoadImage ITRemove

package require ELEP::ViewerBase::ButtonAnnotator
package require ELEP::ViewerBase::AnnotationAndTemplateSpecificationSelector
package require tablelist_tile

oo::class create ELEP::ViewerBase::TemplateFiller {
  superclass ELEP::ViewerBase::ButtonAnnotator

  variable {*}[info class variables ELEP::ViewerBase::ButtonAnnotator] \
           template \
           tables \
           tableFields \
           max_value_number \
           field_type \
           primary_fields \
           field_value_constraints \
           parent_notebook_widget \
           filled_tables_preview_widget \
           filled_tables_compact_preview_widget \
           filled_tables \
           template_filler_widget \
           tf_widget \
           selected_filled_table \
           table_buttons_client_area \
           selected_filled_tables

  constructor {win parent_win parent_notebook {disp_obj {}}} {
    set template {}
    set tables   {}
    # set tableFields          ;# An array mapping keys of "$table" to
    #                           # the list of table fields.
    # set max_value_number     ;# An array mapping keys of "$table $field" to
    #                           # maximum allowed values.
    # set field_type           ;# An array mapping keys of "$table $field" to
    #                           # the field type.
    # set primary_fields       ;# An array mapping keys of $table to
    #                           # the list of primary fields for the table.
    # set field_value_constraints ;# An array mapping keys of "$table $field"
    #                           # to the field value_constraints.
    set parent_notebook_widget {};# A notebook widget, where we will create
                                  # a page that shows previews of filled
                                  # tables.
    set filled_tables_preview_widget {}
    set filled_tables_compact_preview_widget {}
    set template_filler_widget {}
    set tf_widget              {};# The widget holding the template filler.
    set selected_filled_table  {}
    set table_buttons_client_area {}

    next $win $parent_win $disp_obj
    
    set parent_notebook_widget $parent_notebook
    ## Add a page in the $parent_notebook_widget, to preview filled tables (as a
    ## list).
    set filled_tables_compact_preview_widget \
      $parent_notebook_widget.filledTablesListPreviewWidget
    ttk::panedwindow $filled_tables_compact_preview_widget -orient vertical
    $parent_notebook_widget add $filled_tables_compact_preview_widget \
      -text {Filled Tables (Compact)}

    ## Add a page in the $parent_notebook_widget, to preview filled tables.
    ScrolledWindow $parent_notebook_widget.filledTablesPreviewWidget
    ScrollableFrame $parent_notebook_widget.filledTablesPreviewWidget.area \
      -constrainedwidth 1
    $parent_notebook_widget.filledTablesPreviewWidget setwidget \
      $parent_notebook_widget.filledTablesPreviewWidget.area
    $parent_notebook_widget add \
      $parent_notebook_widget.filledTablesPreviewWidget \
      -text {Filled Tables (Extended)}
    set filled_tables_preview_widget \
      [$parent_notebook_widget.filledTablesPreviewWidget.area getframe]
    
    ## Add a page in $widget, to allow a user to fill tables.
    set tf_widget $widget.tableFiller
    ttk::notebook $tf_widget
    $widget insert 1 $tf_widget -text {Table Filler}

    ## Add a page in $widget, to allow a user to fill templates.
    set template_filler_widget $widget.templateFiller
    ScrolledWindow $template_filler_widget
    ScrollableFrame $template_filler_widget.area -constrainedwidth 1
    $template_filler_widget setwidget $template_filler_widget.area
    $widget insert 2 $template_filler_widget -text {Template Filler}
    $widget select 1
  };# constructor

  ##########################################################################
  ## Object Creation
  ##########################################################################
  method createSpecificationSelectorObject {} {
    set selector [ELEP::ViewerBase::AnnotationAndTemplateSpecificationSelector \
                      create [::ELEP::Base::auto ::ELEP::ViewerBase::]]
  };# createSpecificationSelectorObject

  ##########################################################################
  ## Widget Creation
  ##########################################################################
  method init {} {
    next
    set template  [$selector cget -template]
    set tables    [$selector getTemplateTables]
    puts "template: $template"
    puts "tables: $tables"

    my createTemplateManagementWidgetTop
    
    foreach table $tables {
      set selected_filled_tables($table) {}
      ## Filled tables preview area
      set w [my createFilledTablesPreviewWidget $table]
      grid $w -padx 4 -pady 5 -sticky snew
      $filled_tables_compact_preview_widget add \
        [my createFilledTablesCompactPreviewWidget $table] -weight 1
      ## Table filler area
      $tf_widget insert end [my createTableEditingWidget $table] -text $table
      ## Template Table selection area
      set w [my createTemplateEditingWidget $table]
      grid $w -padx 2 -pady 2 -sticky snew -columnspan 3
    }

    my createTemplateManagementWidgetBottom

    grid columnconfigure [$template_filler_widget.area getframe] 0 -weight 1
    grid columnconfigure $filled_tables_preview_widget 0 -weight 1
    grid columnconfigure $filled_tables_compact_preview_widget 0 -weight 1
    $tf_widget select 0
  };# init

  method createFilledTablesPreviewWidget {table} {
    ttk::labelframe $filled_tables_preview_widget.lf$table \
      -text " $table " -height 40
    for {set i 0} {$i < 3} {incr i} {
      grid columnconfigure $filled_tables_preview_widget.lf$table $i \
        -weight 1 -uniform tfFilledTablesPreviewWidget
    }
    return $filled_tables_preview_widget.lf$table
  };# createFilledTablesPreviewWidget

  method createFilledTablesCompactPreviewWidget {table} {
    ttk::labelframe $filled_tables_compact_preview_widget.lf$table \
      -text " $table "
    ScrolledWindow $filled_tables_compact_preview_widget.lf$table.sw
    tablelist::tablelist $filled_tables_compact_preview_widget.lf$table.sw.tbls\
      -stripebackground #e0e8f0 -background gray98 -stretch 1 \
      -columns {4 Id 0 Value} -height 4 -exportselection 0 \
      -listvariable [my varname filled_tables]($table) -selectmode multiple \
      -selectforeground white -selectbackground #6b8e23
    bind $filled_tables_compact_preview_widget.lf$table.sw.tbls \
      <<TablelistSelect>> [list [self] \
      onFilledTablesCompactPreviewWidgetSelectionChanged $table]
    $filled_tables_compact_preview_widget.lf$table.sw setwidget \
      $filled_tables_compact_preview_widget.lf$table.sw.tbls
    pack $filled_tables_compact_preview_widget.lf$table.sw -fill both -expand 1
    return $filled_tables_compact_preview_widget.lf$table
  };# createFilledTablesCompactPreviewWidget

  method createTemplateEditingWidget {table} {
    set win [ttk::labelframe [$template_filler_widget.area getframe].lf$table \
      -text " $table "]
    message $win.value -justify left -anchor nw -background white \
      -foreground navy -relief sunken -borderwidth 1 -aspect 1500 \
      -textvariable [my varname selected_filled_tables]($table)
    ttk::button $win.fill -text {Automatic Fill} -command \
      [list [self] onAutomaticFillTemplateFields $table]
    pack $win.value -side left -fill x -expand 1 -padx 2 -pady 2
    pack $win.fill  -side left -padx 2 -pady 2
    return $win
  };# createTemplateEditingWidget

  method createTemplateManagementWidgetTop {} {
    set frame [$template_filler_widget.area getframe]
    label $frame.templateName -foreground white -background #78B1F3 \
      -text "  \u25ba  Template: $::GUI_Text(New)" \
      -font {arial 10 bold} -anchor w
    grid $frame.templateName -columnspan 2 -sticky snew -padx 0 -pady 2
    ## Add 3 buttons for new template, show/hide tags.
    button $frame.templateNew -image ITNew \
      -background #78B1F3 -activebackground #78B1F3 \
      -relief flat -overrelief raised -highlightthickness 0 \
      -anchor center -state disabled -command \
      [list [self] onSwitchTemplateEditorView new]
    grid $frame.templateNew -column 2 -row 0 -sticky snew -padx 0 -pady 2
    grid columnconfigure $frame 1 -weight 1
  };# createTemplateManagementWidgetTop

  method createTemplateManagementWidgetBottom {} {
    ## Add buttons for creating/updating/deleting filled templates.
    set f [$template_filler_widget.area getframe]
    ttk::frame $f.tbf
      button $f.tbf.createTemplate -text {Add Template} -width 16 \
        -background #6b8e23 -foreground white -font {arial 10 bold} -command \
        [list [self] onCreateTemplateAnnotation]
      button $f.tbf.clearTemplate -text {Clear Fields} -width 16 \
        -background orange -foreground white -font {arial 10 bold} -command \
      [list [self] onSwitchTemplateEditorView new]
      pack $f.tbf.createTemplate -side right -pady 1
      pack $f.tbf.clearTemplate  -side right -padx 4 -pady 1
    grid $f.tbf -padx 2 -pady 1 -sticky snew -columnspan 3

    ## Create a frame for showing the various filled templates...
    set w [ttk::frame [$template_filler_widget.area getframe].filledTables]
      label $w.label -text "  \u25ba  Templates" \
        -font {arial 10 bold} -foreground white -background #78B1F3 -anchor w
      grid $w.label -sticky snew -padx 1 -pady 1
      ScrolledWindow $w.sw
      tablelist::tablelist $w.sw.tables \
        -stripebackground #e0e8f0 -background gray98 -stretch 1 \
        -columns {4 Id 0 Value} -height 16 -exportselection 0 \
        -selectmode extended
      bind $w.sw.tables <<TablelistSelect>> \
        [list [self] onEditTemplateAnnotationSelectionChanged %W]
      $w.sw setwidget $w.sw.tables
      ttk::button $w.del -image ITRemove \
        -command [list [self] onDeleteSelectedTemplateAnnotation]
      grid $w.label -columnspan 2 -sticky snew -padx 1 -pady 1
      grid $w.sw $w.del -sticky snew -padx 1 -pady 1
      grid columnconfigure $w 0 -weight 1
      grid rowconfigure    $w 1 -weight 1
    grid $w -padx 2 -pady 2 -sticky snew -columnspan 3
  };# createTemplateManagementWidgetBottom

  method createTableEditingWidget {table} {
    global GUI_Text
    set frame $tf_widget.p$table
    ttk::panedwindow $frame -orient vertical
    ScrolledWindow $frame.sw
    ScrollableFrame $frame.sw.area -constrainedwidth 1
    set sw $frame.sw.area;# Needed for adding scroll bindings.
    my addScrollBindings $sw $sw
    $frame.sw setwidget $frame.sw.area
    $frame add $frame.sw -weight 6

    ## Create a frame for showing the various filled tables...
    ttk::frame $frame.filledTables
      label $frame.filledTables.label -text "  \u25ba  Tables" \
        -font {arial 10 bold} -foreground white -background #78B1F3 -anchor w
      ScrolledWindow $frame.filledTables.sw
      tablelist::tablelist $frame.filledTables.sw.tables \
        -stripebackground #e0e8f0 -background gray98 -stretch 1 \
        -columns {4 Id 0 Value} -height 6 -exportselection 0 \
        -listvariable [my varname filled_tables]($table) \
        -selectmode extended
      bind $frame.filledTables.sw.tables <<TablelistSelect>> \
        [list [self] onEditTableAnnotationSelectionChanged %W]
      $frame.filledTables.sw setwidget $frame.filledTables.sw.tables
      ttk::button $frame.filledTables.del -image ITRemove \
        -command [list [self] onDeleteSelectedTableAnnotation $table]
      ttk::button $frame.filledTables.dup -text C -width 3 \
        -command [list [self] onDuplicateSelectedTableAnnotation $table]
      grid $frame.filledTables.label -columnspan 2 -sticky snew -padx 1 -pady 1
      grid $frame.filledTables.sw -rowspan 2 -sticky snew -padx 1 -pady 1
      grid $frame.filledTables.dup -column 1 -row 1 -sticky snew -padx 1 -pady 1
      grid $frame.filledTables.del -column 1 -row 2 -sticky snew -padx 1 -pady 1
      grid columnconfigure $frame.filledTables 0 -weight 1
      grid rowconfigure    $frame.filledTables 2 -weight 1
    $frame add $frame.filledTables -weight 4

    set frame [$frame.sw.area getframe]
    my addScrollBindings $sw $frame
    ELEP::WindowUtilities::AutoFocusOnMouseEnter $frame

    ## Create a frame for editing the template table...
    ttk::frame $frame.editTable
    label $frame.editTable.tableName -foreground white -background #78B1F3 \
      -text "  \u25ba  Table: $GUI_Text(New)" \
      -font {arial 10 bold} -anchor w
    grid $frame.editTable.tableName -columnspan 3 -sticky snew -padx 0 -pady 2
    ## Add 3 buttons for new table, show/hide tags.
    frame $frame.editTable.buttons -background #78B1F3
      button $frame.editTable.buttons.tableNew -image ITNew \
        -background #78B1F3 -activebackground #78B1F3 \
        -relief flat -overrelief raised -highlightthickness 0 \
        -anchor center -state disabled -command \
        [list [self] onSwitchTableEditorView $table new]
      pack $frame.editTable.buttons.tableNew -side right -padx 2 -pady 2
    grid $frame.editTable.buttons -column 3 -row 0 \
        -sticky snew -padx 0 -pady 2

    ## Add buttons for creating/updating/deleting filled table.
    ttk::frame $frame.editTable.tbf
    button $frame.editTable.tbf.createTable -text {Add Table} -width 16 \
      -background #6b8e23 -foreground white -font {arial 10 bold} -command \
      [list [self] onCreateTableAnnotation $table]
    bind $frame.editTable.tbf.createTable <<MB_2>> \
      [list [self] onButtonAcceleratorChange $frame.editTable.tbf.createTable]
    button $frame.editTable.tbf.clearTable -text {Clear Fields} -width 16 \
      -background orange -foreground white -font {arial 10 bold} -command \
      [list [self] onSwitchTableEditorView $table new]
    bind $frame.editTable.tbf.clearTable <<MB_2>> \
      [list [self] onButtonAcceleratorChange $frame.editTable.tbf.clearTable]
    button $frame.editTable.tbf.autoAdd -text {Automatic Add} -width 16 \
      -background #778899 -foreground white -font {arial 10 bold} -command \
      [list [self] onAutomaticFieldValueAdd $table]
    bind $frame.editTable.tbf.autoAdd <<MB_2>> \
      [list [self] onButtonAcceleratorChange $frame.editTable.tbf.autoAdd]
    pack $frame.editTable.tbf.createTable -side right -pady 1
    pack $frame.editTable.tbf.clearTable  -side right -padx 4 -pady 1
    pack $frame.editTable.tbf.autoAdd     -side right -padx 4 -pady 1
    grid columnconfigure $frame.editTable 1 -weight 1
    my addScrollBindingsToWidgetList $sw $frame.editTable.tableName \
      $frame.editTable.buttons.tableNew $frame.editTable.buttons \
      $frame.editTable.tbf.createTable $frame.editTable.tbf.clearTable \
      $frame.editTable.tbf

    ## Add field editing facilities for each filed...
    set tableFields($table) [$selector getTemplateTableFields $table]
    set index 0; set row 1
    foreach field $tableFields($table) {
      ## Keep information about the filed.
      set key [list $table $field]
      set max_value_number($key) \
        [$selector getTemplateTableFieldMaxValueNumber $table $field]
      set field_type($key) \
        [$selector getTemplateTableFieldValueType $table $field]
      lappend field_value_constraints($key) [split [string map {:: { }} \
        [$selector getTemplateTableFieldValueConstraints $table $field]]]
      if {[$selector getTemplateTableFieldPrimary $table $field]} {
        lappend primary_fields($table) $field
      }
      # set description [$selector getTemplateDescription $table $field]
      set field_frame $frame.editTable.f$index

      set height 2
      if {$max_value_number($key) && $max_value_number($key) < $height} {
        set height $max_value_number($key)
      }
      ttk::label  ${field_frame}label -anchor w -text \
        [string totitle [string map {_ { }} $field:]]
      switch -exact $field_type($key) {
        annotation::id::from_table {
          ttk::menubutton ${field_frame}addValue -image ITAdd \
            -menu ${field_frame}addValue.menu
          menu ${field_frame}addValue.menu -postcommand \
            [list [self] onFieldValueAdd ${field_frame} $key]
        }
        default {
          ttk::button ${field_frame}addValue -image ITAdd \
            -command [list [self] onFieldValueAdd ${field_frame} $key]
        }
      }
      ttk::button ${field_frame}removeValue -image ITRemove \
        -command [list [self] onFieldValueRemove ${field_frame} $key]
      grid ${field_frame}label x ${field_frame}addValue \
        ${field_frame}removeValue -row $row -sticky snew -padx 1 -pady 1
      my addScrollBindingsToWidgetList $sw \
        ${field_frame}label ${field_frame}addValue ${field_frame}removeValue 
      if {$max_value_number($key) == 1} {
        listbox ${field_frame}listbox -height 1
        grid ${field_frame}listbox -row $row -column 1 \
          -sticky ew -padx 1 -pady 1
        my addScrollBindings $sw ${field_frame}listbox
      } else {
        ttk::label ${field_frame}values_number
        ScrolledWindow ${field_frame}values
        listbox ${field_frame}values.listbox -height $height \
          -selectmode extended
        ${field_frame}values setwidget ${field_frame}values.listbox
        grid ${field_frame}values -row $row -column 1 -rowspan 2 \
          -sticky snew -padx 1 -pady 1
        grid ${field_frame}values_number -row [incr row] -column 2 \
          -columnspan 2 -sticky snew -padx 1 -pady 1
        my addScrollBindingsToWidgetList $sw ${field_frame}values \
          ${field_frame}values.listbox ${field_frame}values_number
      }
      incr index; incr row
    }
    if {![info exists primary_fields($table)]} {
      set primary_fields($table) [list [lindex $tableFields($table) 0]]
    }

    grid $frame.editTable.tbf -columnspan 4 -sticky snew -padx 0 -pady 1
    grid $frame.editTable -sticky snew
    grid columnconfigure $frame 0 -weight 1
    grid rowconfigure    $frame 1 -weight 1
    return $tf_widget.p$table
  };# createTableEditingWidget

  ##########################################################################
  ## Client areas request functions
  ##########################################################################
  method getTableClientArea {table} {
    return [$tf_widget.p$table.sw.area getframe]
  };# getTableClientArea

  method getTableFilledTablesListbox {table} {
    return $tf_widget.p$table.filledTables.sw.tables
  };# getTableFilledTablesListbox

  method getFilledTablesPreviewClientArea {table} {
    return $filled_tables_preview_widget.lf$table
  };# getFilledTablesPreviewClientArea

  method getFilledTablesCompactPreviewClientArea {table} {
    return $filled_tables_compact_preview_widget.lf$table
  };# getFilledTablesCompactPreviewClientArea

  ##########################################################################
  ##########################################################################
  ##
  ## Methods for Table Filling
  ##
  ##########################################################################
  ##########################################################################
  
  ##########################################################################
  ## Event: onTableFieldsClear
  ##########################################################################
  method onTableFieldsClear {table} {
    set index 0; set frame [my getTableClientArea $table].editTable
    foreach field $tableFields($table) {
      set key [list $table $field]
      if {$max_value_number($key) == 1} {
        $frame.f${index}listbox delete 0 end
      } else {
        $frame.f${index}values.listbox delete 0 end
        $frame.f${index}values_number configure -text {}
      }
      incr index
    }
  };# onTableFieldsClear

  ##########################################################################
  ## Event: onAutomaticFieldValueAdd
  ##########################################################################
  method onAutomaticFieldValueAdd {table} {
    set frame [my getTableClientArea $table]
    set index 0
    foreach field $tableFields($table) {
      set field_frame $frame.editTable.f$index
      set key [list $table $field]
      switch -exact $field_type($key) {
        annotation::id::from_table {
          # Fields of this type cannot be added automatically
        }
        default {
          if {![catch {my onFieldValueAdd $field_frame $key}]} {return}
        }
      }
      incr index
    }
    gut_ErrorDlg "Cannot find a suitable field in this table type to\
                  be automatically filled!"
  };# onAutomaticFieldValueAdd

  ##########################################################################
  ## Event: onFieldValueAdd
  ##########################################################################
  method onFieldValueAdd {field_frame key} {
    if {![string length $display_object]} {return}
    ## According to the field value type, request the selection from the display
    ## object.
    switch -exact $field_type($key) {
      annotation::id {
        set id [lindex [$display_object call getSelectedAnnotationIds] 0]
        if {![string length $id]} {return}
        if {![my validateFieldValue $key $id]} {
          foreach constraint $field_value_constraints($key) {
            foreach {type attr val} $constraint {break}
            lappend types $val
          }
          error "Invalid value type for this field.\nValid Types are:\
                 [join $types {, }]"
        }
        set value [my formatFieldValue $field_type($key) $id]
      }
      annotation::id::from_table {
        ## We have to select from the filled tables:
        set len [string length $template:]
        set m ${field_frame}addValue.menu
        $m delete 0 end
        foreach table $field_value_constraints($key) {
          set table [string range $table $len end]
          foreach item [my getFilledTables $table] {
            set value [my formatFieldValue $field_type($key) [lindex $item 0]]
            $m add command -label $item -command \
              [list [self] addFieldValue $field_frame $key $value]
          }
        }
        return
      }
      default {error "unknown field value type: \"$field_type($key)\""}
    }
    my addFieldValue $field_frame $key $value
  };# onFieldValueAdd

  method validateFieldValue {key value} {
    switch -exact $field_type($key) {
      annotation::id {
        set ann [tip_GetAnnotation [$display_object call getDocument] $value]
        foreach constraint $field_value_constraints($key) {
          foreach {type attr val} $constraint {break}
          if {![catch {tip_GetValueValue [tip_GetAttribute $ann $attr]} v]} {
            if {[string match $val $v]} {return 1}
          }
        }
      }
      annotation::id::from_table {return 1}
      default {error "unknown field value type: \"$field_type($key)\""}
    }
    return 0
  };# validateFieldValue
  
  method formatFieldValue {type value} {
    if {![string length $value]} {return {}}
    switch -exact $type {
      annotation::id {
        return "$value [join [$display_object call getAnnotatedTextRanges \
                                              $value] {, }]"
      }
      annotation::id::from_table {
        return "$value [my getTablePrimaryFieldLabel $value]"
      }
      default {return $value}
    }
  };# formatFieldValue

  method normaliseFieldValue {type value} {
    switch -exact $type {
      annotation::id {
        return [lindex [split $value] 0]
      }
      annotation::id::from_table {
        return [lindex [split $value] 0]
      }
      default {return $value}
    }
  };# normaliseFieldValue

  method addFieldValue {field_frame key value} {
    if {$max_value_number($key) == 1} {
      set listbox ${field_frame}listbox
      $listbox delete 0 end
      $listbox insert end $value
    } else {
      set listbox ${field_frame}values.listbox
      if {$max_value_number($key) == 0 ||
          [$listbox size] < $max_value_number($key)} {
        $listbox insert end $value
        $listbox see end
        $listbox selection clear 0 end
        $listbox selection set end
        ${field_frame}values_number configure -text [string map {0 {}} \
          [$listbox size]]
      } else {
        error "Maximum number of values for this table field\
              ($max_value_number($key)) reached."
      }
    }
  };# addFieldValue

  ##########################################################################
  ## Event: onFieldValueRemove
  ##########################################################################
  method onFieldValueRemove {field_frame key} {
    if {$max_value_number($key) == 1} {
      ${field_frame}listbox delete 0 end
    } else {
      foreach index [${field_frame}values.listbox curselection] {
        ${field_frame}values.listbox delete $index
      }
      ${field_frame}values_number configure -text [string map {0 {}} \
        [${field_frame}values.listbox size]]
    }
  };# onFieldValueRemove

  ##########################################################################
  ## Event: onCreateTableAnnotation
  ##########################################################################
  method onCreateTableAnnotation {table} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {![string length $doc]} {return}
    ## As a first step, create an empty annotation, so as to have an id:
    set ann [tip_CreateAnnotation $template:$table \
                 [tip_CreateSpanSet 0 0] [tip_CreateAttributeSet]]
    set id [tip_AddAnnotation $doc $ann]
    ## Add the created table (annotation) to the list of tables:
    my addTableInList $table $id
    ## Add an (empty) table to the page of filled tables:
    my addTableInFilledTablesPreviewPage $table $id
    my arrangeFilledTablesInPreviewPage $table
    ## Allow onUpdateTableAnnotation to fill the structures we have just created
    my onUpdateTableAnnotation $id
  };# onCreateTableAnnotation

  method addTableInList {table id} {
    [my getTableFilledTablesListbox $table] insert end [list $id {}]
  };# addTableInList

  method addTableInFilledTablesPreviewPage {table id} {
    set win [my getFilledTablesPreviewClientArea $table].b$id
    tablelist::tablelist $win -stripebackground #e0e8f0 \
      -columns {8 Field 13 Value} -background gray98 \
      -height 5 -width 25 -stretch 0 -activestyle none
    bind [$win bodytag] <<MB_1>> [list [self] onFilledTableClick $table $id]
    return $win
  };# addTableInFilledTablesPreviewPage

  method arrangeFilledTablesInPreviewPage {table {tables_per_row 3}} {
    set frame [my getFilledTablesPreviewClientArea $table]
    set _col 0; set _row 0
    foreach w [winfo children $frame] {
      if {$_col >= $tables_per_row} {set _col 0; incr _row}
      grid $w -row $_row -column $_col -sticky snew -padx 1 -pady 1; incr _col
    }
  };# arrangeFilledTablesInPreviewPage

  ##########################################################################
  ## Event: onEditTableAnnotation
  ##########################################################################
  method onEditTableAnnotation {id} {
    if {![string length $display_object]} {return}
    set ann [tip_GetAnnotation [$display_object call getDocument] $id]
    set table [string range [tip_GetType $ann] [string length $template:] end]
    my onTableFieldsClear $table
    set index 0; set frame [my getTableClientArea $table].editTable
    foreach field $tableFields($table) {
      set key [list $table $field]
      set field_frame $frame.f$index
      catch {
        set value [tip_GetValueValue [tip_GetAttribute $ann $field]]
        ## Is it a multiple valued field?
        if {$max_value_number($key) == 1} {
          my addFieldValue $field_frame $key \
                        [my formatFieldValue $field_type($key) $value]
        } else {
          foreach i $value {
            my addFieldValue $field_frame $key \
                        [my formatFieldValue $field_type($key) $i]
          }
        }
      }
      incr index
    }
    my onSwitchTableEditorView $table edit $id
  };# onEditTableAnnotation

  ##########################################################################
  ## Event: onEditTableAnnotationSelectionChanged
  ##########################################################################
  method onEditTableAnnotationSelectionChanged {listbox} {
    set sel [$listbox curselection]
    if {![llength $sel]} {return}
    ## Get the id from the selected row item.
    my onEditTableAnnotation [lindex [$listbox get [lindex $sel 0]] 0]
  };# onEditTableAnnotationSelectionChanged

  ##########################################################################
  ## Event: onUpdateTableAnnotation
  ##########################################################################
  method onUpdateTableAnnotation {id} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    set ann [tip_GetAnnotation $doc $id]
    set table [string range [tip_GetType $ann] [string length $template:] end]
    set index 0; set frame [my getTableClientArea $table].editTable
    foreach field $tableFields($table) {
      set key [list $table $field]
      if {$max_value_number($key) == 1} {
        set value_type GDM_STRING
        set value [my normaliseFieldValue $field_type($key) \
                                       [$frame.f${index}listbox get 0]]
      } else {
        set value {}; set value_type GDM_STRING_SET
        foreach i [$frame.f${index}values.listbox get 0 end] {
          lappend value [my normaliseFieldValue $field_type($key) $i]
        }
      }
      incr index
      set ann [tip_PutAttribute $ann [tip_CreateAttribute $field \
                   [tip_CreateAttributeValue $value_type $value]]]
    }
    tip_AddAnnotation $doc $ann
    my updateTableListLabel $id
    my fillFilledTablesPreviewTable $id
    my onSwitchTableEditorView $table new
    return $id 
  };# onUpdateTableAnnotation

  method updateTableListLabel {id} {
    if {![string length $display_object]} {return {}}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    set value [my getTablePrimaryFieldLabel $id]
    ## Locate this table id in the filled tables listbox.
    set listbox [my getTableFilledTablesListbox $table]
    set items [$listbox getcolumns 0]
    set index [lsearch -exact $items $id]
    if {$index == -1} {return}
    $listbox delete $index
    $listbox insert $index [list $id $value]
  };# updateTableListLabel

  method setTableListLabel {id value} {
    if {![string length $display_object]} {return {}}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    ## Locate this table id in the filled tables listbox.
    set listbox [my getTableFilledTablesListbox $table]
    set items [$listbox getcolumns 0]
    set index [lsearch -exact $items $id]
    if {$index == -1} {return}
    $listbox delete $index
    $listbox insert $index [list $id $value]
  };# setTableListLabel

  method configureTableListLabelColours {id {bg {}} {fg {}}} {
    if {![string length $display_object]} {return {}}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    ## Locate this table id in the filled tables listbox.
    set listbox [my getTableFilledTablesListbox $table]
    set items [$listbox getcolumns 0]
    set index [lsearch -exact $items $id]
    if {$index == -1} {return}
    $listbox rowconfigure $index -foreground $fg -background $bg
  };# configureTableListLabelColours

  method getTablePrimaryFieldLabel {id} {
    if {![string length $display_object]} {return {}}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    set value_list {}
    foreach field $primary_fields($table) {
      set attr [tip_GetAttribute $ann $field]
      if {![llength $attr]} {continue}
      set value [tip_GetValueValue $attr]
      ## Is it a multiple valued field?
      set key [list $table $field]
      if {$max_value_number($key) != 1} {
        set value [lindex $value 0]
      }
      if {[string length $value]} {
        lappend value_list "[join [lrange [split \
                  [my formatFieldValue $field_type($key) $value]] 1 end] \ ]"
      }
    }
    if {[llength $value_list]} {return [join $value_list { }]}
    foreach field $tableFields($table) {
      set attr [tip_GetAttribute $ann $field]
      if {![llength $attr]} {continue}
      set value [tip_GetValueValue $attr]
      ## Is it a multiple valued field?
      set key [list $table $field]
      if {$max_value_number($key) != 1} {
        set value [lindex $value 0]
      }
      if {[string length $value]} {
        set key [list $table $field]
        lappend value_list "[join [lrange [split \
                  [my formatFieldValue $field_type($key) $value]] 1 end] \ ]"
      }
    }
    return [join $value_list { }]
    # return "[join [lrange [split \
    #               [my formatFieldValue $field_type($key) $value]] 1 end] \ ]"
  };# getTablePrimaryFieldLabel

  method fillFilledTablesPreviewTable {id} {
    if {![string length $display_object]} {return}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    set win [my getFilledTablesPreviewClientArea $table].b$id
    catch {$win delete 0 end}
    $win insert end [list Id $id]
    $win rowconfigure 0 -selectable 0 -foreground red -font {arial 10 bold}
    foreach field $tableFields($table) {
      catch {
        set key [list $table $field]
        set value [tip_GetValueValue [tip_GetAttribute $ann $field]]
        ## Is it a multiple valued field?
        if {$max_value_number($key) != 1} {
          set value [lindex $value 0]
        }
        set value "[join [lrange [split \
                   [my formatFieldValue $field_type($key) $value]] 1 end] \ ]"
        $win insert end [list $field $value]
        $win rowconfigure end -selectable 0
      }
    }
  };# fillFilledTablesPreviewTable

  ##########################################################################
  ## Event: onDuplicateSelectedTableAnnotation
  ##########################################################################
  method onDuplicateSelectedTableAnnotation {table} {
    set listbox [my getTableFilledTablesListbox $table]
    set sel [$listbox curselection]
    if {![llength $sel]} {return}
    foreach item $sel {
      my onDuplicateTableAnnotation [lindex [$listbox get $item] 0]
    }
    $listbox selection clear 0 end
    $listbox selection set end
  };# onDuplicateSelectedTableAnnotation

  ##########################################################################
  ## Event: onDuplicateTableAnnotation
  ##########################################################################
  method onDuplicateTableAnnotation {id} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {[catch {tip_GetAnnotation $doc $id} ann]} {
      return "invalid table annotation: $id"
    }
    ## Clone this Annotation...
    set clone [tip_CreateAnnotation [tip_GetType $ann] [tip_GetSpans $ann] \
                                    [tip_GetAttributes $ann]]
    set id [tip_AddAnnotation $doc $clone]
    set table [string range [tip_GetType $ann] [string length $template:] end]
    ## Add the created table (annotation) to the list of tables:
    my addTableInList $table $id
    ## Add an (empty) table to the page of filled tables:
    my addTableInFilledTablesPreviewPage $table $id
    my arrangeFilledTablesInPreviewPage $table
    my updateTableListLabel $id
    my fillFilledTablesPreviewTable $id
    my onEditTableAnnotation $id
  };# onDuplicateTableAnnotation

  ##########################################################################
  ## Event: onDeleteSelectedTableAnnotation
  ##########################################################################
  method onDeleteSelectedTableAnnotation {table} {
    set listbox [my getTableFilledTablesListbox $table]
    set sel [$listbox curselection]
    if {![llength $sel]} {return}
    foreach item $sel {
      my onDeleteTableAnnotation [lindex [$listbox get $item] 0]
    }
    $listbox delete $sel
  };# onDeleteSelectedTableAnnotation

  ##########################################################################
  ## Event: onDeleteTableAnnotation
  ##########################################################################
  method onDeleteTableAnnotation {id} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {[catch {tip_GetAnnotation $doc $id} ann]} {
      return "invalid table annotation: $id"
    }
    set table [string range [tip_GetType $ann] [string length $template:] end]
    my onTableFieldsClear $table
    my removeTableFromFilledTablesPreviewPage $table $id
    tip_RemoveAnnotation $doc $id
    my onSwitchTableEditorView $table new
  };# onDeleteTableAnnotation

  method removeTableFromFilledTablesPreviewPage {table id} {
    destroy [my getFilledTablesPreviewClientArea $table].b$id
  };# removeTableFromFilledTablesPreviewPage

  ##########################################################################
  ## Event: getFilledTables
  ##########################################################################
  method getFilledTables {table} {
    return [[my getTableFilledTablesListbox $table] get 0 end]
  };# getFilledTables

  ##########################################################################
  ##########################################################################
  ##
  ## Methods for Template Filling
  ##
  ##########################################################################
  ##########################################################################

  ##########################################################################
  ## Event: onTemplateFieldsClear
  ##########################################################################
  method onTemplateFieldsClear {} {
    foreach table $tables {
      foreach id $selected_filled_tables($table) {
        catch {my onFilledTableClick $table $id}
      }
      set selected_filled_tables($table) {}
    }
  };# onTemplateFieldsClear

  ##########################################################################
  ## Event: onCreateTemplateAnnotation
  ##########################################################################
  method onCreateTemplateAnnotation {} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {![string length $doc]} {return}
    set ann [tip_CreateAnnotation $template \
                 [tip_CreateSpanSet 0 0] [tip_CreateAttributeSet]]

    set id [tip_AddAnnotation $doc $ann]
    ## Add the created template (annotation) to the list of templates:
    my addTemplateInList $id
    ## Allow onUpdateTemplateAnnotation to fill the structures we have just
    ## created.
    my onUpdateTemplateAnnotation $id
  };# onCreateTemplateAnnotation

  method addTemplateInList {id} {
    set listbox [$template_filler_widget.area getframe].filledTables.sw.tables
    $listbox insert end [list $id {}]
  };# addTemplateInList

  ##########################################################################
  ## Event: onEditTemplateAnnotation
  ##########################################################################
  method onEditTemplateAnnotation {id} {
    if {![string length $display_object]} {return}
    if {[catch {tip_GetAnnotation [$display_object call getDocument] $id} ann]} {
      return "invalid table annotation: $id"
    }
    my onTemplateFieldsClear
    foreach table $tables {
      catch {
        foreach i [tip_GetValueValue [tip_GetAttribute $ann $table]] {
          catch {my onFilledTableClick $table $i}
        }
      }
    }
    my onSwitchTemplateEditorView edit $id
  };# onEditTemplateAnnotation

  ##########################################################################
  ## Event: onEditTemplateAnnotationSelectionChanged
  ##########################################################################
  method onEditTemplateAnnotationSelectionChanged {listbox} {
    set sel [$listbox curselection]
    if {![llength $sel]} {return}
    ## Get the id from the selected row item.
    my onEditTemplateAnnotation [lindex [$listbox get [lindex $sel 0]] 0]
  };# onEditTemplateAnnotationSelectionChanged

  ##########################################################################
  ## Event: onUpdateTemplateAnnotation
  ##########################################################################
  method onUpdateTemplateAnnotation {id} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {[catch {tip_GetAnnotation $doc $id} ann]} {
      return "invalid table annotation: $id"
    }
    foreach table $tables {
      set ann [tip_PutAttribute $ann [tip_CreateAttribute $table \
                   [tip_CreateAttributeValue GDM_STRING_SET \
                        $selected_filled_tables($table)]]]
    }
    tip_AddAnnotation $doc $ann
    my onSwitchTemplateEditorView new
    return $id 
  };# onUpdateTemplateAnnotation

  ##########################################################################
  ## Event: onDeleteSelectedTemplateAnnotation
  ##########################################################################
  method onDeleteSelectedTemplateAnnotation {} {
    set listbox [$template_filler_widget.area getframe].filledTables.sw.tables
    set sel [$listbox curselection]
    if {![llength $sel]} {return}
    foreach item $sel {
      my onDeleteTemplateAnnotation [lindex [$listbox get $item] 0]
    }
    $listbox delete $sel
  };# onDeleteSelectedTemplateAnnotation

  ##########################################################################
  ## Event: onDeleteTemplateAnnotation
  ##########################################################################
  method onDeleteTemplateAnnotation {id} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    set ann [tip_GetAnnotation $doc $id]
    my onTemplateFieldsClear
    tip_RemoveAnnotation $doc $id
    my onSwitchTemplateEditorView new
  };# onDeleteTemplateAnnotation

  ##########################################################################
  ## Event: onSwitchTableEditorView
  ##########################################################################
  method onSwitchTableEditorView {table view {id {}}} {
    set frame [my getTableClientArea $table]
    switch -exact $view {
      new  {
        my onTableFieldsClear $table
        #my unselectFilledTable
        $frame.editTable.tableName configure -text \
           "  \u25ba  Table: $::GUI_Text(New)"
        $frame.editTable.tbf.createTable configure -text {Add Table} \
          -command [list [self] onCreateTableAnnotation $table]
        $frame.editTable.tbf.clearTable configure -text {Clear Fields}
        $frame.editTable.buttons.tableNew configure -state disabled
      }
      edit {
        #my selectFilledTable $table $id
        $frame.editTable.tableName configure -text "  \u25ba  Table: $id"
        $frame.editTable.tbf.createTable configure -text {Update Table} \
          -command [list [self] onUpdateTableAnnotation $id]
        $frame.editTable.tbf.clearTable configure -text {Cancel}
        $frame.editTable.buttons.tableNew configure -state normal
      }
    }
  };# onSwitchTableEditorView

  ##########################################################################
  ## Event: onSwitchTemplateEditorView
  ##########################################################################
  method onSwitchTemplateEditorView {view {id {}}} {
    set frame [$template_filler_widget.area getframe]
    switch -exact $view {
      new  {
        my onTemplateFieldsClear
        $frame.templateName configure -text \
           "  \u25ba  Template: $::GUI_Text(New)"
        $frame.templateNew  configure -state disabled
        $frame.tbf.createTemplate configure -text {Add Template} \
          -command [list [self] onCreateTemplateAnnotation]
      }
      edit {
        $frame.templateName configure -text "  \u25ba  Template: $id"
        $frame.templateNew  configure -state normal
        $frame.tbf.createTemplate configure -text {Update Template} \
          -command [list [self] onUpdateTemplateAnnotation $id]
      }
    }
  };# onSwitchTemplateEditorView

  ##########################################################################
  ## Event: onDocumentChange
  ##########################################################################
  method onDocumentChange {col doc} {
    next $col $doc
    my removeAllTemplatesFromFilledTemplateList
    foreach table $tables {
      my onSwitchTableEditorView $table new
      my removeAllTablesFromFilledTableList $table
      my removeAllTablesFromFilledTablesPreviewPage $table
      set selected_filled_tables($table) {}
      set lst [my getFilledTablesCompactPreviewClientArea $table].sw.tbls
      $lst selection clear 1 end
      if {![string length $doc]} {continue}
      foreach ann [tip_SelectAnnotations $doc $template:$table] {
        set id [tip_GetId $ann]
        my addTableInList $table $id
        if {[my validAnnotation $doc $ann]} {
          if {[catch {my updateTableListLabel $id}]} {
            my setTableListLabel $id "## Invalid Fields found in table!"
            my configureTableListLabelColours $id red white
            lappend msg "Invalid Fields in table \"$table\": $id"
          }
        } else {
          my setTableListLabel $id "## Invalid Fields found in table!"
          my configureTableListLabelColours $id red white
          lappend msg "Invalid Fields in table \"$table\": $id"
        }
        my addTableInFilledTablesPreviewPage $table $id
        my fillFilledTablesPreviewTable $id
      }
      my arrangeFilledTablesInPreviewPage $table
    }
    foreach ann [tip_SelectAnnotations $doc $template] {
      set id [tip_GetId $ann]
      my addTemplateInList $id
    }
    if {[info exists msg]} {
      gut_info [join $msg \n]
    }
  };# onDocumentChange

  method removeAllTablesFromFilledTableList {table} {
    [my getTableFilledTablesListbox $table] delete 0 end
  };# removeAllTablesFromFilledTableList

  method removeAllTablesFromFilledTablesPreviewPage {table} {
    set frame [my getFilledTablesPreviewClientArea $table]
    foreach w [winfo children $frame] {
      if {[string equal $w $frame.label]} {continue}
      destroy $w
    }
  };# removeAllTablesFromFilledTablesPreviewPage

  method removeAllTemplatesFromFilledTemplateList {} {
    [$template_filler_widget.area getframe].filledTables.sw.tables delete 0 end
  };# removeAllTemplatesFromFilledTemplateList
  
  ##########################################################################
  ## Event: onFilledTableClick
  ##########################################################################
  method onFilledTableClick {table id} {
    set win [my getFilledTablesPreviewClientArea $table].b$id
    set lst [my getFilledTablesCompactPreviewClientArea $table].sw.tbls
    set ind [my id2TableListIndex $table $id]
    if {![info exists selected_filled_tables($table)]} {
      set selected_filled_tables($table) {}
    }
    set index [lsearch -exact $selected_filled_tables($table) $id]
    if {$index == -1} {
      $win configure -background #8fbc8f \
        -stripebackground #6b8e23 -stripeforeground white
      lappend selected_filled_tables($table) $id
      $lst selection set $ind
      my onEditTableAnnotation $id
    } else {
      set selected_filled_tables($table) [lreplace \
        $selected_filled_tables($table) $index $index]
      $win configure -background gray98 \
        -stripebackground #e0e8f0 -stripeforeground black
      $lst selection clear $ind
      my onSwitchTableEditorView $table new
    }
    $tf_widget select $tf_widget.p$table
  };# onFilledTableClick

  method id2TableListIndex {table id} {
    set index 0
    foreach item $filled_tables($table) {
      if {[string equal $id [lindex $item 0]]} {return $index}
      incr index
    }
  };# id2TableListIndex

  ##########################################################################
  ## Event: onAutomaticFillTemplateFields
  ##########################################################################
  method onFilledTablesCompactPreviewWidgetSelectionChanged {table} {
    set lst [my getFilledTablesCompactPreviewClientArea $table].sw.tbls
    ## Get all selected ids...
    set sel_ids {}
    foreach ind [$lst curselection] {
      lappend sel_ids [lindex $filled_tables($table) $ind 0]
    }
    ## We must find the intersection between the two lists:
    foreach id1 [lsort -unique $sel_ids] \
            id2 [lsort -unique $selected_filled_tables($table)] {
      if {![string equal $id1 $id2]} {
        if {[string length $id1]} {my onFilledTableClick $table $id1}
        if {[string length $id2]} {my onFilledTableClick $table $id2}
      }
    }
  };# onFilledTablesCompactPreviewWidgetSelectionChanged

  ##########################################################################
  ## Event: onAutomaticFillTemplateFields
  ##########################################################################
  method onAutomaticFillTemplateFields {table} {
    if {![string length $display_object]} {return}
    set doc [$display_object call getDocument]
    if {![string length $doc]} {return}
    ## Examine fields, to locate ones with type annotation::id::from_table
    set keys {}
    foreach key [array names field_type] {
      if {[string equal $field_type($key) annotation::id::from_table] &&
          [string equal $field_value_constraints($key) $template:$table]} {
        lappend keys $key
      }
    }
    if {![llength $keys]} {return}
    ## Now, keys holds a list of keys that whose values relates to this table.
    foreach id $selected_filled_tables($table) {
      foreach key $keys {
        foreach {ot attr} $key {break}
        foreach ann [tip_SelectAnnotations $doc $template:$ot \
                       "\[lsearch -exact ann::$attr $id\] != -1"] {
          set rid [tip_GetId $ann]
          set win [my getFilledTablesPreviewClientArea $ot].b$rid
          if {[winfo exists $win]} {
            ## We have found the table! Is it already selected?
            set index [lsearch -exact $selected_filled_tables($ot) $rid]
            if {$index == -1} {my onFilledTableClick $ot $rid}
          }
        }
      }
    }
  };# onAutomaticFillTemplateFields

  ##########################################################################
  ## Method: validAnnotationonAutomaticFillTemplateFields
  ##########################################################################
  #  This method returns true if all attributes whose value is an integer (or a
  #  set of integers)
  method validAnnotation {doc ann} {
    foreach attr [tip_GetAttributes $ann] {
      foreach value [tip_GetValueValue $attr] {
        if {[string is integer $value]} {
          if {[catch {tip_GetAnnotation $doc $value}]} {
            return 0
          }
        }
      }
    }
    return 1
  };# validAnnotation

};# class ELEP::ViewerBase::TemplateFiller

package provide ELEP::ViewerBase::TemplateFiller 1.0
