##############################################################################
##  TextWidgetDisplayAlignment.tcl:                                         ##
## --------------------------------------------------                       ##
##  This class provides facilities for adding tags in a text widget, that   ##
##  visualise text portions annotated by Document Annotations.              ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::ViewerBase::TextWidgetDisplay

oo::class create ELEP::ViewerBase::TextWidgetDisplayAlignment {
  superclass ELEP::ViewerBase::TextWidgetDisplay

  variable {*}[info class variables ELEP::ViewerBase::TextWidgetDisplay] \
           alignment_set alignment_spans

  constructor {tobj textw texti {doc {}}} {
    set alignment_set {}
    set alignment_spans {}
    next $tobj $textw $doc
  };# class constructor

  method setAlignmentSet {set} {
    set alignment_set $set
  };# setAlignmentSet

  method getAlignmentSet {} {
    return $alignment_set
  };# setAlignmentSet

  method setText {val} {
    my restoreMainWidget
    if {![my validTextWidget]} {return}
    ## Search for the alignment-info annotation. Search its attributes to find
    ## one whose value is $alignment_set. Then, restrict the text to the
    ## values of its spans...
    set min_offset [expr {[string length $val] + 100}]
    set alignment_spans {}
    foreach alignment_info [tip_SelectAnnotations $document alignment-info] {
      foreach attr [tip_GetAttributes $alignment_info] {
        if {[string equal $alignment_set [tip_GetValueValue $attr]]} {
          set alignment_spans [tip_GetSpans $alignment_info]
          break
        }
      }
      if {[llength $alignment_spans]} {
        break
      }
    }
    if {[llength $alignment_spans]} {
      set text {}
      foreach span $alignment_spans {
        tip_GetSpanOffsets $span start end
        if {$start < $min_offset} {set min_offset $start}
        append text [string range $val $start [incr end -1]]
      }
      set text_widget_offset $min_offset
    } else {
      ## No alignment information found. Use all text...
      set text $val
    }
    my textWidgetStateNormal
    $text_widget delete 1.0 end
    foreach tag [$text_widget tag names] {$text_widget tag delete $tag}
    $text_widget tag configure stripe1 -background "" -foreground ""
    $text_widget tag configure stripe2 -background #e6e6fa -foreground ""
    foreach {line1 line2} [split [string trim $text] \n] {
      $text_widget insert end $line1\n stripe1 $line2\n stripe2
    }
    $text_widget tag add back 1.0 end
    $text_widget tag bind back <<MB_1>> [list [self] onClick]
    $text_widget tag lower back
    $text_widget tag raise sel
    my textWidgetStateDisabled
  };# setText

  method filterDocumentAnnotations {annotations} {
    if {![llength $alignment_spans]} {return [next $annotations]}
    set anns {}
    foreach span $alignment_spans {
      tip_GetSpanOffsets $span start end
      set anns [concat $anns [tip_AnnotationsInRange $annotations $start $end]]
    }
    return $anns
  };# filterDocumentAnnotations

  method skipSpan {start end} {
    foreach span $alignment_spans {
      tip_GetSpanOffsets $span s e
      if {$s <= $start && $end <= $e} {
        return 0
      }
    }
    return 1
  };# skipSpan

};# class ELEP::ViewerBase::TextWidgetDisplayAlignment

package provide ELEP::ViewerBase::TextWidgetDisplayAlignment 1.0
