##############################################################################
##  AttributesViewer.tcl:                                                   ##
## --------------------------------------------------                       ##
##  This class implements a viewer for Attributes.                          ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ToplevelWindow
package require ELEP::ViewerBase::AttributesWidget
package require ELEP::ViewerBase::DocumentSelector

oo::class create ELEP::Viewers::AttributesViewer {
  superclass ELEP::WindowUtilities::ToplevelWindow

  constructor {{col {}} {doc {}}} {
    next
    my createWidgets $col $doc
  };# constructor

  method createWidgets {col doc} {
    my variable viewer_object
    ## Add a document selector object (toolbar).
    my createDocumentSelector $col $doc
    my createToolbar
    ## Create the viewer object...
    my createViewer
    ## Set the proper title to our toplevel window.
    my setTitle
    ## Load the attributes...
    # $viewer_object  
  };# createWidgets

  method createDocumentSelector {col doc} {
    my variable selector_object
    set selector_object [ELEP::ViewerBase::DocumentSelector new \
                                           [self] {} $col $doc 0 0]
  };# createDocumentSelector

  method createToolbar {} {
  };# createToolbar

  method createViewer {} {
    my variable client_area viewer_object selector_object
    set viewer_object [ELEP::ViewerBase::AttributesWidget new $client_area]
    pack [$viewer_object getframe] -fill both -expand 1
    $selector_object registerListener $viewer_object
    $selector_object registerListener [self]
  };# createLeftAreaWidgets

  method setTitle {} {
    my title {Attributes Viewer...}
  };# setTitle

  method show {} {
    my variable selector_object
    next
    $selector_object show
  };# show

  method loadDocument {doc_id} {
    my variable selector_object
    $selector_object loadDocument $doc_id
  };# loadDocument

  #############################################################################
  ## Event: onUpdateAttributes
  #############################################################################
  method onUpdateAttributes {attributes} {
    my variable viewer_object
    $viewer_object onUpdateAttributes $onUpdateAttributes
  };# onUpdateAttributes

  #############################################################################
  ## Event: onDocumentChange
  #############################################################################
  method onDocumentChange {col doc} {
    my updateStatusFromDocument $col $doc
  };# onDocumentChange

};# class ELEP::Viewers::AttributesViewer

package provide ELEP::Viewers::AttributesViewer 1.0
