##############################################################################
##  VisualRegexp.tcl:                                                       ##
## --------------------------------------------------                       ##
##  This class implements a tool for editing regular expressions.           ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::VisualAnnotators::VisualRegexp {

  common visual_regexp_source [file join $::HomeDir ellogon2.0 lib_elep \
    ELEP_VisualAnnotators visual_regexp-3.1.tcl]

  variable slave_interpreter

  constructor {} {
    my common visual_regexp_source
    set slave_interpreter [interp create]
    $slave_interpreter eval set argc 0
    $slave_interpreter eval set argv {{}}
    $slave_interpreter eval rename exit slave_exit
    $slave_interpreter eval [list proc exit {} {wm withdraw .; set ::exit 1}]
    $slave_interpreter eval package require Tk
    $slave_interpreter eval source $visual_regexp_source
    $slave_interpreter eval wm protocol . WM_DELETE_WINDOW exit
    $slave_interpreter eval regexp::history:init
    $slave_interpreter eval regexp::gui
    $slave_interpreter eval regexp::go
  };# constructor

  method setPattern {pattern} {
    $slave_interpreter eval [list regexp::regexp:set $pattern]
  };# setPattern

  method getPattern {} {
    return [$slave_interpreter eval regexp::regexp:get]
  };# getPattern

  method show {} {
    $slave_interpreter eval update
    $slave_interpreter eval wm deiconify .
    $slave_interpreter eval update
    $slave_interpreter eval tkwait variable ::exit
    return [my getPattern]
  };# show

  method hide {} {
    $slave_interpreter eval wm withdraw .
  };# hide

  destructor {
    if {[string length $slave_interpreter]} {interp delete $slave_interpreter}
  };# destructor

};# class ELEP::VisualAnnotators::VisualRegexp

package provide ELEP::VisualAnnotators::VisualRegexp 1.0
