##############################################################################
##  ObjectListenerClient.tcl:                                               ##
## --------------------------------------------------                       ##
##  This class implements a class for supporting object listeners.          ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

oo::class create ELEP::WindowUtilities::ObjectListenerClient {
  
  variable object_listener_server_objects

  constructor {} {
    set object_listener_server_objects {}
  };# constructor

  method initListenerObject {args} {
    set object_listener_server_objects $args
  };# initListenerObject

  ##########################################################################
  ## Event: registeredAsListener
  ##########################################################################
  method registeredAsListener {server_obj {method {}}} {
    if {[lsearch -exact $object_listener_server_objects $server_obj]} {
      lappend object_listener_server_objects $server_obj
    }
  };# registeredAsListener

  ##########################################################################
  ## Event: unregisteredAsListener
  ##########################################################################
  method unregisteredAsListener {server_obj {method {}}} {
    set index [lsearch -exact $object_listener_server_objects $server_obj]
    if {$index != -1} {
      set object_listener_server_objects \
        [lreplace $object_listener_server_objects $index $index]
    }
  };# unregisteredAsListener

  ##########################################################################
  ## method: notifyListeners
  ##########################################################################
  method notifyListeners {} {
    foreach server_obj $object_listener_server_objects {
      catch {$server_obj onUpdateRequestFromRegisteredListener [self]}
    }
  };# notifyListeners

  destructor {
    foreach obj $object_listener_server_objects {
      catch {$obj unregisterListener [self]}
    }
  };# destructor

};# class ELEP::WindowUtilities::ObjectListenerClient

package provide ELEP::WindowUtilities::ObjectListenerClient 1.0
