##############################################################################
##  RibbonToplevel.tcl:                                                     ##
## --------------------------------------------------                       ##
##  This file implements a class that holds a Toplevel Ellogon window.      ##
##  A Toplevel window is a window that has a menubar, a toolbar, a main     ##
##  and a status bar.                                                       ##
##  This class adds support for Windows Ribbons under Vista and Windows 7.  ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::ToplevelWindow

oo::class create ELEP::WindowUtilities::RibbonToplevelWindow {
  superclass ELEP::WindowUtilities::ToplevelWindow

  variable {*}[info class variables ELEP::WindowUtilities::ToplevelWindow] \
           RibbonLoaded

  constructor {dll resource {allow_exit 0}} {
    my variable RibbonDLL RibbonResource RibbonLoaded
    set RibbonDLL      $dll
    set RibbonResource $resource
    set RibbonLoaded   false
    next $allow_exit
  };# constructor

  ##########################################################################
  ## Method: initialiseRibbon
  ##########################################################################
  method initialiseRibbon {RibbonDLL RibbonResource} {
    package require tkribbon
    my variable win toolbar RibbonLoaded RibbonCommands
    set toolbar [tkribbon::ribbon $win.toolbarFrame \
        -command [list [self] onRibbonUpdatePropertyDispatch] ]
    $toolbar load_resources [file nativename $RibbonDLL]
    foreach event {Execute Preview CancelPreview CreateUICommand ViewChanged
                   DestroyUICommand UpdateProperty} {
      bind $toolbar <<on$event>> \
        [list [self] onRibbonEventDispatch onRibbon$event %d]
    }
    # wm minsize $win 290x250
    ## Is there a .data file?
    set data [file rootname $RibbonDLL].data
    catch {
      set fd [open $data]
      set data [read $fd]
      close $fd
      array set RibbonCommands $data
    }
    $toolbar load_ui [file tail $RibbonDLL] $RibbonResource
    set RibbonLoaded true
  };# initialiseRibbon

  ##########################################################################
  ## Event: onRibbonUpdatePropertyDispatch
  ##########################################################################
  method onRibbonUpdatePropertyDispatch {id property args} {
    return [my onRibbonUpdateProperty [my ribbonId $id] $id $property {*}$args]
  };# onRibbonUpdatePropertyDispatch

  ##########################################################################
  ## Event: onRibbonEventDispatch
  ##########################################################################
  method onRibbonEventDispatch {event id} {
    return [my $event [my ribbonId $id] $id]
  };# onRibbonEventDispatch

  ##########################################################################
  ## Event: onRibbonUpdateProperty
  ##########################################################################
  method onRibbonUpdateProperty {cmd id property args} {
  };# onRibbonUpdateProperty

  ##########################################################################
  ## Event: onRibbonExecute
  ##########################################################################
  method onRibbonExecute {cmd id} {
  };# onRibbonExecute

  ##########################################################################
  ## Event: onRibbonPreview
  ##########################################################################
  method onRibbonPreview {cmd id} {
  };# onRibbonPreview

  ##########################################################################
  ## Event: onRibbonCancelPreview
  ##########################################################################
  method onRibbonCancelPreview {cmd id} {
  };# onRibbonCancelPreview

  ##########################################################################
  ## Event: onRibbonCreateUICommand
  ##########################################################################
  method onRibbonCreateUICommand {cmd id} {
  };# onRibbonCreateUICommand

  ##########################################################################
  ## Event: onRibbonViewChanged
  ##########################################################################
  method onRibbonViewChanged {cmd id} {
  };# onRibbonViewChanged

  ##########################################################################
  ## Event: onRibbonDestroyUICommand
  ##########################################################################
  method onRibbonDestroyUICommand {cmd id} {
  };# onRibbonDestroyUICommand

  ##########################################################################
  ## Method: ribbonId
  ##########################################################################
  method ribbonId {id} {
    my variable RibbonCommands
    if {![info exists RibbonCommands($id)]} {
      set RibbonCommands($id) undefined
    }
    return $RibbonCommands($id)
  };# ribbonId

  ##########################################################################
  ## Method: ribbonModes
  ##########################################################################
  method ribbonModes {cmd args} {
    my variable toolbar
    return [$toolbar $cmd {*}$args]
  };# ribbonModes

  ##########################################################################
  ## Method: createToolBar
  ##########################################################################
  method createToolBar {} {
    my variable win toolbar RibbonDLL RibbonResource
    if {[catch {my initialiseRibbon $RibbonDLL $RibbonResource}]} {
      catch {destroy $toolbar}
      set toolbar [ttk::frame $win.toolbarFrame -class ToolbarFrame]
      return $toolbar
    }
  };# createToolBar

  ##########################################################################
  ## Method: createComponentLayoutSetMenubar
  ##########################################################################
  method createComponentLayoutSetMenubar {} {
    my variable RibbonLoaded
    if {$RibbonLoaded} {return}
    next
  };# createComponentLayoutSetMenubar

};# class ELEP::WindowUtilities::RibbonToplevelWindow

package provide ELEP::WindowUtilities::RibbonToplevelWindow 1.0

