##############################################################################
##  WidgetBase.tcl:                                                         ##
## --------------------------------------------------                       ##
##  This class implements a widget as an TclOO object. Any Tk widget can    ##
##  be contained.                                                           ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities

oo::class create ELEP::WindowUtilities::WidgetBase {
  mixin ELEP::Base::Utilities
  variable widget client_area

  constructor {widget_type name args} {
    set client_area {};# Our client area.

    if {[catch {$widget_type $name {*}$args} widget]} {
      puts "$widget $::errorInfo"; error $widget
    }
    bind itk-delete-$widget <Destroy> [list [self] destroy]
    set tags [bindtags $widget]
    bindtags $widget [linsert $tags 0 itk-delete-$widget]
    # bind $widget <Destroy> "if {\[string equal $widget %W\]}\
    #              {::ELEP::WindowUtilities::DeleteTclOOObject [self]}"
  };# constructor

  method getframe      {} {my variable widget; return $widget}
  method getToplevel   {} {my variable widget; return [winfo toplevel $widget]}
  method getToplevelObject {} {
    return [::ELEP::WindowUtilities::FindToplevelObject [my getToplevel]]
  };# getToplevelObject
  method getClientArea {} {my variable client_area; return $client_area}
  method destroyWindow {} {my variable widget; destroy $widget}

  destructor {
    my variable widget
    if {[info exists widget] && [winfo exists $widget]} {
      set tags [bindtags $widget]
      set i [lsearch $tags itk-delete-$widget]
      if {$i >= 0} {
        bindtags $widget [lreplace $tags $i $i]
      }
      destroy $widget
    }
    catch {bind itk-delete-$widget <Destroy> {}}
  };# destructor
};# class ELEP::WindowUtilities::WidgetBase

package provide ELEP::WindowUtilities::WidgetBase 1.0
