##############################################################################
##  AnnotationsToXML.tcl:                                                   ##
## --------------------------------------------------                       ##
##  This class implements a tool that can insert annotations as XML tags in ##
##  a text portion.                                                         ##
##                                                                          ##
##  This file is part of the Ellogon Language Engineering Platform.         ##
##  Copyright 1998-2010 by:                                                 ##
##  Georgios Petasis,                                                       ##
##  Athens, Greece.                                                         ##
##  E-mail: petasis@iit.demokritos.gr                                       ##
##          petasisg@yahoo.gr                                               ##
##          petasisg@gmail.com                                              ##
##                                                                          ##
##  For licensing information, please refer to the license conditions of    ##
##  "Ellogon" Language Engineering Platform.                                ##
##############################################################################

package require ELEP::WindowUtilities::WidgetBase

oo::class create ELEP::XML::AnnotationsToXML {
  variable tagscmd

  constructor {args} {
    set tagscmd "ELEP::XML::AnnotationsToXML getOpenCloseTags"
    my configure {*}$args
  };# constructor

  ##########################################################################
  ## Method: getXML
  ##########################################################################
  method getXML {text annotations} {
    ## Create a dummy Collection...
    set col [tip_CreateCollection $::CDM_TempDir/TempCol {}]
    set doc [tip_CreateDocument $col TempDoc $text $annotations {}]
    ## Iterate over all Annotations and select their ids...
    set ids {}
    foreach ann [tip_GetAnnotations $doc] {lappend ids [tip_GetId $ann]}
    foreach id $ids {
      set ann [tip_GetAnnotation $doc $id]
      foreach {start_tag end_tag} [my callTagsCmd $doc $ann] {break}
      set spans [llength [tip_GetSpans $ann]]
      for {set i 0} {$i < $spans} {incr i} {
        set ann [tip_GetAnnotation $doc $id]
        tip_GetSpanOffsets [lindex [tip_GetSpans $ann] $i] start end
        tip_ByteSequenceInsertString $doc $start $start_tag
        ## We have just inserted a string before the Annotation. As a result,
        ## the end is invalid...
        set ann [tip_GetAnnotation $doc $id]
        tip_GetSpanOffsets [lindex [tip_GetSpans $ann] $i] start end
        tip_ByteSequenceInsertString $doc $end $end_tag
      };# for each span
      tip_RemoveAnnotation $doc $id
    };# foreach id
    set xml [tip_GetRawData $doc]
    tip_Close $doc
    tip_Close $col
    return $xml
  };# getXML

  ##########################################################################
  ## Method: callTagsCmd
  ##########################################################################
  method callTagsCmd {doc annotation} {
    return [uplevel #0 $tagscmd $doc [list $annotation]]
  };# callTagsCmd

  ##########################################################################
  ## Procedure: getOpenCloseTags
  ##########################################################################
  classmethod getOpenCloseTags {doc annotation} {
    set type [tip_GetType $annotation]
    set tag [list $type "id=\"[tip_GetId $annotation]\""]
    foreach attribute [tip_GetAttributes $annotation] {
      lappend tag "[tip_GetName $attribute]=\"[tip_GetValueValue $attribute]\""
    }
    return [list "<[join $tag]>" "</$type>"]
  };# getOpenCloseTags

  destructor {
  };# destructor

};# class ELEP::XML::AnnotationsToXML

package provide ELEP::XML::AnnotationsToXML 1.0
