/*****************************************************************************
 *  EmbedAPI.hpp:                                                            *
 * ----------------------                                                    *
 *  This file defines an API for embeding Ellogon in a C++ application.      *
 *                                                                           *
 *  Copyright 1998-2009 by:                                                  *
 *  Georgios Petasis,                                                        *
 *  Software and Knowledge Engineering Laboratory,                           *
 *  Institute of Informatics and Telecommunications,                         *
 *  National Center for Scientific Research (N.C.S.R.) "Demokritos",         *
 *  Athens, Greece.                                                          *
 *  E-mail: petasis@iit.demokritos.gr                                        *
 *                                                                           *
 *  This file is part of the "Ellogon" Text Engineering Platform.            *
 *  For licencing information, please refer to the licence conditions of     *
 *  "Ellogon" Text Engineering Platform.                                     *
 *****************************************************************************/

#ifndef _ELLOGON_EMBED_API_HPP_
#define _ELLOGON_EMBED_API_HPP_
#include "tcl.h"

namespace ELLOGON {

class DLLEXPORT Ellogon {
public:
  Ellogon(void);
  Ellogon(const char *path_to_ellogon, /* Path to Ellogon/ellogon, UNIX style */
          int argc = 0,                /* Number of arguments */
          const char **argv = NULL);   /* Array of argument strings */
  ~Ellogon(void);

  int init(const char *path_to_ellogon,/* Path to Ellogon/ellogon, UNIX style */
          int argc = 0,                /* Number of arguments */
          const char **argv = NULL);   /* Array of argument strings */
  void destroy(void);

  int eval(const char *cmd) const;

  const char *result(void) const;
  const char *errorInfo(void) const;
  const char *variable(const char *name, const char *key = NULL) const;

  int InitializeInterpreter(Tcl_Interp *ip = NULL);
  static Tcl_Interp *InitializeTcl(Tcl_Interp *ip = NULL);

  static Tcl_ThreadId GetThreadId(void);
  
  static void debug(bool enable);
private:
  Tcl_Interp *interp;
}; /* class Ellogon */

}; /* namespace ELLOGON */

#endif /* _ELLOGON_EMBED_API_HPP_ */
