##############################################################################
##  MovePlatformBinaries.tcl:                                               ##
## ---------------------------                                              ##
## A script for moving the platform-specific shared libraries into proper   ##
## directories, for working with Ellogon...                                 ##
## Georgios Petasis, 19 Feb 2003                                            ##
##############################################################################

##
## Directories to be seached for binaries...
##
set DirectoryList {
  CDM
  CDM/Java
  CDM/Perl
  CDM/Python
  share/modules/HBrill
  share/modules/HBrillCases
  share/modules/HCFGParser
  share/modules/HGazetteer
  share/modules/HLemmatiser
  share/modules/HMorph
  share/modules/HPcPatr
  share/modules/HStemmer
  share/modules/HTokenizer
  XML/tclxml*/library
  lib/tclodbc*
  lib/Tktable*
  lib/TclControl/install
  lib/tree*/library
  lib/TclAgent
  lib/tmci*
  lib/tkdnd/lib
  lib/tdom*/tnc
  lib/tdom*
  lib/tclblend*
  lib/tclperl*
  lib/tclpython*
  lib/treectrl*
  ellogon2.0/lib/Tkhtml*
  ellogon2.0/lib/Tkzinc*
  ellogon2.0/lib/tkpath*
}

##
## Retrieve the directory of this script, as we will use it as a reference
## for getting the paths...
##
set CWD [file dirname [file dirname [file normalize [info script]]]]
cd $CWD

##
## Iterate over all specified directories...
##
set OS $tcl_platform(os)

catch {
  exec python -V
} version
set Version(Python) [lindex $version end]

##
## Are we packing a compatibility release?
##
if {![info exists env(CC)]}   {set env(CC) gcc}
if {![info exists env(CPP)]}  {set env(CPP) g++}
if {![info exists env(GCC2)]} {set env(GCC2) 0}
if {$env(CC) == "gcc296" || $env(CPP) == "g++296" || $env(GCC2)} {
  puts "@@@@@ Compatibility Release! @@@@@"
  append OS /GCC2x
}

foreach dir [lsort $DirectoryList] {
  puts "## Processing Directory \"$dir\"..."
  ## Search for DLLs...
  foreach dll [glob -nocomplain $CWD/$dir/*.dll] {
    set directory [file dirname $dll]/Windows
    set file      [file tail $dll]
    if {[string match *python* [string tolower $directory]]} {
      set directory [file dirname $dll]/$::Version(Python)/Windows
    }
    file mkdir $directory
    puts "  >> Moving DLL \"$file\" to \"$directory/$file\"..."
    file rename -force -- $dll $directory/$file
  }
  ## Search for .so...
  foreach dll [glob -nocomplain $CWD/$dir/*[info sharedlibextension]] {
    set directory [file dirname $dll]/$OS
    set file      [file tail $dll]
    if {[string match *python* [string tolower $directory]]} {
      set directory [file dirname $dll]/$::Version(Python)/$OS
    }
    file mkdir $directory
    puts "  >> Moving lib \"$file\" to \"$directory/$file\"..."
    file rename -force -- $dll $directory/$file
  }
  ## Search for .so.?.? ...
  foreach dll [glob -nocomplain $CWD/$dir/*[info sharedlibextension].*.*] {
    set directory [file dirname $dll]/$OS
    set file      [file tail $dll]
    if {[string match *python* [string tolower $directory]]} {
      set directory [file dirname $dll]/$::Version(Python)/$OS
    }
    file mkdir $directory
    puts "  >> Moving lib \"$file\" to \"$directory/$file\"..."
    file rename -force -- $dll $directory/$file
  }
  ## Search for .pm...
  foreach dll [glob -nocomplain $CWD/$dir/*.pm] {
    set directory [file dirname $dll]/$OS
    set file      [file tail $dll]
    file mkdir $directory
    puts "  >> Moving perl code \"$file\" to \"$directory/$file\"..."
    file rename -force -- $dll $directory/$file
  }
}
