wm withdraw .
## Load the TclAgent Windows extension...
load ../Windows/TclAgent.dll
cd ../Agents
set Agent {}

## Play a small conversation between Genie and Merlin...
proc conversation {} {
  ## Laod Genie & Merlin...
  set Genie  [agent create ../Agents/Genie.acs]
  set Merlin [agent create ../Agents/Merlin.acs]
  $Genie  move 0 100
  $Merlin move 200 100
  $Merlin wait [$Genie show]
  $Genie wait  [$Merlin show]
  ## Have Genie gesture towards Merlin and say hello...
  $Genie gesture 150 100
  ## Make Merlin wait for Genie to finish speaking
  $Merlin wait [$Genie speak "Hi there Merlin."]
  ## Have Merlin gesture towards Genie..
  $Genie play RestPose
  $Merlin gesture 0 100
  ## And then speak back to Genie...
  $Genie wait [$Merlin speak "Hi Genie. Well I can't believe it. We got called from Tk!"]
  $Merlin play Confused
  $Genie wait [$Merlin speak "Has Tk always supported Agents, or is it something new?"]
  $Merlin play Congratulate
  $Genie play Confused
  $Genie wait [$Merlin speak "Well, anyway, it seems that we have work to do right now, so see you later..."]
  $Merlin play Hide; $Merlin wait [$Genie play Hide]
  $Merlin destroy; $Genie destroy
}
conversation

## Create the AgentManage window...
menu .menubar
menu .menubar.file
menu .menubar.agent
. configure -menu .menubar
.menubar add cascade -label File -menu .menubar.file
.menubar add cascade -label Agent -menu .menubar.agent
set types { {{Agent Character Storage} {.acs}} {{Agent Character File} {.acf}} \
            {{Agents Preview Files} {.acg}}}
.menubar.file add command -label Open... -command \
  "LoadAgent \[tk_getOpenFile -filetypes [list $types] -initialdir [pwd]\]"
.menubar.file add command -label {Show console...} -command {console show}
.menubar.file add command -label Exit -command {catch {$Agent destroy};exit}

## Get available agents...
set agents [glob *.acs]
foreach agent [lsort $agents] {
  set name [string totitle [file root $agent]]
  .menubar.agent add command -label $name -command "LoadAgent $agent"
}

label .anim_label -text Animations
frame .anim_list
  listbox .anim_list.list -yscrollcommand {.anim_list.sc set} \
    -listvar AgentAnims -bg white
  bind .anim_list.list <Double-1> Play
  scrollbar .anim_list.sc -orient vertical -command {.anim_list.list yview}
  pack .anim_list.sc -side right -fill y
  pack .anim_list.list -side left -fill both -expand 1
button .play -text Play -command Play
button .stop -text Stop -command {catch {$Agent stop}}
frame .agent -bg white -width 200 -height 200 -relief sunken -bd 1
frame .text
  text .text.text -width 40 -height 6 -yscrollcommand {.text.sc set}
  scrollbar .text.sc -orient vertical -command {.text.text yview}
  pack .text.sc -side right -fill y
  pack .text.text -side left -fill both -expand 1
  .text.text insert end Hello!
frame .speak
  button .speak.speak -text Speak -command {Cmd speak} -width 6
  button .speak.text  -text Ballon -command {Cmd text} -width 6
  button .speak.think -text Think -command {Cmd think} -width 6
  button .speak.wisper -text Wisper -command {Cmd wisper} -width 6
  button .speak.talk -text Talk -command {Cmd talk} -width 6
  grid .speak.speak .speak.wisper .speak.talk -sticky snew -padx 10 -pady 5
  grid .speak.text .speak.think -sticky snew -padx 15 -pady 5
frame .move
  label .move.xl -text X:
  label .move.yl -text Y:
  entry .move.x -width 5
  entry .move.y -width 5
  .move.x insert end 0
  .move.y insert end 0
  button .move.move -text Move -command \
    {if {$Agent != ""} {$Agent move [.move.x get] [.move.y get]}}
  pack .move.xl .move.x .move.yl .move.y .move.move -side left -padx 40

grid .anim_label -row 0 -column 0 -sticky snew -padx 5 -pady 5
grid .anim_list -row 1 -column 0 -rowspan 2 -sticky snew -padx 5 -pady 5
grid .play -row 1 -column 1 -sticky ew -padx 5 -pady 5
grid .stop -row 2 -column 1 -sticky ew -padx 5 -pady 5
grid .agent -row 1 -column 2 -rowspan 2
grid .text -row 3 -column 0 -sticky snew -padx 5 -pady 5
grid .speak -row 3 -column 1 -sticky snew -padx 5 -pady 5 -columnspan 2
grid .move -row 4 -column 0 -sticky snew -padx 5 -pady 5 -columnspan 3

proc LoadAgent agent {
  global Agent AgentAnims

  if {![string length $agent]} return
  catch {$Agent wait [$Agent hide]}
  catch {$Agent destroy}

  set Agent [agent create $agent]
  set x [winfo rootx .agent]
  set y [winfo rooty .agent]
  incr x 30
  incr y 10
  $Agent move $x $y
  $Agent show
  .text.text delete 1.0 end
  .text.text insert end [$Agent description]
  set AgentAnims [lsort [$Agent animations]]
  $Agent bind <Click> Click
  $Agent bind <DblClick> DblClick
  $Agent bind <DragStart> DragStart
  $Agent bind <DragComplete> DragComplete
}

proc Play {} {
  global Agent

  catch {$Agent stop}
  set sel [.anim_list.list curselection]
  foreach one $sel {
    catch {$Agent play [.anim_list.list get $one]}
  }
}

proc Cmd cmd {
  global Agent

  catch {$Agent $cmd [.text.text get 1.0 end]}
}

proc Click {} {
  global Agent

  $Agent stop
  $Agent play Surprised
  $Agent speak "\\map=\"\\spd=160\\\\pit=120\\Hey! that tickles!\"=\"\"\\|\\map=\"\\spd=160\\\\pit=120\\Stop it!\"=\"\"\\|\\map=\"\\spd=160\\\\pit=120\\Will you stop that!\"=\"\"\\|\\map=\"\\spd=150\\\\pit=150\\Stop \\spd=170\\\\pit=100\\that.\"=\"\"\\|\\map=\"\\spd=160\\\\pit=120\\Come \\pit=100\\on \\pit=70\\. Stop!\"=\"\"\\"
  $Agent play Alert
  $Agent speak "\\mrk=7\\"
}
proc DblClick {} {
}
proc DragStart {} {
  global Agent

  $Agent stop
  $Agent play Surprised
  $Agent speak "\\map=\"\\spd=160\\\\pit=120\\Hey! Where are you dragging me!\"=\"\"\\|\\map=\"\\spd=160\\\\pit=120\\Hey! What are you up to!\"=\"\"\\|\\map=\"\\spd=160\\\\pit=120\\Hey! Careful when you drag me around! \"=\"\"\\"
  $Agent speak "\\mrk=7\\"
}
proc DragComplete {} {
  global Agent

  foreach {x y} [$Agent position] {}
  catch {
    .move.x delete 0 end
    .move.y delete 0 end
    .move.x insert end $x
    .move.y insert end $y
  }
  $Agent play RestPose
}

set ag C:/Windows/MSAgent/Chars
#set agent  [agent create [file join $ag Birdie.acs]]
wm deiconify .
