/* 
 * This is the TclAgent Tcl extension. It provides access to MSAgent
 * ActiveX component...
 * Author: George Petasis, petasis@iit.demokritos.gr
 * This is free software, you can do what ever you want with it!
 * Have fun!
 */

#include <windows.h>
#include <crtdbg.h>
#include "AgtSvr.h"
#include "AgtSvr_i.c"
#include "Notify.h"
#include "tcl.h"
#include "TclAgent.h"

/* This will hold the MSAgent Server instance... */
IAgentEx               *pAgentEx;
static int              TclAgentID = 1;
static AgentNotifySink *pSink = NULL;
static long             lNotifySinkID = -1;
long                    m_lCmdPropertiesID = -1;
long                    TclAgent_WaitID = -1;
/* In the following table we associate the Character id with the
 * character structure. This is needed by notification callbacks...*/
Tcl_HashTable           TclAgent_CharactersIndex;

/* This structure will contain Character related information */
typedef struct TclAgentCharacterInstance_ {
  Tcl_Interp        *interp;
  IAgentCharacterEx *pCharacterEx;  // The associated character
  long               CharID;        // Character ID
  long               RequestID;     // Request ID
  char               Cmd[12];       // Character Tcl Command
  Tcl_Obj           *Click;         // Callback for the "Click" event 
  Tcl_Obj           *DblClick;      // Callback for the "DblClick" event 
  Tcl_Obj           *DragStart;     // Callback for the "DragStart" event 
  Tcl_Obj           *DragComplete;  // Callback for the "DragComplete" event 
} TclAgent_Character;

/* Function Prototypes... */
#define AGENT_ARGS ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]
int  TclAgentObjCmd(AGENT_ARGS);
int  TclAgentInstanceObjCmd(AGENT_ARGS);
void TclAgent_Callback(int event, long dwCharID, short fwKeys, long X, long Y);

/*
 * Main Initialisation Procedure...
 */
extern "C" __declspec( dllexport ) int Tclagent_Init(Tcl_Interp *interp) {
  HRESULT hRes;
  
#ifdef USE_TCL_STUBS 
  if (Tcl_InitStubs(interp, "8.0", 0) == NULL) return TCL_ERROR;
#endif /* USE_TCL_STUBS */
  /* Initialise Hash Table... */
  Tcl_InitHashTable(&TclAgent_CharactersIndex, TCL_ONE_WORD_KEYS);
  // Initialize COM
  if (FAILED(CoInitialize(NULL))) {
    Tcl_SetResult(interp, "There was an error initializing COM.", TCL_STATIC);
    return TCL_ERROR;
  }

  // Create an instance of the Agent 2.0 server. NOTE: by
  // asking for an IAgentEx interface we know we will get
  // at least Agent 2.0. The CLSID also changed between
  // 1.5 and 2.0 so we know we won't get the 1.5 server.
  hRes = CoCreateInstance(CLSID_AgentServer, NULL, CLSCTX_SERVER, IID_IAgentEx,
                          (LPVOID *)&pAgentEx);
  if (FAILED(hRes)) {
    Tcl_SetResult(interp, "There was an error initializing Microsoft Agent",
                  TCL_STATIC);
    CoUninitialize();
    return TCL_ERROR;
  }
  // Create a notify sink
  pSink = new AgentNotifySink;
  if (pSink == NULL) {
    Tcl_SetResult(interp, "There was an error initializing Agent Notify",
                  TCL_STATIC);
    CoUninitialize();
    return TCL_ERROR;
  }
  // AddRef it...
  pSink->AddRef();
  // And register it with Microsoft Agent...
  pAgentEx->Register((IUnknown *)pSink, &lNotifySinkID);
  
  // Register the "agent" command...
  if(Tcl_CreateObjCommand(interp, "agent", TclAgentObjCmd, (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL) {
    CoUninitialize();
    return TCL_ERROR;
  }

  return TCL_OK;
} /* Tclagent_Init */

/*
 * Initialisation for safe interpreters...
 */
extern "C" __declspec( dllexport ) int Tclagent_SafeInit(Tcl_Interp *interp) {
  return Tclagent_Init(interp);
} /* Tclagent_SafeInit */

/* 
 * TclAgentObjCmd --
 *   This is the callback function for the "agent" command.
 */
int TclAgentObjCmd(AGENT_ARGS) {
  TclAgent_Character *charp;
  HRESULT hRes;
  int index;
  static char *Methods[] = {
     "create", "destroy", "names", (char *) NULL
  };
  enum methods {
     CREATE, DESTROY, NAMES
  };
  
  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
    return TCL_ERROR;
  }
  if (Tcl_GetIndexFromObj(interp,objv[1],(const char **) Methods,"method",0,&index) != TCL_OK) 
    return TCL_ERROR;
  Tcl_ResetResult(interp);
 
  /* Handle Arguments... */
  switch ((enum methods) index) {
    case CREATE: {
      VARIANT vPath;
      Tcl_DString ds, path;
      Tcl_HashEntry *entryPtr;
      IAgentCommands *pCommands;
      BSTR bszCommand;
      int new_entry;
      
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "character-file");
        return TCL_ERROR;
      }
      charp = (TclAgent_Character *) Tcl_Alloc(sizeof(TclAgent_Character));
      if (charp == NULL) {
        Tcl_SetResult(interp, "Out if memory!", TCL_STATIC);
        return TCL_OK;
      }
      charp->Click = NULL;
      charp->DblClick = NULL;
      charp->DragStart = NULL;
      charp->DragComplete = NULL;
      
      // Find Character location
      VariantInit(&vPath);
      Tcl_DStringInit(&path);
      if (Tcl_TranslateFileName(interp,Tcl_GetString(objv[2]),&path)==NULL) {
        Tcl_AppendResult(interp, "invalid path \"",
                                 Tcl_GetString(objv[2]), "\"", (char *) NULL);
        return TCL_ERROR;
      }
      if (Tcl_GetPathType(Tcl_DStringValue(&path)) == TCL_PATH_RELATIVE) {
        Tcl_DStringInit(&ds);
	Tcl_GetCwd(interp, &ds);
	Tcl_DStringAppend(&ds, "\\", 1);
	Tcl_DStringAppend(&ds, Tcl_DStringValue(&path), -1);
	Tcl_DStringFree(&path);
	Tcl_DStringInit(&path);
	Tcl_DStringAppend(&path, Tcl_DStringValue(&ds), -1);
	Tcl_DStringFree(&ds);
      }
      Tcl_DStringInit(&ds);
      vPath.bstrVal = SysAllocString(
            Tcl_UtfToUniCharDString(Tcl_DStringValue(&path), -1, &ds));
      vPath.vt = VT_BSTR;
      Tcl_DStringFree(&ds);
	
      // Create the Character instance
      hRes = pAgentEx->Load(vPath, &(charp->CharID), &(charp->RequestID));
      VariantClear(&vPath);
      if (FAILED(hRes)) {
        Tcl_AppendResult(interp, "unable to load \"", 
                         Tcl_DStringValue(&path), "\"", (char *) NULL);
        Tcl_DStringFree(&path);
	Tcl_Free((char *) charp);
        return TCL_ERROR;
      }
      Tcl_DStringFree(&path);
      // Get the IAgentCharacterEx interface
      hRes = pAgentEx->GetCharacterEx(charp->CharID, &(charp->pCharacterEx));
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Cannot get the IAgentCharacterEx interface!",
                      TCL_STATIC);
	Tcl_Free((char *) charp);
        return TCL_ERROR;
      }
      // Set the language of the character (english is a language that always
      // exists!)
      charp->pCharacterEx->SetLanguageID(MAKELANGID(LANG_ENGLISH,
                                                    SUBLANG_ENGLISH_US));
      // Add a menu entry for the character properties along with hide...
      // Query for the IAgentCommands interface
      if (SUCCEEDED(charp->pCharacterEx->QueryInterface(
           IID_IAgentCommands, (LPVOID *)&pCommands))) {
	bszCommand = SysAllocString(L"&Advanced Character Properties");
        if (bszCommand) {
          pCommands->Add(bszCommand, NULL, TRUE, TRUE, &m_lCmdPropertiesID);
          SysFreeString(bszCommand);
        }
      }

      // Register the new Character as a tcl command...
      sprintf(charp->Cmd, "agent%d", TclAgentID++);
      if (Tcl_CreateObjCommand(interp, charp->Cmd, TclAgentInstanceObjCmd,
           (ClientData) charp, (Tcl_CmdDeleteProc *) NULL)  == NULL) {
	charp->pCharacterEx->Release();
	pAgentEx->Unload(charp->CharID);
	Tcl_Free((char *) charp);
	return TCL_ERROR;
      }
      
      /* Add reference to the hash table... */
      entryPtr = Tcl_CreateHashEntry(&TclAgent_CharactersIndex,
                                     (char *) charp->CharID, &new_entry);
      if (entryPtr != NULL) {
        Tcl_SetHashValue(entryPtr, (ClientData) charp);
      }
      charp->interp = interp;
      Tcl_SetResult(interp, charp->Cmd, TCL_STATIC);
      return TCL_OK;
    }
    case DESTROY: {
    }
    case NAMES: {
    }
  }
  return TCL_OK;
} /* TclAgentObjCmd */

/* 
 * TclAgentObjCmd --
 *   This is the callback function for every character instance
 */
int TclAgentInstanceObjCmd(AGENT_ARGS) {
  TclAgent_Character *charp;
  HRESULT hRes;
  int index;
  static char *Methods[] = {
    "activate", "animations", "bind",   "description",
    "destroy",  "gesture",    "hide",   "idle",
    "move",     "name",       "play",   "position",
    "show",     "size",       "sound",  "speak",
    "stop",     "talk",       "text",   "think",
    "visible",  "wait",       "wisper",
    (char *) NULL
  };
  enum methods {
    ACTIVATE, ANIMATIONS, BIND,   DESCRIPTION,
    DESTROY,  GESTURE,    HIDE,   IDLE,
    MOVE,     NAME,       PLAY,   POSITION,
    SHOW,     SIZE,       SOUND,  SPEAK,
    STOP,     TALK,       TEXT,   THINK,
    VISIBLE,  WAIT,       WISPER
  };

  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
    return TCL_ERROR;
  }
  if (Tcl_GetIndexFromObj(interp,objv[1],(const char **) Methods,"method",0,&index) != TCL_OK) 
    return TCL_ERROR;
  Tcl_ResetResult(interp);
  charp = (TclAgent_Character *) clientData;
  if (charp == NULL) return TCL_ERROR;
	 
  /* Handle Arguments... */
  switch ((enum methods) index) {
    case ACTIVATE: {
      int activate;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "boolean");
        return TCL_ERROR;
      }
      if (Tcl_GetBooleanFromObj(interp, objv[2], &activate) != TCL_OK)
        return TCL_ERROR;
      charp->pCharacterEx->Activate((short) activate);
      return TCL_OK;
    } /* ACTIVATE */
    case ANIMATIONS: {
      Tcl_Obj      *ResObj;
      IUnknown	   *punkAnimNames;
      IEnumVARIANT *pevAnimNames;
      VARIANT       vAnimName;
      DWORD         dwRetrieved;
	
	
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      ResObj = Tcl_NewListObj(0, NULL);
      if (ResObj == NULL) return TCL_ERROR;
      // Get the IUnknown for the animation name enumerator
      if (SUCCEEDED(charp->pCharacterEx->GetAnimationNames(&punkAnimNames))) {
        // Then query for IEnumVARIANT
        if (SUCCEEDED(punkAnimNames->QueryInterface(IID_IEnumVARIANT,
                                                    (LPVOID *)&pevAnimNames))) {
          while (TRUE) {
            // Get the next animation name
            hRes = pevAnimNames->Next(1, &vAnimName, &dwRetrieved);
            // Was there one?
            if (hRes != NOERROR) break;
            // Hmmmm
            _ASSERT(dwRetrieved > 0);
            _ASSERT(vAnimName.vt == VT_BSTR);
            _ASSERT(vAnimName.bstrVal != NULL);
            // Add it to the list...
	    Tcl_ListObjAppendElement(interp, ResObj,
                Tcl_NewUnicodeObj(vAnimName.bstrVal, -1));
            // And clean up
            VariantClear(&vAnimName);
          }
          pevAnimNames->Release();
        }
        punkAnimNames->Release();
      }
      Tcl_SetObjResult(interp, ResObj);
      return TCL_OK;
    } /* ANIMATIONS */
    case BIND: {
      char *event;
      if (objc != 3 && objc != 4) {
        Tcl_WrongNumArgs(interp, 2, objv, "event ?script?");
	return TCL_ERROR;
      }
      event = Tcl_GetString(objv[2]);
      if (strcmp(event, "<Click>") == 0) {
        if (objc == 3) {
	  if (charp->Click != NULL) Tcl_SetObjResult(interp, charp->Click);
	} else {
	  if (charp->Click != NULL) Tcl_DecrRefCount(charp->Click);
	  charp->Click = objv[3]; Tcl_IncrRefCount(objv[3]);
        }
      } else if (strcmp(event, "<DblClick>") == 0) {
        if (objc == 3) {
     	  if (charp->DblClick != NULL) Tcl_SetObjResult(interp,charp->DblClick);
  	} else {
          if (charp->DblClick != NULL) Tcl_DecrRefCount(charp->DblClick);
	  charp->DblClick = objv[3]; Tcl_IncrRefCount(objv[3]);
	}
      } else if (strcmp(event, "<DragStart>") == 0) {
        if (objc == 3) {
      	  if (charp->DragStart != NULL)
             Tcl_SetObjResult(interp, charp->DragStart);
        } else {
          if (charp->DragStart != NULL) Tcl_DecrRefCount(charp->DragStart);
          charp->DragStart = objv[3]; Tcl_IncrRefCount(objv[3]);
        }
      } else if (strcmp(event, "<DragComplete>") == 0) {
        if (objc == 3) {
      	  if(charp->DragComplete != NULL)
            Tcl_SetObjResult(interp, charp->DragComplete);
  	} else {
	  if (charp->DragComplete !=NULL) Tcl_DecrRefCount(charp->DragComplete);
  	  charp->DragComplete = objv[3]; Tcl_IncrRefCount(objv[3]);
  	}
      }
      return TCL_OK;
    } /* BIND */
    case DESCRIPTION: {
      Tcl_UniChar *Des;
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->GetDescription(&Des);
      if (Des != NULL) Tcl_SetObjResult(interp, Tcl_NewUnicodeObj(Des,-1));
      return TCL_OK;
    } /* DESCRIPTION */
    case DESTROY: {
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      // Release the character interface
      charp->pCharacterEx->Release();
      // Unload the character.  NOTE:  releasing the character
      // interface does NOT make the character go away.  You must
      // call Unload.
      pAgentEx->Unload(charp->CharID);
      // Delete Character command...
      Tcl_DeleteCommand(interp, charp->Cmd);
      if (charp->Click        != NULL) Tcl_DecrRefCount(charp->Click);
      if (charp->DblClick     != NULL) Tcl_DecrRefCount(charp->DblClick);
      if (charp->DragStart    != NULL) Tcl_DecrRefCount(charp->DragStart);
      if (charp->DragComplete != NULL) Tcl_DecrRefCount(charp->DragComplete);
      Tcl_Free((char *) charp);
      return TCL_OK;
    } /* DESTROY */
    case GESTURE: {
      int x, y;
      if (objc != 4) {
        Tcl_WrongNumArgs(interp, 2, objv, "x y");
        return TCL_ERROR;
      }
      if (Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK) return TCL_ERROR;
      if (Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK) return TCL_ERROR;
      charp->pCharacterEx->StopAll(STOP_TYPE_ALL);
      charp->pCharacterEx->GestureAt((short) x, (short) y, &charp->RequestID);
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* GESTURE */
    case HIDE: {
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->Hide(FALSE, &charp->RequestID);
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* HIDE */
    case IDLE: {
      Tcl_Obj *ResObj;
      long idle;
      if (objc != 2 && objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, " ?boolean?");
        return TCL_ERROR;
      }
      if (objc == 2) { /* Return current status */
        charp->pCharacterEx->GetIdleOn(&idle);
      } else {
        if (Tcl_GetBooleanFromObj(interp, objv[2], (int *) &idle) != TCL_OK)
          return TCL_ERROR;
        charp->pCharacterEx->SetIdleOn(idle);
      }
      ResObj = Tcl_NewLongObj(idle);
      Tcl_SetObjResult(interp, ResObj);
      return TCL_OK;
    } /* IDLE */
    case SHOW: {
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->Show(FALSE, &charp->RequestID);
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* SHOW */
    case MOVE: {
      int x, y, animate = 1;
      if (objc < 4 && objc > 5) {
        Tcl_WrongNumArgs(interp, 2, objv, "left top ?animate?");
        return TCL_ERROR;
      }
      if (Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK) return TCL_ERROR;
      if (Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK) return TCL_ERROR;
      if (objc == 5) {
        if (Tcl_GetBooleanFromObj(interp, objv[4], &animate) != TCL_OK)
          return TCL_ERROR;
      }
      charp->pCharacterEx->MoveTo((short) x, (short) y, animate,
                                  &charp->RequestID);
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* MOVE */
    case NAME: {
      Tcl_UniChar *Name;
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->GetName(&Name);
      Tcl_SetObjResult(interp, Tcl_NewUnicodeObj(Name,-1));
      return TCL_OK;
    } /* NAME */
    case PLAY: {
      BSTR bszPlay;
      Tcl_DString ds;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "animation");
        return TCL_ERROR;
      }
      Tcl_DStringInit(&ds);
      bszPlay = SysAllocString(
                    Tcl_UtfToUniCharDString(Tcl_GetString(objv[2]), -1, &ds));
      charp->pCharacterEx->Play(bszPlay, &charp->RequestID);
      Tcl_DStringFree(&ds);
      SysFreeString(bszPlay);
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* PLAY */
    case POSITION: {
      Tcl_Obj *ResObj;
      long x,y;
      if (objc != 2 && objc != 4) {
        Tcl_WrongNumArgs(interp, 2, objv, "?left top?");
        return TCL_ERROR;
      }
      if (objc == 2) { /* Return the current position */
        charp->pCharacterEx->GetPosition(&x, &y);
	ResObj = Tcl_NewListObj(0, NULL);
	Tcl_ListObjAppendElement(interp, ResObj, Tcl_NewLongObj(x));
	Tcl_ListObjAppendElement(interp, ResObj, Tcl_NewLongObj(y));
	Tcl_SetObjResult(interp, ResObj);
      } else {
        if (Tcl_GetLongFromObj(interp, objv[2], &x) != TCL_OK) return TCL_ERROR;
        if (Tcl_GetLongFromObj(interp, objv[2], &y) != TCL_OK) return TCL_ERROR;
        charp->pCharacterEx->SetPosition(x, y);
      }
      return TCL_OK;
    } /* POSITION */
    case SIZE: {
      Tcl_Obj *ResObj;
      long x,y;
      if (objc != 2 && objc != 4) {
        Tcl_WrongNumArgs(interp, 2, objv, "?width length?");
        return TCL_ERROR;
      }
      if (objc == 2) { /* Return the current size */
        charp->pCharacterEx->GetSize(&x, &y);
	ResObj = Tcl_NewListObj(0, NULL);
	Tcl_ListObjAppendElement(interp, ResObj, Tcl_NewLongObj(x));
	Tcl_ListObjAppendElement(interp, ResObj, Tcl_NewLongObj(y));
	Tcl_SetObjResult(interp, ResObj);
      } else {
        if (Tcl_GetLongFromObj(interp, objv[2], &x) != TCL_OK) return TCL_ERROR;
        if (Tcl_GetLongFromObj(interp, objv[2], &y) != TCL_OK) return TCL_ERROR;
        charp->pCharacterEx->SetSize(x, y);
      }
      return TCL_OK;
    } /* SIZE */
    case SOUND: {
      Tcl_Obj *ResObj;
      long sound;
      if (objc != 2 && objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, " ?boolean?");
        return TCL_ERROR;
      }
      if (objc == 2) { /* Return current status */
        charp->pCharacterEx->GetSoundEffectsOn(&sound);
      } else {
        if (Tcl_GetBooleanFromObj(interp, objv[2], (int *) &sound) != TCL_OK)
          return TCL_ERROR;
        charp->pCharacterEx->SetSoundEffectsOn(sound);
      }
      ResObj = Tcl_NewLongObj(sound);
      Tcl_SetObjResult(interp, ResObj);
      return TCL_OK;
    } /* SOUND */
    case SPEAK: {
      BSTR bszSpeak;
      Tcl_DString ds;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "phrase");
        return TCL_ERROR;
      }
      Tcl_DStringInit(&ds);
      bszSpeak = SysAllocString(
                    Tcl_UtfToUniCharDString(Tcl_GetString(objv[2]), -1, &ds));
      hRes = charp->pCharacterEx->Speak(bszSpeak, NULL, &charp->RequestID);
      Tcl_DStringFree(&ds);
      SysFreeString(bszSpeak);
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Agent failed to speak!", TCL_STATIC);
	return TCL_ERROR;
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* SPEAK */
    case STOP: {
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->StopAll(STOP_TYPE_ALL);
      return TCL_OK;
    } /* STOP */
    case TALK: {
      BSTR bszSpeak;
      Tcl_Obj *text;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "phrase");
        return TCL_ERROR;
      }
      text = Tcl_NewStringObj("\\map=\"", 6);
      if (text == NULL) return TCL_ERROR;
      Tcl_AppendObjToObj(text, objv[2]);
      Tcl_AppendToObj(text, "\"=\"\"\\", 5);
      bszSpeak = SysAllocString(Tcl_GetUnicode(text));
      hRes = charp->pCharacterEx->Speak(bszSpeak, NULL, &charp->RequestID);
      Tcl_DecrRefCount(text);
      SysFreeString(bszSpeak);
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Agent failed to talk!", TCL_STATIC);
        return TCL_ERROR;
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* TALK */
    case TEXT: {
      BSTR bszSpeak;
      Tcl_Obj *text;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "phrase");
        return TCL_ERROR;
      }
      text = Tcl_NewStringObj("\\map=\"\"=\"", 9);
      if (text == NULL) return TCL_ERROR;
      Tcl_AppendObjToObj(text, objv[2]);
      Tcl_AppendToObj(text, "\"\\", 2);
      bszSpeak = SysAllocString(Tcl_GetUnicode(text));
      hRes = charp->pCharacterEx->Speak(bszSpeak, NULL, &charp->RequestID);
      Tcl_DecrRefCount(text);
      SysFreeString(bszSpeak);
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Agent failed to display text!", TCL_STATIC);
        return TCL_ERROR;
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* TEXT */
    case THINK: {
      BSTR bszSpeak;
      Tcl_DString ds;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "phrase");
        return TCL_ERROR;
      }
      Tcl_DStringInit(&ds);
      bszSpeak = SysAllocString(
                 Tcl_UtfToUniCharDString(Tcl_GetString(objv[2]), -1, &ds));
      hRes = charp->pCharacterEx->Think(bszSpeak, &charp->RequestID);
      Tcl_DStringFree(&ds);
      SysFreeString(bszSpeak);
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Agent failed to think!", TCL_STATIC);
     	return TCL_ERROR;
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* THINK */
    case VISIBLE: {
      Tcl_Obj *ResObj;
      long visible;
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      charp->pCharacterEx->ShowPopupMenu(0,0);
      charp->pCharacterEx->GetVisible(&visible);
      ResObj = Tcl_NewLongObj(visible);
      Tcl_SetObjResult(interp, ResObj);
      return TCL_OK;
    } /* VISIBLE */
    case WAIT: {
      long wait;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "job-id");
        return TCL_ERROR;
      }
      if (Tcl_GetLongFromObj(interp, objv[2], &wait) != TCL_OK)
        return TCL_ERROR;
      charp->pCharacterEx->Wait(wait, &charp->RequestID);
      Tcl_SetObjResult(interp, objv[2]);
      /* Wait notification from Agent Server that all requests have
       * been processed by the agents...*/
      TclAgent_WaitID = charp->RequestID;
      while (TclAgent_WaitID > 0) {
        Tcl_DoOneEvent(0);
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* WAIT */
    case WISPER: {
      BSTR bszSpeak;
      Tcl_Obj *text;
      if (objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "phrase");
        return TCL_ERROR;
      }
      text = Tcl_NewStringObj("\\Chr=\"Whisper\"\\", 15);
      if (text == NULL) return TCL_ERROR;
      Tcl_AppendObjToObj(text, objv[2]);
      bszSpeak = SysAllocString(Tcl_GetUnicode(text));
      hRes = charp->pCharacterEx->Speak(bszSpeak, NULL, &charp->RequestID);
      Tcl_DecrRefCount(text);
      SysFreeString(bszSpeak);
      if (FAILED(hRes)) {
        Tcl_SetResult(interp, "Agent failed to wisper!", TCL_STATIC);
        return TCL_ERROR;
      }
      // Return the job id...
      Tcl_SetObjResult(interp, Tcl_NewLongObj(charp->RequestID));
      return TCL_OK;
    } /* WISPER */
  }
  return TCL_OK;
} /* TclAgentInstanceObjCmd */

/*
 * This function handles events captured by the Agent server notification
 * class...
 */
void TclAgent_Callback(int event, long dwCharID, short fwKeys, long X, long Y) {
  Tcl_HashEntry *entryPtr;
  TclAgent_Character *charp;
  Tcl_Obj *Cmd;

  entryPtr = Tcl_FindHashEntry(&TclAgent_CharactersIndex, (char *) dwCharID);
  if (entryPtr == NULL) return;

  charp = (TclAgent_Character *) Tcl_GetHashValue(entryPtr);

  switch (event) {
    case TCL_AGENT_CLICK: {
      Cmd = charp->Click;
      break;
    }
    case TCL_AGENT_DBLCLICK: {
      Cmd = charp->DblClick;
      break;
    }
    case TCL_AGENT_DRAGSTART: {
      Cmd = charp->DragStart;
      break;
    }
    case TCL_AGENT_DRAGCOMPLETE: {
      Cmd = charp->DragComplete;
      break;
    }
  }
  if (Cmd == NULL) return;

  /*
   * Execute the Callback...
   */
  Tcl_EvalObjEx(charp->interp, Cmd, TCL_EVAL_GLOBAL);
  return;
}
/*
 * EOF
 */
