// TclClassFactory.cpp: implementation of the CTclClassFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TclControlPrj2.h"
#include "dlldatax.h"
#include "TclControl.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTclClassFactory::CTclClassFactory() : 
CComClassFactory(), m_hT1Ready(NULL), m_hT2Ready(NULL),
 m_iid(IID_IUnknown), m_pMarshallStm(NULL), m_pu(NULL)
{
	m_hT1Ready = CreateEvent (NULL, FALSE, FALSE, NULL);
	m_hT2Ready = CreateEvent (NULL, FALSE, FALSE, NULL);
}

CTclClassFactory::~CTclClassFactory()
{
	if (m_hT1Ready != NULL) {
		CloseHandle (m_hT1Ready);
		m_hT1Ready = NULL;
	}
	if (m_hT2Ready != NULL) {
		CloseHandle (m_hT2Ready);
		m_hT2Ready = NULL;
	}
}





HRESULT CTclClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, void ** ppvObject)
{
	if ((pUnkOuter != NULL) && !InlineIsEqualUnknown(riid))
	{
		ATLTRACE2(atlTraceCOM, 0, _T("CComClassFactory: asked for non IUnknown interface while creating an aggregated object"));
		m_hres = CLASS_E_NOAGGREGATION;
	}
	else
	{
		m_pUnkOuter = pUnkOuter;
		m_iid = riid;

		DWORD threadid;
		HANDLE hThread;
		hThread = CreateThread (NULL, NULL, CTclClassFactory::TclThreadProc, this, NULL, &threadid);
		if (hThread != NULL)
		{
			if (WaitForSingleObject (m_hT2Ready, INFINITE) == WAIT_OBJECT_0)
			{
				
				if (m_pMarshallStm != NULL) // marshall the interface pointer to this thread
				{
					m_hres = CoGetInterfaceAndReleaseStream (m_pMarshallStm, riid, ppvObject);
					SetEvent (m_hT1Ready);

					if (FAILED (m_hres))
					{
						if (WaitForSingleObject (hThread, 5000) == WAIT_OBJECT_0)
							TerminateThread (hThread, 0);
					}

				}
				/*
				if (m_pu != NULL)
					*ppvObject = m_pu;
				*/

				else 
				{
					// unable to create object ...
					// don't do anything as the error is in m_hres already.
				}
			}
			else // thread didn't respond after being created
			{
				TerminateThread (hThread, 0);
				m_hres = E_FAIL;
			}
			BOOL battach = AttachThreadInput (threadid, GetCurrentThreadId(), TRUE);
			CloseHandle (hThread);
		}
		else // unable to create thread
			m_hres = E_FAIL;
	}
	return m_hres;
}



DWORD WINAPI CTclClassFactory::TclThreadProc (LPVOID lpstruct)
{
	CTclClassFactory *pcf = (CTclClassFactory*)lpstruct;
	ATLASSERT (pcf != NULL);
	MSG msg;

	OleInitialize(NULL);
	IUnknown * pv = NULL;
	pcf->m_hres = (pcf->m_pfnCreateInstance) (pcf->m_pUnkOuter, pcf->m_iid, (void**)&pv);
	if (SUCCEEDED(pcf->m_hres))
	{
		// we've created the object, so now create a stream to marshall the 
		// the interface
		ATLASSERT (pv != NULL);
		pcf->m_hres = CoMarshalInterThreadInterfaceInStream (pcf->m_iid, pv, &(pcf->m_pMarshallStm));
		
	}

	SetEvent (pcf->m_hT2Ready);
	WaitForSingleObject (pcf->m_hT1Ready, INFINITE);

	// interface has been marshalled safetly, so now release our stranglehold on
	// the object if it exists
	
	if (pv != NULL)
	{
		pv->Release();
		pv = NULL;
	}
	

 	if (SUCCEEDED(pcf->m_hres))
	{
		bool bCreated = false;
		int totalwin = 0;

		// the following line ensures that a message queue exists for this thread
		PeekMessage (&msg, NULL, NULL, NULL, PM_NOREMOVE);
		
		// assume that the Tcl interpreter for the object has been intialised.
		while (!bCreated) {
			Tcl_DoOneEvent(0);
			bCreated = Tk_GetNumMainWindows() > 0;
		}

		while (Tk_GetNumMainWindows() > 0)
			Tcl_DoOneEvent(0);
	}

	else // something went wrong in the process of either creation or marshalling
	{
	}


	OleUninitialize();
	return 0;
}