// TclControl.cpp : Implementation of CTclControl
// Author: Farzad Pezeshkpour
// Date: 13 April 1999

#include "stdafx.h"
#include "TclControlPrj2.h"
#include "TclControl.h"


WCHAR* tcldead= L"Tcl interpreter is dead";
bool bInit = false;

/////////////////////////////////////////////////////////////////////////////
// CTclControl

/*
 *-------------------------------------------------------------------------
 * CTclControl --
 *	Constructor - Creates the interpreter for this application and 
 *	initialise it. Sets member variables to default.
 * Result:
 *	
 * Side effects:
 *	None
 *-------------------------------------------------------------------------
 */
CTclControl::CTclControl()
{
	USES_CONVERSION;
	m_hWndTk = NULL;
	m_bWindowOnly = true;
	m_tkWnd = NULL;
	m_pInterp = NULL;
	m_error = "Tcl/Tk not initialised";
	m_pInterp = Tcl_CreateInterp();
}



/*
 *-------------------------------------------------------------------------
 * ~CTclControl --
 *	Destructor - Ensures that allocated resources are released
 * Result:
 *	
 * Side effects:
 *	Interpreter released
 *-------------------------------------------------------------------------
 */
CTclControl::~CTclControl ()
{
	if (m_pInterp != NULL)
	{
		Tcl_DeleteInterp (m_pInterp);
		m_pInterp = NULL;
	}
}

/*
 *-------------------------------------------------------------------------
 * CreateControlWindow --
 *	Virtual function override. Creates control window, and initialises
 *	Tk environement
 * Result:
 *	NULL if failed to create base window
 * Side effects:
 *	None
 *-------------------------------------------------------------------------
 */
HWND CTclControl::CreateControlWindow( HWND hWndParent, RECT& rcPos )
{
	USES_CONVERSION;
	if (m_pInterp == NULL) 
		m_pInterp = Tcl_CreateInterp();

	ATLASSERT (m_hWndTk == NULL);
	HWND hWnd = CComControl<CTclControl>::CreateControlWindow(hWndParent, rcPos);

    /* First try to initialise only tcl. The control may not have been associated with a
	   window... */
	if (m_pInterp != NULL) {
		const char * szCmdLine = T2A(GetCommandLine());
		Tcl_SetVar (m_pInterp, "argv0", (char*)(szCmdLine), TCL_GLOBAL_ONLY);
        if (Tcl_Init (m_pInterp) == TCL_ERROR) {
			char * strerr = NULL;
			if (m_pInterp != NULL && (strerr = Tcl_GetStringResult (m_pInterp)) != NULL)
				m_error = strerr;
			Tcl_DeleteInterp (m_pInterp);
			m_pInterp = NULL;
		} else {
			Tcl_Eval (m_pInterp, "proc exit {args} {}");
			Tcl_SetVar (m_pInterp, "argv", "", TCL_GLOBAL_ONLY);
		}
	}
	
	/*
	 * If tcl is initialised and we have a window handle, try to also initialise Tk...
	 */
	if (m_pInterp != NULL && hWnd != NULL) {
		char buffer[255];
		sprintf(buffer, "-use 0x%8.8x", hWnd);
		Tcl_SetVar (m_pInterp, "argv", buffer, TCL_GLOBAL_ONLY);
	}

	if (Tk_Init(m_pInterp) != TCL_ERROR) {
	    Tcl_StaticPackage(NULL, "Tk", Tk_Init, Tk_SafeInit);
		m_tkWnd = Tk_MainWindow (m_pInterp);
		// this is a bit naughty ... loop on the message queue until 
		// the tk window is initialised, other wise, Tk barfs <<badly>>!
		MSG msg;
		while (m_hWndTk == NULL && GetMessage (&msg, NULL, 0, 0)) 
			DispatchMessage(&msg);
	}
	return hWnd;
}




/*
 *-------------------------------------------------------------------------
 * OnTkClaimFocus --
 *	Message Handler for TK_CLAIMFOCUS - this is probably not implemented
 *	correctly, because focus handling doesn't currently work
 * Result:
 *	0 - I think this means that it was handled
 * Side effects:
 *	Focus changed to toplevel
 *-------------------------------------------------------------------------
 */
LRESULT CTclControl::OnTkClaimFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	bHandled = TRUE;
	if (wParam!=NULL || (::GetFocus() != NULL)) {
		::SetFocus(m_hWndTk);
	}
	return 0;
}

/*
 *-------------------------------------------------------------------------
 * OnParentNotify --
 *	Manages focus according to mouse events
 * Result:
 *	0 to indicate processed
 * Side effects:
 *	Focus changes to Tk window - this probably is not the right way to do
 *	this
 *-------------------------------------------------------------------------
 */
LRESULT CTclControl::OnParentNotify(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	switch (LOWORD(wParam)) {
	case WM_LBUTTONDOWN:
	case WM_MBUTTONDOWN:
	case WM_RBUTTONDOWN:
		::SetFocus(m_hWndTk);
		break;
	}
	return 0;
}


/*
 *-------------------------------------------------------------------------
 * OnTkAttach --
 *	Called by Tk to inform the base window of this control (its container)
 *	that it is attaching. This procedure store the window handle of the
 *	the toplevel and resizes the Tk window to fit in the base window
 * Result:
 *	0 to indicate processed (I think)
 * Side effects:
 *	Tk toplevel resized to that of the base window
 *-------------------------------------------------------------------------
 */
LRESULT CTclControl::OnTkAttach(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	m_hWndTk = (HWND)wParam;
	RECT rect;
	GetClientRect(&rect);
	return	OnSize (WM_SIZE, NULL, MAKELPARAM(rect.right, rect.bottom), bHandled);
}


/*
 *-------------------------------------------------------------------------
 * OnTkGeomReq --
 *	Called by Tk toplevel to request a geometry change - basically we'll
 *	give it only one size; that of the control base window
 * Result:
 *	1 - maybe?
 * Side effects:
 *	Tk toplevel resized
 *-------------------------------------------------------------------------
 */
LRESULT CTclControl::OnTkGeomReq(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	RECT rect;
	if (m_hWndTk != NULL) {
		GetClientRect(&rect);
		::SetWindowPos(m_hWndTk, NULL,
			0, 0, rect.right, rect.bottom, SWP_NOZORDER);
	}
	return 1;
}



/*
 *-------------------------------------------------------------------------
 * OnSize --
 *	Called to update the size of the Tk toplevel according to the (new) size
 *	of the controls base window
 * Result:
 *	0
 * Side effects:
 *
 *-------------------------------------------------------------------------
 */
LRESULT CTclControl::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if (m_tkWnd != NULL)
		Tk_GeometryRequest(m_tkWnd, LOWORD(lParam), HIWORD(lParam));
	return 0;
}

/*
 *-------------------------------------------------------------------------
 * Eval --
 *	Evaluates a command in the interpreter.
 * Result:
 *	pbOk set to -1 iff script evaluated okay; else 0. Returns S_OK iff
 *	interpreter exists.
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::Eval(BSTR command, VARIANT_BOOL *pbOk)
{
	USES_CONVERSION;

	char *szc = NULL;
	int result;

	if (m_pInterp==NULL) {
		Error(A2COLE("tcl interp is dead"));
	}
	else {
		szc = OLE2A(command);
		result = Tcl_Eval (m_pInterp, szc);
		if (pbOk != NULL)
			*pbOk = ((result == TCL_OK)?-1:0);
	}

	return S_OK;
}

/*
 *-------------------------------------------------------------------------
 * get_Result --
 *	Returns the current result of the interpreter in pVal.
 * Result:
 *	S_OK iff interpreter exists.
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::get_Result(BSTR *pVal)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else if (pVal != NULL) {
		CComBSTR str(A2OLE(Tcl_GetStringResult (m_pInterp)));
		*pVal = str.Detach();
		hr = S_OK;
	}

	return hr;
}

/*
 *-------------------------------------------------------------------------
 * TraceVar --
 *	Implements a trace on a Tcl variable, parameterised with a set of flags.
 *	Please read Tcl_TraceVar page in the Tcl help.
 * Result:
 *	*pbOk == -1 iff successful; else 0
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::TraceVar(BSTR name, long flags, VARIANT_BOOL *pbOk)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		int result = Tcl_TraceVar (m_pInterp, OLE2A(name), flags, VarCallback, (ClientData)this);
		if (pbOk != NULL)
			*pbOk = ((result == TCL_OK)?-1:0);
		hr = S_OK;
	}

	return hr;
}

/*
 *-------------------------------------------------------------------------
 * TraceVar2 --
 *	Implements a trace on a Tcl array element, parameterised with a set of
 *	flags. Please read Tcl_TraceVar2 page in the Tcl help.
 * Result:
 *	*pbOk == -1 iff trace was set okay; else 0.
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::TraceVar2(BSTR name1, BSTR name2, long flags, VARIANT_BOOL *pbOk)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		int result = Tcl_TraceVar2 (m_pInterp, OLE2A(name1), OLE2A(name2), flags, VarCallback, (ClientData)this);
		if (pbOk != NULL)
			*pbOk = ((result == TCL_OK)?-1:0);
		hr = S_OK;
	}

	return hr;
}


/*
 *-------------------------------------------------------------------------
 * VarCallback --
 *	Callback from Tcl when a trace has been invoked due to a change
 *	in a variable's state.
 * Result:
 *	NULL
 * Side effects:
 *	Invokes an appropriate COM event from this object.
 *-------------------------------------------------------------------------
 */
char *CTclControl::VarCallback (ClientData cd, Tcl_Interp *pInterp, char *name1, char *name2, int flags)
{
	USES_CONVERSION;
	CTclControl *ptc = reinterpret_cast<CTclControl*>(cd);
	ATLASSERT (ptc != NULL);
	if (ptc == NULL) return NULL;

	if (name2 == NULL) {
		ptc->Fire_OnTraceVar (A2BSTR(name1), flags);
	} else {
		ptc->Fire_OnTraceVar2 (A2BSTR(name1), A2BSTR(name2), flags);
	}
	return NULL;
}

/*
 *-------------------------------------------------------------------------
 * UntraceVar --
 *	Removes an existing trace on a Tcl variable.
 * Result:
 *	S_OK iff interpreter exists.
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::UntraceVar(BSTR name, long flags)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;
	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		Tcl_UntraceVar (m_pInterp, OLE2A(name), flags, VarCallback, (ClientData)this);
		hr = S_OK;
	}
	return hr;
}


/*
 *-------------------------------------------------------------------------
 * UntraceVar2 --
 *	Allows for the removal of an existing trace on a array element
 * Result:
 *	S_OK iff interpreter exists.
 * Side effects:
 *	None.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::UntraceVar2(BSTR name1, BSTR name2, long flags)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		Tcl_UntraceVar2 (m_pInterp, OLE2A(name1), OLE2A(name2), flags, VarCallback, (ClientData)this);
		hr = S_OK;
	}

	return hr;
}


/*
 *-------------------------------------------------------------------------
 * SetVar --
 *	Sets a Tcl Variable
 * Result:
 *	S_OK iff interpreter exists. *pbOk will be true iff set was successful
 * Side effects:
 *	Could invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::SetVar(BSTR name, BSTR value, long flags, VARIANT_BOOL *pbOk)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		char *result = Tcl_SetVar (m_pInterp, OLE2A(name), OLE2A(value), flags);
		if (pbOk != NULL)
			*pbOk = ((result != NULL)?-1:0);
		hr = S_OK;
	}

	return hr;
}

/*
 *-------------------------------------------------------------------------
 * GetVar --
 *	Gets the value of an existing Tcl variable
 * Result:
 *	S_OK iff interpreter and variable exist and variable is not an array.
 * Side effects:
 *	Can invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::GetVar(BSTR name, long flags, BSTR *pValue)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		char *result = Tcl_GetVar (m_pInterp, OLE2A(name), flags);
		if (result != NULL && pValue != NULL) {
			CComBSTR val(A2OLE(result));
			*pValue = val.Detach();
			hr = S_OK;
		}
	}
	return hr;
}



/*
 *-------------------------------------------------------------------------
 * SetVar2 --
 *	Sets the value of an array element.
 * Result:
 *	S_OK iff interpreter exists. *pbOk is true iff set operation succeeded.
 * Side effects:
 *	Could invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::SetVar2(BSTR name1, BSTR name2, BSTR value, long flags, VARIANT_BOOL *pbOk)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		char *result = Tcl_SetVar2 (m_pInterp, OLE2A(name1), OLE2A(name2), OLE2A(value), flags);
		if (pbOk != NULL)
			*pbOk = ((result != NULL)?-1:0);
		hr = S_OK;
	}
	return hr;
}



/*
 *-------------------------------------------------------------------------
 * GetVar2 --
 *	Gets the value of an existing array element.
 * Result:
 *	S_OK iff interpreter exists, and array element exists.
 * Side effects:
 *	Can invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::GetVar2(BSTR name1, BSTR name2, long flags, BSTR *pVal)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		char *result = Tcl_GetVar2 (m_pInterp, OLE2A(name1), OLE2A(name2), flags);
		if (result != NULL && pVal != NULL) {
			CComBSTR val(A2OLE(result));
			*pVal = val.Detach();
			hr = S_OK;
		}
	}
	return hr;
}


/*
 *-------------------------------------------------------------------------
 * UnsetVar --
 *	Unsets a Tcl variable
 * Result:
 *	S_OK if interpreter exists.
 * Side effects:
 *	Can invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::UnsetVar(BSTR name, long flags)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		Tcl_UnsetVar (m_pInterp, OLE2A(name), flags);
		hr = S_OK;
	}
	return hr;
}

/*
 *-------------------------------------------------------------------------
 * UnsetVar2 --
 *	Unsets a Tcl array element.
 * Result:
 *	S_OK iff interpreter exists.
 * Side effects:
 *	Can invoke a variable trace.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::UnsetVar2(BSTR name1, BSTR name2, long flags)
{
	USES_CONVERSION;
	HRESULT hr = E_FAIL;

	if (m_pInterp==NULL)
		Error(tcldead);
	else {
		Tcl_UnsetVar2 (m_pInterp, OLE2A(name1), OLE2A(name2), flags);
		hr = S_OK;
	}
	return hr;
}

/*
 *-------------------------------------------------------------------------
 * LoadTk --
 *	Loads Tk if not loaded.
 * Result:
 *	S_OK if interpreter exists.
 * Side effects:
 *	Tk gets initialised.
 *-------------------------------------------------------------------------
 */
STDMETHODIMP CTclControl::LoadTk(void)
{
	USES_CONVERSION;
	if (m_pInterp == NULL) 
		m_pInterp = Tcl_CreateInterp();

	ATLASSERT (m_hWndTk == NULL);

    /* First try to initialise only tcl. The control may not have been associated with a
	   window... */
	if (m_pInterp != NULL) {
		const char * szCmdLine = T2A(GetCommandLine());
		Tcl_SetVar (m_pInterp, "argv0", (char*)(szCmdLine), TCL_GLOBAL_ONLY);
        if (Tcl_Init (m_pInterp) == TCL_ERROR) {
			char * strerr = NULL;
			if (m_pInterp != NULL && (strerr = Tcl_GetStringResult (m_pInterp)) != NULL)
				m_error = strerr;
			Tcl_DeleteInterp (m_pInterp);
			m_pInterp = NULL;
		} else {
			Tcl_Eval (m_pInterp, "proc exit {args} {}");
			Tcl_SetVar (m_pInterp, "argv", "", TCL_GLOBAL_ONLY);
		}
	}
	
	if (Tk_Init(m_pInterp) != TCL_ERROR) {
	    Tcl_StaticPackage(NULL, "Tk", Tk_Init, Tk_SafeInit);
		m_tkWnd = Tk_MainWindow (m_pInterp);
	}
	return S_OK;
}


STDMETHODIMP CTclControl::put_MyProp(long index, float value)
{
	return S_OK;
}


HRESULT CTclControl::OnDraw(ATL_DRAWINFO& di)
{
	USES_CONVERSION;
	RECT rc = *(RECT*)di.prcBounds;
	Rectangle(di.hdcDraw, rc.left, rc.top, rc.right, rc.bottom);
	InflateRect (&rc, -1, -1);
	SetTextAlign(di.hdcDraw, TA_LEFT|TA_TOP);
	LPCTSTR pszText = A2T((char*)m_error.c_str());
	//ExtTextOut (di.hdcDraw, 0, 0, NULL, &rc, pszText, lstrlen(pszText), NULL);
	DrawText (di.hdcDraw, pszText, lstrlen(pszText), &rc, DT_LEFT|DT_TOP|DT_WORDBREAK);
	return S_OK;
}


STDMETHODIMP CTclControl::InPlaceDeactivate()
{
	if (m_pInterp != NULL)
	{
		Tcl_DeleteInterp (m_pInterp);
		m_pInterp = NULL;
		m_hWndTk = NULL;
		m_tkWnd = NULL;
	}
	return IOleInPlaceObjectWindowlessImpl<CTclControl>::InPlaceDeactivate();
}