package ifneeded tclperl 3.2 [list ::perl::tclperl_load [file normalize $dir]]

namespace eval ::perl {
  proc tclperl_load {dir} {
    package require platform
    switch -glob [platform::generic] {
      *_64    {set bits x86_64}
      default {set bits x86_32}
    }
    switch -- $::tcl_platform(platform) {
      unix    {tclperl_load_unix    $dir $bits}
      windows {tclperl_load_windows $dir $bits}
      default {error "Unsupported platform: \"$::tcl_platform(platform)\""}
    }
  }

  proc tclperl_load_unix {dir bits} {
    set os $::tcl_platform(os)
    set lib [info sharedlibextension]
    set error "$dir/$os/$bits/libtclperl-3.2-*$lib"
    puts ===========>$error
    foreach dll [lsort -dictionary -decreasing [concat \
         [glob -nocomplain $dir/$os/$bits/libtclperl-3.2-*$lib] \
         [glob -nocomplain $dir/$os/libtclperl-3.2-*$lib] \
         [glob -nocomplain $dir/libtclperl-3.2-*$lib]]]  {
      if {![catch {CDM_load $dll Tclperl} error]} {
        return
      }
    }
    error $error
  };# tclperl_load_unix

  ## Try to locate all available DLLs, and try to load all of them, until
  ## a DLL gets loaded.
  proc tclperl_load_windows {dir bits} {
    global env
    ## Try to locate Perl installation path from registry.
    package require registry
    set current_version [registry get \
      {HKEY_LOCAL_MACHINE\SOFTWARE\ActiveState\ActivePerl} CurrentVersion]
    set install_dir [registry get \
      "HKEY_LOCAL_MACHINE\\SOFTWARE\\ActiveState\\ActivePerl\\$current_version"\
      {}]
    set perl.dll_path [file native $install_dir\\bin]
    set oldPath $env(PATH)
    set env(PATH) "$env(PATH);${perl.dll_path}"
    foreach dll [lsort -dictionary -decreasing [concat \
         [glob -nocomplain $dir/Windows/$bits/libtclperl-3.2-*.dll] \
         [glob -nocomplain $dir/Windows/libtclperl-3.2-*.dll] \
         [glob -nocomplain $dir/libtclperl-3.2-*.dll]]]  {
      if {![catch {CDM_load $dll Tclperl} error]} {
        set env(PATH) $oldPath
        return
      }
    }
    set env(PATH) $oldPath
    error $error
  };# tclperl_load_windows
};# namespace ::perl
