##
## Are we packing a compatibility release?
##
if {![info exists env(CC)]}   {set env(CC) gcc}
if {![info exists env(C++)]}  {set env(C++) g++}
if {![info exists env(GCC2)]} {set env(GCC2) 0}
if {$env(CC) == "gcc296" || $env(C++) == "g++296" || $env(GCC2)} {
  puts "@@@@@ Compatibility Release! @@@@@"
  append tcl_platform(os) /GCC2x
}

set Home [pwd]
catch {cd [file join lib tkdnd]}
## This script is a wrapper, for calling TkHtml's configure script. We need to
## pass the following arguments to configure:
##  --x-includes
##  --with-tcl
##  --with-tk
##  --with-tclinclude 
##  --with-tkinclude
switch $::tcl_platform(os) {
  Darwin {
    set version           [info tclversion]
    set --with-tcl        /Library/Frameworks/Tcl.framework/Versions/$version
    set --with-tk         /Library/Frameworks/Tk.framework/Versions/$version
    set --x-includes      /usr/X11R6/include
    set --with-tclinclude ${--with-tcl}/Headers
    set --with-tkinclude  ${--with-tk}/Headers
    set --x-libraries     /usr/X11R6/lib
  }
  default {
    ## Probably this is a unix system. We assume that both the tclConfig.sh &
    ## the tkConfig.sh files are in the same directory, the parent directory
    ## of the [info library].
    set --with-tcl        [info library]
    if {![file exists ${--with-tcl}/tclConfig.sh]} {
      set --with-tcl [file dirname ${--with-tcl}]
    }
    if {[info exists ::tk_library]} {
      set --with-tk $::tk_library
    } else {
      set --with-tk [info library]
    }
    if {![file exists ${--with-tk}/tkConfig.sh]} {
      set --with-tk [file dirname ${--with-tk}]
    }
    if {[file exists ${--with-tk}/tk[info tclversion]/tkConfig.sh]} {
      set --with-tk ${--with-tk}/tk[info tclversion]
    }

    set --x-includes      /usr/X11R6/include
    set --with-tclinclude [file dirname [file dirname [info library]]]/include
    if {[file exists ${--with-tclinclude}/tcl[info tclversion]/tcl.h]} {
      set --with-tclinclude ${--with-tclinclude}/tcl[info tclversion]
    }
    set --with-tkinclude  [file dirname [file dirname [info library]]]/include
    if {[file exists ${--with-tkinclude}/tk[info tclversion]/tk.h]} {
      set --with-tkinclude ${--with-tkinclude}/tk[info tclversion]
    }
    if {[file exists ${--with-tkinclude}/tcl[info tclversion]/tk.h]} {
      set --with-tkinclude ${--with-tkinclude}/tcl[info tclversion]
    }
    set --x-libraries     /usr/X11R6/lib
  }
}
catch {file delete -force config.cache}
set env(TCLTK_ROOT)     ${--with-tcl}
set env(TCL_CONFIG_DIR) ${--with-tcl}

puts "+++ Running 'configure' in directory [pwd]..."
catch {exec sh ./configure --enable-gcc\
       --with-tcl=${--with-tcl} --with-tclinclude=${--with-tclinclude}\
       --with-tk=${--with-tk} --with-tkinclude=${--with-tkinclude} \
       >&@ stdout}
puts "    +++ 'configure' completed successfully..."
puts "+++ Running 'make' in directory [pwd]..."
if {[catch {exec make >&@ stdout} error]} {
  puts "    +++ 'make' failed:$error"
} else {
  puts "    +++ 'make' completed successfully..."
}
puts "*** Saving libraries..."
file mkdir lib
foreach file [concat [glob *libtkdnd*] [glob *pkgIndex*.tcl] \
                     [glob library/*.tcl]] {
  file copy -force $file lib
}
puts "+++ Removing unwanted files..."
catch {exec make distclean}
exit 0
