##############################################################################
## This file contains code that tries to export data to Microsoft
## Excel, through DDE...
##  Georgios Petasis, 12/02/2001
##############################################################################

## This function is a wrapper that tries to use the proper export way,
## according to platform...
proc C45SuiteExport {ResultsFile} {
  global tcl_platform

  switch $tcl_platform(platform) {
    windows {C45SuiteExportExcel $ResultsFile}
    unix    {gut_info "Operation not supported under unix!"}
    default {}
  }
};# C45SuiteExport

proc C45SuiteExportExcel {ResultsFileName} {
  global env tcl_platform
  package require dde
  
  ## Create a new, empty Excel file...
  set ExcelFileName [file rootname $ResultsFileName].xls
  set Topic         [file rootname [file tail $ResultsFileName]]
  set EF [open $ExcelFileName w]
  close $EF
  ## Start Excel...
  switch $tcl_platform(os) {
    {Windows 95} {set Cmd [list start $ExcelFileName &]}
    {Windows NT} {set Cmd [list $env(COMSPEC) /c $ExcelFileName &]}
    default {return}
  }
  eval exec $Cmd
  ## Wait until Excel starts...
  creole_C45SuiteExcelWait 7000
  
  ## Process the Results file...
  set RF [open $ResultsFileName] 
  set i 0
  while {![eof $RF]} {
    gets $RF line
    if {[eof $RF]} {break}
    if {![string length $line]} {continue}
    ## Parse line...
    foreach {m c size accuracy} $line {break}
    set Size([lindex [split $size =] end]) $i
    set M($i)        [lindex [split $m        =] end]
    set C($i)        [lindex [split $c        =] end]
    set Accuracy($i) [lindex [split $accuracy =] end]
    ## Get the various Classes...
    foreach {class_name recall precision} [lrange $line 4 end] {
      lappend Class($i)     [string range $class_name 0  end-1]
      lappend Recall($i)    [lindex [split $recall    =] end]
      lappend Precision($i) [lindex [split $precision =] end]
    }
    incr i
  }
  close $RF
  if {$i == 0} {error "File \"$ResultsFileName\" contains no data!"}
  ## Now, put the result data in Excel...
  dde poke Excel $Topic R1C1 "Tree Size"
  dde poke Excel $Topic R1C2 "m"
  dde poke Excel $Topic R1C3 "c"
  dde poke Excel $Topic R1C4 "Accuracy"
  set i 5
  foreach class $Class(0) {
    dde poke Excel $Topic R1C$i "$class: Recall"
    incr i
    dde poke Excel $Topic R1C$i "$class: Precision"
    incr i
  }
  set row 2;
  foreach size [lsort -real [array names Size]] {
    set index $Size($size)
    dde poke Excel $Topic "R${row}C1" $size
    dde poke Excel $Topic "R${row}C2" $M($index)
    dde poke Excel $Topic "R${row}C3" $C($index)
    dde poke Excel $Topic "R${row}C4" $Accuracy($index)
    set col 5
    foreach class  $Class($index) \
            recall $Recall($index) presicion $Precision($index) {
      dde poke Excel $Topic "R${row}C$col" $recall
      incr col
      dde poke Excel $Topic "R${row}C$col" $presicion
      incr col
    }
    incr row
  }
  ## Create the chart...
  set rows [expr {[llength [array names Size]] +1}]
  set cols [expr {2*[llength $Class(0)] + 4}]
  dde execute Excel $Topic "\[Select(\"R1C1:R${rows}C1\;R1C5:R${rows}C$cols\")\]\[New(2,3)\]"
};# C45SuiteExportExcel

## This function waits for the specified time...
proc creole_C45SuiteExcelWait {time} {
  global creole_C45Suite
  after $time "set creole_C45Suite(ExcelWait) 1"
  tkwait variable creole_C45Suite(ExcelWait)
  unset creole_C45Suite(ExcelWait)
};# creole_C45SuiteExcelWait
