##########################################################################
# 
#       CollectionStatistics.tcl - Saturday November 17 18:05:57 (EET) 2001
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################

## creole_CollectionStatistics
#  
proc creole_CollectionStatistics {doc col args} {
  global GUI_Text GUI_Options
  if {![string length $doc] && ![string length $col]} {
    gut_info $GUI_Text(PleaseOpenColl/Doc)
    return
  }

  GUI_LoadImage ITItemHelp
  ## Create the Statistics window...
  set win [ggi_unused_win .creole_CollectionStatisticsToolWindow]
  toplevel $win
  wm withdraw $win

  ## Header...
  set Header $win.header
  labelframe $Header -bg #d2b48c -fg #8b0000 -padx 2 -pady 2
    label $Header.title -anchor w -justify l -padx 10 -bg #d2b48c -fg #8b0000
    pack $Header.title -fill x -padx 2 -pady 2 -expand 1
  pack $Header -fill x -padx 2 -pady 0 -ipadx 2 -ipady 2

  ## Attributes Section...
  set AttrInfo $win.attributes
  labelframe $AttrInfo -text {Attributes Information:} \
    -bg #8fbc8f -fg #2f4f4f -padx 2 -pady 2
  pack $AttrInfo -fill both -padx 2 -pady 0 -expand 1 -ipadx 2 -ipady 2

  ## 1) Have we got a collection or a document?
  if {[string length $doc]} {
    ## A document:
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations) 0.0]
    set Message "$GUI_Text(Document): [tip_GetId $doc]\
                \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
    wm title $win {Document Statistics Tool...}
    $Header configure -text {Document Info:}
    $Header.title configure -image IDocument -text $Message -compound left

    ## Document Info:
    label $Header.docxid -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Document External Id: [tip_GetExternalId $doc]"
    label $Header.docenc -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Document Encoding: [tip_GetEncoding $doc]"
    label $Header.docfmt -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Document Format: [$doc Format]"
    label $Header.doccmd -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Document Tcl Command: $doc"
    label $Header.docmdf -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Document is not Modified"
    if {[$doc Modified]} {$Header.docmdf conf -text "Document is Modified"}
    pack $Header.docxid -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.docenc -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.docfmt -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.doccmd -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.docmdf -side top -padx 2 -pady 0 -fill x -expand 1

    frame $AttrInfo.info -bg #8fbc8f
      pack [label $AttrInfo.info.number -text \
      "Attribute Number: [llength [tip_GetAttributes $doc]] Attributes" \
      -bg #8fbc8f -fg #2f4f4f -anchor w] \
      -side left -pady 2 -padx 2 -fill x -expand 1
    pack $AttrInfo.info -fill x -padx 0 -pady 0
    ggi_view_Attributes $doc {} 0 {} $AttrInfo
    $AttrInfo.frame.text configure -height 5

    ## Annotation Section...
    set AnnInfo $win.annotations
    set Anns [tip_GetAnnotations $doc]
    set AnnNu [llength $Anns]
    labelframe $AnnInfo -text {Annotations Information:} \
      -bg #b0c4de -fg #0000cd -padx 2 -pady 2
    frame $AnnInfo.info -bg #b0c4de
      pack [label $AnnInfo.info.number -anchor w -text \
      "Annotation Number: $AnnNu Annotations" -bg #b0c4de -fg #0000cd] \
      -side left -pady 2 -padx 2 -fill x -expand 1
    pack $AnnInfo.info -fill x -padx 0 -pady 0
    set listbox_frame $AnnInfo.ann; set listbox $AnnInfo.ann.mclistbox
    ::ScrolledWidgets::ScrolledMclistbox $listbox_frame -font FixedTextFont \
       -selectmode browse -width $GUI_Options(TextWidth) -height 5 \
       -highlightthickness 0 -selectbackground red -maphorizontal 1 \
       -selectforeground white -selectborderwidth 1 -columnrelief flat \
       -labelanchor w -columnborderwidth 0 -labelborderwidth 1 -mapvertical 1 \
       -labelrelief raised -labelforeground navy -labelbackground bisque \
       -fillcolumn type -labelfont TextFont -bg $GUI_Options(InputBackground) \
       -selectcommand ""
    pack $AnnInfo.ann -pady 2 -padx 2 -fill both -expand 1

    $listbox column add id    -label ID         -width 7
    $listbox column add type  -label Type       -width 15
    $listbox column add start -label Start      -width 7
    $listbox column add end   -label End        -width 7
    $listbox column add Attr  -label Attributes 

    ## Sort Bindings...
    $listbox label bind id    <ButtonPress-1> {ggi_view_raw_sort %W id}
    $listbox label bind type  <ButtonPress-1> {ggi_view_raw_sort %W type}
    $listbox label bind start <ButtonPress-1> {ggi_view_raw_sort %W start}
    $listbox label bind end   <ButtonPress-1> {ggi_view_raw_sort %W end}
    bind $listbox <<MB_1>> {+focus %W}
    bind $listbox <<MB_2>> {eval ggi_view_rawContextMenu \
         [::mclistbox::convert %W -W -x %x -y %y] %X %Y; break}
    
    ## Fill the Annotation listbox...
    if {$AnnNu < 1} {set AnnNu 1}
    set ProgressStep [expr {60.0/$AnnNu}]; set Progress 0.0
    set max_attr_width 0
    set Items {}
    foreach ann $Anns {
      set Progress [expr {$Progress + $ProgressStep}]
      ggi_wait_update {} $Progress $wwin
      foreach {width display_ann} [CDM_PresentAnnotationForRawViewer $ann] \
                                  {break}
      lappend Items $display_ann
      if {$width > $max_attr_width} {set max_attr_width $width}
    }
    set AnnNu [llength $Items]
    if {$AnnNu} {
      set Process 61
      ggi_wait_update {} $Process $wwin
      set step [expr {$AnnNu/4}]
      for {set start 0; set end $step; set i 1} {$i < 5} \
          {incr i; incr end; set start $end; set end [expr {$i*$step}]} {
        if {$i == 4} {set end $AnnNu}
        eval $listbox insert end [lrange $Items $start $end]
        incr Process 10
        ggi_wait_update {} $Process $wwin
      }
      ggi_wait_update {} 100 $wwin
    }
    $listbox column configure Attr -width $max_attr_width
    ggi_destroy_wait $wwin
    update
    pack $AnnInfo -fill both -padx 2 -pady 0 -expand 1 -ipadx 2 -ipady 2
  } else {
    ## A collection: Iterate over all documents...
    set Message "$GUI_Text(Collection): [file tail [tip_GetName $col]]\
                 \n([tip_GetName $col])"
    wm title $win {Collection Statistics Tool...}
    $Header configure -text {Collection Info:}
    $Header.title configure -image ICollection -text $Message -compound left

    ## Collection Info:
    label $Header.colxid -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection Name: [tip_GetName $col]"
    label $Header.colenc -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection Encoding: [tip_GetEncoding $col]"
    label $Header.collen -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection Document Number: [tip_Length $col]"
    label $Header.colfmt -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection Format: [$col Format]"
    label $Header.colcmd -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection Tcl Command: $col"
    label $Header.colmdf -anchor w -bg #d2b48c -fg #8b0000 -compound left \
      -image ITItemHelp -text "Collection is not Modified"
    if {[$col Modified]} {$Header.colmdf conf -text "Collection is Modified"}
    pack $Header.colxid -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.colenc -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.collen -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.colfmt -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.colcmd -side top -padx 2 -pady 0 -fill x -expand 1
    pack $Header.colmdf -side top -padx 2 -pady 0 -fill x -expand 1

    frame $AttrInfo.info -bg #8fbc8f
      pack [label $AttrInfo.info.number -text \
      "Attribute Number: [llength [tip_GetAttributes $col]] Attributes" \
      -bg #8fbc8f -fg #2f4f4f -anchor w] \
      -side left -pady 2 -padx 2 -fill x -expand 1
    pack $AttrInfo.info -fill x -padx 0 -pady 0
    ggi_view_Attributes $col {} 0 {} $AttrInfo
    $AttrInfo.frame.text configure -height 5

    ## Documents Section...
    set DocInfo $win.documents
    labelframe $DocInfo -text {Documents Information:} \
      -bg #b0c4de -fg #0000cd -padx 2 -pady 2
    frame $DocInfo.info -bg #b0c4de
      pack [label $DocInfo.info.number -anchor w -bg #b0c4de -fg #0000cd -text \
      "Document Number: [tip_Length $col] Documents"] \
      -side left -pady 2 -padx 2 -fill x -expand 1
    pack $DocInfo.info -fill x -padx 0 -pady 0
    set listbox_frame $DocInfo.doc; set listbox $DocInfo.doc.mclistbox
    ::ScrolledWidgets::ScrolledMclistbox $listbox_frame -font FixedTextFont \
       -selectmode browse -width $GUI_Options(TextWidth) -height 5 \
       -highlightthickness 0 -selectbackground red -maphorizontal 1 \
       -selectforeground white -selectborderwidth 1 -columnrelief flat \
       -labelanchor w -columnborderwidth 0 -labelborderwidth 1 -mapvertical 1 \
       -labelrelief raised -labelforeground navy -labelbackground bisque \
       -fillcolumn id -labelfont TextFont -bg $GUI_Options(InputBackground) \
       -selectcommand ""
    pack $DocInfo.doc -pady 2 -padx 2 -fill both -expand 1

    $listbox column add doc   -label Item          -width 8
    $listbox column add id    -label ID            -width 25
    $listbox column add xid   -label {External ID} -width 40

    ## Sort Bindings...
    bind $listbox <<MB_1>> {+focus %W}
    
    ## Fill the Document listbox...
    foreach {item max_id max_xid} {1 25 40} {}
    foreach id [$col GetFileIds] xid [$col GetFileXIds] {
      $listbox insert end [list $item. $id $xid]
      if {[string length $id]  > $max_id}  {set max_id  [string length $id]}
      if {[string length $xid] > $max_xid} {set max_xid [string length $xid]}
      incr item
    }
    $listbox column configure id  -width $max_id
    $listbox column configure xid -width $max_xid
    pack $DocInfo -fill both -padx 2 -pady 0 -expand 1 -ipadx 2 -ipady 2
  }

  frame $win.buttons
    button $win.buttons.ok -textvariable GUI_Text(Close) -command "destroy $win"
    pack $win.buttons.ok -pady 5
  pack $win.buttons -fill x -padx 2 -pady 2
  update
  wm deiconify $win
};# creole_CollectionStatistics
#
# End of File
#

