##########################################################################
# 
#       CreateVectors.tcl - Tuesday January 16 17:59:15 (EET) 2001
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.50...
#
##########################################################################


# Location of this module (CreateVectors)
global creole_CreateVectors_home

## creole_CreateVectors
#  
proc creole_CreateVectors {doc col args} {
  global GUI_Text
  ## Check if we got an opened Collection/Document...
  if {![string length $doc] && ![string length $col]} {
    gut_info $GUI_Text(PleaseOpenColl/Doc)
    return
  }

  set enableVectorCreation 1
  set optionsFile {}

  foreach {arg value} $args {
    switch -glob -- $arg {
      -enableVectorCreation {set enableVectorCreation $value}
      -optionsFile          {set optionsFile $value}
      default {error "unknown option \"$arg\"!"}
    }
  }

  ## Create the window
  global creole_CreateVectors_home GUI_Text GUI_Options \
         HomeDir UserHomeDir CDMTools_CompareCollections creole_CreateVectors
  ::Annotation::Init
  
  #set win [ggi_unused_win .creole_CreateVectors]
  set win .creole_CreateVectors
  if {[winfo exists $win]} {
    raise $win
    return
  }
  toplevel $win
  wm withdraw $win
  wm title $win {Create Vectors from Annotations...}

  ## 1) Have we got a collection or a document?
  if {[string length $doc]} {
    ## A document:
    set Image IDocument
    set Message "$GUI_Text(EditingDocument): [tip_GetId $doc]\
               \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
  } else {
    ## A collection: Iterate over all documents...
    set Image ICollection
    set Message "$GUI_Text(EditingCollection): [file tail [tip_GetName $col]]\
               \n([tip_GetName $col])"
  }

  set TitleFrame $win.title
  frame $TitleFrame
    label $TitleFrame.image -image $Image
    label $TitleFrame.text -height 2 -anchor w -justify left -text $Message
    label $TitleFrame.cellnul -textvariable GUI_Text(NumberOfCells:)
    spinbox $TitleFrame.cellnu -textvariable creole_CreateVectors(CellNumber) \
      -from 0 -to 25 -width 2 -command [list \
      creole_CreateVectors_CreateCellsWidgets $win.combo.cell]
    $TitleFrame.cellnu set 3
    grid $TitleFrame.image $TitleFrame.text \
         $TitleFrame.cellnul $TitleFrame.cellnu -sticky sew -padx 3 -pady 6
    grid columnconfigure $TitleFrame 1 -weight 1
  pack $TitleFrame -fill x -expand 0

  set Frame $win.combo
  frame $Frame

  ## Vector Annotation...
  labelframe $Frame.vectorann -text $GUI_Text(CreateVectorsFromAnnotation:)
    set VAnn $Frame.vectorann
    label $VAnn.annmsg  -anchor w -justify left -textvar GUI_Text(Annotation)
    ComboBox $VAnn.annval -entrybg $GUI_Options(InputBackground) \
       -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
       -highlightthickness 0 -editable 1 \
       -values [::Annotation::AnnotationTypes] \
       -textvariable creole_CreateVectors(VectorAnn)
    grid $VAnn.annmsg   $VAnn.annval -padx 2 -pady 2 -sticky snew
    label $VAnn.consmsg -anchor w -justify left \
       -textvariable GUI_Text(Constraints)
    entry $VAnn.cons -textvariable creole_CreateVectors(VectorAnnConstraints)
    grid $VAnn.consmsg $VAnn.cons -padx 2 -pady 2 -sticky snew
    grid columnconfigure $VAnn 1 -weight 1
  pack $Frame.vectorann -pady 2 -fill x -expand 0
  
  ## Vector Cell Attributes...
  labelframe $Frame.cell -text $GUI_Text(VectorCellAttributes:)
    set Cell $Frame.cell
    creole_CreateVectors_CreateCellsWidgets $Frame.cell
    grid columnconfigure $Cell 1 -weight 1
    grid columnconfigure $Cell 3 -weight 1
    grid rowconfigure $Cell 25 -weight 1
  pack $Frame.cell -pady 2 -fill x -expand 0
  
  ## Class Value...
  labelframe $Frame.class -text $GUI_Text(ClassValue:)
    set Class $Frame.class
    label $Class.annmsg  -anchor w -justify left -textvar GUI_Text(Annotation)
    ComboBox $Class.annval -entrybg $GUI_Options(InputBackground) \
       -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
       -highlightthickness 0 -editable 1 \
       -values [::Annotation::AnnotationTypes] \
       -textvariable creole_CreateVectors(ClassAnn) \
       -modifycmd "::GUI::FillAttributeComboFromAnnotationCombo \
                          $Class.annval $Class.attrval"
    label $Class.attrmsg -anchor w -justify left -textvar GUI_Text(Attribute)
    ComboBox $Class.attrval -entrybg $GUI_Options(InputBackground) \
       -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
       -highlightthickness 0 -editable 1 \
       -textvariable creole_CreateVectors(ClassAttr)
    label $Class.attrfailmsg -textvariable GUI_Text(Absencevalue:) -anchor w
    if {![info exists creole_CreateVectors(ClassAbsence)]} {
      set creole_CreateVectors(ClassAbsence) NOCLASSVALUE
    }
    entry $Class.attrfailval -textvar creole_CreateVectors(ClassAbsence)
    grid $Class.annmsg   $Class.annval -padx 2 -pady 2 -sticky snew
    grid $Class.attrmsg  $Class.attrval \
         $Class.attrfailmsg $Class.attrfailval -padx 2 -pady 2 -sticky snew
    label $Class.failmsg -textvariable \
         GUI_Text(InCaseOfAnnotationNismatchUseValue:) -anchor w
    entry $Class.fail -textvariable creole_CreateVectors(ClassFail)
    $Class.fail delete 0 end
    $Class.fail insert end FALSE
    grid $Class.failmsg $Class.fail -columnspan 2 -padx 2 -pady 2 -sticky snew
    ## Options for Prefixes...
    labelframe $Class.pref
    $Class.pref configure -labelwidget [checkbutton $Class.pref.contain \
    -textvariable GUI_Text(AnnotationsThatVectorsAreBasedCanBeContainedByClassAnnotations) \
      -variable creole_CreateVectors(ClassCanContain) -command \
      "if {\$creole_CreateVectors(ClassCanContain)} {
         $Class.pref.spref configure -state normal 
         $Class.pref.epref configure -state normal
         if {\$creole_CreateVectors(ClassPrefixModeStart)} {
           $Class.pref.sprefval configure -state normal
         } else {$Class.pref.sprefval configure -state disabled}
         if {\$creole_CreateVectors(ClassPrefixModeEnd)} {
           $Class.pref.eprefval configure -state normal
         } else {$Class.pref.eprefval configure -state disabled}
       } else {
         $Class.pref.spref configure -state disabled
         $Class.pref.sprefval configure -state disabled
         $Class.pref.epref configure -state disabled
         $Class.pref.eprefval configure -state disabled
       }"]
      if {![info exists creole_CreateVectors(ClassPrefixStart)]} {
        set creole_CreateVectors(ClassPrefixStart) S_
      }
      if {![info exists creole_CreateVectors(ClassPrefixEnd)]} {
        set creole_CreateVectors(ClassPrefixEnd) E_
      }
      checkbutton $Class.pref.spref \
        -textvariable GUI_Text(AddPrefixIfAnnotationMatchesStart:) \
        -anchor w -variable creole_CreateVectors(ClassPrefixModeStart) \
        -command \
        "if {\$creole_CreateVectors(ClassPrefixModeStart)} {
           $Class.pref.sprefval configure -state normal
         } else {$Class.pref.sprefval configure -state disabled}"
      entry $Class.pref.sprefval \
        -textvariable creole_CreateVectors(ClassPrefixStart) \
        -state disabled
      checkbutton $Class.pref.epref \
        -textvariable GUI_Text(AddPrefixIfAnnotationMatchesEnd:) \
        -anchor w -variable creole_CreateVectors(ClassPrefixModeEnd) \
        -command \
        "if {\$creole_CreateVectors(ClassPrefixModeEnd)} {
           $Class.pref.eprefval configure -state normal
         } else {$Class.pref.eprefval configure -state disabled}"
      entry $Class.pref.eprefval \
        -textvariable creole_CreateVectors(ClassPrefixEnd) \
        -state disabled
      if {$creole_CreateVectors(ClassCanContain)} {
         $Class.pref.spref configure -state normal 
         $Class.pref.epref configure -state normal
         if {$creole_CreateVectors(ClassPrefixModeStart)} {
           $Class.pref.sprefval configure -state normal
         } else {$Class.pref.sprefval configure -state disabled}
         if {$creole_CreateVectors(ClassPrefixModeEnd)} {
           $Class.pref.eprefval configure -state normal
         } else {$Class.pref.eprefval configure -state disabled}
       } else {
         $Class.pref.spref configure -state disabled
         $Class.pref.sprefval configure -state disabled
         $Class.pref.epref configure -state disabled
         $Class.pref.eprefval configure -state disabled
       }
      grid $Class.pref.spref $Class.pref.sprefval -padx 12 -pady 2 -sticky snew
      grid $Class.pref.epref $Class.pref.eprefval -padx 12 -pady 2 -sticky snew
      grid columnconfigure $Class.pref 1 -weight 1
    grid $Class.pref -columnspan 4 -padx 2 -pady 2 -sticky snew
    grid columnconfigure $Class 1 -weight 1
    grid columnconfigure $Class 3 -weight 1
  pack $Frame.class -pady 2 -fill x -expand 0
  
  ## Vector Format...
  labelframe $Frame.vector -text $GUI_Text(VectorFormat:)
    set Vector $Frame.vector
    image create photo creole_CreateVectorsImg -file \
       [file join $creole_CreateVectors_home Explain.gif]
    label $Vector.img -image creole_CreateVectorsImg
    bind  $Vector.img <Destroy> \
    "if {!\[image inuse creole_CreateVectorsImg\]} {
      catch {image delete creole_CreateVectorsImg}
     }"
    grid  $Vector.img -columnspan 2
    label $Vector.msg -anchor w -justify left \
      -textvariable GUI_Text(VectorFormat:)
    entry $Vector.val -textvar creole_CreateVectors(Format)
    $Vector.val delete 0 end
    $Vector.val insert end {-A2 -A1 A1 A2 -B2 -B1 B1 B2 CLASS}
    #$Vector.val insert end \
    #  {ANNID ROOT_ANN_TEXT -A2 -A1 A1 A2 -B2 -B1 B1 B2 CLASS}
    ## Select the vector format...
    frame $Vector.vector_format
      if {![info exists creole_CreateVectors(OutputFormat)]} {
        set creole_CreateVectors(OutputFormat) arff
      }
      if {![info exists creole_CreateVectors(OutputFormatSpec)]} {
        set creole_CreateVectors(OutputFormatSpec) {$1/$2\ }
      }
      radiobutton $Vector.vector_format.c45 \
        -textvariable GUI_Text(C45C50Format) \
        -variable creole_CreateVectors(OutputFormat) -value c45 \
        -command "$Vector.vector_format.format configure -state disabled"
      radiobutton $Vector.vector_format.arff \
        -textvariable GUI_Text(ARFFFormatWEKA) \
        -variable creole_CreateVectors(OutputFormat) -value arff \
        -command "$Vector.vector_format.format configure -state disabled"
      radiobutton $Vector.vector_format.other -textvariable GUI_Text(Other:) \
        -variable creole_CreateVectors(OutputFormat) -value other \
        -command "$Vector.vector_format.format configure -state normal"
      entry $Vector.vector_format.format -textvariable \
            creole_CreateVectors(OutputFormatSpec)
      if {$creole_CreateVectors(OutputFormat) != "other"} {
        $Vector.vector_format.format configure -state disabled
      }
      pack $Vector.vector_format.c45 $Vector.vector_format.arff \
           $Vector.vector_format.other -padx 2 -side left
      pack $Vector.vector_format.format -padx 2 -side left -fill x -expand 1
    grid $Vector.msg $Vector.val -padx 2 -pady 2 -sticky snew
    grid $Vector.vector_format -columnspan 2 -padx 2 -pady 2 -sticky snew
    grid columnconfigure $Vector 1 -weight 1
  pack $Frame.vector -pady 2 -fill x -expand 0

  ## Output File...
  labelframe $Frame.file -text $GUI_Text(OutputFile:)
    set File $Frame.file
    label  $File.msg -anchor w -justify left -textvariable GUI_Text(OutputFile:)
    entry  $File.val -textvar creole_CreateVectors(Output)
    $File.val delete 0 end
    $File.val insert end [file join $UserHomeDir Vectors.arff]
    button $File.sel -image ITOpen -textvar GUI_Text(SelectFile) -compound left\
      -command "GUI_SelectSaveFile $File.val -defaultextension *.* \
          -filetypes [list [list [list $GUI_Text(All) [list * *.*]]]]"
    GUI_MakeEntrySaveFileDropTarget $File.val
    grid $File.msg $File.val $File.sel -padx 2 -pady 2 -sticky snew
    frame $File.buttons
      button $File.buttons.save -textvariable GUI_Text(SaveSettings...) \
        -command creole_CreateVectors_SaveSettings
      button $File.buttons.open -textvariable GUI_Text(LoadSettings...) \
        -command "creole_CreateVectors_LoadSettings $win"
      set bg [$File.buttons.open cget -background]
      pack $File.buttons.save $File.buttons.open -padx 2 -side left
    grid $File.buttons -columnspan 3 -padx 2 -pady 2 -sticky snew
    grid columnconfigure $File 1 -weight 1
  pack $Frame.file -pady 2 -fill x -expand 0
  pack $Frame -padx 4 -pady 5 -fill x -expand 0

  ## Button Section
  set ButtonFrame $win.buttons
  frame $ButtonFrame
    button $ButtonFrame.apply -textvar GUI_Text(Apply) -command \
      [list creole_CreateVectorsSaveVectors $doc $col $win] -default active
    bind $win <Return> "$ButtonFrame.apply invoke"
    button $ButtonFrame.close -textvar GUI_Text(Close) -command "destroy $win"
    pack $ButtonFrame.apply $ButtonFrame.close -side left -padx 4
    if {!$enableVectorCreation} {
      $ButtonFrame.apply configure -state disabled
    }
  pack $ButtonFrame -padx 4 -pady 5 -expand 1
  update
  wm deiconify $win
  update
  if {[string length $optionsFile]} {
    creole_CreateVectors_LoadSettings $win $optionsFile 1
  }
  catch {
    dnd bindtarget $File.buttons.save Files <DragEnter> \
      {%W configure -bg $GUI_Options(DragEnterBackgound);update;return default}
    dnd bindtarget $File.buttons.save Files <DragLeave> \
      "%W configure -bg $bg;update;return default"
    dnd bindtarget $File.buttons.save Files <Drop> \
      "%W configure -bg $bg
      creole_CreateVectors_SaveSettings \[GUI_NormaliseURI %D\]"
    dnd bindtarget $File.buttons.open Files <DragEnter> \
      {%W configure -bg $GUI_Options(DragEnterBackgound);update;return default}
    dnd bindtarget $File.buttons.open Files <DragLeave> \
      "%W configure -bg $bg;update;return default"
    dnd bindtarget $File.buttons.open Files <Drop> \
      "%W configure -bg $bg
      creole_CreateVectors_LoadSettings $win \[GUI_NormaliseURI %D\]"
  }
} ;# creole_CreateVectors

## creole_CreateVectorsSaveVectors
#   This function actually performs the functionality of creating and saving
#   the vectors in the output file...
proc creole_CreateVectorsSaveVectors {doc col win {encoding {}}} {
  global creole_CreateVectors
  set file [open $creole_CreateVectors(Output) w]
  if {[string length $encoding]} {
    fconfigure $file -encoding $encoding
  }
  if {[string length $doc]} {
    ## Only a single Document was opened...
    if {[catch {creole_CreateVectorsProcessDocument $doc $file} error]} {
      close $file
      error $error
    }
  } else {
    ## A whole Collection was opened. We have to iterate over all available
    ## Documents...
    global GUI_Text
    set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0.0]
    set doc [tip_FirstDocument $col]
    set Length [tip_Length $col]
    set Step [expr {100.0/($Length+1)}]
    set Progress 0.0
    while {[string length $doc]} {
      if {[catch {creole_CreateVectorsProcessDocument $doc $file} error]} {
        close $file
        ggi_destroy_wait $wwin
        tip_Close $doc
        error $error
      }
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
      tip_Close $doc
      set doc [tip_NextDocument $col]
    }
    ggi_destroy_wait $wwin
  }
  close $file
  ## Post-Process file according to the output format...
  if {![info exists creole_CreateVectors(OutputFormat)]} {
    set creole_CreateVectors(OutputFormat) c4.5
  }
  switch $creole_CreateVectors(OutputFormat) {
    arff {
      global GUI_Text
      set file [open $creole_CreateVectors(Output)]
      ## Substitute all \, win ^A, as we will split on ","...
      set data [string map "\\\\, \001" [string trim [read $file]]]
      close $file
      set Vectors [split $data \n]
      unset data
      set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0.0]
      set Step [expr {100.0/([llength $Vectors]+1)}]
      set Progress 0.0
      set NVectors {}
      set VectorFormat $creole_CreateVectors(Format)
      foreach vector $Vectors {
        set nvector {}
        set cell 1
        foreach value [split [string range $vector 0 end-1] ,] {
          switch -exact -- $value {
            ?  {
              CDM_LappendUnique Values($cell) $value
              lappend nvector $value
            }
            default {
              CDM_LappendUnique Values($cell) \'$value\'
              lappend nvector \'$value\'
            }
          }
          incr cell
        }
        lappend NVectors [join $nvector ,]
        set Progress [expr {$Progress+$Step}]
        ggi_wait_update {} $Progress $wwin
      }
      unset Vectors
      ## Write the file again in ARFF format...
      set file [open $creole_CreateVectors(Output) w]
      fconfigure $file -encoding utf-8
      puts $file \
      "% Vectors Generated by $::ApplicationName, version $::ApplicationVersion\
     \n%\
     \n% Vectors generated from Collection: [tip_GetName $col]\
     \n%      Creator: Create Vectors Tool...\
     \n%      Date: [clock format [clock seconds] \
                           -format {%H:%M:%S, %A %d %B %Y} -gmt 1] GMT\
     \n%\
     \n@RELATION \"[tip_GetName $col]\"\
     \n"
      ## Prepare the cell spec...
      set CellSpec {}
      for {set i 0} {$i < $creole_CreateVectors(CellNumber)} {incr i} {
        if {![string length $creole_CreateVectors(CellAnn_$i)]} {continue}
        if {$creole_CreateVectors(AnnotationText_$i)} {
          set value "$creole_CreateVectors(CellAnn_$i)_AnnotationText"
        } else {
          set value \
          "$creole_CreateVectors(CellAnn_$i)_$creole_CreateVectors(CellAttr_$i)"
        }
        lappend CellSpec $value
      }
      ## Remove all "?" from values...
      foreach pos [array names Values] {
        set index [lsearch -exact $Values($pos) ?]
        if {$index != -1} {
          set Values($pos) [lreplace $Values($pos) $index $index]
        }
        set Values($pos) \
            \{[string map "\001 \\\\," [join [lsort $Values($pos)] ,]]\}
      }
      set pos 1
      foreach cell $VectorFormat {
        if {[string equal $Values($pos) "{}"]} {
          gut_info "No values have been found for vector field \"$cell\"!\
                    This may cause reading errors while opening the ARFF file\
                    with WEKA. However, the ARFF file will be written."
        }
        switch -regexp -- $cell {
          ^ANN$            -
          ^-A(\\d)+$       -
          ^-B(\\d)+$       {
            foreach spec $CellSpec {
              puts $file "@ATTRIBUTE \'[format %.2d ${pos}]_${cell}_${spec}\'\
                          $Values($pos)"
              incr pos
            }
          }
          ^A(\\d)+$        -
          ^B(\\d)+$        {
            foreach spec $CellSpec {
              puts $file "@ATTRIBUTE \'[format %.2d ${pos}]_+${cell}_${spec}\'\
                          $Values($pos)"
              incr pos
            }
          }
          ^CLASS$          {
            puts $file "@ATTRIBUTE \'[format %.2d ${pos}]_${cell}\'\
                        $Values($pos)"
            incr pos
          }
          ^ANNID$          -
          ^ROOT_ANN_TEXT$  -
          ^CLASS_ANN_TEXT$ -
          default          {
            puts $file "@ATTRIBUTE \'[format %.2d ${pos}]_${cell}\' STRING"
            incr pos
          }
        }
      }
      puts $file "\n@DATA"
      set data [string map "\001 \\\\," [join $NVectors \n]]
      puts $file $data
      close $file
      ggi_destroy_wait $wwin
    }
    other {
      global GUI_Text
      set file [open $creole_CreateVectors(Output)]
      ## Substitute all \, win ^A, as we will split on ","...
      set data [string map "\\\\, \001" [string trim [read $file]]]
      close $file
      set Vectors [split $data \n]
      unset data
      set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0.0]
      set Step [expr {100.0/([llength $Vectors]+1)}]
      set Progress 0.0
      set NVectors {}
      set CellFormat   $creole_CreateVectors(OutputFormatSpec)
      ## Count how many entries exist for each cell...
      set VarSpec {}
      for {set CellAttrs 1} {$CellAttrs <= $creole_CreateVectors(CellNumber)} \
                            {incr CellAttrs} {
        lappend VarSpec $CellAttrs
        if {![string length $creole_CreateVectors(CellAnn_$CellAttrs)]} {break}
      }
      ## Now, in VarSpec we have the names of the variables we want to define.
      ## These variable names are simply integers. The caller can specify
      ## $1/$2 to refere to the first two Attributes...

      ## Write the file again in the specified format...
      set file [open $creole_CreateVectors(Output) w]
      ## Define all the variables, just to be sure...
      for {set i 0} {$i <= 25} {incr i} {
        set $i "Error in Cell format!"
      }
      set specStart [llength $VarSpec]
      foreach vector $Vectors {
        set vector [split [string range $vector 0 end-1] ,]
        foreach cell $creole_CreateVectors(Format) {
          switch -regexp -- $cell {
            ^ANN$            -
            ^-A(\\d)+$       -
            ^-B(\\d)+$       -
            ^A(\\d)+$        -
            ^B(\\d)+$        {
              foreach $VarSpec $vector {
                eval puts -nonewline $file $CellFormat
                break
              }
              set vector [lrange $vector $specStart end]
            }
            ^CLASS$          -
            ^ANNID$          -
            ^ROOT_ANN_TEXT$  -
            ^CLASS_ANN_TEXT$ -
            default          {
              puts -nonewline $file [lindex $vector 0]
              set vector [lrange $vector 1 end]
            }
          }
        };# foreach cell $creole_CreateVectors(Format)
        puts $file {}
        set Progress [expr {$Progress+$Step}]
        ggi_wait_update {} $Progress $wwin
      }
      close $file
      ggi_destroy_wait $wwin
    }
    c4.5    -
    default {}
  }
};# creole_CreateVectorsSaveVectors

## creole_CreateVectorsProcessDocument
#  This function process a single Document...
proc creole_CreateVectorsProcessDocument {doc file {MsgWin {}}} {
  global creole_CreateVectors

  if {[string length $MsgWin]} {
    set wwin $MsgWin
  } else {
    set wwin [ggi_wait_append \
      "Creating Vectors from Document:\n[tip_GetId $doc]" 0.0]
  }
  # set VectorAnnotation     $creole_CreateVectors(VectorAnn)
  # set VectorAnnConstraints $creole_CreateVectors(VectorAnnConstraints)
  # set VectorFormat         $creole_CreateVectors(Format)
  # set ClassAnnotation      $creole_CreateVectors(ClassAnn)
  # set ClassAttribute       $creole_CreateVectors(ClassAttr)
  # set ClassFail            $creole_CreateVectors(ClassFail)
  # set ClassCanContain      $creole_CreateVectors(ClassCanContain)
  # set ClassPrefixStartMode $creole_CreateVectors(ClassPrefixModeStart)
  # set ClassPrefixEndMode   $creole_CreateVectors(ClassPrefixModeEnd)
  # set ClassPrefixStart     $creole_CreateVectors(ClassPrefixStart)
  # set ClassPrefixEnd       $creole_CreateVectors(ClassPrefixEnd)
  set CellSpec {}
  for {set i 0} {$i < $creole_CreateVectors(CellNumber)} {incr i} {
    if {[string length $creole_CreateVectors(CellAnn_$i)] && (
        [string length $creole_CreateVectors(CellAttr_$i)] ||
        $creole_CreateVectors(AnnotationText_$i) )} {
      lappend CellSpec $creole_CreateVectors(CellAnn_$i) \
                       $creole_CreateVectors(CellAttr_$i) \
                       $creole_CreateVectors(CellFail_$i) \
                       $creole_CreateVectors(Contained_$i) \
                       $creole_CreateVectors(AnnotationText_$i) \
                       $creole_CreateVectors(Expand_$i)
    }
  }
  set Vectors [eval tip_CreateVectors $doc \
                    [list $creole_CreateVectors(VectorAnn)] \
                    [list $creole_CreateVectors(VectorAnnConstraints)] \
                    [list $creole_CreateVectors(Format)] \
                    [list $creole_CreateVectors(ClassAnn)] \
                    [list $creole_CreateVectors(ClassAttr)] \
                    [list $creole_CreateVectors(ClassAbsence)] \
                    [list $creole_CreateVectors(ClassFail)] \
                    [list $creole_CreateVectors(ClassCanContain)] \
                    [list $creole_CreateVectors(ClassPrefixModeStart)] \
                    [list $creole_CreateVectors(ClassPrefixModeEnd)] \
                    [list $creole_CreateVectors(ClassPrefixStart)] \
                    [list $creole_CreateVectors(ClassPrefixEnd)] \
                    [list $wwin] $CellSpec]

  ## Make sure all values inside the vector are proper...
  foreach Vector $Vectors {
    set NV {}
    foreach item $Vector {
      if {[string length $item]} {
        lappend NV [string map \
          {, \\, . \\. \; \\\; : \\: ! \\! ( \\( ) \\) \" \\\" ' \\'
           \\ \\\\ % \\% \{ \\\{ \} \\\} | \\|} $item]
      } else {
        lappend NV ?
      }
    }
    puts $file [join $NV ,].
  }
  ggi_wait_update {} 100.0 $wwin
  if {![string length $MsgWin]} {ggi_destroy_append $wwin}
};# creole_CreateVectorsProcessDocument

## creole_CreateVectors_ToggleWidgetState
#  This function will toggle the state of the provided widgets, according to
#  the value of the first checkbox...
proc creole_CreateVectors_ToggleWidgetState {checkbutton args} {
  set var [$checkbutton cget -variable]
  ## Get the variable value...
  set val [uplevel \#0 set $var]
  if {$val} {
    ## Deactivate widgets...
    foreach widget $args {
      $widget configure -state disabled
    }
  } else {
    ## Activate widgets...
    foreach widget $args {
      $widget configure -state normal
    }
  }
};# creole_CreateVectors_ToggleWidgetState

## creole_CreateVectors_CreateCellsWidgets
proc creole_CreateVectors_CreateCellsWidgets {Cell} {
  global creole_CreateVectors GUI_Text GUI_Options
  for {set attr 0} {$attr < $creole_CreateVectors(CellNumber)} {incr attr} {
    if {[winfo exists $Cell.annmsg_$attr]} {continue}
    label $Cell.annmsg_$attr -anchor w -justify left \
      -textvar GUI_Text(Annotation)
    ComboBox $Cell.annval_$attr -entrybg $GUI_Options(InputBackground) \
     -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
     -highlightthickness 0 -editable 1 \
     -values [::Annotation::AnnotationTypes] \
     -textvariable creole_CreateVectors(CellAnn_$attr) \
     -modifycmd "::GUI::FillAttributeComboFromAnnotationCombo \
                        $Cell.annval_$attr $Cell.attrval_$attr"
    label $Cell.attrmsg_$attr -anchor w -justify left \
      -textvar GUI_Text(Attribute)
    ComboBox $Cell.attrval_$attr -entrybg $GUI_Options(InputBackground) \
     -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
     -highlightthickness 0 -editable 1 \
     -textvariable creole_CreateVectors(CellAttr_$attr)
    label $Cell.failmsg_$attr -textvariable GUI_Text(Absencevalue:) -anchor w
    if {![info exists creole_CreateVectors(CellFail_$attr)]} {
      set creole_CreateVectors(CellFail_$attr) NOVALUE
    }
    entry $Cell.failval_$attr -textvar creole_CreateVectors(CellFail_$attr)
    checkbutton $Cell.text_$attr -textvariable GUI_Text(AnnotatedText) \
      -variable creole_CreateVectors(AnnotationText_$attr) \
      -command [list creole_CreateVectors_ToggleWidgetState $Cell.text_$attr \
                $Cell.attrmsg_$attr $Cell.attrval_$attr \
                $Cell.failmsg_$attr $Cell.failval_$attr $Cell.cont_$attr \
                $Cell.expandl_$attr $Cell.expand_$attr]
    checkbutton $Cell.cont_$attr -textvariable GUI_Text(Contain) \
      -variable creole_CreateVectors(Contained_$attr)
    label $Cell.expandl_$attr -textvariable GUI_Text(Expandinto:)
    spinbox $Cell.expand_$attr -textvariable creole_CreateVectors(Expand_$attr)\
      -from 0 -to 100000 -width 2
    grid $Cell.annmsg_$attr  $Cell.annval_$attr  \
         $Cell.attrmsg_$attr $Cell.attrval_$attr \
         $Cell.failmsg_$attr $Cell.failval_$attr \
         $Cell.text_$attr $Cell.cont_$attr \
         $Cell.expandl_$attr $Cell.expand_$attr -padx 1 -pady 1 -sticky ew
  }
  ## Destroy any unwanted widgets...
  for {set attr 25} {$attr>=$creole_CreateVectors(CellNumber)} {incr attr -1} {
    if {![winfo exists $Cell.annmsg_$attr]} {continue}
    destroy $Cell.annmsg_$attr  $Cell.annval_$attr  \
            $Cell.attrmsg_$attr $Cell.attrval_$attr \
            $Cell.failmsg_$attr $Cell.failval_$attr \
            $Cell.text_$attr    $Cell.cont_$attr \
            $Cell.expandl_$attr $Cell.expand_$attr
  }
  update
  set toplevel [winfo toplevel $Cell]
  set reqw [winfo width  $toplevel]
  set reqh [winfo reqheight $toplevel]
  if {$reqh > 200} {
    wm geometry $toplevel ${reqw}x$reqh
  }
};# creole_CreateVectors_CreateCellsWidgets

## creole_CreateVectors_SaveSettings
#  Save tool settings on a file...
proc creole_CreateVectors_SaveSettings {{File {}}} {
  global creole_CreateVectors
  if {![string length $File]} {
    set File [tk_getSaveFile]
  } else {
    if {[file isdirectory $File]} {
      gut_info [format $::GUI_Text(NotFile-Directory) $File]
      return
    }
    if {[gut_question [format $::GUI_Text(SaveOptionsToFile?) $File]] == "no"} {
      return
    }
    if {[file exists $File]} {
      set reply [gut_question [format $::GUI_Text(OverwriteFile?) $File]] 
      if {[string equal $reply "no"]} {return}
    }
  }
  if {[string length $File]} {
    array set Settings [array get creole_CreateVectors]
    set file [open $File w]
    ## Write some values in a special way...
    foreach key {VectorAnn VectorAnnConstraints} {
      puts -nonewline $file "$key \{$Settings($key)\} "
      unset Settings($key)
    }
    puts $file {}
    puts $file "Format \{$Settings(Format)\}"
    unset Settings(Format)
    for {set cell 0} {$cell < $Settings(CellNumber)} {incr cell} {
      foreach key {CellAnn CellAttr CellFail AnnotationText Contained Expand} {
        set key ${key}_$cell
        puts -nonewline $file "$key \{$Settings($key)\} "
        unset Settings($key)
      }
      puts $file {}
    }
    for {set cell $Settings(CellNumber)} {$cell < 50} {incr cell} {
      foreach key {CellAnn CellAttr CellFail AnnotationText Contained Expand} {
        unset -nocomplain Settings(${key}_$cell)
      }
    }
    foreach key [lsort -dictionary [array names Settings]] {
      puts $file "$key \{$creole_CreateVectors($key)\}"
    }
    #puts $file [array get creole_CreateVectors]
    close $file
    gut_info "Options Saved!"
  }
};# creole_CreateVectors_SaveSettings

## creole_CreateVectors_LoadSettings
#  Load settings stored on a file...
proc creole_CreateVectors_LoadSettings {win {File {}} {silent 0}} {
  global creole_CreateVectors
  if {![string length $File]} {
    set File [tk_getOpenFile]
  } else {
    if {[file isdirectory $File]} {
      gut_info [format $::GUI_Text(NotFile-Directory) $File]
      return
    }
    if {!$silent} {
      if {[gut_question [format $::GUI_Text(LoadOptionsFromFile?) $File]]
                                                               == "no"} {return}
    }
  }
  if {[string length $File]} {
    set file [open $File]
    set data [read $file]
    close $file
    array set creole_CreateVectors $data
    ## Don't forget to update the spinbox...
    set cmd [$win.title.cellnu cget -command]
    eval $cmd
    ## Don't forget to update the checkbutton with Class prefixes...
    set cmd [$win.combo.class.pref.contain cget -command]
    eval $cmd
    gut_info "Options Loaded!"
  }
};# creole_CreateVectors_LoadSettings

#
# End of File
#
