##########################################################################
# 
#	CreateVectors_BagOfWordsRepresentation.tcl -
#                                  Saturday January 18 12:23:38 (EET) 2003
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.5...
#
##########################################################################

package ifneeded WizardDialog 1.0 "source $HomeDir/GUI/Dialogs/Wizard.tcl"

## Create the ::CreateVectors::BagOfWordsRepresentation namespace...
namespace eval CreateVectors {
  namespace eval BagOfWordsRepresentation {
    variable Page; # Inernal information about the current wizard page
    ## The following variable holds the sequence of the wizard pages. The pages
    ## will be navigated according to this order. For each such page, a
    ## procedure named ::CreateVectors::BagOfWordsRepresentation::Page_<page>
    ## should be available...
    variable Pages {Welcome SelectColDoc CheckCollection 
                    SelectWordAnn CollectWords
                    FilterWords SpecifyVector OutputFile Finish}
    variable Info
    variable Words
    variable Frequency2Words
  }
}
## creole_CreateVectors_BagOfWordsRepresentation
#  
proc creole_CreateVectors_BagOfWordsRepresentation {doc col args} {
  ::CreateVectors::BagOfWordsRepresentation::Init $col $doc $args
  ##
  ## As a first step, we need to collect all the words that appear in the
  ## corpus...
  ##
};# creole_CreateVectors_BagOfWordsRepresentation

proc ::CreateVectors::BagOfWordsRepresentation::Init {col doc args} {
  package require WizardDialog
  ## Create the window
  global HomeDir UserAppDir UserHomeDir
  ::Annotation::Init
  variable Info
  if {[info exists Info(WizardActive)]} {return}
  set Info(WizardActive) 1
  set ::WizardDialog::Wizard(CV::BagWords|CancelCommand) \
    {unset -nocomplain ::CreateVectors::BagOfWordsRepresentation::Info(WizardActive)\
                       ::CreateVectors::BagOfWordsRepresentation::Words \
                       ::CreateVectors::BagOfWordsRepresentation::Frequency2Words
     array unset ::WizardDialog::Wizard CV::BagWords|*}
  set ::WizardDialog::Wizard(CV::BagWords|BackCommand) \
    {::CreateVectors::BagOfWordsRepresentation::DisplayPage prev}
  set ::WizardDialog::Wizard(CV::BagWords|NextCommand) \
    {::CreateVectors::BagOfWordsRepresentation::DisplayPage next}
  set ::WizardDialog::Wizard(CV::BagWords|FinishCommand) \
    {::CreateVectors::BagOfWordsRepresentation::DisplayPage last
     unset -nocomplain ::CreateVectors::BagOfWordsRepresentation::Info(WizardActive)\
                       ::CreateVectors::BagOfWordsRepresentation::Frequency2Words
     array unset ::WizardDialog::Wizard CV::BagWords|*}
  GUI_LoadImage Install
  ## Set default values...
  set Info(Collection)         $col
  set Info(Document)           $doc
  set Info(WordAnnotation)     token
  set Info(WordSource)         ann_text
  set Info(WordAnnAttribute)   lemma
  set Info(FilterWords)        keep_all
  set Info(N_MostFrequent)     150
  set Info(OverFrequency)      1
  set Info(ClassAnnotation)    token
  set Info(ClassAttribute)     pos
  set Info(LeftContext)        2
  set Info(RightContext)       2
  set Info(OutputFile)         $::UserHomeDir/Vectors.arff
  set Info(OutputFileEncoding) $::KeyboardLocale
  ## Display the first page...
  set Page(CV::BagWords|Current)   0
  DisplayPage start
};# ::CreateVectors::BagOfWordsRepresentation::Init

##############################################################################
#### Page: Welcome
proc ::CreateVectors::BagOfWordsRepresentation::Page_Welcome {args} {
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 0 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Welcome to the "Create Vectors" Wizard!} \
    -message "This wizard will help you to create vectors from Annotations,\
              that can be used by various Machine Learning algorithms.\
              \n\nThe vector creation process will create vectors based on the\
              \"bag-of-words\" representation.\
              \n\nThe format of the generated vectors will be the ARFF format,\
              (a format used by the WEKA Machine Learning application)."]
};# ::CreateVectors::BagOfWordsRepresentation::Page_Welcome

##############################################################################
#### Page: SelectColDoc
proc ::CreateVectors::BagOfWordsRepresentation::Page_SelectColDoc {args} {
  variable Info
  # ## If the user has specified already a Collection, skip this page...
  # if {[string length $Info(Collection)]} {
  #   DisplayPage [lindex $args 0]
  #   return
  # }
  global GUI_Options GUI_Text
  GUI_LoadImage IForward
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Select a Collection or a Document...} \
    -message "Please select a whole Collection (or a Document) from\
              which the vectors will be generated:"]
    frame $area.select
      set ColNames [GUI_GetCollectionNames]
      label $area.select.collabel -image ICollection -compound left \
            -textvariable GUI_Text(Collection) -padx 5
      ComboBox $area.select.col -editable 0 -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
        -foreground $GUI_Options(InputForeground) -values $ColNames \
        -textvariable ::CreateVectors::BagOfWordsRepresentation::Info(Collection) \
        -modifycmd "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
          $area.select.doc $area.select.col
          $area.select.doc setvalue first
          ::WizardDialog::State $area Next normal"
      label $area.select.doclabel -image IDocument -compound left \
            -textvariable GUI_Text(Document) -padx 5
      ComboBox $area.select.doc -editable 0 -bd $GUI_Options(BorderWidth) \
        -highlightthickness 0 -entrybg $GUI_Options(InputBackground) \
        -foreground $GUI_Options(InputForeground) -values {} \
        -textvariable ::CreateVectors::BagOfWordsRepresentation::Info(Document)
      grid $area.select.collabel $area.select.col -sticky ew -pady 5
      grid $area.select.doclabel $area.select.doc -sticky ew -pady 5
      grid columnconfigure $area.select 1 -weight 1
    pack $area.select -fill both -expand 1 -padx 20
    if {![string length $Info(Collection)]} {
      ## TODO: remove comment
      #::WizardDialog::State $area Next disabled
    }
};# ::CreateVectors::BagOfWordsRepresentation::Page_SelectColDoc

##############################################################################
#### Page: CheckCollection
proc ::CreateVectors::BagOfWordsRepresentation::Page_CheckCollection {args} {
  variable Info
  ## If the user has specified already a Collection, skip this page...
  if {[string length $Info(Collection)]} {
    DisplayPage [lindex $args 0]
    return
  }
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 0 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {No Collection Selected!} \
    -message "No Collection has beed selected! Please return to the previous\
           window and select either a whole Collection or a single Document!"]
};# ::CreateVectors::BagOfWordsRepresentation::Page_CheckCollection

##############################################################################
#### Page: SelectWordAnn
proc ::CreateVectors::BagOfWordsRepresentation::Page_SelectWordAnn {args} {
  variable Info
  global GUI_Options GUI_Text
  GUI_LoadImage IForward
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Select the word Annotation...} \
    -message "According to this representation, the text is separated into\
              basic word elements. Please choose the annotation that specifies\
              what constitutes such a basic element:"]
  frame $area.frame
    label $area.frame.annlabel -anchor w -justify left -image IForward \
      -compound left -textvar GUI_Text(Annotation) -padx 5
    ComboBox $area.frame.ann -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)\
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] \
      -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(WordAnnotation) \
      -modifycmd "::GUI::FillAttributeComboFromAnnotationCombo \
                          $area.frame.ann $area.frame.attr"
    grid $area.frame.annlabel $area.frame.ann -sticky ew -pady 10
    radiobutton $area.frame.use_text -text "Use the Annotated Text as a word" \
      -value ann_text -variable \
      ::CreateVectors::BagOfWordsRepresentation::Info(WordSource) \
      -command "::CreateVectors::BagOfWordsRepresentation::DeactivateWidget \
          $area.frame.attr \
          ::CreateVectors::BagOfWordsRepresentation::Info(WordSource) ann_attr"
    radiobutton $area.frame.use_attr -text \
      "Use the value of an Attribute as a word:" -value ann_attr -variable \
      ::CreateVectors::BagOfWordsRepresentation::Info(WordSource) \
      -command "::CreateVectors::BagOfWordsRepresentation::DeactivateWidget \
          $area.frame.attr \
          ::CreateVectors::BagOfWordsRepresentation::Info(WordSource) ann_attr"
    ComboBox $area.frame.attr -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)\
      -highlightthickness 0 -editable 1 -values {} \
      -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(WordAnnAttribute)
    grid $area.frame.use_text -sticky w -columnspan 2 -pady 10
    grid $area.frame.use_attr -sticky w -columnspan 2
    grid x $area.frame.attr   -sticky snew
    grid columnconfigure $area.frame 1 -weight 1
  pack $area.frame -fill both -expand 1
  ::GUI::FillAttributeComboFromAnnotationCombo $area.frame.ann $area.frame.attr
  ::CreateVectors::BagOfWordsRepresentation::DeactivateWidget $area.frame.attr \
                ::CreateVectors::BagOfWordsRepresentation::Info(WordSource) ann_attr
};# ::CreateVectors::BagOfWordsRepresentation::Page_SelectWordAnn

proc ::CreateVectors::BagOfWordsRepresentation::DeactivateWidget {w var val} {
  if {[string equal [set $var] $val]} {
    $w configure -state normal
  } else {
    ## TODO: remove comment
    # $w configure -state disabled
    $w configure -state normal
  }
};# ::CreateVectors::BagOfWordsRepresentation::DeactivateWidget

##############################################################################
#### Page: CollectWords
proc ::CreateVectors::BagOfWordsRepresentation::Page_CollectWords {args} {
  variable Info
  global GUI_Text
  GUI_LoadImage IQuestionMark
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Collecting Words...} \
    -message "Please wait while collecting all the words from all Documents\
              of the Collection..."]
  ## TODO: remove comment
  #::WizardDialog::State $area Next disabled
  pack [label $area.curr_doc -image IDocument -compound left -anchor w \
              -text $GUI_Text(Document): -padx 10] -fill x -expand 1
  pack [ImageProgress::ImageProgress $area.progress] -fill x -expand 1 \
    -padx 10
  pack [label $area.results -anchor w -image IQuestionMark -compound left \
              -text {                   } -padx 10] -fill x -expand 1
  ## Open the Collection...
  if {[regexp {^Col\d+$} $Info(Collection)]} {
    set col $Info(Collection)
  } else {
    if {[catch {tip_OpenCollection $Info(Collection)} col]} {
      gut_info $col
      DisplayPage SelectColDoc
    }
  }
  variable Words
  variable Frequency2Words
  unset -nocomplain Words Frequency2Words
  set Progress 0.0
  set Step [expr 100.0/([tip_Length $col]+1)]
  ## Place the window underneath our dialog...
  catch {
    foreach {w h x y} [split [winfo geometry [winfo toplevel $area]] x+-] {}
    set wwin [ggi_wait_message {} 0.0]
    wm geometry $wwin +$x+[expr {$y+$h}]
  }
  ## Iterate over all Documents...
  set doc [tip_FirstDocument $col]
  while {[string length $doc]} {
    set Progress [expr {$Progress+$Step}]
    $area.curr_doc configure -text "$GUI_Text(Document): [tip_GetId $doc]"
    ImageProgress::SetImageProgress $area.progress $Progress 100.0
    ggi_wait_update {} $Progress $wwin
    update
    ## Iterate over all the Annotations...
    foreach ann [tip_SelectAnnotations $doc $Info(WordAnnotation)] {
      switch $Info(WordSource) {
        ann_text {set word [tip_GetFirstAnnotatedTextRange $doc $ann]}
        ann_attr {
          set attribute [tip_GetAttribute $ann $Info(WordAnnAttribute)]
          if {[llength $attribute]} {
            set word [tip_GetValueValue $attribute]
          } else {
            set word [tip_GetFirstAnnotatedTextRange $doc $ann]
          }
        }
      }
      if {[catch {incr Words($word)}]} {
        set Words($word) 1
      }
    };# foreach ann [tip_SelectAnnotations $doc $Info(WordAnnotation)]
    tip_Close $doc
    set doc [tip_NextDocument $col]
  }
  ## Calculate frequecy to words statistics...
  foreach word [array names Words] {
    if {[catch {incr Frequency2Words($Words($word))}]} {
      set Frequency2Words($Words($word)) 1
    }
  }
  catch {ggi_destroy_wait $wwin}
  ImageProgress::SetImageProgress $area.progress 100.0 100.0
  set Info(TotalWordsFound) [llength [array names Words]]
  $area.results configure -image IForward \
    -text "Found $Info(TotalWordsFound) words!"
  ::WizardDialog::State $area Next normal
  tip_Close $col
};# ::CreateVectors::BagOfWordsRepresentation::Page_CollectWords

##############################################################################
#### Page: FilterWords
proc ::CreateVectors::BagOfWordsRepresentation::Page_FilterWords {args} {
  variable Info
  variable Words
  variable Frequency2Words
  GUI_LoadImage IForward
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Filter Collected Words...} \
    -message "Please, use any of the following options to reduce the size\
              of the word list:"]
  
  pack [label $area.results -anchor w -image IForward -compound left \
         -text "Found $Info(TotalWordsFound) words!" -padx 10] -fill x
  
  frame $area.frame
    radiobutton $area.frame.keepall -text {Keep all words...} -value keep_all \
      -variable ::CreateVectors::BagOfWordsRepresentation::Info(FilterWords) \
      -anchor w
    radiobutton $area.frame.most_freq -text {Keep N most frequent words:} \
      -value keep_n_most_frequent -anchor w -variable \
      ::CreateVectors::BagOfWordsRepresentation::Info(FilterWords)
    set to $Info(TotalWordsFound)
    if {$to < 1} {set to 1}
    spinbox $area.frame.nfreq -from 1 -to $to -width 7 \
      -justify c -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(N_MostFrequent)
    $area.frame.nfreq set $Info(TotalWordsFound)
    radiobutton $area.frame.over_freq -text \
      {Keep words with frequency qual or greater than:} \
      -value keep_over_freq -anchor w -variable \
      ::CreateVectors::BagOfWordsRepresentation::Info(FilterWords)
    spinbox $area.frame.overfreq -state readonly -values \
      [lsort -integer [array names Frequency2Words]] -width 7 -command \
      "::CreateVectors::BagOfWordsRepresentation::Page_FilterWordsCalculateWords \
       %s $area.frame.word_statistics" -justify c -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(OverFrequency)
    label $area.frame.word_statistics
    grid $area.frame.keepall -sticky w -columnspan 2
    grid $area.frame.most_freq $area.frame.nfreq -sticky w 
    grid $area.frame.over_freq $area.frame.overfreq -sticky w
    grid $area.frame.word_statistics -columnspan 2
  pack $area.frame -fill both -expand 1
};# ::CreateVectors::BagOfWordsRepresentation::Page_FilterWords

proc ::CreateVectors::BagOfWordsRepresentation::Page_FilterWordsCalculateWords {
                                                                      freq w} {
  variable Frequency2Words
  set words 0
  foreach key [lsort -integer -decreasing [array names Frequency2Words]] {
    if {$key < $freq} {break}
    incr words $Frequency2Words($key)
  }
  $w configure -text "($words words with frequency equal or greater to $freq)"
};# ::CreateVectors::BagOfWordsRepresentation::Page_FilterWordsCalculateWords

##############################################################################
#### Page: SpecifyVector
proc ::CreateVectors::BagOfWordsRepresentation::Page_SpecifyVector {args} {
  variable Info
  global GUI_Text GUI_Options
  GUI_LoadImage IForward
  GUI_LoadImage IBack
  GUI_LoadImage IMagnifyingGlass
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 1 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Specify Vector Characteristics...} \
    -message "A vector will be created for each word that has a specific\
              annotation and an attribute of this annotation will be used as\
              the vector's class. Please select this annotation and attribute,\
              as well as the size of the context around the word that will be\
              used to extract the vector's features:"]
  frame $area.frame
    label $area.frame.annlabel -anchor w -justify left -image IForward \
      -compound left -textvar GUI_Text(Annotation) -padx 5
    ComboBox $area.frame.ann -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)\
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] \
      -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(ClassAnnotation) \
      -modifycmd "::GUI::FillAttributeComboFromAnnotationCombo \
                          $area.frame.ann $area.frame.attr"
    grid $area.frame.annlabel $area.frame.ann -sticky ew -pady 10 -padx 10
    label $area.frame.attrlabel -anchor w -justify left -image IForward \
      -compound left -textvar GUI_Text(Attribute) -padx 5
    ComboBox $area.frame.attr -entrybg $GUI_Options(InputBackground) \
      -foreground $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth)\
      -highlightthickness 0 -editable 1 -values {} \
      -textvariable ::CreateVectors::BagOfWordsRepresentation::Info(ClassAttribute)
    grid $area.frame.attrlabel $area.frame.attr -sticky ew -pady 10 -padx 10
    grid columnconfigure $area.frame 1 -weight 1
  pack $area.frame -fill both -expand 1
  ::GUI::FillAttributeComboFromAnnotationCombo $area.frame.ann $area.frame.attr
  frame $area.context
    label $area.context.a -image IForward
    spinbox $area.context.leftContext  -from 1 -to 2000 -width 7 -justify c \
      -textvariable ::CreateVectors::BagOfWordsRepresentation::Info(LeftContext)
    label $area.context.b -image IBack
    label $area.context.c -image IMagnifyingGlass
    label $area.context.d -image IForward
    spinbox $area.context.rightContext -from 1 -to 2000 -width 7 -justify c \
      -textvariable ::CreateVectors::BagOfWordsRepresentation::Info(RightContext)
    label $area.context.e -image IBack
    label $area.context.lcm -text "Left Context\n(words)"
    label $area.context.rcm -text "Right Context\n(words)"
    grid x $area.context.lcm x $area.context.c x $area.context.rcm \
         -sticky snew -padx 2
    grid $area.context.a $area.context.leftContext  $area.context.b \
                                     x \
         $area.context.d $area.context.rightContext $area.context.e \
         -sticky ew -padx 2
  pack $area.context -fill x -expand 1
};# ::CreateVectors::BagOfWordsRepresentation::Page_SpecifyVector

##############################################################################
#### Page: OutputFile
proc ::CreateVectors::BagOfWordsRepresentation::Page_OutputFile {args} {
  variable Info
  set area [::WizardDialog::Dialog -image Install -buttonvisible {1 1 0 1} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Specifying where to store Application...} \
    -message "Please specify the file name into which the generated vectors\
       will be stored.\n\nIf a file with the same file name already\
       exists, it will be overwritten.\nIf a file with the same file name does\
       not exist, it will be created." \
      -fill x -expand 0]
  frame $area.frame
    entry  $area.frame.entry -textvariable \
       ::CreateVectors::BagOfWordsRepresentation::Info(OutputFile)
    button $area.frame.button -image ITOpen -text ... -compound left \
           -command "GUI_SelectSaveFile $area.frame.entry"
    grid $area.frame.entry $area.frame.button -sticky snew -padx 3 -pady 2
    set menu $area.frame.enc.mn
    menubutton $area.frame.enc -relief raised -indicatoron 1 -anchor c \
      -menu $menu -textvariable \
      ::CreateVectors::BagOfWordsRepresentation::Info(OutputFileEncoding) \
      -text $Info(OutputFileEncoding) \
      -image [::Encoding::GetEncodingImage $Info(OutputFileEncoding)] \
      -compound left -padx 10
    menu $menu -tearoff 0
    GUI_AppendEncodingsToMenu $menu \
    "set ::CreateVectors::BagOfWordsRepresentation::Info(OutputFileEncoding)\
         \$Encoding
     $area.frame.enc configure -image \$EncodingImage -text \$Encoding"
    grid $area.frame.enc -sticky snew -padx 3 -pady 2
    grid columnconfigure $area.frame 0 -weight 1
  pack $area.frame -fill x -expand 1
  GUI_MakeEntryFileDropTarget $area.frame.entry
};# ::CreateVectors::BagOfWordsRepresentation::Page_OutputFile

  
##############################################################################
#### Page: Finish
proc ::CreateVectors::BagOfWordsRepresentation::Page_Finish {args} {
  variable Info
  variable Words
  variable Frequency2Words
  unset -nocomplain Info(WizardActive) Frequency2Words
  global GUI_Text
  set area [::WizardDialog::Dialog -image Install -buttonvisible {0 0 0 0} \
    -id CV::BagWords -title {Create Vectors Wizard!} \
    -header {Generating Vectors...} \
    -message "Please wait until the Generation process terminates..."]

  pack [label $area.curr_doc -image IDocument -compound left -anchor w \
              -text $GUI_Text(Document): -padx 10] -fill x -expand 1
  pack [ImageProgress::ImageProgress $area.progress] -fill x -expand 1 \
    -padx 10
  ## Open the Collection...
  if {[regexp {^Col\d+$} $Info(Collection)]} {
    set col $Info(Collection)
  } else {
    if {[catch {tip_OpenCollection $Info(Collection)} col]} {
      gut_info $col
      DisplayPage SelectColDoc
    }
  }
  variable Words
  ## Place the window underneath our dialog...
  catch {
    foreach {w h x y} [split [winfo geometry [winfo toplevel $area]] x+-] {}
    set wwin [ggi_wait_message {} 0.0]
    wm geometry $wwin +$x+[expr {$y+$h}]
  }
  ## Filter words...
  switch $Info(FilterWords) {
    keep_all {}
    keep_n_most_frequent {
      ## We have to sort words according to their frequency and keep the N
      ## most frequent...
      foreach word [array names Words] {
        lappend Freq2Words($Words($word)) $word
      }
      set count 0
      foreach freq [lsort -integer -decreasing [array names Freq2Words]] {
        foreach word $Freq2Words($freq) {
          if {$count == $Info(N_MostFrequent)} {break}
          set NewWords($word) $Words($word)
          incr count
        }
      }
      unset -nocomplain Words
      array set Words [array get NewWords]
      unset -nocomplain NewWords
    }
    keep_over_freq {
      ## Remove all entries below a certain frequency...
      foreach word [array names Words] {
        if {$Words($word) < $Info(OverFrequency)} {unset Words($word)}
      }
    }
  }
  ##
  ## Place the words in a sequence and remember their position, so as to be
  ## able to create the vector...
  ##
  set pos -1; set emptyVector {}
  foreach word [lsort [array names Words]] {
    set Position($word) [incr pos]
    lappend emptyVector 0
  }
  ## Iterate over all Documents...
  set singleDoc 0
  if {[string length $Info(Document)]} {
    switch -regexp $Info(Document) [list \
      {^Doc\d+$}       {set doc $Info(Document); set singleDoc 1} \
      $GUI_Text(All)   {set doc [tip_FirstDocument $col]} \
      default          {set doc [tip_GetDocument $col $Info(Document)]
                        set singleDoc 1}]
  } else {
    set doc [tip_FirstDocument $col]
  }
  set Progress 0.0
  if {$singleDoc} {
    set Step 50.0
  } else {
    set Step [expr 100.0/([tip_Length $col]+1)]
  }
  set Vectors {}
  while {[string length $doc]} {
    set Progress [expr {$Progress+$Step}]
    $area.curr_doc configure -text "$GUI_Text(Document): [tip_GetId $doc]"
    ImageProgress::SetImageProgress $area.progress $Progress 100.0
    ggi_wait_update {} $Progress $wwin
    update
    ## Iterate over all the Annotations...
    set ClassAnns [tip_SelectAnnotationsSorted $doc $Info(ClassAnnotation)]
    set WordAnns  [tip_SelectAnnotationsSorted $doc $Info(WordAnnotation)]
    set max_word  [llength $WordAnns]
    ##
    ## We have to distinguish two possibilities...
    ##
    switch $Info(WordSource) {
      ann_text {
        for {set index 0} {$index < $max_word} {incr index} {
          set ann [lindex $WordAnns $index]
          tip_GetSpanOffsets [tip_GetFirstSpan $ann] start end
          ## Get the class value...
          set class_ann [lindex [\
                         tip_AnnotationsMatchingRange $ClassAnns $start $end] 0]
          set val_attr [tip_GetAttribute $class_ann $Info(ClassAttribute)]
          if {[llength $val_attr]} {
            set class \'[string map \
               {, \\, . \\. \; \\\; : \\: ! \\! ( \\( ) \\) \" \\\" ' \\'
                \\ \\\\ % \\% \{ \\\{ \} \\\}} [tip_GetValueValue $val_attr]]\'
          } else {
            #set class \'FALSE\'
            continue
          }
          set lower_bound [expr {$index-$Info(LeftContext)}]
          set upper_bound [expr {$index+$Info(RightContext)+1}]
          set Vector $emptyVector
          ## Add left context...
          ##for {set i $lower_bound; set pos 1} {$i < $index} {incr i; incr pos}
          for {set i $lower_bound} {$i < $index} {incr i} {
            if {$i >= 0} {
              ## Is value in our word list?
              set word [tip_GetFirstAnnotatedTextRange $doc \
                                                       [lindex $WordAnns $i]]
              if {[info exists Words($word)]} {
                set Vector \
                    [lreplace $Vector $Position($word) $Position($word) 1]
              }
            }
          }
          ## Add the main value...
          ## set word [tip_GetFirstAnnotatedTextRange $doc $ann]
          ## if {[info exists Words($word)]} {
          ##   set Vector [lreplace $Vector $Position($word) $Position($word) 1]
          ## }
          ## Add right context...
          for {set i [expr {$index+1}]} {$i < $upper_bound} {incr i} {
            if {$i < $max_word} {
              ## Is value in our word list?
              set word [tip_GetFirstAnnotatedTextRange $doc \
                                                       [lindex $WordAnns $i]]
              if {[info exists Words($word)]} {
                set Vector \
                    [lreplace $Vector $Position($word) $Position($word) 1]
              }
            }
          };# for {set i [expr {$index+1}]}
          ## Finally, append the class...
          lappend Vector $class
          CDM_LappendUnique Values(class) $class
          lappend Vectors $Vector
        };# foreach ann [tip_SelectAnnotations $doc $Info(WordAnnotation)]
      }
      ann_attr {
        for {set index 0} {$index < $max_word} {incr index} {
          set ann [lindex $WordAnns $index]
          tip_GetSpanOffsets [tip_GetFirstSpan $ann] start end
          ## Get the class value...
          set class_ann [lindex [\
                         tip_AnnotationsMatchingRange $ClassAnns $start $end] 0]
          set val_attr [tip_GetAttribute $class_ann $Info(ClassAttribute)]
          if {[llength $val_attr]} {
            set class \'[string map \
               {, \\, . \\. \; \\\; : \\: ! \\! ( \\( ) \\) \" \\\" ' \\'
                \\ \\\\ % \\% \{ \\\{ \} \\\}} [tip_GetValueValue $val_attr]]\'
          } else {
            #set class \'FALSE\'
            continue
          }
          set lower_bound [expr {$index-$Info(LeftContext)}]
          set upper_bound [expr {$index+$Info(RightContext)+1}]
          set Vector $emptyVector
          ## Add left context...
          for {set i $lower_bound} {$i < $index} {incr i} {
            if {$i >= 0} {
              set attribute [tip_GetAttribute \
                           [lindex $WordAnns $i] $Info(WordAnnAttribute)]
              if {[llength $attribute]} {
                ## Is value in our word list?
                set word [tip_GetValueValue $attribute]
                if {[info exists Words($word)]} {
                  set Vector \
                      [lreplace $Vector $Position($word) $Position($word) 1]
                }
              }
            }
          };# for {set i $lower_bound; set pos 1}...
          ## Add the main value...
          ## set attribute [tip_GetAttribute $ann $Info(WordAnnAttribute)]
          ## if {[llength $attribute]} {
          ##   set word [tip_GetValueValue $attribute]
          ##   if {[info exists Words($word)]} {
          ##     set Vector \
          ##         [lreplace $Vector $Position($word) $Position($word) 1]
          ##   }
          ## }
          ## Add right context...
          for {set i [expr {$index+1}]} {$i < $upper_bound} {incr i} {
            if {$i < $max_word} {
              set attribute [tip_GetAttribute \
                           [lindex $WordAnns $i] $Info(WordAnnAttribute)]
              if {[llength $attribute]} {
                ## Is value in our word list?
                set word [tip_GetValueValue $attribute]
                if {[info exists Words($word)]} {
                  set Vector \
                      [lreplace $Vector $Position($word) $Position($word) 1]
                }
              }
            }
          };# for {set i [expr {$index+1}]}
          ## Finally, append the class...
          lappend Vector $class
          CDM_LappendUnique Values(class) $class
          lappend Vectors $Vector
        };# foreach ann [tip_SelectAnnotations $doc $Info(WordAnnotation)]
      }
    };# switch $Info(WordSource)
    tip_Close $doc
    if {$singleDoc} {
      break
    } else {
      set doc [tip_NextDocument $col]
    }
  }
  array unset ::WizardDialog::Wizard CV::BagWords|*
  ImageProgress::SetImageProgress $area.progress 100.0 100.0
  if {![info exists Values(class)]} {
    gut_info "No values were found for the Class feature. As a result no\
              vectors will be generated."
  } else {
    ##
    ## Write the file...
    ##
    set file [open $Info(OutputFile) w]
    fconfigure $file -encoding $Info(OutputFileEncoding)
    ## Write Comments...
    puts $file \
    "% Vectors Generated by $::ApplicationName, version $::ApplicationVersion\
   \n%\
   \n% Vectors generated from Collection: [tip_GetName $col]\
   \n%      Creator: Create Vectors Tool... (Bag of Words Representation)\
   \n%      Date: [clock format [clock seconds] \
                         -format {%H:%M:%S, %A %d %B %Y} -gmt 1] GMT\
   \n%      File encoding: '$Info(OutputFileEncoding)'\
   \n%\
   \n@RELATION \'[tip_GetName $col]\'
   \n"
    set pos 0
    foreach word [lsort [array names Words]] {
      set word [string map \
                 {, \\, . \\. \; \\\; : \\: ! \\! ( \\( ) \\) \" \\\" ' \\'
                  \\ \\\\ % \\% \{ \\\{ \} \\\}} $word]
      puts $file "@ATTRIBUTE \'[format %.2d [incr pos]]_$word\' NUMERIC"
    }
    puts $file "@ATTRIBUTE class \{[join [lsort $Values(class)] ,]\}"
    puts $file "\n@DATA"
    foreach vector $Vectors {
      puts $file [join $vector ,]
    }
    close $file
  }
  tip_Close $col
  catch {ggi_destroy_wait $wwin}
  unset -nocomplain Info(WizardActive) Words Frequency2Words
  update
  destroy [winfo toplevel $area]
};# ::CreateVectors::BagOfWordsRepresentation::Page_Finish

## ::CreateVectors::BagOfWordsRepresentation::DisplayPage
#  This function will display a page according to the current displayed page.
#  It accepts an argumend which can have one of the following values:
#  "start", "next", "prev", "last"
proc ::CreateVectors::BagOfWordsRepresentation::DisplayPage {page} {
  variable Page
  variable Pages

  #if {![string equal $page last] && [string equal $Page(Current) end]} {return}
  switch $page {
    start {set  Page(CV::BagWords|Current)   0}
    next  {incr Page(CV::BagWords|Current)    }
    prev  {incr Page(CV::BagWords|Current)  -1}
    last  {set  Page(CV::BagWords|Current) end}
    default {
      ## This is one of the pages...
      set Page(CV::BagWords|Current) [lsearch $Pages $page]
    }
  }
  ## Make some routine checks...
  if {![string equal $Page(CV::BagWords|Current) end]} {
    if {$Page(CV::BagWords|Current) < 0} {set Page(CV::BagWords|Current) 0}
    if {$Page(CV::BagWords|Current) >= [llength $Pages]} {return}
  }
  ## Run the function for the specific page...
  ::CreateVectors::BagOfWordsRepresentation::Page_[lindex \
                   $Pages $Page(CV::BagWords|Current)] $page
};# ::CreateVectors::BagOfWordsRepresentation::DisplayPage

#
# End of File CreateVectors_BagOfWordsRepresentation.tcl
#

