##########################################################################
# 
#       EditAnnotationHierarchies.tcl - Thursday January 23 14:52:50 (EET) 2003
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.5...
#
##########################################################################


# Location of this module (EditAnnotationHierarchies)
global creole_EditAnnotationHierarchies_home

namespace eval ::EditAnnotationHierarchiesTool {
  variable Parameters
  variable ValuesSpec
}

## Load some predifined Values...
proc ::EditAnnotationHierarchiesTool::Init {} {
  variable Parameters
  ::Annotation::Init
  if {![info exists ::creole_EditAnnotations_home]} {
    set ::creole_EditAnnotations_home $::creole_EditAnnotationHierarchies_home
  }
  foreach filename [list \
              $::creole_EditAnnotations_home/PredifinedValues.tcl \
              $::creole_EditAnnotationHierarchies_home/PredifinedValues.tcl \
              $::UserAppDir/EditAnnotations.cfg \
              $::UserAppDir/EditAnnotationHierarchies.cfg \
              $::UserHomeDir/.EditAnnotations.cfg \
              $::UserHomeDir/.EditAnnotationHierarchies.cfg] {
    if {![file exists $filename]} {continue}
    set file [open $filename]
      fconfigure $file -encoding utf-8
      set data [read $file]
    close $file
    if {[catch {eval $data} error]} {
      set msg "The following error occured while reading predifined values\
               from file \"$filename\":\n$error"
      catch {bgerror $msg}
    }
  }
  ## Load the images...
  set Parameters(UseImages) 1
  for {set i 0} {$i <= 15} {incr i} {
    if {[catch {image create photo ::EditAnnotationHierarchiesTool::Image$i \
                -file $::creole_EditAnnotationHierarchies_home/Nodes$i.png}]} {
      ## Something went wrong when loading the images. Disable this feature...
      set Parameters(UseImages) 0
    }
  }
};# ::EditAnnotationHierarchiesTool::Init

## creole_EditAnnotationHierarchies
#  
proc creole_EditAnnotationHierarchies {doc col} {
  package require Tree
  ::EditAnnotationHierarchiesTool::Init
  ## Create the main window...
  set win [ggi_unused_win .tooleditannotations_main_window]
  array unset ::EditAnnotationHierarchiesTool::Parameters $win:*

  set ::EditAnnotationHierarchiesTool::Parameters($win:Document)   $doc
  set ::EditAnnotationHierarchiesTool::Parameters($win:Collection) $col
  set ::EditAnnotationHierarchiesTool::Parameters($win:Annotation) syntax
  set ::EditAnnotationHierarchiesTool::Parameters($win:Attribute)  category
  ::EditAnnotationHierarchiesTool::GetParameters $win $doc $col
  ## Has the user cancelled the procedure?
  if {$::EditAnnotationHierarchiesTool::Parameters($win:Cancel)} {
    array unset ::EditAnnotationHierarchiesTool::Parameters $win:*
    return
  }
  if {![string length \
               $::EditAnnotationHierarchiesTool::Parameters($win:Annotation)]} {
    return
  }
  set Annotation $::EditAnnotationHierarchiesTool::Parameters($win:Annotation)
  set Attribute  $::EditAnnotationHierarchiesTool::Parameters($win:Attribute)
  set Values     [split \
                 $::EditAnnotationHierarchiesTool::Parameters($win:UserValues)]
  set Values     [lsort -unique $Values]
  ## If in values we got the empty string, remove it...
  set index [lsearch -exact $Values {}]
  if {$index != -1} {set Values [lreplace $Values $index $index]}
  set ::EditAnnotationHierarchiesTool::Parameters($win:UserValues) $Values
  set Language   $::EditAnnotationHierarchiesTool::Parameters($win:Language)
  #if {![string length $doc]} {
  #  set col {}
  #  set ::EditAnnotationHierarchiesTool::Parameters($win:Collection) {}
  #}
  ::EditAnnotationHierarchiesTool::CreateWindow $win $doc $col \
                 $Annotation $Attribute $Values $Language
  tkwait window $win

  array unset ::EditAnnotationHierarchiesTool::Parameters $win:*
};# creole_EditAnnotationHierarchies

## ::EditAnnotationHierarchiesTool::DocLoad
#   Load the requested document...
proc ::EditAnnotationHierarchiesTool::DocLoad {win ColW DocW {what This}} {
  global GUI_Text
  variable Parameters
  set Col [$ColW getvalue]
  set Doc [$DocW getvalue]

  ## Warn user if Document is modified, before we close it...
  if {[llength [info commands $Parameters($win:Document)]]} {
    ## Clear any images that may be associated with the Document...
    catch {
      ggi_view_documentHTMLRetrieveImagesClean {} $Parameters($win:HtmlViewer)
    }
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  switch $what {
    Prev    {incr Doc -1; $DocW setvalue previous}
    Next    {incr Doc;    set NEXT [$DocW setvalue next]
             if {$NEXT == 0} {return}}
    default {}
  }

  if {$Col == -1 || $Doc == -1} {return}

  catch {tip_Close $Parameters($win:Document)}
  set col $Parameters($win:Collection)
  set doc $Parameters($win:Document)
  if {!$Parameters($win:SystemOwnsCollection)} {
    set Col [lindex [GUI_GetCollectionNames] $Col]
    if {[string length $col]} {
      if {![string equal [file tail [tip_GetName $col]] $Col]} {
        tip_Close $col
        set col [tip_OpenCollection $Col]
      }
    } else {
      set col [tip_OpenCollection $Col]
    }
    ## Is the Collection's Location in accordance with it's Name?
    set ColName [tip_GetName $col]
    if {![string equal $ColName $Col]} {
      GUI_FixCollectionName $col $ColName $Col
    }
  }
  
  set Doc [lindex [$DocW cget -values] $Doc]
  set doc [tip_GetDocument $col $Doc]
  set Parameters($win:Document) $doc
  set Parameters($win:Collection) $col
  set Annotation $Parameters($win:Annotation)
  set Attribute  $Parameters($win:Attribute)
  set Values     $Parameters($win:UserValues)
  set Language   $Parameters($win:Language)

  UpdateWindow $win $doc $col \
                 $Annotation $Attribute $Values $Language
  set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  if {!$Parameters($win:SystemOwnsCollection)} {
    bind $win <Destroy> "catch \"tip_Close $col\""
  }
};# ::EditAnnotationHierarchiesTool::DocLoad

## ::EditAnnotationHierarchiesTool::DocSave
#    Save current document...
proc ::EditAnnotationHierarchiesTool::DocSave {win} {
  variable Parameters
  if {[string length $Parameters($win:Document)]} {
    tip_Sync $Parameters($win:Document)
  }
};# ::EditAnnotationHierarchiesTool::DocSave

## ::EditAnnotationHierarchiesTool::GetParameters
#   Displays a small window, where the user can type the annotation and the
#   the annotation attribute (optional) he wants to edit.
proc ::EditAnnotationHierarchiesTool::GetParameters {win doc col} {
  global GUI_Text GUI_Options
  variable Parameters

  toplevel $win
  wm withdraw $win
  wm title $win "Edit Annotation Hierachies Tool Options..."

  label $win.title -text {Edit Annotation Hierachies Tool Options:}
  pack $win.title -side top -fill x

  set frame $win.frame
  frame $frame -borderwidth 2 -relief groove
    label $frame.ltoken -text {Take word units from Annotation:} -anchor e
    ComboBox $frame.token -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:WordAnnotation) \
      -modifycmd "::GUI::FillAttributeComboFromAnnotationCombo \
                  $frame.token $frame.tokenattr"
    set ::EditAnnotationHierarchiesTool::Parameters($win:WordAnnotation) token
    grid $frame.ltoken $frame.token \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.ltokenattr -text {Display word units Attribute:} -anchor e
    ComboBox $frame.tokenattr -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:WordAnnotationAttr)
    ::GUI::FillAttributeComboFromAnnotationCombo $frame.token $frame.tokenattr
    set ::EditAnnotationHierarchiesTool::Parameters($win:WordAnnotationAttr) {}
    grid $frame.ltokenattr $frame.tokenattr \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lsent -text {Take sentence units from Annotation:} -anchor e
    ComboBox $frame.sent -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:SentenceAnnotation)
    set ::EditAnnotationHierarchiesTool::Parameters($win:SentenceAnnotation) \
        sentence
    grid $frame.lsent $frame.sent \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lann -text {Edit Annotation:} -anchor e
    ComboBox $frame.ann -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:Annotation) \
      -modifycmd \
      "::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr"
    grid $frame.lann $frame.ann -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lattr -text {Annotation Attribute:} -anchor e
    ComboBox $frame.attr -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -textvariable ::EditAnnotationHierarchiesTool::Parameters($win:Attribute)
    ::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr
    set ::EditAnnotationHierarchiesTool::Parameters($win:Attribute) category
    grid $frame.lattr $frame.attr -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.vals -text {Additional Attribute Values:} -anchor e
    entry $frame.vals_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:UserValues)
    grid $frame.vals $frame.vals_e \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lang -text {Language:} -anchor e
    entry $frame.lang_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      ::EditAnnotationHierarchiesTool::Parameters($win:Language)
    set language [tip_GetAttribute $doc language]
    if {![string length $language]} {
      global Locale
      set language [string tolower $Locale]
    } else {
      set language [lindex [tip_GetValue $language] end]
    }
    if {![string length $language]} {
      global Locale
      set language [string tolower $Locale]
    }
    switch [string tolower $language] {
      greek   {set language hellenic}
      default {}
    }
    set Parameters($win:Language) $language
    grid $frame.lang $frame.lang_e \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    grid columnconfigure $frame 0 -weight 0 
    grid columnconfigure $frame 1 -weight 1 
    grid columnconfigure $frame 2 -weight 0 
  pack $frame -fill both -expand 1 -ipadx 5 -ipady 5 -padx 3 -pady 3

  set bframe $win.buttons
  frame $bframe
    button $bframe.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
      -default active
    bind $win <Return> "$bframe.ok invoke"
    button $bframe.cancel -textvariable GUI_Text(Cancel) -command \
     "set ::EditAnnotationHierarchiesTool::Parameters($win:Cancel) 1
      destroy $win"
    pack $bframe.ok $bframe.cancel -padx 3 -pady 5 -side left
  pack $bframe
  focus $frame.ann
  gut_ModalDlg $win
  wm deiconify $win
  set Parameters($win:Cancel) 0
  tkwait window $win
}; #::EditAnnotationHierarchiesTool::GetParameters

## ::EditAnnotationHierarchiesTool::CreateWindow
proc ::EditAnnotationHierarchiesTool::CreateWindow {win doc col 
                  {Annotation {}} {Attribute {}} {Values {}} {Language {}}} {
  package require BWidget
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec
  set title "Edit Annotation Hierachies Tool: Editing Attribute \"$Attribute\"\
             of Annotation \"$Annotation\""

  GUI_LoadImage BackNice
  GUI_LoadImage NextNice
  GUI_LoadImage ITinyDoc
  GUI_LoadImage IWWW
  GUI_LoadImage ITShowTags
  GUI_LoadImage ITStar
  set HaveSystemCol 0
  set HaveSystemDoc 0
  set Parameters($win:SystemOwnsCollection) 0
  if {[string equal $doc $GUI_Text(All)]} {set doc {}}
  if {[string length $col] && ![string length $doc]} {
    set doc [tip_FirstDocument $col]
    set Parameters($win:Document) $doc
    set Parameters($win:SystemOwnsCollection) 1
    set HaveSystemCol 1
  }
  if {![string length $col] || ![string length $doc] || \
      ![string length $Annotation] || ![string length $Attribute]} {
    set Sentences   {}
    set Tokens      {}
    set Annotations {}
    set DocText     {}
  } else {
    ## Get Document Annotations and text
    if {[catch {tip_GetByteSequence $doc} DocText]} {
      error "creole_EditAnnotationHierarchies: Invalid Document!\n($DocText)"
    }
    set Sentences   [tip_SelectAnnotationsSorted $doc \
                      $Parameters($win:SentenceAnnotation)]
    set Tokens      [tip_SelectAnnotationsSorted $doc \
                      $Parameters($win:WordAnnotation)]
    set Annotations [tip_SelectAnnotations $doc $Annotation \
                       "\[string length ann::${Attribute} \]"]
    set HaveSystemDoc 1
  }
  set values $Values
  if {[info exists ValuesSpec($Language:$Annotation:$Attribute)]} {
    foreach item $ValuesSpec($Language:$Annotation:$Attribute) {
      foreach {type description} $item {break}
      if {[string equal $type ---]} {continue}
      lappend values $type
      set Description($type) $description
    }
  }

  ## Create the window
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Status Bar
  if {$HaveSystemDoc} {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  } else {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):( $GUI_Text(none) )    \
       $GUI_Text(Document):( $GUI_Text(none) )"
  }
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  GUI_CreateStatusBar $win.statusbar
  pack forget $win.statusbar

  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
    ## Open Document/Collection
    set toolbar $toolbar_frame.express
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -float 0
    if {$HaveSystemCol || !$HaveSystemDoc} {
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Collection) -tooltipvar GUI_Text(TSelectCollection) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {!$HaveSystemCol} {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -entercmd \
           {%W configure -values [GUI_GetCollectionNameTails]} -modifycmd \
           "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
            $toolbar.widgets.document $toolbar.widgets.collection
            $toolbar.widgets.document configure -values \
            \[lrange \[$toolbar.widgets.document cget -values\] 1 end\]
            $toolbar.widgets.document setvalue first" \
           -tooltipvar GUI_Text(TSelectCollection) -width \
           $GUI_Options(SystemExprToolbarCollectionWidth) -takefocus 0 -pady 3 \
           -statusvar ::GUI_StatusBar::StatusBar($win)
      } else {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -takefocus 0 -pady 3 \
           -values [list [file tail [tip_GetName $col]]] \
           -width $GUI_Options(SystemExprToolbarCollectionWidth)
           $toolbar.widgets.collection setvalue first
           $toolbar.widgets.collection configure -state disabled
      }
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Document) -tooltipvar GUI_Text(TSelectDocument) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -name $toolbar.widgets.document \
         -tooltipvar GUI_Text(TSelectDocument) -width \
         $GUI_Options(SystemExprToolbarDocumentWidth) -takefocus 0 -pady 3 \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {$HaveSystemCol} {
        $toolbar.widgets.document configure -values [$col GetFileIds]
        $toolbar.widgets.document setvalue first
      }
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
         -textvariable GUI_Text(Load) -image ITLoad -command \
         "::EditAnnotationHierarchiesTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document" \
         -tooltipvar GUI_Text(TSelectedLoad) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -image BackNice -command \
         "::EditAnnotationHierarchiesTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Prev"
      Toolbar::addbutton $toolbar -image NextNice -command \
         "::EditAnnotationHierarchiesTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Next"
    }
    Toolbar::addbutton $toolbar -image ITSave -tooltipvar GUI_Text(Save) \
       -statusvar ::GUI_StatusBar::StatusBar($win) \
       -command "::EditAnnotationHierarchiesTool::DocSave $win"
    Toolbar::addseparator $toolbar
    wm protocol $win WM_DELETE_WINDOW \
      "::EditAnnotationHierarchiesTool::CloseWindow $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command \
       "::EditAnnotationHierarchiesTool::CloseWindow $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    set ActiveLabel [Toolbar::addwidget $toolbar label -relief flat -bd 0 \
       -anchor e]
    pack $toolbar -side top -anchor w
  grid $toolbar_frame -sticky snew

  ## Create the main window area...
  frame $win.area
    ## The sentence selection area...
    set SentenceViewer $win.area.sentences
    NoteBook $SentenceViewer -background [$win cget -background]
    set sentenceFrame [$SentenceViewer insert end sentenceList -text \
                       \ $Parameters($win:SentenceAnnotation) -image ITShowTags]
    #::ScrolledWidgets::ScrolledText $SentenceViewer -height 4 -wrap none
    set sw [ScrolledWindow $sentenceFrame.sw -relief flat -borderwidth 0]
    set sf [text $sentenceFrame.sw.text -height 6 -wrap none]
    $sw setwidget $sf
    pack $sentenceFrame.sw -fill both -expand 1

    ## The Document text area, where the user can select the text to edit...
    set textFrame [$SentenceViewer insert end text -text \ $GUI_Text(Text) \
                                   -image ITinyDoc]
    set sw [ScrolledWindow $textFrame.sw -relief flat -borderwidth 0]
    set sf [text $textFrame.sw.text -height 6 -exportselection 0]
    $sw setwidget $sf
    $textFrame.sw.text insert end $DocText
    $textFrame.sw.text configure -state disabled
    pack $textFrame.sw -fill both -expand 1

    ## If Document contains Html, display a rendering...
    set htmlFrame [$SentenceViewer insert end html -text \ Html -image IWWW]
    set sw [ScrolledWindow $htmlFrame.sw -relief flat -borderwidth 0]
    set sf [html $htmlFrame.sw.text -height 6 \
      -bg $GUI_Options(HtmlBg) \
      -borderwidth $GUI_Options(TextBorderWidth) -exportselection 0 \
      -fg $GUI_Options(HtmlFg) -relief $GUI_Options(TextRelief) \
      -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
      -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
      -fontcommand    "::HtmlInfoViewer::PickFont $htmlFrame.sw.text" \
      -imagecommand   "ggi_view_documentHTMLRetrieveImages $doc \
                                                           $htmlFrame.sw.text" \
      -formcommand    "::HtmlInfoViewer::FormCmd $htmlFrame.sw.text" \
      -bgimagecommand "ggi_view_documentHTMLRetrieveBgImages $doc \
                                                           $htmlFrame.sw.text"]
    if {[ggi_view_IsHTMLText $DocText]} {
      catch {$htmlFrame.sw.text configure -base [tip_GetExternalId $doc]}
      $htmlFrame.sw.text parse $DocText
    } else {
      $SentenceViewer itemconfigure html -state disabled
    }
    ::HtmlInfoViewer::DefaultBingings $htmlFrame.sw.text
    ## Disable hyperlinks...
    bind $htmlFrame.sw.text.x <<MB_1>> \
      "set ::HtmlInfoViewer::InfoViewer($htmlFrame.sw.text::SelectionStart) \
                                        \[$htmlFrame.sw.text index @%x,%y\]
       $htmlFrame.sw.text selection clear"
    $sw setwidget $sf
    pack $htmlFrame.sw -fill both -expand 1
    
    ## The canvas for editing the hierarchies...
    set HierarchiesViewer $win.area.hierarchies
    set sw [ScrolledWindow $HierarchiesViewer -relief flat -borderwidth 0]
    set sf [canvas $sw.canvas -background $GUI_Options(TextBg)]
    $sw setwidget $sf
    #::ScrolledWidgets::ScrolledCanvas $HierarchiesViewer \
    #  -background $GUI_Options(TextBg) -regionpadx 2 -regionpady 2
    bind $HierarchiesViewer.canvas <<MB_1>> \
      "::EditAnnotationHierarchiesTool::DeselectEverything $win"
    tree $HierarchiesViewer.canvas.tree -layout vertical
    ## Fill the sentence selection area with sentences...
    foreach sentence $Sentences {
      set ID       [tip_GetId $sentence]
      set text     [tip_GetFirstAnnotatedTextRange $doc $sentence]
      $sentenceFrame.sw.text insert end \
        "$ID. [string map {\n \  \t \  \r \ } $text]\n" id_$ID
      $sentenceFrame.sw.text tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationHierarchiesTool::SelectSentence $win id_$ID]
      $sentenceFrame.sw.text tag bind id_$ID <Enter> \
        "::EditAnnotationHierarchiesTool::HilightSentence \
          $win id_$ID yellow #a020f0"
      $sentenceFrame.sw.text tag bind id_$ID <Leave> \
        "::EditAnnotationHierarchiesTool::HilightSentence \
          $win id_$ID $GUI_Options(TextBg) $GUI_Options(TextFg)"
    }
    $sentenceFrame.sw.text delete end-1c end
    $sentenceFrame.sw.text configure -state disabled
    set Parameters($win:NoteBook)          $SentenceViewer
    set Parameters($win:SentenceViewer)    $sentenceFrame.sw.text
    set Parameters($win:TextViewer)        $textFrame.sw.text
    set Parameters($win:HtmlViewer)        $htmlFrame.sw.text
    set Parameters($win:HierarchiesViewer) $HierarchiesViewer.canvas
    set Parameters($win:TreeWidget)        $HierarchiesViewer.canvas.tree
    set Parameters($win:ActiveLabel)       $ActiveLabel
    set Parameters($win:ImageLabel)        $win.area.selectedImage
    set Parameters($win:HierarchiesViewer:SelectedBG) red
    set Parameters($win:HierarchiesViewer:SelectedFG) white
    set Parameters($win:HierarchiesViewer:BG)         yellow
    set Parameters($win:HierarchiesViewer:FG)         navy
    set Parameters($win:HierarchiesViewer:SelNuBG)    #90ee90
    set Parameters($win:HierarchiesViewer:SelNuFG)    #a52a2a
    ## The buttons area...
    set BFrame $win.area.buttons
    frame $BFrame -relief flat
    grid $HierarchiesViewer -sticky snew -padx 1
    ## The selected image area...
    if {$Parameters(UseImages)} {
      label $Parameters($win:ImageLabel) \
        -image ::EditAnnotationHierarchiesTool::Image0
      grid $BFrame -row 0 -column 1 -sticky snew -padx 1 -pady 1
      grid $Parameters($win:ImageLabel) -row 1 -column 1 -sticky snew
    } else {
      grid $BFrame -row 0 -column 1 -sticky snew -rowspan 2 -padx 1 -pady 1
    }
    grid $SentenceViewer -row 1 -column 0 -sticky snew -padx 1
    grid columnconfigure $win.area 0 -weight 1
    grid rowconfigure $win.area 0 -weight 1
  grid $win.area -sticky snew
  grid $win.statusbar -sticky snew
  grid columnconfigure $win 0 -weight 1
  grid rowconfigure $win 1 -weight 1

  ## Add the Vertical Toolbar, that the user can change type in selected
  ## objects...
  set FFrame $BFrame.functionButtons
  frame $FFrame
    button $FFrame.del_ann -text {Delete Annotation} -background red \
      -foreground white -command \
      "::EditAnnotationHierarchiesTool::Delete $win Annotation"
    button $FFrame.del_attr -text {Delete Attribute} -background orange \
      -foreground white -command "::EditAnnotationHierarchiesTool::Delete $win"
    button $FFrame.autotag -text {Automatic Tag} -background #6b8e23 \
      -foreground white -command "::EditAnnotationHierarchiesTool::AutoTag $win"
    #grid $FFrame.autotag  -sticky snew -padx 1 -pady 1
    #grid $FFrame.del_attr -sticky snew -padx 1 -pady 1
    grid $FFrame.del_ann  -sticky snew -padx 1 -pady 1
    grid columnconfigure $FFrame 0 -weight 1
    bind $FFrame.del_ann <<MB_2>> \
      "::EditAnnotationHierarchiesTool::EditAccelerator %W"

  ## Collect all Attribute values...
  if {[llength $Annotations]} {
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      ## Add a tag
      CDM_LappendUnique values $value
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}

  ##
  ## The values buttons...
  ##
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set values [lsort -unique -ascii $values]
  set row 0; set col 0; set ci 0
  foreach value $values {
    if {$ci == $color_nu} {set ci 0}
    foreach {bg fg} [lindex $Colors $ci] {break}
    set button $BFrame.${row}$col
    set Parameters($win:Colour:$value) [lindex $Colors $ci]
    set Parameters($win:Button:$value) $button
    button $button -text $value  -foreground black -highlightthickness 0\
      -activeforeground $fg -activebackground $bg -command \
      [list ::EditAnnotationHierarchiesTool::Tag $win $button $value]
    ## Add a binding for setting an accelarator...
    bind $button <<MB_2>> "::EditAnnotationHierarchiesTool::EditAccelerator %W"
    bind $button <Enter> "%W configure -background $bg -foreground $fg"
    bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
    grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
    incr col; incr ci
    catch {Tooltip::tooltip $button -text $Description($value)}
    if {$col == 5} {
      incr row; set col 0
    }
  }
  grid $FFrame -columnspan 5 -sticky snew
  ## Add two arrow button for changing selected sentences...
  frame $HierarchiesViewer.sel -borderwidth 0
  GUI_LoadImage ITDown
  GUI_LoadImage ITUp
  button $HierarchiesViewer.sel.prev -image ITUp -highlightthickness 0 -command\
   "::EditAnnotationHierarchiesTool::AdvanceSelectedSentence $win prev"
  button $HierarchiesViewer.sel.next -image ITDown -highlightthickness 0 \
    -command \
    "::EditAnnotationHierarchiesTool::AdvanceSelectedSentence $win next"
  bind $HierarchiesViewer.sel.prev <<MB_2>> \
      {::EditAnnotationHierarchiesTool::EditAccelerator %W}
  bind $HierarchiesViewer.sel.next <<MB_2>> \
      {::EditAnnotationHierarchiesTool::EditAccelerator %W}
  pack $HierarchiesViewer.sel.prev $HierarchiesViewer.sel.next -padx 0 -pady 0

  ## Add a button for editing the selection in Text & Html widgets...
  button $HierarchiesViewer.sel2edit -image ITStar -text Edit\ Selection \
    -command "" -compound left
  $SentenceViewer itemconfigure text -raisecmd \
     "$HierarchiesViewer.sel2edit configure -command \
      \"::EditAnnotationHierarchiesTool::EditSelectedText $win text\"
      place $HierarchiesViewer.sel2edit -in $HierarchiesViewer.canvas \
            -relx 0.0 -rely 1.0 -anchor sw" -leavecmd \
     "place forget $HierarchiesViewer.sel2edit"
  $SentenceViewer itemconfigure html -raisecmd \
     "$HierarchiesViewer.sel2edit configure -command \
      \"::EditAnnotationHierarchiesTool::EditSelectedText $win html\"
      place $HierarchiesViewer.sel2edit -in $HierarchiesViewer.canvas \
            -relx 0.0 -rely 1.0 -anchor sw" -leavecmd \
     "place forget $HierarchiesViewer.sel2edit"

  $SentenceViewer itemconfigure sentenceList -raisecmd \
     "place $HierarchiesViewer.sel -in $HierarchiesViewer.canvas \
            -relx 0.0 -rely 1.0 -anchor sw" -leavecmd \
     "place forget $HierarchiesViewer.sel"
  $SentenceViewer raise sentenceList

  bind $HierarchiesViewer.sel2edit <Destroy> [list \
      ggi_view_documentHTMLRetrieveImagesClean {} $Parameters($win:HtmlViewer)]

  gut_ModalDlg $win
  wm deiconify $win
}; # ::EditAnnotationHierarchiesTool::CreateWindow

## ::EditAnnotationHierarchiesTool::UpdateWindow
proc ::EditAnnotationHierarchiesTool::UpdateWindow {win doc col 
                  {Annotation {}} {Attribute {}} {Values {}} {Language {}}} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec
  ## Get Document Annotations and text
  if {[catch {tip_GetByteSequence $doc} DocText]} {
    error "creole_EditAnnotationHierarchies: Invalid Document!\n($DocText)"
  }
  set Sentences   [tip_SelectAnnotationsSorted $doc \
                      $Parameters($win:SentenceAnnotation)]
  set Tokens      [tip_SelectAnnotationsSorted $doc \
                    $Parameters($win:WordAnnotation)]
  set Annotations [tip_SelectAnnotations $doc $Annotation \
                     "\[string length ann::${Attribute} \]"]
  set values $Values
  if {[info exists ValuesSpec($Language:$Annotation:$Attribute)]} {
    foreach item $ValuesSpec($Language:$Annotation:$Attribute) {
      foreach {type description} $item {break}
      if {[string equal $type ---]} {continue}
      lappend values $type
      set Description($type) $description
    }
  }

  set HierarchiesViewer $win.area.hierarchies
  set NoteBook          $Parameters($win:NoteBook)
  set SentenceViewer    $Parameters($win:SentenceViewer)
  set TextViewer        $Parameters($win:TextViewer)
  set HtmlViewer        $Parameters($win:HtmlViewer)
  ## Destroy & re-create the canvas widget...
  place forget $HierarchiesViewer.sel
  destroy $HierarchiesViewer.canvas
  canvas $HierarchiesViewer.canvas -background $GUI_Options(TextBg)
  $HierarchiesViewer setwidget $HierarchiesViewer.canvas
  #::ScrolledWidgets::ScrolledCanvas $HierarchiesViewer \
  #    -background $GUI_Options(TextBg) -regionpadx 2 -regionpady 2
  grid $HierarchiesViewer -sticky snew -padx 1 -row 0 -column 0
  bind $HierarchiesViewer.canvas <<MB_1>> \
      "::EditAnnotationHierarchiesTool::DeselectEverything $win"
  tree $HierarchiesViewer.canvas.tree -layout vertical
  DeselectEverything $win
  ## The Text selection area...
  $TextViewer configure -state normal
  $TextViewer delete 1.0 end
  $TextViewer insert end $DocText
  $TextViewer configure -state disabled

  ## The Html selection area...
  if {[ggi_view_IsHTMLText $DocText]} {
    $NoteBook itemconfigure html -state normal
    catch {$HtmlViewer configure -base [tip_GetExternalId $doc]}
    $HtmlViewer configure -imagecommand \
       "ggi_view_documentHTMLRetrieveImages $doc $HtmlViewer" -bgimagecommand \
       "ggi_view_documentHTMLRetrieveBgImages $doc $HtmlViewer"
    $HtmlViewer clear
    $HtmlViewer parse $DocText
  } else {
    $NoteBook itemconfigure html -state disabled
  }
  
  ## The sentence selection area...
  $SentenceViewer configure -state normal
  $SentenceViewer delete 1.0 end
    foreach tag [$SentenceViewer tag names] {
      $SentenceViewer tag delete $tag
    }
    ## Fill the sentence selection area with sentences...
    foreach sentence $Sentences {
      set ID       [tip_GetId $sentence]
      set text     [tip_GetFirstAnnotatedTextRange $doc $sentence]
      $SentenceViewer insert end \
        "$ID. [string map {\n \  \t \  \r \ } $text]\n" id_$ID
      $SentenceViewer tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationHierarchiesTool::SelectSentence $win id_$ID]
      $SentenceViewer tag bind id_$ID <Enter> \
        "::EditAnnotationHierarchiesTool::HilightSentence \
          $win id_$ID yellow #a020f0"
      $SentenceViewer tag bind id_$ID <Leave> \
        "::EditAnnotationHierarchiesTool::HilightSentence \
          $win id_$ID $GUI_Options(TextBg) $GUI_Options(TextFg)"
    }
    $SentenceViewer delete end-1c end
  $SentenceViewer configure -state disabled
  set Parameters($win:EditingTokenSetSpan) {}
  ## The buttons area...
  set BFrame $win.area.buttons

  ## Collect all Attribute values...
  if {[llength $Annotations]} {
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      ## Add a tag
      CDM_LappendUnique values $value
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}

  ## Add the Vertical Toolbar, that the user can change type in selected
  ## objects...
  set FFrame $BFrame.functionButtons
  grid forget $FFrame
  ##
  ## The values buttons...
  ##
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set values [lsort -unique -ascii $values]
  set row 0; set col 0; set ci 0
  set window_bg [$win cget -background]
  foreach value $values {
    if {$ci == $color_nu} {set ci 0}
    foreach {bg fg} [lindex $Colors $ci] {break}
    set button $BFrame.${row}$col
    set Parameters($win:Colour:$value) [lindex $Colors $ci]
    set Parameters($win:Button:$value) $button
    if {[winfo exists $button]} {
      $button configure -text $value -foreground black -background $window_bg \
        -relief raised -activeforeground $fg -activebackground $bg -command \
        [list ::EditAnnotationHierarchiesTool::Tag $win $button $value]
    } else {
      button $button -text $value -highlightthickness 0 \
        -foreground black -activeforeground $fg -activebackground $bg -command \
        [list ::EditAnnotationHierarchiesTool::Tag $win $button $value]
      grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
    }
    ## Add a binding for setting an accelarator...
    bind $button <<MB_2>> "::EditAnnotationHierarchiesTool::EditAccelerator %W"
    bind $button <Enter> "%W configure -background $bg -foreground $fg"
    bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
    incr col; incr ci
    catch {Tooltip::tooltip $button -text $Description($value)}
    if {$col == 5} {
      incr row; set col 0
    }
  }
  grid $FFrame -columnspan 5 -sticky snew
  if {[$NoteBook raise] == "sentenceList"} {
    place $HierarchiesViewer.sel -in $HierarchiesViewer.canvas \
          -relx 0.0 -rely 1.0 -anchor sw
  } else {
    place $HierarchiesViewer.sel2edit -in $HierarchiesViewer.canvas \
          -relx 0.0 -rely 1.0 -anchor sw
  }
  raise $HierarchiesViewer.sel $HierarchiesViewer.canvas
  raise $HierarchiesViewer.sel2edit $HierarchiesViewer.canvas
  
  $NoteBook raise [$NoteBook raise]
}; # ::EditAnnotationHierarchiesTool::UpdateWindow

proc ::EditAnnotationHierarchiesTool::CloseWindow {win} {
  variable Parameters
  global GUI_Text
  if {[llength [info commands $Parameters($win:Document)]]} {
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  destroy $win
};# ::EditAnnotationHierarchiesTool::CloseWindow

## ::EditAnnotationHierarchiesTool::SelectSentence
#  Set a sentence for editing in the Hierarchy Viewer...
proc ::EditAnnotationHierarchiesTool::SelectSentence {win tag} {
  variable Parameters
  global GUI_Options
  if {![string length $Parameters($win:Document)]} {return}
  ## Reset the tree widget...
  foreach child [$Parameters($win:TreeWidget) subnodes {}] {
    $Parameters($win:TreeWidget) rmlink $child
  }
  ## Remove all elements from the Hierarchies Viewer canvas...
  $Parameters($win:HierarchiesViewer) delete all
  ## Retrieve the sentence Annotation id from the text widget...
  if {[info exists Parameters($win:LastSelectedSentenceTag)]} {
    $Parameters($win:SentenceViewer) tag configure \
      $Parameters($win:LastSelectedSentenceTag) \
      -background $GUI_Options(TextBg) -foreground $GUI_Options(TextFg)
  }
  ## Hilight the selected sentence...
  set Parameters($win:LastSelectedSentenceTag) $tag
  $Parameters($win:SentenceViewer) tag configure $tag \
      -background red -foreground white

  ## Collect the tokens and place them on the Hierarchies Viewer canvas...
  set id [string range $tag 3 end]
  set sentence [tip_GetAnnotation $Parameters($win:Document) $id]
  tip_GetSpanOffsets [tip_GetFirstSpan $sentence] start end
  SetTokensForEditing $win $start $end
};# ::EditAnnotationHierarchiesTool::SelectSentence

## ::EditAnnotationHierarchiesTool::EditSelectedText
#  Get the selection from a widget and set its tokens for editing...
proc ::EditAnnotationHierarchiesTool::EditSelectedText {win type} {
  variable Parameters
  set doc $Parameters($win:Document)
  if {![string length $doc]} {return}
  switch $type {
    text {
      set widget $Parameters($win:TextViewer)
      ## Is there a selection in the widget?
      if {[catch {$widget index sel.first} first]} {return}
      set last [$widget index sel.last]
      set start [gut_FindCharacterOffset $widget $first]
      set end   [gut_FindCharacterOffset $widget $last]
      SetTokensForEditing $win $start $end
    }
    html {
      set widget $Parameters($win:HtmlViewer)
      ## Is there a selection in the widget?
      set first [$widget index sel.first]
      if {![string length $first]} {return}
      set last  [$widget index sel.last]
      set start [$widget indexToCharacterOffset $first]
      set end   [$widget indexToCharacterOffset $last]
      SetTokensForEditing $win $start $end
    }
  }
};# ::EditAnnotationHierarchiesTool::EditSelectedText

## ::EditAnnotationHierarchiesTool::SetTokensForEditing
#  Set a sentence for editing in the Hierarchy Viewer...
proc ::EditAnnotationHierarchiesTool::SetTokensForEditing {win start end} {
  variable Parameters
  global GUI_Options
  if {![string length $Parameters($win:Document)]} {return}
  set RangeAnns [tip_SortAnnotationSet \
           [tip_AnnotationsInRange $Parameters($win:Document) $start $end]]
  set Parameters($win:EditingTokenSetSpan) [list $start $end]
  ## Reset the tree widget...
  foreach child [$Parameters($win:TreeWidget) subnodes {}] {
    $Parameters($win:TreeWidget) rmlink $child
  }
  ## Remove all elements from the Hierarchies Viewer canvas...
  $Parameters($win:HierarchiesViewer) delete all
  ## As a first step, add all nodes in the tree...
  foreach ann $RangeAnns {
    ## We should add only token & hierarchy related Annotations...
    set type [tip_GetType $ann]
    if {![string equal $type $Parameters($win:WordAnnotation)] &&
        ![string equal $type $Parameters($win:Annotation)]} {continue}
    set ID [tip_GetId $ann]
    switch -exact -- $type [list \
      $Parameters($win:WordAnnotation) {
         set text [tip_GetFirstAnnotatedTextRange \
                                        $Parameters($win:Document) $ann]
      } \
      $Parameters($win:Annotation) {
         set text [tip_GetValueValue \
                       [tip_GetAttribute $ann $Parameters($win:Attribute)]]
      }\
    ]
    ## Create the rect & text items...
    foreach {bg fg} [tag2colours $win $ID] {}
    $Parameters($win:HierarchiesViewer) create text 0 0 -text $text -tags \
      "token text id_$ID text_$ID" -font TextFont -fill $fg
    foreach {x1 y1 x2 y2} [$Parameters($win:HierarchiesViewer) bbox id_$ID] {}
    $Parameters($win:HierarchiesViewer) create rectangle \
      [incr x1 -4] [incr y1 -2] [incr x2 4] [incr y2 2] \
      -tags "token rect id_$ID rect_$ID" -fill $bg
    $Parameters($win:HierarchiesViewer) raise text_$ID rect_$ID
    ## Create the small text & rect items, for showing the selection sequence...
    $Parameters($win:HierarchiesViewer) create text $x2 [incr y2 4] \
      -text 999 -anchor e -tags \
      "token text sst id_$ID sst_$ID" -font ExplorerFontItalic \
      -fill $Parameters($win:HierarchiesViewer:SelNuFG)
    $Parameters($win:HierarchiesViewer) create rectangle \
      [$Parameters($win:HierarchiesViewer) bbox sst_$ID] \
      -tags "token rect id_$ID ssr_$ID" -outline {}
    $Parameters($win:HierarchiesViewer) raise sst_$ID ssr_$ID
    ## If it is a word unit and the user has specified to display a label,
    ## create the needed items...
    if {[string length $Parameters($win:WordAnnotationAttr)] &&
        [string equal $type $Parameters($win:WordAnnotation)]} {
      set attribute [tip_GetAttribute $ann $Parameters($win:WordAnnotationAttr)]
      if {[llength $attribute]} {
        set AttrVal [tip_GetValueValue $attribute]
        $Parameters($win:HierarchiesViewer) create text $x1 [incr y2 4] \
          -text $AttrVal -anchor w -tags \
          "token text id_$ID text_$ID attr_$ID" -font ExplorerFontItalic \
           -fill $fg
         $Parameters($win:HierarchiesViewer) lower attr_$ID ssr_$ID
      }
    }
    ## Create the arrow line...
    set line [$Parameters($win:HierarchiesViewer) create line 0 0 0 0 \
      -arrow last -fill blue -smooth 1 -width 1]
    $Parameters($win:HierarchiesViewer) bind id_$ID <<MB_1>> \
      "::EditAnnotationHierarchiesTool::ToggleItemSelection $win $ID"
    ## Add the node into the tree...
    $Parameters($win:TreeWidget) addlink {} id_$ID $line -border 3
  }
  $Parameters($win:HierarchiesViewer) itemconfigure sst -text {}
  ## As a second step, create the hierarcies...
  foreach ann [tip_SelectAnnotations $RangeAnns $Parameters($win:Annotation)] {
    set ID [tip_GetId $ann]
    ## Check all constituents and make them subtrees of the current node...
    foreach id [tip_GetValueValue [tip_GetAttribute $ann constituents]] {
      $Parameters($win:TreeWidget) movelink id_$id id_$ID
      ## Only top-level nodes should be selectable...
      $Parameters($win:HierarchiesViewer) bind id_$id <<MB_1>> \
        "::EditAnnotationHierarchiesTool::HilightToolButton $win $id"
    }
  }
  DeselectEverything $win
  ::ScrolledWidgets::CanvasUpdateScrollRegion \
    $Parameters($win:HierarchiesViewer)
  ## Center on the first tree node...
  $Parameters($win:TreeWidget) draw
  catch {
    set id [$Parameters($win:TreeWidget) child {}]
    Widget_CanvasSeeItem $Parameters($win:HierarchiesViewer) $id
  }
};# ::EditAnnotationHierarchiesTool::SetTokensForEditing

## ::EditAnnotationHierarchiesTool::AdvanceSelectedSentence
#  A proc for moving the selection to the next/previous sentence...
proc ::EditAnnotationHierarchiesTool::AdvanceSelectedSentence {win dir} {
  variable Parameters
  if {![info exists Parameters($win:LastSelectedSentenceTag)] ||
      ![string length $Parameters($win:LastSelectedSentenceTag)]} {
    ## The user has pressed a button without a selection...
    switch $dir {
      prev {
        set tag [lindex [lsort \
                [$Parameters($win:SentenceViewer) tag names 1.0]] 0]
        set index 1.0
      }
      next {
        set tag [lindex [lsort \
                [$Parameters($win:SentenceViewer) tag names end-1l]] end]
        set index end-1l
      }
    }
    SelectSentence $win $tag
    $Parameters($win:SentenceViewer) see $index
    return
  }
  set tag $Parameters($win:LastSelectedSentenceTag)
  foreach {start end} [$Parameters($win:SentenceViewer) tag ranges $tag] {break}
  switch $dir {
    prev {
      set tag [lindex [lsort \
               [$Parameters($win:SentenceViewer) tag names $start-1c]] 0]
      set index $start-2l
    }
    next {
      set tag [lindex [lsort \
               [$Parameters($win:SentenceViewer) tag names $end+1c]] end]
      set index $start+1l
    }
  }
  if {![string length $tag]} {return}
  SelectSentence $win $tag
  $Parameters($win:SentenceViewer) see $index
};# ::EditAnnotationHierarchiesTool::AdvanceSelectedSentence

## ::EditAnnotationHierarchiesTool::ToggleItemSelection
#  Select or deselect an item...
proc ::EditAnnotationHierarchiesTool::ToggleItemSelection {win id} {
  variable Parameters
  if {[info exists Parameters($win:SelectedTokens)]} {
    ## Is the item already selected?
    set index [lsearch -exact $Parameters($win:SelectedTokens) $id]
    if {$index == -1} {
      ## Select the item...
      lappend Parameters($win:SelectedTokens) $id
      $Parameters($win:HierarchiesViewer) itemconfigure rect_$id \
        -fill $Parameters($win:HierarchiesViewer:SelectedBG)
      $Parameters($win:HierarchiesViewer) itemconfigure text_$id \
        -fill $Parameters($win:HierarchiesViewer:SelectedFG)
      $Parameters($win:HierarchiesViewer) itemconfigure ssr_$id \
        -fill $Parameters($win:HierarchiesViewer:SelNuBG)
    } else {
      ## Deselect the item...
      set Parameters($win:SelectedTokens) \
        [lreplace $Parameters($win:SelectedTokens) $index $index]
      foreach {bg fg} [tag2colours $win $id] {}
      $Parameters($win:HierarchiesViewer) itemconfigure rect_$id -fill $bg
      $Parameters($win:HierarchiesViewer) itemconfigure text_$id -fill $fg
      $Parameters($win:HierarchiesViewer) itemconfigure ssr_$id -fill {}
      $Parameters($win:HierarchiesViewer) itemconfigure sst_$id -text {}
    }
  }
  ## Update all selection counters...
  set i 0
  foreach id $Parameters($win:SelectedTokens) {
    $Parameters($win:HierarchiesViewer) itemconfigure sst_$id -text "[incr i] "
  }
  if {$Parameters(UseImages) && 
      ([llength $Parameters($win:SelectedTokens)] <= 15)} {
    $Parameters($win:ImageLabel) configure -image \
      ::EditAnnotationHierarchiesTool::Image[llength \
                                       $Parameters($win:SelectedTokens)]
  }
  ## Try to hilight the corresponding button, if one available...
  HilightToolButton $win $id
};# ::EditAnnotationHierarchiesTool::ToggleItemSelection

## ::EditAnnotationHierarchiesTool::DeselectEverything
#  Reset the list of selected items in the Hierarchies Viewer...
proc ::EditAnnotationHierarchiesTool::DeselectEverything {win} {
  variable Parameters
  ## If there is an active item, simply return..
  if {[llength [$Parameters($win:HierarchiesViewer) find withtag current]]} {
    return
  }
  if {[info exists Parameters($win:SelectedTokens)]} {
    foreach tag $Parameters($win:SelectedTokens) {
      ## Deselect the items...
      catch {ToggleItemSelection $win $tag}
    }
  }
  set Parameters($win:SelectedTokens) {}
  HilightToolButton $win
};# ::EditAnnotationHierarchiesTool::DeselectEverything

proc ::EditAnnotationHierarchiesTool::HilightSentence {win tag bg fg} {
  variable Parameters
  if {[info exists Parameters($win:LastSelectedSentenceTag)] &&
      [string equal $Parameters($win:LastSelectedSentenceTag) $tag]} {
    return
  }
  $Parameters($win:SentenceViewer) tag configure $tag \
    -background $bg -foreground $fg
};# ::EditAnnotationHierarchiesTool::HilightSentence

## ::EditAnnotationHierarchiesTool::tag2value
#  Retrieve the value of a specific tag
proc ::EditAnnotationHierarchiesTool::tag2value {win tag} {
  variable Parameters
  switch -glob $tag {
    text_*  {set id [string range $tag 5 end]}
    rect_*  {set id [string range $tag 5 end]}
    id_*    {set id [string range $tag 3 end]}
    default {set id $tag}
  }
  if {[catch {tip_GetAnnotation $Parameters($win:Document) $id} ann]} {
    return {}
  }
  ## Is the Annotation of the correct type?
  if {![string equal [tip_GetType $ann] $Parameters($win:Annotation)]} {
    return {}
  }
  return  [tip_GetValueValue \
               [tip_GetAttribute $ann $Parameters($win:Attribute)]]
};# ::EditAnnotationHierarchiesTool::tag2value

## ::EditAnnotationHierarchiesTool::tag2button
#  Retrieve the button widget for a specific tag
proc ::EditAnnotationHierarchiesTool::tag2button {win tag} {
  variable Parameters
  set value [tag2value $win $tag]
  if {![string length $value]} {return {}}
  set Parameters($win:Button:$value)
};# ::EditAnnotationHierarchiesTool::tag2button

## ::EditAnnotationHierarchiesTool::tag2colours
#  Retrieve the colours (bg fg) for a specific tag
proc ::EditAnnotationHierarchiesTool::tag2colours {win tag} {
  variable Parameters
  set value [tag2value $win $tag]
  if {![string length $value]} {
    return [list $Parameters($win:HierarchiesViewer:BG) \
                 $Parameters($win:HierarchiesViewer:FG)]
  }
  set Parameters($win:Colour:$value)
};# ::EditAnnotationHierarchiesTool::tag2colours

## ::EditAnnotationHierarchiesTool::HilightToolButton
#   Hilight the appropriate tool button...
proc ::EditAnnotationHierarchiesTool::HilightToolButton {win {tag {}}} {
  variable Parameters
  set ::GUI_StatusBar::StatusBar($win) $::GUI_StatusBar::StatusBar($win:Def)
  if {[info exists Parameters($win:Selected)]} {
    set sel_tag $Parameters($win:Selected)
    set button [tag2button $win $sel_tag]
    if {[string length $button]} {
      $button configure -background [$win cget -background] -foreground black \
         -relief raised
      $Parameters($win:HierarchiesViewer) itemconfigure rect_$sel_tag \
        -width 1
    }
  }
  unset -nocomplain Parameters($win:Selected)
  if {![string length $tag]} {
    ## The user has clicked on the text background...
    return
  }
  set Parameters($win:Selected) $tag
  set button [tag2button $win $tag]
  if {[string length $button]} {
    if {[info exists Parameters($win:SelectedTokens)] && 
        ![llength $Parameters($win:SelectedTokens)]} {
      foreach {bg fg} [tag2colours $win $tag] {}
      $button configure -background $bg -foreground $fg -relief sunken
      $Parameters($win:HierarchiesViewer) itemconfigure rect_$tag \
          -width 3
    }
  }
}; # ::EditAnnotationHierarchiesTool::HilightToolButton

## ::EditAnnotationHierarchiesTool::AutoTag
#   This function will tag automatically all instances of selected text with
#   the current tag. Use with care :-)
proc ::EditAnnotationHierarchiesTool::AutoTag {win} {
};# ::EditAnnotationHierarchiesTool::AutoTag


## ::EditAnnotationHierarchiesTool::Tag
#   The Callback of the ToolButtons...
proc ::EditAnnotationHierarchiesTool::Tag {win button value} {
  variable Parameters
  if {![info exists Parameters($win:SelectedTokens)]} {return}
  set doc $Parameters($win:Document)
  if {![llength $Parameters($win:SelectedTokens)]} {
    ## The user wants to change the classification of a specific Annotation...
    if {[info exists Parameters($win:Selected)]} {
      set id $Parameters($win:Selected)
      HilightToolButton $win
      set ann [tip_GetAnnotation $doc $id]
      set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                    [tip_CreateAttributeValue GDM_STRING $value]]
      set ann [tip_PutAttribute $ann $attr]
      set ID [tip_AddAnnotation $doc $ann]
      if {[info exists Parameters($win:EditingTokenSetSpan)]} {
        SetTokensForEditing $win \
                    [lindex $Parameters($win:EditingTokenSetSpan) 0] \
                    [lindex $Parameters($win:EditingTokenSetSpan) 1]
      }
      ## Try to make visible the new canvas item...
      $Parameters($win:TreeWidget) draw
      Widget_CanvasSeeItem $Parameters($win:HierarchiesViewer) id_$ID
      HilightToolButton $win $ID
    }
    return
  }
  ## Iterate over all selected items to get the total span...
  foreach id $Parameters($win:SelectedTokens) {
    set ann [tip_GetAnnotation $doc $id]
    foreach span [tip_GetSpans $ann] {
      tip_GetSpanOffsets $span start end
      set Index($start) 0
      set Index($end)   0
    }
  }
  ## The total span is simply the lowest & greatest entries in Index...
  set Indexes [lsort -integer [array names Index]]
  unset Index
  set start [lindex $Indexes 0]; set end [lindex $Indexes end]
  ## Is there an Annotation with the same span?
  ## set Anns [tip_SelectAnnotations \
  ##               [tip_AnnotationsMatchingRange $doc $start $end] \
  ##               $Parameters($win:Annotation)]
  set Anns {}
  if {[llength $Anns]} {
    foreach ann $Anns {
      ## The Selected Span matches an existing annotation
      set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                    [tip_CreateAttributeValue GDM_STRING $value]]
      set ann [tip_PutAttribute $ann $attr]
      set attr [tip_CreateAttribute constituents \
                    [tip_CreateAttributeValue GDM_STRING_SET \
                                              $Parameters($win:SelectedTokens)]]
      set ann [tip_PutAttribute $ann $attr]
      set ID [tip_AddAnnotation $doc $ann]
      ## puts "Changing Annotation with ID $ID: $attr -> $value"
    }
  } else {
    ## A New Annotation must be created
    set attr1 [tip_CreateAttribute $Parameters($win:Attribute) \
                  [tip_CreateAttributeValue GDM_STRING $value]]
    set attr2 [tip_CreateAttribute constituents \
                  [tip_CreateAttributeValue GDM_STRING_SET \
                                            $Parameters($win:SelectedTokens)]]
    set ann  [tip_CreateAnnotation $Parameters($win:Annotation) \
                 [list [tip_CreateSpan $start $end]] [list $attr1 $attr2]]
    set ID [tip_AddAnnotation $doc $ann]
    ## puts "Creating New $Annotation Annotation ($ID, $value)"
  }
  if {[info exists Parameters($win:EditingTokenSetSpan)]} {
    SetTokensForEditing $win [lindex $Parameters($win:EditingTokenSetSpan) 0] \
                             [lindex $Parameters($win:EditingTokenSetSpan) 1]
  }
  ## Try to make visible the new canvas item...
  $Parameters($win:TreeWidget) draw
  Widget_CanvasSeeItem $Parameters($win:HierarchiesViewer) id_$ID
};# ::EditAnnotationHierarchiesTool::Tag

## ::EditAnnotationHierarchiesTool::Delete
#  
proc ::EditAnnotationHierarchiesTool::Delete {win {Annotation {}}} {
  variable Parameters
  variable Parameters
  if {![info exists Parameters($win:SelectedTokens)]} {return}
  if {![llength $Parameters($win:SelectedTokens)]} {return}
  set SelectedTokens $Parameters($win:SelectedTokens)
  DeselectEverything $win
  set doc $Parameters($win:Document)
  foreach id $SelectedTokens {
    ## Is the Annotation type the desired one?
    set ann [tip_GetAnnotation $doc $id]
    if {![string equal $Parameters($win:Annotation) [tip_GetType $ann]]} {
      continue
    }
    if {[string length $Annotation]} {
      tip_RemoveAnnotation $doc $id
    } else {
      ## puts "Deleting $attr $value from Annotation with ID $ID"
      set ann [tip_GetAnnotation $doc $id]
      set ann [tip_RemoveAttribute $ann $Parameters($win:Attribute)]
      tip_AddAnnotation $doc $ann
    }
  }
  if {[info exists Parameters($win:EditingTokenSetSpan)]} {
    SetTokensForEditing $win [lindex $Parameters($win:EditingTokenSetSpan) 0] \
                             [lindex $Parameters($win:EditingTokenSetSpan) 1]
  }
};# ::EditAnnotationHierarchiesTool::Delete

## ::EditAnnotationHierarchiesTool::EditAccelerator
#  Add/Modify an accelerator for a button widget...
proc ::EditAnnotationHierarchiesTool::EditAccelerator {button} {
  ## Create a window for setting the binding...
  set win .tooleditAnnotationsEditAccelerator
  if {[winfo exists $win]} {
    raise $win
    return
  }
  toplevel .tooleditAnnotationsEditAccelerator
  wm withdraw $win
  wm title $win "Setting Button Accelerator..."
  message $win.label -width 250 -text \
    "Please press the keyboard keys combination you want to assign as an\
     accelerator to the button with value \"[$button cget -text]\""
  label $win.acc
  bind $win <Key> \
    [list ::EditAnnotationHierarchiesTool::SetAccelerator $win.acc $button %K]
  button $win.ok -textvariable GUI_Text(Ok) -command "destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -command \
  "destroy $win
   ::EditAnnotationHierarchiesTool::SetAccelerator $win.acc $button"
  grid $win.label -sticky snew -columnspan 3 -sticky snew
  grid $win.acc   -sticky snew -columnspan 3 -sticky snew
  grid $win.ok x $win.cancel -padx 20 -pady 10  -sticky snew
  grid columnconfigure $win 1 -weight 1
  gut_ModalDlg $win
  wm deiconify $win
};# ::EditAnnotationHierarchiesTool::EditAccelerator

proc ::EditAnnotationHierarchiesTool::SetAccelerator {label button {key {}}} {
  catch {$label configure -text "Key: \"$key\""}
  set toplevel [winfo toplevel $button]
  ## Remove any previously set accelerators...
  foreach event [bind $button] {
    switch -glob -- $event {
      ? -
      <Key-*> {
        bind $button   $event {}
        bind $toplevel $event {}
      }
      default {}
    }
  }
  if {![string length $key]} {return}
  bind $button   <Key-$key> "$button invoke; $button flash"
  bind $toplevel <Key-$key> "$button invoke; $button flash"
};# ::EditAnnotationHierarchiesTool::SetAccelerator

#
# End of File EditAnnotationHierarchies.tcl
#

