##########################################################################
# 
#       EditAnnotations.tcl - ??? 04 ???  1999 10:14:52
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.5...
#
##########################################################################


# Location of this module (EditAnnotations)
global creole_EditAnnotations_home

namespace eval ::EditAnnotationsTool {
  variable Parameters
  variable ValuesSpec
}

## Load some predifined Values...
proc ::EditAnnotationsTool::Init {} {
  ::Annotation::Init
  foreach filename [list \
                        $::creole_EditAnnotations_home/PredifinedValues.tcl \
                        $::UserAppDir/EditAnnotations.cfg \
                        $::UserHomeDir/.EditAnnotations.cfg \
                        $::UserHomeDir/.EditAnnotations.cfg] {
    if {![file exists $filename]} {continue}
    set file [open $filename]
      fconfigure $file -encoding utf-8
      set data [read $file]
    close $file
    if {[catch {eval $data} error]} {
      set msg "The following error occured while reading predifined values\
               from file \"$filename\":\n$error"
      catch {bgerror $msg}
    }
  }
};# ::EditAnnotationsTool::Init

## creole_EditAnnotations
#  
proc creole_EditAnnotations {doc col} {
  ::EditAnnotationsTool::Init
  ## Create the main window...
  set win [ggi_unused_win .tooleditannotations_main_window]
  array unset ::EditAnnotationsTool::Parameters $win:*

  set ::EditAnnotationsTool::Parameters($win:Document)   $doc
  set ::EditAnnotationsTool::Parameters($win:Collection) $col
  set ::EditAnnotationsTool::Parameters($win:Annotation) token
  set ::EditAnnotationsTool::Parameters($win:Attribute)  pos
  ::EditAnnotationsTool::GetParameters $win $doc $col
  ## Has the user cancelled the procedure?
  if {$::EditAnnotationsTool::Parameters($win:Cancel)} {
    array unset ::EditAnnotationsTool::Parameters $win:*
    return
  }
  if {![string length $::EditAnnotationsTool::Parameters($win:Annotation)]} {
    return
  }
  set Annotation $::EditAnnotationsTool::Parameters($win:Annotation)
  set Attribute  $::EditAnnotationsTool::Parameters($win:Attribute)
  set Values     [split $::EditAnnotationsTool::Parameters($win:UserValues)]
  set Values     [lsort -unique $Values]
  ## If in values we got the empty string, remove it...
  set index [lsearch -exact $Values {}]
  if {$index != -1} {set Values [lreplace $Values $index $index]}
  set ::EditAnnotationsTool::Parameters($win:UserValues) $Values
  set Language   $::EditAnnotationsTool::Parameters($win:Language)
  #if {![string length $doc]} {
  #  set col {}
  #  set ::EditAnnotationsTool::Parameters($win:Collection) {}
  #}
  ::EditAnnotationsTool::CreateWindow $win $doc $col \
                 $Annotation $Attribute $Values $Language
  tkwait window $win

  array unset ::EditAnnotationsTool::Parameters $win:*
};# creole_EditAnnotations

## ::EditAnnotationsTool::DocLoad
#   Load the requested document...
proc ::EditAnnotationsTool::DocLoad {win ColW DocW {what This}} {
  global GUI_Text
  variable Parameters
  set Col [$ColW getvalue]
  set Doc [$DocW getvalue]

  ## Warn user if Document is modified, before we close it...
  if {[llength [info commands $Parameters($win:Document)]]} {
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  switch $what {
    Prev    {incr Doc -1; $DocW setvalue previous}
    Next    {incr Doc;    set NEXT [$DocW setvalue next]
             if {$NEXT == 0} {return}}
    default {}
  }

  if {$Col == -1 || $Doc == -1} {return}

  catch {tip_Close $Parameters($win:Document)}
  set col $Parameters($win:Collection)
  set doc $Parameters($win:Document)
  if {!$Parameters($win:SystemOwnsCollection)} {
    set Col [lindex [GUI_GetCollectionNames] $Col]
    if {[string length $col]} {
      if {![string equal [file tail [tip_GetName $col]] $Col]} {
        tip_Close $col
        set col [tip_OpenCollection $Col]
      }
    } else {
      set col [tip_OpenCollection $Col]
    }
    ## Is the Collection's Location in accordance with it's Name?
    set ColName [tip_GetName $col]
    if {![string equal $ColName $Col]} {
      GUI_FixCollectionName $col $ColName $Col
    }
  }
  
  set Doc [lindex [$DocW cget -values] $Doc]
  set doc [tip_GetDocument $col $Doc]
  set Parameters($win:Document) $doc
  set Parameters($win:Collection) $col
  set Annotation $Parameters($win:Annotation)
  set Attribute  $Parameters($win:Attribute)
  set Values     $Parameters($win:UserValues)
  set Language   $Parameters($win:Language)

  UpdateWindow $win $doc $col \
                 $Annotation $Attribute $Values $Language
  set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  if {!$Parameters($win:SystemOwnsCollection)} {
    bind $win <Destroy> "catch \"tip_Close $col\""
  }
};# ::EditAnnotationsTool::DocLoad

## ::EditAnnotationsTool::DocSave
#    Save current document...
proc ::EditAnnotationsTool::DocSave {win} {
  variable Parameters
  if {[string length $Parameters($win:Document)]} {
    tip_Sync $Parameters($win:Document)
  }
};# ::EditAnnotationsTool::DocSave

## ::EditAnnotationsTool::GetParameters
#   Displays a small window, where the user can type the annotation and the
#   the annotation attribute (optional) he wants to edit.
proc ::EditAnnotationsTool::GetParameters {win doc col} {
  global GUI_Text GUI_Options
  variable Parameters

  toplevel $win
  wm withdraw $win
  wm title $win "Edit Annotation Tool Options..."

  label $win.title -text {Edit Annotation Tool Options:}
  pack $win.title -side top -fill x

  set frame $win.frame
  frame $frame -borderwidth 2 -relief groove
    label $frame.lann -text {Edit Annotation:}
    ComboBox $frame.ann -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] \
      -textvariable ::EditAnnotationsTool::Parameters($win:Annotation) \
      -modifycmd \
      "::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr"
    grid $frame.lann $frame.ann -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lattr -text {Annotation Type:}
    ComboBox $frame.attr -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -textvariable ::EditAnnotationsTool::Parameters($win:Attribute)
    ::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr
    set ::EditAnnotationsTool::Parameters($win:Attribute)  pos
    grid $frame.lattr $frame.attr -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.vals -text {Edit Attribute Values:}
    entry $frame.vals_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      ::EditAnnotationsTool::Parameters($win:UserValues)
    grid $frame.vals $frame.vals_e \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lang -text {Language:}
    entry $frame.lang_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      ::EditAnnotationsTool::Parameters($win:Language)
    set language [tip_GetAttribute $doc language]
    if {![string length $language]} {
      global Locale
      set language [string tolower $Locale]
    } else {
      set language [lindex [tip_GetValue $language] end]
    }
    if {![string length $language]} {
      global Locale
      set language [string tolower $Locale]
    }
    switch [string tolower $language] {
      greek   {set language hellenic}
      default {}
    }
    set Parameters($win:Language) $language
    grid $frame.lang $frame.lang_e \
      -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    grid columnconfigure $frame 0 -weight 0 
    grid columnconfigure $frame 1 -weight 1 
    grid columnconfigure $frame 2 -weight 0 
  pack $frame -fill both -expand 1 -ipadx 5 -ipady 5 -padx 3 -pady 3

  set bframe $win.buttons
  frame $bframe
    button $bframe.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
      -default active
    bind $win <Return> "$bframe.ok invoke"
    button $bframe.cancel -textvariable GUI_Text(Cancel) -command \
     "set ::EditAnnotationsTool::Parameters($win:Cancel) 1
      destroy $win"
    pack $bframe.ok $bframe.cancel -padx 3 -pady 5 -side left
  pack $bframe
  focus $frame.ann
  gut_ModalDlg $win
  wm deiconify $win
  set Parameters($win:Cancel) 0
  tkwait window $win
}; #::EditAnnotationsTool::GetParameters

## ::EditAnnotationsTool::CreateWindow
proc ::EditAnnotationsTool::CreateWindow {win doc col 
                  {Annotation {}} {Attribute {}} {Values {}} {Language {}}} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec
  set title "Edit Annotations Tool: Editing Attribute \"$Attribute\"\
             of Annotation \"$Annotation\""

  GUI_LoadImage BackNice
  GUI_LoadImage NextNice
  set HaveSystemCol 0
  set HaveSystemDoc 0
  set Parameters($win:SystemOwnsCollection) 0
  if {[string equal $doc $GUI_Text(All)]} {set doc {}}
  if {[string length $col] && ![string length $doc]} {
    set doc [tip_FirstDocument $col]
    set Parameters($win:SystemOwnsCollection) 1
    set HaveSystemCol 1
  }
  if {![string length $col] || ![string length $doc] || \
      ![string length $Annotation] || ![string length $Attribute]} {
    set DocText {}
    set Annotations {}
  } else {
    ## Get Document Annotations and text
    if {[catch {tip_GetByteSequence $doc} DocText]} {
      error "creole_EditAnnotations: Invalid Document!\n($DocText)"
    }
    set Annotations [tip_SelectAnnotations $doc $Annotation \
                       "\[string length ann::${Attribute} \]"]
    set HaveSystemDoc 1
  }
  set values $Values
  if {[info exists ValuesSpec($Language:$Annotation:$Attribute)]} {
    foreach item $ValuesSpec($Language:$Annotation:$Attribute) {
      foreach {type description} $item {break}
      if {[string equal $type ---]} {continue}
      lappend values $type
      set Description($type) $description
    }
  }

  ## Create the window
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Status Bar
  if {$HaveSystemDoc} {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  } else {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):( $GUI_Text(none) )    \
       $GUI_Text(Document):( $GUI_Text(none) )"
  }
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  GUI_CreateStatusBar $win.statusbar

  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
    ## Open Document/Collection
    set toolbar $toolbar_frame.express
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -float 0
    if {$HaveSystemCol || !$HaveSystemDoc} {
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Collection) -tooltipvar GUI_Text(TSelectCollection) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {!$HaveSystemCol} {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -entercmd \
           {%W configure -values [GUI_GetCollectionNameTails]} -modifycmd \
           "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
            $toolbar.widgets.document $toolbar.widgets.collection
            $toolbar.widgets.document configure -values \
            \[lrange \[$toolbar.widgets.document cget -values\] 1 end\]
            $toolbar.widgets.document setvalue first" \
           -tooltipvar GUI_Text(TSelectCollection) -width \
           $GUI_Options(SystemExprToolbarCollectionWidth) -takefocus 0 -pady 3 \
           -statusvar ::GUI_StatusBar::StatusBar($win)
      } else {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -takefocus 0 -pady 3 \
           -values [list [file tail [tip_GetName $col]]] \
           -width $GUI_Options(SystemExprToolbarCollectionWidth)
           $toolbar.widgets.collection setvalue first
           $toolbar.widgets.collection configure -state disabled
      }
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Document) -tooltipvar GUI_Text(TSelectDocument) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -name $toolbar.widgets.document \
         -tooltipvar GUI_Text(TSelectDocument) -width \
         $GUI_Options(SystemExprToolbarDocumentWidth) -takefocus 0 -pady 3 \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {$HaveSystemCol} {
        $toolbar.widgets.document configure -values [$col GetFileIds]
        $toolbar.widgets.document setvalue first
      }
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
         -textvariable GUI_Text(Load) -image ITLoad -command \
         "::EditAnnotationsTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document" \
         -tooltipvar GUI_Text(TSelectedLoad) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -image BackNice -command \
         "::EditAnnotationsTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Prev"
      Toolbar::addbutton $toolbar -image NextNice -command \
         "::EditAnnotationsTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Next"
    }
    Toolbar::addbutton $toolbar -image ITSave -tooltipvar GUI_Text(Save) \
       -statusvar ::GUI_StatusBar::StatusBar($win) \
       -command "::EditAnnotationsTool::DocSave $win"
    Toolbar::addseparator $toolbar
    wm protocol $win WM_DELETE_WINDOW "::EditAnnotationsTool::CloseWindow $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command "::EditAnnotationsTool::CloseWindow $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    set ActiveLabel [Toolbar::addwidget $toolbar label -relief flat -bd 0 \
       -anchor e]
    pack $toolbar -side top -anchor w
  pack $toolbar_frame -side top -fill x

  ## Create the main window area...
  frame $win.area
    ## The text area...
    set Viewer $win.area.text
    ::ScrolledWidgets::ScrolledText $Viewer
    $Viewer.text insert end $DocText
    set Parameters($win:TextObj) $Viewer.text
    set Parameters($win:ActiveLabel) $ActiveLabel
    ## The Combobox for selecting overlapping ranges...
    frame $win.select
      label $win.select.label -textvariable GUI_Text(SelectOverlapingTag:)
      ComboBox $win.select.combo -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -takefocus 0 -state disabled -modifycmd \
         "::EditAnnotationsTool::SelectOverlapped $win $win.select.combo"
      grid $win.select.label $win.select.combo -sticky snew
      grid columnconfigure $win.select 1 -weight 1
    set Parameters($win:SelectRangeWidget) $win.select.combo
    ## The buttons area...
    set BFrame $win.area.buttons
    frame $BFrame -relief flat
  pack $BFrame -side right -fill y    -expand 0 -padx 2
  pack $Viewer -side left  -fill both -expand 1 -padx 2

  ## Add Tags to Text Viewer...
  #   If the attribute type is in the Values variable, just add the two tags.
  #   Else, add the "gray" tag, and add the span to an array for further
  #   reference...
  if {[llength $Annotations]} {
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      set span     [tip_GetFirstSpan $ann]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      ## Add a tag
      CDM_LappendUnique values $value
      tip_GetSpanOffsets $span start end
      $Viewer.text tag add id_$ID 1.0+${start}c 1.0+${end}c
      $Viewer.text tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationsTool::HilightToolButton $win id_$ID]
      $Viewer.text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $value]
      $Viewer.text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
      ggi_wait_update {} $Progress $wwin
    }
  };# if {[llength $Annotations]}

  ## Add the Vertical Toolbar, that the user can change type in selected
  ## objects...
  set FFrame $BFrame.functionButtons
  frame $FFrame
    button $FFrame.del_ann -text {Delete Annotation} -background red \
      -foreground white -command \
      "::EditAnnotationsTool::Delete $win Annotation"
    button $FFrame.del_attr -text {Delete Attribute} -background orange \
      -foreground white -command "::EditAnnotationsTool::Delete $win"
    button $FFrame.autotag -text {Automatic Tag} -background #6b8e23 \
      -foreground white -command "::EditAnnotationsTool::AutoTag $win"
    grid $FFrame.autotag  -sticky snew -padx 1 -pady 1
    grid $FFrame.del_attr -sticky snew -padx 1 -pady 1
    grid $FFrame.del_ann  -sticky snew -padx 1 -pady 1
    grid columnconfigure $FFrame 0 -weight 1
  ##
  ## The values buttons...
  ##
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set values [lsort -unique -ascii $values]
  set row 0; set col 0; set ci 0
  foreach value $values {
    if {$ci == $color_nu} {set ci 0}
    foreach {bg fg} [lindex $Colors $ci] {break}
    set button $BFrame.${row}$col
    set Parameters($win:Colour:$value) [lindex $Colors $ci]
    set Parameters($win:Button:$value) $button
    button $button -text $value -foreground black -highlightthickness 0\
      -activeforeground $fg -activebackground $bg -command \
      [list ::EditAnnotationsTool::Tag $win $button $value]
    ## Add a binding for setting an accelarator...
    bind $button <<MB_2>> "::EditAnnotationsTool::EditAccelerator %W"
    bind $button <<MB_M>> \
      [list ::EditAnnotationsTool::TagWithNewAnnotation $win $button $value]
    bind $button <Enter> "%W configure -background $bg -foreground $fg"
    bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
    grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
    incr col; incr ci
    catch {Tooltip::tooltip $button -text $Description($value)}
    if {$col == 5} {
      incr row; set col 0
    }
  }
  grid $FFrame -columnspan 5 -sticky snew
  pack $win.area -fill both -expand 1 -ipadx 10 -ipady 10
  pack $win.select -side bottom -fill x -expand 0 \
       -padx 40 -pady 5

  ##
  ## Colour the text tags according to their value...
  ##
  if {[llength $Annotations]} {
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      foreach {bg fg} $Parameters($win:Colour:$value) {break}
      $Viewer.text tag configure id_$ID -foreground $fg -background $bg
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}

  gut_ModalDlg $win
  wm deiconify $win

  $Viewer.text configure -state disabled -exportselection 1
  $Viewer.text tag add back 1.0 end
  $Viewer.text tag bind back <<MB_1>> \
    "::EditAnnotationsTool::HilightToolButton $win"
  $Viewer.text tag lower back
  focus $Viewer.text
  $Viewer.text tag raise sel
  return [list $win $Viewer.text]
}; # ::EditAnnotationsTool::CreateWindow

## ::EditAnnotationsTool::UpdateWindow
proc ::EditAnnotationsTool::UpdateWindow {win doc col 
                  {Annotation {}} {Attribute {}} {Values {}} {Language {}}} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec
  ## Get Document Annotations and text
  if {[catch {tip_GetByteSequence $doc} DocText]} {
    error "creole_EditAnnotations: Invalid Document!\n($DocText)"
  }
  set Annotations [tip_SelectAnnotations $doc $Annotation \
                     "\[string length ann::${Attribute} \]"]
  set values $Values
  if {[info exists ValuesSpec($Language:$Annotation:$Attribute)]} {
    foreach item $ValuesSpec($Language:$Annotation:$Attribute) {
      foreach {type description} $item {break}
      if {[string equal $type ---]} {continue}
      lappend values $type
      set Description($type) $description
    }
  }

  ## The text area...
  set Viewer $win.area.text
  $Viewer.text configure -state normal
  $Viewer.text delete 1.0 end
  foreach tag [$Viewer.text tag names] {
    $Viewer.text tag delete $tag
  }
  unset -nocomplain Parameters($win:Selected)
  $Viewer.text insert end $DocText
  $Parameters($win:SelectRangeWidget) configure -values {} -text {} \
                                                -state disabled
  unset -nocomplain Parameters($win:SelectRangeWidget_LastRange)
  ## The buttons area...
  set BFrame $win.area.buttons

  ## Add Tags to Text Viewer...
  #   If the attribute type is in the Values variable, just add the two tags.
  #   Else, add the "gray" tag, and add the span to an array for further
  #   reference...
  if {[llength $Annotations]} {
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      set span     [tip_GetFirstSpan $ann]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      ## Add a tag
      CDM_LappendUnique values $value
      tip_GetSpanOffsets $span start end
      $Viewer.text tag add id_$ID 1.0+${start}c 1.0+${end}c
      $Viewer.text tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationsTool::HilightToolButton $win id_$ID]
      $Viewer.text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $value]
      $Viewer.text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
      ggi_wait_update {} $Progress $wwin
    }
  };# if {[llength $Annotations]}

  ## Add the Vertical Toolbar, that the user can change type in selected
  ## objects...
  set FFrame $BFrame.functionButtons
  grid forget $FFrame
  ##
  ## The values buttons...
  ##
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set values [lsort -unique -ascii $values]
  set row 0; set col 0; set ci 0
  set window_bg [$win cget -background]
  foreach value $values {
    if {$ci == $color_nu} {set ci 0}
    foreach {bg fg} [lindex $Colors $ci] {break}
    set button $BFrame.${row}$col
    set Parameters($win:Colour:$value) [lindex $Colors $ci]
    set Parameters($win:Button:$value) $button
    if {[winfo exists $button]} {
      $button configure -text $value -background $window_bg -relief raised \
        -foreground black -activeforeground $fg -activebackground $bg -command \
        [list ::EditAnnotationsTool::Tag $win $button $value]
    } else {
      button $button -text $value -highlightthickness 0 \
        -foreground black -activeforeground $fg -activebackground $bg -command \
        [list ::EditAnnotationsTool::Tag $win $button $value]
      grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
    }
    ## Add a binding for setting an accelarator...
    bind $button <<MB_2>> "::EditAnnotationsTool::EditAccelerator %W"
    bind $button <Enter> "%W configure -background $bg -foreground $fg"
    bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
    incr col; incr ci
    catch {Tooltip::tooltip $button -text $Description($value)}
    if {$col == 5} {
      incr row; set col 0
    }
  }
  grid $FFrame -columnspan 5 -sticky snew

  ##
  ## Colour the text tags according to their value...
  ##
  if {[llength $Annotations]} {
    foreach ann $Annotations {
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      set value    [tip_GetValueValue [tip_GetAttribute $ann $Attribute]]
      foreach {bg fg} $Parameters($win:Colour:$value) {break}
      $Viewer.text tag configure id_$ID -foreground $fg -background $bg
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}

  $Viewer.text configure -state disabled -exportselection 1
  $Viewer.text tag add back 1.0 end
  $Viewer.text tag bind back <<MB_1>> \
    "::EditAnnotationsTool::HilightToolButton $win"
  $Viewer.text tag lower back
  focus $Viewer.text
  $Viewer.text tag raise sel
}; # ::EditAnnotationsTool::UpdateWindow

proc ::EditAnnotationsTool::CloseWindow {win} {
  variable Parameters
  global GUI_Text
  if {[llength [info commands $Parameters($win:Document)]]} {
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  destroy $win
};# ::EditAnnotationsTool::CloseWindow

## ::EditAnnotationsTool::tag2value
#  Retrieve the value of a specific tag
proc ::EditAnnotationsTool::tag2value {win tag} {
  set id  [string range $tag 3 end]
  #if {![string length $id]} {return {}}
  variable Parameters
  set ann [tip_GetAnnotation $Parameters($win:Document) $id]
  return  [tip_GetValueValue \
               [tip_GetAttribute $ann $Parameters($win:Attribute)]]
};# ::EditAnnotationsTool::tag2value

## ::EditAnnotationsTool::tag2button
#  Retrieve the button widget for a specific tag
proc ::EditAnnotationsTool::tag2button {win tag} {
  variable Parameters
  set Parameters($win:Button:[tag2value $win $tag])
};# ::EditAnnotationsTool::tag2button

## ::EditAnnotationsTool::tag2colours
#  Retrieve the colours (bg fg) for a specific tag
proc ::EditAnnotationsTool::tag2colours {win tag} {
  variable Parameters
  set Parameters($win:Colour:[tag2value $win $tag])
};# ::EditAnnotationsTool::tag2colours

## ::EditAnnotationsTool::HilightToolButton
#   Hilight the appropriate tool button...
proc ::EditAnnotationsTool::HilightToolButton {win {tag {}}} {
  variable Parameters
  set text $Parameters($win:TextObj)
  set ::GUI_StatusBar::StatusBar($win) $::GUI_StatusBar::StatusBar($win:Def)
  if {[info exists Parameters($win:Selected)]} {
    set sel_tag $Parameters($win:Selected)
    $text tag configure $sel_tag -relief flat -borderwidth 0
    [tag2button $win $sel_tag] configure -background [$win cget -background] \
      -foreground black -relief raised
  }
  unset -nocomplain Parameters($win:Selected)
  if {![string length $tag] || ![string match id_* $tag]} {
    ## The user has clicked on the text background...
    unset -nocomplain Parameters($win:SelectRangeWidget_LastRange)
    $Parameters($win:SelectRangeWidget) configure -values {} \
                                        -text {} -state disabled
    return
  }
  $text tag configure $tag -relief ridge -borderwidth 4
  set Parameters($win:Selected) $tag
  foreach {bg fg} [tag2colours $win $tag] {break}
  [tag2button $win $tag] configure -background $bg -foreground $fg \
    -relief sunken
  ## Check to see whether they are overlapped tags by the selected one...
  set start [$text index $tag.first]
  set end   [$text index $tag.last]
  set ::GUI_StatusBar::StatusBar($win) \
      "$::GUI_Text(Text): \"[$text get $start $end]\",\
       $::GUI_Text(Value): [tag2value $win $tag],\
       $::GUI_Text(Span): ($start,$end), ID: [string range $tag 3 end]"
  ## Ensure that this range is not contained within a range from a previous
  ## binding. (Tk will call all bindings for all text tags when an index is
  ## clicked)
  if {[info exists Parameters($win:SelectRangeWidget_LastRange)]} {
    foreach {ls le} $Parameters($win:SelectRangeWidget_LastRange) {break}
    #if {$ls <= $start && $le >= $end} {return}
    ## If we got here, our range is outside the previous one, so reset the
    ## select widget...
    $Parameters($win:SelectRangeWidget) configure -values {} -text {} \
                                                  -state disabled
  }
  for {set index $start} {$index < $end} {set index [$text index $index+1c]} {
    foreach ptag [$text tag names $index] {
      switch $ptag [list \
        $tag {}  \
        back {}  \
        sel  {}  \
        default {
          lappend Tags($ptag) $index
        } \
      ]
    }
  }
  ## Iterate over all overlapped tags...
  if {[info exists Tags]} {
    set Parameters($win:SelectRangeWidget_LastRange) [list $start $end]
    lappend Values \
      "$::GUI_Text(Text): \"[$text get $start $end]\",\
       $::GUI_Text(Value): [tag2value $win $tag],\
       $::GUI_Text(Span): ($start,$end), ID: [string range $tag 3 end]"
    foreach ptag [lsort [array names Tags]] {
      set range [$text tag nextrange $ptag $start $end]
      while {[llength $range]} {
        foreach {p_start p_end} $range {break}
        lappend Values \
          "$::GUI_Text(Text): \"[$text get $p_start $p_end]\",\
           $::GUI_Text(Value): [tag2value $win $ptag],\
           $::GUI_Text(Span): ($p_start,$p_end), ID: [string range $ptag 3 end]"
        set range [$text tag nextrange $ptag $p_end $end]
      }
    }
    $Parameters($win:SelectRangeWidget) configure -values $Values \
                                                  -state normal
    $Parameters($win:SelectRangeWidget) setvalue @0
  }
}; # ::EditAnnotationsTool::HilightToolButton

## ::EditAnnotationsTool::SelectOverlapped
#  Select a tag from the overlaped tags selection combo box
proc ::EditAnnotationsTool::SelectOverlapped {win combo} {
  set value [lindex [$combo cget -values] [$combo getvalue]]
  set id [lindex [split $value] end]
  variable Parameters
  $Parameters($win:TextObj) tag raise id_$id
  HilightToolButton $win id_$id
};# ::EditAnnotationsTool::SelectOverlapped

## ::EditAnnotationsTool::EditAccelerator
#  Add/Modify an accelerator for a button widget...
proc ::EditAnnotationsTool::EditAccelerator {button} {
  ## Create a window for setting the binding...
  set win .tooleditAnnotationsEditAccelerator
  if {[winfo exists $win]} {
    raise $win
    return
  }
  toplevel .tooleditAnnotationsEditAccelerator
  wm withdraw $win
  wm title $win "Setting Button Accelerator..."
  message $win.label -width 250 -text \
    "Please press the keyboard keys combination you want to assign as an\
     accelerator to the button with value \"[$button cget -text]\""
  label $win.acc
  bind $win <Key> \
    [list ::EditAnnotationsTool::SetAccelerator $win.acc $button %K]
  button $win.ok -textvariable GUI_Text(Ok) -command "destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -command \
  "destroy $win; ::EditAnnotationsTool::SetAccelerator $win.acc $button"
  grid $win.label -sticky snew -columnspan 3 -sticky snew
  grid $win.acc   -sticky snew -columnspan 3 -sticky snew
  grid $win.ok x $win.cancel -padx 20 -pady 10  -sticky snew
  grid columnconfigure $win 1 -weight 1
  gut_ModalDlg $win
  wm deiconify $win
};# ::EditAnnotationsTool::EditAccelerator

proc ::EditAnnotationsTool::SetAccelerator {label button {key {}}} {
  catch {$label configure -text "Key: \"$key\""}
  set toplevel [winfo toplevel $button]
  ## Remove any previously set accelerators...
  foreach event [bind $button] {
    switch -glob -- $event {
      ? -
      <Key-*> {
        bind $button   $event {}
        bind $toplevel $event {}
      }
      default {}
    }
  }
  if {![string length $key]} {return}
  bind $button   <Key-$key> "$button invoke; $button flash"
  bind $toplevel <Key-$key> "$button invoke; $button flash"
};# ::EditAnnotationsTool::EditAccelerator

## ::EditAnnotationsTool::AutoTag
#   This function will tag automatically all instances of selected text with
#   the current tag. Use with care :-)
proc ::EditAnnotationsTool::AutoTag {win} {
  variable Parameters
  
  if {![info exists Parameters($win:Selected)]} {return}
  set text $Parameters($win:TextObj)
  set ptag $Parameters($win:Selected)

  ## Get the staring point that we will search (towards the end of the text)...
  set index [$text index $ptag.first]

  ## Get the text that we should search for...
  set pattern [$text get $index $ptag.last]
  set attr [tag2value $win $ptag]

  ## Well, its a good idea to ask the user first...
  set ans [gut_question "Do you want to tag all occurances of the token\
    \"$pattern\" with the attribute value \"$attr\"?"]
  if {[string equal $ans no]} {return}
  set count [string length $pattern]
  incr count
  set pattern "(\\s|\\W)${pattern}(\\s|\\W)"

  ## Start searching forward...
  while {1} {
    set index [$text search -forwards -regexp -- $pattern $index end]
    if {![string length $index]} {break}
    $text tag delete sel
    $text tag add sel $index+1c $index+${count}c
    ::EditAnnotationsTool::Tag $win {} $attr
    set index $index+${count}c
  }
};# ::EditAnnotationsTool::AutoTag


## ::EditAnnotationsTool::Tag
#   The Callback of the ToolButtons...
proc ::EditAnnotationsTool::Tag {win button value} {
  variable Parameters
  set text $Parameters($win:TextObj)
  set doc  $Parameters($win:Document)

  ## If there is selected text...
  if {![catch {$text index sel.first}]} {
    set first [$text index sel.first]
    set last  [$text index sel.last]
    set start [gut_FindCharacterOffset $text $first]
    set end   [gut_FindCharacterOffset $text $last]
    ## Is there an Annotation with the same span?
    set Anns [tip_SelectAnnotations \
                  [tip_AnnotationsMatchingRange $doc $start $end] \
                  $Parameters($win:Annotation)]
    if {[llength $Anns]} {
      foreach ann $Anns {
        ## The Selected Span matches an existing annotation
        #unset -nocomplain Parameters($win:Selected)
        set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                      [tip_CreateAttributeValue GDM_STRING $value]]
        set ann [tip_PutAttribute $ann $attr]
        set ID [tip_AddAnnotation $doc $ann]
        ## puts "Changing Annotation with ID $ID: $attr -> $value"
      }
    } else {
      ## A New Annotation must be created
      set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                   [tip_CreateAttributeValue GDM_STRING $value]]
      set ann  [tip_CreateAnnotation $Parameters($win:Annotation) \
                   [list [tip_CreateSpan $start $end]] [list $attr]]
      set ID [tip_AddAnnotation $doc $ann]
      ## puts "Creating New $Annotation Annotation ($ID, $value)"
    }
    $text tag add id_$ID $first $last
    foreach {bg fg} [tag2colours $win id_$ID] {break}
    $text tag configure id_$ID -foreground $fg -background $bg
    $text tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationsTool::HilightToolButton $win id_$ID]
    $text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $value]
    $text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
    $text tag remove sel $first $last
  } elseif {[info exists Parameters($win:Selected)]} {
    ## If there is a selected tag...
    set ptag $Parameters($win:Selected)
    set but  [::EditAnnotationsTool::tag2button $win $ptag]
    ## No need to clasify text in the category it already is...
    if {[string equal $but $button]} {return}
    $but configure -background [$win cget -background] -relief raised
    ## Change the Attribute
    set ID   [string range $ptag 3 end]
    set ann  [tip_GetAnnotation $doc $ID]
    set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                  [tip_CreateAttributeValue GDM_STRING $value]]
    set ann  [tip_PutAttribute $ann $attr]
    tip_AddAnnotation $doc $ann
    foreach {bg fg} [tag2colours $win $ptag] {break}
    $button configure -background $bg -relief sunken
    $text tag configure id_$ID -foreground $fg -background $bg
    $text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $value]
    $text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
    ## puts "Changing Annotation with ID $ID: $attr -> $value"
  }
};# ::EditAnnotationsTool::Tag

## ::EditAnnotationsTool::TagWithNewAnnotation
proc ::EditAnnotationsTool::TagWithNewAnnotation {win button value} {
  variable Parameters
  set text $Parameters($win:TextObj)
  set doc  $Parameters($win:Document)

  if {[info exists Parameters($win:Selected)]} {
    ## If there is a selected tag...
    set ptag $Parameters($win:Selected)
    set but  [::EditAnnotationsTool::tag2button $win $ptag]
    ## No need to clasify text in the category it already is...
    if {[string equal $but $button]} {return}
    $but configure -background [$win cget -background] -relief raised
    ## Create a new Annotation, with the same span as the existing one...
    set ID   [string range $ptag 3 end]
    set ann  [tip_GetAnnotation $doc $ID]
    tip_GetFirstSpanOffsets $ann start end
    ## A New Annotation must be created
    set attr [tip_CreateAttribute $Parameters($win:Attribute) \
                 [tip_CreateAttributeValue GDM_STRING $value]]
    set ann  [tip_CreateAnnotation $Parameters($win:Annotation) \
                 [list [tip_CreateSpan $start $end]] [list $attr]]
    set ID [tip_AddAnnotation $doc $ann]

    ## Add a new tag, that overlaps the one in selection...
    $text tag add id_$ID [$text index ${ptag}.first] [$text index ${ptag}.last]
    foreach {bg fg} [tag2colours $win id_$ID] {break}
    $text tag configure id_$ID -foreground $fg -background $bg
    $text tag bind id_$ID <<MB_1>> \
        [list ::EditAnnotationsTool::HilightToolButton $win id_$ID]
    $text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $value]
    $text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
    $button configure -background $bg -relief sunken
    ## puts "Changing Annotation with ID $ID: $attr -> $value"
  }
};# ::EditAnnotationsTool::TagWithNewAnnotation

## ::EditAnnotationsTool::Delete
#  
proc ::EditAnnotationsTool::Delete {win {Annotation {}}} {
  variable Parameters
  set text $Parameters($win:TextObj)
  set doc  $Parameters($win:Document)
  if {[info exists Parameters($win:Selected)]} {
    set ptag $Parameters($win:Selected)
    unset Parameters($win:Selected)
    set ID [string range $ptag 3 end]
    [::EditAnnotationsTool::tag2button $win $ptag] \
      configure -background [$win cget -background] -relief raised
    $text tag delete $ptag
    if {[string length $Annotation]} {
      ## puts "Deleting whole Annotation with ID $ID"
      tip_RemoveAnnotation $doc $ID
    } else {
      ## puts "Deleting $attr $value from Annotation with ID $ID"
      set ann [tip_GetAnnotation $doc $ID]
      set ann [tip_RemoveAttribute $ann $Parameters($win:Attribute)]
      tip_AddAnnotation $doc $ann
    }
  }
};# ::EditAnnotationsTool::Delete
