##########################################################################
# 
#	EditAttributes.tcl - Monday December 09 21:54:50 (EET) 2002
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.5...
#
##########################################################################


# Location of this module (EditAttributes)
global creole_EditAttributes_home

## creole_EditAttributes
#  
proc creole_EditAttributes {doc col args} {
  package require Tktable
  global GUI_Text GUI_Options creole_EditAttributes
  set win [ggi_unused_win .creole_EditAttributesTool]
  toplevel $win
  wm withdraw $win
  wm title $win "Tool for Editing Attributes..."
  GUI_LoadImage BackNice
  GUI_LoadImage NextNice

  ## Status Bar
  set ::GUI_StatusBar::StatusBar($win) \
    "$GUI_Text(Collection):( $GUI_Text(none) )    \
     $GUI_Text(Document):( $GUI_Text(none) )"
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  GUI_CreateStatusBar $win.statusbar
  
  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
    ## Open Document/Collection Express Toolbar
    set toolbar $toolbar_frame.express
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
       -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -float 0
    Toolbar::addwidget $toolbar label -bd 0 -textvariable \
       GUI_Text(Collection) -tooltipvar GUI_Text(TSelectCollection) \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
       $GUI_Options(InputBackground) -highlightthickness 0 \
       -foreground $GUI_Options(InputForeground) \
       -name $toolbar.widgets.collection -entercmd \
       {%W configure -values [GUI_GetCollectionNameTails]} -modifycmd \
       "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
        $toolbar.widgets.document $toolbar.widgets.collection
        $toolbar.widgets.document configure -values \
        \[lrange \[$toolbar.widgets.document cget -values\] 1 end\]
        $toolbar.widgets.document setvalue first" \
       -tooltipvar GUI_Text(TSelectCollection) -width \
       $GUI_Options(SystemExprToolbarCollectionWidth) -takefocus 0 -pady 3 \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar label -bd 0 -textvariable \
       GUI_Text(Document) -tooltipvar GUI_Text(TSelectDocument) \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
       $GUI_Options(InputBackground) -highlightthickness 0 \
       -foreground $GUI_Options(InputForeground) \
       -name $toolbar.widgets.document \
       -tooltipvar GUI_Text(TSelectDocument) -width \
       $GUI_Options(SystemExprToolbarDocumentWidth) -takefocus 0 -pady 3 \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
       -textvariable GUI_Text(Load) -image ITLoad -command \
       "creole_EditAttributes_DocLoadRequest $win \
        $toolbar.widgets.collection $toolbar.widgets.document" \
       -tooltipvar GUI_Text(TSelectedLoad) \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addbutton $toolbar -image BackNice -command \
       "creole_EditAttributes_DocLoadRequest $win \
        $toolbar.widgets.collection $toolbar.widgets.document Prev"
    Toolbar::addbutton $toolbar -image NextNice -command \
       "creole_EditAttributes_DocLoadRequest $win \
        $toolbar.widgets.collection $toolbar.widgets.document Next"
    Toolbar::addbutton $toolbar -image ITSave -tooltipvar GUI_Text(Save) \
       -statusvar ::GUI_StatusBar::StatusBar($win) \
       -command "if {\[string length \$creole_EditAttributes($win:Document)]} \
                    {tip_Sync \$creole_EditAttributes($win:Document)}"
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -image ITAddCurveP \
       -command "creole_EditAttributes_AddNewAttribute $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TViewDocAttrs) \
        -image ITViewDocAtt -command \
        "ggi_view_Attributes \$creole_EditAttributes($win:Document) {} 1" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    Toolbar::addseparator $toolbar
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command "destroy $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    pack $toolbar -side top -anchor w
  pack $toolbar_frame -side top -fill x
  
  ## Create the main window area...
  panedwindow $win.area -orient horizontal
    ## The text area...
    ::ScrolledWidgets::ScrolledText $win.area.text -state disabled
    ## The attributes area...
    ScrolledWindow  $win.area.attrs
    table $win.area.attrs.table -cols 4 -rows 1 -colstretchmode unset \
          -cache 1 -selecttype row -state disabled -titlerows 1 -titlecols 1
    $win.area.attrs setwidget $win.area.attrs.table
    update
  $win.area add $win.area.text $win.area.attrs -sticky snew
  pack $win.area -fill both -expand 1 -ipadx 10 -ipady 10
  $win.area sash place 0 350 2
  update
  wm deiconify $win
  set creole_EditAttributes($win:Document)   {}
  set creole_EditAttributes($win:Collection) {}
};# creole_EditAttributes

proc creole_EditAttributes_DocLoadRequest {win ColW DocW {what This}} {
  global GUI_Text creole_EditAttributes
  set Col [$ColW getvalue]
  set Doc [$DocW getvalue]

  ## Warn user if Document is modified, before we close it...
  if {[llength [info commands $creole_EditAttributes($win:Document)]]} {
    if {[$creole_EditAttributes($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $creole_EditAttributes($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $creole_EditAttributes($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  switch $what {
    Prev    {incr Doc -1; $DocW setvalue previous}
    Next    {incr Doc;    set NEXT [$DocW setvalue next]
             if {$NEXT == 0} {return}}
    default {}
  }

  if {$Col == -1 || $Doc == -1} {return}

  catch {
    tip_Close $creole_EditAttributes($win:Document)
  }
  set creole_EditAttributes($win:Document) {}
  set col $creole_EditAttributes($win:Collection)
  set Col [lindex [GUI_GetCollectionNames] $Col]
  if {[string length $col]} {
    if {[llength [info commands $col]]} {
      if {![string equal [tip_GetName $col] $Col]} {
        tip_Close $col
      }
    }
    set col [tip_OpenCollection $Col]
  } else {
    set col [tip_OpenCollection $Col]
  }
  ## Is the Collection's Location in accordance with it's Name?
  set ColName [tip_GetName $col]
  if {![string equal $ColName $Col]} {
    GUI_FixCollectionName $col $ColName $Col
  }
  
  set Doc [lindex [gut_GetCollectionDocuments $Col] $Doc]
  set doc [tip_GetDocument $col $Doc]
  set creole_EditAttributes($win:Document) $doc
  set creole_EditAttributes($win:Collection) $col

  bind $win.area.attrs.table <Destroy> "catch \"$col Close\""

  ## Set the text area...
  $win.area.text.text configure -state normal
  $win.area.text.text delete 1.0 end
  $win.area.text.text insert end [tip_GetRawData $doc]
  $win.area.text.text configure -state disabled

  ## Update the Attribute Information...
  foreach attr [tip_GetAttributes $doc] {
    set name  [tip_GetName       $attr]
    set type  [tip_GetValueType  $attr]
    set value [tip_GetValueValue $attr]
    set creole_EditAttributes($win:Attr:$name:$type:$value) 0
  }
  creole_EditAttributes_UpdateAttributesDisplay $win
  global GUI_Text
  set ::GUI_StatusBar::StatusBar($win:Def) \
        "$GUI_Text(Collection):($Col), $GUI_Text(Document):($Doc)"
  set ::GUI_StatusBar::StatusBar($win) $::GUI_StatusBar::StatusBar($win:Def)
};# creole_EditAttributes_DocLoadRequest

proc creole_EditAttributes_UpdateAttributesDisplay {win} {
  global creole_EditAttributes
  $win.area.attrs.table clear all
  foreach child [winfo children $win.area.attrs.table] {
    destroy $child
  }
  set keys [array names creole_EditAttributes $win:Attr:*]
  $win.area.attrs.table configure -rows [expr {[llength $keys]+1}] -state normal
  $win.area.attrs.table set 0,1 Name 0,2 Value 0,3 Type
  set i 1
  foreach key [lsort $keys] {
    foreach {_ _ name type value} [split $key :] {break}
    ## Does the Attribute exists in the Document?
    set attr [tip_GetAttribute $creole_EditAttributes($win:Document) $name]
    if {[llength $attr] &&
        [string equal [tip_GetValueValue $attr] $value] &&
        [string equal [tip_GetValueType  $attr] $type]} {
      button $win.area.attrs.table.t$i -text { - } -bg red     -fg white \
        -command [list creole_EditAttributes_ToggleAttribute $win \
                       $win.area.attrs.table.t$i $name $type $value]
    } else {
      button $win.area.attrs.table.t$i -text { + } -bg #8fbc8f -fg white \
        -command [list creole_EditAttributes_ToggleAttribute $win \
                       $win.area.attrs.table.t$i $name $type $value]
    }
    $win.area.attrs.table set $i,1 $name $i,2 $value $i,3 ($type)
    $win.area.attrs.table window configure $i,0 \
           -window $win.area.attrs.table.t$i -relief flat -sticky snew
    incr i
  }
  $win.area.attrs.table tag configure button -relief flat 
  $win.area.attrs.table tag col button 0
  $win.area.attrs.table width 0 8
  $win.area.attrs.table configure -state disabled
};# creole_EditAttributes_UpdateAttributesDisplay

## creole_EditAttributes_ToggleAttribute
## Adds/Removes an Attribute from the Document...
proc creole_EditAttributes_ToggleAttribute {win button name type value} {
  global creole_EditAttributes
  set doc $creole_EditAttributes($win:Document)
  if {![string length $doc]} {return}
  set attr [tip_GetAttribute $creole_EditAttributes($win:Document) $name]
  if {[llength $attr] &&
      [string equal [tip_GetValueValue $attr] $value] &&
      [string equal [tip_GetValueType  $attr] $type]} {
    ## Attribute exists. Remove it...
    tip_RemoveAttribute $doc $name
  } else {
    ## The Attribute does not exists or has a different value. Add/Replace
    ## it...
    tip_PutAttribute $doc [tip_CreateAttribute $name \
                               [tip_CreateAttributeValue $type $value]]
  }
  creole_EditAttributes_UpdateAttributesDisplay $win
};# creole_EditAttributes_ToggleAttribute

## creole_EditAttributes_AddNewAttribute
## Create a dialog where the user can add a new Attribute...
proc creole_EditAttributes_AddNewAttribute {win} {
  set dialog $win.add_newAttribute
  if {[winfo exists $dialog]} {
    raise $win
    return
  }
  set Values {GDM_STRING GDM_STRING_SET}
  toplevel $dialog
  wm withdraw $dialog
  wm transient $dialog $win
  wm title $dialog {Add a new Attribute...}
  grid [label $dialog.lname -text Name:] \
       [entry $dialog.name] -padx 2 -pady 2 -sticky snew
  grid [label $dialog.ltype -text Type:] \
       [ComboBox $dialog.type -values $Values -editable 0] \
       -padx 2 -pady 2 -sticky snew
  $dialog.type setvalue first
  grid [label $dialog.lval -text Value:] \
       [entry $dialog.val] -padx 2 -pady 2 -sticky snew
  frame $dialog.buttons
    button $dialog.buttons.ok -text $::GUI_Text(Ok) -command \
       "set creole_EditAttributes($win:Attr:\[$dialog.name get\]:\[lindex \"$Values\" \[$dialog.type getvalue\]\]:\[$dialog.val get\]) 0;creole_EditAttributes_UpdateAttributesDisplay $win;destroy $dialog"
    button $dialog.buttons.cancel -text $::GUI_Text(Cancel) \
               -command "destroy $dialog"
  pack $dialog.buttons.ok $dialog.buttons.cancel -padx 10 -pady 10 -side left
  grid $dialog.buttons -columnspan 2
  grid columnconfigure $dialog 1 -weight 1
  grid rowconfigure $dialog 3 -weight 1
  gut_ModalDlg $dialog
  wm deiconify $dialog
};# creole_EditAttributes_AddNewAttribute

## Procedure: creole_EditAttributes_Initialize
#   Use this function in order to do some initialisation. This function will
#   be called just before creole_EditAttributes in the following situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just before the first document in Collection gets proccessed 
#       with creole_EditAttributes
#    *) If the user has opened a single Document, this function will be called
#       just before calling creole_EditAttributes
#   Please, refer to the Ellogon's Programming Manual for more 
#   information on this function...
proc creole_EditAttributes_Initialize {col doc args} {
  global creole_EditAttributes_home
  ## Do some initilisation here...
};# creole_EditAttributes_Initialize

## Procedure: creole_EditAttributes_Finish
#   Use this function in order to do some Clean-Up. This function will
#   be called after all calls to creole_EditAttributes in the following 
#   situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just after the last document in Collection gets proccessed with 
#       creole_EditAttributes
#    *) If the user has opened a single Document, this function will be called
#       just after calling creole_EditAttributes
#   Please, refer to the Ellogon's Programming Manual for more 
#   information on this function...
proc creole_EditAttributes_Finish {col doc args} {
  global creole_EditAttributes_home
  ## Clean-Up...
};# creole_EditAttributes_Finish

#
# End of File EditAttributes.tcl
#

