##########################################################################
# 
#       EditCoreferenceOnHTML.tcl - Fri May 16  2003 10:14:52
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.5...
#
##########################################################################


# Location of this module (EditCoreferenceOnHTML)
global creole_EditCoreferenceOnHTML_home

namespace eval ::EditCoreferenceOnHTMLTool {
  variable Parameters
  variable ValuesSpec
}
namespace eval ::HtmlInfoViewer {}

## Load some predifined Values...
proc ::EditCoreferenceOnHTMLTool::Init {} {
  ::Annotation::Init
  foreach filename [list \
                     $::creole_EditCoreferenceOnHTML_home/PredifinedValues.tcl \
                     $::UserAppDir/EditCoreference.cfg \
                     $::UserHomeDir/.EditCoreference.cfg \
                     $::UserHomeDir/EditCoreference.cfg] {
    if {![file exists $filename]} {continue}
    set file [open $filename]
      fconfigure $file -encoding utf-8
      set data [read $file]
    close $file
    if {[catch {eval $data} error]} {
      set msg "The following error occured while reading predifined values\
               from file \"$filename\":\n$error"
      catch {bgerror $msg}
    }
  }
};# ::EditCoreferenceOnHTMLTool::Init

## creole_EditCoreferenceOnHTML
#  
proc creole_EditCoreferenceOnHTML {doc col} {
  ::EditCoreferenceOnHTMLTool::Init
  ## Create the main window...
  set win [ggi_unused_win .tooleditcoreferences_main_window]
  array unset ::EditCoreferenceOnHTMLTool::Parameters $win:*

  set ::EditCoreferenceOnHTMLTool::Parameters($win:Document)   $doc
  set ::EditCoreferenceOnHTMLTool::Parameters($win:Collection) $col
  ::EditCoreferenceOnHTMLTool::GetParameters $win $doc $col
  ## Has the user cancelled the procedure?
  if {$::EditCoreferenceOnHTMLTool::Parameters($win:Cancel)} {
    array unset ::EditCoreferenceOnHTMLTool::Parameters $win:*
    return
  }
  if {![string length \
               $::EditCoreferenceOnHTMLTool::Parameters($win:Annotation)]} {
    return
  }
  set Annotation $::EditCoreferenceOnHTMLTool::Parameters($win:Annotation)
  set i 0
  set Attributes {}
  while {[incr i]} {
    if {![info exists ::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)]} {
      break
    }
    set attr_name $::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)
    if {![string length $attr_name]} {break}
    lappend Attributes $attr_name
  }
  ::EditCoreferenceOnHTMLTool::CreateWindow $win $doc $col \
                 $Annotation $Attributes
  tkwait window $win
  array unset ::EditCoreferenceOnHTMLTool::Parameters $win:*
};# creole_EditCoreferenceOnHTML

## ::EditCoreferenceOnHTMLTool::DocLoad
#   Load the requested document...
proc ::EditCoreferenceOnHTMLTool::DocLoad {win ColW DocW {what This}} {
  global GUI_Text
  variable Parameters
  set Col [$ColW getvalue]
  set Doc [$DocW getvalue]

  ## Warn user if Document is modified, before we close it...
  if {[llength [info commands $Parameters($win:Document)]]} {
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  switch $what {
    Prev    {incr Doc -1; $DocW setvalue previous}
    Next    {incr Doc;    set NEXT [$DocW setvalue next]
             if {$NEXT == 0} {return}}
    default {}
  }

  if {$Col == -1 || $Doc == -1} {return}

  catch {tip_Close $Parameters($win:Document)}
  set col $Parameters($win:Collection)
  set doc $Parameters($win:Document)
  if {!$Parameters($win:SystemOwnsCollection)} {
    set Col [lindex [GUI_GetCollectionNames] $Col]
    if {[string length $col]} {
      if {![string equal [file tail [tip_GetName $col]] $Col]} {
        tip_Close $col
        set col [tip_OpenCollection $Col]
      }
    } else {
      set col [tip_OpenCollection $Col]
    }
    ## Is the Collection's Location in accordance with it's Name?
    set ColName [tip_GetName $col]
    if {![string equal $ColName $Col]} {
      GUI_FixCollectionName $col $ColName $Col
    }
  }
  
  set Doc [lindex [$DocW cget -values] $Doc]
  set doc [tip_GetDocument $col $Doc]
  set Parameters($win:Document) $doc
  set Parameters($win:Collection) $col
  set Annotation $::EditCoreferenceOnHTMLTool::Parameters($win:Annotation)
  set i 0
  set Attributes {}
  while {[incr i]} {
    if {![info exists ::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)]} {
      break
    }
    set attr_name $::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)
    if {![string length $attr_name]} {break}
    lappend Attributes $attr_name
  }

  UpdateWindow $win $doc $col $Annotation $Attributes
  set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  if {!$Parameters($win:SystemOwnsCollection)} {
    bind $win.area.text.text <Destroy> "catch \"tip_Close $col\""
  }
};# ::EditCoreferenceOnHTMLTool::DocLoad

## ::EditCoreferenceOnHTMLTool::DocSave
#    Save current document...
proc ::EditCoreferenceOnHTMLTool::DocSave {win} {
  variable Parameters
  if {[string length $Parameters($win:Document)]} {
    tip_Sync $Parameters($win:Document)
  }
};# ::EditCoreferenceOnHTMLTool::DocSave

## ::EditCoreferenceOnHTMLTool::GetParameters
#   Displays a small window, where the user can type the annotation and the
#   the annotation attribute (optional) he wants to edit.
proc ::EditCoreferenceOnHTMLTool::GetParameters {win doc col} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec

  ## Collect the predefined values...
  set PredefinedEntries {}
  foreach entry [array names ValuesSpec] {
    set index [lsearch -exact $ValuesSpec($entry) title]
    if {$index != -1} {
      ## A title was found...
      lappend PredefinedEntries [lindex $ValuesSpec($entry) [incr index]]
    } else {
      lappend PredefinedEntries $entry
    }
  }
  set PredefinedEntries [linsert \
          [lsort -unique -dictionary $PredefinedEntries] 0 {}]
  toplevel $win
  wm withdraw $win
  wm title $win "Edit Coreferences Tool Options..."

  label $win.title -text {Edit Coreferences Tool Options:}
  pack $win.title -side top -fill x

  set frame $win.frame
  frame $frame -borderwidth 2 -relief groove
    label $frame.lschema -textvariable GUI_Text(PredefinedSchema:) -anchor e
    ComboBox $frame.schema -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 0 -width 40 \
      -values $PredefinedEntries \
      -textvariable ::EditCoreferenceOnHTMLTool::Parameters($win:Schema) \
      -modifycmd \
      "::EditCoreferenceOnHTMLTool::GetParametersChangeCategory $frame"
    grid $frame.lschema $frame.schema -sticky snew -ipadx 3 -ipady 3 \
      -padx 6 -pady 2
    label $frame.lann -textvariable GUI_Text(AnnotationType:) -anchor e
    entry $frame.ann \
      -textvariable ::EditCoreferenceOnHTMLTool::Parameters($win:Annotation)
    grid $frame.lann $frame.ann -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    #for {set i 1} {$i <= 5} {incr i} {
    #  label $frame.label$i -text "$GUI_Text(Attribute) $i:" -anchor e
    #  entry $frame.attr$i -textvariable \
    #    ::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)
    #  grid $frame.label$i $frame.attr$i \
    #    -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    #}
    grid columnconfigure $frame 0 -weight 0 
    grid columnconfigure $frame 1 -weight 1 
  pack $frame -fill both -expand 1 -ipadx 5 -ipady 5 -padx 3 -pady 3

  set bframe $win.buttons
  frame $bframe
    button $bframe.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
      -default active
    bind $win <Return> "$bframe.ok invoke"
    button $bframe.cancel -textvariable GUI_Text(Cancel) -command \
     "set ::EditCoreferenceOnHTMLTool::Parameters($win:Cancel) 1
      destroy $win"
    pack $bframe.ok $bframe.cancel -padx 3 -pady 5 -side left
  pack $bframe
  focus $frame.ann
  $frame.schema setvalue @1
  GetParametersChangeCategory $frame
  gut_ModalDlg $win
  wm deiconify $win
  set Parameters($win:Cancel) 0
  tkwait window $win
}; #::EditCoreferenceOnHTMLTool::GetParameters

proc ::EditCoreferenceOnHTMLTool::GetParametersChangeCategory {frame} {
  global GUI_Text
  variable Parameters
  set win [winfo toplevel $frame]
  set sel [lindex [$frame.schema cget -values] [$frame.schema getvalue]]
  if {[string length $sel]} {
    variable ValuesSpec
    ## Locate the predefined value...
    foreach entry [array names ValuesSpec] {
      set index [lsearch -exact $ValuesSpec($entry) title]
      if {$index != -1} {
        ## A title was found...
        set title [lindex $ValuesSpec($entry) [incr index]]
        if {[string equal $title $sel]} {break}
      }
    }
    ## Get Annotation...
    set index [lsearch -exact $ValuesSpec($entry) annotation]
    set ann [lindex $ValuesSpec($entry) [incr index]]
    set Parameters($win:Annotation) $ann
    $frame.ann configure -state readonly
    set i 1
    while {[set index [lsearch -exact -start \
                               $index $ValuesSpec($entry) attribute]] != -1} {
      set Parameters($win:Attr$i) [lindex $ValuesSpec($entry) [incr index]]
      if {![winfo exists $frame.attr$i]} {
        label $frame.label$i -text "$GUI_Text(Attribute) $i:" -anchor e
        entry $frame.attr$i -textvariable \
          ::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)
        grid $frame.label$i $frame.attr$i \
          -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
      }
      $frame.attr$i configure -state readonly
      incr i
    }
    while {1} {
      if {[winfo exists $frame.attr$i]} {
        destroy $frame.label$i $frame.attr$i
        unset Parameters($win:Attr$i)
      } else {
        break
      }
      incr i
    }
  } else {
    $frame.ann configure -state normal
    for {set i 1} {$i <= 5} {incr i} {
      if {[winfo exists $frame.attr$i]} {
        $frame.attr$i configure -state normal
      } else {
        label $frame.label$i -text "$GUI_Text(Attribute) $i:" -anchor e
        entry $frame.attr$i -textvariable \
          ::EditCoreferenceOnHTMLTool::Parameters($win:Attr$i)
        grid $frame.label$i $frame.attr$i \
          -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
      }
    }
    for {set i 6} {1} {incr i} {
      if {[winfo exists $frame.attr$i]} {
        destroy $frame.label$i $frame.attr$i
        unset Parameters($win:Attr$i)
      } else {
        break
      }
    }
  }
};# ::EditCoreferenceOnHTMLTool::GetParametersChangeCategory

## ::EditCoreferenceOnHTMLTool::CreateWindow
proc ::EditCoreferenceOnHTMLTool::CreateWindow {win doc col 
                                  {Annotation {}} {Attributes {}}} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec
  set title "Edit Coreferences Tool: Editing Annotation \"$Annotation\"..."

  GUI_LoadImage BackNice
  GUI_LoadImage NextNice
  GUI_LoadImage ITShowTags
  GUI_LoadImage ITHideTags
  set HaveSystemCol 0
  set HaveSystemDoc 0
  set Parameters($win:SystemOwnsCollection) 0
  if {[string equal $doc $GUI_Text(All)]} {set doc {}}
  if {[string length $col] && ![string length $doc]} {
    set doc [tip_FirstDocument $col]
    set Parameters($win:SystemOwnsCollection) 1
    set HaveSystemCol 1
  }
  if {![string length $col] || ![string length $doc] || \
      ![string length $Annotation] || ![llength $Attributes]} {
    set DocText {}
    set Annotations {}
  } else {
    ## Get Document Annotations and text
    if {[catch {tip_GetByteSequence $doc} DocText]} {
      error "creole_EditCoreferenceOnHTML: Invalid Document!\n($DocText)"
    }
    set Annotations [tip_SelectAnnotations $doc $Annotation]
    set HaveSystemDoc 1
  }

  ## Create the window
  toplevel $win
  wm withdraw $win
  wm title $win $title

  ## Status Bar
  if {$HaveSystemDoc} {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):([file tail [tip_GetName $col]]), \
       $GUI_Text(Document):([tip_GetId $doc])"
  } else {
    set ::GUI_StatusBar::StatusBar($win) \
      "$GUI_Text(Collection):( $GUI_Text(none) )    \
       $GUI_Text(Document):( $GUI_Text(none) )"
  }
  set ::GUI_StatusBar::StatusBar($win:Def) $::GUI_StatusBar::StatusBar($win)
  GUI_CreateStatusBar $win.statusbar

  ## Toolbar...
  set toolbar_frame $win.tf; 
  Toolbar::ToolbarFrame $toolbar_frame \
        -relief $GUI_Options(ToolbarFrameRelief) \
        -borderwidth $GUI_Options(ToolbarFrameReliefBorderWidth) \
        -decorate $GUI_Options(ToolbarDecorate)
    ## Open Document/Collection
    set toolbar $toolbar_frame.express
    Toolbar::create $toolbar -relief $GUI_Options(ToolbarRelief) \
         -borderwidth $GUI_Options(ToolbarReliefBorderWidth) -float 0
    if {$HaveSystemCol || !$HaveSystemDoc} {
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Collection) -tooltipvar GUI_Text(TSelectCollection) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {!$HaveSystemCol} {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -entercmd \
           {%W configure -values [GUI_GetCollectionNameTails]} -modifycmd \
           "GUI_CreateSystemWindow_UpdateDocExpressToolbar \
            $toolbar.widgets.document $toolbar.widgets.collection
            $toolbar.widgets.document configure -values \
            \[lrange \[$toolbar.widgets.document cget -values\] 1 end\]
            $toolbar.widgets.document setvalue first" \
           -tooltipvar GUI_Text(TSelectCollection) -width \
           $GUI_Options(SystemExprToolbarCollectionWidth) -takefocus 0 -pady 3 \
           -statusvar ::GUI_StatusBar::StatusBar($win)
      } else {
        Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
           $GUI_Options(InputBackground) -highlightthickness 0 \
           -foreground $GUI_Options(InputForeground) \
           -name $toolbar.widgets.collection -takefocus 0 -pady 3 \
           -values [list [file tail [tip_GetName $col]]] \
           -width $GUI_Options(SystemExprToolbarCollectionWidth)
           $toolbar.widgets.collection setvalue first
           $toolbar.widgets.collection configure -state disabled
      }
      Toolbar::addwidget $toolbar label -bd 0 -textvariable \
         GUI_Text(Document) -tooltipvar GUI_Text(TSelectDocument) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addwidget $toolbar ComboBox -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -name $toolbar.widgets.document \
         -tooltipvar GUI_Text(TSelectDocument) -width \
         $GUI_Options(SystemExprToolbarDocumentWidth) -takefocus 0 -pady 3 \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      if {$HaveSystemCol} {
        $toolbar.widgets.document configure -values [$col GetFileIds]
        $toolbar.widgets.document setvalue first
      }
      Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TOpenCol_Doc) \
         -textvariable GUI_Text(Load) -image ITLoad -command \
         "::EditCoreferenceOnHTMLTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document" \
         -tooltipvar GUI_Text(TSelectedLoad) \
         -statusvar ::GUI_StatusBar::StatusBar($win)
      Toolbar::addbutton $toolbar -image BackNice -command \
         "::EditCoreferenceOnHTMLTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Prev"
      Toolbar::addbutton $toolbar -image NextNice -command \
         "::EditCoreferenceOnHTMLTool::DocLoad $win \
          $toolbar.widgets.collection $toolbar.widgets.document Next"
    }
    Toolbar::addbutton $toolbar -image ITSave -tooltipvar GUI_Text(Save) \
       -statusvar ::GUI_StatusBar::StatusBar($win) \
       -command "::EditCoreferenceOnHTMLTool::DocSave $win"
    Toolbar::addseparator $toolbar
    wm protocol $win WM_DELETE_WINDOW \
       "::EditCoreferenceOnHTMLTool::CloseWindow $win"
    Toolbar::addbutton $toolbar -tooltipvar GUI_Text(TExitSystem) \
       -image ITExit -command "::EditCoreferenceOnHTMLTool::CloseWindow $win" \
       -statusvar ::GUI_StatusBar::StatusBar($win)
    set ActiveLabel [Toolbar::addwidget $toolbar label -relief flat -bd 0 \
       -anchor e]
    pack $toolbar -side top -anchor w
  pack $toolbar_frame -side top -fill x

  ## Create the main window area...
  panedwindow $win.area -orient horizontal
    ## The text area...
    set Viewer $win.area.text
    ::ScrolledWidgets::ScrolledHtml $Viewer -widgetname text \
      -bg $GUI_Options(HtmlBg) \
      -borderwidth $GUI_Options(TextBorderWidth) -exportselection 1 \
      -fg $GUI_Options(HtmlFg) -relief $GUI_Options(TextRelief) \
      -unvisitedcolor blue -selectioncolor $GUI_Options(SelectBackground) \
      -tablerelief raised -padx 0 -pady 0 -underlinehyperlinks 1 \
      -fontcommand    "::HtmlInfoViewer::PickFont $Viewer.text" \
      -imagecommand   "ggi_view_documentHTMLRetrieveImages $doc $Viewer.text" \
      -formcommand    "::HtmlInfoViewer::FormCmd $Viewer.text" \
      -bgimagecommand "ggi_view_documentHTMLRetrieveBgImages $doc $Viewer.text"
    bind $Viewer.text.x <<MB_1>> \
      "set ::HtmlInfoViewer::InfoViewer($Viewer.text::SelectionStart) \
                                        \[$Viewer.text index @%x,%y\]
       $Viewer.text selection clear"
    #bind $Viewer.text.x <Motion> {}
    set ::HtmlInfoViewer::InfoViewer($Viewer.text:showImages)   1
    set ::HtmlInfoViewer::InfoViewer($Viewer.text:showBgImages) 1
    bind $Viewer.text.x <<MB_2>> \
      [list ::HtmlInfoViewer::ContextualMenuHandler $Viewer.text $doc]
    bind $Viewer.text <Destroy> \
      [list +ggi_view_documentHTMLRetrieveImagesClean {} $Viewer.text]
    catch {$Viewer.text configure -base [tip_GetExternalId $doc]}
    if {[string length $DocText]} {
      $Viewer.text parse $DocText
    } else {
      $Viewer.text parse {<html></html>}
    }
    ## The Combobox for selecting overlapping ranges...
    frame $win.select
      label $win.select.label -textvariable GUI_Text(SelectOverlapingTag:)
      ComboBox $win.select.combo -editable 0 -entrybg \
         $GUI_Options(InputBackground) -highlightthickness 0 \
         -foreground $GUI_Options(InputForeground) \
         -takefocus 0 -state disabled -modifycmd \
         "::EditCoreferenceOnHTMLTool::SelectOverlapped $win $win.select.combo"
      grid $win.select.label $win.select.combo -sticky snew
      grid columnconfigure $win.select 1 -weight 1
    ## The buttons area...
    set ScrolledWindow $win.area.buttons
    ScrolledWindow $ScrolledWindow
    ScrollableFrame $ScrolledWindow.w
    $ScrolledWindow setwidget $ScrolledWindow.w
    set BFrame [$ScrolledWindow.w getframe]
    bind $BFrame <MouseWheel> \
       "$ScrolledWindow.w yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $BFrame <4> "$ScrolledWindow.w yview scroll -5 units"
    bind $BFrame <5> "$ScrolledWindow.w yview scroll  5 units"

  ## Prepare the frame for the various group buttons...
  set max_column 10
  frame $BFrame.groups
    label $BFrame.groups.label -text "  \u25ba  $GUI_Text(Groups)" \
     -font {arial 10 bold} -foreground white -background #78B1F3 -anchor w
    grid $BFrame.groups.label -columnspan $max_column \
      -sticky snew -padx 1 -pady 1
    bind $BFrame.groups.label <MouseWheel> \
       "$ScrolledWindow.w yview scroll \[expr {- (%D / 120) * 4}\] units"
    bind $BFrame.groups.label <4> "$ScrolledWindow.w yview scroll -5 units"
    bind $BFrame.groups.label <5> "$ScrolledWindow.w yview scroll  5 units"
    bind $BFrame.groups.label <<MB_1>> "focus $BFrame"

  ## Store some needed Parameters...
  set Parameters($win:TextObj)           $Viewer.text
  set Parameters($win:ActiveLabel)       $ActiveLabel
  set Parameters($win:Attributes)        $Attributes
  set Parameters($win:SelectRangeWidget) $win.select.combo
  set Parameters($win:ScrolledWindow)    $ScrolledWindow
  set Parameters($win:EditFrame)         $BFrame.edit
  set Parameters($win:ButtonFrame)       $BFrame.groups

  ## Add Tags to Text Viewer...
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set groups {}; set ci 0; set row 1; set col 0;
  set columnBreak 8
  if {[llength $Annotations]} {
    #$Viewer.text tag add back begin end
    $Viewer.text tag bind back <<MB_1>> \
      "::EditCoreferenceOnHTMLTool::HilightToolButton $win"
    $Viewer.text tag lower back
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      if {$ci == $color_nu} {set ci 0}
      foreach {bg fg} [lindex $Colors $ci] {break}
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      lappend groups $ID
      foreach span [tip_GetSpans $ann] {
        tip_GetSpanOffsets $span start end
        $Viewer.text tag add id_$ID \
             [$Viewer.text characterOffsetToIndex $start] \
             [$Viewer.text characterOffsetToIndex $end]
        $Viewer.text tag bind id_$ID <<MB_1>> \
          [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
        $Viewer.text tag bind id_$ID <Enter> \
          [list $Parameters($win:ActiveLabel) configure -text $ID]
        $Viewer.text tag bind id_$ID <Leave> \
          [list $Parameters($win:ActiveLabel) configure -text {}]
      }
      $Viewer.text tag configure id_$ID -background $bg -foreground $fg
      ## Add the group button...
      set button $BFrame.groups.g${row}$col
      set Parameters($win:Colour:$ID) [lindex $Colors $ci]
      set Parameters($win:Button:$ID) $button
      image create photo $button.image -width 12 -height 18
      $button.image put $bg -to 0 0 8 18
      button $button -text $ID -foreground black -highlightthickness 0\
        -activeforeground $fg -activebackground $bg -anchor w \
        -image $button.image -compound left -padx 2 -pady 0 -command \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
      ## Ensure that image gets destroyed with the widget...
      bind $button <Destroy> {image delete %W.image}
      ## Enabling scrolling for mouse over a widget...
      bind $button <MouseWheel> \
        "$ScrolledWindow.w yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $button <4> "$ScrolledWindow.w yview scroll -5 units"
      bind $button <5> "$ScrolledWindow.w yview scroll  5 units"
      ## Add a binding for setting an accelarator...
      bind $button <<MB_2>> "::EditCoreferenceOnHTMLTool::EditAccelerator %W"
      bind $button <Enter> "%W configure -background $bg -foreground $fg"
      bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
      grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
      incr col; incr ci
      if {$col == $columnBreak} {
        incr row; set col 0
      }
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}
  for {set i 0} {$i < $columnBreak} {incr i} {
    grid columnconfigure $BFrame.groups $i -weight 1
  }

  ## Place the Editing facilities...
  GUI_LoadImage IBack
  frame $BFrame.edit
    label $BFrame.edit.group_name -foreground white -background #78B1F3 -text \
      "  \u25ba  $GUI_Text(Group): $GUI_Text(New)" -font {arial 10 bold} \
      -anchor w
    grid $BFrame.edit.group_name -columnspan 3 -sticky snew -padx 0 -pady 2
    ## Place some buttons in a frame...
    frame $BFrame.edit.wmbuttons -background #78B1F3
      button $BFrame.edit.wmbuttons.group_new -image ITNew -background #78B1F3 \
        -activebackground #78B1F3 -relief flat -overrelief raised \
        -highlightthickness 0 -anchor center -state disabled -command \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win]
      button $BFrame.edit.wmbuttons.show -image ITShowTags -background #78B1F3 \
        -activebackground #78B1F3 -relief flat -overrelief raised \
        -highlightthickness 0 -anchor center -state disabled -command \
        [list ::EditCoreferenceOnHTMLTool::ShowHideTags $win show]
      button $BFrame.edit.wmbuttons.hide -image ITHideTags -background #78B1F3 \
        -activebackground #78B1F3 -relief flat -overrelief raised \
        -highlightthickness 0 -anchor center -state disabled -command \
        [list ::EditCoreferenceOnHTMLTool::ShowHideTags $win hide]
      pack $BFrame.edit.wmbuttons.hide $BFrame.edit.wmbuttons.show \
          $BFrame.edit.wmbuttons.group_new -side right -padx 2 -pady 2
    grid $BFrame.edit.wmbuttons -column 3 -row 0 -columnspan 2 \
        -sticky snew -padx 0 -pady 2
    label $BFrame.edit.label1 -textvariable GUI_Text(Attribute) \
      -borderwidth 1 -relief flat
    label $BFrame.edit.label2 -textvariable GUI_Text(Text) \
      -borderwidth 1 -relief flat -width 16
    label $BFrame.edit.label3 -textvariable GUI_Text(Start) \
      -borderwidth 0 -relief flat -width 7
    label $BFrame.edit.label4 -textvariable GUI_Text(End) \
      -borderwidth 0 -relief flat -width 7
    grid x $BFrame.edit.label2 $BFrame.edit.label3 $BFrame.edit.label4 \
         -sticky ew -padx 1 -pady 1
    set i 1
    foreach attr $Attributes {
      label $BFrame.edit.lattr$i -text $attr -anchor e \
        -borderwidth 1 -relief flat
      entry $BFrame.edit.attr$i -borderwidth 1 -relief sunken -state readonly \
        -readonlybackground $GUI_Options(InputBackground)
      label $BFrame.edit.start$i -text {} \
        -foreground $GUI_Options(InputForeground) \
        -background $GUI_Options(InputBackground) \
        -borderwidth 1 -relief sunken
      label $BFrame.edit.end$i -text {} \
        -foreground $GUI_Options(InputForeground) \
        -background $GUI_Options(InputBackground) \
        -borderwidth 1 -relief sunken
      button $BFrame.edit.cmd$i -image IBack -command \
        "::EditCoreferenceOnHTMLTool::Selection2Entry $Viewer.text \
         $BFrame.edit.attr$i $BFrame.edit.start$i $BFrame.edit.end$i"
      grid $BFrame.edit.lattr$i $BFrame.edit.attr$i \
           $BFrame.edit.start$i $BFrame.edit.end$i $BFrame.edit.cmd$i \
        -sticky ew -padx 1 -pady 1
      incr i
    }
    button $BFrame.edit.group -textvariable GUI_Text(CreateGroup) \
      -background #6b8e23 -foreground white -font {arial 10 bold} -command \
      [list ::EditCoreferenceOnHTMLTool::CreateGroup $win]
    button $BFrame.edit.delgroup -textvariable GUI_Text(DeleteGroup) \
      -background red -foreground white -font {arial 10 bold} -command \
      [list ::EditCoreferenceOnHTMLTool::Delete $win]
    button $BFrame.edit.updategroup -textvariable GUI_Text(UpdateGroup) \
      -background orange -foreground white -font {arial 10 bold} -command \
      [list ::EditCoreferenceOnHTMLTool::UpdateGroup $win]
    grid $BFrame.edit.group -columnspan 5 -sticky snew -padx 1 -pady 1
  grid columnconfigure $BFrame.edit 1 -weight 1
  grid $BFrame.edit -sticky snew -padx 2 -pady 2
  grid $BFrame.groups -sticky snew -padx 2 -pady 2

  update
  $win.area add $Viewer $win.area.buttons -sticky snew
  pack $win.area -fill both -expand 1 -ipadx 10 -ipady 10
  pack $win.select -side bottom -fill x -expand 0 -padx 40 -pady 5

  gut_ModalDlg $win
  wm deiconify $win

  focus $Viewer.text
  return [list $win $Viewer.text]
}; # ::EditCoreferenceOnHTMLTool::CreateWindow

## ::EditCoreferenceOnHTMLTool::UpdateWindow
proc ::EditCoreferenceOnHTMLTool::UpdateWindow {win doc col 
                  {Annotation {}} {Attributes {}}} {
  global GUI_Text GUI_Options
  variable Parameters
  variable ValuesSpec

  ## Get Document Annotations and text
  if {[catch {tip_GetByteSequence $doc} DocText]} {
    error "creole_EditCoreferenceOnHTML: Invalid Document!\n($DocText)"
  }
  set Annotations [tip_SelectAnnotations $doc $Annotation]

  ## The text area...
  set Viewer $win.area.text
  ggi_view_documentHTMLRetrieveImagesClean {} $Viewer.text
  $Viewer.text configure \
    -imagecommand   "ggi_view_documentHTMLRetrieveImages $doc $Viewer.text" \
    -bgimagecommand "ggi_view_documentHTMLRetrieveBgImages $doc $Viewer.text"
  $Viewer.text clear
  unset -nocomplain Parameters($win:Selected)
  catch {$Viewer.text configure -base [tip_GetExternalId $doc]}
  $Viewer.text parse $DocText
  $Parameters($win:SelectRangeWidget) configure -values {} -text {} \
                                                -state disabled
  unset -nocomplain Parameters($win:SelectRangeWidget_LastRange)
  ## The buttons area...
  set ScrolledWindow $win.area.buttons
  set BFrame [$ScrolledWindow.w getframe]
  
  ## Hide the html widget, as configuring the tags results in much drawing...
  grid forget $Viewer.text

  ## Remove all previous group buttons...
  set columnBreak 8
  for {set row 1} {1} {incr row} {
    if {![winfo exists $BFrame.groups.g${row}0]} {break}
    for {set col 0} {$col < $columnBreak} {incr col} {
      if {[winfo exists $BFrame.groups.g${row}$col]} {
        destroy $BFrame.groups.g${row}$col
      } else {
        break
      }
    }
  }
  array unset Parameters $win:Colour:*
  array unset Parameters $win:Button:*

  ## Add Tags to Text Viewer...
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set groups {}; set ci 0; set row 1; set col 0; 
  if {[llength $Annotations]} {
    # $Viewer.text tag add back begin end
    $Viewer.text tag bind back <<MB_1>> \
      "::EditCoreferenceOnHTMLTool::HilightToolButton $win"
    $Viewer.text tag lower back
    set wwin [ggi_wait_message $GUI_Text(LoadingAnnotations)]
    set Progress 0.0
    set ProgressStep [expr {100.0/([llength $Annotations]+1)/2.0}]
    foreach ann $Annotations {
      if {$ci == $color_nu} {set ci 0}
      foreach {bg fg} [lindex $Colors $ci] {break}
      set Progress [expr {$Progress + $ProgressStep}]
      set ID       [tip_GetId $ann]
      lappend groups $ID
      foreach span [tip_GetSpans $ann] {
        tip_GetSpanOffsets $span start end
        $Viewer.text tag add id_$ID \
             [$Viewer.text characterOffsetToIndex $start] \
             [$Viewer.text characterOffsetToIndex $end]
      }
      $Viewer.text tag bind id_$ID <<MB_1>> \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
      $Viewer.text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $ID]
      $Viewer.text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
      $Viewer.text tag configure id_$ID -background $bg -foreground $fg
      ## Add the group button...
      set button $BFrame.groups.g${row}$col
      set Parameters($win:Colour:$ID) [lindex $Colors $ci]
      set Parameters($win:Button:$ID) $button
      image create photo $button.image -width 12 -height 18
      $button.image put $bg -to 0 0 8 18
      button $button -text $ID -foreground black -highlightthickness 0\
        -activeforeground $fg -activebackground $bg -anchor w \
        -image $button.image -compound left -padx 2 -pady 0 -command \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
      ## Ensure that image gets destroyed with the widget...
      bind $button <Destroy> {image delete %W.image}
      ## Enabling scrolling for mouse over a widget...
      bind $button <MouseWheel> \
        "$ScrolledWindow.w yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $button <4> "$ScrolledWindow.w yview scroll -5 units"
      bind $button <5> "$ScrolledWindow.w yview scroll  5 units"
      ## Add a binding for setting an accelarator...
      bind $button <<MB_2>> "::EditCoreferenceOnHTMLTool::EditAccelerator %W"
      bind $button <Enter> "%W configure -background $bg -foreground $fg"
      bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
      grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
      incr col; incr ci
      if {$col == $columnBreak} {
        incr row; set col 0
      }
      ggi_wait_update {} $Progress $wwin
    }
    ggi_destroy_wait $wwin
  };# if {[llength $Annotations]}
  grid $Viewer.text -column 0 -row 0 -sticky snew
  focus $Viewer.text
  ResetGroupEditingPane $win
}; # ::EditCoreferenceOnHTMLTool::UpdateWindow

proc ::EditCoreferenceOnHTMLTool::CloseWindow {win} {
  variable Parameters
  global GUI_Text
  if {[llength [info commands $Parameters($win:Document)]]} {
    if {[$Parameters($win:Document) Modified]} {
      set ans [gut_tk_dialog {Modified Document:} [format \
        $GUI_Text(SaveModifiedDocument?) [tip_GetId \
        $Parameters($win:Document)]] yesnocancel]
      switch $ans {
        cancel  {return}
        no      {}
        yes     {tip_Sync $Parameters($win:Document)}
        default {error "Unknown answer \"$ans\""}
      }
    }
  }
  destroy $win
};# ::EditCoreferenceOnHTMLTool::CloseWindow

## ::EditCoreferenceOnHTMLTool::Selection2Entry
#  Fill the provided labels with information from the current selection...
proc ::EditCoreferenceOnHTMLTool::Selection2Entry {text text_label
                                                        start_label end_label} {
  variable Parameters
  ## If there is selected text...
  set first [$text index sel.first]
  if {[string length $first]} {
    set win [winfo toplevel $text]
    set last  [$text index sel.last]
    set start [$text indexToCharacterOffset $first]
    set end   [$text indexToCharacterOffset $last]
    $start_label configure -text $start
    $end_label   configure -text $end
    $text_label configure -state normal
    $text_label delete 0 end
    $text_label insert end  [string map {\n \\n} [string range \
      [tip_GetRawData $Parameters($win:Document)] $start [incr end -1]]]
    $text_label configure -state readonly
  } else {
    $text_label configure -state normal
    $text_label delete 0 end
    $text_label configure -state readonly
    $start_label configure -text {}
    $end_label   configure -text {}
  }
};# ::EditCoreferenceOnHTMLTool::Selection2Entry

## ::EditCoreferenceOnHTMLTool::tag2value
#  Retrieve the value of a specific tag
proc ::EditCoreferenceOnHTMLTool::tag2value {win tag} {
  variable Parameters
  return [string range $tag 3 end]
};# ::EditCoreferenceOnHTMLTool::tag2value

## ::EditCoreferenceOnHTMLTool::tag2button
#  Retrieve the button widget for a specific tag
proc ::EditCoreferenceOnHTMLTool::tag2button {win tag} {
  variable Parameters
  set Parameters($win:Button:[tag2value $win $tag])
};# ::EditCoreferenceOnHTMLTool::tag2button

## ::EditCoreferenceOnHTMLTool::tag2colours
#  Retrieve the colours (bg fg) for a specific tag
proc ::EditCoreferenceOnHTMLTool::tag2colours {win tag} {
  variable Parameters
  set Parameters($win:Colour:[tag2value $win $tag])
};# ::EditCoreferenceOnHTMLTool::tag2colours

## ::EditCoreferenceOnHTMLTool::HilightToolButton
#   Hilight the appropriate tool button...
proc ::EditCoreferenceOnHTMLTool::HilightToolButton {win {tag {}}} {
  variable Parameters
  set text $Parameters($win:TextObj)
  set ::GUI_StatusBar::StatusBar($win) $::GUI_StatusBar::StatusBar($win:Def)
  set ScrollableFrame $win.area.buttons.w
  if {[info exists Parameters($win:Selected)]} {
    set sel_tag $Parameters($win:Selected)
    $text tag configure $sel_tag -relief flat -borderwidth 0
    [tag2button $win $sel_tag] configure -background [$win cget -background] \
      -foreground black -relief raised
    $ScrollableFrame see [tag2button $win $sel_tag]
  }
  unset -nocomplain Parameters($win:Selected)
  if {![string length $tag]} {
    ## The user has clicked on the text background...
    unset -nocomplain Parameters($win:SelectRangeWidget_LastRange)
    $Parameters($win:SelectRangeWidget) configure -values {} \
                                        -text {} -state disabled
    ResetGroupEditingPane $win
    return
  }
  $text tag configure $tag -relief ridge -borderwidth 4
  $text tag raise $tag
  set Parameters($win:Selected) $tag
  foreach {bg fg} [tag2colours $win $tag] {break}
  [tag2button $win $tag] configure -background $bg -foreground $fg \
    -relief sunken
  $ScrollableFrame see [tag2button $win $tag]
  SetGroupForEditing $win $tag
  if {![llength [$text tag ranges $tag]]} {
    ## The tags have been hidden. Return...
    return
  }
  ## Check to see whether they are overlapped tags by the selected one...
  set start [$text index $tag.first]
  set end   [$text index $tag.last]
  set ::GUI_StatusBar::StatusBar($win) \
      "$::GUI_Text(Value): [tag2value $win $tag],\
       $::GUI_Text(Span): ($start,$end)"
  ## Ensure that this range is not contained within a range from a previous
  ## binding. (Tk will call all bindings for all text tags when an index is
  ## clicked)
  if {[info exists Parameters($win:SelectRangeWidget_LastRange)]} {
    foreach {ls le} $Parameters($win:SelectRangeWidget_LastRange) {break}
    if {$ls <= $start && $le >= $end} {return}
    ## If we got here, our range is outside the previous one, so reset the
    ## select widget...
    $Parameters($win:SelectRangeWidget) configure -values {} -text {} \
                                                  -state disabled
  }
  for {set index $start} {$index < $end} {} {
    foreach ptag [$text tag names $index] {
      switch $ptag [list \
        $tag {}  \
        back {}  \
        default {
          lappend Tags($ptag) $index
        } \
      ]
    }
    ## Advance index by 1 character...
    set index [$text characterOffsetToIndex \
                     [expr {[$text indexToCharacterOffset $index]+1}]]
  }
  ## Iterate over all overlapped tags...
  if {[info exists Tags]} {
    set Parameters($win:SelectRangeWidget_LastRange) [list $start $end]
    lappend Values \
      "$::GUI_Text(Text):\
       \"[::EditCoreferenceOnHTMLTool::GetTaggedText $win $tag]\",\
       $::GUI_Text(Value): [tag2value $win $tag],\
       $::GUI_Text(Span): ($start,$end), ID: [string range $tag 3 end]"
    foreach ptag [lsort [array names Tags]] {
      set range [$text tag nextrange $ptag $start $end]
      while {[llength $range]} {
        foreach {p_start p_end} $range {break}
        lappend Values \
          "$::GUI_Text(Text):\
           \"[::EditCoreferenceOnHTMLTool::GetTaggedText $win $tag]\",\
           $::GUI_Text(Value): [tag2value $win $ptag],\
           $::GUI_Text(Span): ($p_start,$p_end), ID: [string range $ptag 3 end]"
        set range [$text tag nextrange $ptag $p_end $end]
      }
    }
    $Parameters($win:SelectRangeWidget) configure -values $Values \
                                                  -state normal
    $Parameters($win:SelectRangeWidget) setvalue first
  }
}; # ::EditCoreferenceOnHTMLTool::HilightToolButton

## ::EditCoreferenceOnHTMLTool::SelectOverlapped
#  Select a tag from the overlaped tags selection combo box
proc ::EditCoreferenceOnHTMLTool::SelectOverlapped {win combo} {
  set value [lindex [$combo cget -values] [$combo getvalue]]
  set id [lindex [split $value] end]
  variable Parameters
  $Parameters($win:TextObj) tag raise id_$id
  HilightToolButton $win id_$id
};# ::EditCoreferenceOnHTMLTool::SelectOverlapped

## ::EditCoreferenceOnHTMLTool::EditAccelerator
#  Add/Modify an accelerator for a button widget...
proc ::EditCoreferenceOnHTMLTool::EditAccelerator {button} {
  ## Create a window for setting the binding...
  set win .tooleditAnnotationsEditAccelerator
  if {[winfo exists $win]} {
    raise $win
    return
  }
  toplevel .tooleditAnnotationsEditAccelerator
  wm withdraw $win
  wm title $win "Setting Button Accelerator..."
  message $win.label -width 250 -text \
    "Please press the keyboard keys combination you want to assign as an\
     accelerator to the button with value \"[$button cget -text]\""
  label $win.acc
  bind $win <Key> \
    [list ::EditCoreferenceOnHTMLTool::SetAccelerator $win.acc $button %K]
  button $win.ok -textvariable GUI_Text(Ok) -command "destroy $win"
  button $win.cancel -textvariable GUI_Text(Cancel) -command \
  "destroy $win; ::EditCoreferenceOnHTMLTool::SetAccelerator $win.acc $button"
  grid $win.label -sticky snew -columnspan 3 -sticky snew
  grid $win.acc   -sticky snew -columnspan 3 -sticky snew
  grid $win.ok x $win.cancel -padx 20 -pady 10  -sticky snew
  grid columnconfigure $win 1 -weight 1
  gut_ModalDlg $win
  wm deiconify $win
};# ::EditCoreferenceOnHTMLTool::EditAccelerator

proc ::EditCoreferenceOnHTMLTool::SetAccelerator {label button {key {}}} {
  catch {$label configure -text "Key: \"$key\""}
  set toplevel [winfo toplevel $button]
  ## Remove any previously set accelerators...
  foreach event [bind $button] {
    switch -glob -- $event {
      ? -
      <Key-*> {
        bind $button   $event {}
        bind $toplevel $event {}
      }
      default {}
    }
  }
  if {![string length $key]} {return}
  bind $button   <Key-$key> "$button invoke; $button flash"
  bind $toplevel <Key-$key> "$button invoke; $button flash"
};# ::EditCoreferenceOnHTMLTool::EditAccelerator

## ::EditCoreferenceOnHTMLTool::CreateGroup
#  Create a new group...
proc ::EditCoreferenceOnHTMLTool::CreateGroup {win} {
  global GUI_Options GUI_Text
  variable Parameters
  set text           $Parameters($win:TextObj)
  set doc            $Parameters($win:Document)
  set frame          $Parameters($win:EditFrame)
  set bframe         $Parameters($win:ButtonFrame)
  set ScrolledWindow $Parameters($win:ScrolledWindow)
  set Attributes     $Parameters($win:Attributes)

  ## Create the Annotation to be put in the Document...
  set spans {}; set attrs {};
  set i 1
  foreach attr_name $Attributes {
    set start     [$frame.start$i cget -text]
    if {![string length $start]} {
      incr i
      continue
    }
    set end       [$frame.end$i   cget -text]
    set span      [tip_CreateSpan $start $end]
    lappend spans $span
    set attr [tip_CreateAttribute $attr_name \
                 [tip_CreateAttributeValue GDM_STRING_SET $span]]
    lappend attrs $attr
    incr i
  }
  if {![llength $spans]} {return}
  set ann [tip_CreateAnnotation $Parameters($win:Annotation) $spans $attrs]
  set ID [tip_AddAnnotation $doc $ann]

  ## Update the group button section...
  set Colors $GUI_Options(Colors); set color_nu [llength $Colors]
  set ci 0; set row 1; set col 0;
  set columnBreak 8; set width -1
  while {1} {
    if {$ci == $color_nu} {set ci 0}
    set button $bframe.g${row}$col
    if {![winfo exists $button]} {
      foreach {bg fg} [lindex $Colors $ci] {break}
      ## Create the button widget...
      set Parameters($win:Colour:$ID) [lindex $Colors $ci]
      set Parameters($win:Button:$ID) $button
      image create photo $button.image -width 12 -height 18
      $button.image put $bg -to 0 0 8 18
      button $button -text $ID -foreground black -highlightthickness 0\
        -activeforeground $fg -activebackground $bg -anchor w -width $width \
        -image $button.image -compound left -padx 2 -pady 0 -command \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
      ## Ensure that image gets destroyed with the widget...
      bind $button <Destroy> {image delete %W.image}
      ## Enabling scrolling for mouse over a widget...
      bind $button <MouseWheel> \
        "$ScrolledWindow.w yview scroll \[expr {- (%D / 120) * 4}\] units"
      bind $button <4> "$ScrolledWindow.w yview scroll -5 units"
      bind $button <5> "$ScrolledWindow.w yview scroll  5 units"
      ## Add a binding for setting an accelarator...
      bind $button <<MB_2>> "::EditCoreferenceOnHTMLTool::EditAccelerator %W"
      bind $button <Enter> "%W configure -background $bg -foreground $fg"
      bind $button <Leave> "if {\[%W cget -relief\] != \"sunken\"} \
         {%W configure -background \[$win cget -background\] -foreground black}"
      grid $button -row $row -column $col -sticky snew -padx 1 -pady 1
      break
    }
    incr col; incr ci
    if {$col == $columnBreak} {
      incr row; set col 0
    }
  }

  ## Add the tags to the text viewer
  foreach span $spans {
    tip_GetSpanOffsets $span start end
    $text tag add id_$ID \
         [$text characterOffsetToIndex $start] \
         [$text characterOffsetToIndex $end]
  }
  $text tag bind id_$ID <<MB_1>> \
    [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
  $text tag bind id_$ID <Enter> \
    [list $Parameters($win:ActiveLabel) configure -text $ID]
  $text tag bind id_$ID <Leave> \
    [list $Parameters($win:ActiveLabel) configure -text {}]
  $text tag configure id_$ID -foreground $fg -background $bg
  $frame.group_name configure -text "  \u25ba  $GUI_Text(Group): $ID"
  $text selection clear
  ## The following code keeps the newly created group in the editing pane...
  #grid forget $frame.group
  #grid $frame.updategroup -columnspan 5 -sticky snew -padx 1 -pady 1
  #grid $frame.delgroup -columnspan 5 -sticky snew -padx 1 -pady 1
  #$frame.wmbuttons.group_new configure -state normal
  #$frame.wmbuttons.show      configure -state disabled
  #$frame.wmbuttons.hide      configure -state normal
  #HilightToolButton $win id_$ID
  ## The following code resets the editing pane into the "new" state...
  ResetGroupEditingPane $win
  HilightToolButton $win 
};# ::EditCoreferenceOnHTMLTool::CreateGroup

## ::EditCoreferenceOnHTMLTool::UpdateGroup
#  Update an existing group...
proc ::EditCoreferenceOnHTMLTool::UpdateGroup {win} {
  global GUI_Options GUI_Text
  variable Parameters
  if {![info exists Parameters($win:Selected)]} {
    ## There is nothing selected - nothing to update :-)
    return
  }
  set tag            $Parameters($win:Selected)
  set ID             [tag2value $win $tag]
  set text           $Parameters($win:TextObj)
  set doc            $Parameters($win:Document)
  set frame          $Parameters($win:EditFrame)
  set bframe         $Parameters($win:ButtonFrame)
  set ScrolledWindow $Parameters($win:ScrolledWindow)
  set Attributes     $Parameters($win:Attributes)

  ## Update the Annotation to be put in the Document...
  set spans {}; set attrs {};
  set i 1
  foreach attr $Attributes {
    set attr_name [$frame.lattr$i cget -text]
    set start     [$frame.start$i cget -text]
    if {![string length $start]} {
      incr i
      continue
    }
    set end       [$frame.end$i   cget -text]
    set span      [tip_CreateSpan $start $end]
    lappend spans $span
    set attr [tip_CreateAttribute $attr_name \
                 [tip_CreateAttributeValue GDM_STRING_SET $span]]
    lappend attrs $attr
    incr i
  }
  if {![llength $spans]} {return}
  set ann [list $ID $Parameters($win:Annotation) $spans $attrs]
  tip_AddAnnotation $doc $ann
  
  ## Add the tags to the text viewer
  $text tag delete id_$ID
  foreach span $spans {
    tip_GetSpanOffsets $span start end
    $text tag add id_$ID \
         [$text characterOffsetToIndex $start] \
         [$text characterOffsetToIndex $end]
  }
  $text tag bind id_$ID <<MB_1>> \
    [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
  $text tag bind id_$ID <Enter> \
    [list $Parameters($win:ActiveLabel) configure -text $ID]
  $text tag bind id_$ID <Leave> \
    [list $Parameters($win:ActiveLabel) configure -text {}]
  foreach {bg fg} [tag2colours $win $tag] {break}
  $text tag configure id_$ID -foreground $fg -background $bg
  HilightToolButton $win id_$ID
  $text selection clear
};# ::EditCoreferenceOnHTMLTool::UpdateGroup

## ::EditCoreferenceOnHTMLTool::Delete
#  
proc ::EditCoreferenceOnHTMLTool::Delete {win} {
  variable Parameters
  global GUI_Text
  set text           $Parameters($win:TextObj)
  set doc            $Parameters($win:Document)
  set frame          $Parameters($win:EditFrame)
  set bframe         $Parameters($win:ButtonFrame)
  set ScrolledWindow $Parameters($win:ScrolledWindow)
  set Attributes     $Parameters($win:Attributes)
  if {[info exists Parameters($win:Selected)]} {
    ## Delete the Annotation and the tags from the viewer...
    set ptag $Parameters($win:Selected)
    unset Parameters($win:Selected)
    set ID [string range $ptag 3 end]
    [::EditCoreferenceOnHTMLTool::tag2button $win $ptag] \
      configure -background [$win cget -background] -relief raised
    $text tag delete $ptag
    ## puts "Deleting whole Annotation with ID $ID"
    tip_RemoveAnnotation $doc $ID
    ## Delete the group button...
    set button $Parameters($win:Button:[tag2value $win $ptag])
    $button configure -state disabled -relief flat
    bind $button <Enter> {}; bind $button <Leave> {}
    unset Parameters($win:Button:[tag2value $win $ptag])
    unset Parameters($win:Colour:[tag2value $win $ptag])
    ## And finally, re-arrange the buttons...
    ## Ensure that the edit tools get back to the "new" mode...
    ResetGroupEditingPane $win
  }
};# ::EditCoreferenceOnHTMLTool::Delete

## ::EditCoreferenceOnHTMLTool::ResetGroupEditingPane
#  Reset the group editing facilities to its default state...
proc ::EditCoreferenceOnHTMLTool::ResetGroupEditingPane {win} {
  variable Parameters
  global GUI_Text
  set text           $Parameters($win:TextObj)
  set doc            $Parameters($win:Document)
  set frame          $Parameters($win:EditFrame)
  set Attributes     $Parameters($win:Attributes)

  ## Deselect any selected button...
  if {[info exists Parameters($win:Selected)]} {
    HilightToolButton $win
  }

  ## Reset the label of the group...
  $frame.group_name configure -text \
       "  \u25ba  $GUI_Text(Group): $GUI_Text(New)"
  ## Place only the create group button...
  grid forget $frame.delgroup $frame.updategroup
  grid $frame.group -columnspan 5 -sticky snew -padx 1 -pady 1
  ## Clear all contents from all entries...
  set i 1
  foreach attr $Attributes {
    $frame.attr$i  configure -state normal
    $frame.attr$i  delete 0 end
    $frame.attr$i  configure -state readonly
    $frame.start$i configure -text {}
    $frame.end$i   configure -text {}
    incr i
  }
  $frame.wmbuttons.group_new configure -state disabled
  $frame.wmbuttons.show      configure -state disabled
  $frame.wmbuttons.hide      configure -state disabled
};# ::EditCoreferenceOnHTMLTool::ResetGroupEditingPane

## ::EditCoreferenceOnHTMLTool::SetGroupForEditing
#  Reset the group editing facilities to its default state...
proc ::EditCoreferenceOnHTMLTool::SetGroupForEditing {win tag} {
  variable Parameters
  global GUI_Text
  set text           $Parameters($win:TextObj)
  set doc            $Parameters($win:Document)
  set frame          $Parameters($win:EditFrame)
  set Attributes     $Parameters($win:Attributes)

  set ID [tag2value $win $tag]

  ## Change the label of the group...
  $frame.group_name configure -text "  \u25ba  $GUI_Text(Group): $ID"
  
  ## Place the delete/update group button...
  grid forget $frame.group
  grid $frame.updategroup -columnspan 5 -sticky snew -padx 1 -pady 1
  grid $frame.delgroup    -columnspan 5 -sticky snew -padx 1 -pady 1

  ## Get all Attributes from the Annotation...
  set ann [tip_GetAnnotation $doc $ID]
  foreach attr [tip_GetAttributes $ann] {
    set name [tip_GetName $attr]
    set val  [tip_GetValueValue $attr]
    set Attribute($name) $val
  }
  ## Clear all contents from all entries...
  set i 1
  foreach name $Attributes {
    $frame.attr$i  configure -state normal
    $frame.attr$i  delete 0 end
    if {[info exists Attribute($name)]} {
      foreach {start end} $Attribute($name) {break}
      $frame.start$i configure -text $start
      $frame.end$i   configure -text $end
      $frame.attr$i insert end [string map {\n \\n} [string range \
                                  [tip_GetRawData $doc] $start [incr end -1]]]
    } else {
      $frame.start$i configure -text {}
      $frame.end$i   configure -text {}
    }
    $frame.attr$i  configure -state readonly
    incr i
  }
  $frame.wmbuttons.group_new configure -state normal
  if {[llength [$text tag ranges $tag]]} {
    ## The tags are visible...
    $frame.wmbuttons.show      configure -state disabled
    $frame.wmbuttons.hide      configure -state normal
  } else {
    $frame.wmbuttons.show      configure -state normal
    $frame.wmbuttons.hide      configure -state disabled
  }
};# ::EditCoreferenceOnHTMLTool::SetGroupForEditing

## ::EditCoreferenceOnHTMLTool::ShowHideTags
#
proc ::EditCoreferenceOnHTMLTool::ShowHideTags {win mode} {
  variable Parameters
  set text  $Parameters($win:TextObj)
  set frame $Parameters($win:EditFrame)
  set tag   $Parameters($win:Selected)
  switch $mode {
    show {
      set doc        $Parameters($win:Document)
      set Attributes $Parameters($win:Attributes)
      ## Get all Attributes from the Annotation...
      set ID [tag2value $win $tag]
      set ann [tip_GetAnnotation $doc $ID]
      ## Add the tags to the text viewer
      foreach span [tip_GetSpans $ann] {
        tip_GetSpanOffsets $span start end
        $text tag add id_$ID \
             [$text characterOffsetToIndex $start] \
             [$text characterOffsetToIndex $end]
      }
      $text tag bind id_$ID <<MB_1>> \
        [list ::EditCoreferenceOnHTMLTool::HilightToolButton $win id_$ID]
      $text tag bind id_$ID <Enter> \
        [list $Parameters($win:ActiveLabel) configure -text $ID]
      $text tag bind id_$ID <Leave> \
        [list $Parameters($win:ActiveLabel) configure -text {}]
      foreach {bg fg} [tag2colours $win $tag] {break}
      $text tag configure id_$ID -foreground $fg -background $bg
      $frame.wmbuttons.show      configure -state disabled
      $frame.wmbuttons.hide      configure -state normal
    }
    hide {
      $text tag delete $tag
      $frame.wmbuttons.show      configure -state normal
      $frame.wmbuttons.hide      configure -state disabled
    }
  }
};# ::EditCoreferenceOnHTMLTool::ShowHideTags

## ::EditCoreferenceOnHTMLTool::GetTaggedText
#  Given a tag, retrieve the text tagged with the tag
proc ::EditCoreferenceOnHTMLTool::GetTaggedText {win tag} {
  variable Parameters
  set doc $Parameters($win:Document)
  set id  [string range $tag 3 end]
  set ann [tip_GetAnnotation $doc $id]
  return [string map {\n \\n} [tip_GetFirstAnnotatedTextRange $doc $ann]]
};# ::EditCoreferenceOnHTMLTool::GetTaggedText
