##########################################################################
# 
#        ExportAttribute.tcl - Monday October 16 19:57:16 (EEST) 2000
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################


# Location of this module (ExportAttribute)
global creole_ExportAttribute_home

## creole_ExportAttribute
#  
proc creole_ExportAttribute {doc col args} {
  global creole_ExportAttribute_GetParameters GUI_Text

  ## Has the user opened anything?
  if {![string length $doc] && ![string length $col]} {
    gut_info $::GUI_Text(PleaseOpenColl/Doc)
    return
  }

  set win [creole_ExportAttribute_GetParameters]
  tkwait window $win
  if {$creole_ExportAttribute_GetParameters($win:Cancel)} {
    #array unset creole_ExportAttribute_GetParameters $win:*
    return
  } else {
    set ann     $creole_ExportAttribute_GetParameters($win:KeyAnnType)
    set cons    $creole_ExportAttribute_GetParameters($win:SelectionConstraints)
    set attr    $creole_ExportAttribute_GetParameters($win:Attrs)
    set pattern $creole_ExportAttribute_GetParameters($win:Pattern)
    set failval $creole_ExportAttribute_GetParameters($win:FailValue)
    set doc_id  $creole_ExportAttribute_GetParameters($win:InsertDocId)
  }
  #array unset creole_ExportAttribute_GetParameters $win:*

  ## Has the user opened a single document?
  if {[string length $doc]} {
    set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]
    set Data [creole_ExportAttribute_ProcessDocument $doc $ann $attr $pattern \
                                                     $failval $cons]
  } else {
    ## Iterate over Collection Documents...
    set files [$col GetFileIds]; set file_nu [llength $files]
    set progress 0.0
    set step [expr {100.0/$file_nu}]
    set wwin [ggi_wait_message $GUI_Text(PleaseWait) 0]
    set Data {}
    foreach file $files {
      set doc [tip_GetDocument $col $file]
      if {$doc_id} {
        append Data "Processing File: \"[tip_GetId $doc]\"\n"
      }
      append Data \
             [creole_ExportAttribute_ProcessDocument $doc $ann $attr $pattern \
                                                     $failval $cons]
      tip_Close $doc
      set progress [expr {$progress+$step}]
      catch {ggi_wait_update {} $progress $wwin}
    }
  }
  ggi_destroy_wait $wwin
  ggi_view_text_file $Data {} {} "Exported Attribute \"$attr\"..."
} ;# creole_ExportAttribute


proc creole_ExportAttribute_ProcessDocument {doc ann attr pattern failval
                                             {cons {}}} {
  set text [tip_GetRawData $doc]
  set data {}
  if {[string equal $ann token]} {
    ## Here we have two possibilities: If "sentence" annotations exist and
    ## contain a "constituents" attribute, then we want to use them...
    set done 0
    if {![string length $cons]} {
      foreach sent [tip_SelectAnnotations $doc sentence] {
        set attribute [tip_GetAttribute $sent constituents]
        if {![llength $attribute]} {
          set data {}
          set done 0
          break
        }
        set IDs [tip_GetValueValue $attribute]
        if {[llength $IDs]} {
          foreach id $IDs {
            set one        [tip_GetAnnotation $doc $id]
            set attribute  [tip_GetAttribute $one $attr]
            set string     [tip_GetFirstAnnotatedTextRange $text $one]
            tip_GetSpanOffsets [tip_GetFirstSpan $one] start end
            if {![llength $attribute]} {
              eval set value $failval
            } else {
              set value [tip_GetValueValue $attribute]
            }
            eval append data "$pattern"
          }
          append data "\n"
          set done 1
        };# if {[llength $IDs]}
      }
      if {$done} {return $data}
    };# if {![string length $cons]}
    ## If we haven't emmited anything, then something was wrong with the
    ## "sentence"->"constituents". Try to search the tokens directly...
    foreach one [tip_SelectAnnotationsSorted $doc $ann $cons] {
      set attribute  [tip_GetAttribute $one $attr]
      set string     [tip_GetFirstAnnotatedTextRange $text $one]
      tip_GetSpanOffsets [tip_GetFirstSpan $one] start end
      if {![llength $attribute]} {
        eval set value $failval
      } else {
        set value  [tip_GetValueValue $attribute]
      }
      eval append data "$pattern"
    }
  } else {
    foreach one  [tip_SelectAnnotationsSorted $doc $ann $cons] {
      set attribute  [tip_GetAttribute $one $attr]
      set string     [tip_GetFirstAnnotatedTextRange $text $one]
      tip_GetSpanOffsets [tip_GetFirstSpan $one] start end
      if {![llength $attribute]} {
        eval set value $failval
      } else {
        set value  [tip_GetValueValue $attribute]
      }
      eval append data "$pattern"
    }
  }
  return $data
};# creole_ExportAttribute_ProcessDocument

proc creole_ExportAttribute_GetParameters {} {
  global creole_ExportAttribute_GetParameters GUI_Text GUI_Options \
         HomeDir UserHomeDir
 
  set win [ggi_unused_win .toolExportAttributeDlg]

  ::Annotation::Init

  toplevel $win
  wm withdraw $win
  wm title $win "Export Annotation Attribute Tool Options..."
  
  label $win.title -text {Export Annotation Attribute Tool Options:}
  pack $win.title -side top -fill x

  set frame $win.frame
  frame $frame -borderwidth 2 -relief groove
    label $frame.lann -text {Export Annotation:}
    if {![info exists creole_ExportAttribute_GetParameters($win:Pattern)]} {
      set creole_ExportAttribute_GetParameters($win:Pattern) {$string/$value\ }
      set creole_ExportAttribute_GetParameters($win:FailValue) {$string\ }
    }
    ComboBox $frame.ann -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -values [::Annotation::AnnotationTypes] \
      -textvariable creole_ExportAttribute_GetParameters($win:KeyAnnType) \
      -modifycmd \
      "::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr"
    grid $frame.lann $frame.ann -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lattr -text {Annotation Attribute:}
    ComboBox $frame.attr -entrybg $GUI_Options(InputBackground) \
      -fg $GUI_Options(InputForeground) -bd $GUI_Options(BorderWidth) \
      -highlightthickness 0 -editable 1 \
      -textvariable creole_ExportAttribute_GetParameters($win:Attrs)
    set tokenIndex [lsearch [$frame.ann cget -values] token]
    if {$tokenIndex != -1} {
      $frame.ann setvalue @$tokenIndex
    } else {
      $frame.ann setvalue first
    }
    ::GUI::FillAttributeComboFromAnnotationCombo $frame.ann $frame.attr
    set posIndex [lsearch [$frame.attr cget -values] pos]
    if {$posIndex != -1} {
      $frame.attr setvalue @$posIndex
    }
    grid $frame.lattr $frame.attr -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.lcons -text {Annotation Selection Constrains:}
    entry $frame.cons -textvariable \
          creole_ExportAttribute_GetParameters($win:SelectionConstraints)
    grid $frame.lcons $frame.cons -sticky snew -ipadx 3 -ipady 3 -padx 6 -pady 2
    label $frame.vals -text \
      "Pattern: (use \"\$string\" for text and \"\$value\" for value)\
       \nExample: <\$value>\$string</\$value> for sgml-like output..."
    entry $frame.vals_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      creole_ExportAttribute_GetParameters($win:Pattern)
    grid $frame.vals $frame.vals_e -sticky snew -ipadx 3 -ipady 3 \
                                   -padx 6 -pady 2
    label $frame.failval -text {In case of failure use value:}
    entry $frame.failval_e -foreground $GUI_Options(InputForeground) \
      -background $GUI_Options(InputBackground) -textvariable \
      creole_ExportAttribute_GetParameters($win:FailValue)
     grid $frame.failval $frame.failval_e -sticky snew -ipadx 3 -ipady 3 \
                                   -padx 6 -pady 2
    if {![info exists \
          ::creole_ExportAttribute_GetParameters($win:InsertDocId)]} {
     set creole_ExportAttribute_GetParameters($win:InsertDocId) 0
    }
    checkbutton $frame.docid -text {Insert Document Identification label} \
      -variable creole_ExportAttribute_GetParameters($win:InsertDocId)
    grid $frame.docid -sticky snew -ipadx 3 -ipady 3 \
                                   -padx 6 -pady 2 -columnspan 2
    grid columnconfigure $frame 0 -weight 0 
    grid columnconfigure $frame 1 -weight 1 
    grid columnconfigure $frame 2 -weight 0 
  pack $frame -fill both -expand 1 -ipadx 5 -ipady 5 -padx 3 -pady 3

  set bframe $win.buttons
  frame $bframe
    button $bframe.ok -textvariable GUI_Text(Ok) -command "destroy $win" \
      -default active
    bind $win <Return> "$bframe.ok invoke"
    button $bframe.cancel -textvariable GUI_Text(Cancel) -command \
     "set creole_ExportAttribute_GetParameters($win:Cancel) 1
      destroy $win"
    pack $bframe.ok $bframe.cancel -padx 3 -pady 5 -side left
  pack $bframe
  focus $frame.ann
  gut_ModalDlg $win
  set creole_ExportAttribute_GetParameters($win:Cancel) 0
  wm deiconify $win
  return $win
} ;# creole_ExportAttribute_GetParameters

#
# End of File
#

