##########################################################################
# 
#        Ext_RunComponentTool.tcl - Friday March 16 19:02:26 (EET) 2001
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################


# Location of this module (Ext_RunComponentTool)
global creole_Ext_RunComponentTool_home

## creole_Ext_RunComponentTool
#  
proc creole_Ext_RunComponentTool {doc col {Annotators {}} {force 1} args} {
  global GUI_Text GUI_Progress
  if {![string length $doc] && ![string length $col]} {
    gut_info $GUI_Text(PleaseOpenColl/Doc)
    return
  }

  ## Get the Components to run...
  if {[llength $Annotators] == 0} {
    set Annotators [GUI_MW_GetModule2Reload {Select Component to Run...} \
                                            {Select Component...}]
  }

  set autosave 0
  set ignore   0
  catch {
    set autosave [GUI_SystemWindowGetInfo $col AutoSave]
    set ignore   [GUI_SystemWindowGetInfo $col IgnoreErrors]
  }
  
  if {[string length $doc]} {
    set wwin [ggi_wait_message {} 0.0]
    ## Initialise Annotators...
    foreach annotator $Annotators {
      set result [tip_InitialiseAnnotator $doc [file tail $annotator]]
      if {![string equal [lindex $result 0] Annotate]} {
        global GUI_Text
        bgerror [format $GUI_Text(ModuleInitialize) \
          [file tail $annotator] [file tail $annotator] [lindex $result 1] \
          [lindex $result 2]]
        return
      }
    }
    
    ## Run Selected Annotators...
    foreach annotator $Annotators {
      if {!$force} {
        ## Perhaps the Annotator has already been ran?
        if {[llength [tip_GetAttribute $doc $annotator]]} {continue}
      }
      catch {ggi_wait_append "$GUI_Text(PleaseWait)\n$GUI_Text(RunningModule)\
                  [file tail $annotator] $GUI_Text(OnDocument)\n\
                  [tip_GetId $doc]\n" -1.0}
      set GUI_Progress(WindowID:$doc) $wwin
      set result [tip_RunAnnotator $doc [file tail $annotator]]
      if {![string equal [lindex $result 0] Annotate]} {
        global GUI_Text
        error "\n$GUI_Text(ErrorWhileExecutingModule:) [file tail $annotator]\
               \n$GUI_Text(OnDocument:) [tip_GetId $doc] \
               \n\n[lindex $result 1]" "[lindex $result 2]"
      }
      catch {ggi_wait_update {} 100 $wwin}
      catch {ggi_destroy_append $wwin}
    }

    ## Finalise Annotators...
    foreach annotator $Annotators {
      set result [tip_FinishAnnotator $doc [file tail $annotator]]
      if {![string equal [lindex $result 0] Annotate]} {
        global GUI_Text
        bgerror [format $GUI_Text(ModuleFinalize) \
          [file tail $annotator] [file tail $annotator] [lindex $result 1] \
          [lindex $result 2]]
        return
      }
    }
    
    if {$autosave} {tip_Sync $doc}
    ggi_destroy_wait $wwin
  } else {
    set wwin [ggi_wait_message {} 0.0]
    set doc [tip_FirstDocument $col]
    set Length [tip_Length $col]
    set Step [expr {100.0/($Length+1)}]
    set Progress 0.0
    
    ## Initialise Annotators...
    foreach annotator $Annotators {
      set result [tip_InitialiseAnnotator $doc [file tail $annotator]]
      if {![string equal [lindex $result 0] Annotate]} {
        global GUI_Text
        bgerror [format $GUI_Text(ModuleInitialize) \
          [file tail $annotator] [file tail $annotator] [lindex $result 1] \
          [lindex $result 2]]
        return
      }
    }

    ## Iterate Annotators over Collection's Documents
    while {[string length $doc]} {
      foreach annotator $Annotators {
        if {!$force} {
          ## Perhaps the Annotator has already been ran?
          if {[llength [tip_GetAttribute $doc $annotator]]} {continue}
        }
        catch {ggi_wait_append "$GUI_Text(PleaseWait)\n$GUI_Text(RunningModule)\
                  [file tail $annotator] $GUI_Text(OnDocument)\n\
                  [tip_GetId $doc]\n" -1.0}
        set GUI_Progress(WindowID:$doc) $wwin
        set result [tip_RunAnnotator $doc [file tail $annotator]]
        if {![string equal [lindex $result 0] Annotate]} {
          global GUI_Text
          error "\n$GUI_Text(ErrorWhileExecutingModule:) [file tail $annotator]\
                 \n$GUI_Text(OnDocument:) [tip_GetId $doc] \
                 \n\n[lindex $result 1]" "[lindex $result 2]"
        }
        catch {ggi_wait_update {} 100 $wwin}
        catch {ggi_destroy_append $wwin}
      }
      if {$autosave} {
        tip_Sync  $doc
	tip_Close $doc
      }
      set doc [tip_NextDocument $col]
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
    }

    ## Finalise Annotators...
    foreach annotator $Annotators {
      set result [tip_FinishAnnotator $doc [file tail $annotator] $col]
      if {![string equal [lindex $result 0] Annotate]} {
        global GUI_Text
        bgerror [format $GUI_Text(ModuleFinalize) \
          [file tail $annotator] [file tail $annotator] [lindex $result 1] \
          [lindex $result 2]]
        return
      }
    }
    
    ggi_destroy_wait $wwin
    if {$autosave} {
      tip_Sync $col
    }
  }

  ## If we got here, then the processing has been succesfully completed.
  ## Update the system graph, if one is available...
  catch {
    set info [tip_GetAssociatedInfo $col]
    if {[llength $info]} {
      global GUI_GraphInfo
      set canvas [lindex $info 1]
      foreach Annotator $Annotators {
        set annotator [file tail $Annotator]
        set nmods [lrange $GUI_GraphInfo(${canvas}${annotator}) 1 end]
        set GUI_GraphInfo(${canvas}${annotator}) [concat 3 $nmods]
        GUI_Draw_Module_Status $canvas $annotator {}
      }
      set GUI_GraphInfo(CollectionChanged_${canvas}) 1
    }
  }
};# creole_Ext_RunComponentTool

#
# End of File
#

