##########################################################################
# 
#	RecogniseContainedAnnotations.tcl - Monday September 23 09:57:17 (EEST) 2002
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.02...
#
##########################################################################


# Location of this module (RecogniseContainedAnnotations)
global creole_RecogniseContainedAnnotations_home

## creole_RecogniseContainedAnnotations
#  
proc creole_RecogniseContainedAnnotations {doc col args} {
  global creole_RecogniseContainedAnnotations GUI_Options GUI_Text
  ## Has the user opened anything?
  if {![string length $doc] && ![string length $col]} {
    gut_info $::GUI_Text(PleaseOpenColl/Doc)
    return
  }
  set win [ggi_unused_win .creole_RecogniseContainedAnnotationsToolWindow]

  set creole_RecogniseContainedAnnotations($win:Col) $col
  set creole_RecogniseContainedAnnotations($win:Doc) $doc
  ## 1) Have we got a collection or a document?
  if {[string length $doc]} {
    ## A document:
    set Image IDocument
    set Message "$GUI_Text(EditingDocument): [tip_GetId $doc]\
               \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
    set Types [$doc GetAnnotationTypes]
  } else {
    ## A collection: Iterate over all documents...
    set Image ICollection
    set Message "$GUI_Text(EditingCollection): [file tail [tip_GetName $col]]\
               \n([tip_GetName $col])"
    set wwin [ggi_wait_message {} 0.0]
    set doc [tip_FirstDocument $col]
    set Length [tip_Length $col]
    if {$Length > 50} {set Length 50}
    set Step [expr {100.0/($Length+1.0)}]
    set Progress 0.0
    set index 0
    while {[string length $doc]} {
      ## Get all annotation types that the document contains
      foreach type [$doc GetAnnotationTypes] {
        CDM_LappendUnique Types $type
      }
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
      if {[incr index] >= 50} {break}
      tip_Close $doc
      set doc [tip_NextDocument $col]
    }
    set doc {}
    ggi_destroy_wait $wwin
  }
  if {![llength $Types]} {
    gut_info $GUI_Text(EmptyCol/Doc)
    return
  }
  set Types [lsort $Types]

  ## Tool Dialog Code
  toplevel $win
  wm withdraw $win
  wm title $win {Remove Annotations/Attributes Tool...}
  
  ## Header Section...
  set TitleFrame $win.title
  frame $TitleFrame
    label $TitleFrame.image -image $Image
    label $TitleFrame.text -height 2 -anchor w -justify left -text $Message
    grid $TitleFrame.image $TitleFrame.text -sticky snew -padx 3 -pady 6
    grid columnconfigure $TitleFrame 1 -weight 1
  pack $TitleFrame -fill x -expand 1

  ## Main area...
  set Frame $win.main
  frame $Frame
    label $Frame.containerl -anchor w -justify left \
          -text {Container Annotation:}
    ComboBox $Frame.container -editable 0 -highlightthickness 0 \
          -entrybg $GUI_Options(InputBackground) \
          -fg $GUI_Options(InputForeground) \
          -values  $Types \
          -textvariable creole_RecogniseContainedAnnotations($win:Container)
    $Frame.container setvalue first
    grid $Frame.containerl $Frame.container -padx 2 -pady 6 -sticky snew
    for {set i 1} {$i <= 3} {incr i} {
      label $Frame.containedl_$i -anchor w -justify left \
            -text "Contained Annotation $i:"
      ComboBox $Frame.contained_$i -editable 0 -highlightthickness 0 \
            -entrybg $GUI_Options(InputBackground) \
            -fg $GUI_Options(InputForeground) \
            -values  $Types \
            -textvariable creole_RecogniseContainedAnnotations($win:Contained$i)
      grid $Frame.containedl_$i $Frame.contained_$i -padx 2 -pady 2 -sticky snew
    }

    label $Frame.attrl -anchor w -justify left \
          -text {Place in Attribute:}
    entry $Frame.attr \
          -textvariable creole_RecogniseContainedAnnotations($win:Attribute)
    set creole_RecogniseContainedAnnotations($win:Attribute) constituents
    grid $Frame.attrl $Frame.attr -padx 2 -pady 8 -sticky snew
    grid columnconfigure $Frame 1 -weight 1
   pack $Frame -padx 4 -pady 5 -fill x -expand 1
  
  ## Button Section
  set ButtonFrame $win.buttons
  frame $ButtonFrame
    button $ButtonFrame.ok    -textvar GUI_Text(Ok) -command \
      "creole_RecogniseContainedAnnotations_Recognise $win; destroy $win"
    button $ButtonFrame.close -textvar GUI_Text(Close) -command "destroy $win"
    bind $ButtonFrame.close <Destroy> \
      "array unset creole_RecogniseContainedAnnotations \"$win:*\""
    pack $ButtonFrame.ok $ButtonFrame.close -padx 10 -side left
  pack $ButtonFrame -padx 4 -pady 5 -expand 1
  wm deiconify $win
};# creole_RecogniseContainedAnnotations

proc creole_RecogniseContainedAnnotations_Recognise {win} {
  global creole_RecogniseContainedAnnotations
  set col $creole_RecogniseContainedAnnotations($win:Col)
  set doc $creole_RecogniseContainedAnnotations($win:Doc)
  if {[string length $doc]} {
    ## A document:
    creole_RecogniseContainedAnnotations_RecogniseDocument $win $doc
  } else {
    ## A collection: Iterate over all documents...
    set wwin [ggi_wait_message {} 0.0]
    set doc [tip_FirstDocument $col]
    set Step [expr {100.0/([tip_Length $col]+1.0)}]
    set Progress 0.0
    while {[string length $doc]} {
      creole_RecogniseContainedAnnotations_RecogniseDocument $win $doc
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
      tip_Close $doc
      set doc [tip_NextDocument $col]
    }
    ggi_destroy_wait $wwin
  }
};# creole_RecogniseContainedAnnotations_Initialize

proc creole_RecogniseContainedAnnotations_RecogniseDocument {win doc} {
  global creole_RecogniseContainedAnnotations
  set name $creole_RecogniseContainedAnnotations($win:Attribute)
  set Container $creole_RecogniseContainedAnnotations($win:Container)
  set ContainerAnns [tip_SelectAnnotations $doc $Container]
  set ContainedAnns {}
  foreach index [array names creole_RecogniseContainedAnnotations \
                                                      $win:Contained*] {
    set ContainedAnns [concat $ContainedAnns [tip_SelectAnnotations $doc \
        $creole_RecogniseContainedAnnotations($index)]]
  }
  ## The algorithm is simple: each annotation is represented as a list:
  ##   {start.end id x}. Then we request from Tcl to sort this list as a
  ## sequence of real numbers. Each time we find the id of a container
  ## annotation, we get all annotation before this as its costituents...
  set List {}
  foreach ann $ContainerAnns {
    foreach span [tip_GetSpans $ann] {
      tip_GetSpanOffsets $span start end
      lappend List [list $end.$start [tip_GetId $ann] c]
    }
  }
  unset ContainerAnns
  foreach ann $ContainedAnns {
    foreach span [tip_GetSpans $ann] {
      tip_GetSpanOffsets $span start end
      lappend List [list $start.$end [tip_GetId $ann] d]
    }
  }
  unset ContainedAnns
  ## Sort the list...
  set List [lsort -real -index 0 -increasing $List]
  set cons {}
  foreach element $List {
    puts $element
    foreach {number id type} $element {break}
    switch $type {
      c {
        set attr [tip_CreateAttribute $name \
                     [tip_CreateAttributeValue CDM_STRING_SET $cons]]
        set ann [tip_PutAttribute [tip_GetAnnotation $doc $id] $attr]
        tip_AddAnnotation $doc $ann
        set cons {}
      }
      d {lappend cons $id}
    }
  }
};# creole_RecogniseContainedAnnotations_Finish

#
# End of File RecogniseContainedAnnotations.tcl
#

