##########################################################################
# 
#       RemoveAnnotationsTool.tcl - Tuesday December 19 08:03:44 (EET) 2000
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################

# Location of this module (RemoveAnnotationsTool)
global creole_RemoveAnnotationsTool_home

## creole_RemoveAnnotationsTool
#  
proc creole_RemoveAnnotationsTool {doc col args} {
  puts $args
  if {![string length $doc] && ![string length $col]} {
    gut_info $::GUI_Text(PleaseOpenColl/Doc)
    return
  }

  ## Create the window
  set win [creole_RemoveAnnotationsToolCreateWindow $col $doc]
} ;# creole_RemoveAnnotationsTool

## This function will:
## 1) Create the tool's window
## 2) Check given collection/document for all its attributes/annotations
proc creole_RemoveAnnotationsToolCreateWindow {col doc} {
  global creole_RemoveAnnotationsTool GUI_Options GUI_Text
  set win [ggi_unused_win .creole_RemoveAnnotationsToolWindow]

  ## Reset our info structure, just in case...
  array unset creole_RemoveAnnotationsTool $win:*
  set creole_RemoveAnnotationsTool($win:DocAttrs) {}
  set creole_RemoveAnnotationsTool($win:AnnotationTypes) {}
  set creole_RemoveAnnotationsTool($win:ColAttrs) {}
  
  ## 1) Have we got a collection or a document?
  if {[string length $doc]} {
    ## A document:
    set Image IDocument
    set Message "$GUI_Text(EditingDocument): [tip_GetId $doc]\
               \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
    set creole_RemoveAnnotationsTool($win:Collection) $col
    set creole_RemoveAnnotationsTool($win:Document)   $doc
    ## Foreach annotation type, get all its defined attributes...
    foreach type [$doc GetAnnotationTypes] {
      lappend creole_RemoveAnnotationsTool($win:AnnotationTypes) $type
      set creole_RemoveAnnotationsTool($win:AnnAttrs:$type) \
          [$doc GetAnnotationAttributeNames $type]
    }
    ## Get Document's attributes...
    foreach attr [$doc GetAttributes] {
      lappend creole_RemoveAnnotationsTool($win:DocAttrs) [tip_GetName $attr]
    }
  } else {
    ## A collection: Iterate over all documents...
    set Image ICollection
    set Message "$GUI_Text(EditingCollection): [file tail [tip_GetName $col]]\
               \n([tip_GetName $col])"
    set creole_RemoveAnnotationsTool($win:Collection) $col
    set creole_RemoveAnnotationsTool($win:Document)   $doc
    foreach attr [$col GetAttributes] {
      lappend creole_RemoveAnnotationsTool($win:ColAttrs) [tip_GetName $attr]
    }
    set wwin [ggi_wait_message {} 0.0]
    set doc [tip_FirstDocument $col]
    set Length [tip_Length $col]
    if {$Length > 500} {set Length 500}
    set Step [expr {100.0/($Length+1)}]
    set Progress 0.0
    set index 0
    while {[string length $doc]} {
      ## Get all annotation types that the document contains
      foreach type [$doc GetAnnotationTypes] {
        CDM_LappendUnique creole_RemoveAnnotationsTool($win:AnnotationTypes) \
                          $type
        ## Foreach annotation type, get all its defined attributes...
        foreach attrname [$doc GetAnnotationAttributeNames $type] {
          CDM_LappendUnique creole_RemoveAnnotationsTool($win:AnnAttrs:$type) \
                            $attrname
        }
      }
      ## Get Document's attributes...
      foreach attr [$doc GetAttributes] {
        CDM_LappendUnique creole_RemoveAnnotationsTool($win:DocAttrs) \
                          [tip_GetName $attr]
      }

      ## We got this Collection from within a system. We must not close any of
      ## the documents, as they may contain unsaved changes. We could check
      ## whether each document is modified or not, though...
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
      if {$index >= 500} {break}
      incr index
      tip_Close $doc
      set doc [tip_NextDocument $col]
    }
    set doc {}
    ggi_destroy_wait $wwin
  }
  if {![llength $creole_RemoveAnnotationsTool($win:AnnotationTypes)]} {
    gut_info $GUI_Text(EmptyCol/Doc)
    return
  }
  set creole_RemoveAnnotationsTool($win:AnnotationTypes) \
      [lsort $creole_RemoveAnnotationsTool($win:AnnotationTypes)]
  foreach type $creole_RemoveAnnotationsTool($win:AnnotationTypes) {
    if {![info exists creole_RemoveAnnotationsTool($win:AnnAttrs:$type)]} {
      set creole_RemoveAnnotationsTool($win:AnnAttrs:$type) {}
    }
  }
  set Type [lindex $creole_RemoveAnnotationsTool($win:AnnotationTypes) 0]
  set creole_RemoveAnnotationsTool($win:RemAnn) $Type 
  set creole_RemoveAnnotationsTool($win:RemAtt) [lindex \
      $creole_RemoveAnnotationsTool($win:AnnAttrs:$Type) 0]

  ## Tool Dialog Code
  toplevel $win
  wm withdraw $win
  wm title $win {Remove Annotations/Attributes Tool...}
  
  ## Header Section...
  set TitleFrame $win.title
  frame $TitleFrame
    label $TitleFrame.image -image $Image
    label $TitleFrame.text -height 2 -anchor w -justify left -text $Message
    grid $TitleFrame.image $TitleFrame.text -sticky snew -padx 3 -pady 6
    grid columnconfigure $TitleFrame 1 -weight 1
  pack $TitleFrame -fill x -expand 1

  set ComboFrame $win.combo
  frame $ComboFrame
    ## Remove Attributes from Collection
    if {![string length $doc]} {
      TitleFrame $ComboFrame.remColAttr -text \
         $GUI_Text(RemoveCollectionAttribute)
      set RemCAtt [$ComboFrame.remColAttr getframe]
        label $RemCAtt.mesg -anchor w -justify left \
          -textvar GUI_Text(RemoveAttribute)
        ComboBox $RemCAtt.attcb -editable 0 -highlightthickness 0 \
          -entrybg $GUI_Options(InputBackground) \
          -fg $GUI_Options(InputForeground) \
          -values  $creole_RemoveAnnotationsTool($win:ColAttrs) \
          -textvariable creole_RemoveAnnotationsTool($win:RemColAtt)
        $RemCAtt.attcb setvalue first
        button $RemCAtt.rematt -textvar GUI_Text(Remove) -command \
          "creole_RemoveAnnotationsToolExecute ColAttribute $win $RemCAtt.attcb"
        grid $RemCAtt.mesg $RemCAtt.attcb $RemCAtt.rematt \
          -padx 2 -pady 2 -sticky snew
        grid columnconfigure $RemCAtt 1 -weight 1
      pack $ComboFrame.remColAttr -pady 2 -fill x -expand 1
    }
    
    ## Remove Attributes from Documents
    TitleFrame $ComboFrame.remDocAttr -text $GUI_Text(RemoveDocumentAttribute)
    set RemDAtt [$ComboFrame.remDocAttr getframe]
      label $RemDAtt.mesg -anchor w -justify left \
        -textvar GUI_Text(RemoveAttribute)
      ComboBox $RemDAtt.attcb -editable 0 -highlightthickness 0 \
        -entrybg $GUI_Options(InputBackground) \
        -fg $GUI_Options(InputForeground) \
        -values  $creole_RemoveAnnotationsTool($win:DocAttrs) \
        -textvariable creole_RemoveAnnotationsTool($win:RemDocAtt)
      $RemDAtt.attcb setvalue first
      button $RemDAtt.rematt -textvar GUI_Text(Remove) -command \
        "creole_RemoveAnnotationsToolExecute DocAttribute $win $RemDAtt.attcb"
      grid $RemDAtt.mesg $RemDAtt.attcb $RemDAtt.rematt \
        -padx 2 -pady 2 -sticky snew
      grid columnconfigure $RemDAtt 1 -weight 1
    pack $ComboFrame.remDocAttr -pady 2 -fill x -expand 1
    
    ## Remove Annotations/Attributes from Documents
    TitleFrame $ComboFrame.remDocAnn -text $GUI_Text(RemoveDocumentAnnotations)
    set RemDAnn [$ComboFrame.remDocAnn getframe]
      label $RemDAnn.annmesg -anchor w -justify left \
        -textvar GUI_Text(RemoveAnnotation)
      label $RemDAnn.attmesg    -anchor w -justify left \
        -textvar GUI_Text(RemoveAnnotationAttribute)
      ComboBox $RemDAnn.anncb -editable 0 -highlightthickness 0 \
        -entrybg $GUI_Options(InputBackground) \
        -fg $GUI_Options(InputForeground) \
        -values $creole_RemoveAnnotationsTool($win:AnnotationTypes) \
        -modifycmd "creole_RemoveAnnotationsToolUpdateCombo $win $RemDAnn" \
        -textvariable creole_RemoveAnnotationsTool($win:RemAnn)
      ComboBox $RemDAnn.attcb -editable 0 -highlightthickness 0 \
        -entrybg $GUI_Options(InputBackground) \
        -fg $GUI_Options(InputForeground) \
        -values  $creole_RemoveAnnotationsTool($win:AnnAttrs:$Type) \
        -textvariable creole_RemoveAnnotationsTool($win:RemAtt)
      $RemDAnn.attcb setvalue first
      button $RemDAnn.remann -textvar GUI_Text(Remove) -command \
        "creole_RemoveAnnotationsToolExecute DocAnnotation $win $RemDAnn.anncb"
      button $RemDAnn.rematt -textvar GUI_Text(Remove) -command \
        "creole_RemoveAnnotationsToolExecute DocAnnAttr $win $RemDAnn.attcb"
      grid $RemDAnn.annmesg $RemDAnn.anncb $RemDAnn.remann \
        -padx 2 -pady 2 -sticky snew
      grid $RemDAnn.attmesg $RemDAnn.attcb $RemDAnn.rematt \
        -padx 2 -pady 2 -sticky snew
      grid columnconfigure $RemDAnn 1 -weight 1
    pack $ComboFrame.remDocAnn -pady 2 -fill x -expand 1
  pack $ComboFrame -padx 4 -pady 5 -fill x -expand 1

  ## Button Section
  set ButtonFrame $win.buttons
  frame $ButtonFrame
    button $ButtonFrame.close -textvar GUI_Text(Close) -command "destroy $win"
    bind $ButtonFrame.close <Destroy> \
      "array unset creole_RemoveAnnotationsTool \"$win:*\""
    pack $ButtonFrame.close
  pack $ButtonFrame -padx 4 -pady 5 -fill x -expand 1
  wm deiconify $win
};# creole_RemoveAnnotationsToolCreateWindow

##
proc creole_RemoveAnnotationsToolExecute {what win combo} {
  global creole_RemoveAnnotationsTool
  set col $creole_RemoveAnnotationsTool($win:Collection)
  set doc $creole_RemoveAnnotationsTool($win:Document)

  set DocAttr $creole_RemoveAnnotationsTool($win:RemDocAtt)
  set type    $creole_RemoveAnnotationsTool($win:RemAnn)
  set attr    $creole_RemoveAnnotationsTool($win:RemAtt)

  if {[string length $doc]} {
    switch -exact -- $what {
      DocAttribute  {catch {tip_RemoveAttribute $doc $DocAttr}}
      DocAnnotation {tip_DeleteAnnotations $doc $type}
      DocAnnAttr    {tip_DeleteAnnotations $doc "$type $attr"}
      default {error "Unknown type \"$what\""}
    }
  } else {
    set ColAttr $creole_RemoveAnnotationsTool($win:RemColAtt)
    set wwin [ggi_wait_message {} 0.0]
    set doc [tip_FirstDocument $col]
    set Length [tip_Length $col]
    set Step [expr {100.0/($Length+1)}]
    set Progress 0.0
    ## Is auto-save enebled for the Collection?
    set autosave [GUI_SystemWindowGetInfo $col AutoSave]
    while {[string length $doc]} {
      switch -exact -- $what {
        ColAttribute  {catch {tip_RemoveAttribute $col $ColAttr}}
        DocAttribute  {catch {tip_RemoveAttribute $doc $DocAttr}}
        DocAnnotation {tip_DeleteAnnotations $doc $type}
        DocAnnAttr    {tip_DeleteAnnotations $doc "$type $attr"}
        default {error "Unknown type \"$what\""}
      }
      if {$autosave} {
        tip_Sync $doc
        tip_Close $doc
        CDM_ClearUnusedSavedValues
      }
      set doc [tip_NextDocument $col]
      set Progress [expr {$Progress+$Step}]
      ggi_wait_update {} $Progress $wwin
    }
    set doc {}
    ggi_destroy_wait $wwin
  }

  ## Remove the entry from the ComboBox values...
  switch -exact -- $what {
    ColAttribute  {
      set val $ColAttr
      set creole_RemoveAnnotationsTool($win:RemColAtt) {}
    }
    DocAttribute  {
      set val $DocAttr
      set creole_RemoveAnnotationsTool($win:RemDocAtt) {}
    }
    DocAnnotation {
      set val $type
      set creole_RemoveAnnotationsTool($win:RemAnn) {}
      set creole_RemoveAnnotationsTool($win:RemAtt) {}
    }
    DocAnnAttr    {
      set val $attr
      set creole_RemoveAnnotationsTool($win:RemAtt) {}
      ## 25/12/2002: Bug reported by Sigletos: If we remove an Attribute from
      ## an Annotation, this Attribute shows up again for removal if we
      ## re-select the same Annotation...
      if {[info exists creole_RemoveAnnotationsTool($win:AnnAttrs:$type)]} {
        set index [lsearch $creole_RemoveAnnotationsTool($win:AnnAttrs:$type) \
                           $attr]
        if {$index != -1} {
          set creole_RemoveAnnotationsTool($win:AnnAttrs:$type) \
            [lreplace $creole_RemoveAnnotationsTool($win:AnnAttrs:$type) \
                      $index $index]
        }
      }
    }
  }
  set values [$combo cget -values]
  set index [lsearch $values $val]
  if {$index != -1} {
    $combo configure -values [lreplace $values $index $index]
    $combo setvalue first
  }
};# creole_RemoveAnnotationsToolExecute

## This function updates our second combo box, if the selection changes in the
## first one...
proc creole_RemoveAnnotationsToolUpdateCombo {win frame} {
  global creole_RemoveAnnotationsTool

  set type $creole_RemoveAnnotationsTool($win:RemAnn)
  if {![string length $type]} {return}
  set attrs $creole_RemoveAnnotationsTool($win:AnnAttrs:$type)
  set cb2 $frame.attcb
  $cb2 configure -values $attrs
  $cb2 setvalue first
};# creole_RemoveAnnotationsToolUpdateCombo
#
# End of File
#

