##########################################################################
# 
#        RenameAnnotationsTool.tcl - Wednesday January 09 19:52:16 (EET) 2002
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################


# Location of this module (RenameAnnotationsTool)
global creole_RenameAnnotationsTool_home

## creole_RenameAnnotationsTool
#  
proc creole_RenameAnnotationsTool {doc col args} {
  if {![string length $doc] && ![string length $col]} {
    global GUI_Text
    gut_info $GUI_Text(PleaseOpenColl/Doc)
    return
  }

  global GUI_Text GUI_Options
  set win [ggi_unused_win .creole_RemoveAnnotationsToolWindow]

  ## 1) Have we got a collection or a document?
  if {[string length $doc]} {
    ## A document:
    set Image IDocument
    set Message "$GUI_Text(EditingDocument): [tip_GetId $doc]\
               \n$GUI_Text(FromCollection): [file tail [tip_GetName $col]]"
    set Types [lsort [$doc GetAnnotationTypes]]
  } else {
    ## A collection: Iterate over all documents...
    set Image ICollection
    set Message "$GUI_Text(EditingCollection): [file tail [tip_GetName $col]]\
               \n([tip_GetName $col])"
    set Doc [tip_FirstDocument $col]
    set Types [lsort [$Doc GetAnnotationTypes]]
  }

  ## Tool Dialog Code
  toplevel $win
  wm withdraw $win
  wm title $win {Rename Annotations Tool...}
  
  set TitleFrame $win.title
  ttk::frame $TitleFrame
    ttk::label $TitleFrame.image -image $Image
    ttk::label $TitleFrame.text -anchor w -justify left -text $Message
    grid $TitleFrame.image $TitleFrame.text -sticky snew -padx 3 -pady 6
    grid columnconfigure $TitleFrame 1 -weight 1
  pack $TitleFrame -fill x -expand 1

  ## The two ComboBoxes that will hold the two names...
  ttk::labelframe $win.combo -text {Rename Annotation Type:}
  ttk::label $win.combo.lbl -text {Annotation type:} -anchor w
  ttk::combobox $win.combo.original -state normal -values $Types
  $win.combo.original set [lindex $Types 0]
  ttk::combobox $win.combo.rename -state normal -values $Types
  $win.combo.rename set [lindex $Types 0]
  ArrowButton $win.combo.arrow -dir right -height 24 -width 24 \
    -bg [$win cget -bg]
  ArrowButton $win.combo.arrow2 -dir right -height 24 -width 24 \
    -bg [$win cget -bg]

  ttk::label $win.combo.consl -text {Selection Constraints:} -anchor w
  ttk::entry $win.combo.cons -text {}

  grid $win.combo.lbl $win.combo.original $win.combo.arrow $win.combo.arrow2 \
       $win.combo.rename -sticky snew -padx 4 -pady 4
  grid $win.combo.consl $win.combo.cons - - - -sticky snew -padx 4 -pady 4
  grid columnconfigure $win.combo 1 -weight 1
  grid columnconfigure $win.combo 4 -weight 1
  pack $win.combo -fill x -expand 1 -padx 2 -pady 2

  ## Button Section
  set ButtonFrame $win.buttons
  ttk::frame $ButtonFrame
    ttk::button $ButtonFrame.ok -textvar GUI_Text(Ok) -command \
      "set ::creole_RenameAnnotationsTool($win) ok"
    ttk::button $ButtonFrame.close -textvar GUI_Text(Cancel) -command \
      "set ::creole_RenameAnnotationsTool($win) cancel"
    pack $ButtonFrame.ok $ButtonFrame.close -side left -padx 10 -pady 10
  pack $ButtonFrame
  
  wm deiconify $win

  tkwait variable ::creole_RenameAnnotationsTool($win)
  ## Has the user pressed "Ok"?
  if {[string equal $::creole_RenameAnnotationsTool($win) ok]} {
    ## Get the two values from the ComboBoxes...
    set original    [$win.combo.original get]
    set rename      [$win.combo.rename   get]
    set constraints [$win.combo.cons     get]
    ## Is it a Collection or a Document?
    if {[string length $doc]} {
      ## A Document...
      foreach ann [tip_SelectAnnotations $doc $original $constraints] {
        tip_AddAnnotation $doc [lreplace $ann 1 1 $rename]
      }
    } else {
      ## A Collection...
      set Length [tip_Length $col]
      set Step [expr {100.0/($Length+1)}]
      set Progress 0.0
      set wwin [ggi_wait_message {} 0.0]
      for {set doc [tip_FirstDocument $col]} {[string length $doc]} \
          {set doc [tip_NextDocument $col]} {
        foreach ann [tip_SelectAnnotations $doc $original $constraints] {
          tip_AddAnnotation $doc [lreplace $ann 1 1 $rename]
        }
        set Progress [expr {$Progress+$Step}]
        ggi_wait_update {} $Progress $wwin
      }
      ggi_destroy_wait $wwin
    }
  }
  unset ::creole_RenameAnnotationsTool($win)
  destroy $win
};# creole_RenameAnnotationsTool
#
# End of File
#

