/***************************************************************************
*
*       HBrill.cpp - Saturday October 30 12:07:45 (EEST) 1999
*       This is a dynamic/tight coupled (C) Module for use with 
*       Ellogon, version 1.0...
*
***************************************************************************/

/*
 * Use the following define, if you want to emulate the bugs of the original
 * Brill implementation...
 */
#define EMULATE_BRILL_BUGS


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CDM.h"

#ifdef WINDOWS
#ifndef R_OK
#define R_OK 04
#endif /* R_OK */
#endif /* WINDOWS */

/* Prototype Definitions */
extern "C" CDM_EXPORT int Creole_hbrill_Init(Tcl_Interp *interp);
int creole_HBrill(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HBrill_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HBrill_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);

#define MAX_LINE_LENGTH 1024 /* There must not be a line in all input files
                                greater that 1024 characters... */
#define LANGUAGE_GREEK   0
#define LANGUAGE_ENGLISH 1
#define LANGUAGE_TCL     2

CDM_BEGIN_GLOBAL_VARIABLE_BLOCK(HBrill)
/* 
 * Module's Home Path: Its value will be updated when the module will be
 * loaded... 
 */
CDM_DEFINE_GLOBAL(char *,        creole_HBrill_home);
CDM_DEFINE_GLOBAL(int,           HBrill_WordList);
CDM_DEFINE_GLOBAL(int,           HBrill_HashTablesInitialised);
CDM_DEFINE_GLOBAL(int,           HBrill_RestrictMove);
CDM_DEFINE_GLOBAL(int,           StartStateTagger);
CDM_DEFINE_GLOBAL(Tcl_HashTable, LexiconHash);
CDM_DEFINE_GLOBAL(Tcl_HashTable, WordListHash);
CDM_DEFINE_GLOBAL(Tcl_HashTable, BigramsHash);
CDM_DEFINE_GLOBAL(Tcl_HashTable, SeenTags);
CDM_DEFINE_GLOBAL(Tcl_Obj *,     LexicalRuleList);
CDM_DEFINE_GLOBAL(Tcl_Obj *,     ContextualRuleList);
CDM_DEFINE_GLOBAL(int,           HBrill_StartStateAlgorithmLanguage);
CDM_DEFINE_GLOBAL(Tcl_Obj*,      HBrill_StartStateAlgorithmCommand);
CDM_DEFINE_GLOBAL(char,          HBrill_StartStateAlgorithmCommandResult[128]);
CDM_END_GLOBAL_VARIABLE_BLOCK(HBrill)
/*
 * Make global variables available everywhere, without using
 * CDM_GLOBAL(var)...
 */
#define creole_HBrill_home           (tsdPtr->creole_HBrill_home)
#define HBrill_WordList              (tsdPtr->HBrill_WordList)
#define HBrill_HashTablesInitialised (tsdPtr->HBrill_HashTablesInitialised)
#define HBrill_RestrictMove          (tsdPtr->HBrill_RestrictMove)
#define StartStateTagger             (tsdPtr->StartStateTagger)
#define LexiconHash                  (tsdPtr->LexiconHash)
#define WordListHash                 (tsdPtr->WordListHash)
#define BigramsHash                  (tsdPtr->BigramsHash)
#define SeenTags                     (tsdPtr->SeenTags)
#define LexicalRuleList              (tsdPtr->LexicalRuleList)
#define ContextualRuleList           (tsdPtr->ContextualRuleList)
#define HBrill_StartStateAlgorithmLanguage \
               (tsdPtr->HBrill_StartStateAlgorithmLanguage)
#define HBrill_StartStateAlgorithmCommand \
               (tsdPtr->HBrill_StartStateAlgorithmCommand)
#define HBrill_StartStateAlgorithmCommandResult \
               (tsdPtr->HBrill_StartStateAlgorithmCommandResult)

int      creole_HBrill_InitHashTables(char *LexiconFile, char *BigramsFile,
             char *LexRuleFile, char *ConRuleFile, char *WordListFile);
int      creole_HBrill_FreeHashTables(void);
const char *
         creole_HBrill_StartStateAlgorithm(Tcl_Interp *interp, char *word);
int      creole_HBrill_TagLine(Tcl_Interp *interp, CDM_Document Doc,
                               Tcl_Obj *Anns, Tcl_Obj *Line);
int      HBrill_ApplyLexicalRules(Tcl_Obj *Line, Tcl_HashTable *Token2Tags);
int      HBrill_ApplyContextualRules(Tcl_Obj *Line, Tcl_Obj *TaggedLine);
int      HBrill_NotJustBlank(char *line);
/* static   char *HBrillFixStr(char *line); */

/*
 *  The followin routine implements the Initial Start State Algorithm.
 *  If you change it, make sure that these changes are reflected to the
 *  learner's code!
 */
const char *creole_HBrill_StartStateAlgorithm(Tcl_Interp *interp, char *word) {
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  switch (HBrill_StartStateAlgorithmLanguage) {
    case LANGUAGE_GREEK: {
      if ((*word >='A' && *word <='Z') || (*word >='a' && *word <='z')) {
        return ("FW");
      } else if (*word >='0' && *word <='9') {
        return ("CD");
      } else if (strchr("ÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÓÔÕÖ×Ø٢¸º¼¾¿ÚÛ", *word) != NULL) {
        return ("NNPM");
      }
      return ("NNF");
    }
    case LANGUAGE_ENGLISH: {
      if ((*word >='A' && *word <='Z')) {
        return ("NNP");
      } else {
        return ("NN");
      }
    }
    case LANGUAGE_TCL: {
      Tcl_Obj *objPtr[2];
      objPtr[0] = HBrill_StartStateAlgorithmCommand;
      objPtr[1] = Tcl_NewStringObj(word, -1);
      for (int i = 0; i < 2; ++i) Tcl_IncrRefCount(objPtr[i]);
      int status = Tcl_EvalObjv(interp, 2, objPtr, TCL_EVAL_GLOBAL);
      for (int i = 0; i < 2; ++i) Tcl_DecrRefCount(objPtr[i]);
      if (status == TCL_OK) {
        strncpy(HBrill_StartStateAlgorithmCommandResult,
                Tcl_GetStringResult(interp), 128);
        return HBrill_StartStateAlgorithmCommandResult;
      }
    }
  }
  return "";
} /* creole_HBrill_StartStateAlgorithm */

/* Function: creole_HBrill */
int creole_HBrill(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  CDM_Document Doc;
  CDM_AnnotationSet Sentences, Annotations=NULL;
  CDM_Annotation Ann;
  CDM_Attribute Constituents;
  CDM_AttributeValue Value;
  CDM_Span Span;
  Tcl_Obj **Sentence, **Id, *LineObj=NULL, *ObjPtr;
  double progress, step;
  long start, end;
  char *Text;
  int sentences, sent_id, id, ids, last_progress;
  
  CDM_SetActiveInterpreter(interp); /* Do not remove this command... */
  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "Document Lexicon Bigrams LexicalRuleFile"
        " ContextualRuleFile ?-w WordList? ?-s SplitNumber? ?-i IntermedFile? "
        "?-S? ?-F?");
    return TCL_ERROR;
  }
  Doc = CDM_GetDocumentPointer(objv[1]);
  if (Doc == NULL) 
    return CDM_ReportError("creole_HBrill: Invalid Document Pointer");
  CDM_SetProcessStatus(Doc, 0);

  /*
   * Create the string buffer...
   */
  /* Get Document Text... */
  Text = CDM_GetByteSequence(Doc, "iso8859-7");
  /* Get Sentence Annotations... */
  Sentences = CDM_SelectAnnotations(Doc, "sentence");

  /* Foreach Sentence Annotation */
  Tcl_ListObjGetElements(interp, Sentences, &sentences, &Sentence);
  if (sentences < 1) {
    CDM_ReportError("No Sentence Annotations!");
    goto error;
  }
  
  progress = 0.0;
  last_progress = 0;
  step = 100.0/float(sentences);
  for (sent_id=0; sent_id<sentences; sent_id++) {
    progress += step;
    if (int(progress) - last_progress == 10) {
      last_progress = int(progress);
      CDM_SetProcessStatus(Doc, last_progress);
    }
    LineObj = Tcl_NewListObj(0, NULL); Annotations = Tcl_NewListObj(0, NULL);
    if (LineObj == NULL || Annotations == NULL) 
       {CDM_ReportError("Out of memory"); goto error;}
    Tcl_IncrRefCount(LineObj); Tcl_IncrRefCount(Annotations);
    /* Get Sentence Constituents... */
    Constituents = CDM_GetAttribute(Sentence[sent_id], "constituents");
    if (Constituents == NULL) continue;
    Value = CDM_GetValueValue(Constituents);
    /* Foreach ID... */
    Tcl_ListObjGetElements(interp, Value, &ids, &Id);
    for (id=0; id<ids; id++) {
      Ann = CDM_GetAnnotation(Doc, Id[id]);
      if (Ann == NULL) {
        goto error;
      }
      if (Tcl_ListObjAppendElement(interp,Annotations,Ann) !=TCL_OK) goto error;
      Span = CDM_GetFirstSpan(Ann);
      CDM_GetSpanOffsets(Span, &start, &end);
      ObjPtr = Tcl_NewStringObj((Text+start), end - start);
      if (ObjPtr == NULL) {CDM_ReportError("Out of memory"); goto error;}
      if (Tcl_ListObjAppendElement(interp,LineObj,ObjPtr) != TCL_OK) goto error;
    }
    
    /*
     * Call Brill Tagger on buffer...
     */
    creole_HBrill_TagLine(interp, Doc, Annotations, LineObj);

    /* Free Buffer... */
    Tcl_DecrRefCount(LineObj); Tcl_DecrRefCount(Annotations);
    LineObj = Annotations = NULL;
  }

  /* Free Memory... */
  Tcl_Free(Text);
  Tcl_DecrRefCount(Sentences);

  /* Report Succesful Completion */
  CDM_SetProcessStatus(Doc, 100);
  CDM_PutAttribute(Doc, CDM_CreateAttribute("HBrill",
                        CDM_CreateAttributeValue(CDM_STRING, "")));
  return TCL_OK;
error:
  if (LineObj != NULL) Tcl_DecrRefCount(LineObj);
  if (Annotations != NULL) Tcl_DecrRefCount(Annotations);
  creole_HBrill_FreeHashTables();
  Tcl_Free(Text);
  Tcl_DecrRefCount(Sentences);
  return TCL_ERROR;
} /* creole_HBrill */

/*
 * Apply Brill's Algorithm on text in "Line" object...
 */
int creole_HBrill_TagLine(Tcl_Interp *interp, CDM_Document Doc,
                          Tcl_Obj *Anns, Tcl_Obj *Line) {
  CDM_Attribute Attr;
  Tcl_HashTable TokenTable;
  Tcl_HashSearch searchPtr;
  Tcl_HashEntry *entry;
  Tcl_Obj **Tokens, **Ann, **Tags, *Tag, *TaggedLine, *objv[2];
  char *token;
  int items, anns, tags, i, newEntry;
  CDM_ACCESS_GLOBAL_DATA(HBrill);

  Tcl_InitHashTable(&TokenTable, TCL_STRING_KEYS);
  if (Tcl_ListObjGetElements(NULL, Line, &items, &Tokens)!=TCL_OK) {
    CDM_ReportError("Cannot get tokens of sentence: ");
    return CDM_AppendError(Tcl_GetString(Line));
  }
  for (i=0; i<items; i++) {
    token = Tcl_GetString(Tokens[i]);
    if ((entry = Tcl_FindHashEntry(&LexiconHash, token)) == NULL) {
      Tag = CDM_NewStringObj(creole_HBrill_StartStateAlgorithm(interp, token));
      entry = Tcl_CreateHashEntry(&TokenTable, token, &newEntry);
      if (newEntry) {
        Tcl_SetHashValue(entry, (char *) Tag); Tcl_IncrRefCount(Tag);
      }
    }
  }
  
  /*
   * Now, all tokens not in lexicon are in "TokenTable" Hash Table, and all
   * have tags according to Lexicon and start state algorithm, 
   * for unknown words...
   */
  HBrill_ApplyLexicalRules(Line, &TokenTable);

  /*
   * Now, get the final (unknown word) lexical tags from hash table and create
   * a list with all tags (known and unknown). This list will be given to
   * apply contextual rules...
   */
  objv[0] = objv[1] = CDM_NewStringObj("STAART");
  TaggedLine = Tcl_NewListObj(2, objv);
  if (TaggedLine == NULL) return CDM_ReportError("Out of memory");
  Tcl_IncrRefCount(TaggedLine);
  
  for (i=0; i<items; i++) {
    token = Tcl_GetString(Tokens[i]);
    if ((entry = Tcl_FindHashEntry(&LexiconHash, token)) == NULL) {
      /* Word was not in the lexicon. We have to find out how the lexical
       * rules have tagged it... */
      entry = Tcl_FindHashEntry(&TokenTable, token);
      Tag = (Tcl_Obj *) Tcl_GetHashValue(entry);
      Tcl_ListObjAppendElement(NULL, TaggedLine, Tag);
    } else {
      /* Word is in the lexicon. Just use as tag the most frequent tag... */
      Tag = (Tcl_Obj *) Tcl_GetHashValue(entry);
      Tcl_ListObjAppendElement(NULL, TaggedLine, Tag);
    }
  }
  Tcl_ListObjAppendElement(NULL, TaggedLine, objv[0]);
  Tcl_ListObjAppendElement(NULL, TaggedLine, objv[0]);

  /* Clear Hash Table... */
  entry = Tcl_FirstHashEntry(&TokenTable, &searchPtr); 
  while (entry != NULL) {
    Tcl_DecrRefCount((Tcl_Obj *) Tcl_GetHashValue(entry));
    entry = Tcl_NextHashEntry(&searchPtr);
  }
  Tcl_DeleteHashTable(&TokenTable);

  /*
   * Also, add the special words "STAART" at the beginning and end of the word
   * list...
   */
  Tcl_ListObjReplace(NULL, Line, items, 0, 2, objv);
  Tcl_ListObjReplace(NULL, Line, 0    , 0, 2, objv);

  /*
   * Finally, apply Contextual Rules...
   */
  if (!StartStateTagger) {
    HBrill_ApplyContextualRules(Line, TaggedLine);
  }
   
  /*
   * "TaggedLine" now has the final tags. Add them to the Document...
   */
  Tcl_ListObjGetElements(NULL, Anns, &anns, &Ann);
  Tcl_ListObjGetElements(NULL, TaggedLine, &tags, &Tags);
  for (i=2; i<tags-2; i++) {
    Attr = CDM_CreateAttribute("pos", 
               CDM_CreateAttributeValue(CDM_STRING, Tcl_GetString(Tags[i])));
    CDM_AddAnnotation(Doc, CDM_PutAttribute(Ann[i-2], Attr));
  }
  Tcl_DecrRefCount(TaggedLine);
  return TCL_OK;
} /* creole_HBrill_TagLine */

/*
 * Initialise Hash Tables
 */
int creole_HBrill_InitHashTables(char *LexiconFile, char *BigramsFile,
    char *LexRuleFile, char *ConRuleFile, char *WordListFile) {
  FILE *File;
  Tcl_HashEntry *entry;
  Tcl_Obj *LineObj, **Item, *Obj;
  char line[MAX_LINE_LENGTH], tmpstr[MAX_LINE_LENGTH+1], *word, *tag;
  int integer, items, i;
  CDM_ACCESS_GLOBAL_DATA(HBrill);

  if (HBrill_HashTablesInitialised) return TCL_RETURN;
  HBrill_HashTablesInitialised = 1;
  
  /*
   * Create Lexicon Hash Table.
   * Lexicon Hash stores the most likely tag for all known words.
   * We can have a separate wordlist and lexicon file because unsupervised
   * learning can add to wordlist, while not adding to lexicon.  For
   * example, if a big untagged corpus is about to be tagged, the wordlist
   * can be extended to include words in that corpus, while the lexicon
   * remains static. Lexicon is file of form: 
   *           word t1 t2 ... tn 
   * where t1 is the most likely tag for the word, and t2...tn are alternate
   * tags, in no particular order.
   */
  Tcl_InitHashTable(&LexiconHash, TCL_STRING_KEYS);
  Tcl_InitHashTable(&SeenTags, TCL_STRING_KEYS);
  File = fopen(LexiconFile, "r");
  if (File == NULL) {
    CDM_ReportError("Cannot open file: ");
    return CDM_AppendError(LexiconFile);
  }
  /* Read Lexicon file... */
  while(fgets(line, MAX_LINE_LENGTH, File) != NULL) {
    line[strlen(line) -1] = '\0';
    LineObj = CDM_SplitString(line);
    if (LineObj == NULL ||
        Tcl_ListObjGetElements(NULL, LineObj, &items, &Item) != TCL_OK) {
      fclose(File);
      if (LineObj) Tcl_DecrRefCount(LineObj);
      return CDM_ReportError("Cannot Parse Lexicon File!\n");
    }
    if (items < 2) {
      fclose(File);
      Tcl_DecrRefCount(LineObj);
      CDM_ReportError("Less than two items in a line found (Lexicon File)!\n");
      return CDM_AppendError(line);
    }
    word = Tcl_GetString(Item[0]); tag = Tcl_GetString(Item[1]);
    entry = Tcl_CreateHashEntry(&LexiconHash, word, &integer);
    Obj = CDM_NewStringObj(tag); Tcl_IncrRefCount(Obj);
    Tcl_SetHashValue(entry, (char *) Obj);
    for (i=1; i<items; i++) {
      sprintf(tmpstr, "%s %s", word, Tcl_GetString(Item[i]));
      entry = Tcl_CreateHashEntry(&SeenTags, tmpstr, &integer);
    }
    Tcl_DecrRefCount(LineObj);
  }
  fclose(File);
#ifdef DEBUG
  printf("Start State Tagger: Lexicon Read ok...\n");
#endif /* DEBUG */
  
  /* Create WordList Hash Table.
   * WordList Hash table contains a list of words.  This is used 
   * for tagging unknown words in the "add prefix/suffix" and
   * "delete prefix/suffix" rules.
   * This contains words not in LEXICON. 
   */
  Tcl_InitHashTable(&WordListHash, TCL_STRING_KEYS);
  HBrill_WordList = 0;
  if (WordListFile != NULL) {
    File = fopen(WordListFile, "r");
    if (File == NULL) {
      CDM_ReportError("Cannot open File: ");
      return CDM_AppendError(WordListFile);
    }
    /* Read WordList File... */
    while(fgets(line, MAX_LINE_LENGTH, File) != NULL) {
      if (integer && HBrill_NotJustBlank(line)) {
        entry = Tcl_CreateHashEntry(&LexiconHash, line, &integer);
        Tcl_SetHashValue(entry, (char *)1);
      }
    }
    fclose(File);
    HBrill_WordList = 1;
#ifdef DEBUG
    printf("Start State Tagger: WordList Read ok...\n");
#endif /* DEBUG */
  }
  
  /*
   * Read Lexical Rules...
   */
  LexicalRuleList = Tcl_NewListObj(0, NULL);
  if (LexicalRuleList == NULL) return CDM_ReportError("Out of memory");
  Tcl_IncrRefCount(LexicalRuleList);
  File = fopen(LexRuleFile, "r");
  if (File == NULL) {
    CDM_ReportError("Cannot open File: ");
    return CDM_AppendError(LexRuleFile);
  }
  /* Read Lexical Rule File... */
  while(fgets(line, MAX_LINE_LENGTH, File) != NULL) {
    line[strlen(line) -1] = '\0';
    LineObj = CDM_SplitString(line);
    if (LineObj == NULL ||
        Tcl_ListObjGetElements(NULL, LineObj, &items, &Item) != TCL_OK) {
      fclose(File);
      if (LineObj) Tcl_DecrRefCount(LineObj);
      CDM_ReportError("Cannot process lexical rule: ");
      return CDM_AppendError(line);
    }
    if (items < 1) {
      printf("Skipping Lexical Rule: (%s)\n", line);
      Tcl_DecrRefCount(LineObj);
      continue;
    }
    Tcl_ListObjAppendElement(NULL, LexicalRuleList, LineObj);
  }
  fclose(File);
#ifdef DEBUG
  printf("Start State Tagger: Lexical Rules Read ok...\n");
#endif /* DEBUG */

  /*
   * Read Contextual Rules...
   */
  ContextualRuleList = Tcl_NewListObj(0, NULL);
  if (ContextualRuleList == NULL) return CDM_ReportError("Out of memory");
  Tcl_IncrRefCount(ContextualRuleList);
  File = fopen(ConRuleFile, "r");
  if (File == NULL) {
    CDM_ReportError("Cannot open File: ");
    return CDM_AppendError(ConRuleFile);
  }
  /* Read Contextual Rule File... */
  while(fgets(line, MAX_LINE_LENGTH, File) != NULL) {
    line[strlen(line) -1] = '\0';
    LineObj = CDM_SplitString(line);
    if (LineObj == NULL ||
        Tcl_ListObjGetElements(NULL, LineObj, &items, &Item) != TCL_OK) {
      fclose(File);
      if (LineObj) Tcl_DecrRefCount(LineObj);
      CDM_ReportError("Cannot process contextual rule: ");
      return CDM_AppendError(line);
    }
    if (items < 1) {
      printf("Skipping Contextual rule: (%s)\n", line);
      Tcl_DecrRefCount(LineObj);
      continue;
    }
    Tcl_ListObjAppendElement(NULL, ContextualRuleList, LineObj);
  }
  fclose(File);
#ifdef DEBUG
  printf("Start State Tagger: Contextual Rules Read ok...\n");
#endif /* DEBUG */

  /*
   * Read Bigrams File...
   */
  Tcl_InitHashTable(&BigramsHash, TCL_STRING_KEYS);
  File = fopen(BigramsFile, "r");
  if (File == NULL) {
    CDM_ReportError("Cannot open File: ");
    return CDM_AppendError(BigramsFile);
  }
  /* Read Bigrams File... */
  while(fgets(line, MAX_LINE_LENGTH, File) != NULL) {
    line[strlen(line) -1] = '\0';
    LineObj = CDM_SplitString(line);
    if (LineObj == NULL ||
        Tcl_ListObjGetElements(NULL, LineObj, &items, &Item) != TCL_OK) {
      fclose(File);
      if (LineObj) Tcl_DecrRefCount(LineObj);
      CDM_ReportError("Cannot process Bigram: ");
      return CDM_AppendError(line);
    }
    if (items != 2) {
      Tcl_DecrRefCount(LineObj);
      printf("Skipping Bigram: (%s)\n", line);
      continue;
    }
    word = Tcl_GetString(Item[0]); tag = Tcl_GetString(Item[1]);
    snprintf(tmpstr, MAX_LINE_LENGTH, "%s %s", word, tag);
    entry = Tcl_CreateHashEntry(&BigramsHash, tmpstr, &integer);
    Tcl_DecrRefCount(LineObj);
  }
  fclose(File);
#ifdef DEBUG
  printf("Start State Tagger: Bigrams Read ok...\n");
#endif /* DEBUG */

  return TCL_OK;
} /* creole_HBrill_InitHashTables */

/*
 * Free Hash Tables
 */
int creole_HBrill_FreeHashTables(void) {
  Tcl_HashEntry *entryPtr;
  Tcl_HashSearch searchPtr;
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  if (!HBrill_HashTablesInitialised) return TCL_RETURN;
  HBrill_HashTablesInitialised = 0;

  entryPtr = Tcl_FirstHashEntry(&LexiconHash, &searchPtr);
  while (entryPtr != NULL) {
    Tcl_DecrRefCount((Tcl_Obj *)Tcl_GetHashValue(entryPtr));
    entryPtr = Tcl_NextHashEntry(&searchPtr);
  }
  Tcl_DeleteHashTable(&LexiconHash);
  Tcl_DeleteHashTable(&WordListHash);
  Tcl_DeleteHashTable(&BigramsHash);
  Tcl_DeleteHashTable(&SeenTags);
  Tcl_DecrRefCount(LexicalRuleList);
  Tcl_DecrRefCount(ContextualRuleList);
  return TCL_OK;
} /* creole_HBrill_FreeHashTables */


/*
 * Apply lexical rules...
 */
int HBrill_ApplyLexicalRules(Tcl_Obj *Line, Tcl_HashTable *Token2Tags) {
  Tcl_HashEntry *entry;
  Tcl_DString ds;
  Tcl_Obj **Tokens, **Rules, **Rule, *Tag, *OldTag;
  char *str, *str0, *str1, *str2, *str3, *token, *tag, *old_tag;
  int items, rules, rule_tokens, rule_count, last, i, integer;
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  
  if (Tcl_ListObjGetElements(NULL, LexicalRuleList, &rules, &Rules)!=TCL_OK) {
    return CDM_ReportError("Cannot split lexical rule list");
  }
  //printf("Lexical rules: %d\n", rules);
  if (Tcl_ListObjGetElements(NULL, Line, &items, &Tokens)!=TCL_OK) {
    return CDM_ReportError("Cannot split sentence list");
  }
  //printf("Number of tokens: %d\n", items);
  for (rule_count=0; rule_count<rules; rule_count++) {
    if (Tcl_ListObjGetElements(NULL, Rules[rule_count], &rule_tokens, &Rule) 
        != TCL_OK) {
       CDM_ReportError("Cannot split lexical rule: ");
       return CDM_AppendError(Tcl_GetString(Rules[rule_count]));
    }
    str0  = Tcl_GetString(Rule[0]);
    str1  = Tcl_GetString(Rule[1]);
    str2  = Tcl_GetString(Rule[2]);
    str3  = Tcl_GetString(Rule[3]);
    last = rule_tokens - 2;

    /* Rule Type: "char" */
    if (strcmp(str1, "char") == 0) {
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) {
          if (strpbrk(token, str0) != NULL) {
#ifdef DEBUG
          printf("\"%s\" %s->%s (%s %d)\n", token, old_tag,
          Tcl_GetString(Rule[last]), "char", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
            Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "fchar" */
    if (strcmp(str2, "fchar") == 0) {
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          if (strpbrk(token, str1) != NULL) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "fchar", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tag = Rule[last];
            Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "deletepref" */
    if (strcmp(str1, "deletepref") == 0) {
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      integer = atoi(str2);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) {
          if (strncmp(token, str0, integer) == 0) {
            str = token + integer;
            if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "deletepref", rule_count);
#endif
               Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
               Tcl_SetHashValue(entry, Tag);
            }
          }
        }
      }
    } else 

    /* Rule Type: "fdeletepref" */
    if (strcmp(str2, "fdeletepref") == 0) {
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      integer = atoi(str3);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          if (strncmp(token, str1, integer) == 0) {
            str = token + integer;
            if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "fdeletepref", rule_count);
#endif
               Tcl_DecrRefCount(OldTag); Tag = Rule[last];
               Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
            }
          }
        }
      }
    } else 
    
    /* Rule Type: "haspref" */
    if (strcmp(str1, "haspref") == 0) {
      //printf("haspref: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      integer = atoi(str2);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) {
          if (strncmp(token, str0, integer) == 0) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "haspref", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
            Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "fhaspref" */
    if (strcmp(str2, "fhaspref") == 0) {
      //printf("fhaspref: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      integer = atoi(str3);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          if (strncmp(token, str1, integer) == 0) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "fhaspref", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tag = Rule[last];
            Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "deletesuf" */
    if (strcmp(str1, "deletesuf") == 0) {
      //printf("deletesuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      integer = atoi(str2);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) {
          str = &token[strlen(token) - integer];
          if (strncmp(str, str0, (strlen(token) - integer)) == 0) {
            str = CDM_Strdup(token);
            str[strlen(token) - integer] = '\0';
            if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "deletesuf", rule_count);
#endif
               Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
               Tcl_SetHashValue(entry, Tag);
            }
            CDM_Free(str);
          }
        }
      }
    } else
    
    /* Rule Type: "fdeletesuf" */
    if (strcmp(str2, "fdeletesuf") == 0) {
      //printf("fdeletesuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      integer = atoi(str3);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          str = &token[strlen(token) - integer];
          if (strncmp(str, str1, integer) == 0) {
            str = CDM_Strdup(token);
            str[strlen(token) - integer] = '\0';
            if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "fdeletesuf", rule_count);
#endif
               Tcl_DecrRefCount(OldTag); Tag = Rule[last];
               Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
            }
            CDM_Free(str);
          }
        }
      }
    } else

    /* Rule Type: "hassuf" */
    if (strcmp(str1, "hassuf") == 0) {
      //printf("hassuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      integer = atoi(str2);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) {
          str = &token[strlen(token) - integer];
          if (strncmp(str, str0, integer) == 0) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "hassuf", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
            Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "fhassuf" */
    if (strcmp(str2, "fhassuf") == 0) {
      //printf("fhassuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      integer = atoi(str3);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          str = &token[strlen(token) - integer];
          if (strncmp(str, str1, integer) == 0) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "fhassuf", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tag = Rule[last];
            Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
        }
      }
    } else 

    /* Rule Type: "addpref" */
    if (strcmp(str1, "addpref") == 0) {
      //printf("addpref: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) { // ERROR in original Tagger!
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, str0, -1);
          Tcl_DStringAppend(&ds, token, -1); str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "addpref", rule_count);
#endif
             Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
             Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else 

    /* Rule Type: "faddpref" */ // ERROR in original Tagger!
    if (strcmp(str2, "faddpref") == 0) {
      //printf("faddpref: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag); // ERROR in original Tagger!
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) { 
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, str1, -1);
          Tcl_DStringAppend(&ds, token, -1); str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "faddpref", rule_count);
#endif
             Tcl_DecrRefCount(OldTag); Tag = Rule[last];
             Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else

    /* Rule Type: "addsuf" */
    if (strcmp(str1, "addsuf") == 0) {
      //printf("addsuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) { 
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, token, -1);
          Tcl_DStringAppend(&ds, str0, -1); str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "addsuf", rule_count);
#endif
             Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
             Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else 

    /* Rule Type: "faddsuf" */
    if (strcmp(str2, "faddsuf") == 0) {
      //printf("faddsuf: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, token, -1);
          Tcl_DStringAppend(&ds, str1, -1); str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&LexiconHash, str) != NULL || (HBrill_WordList
                && Tcl_FindHashEntry(&WordListHash, str) != NULL)) {
#ifdef DEBUG
               printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
               Tcl_GetString(Rule[last]), "faddsuf", rule_count);
#endif
             Tcl_DecrRefCount(OldTag); Tag = Rule[last];
             Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else
    
    /* Rule Type: "goodleft" */
    if (strcmp(str1, "goodleft") == 0) {
      //printf("goodleft: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
#ifndef EMULATE_BRILL_BUGS
        if (i==items-1) str="S-T-A-R-T"; else str=Tcl_GetString(Tokens[i+1]);
        if (strcmp(str, str0) != 0) continue;
#endif
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) { 
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, token, -1);
          Tcl_DStringAppend(&ds, " ", -1); Tcl_DStringAppend(&ds, str0, -1); 
          str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&BigramsHash, str) != NULL) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "goodleft", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
            Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else 

    /* Rule Type: "fgoodleft" */
    if (strcmp(str2, "fgoodleft") == 0) {
      //printf("fgoodleft: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
#ifndef EMULATE_BRILL_BUGS
        if (i==items-1) str="S-T-A-R-T"; else str=Tcl_GetString(Tokens[i+1]);
        if (strcmp(str, str0) != 0) continue;
#endif
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, token, -1);
          Tcl_DStringAppend(&ds, " ", -1); Tcl_DStringAppend(&ds, str1, -1); 
          str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&BigramsHash, str) != NULL) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "fgoodleft", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tag = Rule[last];
            Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else

    /* Rule Type: "goodright" */
    if (strcmp(str1, "goodright") == 0) {
      //printf("goodright: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[last]; tag = Tcl_GetString(Tag);
      for (i=0; i<items; i++) {
#ifndef EMULATE_BRILL_BUGS
        if (i==0) str="S-T-A-R-T"; else str=Tcl_GetString(Tokens[i-1]);
        if (strcmp(str, str0) != 0) continue;
#endif
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) != 0) { 
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, str0, -1);
          Tcl_DStringAppend(&ds, " ", -1); Tcl_DStringAppend(&ds, token, -1); 
          str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&BigramsHash, str) != NULL) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "goodright", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tcl_IncrRefCount(Tag);
            Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    } else 

    /* Rule Type: "fgoodright" */
    if (strcmp(str2, "fgoodright") == 0) {
      //printf("fgoodright: %s\n", Tcl_GetString(Rules[rule_count]));
      Tag = Rule[0]; tag = str0;
      for (i=0; i<items; i++) {
#ifndef EMULATE_BRILL_BUGS
        if (i==0) str="S-T-A-R-T"; else str=Tcl_GetString(Tokens[i-1]);
        if (strcmp(str, str0) != 0) continue;
#endif
        token = Tcl_GetString(Tokens[i]);
        entry = Tcl_FindHashEntry(Token2Tags, token);
        if (entry == NULL) continue; // Word is in Lexicon, do not re-tag it...
        OldTag=(Tcl_Obj*)Tcl_GetHashValue(entry);old_tag=Tcl_GetString(OldTag);
        if (strcmp(tag, old_tag) == 0) {
          Tcl_DStringInit(&ds); Tcl_DStringAppend(&ds, str1, -1);
          Tcl_DStringAppend(&ds, " ", -1); Tcl_DStringAppend(&ds, token, -1); 
          str = Tcl_DStringValue(&ds);
          if (Tcl_FindHashEntry(&BigramsHash, str) != NULL) {
#ifdef DEBUG
            printf("\"%s\" %s->%s (%s %d)\n", token, old_tag, 
            Tcl_GetString(Rule[last]), "fgoodright", rule_count);
#endif
            Tcl_DecrRefCount(OldTag); Tag = Rule[last];
            Tcl_IncrRefCount(Tag); Tcl_SetHashValue(entry, Tag);
          }
          Tcl_DStringFree(&ds);
        }
      }
    }
  } /* Iteration on Lexical Rules... */
  return TCL_OK;
} /* HBrill_ApplyLexicalRules */

#define CHANGE_TAG(tag,pos)  { \
  Tcl_ListObjReplace(NULL, TaggedLine, pos, 1, 1, &tag); \
  Tcl_ListObjGetElements(NULL, TaggedLine, &tempi2, &Tags);}
int HBrill_ApplyContextualRules(Tcl_Obj *Line, Tcl_Obj *TaggedLine) {
  Tcl_Obj **Tokens, **Rules, **Rule, **Tags, *OldTag, *_new;
  char *old = NULL, *when, *tag = NULL, *lft = NULL, *rght = NULL,
       *prev1 = NULL, *prev2 = NULL, *next1 = NULL, *next2 = NULL,
       *curtag, *curwd, tempstr[256], *word = NULL;
  int items, rules, rule_tokens, rule_count, i, tempi1, tempi2;
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  
  Tcl_ListObjGetElements(NULL, ContextualRuleList, &rules, &Rules);
  Tcl_ListObjGetElements(NULL, Line, &items, &Tokens);
  Tcl_ListObjGetElements(NULL, TaggedLine, &i, &Tags);
  //printf("%s\n", Tcl_GetString(Line));
  items--;
  for (rule_count=0; rule_count<rules; rule_count++) {
    //printf("Applying rule \"%s\"\n", Tcl_GetString(Rules[rule_count]));
    Tcl_ListObjGetElements(NULL, Rules[rule_count], &rule_tokens, &Rule);
    old   = Tcl_GetString(Rule[0]);
    _new  = Rule[1];
    when  = Tcl_GetString(Rule[2]);

    if (strcmp(when, "NEXTTAG")==0 || strcmp(when, "NEXT2TAG")==0 ||
        strcmp(when, "NEXT1OR2TAG")==0 || strcmp(when, "NEXT1OR2OR3TAG")==0 ||
        strcmp(when, "PREVTAG")==0 || strcmp(when, "PREV2TAG")==0 ||
        strcmp(when, "PREV1OR2TAG")==0 || strcmp(when, "PREV1OR2OR3TAG")==0) {
             tag = Tcl_GetString(Rule[3]);
    } else if (strcmp(when, "NEXTWD")==0 || strcmp(when,"CURWD")==0 ||
        strcmp(when, "NEXT2WD")==0 || strcmp(when, "NEXT1OR2WD")==0 ||
        strcmp(when, "NEXT1OR2OR3WD")==0 || strcmp(when, "PREVWD")==0 ||
        strcmp(when, "PREV2WD")==0 || strcmp(when, "PREV1OR2WD")==0 ||
        strcmp(when, "PREV1OR2OR3WD") == 0) {
             word = Tcl_GetString(Rule[3]);
    } else if (strcmp(when, "SURROUNDTAG")==0) {
             lft  = Tcl_GetString(Rule[3]);
             rght = Tcl_GetString(Rule[4]);
    } else if (strcmp(when, "PREVBIGRAM")==0) {
             prev1 = Tcl_GetString(Rule[3]);
             prev2 = Tcl_GetString(Rule[4]);
    } else if (strcmp(when, "NEXTBIGRAM") == 0) {
             next1 = Tcl_GetString(Rule[3]);
             next2 = Tcl_GetString(Rule[4]);
    } else if (strcmp(when, "LBIGRAM")==0 || strcmp(when, "WDPREVTAG")==0) {
             prev1 = Tcl_GetString(Rule[3]);
             word  = Tcl_GetString(Rule[4]); 
    } else if (strcmp(when, "RBIGRAM")==0 || strcmp(when, "WDNEXTTAG")==0) {
             word  = Tcl_GetString(Rule[3]);
             next1 = Tcl_GetString(Rule[4]); 
    } else if (strcmp(when, "WDAND2BFR")==0 || strcmp(when,"WDAND2TAGBFR")==0) {
             prev2 = Tcl_GetString(Rule[3]);
             word  = Tcl_GetString(Rule[4]);
    } else if (strcmp(when,"WDAND2AFT")==0 || strcmp(when,"WDAND2TAGAFT")==0) {
             next2 = Tcl_GetString(Rule[4]);
             word  = Tcl_GetString(Rule[3]);
    }

    for (i=0; i<=items; i++) {
      curwd  = Tcl_GetString(Tokens[i]);
      //printf("%s : %s (%d of %d)\n",when, curwd, i, items);
      OldTag = Tags[i];
      curtag = Tcl_GetString(OldTag);
      if (strcmp(old, curtag) != 0) continue;
      snprintf(tempstr, 255, "%s %s", curwd, Tcl_GetString(_new));
      if (!HBrill_RestrictMove || 
            Tcl_FindHashEntry(&LexiconHash, curwd) == NULL ||
            Tcl_FindHashEntry(&SeenTags, tempstr) != NULL) {
        if (strcmp(when, "SURROUNDTAG") == 0) {
          if (i < items && i > 0) {
            if (strcmp(lft,  Tcl_GetString(Tags[i - 1])) == 0 &&
                strcmp(rght, Tcl_GetString(Tags[i + 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXTTAG") == 0) {
          if (i < items) {
            if (strcmp(tag, Tcl_GetString(Tags[i + 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "CURWD") == 0) {
          if (strcmp(word, curwd) == 0) {
             CHANGE_TAG(_new, i);
          } 
        } else if (strcmp(when, "NEXTWD") == 0) {
          if (i < items) {
            if (strcmp(word, Tcl_GetString(Tokens[i + 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "RBIGRAM") == 0) {
          if (i < items) {
            if (strcmp(word, curwd) == 0 &&
                strcmp(next1, Tcl_GetString(Tokens[i + 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDNEXTTAG") == 0) {
          if (i < items) {
            if (strcmp(word,  curwd                     ) == 0 &&
                strcmp(next1, Tcl_GetString(Tags[i + 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDAND2AFT") == 0) {
          if (i < items-1) {
            if (strcmp(word,  curwd                       ) == 0 &&
                strcmp(next2, Tcl_GetString(Tokens[i + 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDAND2TAGAFT") == 0) {
          if (i < items-1) {
            if (strcmp(word,                       curwd) == 0 &&
                strcmp(next2, Tcl_GetString(Tags[i + 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT2TAG") == 0) {
          if (i < items - 1) {
            if (strcmp(tag, Tcl_GetString(Tags[i + 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT2WD") == 0) {
          if (i < items - 1) {
            if (strcmp(word, Tcl_GetString(Tokens[i + 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXTBIGRAM") == 0) {
          if (i < items - 1) {
            if (strcmp(next1, Tcl_GetString(Tags[i + 1])) == 0 &&
                strcmp(next2, Tcl_GetString(Tags[i + 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT1OR2TAG") == 0) {
          if (i < items) {
            if (i < items-1) tempi1 = i+2;
            else tempi1 = i+1;
            if (strcmp(tag, Tcl_GetString(Tags[i + 1]) ) == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT1OR2WD") == 0) {
          if (i < items) {
            if (i < items-1) tempi1 = i+2;
            else tempi1 = i+1;
            if (strcmp(word, Tcl_GetString(Tokens[i + 1])) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT1OR2OR3TAG") == 0) {
          if (i < items) {
            if (i < items -1) tempi1 = i+2;
            else tempi1 = i+1;
            if (i < items-2) tempi2 = i+3;
            else tempi2 =i+1;
            if (strcmp(tag, Tcl_GetString(Tags[i + 1]) ) == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi1])) == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "NEXT1OR2OR3WD") == 0) {
          if (i < items) {
            if (i < items -1) tempi1 = i+2;
            else tempi1 = i+1;
            if (i < items-2) tempi2 = i+3;
            else tempi2 =i+1;
            if (strcmp(word, Tcl_GetString(Tokens[i + 1]) ) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi1])) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREVTAG") == 0) {
          if (i > 0) {
            if (strcmp(tag, Tcl_GetString(Tags[i - 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREVWD") == 0) {
          if (i > 0) {
            if (strcmp(word, Tcl_GetString(Tokens[i - 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "LBIGRAM") == 0) {
          if (i > 0) {
            if (strcmp(word,  curwd                       ) == 0 &&
                strcmp(prev1, Tcl_GetString(Tokens[i - 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDPREVTAG") == 0) {
          if (i > 0) {
            if (strcmp(word,  curwd                     ) == 0 &&
                strcmp(prev1, Tcl_GetString(Tags[i - 1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDAND2BFR") == 0) {
          if (i > 1) {
            if (strcmp(word,  curwd                       ) == 0 &&
                strcmp(prev2, Tcl_GetString(Tokens[i - 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "WDAND2TAGBFR") == 0) {
          if (i > 1) {
            if (strcmp(word,  curwd                     ) == 0 &&
                strcmp(prev2, Tcl_GetString(Tags[i - 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV2TAG") == 0) {
          if (i > 1) {
            if (strcmp(tag, Tcl_GetString(Tags[i - 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV2WD") == 0) {
          if (i > 1) {
            if (strcmp(word, Tcl_GetString(Tokens[i - 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV1OR2TAG") == 0) {
          if (i > 0) {
            if (i > 1) tempi1 = i-2;
            else tempi1 = i-1;
            if (strcmp(tag, Tcl_GetString(Tags[i - 1]))  == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV1OR2WD") == 0) {
          if (i > 0) {
            if (i > 1) tempi1 = i-2;
            else tempi1 = i-1;
            if (strcmp(word, Tcl_GetString(Tokens[i - 1])) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi1])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV1OR2OR3TAG") == 0) {
          if (i > 0) {
            if (i>1) tempi1 = i-2;
            else tempi1 = i-1;
            if (i >2) tempi2 = i-3;
            else tempi2 = i-1;
            if (strcmp(tag, Tcl_GetString(Tags[i - 1]) ) == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi1])) == 0 ||
                strcmp(tag, Tcl_GetString(Tags[tempi2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREV1OR2OR3WD") == 0) {
          if (i > 0) {
            if (i>1) tempi1 = i-2;
            else tempi1 = i-1;
            if (i >2) tempi2 = i-3;
            else tempi2 = i-1;
            if (strcmp(word, Tcl_GetString(Tokens[i - 1]) ) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi1])) == 0 ||
                strcmp(word, Tcl_GetString(Tokens[tempi2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else if (strcmp(when, "PREVBIGRAM") == 0) {
          if (i > 1) {
            if (strcmp(prev2, Tcl_GetString(Tags[i - 1])) == 0 &&
                strcmp(prev1, Tcl_GetString(Tags[i - 2])) == 0) {
               CHANGE_TAG(_new, i);
            }
          }
        } else {
          CDM_ReportError("Error: "); CDM_AppendError(when);
          CDM_AppendError(" is not an allowable transform type");
        }
      }
    }
  }
  return TCL_OK;
} /* HBrill_ApplyContextualRules */

int HBrill_NotJustBlank(char *line) {
  while (*line != '\0') {
    if (*line != ' ' && *line != '\t' && *line != '\n') return 1;
    line++;
  }
  return 0;
} /* HBrill_NotJustBlank */

/*
 This is now obsolete, since we now use CDM_SplitString...
char *HBrillFixStr(char *line) {
  static char val[MAX_LINE_LENGTH], *str1, *str2;

  if (strchr(line,'{')!=NULL||strchr(line,'}')!= NULL||strchr(line,'\"')!=NULL){
    str1 = line; str2 = val;
    while (*str1 != '\0') {
      if (*str1 == '{' || *str1 == '}' || *str1 == '\"' || *str1 == '\\') {
        *str2 = '\\'; str2++; *str2 = *str1; str2++;
      } else {
        *str2 = *str1; str2++;
      }
      str1++;
    }
    *str2 = '\0';
  }
  return val;
}
*/

/* Function: creole_HBrill_Initialize
    Use this function in order to do some initialization. This function will
    be called just before creole_HBrill in the following situations:
     *) If the user has opened a whole Collection, this function will be 
        called just before the first document in Collection gets proccessed 
        with creole_HBrill
     *) If the user has opened a single Document, this function will be called
        just before calling creole_HBrill
    Please, refer to the Logos's Programming Manual for more 
    information on this function... */
int creole_HBrill_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /* Do some initilization here... */
  char *LexiconFile, *BigramsFile, *LexRuleFile, *ConRuleFile,
       *WordListFile = NULL, *arg;
  int i;
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  HBrill_StartStateAlgorithmCommand = NULL;
  
  if (objc < 7) {
    Tcl_WrongNumArgs(interp, 1, objv, "Document Collection"
        " Lexicon Bigrams LexicalRuleFile"
        " ContextualRuleFile ?-w WordList? ?-s SplitNumber? ?-i IntermedFile? "
        "?-S? ?-F? ?-l greek|english?");
    return TCL_ERROR;
  }

  /*
   * Handle Arguments...
   */
  LexiconFile = Tcl_GetString(objv[3]);
  if (Tcl_Access(LexiconFile, R_OK)) {
    CDM_ReportError("Unreadable Lexicon File: ");
    return CDM_AppendError(LexiconFile);
  }
  BigramsFile = Tcl_GetString(objv[4]);
  if (Tcl_Access(BigramsFile, R_OK)) {
    CDM_ReportError("Unreadable Bigrams File: ");
    return CDM_AppendError(BigramsFile);
  }
  LexRuleFile = Tcl_GetString(objv[5]);
  if (Tcl_Access(LexRuleFile, R_OK)) {
    CDM_ReportError("Unreadable Lexical Rule File: ");
    return CDM_AppendError(LexRuleFile);
  }
  ConRuleFile = Tcl_GetString(objv[6]);
  if (Tcl_Access(ConRuleFile, R_OK)) {
    CDM_ReportError("Unreadable Contextual Rule File: ");
    return CDM_AppendError(ConRuleFile);
  }

  /*
   * Handle optional arguments...
   */
  for (i=7; i<objc; i++) {
    arg = Tcl_GetString(objv[i]);
    /* printf("arg %d = %s\n", i, arg); */
    if (strcmp(arg, "-w") == 0) {
      /* Additional WordList */
      i++; /* This takes a value */
    } else if (strcmp(arg, "-s") == 0) {
      /* Process only first n lines... */
      i++; /* This takes a value */
    } else if (strcmp(arg, "-i") == 0) {
      /* Intermediate file */
      i++; /* This takes a value */
    } else if (strcmp(arg, "-S") == 0) {
      /* Start state only tagger... */
      StartStateTagger = 1;
    } else if (strcmp(arg, "-F") == 0) {
      /* ???... */
    } else if (strcmp(arg, "-l") == 0) {
      /* Language: one of greek, english */
      i++; /* This takes a value */
      arg = Tcl_GetString(objv[i]);
      if (strcmp(arg, "greek") == 0) {
        HBrill_StartStateAlgorithmLanguage = LANGUAGE_GREEK;
      } else if (strcmp(arg, "english") == 0) {
        HBrill_StartStateAlgorithmLanguage = LANGUAGE_ENGLISH;
      } else {
        HBrill_StartStateAlgorithmLanguage = LANGUAGE_TCL;
        HBrill_StartStateAlgorithmCommand  = objv[i];
        Tcl_IncrRefCount(HBrill_StartStateAlgorithmCommand);
      }
    }
  }

  /*
   * Read from disk all the required files (lexicons, rules, etc) and fill the
   * appropriate Hash tables...
   */
  if (HBrill_HashTablesInitialised) creole_HBrill_FreeHashTables();
  if (creole_HBrill_InitHashTables(LexiconFile, BigramsFile, LexRuleFile, 
        ConRuleFile, WordListFile) == TCL_ERROR) return TCL_ERROR;
  return TCL_OK;
} /* creole_HBrill_Initialize */


/*  Function: creole_HBrill_Finish
    Use this function in order to do some Clean-Up. This function will
    be called after all calls to creole_HBrill in the following 
    situations:
     *) If the user has opened a whole Collection, this function will be 
        called just after the last document in Collection gets proccessed with 
        creole_HBrill
     *) If the user has opened a single Document, this function will be called
        just after calling creole_HBrill
    Please, refer to the Logos's Programming Manual for more 
    information on this function... */
int creole_HBrill_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HBrill);
  /* Clean-Up... */
  creole_HBrill_FreeHashTables();
  if (HBrill_StartStateAlgorithmCommand) 
      Tcl_DecrRefCount(HBrill_StartStateAlgorithmCommand);
  HBrill_StartStateAlgorithmCommand = NULL;
  return TCL_OK;
} /* creole_HBrill_Finish */


/* ######## Ignore the Following Code ################################# */
/* This command registers the Module commands with the tcl interpreter.
   Modify with care...                                                  */
int Creole_hbrill_Init(Tcl_Interp *interp) {
#ifdef CDM_USE_STUBS 
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
    return TCL_ERROR;
  }
#endif /* CDM_USE_STUBS */

  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HBrill);

  /* Initialise global variables... */
  /* 
   * If HBrill_RestrictMove is set to 1, then a rule "change a tag from x to y"
   * will only apply to a word if:
   *     a) the word was not in the training set or
   *     b) the word was tagged with y at least once in the training set  
   * When training on a very small corpus, better performance might be 
   * obtained by setting this to 0, but for most uses it should be set to 1
   */
  HBrill_RestrictMove                = 1;
  HBrill_HashTablesInitialised       = 0;
  StartStateTagger                   = 0;
  LexicalRuleList                    = NULL;
  ContextualRuleList                 = NULL;
  HBrill_WordList                    = 0;
  HBrill_StartStateAlgorithmLanguage = LANGUAGE_GREEK;
  creole_HBrill_home                 = NULL;

  /* Link our location var with Tcl */
  CDM_LinkVar(interp, "creole_HBrill_home", &creole_HBrill_home);

  /* Register the Module Main Command */
  if (Tcl_CreateObjCommand(interp, "creole_HBrill", 
                           (Tcl_ObjCmdProc*) creole_HBrill,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;
  if( Tcl_CreateObjCommand(interp, "creole_HBrill_Initialize", 
                           (Tcl_ObjCmdProc*) creole_HBrill_Initialize,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if( Tcl_CreateObjCommand(interp, "creole_HBrill_Finish", 
                           (Tcl_ObjCmdProc*) creole_HBrill_Finish,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  /* Report Success... */
  return TCL_OK;
} /* Creole_hbrill_Init */

/* 
 * End of HBrill.c
 */
