##########################################################################
# 
#	HBrill.tcl - Saturday September 11 14:31:19 (EEST) 1999
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.0...
#
##########################################################################


# Location of this module (HBrill)
global creole_HBrill_home HBrill_tagger_home
set HBrill_tagger_home \
    [file join $creole_HBrill_home RULE_BASED_TAGGER_V1.14 Bin_and_Data]

## creole_HBrill
#  
proc creole_HBrill {doc args} {
  global creole_HBrill_home HBrill_tagger_home CDM_TempDir
  set current_dir [pwd]
  cd $creole_HBrill_home

  ## Under Ellogon, the window that desplays the "wait" message can
  ## additionally display a progress bar. In order to activate this feature,
  ## you must execute the command "ggi_wait_update document percent"
  ## In fact, if you enclose the command in a catch block, this module will
  ## also run under GATE without problems. The second argument to the 
  ## ggi_wait_update procedure is the percent, which is in the range [0,100]
  ## Setting percent a negative value causes the progress bar to disappear.
  catch {ggi_wait_update $doc 0}
  
  ## Create temporary file names...
  if {![info exists CDM_TempDir]} {set CDM_TempDir /tmp}
  set tmp_dump_fname [file join $CDM_TempDir HBrill_tmp[pid]_dump]
  set tmp_read_fname [file join $CDM_TempDir HBrill_tmp[pid]_read]
  set dev-null       [file join $CDM_TempDir HBrill_tmp[pid]_delete]

  ## The first four arguments are the needed files...
  foreach {lexicon_file bigrams_file lexrule_file context_file} $args {break}
  if {![file readable $lexicon_file]} {
    error "unreadable lexicon file parameter $lexicon_file: creole_HBrill"
  }
  if {![file readable $bigrams_file]} {
    error "unreadable bigrams file parameter $bigrams_file: creole_HBrill"
  }
  if {![file readable $lexrule_file]} {
    error "unreadable lexical rule file parameter $lexrule_file:\
           creole_HBrill"
  }
  if {![file readable $context_file]} {
    error "unreadable contextual rule file parameter $context_file:\
           creole_HBrill"
  }
  set other_args [lrange $args 4 end]
  foreach arg $other_args {
    set flag [string range [lindex $arg 0] 1 end]
    set value [lindex $arg 1]
    switch -exact -- $flag {
      w {
        if {![file readable $value]} {
          error "unreadable wordlist file parameter $value: creole_HBrill"}
      }
      i {
        if {[file exists $value]} {
          if {![file writable $value]} {
            error "unwritable intermediate file parameter $value:\
                   creole_HBrill"
          }
        } else {
          if {![file writable [file dirname $value]]} {
            error "unwritable directory for intermediate file parameter\
                   $value: creole_HBrill"
          }
        }
      }
      s {
        if {$value < 100} {
          error "too small value for process lines parameter $value:\
                 creole_HBrill"}
      }
      S {}
      default {error "bad option parameter $flag: creole_HBrill"}
    }
  }

  # open file to use as input to external process
  if {[catch {open $tmp_dump_fname w} tmp_dump]} {
      error "Cannot open $tmp_dump_fname: $tmp_dump"
  }
  # write out required info
  creole_dump_HBrill_file $tmp_dump $doc
  close $tmp_dump

  # call the external process

  # (HBrill needs to run in its own directory)
  set current_dir [pwd]
  cd $HBrill_tagger_home
  # (and needs its directory in the path as well, so use a subshell)
  if {[catch {
    eval exec tagger [list [file nativename $lexicon_file] \
                [file nativename $tmp_dump_fname] \
		[file nativename $bigrams_file] \
                [file nativename $lexrule_file] \
		[file nativename $context_file]] \
		[join $other_args] > $tmp_read_fname 2> ${dev-null}} error]} {
    # (HBrill's exit status is garbage so ignore it, but report others)
    global errorCode
    catch [file delete -force ${dev-null}]
    if {[lindex $errorCode 0] != "CHILDSTATUS"} {
      error "HBrill exited abnormally: $error"
    }
  }
  cd $current_dir
  catch [file delete -force ${dev-null}]

  # open the external process's output file
  if {[catch {open $tmp_read_fname r} tmp_read]} {
    error ">>Cannot open $tmp_read_fname: $tmp_read"
  }
  # parse and add info to the database
  if {[catch {creole_read_HBrill_file $tmp_read $doc} error]} {
    close $tmp_read
    error $error
  }
  close $tmp_read
  catch {ggi_wait_update $doc 100}

  # delete tmp files
  catch [file delete -force $tmp_dump_fname]
  catch [file delete -force $tmp_read_fname]

  # record the fact that we ran and exit normally
  cd $current_dir
  tip_PutAttribute $doc [tip_CreateAttribute HBrill \
                             [tip_CreateAttributeValue GDM_STRING {}]]
  return
} ;# creole_HBrill


# creole_dump_HBrill_file
#
proc creole_dump_HBrill_file { dump_file doc } {

  # get text string
  set ByteSequence [tip_GetByteSequence $doc]

  # find sentences
  set SentenceAnns [tip_SelectAnnotations $doc sentence {}]
  set items [llength $SentenceAnns]
  if {!$items} {
    error "creole_HBrill: No Sentence Annotations Found!"
  }
  set step [expr {33.0/$items}]
  set progress $step

  foreach sentence $SentenceAnns {
    catch {ggi_wait_update $doc $progress}
    set progress [expr {$progress + $step}]
    set constituents [tip_GetValueValue \
                          [tip_GetAttribute $sentence constituents]]
    foreach token_id $constituents {
      set token [tip_GetAnnotation $doc $token_id]
      set span  [lindex [tip_GetSpans $token] 0]
      set start [tip_GetStart $span]
      set end   [tip_GetEnd   $span]
      set text  [string range $ByteSequence $start [expr {$end - 1}]]
      puts -nonewline $dump_file "$text "
    }
    puts $dump_file {};#newline
  }
} ;# creole_dump_HBrill_file


# creole_read_HBrill_file
#
proc creole_read_HBrill_file {read_file doc} {

  # find sentences
  set SentenceAnns [tip_SelectAnnotations $doc sentence {}]
  set items [llength $SentenceAnns]
  if {!$items} {
    error "creole_HBrill: No Sentence Annotations Found!"
  }
  set step     [expr {33.0/$items}]
  set progress [expr {66.0 + $step}]

  foreach sentence $SentenceAnns {
    catch {ggi_wait_update $doc $progress}
    set progress [expr {$progress + $step}]
    set constituents [tip_GetValueValue \
                          [tip_GetAttribute $sentence constituents]]
    set line [gets $read_file]
    if {[eof $read_file]} {break}
    set line [split $line]
    set TaggedTokens {}
    foreach token $line {
      if {[string length $token]} {lappend TaggedTokens $token}
    }
    foreach id $constituents TaggedToken $TaggedTokens {
      set token [tip_GetAnnotation $doc $id]
      # get pos tag from tagger output
      if {[regexp {.+/([^/]+)} $TaggedToken match tag]} {
        # create a new attribute
        set attribute [tip_CreateAttribute pos \
                           [tip_CreateAttributeValue GDM_STRING $tag]]
        # add it to the current token annotation
        set token [tip_PutAttribute $token $attribute]
        # add the annotation back to the doc
        tip_AddAnnotation $doc $token
      }
    }
  }
} ;# creole_read_HBrill_file

## Procedure: creole_HBrill_Initialize
#   Use this function in order to do some initialization. This function will
#   be called just before creole_HBrill in the following situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just before the first document in Collection gets proccessed 
#       with creole_HBrill
#    *) If the user has opened a single Document, this function will be called
#       just before calling creole_HBrill
#   Please, refer to the Ellogon's Programming Manual for more 
#   information on this function...
proc creole_HBrill_Initialize { col doc args } {
  global creole_HBrill_home
  ## Do some initilization here...
} ;# creole_HBrill_Initialize

## Procedure: creole_HBrill_Finish
#   Use this function in order to do some Clean-Up. This function will
#   be called after all calls to creole_HBrill in the following 
#   situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just after the last document in Collection gets proccessed with 
#       creole_HBrill
#    *) If the user has opened a single Document, this function will be called
#       just after calling creole_HBrill
#   Please, refer to the Ellogon's Programming Manual for more 
#   information on this function...
proc creole_HBrill_Finish { col doc args } {
  global creole_HBrill_home
  ## Clean-Up...
} ;# creole_HBrill_Finish

#
# End of File
#

