#!/bin/sh
# \
  exec tclsh "$0" "$@"

set Root [lindex $argv 0]
set dirs [glob Results_$Root*]
set files {}
foreach dir $dirs {
  if {[file isdir $dir] && [file exists [file join $dir TrainResults]]} {
    lappend files [file join $dir TrainResults]
  }
}

set sizes {}
if {$files != {}} {
  foreach Dir $files {
    set dir [file dirname  $Dir]; set size 0
    regexp -- "_${Root}(\[0-9\]+)" $dir tmp size
    foreach subdir [lsort [glob [file join $dir Exper_*]]] {
      set ContextualSize($size:$subdir) \
        [lindex [exec wc [file join $subdir ContextualRules]] 0]
      set LexicalSize($size:$subdir) \
        [lindex [exec wc [file join $subdir LexicalRules]] 0]
    }
    lappend sizes $size
  }
}
## Get word cound...
if {$files != {}} {
  set data [eval exec grep -- words  $files]
  set data [split $data "\n"]
  foreach line $data {
    set list [split $line ", "]
    set ind [expr [llength $list] -2]
    set words [lindex $list $ind]
    incr ind -5
    set file [lindex $list $ind]
    regexp "${Root}(\[0-9\]+)" $file tmp size
    set Words($size) $words
  }
}

foreach size [lsort -integer $sizes] {
  set Con_size 0.0
  set Lex_size 0.0
  set i 0
  foreach one [array names ContextualSize $size:*] {
    incr i
    set Con_size [expr $Con_size + double($ContextualSize($one))]
    set Lex_size [expr $Lex_size + double($LexicalSize($one))]
  }
  set N($size) $i
  set Con_Size($size) [expr $Con_size/double($i)]
  set Lex_Size($size) [expr $Lex_size/double($i)]
}
## Standard Deviation
foreach size [lsort -integer $sizes] {
  set con_tmp 0.0; set lex_tmp 0.0
  set Con_mean $Con_Size($size); set Lex_mean $Lex_Size($size)
  foreach one [array names ContextualSize $size:*] {
    set con_tmp [expr $con_tmp + \
        pow( (double($ContextualSize($one)) - $Con_mean ) , 2)]
    set lex_tmp [expr $lex_tmp + \
        pow( (double($LexicalSize($one)) - $Lex_mean ) , 2)]
  }
  set Con_Deviation($size) [expr sqrt($con_tmp/double($N($size) - 1))]
  set Lex_Deviation($size) [expr sqrt($lex_tmp/double($N($size) - 1))]
  puts "$Words($size) $Con_Size($size) $Con_Deviation($size) $Lex_Size($size) $Lex_Deviation($size)"
}
