#!/bin/sh
# Restart using tclsh \
  exec wish "$0" "$@"
catch {encoding system iso8859-7}

proc CreateLogWin {} {
  global argv Text Button Log
  wm withdraw .
  set win .lgwin
  
  toplevel $win
  wm title $win [lindex $argv 0]
  wm geometry $win +0+24
  wm protocol $win WM_DELETE_WINDOW {puts {Window stays!}}
  frame $win.f
  text ${win}.f.text -bg white -fg navy -width 80 -height 20 -wrap word \
       -yscrollcommand "$win.f.sc set"
  scrollbar $win.f.sc -orient vertical -bd 1 -command [list $win.f.text yview]
  button $win.done -text Done -command {exit 0} -state disabled
  pack $win.f.text -side left -fill both -expand 1
  pack $win.f.sc -side right -fill y
  pack $win.f -side top -fill both -expand 1
  pack $win.done -side top
  set Text $win.f.text
  set Button $win.done
  $Text configure -state disabled
  update
  
  set win .make_win
  toplevel $win
  wm title $win "([lindex $argv 0]) Train Output..."
  wm geometry $win -1-46
  wm protocol $win WM_DELETE_WINDOW {puts {Window Stays!}}
  frame $win.f
  text ${win}.f.text -bg white -fg navy -width 80 -height 20 -wrap word \
       -yscrollcommand "$win.f.sc set"
  scrollbar $win.f.sc -orient vertical -bd 1 -command [list $win.f.text yview]
  button $win.done -text Done -command {exit 0} -state disabled
  pack $win.f.text -side left -fill both -expand 1
  pack $win.f.sc -side right -fill y
  pack $win.f -side top -fill both -expand 1
  set Log $win.f.text
  $Log configure -state disabled
  update
}

proc message message {
  global Text
  $Text configure -state normal
  $Text insert end $message
  $Text see end
  $Text configure -state disabled
  update
}
proc Log message {
  global Log LogContents
  lappend LogContents $message
  $Log configure -state normal
  $Log delete 1.0 end
  $Log insert end [join $LogContents {}]
  $Log see end
  $Log configure -state disabled
  set end [llength $LogContents]
  set start [expr $end - 500]
  if {$start > 0} {set LogContents [lrange $LogContents $start end]}
}

if {$argc < 1} {
  puts stderr "$argv0: Usage $argv0 TaggedCorpus"
  exit 1
}

set TaggedCorpus [lindex $argv 0]
if {![file exists $TaggedCorpus]} {
  puts stderr "$argv0: Tagged Corpus Filename $TaggedCorpus does not exist!"
  exit 2
}

CreateLogWin

## Get Tagged Corpus line number
set result [list 0 0 0 $TaggedCorpus]
catch {set result [exec wc $TaggedCorpus]}
set lines [lindex $result 0]
set words [lindex $result 1]
message "+ Train File: $TaggedCorpus, $lines lines, $words words.\n"
message "+ Creating Results Directory: Results_${TaggedCorpus}...\n"
file mkdir Results_${TaggedCorpus}

message "+ Creating Data Files for Ten fold cross validation...\n"
set Input [open ${TaggedCorpus} r]

for {set i 1} {$i <= 10} {incr i} {
  set File_$i [open ${TaggedCorpus}_$i w]
}

set i 1
while {1} {
  set line [gets $Input]
  if {$line == {}} break
  puts [set File_$i] $line
  incr i
  if {$i > 10} {set i 1}
}
close $Input
for {set i 1} {$i <= 10} {incr i} {
  close [set File_$i]
  file copy -force ${TaggedCorpus}_$i Results_${TaggedCorpus}
}

message "+ Cleaning Up...\n"
exec make clean
message "+ Done.\n\n"

message "+ Starting Training...\n"
set After {}
for {set ind 1} {$ind <= 10} {incr ind} {
  message "Phase # $ind: \n"
  set command {}
  lappend command exec cat
  for {set i 1} {$i <= 10} {incr i} {
    if {$i != $ind} {lappend command ${TaggedCorpus}_$i}
  }
  lappend command > TText
  file delete -force TText
  eval $command
  catch {set lines [lindex  [exec wc $TaggedCorpus] 1]}
  set Pipe [open "|make" r]
  fconfigure $Pipe -blocking 0
  fileevent $Pipe readable {
    if {[eof $Pipe]} {
      catch {close $Pipe}
      ##puts "Phase $ind Copleted!"
      unset Pipe
    } else {
      foreach el [split [read $Pipe] "\n"] {
        Log "$el\n"
      }
      update
    }
  }
  tkwait variable Pipe
  set error_line -1
  set SearchData $LogContents
  while {[lsearch -regexp $SearchData "NUMBER OF ERRORS:.*"] != -1} {
    set error_line [lsearch -regexp $SearchData "NUMBER OF ERRORS:.*"]
    set error_text [lindex $SearchData $error_line]
    set SearchData [lrange $SearchData [expr $error_line +1] end]
  }
  if {$error_line == -1} {
    message "Warning: Train Stopped????\n"
  } else {
    set error [expr double([lindex $error_text end])]
    set error [expr ($lines - $error)/$lines]
    message "  ++Training Accuracy: $error, $error_text"
  }
  unset SearchData
  set TestFile ${TaggedCorpus}_$ind
  ## Run Tagger for Evaluation...
  message "  ++Evaluating Trained Tagger...\n"
  exec cat $TestFile | Utilities/tagged-to-untagged.prl > tmp_[pid]
  if {[catch {exec tagger FinalLexicon tmp_[pid] Bigrams LexicalRules \
             ContextualRules > [file join Results_${TaggedCorpus} Eval.txt]}]} {
    ##message "  ++ ERROR: Cannot run tagger!"
  }
  catch {file delete -force tmp_[pid]}
  ## Compute Deferences...
  set file [open $TestFile r]
  set Correct {}
  foreach word [split [read $file] "\ \n\t"] {
    if {$word != {}} {lappend Correct $word}
  }
  close $file
  set file [open [file join Results_${TaggedCorpus} Eval.txt] r]
  set Eval {}
  foreach word [split [read $file] "\ \n\t"] {
    if {$word != {}} {lappend Eval $word}
  }
  close $file
  set errors 0; set how_many [llength $Correct]
  for {set index 0} {$index < $how_many} {incr index} {
    set C [lindex $Correct $index]
    set E [lindex $Eval    $index]
    if {[string compare $C $E] != 0} {incr errors;Log "-> $C != $E\n";update}
  }
  unset Eval Correct
  set error [expr ($how_many - $errors)/double($how_many)]
  message "  --Accuracy: $error ($errors errors/$how_many total)\n"
  set dir [file join Results_${TaggedCorpus} Exper_$ind]
  message "+   Saving Results in $dir...\n"
  file mkdir $dir
  file copy -force Bigrams LexicalRules FinalLexicon ContextualRules $dir
}

message "+ Saving Results...\n"
file copy -force Bigrams LexicalRules FinalLexicon ContextualRules \
     Results_${TaggedCorpus}
set Results [open [file join Results_${TaggedCorpus} TrainResults] w]
message "+ Done!\n"
message "+   --- Training Completed! ---\n"
puts $Results [$Text get 1.0 end]
close $Results
exec make clean
for {set i 1} {$i <= 10} {incr i} {
  file delete -force ${TaggedCorpus}_$i
}


$Button configure -state normal
after 60000 {exit 0}
