/***************************************************************************
*
*       HGazetteer.c - Friday October 08 23:18:20 (EEST) 1999
*       This is a dynamic/tight coupled (C) Module for use with 
*       Ellogon, version 1.0...
*
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "CDM.h"

/* Prototype Definitions */
extern "C" CDM_EXPORT int Creole_hgazetteer_Init(Tcl_Interp *interp);
int creole_HGazetteer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HGazetteer_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HGazetteer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);

CDM_BEGIN_GLOBAL_VARIABLE_BLOCK(HGazetteer)
/* 
 * Module's Home Path: Its value will be updated when the module will be
 * loaded... 
 */
CDM_DEFINE_GLOBAL(char *,       creole_HGazetteer_home);
CDM_DEFINE_GLOBAL(CDM_Document, HGazetteer_GlobalDoc);
CDM_END_GLOBAL_VARIABLE_BLOCK(HGazetteer)
#define creole_HGazetteer_home  (tsdPtr->creole_HGazetteer_home)
#define HGazetteer_GlobalDoc    (tsdPtr->HGazetteer_GlobalDoc)

extern int HGazetteerStartlex(char *);

/* Function: creole_HGazetteer */
int creole_HGazetteer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  CDM_Document Doc;
  CDM_AnnotationSet Sentences = NULL;
  CDM_Annotation Ann;
  CDM_Attribute Constituents, Stem;
  CDM_AttributeValue Value, StemVal;
  CDM_Span Span;
  Tcl_Obj **Sentence, **Id;
  Tcl_DString ds;
  Tcl_Encoding enc = NULL;
  char *Text = NULL, *ptr, line[100], *TokenAttribute;
  long ann_id, start, end;
  double progress, step;
  int status, sentences, sent_id, id, ids, last_progress;
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HGazetteer);

  CDM_SetActiveInterpreter(interp); /* Do not remove this command... */
  if (objc < 3) {
    Tcl_WrongNumArgs(interp, 1, objv, "Document token-attribute ?to-lower?");
    return TCL_ERROR;
  }
  Doc = CDM_GetDocumentPointer(objv[1]);
  if (Doc == NULL) 
    return CDM_ReportError("creole_HGazetteer: Invalid Document Pointer");
  TokenAttribute = Tcl_GetString(objv[2]);
  /* Get an Encoding object. This is done for efficiency reasons, so as Tcl
   * not to allocate and free this encoding each time we use it... */
  enc = Tcl_GetEncoding(interp, "iso8859-7");
  if (enc == NULL) goto error;
  
  CDM_SetProcessStatus(Doc, 0);
  HGazetteer_GlobalDoc = Doc;
  
  /* Get Document Text... */
  if (objc > 3) {
    if (strcmp(Tcl_GetString(objv[3]), "ToLower") == 0) {
      ptr = CDM_Strdup(CDM_GetByteSequence(Doc));
      if (ptr == NULL) goto error;
      Tcl_UtfToLower(ptr);
      Text = CDM_UtfToExternal("iso8859-7", ptr);
      CDM_Free(ptr);
    }
  }
  if (Text == NULL) Text = CDM_GetByteSequence(Doc, "iso8859-7");
  
  Sentences = CDM_SelectAnnotations(Doc, "sentence");

  /* Foreach Sentence Annotation */
  Tcl_ListObjGetElements(interp, Sentences, &sentences, &Sentence);
  if (sentences < 1) {
    CDM_ReportError("No Sentence Annotations!");
    goto error;
  }

  progress = 0.0;
  last_progress = 0;
  step = 100.0/float(sentences);
  for (sent_id=0; sent_id<sentences; sent_id++) {
    progress += step;
    if (int(progress) - last_progress == 10) {
      last_progress = int(progress);
      CDM_SetProcessStatus(Doc, last_progress);
    }
    Tcl_DStringInit(&ds);
    /* Get Sentence Constituents... */
    Constituents = CDM_GetAttribute(Sentence[sent_id], "constituents");
    Value = CDM_GetValueValue(Constituents);
    /* Foreach ID... */
    Tcl_ListObjGetElements(interp, Value, &ids, &Id);
    for (id=0; id<ids; id++) {
      status = Tcl_GetLongFromObj(interp, Id[id], &ann_id);
      if (status != TCL_OK) {
        CDM_ReportError("Invalid \"constituents\" attribute!");
        goto error;
      }
      Ann = CDM_GetAnnotation(Doc, ann_id);
      if (Ann == NULL) {
        CDM_ReportError("No Annotation ID as in \"constituents\" attribute!");
        goto error;
      }
      /* Get the first span range of the "token" annotation... */
      Span  = CDM_Nth(CDM_GetSpans(Ann), 0);
      start = CDM_GetStart(Span);
      end   = CDM_GetEnd(Span);
      
      /* Does the Annotation has a "stem" (or the one given from arguments)
         attribute? */
      Stem = CDM_GetAttribute(Ann, TokenAttribute);
      if (Stem != NULL) {
        StemVal = Value = CDM_GetValueValue(Stem);
        ptr = Tcl_GetString(StemVal);
        if (strlen(ptr) == 0) {
          /* In case the attribute does not have a value, use token string */
          ptr = (Text+start);
          if (CDM_Isdigit(*ptr)) Tcl_DStringAppend(&ds, " ", -1);
          Tcl_DStringAppend(&ds, ptr, end - start);
        } else {
          if (CDM_Isdigit(*ptr)) Tcl_DStringAppend(&ds, " ", -1);
          ptr = CDM_UtfToExternal("iso8859-7", ptr);
          Tcl_DStringAppend(&ds, ptr, -1);
          CDM_Free(ptr);
        }
      } else {
        ptr = (Text+start);
        if (CDM_Isdigit(*ptr)) Tcl_DStringAppend(&ds, " ", -1);
        Tcl_DStringAppend(&ds, ptr, end - start);
      }
      sprintf(line, ",%ld+%ld>", start, end);
      Tcl_DStringAppend(&ds, line, -1);
    }
  
    /* Call Lexer! */
    HGazetteerStartlex(Tcl_DStringValue(&ds));
    Tcl_DStringFree(&ds);
  }

  /* Report Succesful Completion */
  Tcl_Free(Text);
  Tcl_DecrRefCount(Sentences);
  Tcl_FreeEncoding(enc);
  CDM_SetProcessStatus(Doc, 100);
  CDM_PutAttribute(Doc, CDM_CreateAttribute("HGazetteer",
                        CDM_CreateAttributeValue(CDM_STRING, "")));
  return TCL_OK;
error:
  if (Text) Tcl_Free(Text);
  if (Sentences) Tcl_DecrRefCount(Sentences);
  if (enc) Tcl_FreeEncoding(enc);
  return TCL_ERROR;
} /* creole_HGazetteer */

/*
 * Process Matched Text...
 */
void HGazetteer_ProcessMatch(unsigned char *text, const char *type,
                                                  const char *subtype) {
  CDM_AttributeSet Attrs;
  CDM_Attribute    Attr;
  CDM_SpanSet SpanSet;
  CDM_Annotation Ann;
  unsigned char *ptr = text, *last_plus = NULL;
  long start, end;
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HGazetteer);

  /*printf("HGaz: \"%s\" -> (%s,%s)\n", text, type, subtype);*/
  /* Get Annotation Start... */
  ptr = (unsigned char *) strstr((const char *) text, ",");
  if (ptr == NULL) return;
  sscanf((const char *) ++ptr, "%ld+", &start);
  
  /* Get Annotation End */
  while (*ptr != '\0') {
    if (*ptr == '+') last_plus = ++ptr;
    ptr++;
  }
  /* sscanf(last_plus, "%ld>", &end); */
  sscanf((const char *) last_plus, "%ld", &end);

  /* Create "lookup" Annotation... */
  Attrs = CDM_CreateAttributeSet("type", 
          CDM_CreateAttributeValue(CDM_STRING, type));
  Attr = CDM_CreateAttribute("tag", 
          CDM_CreateAttributeValue(CDM_STRING, subtype));
  Tcl_ListObjAppendElement(NULL, Attrs, Attr);
  SpanSet = CDM_CreateSpanSet(start, end);
  Ann = CDM_CreateAnnotation("lookup", SpanSet, Attrs);
  if (Ann != NULL) CDM_AddAnnotation(HGazetteer_GlobalDoc, Ann);
}

/* Function: creole_HGazetteer_Initialize
    Use this function in order to do some initialization. This function will
    be called just before creole_HGazetteer in the following situations:
     *) If the user has opened a whole Collection, this function will be 
        called just before the first document in Collection gets proccessed 
        with creole_HGazetteer
     *) If the user has opened a single Document, this function will be called
        just before calling creole_HGazetteer
    Please, refer to the Logos's Programming Manual for more 
    information on this function... */
int creole_HGazetteer_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /* Do some initilization here... */
  return TCL_OK;
} /* creole_HGazetteer_Initialize */


/*  Function: creole_HGazetteer_Finish
    Use this function in order to do some Clean-Up. This function will
    be called after all calls to creole_HGazetteer in the following 
    situations:
     *) If the user has opened a whole Collection, this function will be 
        called just after the last document in Collection gets proccessed with 
        creole_HGazetteer
     *) If the user has opened a single Document, this function will be called
        just after calling creole_HGazetteer
    Please, refer to the Logos's Programming Manual for more 
    information on this function... */
int creole_HGazetteer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /* Clean-Up... */
  return TCL_OK;
} /* creole_HGazetteer_Finish */



/* ######## Ignore the Following Code ################################# */

/* This command registers the Module commands with the tcl interpreter.
   Modify with care...                                                  */
int Creole_hgazetteer_Init(Tcl_Interp *interp) {
#ifdef CDM_USE_STUBS 
  if (Tcl_InitStubs(interp, "8.2", 0) == NULL) {
    return TCL_ERROR;
  }
#endif /* CDM_USE_STUBS */

  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HGazetteer);

  /* Link our location var with Tcl */
  CDM_LinkVar(interp, "creole_HGazetteer_home",
                      &creole_HGazetteer_home);

  /* Register the Module Main Command */
  if( Tcl_CreateObjCommand(interp, "creole_HGazetteer", 
                           (Tcl_ObjCmdProc*) creole_HGazetteer,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if( Tcl_CreateObjCommand(interp, "creole_HGazetteer_Initialize", 
                           (Tcl_ObjCmdProc*) creole_HGazetteer_Initialize,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if( Tcl_CreateObjCommand(interp, "creole_HGazetteer_Finish", 
                           (Tcl_ObjCmdProc*) creole_HGazetteer_Finish,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  /* Report Success... */
  return TCL_OK;
} /* Creole_hgazetteer_Init */
/* 
 * End of HGazetteer.c 
 */
