%{
 /*  This is a simple tokenizer & sentence splitter for the Greek language.
  *  Author: George Petasis, petasis@iit.demokritos.gr
  *         15/02/1999
  *  Updated: 27 November 2006:
  *    Updated to flex 2.5.4 (Fedora Core 5)
  */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __WIN32__
#include <io.h>
#define snprintf _snprintf
#define isatty   _isatty
#endif /* __WIN32__ */


#define GREEK_WORD                 "GW"
#define GREEK_LOWERCASE_WORD       "GLW"
#define GREEK_UPERCASE_WORD        "GUW"
#define GREEK_FIRSTCAP_WORD        "GFW"
#define ENGLISH_WORD               "EW"
#define ENGLISH_LOWERCASE_WORD     "ELW"
#define ENGLISH_UPERCASE_WORD      "EUW"
#define ENGLISH_FIRSTCAP_WORD      "EFW"
#define MIXED_WORD                 "MW"
#define MIXED_LOWERCASE_WORD       "MLW"
#define MIXED_UPERCASE_WORD        "MUW"
#define MIXED_FIRSTCAP_WORD        "MFW"

#define TIME                       "TIME"
#define DATE                       "DATE"
#define NUMBER                     "NUMBER"
#define SYMBOL                     "SYMBOL"
#define PUNCTUATION                "PUNC"
#define ABBREVIATION               "ABBREV"

#define GREEK_LOWERCASE_LETTER     "GLW"
#define GREEK_UPERCASE_LETTER      "GUW"
#define ENGLISH_LOWERCASE_LETTER   "ELW"
#define ENGLISH_UPERCASE_LETTER    "EUW"

#define APOSTROPH                  "APOSTR"
#define LPAR                       "LPAR"
#define RPAR                       "RPAR"

#define SPECIAL_WORD               "SPECIAL"
#define UNKNOWN                    "UNKNOWN"

#define HTML                       "HTML"
#define HTML_DOCTYPE               "HTML_DOCTYPE"
#define HTML_SCRIPT                "HTML_SCRIPT"
#define HTML_COMMENT               "HTML_COMMENT"
#define HTML_STYLE                 "HTML_STYLE"
#define HTML_ENTITY                "HTML_ENTITY"
#define HTML_EMAIL                 "EMAIL"
#define HTML_URL                   "URL"

#ifndef HT_STATIC
#define HT_STATIC static

#define HANDLE_HTML_ENTITIES if (HtmlFile) {\
                 htok_ptr = (YY_CHAR *) strchr((const char *)yytext, '&'); \
                 if (htok_ptr != NULL) yyless(int(htok_ptr-(YY_CHAR *)yytext));}
#endif /* HT_STATIC */

#ifndef YY_CHAR
#define YY_CHAR unsigned char
#endif /* YY_CHAR */

/*
 * The following code is for building a Dynamic module...
 */
#ifdef DYNAMIC_MODULE
#include "CDM.h"
/* Prototype Definitions */
extern "C" CDM_EXPORT int Creole_htokenizer_Init(Tcl_Interp *interp);
int creole_HTokenizer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HTokenizer_Init(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_HTokenizer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
#endif /* DYNAMIC_MODULE */

HT_STATIC int  HTok_add_token(const char *type);
HT_STATIC int  HTok_AddSentenceAnnotation(void);
HT_STATIC void HTok_StartNewSentence(void);
HT_STATIC void HTok_CloseCurrentSentence(void);

CDM_BEGIN_GLOBAL_VARIABLE_BLOCK(HTokenizer)
CDM_DEFINE_GLOBAL(char *,       creole_HTokenizer_home);
CDM_DEFINE_GLOBAL(int,          start);
CDM_DEFINE_GLOBAL(int,          end);
CDM_DEFINE_GLOBAL(int,          char_count);
CDM_DEFINE_GLOBAL(int,          sentence_start);
CDM_DEFINE_GLOBAL(int,          sentence_end);
CDM_DEFINE_GLOBAL(int,          first_token);
CDM_DEFINE_GLOBAL(int,          sentence_completted);
CDM_DEFINE_GLOBAL(int,          HtmlFile);
CDM_DEFINE_GLOBAL(int,          InsideOtherToken);
CDM_DEFINE_GLOBAL(int,          ComputeStart);
CDM_DEFINE_GLOBAL(int,          EmmitSentences);
CDM_DEFINE_GLOBAL(YY_CHAR *,    htok_ptr);
CDM_DEFINE_GLOBAL(CDM_Document, HTokenizer_Doc);
CDM_DEFINE_GLOBAL(Tcl_Obj *,    HTokenizer_Constituents);
CDM_DEFINE_GLOBAL(Tcl_Obj *,    HTokenizer_HTMLConstituents);
CDM_DEFINE_GLOBAL(int ,         HTokenizer_StringSize);
CDM_DEFINE_GLOBAL(int ,         HTokenizer_LastPercent);
#ifndef DYNAMIC_MODULE
CDM_DEFINE_GLOBAL(FILE *,       OutputFile);
CDM_DEFINE_GLOBAL(FILE *,       InputFile);
CDM_DEFINE_GLOBAL(int,          debug);
#endif
CDM_END_GLOBAL_VARIABLE_BLOCK(HTokenizer)
/*
 * Make global variables available everywhere, without using
 * CDM_GLOBAL(var)...
 */
#define start                       (tsdPtr->start)
#define end                         (tsdPtr->end)
#define char_count                  (tsdPtr->char_count)
#define sentence_start              (tsdPtr->sentence_start)
#define sentence_end                (tsdPtr->sentence_end)
#define first_token                 (tsdPtr->first_token)
#define sentence_completted         (tsdPtr->sentence_completted)
#define HtmlFile                    (tsdPtr->HtmlFile)
#define InsideOtherToken            (tsdPtr->InsideOtherToken)
#define ComputeStart                (tsdPtr->ComputeStart)
#define EmmitSentences              (tsdPtr->EmmitSentences)
#define htok_ptr                    (tsdPtr->htok_ptr)
#define OutputFile                  (tsdPtr->OutputFile)
#define InputFile                   (tsdPtr->InputFile)
#define debug                       (tsdPtr->debug)
#define creole_HTokenizer_home      (tsdPtr->creole_HTokenizer_home)
#define HTokenizer_Doc              (tsdPtr->HTokenizer_Doc)
#define HTokenizer_Constituents     (tsdPtr->HTokenizer_Constituents)
#define HTokenizer_HTMLConstituents (tsdPtr->HTokenizer_HTMLConstituents)
#define HTokenizer_StringSize       (tsdPtr->HTokenizer_StringSize)
#define HTokenizer_LastPercent      (tsdPtr->HTokenizer_LastPercent)
%}

REAL_NUMBER            [-+]?[0-9]*[.,]?[0-9]+([eE][-+]?[0-9]+)?
INT_NUMBER             [0-9]+
NUMBER                 {INT_NUMBER}
AMOUNT                 ({INT_NUMBER}[,.])+{INT_NUMBER}
                         
ENGLISH_LOWER          [a-z]
ENGLISH_CAPITAL        [A-Z]
ENGLISH_LCONSONANT     [bcdfghjklmnpqrstvwxz]
ENGLISH_UCONSONANT     [BCDFGHJKLMNPQRSTVWXZ]
ENGLISH_LOWERS         {ENGLISH_LOWER}+
ENGLISH_CAPITALS       {ENGLISH_CAPITAL}+
ENGLISH_CONSONANT      {ENGLISH_LCONSONANT}|{ENGLISH_UCONSONANT}

GREEK_LATONA           []
GREEK_UATONA           []
GREEK_LTONOUMENA       []
GREEK_UTONOUMENA       [?]
GREEK_LCONSONANT       []
GREEK_UCONSONANT       []
GREEK_LOWER            {GREEK_LTONOUMENA}|{GREEK_LATONA}
GREEK_CAPITAL          {GREEK_UTONOUMENA}|{GREEK_UATONA}
GREEK_LOWERS           {GREEK_LOWER}+
GREEK_CAPITALS         {GREEK_CAPITAL}+
GREEK_CONSONANT        {GREEK_LCONSONANT}|{GREEK_UCONSONANT}


PERIOD                 [.]
THREEDOTS              [.]{3}
COMMA                  [\,]
PUNCTUATION            [\.\!\:\?\;]
SYMBOL                 [-\@\#\$\%\&\*\+\=\<\>\|\~\\\/]
APOSTROPH              [\"\'\`]
LPAR                   [\[\{\(\]
RPAR                   [\]\}\)\]
CONNECTIVES            [-\/\&"'"]
CAPITAL                {GREEK_CAPITAL}|{ENGLISH_CAPITAL}
CAPITALS               {CAPITAL}+
LOWER                  {GREEK_LOWER}|{ENGLISH_LOWER}
LOWERS                 {LOWER}+

ENGLISH_CHAR           {ENGLISH_LOWER}|{ENGLISH_CAPITAL}
ENGLISH_CHARS          {ENGLISH_CHAR}+
GREEK_CHAR             {GREEK_LOWER}|{GREEK_CAPITAL}
GREEK_CHARS            {GREEK_CHAR}+
CHAR                   {LOWER}|{CAPITAL}
CHARS                  {CHAR}+
LCONSONANT             {GREEK_LCONSONANT}|{ENGLISH_LCONSONANT}
LCONSONANTS            {GREEK_LCONSONANT}|{ENGLISH_LCONSONANT}+
UCONSONANT             {GREEK_UCONSONANT}|{ENGLISH_UCONSONANT}
UCONSONANTS            {GREEK_UCONSONANT}|{ENGLISH_UCONSONANT}+
CONSONANT              {LCONSONANT}|{UCONSONANT}
CONSONANTS             {CONSONANT}+
GREEK_ATONA            ({GREEK_LATONA}|{GREEK_UATONA})+

TIME                   [0-9]?[0-9]:[0-9][0-9]?(-[0-9]?[0-9]:[0-9][0-9]?)?
DATE                   [0-9]{1,4}[-/][0-9]{1,4}([-/][0-9]{1,4})?

HTML_SPACE             [[:space:]\n]+
HTML_ATTRIBUTE_NAME    [A-Za-z0-9_:.-]+
HTML_ATTRIBUTE_VALUE   \"[^"]+\"|\'[^']+\'|[^\> \t\n]+
HTML_ATTRIBUTE         {HTML_ATTRIBUTE_NAME}(({HTML_SPACE})*\=({HTML_SPACE})*{HTML_ATTRIBUTE_VALUE})?
HTML_TAG_NAME          [!]?[1-9A-Za-z:_]+[/]?
HTML_TAG_ATTRS         ({HTML_SPACE})?({HTML_SPACE}{HTML_ATTRIBUTE})*({HTML_SPACE})*
HTML_TAG               \<\/?{HTML_TAG_NAME}{HTML_TAG_ATTRS}[/]?\>
XML_START_TAG          \<\?(xml|XML)({HTML_SPACE})?({HTML_SPACE}{HTML_ATTRIBUTE})+\?\>
HTML_DOCTYPE           (\<\!DOCTYPE|\<\!doctype)[^\>]+\>
HTML_SCRIPT_START      \<script|\<SCRIPT|(\<script|\<SCRIPT)[^\>]*\>
HTML_SCRIPT_END        \<\/script\>|\<\/SCRIPT\>
HTML_COMMENT_START     \<\!
HTML_COMMENT_END       --\>
HTML_STYLE_START       \<style|\<STYLE|(\<style|\<STYLE)[^\>]*\>
HTML_STYLE_END         \<\/style\>|\<\/STYLE\>
HTML_ENTITY            \&[A-Za-z]+\;|\&\#[0-9][0-9][0-9]\;
HTML_EMAIL             {HTML_ATTRIBUTE_NAME}\@({HTML_ATTRIBUTE_NAME}\.)+{HTML_ATTRIBUTE_NAME}
HTML_URL               (http|ftp|nntp)\:\/\/({HTML_ATTRIBUTE_NAME}\.)+{HTML_ATTRIBUTE_NAME}|www\.({HTML_ATTRIBUTE_NAME}\.)+{HTML_ATTRIBUTE_NAME}
HTML_START_SENTENCE_TAG_NAMES p|li|td|title|table|tbody|tr|td|div|P|LI|TD|TITLE|TABLE|TBODY|TR|TD|DIV
HTML_START_SENTENCE    \<{HTML_START_SENTENCE_TAG_NAMES}{HTML_TAG_ATTRS}\>
HTML_END_SENTENCE      \<\/?({HTML_START_SENTENCE_TAG_NAMES}|br|BR){HTML_SPACE}*[/]?\>

COMMON_KEYWORDS        (C\+\+|C\/C\+\+|VC\+\+|g\+\+|http\:\/\/|ftp:\/\/|DB2|TCP\/IP)

COMMON_GREEK_ABBREVIATIONS    (\.\.|\.|\.\.\.|\.\.|\.\.\.|A\.E\.|\.\.\.|\:|\.|\.|\.\.|\.\.\.|\.\.\.)
COMMON_ENGLISH_ABBREVIATIONS  (etc\.|ref\.|S\.A\.|Ave\.)

%x script comment style
%%

 /* Enable access to the global variables of this component... */
 CDM_ACCESS_GLOBAL_DATA(HTokenizer);

 /* Rule Section */

 /* HTML Section. HTML rules must be in front of plain text rules... */
\<HTML[[:space:]]*\>|\<html[[:space:]]*\>|{XML_START_TAG} {HtmlFile = 1;
                         //EmmitSentences = 0;
                         if (HTok_add_token(HTML)) return 10;}
{HTML_DOCTYPE}           {HtmlFile = 1;
                         if (HTok_add_token(HTML_DOCTYPE)) return 10;}
{HTML_SCRIPT_START}      {InsideOtherToken = 1; start = char_count;
                         char_count += yyleng;
			 BEGIN(script);
			 }
<script>{HTML_SCRIPT_END} {InsideOtherToken = 0; ComputeStart = 0;
                         HTok_CloseCurrentSentence();
                         if (HTok_add_token(HTML_SCRIPT)) return 10;
                         ComputeStart = 1;
                         if (HTok_AddSentenceAnnotation()) return 10;
			 BEGIN(INITIAL);
			 }
{HTML_COMMENT_START}     {InsideOtherToken = 1; start = char_count;
                         char_count += yyleng;
			 HtmlFile = 1;
			 BEGIN(comment);
			 }
<comment>{HTML_COMMENT_END} {InsideOtherToken = 0; ComputeStart = 0;
                         HTok_CloseCurrentSentence();
                         if (HTok_add_token(HTML_COMMENT)) return 10;
                         ComputeStart = 1;
                         if (HTok_AddSentenceAnnotation()) return 10;
			 BEGIN(INITIAL);
			 }
{HTML_STYLE_START}       {InsideOtherToken = 1; start = char_count;
                         char_count += yyleng;
			 HtmlFile = 1;
			 BEGIN(style);
			 }
<style>{HTML_STYLE_END}  {InsideOtherToken = 0; ComputeStart = 0;
                         HTok_CloseCurrentSentence();
                         if (HTok_add_token(HTML_STYLE)) return 10;
                         ComputeStart = 1;
                         if (HTok_AddSentenceAnnotation()) return 10;
			 BEGIN(INITIAL);
			 }
{HTML_START_SENTENCE}   {
                         if (!InsideOtherToken) HTok_CloseCurrentSentence();
                         if (!InsideOtherToken) {
                           if (HTok_AddSentenceAnnotation()) return 10;
                         }
                         if (HTok_add_token(HTML)) return 10;
                        }
{HTML_END_SENTENCE}     {
                         if (!InsideOtherToken) HTok_CloseCurrentSentence();
                         if (HTok_add_token(HTML)) return 10;
                         if (!InsideOtherToken) {
                           if (HTok_AddSentenceAnnotation()) return 10;
                         }
                        }       
{HTML_EMAIL}            {if (HTok_add_token(HTML_EMAIL)) return 10;}
{HTML_URL}              {if (HTok_add_token(HTML_URL)) return 10;}
{HTML_ENTITY}           {if (HTok_add_token(HTML_ENTITY)) return 10;}
{HTML_TAG}              {if (HTok_add_token(HTML)) return 10;}

 /* Greek Tokens */
{GREEK_LOWER}"'"?                                                    {
               if (HTok_add_token(GREEK_LOWERCASE_LETTER)) return 10;
               }
               
{GREEK_CAPITAL}"'"?                                                  {
               if (HTok_add_token(GREEK_UPERCASE_LETTER)) return 10;
               }
               
{GREEK_LOWERS}({CONNECTIVES}{GREEK_LOWERS})*"'"?                     {
	       HANDLE_HTML_ENTITIES
               if (HTok_add_token(GREEK_LOWERCASE_WORD)) return 10;
               }
               
{GREEK_CAPITALS}({CONNECTIVES}{GREEK_CAPITALS})*"'"?                 {
	       HANDLE_HTML_ENTITIES
               if (HTok_add_token(GREEK_UPERCASE_WORD)) return 10;
               }
               
{GREEK_CAPITAL}{GREEK_LOWERS}({CONNECTIVES}{GREEK_CAPITAL}?{GREEK_LOWERS})*"'"? {
	       HANDLE_HTML_ENTITIES
               if (HTok_add_token(GREEK_FIRSTCAP_WORD)) return 10;
               }

{GREEK_CHARS}({CONNECTIVES}{GREEK_CHARS})*"'"?                       {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(GREEK_WORD)) return 10;
               }


 /* English Tokens */
{ENGLISH_LOWER}("'"{ENGLISH_LOWER})?                                 {
               if (HTok_add_token(ENGLISH_LOWERCASE_LETTER)) return 10;
               }

{ENGLISH_CAPITAL}("'"{ENGLISH_LOWER})?                               {
               if (HTok_add_token(ENGLISH_UPERCASE_LETTER)) return 10;
               }
              
{ENGLISH_LOWERS}({CONNECTIVES}{ENGLISH_LOWERS})*("'"(s)?)?           {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(ENGLISH_LOWERCASE_WORD)) return 10;
               }
               
{ENGLISH_CAPITALS}({CONNECTIVES}{ENGLISH_CAPITALS})*("'"(s)?)?       {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(ENGLISH_UPERCASE_WORD)) return 10;
               }
               
{ENGLISH_CAPITAL}{ENGLISH_LOWERS}({CONNECTIVES}{ENGLISH_CAPITAL}?{ENGLISH_LOWERS})*("'"(s)?)? {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(ENGLISH_FIRSTCAP_WORD)) return 10;
               }

{ENGLISH_CHARS}({CONNECTIVES}{ENGLISH_CHARS})*("'"(s)?)?            {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(ENGLISH_WORD)) return 10;
               }

 /* Mixed Language */
{LOWERS}({CONNECTIVES}{LOWERS})*("'"{LOWER}?)?                      {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(MIXED_LOWERCASE_WORD)) return 10;
               }
               
{CAPITALS}({CONNECTIVES}{CAPITALS})*("'"(s)?)?                      {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(MIXED_UPERCASE_WORD)) return 10;
               }
               
{CAPITAL}{LOWERS}({CONNECTIVES}{CAPITAL}*{LOWERS})?("'"(s)?)?       {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(MIXED_FIRSTCAP_WORD)) return 10;
               }

{CHARS}({CONNECTIVES}{CHARS})*("'"(s)?)?                            {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(MIXED_WORD)) return 10;
               }

 /* Special Cases */
{ENGLISH_CHARS}({CONNECTIVES}{ENGLISH_CHARS})+ |
{NUMBER}{CHARS}                                      {
	       HANDLE_HTML_ENTITIES
	       if (HTok_add_token(SPECIAL_WORD)) return 10;
               }

 /* Amounts... */
{AMOUNT} {
               HANDLE_HTML_ENTITIES
	       if (HTok_add_token(NUMBER)) return 10;
               }

 /* Abbreviations */

 /* Handle a special case, as ". " or ". " */
{GREEK_CAPITAL}{GREEK_LATONA}*{GREEK_LCONSONANT}{PERIOD}/[[:space:]]{CAPITAL}{LOWER} |
{GREEK_CAPITAL}{PERIOD}/[[:space:]]{CAPITAL}{LOWER} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }

 /* Special case 2: " .. " */
{GREEK_CAPITAL}{PERIOD}({GREEK_CAPITAL}{PERIOD})+/[[:space:]]{CAPITAL}{LOWER} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }

 /* Special case 3: "  559 .19" */
({CHARS}{APOSTROPH})*{GREEK_LATONA}*{GREEK_LCONSONANT}{PERIOD}/([[:space:]])*{INT_NUMBER} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }

 /* Special case 4: "...," */
({CHARS}{PERIOD})+/{COMMA} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }

 /* Special case 5: ". . " */
{CAPITAL}{PERIOD}/[[:space:]]+{CAPITAL}{PERIOD} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }
 /* Special case 6: " .  " */
(|)\./[[:space:]]+{CAPITAL} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }
 /* Special case 6: ". " */
{CAPITAL}{PERIOD}/[[:space:]]+{CAPITAL}{LOWERS} {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }

 /* Versions are usually specified as 1.5 or 1.x */
{INT_NUMBER}{PERIOD}{INT_NUMBER}/[[:space:]] |
{INT_NUMBER}{PERIOD}x/[[:space:]] |
{INT_NUMBER}{PERIOD}X/[[:space:]] {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }
 /* Handle other Abreviations... */
{COMMON_GREEK_ABBREVIATIONS} |
{COMMON_ENGLISH_ABBREVIATIONS} |
({CHARS}{PERIOD}{CHARS})+ |
({CHARS}{PERIOD})+/{APOSTROPH}|{RPAR} |
{LCONSONANTS}{PERIOD}/[[:space:]]{LOWER} |
{LCONSONANTS}/{PERIOD}[[:space:]]{CAPITAL} |
({GREEK_ATONA}{PERIOD})+{GREEK_CONSONANT}+ |
{GREEK_ATONA}{GREEK_CONSONANT}{PERIOD}/[[:space:]]{LOWER} |
{GREEK_ATONA}{GREEK_CONSONANT}/{PERIOD}[[:space:]]{CAPITAL}                 {
                if (HTok_add_token(ABBREVIATION)) return 10;
                }
 /* Recognise common tokens... */
{COMMON_KEYWORDS}                      {if (HTok_add_token(SPECIAL_WORD))
                                           return 10;}
 /* Various */
{THREEDOTS} |
{PUNCTUATION}   {  
                HTok_CloseCurrentSentence();
                if (HTok_add_token(PUNCTUATION)) return 10;
                if (HTok_AddSentenceAnnotation()) return 10;
}
{THREEDOTS}/[[:space:]]({APOSTROPH}|{LPAR}|{CONNECTIVES})?{LOWER} |
{COMMA}                                {if (HTok_add_token(PUNCTUATION))
                                           return 10;}
{NUMBER}([-/]{NUMBER})?\%?             {if (HTok_add_token(NUMBER)) return 10;}
{SYMBOL}                               {if (HTok_add_token(SYMBOL)) return 10;}
{DATE}                                 {if (HTok_add_token(DATE)) return 10;}
{TIME}                                 {if (HTok_add_token(TIME)) return 10;}
{APOSTROPH}                            {if (HTok_add_token(APOSTROPH))
                                           return 10;}
{LPAR}                                 {if (HTok_add_token(LPAR)) return 10;}
{RPAR}                                 {if (HTok_add_token(RPAR)) return 10;}
<*>[[:space:]]                         {char_count++;}
<*>.                                   {if (yytext[0] == '\240') {char_count++;}
                                        else
                                        if (HTok_add_token(UNKNOWN)) return 10;}

%%

/*
 *  Add a token Annotation (what ever that means)
 */

int HTok_add_token(const char *type) {
#ifdef DYNAMIC_MODULE
  CDM_AttributeValue AttributeValue;
  CDM_AttributeSet   AttributeSet;
  CDM_Annotation     Annotation;
  CDM_SpanSet        SpanSet;
  long               AnnId;
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);
#endif /* DYNAMIC_MODULE */

  if (InsideOtherToken) {
    /* If we are inside something (like an HTML script), we should not add
       other tokens... */
    char_count += yyleng;
    return 0;
  } else {
    if (ComputeStart) start = char_count;
    char_count += yyleng;
    end = char_count;
  }

#ifdef DYNAMIC_MODULE
  /* Create Attributes */
  AttributeValue = CDM_CreateAttributeValue(CDM_STRING, type);
  if (AttributeValue == NULL) return 1;
  AttributeSet = CDM_CreateAttributeSet("type", AttributeValue);
  if (AttributeSet == NULL) return 1;
  /* Create Spans */
  SpanSet = CDM_CreateSpanSet((long) start, (long) end);
  if (SpanSet == NULL) return 1;
  /* Finally, Create Annotation... */
  Annotation = CDM_CreateAnnotation("token", SpanSet, AttributeSet);
  if (Annotation == NULL) return 1;

  /* Add Annotation into Document... */
  AnnId = CDM_AddAnnotation(HTokenizer_Doc, Annotation);
  if (AnnId < 0) return 1;

  /* Should we add html in constituents? */
  if (strcmp(type, HTML)        != 0 && strcmp(type, HTML_SCRIPT)  != 0 &&
      strcmp(type, HTML_ENTITY) != 0 && strcmp(type, HTML_COMMENT) != 0 &&
      strcmp(type, HTML_STYLE)  != 0 && strcmp(type, HTML_DOCTYPE)) {
      Tcl_ListObjAppendElement(NULL, HTokenizer_Constituents,
                               CDM_NewLongObj(AnnId));
  } else {
    if (HtmlFile) {
      Tcl_ListObjAppendElement(NULL, HTokenizer_HTMLConstituents,
                               CDM_NewLongObj(AnnId));
    } else {
      Tcl_ListObjAppendElement(NULL, HTokenizer_Constituents,
                               CDM_NewLongObj(AnnId));
    }
  }
#else 
  if (debug) {
    fprintf(OutputFile, "%s\n", yytext);
  } else {
    fprintf(OutputFile, "{} token {{%d %d}} {{type {GDM_STRING %s}}}\n",
             start, end, type);
  }
#endif /* DYNAMIC_MODULE */
  
  /* Initialise new sentence, if this is the first token in sentence */
  if (first_token) {
    sentence_start = start;
    first_token = 0;
    sentence_completted=0;
  }
  sentence_end = end;
  return 0;
}; /* HTok_add_token */

/* 
 *  Start up a new sentence annotation
 */
void HTok_StartNewSentence(void) {
#ifdef DYNAMIC_MODULE
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);
  if (HTokenizer_Constituents != NULL)
    Tcl_DecrRefCount(HTokenizer_Constituents);
  if (HTokenizer_HTMLConstituents != NULL)
    Tcl_DecrRefCount(HTokenizer_HTMLConstituents);
  HTokenizer_Constituents     = Tcl_NewListObj(0, NULL);
  HTokenizer_HTMLConstituents = Tcl_NewListObj(0, NULL);
  Tcl_IncrRefCount(HTokenizer_Constituents);
  Tcl_IncrRefCount(HTokenizer_HTMLConstituents);
#endif /* DYNAMIC_MODULE */
  sentence_start = char_count;
  first_token=1;
}

/*
 *  End the current sentence
 */
void HTok_CloseCurrentSentence(void) {
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);
  sentence_completted=1;
}

/*
 *  Add a sentence Annotation
 */
int HTok_AddSentenceAnnotation(void) {
#ifdef DYNAMIC_MODULE
  CDM_AttributeValue AttributeValue;
  CDM_AttributeSet   AttributeSet;
  CDM_Annotation     Annotation;
  CDM_SpanSet        SpanSet;
  int                percent;
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);

  if (sentence_end <= sentence_start) return 0;
  
  if (EmmitSentences) {
    /* Create Attributes */
    AttributeValue = CDM_CreateAttributeValue(CDM_STRING_SET,
                                              HTokenizer_Constituents);
    if (AttributeValue == NULL) return 1;
    AttributeSet=CDM_CreateAttributeSet("constituents", AttributeValue);
    if (AttributeSet == NULL) return 1;
    if (HtmlFile) {
      CDM_Attribute htmlCons;
      AttributeValue = CDM_CreateAttributeValue(CDM_STRING_SET,
                                                HTokenizer_HTMLConstituents);
      if (AttributeValue == NULL) return 1;
      htmlCons = CDM_CreateAttribute("html_constituents", AttributeValue);
      if (htmlCons == NULL) return 1;
      Tcl_ListObjAppendElement(NULL, AttributeSet, htmlCons);
    }
    /* Create Spans */
    SpanSet = CDM_CreateSpanSet((long) sentence_start, (long) sentence_end);
    if (SpanSet == NULL) return 1;
    /* Finally, Create Annotation... */
    Annotation = CDM_CreateAnnotation("sentence", SpanSet, AttributeSet);
    if (Annotation == NULL) return 1;

    /* Add Annotation into Document... */
    if (CDM_AddAnnotation(HTokenizer_Doc, Annotation) < 0) return 1;

    percent = int(float(sentence_end)/float(HTokenizer_StringSize)*100.0);
    if (percent > HTokenizer_LastPercent) {
      CDM_SetProcessStatus(HTokenizer_Doc, percent);
      HTokenizer_LastPercent += 25;
    }
  }; /* if (EmmitSentences) */
#else
  if (sentence_end <= sentence_start) return 0;
  if (EmmitSentences) {
    /*sentence_end = char_count;*/
    if (debug) {
      fprintf(OutputFile, "s: --%s--\n", yytext);
    } else {
      fprintf(OutputFile, "{} sentence {{%d %d}} {}\n",
                          sentence_start, sentence_end);
    }
  }; /* if (EmmitSentences) */
#endif /* DYNAMIC_MODULE */
  HTok_CloseCurrentSentence();
  HTok_StartNewSentence();
  return 0;
}

int yywrap() {return 1;}

/*
 * The following code is for building a Dynamic module...
 */
#ifdef DYNAMIC_MODULE
/* Procedure: creole_HTokenizer */
int creole_HTokenizer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  char *ByteSequence;
  register int size;
  YY_BUFFER_STATE flex_buffer;
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);

  CDM_SetActiveInterpreter(interp);
  EmmitSentences = 1;
  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "Document ?args?");
    return TCL_ERROR;
  }

  HTokenizer_Doc = CDM_GetDocumentPointer(objv[1]);
  if (HTokenizer_Doc == NULL) 
    return CDM_AppendError("\ncreole_HTokenizer: Invalid Document Pointer");
    
  /* Check for arguments */
  HTokenizer_LastPercent = 25;
  size = 0;
  if (objc > 2) {
    if (Tcl_GetBooleanFromObj(interp, objv[2], &size) != TCL_OK)
                                                      return TCL_ERROR;
    if (size) HTokenizer_LastPercent = 200;
    if (objc > 3) {
      if (Tcl_GetBooleanFromObj(interp, objv[3], &EmmitSentences) != TCL_OK)
                                                      return TCL_ERROR;
    }
  }
  if (!size) CDM_SetProcessStatus(HTokenizer_Doc, 0);
  
  char_count = sentence_start = sentence_end = first_token = 0;
  sentence_completted = 0;
  
  ByteSequence = CDM_GetByteSequence(HTokenizer_Doc, "iso8859-7");
  if (ByteSequence == NULL) return TCL_ERROR;

  HTok_StartNewSentence();
  HTokenizer_StringSize = size = strlen(ByteSequence);
  
  flex_buffer = yy_scan_string((yyconst char *) ByteSequence);
  if (yylex() == 10) {
    yy_delete_buffer(flex_buffer);
    Tcl_Free(ByteSequence);
    CDM_AppendError("HTokenizer: internal error - stopping...");
    return TCL_ERROR;
  }
  
  if (!sentence_completted) {
    HTok_CloseCurrentSentence();
    HTok_AddSentenceAnnotation();
  }
  yy_delete_buffer(flex_buffer);
  Tcl_Free(ByteSequence);
  CDM_PutAttribute(HTokenizer_Doc, CDM_CreateAttribute("language",
                        CDM_CreateAttributeValue(CDM_STRING, "hellenic")));
  CDM_PutAttribute(HTokenizer_Doc, CDM_CreateAttribute("language_greek",
                        CDM_CreateAttributeValue(CDM_STRING, "")));
  CDM_PutAttribute(HTokenizer_Doc, CDM_CreateAttribute("HTokenizer",
                        CDM_CreateAttributeValue(CDM_STRING, "")));
  if (HtmlFile) {
    CDM_PutAttribute(HTokenizer_Doc, CDM_CreateAttribute("HTML",
                        CDM_CreateAttributeValue(CDM_STRING, "true")));
  }
  return TCL_OK;
} /* creole_HTokenizer */

/* Procedure: creole_HTokenizer_Init */
int creole_HTokenizer_Init(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  return TCL_OK;
} /* creole_HTokenizer_Init */


/* Procedure: creole_HTokenizer_Finish */
int creole_HTokenizer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  return TCL_OK;
} /* creole_HTokenizer_Finish */

/* ######## Ignore the Following Code ################################# */
/* This command registers the Module commands with the tcl interpreter.
   Modify with care...                                                  */
extern "C" CDM_EXPORT int Creole_htokenizer_Init(Tcl_Interp *interp) {
#ifdef CDM_USE_STUBS 
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
    return TCL_ERROR;
  }
#endif /* CDM_USE_STUBS */
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(HTokenizer);

  /* Initialise global variables... */
  char_count                  = 0;
  sentence_start              = 0;
  sentence_end                = 0;
  first_token                 = 0;
  sentence_completted         = 0;
  HtmlFile                    = 0;
  InsideOtherToken            = 0;
  ComputeStart                = 1;
  EmmitSentences              = 1;
  HTokenizer_Constituents     = NULL;
  HTokenizer_HTMLConstituents = NULL;
  creole_HTokenizer_home      = NULL;
#ifndef DYNAMIC_MODULE
  debug                       = 0;
#endif

  /* Link our location var with Tcl */
  CDM_LinkVar(interp, "creole_HTokenizer_home", 
                      &creole_HTokenizer_home);

  /* Register the Module Main Command */
  if( Tcl_CreateObjCommand(interp, "creole_HTokenizer", 
                           (Tcl_ObjCmdProc*) creole_HTokenizer,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if( Tcl_CreateObjCommand(interp, "creole_HTokenizer_Init", 
                           (Tcl_ObjCmdProc*) creole_HTokenizer_Init,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if( Tcl_CreateObjCommand(interp, "creole_HTokenizer_Finish", 
                           (Tcl_ObjCmdProc*) creole_HTokenizer_Finish,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  /* Report Success... */
  return TCL_OK;
} /* Creole_htokenizer_Init */
#else 
int main(int argc, char *argv[]) {
  char *ptr;
  int i;
  InputFile = stdin;
  OutputFile = stdout;
  
  for (i=1; i<argc; i++) {
    if (strcmp(argv[i], "-i") == 0) {
      ptr=argv[++i];
      InputFile = fopen( ptr, "r");
      if (InputFile == NULL) {
        fprintf(stderr, "Cannot Open Input File: %s!\n", ptr);
        exit(-1);
      }
    }
    if (strcmp(argv[i], "-o") == 0) {
      ptr=argv[++i];
      OutputFile = fopen( ptr, "w");
      if (OutputFile == NULL) {
        fprintf(stderr, "Cannot Open Output File: %s!\n", ptr);
        exit(-1);
      }
    }
    if (strcmp(argv[i], "-h") == 0) {
      fprintf(stderr, "Usage: %s -i InputFile -o OutputFile or\n"
              "       %s  (for reading from stdin and write to stdout)\n",
              argv[0], argv[0]);
      exit(-1);
    }
    if (strcmp(argv[i], "-v") == 0) {
      fprintf(stderr, 
              "%s: A text tokenizer and sentence splitter for the Greek "
              "language.\nCopyright: NCSR \"Demokritos\"\n"
              "Author: G. Petasis, petasis@iit.demokritos.gr\n",
              argv[0]);
      exit(-1);
    }
    if (strcmp(argv[i], "-d") == 0) {
      debug = 1;
    }
  }
  yyin = InputFile;
  HTok_StartNewSentence();
  yylex();
  if (!sentence_completted) {
    HTok_CloseCurrentSentence();
    HTok_AddSentenceAnnotation();
  }
  return 0;
}
#endif /* DYNAMIC_MODULE */
