##########################################################################
# 
#      HTokenizer.tcl - Monday February 15 10:04:06 (EET) 1999
#       This is a loosed coupled (Tcl) Module for use with 
#       NGate, version 0.1...
#
##########################################################################


# Location of this module (HTokenizer)
global creole_HTokenizer_home

## creole_HTokenizer
#  
proc creole_HTokenizer {doc args} {
  global creole_HTokenizer_home CDM_TempDir
  catch {ggi_wait_update $doc 0}
  set current_dir [pwd]
  cd $creole_HTokenizer_home
  
  set inputFileName  [file join $CDM_TempDir creole_HTokenizer_input[pid]]
  set outputFileName [file join $CDM_TempDir creole_HTokenizer_output[pid]]
  
  ## Copy Text to the Input File... ##
  if [catch {open $inputFileName w} inputFile] {
    error "Cannot open $inputFileName"
  }
  catch {fconfigure $inputFile -encoding iso8859-7}
  set Text [tip_GetByteSequence $doc]
  puts -nonewline $inputFile $Text
  close $inputFile
  catch {ggi_wait_update $doc 10}

  if [catch {exec tokenizer.exe -i $inputFileName -o $outputFileName } error] {
    catch {file delete -force $inputFileName}
    catch {file delete -force $outputFileName}
    error "Cannot execute tokenizer!\n($error)" {}
  }
  catch {ggi_wait_update $doc 20}

  ## Read Output File, and pass data to the GDM... ##
  if {[catch {open $outputFileName r} outputFile]} {
    error "Cannot read $outputFileName!\n($outputFile)" {}
  }

  if {[catch {set lines [lindex [exec wc $outputFileName] 0]}]} {
    set lines 100000
  }
  set step [expr 80.0/$lines]; set Progress 20.0
  set Constituents {}
  while {![eof $outputFile]} {
    set Progress [expr {$Progress + $step}]
    catch {ggi_wait_update $doc $Progress}
    gets $outputFile Ann
    if {![llength $Ann]} continue
    foreach {ann_id ann_type ann_spans ann_attrs} $Ann {break}
    foreach {start end} [lindex $ann_spans 0] {break}
    set ann_spans [list [tip_CreateSpan $start $end]]
    if {[string equal $ann_type "sentence"]} {
      set attrs [list [tip_CreateAttribute constituents \
                      [tip_CreateAttributeValue GDM_STRING_SET $Constituents]]]
      set ann [tip_CreateAnnotation $ann_type $ann_spans $attrs]
      tip_AddAnnotation $doc $ann
      set Constituents {}
    } else {
      set val   [tip_GetValueValue [lindex [lindex $Ann 3] 0]]
      set attrs [list [tip_CreateAttribute type \
                      [tip_CreateAttributeValue GDM_STRING $val] \
                ]]
      set ann [tip_CreateAnnotation $ann_type $ann_spans $attrs]
      lappend Constituents [tip_AddAnnotation $doc $ann]
    }
  }
  close $outputFile
  file delete -force $inputFileName
  file delete -force $outputFileName

  # record the fact that we ran and exit normally
  cd $current_dir
  tip_PutAttribute $doc [tip_CreateAttribute HTokenizer \
      [tip_CreateAttributeValue GDM_STRING {}]]
  tip_PutAttribute $doc [tip_CreateAttribute language  \
      [tip_CreateAttributeValue GDM_STRING greek]]
  tip_PutAttribute $doc [tip_CreateAttribute language_greek  \
      [tip_CreateAttributeValue GDM_STRING {}]]
  return

} ;# creole_HTokenizer


## Procedure: creole_HTokenizer_Initialize
#   Use this function in order to do some initialization. This function will
#   be called just before creole_HTokenizer in the following situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just before the first document in Collection gets proccessed 
#       with creole_HTokenizer
#    *) If the user has opened a single Document, this function will be called
#       just before calling creole_HTokenizer
#   Please, refer to the NGate's Programming Manual for more 
#   information on this function...
proc creole_HTokenizer_Initialize { col doc args } {
  global creole_HTokenizer_home
  ## Do some initilization here...
} ;# creole_HTokenizer_Initialize

## Procedure: creole_HTokenizer_Finish
#   Use this function in order to do some Clean-Up. This function will
#   be called after all calls to creole_HTokenizer in the following 
#   situations:
#    *) If the user has opened a whole Collection, this function will be 
#       called just after the last document in Collection gets proccessed with 
#       creole_HTokenizer
#    *) If the user has opened a single Document, this function will be called
#       just after calling creole_HTokenizer
#   Please, refer to the NGate's Programming Manual for more 
#   information on this function...
proc creole_HTokenizer_Finish { col doc args } {
  global creole_HTokenizer_home
  ## Clean-Up...
} ;# creole_HTokenizer_Finish


