/***************************************************************************
*
*       JavaTokeniserSplitter.java - Tuesday November 19 22:31:58 (EET) 2002
*       This is a java coupled (Java) Component for use with 
*       Ellogon, version 1.5...
*
***************************************************************************/

import tcl.lang.*;
import Ellogon.JCDM.*;
import java.text.*;
import java.util.*;

/* 
 * Class: creole_JavaTokeniserSplitter
 * This is the class of interest, as it implements the basic function
 * required by the Ellogon platform in order to succesfully load this plug-in
 * module. This function is the main function of the module, responsible
 * for doing the basic processing...
 */
class creole_JavaTokeniserSplitter implements Command {
  /* Location of this module (JavaTokeniserSplitter) */
  public static String creole_JavaTokeniserSplitter_home;
  
  public void cmdProc(Interp interp, TclObject argv[]) throws TclException {
    /*
     * Ellogon Components require at least one argument. If more are
     * required by the component, please update the following three lines of
     * code accordingly...
     */
    if (argv.length < 2) {
      throw new TclNumArgsException(interp, 1, argv, "Document ?args?");
    }

    OCDM_Document Doc = new OCDM_Document(argv[1].toString());
    Doc.SetProcessStatus(0); // Status Value: [0-100], Remove method to disable

    // Get the Document text and convert it into a String object...
    OCDM_RawData rawdata = Doc.GetRawData();
    String       Text    = rawdata.toString();

    int start_of_word, end_of_word, start_of_sentence, end_of_sentence;
    
    // Tokenise text...
    BreakIterator worditerator = BreakIterator.getWordInstance();
    worditerator.setText(Text);
    start_of_word = worditerator.first();
    end_of_word   = worditerator.next();
    while (end_of_word != BreakIterator.DONE) {
      String word = Text.substring(start_of_word, end_of_word);
      if (!Character.isWhitespace(word.charAt(0))) {
        // Create a new Annotation:
        // First, create the set of spans...
	OCDM_SpanSet spans = new OCDM_SpanSet(start_of_word, end_of_word);
        // Then an attribute value (as we will add an attribute in the
        // annotation)...
        OCDM_AttributeValue attrVal = new OCDM_AttributeValue(OCDM.OCDM_STRING,
                                                              "word");
        OCDM_AttributeSet attrs = new OCDM_AttributeSet ("type", attrVal);
        // And finally create the Annotation...
        OCDM_Annotation annotation = new OCDM_Annotation("token", spans, attrs);
        // Add the Annotation into the document...
        Doc.AddAnnotation(annotation); 
      }  
      start_of_word = end_of_word;
      end_of_word = worditerator.next();
    } // while (end_of_word != BreakIterator.DONE)

    // Identify sentences...
    BreakIterator sentenceiterator = BreakIterator.getSentenceInstance();
    sentenceiterator.setText(Text);
    start_of_sentence = sentenceiterator.first();
    end_of_sentence   = sentenceiterator.next();
    OCDM_AttributeSet attrs = new OCDM_AttributeSet ();
    while (end_of_sentence != BreakIterator.DONE) {
      OCDM_SpanSet spans = new OCDM_SpanSet(start_of_sentence, end_of_sentence);
      OCDM_AttributeValue attrVal = new OCDM_AttributeValue(OCDM.OCDM_STRING,
                                                            "sentence");
      OCDM_Annotation annotation = new OCDM_Annotation("sentence", spans,
                                                                   attrs);
      Doc.AddAnnotation(annotation); 
      start_of_sentence = end_of_sentence;
      end_of_sentence = sentenceiterator.next();
    }

    // Record the fact that we ran and exit normally
    Doc.PutAttribute(new OCDM_Attribute("JavaTokeniserSplitter",
                         new OCDM_AttributeValue(OCDM.OCDM_STRING, "")));
  } /* creole_JavaTokeniserSplitter::cmdProc */
} /* class creole_JavaTokeniserSplitter */


/* Function: creole_JavaTokeniserSplitter_Initialize
    Use this function in order to do some initialisation. This function will
    be called just before creole_JavaTokeniserSplitter in the following
    situations:
     *) If the user has opened a whole Collection, this function will be 
        called just before the first document in Collection gets proccessed 
        with creole_JavaTokeniserSplitter
     *) If the user has opened a single Document, this function will be called
        just before calling creole_JavaTokeniserSplitter
    Please, refer to the Ellogon's Programming Manual for more 
    information on this function... */
class creole_JavaTokeniserSplitter_Initialize implements Command {
  /* Location of this module (JavaTokeniserSplitter) */
  public static String creole_JavaTokeniserSplitter_home;
  
  public void cmdProc(Interp interp, TclObject argv[]) throws TclException {
    // if (argv.length < 3) {
    //   throw new TclNumArgsException(interp, 1, argv,
    //                                            "Collection Document ?args?");
    // }
    // OCDM_Collection Col = new OCDM_Collection(argv[1].toString());
    // OCDM_Document   Doc = new OCDM_Document(argv[2].toString());
    
    /* Do some initilization here... */
    
  } /* creole_JavaTokeniserSplitter_Initialize::cmdProc */
} /* class creole_JavaTokeniserSplitter_Initialize */


/*  Function: creole_JavaTokeniserSplitter_Finish
    Use this function in order to do some Clean-Up. This function will
    be called after all calls to creole_JavaTokeniserSplitter in the following 
    situations:
     *) If the user has opened a whole Collection, this function will be 
        called just after the last document in Collection gets proccessed with 
        creole_JavaTokeniserSplitter
     *) If the user has opened a single Document, this function will be called
        just after calling creole_JavaTokeniserSplitter
    Please, refer to the Ellogon's Programming Manual for more 
    information on this function... */
class creole_JavaTokeniserSplitter_Finish implements Command {
  /* Location of this module (JavaTokeniserSplitter) */
  public static String creole_JavaTokeniserSplitter_home;

  public void cmdProc(Interp interp, TclObject argv[]) throws TclException {
    // if (argv.length < 3) {
    //   throw new TclNumArgsException(interp, 1, argv,
    //                                            "Collection Document ?args?");
    // }
    // OCDM_Collection Col = new OCDM_Collection(argv[1].toString());
    // OCDM_Document   Doc = new OCDM_Document(argv[2].toString());
    
    /* Clean-Up... */

  } /* creole_JavaTokeniserSplitter_Finish::cmdProc */
} /* class creole_JavaTokeniserSplitter_Finish */

/**********************************************************************
 **               --   Ignore the Following Code   --              
 **  The following Java code takes care the interfaces to the Ellogon
 **  platform. Please modify with care...
 **********************************************************************/

/* 
 * This class implements a simple Tcl extension package and registers the
 * Components commands with the main Ellogon Tcl interpreter.
 * Modify with care...
 */
public class JavaTokeniserSplitter extends Extension {
  public void init(Interp interp) {
    /* Register the Module Main Command */
    interp.createCommand("creole_JavaTokeniserSplitter",
                         new creole_JavaTokeniserSplitter());
    interp.createCommand("creole_JavaTokeniserSplitter_Initialize",
                         new creole_JavaTokeniserSplitter_Initialize());
    interp.createCommand("creole_JavaTokeniserSplitter_Finish",
                         new creole_JavaTokeniserSplitter_Finish());
  }

  public static void SetComponentHome(String path) {
    creole_JavaTokeniserSplitter.creole_JavaTokeniserSplitter_home = 
    creole_JavaTokeniserSplitter_Initialize.creole_JavaTokeniserSplitter_home =
    creole_JavaTokeniserSplitter_Finish.creole_JavaTokeniserSplitter_home =
                                                                           path;
  }
} /* JavaTokeniserSplitter */

/* 
 * End of JavaTokeniserSplitter.java
 */
