##########################################################################
# 
#	LanguageIdentifier.tcl - Monday September 21 21:31:37 (EEST) 2009
#       This is a loosed coupled (Tcl) Component for use with 
#       Ellogon, version 1.8.1...
#
##########################################################################


# Location of this module (LanguageIdentifier)
global creole_LanguageIdentifier_home

## creole_LanguageIdentifier
#  
proc creole_LanguageIdentifier {doc args} {
  set rough  {}
  set detail {}
  foreach language [creole_LanguageIdentifier_Identify \
                      [tip_GetRawData $doc]] {
    lappend detail $language
    set parts [split $language -]
    if {[llength $parts] > 1} {
      lappend rough [lindex $parts 0]
    } else {
      lappend rough $language
    }
  }
  ## Put the attributes...
  tip_PutAttribute $doc [tip_CreateAttribute auto-language \
                             [tip_CreateAttributeValue GDM_STRING_SET $rough]]
  tip_PutAttribute $doc [tip_CreateAttribute auto-language-detail \
                             [tip_CreateAttributeValue GDM_STRING_SET $detail]]
  # record the fact that we ran and exit normally
  tip_PutAttribute $doc [tip_CreateAttribute LanguageIdentifier \
                             [tip_CreateAttributeValue GDM_STRING {}]]
};# creole_LanguageIdentifier

proc creole_LanguageIdentifier_Identify {text {topmost_ngrams 400}} {
  global creole_LanguageIdentifier
  set t_ngrams [creole_LanguageIdentifier_CreateLanguageModel \
                                         $text $topmost_ngrams]
  foreach language [array names creole_LanguageIdentifier] {
    set l_ngrams $creole_LanguageIdentifier($language)
    set p 0
    set i 0
    dict for {word freq} $t_ngrams {
      if {[dict exists $l_ngrams $word]} {
        incr p [expr {abs([dict get $l_ngrams $word] - $i)}]
      } else {
        incr p $topmost_ngrams
      }
      incr i
    }
    lappend results($p) $language
  }
  ## Return the best scoring language...
  set scores [lsort -integer [array names results]]
  set best_score [lindex $scores 0]
  return $results($best_score)
};# creole_LanguageIdentifier_Identify

proc creole_LanguageIdentifier_CreateLanguageModel {text {topmost_ngrams 400}} {
  package require ELEP::DocumentUtilities::NaiveTokenizer
  ## Generate the n-grams...
  set ngram [dict create]
  foreach word [ELEP::DocumentUtilities::NaiveTokenizer tokenize \
                                              [string tolower $text]] {
    set word "_${word}_"
    set len  [string length $word]
    set flen $len
    for {set i 0} {$i < $flen} {incr i} {
      if {$len > 4} {dict incr ngram [string range $word $i $i+5]}
      if {$len > 3} {dict incr ngram [string range $word $i $i+4]}
      if {$len > 2} {dict incr ngram [string range $word $i $i+3]}
      if {$len > 1} {dict incr ngram [string range $word $i $i+2]}
      dict incr ngram [string range $word $i $i+1]
      incr len -1
    }
  }

  ## Sort the n-grams...
  dict for {key val} $ngram {
    lappend NGRAM($val) $key
  }
  set ngram [dict create]
  set count 0
  foreach freq [lsort -integer -decreasing [array names NGRAM]] {
    foreach one [lsort -dictionary $NGRAM($freq)] {
      dict set ngram $one $freq
      if {[incr count] >= $topmost_ngrams} {return $ngram}
    }
  }
  return $ngram
};# creole_LanguageIdentifier_CreateLanguageModel

## Procedure: creole_LanguageIdentifier_Initialize
proc creole_LanguageIdentifier_Initialize {col doc args} {
  global creole_LanguageIdentifier
  set LM [lindex $args 0]
  ## Load the language models...
  foreach file [glob -directory $LM *.lm] {
    set language [file rootname [file tail $file]]
    # Is there an encoding in the filename?
    set encoding [lindex [split $language -] end]
    set fd [open $file]
    catch {fconfigure $fd -encoding $encoding}
    set rank 0
    set ngrams [dict create]
    while {[gets $fd line] > 0} {
      set key [lindex [split $line] 0]
      dict set ngrams $key [incr rank]
    }
    close $fd
    set creole_LanguageIdentifier($language) $ngrams
  }
};# creole_LanguageIdentifier_Initialize

## Procedure: creole_LanguageIdentifier_Finish
proc creole_LanguageIdentifier_Finish {col doc args} {
};# creole_LanguageIdentifier_Finish

#
# End of File LanguageIdentifier.tcl
#
