/***************************************************************************
*
* PorterEnglishStemmer.cpp - Wednesday July 02 13:54:23 (GTB Daylight Time) 2003
* This is a dynamic/tight coupled (C) Component for use with 
* Ellogon, version 1.5...
*
***************************************************************************/

//#include <stdio.h>
//#include <stdlib.h>
//#include "CDM.h"
//#include <string.h>
//#include <ctype.h>

#define TRUE 1
#define FALSE 0

/* If you want to use the object oriented interface, uncomment the following
   include definition: */
#include "OCDM.h"

/* 
 * Component's Home Path: Its value will be updated when the component will be
 * loaded... 
 */
CDM_BEGIN_GLOBAL_VARIABLE_BLOCK(PorterEnglishStemmer)
CDM_DEFINE_GLOBAL(char *, creole_PorterEnglishStemmer_home);
/* Global variables from stem.c */
CDM_DEFINE_GLOBAL(char *, b);
CDM_DEFINE_GLOBAL(int, k);
CDM_DEFINE_GLOBAL(int, k0);
CDM_DEFINE_GLOBAL(int, j);
CDM_END_GLOBAL_VARIABLE_BLOCK(PorterEnglishStemmer)
#define creole_PorterEnglishStemmer_home \
        (tsdPtr->creole_PorterEnglishStemmer_home)
#define b  (tsdPtr->b)
#define k  (tsdPtr->k)
#define k0 (tsdPtr->k0)
#define j  (tsdPtr->j)

#include "stem.c"

/* Prototype Definitions */
extern "C" CDM_EXPORT int Creole_porterenglishstemmer_Init(Tcl_Interp *interp);
int creole_PorterEnglishStemmer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_PorterEnglishStemmer_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_PorterEnglishStemmer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);
int creole_PorterEnglishStemmer_WordStem(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]);

/*
 * Helper function to stem a single word...
 */
int creole_PorterEnglishStemmer_WordStem(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  Tcl_Obj *result;
  char *str;
  int len;
  CDM_SetActiveInterpreter(interp); /* Do not remove this command... */
  if (objc != 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "word");
    return TCL_ERROR;
  }
  str = CDM_Strdup(Tcl_GetStringFromObj(objv[1], &len));
  str[stem(str, 0 , len-1)+1] = '\0';
  result = Tcl_NewStringObj(str, -1);
  CDM_Free(str);
  Tcl_SetObjResult(interp, result);
  return TCL_OK;
}; /* creole_PorterEnglishStemmer_WordStem */


/* Function: creole_PorterEnglishStemmer */
int creole_PorterEnglishStemmer(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
#ifdef OCDM_PRESENT
  OCDM_Document Doc;
#else  /* OCDM_PRESENT */
  CDM_Document Doc;
#endif /* OCDM_PRESENT */
  /* prevProgress will be used to call SetProcessStatus on the Document not to
   * often. When SetProcessStatus is called, Ellogon will update its windows
   * and process user requests. However, we do not want to call it too
   * frequently, i.e. for each Annotation processed, as doing so will slow
   * down our execution. We will call SetProcessStatus only ten times, just to
   * notify the user we are running... */
  double progress = 0.0, step, prevProgress = 0.0;
  
  CDM_SetActiveInterpreter(interp); /* Do not remove this command... */
  /*
   * Ellogon Components require at least one argument. If more are
   * required by the component, please update the following three lines of
   * code accordingly...
   */
  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "Document ?args?");
    return TCL_ERROR;
  }
  /* Retrieve the Document from the first argument... */
#ifdef OCDM_PRESENT
  try {
    Doc = OCDM_Document(objv[1]);
  } catch (OCDM_Exception e) {
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Document Pointer");
  }
  Doc.SetProcessStatus(0);
#else  /* OCDM_PRESENT */
  Doc = CDM_GetDocumentPointer(objv[1]);
  if (Doc == NULL) 
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Document Pointer");
  CDM_SetProcessStatus(Doc, 0);
#endif /* OCDM_PRESENT */

#ifdef OCDM_PRESENT
  /* This section of the code is an example on how to use the C++ interface */
  try {
    const OCDM_AnnotationSet Tokens = Doc.SelectAnnotations("token");
    step = 100.0/(Tokens.size()+1);
    for (int i = 0; i < Tokens.size(); ++i) {
      OCDM_Annotation token = Tokens[i];
      OCDM_ByteSequence tokenText = Doc.GetFirstAnnotatedTextRange(token);
      /* Convert all the UTF-8 characters of the text object into lower... */
      tokenText.toLower();
      /* CDM_Strdup will always return a valid (non-NULL) pointer... */
      char *str = CDM_Strdup(tokenText.toString());
      str[stem(str, 0 , tokenText.length()-1)+1] = '\0';
      token.PutAttribute(OCDM_Attribute("stem",
                                        OCDM_AttributeValue(CDM_STRING, str)));
      CDM_Free(str);
      Doc.AddAnnotation(token);
      progress += step;
      if (progress - prevProgress < 10.0) {
        Doc.SetProcessStatus((int) progress);
        prevProgress = progress;
      }
    }
  } catch (OCDM_Exception e) {
    return CDM_ReportError(e.what());
  }
#else
  /* This section of the code is an example on how to use the C interface */
  int i, tokens, length;
  char *str = NULL;
  CDM_Object *Token;
  CDM_ByteSequence tokenText = NULL;
  CDM_Attribute Value = NULL;
  CDM_Annotation ann = NULL;
  CDM_AnnotationSet Tokens = CDM_SelectAnnotations(Doc, "token");
  if (Tokens == NULL) goto error;
  CDM_SetGetElements(Tokens, &tokens, &Token);
  step = 100.0/(tokens+1);
  for (i = 0; i < tokens; i++) {
    tokenText = CDM_GetFirstAnnotatedTextRange(Doc, Token[i]);
    if (tokenText == NULL) goto error;
    /* Convert all the UTF-8 characters of the text object into lower... */
    /* CDM_Strdup will always return a valid (non-NULL) pointer... */
    str = CDM_Strdup(tokenText);
    /* Free the tokenText, is not needed any more as we got a copy of its
     * string in str... */
    CDM_Free(tokenText); tokenText = NULL;
    length = Tcl_UtfToLower(str);
    str[stem(str, 0 , length-1)+1] = '\0';
    Value = CDM_CreateAttribute("stem",
               CDM_CreateAttributeValue(CDM_STRING, str));
    if (Value == NULL) goto error;
    CDM_Free(str); str = NULL;
    ann = CDM_PutAttribute(Token[i], Value);
    if (ann == NULL) goto error;
    Value = NULL;
    if (CDM_AddAnnotation(Doc, ann) < 0) goto error;
    ann = NULL;
    /* It is an error to free Value & ann, as we pass them back to CDM... */
    progress += step;
    if (progress - prevProgress < 10.0) {
      CDM_SetProcessStatus(Doc, (int) progress);
      prevProgress = progress;
    }
  }
  CDM_Free(Tokens); Tokens = NULL;
#endif /* OCDM_PRESENT */

  /* Report Succesful Completion */
#ifdef OCDM_PRESENT
  Doc.SetProcessStatus(0);
  Doc.PutAttribute(OCDM_Attribute("PorterEnglishStemmer",
                        OCDM_AttributeValue(CDM_STRING, "")));
#else  /* OCDM_PRESENT */
  CDM_SetProcessStatus(Doc, 100);
  CDM_PutAttribute(Doc, CDM_CreateAttribute("PorterEnglishStemmer",
                        CDM_CreateAttributeValue(CDM_STRING, "")));
#endif /* OCDM_PRESENT */
  return TCL_OK;
error:
#ifdef OCDM_PRESENT
  // All resources have been already freed automatically...
#else  /* OCDM_PRESENT */
  if (str)       CDM_Free(str);
  if (tokenText) CDM_Free(tokenText);
  if (Value)     CDM_Free(Value);
  if (ann)       CDM_Free(ann);
  if (Tokens)    CDM_Free(Tokens);
#endif /* OCDM_PRESENT */
  return TCL_ERROR;
} /* creole_PorterEnglishStemmer */


/* Function: creole_PorterEnglishStemmer_Initialize
    Use this function in order to do some initialisation. This function will
    be called just before creole_PorterEnglishStemmer in the following situations:
     *) If the user has opened a whole Collection, this function will be 
        called just before the first document in Collection gets proccessed 
        with creole_PorterEnglishStemmer
     *) If the user has opened a single Document, this function will be called
        just before calling creole_PorterEnglishStemmer
    Please, refer to the Ellogon's Programming Manual for more 
    information on this function... */
int creole_PorterEnglishStemmer_Initialize(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /*
  if (objc < 3) {
    Tcl_WrongNumArgs(interp, 1, objv, "Collection Document ?args?");
    return TCL_ERROR;
  }
  CDM_Collection Col = CDM_GetCollectionPointer(objv[1]);
  CDM_Document   Doc = CDM_GetDocumentPointer(objv[2]);
  if (Col == NULL) 
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Collection Pointer");
  if (Doc == NULL) 
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Document Pointer");
  */

  /* Do some initilization here... */
  
  return TCL_OK;
} /* creole_PorterEnglishStemmer_Initialize */


/*  Function: creole_PorterEnglishStemmer_Finish
    Use this function in order to do some Clean-Up. This function will
    be called after all calls to creole_PorterEnglishStemmer in the following 
    situations:
     *) If the user has opened a whole Collection, this function will be 
        called just after the last document in Collection gets proccessed with 
        creole_PorterEnglishStemmer
     *) If the user has opened a single Document, this function will be called
        just after calling creole_PorterEnglishStemmer
    Please, refer to the Ellogon's Programming Manual for more 
    information on this function... */
int creole_PorterEnglishStemmer_Finish(
      ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *objv[]) {
  /*
  if (objc < 3) {
    Tcl_WrongNumArgs(interp, 1, objv, "Collection Document ?args?");
    return TCL_ERROR;
  }
  CDM_Collection Col = CDM_GetCollectionPointer(objv[1]);
  CDM_Document   Doc = CDM_GetDocumentPointer(objv[2]);
  if (Col == NULL) 
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Collection Pointer");
  if (Doc == NULL) 
    return CDM_ReportError("creole_PorterEnglishStemmer: "
                           "Invalid Document Pointer");
  */
  
  /* Clean-Up... */
  
  return TCL_OK;
} /* creole_PorterEnglishStemmer_Finish */


/**********************************************************************
 **               -->>   Ignore the Following Code   <<--              
 **  The following C++ code takes care the interfaces to the Ellogon
 **  platform. Please modify with care...
 **********************************************************************/

/* 
 * This command registers the Component commands with the tcl interpreter in
 * which the Component is loaded. Please modify with care...
 */
int Creole_porterenglishstemmer_Init(Tcl_Interp *interp) {
  /*
   * Use the stubs mechanism so as not to depend on the specific Tcl version.
   * Doing so will enable the binaries compiled under one system to run on
   * other systems, even if a different version of Tcl is installed.
   * Note that we pose a limit of the lowest version that can be used.
   * In order to run, we require a Tcl version >= 8.4...
   */
#ifdef CDM_USE_STUBS 
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
    return TCL_ERROR;
  }
#endif /* CDM_USE_STUBS */
  /* Enable access to the global variables of this component... */
  CDM_ACCESS_GLOBAL_DATA(PorterEnglishStemmer);

  /*
   * Link our location var with the Tcl interpreter.
   * The correct value will be set by Tcl, when the module is actually
   * loaded...
   */
  CDM_LinkVar(interp, "creole_PorterEnglishStemmer_home",
                      &creole_PorterEnglishStemmer_home);

  /* 
   * Register the Module Main Command 
   */
  if (Tcl_CreateObjCommand(interp, "creole_PorterEnglishStemmer", 
                           (Tcl_ObjCmdProc*) creole_PorterEnglishStemmer,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if (Tcl_CreateObjCommand(interp, "creole_PorterEnglishStemmer_Initialize", 
                           (Tcl_ObjCmdProc*) creole_PorterEnglishStemmer_Initialize,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  if (Tcl_CreateObjCommand(interp, "creole_PorterEnglishStemmer_Finish", 
                           (Tcl_ObjCmdProc*) creole_PorterEnglishStemmer_Finish,
                           (ClientData) NULL,
                           (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;

  /*
   * Register a helper function to stem a string...
   */
  if (Tcl_CreateObjCommand(interp, "creole_PorterEnglishStemmer_WordStem", 
                        (Tcl_ObjCmdProc*) creole_PorterEnglishStemmer_WordStem,
                        (ClientData) NULL,
                        (Tcl_CmdDeleteProc *) NULL)  == NULL )
       return TCL_ERROR;
  /*
   * Report Loading Success...
   */
  return TCL_OK;
} /* Creole_porterenglishstemmer_Init */

/* 
 * End of PorterEnglishStemmer.cpp
 */
