##########################################################################
# 
#       UnicodeListLookup.tcl - Wednesday November 21 22:02:16 (EET) 2001
#       This is a loosed coupled (Tcl) Module for use with 
#       Ellogon, version 1.00...
#
##########################################################################

## creole_UnicodeListLookup
#  
proc creole_UnicodeListLookup {doc args} {
  global creole_UnicodeListLookup
  ggi_wait_update $doc 0
  ## Get Document Text
  set Text [string toupper [tip_GetRawData $doc]]
  set Tokens [tip_SelectAnnotations $doc token]

  set Progress 0.0
  set Step [expr {100.0/($creole_UnicodeListLookup(LookupListFileSize)+1.0)}]
  foreach pattern $creole_UnicodeListLookup(Rules) \
          type    $creole_UnicodeListLookup(Types) \
          subtype $creole_UnicodeListLookup(SubTypes) {
    foreach index [regexp -indices -inline -all -- $pattern $Text] {
      foreach {start end} $index {break}
      incr end
      ## What is recognised by our gazetteer must never be smaller than a
      ## token, i.e. it must never be a token sub-part...
      set st_token [lindex [tip_AnnotationsAt $Tokens $start] 0]
      if {![llength $st_token]} {continue}
      set en_token [lindex [tip_AnnotationsContaining $Tokens $end]   0]
      if {![llength $en_token]} {continue}
      if {$end != [lindex [tip_GetFirstSpan $en_token] 1]} {continue}
      
      ## Add the Annotation
      set TypeAttr    [tip_CreateAttribute type \
                             [tip_CreateAttributeValue GDM_STRING $type]]
      set SubtypeAttr [tip_CreateAttribute subtype \
                             [tip_CreateAttributeValue GDM_STRING $subtype]]
      set ann [tip_CreateAnnotation lookup [list [list $start $end]] \
                    [list $TypeAttr $SubtypeAttr]]
      tip_AddAnnotation $doc $ann
    }
    set Progress [expr {$Progress+$Step}]
    ggi_wait_update $doc $Progress
  }

  tip_PutAttribute $doc [tip_CreateAttribute UnicodeListLookup \
                             [tip_CreateAttributeValue GDM_STRING {}]]
};# creole_UnicodeListLookup

proc creole_UnicodeListLookup_Initialize {col doc args} {
  global creole_UnicodeListLookup
  foreach {FileName Encoding} $args {break}

  ## Do a simple line count in the file...
  set lines 0
  set file [open $FileName]
  fconfigure $file -encoding $Encoding
  unset -nocomplain creole_UnicodeListLookup(Rules) \
                    creole_UnicodeListLookup(Types) \
                    creole_UnicodeListLookup(SubTypes)
  while {[gets $file line] >= 0} {
    foreach {pattern type subtype} $line {break}
    lappend creole_UnicodeListLookup(Rules)    $pattern
    lappend creole_UnicodeListLookup(Types)    $type
    lappend creole_UnicodeListLookup(SubTypes) $subtype
    incr lines
  }
  set creole_UnicodeListLookup(LookupListFileSize) $lines
  close $file
};# creole_UnicodeListLookup_Initialize

proc creole_UnicodeListLookup_Finish {col doc args} {
  global creole_UnicodeListLookup
  unset -nocomplain creole_UnicodeListLookup
};# creole_UnicodeListLookup_Finish

## creole_UnicodeListLookupCreateFile
#  Function to create the needed lists out of a set of files...
proc creole_UnicodeListLookupCreateFile {args} {
  global GUI_Text GUI_Options
  set win [ggi_unused_win .creole_UnicodeListLookupCreateFile]
  toplevel $win
  wm withdraw $win
  wm title $win {Create List Lookup Rule File...}

  labelframe $win.files -text {Select List Files:}
    ScrolledWidgets::ScrolledListbox $win.files.files \
      -bd $GUI_Options(BorderWidth) -height 1 -width 45
    $win.files.files.listbox insert end \
      $::creole_UnicodeListLookup_home/Lists
    $win.files.files.listbox xview moveto 1.0
    button $win.files.add -compound left -image ITOpen \
       -textvariable GUI_Text(SelectFile) \
       -command "GUI_SelectFiles $win.files.files.listbox"
    grid $win.files.files $win.files.add -sticky snew -padx 3 -pady 2
    grid columnconfigure $win.files 0 -weight 1
    grid rowconfigure    $win.files 0 -weight 1
    ## File encoding:
    set menu $win.files.encoding.menu
    menubutton $win.files.encoding -relief raised -indicatoron 1 -anchor c \
        -menu $menu -width 150 -image [::Encoding::GetEncodingImage \
        $::KeyboardLocale] -text $::KeyboardLocale -compound left -padx 10 \
        -bd $GUI_Options(BorderWidth) -fg #ffff00 -bg #009900
    menu $menu -tearoff 0
    GUI_AppendEncodingsToMenu $menu \
      "$win.files.encoding configure -image \$EncodingImage -text \$Encoding" 0
    grid $win.files.encoding -sticky snew -padx 20 -pady 2 -columnspan 2
  pack $win.files -fill both -expand 1 -padx 2 -pady 2

  labelframe $win.file -text {Select Rule File:}
    entry $win.file.file -width 50
    $win.file.file insert end $::creole_UnicodeListLookup_home/Rules.txt
    $win.file.file xview end
    button $win.file.buttton -compound left -image ITOpen -textvariable \
      GUI_Text(SelectFile) -command "GUI_SelectFile $win.file.file"
    GUI_MakeEntryFileDropTarget $win.file.file
    grid $win.file.file $win.file.buttton -sticky snew -padx 3 -pady 2
    grid columnconfigure $win.file 0 -weight 1
  pack $win.file -fill x -padx 2 -pady 2

  frame $win.buttons
    button $win.buttons.ok     -textvariable GUI_Text(Ok) -command \
      "creole_UnicodeListLookupCreateRuleFile $win.files.files.listbox \
                                              $win.file.file $win.files.encoding
       destroy $win"
    button $win.buttons.cancel -textvariable GUI_Text(Cancel) -command \
      "destroy $win"
    grid $win.buttons.ok $win.buttons.cancel -padx 8 -pady 2
  pack $win.buttons -fill x -padx 6 -pady 12

  gut_ModalDlg $win 450 300
  wm deiconify $win
  update
  ## Make the listbox a Drop target...
  catch {
    dnd bindtarget $win.files.files.listbox text/uri-list <DragEnter> \
      {%W configure -bg $GUI_Options(DragEnterBackgound);update;return %a}
    dnd bindtarget $win.files.files.listbox text/uri-list <DragLeave> \
      {%W configure -bg $GUI_Options(InputBackground);update}
    dnd bindtarget $win.files.files.listbox text/uri-list <Drop> \
      {%W configure -bg $GUI_Options(InputBackground);GUI_AddToListBox %W %D}
  }
};# creole_UnicodeListLookupCreateFile

proc creole_UnicodeListLookupEscapeRegexpSpecialCharactersMap {} {
  return {( \\( ) \\) \" \\\" \' \\\' \\ \\\\
      \{ \\\{ \} \\\} . \\. / \\/ * \\* [ \\[ ] \\] + \\+ ? \\? $ \\$ ^ \\^
      : \\: \# \\\#}
};# creole_UnicodeListLookupEscapeRegexpSpecialCharactersMap

proc creole_UnicodeListLookupCreateRuleFile {listbox entry menubutton} {
  if {![string length [$entry get]]} {return}
  set encoding [$menubutton cget -text]

  ## Collect the data from the list files...
  foreach file [$listbox get 0 end] {
    if {![file exists $file] || ![file isfile $file]} {continue}
    set type [file rootname [file tail $file]]
    set Types($type) 0
    set list [open $file]
    fconfigure $list -encoding $encoding
    while {[gets $list line] >= 0} {
      set line [string trim $line]
      if {[string length $line]} {
        lappend Type($type) [string toupper $line]
      }
    }
    close $list
  }
  set map [creole_UnicodeListLookupEscapeRegexpSpecialCharactersMap]
  lappend map { } {\s+}

  ## Now we have all the information. Create the rule file...
  set Rules [open [$entry get] w]
  fconfigure $Rules -encoding utf-8
  foreach type [array names Types] {
    set Patterns [lsort $Type($type)]
    ## We have to split this list in sublists of 100 elements...
    for {set i 0} {$i < [llength $Patterns]} {incr i 100} {
      set sublist [lrange $Patterns $i [expr {$i+99}]]
      puts $Rules [list [string map $map [join $sublist |]] $type $type]
    }
  }
  close $Rules
};# creole_UnicodeListLookupCreateRuleFile
#
# End of File
#
